//This file is automatically generated. DO NOT EDIT!
package com.robotraconteur.testing.TestService3;
import java.util.*;
import com.robotraconteur.*;
public class obj1_default_impl implements obj1{
    protected double[] rrvar_d1;
    public double[] get_d1() { return rrvar_d1; }
    public void set_d1(double[] value) { rrvar_d1 = value; }
}
