//This file is automatically generated. DO NOT EDIT!
package com.robotraconteur.testing.TestService3;
import java.util.*;
import com.robotraconteur.*;
public class transform_stub extends NamedArrayStub<transform,double[]> {
    public double[] getNumericArrayFromNamedArrayStruct(transform s) {
    return s.getNumericArray();
    }
    public transform getNamedArrayStructFromNumericArray(double[] m) {
    transform s = new transform();
    s.assignFromNumericArray(m,0);
    return s;
    }
    public double[] getNumericArrayFromNamedArray(transform[] s) {
    return transform.getNumericArray(s);
    }
    public transform[] getNamedArrayFromNumericArray(double[] m) {
    transform[] s = new transform[m.length / 7];
    for (int i=0; i<s.length; i++) s[i] = new transform();
    transform.assignFromNumericArray(s,m,0);
    return s;
    }
    public String getTypeName() { return "com.robotraconteur.testing.TestService3.transform"; }}
