//This file is automatically generated. DO NOT EDIT!
package com.robotraconteur.testing.TestService1;
import java.util.*;
import com.robotraconteur.*;
class com__robotraconteur__testing__TestService1Constants 
{
    public static class testroot
    {
    public static final double[] doubleconst={3.4, 4.8, 14372.8};
    public static final String strconst="This is a constant";
    }
}
