//This file is automatically generated. DO NOT EDIT!
package com.robotraconteur.testing.TestService1;
import java.util.*;
import com.robotraconteur.*;
public class sub3_default_impl implements sub3{
    protected String rrvar_ind;
    public String get_ind() { return rrvar_ind; }
    public void set_ind(String value) { rrvar_ind = value; }
    protected String rrvar_data2;
    public String get_data2() { return rrvar_data2; }
    public void set_data2(String value) { rrvar_data2 = value; }
    protected double rrvar_data3;
    public double get_data3() { return rrvar_data3; }
    public void set_data3(double value) { rrvar_data3 = value; }
    public double add(double d) {
    throw new UnsupportedOperationException();    }
}
