//This file is automatically generated. DO NOT EDIT!
package com.robotraconteur.testing.TestService3;
import java.util.*;
import com.robotraconteur.*;
public class vector3 implements RRNamedArray 
{
    public double x;
    public double y;
    public double z;
    public vector3()
    {
    x = 0.0;
    y = 0.0;
    z = 0.0;
    }
    public double[] getNumericArray()
    {
    double[] a = new double[3];
    getNumericArray(a,0);
    return a;
    }
    public void getNumericArray(double[] buffer, int offset)
    {
    buffer[offset + 0] = x;
    buffer[offset + 1] = y;
    buffer[offset + 2] = z;
    }
    public void assignFromNumericArray(double[] buffer, int offset)
    {
    x = buffer[offset + 0];
    y = buffer[offset + 1];
    z = buffer[offset + 2];
    }
    public static double[] getNumericArray(vector3[] s)
    {
    double[] a = new double[3 * s.length];
    getNumericArray(s,a,0);
    return a;
    }
    public static void getNumericArray(vector3[] s, double[] a, int offset)
    {
    for (int i=0; i<s.length; i++)
    {
    s[i].getNumericArray(a, offset + 3 * i);
    }
    }
    public static void assignFromNumericArray(vector3[] s, double[] a, int offset)
    {
    for (int i=0; i<s.length; i++)
    {
    s[i].assignFromNumericArray(a, offset + 3 * i);
    }
    }
}

