//This file is automatically generated. DO NOT EDIT!
package com.robotraconteur.testing.TestService3;
import java.util.*;
import com.robotraconteur.*;
public class testroot3_default_impl implements testroot3{
    protected PipeBroadcaster<int[]> rrvar_unreliable1;
    protected WireBroadcaster<int[]> rrvar_peekwire;
    protected WireUnicastReceiver<int[]> rrvar_pokewire;
    protected int rrvar_readme;
    public int get_readme() { return rrvar_readme; }
    public void set_readme(int value) { rrvar_readme = value; }
    protected int rrvar_writeme;
    public int get_writeme() { return rrvar_writeme; }
    public void set_writeme(int value) { rrvar_writeme = value; }
    protected int rrvar_unknown_modifier;
    public int get_unknown_modifier() { return rrvar_unknown_modifier; }
    public void set_unknown_modifier(int value) { rrvar_unknown_modifier = value; }
    protected testenum1 rrvar_testenum1_prop;
    public testenum1 get_testenum1_prop() { return rrvar_testenum1_prop; }
    public void set_testenum1_prop(testenum1 value) { rrvar_testenum1_prop = value; }
    protected testpod1 rrvar_testpod1_prop;
    public testpod1 get_testpod1_prop() { return rrvar_testpod1_prop; }
    public void set_testpod1_prop(testpod1 value) { rrvar_testpod1_prop = value; }
    protected teststruct3 rrvar_teststruct3_prop;
    public teststruct3 get_teststruct3_prop() { return rrvar_teststruct3_prop; }
    public void set_teststruct3_prop(teststruct3 value) { rrvar_teststruct3_prop = value; }
    protected List<double[]> rrvar_d1;
    public List<double[]> get_d1() { return rrvar_d1; }
    public void set_d1(List<double[]> value) { rrvar_d1 = value; }
    protected List<double[]> rrvar_d2;
    public List<double[]> get_d2() { return rrvar_d2; }
    public void set_d2(List<double[]> value) { rrvar_d2 = value; }
    protected Map<Integer,double[]> rrvar_d3;
    public Map<Integer,double[]> get_d3() { return rrvar_d3; }
    public void set_d3(Map<Integer,double[]> value) { rrvar_d3 = value; }
    protected Map<Integer,double[]> rrvar_d4;
    public Map<Integer,double[]> get_d4() { return rrvar_d4; }
    public void set_d4(Map<Integer,double[]> value) { rrvar_d4 = value; }
    protected List<MultiDimArray> rrvar_d5;
    public List<MultiDimArray> get_d5() { return rrvar_d5; }
    public void set_d5(List<MultiDimArray> value) { rrvar_d5 = value; }
    protected Map<Integer,MultiDimArray> rrvar_d6;
    public Map<Integer,MultiDimArray> get_d6() { return rrvar_d6; }
    public void set_d6(Map<Integer,MultiDimArray> value) { rrvar_d6 = value; }
    protected vector3 rrvar_testnamedarray1;
    public vector3 get_testnamedarray1() { return rrvar_testnamedarray1; }
    public void set_testnamedarray1(vector3 value) { rrvar_testnamedarray1 = value; }
    protected transform rrvar_testnamedarray2;
    public transform get_testnamedarray2() { return rrvar_testnamedarray2; }
    public void set_testnamedarray2(transform value) { rrvar_testnamedarray2 = value; }
    protected transform[] rrvar_testnamedarray3;
    public transform[] get_testnamedarray3() { return rrvar_testnamedarray3; }
    public void set_testnamedarray3(transform[] value) { rrvar_testnamedarray3 = value; }
    protected NamedMultiDimArray rrvar_testnamedarray4;
    public NamedMultiDimArray get_testnamedarray4() { return rrvar_testnamedarray4; }
    public void set_testnamedarray4(NamedMultiDimArray value) { rrvar_testnamedarray4 = value; }
    protected NamedMultiDimArray rrvar_testnamedarray5;
    public NamedMultiDimArray get_testnamedarray5() { return rrvar_testnamedarray5; }
    public void set_testnamedarray5(NamedMultiDimArray value) { rrvar_testnamedarray5 = value; }
    protected CDouble rrvar_c1;
    public CDouble get_c1() { return rrvar_c1; }
    public void set_c1(CDouble value) { rrvar_c1 = value; }
    protected CDouble[] rrvar_c2;
    public CDouble[] get_c2() { return rrvar_c2; }
    public void set_c2(CDouble[] value) { rrvar_c2 = value; }
    protected MultiDimArray rrvar_c3;
    public MultiDimArray get_c3() { return rrvar_c3; }
    public void set_c3(MultiDimArray value) { rrvar_c3 = value; }
    protected List<CDouble[]> rrvar_c4;
    public List<CDouble[]> get_c4() { return rrvar_c4; }
    public void set_c4(List<CDouble[]> value) { rrvar_c4 = value; }
    protected List<CDouble[]> rrvar_c5;
    public List<CDouble[]> get_c5() { return rrvar_c5; }
    public void set_c5(List<CDouble[]> value) { rrvar_c5 = value; }
    protected List<MultiDimArray> rrvar_c6;
    public List<MultiDimArray> get_c6() { return rrvar_c6; }
    public void set_c6(List<MultiDimArray> value) { rrvar_c6 = value; }
    protected CSingle rrvar_c7;
    public CSingle get_c7() { return rrvar_c7; }
    public void set_c7(CSingle value) { rrvar_c7 = value; }
    protected CSingle[] rrvar_c8;
    public CSingle[] get_c8() { return rrvar_c8; }
    public void set_c8(CSingle[] value) { rrvar_c8 = value; }
    protected MultiDimArray rrvar_c9;
    public MultiDimArray get_c9() { return rrvar_c9; }
    public void set_c9(MultiDimArray value) { rrvar_c9 = value; }
    protected List<CSingle[]> rrvar_c10;
    public List<CSingle[]> get_c10() { return rrvar_c10; }
    public void set_c10(List<CSingle[]> value) { rrvar_c10 = value; }
    protected List<CSingle[]> rrvar_c11;
    public List<CSingle[]> get_c11() { return rrvar_c11; }
    public void set_c11(List<CSingle[]> value) { rrvar_c11 = value; }
    protected List<MultiDimArray> rrvar_c12;
    public List<MultiDimArray> get_c12() { return rrvar_c12; }
    public void set_c12(List<MultiDimArray> value) { rrvar_c12 = value; }
    protected boolean rrvar_b1;
    public boolean get_b1() { return rrvar_b1; }
    public void set_b1(boolean value) { rrvar_b1 = value; }
    protected boolean[] rrvar_b2;
    public boolean[] get_b2() { return rrvar_b2; }
    public void set_b2(boolean[] value) { rrvar_b2 = value; }
    protected MultiDimArray rrvar_b3;
    public MultiDimArray get_b3() { return rrvar_b3; }
    public void set_b3(MultiDimArray value) { rrvar_b3 = value; }
    protected List<boolean[]> rrvar_b4;
    public List<boolean[]> get_b4() { return rrvar_b4; }
    public void set_b4(List<boolean[]> value) { rrvar_b4 = value; }
    protected List<boolean[]> rrvar_b5;
    public List<boolean[]> get_b5() { return rrvar_b5; }
    public void set_b5(List<boolean[]> value) { rrvar_b5 = value; }
    protected List<MultiDimArray> rrvar_b6;
    public List<MultiDimArray> get_b6() { return rrvar_b6; }
    public void set_b6(List<MultiDimArray> value) { rrvar_b6 = value; }
    public void testpod1_func1(testpod1 s) {
    throw new UnsupportedOperationException();    }
    public testpod1 testpod1_func2() {
    throw new UnsupportedOperationException();    }
    public Generator2<double[]> gen_func1() {
    throw new UnsupportedOperationException();    }
    public Generator2<UnsignedBytes> gen_func2(String name) {
    throw new UnsupportedOperationException();    }
    public Generator3<UnsignedBytes> gen_func3(String name) {
    throw new UnsupportedOperationException();    }
    public Generator1<UnsignedBytes,UnsignedBytes> gen_func4() {
    throw new UnsupportedOperationException();    }
    public Generator1<com.robotraconteur.testing.TestService1.teststruct2,com.robotraconteur.testing.TestService1.teststruct2> gen_func5() {
    throw new UnsupportedOperationException();    }
    public void test_exception_params1() {
    throw new UnsupportedOperationException();    }
    public void test_exception_params2() {
    throw new UnsupportedOperationException();    }
    public Generator2<testenum1[]> enum_generator1() {
    throw new UnsupportedOperationException();    }
    public Generator1<testenum1[],testenum1[]> enum_generator2(int a, int b) {
    throw new UnsupportedOperationException();    }
    public obj4 get_o4() {
    throw new UnsupportedOperationException();
    }
    public obj5 get_nolock_test() {
    throw new UnsupportedOperationException();
    }
    public Pipe<int[]> get_unreliable1()
    { return rrvar_unreliable1.getPipe();  }
    public void set_unreliable1(Pipe<int[]> value)
    {
    if (rrvar_unreliable1!=null) throw new IllegalStateException("Pipe already set");
    rrvar_unreliable1= new PipeBroadcaster<int[]>(value);
    }
    public Pipe<int[]> get_p1()
    { throw new UnsupportedOperationException(); }
    public void set_p1(Pipe<int[]> value)
    { throw new IllegalStateException();}
    public Pipe<int[]> get_p2()
    { throw new UnsupportedOperationException(); }
    public void set_p2(Pipe<int[]> value)
    { throw new IllegalStateException();}
    public Pipe<MultiDimArray> get_p3()
    { throw new UnsupportedOperationException(); }
    public void set_p3(Pipe<MultiDimArray> value)
    { throw new IllegalStateException();}
    public Wire<int[]> get_peekwire()
    { return rrvar_peekwire.getWire();  }
    public void set_peekwire(Wire<int[]> value)
    {
    if (rrvar_peekwire!=null) throw new IllegalStateException("Pipe already set");
    rrvar_peekwire= new WireBroadcaster<int[]>(value);
    }
    public Wire<int[]> get_pokewire()
    { return rrvar_pokewire.getWire();  }
    public void set_pokewire(Wire<int[]> value)
    {
    if (rrvar_pokewire!=null) throw new IllegalStateException("Pipe already set");
    rrvar_pokewire= new WireUnicastReceiver<int[]>(value);
    }
    public Wire<int[]> get_w1()
    { throw new UnsupportedOperationException(); }
    public void set_w1(Wire<int[]> value)
    { throw new UnsupportedOperationException();}
    public Wire<int[]> get_w2()
    { throw new UnsupportedOperationException(); }
    public void set_w2(Wire<int[]> value)
    { throw new UnsupportedOperationException();}
    public Wire<MultiDimArray> get_w3()
    { throw new UnsupportedOperationException(); }
    public void set_w3(Wire<MultiDimArray> value)
    { throw new UnsupportedOperationException();}
    public ArrayMemory<double[]> get_readmem()
    { throw new UnsupportedOperationException(); }
    public PodArrayMemory<testpod2[]> get_pod_m1()
    { throw new UnsupportedOperationException(); }
    public PodMultiDimArrayMemory<testpod2[]> get_pod_m2()
    { throw new UnsupportedOperationException(); }
    public NamedArrayMemory<transform[]> get_namedarray_m1()
    { throw new UnsupportedOperationException(); }
    public NamedMultiDimArrayMemory<transform[]> get_namedarray_m2()
    { throw new UnsupportedOperationException(); }
    public ArrayMemory<CDouble[]> get_c_m1()
    { throw new UnsupportedOperationException(); }
    public MultiDimArrayMemory<CDouble[]> get_c_m2()
    { throw new UnsupportedOperationException(); }
    public ArrayMemory<CDouble[]> get_c_m3()
    { throw new UnsupportedOperationException(); }
    public MultiDimArrayMemory<CDouble[]> get_c_m4()
    { throw new UnsupportedOperationException(); }
    public ArrayMemory<boolean[]> get_c_m5()
    { throw new UnsupportedOperationException(); }
    public MultiDimArrayMemory<boolean[]> get_c_m6()
    { throw new UnsupportedOperationException(); }
}
