/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 4.0.2
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package com.robotraconteur;

public class WrappedRobotRaconteurNodeSetup {
  private transient long swigCPtr;
  private transient boolean swigCMemOwn;

  protected WrappedRobotRaconteurNodeSetup(long cPtr, boolean cMemoryOwn) {
    swigCMemOwn = cMemoryOwn;
    swigCPtr = cPtr;
  }

  protected static long getCPtr(WrappedRobotRaconteurNodeSetup obj) {
    return (obj == null) ? 0 : obj.swigCPtr;
  }

  protected void swigSetCMemOwn(boolean own) {
    swigCMemOwn = own;
  }

  @SuppressWarnings("deprecation")
  protected void finalize() {
    delete();
  }

  public synchronized void delete() {
    if (swigCPtr != 0) {
      if (swigCMemOwn) {
        swigCMemOwn = false;
        RobotRaconteurJavaJNI.delete_WrappedRobotRaconteurNodeSetup(swigCPtr);
      }
      swigCPtr = 0;
    }
  }

  public LocalTransport getLocalTransport() {
    long cPtr = RobotRaconteurJavaJNI.WrappedRobotRaconteurNodeSetup_getLocalTransport(swigCPtr, this);
    return (cPtr == 0) ? null : new LocalTransport(cPtr, true);
  }

  public IntraTransport getIntraTransport() {
    long cPtr = RobotRaconteurJavaJNI.WrappedRobotRaconteurNodeSetup_getIntraTransport(swigCPtr, this);
    return (cPtr == 0) ? null : new IntraTransport(cPtr, true);
  }

  public TcpTransport getTcpTransport() {
    long cPtr = RobotRaconteurJavaJNI.WrappedRobotRaconteurNodeSetup_getTcpTransport(swigCPtr, this);
    return (cPtr == 0) ? null : new TcpTransport(cPtr, true);
  }

  public HardwareTransport getHardwareTransport() {
    long cPtr = RobotRaconteurJavaJNI.WrappedRobotRaconteurNodeSetup_getHardwareTransport(swigCPtr, this);
    return (cPtr == 0) ? null : new HardwareTransport(cPtr, true);
  }

  public CommandLineConfigParser getCommandLineConfig() {
    long cPtr = RobotRaconteurJavaJNI.WrappedRobotRaconteurNodeSetup_getCommandLineConfig(swigCPtr, this);
    return (cPtr == 0) ? null : new CommandLineConfigParser(cPtr, true);
  }

  public void releaseNode() {
    RobotRaconteurJavaJNI.WrappedRobotRaconteurNodeSetup_releaseNode(swigCPtr, this);
  }

  public WrappedRobotRaconteurNodeSetup(RobotRaconteurNode node, String node_name, int tcp_port, long flags) {
    this(RobotRaconteurJavaJNI.new_WrappedRobotRaconteurNodeSetup__SWIG_0(RobotRaconteurNode.getCPtr(node), node, node_name, tcp_port, flags), true);
  }

  public WrappedRobotRaconteurNodeSetup(RobotRaconteurNode node, CommandLineConfigParser config) {
    this(RobotRaconteurJavaJNI.new_WrappedRobotRaconteurNodeSetup__SWIG_1(RobotRaconteurNode.getCPtr(node), node, CommandLineConfigParser.getCPtr(config), config), true);
  }

  public WrappedRobotRaconteurNodeSetup(RobotRaconteurNode node, String node_name, int tcp_port, long flags, long allowed_overrides, vectorstring args) {
    this(RobotRaconteurJavaJNI.new_WrappedRobotRaconteurNodeSetup__SWIG_2(RobotRaconteurNode.getCPtr(node), node, node_name, tcp_port, flags, allowed_overrides, vectorstring.getCPtr(args), args), true);
  }

}
