//This file is automatically generated. DO NOT EDIT!
package com.robotraconteur.testing.TestService1;
import java.util.*;
import com.robotraconteur.*;
public class sub1_default_impl implements sub1{
    protected double[] rrvar_d1;
    public double[] get_d1() { return rrvar_d1; }
    public void set_d1(double[] value) { rrvar_d1 = value; }
    protected MultiDimArray rrvar_d2;
    public MultiDimArray get_d2() { return rrvar_d2; }
    public void set_d2(MultiDimArray value) { rrvar_d2 = value; }
    protected String rrvar_s_ind;
    public String get_s_ind() { return rrvar_s_ind; }
    public void set_s_ind(String value) { rrvar_s_ind = value; }
    protected int rrvar_i_ind;
    public int get_i_ind() { return rrvar_i_ind; }
    public void set_i_ind(int value) { rrvar_i_ind = value; }
    public sub2 get_o2_1() {
    throw new UnsupportedOperationException();
    }
    public sub2 get_o2_2(int ind) {
    throw new UnsupportedOperationException();
    }
    public sub2 get_o2_3(String ind) {
    throw new UnsupportedOperationException();
    }
}
