//This file is automatically generated. DO NOT EDIT!
package com.robotraconteur.testing.TestService3;
import java.util.*;
import com.robotraconteur.*;
public class pixel2_stub extends NamedArrayStub<pixel2,UnsignedBytes> {
    public UnsignedBytes getNumericArrayFromNamedArrayStruct(pixel2 s) {
    return s.getNumericArray();
    }
    public pixel2 getNamedArrayStructFromNumericArray(UnsignedBytes m) {
    pixel2 s = new pixel2();
    s.assignFromNumericArray(m,0);
    return s;
    }
    public UnsignedBytes getNumericArrayFromNamedArray(pixel2[] s) {
    return pixel2.getNumericArray(s);
    }
    public pixel2[] getNamedArrayFromNumericArray(UnsignedBytes m) {
    pixel2[] s = new pixel2[m.value.length / 31];
    for (int i=0; i<s.length; i++) s[i] = new pixel2();
    pixel2.assignFromNumericArray(s,m,0);
    return s;
    }
    public String getTypeName() { return "com.robotraconteur.testing.TestService3.pixel2"; }}
