/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 4.0.2
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package com.robotraconteur;

public class ServiceSubscriptionFilterAttribute {
  private transient long swigCPtr;
  protected transient boolean swigCMemOwn;

  protected ServiceSubscriptionFilterAttribute(long cPtr, boolean cMemoryOwn) {
    swigCMemOwn = cMemoryOwn;
    swigCPtr = cPtr;
  }

  protected static long getCPtr(ServiceSubscriptionFilterAttribute obj) {
    return (obj == null) ? 0 : obj.swigCPtr;
  }

  @SuppressWarnings("deprecation")
  protected void finalize() {
    delete();
  }

  public synchronized void delete() {
    if (swigCPtr != 0) {
      if (swigCMemOwn) {
        swigCMemOwn = false;
        RobotRaconteurJavaJNI.delete_ServiceSubscriptionFilterAttribute(swigCPtr);
      }
      swigCPtr = 0;
    }
  }

  public void setName(String value) {
    RobotRaconteurJavaJNI.ServiceSubscriptionFilterAttribute_Name_set(swigCPtr, this, value);
  }

  public String getName() {
    return RobotRaconteurJavaJNI.ServiceSubscriptionFilterAttribute_Name_get(swigCPtr, this);
  }

  public void setValue(String value) {
    RobotRaconteurJavaJNI.ServiceSubscriptionFilterAttribute_Value_set(swigCPtr, this, value);
  }

  public String getValue() {
    return RobotRaconteurJavaJNI.ServiceSubscriptionFilterAttribute_Value_get(swigCPtr, this);
  }

  public void setUseRegex(boolean value) {
    RobotRaconteurJavaJNI.ServiceSubscriptionFilterAttribute_UseRegex_set(swigCPtr, this, value);
  }

  public boolean getUseRegex() {
    return RobotRaconteurJavaJNI.ServiceSubscriptionFilterAttribute_UseRegex_get(swigCPtr, this);
  }

  public ServiceSubscriptionFilterAttribute(String value) {
    this(RobotRaconteurJavaJNI.new_ServiceSubscriptionFilterAttribute__SWIG_0(value), true);
  }

  public ServiceSubscriptionFilterAttribute(String name, String value) {
    this(RobotRaconteurJavaJNI.new_ServiceSubscriptionFilterAttribute__SWIG_1(name, value), true);
  }

  public boolean isMatch(String value) {
    return RobotRaconteurJavaJNI.ServiceSubscriptionFilterAttribute_isMatch__SWIG_0(swigCPtr, this, value);
  }

  public boolean isMatch(String name, String value) {
    return RobotRaconteurJavaJNI.ServiceSubscriptionFilterAttribute_isMatch__SWIG_1(swigCPtr, this, name, value);
  }

  public boolean isMatch(vectorstring values) {
    return RobotRaconteurJavaJNI.ServiceSubscriptionFilterAttribute_isMatch__SWIG_2(swigCPtr, this, vectorstring.getCPtr(values), values);
  }

  public boolean isMatch(map_strstr values) {
    return RobotRaconteurJavaJNI.ServiceSubscriptionFilterAttribute_isMatch__SWIG_3(swigCPtr, this, map_strstr.getCPtr(values), values);
  }

}
