//This file is automatically generated. DO NOT EDIT!
package com.robotraconteur.testing.TestService1;
import java.util.*;
import com.robotraconteur.*;
@RobotRaconteurServiceObjectInterface
public interface sub1
{
    double[] get_d1();
    void set_d1(double[] value);
    MultiDimArray get_d2();
    void set_d2(MultiDimArray value);
    String get_s_ind();
    void set_s_ind(String value);
    int get_i_ind();
    void set_i_ind(int value);
    sub2 get_o2_1();
    sub2 get_o2_2(int ind);
    sub2 get_o2_3(String ind);
}

