/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 4.0.2
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package com.robotraconteur;

public enum MemberDefinition_Direction {
  MemberDefinition_Direction_both(0),
  MemberDefinition_Direction_readonly,
  MemberDefinition_Direction_writeonly;

  public final int swigValue() {
    return swigValue;
  }

  public static MemberDefinition_Direction swigToEnum(int swigValue) {
    MemberDefinition_Direction[] swigValues = MemberDefinition_Direction.class.getEnumConstants();
    if (swigValue < swigValues.length && swigValue >= 0 && swigValues[swigValue].swigValue == swigValue)
      return swigValues[swigValue];
    for (MemberDefinition_Direction swigEnum : swigValues)
      if (swigEnum.swigValue == swigValue)
        return swigEnum;
    throw new IllegalArgumentException("No enum " + MemberDefinition_Direction.class + " with value " + swigValue);
  }

  @SuppressWarnings("unused")
  private MemberDefinition_Direction() {
    this.swigValue = SwigNext.next++;
  }

  @SuppressWarnings("unused")
  private MemberDefinition_Direction(int swigValue) {
    this.swigValue = swigValue;
    SwigNext.next = swigValue+1;
  }

  @SuppressWarnings("unused")
  private MemberDefinition_Direction(MemberDefinition_Direction swigEnum) {
    this.swigValue = swigEnum.swigValue;
    SwigNext.next = this.swigValue+1;
  }

  private final int swigValue;

  private static class SwigNext {
    private static int next = 0;
  }
}

