//This file is automatically generated. DO NOT EDIT!
package com.robotraconteur.testing.TestService2;
import java.util.*;
import com.robotraconteur.*;
@RobotRaconteurServiceObjectInterface
public interface baseobj
{
    double get_d1();
    void set_d1(double value);
    double[] get_d2();
    void set_d2(double[] value);
    double func3(double d1, double d2);
    void  addev1Listener(Action listener); 
    void  removeev1Listener(Action listener); 
    subobj get_o5();
    Pipe<double[]> get_p1();
    void set_p1(Pipe<double[]> value);
    Callback<Action2<Double, Double>> get_cb2();
    void set_cb2(Callback<Action2<Double, Double>> value);
    Wire<double[]> get_w1();
    void set_w1(Wire<double[]> value);
    ArrayMemory<double[]> get_m1();
}

