//This file is automatically generated. DO NOT EDIT!
package com.robotraconteur.testing.TestService3;
import java.util.*;
import com.robotraconteur.*;
public class vector3_stub extends NamedArrayStub<vector3,double[]> {
    public double[] getNumericArrayFromNamedArrayStruct(vector3 s) {
    return s.getNumericArray();
    }
    public vector3 getNamedArrayStructFromNumericArray(double[] m) {
    vector3 s = new vector3();
    s.assignFromNumericArray(m,0);
    return s;
    }
    public double[] getNumericArrayFromNamedArray(vector3[] s) {
    return vector3.getNumericArray(s);
    }
    public vector3[] getNamedArrayFromNumericArray(double[] m) {
    vector3[] s = new vector3[m.length / 3];
    for (int i=0; i<s.length; i++) s[i] = new vector3();
    vector3.assignFromNumericArray(s,m,0);
    return s;
    }
    public String getTypeName() { return "com.robotraconteur.testing.TestService3.vector3"; }}
