//This file is automatically generated. DO NOT EDIT!
package com.robotraconteur.testing.TestService2;
import java.util.*;
import com.robotraconteur.*;
public class testexception3 extends RobotRaconteurRemoteException
{
    public testexception3(String message)  {
    super("com.robotraconteur.testing.TestService2.testexception3",message);
    }

    public testexception3(String message, String subname, Object param_)  {
    super("com.robotraconteur.testing.TestService2.testexception3",message, subname, param_);
    }
};
