//This file is automatically generated. DO NOT EDIT!
package com.robotraconteur.testing.TestService3;
import java.util.*;
import com.robotraconteur.*;
public class obj4_default_impl implements obj4{
    protected String rrvar_s_ind;
    public String get_s_ind() { return rrvar_s_ind; }
    public void set_s_ind(String value) { rrvar_s_ind = value; }
    protected int rrvar_i_ind;
    public int get_i_ind() { return rrvar_i_ind; }
    public void set_i_ind(int value) { rrvar_i_ind = value; }
    protected String rrvar_data;
    public String get_data() { return rrvar_data; }
    public void set_data(String value) { rrvar_data = value; }
    public com.robotraconteur.testing.TestService1.sub3 get_o3_1(String ind) {
    throw new UnsupportedOperationException();
    }
}
