//This file is automatically generated. DO NOT EDIT!
package com.robotraconteur.testing.TestService3;
import java.util.*;
import com.robotraconteur.*;
@RobotRaconteurServiceObjectInterface
public interface obj5
{
    double get_p1();
    void set_p1(double value);
    double get_p2();
    void set_p2(double value);
    double get_p3();
    void set_p3(double value);
    int f1();
    int f2();
    Pipe<int[]> get_q1();
    void set_q1(Pipe<int[]> value);
    Pipe<int[]> get_q2();
    void set_q2(Pipe<int[]> value);
    Wire<int[]> get_w1();
    void set_w1(Wire<int[]> value);
    Wire<int[]> get_w2();
    void set_w2(Wire<int[]> value);
    ArrayMemory<int[]> get_m1();
    ArrayMemory<int[]> get_m2();
    ArrayMemory<int[]> get_m3();
}

