//This file is automatically generated. DO NOT EDIT!
package com.robotraconteur.testing.TestService1;
import java.util.*;
import com.robotraconteur.*;
@RobotRaconteurServiceObjectInterface
public interface sub3
{
    String get_ind();
    void set_ind(String value);
    String get_data2();
    void set_data2(String value);
    double get_data3();
    void set_data3(double value);
    double add(double d);
}

