//This file is automatically generated. DO NOT EDIT!
package com.robotraconteur.testing.TestService2;
import java.util.*;
import com.robotraconteur.*;
public interface async_subobj
{
    void async_add_val(double v,Action2<Double,RuntimeException> rr_handler,int rr_timeout);
}

