//This file is automatically generated. DO NOT EDIT!
package com.robotraconteur.testing.TestService3;
import java.util.*;
import com.robotraconteur.*;
class com__robotraconteur__testing__TestService3Constants 
{
    public static final String strconst="This is a\n \"string constant\" \\/\b\f \r\u00ff tabme\ttabme\n smile! \ud83d\ude01";
    public static final int int32const=3856384;
    public static final int[] int32const_array={182476, 56483, -2947};
    public static final int int32hexconst=0x082bc7;
    public static final int int32hexconst2=-0x7264c17;
    public static final int[] int32hexconst_array={ 0x8274ec, -0x0001, +0xABCDEF, 0xabcdef, 0x012345, 0x6789 };
    public static final double[] doubleconst_array={1.5847, 3.14, -548e3, 3452.67e2, 485e-21};
    public static class structconst { public static final String strconst="This is a\n \"string constant\" \\/\b\f \r\u00ff tabme\ttabme\n smile! \ud83d\ude01"; public static final int[] int32const_array={182476, 56483, -2947}; }
    public static class structconst2 { public static class structconst { public static final String strconst="This is a\n \"string constant\" \\/\b\f \r\u00ff tabme\ttabme\n smile! \ud83d\ude01"; public static final int[] int32const_array={182476, 56483, -2947}; } public static final int int32const=3856384; }
    public static class testroot3
    {
    public static final String unknown_modifier_hello="hello world!";
    }
}
