//This file is automatically generated. DO NOT EDIT!
package com.robotraconteur.testing.TestService1;
import java.util.*;
import com.robotraconteur.*;
@RobotRaconteurServiceObjectInterface
public interface sub2
{
    String get_s_ind();
    void set_s_ind(String value);
    int get_i_ind();
    void set_i_ind(int value);
    String get_data();
    void set_data(String value);
    sub3 get_o3_1(String ind);
}

