//This file is automatically generated. DO NOT EDIT!
package com.robotraconteur.testing.TestService3;
import java.util.*;
import com.robotraconteur.*;
public interface async_obj3 extends async_obj1, async_obj2
{
    void async_get_d1(Action2<double[],RuntimeException> rr_handler, int rr_timeout);
    void async_set_d1(double[] value, Action1<RuntimeException> rr_handler, int rr_timeout);
}

