/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 4.0.2
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package com.robotraconteur;

public enum DataTypes {
  DataTypes_void_t(0),
  DataTypes_double_t,
  DataTypes_single_t,
  DataTypes_int8_t,
  DataTypes_uint8_t,
  DataTypes_int16_t,
  DataTypes_uint16_t,
  DataTypes_int32_t,
  DataTypes_uint32_t,
  DataTypes_int64_t,
  DataTypes_uint64_t,
  DataTypes_string_t,
  DataTypes_cdouble_t,
  DataTypes_csingle_t,
  DataTypes_bool_t,
  DataTypes_structure_t(101),
  DataTypes_vector_t,
  DataTypes_dictionary_t,
  DataTypes_object_t,
  DataTypes_varvalue_t,
  DataTypes_varobject_t,
  DataTypes_list_t(108),
  DataTypes_pod_t,
  DataTypes_pod_array_t,
  DataTypes_pod_multidimarray_t,
  DataTypes_enum_t,
  DataTypes_namedtype_t,
  DataTypes_namedarray_t,
  DataTypes_namedarray_array_t,
  DataTypes_namedarray_multidimarray_t,
  DataTypes_multidimarray_t;

  public final int swigValue() {
    return swigValue;
  }

  public static DataTypes swigToEnum(int swigValue) {
    DataTypes[] swigValues = DataTypes.class.getEnumConstants();
    if (swigValue < swigValues.length && swigValue >= 0 && swigValues[swigValue].swigValue == swigValue)
      return swigValues[swigValue];
    for (DataTypes swigEnum : swigValues)
      if (swigEnum.swigValue == swigValue)
        return swigEnum;
    throw new IllegalArgumentException("No enum " + DataTypes.class + " with value " + swigValue);
  }

  @SuppressWarnings("unused")
  private DataTypes() {
    this.swigValue = SwigNext.next++;
  }

  @SuppressWarnings("unused")
  private DataTypes(int swigValue) {
    this.swigValue = swigValue;
    SwigNext.next = swigValue+1;
  }

  @SuppressWarnings("unused")
  private DataTypes(DataTypes swigEnum) {
    this.swigValue = swigEnum.swigValue;
    SwigNext.next = this.swigValue+1;
  }

  private final int swigValue;

  private static class SwigNext {
    private static int next = 0;
  }
}

