/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 4.0.2
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package com.robotraconteur;

public class TimerEvent {
  private transient long swigCPtr;
  protected transient boolean swigCMemOwn;

  protected TimerEvent(long cPtr, boolean cMemoryOwn) {
    swigCMemOwn = cMemoryOwn;
    swigCPtr = cPtr;
  }

  protected static long getCPtr(TimerEvent obj) {
    return (obj == null) ? 0 : obj.swigCPtr;
  }

  @SuppressWarnings("deprecation")
  protected void finalize() {
    delete();
  }

  public synchronized void delete() {
    if (swigCPtr != 0) {
      if (swigCMemOwn) {
        swigCMemOwn = false;
        RobotRaconteurJavaJNI.delete_TimerEvent(swigCPtr);
      }
      swigCPtr = 0;
    }
  }

  public boolean getStopped() {
    return RobotRaconteurJavaJNI.TimerEvent_stopped_get(swigCPtr, this);
  }

  public com.robotraconteur.TimeSpec getLast_expected() {
    return new TimeSpec(RobotRaconteurJavaJNI.TimerEvent_last_expected_get(swigCPtr, this),true);
}

  public com.robotraconteur.TimeSpec getLast_real() {
    return new TimeSpec(RobotRaconteurJavaJNI.TimerEvent_last_real_get(swigCPtr, this),true);
}

  public com.robotraconteur.TimeSpec getCurrent_expected() {
    return new TimeSpec(RobotRaconteurJavaJNI.TimerEvent_current_expected_get(swigCPtr, this),true);
}

  public com.robotraconteur.TimeSpec getCurrent_real() {
    return new TimeSpec(RobotRaconteurJavaJNI.TimerEvent_current_real_get(swigCPtr, this),true);
}

  public TimerEvent() {
    this(RobotRaconteurJavaJNI.new_TimerEvent(), true);
  }

}
