/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 4.0.2
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package com.robotraconteur;

public class GetUuidForNameAndLockResult {
  private transient long swigCPtr;
  protected transient boolean swigCMemOwn;

  protected GetUuidForNameAndLockResult(long cPtr, boolean cMemoryOwn) {
    swigCMemOwn = cMemoryOwn;
    swigCPtr = cPtr;
  }

  protected static long getCPtr(GetUuidForNameAndLockResult obj) {
    return (obj == null) ? 0 : obj.swigCPtr;
  }

  @SuppressWarnings("deprecation")
  protected void finalize() {
    delete();
  }

  public synchronized void delete() {
    if (swigCPtr != 0) {
      if (swigCMemOwn) {
        swigCMemOwn = false;
        RobotRaconteurJavaJNI.delete_GetUuidForNameAndLockResult(swigCPtr);
      }
      swigCPtr = 0;
    }
  }

  public void setUuid(NodeID value) {
    RobotRaconteurJavaJNI.GetUuidForNameAndLockResult_uuid_set(swigCPtr, this, NodeID.getCPtr(value), value);
  }

  public NodeID getUuid() {
    long cPtr = RobotRaconteurJavaJNI.GetUuidForNameAndLockResult_uuid_get(swigCPtr, this);
    return (cPtr == 0) ? null : new NodeID(cPtr, false);
  }

  public void setName(String value) {
    RobotRaconteurJavaJNI.GetUuidForNameAndLockResult_name_set(swigCPtr, this, value);
  }

  public String getName() {
    return RobotRaconteurJavaJNI.GetUuidForNameAndLockResult_name_get(swigCPtr, this);
  }

  public void setScope(vectorstring value) {
    RobotRaconteurJavaJNI.GetUuidForNameAndLockResult_scope_set(swigCPtr, this, vectorstring.getCPtr(value), value);
  }

  public vectorstring getScope() {
    long cPtr = RobotRaconteurJavaJNI.GetUuidForNameAndLockResult_scope_get(swigCPtr, this);
    return (cPtr == 0) ? null : new vectorstring(cPtr, false);
  }

  public void setFd(NodeDirectoriesFD value) {
    RobotRaconteurJavaJNI.GetUuidForNameAndLockResult_fd_set(swigCPtr, this, NodeDirectoriesFD.getCPtr(value), value);
  }

  public NodeDirectoriesFD getFd() {
    long cPtr = RobotRaconteurJavaJNI.GetUuidForNameAndLockResult_fd_get(swigCPtr, this);
    return (cPtr == 0) ? null : new NodeDirectoriesFD(cPtr, true);
  }

  public GetUuidForNameAndLockResult() {
    this(RobotRaconteurJavaJNI.new_GetUuidForNameAndLockResult(), true);
  }

}
