//This file is automatically generated. DO NOT EDIT!
package com.robotraconteur.testing.TestService2;
import java.util.*;
import com.robotraconteur.*;
public interface async_baseobj
{
    void async_get_d1(Action2<Double,RuntimeException> rr_handler, int rr_timeout);
    void async_set_d1(double value, Action1<RuntimeException> rr_handler, int rr_timeout);
    void async_get_d2(Action2<double[],RuntimeException> rr_handler, int rr_timeout);
    void async_set_d2(double[] value, Action1<RuntimeException> rr_handler, int rr_timeout);
    void async_func3(double d1, double d2,Action2<Double,RuntimeException> rr_handler,int rr_timeout);
    void async_get_o5(Action2<subobj,RuntimeException> handler, int timeout);
}

