//This file is automatically generated. DO NOT EDIT!
package com.robotraconteur.testing.TestService3;
import java.util.*;
import com.robotraconteur.*;
public class obj4_skel extends ServiceSkel {
    protected obj4 obj;
    public obj4_skel(Object o) { 
    super(o);
    obj=(obj4)o;
    }
    public void releaseCastObject() { 
    obj=null;
    super.releaseCastObject();
    }
    public MessageElement callGetProperty(String membername) {
    if(membername.equals( "s_ind"))
    {
    String ret=obj.get_s_ind();
    return MessageElementUtil.packString("return",ret);
    }
    if(membername.equals( "i_ind"))
    {
    int ret=obj.get_i_ind();
    return MessageElementUtil.<int[]>packArray("return",new int[] {ret});
    }
    if(membername.equals( "data"))
    {
    String ret=obj.get_data();
    return MessageElementUtil.packString("return",ret);
    }
    throw new MemberNotFoundException("Member not found");
    }
    public void callSetProperty(String membername, MessageElement m) {
    if(membername.equals( "s_ind"))
    {
    obj.set_s_ind(MessageElementUtil.unpackString(m));
    return;
    }
    if(membername.equals( "i_ind"))
    {
    obj.set_i_ind((MessageElementUtil.<int[]>castDataAndDispose(m))[0]);
    return;
    }
    if(membername.equals( "data"))
    {
    obj.set_data(MessageElementUtil.unpackString(m));
    return;
    }
    throw new MemberNotFoundException("Member not found");
    }
    public MessageElement callFunction(String rr_membername, vectorptr_messageelement rr_m) {
    throw new MemberNotFoundException("Member not found");
    }
    public Object getSubObj(String name, String ind) {
    if(name.equals( "o3_1")) {
    return obj.get_o3_1(ind);
    }
    throw new MemberNotFoundException("");
    }
    public void registerEvents(Object obj1) {
    obj=(obj4)obj1;
    }
    public void unregisterEvents(Object obj1) {
    obj=(obj4)obj1;
    }
    public Object getCallbackFunction(long endpoint, String membername) {
    throw new MemberNotFoundException("Member not found");
    }
    public  void initPipeServers(Object obj1) {
    obj=(obj4)obj1;
    }
    public void initCallbackServers(Object obj1) {
    obj=(obj4)obj1;
    }
    public void initWireServers(Object obj1) {
    obj=(obj4)obj1;
    }
    public WrappedArrayMemoryDirector getArrayMemory(String name) {
    throw new MemberNotFoundException("Member Not Found");
    }
    public WrappedMultiDimArrayMemoryDirector getMultiDimArrayMemory(String name) {
    throw new MemberNotFoundException("Member Not Found");
    }
    public WrappedPodArrayMemoryDirector getPodArrayMemory(String name) {
    throw new MemberNotFoundException("Member Not Found");
    }
    public WrappedPodMultiDimArrayMemoryDirector getPodMultiDimArrayMemory(String name) {
    throw new MemberNotFoundException("Member Not Found");
    }
    public WrappedNamedArrayMemoryDirector getNamedArrayMemory(String name) {
    throw new MemberNotFoundException("Member Not Found");
    }
    public WrappedNamedMultiDimArrayMemoryDirector getNamedMultiDimArrayMemory(String name) {
    throw new MemberNotFoundException("Member Not Found");
    }
    public String getRRType() { return "com.robotraconteur.testing.TestService3.obj4"; }
}
