//This file is automatically generated. DO NOT EDIT!
package com.robotraconteur.testing.TestService3;
import java.util.*;
import com.robotraconteur.*;
public interface async_obj5
{
    void async_get_p1(Action2<Double,RuntimeException> rr_handler, int rr_timeout);
    void async_set_p1(double value, Action1<RuntimeException> rr_handler, int rr_timeout);
    void async_get_p2(Action2<Double,RuntimeException> rr_handler, int rr_timeout);
    void async_set_p2(double value, Action1<RuntimeException> rr_handler, int rr_timeout);
    void async_get_p3(Action2<Double,RuntimeException> rr_handler, int rr_timeout);
    void async_set_p3(double value, Action1<RuntimeException> rr_handler, int rr_timeout);
    void async_f1(Action2<Integer,RuntimeException> rr_handler,int rr_timeout);
    void async_f2(Action2<Integer,RuntimeException> rr_handler,int rr_timeout);
}

