/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 4.0.2
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package com.robotraconteur;

public class WrappedServiceSubscriptionClientID {
  private transient long swigCPtr;
  protected transient boolean swigCMemOwn;

  protected WrappedServiceSubscriptionClientID(long cPtr, boolean cMemoryOwn) {
    swigCMemOwn = cMemoryOwn;
    swigCPtr = cPtr;
  }

  protected static long getCPtr(WrappedServiceSubscriptionClientID obj) {
    return (obj == null) ? 0 : obj.swigCPtr;
  }

  @SuppressWarnings("deprecation")
  protected void finalize() {
    delete();
  }

  public synchronized void delete() {
    if (swigCPtr != 0) {
      if (swigCMemOwn) {
        swigCMemOwn = false;
        RobotRaconteurJavaJNI.delete_WrappedServiceSubscriptionClientID(swigCPtr);
      }
      swigCPtr = 0;
    }
  }

  public void setNodeID(NodeID value) {
    RobotRaconteurJavaJNI.WrappedServiceSubscriptionClientID_NodeID_set(swigCPtr, this, NodeID.getCPtr(value), value);
  }

  public NodeID getNodeID() {
    long cPtr = RobotRaconteurJavaJNI.WrappedServiceSubscriptionClientID_NodeID_get(swigCPtr, this);
    return (cPtr == 0) ? null : new NodeID(cPtr, false);
  }

  public void setServiceName(String value) {
    RobotRaconteurJavaJNI.WrappedServiceSubscriptionClientID_ServiceName_set(swigCPtr, this, value);
  }

  public String getServiceName() {
    return RobotRaconteurJavaJNI.WrappedServiceSubscriptionClientID_ServiceName_get(swigCPtr, this);
  }

  public boolean eq(WrappedServiceSubscriptionClientID id2) {
    return RobotRaconteurJavaJNI.WrappedServiceSubscriptionClientID_eq(swigCPtr, this, WrappedServiceSubscriptionClientID.getCPtr(id2), id2);
  }

  public boolean ne(WrappedServiceSubscriptionClientID id2) {
    return RobotRaconteurJavaJNI.WrappedServiceSubscriptionClientID_ne(swigCPtr, this, WrappedServiceSubscriptionClientID.getCPtr(id2), id2);
  }

}
