//This file is automatically generated. DO NOT EDIT!
package com.robotraconteur.testing.TestService2;
import java.util.*;
import com.robotraconteur.*;
public class subobj_default_impl implements subobj{
    public double add_val(double v) {
    throw new UnsupportedOperationException();    }
}
