/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 4.0.2
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package com.robotraconteur;

public class UserLogRecordHandlerDirector {
  private transient long swigCPtr;
  protected transient boolean swigCMemOwn;

  protected UserLogRecordHandlerDirector(long cPtr, boolean cMemoryOwn) {
    swigCMemOwn = cMemoryOwn;
    swigCPtr = cPtr;
  }

  protected static long getCPtr(UserLogRecordHandlerDirector obj) {
    return (obj == null) ? 0 : obj.swigCPtr;
  }

  @SuppressWarnings("deprecation")
  protected void finalize() {
    delete();
  }

  public synchronized void delete() {
    if (swigCPtr != 0) {
      if (swigCMemOwn) {
        swigCMemOwn = false;
        RobotRaconteurJavaJNI.delete_UserLogRecordHandlerDirector(swigCPtr);
      }
      swigCPtr = 0;
    }
  }

  protected void swigDirectorDisconnect() {
    swigCMemOwn = false;
    delete();
  }

  public void swigReleaseOwnership() {
    swigCMemOwn = false;
    RobotRaconteurJavaJNI.UserLogRecordHandlerDirector_change_ownership(this, swigCPtr, false);
  }

  public void swigTakeOwnership() {
    swigCMemOwn = true;
    RobotRaconteurJavaJNI.UserLogRecordHandlerDirector_change_ownership(this, swigCPtr, true);
  }

  public void handleLogRecord(RRLogRecord record) {
    RobotRaconteurJavaJNI.UserLogRecordHandlerDirector_handleLogRecord(swigCPtr, this, RRLogRecord.getCPtr(record), record);
  }

  public UserLogRecordHandlerDirector() {
    this(RobotRaconteurJavaJNI.new_UserLogRecordHandlerDirector(), true);
    RobotRaconteurJavaJNI.UserLogRecordHandlerDirector_director_connect(this, swigCPtr, true, true);
  }

}
