/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 4.0.2
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package com.robotraconteur;

public class EnumDefinitionValue {
  private transient long swigCPtr;
  protected transient boolean swigCMemOwn;

  protected EnumDefinitionValue(long cPtr, boolean cMemoryOwn) {
    swigCMemOwn = cMemoryOwn;
    swigCPtr = cPtr;
  }

  protected static long getCPtr(EnumDefinitionValue obj) {
    return (obj == null) ? 0 : obj.swigCPtr;
  }

  @SuppressWarnings("deprecation")
  protected void finalize() {
    delete();
  }

  public synchronized void delete() {
    if (swigCPtr != 0) {
      if (swigCMemOwn) {
        swigCMemOwn = false;
        RobotRaconteurJavaJNI.delete_EnumDefinitionValue(swigCPtr);
      }
      swigCPtr = 0;
    }
  }

  public EnumDefinitionValue() {
    this(RobotRaconteurJavaJNI.new_EnumDefinitionValue(), true);
  }

  public void setName(String value) {
    RobotRaconteurJavaJNI.EnumDefinitionValue_Name_set(swigCPtr, this, value);
  }

  public String getName() {
    return RobotRaconteurJavaJNI.EnumDefinitionValue_Name_get(swigCPtr, this);
  }

  public void setDocString(String value) {
    RobotRaconteurJavaJNI.EnumDefinitionValue_DocString_set(swigCPtr, this, value);
  }

  public String getDocString() {
    return RobotRaconteurJavaJNI.EnumDefinitionValue_DocString_get(swigCPtr, this);
  }

  public void setValue(int value) {
    RobotRaconteurJavaJNI.EnumDefinitionValue_Value_set(swigCPtr, this, value);
  }

  public int getValue() {
    return RobotRaconteurJavaJNI.EnumDefinitionValue_Value_get(swigCPtr, this);
  }

  public void setImplicitValue(boolean value) {
    RobotRaconteurJavaJNI.EnumDefinitionValue_ImplicitValue_set(swigCPtr, this, value);
  }

  public boolean getImplicitValue() {
    return RobotRaconteurJavaJNI.EnumDefinitionValue_ImplicitValue_get(swigCPtr, this);
  }

  public void setHexValue(boolean value) {
    RobotRaconteurJavaJNI.EnumDefinitionValue_HexValue_set(swigCPtr, this, value);
  }

  public boolean getHexValue() {
    return RobotRaconteurJavaJNI.EnumDefinitionValue_HexValue_get(swigCPtr, this);
  }

}
