/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 4.0.2
 *
 * This file is not intended to be easily readable and contains a number of
 * coding conventions designed to improve portability and efficiency. Do not make
 * changes to this file unless you know what you are doing--modify the SWIG
 * interface file instead.
 * ----------------------------------------------------------------------------- */


#ifndef SWIGPYTHON
#define SWIGPYTHON
#endif

#define SWIG_DIRECTORS
#define SWIG_PYTHON_THREADS
#define SWIG_PYTHON_DIRECTOR_NO_VTABLE


#ifdef __cplusplus
/* SwigValueWrapper is described in swig.swg */
template<typename T> class SwigValueWrapper {
  struct SwigMovePointer {
    T *ptr;
    SwigMovePointer(T *p) : ptr(p) { }
    ~SwigMovePointer() { delete ptr; }
    SwigMovePointer& operator=(SwigMovePointer& rhs) { T* oldptr = ptr; ptr = 0; delete oldptr; ptr = rhs.ptr; rhs.ptr = 0; return *this; }
  } pointer;
  SwigValueWrapper& operator=(const SwigValueWrapper<T>& rhs);
  SwigValueWrapper(const SwigValueWrapper<T>& rhs);
public:
  SwigValueWrapper() : pointer(0) { }
  SwigValueWrapper& operator=(const T& t) { SwigMovePointer tmp(new T(t)); pointer = tmp; return *this; }
  operator T&() const { return *pointer.ptr; }
  T *operator&() { return pointer.ptr; }
};

template <typename T> T SwigValueInit() {
  return T();
}
#endif

/* -----------------------------------------------------------------------------
 *  This section contains generic SWIG labels for method/variable
 *  declarations/attributes, and other compiler dependent labels.
 * ----------------------------------------------------------------------------- */

/* template workaround for compilers that cannot correctly implement the C++ standard */
#ifndef SWIGTEMPLATEDISAMBIGUATOR
# if defined(__SUNPRO_CC) && (__SUNPRO_CC <= 0x560)
#  define SWIGTEMPLATEDISAMBIGUATOR template
# elif defined(__HP_aCC)
/* Needed even with `aCC -AA' when `aCC -V' reports HP ANSI C++ B3910B A.03.55 */
/* If we find a maximum version that requires this, the test would be __HP_aCC <= 35500 for A.03.55 */
#  define SWIGTEMPLATEDISAMBIGUATOR template
# else
#  define SWIGTEMPLATEDISAMBIGUATOR
# endif
#endif

/* inline attribute */
#ifndef SWIGINLINE
# if defined(__cplusplus) || (defined(__GNUC__) && !defined(__STRICT_ANSI__))
#   define SWIGINLINE inline
# else
#   define SWIGINLINE
# endif
#endif

/* attribute recognised by some compilers to avoid 'unused' warnings */
#ifndef SWIGUNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define SWIGUNUSED __attribute__ ((__unused__))
#   else
#     define SWIGUNUSED
#   endif
# elif defined(__ICC)
#   define SWIGUNUSED __attribute__ ((__unused__))
# else
#   define SWIGUNUSED
# endif
#endif

#ifndef SWIG_MSC_UNSUPPRESS_4505
# if defined(_MSC_VER)
#   pragma warning(disable : 4505) /* unreferenced local function has been removed */
# endif
#endif

#ifndef SWIGUNUSEDPARM
# ifdef __cplusplus
#   define SWIGUNUSEDPARM(p)
# else
#   define SWIGUNUSEDPARM(p) p SWIGUNUSED
# endif
#endif

/* internal SWIG method */
#ifndef SWIGINTERN
# define SWIGINTERN static SWIGUNUSED
#endif

/* internal inline SWIG method */
#ifndef SWIGINTERNINLINE
# define SWIGINTERNINLINE SWIGINTERN SWIGINLINE
#endif

/* exporting methods */
#if defined(__GNUC__)
#  if (__GNUC__ >= 4) || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4)
#    ifndef GCC_HASCLASSVISIBILITY
#      define GCC_HASCLASSVISIBILITY
#    endif
#  endif
#endif

#ifndef SWIGEXPORT
# if defined(_WIN32) || defined(__WIN32__) || defined(__CYGWIN__)
#   if defined(STATIC_LINKED)
#     define SWIGEXPORT
#   else
#     define SWIGEXPORT __declspec(dllexport)
#   endif
# else
#   if defined(__GNUC__) && defined(GCC_HASCLASSVISIBILITY)
#     define SWIGEXPORT __attribute__ ((visibility("default")))
#   else
#     define SWIGEXPORT
#   endif
# endif
#endif

/* calling conventions for Windows */
#ifndef SWIGSTDCALL
# if defined(_WIN32) || defined(__WIN32__) || defined(__CYGWIN__)
#   define SWIGSTDCALL __stdcall
# else
#   define SWIGSTDCALL
# endif
#endif

/* Deal with Microsoft's attempt at deprecating C standard runtime functions */
#if !defined(SWIG_NO_CRT_SECURE_NO_DEPRECATE) && defined(_MSC_VER) && !defined(_CRT_SECURE_NO_DEPRECATE)
# define _CRT_SECURE_NO_DEPRECATE
#endif

/* Deal with Microsoft's attempt at deprecating methods in the standard C++ library */
#if !defined(SWIG_NO_SCL_SECURE_NO_DEPRECATE) && defined(_MSC_VER) && !defined(_SCL_SECURE_NO_DEPRECATE)
# define _SCL_SECURE_NO_DEPRECATE
#endif

/* Deal with Apple's deprecated 'AssertMacros.h' from Carbon-framework */
#if defined(__APPLE__) && !defined(__ASSERT_MACROS_DEFINE_VERSIONS_WITHOUT_UNDERSCORES)
# define __ASSERT_MACROS_DEFINE_VERSIONS_WITHOUT_UNDERSCORES 0
#endif

/* Intel's compiler complains if a variable which was never initialised is
 * cast to void, which is a common idiom which we use to indicate that we
 * are aware a variable isn't used.  So we just silence that warning.
 * See: https://github.com/swig/swig/issues/192 for more discussion.
 */
#ifdef __INTEL_COMPILER
# pragma warning disable 592
#endif


#if defined(__GNUC__) && defined(_WIN32) && !defined(SWIG_PYTHON_NO_HYPOT_WORKAROUND)
/* Workaround for '::hypot' has not been declared', see https://bugs.python.org/issue11566 */
# include <math.h>
#endif

#if defined(_DEBUG) && defined(SWIG_PYTHON_INTERPRETER_NO_DEBUG)
/* Use debug wrappers with the Python release dll */
# undef _DEBUG
# include <Python.h>
# define _DEBUG 1
#else
# include <Python.h>
#endif

/* -----------------------------------------------------------------------------
 * swigrun.swg
 *
 * This file contains generic C API SWIG runtime support for pointer
 * type checking.
 * ----------------------------------------------------------------------------- */

/* This should only be incremented when either the layout of swig_type_info changes,
   or for whatever reason, the runtime changes incompatibly */
#define SWIG_RUNTIME_VERSION "4"

/* define SWIG_TYPE_TABLE_NAME as "SWIG_TYPE_TABLE" */
#ifdef SWIG_TYPE_TABLE
# define SWIG_QUOTE_STRING(x) #x
# define SWIG_EXPAND_AND_QUOTE_STRING(x) SWIG_QUOTE_STRING(x)
# define SWIG_TYPE_TABLE_NAME SWIG_EXPAND_AND_QUOTE_STRING(SWIG_TYPE_TABLE)
#else
# define SWIG_TYPE_TABLE_NAME
#endif

/*
  You can use the SWIGRUNTIME and SWIGRUNTIMEINLINE macros for
  creating a static or dynamic library from the SWIG runtime code.
  In 99.9% of the cases, SWIG just needs to declare them as 'static'.

  But only do this if strictly necessary, ie, if you have problems
  with your compiler or suchlike.
*/

#ifndef SWIGRUNTIME
# define SWIGRUNTIME SWIGINTERN
#endif

#ifndef SWIGRUNTIMEINLINE
# define SWIGRUNTIMEINLINE SWIGRUNTIME SWIGINLINE
#endif

/*  Generic buffer size */
#ifndef SWIG_BUFFER_SIZE
# define SWIG_BUFFER_SIZE 1024
#endif

/* Flags for pointer conversions */
#define SWIG_POINTER_DISOWN        0x1
#define SWIG_CAST_NEW_MEMORY       0x2
#define SWIG_POINTER_NO_NULL       0x4

/* Flags for new pointer objects */
#define SWIG_POINTER_OWN           0x1


/*
   Flags/methods for returning states.

   The SWIG conversion methods, as ConvertPtr, return an integer
   that tells if the conversion was successful or not. And if not,
   an error code can be returned (see swigerrors.swg for the codes).

   Use the following macros/flags to set or process the returning
   states.

   In old versions of SWIG, code such as the following was usually written:

     if (SWIG_ConvertPtr(obj,vptr,ty.flags) != -1) {
       // success code
     } else {
       //fail code
     }

   Now you can be more explicit:

    int res = SWIG_ConvertPtr(obj,vptr,ty.flags);
    if (SWIG_IsOK(res)) {
      // success code
    } else {
      // fail code
    }

   which is the same really, but now you can also do

    Type *ptr;
    int res = SWIG_ConvertPtr(obj,(void **)(&ptr),ty.flags);
    if (SWIG_IsOK(res)) {
      // success code
      if (SWIG_IsNewObj(res) {
        ...
	delete *ptr;
      } else {
        ...
      }
    } else {
      // fail code
    }

   I.e., now SWIG_ConvertPtr can return new objects and you can
   identify the case and take care of the deallocation. Of course that
   also requires SWIG_ConvertPtr to return new result values, such as

      int SWIG_ConvertPtr(obj, ptr,...) {
        if (<obj is ok>) {
          if (<need new object>) {
            *ptr = <ptr to new allocated object>;
            return SWIG_NEWOBJ;
          } else {
            *ptr = <ptr to old object>;
            return SWIG_OLDOBJ;
          }
        } else {
          return SWIG_BADOBJ;
        }
      }

   Of course, returning the plain '0(success)/-1(fail)' still works, but you can be
   more explicit by returning SWIG_BADOBJ, SWIG_ERROR or any of the
   SWIG errors code.

   Finally, if the SWIG_CASTRANK_MODE is enabled, the result code
   allows to return the 'cast rank', for example, if you have this

       int food(double)
       int fooi(int);

   and you call

      food(1)   // cast rank '1'  (1 -> 1.0)
      fooi(1)   // cast rank '0'

   just use the SWIG_AddCast()/SWIG_CheckState()
*/

#define SWIG_OK                    (0)
#define SWIG_ERROR                 (-1)
#define SWIG_IsOK(r)               (r >= 0)
#define SWIG_ArgError(r)           ((r != SWIG_ERROR) ? r : SWIG_TypeError)

/* The CastRankLimit says how many bits are used for the cast rank */
#define SWIG_CASTRANKLIMIT         (1 << 8)
/* The NewMask denotes the object was created (using new/malloc) */
#define SWIG_NEWOBJMASK            (SWIG_CASTRANKLIMIT  << 1)
/* The TmpMask is for in/out typemaps that use temporal objects */
#define SWIG_TMPOBJMASK            (SWIG_NEWOBJMASK << 1)
/* Simple returning values */
#define SWIG_BADOBJ                (SWIG_ERROR)
#define SWIG_OLDOBJ                (SWIG_OK)
#define SWIG_NEWOBJ                (SWIG_OK | SWIG_NEWOBJMASK)
#define SWIG_TMPOBJ                (SWIG_OK | SWIG_TMPOBJMASK)
/* Check, add and del mask methods */
#define SWIG_AddNewMask(r)         (SWIG_IsOK(r) ? (r | SWIG_NEWOBJMASK) : r)
#define SWIG_DelNewMask(r)         (SWIG_IsOK(r) ? (r & ~SWIG_NEWOBJMASK) : r)
#define SWIG_IsNewObj(r)           (SWIG_IsOK(r) && (r & SWIG_NEWOBJMASK))
#define SWIG_AddTmpMask(r)         (SWIG_IsOK(r) ? (r | SWIG_TMPOBJMASK) : r)
#define SWIG_DelTmpMask(r)         (SWIG_IsOK(r) ? (r & ~SWIG_TMPOBJMASK) : r)
#define SWIG_IsTmpObj(r)           (SWIG_IsOK(r) && (r & SWIG_TMPOBJMASK))

/* Cast-Rank Mode */
#if defined(SWIG_CASTRANK_MODE)
#  ifndef SWIG_TypeRank
#    define SWIG_TypeRank             unsigned long
#  endif
#  ifndef SWIG_MAXCASTRANK            /* Default cast allowed */
#    define SWIG_MAXCASTRANK          (2)
#  endif
#  define SWIG_CASTRANKMASK          ((SWIG_CASTRANKLIMIT) -1)
#  define SWIG_CastRank(r)           (r & SWIG_CASTRANKMASK)
SWIGINTERNINLINE int SWIG_AddCast(int r) {
  return SWIG_IsOK(r) ? ((SWIG_CastRank(r) < SWIG_MAXCASTRANK) ? (r + 1) : SWIG_ERROR) : r;
}
SWIGINTERNINLINE int SWIG_CheckState(int r) {
  return SWIG_IsOK(r) ? SWIG_CastRank(r) + 1 : 0;
}
#else /* no cast-rank mode */
#  define SWIG_AddCast(r) (r)
#  define SWIG_CheckState(r) (SWIG_IsOK(r) ? 1 : 0)
#endif


#include <string.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef void *(*swig_converter_func)(void *, int *);
typedef struct swig_type_info *(*swig_dycast_func)(void **);

/* Structure to store information on one type */
typedef struct swig_type_info {
  const char             *name;			/* mangled name of this type */
  const char             *str;			/* human readable name of this type */
  swig_dycast_func        dcast;		/* dynamic cast function down a hierarchy */
  struct swig_cast_info  *cast;			/* linked list of types that can cast into this type */
  void                   *clientdata;		/* language specific type data */
  int                    owndata;		/* flag if the structure owns the clientdata */
} swig_type_info;

/* Structure to store a type and conversion function used for casting */
typedef struct swig_cast_info {
  swig_type_info         *type;			/* pointer to type that is equivalent to this type */
  swig_converter_func     converter;		/* function to cast the void pointers */
  struct swig_cast_info  *next;			/* pointer to next cast in linked list */
  struct swig_cast_info  *prev;			/* pointer to the previous cast */
} swig_cast_info;

/* Structure used to store module information
 * Each module generates one structure like this, and the runtime collects
 * all of these structures and stores them in a circularly linked list.*/
typedef struct swig_module_info {
  swig_type_info         **types;		/* Array of pointers to swig_type_info structures that are in this module */
  size_t                 size;		        /* Number of types in this module */
  struct swig_module_info *next;		/* Pointer to next element in circularly linked list */
  swig_type_info         **type_initial;	/* Array of initially generated type structures */
  swig_cast_info         **cast_initial;	/* Array of initially generated casting structures */
  void                    *clientdata;		/* Language specific module data */
} swig_module_info;

/*
  Compare two type names skipping the space characters, therefore
  "char*" == "char *" and "Class<int>" == "Class<int >", etc.

  Return 0 when the two name types are equivalent, as in
  strncmp, but skipping ' '.
*/
SWIGRUNTIME int
SWIG_TypeNameComp(const char *f1, const char *l1,
		  const char *f2, const char *l2) {
  for (;(f1 != l1) && (f2 != l2); ++f1, ++f2) {
    while ((*f1 == ' ') && (f1 != l1)) ++f1;
    while ((*f2 == ' ') && (f2 != l2)) ++f2;
    if (*f1 != *f2) return (*f1 > *f2) ? 1 : -1;
  }
  return (int)((l1 - f1) - (l2 - f2));
}

/*
  Check type equivalence in a name list like <name1>|<name2>|...
  Return 0 if equal, -1 if nb < tb, 1 if nb > tb
*/
SWIGRUNTIME int
SWIG_TypeCmp(const char *nb, const char *tb) {
  int equiv = 1;
  const char* te = tb + strlen(tb);
  const char* ne = nb;
  while (equiv != 0 && *ne) {
    for (nb = ne; *ne; ++ne) {
      if (*ne == '|') break;
    }
    equiv = SWIG_TypeNameComp(nb, ne, tb, te);
    if (*ne) ++ne;
  }
  return equiv;
}

/*
  Check type equivalence in a name list like <name1>|<name2>|...
  Return 0 if not equal, 1 if equal
*/
SWIGRUNTIME int
SWIG_TypeEquiv(const char *nb, const char *tb) {
  return SWIG_TypeCmp(nb, tb) == 0 ? 1 : 0;
}

/*
  Check the typename
*/
SWIGRUNTIME swig_cast_info *
SWIG_TypeCheck(const char *c, swig_type_info *ty) {
  if (ty) {
    swig_cast_info *iter = ty->cast;
    while (iter) {
      if (strcmp(iter->type->name, c) == 0) {
        if (iter == ty->cast)
          return iter;
        /* Move iter to the top of the linked list */
        iter->prev->next = iter->next;
        if (iter->next)
          iter->next->prev = iter->prev;
        iter->next = ty->cast;
        iter->prev = 0;
        if (ty->cast) ty->cast->prev = iter;
        ty->cast = iter;
        return iter;
      }
      iter = iter->next;
    }
  }
  return 0;
}

/*
  Identical to SWIG_TypeCheck, except strcmp is replaced with a pointer comparison
*/
SWIGRUNTIME swig_cast_info *
SWIG_TypeCheckStruct(swig_type_info *from, swig_type_info *ty) {
  if (ty) {
    swig_cast_info *iter = ty->cast;
    while (iter) {
      if (iter->type == from) {
        if (iter == ty->cast)
          return iter;
        /* Move iter to the top of the linked list */
        iter->prev->next = iter->next;
        if (iter->next)
          iter->next->prev = iter->prev;
        iter->next = ty->cast;
        iter->prev = 0;
        if (ty->cast) ty->cast->prev = iter;
        ty->cast = iter;
        return iter;
      }
      iter = iter->next;
    }
  }
  return 0;
}

/*
  Cast a pointer up an inheritance hierarchy
*/
SWIGRUNTIMEINLINE void *
SWIG_TypeCast(swig_cast_info *ty, void *ptr, int *newmemory) {
  return ((!ty) || (!ty->converter)) ? ptr : (*ty->converter)(ptr, newmemory);
}

/*
   Dynamic pointer casting. Down an inheritance hierarchy
*/
SWIGRUNTIME swig_type_info *
SWIG_TypeDynamicCast(swig_type_info *ty, void **ptr) {
  swig_type_info *lastty = ty;
  if (!ty || !ty->dcast) return ty;
  while (ty && (ty->dcast)) {
    ty = (*ty->dcast)(ptr);
    if (ty) lastty = ty;
  }
  return lastty;
}

/*
  Return the name associated with this type
*/
SWIGRUNTIMEINLINE const char *
SWIG_TypeName(const swig_type_info *ty) {
  return ty->name;
}

/*
  Return the pretty name associated with this type,
  that is an unmangled type name in a form presentable to the user.
*/
SWIGRUNTIME const char *
SWIG_TypePrettyName(const swig_type_info *type) {
  /* The "str" field contains the equivalent pretty names of the
     type, separated by vertical-bar characters.  We choose
     to print the last name, as it is often (?) the most
     specific. */
  if (!type) return NULL;
  if (type->str != NULL) {
    const char *last_name = type->str;
    const char *s;
    for (s = type->str; *s; s++)
      if (*s == '|') last_name = s+1;
    return last_name;
  }
  else
    return type->name;
}

/*
   Set the clientdata field for a type
*/
SWIGRUNTIME void
SWIG_TypeClientData(swig_type_info *ti, void *clientdata) {
  swig_cast_info *cast = ti->cast;
  /* if (ti->clientdata == clientdata) return; */
  ti->clientdata = clientdata;

  while (cast) {
    if (!cast->converter) {
      swig_type_info *tc = cast->type;
      if (!tc->clientdata) {
	SWIG_TypeClientData(tc, clientdata);
      }
    }
    cast = cast->next;
  }
}
SWIGRUNTIME void
SWIG_TypeNewClientData(swig_type_info *ti, void *clientdata) {
  SWIG_TypeClientData(ti, clientdata);
  ti->owndata = 1;
}

/*
  Search for a swig_type_info structure only by mangled name
  Search is a O(log #types)

  We start searching at module start, and finish searching when start == end.
  Note: if start == end at the beginning of the function, we go all the way around
  the circular list.
*/
SWIGRUNTIME swig_type_info *
SWIG_MangledTypeQueryModule(swig_module_info *start,
                            swig_module_info *end,
		            const char *name) {
  swig_module_info *iter = start;
  do {
    if (iter->size) {
      size_t l = 0;
      size_t r = iter->size - 1;
      do {
	/* since l+r >= 0, we can (>> 1) instead (/ 2) */
	size_t i = (l + r) >> 1;
	const char *iname = iter->types[i]->name;
	if (iname) {
	  int compare = strcmp(name, iname);
	  if (compare == 0) {
	    return iter->types[i];
	  } else if (compare < 0) {
	    if (i) {
	      r = i - 1;
	    } else {
	      break;
	    }
	  } else if (compare > 0) {
	    l = i + 1;
	  }
	} else {
	  break; /* should never happen */
	}
      } while (l <= r);
    }
    iter = iter->next;
  } while (iter != end);
  return 0;
}

/*
  Search for a swig_type_info structure for either a mangled name or a human readable name.
  It first searches the mangled names of the types, which is a O(log #types)
  If a type is not found it then searches the human readable names, which is O(#types).

  We start searching at module start, and finish searching when start == end.
  Note: if start == end at the beginning of the function, we go all the way around
  the circular list.
*/
SWIGRUNTIME swig_type_info *
SWIG_TypeQueryModule(swig_module_info *start,
                     swig_module_info *end,
		     const char *name) {
  /* STEP 1: Search the name field using binary search */
  swig_type_info *ret = SWIG_MangledTypeQueryModule(start, end, name);
  if (ret) {
    return ret;
  } else {
    /* STEP 2: If the type hasn't been found, do a complete search
       of the str field (the human readable name) */
    swig_module_info *iter = start;
    do {
      size_t i = 0;
      for (; i < iter->size; ++i) {
	if (iter->types[i]->str && (SWIG_TypeEquiv(iter->types[i]->str, name)))
	  return iter->types[i];
      }
      iter = iter->next;
    } while (iter != end);
  }

  /* neither found a match */
  return 0;
}

/*
   Pack binary data into a string
*/
SWIGRUNTIME char *
SWIG_PackData(char *c, void *ptr, size_t sz) {
  static const char hex[17] = "0123456789abcdef";
  const unsigned char *u = (unsigned char *) ptr;
  const unsigned char *eu =  u + sz;
  for (; u != eu; ++u) {
    unsigned char uu = *u;
    *(c++) = hex[(uu & 0xf0) >> 4];
    *(c++) = hex[uu & 0xf];
  }
  return c;
}

/*
   Unpack binary data from a string
*/
SWIGRUNTIME const char *
SWIG_UnpackData(const char *c, void *ptr, size_t sz) {
  unsigned char *u = (unsigned char *) ptr;
  const unsigned char *eu = u + sz;
  for (; u != eu; ++u) {
    char d = *(c++);
    unsigned char uu;
    if ((d >= '0') && (d <= '9'))
      uu = (unsigned char)((d - '0') << 4);
    else if ((d >= 'a') && (d <= 'f'))
      uu = (unsigned char)((d - ('a'-10)) << 4);
    else
      return (char *) 0;
    d = *(c++);
    if ((d >= '0') && (d <= '9'))
      uu |= (unsigned char)(d - '0');
    else if ((d >= 'a') && (d <= 'f'))
      uu |= (unsigned char)(d - ('a'-10));
    else
      return (char *) 0;
    *u = uu;
  }
  return c;
}

/*
   Pack 'void *' into a string buffer.
*/
SWIGRUNTIME char *
SWIG_PackVoidPtr(char *buff, void *ptr, const char *name, size_t bsz) {
  char *r = buff;
  if ((2*sizeof(void *) + 2) > bsz) return 0;
  *(r++) = '_';
  r = SWIG_PackData(r,&ptr,sizeof(void *));
  if (strlen(name) + 1 > (bsz - (r - buff))) return 0;
  strcpy(r,name);
  return buff;
}

SWIGRUNTIME const char *
SWIG_UnpackVoidPtr(const char *c, void **ptr, const char *name) {
  if (*c != '_') {
    if (strcmp(c,"NULL") == 0) {
      *ptr = (void *) 0;
      return name;
    } else {
      return 0;
    }
  }
  return SWIG_UnpackData(++c,ptr,sizeof(void *));
}

SWIGRUNTIME char *
SWIG_PackDataName(char *buff, void *ptr, size_t sz, const char *name, size_t bsz) {
  char *r = buff;
  size_t lname = (name ? strlen(name) : 0);
  if ((2*sz + 2 + lname) > bsz) return 0;
  *(r++) = '_';
  r = SWIG_PackData(r,ptr,sz);
  if (lname) {
    strncpy(r,name,lname+1);
  } else {
    *r = 0;
  }
  return buff;
}

SWIGRUNTIME const char *
SWIG_UnpackDataName(const char *c, void *ptr, size_t sz, const char *name) {
  if (*c != '_') {
    if (strcmp(c,"NULL") == 0) {
      memset(ptr,0,sz);
      return name;
    } else {
      return 0;
    }
  }
  return SWIG_UnpackData(++c,ptr,sz);
}

#ifdef __cplusplus
}
#endif

/*  Errors in SWIG */
#define  SWIG_UnknownError    	   -1
#define  SWIG_IOError        	   -2
#define  SWIG_RuntimeError   	   -3
#define  SWIG_IndexError     	   -4
#define  SWIG_TypeError      	   -5
#define  SWIG_DivisionByZero 	   -6
#define  SWIG_OverflowError  	   -7
#define  SWIG_SyntaxError    	   -8
#define  SWIG_ValueError     	   -9
#define  SWIG_SystemError    	   -10
#define  SWIG_AttributeError 	   -11
#define  SWIG_MemoryError    	   -12
#define  SWIG_NullReferenceError   -13



/* Compatibility macros for Python 3 */
#if PY_VERSION_HEX >= 0x03000000

#define PyClass_Check(obj) PyObject_IsInstance(obj, (PyObject *)&PyType_Type)
#define PyInt_Check(x) PyLong_Check(x)
#define PyInt_AsLong(x) PyLong_AsLong(x)
#define PyInt_FromLong(x) PyLong_FromLong(x)
#define PyInt_FromSize_t(x) PyLong_FromSize_t(x)
#define PyString_Check(name) PyBytes_Check(name)
#define PyString_FromString(x) PyUnicode_FromString(x)
#define PyString_Format(fmt, args)  PyUnicode_Format(fmt, args)
#define PyString_AsString(str) PyBytes_AsString(str)
#define PyString_Size(str) PyBytes_Size(str)	
#define PyString_InternFromString(key) PyUnicode_InternFromString(key)
#define Py_TPFLAGS_HAVE_CLASS Py_TPFLAGS_BASETYPE
#define PyString_AS_STRING(x) PyUnicode_AS_STRING(x)
#define _PyLong_FromSsize_t(x) PyLong_FromSsize_t(x)

#endif

#ifndef Py_TYPE
#  define Py_TYPE(op) ((op)->ob_type)
#endif

/* SWIG APIs for compatibility of both Python 2 & 3 */

#if PY_VERSION_HEX >= 0x03000000
#  define SWIG_Python_str_FromFormat PyUnicode_FromFormat
#else
#  define SWIG_Python_str_FromFormat PyString_FromFormat
#endif


/* Warning: This function will allocate a new string in Python 3,
 * so please call SWIG_Python_str_DelForPy3(x) to free the space.
 */
SWIGINTERN char*
SWIG_Python_str_AsChar(PyObject *str)
{
#if PY_VERSION_HEX >= 0x03030000
  return (char *)PyUnicode_AsUTF8(str);
#elif PY_VERSION_HEX >= 0x03000000
  char *newstr = 0;
  str = PyUnicode_AsUTF8String(str);
  if (str) {
    char *cstr;
    Py_ssize_t len;
    if (PyBytes_AsStringAndSize(str, &cstr, &len) != -1) {
      newstr = (char *) malloc(len+1);
      if (newstr)
        memcpy(newstr, cstr, len+1);
    }
    Py_XDECREF(str);
  }
  return newstr;
#else
  return PyString_AsString(str);
#endif
}

#if PY_VERSION_HEX >= 0x03030000 || PY_VERSION_HEX < 0x03000000
#  define SWIG_Python_str_DelForPy3(x)
#else
#  define SWIG_Python_str_DelForPy3(x) free( (void*) (x) )
#endif


SWIGINTERN PyObject*
SWIG_Python_str_FromChar(const char *c)
{
#if PY_VERSION_HEX >= 0x03000000
  return PyUnicode_FromString(c); 
#else
  return PyString_FromString(c);
#endif
}

#ifndef PyObject_DEL
# define PyObject_DEL PyObject_Del
#endif

// SWIGPY_USE_CAPSULE is no longer used within SWIG itself, but some user
// interface files check for it.
# define SWIGPY_USE_CAPSULE
# define SWIGPY_CAPSULE_NAME ("swig_runtime_data" SWIG_RUNTIME_VERSION ".type_pointer_capsule" SWIG_TYPE_TABLE_NAME)

#if PY_VERSION_HEX < 0x03020000
#define PyDescr_TYPE(x) (((PyDescrObject *)(x))->d_type)
#define PyDescr_NAME(x) (((PyDescrObject *)(x))->d_name)
#define Py_hash_t long
#endif

/* -----------------------------------------------------------------------------
 * error manipulation
 * ----------------------------------------------------------------------------- */

SWIGRUNTIME PyObject*
SWIG_Python_ErrorType(int code) {
  PyObject* type = 0;
  switch(code) {
  case SWIG_MemoryError:
    type = PyExc_MemoryError;
    break;
  case SWIG_IOError:
    type = PyExc_IOError;
    break;
  case SWIG_RuntimeError:
    type = PyExc_RuntimeError;
    break;
  case SWIG_IndexError:
    type = PyExc_IndexError;
    break;
  case SWIG_TypeError:
    type = PyExc_TypeError;
    break;
  case SWIG_DivisionByZero:
    type = PyExc_ZeroDivisionError;
    break;
  case SWIG_OverflowError:
    type = PyExc_OverflowError;
    break;
  case SWIG_SyntaxError:
    type = PyExc_SyntaxError;
    break;
  case SWIG_ValueError:
    type = PyExc_ValueError;
    break;
  case SWIG_SystemError:
    type = PyExc_SystemError;
    break;
  case SWIG_AttributeError:
    type = PyExc_AttributeError;
    break;
  default:
    type = PyExc_RuntimeError;
  }
  return type;
}


SWIGRUNTIME void
SWIG_Python_AddErrorMsg(const char* mesg)
{
  PyObject *type = 0;
  PyObject *value = 0;
  PyObject *traceback = 0;

  if (PyErr_Occurred())
    PyErr_Fetch(&type, &value, &traceback);
  if (value) {
    PyObject *old_str = PyObject_Str(value);
    const char *tmp = SWIG_Python_str_AsChar(old_str);
    PyErr_Clear();
    Py_XINCREF(type);
    if (tmp)
      PyErr_Format(type, "%s %s", tmp, mesg);
    else
      PyErr_Format(type, "%s", mesg);
    SWIG_Python_str_DelForPy3(tmp);
    Py_DECREF(old_str);
    Py_DECREF(value);
  } else {
    PyErr_SetString(PyExc_RuntimeError, mesg);
  }
}

SWIGRUNTIME int
SWIG_Python_TypeErrorOccurred(PyObject *obj)
{
  PyObject *error;
  if (obj)
    return 0;
  error = PyErr_Occurred();
  return error && PyErr_GivenExceptionMatches(error, PyExc_TypeError);
}

SWIGRUNTIME void
SWIG_Python_RaiseOrModifyTypeError(const char *message)
{
  if (SWIG_Python_TypeErrorOccurred(NULL)) {
    /* Use existing TypeError to preserve stacktrace and enhance with given message */
    PyObject *newvalue;
    PyObject *type = NULL, *value = NULL, *traceback = NULL;
    PyErr_Fetch(&type, &value, &traceback);
#if PY_VERSION_HEX >= 0x03000000
    newvalue = PyUnicode_FromFormat("%S\nAdditional information:\n%s", value, message);
#else
    newvalue = PyString_FromFormat("%s\nAdditional information:\n%s", PyString_AsString(value), message);
#endif
    Py_XDECREF(value);
    PyErr_Restore(type, newvalue, traceback);
  } else {
    /* Raise TypeError using given message */
    PyErr_SetString(PyExc_TypeError, message);
  }
}

#if defined(SWIG_PYTHON_NO_THREADS)
#  if defined(SWIG_PYTHON_THREADS)
#    undef SWIG_PYTHON_THREADS
#  endif
#endif
#if defined(SWIG_PYTHON_THREADS) /* Threading support is enabled */
#  if !defined(SWIG_PYTHON_USE_GIL) && !defined(SWIG_PYTHON_NO_USE_GIL)
#    define SWIG_PYTHON_USE_GIL
#  endif
#  if defined(SWIG_PYTHON_USE_GIL) /* Use PyGILState threads calls */
#    ifndef SWIG_PYTHON_INITIALIZE_THREADS
#     define SWIG_PYTHON_INITIALIZE_THREADS  PyEval_InitThreads() 
#    endif
#    ifdef __cplusplus /* C++ code */
       class SWIG_Python_Thread_Block {
         bool status;
         PyGILState_STATE state;
       public:
         void end() { if (status) { PyGILState_Release(state); status = false;} }
         SWIG_Python_Thread_Block() : status(true), state(PyGILState_Ensure()) {}
         ~SWIG_Python_Thread_Block() { end(); }
       };
       class SWIG_Python_Thread_Allow {
         bool status;
         PyThreadState *save;
       public:
         void end() { if (status) { PyEval_RestoreThread(save); status = false; }}
         SWIG_Python_Thread_Allow() : status(true), save(PyEval_SaveThread()) {}
         ~SWIG_Python_Thread_Allow() { end(); }
       };
#      define SWIG_PYTHON_THREAD_BEGIN_BLOCK   SWIG_Python_Thread_Block _swig_thread_block
#      define SWIG_PYTHON_THREAD_END_BLOCK     _swig_thread_block.end()
#      define SWIG_PYTHON_THREAD_BEGIN_ALLOW   SWIG_Python_Thread_Allow _swig_thread_allow
#      define SWIG_PYTHON_THREAD_END_ALLOW     _swig_thread_allow.end()
#    else /* C code */
#      define SWIG_PYTHON_THREAD_BEGIN_BLOCK   PyGILState_STATE _swig_thread_block = PyGILState_Ensure()
#      define SWIG_PYTHON_THREAD_END_BLOCK     PyGILState_Release(_swig_thread_block)
#      define SWIG_PYTHON_THREAD_BEGIN_ALLOW   PyThreadState *_swig_thread_allow = PyEval_SaveThread()
#      define SWIG_PYTHON_THREAD_END_ALLOW     PyEval_RestoreThread(_swig_thread_allow)
#    endif
#  else /* Old thread way, not implemented, user must provide it */
#    if !defined(SWIG_PYTHON_INITIALIZE_THREADS)
#      define SWIG_PYTHON_INITIALIZE_THREADS
#    endif
#    if !defined(SWIG_PYTHON_THREAD_BEGIN_BLOCK)
#      define SWIG_PYTHON_THREAD_BEGIN_BLOCK
#    endif
#    if !defined(SWIG_PYTHON_THREAD_END_BLOCK)
#      define SWIG_PYTHON_THREAD_END_BLOCK
#    endif
#    if !defined(SWIG_PYTHON_THREAD_BEGIN_ALLOW)
#      define SWIG_PYTHON_THREAD_BEGIN_ALLOW
#    endif
#    if !defined(SWIG_PYTHON_THREAD_END_ALLOW)
#      define SWIG_PYTHON_THREAD_END_ALLOW
#    endif
#  endif
#else /* No thread support */
#  define SWIG_PYTHON_INITIALIZE_THREADS
#  define SWIG_PYTHON_THREAD_BEGIN_BLOCK
#  define SWIG_PYTHON_THREAD_END_BLOCK
#  define SWIG_PYTHON_THREAD_BEGIN_ALLOW
#  define SWIG_PYTHON_THREAD_END_ALLOW
#endif

/* -----------------------------------------------------------------------------
 * Python API portion that goes into the runtime
 * ----------------------------------------------------------------------------- */

#ifdef __cplusplus
extern "C" {
#endif

/* -----------------------------------------------------------------------------
 * Constant declarations
 * ----------------------------------------------------------------------------- */

/* Constant Types */
#define SWIG_PY_POINTER 4
#define SWIG_PY_BINARY  5

/* Constant information structure */
typedef struct swig_const_info {
  int type;
  const char *name;
  long lvalue;
  double dvalue;
  void   *pvalue;
  swig_type_info **ptype;
} swig_const_info;

#ifdef __cplusplus
}
#endif


/* -----------------------------------------------------------------------------
 * pyrun.swg
 *
 * This file contains the runtime support for Python modules
 * and includes code for managing global variables and pointer
 * type checking.
 *
 * ----------------------------------------------------------------------------- */

#if PY_VERSION_HEX < 0x02070000 /* 2.7.0 */
# error "This version of SWIG only supports Python >= 2.7"
#endif

#if PY_VERSION_HEX >= 0x03000000 && PY_VERSION_HEX < 0x03020000
# error "This version of SWIG only supports Python 3 >= 3.2"
#endif

/* Common SWIG API */

/* for raw pointers */
#define SWIG_Python_ConvertPtr(obj, pptr, type, flags)  SWIG_Python_ConvertPtrAndOwn(obj, pptr, type, flags, 0)
#define SWIG_ConvertPtr(obj, pptr, type, flags)         SWIG_Python_ConvertPtr(obj, pptr, type, flags)
#define SWIG_ConvertPtrAndOwn(obj,pptr,type,flags,own)  SWIG_Python_ConvertPtrAndOwn(obj, pptr, type, flags, own)

#ifdef SWIGPYTHON_BUILTIN
#define SWIG_NewPointerObj(ptr, type, flags)            SWIG_Python_NewPointerObj(self, ptr, type, flags)
#else
#define SWIG_NewPointerObj(ptr, type, flags)            SWIG_Python_NewPointerObj(NULL, ptr, type, flags)
#endif

#define SWIG_InternalNewPointerObj(ptr, type, flags)	SWIG_Python_NewPointerObj(NULL, ptr, type, flags)

#define SWIG_CheckImplicit(ty)                          SWIG_Python_CheckImplicit(ty) 
#define SWIG_AcquirePtr(ptr, src)                       SWIG_Python_AcquirePtr(ptr, src)
#define swig_owntype                                    int

/* for raw packed data */
#define SWIG_ConvertPacked(obj, ptr, sz, ty)            SWIG_Python_ConvertPacked(obj, ptr, sz, ty)
#define SWIG_NewPackedObj(ptr, sz, type)                SWIG_Python_NewPackedObj(ptr, sz, type)

/* for class or struct pointers */
#define SWIG_ConvertInstance(obj, pptr, type, flags)    SWIG_ConvertPtr(obj, pptr, type, flags)
#define SWIG_NewInstanceObj(ptr, type, flags)           SWIG_NewPointerObj(ptr, type, flags)

/* for C or C++ function pointers */
#define SWIG_ConvertFunctionPtr(obj, pptr, type)        SWIG_Python_ConvertFunctionPtr(obj, pptr, type)
#define SWIG_NewFunctionPtrObj(ptr, type)               SWIG_Python_NewPointerObj(NULL, ptr, type, 0)

/* for C++ member pointers, ie, member methods */
#define SWIG_ConvertMember(obj, ptr, sz, ty)            SWIG_Python_ConvertPacked(obj, ptr, sz, ty)
#define SWIG_NewMemberObj(ptr, sz, type)                SWIG_Python_NewPackedObj(ptr, sz, type)


/* Runtime API */

#define SWIG_GetModule(clientdata)                      SWIG_Python_GetModule(clientdata)
#define SWIG_SetModule(clientdata, pointer)             SWIG_Python_SetModule(pointer)
#define SWIG_NewClientData(obj)                         SwigPyClientData_New(obj)

#define SWIG_SetErrorObj                                SWIG_Python_SetErrorObj                            
#define SWIG_SetErrorMsg                        	SWIG_Python_SetErrorMsg				   
#define SWIG_ErrorType(code)                    	SWIG_Python_ErrorType(code)                        
#define SWIG_Error(code, msg)            		SWIG_Python_SetErrorMsg(SWIG_ErrorType(code), msg) 
#define SWIG_fail                        		goto fail					   


/* Runtime API implementation */

/* Error manipulation */

SWIGINTERN void 
SWIG_Python_SetErrorObj(PyObject *errtype, PyObject *obj) {
  SWIG_PYTHON_THREAD_BEGIN_BLOCK; 
  PyErr_SetObject(errtype, obj);
  Py_DECREF(obj);
  SWIG_PYTHON_THREAD_END_BLOCK;
}

SWIGINTERN void 
SWIG_Python_SetErrorMsg(PyObject *errtype, const char *msg) {
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  PyErr_SetString(errtype, msg);
  SWIG_PYTHON_THREAD_END_BLOCK;
}

#define SWIG_Python_Raise(obj, type, desc)  SWIG_Python_SetErrorObj(SWIG_Python_ExceptionType(desc), obj)

/* Set a constant value */

#if defined(SWIGPYTHON_BUILTIN)

SWIGINTERN void
SwigPyBuiltin_AddPublicSymbol(PyObject *seq, const char *key) {
  PyObject *s = PyString_InternFromString(key);
  PyList_Append(seq, s);
  Py_DECREF(s);
}

SWIGINTERN void
SWIG_Python_SetConstant(PyObject *d, PyObject *public_interface, const char *name, PyObject *obj) {   
  PyDict_SetItemString(d, name, obj);
  Py_DECREF(obj);
  if (public_interface)
    SwigPyBuiltin_AddPublicSymbol(public_interface, name);
}

#else

SWIGINTERN void
SWIG_Python_SetConstant(PyObject *d, const char *name, PyObject *obj) {   
  PyDict_SetItemString(d, name, obj);
  Py_DECREF(obj);                            
}

#endif

/* Append a value to the result obj */

SWIGINTERN PyObject*
SWIG_Python_AppendOutput(PyObject* result, PyObject* obj) {
  if (!result) {
    result = obj;
  } else if (result == Py_None) {
    Py_DECREF(result);
    result = obj;
  } else {
    if (!PyList_Check(result)) {
      PyObject *o2 = result;
      result = PyList_New(1);
      PyList_SetItem(result, 0, o2);
    }
    PyList_Append(result,obj);
    Py_DECREF(obj);
  }
  return result;
}

/* Unpack the argument tuple */

SWIGINTERN Py_ssize_t
SWIG_Python_UnpackTuple(PyObject *args, const char *name, Py_ssize_t min, Py_ssize_t max, PyObject **objs)
{
  if (!args) {
    if (!min && !max) {
      return 1;
    } else {
      PyErr_Format(PyExc_TypeError, "%s expected %s%d arguments, got none", 
		   name, (min == max ? "" : "at least "), (int)min);
      return 0;
    }
  }  
  if (!PyTuple_Check(args)) {
    if (min <= 1 && max >= 1) {
      Py_ssize_t i;
      objs[0] = args;
      for (i = 1; i < max; ++i) {
	objs[i] = 0;
      }
      return 2;
    }
    PyErr_SetString(PyExc_SystemError, "UnpackTuple() argument list is not a tuple");
    return 0;
  } else {
    Py_ssize_t l = PyTuple_GET_SIZE(args);
    if (l < min) {
      PyErr_Format(PyExc_TypeError, "%s expected %s%d arguments, got %d", 
		   name, (min == max ? "" : "at least "), (int)min, (int)l);
      return 0;
    } else if (l > max) {
      PyErr_Format(PyExc_TypeError, "%s expected %s%d arguments, got %d", 
		   name, (min == max ? "" : "at most "), (int)max, (int)l);
      return 0;
    } else {
      Py_ssize_t i;
      for (i = 0; i < l; ++i) {
	objs[i] = PyTuple_GET_ITEM(args, i);
      }
      for (; l < max; ++l) {
	objs[l] = 0;
      }
      return i + 1;
    }    
  }
}

SWIGINTERN int
SWIG_Python_CheckNoKeywords(PyObject *kwargs, const char *name) {
  int no_kwargs = 1;
  if (kwargs) {
    assert(PyDict_Check(kwargs));
    if (PyDict_Size(kwargs) > 0) {
      PyErr_Format(PyExc_TypeError, "%s() does not take keyword arguments", name);
      no_kwargs = 0;
    }
  }
  return no_kwargs;
}

/* A functor is a function object with one single object argument */
#define SWIG_Python_CallFunctor(functor, obj)	        PyObject_CallFunctionObjArgs(functor, obj, NULL);

/*
  Helper for static pointer initialization for both C and C++ code, for example
  static PyObject *SWIG_STATIC_POINTER(MyVar) = NewSomething(...);
*/
#ifdef __cplusplus
#define SWIG_STATIC_POINTER(var)  var
#else
#define SWIG_STATIC_POINTER(var)  var = 0; if (!var) var
#endif

/* -----------------------------------------------------------------------------
 * Pointer declarations
 * ----------------------------------------------------------------------------- */

/* Flags for new pointer objects */
#define SWIG_POINTER_NOSHADOW       (SWIG_POINTER_OWN      << 1)
#define SWIG_POINTER_NEW            (SWIG_POINTER_NOSHADOW | SWIG_POINTER_OWN)

#define SWIG_POINTER_IMPLICIT_CONV  (SWIG_POINTER_DISOWN   << 1)

#define SWIG_BUILTIN_TP_INIT	    (SWIG_POINTER_OWN << 2)
#define SWIG_BUILTIN_INIT	    (SWIG_BUILTIN_TP_INIT | SWIG_POINTER_OWN)

#ifdef __cplusplus
extern "C" {
#endif

/* The python void return value */

SWIGRUNTIMEINLINE PyObject * 
SWIG_Py_Void(void)
{
  PyObject *none = Py_None;
  Py_INCREF(none);
  return none;
}

/* SwigPyClientData */

typedef struct {
  PyObject *klass;
  PyObject *newraw;
  PyObject *newargs;
  PyObject *destroy;
  int delargs;
  int implicitconv;
  PyTypeObject *pytype;
} SwigPyClientData;

SWIGRUNTIMEINLINE int 
SWIG_Python_CheckImplicit(swig_type_info *ty)
{
  SwigPyClientData *data = (SwigPyClientData *)ty->clientdata;
  int fail = data ? data->implicitconv : 0;
  if (fail)
    PyErr_SetString(PyExc_TypeError, "Implicit conversion is prohibited for explicit constructors.");
  return fail;
}

SWIGRUNTIMEINLINE PyObject *
SWIG_Python_ExceptionType(swig_type_info *desc) {
  SwigPyClientData *data = desc ? (SwigPyClientData *) desc->clientdata : 0;
  PyObject *klass = data ? data->klass : 0;
  return (klass ? klass : PyExc_RuntimeError);
}


SWIGRUNTIME SwigPyClientData * 
SwigPyClientData_New(PyObject* obj)
{
  if (!obj) {
    return 0;
  } else {
    SwigPyClientData *data = (SwigPyClientData *)malloc(sizeof(SwigPyClientData));
    /* the klass element */
    data->klass = obj;
    Py_INCREF(data->klass);
    /* the newraw method and newargs arguments used to create a new raw instance */
    if (PyClass_Check(obj)) {
      data->newraw = 0;
      data->newargs = obj;
      Py_INCREF(obj);
    } else {
      data->newraw = PyObject_GetAttrString(data->klass, "__new__");
      if (data->newraw) {
	Py_INCREF(data->newraw);
	data->newargs = PyTuple_New(1);
	PyTuple_SetItem(data->newargs, 0, obj);
      } else {
	data->newargs = obj;
      }
      Py_INCREF(data->newargs);
    }
    /* the destroy method, aka as the C++ delete method */
    data->destroy = PyObject_GetAttrString(data->klass, "__swig_destroy__");
    if (PyErr_Occurred()) {
      PyErr_Clear();
      data->destroy = 0;
    }
    if (data->destroy) {
      int flags;
      Py_INCREF(data->destroy);
      flags = PyCFunction_GET_FLAGS(data->destroy);
      data->delargs = !(flags & (METH_O));
    } else {
      data->delargs = 0;
    }
    data->implicitconv = 0;
    data->pytype = 0;
    return data;
  }
}

SWIGRUNTIME void 
SwigPyClientData_Del(SwigPyClientData *data) {
  Py_XDECREF(data->newraw);
  Py_XDECREF(data->newargs);
  Py_XDECREF(data->destroy);
}

/* =============== SwigPyObject =====================*/

typedef struct {
  PyObject_HEAD
  void *ptr;
  swig_type_info *ty;
  int own;
  PyObject *next;
#ifdef SWIGPYTHON_BUILTIN
  PyObject *dict;
#endif
} SwigPyObject;


#ifdef SWIGPYTHON_BUILTIN

SWIGRUNTIME PyObject *
SwigPyObject_get___dict__(PyObject *v, PyObject *SWIGUNUSEDPARM(args))
{
  SwigPyObject *sobj = (SwigPyObject *)v;

  if (!sobj->dict)
    sobj->dict = PyDict_New();

  Py_INCREF(sobj->dict);
  return sobj->dict;
}

#endif

SWIGRUNTIME PyObject *
SwigPyObject_long(SwigPyObject *v)
{
  return PyLong_FromVoidPtr(v->ptr);
}

SWIGRUNTIME PyObject *
SwigPyObject_format(const char* fmt, SwigPyObject *v)
{
  PyObject *res = NULL;
  PyObject *args = PyTuple_New(1);
  if (args) {
    if (PyTuple_SetItem(args, 0, SwigPyObject_long(v)) == 0) {
      PyObject *ofmt = SWIG_Python_str_FromChar(fmt);
      if (ofmt) {
#if PY_VERSION_HEX >= 0x03000000
	res = PyUnicode_Format(ofmt,args);
#else
	res = PyString_Format(ofmt,args);
#endif
	Py_DECREF(ofmt);
      }
      Py_DECREF(args);
    }
  }
  return res;
}

SWIGRUNTIME PyObject *
SwigPyObject_oct(SwigPyObject *v)
{
  return SwigPyObject_format("%o",v);
}

SWIGRUNTIME PyObject *
SwigPyObject_hex(SwigPyObject *v)
{
  return SwigPyObject_format("%x",v);
}

SWIGRUNTIME PyObject *
SwigPyObject_repr(SwigPyObject *v)
{
  const char *name = SWIG_TypePrettyName(v->ty);
  PyObject *repr = SWIG_Python_str_FromFormat("<Swig Object of type '%s' at %p>", (name ? name : "unknown"), (void *)v);
  if (v->next) {
    PyObject *nrep = SwigPyObject_repr((SwigPyObject *)v->next);
# if PY_VERSION_HEX >= 0x03000000
    PyObject *joined = PyUnicode_Concat(repr, nrep);
    Py_DecRef(repr);
    Py_DecRef(nrep);
    repr = joined;
# else
    PyString_ConcatAndDel(&repr,nrep);
# endif
  }
  return repr;  
}

/* We need a version taking two PyObject* parameters so it's a valid
 * PyCFunction to use in swigobject_methods[]. */
SWIGRUNTIME PyObject *
SwigPyObject_repr2(PyObject *v, PyObject *SWIGUNUSEDPARM(args))
{
  return SwigPyObject_repr((SwigPyObject*)v);
}

SWIGRUNTIME int
SwigPyObject_compare(SwigPyObject *v, SwigPyObject *w)
{
  void *i = v->ptr;
  void *j = w->ptr;
  return (i < j) ? -1 : ((i > j) ? 1 : 0);
}

/* Added for Python 3.x, would it also be useful for Python 2.x? */
SWIGRUNTIME PyObject*
SwigPyObject_richcompare(SwigPyObject *v, SwigPyObject *w, int op)
{
  PyObject* res;
  if( op != Py_EQ && op != Py_NE ) {
    Py_INCREF(Py_NotImplemented);
    return Py_NotImplemented;
  }
  res = PyBool_FromLong( (SwigPyObject_compare(v, w)==0) == (op == Py_EQ) ? 1 : 0);
  return res;  
}


SWIGRUNTIME PyTypeObject* SwigPyObject_TypeOnce(void);

#ifdef SWIGPYTHON_BUILTIN
static swig_type_info *SwigPyObject_stype = 0;
SWIGRUNTIME PyTypeObject*
SwigPyObject_type(void) {
    SwigPyClientData *cd;
    assert(SwigPyObject_stype);
    cd = (SwigPyClientData*) SwigPyObject_stype->clientdata;
    assert(cd);
    assert(cd->pytype);
    return cd->pytype;
}
#else
SWIGRUNTIME PyTypeObject*
SwigPyObject_type(void) {
  static PyTypeObject *SWIG_STATIC_POINTER(type) = SwigPyObject_TypeOnce();
  return type;
}
#endif

SWIGRUNTIMEINLINE int
SwigPyObject_Check(PyObject *op) {
#ifdef SWIGPYTHON_BUILTIN
  PyTypeObject *target_tp = SwigPyObject_type();
  if (PyType_IsSubtype(op->ob_type, target_tp))
    return 1;
  return (strcmp(op->ob_type->tp_name, "SwigPyObject") == 0);
#else
  return (Py_TYPE(op) == SwigPyObject_type())
    || (strcmp(Py_TYPE(op)->tp_name,"SwigPyObject") == 0);
#endif
}

SWIGRUNTIME PyObject *
SwigPyObject_New(void *ptr, swig_type_info *ty, int own);

SWIGRUNTIME void
SwigPyObject_dealloc(PyObject *v)
{
  SwigPyObject *sobj = (SwigPyObject *) v;
  PyObject *next = sobj->next;
  if (sobj->own == SWIG_POINTER_OWN) {
    swig_type_info *ty = sobj->ty;
    SwigPyClientData *data = ty ? (SwigPyClientData *) ty->clientdata : 0;
    PyObject *destroy = data ? data->destroy : 0;
    if (destroy) {
      /* destroy is always a VARARGS method */
      PyObject *res;

      /* PyObject_CallFunction() has the potential to silently drop
         the active exception.  In cases of unnamed temporary
         variable or where we just finished iterating over a generator
         StopIteration will be active right now, and this needs to
         remain true upon return from SwigPyObject_dealloc.  So save
         and restore. */
      
      PyObject *type = NULL, *value = NULL, *traceback = NULL;
      PyErr_Fetch(&type, &value, &traceback);

      if (data->delargs) {
        /* we need to create a temporary object to carry the destroy operation */
        PyObject *tmp = SwigPyObject_New(sobj->ptr, ty, 0);
        res = SWIG_Python_CallFunctor(destroy, tmp);
        Py_DECREF(tmp);
      } else {
        PyCFunction meth = PyCFunction_GET_FUNCTION(destroy);
        PyObject *mself = PyCFunction_GET_SELF(destroy);
        res = ((*meth)(mself, v));
      }
      if (!res)
        PyErr_WriteUnraisable(destroy);

      PyErr_Restore(type, value, traceback);

      Py_XDECREF(res);
    } 
#if !defined(SWIG_PYTHON_SILENT_MEMLEAK)
    else {
      const char *name = SWIG_TypePrettyName(ty);
      printf("swig/python detected a memory leak of type '%s', no destructor found.\n", (name ? name : "unknown"));
    }
#endif
  } 
  Py_XDECREF(next);
  PyObject_DEL(v);
}

SWIGRUNTIME PyObject* 
SwigPyObject_append(PyObject* v, PyObject* next)
{
  SwigPyObject *sobj = (SwigPyObject *) v;
  if (!SwigPyObject_Check(next)) {
    PyErr_SetString(PyExc_TypeError, "Attempt to append a non SwigPyObject");
    return NULL;
  }
  sobj->next = next;
  Py_INCREF(next);
  return SWIG_Py_Void();
}

SWIGRUNTIME PyObject* 
SwigPyObject_next(PyObject* v, PyObject *SWIGUNUSEDPARM(args))
{
  SwigPyObject *sobj = (SwigPyObject *) v;
  if (sobj->next) {    
    Py_INCREF(sobj->next);
    return sobj->next;
  } else {
    return SWIG_Py_Void();
  }
}

SWIGINTERN PyObject*
SwigPyObject_disown(PyObject* v, PyObject *SWIGUNUSEDPARM(args))
{
  SwigPyObject *sobj = (SwigPyObject *)v;
  sobj->own = 0;
  return SWIG_Py_Void();
}

SWIGINTERN PyObject*
SwigPyObject_acquire(PyObject* v, PyObject *SWIGUNUSEDPARM(args))
{
  SwigPyObject *sobj = (SwigPyObject *)v;
  sobj->own = SWIG_POINTER_OWN;
  return SWIG_Py_Void();
}

SWIGINTERN PyObject*
SwigPyObject_own(PyObject *v, PyObject *args)
{
  PyObject *val = 0;
  if (!PyArg_UnpackTuple(args, "own", 0, 1, &val)) {
    return NULL;
  } else {
    SwigPyObject *sobj = (SwigPyObject *)v;
    PyObject *obj = PyBool_FromLong(sobj->own);
    if (val) {
      if (PyObject_IsTrue(val)) {
        SwigPyObject_acquire(v,args);
      } else {
        SwigPyObject_disown(v,args);
      }
    } 
    return obj;
  }
}

static PyMethodDef
swigobject_methods[] = {
  {"disown",  SwigPyObject_disown,  METH_NOARGS,  "releases ownership of the pointer"},
  {"acquire", SwigPyObject_acquire, METH_NOARGS,  "acquires ownership of the pointer"},
  {"own",     SwigPyObject_own,     METH_VARARGS, "returns/sets ownership of the pointer"},
  {"append",  SwigPyObject_append,  METH_O,       "appends another 'this' object"},
  {"next",    SwigPyObject_next,    METH_NOARGS,  "returns the next 'this' object"},
  {"__repr__",SwigPyObject_repr2,   METH_NOARGS,  "returns object representation"},
  {0, 0, 0, 0}  
};

SWIGRUNTIME PyTypeObject*
SwigPyObject_TypeOnce(void) {
  static char swigobject_doc[] = "Swig object carries a C/C++ instance pointer";

  static PyNumberMethods SwigPyObject_as_number = {
    (binaryfunc)0, /*nb_add*/
    (binaryfunc)0, /*nb_subtract*/
    (binaryfunc)0, /*nb_multiply*/
    /* nb_divide removed in Python 3 */
#if PY_VERSION_HEX < 0x03000000
    (binaryfunc)0, /*nb_divide*/
#endif
    (binaryfunc)0, /*nb_remainder*/
    (binaryfunc)0, /*nb_divmod*/
    (ternaryfunc)0,/*nb_power*/
    (unaryfunc)0,  /*nb_negative*/
    (unaryfunc)0,  /*nb_positive*/
    (unaryfunc)0,  /*nb_absolute*/
    (inquiry)0,    /*nb_nonzero*/
    0,		   /*nb_invert*/
    0,		   /*nb_lshift*/
    0,		   /*nb_rshift*/
    0,		   /*nb_and*/
    0,		   /*nb_xor*/
    0,		   /*nb_or*/
#if PY_VERSION_HEX < 0x03000000
    0,   /*nb_coerce*/
#endif
    (unaryfunc)SwigPyObject_long, /*nb_int*/
#if PY_VERSION_HEX < 0x03000000
    (unaryfunc)SwigPyObject_long, /*nb_long*/
#else
    0, /*nb_reserved*/
#endif
    (unaryfunc)0,                 /*nb_float*/
#if PY_VERSION_HEX < 0x03000000
    (unaryfunc)SwigPyObject_oct,  /*nb_oct*/
    (unaryfunc)SwigPyObject_hex,  /*nb_hex*/
#endif
#if PY_VERSION_HEX >= 0x03050000 /* 3.5 */
    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 /* nb_inplace_add -> nb_inplace_matrix_multiply */
#elif PY_VERSION_HEX >= 0x03000000 /* 3.0 */
    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 /* nb_inplace_add -> nb_index, nb_inplace_divide removed */
#else
    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 /* nb_inplace_add -> nb_index */
#endif
  };

  static PyTypeObject swigpyobject_type;
  static int type_init = 0;
  if (!type_init) {
    const PyTypeObject tmp = {
#if PY_VERSION_HEX >= 0x03000000
      PyVarObject_HEAD_INIT(NULL, 0)
#else
      PyObject_HEAD_INIT(NULL)
      0,                                    /* ob_size */
#endif
      "SwigPyObject",                       /* tp_name */
      sizeof(SwigPyObject),                 /* tp_basicsize */
      0,                                    /* tp_itemsize */
      (destructor)SwigPyObject_dealloc,     /* tp_dealloc */
      0,                                    /* tp_print */
      (getattrfunc)0,                       /* tp_getattr */
      (setattrfunc)0,                       /* tp_setattr */
#if PY_VERSION_HEX >= 0x03000000
      0, /* tp_reserved in 3.0.1, tp_compare in 3.0.0 but not used */
#else
      (cmpfunc)SwigPyObject_compare,        /* tp_compare */
#endif
      (reprfunc)SwigPyObject_repr,          /* tp_repr */
      &SwigPyObject_as_number,              /* tp_as_number */
      0,                                    /* tp_as_sequence */
      0,                                    /* tp_as_mapping */
      (hashfunc)0,                          /* tp_hash */
      (ternaryfunc)0,                       /* tp_call */
      0,                                    /* tp_str */
      PyObject_GenericGetAttr,              /* tp_getattro */
      0,                                    /* tp_setattro */
      0,                                    /* tp_as_buffer */
      Py_TPFLAGS_DEFAULT,                   /* tp_flags */
      swigobject_doc,                       /* tp_doc */
      0,                                    /* tp_traverse */
      0,                                    /* tp_clear */
      (richcmpfunc)SwigPyObject_richcompare,/* tp_richcompare */
      0,                                    /* tp_weaklistoffset */
      0,                                    /* tp_iter */
      0,                                    /* tp_iternext */
      swigobject_methods,                   /* tp_methods */
      0,                                    /* tp_members */
      0,                                    /* tp_getset */
      0,                                    /* tp_base */
      0,                                    /* tp_dict */
      0,                                    /* tp_descr_get */
      0,                                    /* tp_descr_set */
      0,                                    /* tp_dictoffset */
      0,                                    /* tp_init */
      0,                                    /* tp_alloc */
      0,                                    /* tp_new */
      0,                                    /* tp_free */
      0,                                    /* tp_is_gc */
      0,                                    /* tp_bases */
      0,                                    /* tp_mro */
      0,                                    /* tp_cache */
      0,                                    /* tp_subclasses */
      0,                                    /* tp_weaklist */
      0,                                    /* tp_del */
      0,                                    /* tp_version_tag */
#if PY_VERSION_HEX >= 0x03040000
      0,                                    /* tp_finalize */
#endif
#if PY_VERSION_HEX >= 0x03080000
      0,                                    /* tp_vectorcall */
#endif
#if (PY_VERSION_HEX >= 0x03080000) && (PY_VERSION_HEX < 0x03090000)
      0,                                    /* tp_print */
#endif
#ifdef COUNT_ALLOCS
      0,                                    /* tp_allocs */
      0,                                    /* tp_frees */
      0,                                    /* tp_maxalloc */
      0,                                    /* tp_prev */
      0                                     /* tp_next */
#endif
    };
    swigpyobject_type = tmp;
    type_init = 1;
    if (PyType_Ready(&swigpyobject_type) < 0)
      return NULL;
  }
  return &swigpyobject_type;
}

SWIGRUNTIME PyObject *
SwigPyObject_New(void *ptr, swig_type_info *ty, int own)
{
  SwigPyObject *sobj = PyObject_NEW(SwigPyObject, SwigPyObject_type());
  if (sobj) {
    sobj->ptr  = ptr;
    sobj->ty   = ty;
    sobj->own  = own;
    sobj->next = 0;
  }
  return (PyObject *)sobj;
}

/* -----------------------------------------------------------------------------
 * Implements a simple Swig Packed type, and use it instead of string
 * ----------------------------------------------------------------------------- */

typedef struct {
  PyObject_HEAD
  void *pack;
  swig_type_info *ty;
  size_t size;
} SwigPyPacked;

SWIGRUNTIME PyObject *
SwigPyPacked_repr(SwigPyPacked *v)
{
  char result[SWIG_BUFFER_SIZE];
  if (SWIG_PackDataName(result, v->pack, v->size, 0, sizeof(result))) {
    return SWIG_Python_str_FromFormat("<Swig Packed at %s%s>", result, v->ty->name);
  } else {
    return SWIG_Python_str_FromFormat("<Swig Packed %s>", v->ty->name);
  }  
}

SWIGRUNTIME PyObject *
SwigPyPacked_str(SwigPyPacked *v)
{
  char result[SWIG_BUFFER_SIZE];
  if (SWIG_PackDataName(result, v->pack, v->size, 0, sizeof(result))){
    return SWIG_Python_str_FromFormat("%s%s", result, v->ty->name);
  } else {
    return SWIG_Python_str_FromChar(v->ty->name);
  }  
}

SWIGRUNTIME int
SwigPyPacked_compare(SwigPyPacked *v, SwigPyPacked *w)
{
  size_t i = v->size;
  size_t j = w->size;
  int s = (i < j) ? -1 : ((i > j) ? 1 : 0);
  return s ? s : strncmp((const char *)v->pack, (const char *)w->pack, 2*v->size);
}

SWIGRUNTIME PyTypeObject* SwigPyPacked_TypeOnce(void);

SWIGRUNTIME PyTypeObject*
SwigPyPacked_type(void) {
  static PyTypeObject *SWIG_STATIC_POINTER(type) = SwigPyPacked_TypeOnce();
  return type;
}

SWIGRUNTIMEINLINE int
SwigPyPacked_Check(PyObject *op) {
  return ((op)->ob_type == SwigPyPacked_TypeOnce()) 
    || (strcmp((op)->ob_type->tp_name,"SwigPyPacked") == 0);
}

SWIGRUNTIME void
SwigPyPacked_dealloc(PyObject *v)
{
  if (SwigPyPacked_Check(v)) {
    SwigPyPacked *sobj = (SwigPyPacked *) v;
    free(sobj->pack);
  }
  PyObject_DEL(v);
}

SWIGRUNTIME PyTypeObject*
SwigPyPacked_TypeOnce(void) {
  static char swigpacked_doc[] = "Swig object carries a C/C++ instance pointer";
  static PyTypeObject swigpypacked_type;
  static int type_init = 0;
  if (!type_init) {
    const PyTypeObject tmp = {
#if PY_VERSION_HEX>=0x03000000
      PyVarObject_HEAD_INIT(NULL, 0)
#else
      PyObject_HEAD_INIT(NULL)
      0,                                    /* ob_size */
#endif
      "SwigPyPacked",                       /* tp_name */
      sizeof(SwigPyPacked),                 /* tp_basicsize */
      0,                                    /* tp_itemsize */
      (destructor)SwigPyPacked_dealloc,     /* tp_dealloc */
      0,                                    /* tp_print */
      (getattrfunc)0,                       /* tp_getattr */
      (setattrfunc)0,                       /* tp_setattr */
#if PY_VERSION_HEX>=0x03000000
      0, /* tp_reserved in 3.0.1 */
#else
      (cmpfunc)SwigPyPacked_compare,        /* tp_compare */
#endif
      (reprfunc)SwigPyPacked_repr,          /* tp_repr */
      0,                                    /* tp_as_number */
      0,                                    /* tp_as_sequence */
      0,                                    /* tp_as_mapping */
      (hashfunc)0,                          /* tp_hash */
      (ternaryfunc)0,                       /* tp_call */
      (reprfunc)SwigPyPacked_str,           /* tp_str */
      PyObject_GenericGetAttr,              /* tp_getattro */
      0,                                    /* tp_setattro */
      0,                                    /* tp_as_buffer */
      Py_TPFLAGS_DEFAULT,                   /* tp_flags */
      swigpacked_doc,                       /* tp_doc */
      0,                                    /* tp_traverse */
      0,                                    /* tp_clear */
      0,                                    /* tp_richcompare */
      0,                                    /* tp_weaklistoffset */
      0,                                    /* tp_iter */
      0,                                    /* tp_iternext */
      0,                                    /* tp_methods */
      0,                                    /* tp_members */
      0,                                    /* tp_getset */
      0,                                    /* tp_base */
      0,                                    /* tp_dict */
      0,                                    /* tp_descr_get */
      0,                                    /* tp_descr_set */
      0,                                    /* tp_dictoffset */
      0,                                    /* tp_init */
      0,                                    /* tp_alloc */
      0,                                    /* tp_new */
      0,                                    /* tp_free */
      0,                                    /* tp_is_gc */
      0,                                    /* tp_bases */
      0,                                    /* tp_mro */
      0,                                    /* tp_cache */
      0,                                    /* tp_subclasses */
      0,                                    /* tp_weaklist */
      0,                                    /* tp_del */
      0,                                    /* tp_version_tag */
#if PY_VERSION_HEX >= 0x03040000
      0,                                    /* tp_finalize */
#endif
#if PY_VERSION_HEX >= 0x03080000
      0,                                    /* tp_vectorcall */
#endif
#if (PY_VERSION_HEX >= 0x03080000) && (PY_VERSION_HEX < 0x03090000)
      0,                                    /* tp_print */
#endif
#ifdef COUNT_ALLOCS
      0,                                    /* tp_allocs */
      0,                                    /* tp_frees */
      0,                                    /* tp_maxalloc */
      0,                                    /* tp_prev */
      0                                     /* tp_next */
#endif
    };
    swigpypacked_type = tmp;
    type_init = 1;
    if (PyType_Ready(&swigpypacked_type) < 0)
      return NULL;
  }
  return &swigpypacked_type;
}

SWIGRUNTIME PyObject *
SwigPyPacked_New(void *ptr, size_t size, swig_type_info *ty)
{
  SwigPyPacked *sobj = PyObject_NEW(SwigPyPacked, SwigPyPacked_type());
  if (sobj) {
    void *pack = malloc(size);
    if (pack) {
      memcpy(pack, ptr, size);
      sobj->pack = pack;
      sobj->ty   = ty;
      sobj->size = size;
    } else {
      PyObject_DEL((PyObject *) sobj);
      sobj = 0;
    }
  }
  return (PyObject *) sobj;
}

SWIGRUNTIME swig_type_info *
SwigPyPacked_UnpackData(PyObject *obj, void *ptr, size_t size)
{
  if (SwigPyPacked_Check(obj)) {
    SwigPyPacked *sobj = (SwigPyPacked *)obj;
    if (sobj->size != size) return 0;
    memcpy(ptr, sobj->pack, size);
    return sobj->ty;
  } else {
    return 0;
  }
}

/* -----------------------------------------------------------------------------
 * pointers/data manipulation
 * ----------------------------------------------------------------------------- */

static PyObject *Swig_This_global = NULL;

SWIGRUNTIME PyObject *
SWIG_This(void)
{
  if (Swig_This_global == NULL)
    Swig_This_global = SWIG_Python_str_FromChar("this");
  return Swig_This_global;
}

/* #define SWIG_PYTHON_SLOW_GETSET_THIS */

/* TODO: I don't know how to implement the fast getset in Python 3 right now */
#if PY_VERSION_HEX>=0x03000000
#define SWIG_PYTHON_SLOW_GETSET_THIS 
#endif

SWIGRUNTIME SwigPyObject *
SWIG_Python_GetSwigThis(PyObject *pyobj) 
{
  PyObject *obj;

  if (SwigPyObject_Check(pyobj))
    return (SwigPyObject *) pyobj;

#ifdef SWIGPYTHON_BUILTIN
  (void)obj;
# ifdef PyWeakref_CheckProxy
  if (PyWeakref_CheckProxy(pyobj)) {
    pyobj = PyWeakref_GET_OBJECT(pyobj);
    if (pyobj && SwigPyObject_Check(pyobj))
      return (SwigPyObject*) pyobj;
  }
# endif
  return NULL;
#else

  obj = 0;

#if !defined(SWIG_PYTHON_SLOW_GETSET_THIS)
  if (PyInstance_Check(pyobj)) {
    obj = _PyInstance_Lookup(pyobj, SWIG_This());      
  } else {
    PyObject **dictptr = _PyObject_GetDictPtr(pyobj);
    if (dictptr != NULL) {
      PyObject *dict = *dictptr;
      obj = dict ? PyDict_GetItem(dict, SWIG_This()) : 0;
    } else {
#ifdef PyWeakref_CheckProxy
      if (PyWeakref_CheckProxy(pyobj)) {
	PyObject *wobj = PyWeakref_GET_OBJECT(pyobj);
	return wobj ? SWIG_Python_GetSwigThis(wobj) : 0;
      }
#endif
      obj = PyObject_GetAttr(pyobj,SWIG_This());
      if (obj) {
	Py_DECREF(obj);
      } else {
	if (PyErr_Occurred()) PyErr_Clear();
	return 0;
      }
    }
  }
#else
  obj = PyObject_GetAttr(pyobj,SWIG_This());
  if (obj) {
    Py_DECREF(obj);
  } else {
    if (PyErr_Occurred()) PyErr_Clear();
    return 0;
  }
#endif
  if (obj && !SwigPyObject_Check(obj)) {
    /* a PyObject is called 'this', try to get the 'real this'
       SwigPyObject from it */ 
    return SWIG_Python_GetSwigThis(obj);
  }
  return (SwigPyObject *)obj;
#endif
}

/* Acquire a pointer value */

SWIGRUNTIME int
SWIG_Python_AcquirePtr(PyObject *obj, int own) {
  if (own == SWIG_POINTER_OWN) {
    SwigPyObject *sobj = SWIG_Python_GetSwigThis(obj);
    if (sobj) {
      int oldown = sobj->own;
      sobj->own = own;
      return oldown;
    }
  }
  return 0;
}

/* Convert a pointer value */

SWIGRUNTIME int
SWIG_Python_ConvertPtrAndOwn(PyObject *obj, void **ptr, swig_type_info *ty, int flags, int *own) {
  int res;
  SwigPyObject *sobj;
  int implicit_conv = (flags & SWIG_POINTER_IMPLICIT_CONV) != 0;

  if (!obj)
    return SWIG_ERROR;
  if (obj == Py_None && !implicit_conv) {
    if (ptr)
      *ptr = 0;
    return (flags & SWIG_POINTER_NO_NULL) ? SWIG_NullReferenceError : SWIG_OK;
  }

  res = SWIG_ERROR;

  sobj = SWIG_Python_GetSwigThis(obj);
  if (own)
    *own = 0;
  while (sobj) {
    void *vptr = sobj->ptr;
    if (ty) {
      swig_type_info *to = sobj->ty;
      if (to == ty) {
        /* no type cast needed */
        if (ptr) *ptr = vptr;
        break;
      } else {
        swig_cast_info *tc = SWIG_TypeCheck(to->name,ty);
        if (!tc) {
          sobj = (SwigPyObject *)sobj->next;
        } else {
          if (ptr) {
            int newmemory = 0;
            *ptr = SWIG_TypeCast(tc,vptr,&newmemory);
            if (newmemory == SWIG_CAST_NEW_MEMORY) {
              assert(own); /* badly formed typemap which will lead to a memory leak - it must set and use own to delete *ptr */
              if (own)
                *own = *own | SWIG_CAST_NEW_MEMORY;
            }
          }
          break;
        }
      }
    } else {
      if (ptr) *ptr = vptr;
      break;
    }
  }
  if (sobj) {
    if (own)
      *own = *own | sobj->own;
    if (flags & SWIG_POINTER_DISOWN) {
      sobj->own = 0;
    }
    res = SWIG_OK;
  } else {
    if (implicit_conv) {
      SwigPyClientData *data = ty ? (SwigPyClientData *) ty->clientdata : 0;
      if (data && !data->implicitconv) {
        PyObject *klass = data->klass;
        if (klass) {
          PyObject *impconv;
          data->implicitconv = 1; /* avoid recursion and call 'explicit' constructors*/
          impconv = SWIG_Python_CallFunctor(klass, obj);
          data->implicitconv = 0;
          if (PyErr_Occurred()) {
            PyErr_Clear();
            impconv = 0;
          }
          if (impconv) {
            SwigPyObject *iobj = SWIG_Python_GetSwigThis(impconv);
            if (iobj) {
              void *vptr;
              res = SWIG_Python_ConvertPtrAndOwn((PyObject*)iobj, &vptr, ty, 0, 0);
              if (SWIG_IsOK(res)) {
                if (ptr) {
                  *ptr = vptr;
                  /* transfer the ownership to 'ptr' */
                  iobj->own = 0;
                  res = SWIG_AddCast(res);
                  res = SWIG_AddNewMask(res);
                } else {
                  res = SWIG_AddCast(res);		    
                }
              }
            }
            Py_DECREF(impconv);
          }
        }
      }
      if (!SWIG_IsOK(res) && obj == Py_None) {
        if (ptr)
          *ptr = 0;
        if (PyErr_Occurred())
          PyErr_Clear();
        res = SWIG_OK;
      }
    }
  }
  return res;
}

/* Convert a function ptr value */

SWIGRUNTIME int
SWIG_Python_ConvertFunctionPtr(PyObject *obj, void **ptr, swig_type_info *ty) {
  if (!PyCFunction_Check(obj)) {
    return SWIG_ConvertPtr(obj, ptr, ty, 0);
  } else {
    void *vptr = 0;
    swig_cast_info *tc;

    /* here we get the method pointer for callbacks */
    const char *doc = (((PyCFunctionObject *)obj) -> m_ml -> ml_doc);
    const char *desc = doc ? strstr(doc, "swig_ptr: ") : 0;
    if (desc)
      desc = ty ? SWIG_UnpackVoidPtr(desc + 10, &vptr, ty->name) : 0;
    if (!desc)
      return SWIG_ERROR;
    tc = SWIG_TypeCheck(desc,ty);
    if (tc) {
      int newmemory = 0;
      *ptr = SWIG_TypeCast(tc,vptr,&newmemory);
      assert(!newmemory); /* newmemory handling not yet implemented */
    } else {
      return SWIG_ERROR;
    }
    return SWIG_OK;
  }
}

/* Convert a packed pointer value */

SWIGRUNTIME int
SWIG_Python_ConvertPacked(PyObject *obj, void *ptr, size_t sz, swig_type_info *ty) {
  swig_type_info *to = SwigPyPacked_UnpackData(obj, ptr, sz);
  if (!to) return SWIG_ERROR;
  if (ty) {
    if (to != ty) {
      /* check type cast? */
      swig_cast_info *tc = SWIG_TypeCheck(to->name,ty);
      if (!tc) return SWIG_ERROR;
    }
  }
  return SWIG_OK;
}  

/* -----------------------------------------------------------------------------
 * Create a new pointer object
 * ----------------------------------------------------------------------------- */

/*
  Create a new instance object, without calling __init__, and set the
  'this' attribute.
*/

SWIGRUNTIME PyObject* 
SWIG_Python_NewShadowInstance(SwigPyClientData *data, PyObject *swig_this)
{
  PyObject *inst = 0;
  PyObject *newraw = data->newraw;
  if (newraw) {
    inst = PyObject_Call(newraw, data->newargs, NULL);
    if (inst) {
#if !defined(SWIG_PYTHON_SLOW_GETSET_THIS)
      PyObject **dictptr = _PyObject_GetDictPtr(inst);
      if (dictptr != NULL) {
	PyObject *dict = *dictptr;
	if (dict == NULL) {
	  dict = PyDict_New();
	  *dictptr = dict;
	  PyDict_SetItem(dict, SWIG_This(), swig_this);
	}
      }
#else
      if (PyObject_SetAttr(inst, SWIG_This(), swig_this) == -1) {
        Py_DECREF(inst);
        inst = 0;
      }
#endif
    }
  } else {
#if PY_VERSION_HEX >= 0x03000000
    PyObject *empty_args = PyTuple_New(0);
    if (empty_args) {
      PyObject *empty_kwargs = PyDict_New();
      if (empty_kwargs) {
        inst = ((PyTypeObject *)data->newargs)->tp_new((PyTypeObject *)data->newargs, empty_args, empty_kwargs);
        Py_DECREF(empty_kwargs);
        if (inst) {
          if (PyObject_SetAttr(inst, SWIG_This(), swig_this) == -1) {
            Py_DECREF(inst);
            inst = 0;
          } else {
            Py_TYPE(inst)->tp_flags &= ~Py_TPFLAGS_VALID_VERSION_TAG;
          }
        }
      }
      Py_DECREF(empty_args);
    }
#else
    PyObject *dict = PyDict_New();
    if (dict) {
      PyDict_SetItem(dict, SWIG_This(), swig_this);
      inst = PyInstance_NewRaw(data->newargs, dict);
      Py_DECREF(dict);
    }
#endif
  }
  return inst;
}

SWIGRUNTIME int
SWIG_Python_SetSwigThis(PyObject *inst, PyObject *swig_this)
{
#if !defined(SWIG_PYTHON_SLOW_GETSET_THIS)
  PyObject **dictptr = _PyObject_GetDictPtr(inst);
  if (dictptr != NULL) {
    PyObject *dict = *dictptr;
    if (dict == NULL) {
      dict = PyDict_New();
      *dictptr = dict;
    }
    return PyDict_SetItem(dict, SWIG_This(), swig_this);
  }
#endif
  return PyObject_SetAttr(inst, SWIG_This(), swig_this);
} 


SWIGINTERN PyObject *
SWIG_Python_InitShadowInstance(PyObject *args) {
  PyObject *obj[2];
  if (!SWIG_Python_UnpackTuple(args, "swiginit", 2, 2, obj)) {
    return NULL;
  } else {
    SwigPyObject *sthis = SWIG_Python_GetSwigThis(obj[0]);
    if (sthis) {
      SwigPyObject_append((PyObject*) sthis, obj[1]);
    } else {
      if (SWIG_Python_SetSwigThis(obj[0], obj[1]) != 0)
        return NULL;
    }
    return SWIG_Py_Void();
  }
}

/* Create a new pointer object */

SWIGRUNTIME PyObject *
SWIG_Python_NewPointerObj(PyObject *self, void *ptr, swig_type_info *type, int flags) {
  SwigPyClientData *clientdata;
  PyObject * robj;
  int own;

  if (!ptr)
    return SWIG_Py_Void();

  clientdata = type ? (SwigPyClientData *)(type->clientdata) : 0;
  own = (flags & SWIG_POINTER_OWN) ? SWIG_POINTER_OWN : 0;
  if (clientdata && clientdata->pytype) {
    SwigPyObject *newobj;
    if (flags & SWIG_BUILTIN_TP_INIT) {
      newobj = (SwigPyObject*) self;
      if (newobj->ptr) {
        PyObject *next_self = clientdata->pytype->tp_alloc(clientdata->pytype, 0);
        while (newobj->next)
	  newobj = (SwigPyObject *) newobj->next;
        newobj->next = next_self;
        newobj = (SwigPyObject *)next_self;
#ifdef SWIGPYTHON_BUILTIN
        newobj->dict = 0;
#endif
      }
    } else {
      newobj = PyObject_New(SwigPyObject, clientdata->pytype);
#ifdef SWIGPYTHON_BUILTIN
      newobj->dict = 0;
#endif
    }
    if (newobj) {
      newobj->ptr = ptr;
      newobj->ty = type;
      newobj->own = own;
      newobj->next = 0;
      return (PyObject*) newobj;
    }
    return SWIG_Py_Void();
  }

  assert(!(flags & SWIG_BUILTIN_TP_INIT));

  robj = SwigPyObject_New(ptr, type, own);
  if (robj && clientdata && !(flags & SWIG_POINTER_NOSHADOW)) {
    PyObject *inst = SWIG_Python_NewShadowInstance(clientdata, robj);
    Py_DECREF(robj);
    robj = inst;
  }
  return robj;
}

/* Create a new packed object */

SWIGRUNTIMEINLINE PyObject *
SWIG_Python_NewPackedObj(void *ptr, size_t sz, swig_type_info *type) {
  return ptr ? SwigPyPacked_New((void *) ptr, sz, type) : SWIG_Py_Void();
}

/* -----------------------------------------------------------------------------*
 *  Get type list 
 * -----------------------------------------------------------------------------*/

#ifdef SWIG_LINK_RUNTIME
void *SWIG_ReturnGlobalTypeList(void *);
#endif

SWIGRUNTIME swig_module_info *
SWIG_Python_GetModule(void *SWIGUNUSEDPARM(clientdata)) {
  static void *type_pointer = (void *)0;
  /* first check if module already created */
  if (!type_pointer) {
#ifdef SWIG_LINK_RUNTIME
    type_pointer = SWIG_ReturnGlobalTypeList((void *)0);
#else
    type_pointer = PyCapsule_Import(SWIGPY_CAPSULE_NAME, 0);
    if (PyErr_Occurred()) {
      PyErr_Clear();
      type_pointer = (void *)0;
    }
#endif
  }
  return (swig_module_info *) type_pointer;
}

SWIGRUNTIME void
SWIG_Python_DestroyModule(PyObject *obj)
{
  swig_module_info *swig_module = (swig_module_info *) PyCapsule_GetPointer(obj, SWIGPY_CAPSULE_NAME);
  swig_type_info **types = swig_module->types;
  size_t i;
  for (i =0; i < swig_module->size; ++i) {
    swig_type_info *ty = types[i];
    if (ty->owndata) {
      SwigPyClientData *data = (SwigPyClientData *) ty->clientdata;
      if (data) SwigPyClientData_Del(data);
    }
  }
  Py_DECREF(SWIG_This());
  Swig_This_global = NULL;
}

SWIGRUNTIME void
SWIG_Python_SetModule(swig_module_info *swig_module) {
#if PY_VERSION_HEX >= 0x03000000
 /* Add a dummy module object into sys.modules */
  PyObject *module = PyImport_AddModule("swig_runtime_data" SWIG_RUNTIME_VERSION);
#else
  static PyMethodDef swig_empty_runtime_method_table[] = { {NULL, NULL, 0, NULL} }; /* Sentinel */
  PyObject *module = Py_InitModule("swig_runtime_data" SWIG_RUNTIME_VERSION, swig_empty_runtime_method_table);
#endif
  PyObject *pointer = PyCapsule_New((void *) swig_module, SWIGPY_CAPSULE_NAME, SWIG_Python_DestroyModule);
  if (pointer && module) {
    PyModule_AddObject(module, "type_pointer_capsule" SWIG_TYPE_TABLE_NAME, pointer);
  } else {
    Py_XDECREF(pointer);
  }
}

/* The python cached type query */
SWIGRUNTIME PyObject *
SWIG_Python_TypeCache(void) {
  static PyObject *SWIG_STATIC_POINTER(cache) = PyDict_New();
  return cache;
}

SWIGRUNTIME swig_type_info *
SWIG_Python_TypeQuery(const char *type)
{
  PyObject *cache = SWIG_Python_TypeCache();
  PyObject *key = SWIG_Python_str_FromChar(type); 
  PyObject *obj = PyDict_GetItem(cache, key);
  swig_type_info *descriptor;
  if (obj) {
    descriptor = (swig_type_info *) PyCapsule_GetPointer(obj, NULL);
  } else {
    swig_module_info *swig_module = SWIG_GetModule(0);
    descriptor = SWIG_TypeQueryModule(swig_module, swig_module, type);
    if (descriptor) {
      obj = PyCapsule_New((void*) descriptor, NULL, NULL);
      PyDict_SetItem(cache, key, obj);
      Py_DECREF(obj);
    }
  }
  Py_DECREF(key);
  return descriptor;
}

/* 
   For backward compatibility only
*/
#define SWIG_POINTER_EXCEPTION  0
#define SWIG_arg_fail(arg)      SWIG_Python_ArgFail(arg)
#define SWIG_MustGetPtr(p, type, argnum, flags)  SWIG_Python_MustGetPtr(p, type, argnum, flags)

SWIGRUNTIME int
SWIG_Python_AddErrMesg(const char* mesg, int infront)
{  
  if (PyErr_Occurred()) {
    PyObject *type = 0;
    PyObject *value = 0;
    PyObject *traceback = 0;
    PyErr_Fetch(&type, &value, &traceback);
    if (value) {
      PyObject *old_str = PyObject_Str(value);
      const char *tmp = SWIG_Python_str_AsChar(old_str);
      const char *errmesg = tmp ? tmp : "Invalid error message";
      Py_XINCREF(type);
      PyErr_Clear();
      if (infront) {
	PyErr_Format(type, "%s %s", mesg, errmesg);
      } else {
	PyErr_Format(type, "%s %s", errmesg, mesg);
      }
      SWIG_Python_str_DelForPy3(tmp);
      Py_DECREF(old_str);
    }
    return 1;
  } else {
    return 0;
  }
}
  
SWIGRUNTIME int
SWIG_Python_ArgFail(int argnum)
{
  if (PyErr_Occurred()) {
    /* add information about failing argument */
    char mesg[256];
    PyOS_snprintf(mesg, sizeof(mesg), "argument number %d:", argnum);
    return SWIG_Python_AddErrMesg(mesg, 1);
  } else {
    return 0;
  }
}

SWIGRUNTIMEINLINE const char *
SwigPyObject_GetDesc(PyObject *self)
{
  SwigPyObject *v = (SwigPyObject *)self;
  swig_type_info *ty = v ? v->ty : 0;
  return ty ? ty->str : "";
}

SWIGRUNTIME void
SWIG_Python_TypeError(const char *type, PyObject *obj)
{
  if (type) {
#if defined(SWIG_COBJECT_TYPES)
    if (obj && SwigPyObject_Check(obj)) {
      const char *otype = (const char *) SwigPyObject_GetDesc(obj);
      if (otype) {
	PyErr_Format(PyExc_TypeError, "a '%s' is expected, 'SwigPyObject(%s)' is received",
		     type, otype);
	return;
      }
    } else 
#endif      
    {
      const char *otype = (obj ? obj->ob_type->tp_name : 0); 
      if (otype) {
	PyObject *str = PyObject_Str(obj);
	const char *cstr = str ? SWIG_Python_str_AsChar(str) : 0;
	if (cstr) {
	  PyErr_Format(PyExc_TypeError, "a '%s' is expected, '%s(%s)' is received",
		       type, otype, cstr);
          SWIG_Python_str_DelForPy3(cstr);
	} else {
	  PyErr_Format(PyExc_TypeError, "a '%s' is expected, '%s' is received",
		       type, otype);
	}
	Py_XDECREF(str);
	return;
      }
    }   
    PyErr_Format(PyExc_TypeError, "a '%s' is expected", type);
  } else {
    PyErr_Format(PyExc_TypeError, "unexpected type is received");
  }
}


/* Convert a pointer value, signal an exception on a type mismatch */
SWIGRUNTIME void *
SWIG_Python_MustGetPtr(PyObject *obj, swig_type_info *ty, int SWIGUNUSEDPARM(argnum), int flags) {
  void *result;
  if (SWIG_Python_ConvertPtr(obj, &result, ty, flags) == -1) {
    PyErr_Clear();
#if SWIG_POINTER_EXCEPTION
    if (flags) {
      SWIG_Python_TypeError(SWIG_TypePrettyName(ty), obj);
      SWIG_Python_ArgFail(argnum);
    }
#endif
  }
  return result;
}

#ifdef SWIGPYTHON_BUILTIN
SWIGRUNTIME int
SWIG_Python_NonDynamicSetAttr(PyObject *obj, PyObject *name, PyObject *value) {
  PyTypeObject *tp = obj->ob_type;
  PyObject *descr;
  PyObject *encoded_name;
  descrsetfunc f;
  int res = -1;

# ifdef Py_USING_UNICODE
  if (PyString_Check(name)) {
    name = PyUnicode_Decode(PyString_AsString(name), PyString_Size(name), NULL, NULL);
    if (!name)
      return -1;
  } else if (!PyUnicode_Check(name))
# else
  if (!PyString_Check(name))
# endif
  {
    PyErr_Format(PyExc_TypeError, "attribute name must be string, not '%.200s'", name->ob_type->tp_name);
    return -1;
  } else {
    Py_INCREF(name);
  }

  if (!tp->tp_dict) {
    if (PyType_Ready(tp) < 0)
      goto done;
  }

  descr = _PyType_Lookup(tp, name);
  f = NULL;
  if (descr != NULL)
    f = descr->ob_type->tp_descr_set;
  if (!f) {
    if (PyString_Check(name)) {
      encoded_name = name;
      Py_INCREF(name);
    } else {
      encoded_name = PyUnicode_AsUTF8String(name);
      if (!encoded_name)
        return -1;
    }
    PyErr_Format(PyExc_AttributeError, "'%.100s' object has no attribute '%.200s'", tp->tp_name, PyString_AsString(encoded_name));
    Py_DECREF(encoded_name);
  } else {
    res = f(descr, obj, value);
  }
  
  done:
  Py_DECREF(name);
  return res;
}
#endif


#ifdef __cplusplus
}
#endif



#define SWIG_exception_fail(code, msg) do { SWIG_Error(code, msg); SWIG_fail; } while(0) 

#define SWIG_contract_assert(expr, msg) if (!(expr)) { SWIG_Error(SWIG_RuntimeError, msg); SWIG_fail; } else 



#ifdef __cplusplus
extern "C" {
#endif

/* Method creation and docstring support functions */

SWIGINTERN PyMethodDef *SWIG_PythonGetProxyDoc(const char *name);
SWIGINTERN PyObject *SWIG_PyInstanceMethod_New(PyObject *SWIGUNUSEDPARM(self), PyObject *func);
SWIGINTERN PyObject *SWIG_PyStaticMethod_New(PyObject *SWIGUNUSEDPARM(self), PyObject *func);

#ifdef __cplusplus
}
#endif


  #define SWIG_exception(code, msg) do { SWIG_Error(code, msg); SWIG_fail;; } while(0) 

/* -----------------------------------------------------------------------------
 * director_common.swg
 *
 * This file contains support for director classes which is common between
 * languages.
 * ----------------------------------------------------------------------------- */

/*
  Use -DSWIG_DIRECTOR_STATIC if you prefer to avoid the use of the
  'Swig' namespace. This could be useful for multi-modules projects.
*/
#ifdef SWIG_DIRECTOR_STATIC
/* Force anonymous (static) namespace */
#define Swig
#endif
/* -----------------------------------------------------------------------------
 * director.swg
 *
 * This file contains support for director classes so that Python proxy
 * methods can be called from C++.
 * ----------------------------------------------------------------------------- */

#ifndef SWIG_DIRECTOR_PYTHON_HEADER_
#define SWIG_DIRECTOR_PYTHON_HEADER_

#include <string>
#include <iostream>
#include <exception>
#include <vector>
#include <map>


/*
  Use -DSWIG_PYTHON_DIRECTOR_NO_VTABLE if you don't want to generate a 'virtual
  table', and avoid multiple GetAttr calls to retrieve the python
  methods.
*/

#ifndef SWIG_PYTHON_DIRECTOR_NO_VTABLE
#ifndef SWIG_PYTHON_DIRECTOR_VTABLE
#define SWIG_PYTHON_DIRECTOR_VTABLE
#endif
#endif



/*
  Use -DSWIG_DIRECTOR_NO_UEH if you prefer to avoid the use of the
  Undefined Exception Handler provided by swig.
*/
#ifndef SWIG_DIRECTOR_NO_UEH
#ifndef SWIG_DIRECTOR_UEH
#define SWIG_DIRECTOR_UEH
#endif
#endif


/*
  Use -DSWIG_DIRECTOR_NORTTI if you prefer to avoid the use of the
  native C++ RTTI and dynamic_cast<>. But be aware that directors
  could stop working when using this option.
*/
#ifdef SWIG_DIRECTOR_NORTTI
/*
   When we don't use the native C++ RTTI, we implement a minimal one
   only for Directors.
*/
# ifndef SWIG_DIRECTOR_RTDIR
# define SWIG_DIRECTOR_RTDIR

namespace Swig {
  class Director;
  SWIGINTERN std::map<void *, Director *>& get_rtdir_map() {
    static std::map<void *, Director *> rtdir_map;
    return rtdir_map;
  }

  SWIGINTERNINLINE void set_rtdir(void *vptr, Director *rtdir) {
    get_rtdir_map()[vptr] = rtdir;
  }

  SWIGINTERNINLINE Director *get_rtdir(void *vptr) {
    std::map<void *, Director *>::const_iterator pos = get_rtdir_map().find(vptr);
    Director *rtdir = (pos != get_rtdir_map().end()) ? pos->second : 0;
    return rtdir;
  }
}
# endif /* SWIG_DIRECTOR_RTDIR */

# define SWIG_DIRECTOR_CAST(ARG) Swig::get_rtdir(static_cast<void *>(ARG))
# define SWIG_DIRECTOR_RGTR(ARG1, ARG2) Swig::set_rtdir(static_cast<void *>(ARG1), ARG2)

#else

# define SWIG_DIRECTOR_CAST(ARG) dynamic_cast<Swig::Director *>(ARG)
# define SWIG_DIRECTOR_RGTR(ARG1, ARG2)

#endif /* SWIG_DIRECTOR_NORTTI */

extern "C" {
  struct swig_type_info;
}

namespace Swig {

  /* memory handler */
  struct GCItem {
    virtual ~GCItem() {}

    virtual int get_own() const {
      return 0;
    }
  };

  struct GCItem_var {
    GCItem_var(GCItem *item = 0) : _item(item) {
    }

    GCItem_var& operator=(GCItem *item) {
      GCItem *tmp = _item;
      _item = item;
      delete tmp;
      return *this;
    }

    ~GCItem_var() {
      delete _item;
    }

    GCItem * operator->() const {
      return _item;
    }

  private:
    GCItem *_item;
  };

  struct GCItem_Object : GCItem {
    GCItem_Object(int own) : _own(own) {
    }

    virtual ~GCItem_Object() {
    }

    int get_own() const {
      return _own;
    }

  private:
    int _own;
  };

  template <typename Type>
  struct GCItem_T : GCItem {
    GCItem_T(Type *ptr) : _ptr(ptr) {
    }

    virtual ~GCItem_T() {
      delete _ptr;
    }

  private:
    Type *_ptr;
  };

  template <typename Type>
  struct GCArray_T : GCItem {
    GCArray_T(Type *ptr) : _ptr(ptr) {
    }

    virtual ~GCArray_T() {
      delete[] _ptr;
    }

  private:
    Type *_ptr;
  };

  /* base class for director exceptions */
  class DirectorException : public std::exception {
  protected:
    std::string swig_msg;
  public:
    DirectorException(PyObject *error, const char *hdr ="", const char *msg ="") : swig_msg(hdr) {
      SWIG_PYTHON_THREAD_BEGIN_BLOCK;
      if (msg[0]) {
        swig_msg += " ";
        swig_msg += msg;
      }
      if (!PyErr_Occurred()) {
        PyErr_SetString(error, what());
      }
      SWIG_PYTHON_THREAD_END_BLOCK;
    }

    virtual ~DirectorException() throw() {
    }

    /* Deprecated, use what() instead */
    const char *getMessage() const {
      return what();
    }

    const char *what() const throw() {
      return swig_msg.c_str();
    }

    static void raise(PyObject *error, const char *msg) {
      throw DirectorException(error, msg);
    }

    static void raise(const char *msg) {
      raise(PyExc_RuntimeError, msg);
    }
  };

  /* type mismatch in the return value from a python method call */
  class DirectorTypeMismatchException : public DirectorException {
  public:
    DirectorTypeMismatchException(PyObject *error, const char *msg="")
      : DirectorException(error, "SWIG director type mismatch", msg) {
    }

    DirectorTypeMismatchException(const char *msg="")
      : DirectorException(PyExc_TypeError, "SWIG director type mismatch", msg) {
    }

    static void raise(PyObject *error, const char *msg) {
      throw DirectorTypeMismatchException(error, msg);
    }

    static void raise(const char *msg) {
      throw DirectorTypeMismatchException(msg);
    }
  };

  /* any python exception that occurs during a director method call */
  class DirectorMethodException : public DirectorException {
  public:
    DirectorMethodException(const char *msg = "")
      : DirectorException(PyExc_RuntimeError, "SWIG director method error.", msg) {
    }

    static void raise(const char *msg) {
      throw DirectorMethodException(msg);
    }
  };

  /* attempt to call a pure virtual method via a director method */
  class DirectorPureVirtualException : public DirectorException {
  public:
    DirectorPureVirtualException(const char *msg = "")
      : DirectorException(PyExc_RuntimeError, "SWIG director pure virtual method called", msg) {
    }

    static void raise(const char *msg) {
      throw DirectorPureVirtualException(msg);
    }
  };


#if defined(SWIG_PYTHON_THREADS)
/*  __THREAD__ is the old macro to activate some thread support */
# if !defined(__THREAD__)
#   define __THREAD__ 1
# endif
#endif

#ifdef __THREAD__
# include "pythread.h"
  class Guard {
    PyThread_type_lock &mutex_;

  public:
    Guard(PyThread_type_lock & mutex) : mutex_(mutex) {
      PyThread_acquire_lock(mutex_, WAIT_LOCK);
    }

    ~Guard() {
      PyThread_release_lock(mutex_);
    }
  };
# define SWIG_GUARD(mutex) Guard _guard(mutex)
#else
# define SWIG_GUARD(mutex)
#endif

  /* director base class */
  class Director {
  private:
    /* pointer to the wrapped python object */
    PyObject *swig_self;
    /* flag indicating whether the object is owned by python or c++ */
    mutable bool swig_disown_flag;

    /* decrement the reference count of the wrapped python object */
    void swig_decref() const {
      if (swig_disown_flag) {
        SWIG_PYTHON_THREAD_BEGIN_BLOCK;
        Py_DECREF(swig_self);
        SWIG_PYTHON_THREAD_END_BLOCK;
      }
    }

  public:
    /* wrap a python object. */
    Director(PyObject *self) : swig_self(self), swig_disown_flag(false) {
    }

    /* discard our reference at destruction */
    virtual ~Director() {
      swig_decref();
    }

    /* return a pointer to the wrapped python object */
    PyObject *swig_get_self() const {
      return swig_self;
    }

    /* acquire ownership of the wrapped python object (the sense of "disown" is from python) */
    void swig_disown() const {
      if (!swig_disown_flag) {
        swig_disown_flag=true;
        swig_incref();
      }
    }

    /* increase the reference count of the wrapped python object */
    void swig_incref() const {
      if (swig_disown_flag) {
        Py_INCREF(swig_self);
      }
    }

    /* methods to implement pseudo protected director members */
    virtual bool swig_get_inner(const char * /* swig_protected_method_name */) const {
      return true;
    }

    virtual void swig_set_inner(const char * /* swig_protected_method_name */, bool /* swig_val */) const {
    }

  /* ownership management */
  private:
    typedef std::map<void *, GCItem_var> swig_ownership_map;
    mutable swig_ownership_map swig_owner;
#ifdef __THREAD__
    static PyThread_type_lock swig_mutex_own;
#endif

  public:
    template <typename Type>
    void swig_acquire_ownership_array(Type *vptr) const {
      if (vptr) {
        SWIG_GUARD(swig_mutex_own);
        swig_owner[vptr] = new GCArray_T<Type>(vptr);
      }
    }

    template <typename Type>
    void swig_acquire_ownership(Type *vptr) const {
      if (vptr) {
        SWIG_GUARD(swig_mutex_own);
        swig_owner[vptr] = new GCItem_T<Type>(vptr);
      }
    }

    void swig_acquire_ownership_obj(void *vptr, int own) const {
      if (vptr && own) {
        SWIG_GUARD(swig_mutex_own);
        swig_owner[vptr] = new GCItem_Object(own);
      }
    }

    int swig_release_ownership(void *vptr) const {
      int own = 0;
      if (vptr) {
        SWIG_GUARD(swig_mutex_own);
        swig_ownership_map::iterator iter = swig_owner.find(vptr);
        if (iter != swig_owner.end()) {
          own = iter->second->get_own();
          swig_owner.erase(iter);
        }
      }
      return own;
    }

    template <typename Type>
    static PyObject *swig_pyobj_disown(PyObject *pyobj, PyObject *SWIGUNUSEDPARM(args)) {
      SwigPyObject *sobj = (SwigPyObject *)pyobj;
      sobj->own = 0;
      Director *d = SWIG_DIRECTOR_CAST(reinterpret_cast<Type *>(sobj->ptr));
      if (d)
        d->swig_disown();
      return PyWeakref_NewProxy(pyobj, NULL);
    }
  };

#ifdef __THREAD__
  PyThread_type_lock Director::swig_mutex_own = PyThread_allocate_lock();
#endif
}

#endif

/* -------- TYPES TABLE (BEGIN) -------- */

#define SWIGTYPE_p_ClientServiceListenerDirector swig_types[0]
#define SWIGTYPE_p_MemberDefinitionUtil swig_types[1]
#define SWIGTYPE_p_MessageElementDataUtil swig_types[2]
#define SWIGTYPE_p_RobotRaconteur__ArrayMemoryBase swig_types[3]
#define SWIGTYPE_p_RobotRaconteur__AsyncGeneratorClientReturnDirector swig_types[4]
#define SWIGTYPE_p_RobotRaconteur__AsyncNodeInfo2VectorReturnDirector swig_types[5]
#define SWIGTYPE_p_RobotRaconteur__AsyncPipeEndpointReturnDirector swig_types[6]
#define SWIGTYPE_p_RobotRaconteur__AsyncRequestDirector swig_types[7]
#define SWIGTYPE_p_RobotRaconteur__AsyncServiceInfo2VectorReturnDirector swig_types[8]
#define SWIGTYPE_p_RobotRaconteur__AsyncStringReturnDirector swig_types[9]
#define SWIGTYPE_p_RobotRaconteur__AsyncStubReturnDirector swig_types[10]
#define SWIGTYPE_p_RobotRaconteur__AsyncTimerEventReturnDirector swig_types[11]
#define SWIGTYPE_p_RobotRaconteur__AsyncUInt32ReturnDirector swig_types[12]
#define SWIGTYPE_p_RobotRaconteur__AsyncVoidNoErrReturnDirector swig_types[13]
#define SWIGTYPE_p_RobotRaconteur__AsyncVoidReturnDirector swig_types[14]
#define SWIGTYPE_p_RobotRaconteur__AsyncWireConnectionReturnDirector swig_types[15]
#define SWIGTYPE_p_RobotRaconteur__AsyncWirePeekReturnDirector swig_types[16]
#define SWIGTYPE_p_RobotRaconteur__AuthenticatedUser swig_types[17]
#define SWIGTYPE_p_RobotRaconteur__AutoResetEvent swig_types[18]
#define SWIGTYPE_p_RobotRaconteur__BroadcastDownsampler swig_types[19]
#define SWIGTYPE_p_RobotRaconteur__CallbackDefinition swig_types[20]
#define SWIGTYPE_p_RobotRaconteur__CommandLineConfigParser swig_types[21]
#define SWIGTYPE_p_RobotRaconteur__ConstantDefinition swig_types[22]
#define SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField swig_types[23]
#define SWIGTYPE_p_RobotRaconteur__EnumDefinition swig_types[24]
#define SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue swig_types[25]
#define SWIGTYPE_p_RobotRaconteur__EventDefinition swig_types[26]
#define SWIGTYPE_p_RobotRaconteur__ExceptionDefinition swig_types[27]
#define SWIGTYPE_p_RobotRaconteur__FileLogRecordHandler swig_types[28]
#define SWIGTYPE_p_RobotRaconteur__FunctionDefinition swig_types[29]
#define SWIGTYPE_p_RobotRaconteur__GetUuidForNameAndLockResult swig_types[30]
#define SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo swig_types[31]
#define SWIGTYPE_p_RobotRaconteur__HardwareTransport swig_types[32]
#define SWIGTYPE_p_RobotRaconteur__IntraTransport swig_types[33]
#define SWIGTYPE_p_RobotRaconteur__LocalTransport swig_types[34]
#define SWIGTYPE_p_RobotRaconteur__LogRecordHandler swig_types[35]
#define SWIGTYPE_p_RobotRaconteur__MemberDefinition swig_types[36]
#define SWIGTYPE_p_RobotRaconteur__MemoryDefinition swig_types[37]
#define SWIGTYPE_p_RobotRaconteur__Message swig_types[38]
#define SWIGTYPE_p_RobotRaconteur__MessageElement swig_types[39]
#define SWIGTYPE_p_RobotRaconteur__MessageElementData swig_types[40]
#define SWIGTYPE_p_RobotRaconteur__MessageElementNestedElementList swig_types[41]
#define SWIGTYPE_p_RobotRaconteur__MessageEntry swig_types[42]
#define SWIGTYPE_p_RobotRaconteur__MessageHeader swig_types[43]
#define SWIGTYPE_p_RobotRaconteur__MultiDimArrayMemoryBase swig_types[44]
#define SWIGTYPE_p_RobotRaconteur__NodeDirectories swig_types[45]
#define SWIGTYPE_p_RobotRaconteur__NodeDirectoriesFD swig_types[46]
#define SWIGTYPE_p_RobotRaconteur__NodeDirectoriesResourceAlreadyInUse swig_types[47]
#define SWIGTYPE_p_RobotRaconteur__NodeDirectoriesUtil swig_types[48]
#define SWIGTYPE_p_RobotRaconteur__NodeID swig_types[49]
#define SWIGTYPE_p_RobotRaconteur__NodeInfo2 swig_types[50]
#define SWIGTYPE_p_RobotRaconteur__ObjRefDefinition swig_types[51]
#define SWIGTYPE_p_RobotRaconteur__PasswordFileUserAuthenticator swig_types[52]
#define SWIGTYPE_p_RobotRaconteur__PipeDefinition swig_types[53]
#define SWIGTYPE_p_RobotRaconteur__PropertyDefinition swig_types[54]
#define SWIGTYPE_p_RobotRaconteur__RRBaseArray swig_types[55]
#define SWIGTYPE_p_RobotRaconteur__RRLogRecord swig_types[56]
#define SWIGTYPE_p_RobotRaconteur__RRMultiDimArrayUntyped swig_types[57]
#define SWIGTYPE_p_RobotRaconteur__RRNativeDirectorSupport swig_types[58]
#define SWIGTYPE_p_RobotRaconteur__RRObject swig_types[59]
#define SWIGTYPE_p_RobotRaconteur__RRValue swig_types[60]
#define SWIGTYPE_p_RobotRaconteur__Rate swig_types[61]
#define SWIGTYPE_p_RobotRaconteur__RobotRaconteurNode swig_types[62]
#define SWIGTYPE_p_RobotRaconteur__RobotRaconteurNodeSetup swig_types[63]
#define SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion swig_types[64]
#define SWIGTYPE_p_RobotRaconteur__ServerContext swig_types[65]
#define SWIGTYPE_p_RobotRaconteur__ServerEndpoint swig_types[66]
#define SWIGTYPE_p_RobotRaconteur__ServiceDefinition swig_types[67]
#define SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo swig_types[68]
#define SWIGTYPE_p_RobotRaconteur__ServiceEntryDefinition swig_types[69]
#define SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped swig_types[70]
#define SWIGTYPE_p_RobotRaconteur__ServicePathSegment swig_types[71]
#define SWIGTYPE_p_RobotRaconteur__ServiceSecurityPolicy swig_types[72]
#define SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID swig_types[73]
#define SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute swig_types[74]
#define SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup swig_types[75]
#define SWIGTYPE_p_RobotRaconteur__TcpTransport swig_types[76]
#define SWIGTYPE_p_RobotRaconteur__TimeSpec swig_types[77]
#define SWIGTYPE_p_RobotRaconteur__Timer swig_types[78]
#define SWIGTYPE_p_RobotRaconteur__TimerEvent swig_types[79]
#define SWIGTYPE_p_RobotRaconteur__Transport swig_types[80]
#define SWIGTYPE_p_RobotRaconteur__TryGetValueResult swig_types[81]
#define SWIGTYPE_p_RobotRaconteur__TypeDefinition swig_types[82]
#define SWIGTYPE_p_RobotRaconteur__UserAuthenticator swig_types[83]
#define SWIGTYPE_p_RobotRaconteur__UserLogRecordHandlerBase swig_types[84]
#define SWIGTYPE_p_RobotRaconteur__UserLogRecordHandlerDirector swig_types[85]
#define SWIGTYPE_p_RobotRaconteur__UsingDefinition swig_types[86]
#define SWIGTYPE_p_RobotRaconteur__WallRate swig_types[87]
#define SWIGTYPE_p_RobotRaconteur__WallTimer swig_types[88]
#define SWIGTYPE_p_RobotRaconteur__WireDefinition swig_types[89]
#define SWIGTYPE_p_RobotRaconteur__WrappedArrayMemoryClientUtil swig_types[90]
#define SWIGTYPE_p_RobotRaconteur__WrappedArrayMemoryDirector swig_types[91]
#define SWIGTYPE_p_RobotRaconteur__WrappedDirectorLock swig_types[92]
#define SWIGTYPE_p_RobotRaconteur__WrappedGeneratorClient swig_types[93]
#define SWIGTYPE_p_RobotRaconteur__WrappedGeneratorClient_TryGetNextResult swig_types[94]
#define SWIGTYPE_p_RobotRaconteur__WrappedGeneratorServerDirector swig_types[95]
#define SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryClientUtil swig_types[96]
#define SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryDirector swig_types[97]
#define SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryParams swig_types[98]
#define SWIGTYPE_p_RobotRaconteur__WrappedNamedArrayMemoryClient swig_types[99]
#define SWIGTYPE_p_RobotRaconteur__WrappedNamedArrayMemoryClientBuffer swig_types[100]
#define SWIGTYPE_p_RobotRaconteur__WrappedNamedArrayMemoryDirector swig_types[101]
#define SWIGTYPE_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClient swig_types[102]
#define SWIGTYPE_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClientBuffer swig_types[103]
#define SWIGTYPE_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryDirector swig_types[104]
#define SWIGTYPE_p_RobotRaconteur__WrappedPipeBroadcaster swig_types[105]
#define SWIGTYPE_p_RobotRaconteur__WrappedPipeBroadcasterPredicateDirector swig_types[106]
#define SWIGTYPE_p_RobotRaconteur__WrappedPipeClient swig_types[107]
#define SWIGTYPE_p_RobotRaconteur__WrappedPipeEndpoint swig_types[108]
#define SWIGTYPE_p_RobotRaconteur__WrappedPipeEndpointDirector swig_types[109]
#define SWIGTYPE_p_RobotRaconteur__WrappedPipeServer swig_types[110]
#define SWIGTYPE_p_RobotRaconteur__WrappedPipeServerConnectDirector swig_types[111]
#define SWIGTYPE_p_RobotRaconteur__WrappedPipeSubscription swig_types[112]
#define SWIGTYPE_p_RobotRaconteur__WrappedPipeSubscriptionDirector swig_types[113]
#define SWIGTYPE_p_RobotRaconteur__WrappedPipeSubscription_send_iterator swig_types[114]
#define SWIGTYPE_p_RobotRaconteur__WrappedPodArrayMemoryClient swig_types[115]
#define SWIGTYPE_p_RobotRaconteur__WrappedPodArrayMemoryClientBuffer swig_types[116]
#define SWIGTYPE_p_RobotRaconteur__WrappedPodArrayMemoryDirector swig_types[117]
#define SWIGTYPE_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryClient swig_types[118]
#define SWIGTYPE_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryClientBuffer swig_types[119]
#define SWIGTYPE_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryDirector swig_types[120]
#define SWIGTYPE_p_RobotRaconteur__WrappedRRObject swig_types[121]
#define SWIGTYPE_p_RobotRaconteur__WrappedServiceInfo2Subscription swig_types[122]
#define SWIGTYPE_p_RobotRaconteur__WrappedServiceInfo2SubscriptionDirector swig_types[123]
#define SWIGTYPE_p_RobotRaconteur__WrappedServiceSkel swig_types[124]
#define SWIGTYPE_p_RobotRaconteur__WrappedServiceSkelAsyncAdapter swig_types[125]
#define SWIGTYPE_p_RobotRaconteur__WrappedServiceSkelDirector swig_types[126]
#define SWIGTYPE_p_RobotRaconteur__WrappedServiceStub swig_types[127]
#define SWIGTYPE_p_RobotRaconteur__WrappedServiceStubDirector swig_types[128]
#define SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscription swig_types[129]
#define SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionDirector swig_types[130]
#define SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionFilter swig_types[131]
#define SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionFilterNode swig_types[132]
#define SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionFilterPredicateDirector swig_types[133]
#define SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManager swig_types[134]
#define SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails swig_types[135]
#define SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscription_TryDefaultClientRes swig_types[136]
#define SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet swig_types[137]
#define SWIGTYPE_p_RobotRaconteur__WrappedSubObjectSubscription swig_types[138]
#define SWIGTYPE_p_RobotRaconteur__WrappedTryReceivePacketWaitResult swig_types[139]
#define SWIGTYPE_p_RobotRaconteur__WrappedUserAuthenticator swig_types[140]
#define SWIGTYPE_p_RobotRaconteur__WrappedUserAuthenticatorDirector swig_types[141]
#define SWIGTYPE_p_RobotRaconteur__WrappedWireBroadcaster swig_types[142]
#define SWIGTYPE_p_RobotRaconteur__WrappedWireBroadcasterPredicateDirector swig_types[143]
#define SWIGTYPE_p_RobotRaconteur__WrappedWireClient swig_types[144]
#define SWIGTYPE_p_RobotRaconteur__WrappedWireConnection swig_types[145]
#define SWIGTYPE_p_RobotRaconteur__WrappedWireConnectionDirector swig_types[146]
#define SWIGTYPE_p_RobotRaconteur__WrappedWireServer swig_types[147]
#define SWIGTYPE_p_RobotRaconteur__WrappedWireServerConnectDirector swig_types[148]
#define SWIGTYPE_p_RobotRaconteur__WrappedWireServerPeekValueDirector swig_types[149]
#define SWIGTYPE_p_RobotRaconteur__WrappedWireServerPokeValueDirector swig_types[150]
#define SWIGTYPE_p_RobotRaconteur__WrappedWireSubscription swig_types[151]
#define SWIGTYPE_p_RobotRaconteur__WrappedWireSubscriptionDirector swig_types[152]
#define SWIGTYPE_p_RobotRaconteur__WrappedWireSubscription_send_iterator swig_types[153]
#define SWIGTYPE_p_RobotRaconteur__WrappedWireUnicastReceiver swig_types[154]
#define SWIGTYPE_p_RobotRaconteur__detail__MultiDimArray_CalculateCopyIndicesIter swig_types[155]
#define SWIGTYPE_p_ServerServiceListenerDirector swig_types[156]
#define SWIGTYPE_p_allocator_type swig_types[157]
#define SWIGTYPE_p_boost__arrayT_unsigned_char_16_t swig_types[158]
#define SWIGTYPE_p_boost__filesystem__path swig_types[159]
#define SWIGTYPE_p_boost__optionalT_std__string_t swig_types[160]
#define SWIGTYPE_p_boost__posix_time__ptime swig_types[161]
#define SWIGTYPE_p_boost__posix_time__time_duration swig_types[162]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ArrayMemoryBase_t swig_types[163]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__AuthenticatedUser_t swig_types[164]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__AutoResetEvent_t swig_types[165]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__BroadcastDownsampler_t swig_types[166]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CallbackDefinition_t swig_types[167]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t swig_types[168]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t swig_types[169]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t swig_types[170]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EventDefinition_t swig_types[171]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t swig_types[172]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__FileLogRecordHandler_t swig_types[173]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_t swig_types[174]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t swig_types[175]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__IntraTransport_t swig_types[176]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t swig_types[177]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LogRecordHandler_t swig_types[178]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t swig_types[179]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemoryDefinition_t swig_types[180]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t swig_types[181]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t swig_types[182]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t swig_types[183]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t swig_types[184]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t swig_types[185]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Message_t swig_types[186]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MultiDimArrayMemoryBase_t swig_types[187]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__NodeDirectoriesFD_t swig_types[188]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_t swig_types[189]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PasswordFileUserAuthenticator_t swig_types[190]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PipeDefinition_t swig_types[191]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_t swig_types[192]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t swig_types[193]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRMultiDimArrayUntyped_t swig_types[194]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRObject_t swig_types[195]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRValue_t swig_types[196]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Rate_t swig_types[197]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNodeSetup_t swig_types[198]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t swig_types[199]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t swig_types[200]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerEndpoint_t swig_types[201]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t swig_types[202]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t swig_types[203]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceSecurityPolicy_t swig_types[204]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t swig_types[205]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Timer_t swig_types[206]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Transport_t swig_types[207]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t swig_types[208]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UserAuthenticator_t swig_types[209]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UserLogRecordHandlerBase_t swig_types[210]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t swig_types[211]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WallRate_t swig_types[212]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WallTimer_t swig_types[213]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WireDefinition_t swig_types[214]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedDirectorLock_t swig_types[215]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedGeneratorClient_t swig_types[216]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedNamedArrayMemoryClient_t swig_types[217]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClient_t swig_types[218]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeBroadcaster_t swig_types[219]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeClient_t swig_types[220]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t swig_types[221]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeServer_t swig_types[222]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t swig_types[223]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPodArrayMemoryClient_t swig_types[224]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPodMultiDimArrayMemoryClient_t swig_types[225]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedRRObject_t swig_types[226]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceInfo2Subscription_t swig_types[227]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t swig_types[228]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t swig_types[229]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t swig_types[230]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t swig_types[231]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t swig_types[232]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t swig_types[233]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t swig_types[234]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t swig_types[235]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedUserAuthenticator_t swig_types[236]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireBroadcaster_t swig_types[237]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireClient_t swig_types[238]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t swig_types[239]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireServer_t swig_types[240]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t swig_types[241]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireUnicastReceiver_t swig_types[242]
#define SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__detail__MultiDimArray_CalculateCopyIndicesIter_t swig_types[243]
#define SWIGTYPE_p_boost__string_ref swig_types[244]
#define SWIGTYPE_p_char swig_types[245]
#define SWIGTYPE_p_difference_type swig_types[246]
#define SWIGTYPE_p_int swig_types[247]
#define SWIGTYPE_p_key_type swig_types[248]
#define SWIGTYPE_p_long swig_types[249]
#define SWIGTYPE_p_mapped_type swig_types[250]
#define SWIGTYPE_p_p_PyObject swig_types[251]
#define SWIGTYPE_p_short swig_types[252]
#define SWIGTYPE_p_signed_char swig_types[253]
#define SWIGTYPE_p_size_type swig_types[254]
#define SWIGTYPE_p_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t swig_types[255]
#define SWIGTYPE_p_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t swig_types[256]
#define SWIGTYPE_p_std__allocatorT_RobotRaconteur__NodeInfo2_t swig_types[257]
#define SWIGTYPE_p_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t swig_types[258]
#define SWIGTYPE_p_std__allocatorT_RobotRaconteur__ServicePathSegment_t swig_types[259]
#define SWIGTYPE_p_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t swig_types[260]
#define SWIGTYPE_p_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t swig_types[261]
#define SWIGTYPE_p_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t swig_types[262]
#define SWIGTYPE_p_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t swig_types[263]
#define SWIGTYPE_p_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t swig_types[264]
#define SWIGTYPE_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t swig_types[265]
#define SWIGTYPE_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t swig_types[266]
#define SWIGTYPE_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t swig_types[267]
#define SWIGTYPE_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t swig_types[268]
#define SWIGTYPE_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t swig_types[269]
#define SWIGTYPE_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t swig_types[270]
#define SWIGTYPE_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t swig_types[271]
#define SWIGTYPE_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t swig_types[272]
#define SWIGTYPE_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t swig_types[273]
#define SWIGTYPE_p_std__allocatorT_int_t swig_types[274]
#define SWIGTYPE_p_std__allocatorT_signed_char_t swig_types[275]
#define SWIGTYPE_p_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t swig_types[276]
#define SWIGTYPE_p_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t swig_types[277]
#define SWIGTYPE_p_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t swig_types[278]
#define SWIGTYPE_p_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t swig_types[279]
#define SWIGTYPE_p_std__allocatorT_std__pairT_std__string_const_std__string_t_t swig_types[280]
#define SWIGTYPE_p_std__allocatorT_std__string_t swig_types[281]
#define SWIGTYPE_p_std__allocatorT_unsigned_char_t swig_types[282]
#define SWIGTYPE_p_std__allocatorT_unsigned_int_t swig_types[283]
#define SWIGTYPE_p_std__allocatorT_unsigned_long_t swig_types[284]
#define SWIGTYPE_p_std__invalid_argument swig_types[285]
#define SWIGTYPE_p_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t swig_types[286]
#define SWIGTYPE_p_std__lessT_std__string_t swig_types[287]
#define SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t swig_types[288]
#define SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t swig_types[289]
#define SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t swig_types[290]
#define SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t swig_types[291]
#define SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t swig_types[292]
#define SWIGTYPE_p_std__runtime_error swig_types[293]
#define SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t swig_types[294]
#define SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t swig_types[295]
#define SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t swig_types[296]
#define SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t swig_types[297]
#define SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t swig_types[298]
#define SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t swig_types[299]
#define SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t swig_types[300]
#define SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t swig_types[301]
#define SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t swig_types[302]
#define SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t swig_types[303]
#define SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t swig_types[304]
#define SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t swig_types[305]
#define SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t swig_types[306]
#define SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t swig_types[307]
#define SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t swig_types[308]
#define SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t swig_types[309]
#define SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t swig_types[310]
#define SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t swig_types[311]
#define SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t swig_types[312]
#define SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t swig_types[313]
#define SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t swig_types[314]
#define SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t swig_types[315]
#define SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t swig_types[316]
#define SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t swig_types[317]
#define SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t swig_types[318]
#define SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t swig_types[319]
#define SWIGTYPE_p_swig__SwigPyIterator swig_types[320]
#define SWIGTYPE_p_unsigned_char swig_types[321]
#define SWIGTYPE_p_unsigned_int swig_types[322]
#define SWIGTYPE_p_unsigned_long swig_types[323]
#define SWIGTYPE_p_unsigned_short swig_types[324]
#define SWIGTYPE_p_value_type swig_types[325]
static swig_type_info *swig_types[327];
static swig_module_info swig_module = {swig_types, 326, 0, 0, 0, 0};
#define SWIG_TypeQuery(name) SWIG_TypeQueryModule(&swig_module, &swig_module, name)
#define SWIG_MangledTypeQuery(name) SWIG_MangledTypeQueryModule(&swig_module, &swig_module, name)

/* -------- TYPES TABLE (END) -------- */

#ifdef SWIG_TypeQuery
# undef SWIG_TypeQuery
#endif
#define SWIG_TypeQuery SWIG_Python_TypeQuery

/*-----------------------------------------------
              @(target):= _RobotRaconteurPython.so
  ------------------------------------------------*/
#if PY_VERSION_HEX >= 0x03000000
#  define SWIG_init    PyInit__RobotRaconteurPython

#else
#  define SWIG_init    init_RobotRaconteurPython

#endif
#define SWIG_name    "_RobotRaconteurPython"

#define SWIGVERSION 0x040002 
#define SWIG_VERSION SWIGVERSION


#define SWIG_as_voidptr(a) const_cast< void * >(static_cast< const void * >(a)) 
#define SWIG_as_voidptrptr(a) ((void)SWIG_as_voidptr(*a),reinterpret_cast< void** >(a)) 


#include <stdexcept>


namespace swig {
  class SwigPtr_PyObject {
  protected:
    PyObject *_obj;

  public:
    SwigPtr_PyObject() :_obj(0)
    {
    }

    SwigPtr_PyObject(const SwigPtr_PyObject& item) : _obj(item._obj)
    {
      SWIG_PYTHON_THREAD_BEGIN_BLOCK;
      Py_XINCREF(_obj);      
      SWIG_PYTHON_THREAD_END_BLOCK;
    }
    
    SwigPtr_PyObject(PyObject *obj, bool initial_ref = true) :_obj(obj)
    {
      if (initial_ref) {
        SWIG_PYTHON_THREAD_BEGIN_BLOCK;
        Py_XINCREF(_obj);
        SWIG_PYTHON_THREAD_END_BLOCK;
      }
    }
    
    SwigPtr_PyObject & operator=(const SwigPtr_PyObject& item) 
    {
      SWIG_PYTHON_THREAD_BEGIN_BLOCK;
      Py_XINCREF(item._obj);
      Py_XDECREF(_obj);
      _obj = item._obj;
      SWIG_PYTHON_THREAD_END_BLOCK;
      return *this;      
    }
    
    ~SwigPtr_PyObject() 
    {
      SWIG_PYTHON_THREAD_BEGIN_BLOCK;
      Py_XDECREF(_obj);
      SWIG_PYTHON_THREAD_END_BLOCK;
    }
    
    operator PyObject *() const
    {
      return _obj;
    }

    PyObject *operator->() const
    {
      return _obj;
    }
  };
}


namespace swig {
  struct SwigVar_PyObject : SwigPtr_PyObject {
    SwigVar_PyObject(PyObject* obj = 0) : SwigPtr_PyObject(obj, false) { }
    
    SwigVar_PyObject & operator = (PyObject* obj)
    {
      Py_XDECREF(_obj);
      _obj = obj;
      return *this;      
    }
  };
}


#include <RobotRaconteur.h>
#include "RobotRaconteurWrapped.h"
#include <boost/lexical_cast.hpp>
#include <RobotRaconteur/RobotRaconteurServiceIndex_stubskel.h>
#include <boost/foreach.hpp>
using namespace RobotRaconteur;
using namespace std;


SWIGINTERNINLINE PyObject*
  SWIG_From_int  (int value)
{
  return PyInt_FromLong((long) value);
}


#include <string>


#include <iostream>

#if PY_VERSION_HEX >= 0x03020000
# define SWIGPY_SLICE_ARG(obj) ((PyObject*) (obj))
#else
# define SWIGPY_SLICE_ARG(obj) ((PySliceObject*) (obj))
#endif


#include <typeinfo>
#include <stdexcept>


#if defined(__GNUC__)
#  if __GNUC__ == 2 && __GNUC_MINOR <= 96
#     define SWIG_STD_NOMODERN_STL
#  endif
#endif


#include <stddef.h>


namespace swig {
  struct stop_iteration {
  };

  struct SwigPyIterator {
  private:
    SwigPtr_PyObject _seq;

  protected:
    SwigPyIterator(PyObject *seq) : _seq(seq)
    {
    }
      
  public:
    virtual ~SwigPyIterator() {}

    // Access iterator method, required by Python
    virtual PyObject *value() const = 0;

    // Forward iterator method, required by Python
    virtual SwigPyIterator *incr(size_t n = 1) = 0;
    
    // Backward iterator method, very common in C++, but not required in Python
    virtual SwigPyIterator *decr(size_t /*n*/ = 1)
    {
      throw stop_iteration();
    }

    // Random access iterator methods, but not required in Python
    virtual ptrdiff_t distance(const SwigPyIterator &/*x*/) const
    {
      throw std::invalid_argument("operation not supported");
    }

    virtual bool equal (const SwigPyIterator &/*x*/) const
    {
      throw std::invalid_argument("operation not supported");
    }
    
    // C++ common/needed methods
    virtual SwigPyIterator *copy() const = 0;

    PyObject *next()     
    {
      SWIG_PYTHON_THREAD_BEGIN_BLOCK; // disable threads       
      PyObject *obj = value();
      incr();       
      SWIG_PYTHON_THREAD_END_BLOCK; // re-enable threads
      return obj;     
    }

    /* Make an alias for Python 3.x */
    PyObject *__next__()
    {
      return next();
    }

    PyObject *previous()
    {
      SWIG_PYTHON_THREAD_BEGIN_BLOCK; // disable threads       
      decr();
      PyObject *obj = value();
      SWIG_PYTHON_THREAD_END_BLOCK; // re-enable threads       
      return obj;
    }

    SwigPyIterator *advance(ptrdiff_t n)
    {
      return  (n > 0) ?  incr(n) : decr(-n);
    }
      
    bool operator == (const SwigPyIterator& x)  const
    {
      return equal(x);
    }
      
    bool operator != (const SwigPyIterator& x) const
    {
      return ! operator==(x);
    }
      
    SwigPyIterator& operator += (ptrdiff_t n)
    {
      return *advance(n);
    }

    SwigPyIterator& operator -= (ptrdiff_t n)
    {
      return *advance(-n);
    }
      
    SwigPyIterator* operator + (ptrdiff_t n) const
    {
      return copy()->advance(n);
    }

    SwigPyIterator* operator - (ptrdiff_t n) const
    {
      return copy()->advance(-n);
    }
      
    ptrdiff_t operator - (const SwigPyIterator& x) const
    {
      return x.distance(*this);
    }
      
    static swig_type_info* descriptor() {
      static int init = 0;
      static swig_type_info* desc = 0;
      if (!init) {
	desc = SWIG_TypeQuery("swig::SwigPyIterator *");
	init = 1;
      }	
      return desc;
    }    
  };

#if defined(SWIGPYTHON_BUILTIN)
  inline PyObject* make_output_iterator_builtin (PyObject *pyself)
  {
    Py_INCREF(pyself);
    return pyself;
  }
#endif
}


SWIGINTERN int
SWIG_AsVal_double (PyObject *obj, double *val)
{
  int res = SWIG_TypeError;
  if (PyFloat_Check(obj)) {
    if (val) *val = PyFloat_AsDouble(obj);
    return SWIG_OK;
#if PY_VERSION_HEX < 0x03000000
  } else if (PyInt_Check(obj)) {
    if (val) *val = (double) PyInt_AsLong(obj);
    return SWIG_OK;
#endif
  } else if (PyLong_Check(obj)) {
    double v = PyLong_AsDouble(obj);
    if (!PyErr_Occurred()) {
      if (val) *val = v;
      return SWIG_OK;
    } else {
      PyErr_Clear();
    }
  }
#ifdef SWIG_PYTHON_CAST_MODE
  {
    int dispatch = 0;
    double d = PyFloat_AsDouble(obj);
    if (!PyErr_Occurred()) {
      if (val) *val = d;
      return SWIG_AddCast(SWIG_OK);
    } else {
      PyErr_Clear();
    }
    if (!dispatch) {
      long v = PyLong_AsLong(obj);
      if (!PyErr_Occurred()) {
	if (val) *val = v;
	return SWIG_AddCast(SWIG_AddCast(SWIG_OK));
      } else {
	PyErr_Clear();
      }
    }
  }
#endif
  return res;
}


#include <float.h>


#include <math.h>


SWIGINTERNINLINE int
SWIG_CanCastAsInteger(double *d, double min, double max) {
  double x = *d;
  if ((min <= x && x <= max)) {
   double fx = floor(x);
   double cx = ceil(x);
   double rd =  ((x - fx) < 0.5) ? fx : cx; /* simple rint */
   if ((errno == EDOM) || (errno == ERANGE)) {
     errno = 0;
   } else {
     double summ, reps, diff;
     if (rd < x) {
       diff = x - rd;
     } else if (rd > x) {
       diff = rd - x;
     } else {
       return 1;
     }
     summ = rd + x;
     reps = diff/summ;
     if (reps < 8*DBL_EPSILON) {
       *d = rd;
       return 1;
     }
   }
  }
  return 0;
}


SWIGINTERN int
SWIG_AsVal_unsigned_SS_long (PyObject *obj, unsigned long *val) 
{
#if PY_VERSION_HEX < 0x03000000
  if (PyInt_Check(obj)) {
    long v = PyInt_AsLong(obj);
    if (v >= 0) {
      if (val) *val = v;
      return SWIG_OK;
    } else {
      return SWIG_OverflowError;
    }
  } else
#endif
  if (PyLong_Check(obj)) {
    unsigned long v = PyLong_AsUnsignedLong(obj);
    if (!PyErr_Occurred()) {
      if (val) *val = v;
      return SWIG_OK;
    } else {
      PyErr_Clear();
      return SWIG_OverflowError;
    }
  }
#ifdef SWIG_PYTHON_CAST_MODE
  {
    int dispatch = 0;
    unsigned long v = PyLong_AsUnsignedLong(obj);
    if (!PyErr_Occurred()) {
      if (val) *val = v;
      return SWIG_AddCast(SWIG_OK);
    } else {
      PyErr_Clear();
    }
    if (!dispatch) {
      double d;
      int res = SWIG_AddCast(SWIG_AsVal_double (obj,&d));
      if (SWIG_IsOK(res) && SWIG_CanCastAsInteger(&d, 0, ULONG_MAX)) {
	if (val) *val = (unsigned long)(d);
	return res;
      }
    }
  }
#endif
  return SWIG_TypeError;
}


#include <limits.h>
#if !defined(SWIG_NO_LLONG_MAX)
# if !defined(LLONG_MAX) && defined(__GNUC__) && defined (__LONG_LONG_MAX__)
#   define LLONG_MAX __LONG_LONG_MAX__
#   define LLONG_MIN (-LLONG_MAX - 1LL)
#   define ULLONG_MAX (LLONG_MAX * 2ULL + 1ULL)
# endif
#endif


#if defined(LLONG_MAX) && !defined(SWIG_LONG_LONG_AVAILABLE)
#  define SWIG_LONG_LONG_AVAILABLE
#endif


#ifdef SWIG_LONG_LONG_AVAILABLE
SWIGINTERN int
SWIG_AsVal_unsigned_SS_long_SS_long (PyObject *obj, unsigned long long *val)
{
  int res = SWIG_TypeError;
  if (PyLong_Check(obj)) {
    unsigned long long v = PyLong_AsUnsignedLongLong(obj);
    if (!PyErr_Occurred()) {
      if (val) *val = v;
      return SWIG_OK;
    } else {
      PyErr_Clear();
      res = SWIG_OverflowError;
    }
  } else {
    unsigned long v;
    res = SWIG_AsVal_unsigned_SS_long (obj,&v);
    if (SWIG_IsOK(res)) {
      if (val) *val = v;
      return res;
    }
  }
#ifdef SWIG_PYTHON_CAST_MODE
  {
    const double mant_max = 1LL << DBL_MANT_DIG;
    double d;
    res = SWIG_AsVal_double (obj,&d);
    if (SWIG_IsOK(res) && !SWIG_CanCastAsInteger(&d, 0, mant_max))
      return SWIG_OverflowError;
    if (SWIG_IsOK(res) && SWIG_CanCastAsInteger(&d, 0, mant_max)) {
      if (val) *val = (unsigned long long)(d);
      return SWIG_AddCast(res);
    }
    res = SWIG_TypeError;
  }
#endif
  return res;
}
#endif


SWIGINTERNINLINE int
SWIG_AsVal_size_t (PyObject * obj, size_t *val)
{
  int res = SWIG_TypeError;
#ifdef SWIG_LONG_LONG_AVAILABLE
  if (sizeof(size_t) <= sizeof(unsigned long)) {
#endif
    unsigned long v;
    res = SWIG_AsVal_unsigned_SS_long (obj, val ? &v : 0);
    if (SWIG_IsOK(res) && val) *val = static_cast< size_t >(v);
#ifdef SWIG_LONG_LONG_AVAILABLE
  } else if (sizeof(size_t) <= sizeof(unsigned long long)) {
    unsigned long long v;
    res = SWIG_AsVal_unsigned_SS_long_SS_long (obj, val ? &v : 0);
    if (SWIG_IsOK(res) && val) *val = static_cast< size_t >(v);
  }
#endif
  return res;
}


  #define SWIG_From_long   PyInt_FromLong 


#ifdef SWIG_LONG_LONG_AVAILABLE
SWIGINTERNINLINE PyObject* 
SWIG_From_long_SS_long  (long long value)
{
  return ((value < LONG_MIN) || (value > LONG_MAX)) ?
    PyLong_FromLongLong(value) : PyInt_FromLong(static_cast< long >(value));
}
#endif


SWIGINTERNINLINE PyObject *
SWIG_From_ptrdiff_t  (ptrdiff_t value)
{    
#ifdef SWIG_LONG_LONG_AVAILABLE
  if (sizeof(ptrdiff_t) <= sizeof(long)) {
#endif
    return SWIG_From_long  (static_cast< long >(value));
#ifdef SWIG_LONG_LONG_AVAILABLE
  } else {
    /* assume sizeof(ptrdiff_t) <= sizeof(long long) */
    return SWIG_From_long_SS_long  (static_cast< long long >(value));
  }
#endif
}


SWIGINTERNINLINE PyObject*
  SWIG_From_bool  (bool value)
{
  return PyBool_FromLong(value ? 1 : 0);
}


SWIGINTERN int
SWIG_AsVal_long (PyObject *obj, long* val)
{
#if PY_VERSION_HEX < 0x03000000
  if (PyInt_Check(obj)) {
    if (val) *val = PyInt_AsLong(obj);
    return SWIG_OK;
  } else
#endif
  if (PyLong_Check(obj)) {
    long v = PyLong_AsLong(obj);
    if (!PyErr_Occurred()) {
      if (val) *val = v;
      return SWIG_OK;
    } else {
      PyErr_Clear();
      return SWIG_OverflowError;
    }
  }
#ifdef SWIG_PYTHON_CAST_MODE
  {
    int dispatch = 0;
    long v = PyInt_AsLong(obj);
    if (!PyErr_Occurred()) {
      if (val) *val = v;
      return SWIG_AddCast(SWIG_OK);
    } else {
      PyErr_Clear();
    }
    if (!dispatch) {
      double d;
      int res = SWIG_AddCast(SWIG_AsVal_double (obj,&d));
      if (SWIG_IsOK(res) && SWIG_CanCastAsInteger(&d, LONG_MIN, LONG_MAX)) {
	if (val) *val = (long)(d);
	return res;
      }
    }
  }
#endif
  return SWIG_TypeError;
}


#ifdef SWIG_LONG_LONG_AVAILABLE
SWIGINTERN int
SWIG_AsVal_long_SS_long (PyObject *obj, long long *val)
{
  int res = SWIG_TypeError;
  if (PyLong_Check(obj)) {
    long long v = PyLong_AsLongLong(obj);
    if (!PyErr_Occurred()) {
      if (val) *val = v;
      return SWIG_OK;
    } else {
      PyErr_Clear();
      res = SWIG_OverflowError;
    }
  } else {
    long v;
    res = SWIG_AsVal_long (obj,&v);
    if (SWIG_IsOK(res)) {
      if (val) *val = v;
      return res;
    }
  }
#ifdef SWIG_PYTHON_CAST_MODE
  {
    const double mant_max = 1LL << DBL_MANT_DIG;
    const double mant_min = -mant_max;
    double d;
    res = SWIG_AsVal_double (obj,&d);
    if (SWIG_IsOK(res) && !SWIG_CanCastAsInteger(&d, mant_min, mant_max))
      return SWIG_OverflowError;
    if (SWIG_IsOK(res) && SWIG_CanCastAsInteger(&d, mant_min, mant_max)) {
      if (val) *val = (long long)(d);
      return SWIG_AddCast(res);
    }
    res = SWIG_TypeError;
  }
#endif
  return res;
}
#endif


SWIGINTERNINLINE int
SWIG_AsVal_ptrdiff_t (PyObject * obj, ptrdiff_t *val)
{
  int res = SWIG_TypeError;
#ifdef SWIG_LONG_LONG_AVAILABLE
  if (sizeof(ptrdiff_t) <= sizeof(long)) {
#endif
    long v;
    res = SWIG_AsVal_long (obj, val ? &v : 0);
    if (SWIG_IsOK(res) && val) *val = static_cast< ptrdiff_t >(v);
#ifdef SWIG_LONG_LONG_AVAILABLE
  } else if (sizeof(ptrdiff_t) <= sizeof(long long)) {
    long long v;
    res = SWIG_AsVal_long_SS_long (obj, val ? &v : 0);
    if (SWIG_IsOK(res) && val) *val = static_cast< ptrdiff_t >(v);
  }
#endif
  return res;
}


#include <algorithm>


#include <vector>


#include <utility>


#include <map>


#include <algorithm>


#include <stdint.h>		// Use the C99 official header


#define SWIGWORDSIZE64
#ifndef LONG_MAX
#include <limits.h>
#endif
#if (__WORDSIZE == 32) || (LONG_MAX == INT_MAX)
# error "SWIG wrapped code invalid in 32 bit architecture, regenerate code using -DSWIGWORDSIZE32"
#endif


#include "PythonTypeSupport.h"


#include <datetime.h>


#include <boost/filesystem/path.hpp>


SWIGINTERN swig_type_info*
SWIG_pchar_descriptor(void)
{
  static int init = 0;
  static swig_type_info* info = 0;
  if (!init) {
    info = SWIG_TypeQuery("_p_char");
    init = 1;
  }
  return info;
}


SWIGINTERN int
SWIG_AsCharPtrAndSize(PyObject *obj, char** cptr, size_t* psize, int *alloc)
{
#if PY_VERSION_HEX>=0x03000000
#if defined(SWIG_PYTHON_STRICT_BYTE_CHAR)
  if (PyBytes_Check(obj))
#else
  if (PyUnicode_Check(obj))
#endif
#else  
  if (PyString_Check(obj))
#endif
  {
    char *cstr; Py_ssize_t len;
    int ret = SWIG_OK;
#if PY_VERSION_HEX>=0x03000000
#if !defined(SWIG_PYTHON_STRICT_BYTE_CHAR)
    if (!alloc && cptr) {
        /* We can't allow converting without allocation, since the internal
           representation of string in Python 3 is UCS-2/UCS-4 but we require
           a UTF-8 representation.
           TODO(bhy) More detailed explanation */
        return SWIG_RuntimeError;
    }
    obj = PyUnicode_AsUTF8String(obj);
    if (!obj)
      return SWIG_TypeError;
    if (alloc)
      *alloc = SWIG_NEWOBJ;
#endif
    if (PyBytes_AsStringAndSize(obj, &cstr, &len) == -1)
      return SWIG_TypeError;
#else
    if (PyString_AsStringAndSize(obj, &cstr, &len) == -1)
      return SWIG_TypeError;
#endif
    if (cptr) {
      if (alloc) {
	if (*alloc == SWIG_NEWOBJ) {
	  *cptr = reinterpret_cast< char* >(memcpy(new char[len + 1], cstr, sizeof(char)*(len + 1)));
	  *alloc = SWIG_NEWOBJ;
	} else {
	  *cptr = cstr;
	  *alloc = SWIG_OLDOBJ;
	}
      } else {
#if PY_VERSION_HEX>=0x03000000
#if defined(SWIG_PYTHON_STRICT_BYTE_CHAR)
	*cptr = PyBytes_AsString(obj);
#else
	assert(0); /* Should never reach here with Unicode strings in Python 3 */
#endif
#else
	*cptr = SWIG_Python_str_AsChar(obj);
        if (!*cptr)
          ret = SWIG_TypeError;
#endif
      }
    }
    if (psize) *psize = len + 1;
#if PY_VERSION_HEX>=0x03000000 && !defined(SWIG_PYTHON_STRICT_BYTE_CHAR)
    Py_XDECREF(obj);
#endif
    return ret;
  } else {
#if defined(SWIG_PYTHON_2_UNICODE)
#if defined(SWIG_PYTHON_STRICT_BYTE_CHAR)
#error "Cannot use both SWIG_PYTHON_2_UNICODE and SWIG_PYTHON_STRICT_BYTE_CHAR at once"
#endif
#if PY_VERSION_HEX<0x03000000
    if (PyUnicode_Check(obj)) {
      char *cstr; Py_ssize_t len;
      if (!alloc && cptr) {
        return SWIG_RuntimeError;
      }
      obj = PyUnicode_AsUTF8String(obj);
      if (!obj)
        return SWIG_TypeError;
      if (PyString_AsStringAndSize(obj, &cstr, &len) != -1) {
        if (cptr) {
          if (alloc) *alloc = SWIG_NEWOBJ;
          *cptr = reinterpret_cast< char* >(memcpy(new char[len + 1], cstr, sizeof(char)*(len + 1)));
        }
        if (psize) *psize = len + 1;

        Py_XDECREF(obj);
        return SWIG_OK;
      } else {
        Py_XDECREF(obj);
      }
    }
#endif
#endif

    swig_type_info* pchar_descriptor = SWIG_pchar_descriptor();
    if (pchar_descriptor) {
      void* vptr = 0;
      if (SWIG_ConvertPtr(obj, &vptr, pchar_descriptor, 0) == SWIG_OK) {
	if (cptr) *cptr = (char *) vptr;
	if (psize) *psize = vptr ? (strlen((char *)vptr) + 1) : 0;
	if (alloc) *alloc = SWIG_OLDOBJ;
	return SWIG_OK;
      }
    }
  }
  return SWIG_TypeError;
}


SWIGINTERN int
SWIG_AsPtr_std_string (PyObject * obj, std::string **val) 
{
  char* buf = 0 ; size_t size = 0; int alloc = SWIG_OLDOBJ;
  if (SWIG_IsOK((SWIG_AsCharPtrAndSize(obj, &buf, &size, &alloc)))) {
    if (buf) {
      if (val) *val = new std::string(buf, size - 1);
      if (alloc == SWIG_NEWOBJ) delete[] buf;
      return SWIG_NEWOBJ;
    } else {
      if (val) *val = 0;
      return SWIG_OLDOBJ;
    }
  } else {
    static int init = 0;
    static swig_type_info* descriptor = 0;
    if (!init) {
      descriptor = SWIG_TypeQuery("std::string" " *");
      init = 1;
    }
    if (descriptor) {
      std::string *vptr;
      int res = SWIG_ConvertPtr(obj, (void**)&vptr, descriptor, 0);
      if (SWIG_IsOK(res) && val) *val = vptr;
      return res;
    }
  }
  return SWIG_ERROR;
}


SWIGINTERNINLINE PyObject *
SWIG_FromCharPtrAndSize(const char* carray, size_t size)
{
  if (carray) {
    if (size > INT_MAX) {
      swig_type_info* pchar_descriptor = SWIG_pchar_descriptor();
      return pchar_descriptor ? 
	SWIG_InternalNewPointerObj(const_cast< char * >(carray), pchar_descriptor, 0) : SWIG_Py_Void();
    } else {
#if PY_VERSION_HEX >= 0x03000000
#if defined(SWIG_PYTHON_STRICT_BYTE_CHAR)
      return PyBytes_FromStringAndSize(carray, static_cast< Py_ssize_t >(size));
#else
      return PyUnicode_DecodeUTF8(carray, static_cast< Py_ssize_t >(size), "surrogateescape");
#endif
#else
      return PyString_FromStringAndSize(carray, static_cast< Py_ssize_t >(size));
#endif
    }
  } else {
    return SWIG_Py_Void();
  }
}


SWIGINTERNINLINE PyObject *
SWIG_From_std_string  (const std::string& s)
{
  return SWIG_FromCharPtrAndSize(s.data(), s.size());
}



static void RRExceptionToPythonError(RobotRaconteurException& rrexp)
{
	PyObject* modules_dict= PyImport_GetModuleDict();
	if (modules_dict==NULL)
	{
		PyErr_SetString(PyExc_Exception, "Could not load RobotRaconteurPythonError module");
		return;
	}

	PyObject* err_module=PyDict_GetItemString(modules_dict, "RobotRaconteur.RobotRaconteurPythonError");
	if (err_module==NULL)
	{
		PyErr_SetString(PyExc_Exception, "Could not load RobotRaconteurPythonError module");
		return;
	}

	swig::SwigVar_PyObject exceptionUtil=PyObject_GetAttrString(err_module, "RobotRaconteurExceptionUtil");
	if (exceptionUtil==NULL)
	{
		PyErr_SetString(PyExc_Exception, "Could not load RobotRaconteurPythonError.RobotRaconteurExceptionUtil class");
		return;
	}

	swig::SwigVar_PyObject errorCodeToException=PyObject_GetAttrString(exceptionUtil, "ErrorInfoToException");
	if (errorCodeToException==NULL)
	{
		PyErr_SetString(PyExc_Exception, "Could not load RobotRaconteurExceptionUtil.ErrorInfoToException function");
		return;
	}

	RobotRaconteur::HandlerErrorInfo error(rrexp);

	swig::SwigVar_PyObject error_obj;
  	error_obj = SWIG_NewPointerObj(SWIG_as_voidptr(&error), SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0 );

	swig::SwigVar_PyObject pyErr=PyObject_CallFunction(errorCodeToException, "O", (PyObject*)error_obj);
	swig::SwigVar_PyObject pyErrType=PyObject_Type(pyErr);

	PyErr_SetObject(pyErrType,pyErr);
}




static void ThrowPythonError()
{
	PyObject* exc1;
	PyObject* val1;
	PyObject* tb1;
	PyErr_Fetch(&exc1,&val1,&tb1);
	PyErr_NormalizeException(&exc1,&val1,&tb1);
	swig::SwigVar_PyObject exc = exc1;
	swig::SwigVar_PyObject val = val1;
	swig::SwigVar_PyObject tb = tb1;

	if (RobotRaconteur::PythonTracebackPrintExc)
	{
		std::cerr << "RobotRaconteurPython caught exception:" << std::endl;
		if (exc && val && tb)
		{
			PyErr_Display(exc,val,tb);
		}
		PyErr_Clear();
	}
	else
	{
		PyErr_Clear();
	}

	PyObject* modules_dict= PyImport_GetModuleDict();
	if (modules_dict==NULL)
	{
		throw InternalErrorException("Could not load RobotRaconteurPythonError module");
	}

	PyObject* err_module=PyDict_GetItemString(modules_dict, "RobotRaconteur.RobotRaconteurPythonError");
	if (err_module==NULL)
	{
		throw InternalErrorException("Could not load RobotRaconteurPythonError module");
	}

	swig::SwigVar_PyObject rr_py_RobotRaconteurException=PyObject_GetAttrString(err_module, "RobotRaconteurException");
	if (rr_py_RobotRaconteurException==NULL)
	{
		throw InternalErrorException("Could not load RobotRaconteurPythonError.RobotRaconteurException type");
	}

	if (PyErr_GivenExceptionMatches(exc,rr_py_RobotRaconteurException))
	{
		swig::SwigVar_PyObject message1 = PyObject_GetAttrString(val,"message");
		swig::SwigVar_PyObject errorname1 = PyObject_GetAttrString(val,"errorname");
		swig::SwigVar_PyObject errorcode1 = PyObject_GetAttrString(val,"errorcode");

		swig::SwigVar_PyObject message=PyObject_Str(message1);
		swig::SwigVar_PyObject errorname=PyObject_Str(errorname1);
		swig::SwigVar_PyObject errorcode=PyNumber_Long(errorcode1);

		if (!(PyObject*)message || !(PyObject*)errorcode || !(PyObject*)errorname)
		{
			throw InternalErrorException("Exception occurred in Python code");
		}

		std::string message2 = PyObjectToUTF8(message);
		std::string errorname2=PyObjectToUTF8(errorname);
		long errorcode2=PyLong_AsLong(errorcode);
		if (errorcode2 > 65535 || errorcode2 < 0) errorcode2=MessageErrorType_UnknownError;

		boost::intrusive_ptr<MessageEntry> m=CreateMessageEntry();
		m->Error=(MessageErrorType)(uint16_t)errorcode2;
		m->AddElement("errorname",stringToRRArray(errorname2));
		m->AddElement("errorstring", stringToRRArray(message2));

		swig::SwigVar_PyObject errorsubname = PyObject_GetAttrString(val,"errorsubname");
		if ((PyObject*)errorsubname && (PyObject*)errorsubname != Py_None)
		{
			swig::SwigVar_PyObject errorsubname2 = PyObject_Str(errorsubname);
			if ((PyObject*)errorsubname2)
			{
				m->AddElement("errorsubname",stringToRRArray(PyObjectToUTF8(errorsubname2)));
			}
		}

		swig::SwigVar_PyObject errorparam = PyObject_GetAttrString(val,"errorparam");
		if ((PyObject*)errorparam && (PyObject*)errorparam != Py_None)
		{
			PyObject* util_module=PyDict_GetItemString(modules_dict, "RobotRaconteur.RobotRaconteurPythonUtil");
			if (util_module)
			{
				swig::SwigVar_PyObject python_pack_element=PyObject_GetAttrString(util_module, "PackMessageElement");
				if ((PyObject*)python_pack_element)
				{
					swig::SwigVar_PyObject py_errorparam_elem = PyObject_CallFunction(python_pack_element,"OsOO",(PyObject*)errorparam,"varvalue errorparam",Py_None,Py_None);
					if ((PyObject*)py_errorparam_elem)
					{
						try
						{
							boost::intrusive_ptr< RobotRaconteur::MessageElement > arg1 ;
							void *argp1 ;
							int newmem1 = 0 ;
							// Extract swig pointer
							int res1 = SWIG_ConvertPtrAndOwn(py_errorparam_elem, &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem1);
							if (SWIG_IsOK(res1)) {
								if (argp1) arg1 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp1)->get(),true);
								if (newmem1 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp1);
								if (arg1)
								{
									arg1->ElementName = "errorparam";
									m->AddElement(arg1);
								}
							}

						}
						catch (std::exception)
						{
							//TODO: log error
						}
					}
				}
			}
		}


		RobotRaconteurExceptionUtil::ThrowMessageEntryException(m);
	}
	else
	{
		swig::SwigVar_PyObject str=PyObject_Str(val);
		swig::SwigVar_PyObject strname1 = PyObject_GetAttrString(exc,"__name__");
		swig::SwigVar_PyObject strname=PyObject_Str(strname1);
		std::string str2=PyObjectToUTF8(str);
		std::string strname2=PyObjectToUTF8(strname);

		throw UnknownException(strname2, str2);
	}
}



SWIGINTERNINLINE PyObject* 
SWIG_From_unsigned_SS_long  (unsigned long value)
{
  return (value > LONG_MAX) ?
    PyLong_FromUnsignedLong(value) : PyInt_FromLong(static_cast< long >(value));
}


SWIGINTERNINLINE PyObject *
SWIG_From_unsigned_SS_char  (unsigned char value)
{    
  return SWIG_From_unsigned_SS_long  (value);
}


SWIGINTERNINLINE PyObject*
  SWIG_From_unsigned_SS_int  (unsigned int value)
{
  return PyInt_FromSize_t((size_t) value);
}


namespace swig {
  template <class Type>
  struct noconst_traits {
    typedef Type noconst_type;
  };

  template <class Type>
  struct noconst_traits<const Type> {
    typedef Type noconst_type;
  };

  /*
    type categories
  */
  struct pointer_category { };
  struct value_category { };

  /*
    General traits that provides type_name and type_info
  */
  template <class Type> struct traits { };

  template <class Type>
  inline const char* type_name() {
    return traits<typename noconst_traits<Type >::noconst_type >::type_name();
  }

  template <class Type> struct traits_info {
    static swig_type_info *type_query(std::string name) {
      name += " *";
      return SWIG_TypeQuery(name.c_str());
    }
    static swig_type_info *type_info() {
      static swig_type_info *info = type_query(type_name<Type>());
      return info;
    }
  };

  /*
    Partial specialization for pointers (traits_info)
  */
  template <class Type> struct traits_info<Type *> {
    static swig_type_info *type_query(std::string name) {
      name += " *";
      return SWIG_TypeQuery(name.c_str());
    }
    static swig_type_info *type_info() {
      static swig_type_info *info = type_query(type_name<Type>());
      return info;
    }
  };

  template <class Type>
  inline swig_type_info *type_info() {
    return traits_info<Type>::type_info();
  }

  /*
    Partial specialization for pointers (traits)
  */
  template <class Type> struct traits <Type *> {
    typedef pointer_category category;
    static std::string make_ptr_name(const char* name) {
      std::string ptrname = name;
      ptrname += " *";
      return ptrname;
    }
    static const char* type_name() {
      static std::string name = make_ptr_name(swig::type_name<Type>());
      return name.c_str();
    }
  };

  template <class Type, class Category>
  struct traits_as { };

  template <class Type, class Category>
  struct traits_check { };

}


namespace swig {  
  /*
    Traits that provides the from method
  */
  template <class Type> struct traits_from_ptr {
    static PyObject *from(Type *val, int owner = 0) {
      return SWIG_InternalNewPointerObj(val, type_info<Type>(), owner);
    }
  };

  template <class Type> struct traits_from {
    static PyObject *from(const Type& val) {
      return traits_from_ptr<Type>::from(new Type(val), 1);
    }
  };

  template <class Type> struct traits_from<Type *> {
    static PyObject *from(Type* val) {
      return traits_from_ptr<Type>::from(val, 0);
    }
  };

  template <class Type> struct traits_from<const Type *> {
    static PyObject *from(const Type* val) {
      return traits_from_ptr<Type>::from(const_cast<Type*>(val), 0);
    }
  };


  template <class Type>
  inline PyObject *from(const Type& val) {
    return traits_from<Type>::from(val);
  }

  template <class Type>
  inline PyObject *from_ptr(Type* val, int owner) {
    return traits_from_ptr<Type>::from(val, owner);
  }

  /*
    Traits that provides the asval/as/check method
  */
  template <class Type>
  struct traits_asptr {   
    static int asptr(PyObject *obj, Type **val) {
      int res = SWIG_ERROR;
      swig_type_info *descriptor = type_info<Type>();
      if (val) {
        Type *p = 0;
        int newmem = 0;
        res = descriptor ? SWIG_ConvertPtrAndOwn(obj, (void **)&p, descriptor, 0, &newmem) : SWIG_ERROR;
        if (SWIG_IsOK(res)) {
          if (newmem & SWIG_CAST_NEW_MEMORY) {
            res |= SWIG_NEWOBJMASK;
          }
          *val = p;
        }
      } else {
        res = descriptor ? SWIG_ConvertPtr(obj, 0, descriptor, 0) : SWIG_ERROR;
      }
      return res;
    }
  }; 

  template <class Type>
  inline int asptr(PyObject *obj, Type **vptr) {
    return traits_asptr<Type>::asptr(obj, vptr);
  }

  template <class Type> 
  struct traits_asval {
    static int asval(PyObject *obj, Type *val) {
      if (val) {
	Type *p = 0;
	int res = traits_asptr<Type>::asptr(obj, &p);
	if (!SWIG_IsOK(res)) return res;	
	if (p) {
	  typedef typename noconst_traits<Type>::noconst_type noconst_type;
	  *(const_cast<noconst_type*>(val)) = *p;
	  if (SWIG_IsNewObj(res)){
	    delete p;
	    res = SWIG_DelNewMask(res);
	  }
	  return res;
	} else {
	  return SWIG_ERROR;
	}
      } else {
	return traits_asptr<Type>::asptr(obj, (Type **)(0));
      }
    }
  };

  template <class Type> struct traits_asval<Type*> {
    static int asval(PyObject *obj, Type **val) {
      if (val) {
        typedef typename noconst_traits<Type>::noconst_type noconst_type;
        noconst_type *p = 0;
        int res = traits_asptr<noconst_type>::asptr(obj,  &p);
        if (SWIG_IsOK(res)) {
          *(const_cast<noconst_type**>(val)) = p;
	}
	return res;
      } else {
	return traits_asptr<Type>::asptr(obj, (Type **)(0));
      }
    }
  };
  
  template <class Type>
  inline int asval(PyObject *obj, Type *val) {
    return traits_asval<Type>::asval(obj, val);
  }

  template <class Type> 
  struct traits_as<Type, value_category> {
    static Type as(PyObject *obj) {
      Type v;
      int res = asval(obj, &v);
      if (!obj || !SWIG_IsOK(res)) {
	if (!PyErr_Occurred()) {
	  ::SWIG_Error(SWIG_TypeError,  swig::type_name<Type>());
	}
	throw std::invalid_argument("bad type");
      }
      return v;
    }
  };

  template <class Type> 
  struct traits_as<Type, pointer_category> {
    static Type as(PyObject *obj) {
      Type *v = 0;      
      int res = (obj ? traits_asptr<Type>::asptr(obj, &v) : SWIG_ERROR);
      if (SWIG_IsOK(res) && v) {
	if (SWIG_IsNewObj(res)) {
	  Type r(*v);
	  delete v;
	  return r;
	} else {
	  return *v;
	}
      } else {
	if (!PyErr_Occurred()) {
	  SWIG_Error(SWIG_TypeError,  swig::type_name<Type>());
	}
	throw std::invalid_argument("bad type");
      }
    }
  };

  template <class Type> 
  struct traits_as<Type*, pointer_category> {
    static Type* as(PyObject *obj) {
      Type *v = 0;      
      int res = (obj ? traits_asptr<Type>::asptr(obj, &v) : SWIG_ERROR);
      if (SWIG_IsOK(res)) {
	return v;
      } else {
	if (!PyErr_Occurred()) {
	  SWIG_Error(SWIG_TypeError,  swig::type_name<Type>());
	}
	throw std::invalid_argument("bad type");
      }
    }
  };
    
  template <class Type>
  inline Type as(PyObject *obj) {
    return traits_as<Type, typename traits<Type>::category>::as(obj);
  }

  template <class Type> 
  struct traits_check<Type, value_category> {
    static bool check(PyObject *obj) {
      int res = obj ? asval(obj, (Type *)(0)) : SWIG_ERROR;
      return SWIG_IsOK(res) ? true : false;
    }
  };

  template <class Type> 
  struct traits_check<Type, pointer_category> {
    static bool check(PyObject *obj) {
      int res = obj ? asptr(obj, (Type **)(0)) : SWIG_ERROR;
      return SWIG_IsOK(res) ? true : false;
    }
  };

  template <class Type>
  inline bool check(PyObject *obj) {
    return traits_check<Type, typename traits<Type>::category>::check(obj);
  }
}


#include <functional>

namespace std {
  template <>
  struct less <PyObject *>
  {
    bool
    operator()(PyObject * v, PyObject *w) const
    { 
      bool res;
      SWIG_PYTHON_THREAD_BEGIN_BLOCK;
      res = PyObject_RichCompareBool(v, w, Py_LT) ? true : false;
      /* This may fall into a case of inconsistent
               eg. ObjA > ObjX > ObjB
               but ObjA < ObjB
      */
      if( PyErr_Occurred() && PyErr_ExceptionMatches(PyExc_TypeError) )
      {
        /* Objects can't be compared, this mostly occurred in Python 3.0 */
        /* Compare their ptr directly for a workaround */
        res = (v < w);
        PyErr_Clear();
      }
      SWIG_PYTHON_THREAD_END_BLOCK;
      return res;
    }
  };

  template <>
  struct less <swig::SwigPtr_PyObject>
  {
    bool
    operator()(const swig::SwigPtr_PyObject& v, const swig::SwigPtr_PyObject& w) const
    {
      return std::less<PyObject *>()(v, w);
    }
  };

  template <>
  struct less <swig::SwigVar_PyObject>
  {
    bool
    operator()(const swig::SwigVar_PyObject& v, const swig::SwigVar_PyObject& w) const
    {
      return std::less<PyObject *>()(v, w);
    }
  };

}

namespace swig {
  template <> struct traits<PyObject *> {
    typedef value_category category;
    static const char* type_name() { return "PyObject *"; }
  };  

  template <>  struct traits_asval<PyObject * > {   
    typedef PyObject * value_type;
    static int asval(PyObject *obj, value_type *val) {
      if (val) *val = obj;
      return SWIG_OK;
    }
  };

  template <> 
  struct traits_check<PyObject *, value_category> {
    static bool check(PyObject *) {
      return true;
    }
  };

  template <>  struct traits_from<PyObject *> {
    typedef PyObject * value_type;
    static PyObject *from(const value_type& val) {
      Py_XINCREF(val);
      return val;
    }
  };
  
}

namespace swig {
  template <class Difference>
  inline size_t
  check_index(Difference i, size_t size, bool insert = false) {
    if ( i < 0 ) {
      if ((size_t) (-i) <= size)
	return (size_t) (i + size);
    } else if ( (size_t) i < size ) {
      return (size_t) i;
    } else if (insert && ((size_t) i == size)) {
      return size;
    }
    throw std::out_of_range("index out of range");
  }

  template <class Difference>
  void
  slice_adjust(Difference i, Difference j, Py_ssize_t step, size_t size, Difference &ii, Difference &jj, bool insert = false) {
    if (step == 0) {
      throw std::invalid_argument("slice step cannot be zero");
    } else if (step > 0) {
      // Required range: 0 <= i < size, 0 <= j < size, i <= j
      if (i < 0) {
        ii = 0;
      } else if (i < (Difference)size) {
        ii = i;
      } else if (insert && (i >= (Difference)size)) {
        ii = (Difference)size;
      }
      if (j < 0) {
        jj = 0;
      } else {
        jj = (j < (Difference)size) ? j : (Difference)size;
      }
      if (jj < ii)
        jj = ii;
    } else {
      // Required range: -1 <= i < size-1, -1 <= j < size-1, i >= j
      if (i < -1) {
        ii = -1;
      } else if (i < (Difference) size) {
        ii = i;
      } else if (i >= (Difference)(size-1)) {
        ii = (Difference)(size-1);
      }
      if (j < -1) {
        jj = -1;
      } else {
        jj = (j < (Difference)size ) ? j : (Difference)(size-1);
      }
      if (ii < jj)
        ii = jj;
    }
  }

  template <class Sequence, class Difference>
  inline typename Sequence::iterator
  getpos(Sequence* self, Difference i)  {
    typename Sequence::iterator pos = self->begin();
    std::advance(pos, check_index(i,self->size()));
    return pos;
  }

  template <class Sequence, class Difference>
  inline typename Sequence::const_iterator
  cgetpos(const Sequence* self, Difference i)  {
    typename Sequence::const_iterator pos = self->begin();
    std::advance(pos, check_index(i,self->size()));
    return pos;
  }

  template <class Sequence>
  inline void
  erase(Sequence* seq, const typename Sequence::iterator& position) {
    seq->erase(position);
  }

  template <class Sequence>
  struct traits_reserve {
    static void reserve(Sequence & /*seq*/, typename Sequence::size_type /*n*/) {
      // This should be specialized for types that support reserve
    }
  };

  template <class Sequence, class Difference>
  inline Sequence*
  getslice(const Sequence* self, Difference i, Difference j, Py_ssize_t step) {
    typename Sequence::size_type size = self->size();
    Difference ii = 0;
    Difference jj = 0;
    swig::slice_adjust(i, j, step, size, ii, jj);

    if (step > 0) {
      typename Sequence::const_iterator sb = self->begin();
      typename Sequence::const_iterator se = self->begin();
      std::advance(sb,ii);
      std::advance(se,jj);
      if (step == 1) {
        return new Sequence(sb, se);
      } else {
        Sequence *sequence = new Sequence();
        swig::traits_reserve<Sequence>::reserve(*sequence, (jj - ii + step - 1) / step);
        typename Sequence::const_iterator it = sb;
        while (it!=se) {
          sequence->push_back(*it);
          for (Py_ssize_t c=0; c<step && it!=se; ++c)
            it++;
        }
        return sequence;
      } 
    } else {
      Sequence *sequence = new Sequence();
      swig::traits_reserve<Sequence>::reserve(*sequence, (ii - jj - step - 1) / -step);
      typename Sequence::const_reverse_iterator sb = self->rbegin();
      typename Sequence::const_reverse_iterator se = self->rbegin();
      std::advance(sb,size-ii-1);
      std::advance(se,size-jj-1);
      typename Sequence::const_reverse_iterator it = sb;
      while (it!=se) {
        sequence->push_back(*it);
        for (Py_ssize_t c=0; c<-step && it!=se; ++c)
          it++;
      }
      return sequence;
    }
  }

  template <class Sequence, class Difference, class InputSeq>
  inline void
  setslice(Sequence* self, Difference i, Difference j, Py_ssize_t step, const InputSeq& is = InputSeq()) {
    typename Sequence::size_type size = self->size();
    Difference ii = 0;
    Difference jj = 0;
    swig::slice_adjust(i, j, step, size, ii, jj, true);
    if (step > 0) {
      if (step == 1) {
        size_t ssize = jj - ii;
        if (ssize <= is.size()) {
          // expanding/staying the same size
          swig::traits_reserve<Sequence>::reserve(*self, self->size() - ssize + is.size());
          typename Sequence::iterator sb = self->begin();
          typename InputSeq::const_iterator isit = is.begin();
          std::advance(sb,ii);
          std::advance(isit, jj - ii);
          self->insert(std::copy(is.begin(), isit, sb), isit, is.end());
        } else {
          // shrinking
          typename Sequence::iterator sb = self->begin();
          typename Sequence::iterator se = self->begin();
          std::advance(sb,ii);
          std::advance(se,jj);
          self->erase(sb,se);
          sb = self->begin();
          std::advance(sb,ii);
          self->insert(sb, is.begin(), is.end());
        }
      } else {
        size_t replacecount = (jj - ii + step - 1) / step;
        if (is.size() != replacecount) {
          char msg[1024];
          sprintf(msg, "attempt to assign sequence of size %lu to extended slice of size %lu", (unsigned long)is.size(), (unsigned long)replacecount);
          throw std::invalid_argument(msg);
        }
        typename Sequence::const_iterator isit = is.begin();
        typename Sequence::iterator it = self->begin();
        std::advance(it,ii);
        for (size_t rc=0; rc<replacecount && it != self->end(); ++rc) {
          *it++ = *isit++;
          for (Py_ssize_t c=0; c<(step-1) && it != self->end(); ++c)
            it++;
        }
      }
    } else {
      size_t replacecount = (ii - jj - step - 1) / -step;
      if (is.size() != replacecount) {
        char msg[1024];
        sprintf(msg, "attempt to assign sequence of size %lu to extended slice of size %lu", (unsigned long)is.size(), (unsigned long)replacecount);
        throw std::invalid_argument(msg);
      }
      typename Sequence::const_iterator isit = is.begin();
      typename Sequence::reverse_iterator it = self->rbegin();
      std::advance(it,size-ii-1);
      for (size_t rc=0; rc<replacecount && it != self->rend(); ++rc) {
        *it++ = *isit++;
        for (Py_ssize_t c=0; c<(-step-1) && it != self->rend(); ++c)
          it++;
      }
    }
  }

  template <class Sequence, class Difference>
  inline void
  delslice(Sequence* self, Difference i, Difference j, Py_ssize_t step) {
    typename Sequence::size_type size = self->size();
    Difference ii = 0;
    Difference jj = 0;
    swig::slice_adjust(i, j, step, size, ii, jj, true);
    if (step > 0) {
      typename Sequence::iterator sb = self->begin();
      std::advance(sb,ii);
      if (step == 1) {
        typename Sequence::iterator se = self->begin();
        std::advance(se,jj);
        self->erase(sb,se);
      } else {
        typename Sequence::iterator it = sb;
        size_t delcount = (jj - ii + step - 1) / step;
        while (delcount) {
          it = self->erase(it);
          for (Py_ssize_t c=0; c<(step-1) && it != self->end(); ++c)
            it++;
          delcount--;
        }
      }
    } else {
      typename Sequence::reverse_iterator sb = self->rbegin();
      std::advance(sb,size-ii-1);
      typename Sequence::reverse_iterator it = sb;
      size_t delcount = (ii - jj - step - 1) / -step;
      while (delcount) {
        it = typename Sequence::reverse_iterator(self->erase((++it).base()));
        for (Py_ssize_t c=0; c<(-step-1) && it != self->rend(); ++c)
          it++;
        delcount--;
      }
    }
  }
}


#if defined(__SUNPRO_CC) && defined(_RWSTD_VER)
#  if !defined(SWIG_NO_STD_NOITERATOR_TRAITS_STL)
#    define SWIG_STD_NOITERATOR_TRAITS_STL
#  endif
#endif

#if !defined(SWIG_STD_NOITERATOR_TRAITS_STL)
#include <iterator>
#else
namespace std {
  template <class Iterator>
  struct iterator_traits {
    typedef ptrdiff_t difference_type;
    typedef typename Iterator::value_type value_type;
  };

  template <class Iterator, class Category,class T, class Reference, class Pointer, class Distance>
  struct iterator_traits<__reverse_bi_iterator<Iterator,Category,T,Reference,Pointer,Distance> > {
    typedef Distance difference_type;
    typedef T value_type;
  };

  template <class T>
  struct iterator_traits<T*> {
    typedef T value_type;
    typedef ptrdiff_t difference_type;
  };

  template<typename _InputIterator>
  inline typename iterator_traits<_InputIterator>::difference_type
  distance(_InputIterator __first, _InputIterator __last)
  {
    typename iterator_traits<_InputIterator>::difference_type __n = 0;
    while (__first != __last) {
      ++__first; ++__n;
    }
    return __n;
  }
}
#endif


namespace swig {
  template<typename OutIterator>
  class SwigPyIterator_T :  public SwigPyIterator
  {
  public:
    typedef OutIterator out_iterator;
    typedef typename std::iterator_traits<out_iterator>::value_type value_type;    
    typedef SwigPyIterator_T<out_iterator> self_type;

    SwigPyIterator_T(out_iterator curr, PyObject *seq)
      : SwigPyIterator(seq), current(curr)
    {
    }

    const out_iterator& get_current() const
    {
      return current;
    }

    
    bool equal (const SwigPyIterator &iter) const
    {
      const self_type *iters = dynamic_cast<const self_type *>(&iter);
      if (iters) {
	return (current == iters->get_current());
      } else {
	throw std::invalid_argument("bad iterator type");
      }
    }
    
    ptrdiff_t distance(const SwigPyIterator &iter) const
    {
      const self_type *iters = dynamic_cast<const self_type *>(&iter);
      if (iters) {
	return std::distance(current, iters->get_current());
      } else {
	throw std::invalid_argument("bad iterator type");
      }
    }    
    
  protected:
    out_iterator current;
  };
  
  template <class ValueType>
  struct from_oper 
  {
    typedef const ValueType& argument_type;
    typedef PyObject *result_type;
    result_type operator()(argument_type v) const
    {
      return swig::from(v);
    }
  };

  template<typename OutIterator, 
	   typename ValueType = typename std::iterator_traits<OutIterator>::value_type,
	   typename FromOper = from_oper<ValueType> >
  class SwigPyForwardIteratorOpen_T :  public SwigPyIterator_T<OutIterator>
  {
  public:
    FromOper from;
    typedef OutIterator out_iterator;
    typedef ValueType value_type;
    typedef SwigPyIterator_T<out_iterator>  base;
    typedef SwigPyForwardIteratorOpen_T<OutIterator, ValueType, FromOper> self_type;
    
    SwigPyForwardIteratorOpen_T(out_iterator curr, PyObject *seq)
      : SwigPyIterator_T<OutIterator>(curr, seq)
    {
    }
    
    PyObject *value() const {
      return from(static_cast<const value_type&>(*(base::current)));
    }
    
    SwigPyIterator *copy() const
    {
      return new self_type(*this);
    }

    SwigPyIterator *incr(size_t n = 1)
    {
      while (n--) {
	++base::current;
      }
      return this;
    }

  };

  template<typename OutIterator, 
	   typename ValueType = typename std::iterator_traits<OutIterator>::value_type,
	   typename FromOper = from_oper<ValueType> >
  class SwigPyIteratorOpen_T :  public SwigPyForwardIteratorOpen_T<OutIterator, ValueType, FromOper>
  {
  public:
    FromOper from;
    typedef OutIterator out_iterator;
    typedef ValueType value_type;
    typedef SwigPyIterator_T<out_iterator>  base;
    typedef SwigPyIteratorOpen_T<OutIterator, ValueType, FromOper> self_type;
    
    SwigPyIteratorOpen_T(out_iterator curr, PyObject *seq)
      : SwigPyForwardIteratorOpen_T<OutIterator>(curr, seq)
    {
    }

    SwigPyIterator *decr(size_t n = 1)
    {
      while (n--) {
	--base::current;
      }
      return this;
    }
  };

  template<typename OutIterator, 
	   typename ValueType = typename std::iterator_traits<OutIterator>::value_type,
	   typename FromOper = from_oper<ValueType> >
  class SwigPyForwardIteratorClosed_T :  public SwigPyIterator_T<OutIterator>
  {
  public:
    FromOper from;
    typedef OutIterator out_iterator;
    typedef ValueType value_type;
    typedef SwigPyIterator_T<out_iterator>  base;    
    typedef SwigPyForwardIteratorClosed_T<OutIterator, ValueType, FromOper> self_type;
    
    SwigPyForwardIteratorClosed_T(out_iterator curr, out_iterator first, out_iterator last, PyObject *seq)
      : SwigPyIterator_T<OutIterator>(curr, seq), begin(first), end(last)
    {
    }
    
    PyObject *value() const {
      if (base::current == end) {
	throw stop_iteration();
      } else {
	return from(static_cast<const value_type&>(*(base::current)));
      }
    }
    
    SwigPyIterator *copy() const
    {
      return new self_type(*this);
    }

    SwigPyIterator *incr(size_t n = 1)
    {
      while (n--) {
	if (base::current == end) {
	  throw stop_iteration();
	} else {
	  ++base::current;
	}
      }
      return this;
    }

  protected:
    out_iterator begin;
    out_iterator end;
  };

  template<typename OutIterator, 
	   typename ValueType = typename std::iterator_traits<OutIterator>::value_type,
	   typename FromOper = from_oper<ValueType> >
  class SwigPyIteratorClosed_T :  public SwigPyForwardIteratorClosed_T<OutIterator,ValueType,FromOper>
  {
  public:
    FromOper from;
    typedef OutIterator out_iterator;
    typedef ValueType value_type;
    typedef SwigPyIterator_T<out_iterator>  base;
    typedef SwigPyForwardIteratorClosed_T<OutIterator, ValueType, FromOper> base0;
    typedef SwigPyIteratorClosed_T<OutIterator, ValueType, FromOper> self_type;
    
    SwigPyIteratorClosed_T(out_iterator curr, out_iterator first, out_iterator last, PyObject *seq)
      : SwigPyForwardIteratorClosed_T<OutIterator,ValueType,FromOper>(curr, first, last, seq)
    {
    }

    SwigPyIterator *decr(size_t n = 1)
    {
      while (n--) {
	if (base::current == base0::begin) {
	  throw stop_iteration();
	} else {
	  --base::current;
	}
      }
      return this;
    }
  };


  template<typename OutIter>
  inline SwigPyIterator*
  make_output_forward_iterator(const OutIter& current, const OutIter& begin,const OutIter& end, PyObject *seq = 0)
  {
    return new SwigPyForwardIteratorClosed_T<OutIter>(current, begin, end, seq);
  }

  template<typename OutIter>
  inline SwigPyIterator*
  make_output_iterator(const OutIter& current, const OutIter& begin,const OutIter& end, PyObject *seq = 0)
  {
    return new SwigPyIteratorClosed_T<OutIter>(current, begin, end, seq);
  }

  template<typename OutIter>
  inline SwigPyIterator*
  make_output_forward_iterator(const OutIter& current, PyObject *seq = 0)
  {
    return new SwigPyForwardIteratorOpen_T<OutIter>(current, seq);
  }

  template<typename OutIter>
  inline SwigPyIterator*
  make_output_iterator(const OutIter& current, PyObject *seq = 0)
  {
    return new SwigPyIteratorOpen_T<OutIter>(current, seq);
  }

}


namespace swig
{
  template <class T>
  struct SwigPySequence_Ref
  {
    SwigPySequence_Ref(PyObject* seq, Py_ssize_t index)
      : _seq(seq), _index(index)
    {
    }
    
    operator T () const
    {
      swig::SwigVar_PyObject item = PySequence_GetItem(_seq, _index);
      try {
	return swig::as<T>(item);
      } catch (const std::invalid_argument& e) {
	char msg[1024];
	sprintf(msg, "in sequence element %d ", (int)_index);
	if (!PyErr_Occurred()) {
	  ::SWIG_Error(SWIG_TypeError,  swig::type_name<T>());
	}
	SWIG_Python_AddErrorMsg(msg);
	SWIG_Python_AddErrorMsg(e.what());
	throw;
      }
    }

    SwigPySequence_Ref& operator=(const T& v)
    {
      PySequence_SetItem(_seq, _index, swig::from<T>(v));
      return *this;
    }

  private:
    PyObject* _seq;
    Py_ssize_t _index;
  };

  template <class T>
  struct SwigPySequence_ArrowProxy
  {
    SwigPySequence_ArrowProxy(const T& x): m_value(x) {}
    const T* operator->() const { return &m_value; }
    operator const T*() const { return &m_value; }
    T m_value;
  };

  template <class T, class Reference >
  struct SwigPySequence_InputIterator
  {
    typedef SwigPySequence_InputIterator<T, Reference > self;

    typedef std::random_access_iterator_tag iterator_category;
    typedef Reference reference;
    typedef T value_type;
    typedef T* pointer;
    typedef Py_ssize_t difference_type;

    SwigPySequence_InputIterator()
    {
    }

    SwigPySequence_InputIterator(PyObject* seq, Py_ssize_t index)
      : _seq(seq), _index(index)
    {
    }

    reference operator*() const
    {
      return reference(_seq, _index);
    }

    SwigPySequence_ArrowProxy<T>
    operator->() const {
      return SwigPySequence_ArrowProxy<T>(operator*());
    }

    bool operator==(const self& ri) const
    {
      return (_index == ri._index) && (_seq == ri._seq);
    }

    bool operator!=(const self& ri) const
    {
      return !(operator==(ri));
    }

    self& operator ++ ()
    {
      ++_index;
      return *this;
    }

    self& operator -- ()
    {
      --_index;
      return *this;
    }

    self& operator += (difference_type n)
    {
      _index += n;
      return *this;
    }

    self operator +(difference_type n) const
    {
      return self(_seq, _index + n);
    }

    self& operator -= (difference_type n)
    {
      _index -= n;
      return *this;
    }

    self operator -(difference_type n) const
    {
      return self(_seq, _index - n);
    }

    difference_type operator - (const self& ri) const
    {
      return _index - ri._index;
    }

    bool operator < (const self& ri) const
    {
      return _index < ri._index;
    }

    reference
    operator[](difference_type n) const
    {
      return reference(_seq, _index + n);
    }

  private:
    PyObject* _seq;
    difference_type _index;
  };

  // STL container wrapper around a Python sequence
  template <class T>
  struct SwigPySequence_Cont
  {
    typedef SwigPySequence_Ref<T> reference;
    typedef const SwigPySequence_Ref<T> const_reference;
    typedef T value_type;
    typedef T* pointer;
    typedef Py_ssize_t difference_type;
    typedef size_t size_type;
    typedef const pointer const_pointer;
    typedef SwigPySequence_InputIterator<T, reference> iterator;
    typedef SwigPySequence_InputIterator<T, const_reference> const_iterator;

    SwigPySequence_Cont(PyObject* seq) : _seq(0)
    {
      if (!PySequence_Check(seq)) {
	throw std::invalid_argument("a sequence is expected");
      }
      _seq = seq;
      Py_INCREF(_seq);
    }

    ~SwigPySequence_Cont()
    {
      Py_XDECREF(_seq);
    }

    size_type size() const
    {
      return static_cast<size_type>(PySequence_Size(_seq));
    }

    bool empty() const
    {
      return size() == 0;
    }

    iterator begin()
    {
      return iterator(_seq, 0);
    }

    const_iterator begin() const
    {
      return const_iterator(_seq, 0);
    }

    iterator end()
    {
      return iterator(_seq, size());
    }

    const_iterator end() const
    {
      return const_iterator(_seq, size());
    }

    reference operator[](difference_type n)
    {
      return reference(_seq, n);
    }

    const_reference operator[](difference_type n)  const
    {
      return const_reference(_seq, n);
    }

    bool check() const
    {
      Py_ssize_t s = size();
      for (Py_ssize_t i = 0; i < s; ++i) {
	swig::SwigVar_PyObject item = PySequence_GetItem(_seq, i);
	if (!swig::check<value_type>(item))
	  return false;
      }
      return true;
    }

  private:
    PyObject* _seq;
  };

}


SWIGINTERN int
SWIG_AsVal_std_string (PyObject * obj, std::string *val)
{
  std::string* v = (std::string *) 0;
  int res = SWIG_AsPtr_std_string (obj, &v);
  if (!SWIG_IsOK(res)) return res;
  if (v) {
    if (val) *val = *v;
    if (SWIG_IsNewObj(res)) {
      delete v;
      res = SWIG_DelNewMask(res);
    }
    return res;
  }
  return SWIG_ERROR;
}


namespace swig {
  template <> struct traits< std::string > {
    typedef value_category category;
    static const char* type_name() { return"std::string"; }
  };
  template <>  struct traits_asval< std::string > {
    typedef std::string value_type;
    static int asval(PyObject *obj, value_type *val) {
      return SWIG_AsVal_std_string (obj, val);
    }
  };
  template <>  struct traits_from< std::string > {
    typedef std::string value_type;
    static PyObject *from(const value_type& val) {
      return SWIG_From_std_string  (val);
    }
  };
}


namespace swig {
  template <class SwigPySeq, class Seq>
  inline void
  assign(const SwigPySeq& swigpyseq, Seq* seq) {
    // seq->assign(swigpyseq.begin(), swigpyseq.end()); // not used as not always implemented
    typedef typename SwigPySeq::value_type value_type;
    typename SwigPySeq::const_iterator it = swigpyseq.begin();
    for (;it != swigpyseq.end(); ++it) {
      seq->insert(seq->end(),(value_type)(*it));
    }
  }

  template <class Seq, class T = typename Seq::value_type >
  struct traits_asptr_stdseq {
    typedef Seq sequence;
    typedef T value_type;

    static int asptr(PyObject *obj, sequence **seq) {
      if (obj == Py_None || SWIG_Python_GetSwigThis(obj)) {
	sequence *p;
	swig_type_info *descriptor = swig::type_info<sequence>();
	if (descriptor && SWIG_IsOK(::SWIG_ConvertPtr(obj, (void **)&p, descriptor, 0))) {
	  if (seq) *seq = p;
	  return SWIG_OLDOBJ;
	}
      } else if (PySequence_Check(obj)) {
	try {
	  SwigPySequence_Cont<value_type> swigpyseq(obj);
	  if (seq) {
	    sequence *pseq = new sequence();
	    assign(swigpyseq, pseq);
	    *seq = pseq;
	    return SWIG_NEWOBJ;
	  } else {
	    return swigpyseq.check() ? SWIG_OK : SWIG_ERROR;
	  }
	} catch (std::exception& e) {
	  if (seq) {
	    if (!PyErr_Occurred()) {
	      PyErr_SetString(PyExc_TypeError, e.what());
	    }
	  }
	  return SWIG_ERROR;
	}
      }
      return SWIG_ERROR;
    }
  };

  template <class Seq, class T = typename Seq::value_type >
  struct traits_from_stdseq {
    typedef Seq sequence;
    typedef T value_type;
    typedef typename Seq::size_type size_type;
    typedef typename sequence::const_iterator const_iterator;

    static PyObject *from(const sequence& seq) {
#ifdef SWIG_PYTHON_EXTRA_NATIVE_CONTAINERS
      swig_type_info *desc = swig::type_info<sequence>();
      if (desc && desc->clientdata) {
	return SWIG_InternalNewPointerObj(new sequence(seq), desc, SWIG_POINTER_OWN);
      }
#endif
      size_type size = seq.size();
      if (size <= (size_type)INT_MAX) {
	PyObject *obj = PyTuple_New((Py_ssize_t)size);
	Py_ssize_t i = 0;
	for (const_iterator it = seq.begin(); it != seq.end(); ++it, ++i) {
	  PyTuple_SetItem(obj,i,swig::from<value_type>(*it));
	}
	return obj;
      } else {
	PyErr_SetString(PyExc_OverflowError,"sequence size not valid in python");
	return NULL;
      }
    }
  };
}


  namespace swig {
    template <class T>
    struct traits_reserve<std::vector<T> > {
      static void reserve(std::vector<T> &seq, typename std::vector<T>::size_type n) {
        seq.reserve(n);
      }
    };

    template <class T>
    struct traits_asptr<std::vector<T> >  {
      static int asptr(PyObject *obj, std::vector<T> **vec) {
	return traits_asptr_stdseq<std::vector<T> >::asptr(obj, vec);
      }
    };
    
    template <class T>
    struct traits_from<std::vector<T> > {
      static PyObject *from(const std::vector<T>& vec) {
	return traits_from_stdseq<std::vector<T> >::from(vec);
      }
    };
  }


      namespace swig {
	template <>  struct traits<std::vector< std::string, std::allocator< std::string > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "std::string" "," "std::allocator< std::string >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_std_string_Sg__iterator(std::vector< std::string > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_std_string_Sg____nonzero__(std::vector< std::string > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_std_string_Sg____bool__(std::vector< std::string > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< std::string >::size_type std_vector_Sl_std_string_Sg____len__(std::vector< std::string > const *self){
      return self->size();
    }

#ifdef SWIG_LONG_LONG_AVAILABLE
SWIGINTERNINLINE PyObject* 
SWIG_From_unsigned_SS_long_SS_long  (unsigned long long value)
{
  return (value > LONG_MAX) ?
    PyLong_FromUnsignedLongLong(value) : PyInt_FromLong(static_cast< long >(value));
}
#endif


SWIGINTERNINLINE PyObject *
SWIG_From_size_t  (size_t value)
{    
#ifdef SWIG_LONG_LONG_AVAILABLE
  if (sizeof(size_t) <= sizeof(unsigned long)) {
#endif
    return SWIG_From_unsigned_SS_long  (static_cast< unsigned long >(value));
#ifdef SWIG_LONG_LONG_AVAILABLE
  } else {
    /* assume sizeof(size_t) <= sizeof(unsigned long long) */
    return SWIG_From_unsigned_SS_long_SS_long  (static_cast< unsigned long long >(value));
  }
#endif
}

SWIGINTERN std::vector< std::string,std::allocator< std::string > > *std_vector_Sl_std_string_Sg____getslice__(std::vector< std::string > *self,std::vector< std::string >::difference_type i,std::vector< std::string >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_std_string_Sg____setslice____SWIG_0(std::vector< std::string > *self,std::vector< std::string >::difference_type i,std::vector< std::string >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< std::string,std::allocator< std::string > >());
    }
SWIGINTERN void std_vector_Sl_std_string_Sg____setslice____SWIG_1(std::vector< std::string > *self,std::vector< std::string >::difference_type i,std::vector< std::string >::difference_type j,std::vector< std::string,std::allocator< std::string > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_std_string_Sg____delslice__(std::vector< std::string > *self,std::vector< std::string >::difference_type i,std::vector< std::string >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_std_string_Sg____delitem____SWIG_0(std::vector< std::string > *self,std::vector< std::string >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< std::string,std::allocator< std::string > > *std_vector_Sl_std_string_Sg____getitem____SWIG_0(std::vector< std::string > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< std::string,std::allocator< std::string > >::difference_type id = i;
      std::vector< std::string,std::allocator< std::string > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_std_string_Sg____setitem____SWIG_0(std::vector< std::string > *self,PySliceObject *slice,std::vector< std::string,std::allocator< std::string > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< std::string,std::allocator< std::string > >::difference_type id = i;
      std::vector< std::string,std::allocator< std::string > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_std_string_Sg____setitem____SWIG_1(std::vector< std::string > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< std::string,std::allocator< std::string > >::difference_type id = i;
      std::vector< std::string,std::allocator< std::string > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_std_string_Sg____delitem____SWIG_1(std::vector< std::string > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< std::string,std::allocator< std::string > >::difference_type id = i;
      std::vector< std::string,std::allocator< std::string > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< std::string >::value_type const &std_vector_Sl_std_string_Sg____getitem____SWIG_1(std::vector< std::string > const *self,std::vector< std::string >::difference_type i){
      return *(swig::cgetpos(self, i));
    }

namespace swig {
  static PyObject* container_owner_attribute() {
    static PyObject* attr = SWIG_Python_str_FromChar("__swig_container");
    return attr;
  }

  template <typename T>
  struct container_owner {
    // By default, do not add the back-reference (for value types)
    // Specialization below will check the reference for pointer types.
    static bool back_reference(PyObject* /*child*/, PyObject* /*owner*/) {
      return false;
    }
  };

  template <>
  struct container_owner<swig::pointer_category> {  
    /*
     * Call to add a back-reference to the owning object when returning a 
     * reference from a container.  Will only set the reference if child
     * is a SWIG wrapper object that does not own the pointer.
     *
     * returns whether the reference was set or not
     */
    static bool back_reference(PyObject* child, PyObject* owner) {
      SwigPyObject* swigThis = SWIG_Python_GetSwigThis(child);
      if (swigThis && (swigThis->own & SWIG_POINTER_OWN) != SWIG_POINTER_OWN) {
        return PyObject_SetAttr(child, container_owner_attribute(), owner) != -1;
      }
      return false;
    }
  };
}

SWIGINTERN void std_vector_Sl_std_string_Sg____setitem____SWIG_2(std::vector< std::string > *self,std::vector< std::string >::difference_type i,std::vector< std::string >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< std::string >::value_type std_vector_Sl_std_string_Sg__pop(std::vector< std::string > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< std::string,std::allocator< std::string > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_std_string_Sg__append(std::vector< std::string > *self,std::vector< std::string >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< std::string >::iterator std_vector_Sl_std_string_Sg__erase__SWIG_0(std::vector< std::string > *self,std::vector< std::string >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< std::string >::iterator std_vector_Sl_std_string_Sg__erase__SWIG_1(std::vector< std::string > *self,std::vector< std::string >::iterator first,std::vector< std::string >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< std::string >::iterator std_vector_Sl_std_string_Sg__insert__SWIG_0(std::vector< std::string > *self,std::vector< std::string >::iterator pos,std::vector< std::string >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_std_string_Sg__insert__SWIG_1(std::vector< std::string > *self,std::vector< std::string >::iterator pos,std::vector< std::string >::size_type n,std::vector< std::string >::value_type const &x){ self->insert(pos, n, x); }

SWIGINTERN int
SWIG_AsVal_int (PyObject * obj, int *val)
{
  long v;
  int res = SWIG_AsVal_long (obj, &v);
  if (SWIG_IsOK(res)) {
    if ((v < INT_MIN || v > INT_MAX)) {
      return SWIG_OverflowError;
    } else {
      if (val) *val = static_cast< int >(v);
    }
  }  
  return res;
}


namespace swig {
  template <> struct traits< int > {
    typedef value_category category;
    static const char* type_name() { return"int"; }
  };
  template <>  struct traits_asval< int > {
    typedef int value_type;
    static int asval(PyObject *obj, value_type *val) {
      return SWIG_AsVal_int (obj, val);
    }
  };
  template <>  struct traits_from< int > {
    typedef int value_type;
    static PyObject *from(const value_type& val) {
      return SWIG_From_int  (val);
    }
  };
}


      namespace swig {
	template <>  struct traits<std::vector< int, std::allocator< int > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "int" "," "std::allocator< int >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_int32_t_Sg__iterator(std::vector< int32_t > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_int32_t_Sg____nonzero__(std::vector< int32_t > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_int32_t_Sg____bool__(std::vector< int32_t > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< int >::size_type std_vector_Sl_int32_t_Sg____len__(std::vector< int32_t > const *self){
      return self->size();
    }
SWIGINTERN std::vector< int,std::allocator< int > > *std_vector_Sl_int32_t_Sg____getslice__(std::vector< int32_t > *self,std::vector< int >::difference_type i,std::vector< int >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_int32_t_Sg____setslice____SWIG_0(std::vector< int32_t > *self,std::vector< int >::difference_type i,std::vector< int >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< int,std::allocator< int > >());
    }
SWIGINTERN void std_vector_Sl_int32_t_Sg____setslice____SWIG_1(std::vector< int32_t > *self,std::vector< int >::difference_type i,std::vector< int >::difference_type j,std::vector< int,std::allocator< int > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_int32_t_Sg____delslice__(std::vector< int32_t > *self,std::vector< int >::difference_type i,std::vector< int >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_int32_t_Sg____delitem____SWIG_0(std::vector< int32_t > *self,std::vector< int >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< int,std::allocator< int > > *std_vector_Sl_int32_t_Sg____getitem____SWIG_0(std::vector< int32_t > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< int,std::allocator< int > >::difference_type id = i;
      std::vector< int,std::allocator< int > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_int32_t_Sg____setitem____SWIG_0(std::vector< int32_t > *self,PySliceObject *slice,std::vector< int,std::allocator< int > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< int,std::allocator< int > >::difference_type id = i;
      std::vector< int,std::allocator< int > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_int32_t_Sg____setitem____SWIG_1(std::vector< int32_t > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< int,std::allocator< int > >::difference_type id = i;
      std::vector< int,std::allocator< int > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_int32_t_Sg____delitem____SWIG_1(std::vector< int32_t > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< int,std::allocator< int > >::difference_type id = i;
      std::vector< int,std::allocator< int > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< int >::value_type const &std_vector_Sl_int32_t_Sg____getitem____SWIG_1(std::vector< int32_t > const *self,std::vector< int >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_int32_t_Sg____setitem____SWIG_2(std::vector< int32_t > *self,std::vector< int >::difference_type i,std::vector< int >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< int >::value_type std_vector_Sl_int32_t_Sg__pop(std::vector< int32_t > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< int,std::allocator< int > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_int32_t_Sg__append(std::vector< int32_t > *self,std::vector< int >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< int >::iterator std_vector_Sl_int32_t_Sg__erase__SWIG_0(std::vector< int32_t > *self,std::vector< int >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< int >::iterator std_vector_Sl_int32_t_Sg__erase__SWIG_1(std::vector< int32_t > *self,std::vector< int >::iterator first,std::vector< int >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< int >::iterator std_vector_Sl_int32_t_Sg__insert__SWIG_0(std::vector< int32_t > *self,std::vector< int >::iterator pos,std::vector< int >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_int32_t_Sg__insert__SWIG_1(std::vector< int32_t > *self,std::vector< int >::iterator pos,std::vector< int >::size_type n,std::vector< int >::value_type const &x){ self->insert(pos, n, x); }

SWIGINTERN int
SWIG_AsVal_unsigned_SS_int (PyObject * obj, unsigned int *val)
{
  unsigned long v;
  int res = SWIG_AsVal_unsigned_SS_long (obj, &v);
  if (SWIG_IsOK(res)) {
    if ((v > UINT_MAX)) {
      return SWIG_OverflowError;
    } else {
      if (val) *val = static_cast< unsigned int >(v);
    }
  }  
  return res;
}


namespace swig {
  template <> struct traits< unsigned int > {
    typedef value_category category;
    static const char* type_name() { return"unsigned int"; }
  };
  template <>  struct traits_asval< unsigned int > {
    typedef unsigned int value_type;
    static int asval(PyObject *obj, value_type *val) {
      return SWIG_AsVal_unsigned_SS_int (obj, val);
    }
  };
  template <>  struct traits_from< unsigned int > {
    typedef unsigned int value_type;
    static PyObject *from(const value_type& val) {
      return SWIG_From_unsigned_SS_int  (val);
    }
  };
}


      namespace swig {
	template <>  struct traits<std::vector< unsigned int, std::allocator< unsigned int > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "unsigned int" "," "std::allocator< unsigned int >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_uint32_t_Sg__iterator(std::vector< uint32_t > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_uint32_t_Sg____nonzero__(std::vector< uint32_t > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_uint32_t_Sg____bool__(std::vector< uint32_t > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< unsigned int >::size_type std_vector_Sl_uint32_t_Sg____len__(std::vector< uint32_t > const *self){
      return self->size();
    }
SWIGINTERN std::vector< unsigned int,std::allocator< unsigned int > > *std_vector_Sl_uint32_t_Sg____getslice__(std::vector< uint32_t > *self,std::vector< unsigned int >::difference_type i,std::vector< unsigned int >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_uint32_t_Sg____setslice____SWIG_0(std::vector< uint32_t > *self,std::vector< unsigned int >::difference_type i,std::vector< unsigned int >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< unsigned int,std::allocator< unsigned int > >());
    }
SWIGINTERN void std_vector_Sl_uint32_t_Sg____setslice____SWIG_1(std::vector< uint32_t > *self,std::vector< unsigned int >::difference_type i,std::vector< unsigned int >::difference_type j,std::vector< unsigned int,std::allocator< unsigned int > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_uint32_t_Sg____delslice__(std::vector< uint32_t > *self,std::vector< unsigned int >::difference_type i,std::vector< unsigned int >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_uint32_t_Sg____delitem____SWIG_0(std::vector< uint32_t > *self,std::vector< unsigned int >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< unsigned int,std::allocator< unsigned int > > *std_vector_Sl_uint32_t_Sg____getitem____SWIG_0(std::vector< uint32_t > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< unsigned int,std::allocator< unsigned int > >::difference_type id = i;
      std::vector< unsigned int,std::allocator< unsigned int > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_uint32_t_Sg____setitem____SWIG_0(std::vector< uint32_t > *self,PySliceObject *slice,std::vector< unsigned int,std::allocator< unsigned int > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< unsigned int,std::allocator< unsigned int > >::difference_type id = i;
      std::vector< unsigned int,std::allocator< unsigned int > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_uint32_t_Sg____setitem____SWIG_1(std::vector< uint32_t > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< unsigned int,std::allocator< unsigned int > >::difference_type id = i;
      std::vector< unsigned int,std::allocator< unsigned int > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_uint32_t_Sg____delitem____SWIG_1(std::vector< uint32_t > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< unsigned int,std::allocator< unsigned int > >::difference_type id = i;
      std::vector< unsigned int,std::allocator< unsigned int > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< unsigned int >::value_type const &std_vector_Sl_uint32_t_Sg____getitem____SWIG_1(std::vector< uint32_t > const *self,std::vector< unsigned int >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_uint32_t_Sg____setitem____SWIG_2(std::vector< uint32_t > *self,std::vector< unsigned int >::difference_type i,std::vector< unsigned int >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< unsigned int >::value_type std_vector_Sl_uint32_t_Sg__pop(std::vector< uint32_t > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< unsigned int,std::allocator< unsigned int > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_uint32_t_Sg__append(std::vector< uint32_t > *self,std::vector< unsigned int >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< unsigned int >::iterator std_vector_Sl_uint32_t_Sg__erase__SWIG_0(std::vector< uint32_t > *self,std::vector< unsigned int >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< unsigned int >::iterator std_vector_Sl_uint32_t_Sg__erase__SWIG_1(std::vector< uint32_t > *self,std::vector< unsigned int >::iterator first,std::vector< unsigned int >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< unsigned int >::iterator std_vector_Sl_uint32_t_Sg__insert__SWIG_0(std::vector< uint32_t > *self,std::vector< unsigned int >::iterator pos,std::vector< unsigned int >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_uint32_t_Sg__insert__SWIG_1(std::vector< uint32_t > *self,std::vector< unsigned int >::iterator pos,std::vector< unsigned int >::size_type n,std::vector< unsigned int >::value_type const &x){ self->insert(pos, n, x); }

SWIGINTERN int
SWIG_AsVal_signed_SS_char (PyObject * obj, signed char *val)
{
  long v;
  int res = SWIG_AsVal_long (obj, &v);
  if (SWIG_IsOK(res)) {
    if ((v < SCHAR_MIN || v > SCHAR_MAX)) {
      return SWIG_OverflowError;
    } else {
      if (val) *val = static_cast< signed char >(v);
    }
  }  
  return res;
}


SWIGINTERNINLINE PyObject *
SWIG_From_signed_SS_char  (signed char value)
{    
  return SWIG_From_long  (value);
}


namespace swig {
  template <> struct traits< signed char > {
    typedef value_category category;
    static const char* type_name() { return"signed char"; }
  };
  template <>  struct traits_asval< signed char > {
    typedef signed char value_type;
    static int asval(PyObject *obj, value_type *val) {
      return SWIG_AsVal_signed_SS_char (obj, val);
    }
  };
  template <>  struct traits_from< signed char > {
    typedef signed char value_type;
    static PyObject *from(const value_type& val) {
      return SWIG_From_signed_SS_char  (val);
    }
  };
}


      namespace swig {
	template <>  struct traits<std::vector< signed char, std::allocator< signed char > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "signed char" "," "std::allocator< signed char >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_int8_t_Sg__iterator(std::vector< int8_t > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_int8_t_Sg____nonzero__(std::vector< int8_t > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_int8_t_Sg____bool__(std::vector< int8_t > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< signed char >::size_type std_vector_Sl_int8_t_Sg____len__(std::vector< int8_t > const *self){
      return self->size();
    }
SWIGINTERN std::vector< signed char,std::allocator< signed char > > *std_vector_Sl_int8_t_Sg____getslice__(std::vector< int8_t > *self,std::vector< signed char >::difference_type i,std::vector< signed char >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_int8_t_Sg____setslice____SWIG_0(std::vector< int8_t > *self,std::vector< signed char >::difference_type i,std::vector< signed char >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< signed char,std::allocator< signed char > >());
    }
SWIGINTERN void std_vector_Sl_int8_t_Sg____setslice____SWIG_1(std::vector< int8_t > *self,std::vector< signed char >::difference_type i,std::vector< signed char >::difference_type j,std::vector< signed char,std::allocator< signed char > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_int8_t_Sg____delslice__(std::vector< int8_t > *self,std::vector< signed char >::difference_type i,std::vector< signed char >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_int8_t_Sg____delitem____SWIG_0(std::vector< int8_t > *self,std::vector< signed char >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< signed char,std::allocator< signed char > > *std_vector_Sl_int8_t_Sg____getitem____SWIG_0(std::vector< int8_t > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< signed char,std::allocator< signed char > >::difference_type id = i;
      std::vector< signed char,std::allocator< signed char > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_int8_t_Sg____setitem____SWIG_0(std::vector< int8_t > *self,PySliceObject *slice,std::vector< signed char,std::allocator< signed char > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< signed char,std::allocator< signed char > >::difference_type id = i;
      std::vector< signed char,std::allocator< signed char > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_int8_t_Sg____setitem____SWIG_1(std::vector< int8_t > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< signed char,std::allocator< signed char > >::difference_type id = i;
      std::vector< signed char,std::allocator< signed char > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_int8_t_Sg____delitem____SWIG_1(std::vector< int8_t > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< signed char,std::allocator< signed char > >::difference_type id = i;
      std::vector< signed char,std::allocator< signed char > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< signed char >::value_type const &std_vector_Sl_int8_t_Sg____getitem____SWIG_1(std::vector< int8_t > const *self,std::vector< signed char >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_int8_t_Sg____setitem____SWIG_2(std::vector< int8_t > *self,std::vector< signed char >::difference_type i,std::vector< signed char >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< signed char >::value_type std_vector_Sl_int8_t_Sg__pop(std::vector< int8_t > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< signed char,std::allocator< signed char > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_int8_t_Sg__append(std::vector< int8_t > *self,std::vector< signed char >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< signed char >::iterator std_vector_Sl_int8_t_Sg__erase__SWIG_0(std::vector< int8_t > *self,std::vector< signed char >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< signed char >::iterator std_vector_Sl_int8_t_Sg__erase__SWIG_1(std::vector< int8_t > *self,std::vector< signed char >::iterator first,std::vector< signed char >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< signed char >::iterator std_vector_Sl_int8_t_Sg__insert__SWIG_0(std::vector< int8_t > *self,std::vector< signed char >::iterator pos,std::vector< signed char >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_int8_t_Sg__insert__SWIG_1(std::vector< int8_t > *self,std::vector< signed char >::iterator pos,std::vector< signed char >::size_type n,std::vector< signed char >::value_type const &x){ self->insert(pos, n, x); }

SWIGINTERN int
SWIG_AsVal_unsigned_SS_char (PyObject * obj, unsigned char *val)
{
  unsigned long v;
  int res = SWIG_AsVal_unsigned_SS_long (obj, &v);
  if (SWIG_IsOK(res)) {
    if ((v > UCHAR_MAX)) {
      return SWIG_OverflowError;
    } else {
      if (val) *val = static_cast< unsigned char >(v);
    }
  }  
  return res;
}


namespace swig {
  template <> struct traits< unsigned char > {
    typedef value_category category;
    static const char* type_name() { return"unsigned char"; }
  };
  template <>  struct traits_asval< unsigned char > {
    typedef unsigned char value_type;
    static int asval(PyObject *obj, value_type *val) {
      return SWIG_AsVal_unsigned_SS_char (obj, val);
    }
  };
  template <>  struct traits_from< unsigned char > {
    typedef unsigned char value_type;
    static PyObject *from(const value_type& val) {
      return SWIG_From_unsigned_SS_char  (val);
    }
  };
}


      namespace swig {
	template <>  struct traits<std::vector< unsigned char, std::allocator< unsigned char > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "unsigned char" "," "std::allocator< unsigned char >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_uint8_t_Sg__iterator(std::vector< uint8_t > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_uint8_t_Sg____nonzero__(std::vector< uint8_t > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_uint8_t_Sg____bool__(std::vector< uint8_t > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< unsigned char >::size_type std_vector_Sl_uint8_t_Sg____len__(std::vector< uint8_t > const *self){
      return self->size();
    }
SWIGINTERN std::vector< unsigned char,std::allocator< unsigned char > > *std_vector_Sl_uint8_t_Sg____getslice__(std::vector< uint8_t > *self,std::vector< unsigned char >::difference_type i,std::vector< unsigned char >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_uint8_t_Sg____setslice____SWIG_0(std::vector< uint8_t > *self,std::vector< unsigned char >::difference_type i,std::vector< unsigned char >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< unsigned char,std::allocator< unsigned char > >());
    }
SWIGINTERN void std_vector_Sl_uint8_t_Sg____setslice____SWIG_1(std::vector< uint8_t > *self,std::vector< unsigned char >::difference_type i,std::vector< unsigned char >::difference_type j,std::vector< unsigned char,std::allocator< unsigned char > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_uint8_t_Sg____delslice__(std::vector< uint8_t > *self,std::vector< unsigned char >::difference_type i,std::vector< unsigned char >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_uint8_t_Sg____delitem____SWIG_0(std::vector< uint8_t > *self,std::vector< unsigned char >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< unsigned char,std::allocator< unsigned char > > *std_vector_Sl_uint8_t_Sg____getitem____SWIG_0(std::vector< uint8_t > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< unsigned char,std::allocator< unsigned char > >::difference_type id = i;
      std::vector< unsigned char,std::allocator< unsigned char > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_uint8_t_Sg____setitem____SWIG_0(std::vector< uint8_t > *self,PySliceObject *slice,std::vector< unsigned char,std::allocator< unsigned char > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< unsigned char,std::allocator< unsigned char > >::difference_type id = i;
      std::vector< unsigned char,std::allocator< unsigned char > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_uint8_t_Sg____setitem____SWIG_1(std::vector< uint8_t > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< unsigned char,std::allocator< unsigned char > >::difference_type id = i;
      std::vector< unsigned char,std::allocator< unsigned char > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_uint8_t_Sg____delitem____SWIG_1(std::vector< uint8_t > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< unsigned char,std::allocator< unsigned char > >::difference_type id = i;
      std::vector< unsigned char,std::allocator< unsigned char > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< unsigned char >::value_type const &std_vector_Sl_uint8_t_Sg____getitem____SWIG_1(std::vector< uint8_t > const *self,std::vector< unsigned char >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_uint8_t_Sg____setitem____SWIG_2(std::vector< uint8_t > *self,std::vector< unsigned char >::difference_type i,std::vector< unsigned char >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< unsigned char >::value_type std_vector_Sl_uint8_t_Sg__pop(std::vector< uint8_t > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< unsigned char,std::allocator< unsigned char > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_uint8_t_Sg__append(std::vector< uint8_t > *self,std::vector< unsigned char >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< unsigned char >::iterator std_vector_Sl_uint8_t_Sg__erase__SWIG_0(std::vector< uint8_t > *self,std::vector< unsigned char >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< unsigned char >::iterator std_vector_Sl_uint8_t_Sg__erase__SWIG_1(std::vector< uint8_t > *self,std::vector< unsigned char >::iterator first,std::vector< unsigned char >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< unsigned char >::iterator std_vector_Sl_uint8_t_Sg__insert__SWIG_0(std::vector< uint8_t > *self,std::vector< unsigned char >::iterator pos,std::vector< unsigned char >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_uint8_t_Sg__insert__SWIG_1(std::vector< uint8_t > *self,std::vector< unsigned char >::iterator pos,std::vector< unsigned char >::size_type n,std::vector< unsigned char >::value_type const &x){ self->insert(pos, n, x); }

namespace swig {
  template <> struct traits< unsigned long > {
    typedef value_category category;
    static const char* type_name() { return"unsigned long"; }
  };
  template <>  struct traits_asval< unsigned long > {
    typedef unsigned long value_type;
    static int asval(PyObject *obj, value_type *val) {
      return SWIG_AsVal_unsigned_SS_long (obj, val);
    }
  };
  template <>  struct traits_from< unsigned long > {
    typedef unsigned long value_type;
    static PyObject *from(const value_type& val) {
      return SWIG_From_unsigned_SS_long  (val);
    }
  };
}


      namespace swig {
	template <>  struct traits<std::vector< unsigned long, std::allocator< unsigned long > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "unsigned long" "," "std::allocator< unsigned long >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_uint64_t_Sg__iterator(std::vector< uint64_t > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_uint64_t_Sg____nonzero__(std::vector< uint64_t > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_uint64_t_Sg____bool__(std::vector< uint64_t > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< unsigned long >::size_type std_vector_Sl_uint64_t_Sg____len__(std::vector< uint64_t > const *self){
      return self->size();
    }
SWIGINTERN std::vector< unsigned long,std::allocator< unsigned long > > *std_vector_Sl_uint64_t_Sg____getslice__(std::vector< uint64_t > *self,std::vector< unsigned long >::difference_type i,std::vector< unsigned long >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_uint64_t_Sg____setslice____SWIG_0(std::vector< uint64_t > *self,std::vector< unsigned long >::difference_type i,std::vector< unsigned long >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< unsigned long,std::allocator< unsigned long > >());
    }
SWIGINTERN void std_vector_Sl_uint64_t_Sg____setslice____SWIG_1(std::vector< uint64_t > *self,std::vector< unsigned long >::difference_type i,std::vector< unsigned long >::difference_type j,std::vector< unsigned long,std::allocator< unsigned long > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_uint64_t_Sg____delslice__(std::vector< uint64_t > *self,std::vector< unsigned long >::difference_type i,std::vector< unsigned long >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_uint64_t_Sg____delitem____SWIG_0(std::vector< uint64_t > *self,std::vector< unsigned long >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< unsigned long,std::allocator< unsigned long > > *std_vector_Sl_uint64_t_Sg____getitem____SWIG_0(std::vector< uint64_t > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< unsigned long,std::allocator< unsigned long > >::difference_type id = i;
      std::vector< unsigned long,std::allocator< unsigned long > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_uint64_t_Sg____setitem____SWIG_0(std::vector< uint64_t > *self,PySliceObject *slice,std::vector< unsigned long,std::allocator< unsigned long > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< unsigned long,std::allocator< unsigned long > >::difference_type id = i;
      std::vector< unsigned long,std::allocator< unsigned long > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_uint64_t_Sg____setitem____SWIG_1(std::vector< uint64_t > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< unsigned long,std::allocator< unsigned long > >::difference_type id = i;
      std::vector< unsigned long,std::allocator< unsigned long > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_uint64_t_Sg____delitem____SWIG_1(std::vector< uint64_t > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< unsigned long,std::allocator< unsigned long > >::difference_type id = i;
      std::vector< unsigned long,std::allocator< unsigned long > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< unsigned long >::value_type const &std_vector_Sl_uint64_t_Sg____getitem____SWIG_1(std::vector< uint64_t > const *self,std::vector< unsigned long >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_uint64_t_Sg____setitem____SWIG_2(std::vector< uint64_t > *self,std::vector< unsigned long >::difference_type i,std::vector< unsigned long >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< unsigned long >::value_type std_vector_Sl_uint64_t_Sg__pop(std::vector< uint64_t > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< unsigned long,std::allocator< unsigned long > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_uint64_t_Sg__append(std::vector< uint64_t > *self,std::vector< unsigned long >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< unsigned long >::iterator std_vector_Sl_uint64_t_Sg__erase__SWIG_0(std::vector< uint64_t > *self,std::vector< unsigned long >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< unsigned long >::iterator std_vector_Sl_uint64_t_Sg__erase__SWIG_1(std::vector< uint64_t > *self,std::vector< unsigned long >::iterator first,std::vector< unsigned long >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< unsigned long >::iterator std_vector_Sl_uint64_t_Sg__insert__SWIG_0(std::vector< uint64_t > *self,std::vector< unsigned long >::iterator pos,std::vector< unsigned long >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_uint64_t_Sg__insert__SWIG_1(std::vector< uint64_t > *self,std::vector< unsigned long >::iterator pos,std::vector< unsigned long >::size_type n,std::vector< unsigned long >::value_type const &x){ self->insert(pos, n, x); }

  namespace swig {


















































    template <class T, class U >
    struct traits_asptr<std::pair<T,U> >  {
      typedef std::pair<T,U> value_type;

      static int get_pair(PyObject* first, PyObject* second,
			  std::pair<T,U> **val) 
      {
	if (val) {
	  value_type *vp = (new std::pair<T,U>());
	  T *pfirst = &(vp->first);
	  int res1 = swig::asval((PyObject*)first, pfirst);
	  if (!SWIG_IsOK(res1)) {
	    delete vp;
	    return res1;
	  }
	  U *psecond = &(vp->second);
	  int res2 = swig::asval((PyObject*)second, psecond);
	  if (!SWIG_IsOK(res2)) {
	    delete vp;
	    return res2;
	  }
	  *val = vp;
	  return SWIG_AddNewMask(res1 > res2 ? res1 : res2);
	} else {
	  T *pfirst = 0;
	  int res1 = swig::asval((PyObject*)first, pfirst);
	  if (!SWIG_IsOK(res1)) return res1;
	  U *psecond = 0;
	  int res2 = swig::asval((PyObject*)second, psecond);
	  if (!SWIG_IsOK(res2)) return res2;
	  return res1 > res2 ? res1 : res2;
	}
      }

      static int asptr(PyObject *obj, std::pair<T,U> **val) {
	int res = SWIG_ERROR;
	if (PyTuple_Check(obj)) {
	  if (PyTuple_GET_SIZE(obj) == 2) {
	    res = get_pair(PyTuple_GET_ITEM(obj,0),PyTuple_GET_ITEM(obj,1), val);
	  }
	} else if (PySequence_Check(obj)) {
	  if (PySequence_Size(obj) == 2) {
	    swig::SwigVar_PyObject first = PySequence_GetItem(obj,0);
	    swig::SwigVar_PyObject second = PySequence_GetItem(obj,1);
	    res = get_pair(first, second, val);
	  }
	} else {
	  value_type *p = 0;
	  swig_type_info *descriptor = swig::type_info<value_type>();
	  res = descriptor ? SWIG_ConvertPtr(obj, (void **)&p, descriptor, 0) : SWIG_ERROR;
	  if (SWIG_IsOK(res) && val)  *val = p;
	}
	return res;
      }
    };


    template <class T, class U >
    struct traits_from<std::pair<T,U> >   {
      static PyObject *from(const std::pair<T,U>& val) {
	PyObject* obj = PyTuple_New(2);
	PyTuple_SetItem(obj,0,swig::from(val.first));
	PyTuple_SetItem(obj,1,swig::from(val.second));
	return obj;
      }
    };
  }





































      namespace swig {
	template <>  struct traits<std::pair< std::string, std::string > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::pair<" "std::string" "," "std::string" " >";
	  }
	};
      }
    

  namespace swig {
    template <class ValueType>
    struct from_key_oper 
    {
      typedef const ValueType& argument_type;
      typedef  PyObject *result_type;
      result_type operator()(argument_type v) const
      {
	return swig::from(v.first);
      }
    };

    template <class ValueType>
    struct from_value_oper 
    {
      typedef const ValueType& argument_type;
      typedef  PyObject *result_type;
      result_type operator()(argument_type v) const
      {
	return swig::from(v.second);
      }
    };

    template<class OutIterator, class FromOper, class ValueType = typename OutIterator::value_type>
    struct SwigPyMapIterator_T : SwigPyIteratorClosed_T<OutIterator, ValueType, FromOper>
    {
      SwigPyMapIterator_T(OutIterator curr, OutIterator first, OutIterator last, PyObject *seq)
	: SwigPyIteratorClosed_T<OutIterator,ValueType,FromOper>(curr, first, last, seq)
      {
      }
    };


    template<class OutIterator,
	     class FromOper = from_key_oper<typename OutIterator::value_type> >
    struct SwigPyMapKeyIterator_T : SwigPyMapIterator_T<OutIterator, FromOper>
    {
      SwigPyMapKeyIterator_T(OutIterator curr, OutIterator first, OutIterator last, PyObject *seq)
	: SwigPyMapIterator_T<OutIterator, FromOper>(curr, first, last, seq)
      {
      }
    };

    template<typename OutIter>
    inline SwigPyIterator*
    make_output_key_iterator(const OutIter& current, const OutIter& begin, const OutIter& end, PyObject *seq = 0)
    {
      return new SwigPyMapKeyIterator_T<OutIter>(current, begin, end, seq);
    }

    template<class OutIterator,
	     class FromOper = from_value_oper<typename OutIterator::value_type> >
    struct SwigPyMapValueIterator_T : SwigPyMapIterator_T<OutIterator, FromOper>
    {
      SwigPyMapValueIterator_T(OutIterator curr, OutIterator first, OutIterator last, PyObject *seq)
	: SwigPyMapIterator_T<OutIterator, FromOper>(curr, first, last, seq)
      {
      }
    };
    

    template<typename OutIter>
    inline SwigPyIterator*
    make_output_value_iterator(const OutIter& current, const OutIter& begin, const OutIter& end, PyObject *seq = 0)
    {
      return new SwigPyMapValueIterator_T<OutIter>(current, begin, end, seq);
    }
  }


  namespace swig {
    template <class SwigPySeq, class K, class T, class Compare, class Alloc >
    inline void
    assign(const SwigPySeq& swigpyseq, std::map<K,T,Compare,Alloc > *map) {
      typedef typename std::map<K,T,Compare,Alloc >::value_type value_type;
      typename SwigPySeq::const_iterator it = swigpyseq.begin();
      for (;it != swigpyseq.end(); ++it) {
	map->insert(value_type(it->first, it->second));
      }
    }

    template <class K, class T, class Compare, class Alloc>
    struct traits_asptr<std::map<K,T,Compare,Alloc > >  {
      typedef std::map<K,T,Compare,Alloc > map_type;
      static int asptr(PyObject *obj, map_type **val) {
	int res = SWIG_ERROR;
	SWIG_PYTHON_THREAD_BEGIN_BLOCK;
	if (PyDict_Check(obj)) {
	  SwigVar_PyObject items = PyObject_CallMethod(obj,(char *)"items",NULL);
#if PY_VERSION_HEX >= 0x03000000
          /* In Python 3.x the ".items()" method returns a dict_items object */
          items = PySequence_Fast(items, ".items() didn't return a sequence!");
#endif
	  res = traits_asptr_stdseq<map_type, std::pair<K, T> >::asptr(items, val);
	} else {
	  map_type *p = 0;
	  swig_type_info *descriptor = swig::type_info<map_type>();
	  res = descriptor ? SWIG_ConvertPtr(obj, (void **)&p, descriptor, 0) : SWIG_ERROR;
	  if (SWIG_IsOK(res) && val)  *val = p;
	}
	SWIG_PYTHON_THREAD_END_BLOCK;
	return res;
      }      
    };
      
    template <class K, class T, class Compare, class Alloc >
    struct traits_from<std::map<K,T,Compare,Alloc > >  {
      typedef std::map<K,T,Compare,Alloc > map_type;
      typedef typename map_type::const_iterator const_iterator;
      typedef typename map_type::size_type size_type;

      static PyObject *asdict(const map_type& map) {
	SWIG_PYTHON_THREAD_BEGIN_BLOCK;
	size_type size = map.size();
	Py_ssize_t pysize = (size <= (size_type) INT_MAX) ? (Py_ssize_t) size : -1;
	if (pysize < 0) {
	  PyErr_SetString(PyExc_OverflowError, "map size not valid in python");
	  SWIG_PYTHON_THREAD_END_BLOCK;
	  return NULL;
	}
	PyObject *obj = PyDict_New();
	for (const_iterator i= map.begin(); i!= map.end(); ++i) {
	  swig::SwigVar_PyObject key = swig::from(i->first);
	  swig::SwigVar_PyObject val = swig::from(i->second);
	  PyDict_SetItem(obj, key, val);
	}
	SWIG_PYTHON_THREAD_END_BLOCK;
	return obj;
      }
                
      static PyObject *from(const map_type& map) {
	swig_type_info *desc = swig::type_info<map_type>();
	if (desc && desc->clientdata) {
	  return SWIG_InternalNewPointerObj(new map_type(map), desc, SWIG_POINTER_OWN);
	} else {
	  return asdict(map);
	}
      }
    };
  }


      namespace swig {
	template <>  struct traits<std::map< std::string, std::string, std::less< std::string >, std::allocator< std::pair< std::string const,std::string > > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::map<" "std::string" "," "std::string" "," "std::less< std::string >" "," "std::allocator< std::pair< std::string const,std::string > >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_map_Sl_std_string_Sc_std_string_Sg__iterator(std::map< std::string,std::string > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_map_Sl_std_string_Sc_std_string_Sg____nonzero__(std::map< std::string,std::string > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_map_Sl_std_string_Sc_std_string_Sg____bool__(std::map< std::string,std::string > const *self){
      return !(self->empty());
    }
SWIGINTERN std::map< std::string,std::string >::size_type std_map_Sl_std_string_Sc_std_string_Sg____len__(std::map< std::string,std::string > const *self){
      return self->size();
    }
SWIGINTERN std::map< std::string,std::string >::mapped_type const &std_map_Sl_std_string_Sc_std_string_Sg____getitem__(std::map< std::string,std::string > *self,std::map< std::string,std::string >::key_type const &key){
      std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > >::const_iterator i = self->find(key);
      if (i != self->end())
	return i->second;
      else
	throw std::out_of_range("key not found");
    }
SWIGINTERN void std_map_Sl_std_string_Sc_std_string_Sg____delitem__(std::map< std::string,std::string > *self,std::map< std::string,std::string >::key_type const &key){
      std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > >::iterator i = self->find(key);
      if (i != self->end())
	self->erase(i);
      else
	throw std::out_of_range("key not found");
    }
SWIGINTERN bool std_map_Sl_std_string_Sc_std_string_Sg__has_key(std::map< std::string,std::string > const *self,std::map< std::string,std::string >::key_type const &key){
      std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > >::const_iterator i = self->find(key);
      return i != self->end();
    }
SWIGINTERN PyObject *std_map_Sl_std_string_Sc_std_string_Sg__keys(std::map< std::string,std::string > *self){
      std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > >::size_type size = self->size();
      Py_ssize_t pysize = (size <= (std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > >::size_type) INT_MAX) ? (Py_ssize_t) size : -1;
      SWIG_PYTHON_THREAD_BEGIN_BLOCK;
      if (pysize < 0) {
	PyErr_SetString(PyExc_OverflowError, "map size not valid in python");
	SWIG_PYTHON_THREAD_END_BLOCK;
	return NULL;
      }
      PyObject* keyList = PyList_New(pysize);
      std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > >::const_iterator i = self->begin();
      for (Py_ssize_t j = 0; j < pysize; ++i, ++j) {
	PyList_SET_ITEM(keyList, j, swig::from(i->first));
      }
      SWIG_PYTHON_THREAD_END_BLOCK;
      return keyList;
    }
SWIGINTERN PyObject *std_map_Sl_std_string_Sc_std_string_Sg__values(std::map< std::string,std::string > *self){
      std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > >::size_type size = self->size();
      Py_ssize_t pysize = (size <= (std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > >::size_type) INT_MAX) ? (Py_ssize_t) size : -1;
      SWIG_PYTHON_THREAD_BEGIN_BLOCK;
      if (pysize < 0) {
	PyErr_SetString(PyExc_OverflowError, "map size not valid in python");
	SWIG_PYTHON_THREAD_END_BLOCK;
	return NULL;
      }
      PyObject* valList = PyList_New(pysize);
      std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > >::const_iterator i = self->begin();
      for (Py_ssize_t j = 0; j < pysize; ++i, ++j) {
	PyList_SET_ITEM(valList, j, swig::from(i->second));
      }
      SWIG_PYTHON_THREAD_END_BLOCK;
      return valList;
    }
SWIGINTERN PyObject *std_map_Sl_std_string_Sc_std_string_Sg__items(std::map< std::string,std::string > *self){
      std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > >::size_type size = self->size();
      Py_ssize_t pysize = (size <= (std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > >::size_type) INT_MAX) ? (Py_ssize_t) size : -1;
      SWIG_PYTHON_THREAD_BEGIN_BLOCK;
      if (pysize < 0) {
	PyErr_SetString(PyExc_OverflowError, "map size not valid in python");
	SWIG_PYTHON_THREAD_END_BLOCK;
	return NULL;
      }    
      PyObject* itemList = PyList_New(pysize);
      std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > >::const_iterator i = self->begin();
      for (Py_ssize_t j = 0; j < pysize; ++i, ++j) {
	PyList_SET_ITEM(itemList, j, swig::from(*i));
      }
      SWIG_PYTHON_THREAD_END_BLOCK;
      return itemList;
    }
SWIGINTERN bool std_map_Sl_std_string_Sc_std_string_Sg____contains__(std::map< std::string,std::string > *self,std::map< std::string,std::string >::key_type const &key){
      return self->find(key) != self->end();
    }
SWIGINTERN swig::SwigPyIterator *std_map_Sl_std_string_Sc_std_string_Sg__key_iterator(std::map< std::string,std::string > *self,PyObject **PYTHON_SELF){
      return swig::make_output_key_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN swig::SwigPyIterator *std_map_Sl_std_string_Sc_std_string_Sg__value_iterator(std::map< std::string,std::string > *self,PyObject **PYTHON_SELF){
      return swig::make_output_value_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN void std_map_Sl_std_string_Sc_std_string_Sg____setitem____SWIG_0(std::map< std::string,std::string > *self,std::map< std::string,std::string >::key_type const &key){
      self->erase(key);
    }
SWIGINTERN void std_map_Sl_std_string_Sc_std_string_Sg____setitem____SWIG_1(std::map< std::string,std::string > *self,std::map< std::string,std::string >::key_type const &key,std::map< std::string,std::string >::mapped_type const &x){
      (*self)[key] = x;
    }
SWIGINTERN PyObject *std_map_Sl_std_string_Sc_std_string_Sg__asdict(std::map< std::string,std::string > *self){
      return swig::traits_from< std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > >::asdict(*self);
    }
SWIGINTERN void std_map_Sl_std_string_Sc_std_string_Sg__erase__SWIG_1(std::map< std::string,std::string > *self,std::map< std::string,std::string >::iterator position){ self->erase(position); }
SWIGINTERN void std_map_Sl_std_string_Sc_std_string_Sg__erase__SWIG_2(std::map< std::string,std::string > *self,std::map< std::string,std::string >::iterator first,std::map< std::string,std::string >::iterator last){ self->erase(first, last); }

		SWIGINTERNINLINE PyObject *
		SWIG_From_MessageStringPtr    (const RobotRaconteur::MessageStringPtr& m)
		{
			return SWIG_From_std_string(m.str().to_string());
		}
	

template<class T> struct SWIG_intrusive_deleter {
    void operator()(T *p) {
        if (p) 
          intrusive_ptr_release(p);
    }
};


struct SWIG_null_deleter {
  void operator() (void const *) const {
  }
};
#define SWIG_NO_NULL_DELETER_0 , SWIG_null_deleter()
#define SWIG_NO_NULL_DELETER_1
#define SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW
#define SWIG_NO_NULL_DELETER_SWIG_POINTER_OWN


#define SWIG_NO_NULL_DELETER_SWIG_BUILTIN_INIT


  namespace swig {
    template <>  struct traits< RobotRaconteur::ServicePathSegment > {
      typedef pointer_category category;
      static const char* type_name() { return"RobotRaconteur::ServicePathSegment"; }
    };
  }


      namespace swig {
	template <>  struct traits<std::vector< RobotRaconteur::ServicePathSegment, std::allocator< RobotRaconteur::ServicePathSegment > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "RobotRaconteur::ServicePathSegment" "," "std::allocator< RobotRaconteur::ServicePathSegment >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg__iterator(std::vector< RobotRaconteur::ServicePathSegment > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____nonzero__(std::vector< RobotRaconteur::ServicePathSegment > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____bool__(std::vector< RobotRaconteur::ServicePathSegment > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< RobotRaconteur::ServicePathSegment >::size_type std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____len__(std::vector< RobotRaconteur::ServicePathSegment > const *self){
      return self->size();
    }
SWIGINTERN std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > *std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____getslice__(std::vector< RobotRaconteur::ServicePathSegment > *self,std::vector< RobotRaconteur::ServicePathSegment >::difference_type i,std::vector< RobotRaconteur::ServicePathSegment >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____setslice____SWIG_0(std::vector< RobotRaconteur::ServicePathSegment > *self,std::vector< RobotRaconteur::ServicePathSegment >::difference_type i,std::vector< RobotRaconteur::ServicePathSegment >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >());
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____setslice____SWIG_1(std::vector< RobotRaconteur::ServicePathSegment > *self,std::vector< RobotRaconteur::ServicePathSegment >::difference_type i,std::vector< RobotRaconteur::ServicePathSegment >::difference_type j,std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____delslice__(std::vector< RobotRaconteur::ServicePathSegment > *self,std::vector< RobotRaconteur::ServicePathSegment >::difference_type i,std::vector< RobotRaconteur::ServicePathSegment >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____delitem____SWIG_0(std::vector< RobotRaconteur::ServicePathSegment > *self,std::vector< RobotRaconteur::ServicePathSegment >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > *std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____getitem____SWIG_0(std::vector< RobotRaconteur::ServicePathSegment > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >::difference_type id = i;
      std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____setitem____SWIG_0(std::vector< RobotRaconteur::ServicePathSegment > *self,PySliceObject *slice,std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >::difference_type id = i;
      std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____setitem____SWIG_1(std::vector< RobotRaconteur::ServicePathSegment > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >::difference_type id = i;
      std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____delitem____SWIG_1(std::vector< RobotRaconteur::ServicePathSegment > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >::difference_type id = i;
      std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< RobotRaconteur::ServicePathSegment >::value_type const &std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____getitem____SWIG_1(std::vector< RobotRaconteur::ServicePathSegment > const *self,std::vector< RobotRaconteur::ServicePathSegment >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____setitem____SWIG_2(std::vector< RobotRaconteur::ServicePathSegment > *self,std::vector< RobotRaconteur::ServicePathSegment >::difference_type i,std::vector< RobotRaconteur::ServicePathSegment >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< RobotRaconteur::ServicePathSegment >::value_type std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg__pop(std::vector< RobotRaconteur::ServicePathSegment > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg__append(std::vector< RobotRaconteur::ServicePathSegment > *self,std::vector< RobotRaconteur::ServicePathSegment >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< RobotRaconteur::ServicePathSegment >::iterator std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg__erase__SWIG_0(std::vector< RobotRaconteur::ServicePathSegment > *self,std::vector< RobotRaconteur::ServicePathSegment >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< RobotRaconteur::ServicePathSegment >::iterator std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg__erase__SWIG_1(std::vector< RobotRaconteur::ServicePathSegment > *self,std::vector< RobotRaconteur::ServicePathSegment >::iterator first,std::vector< RobotRaconteur::ServicePathSegment >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< RobotRaconteur::ServicePathSegment >::iterator std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg__insert__SWIG_0(std::vector< RobotRaconteur::ServicePathSegment > *self,std::vector< RobotRaconteur::ServicePathSegment >::iterator pos,std::vector< RobotRaconteur::ServicePathSegment >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg__insert__SWIG_1(std::vector< RobotRaconteur::ServicePathSegment > *self,std::vector< RobotRaconteur::ServicePathSegment >::iterator pos,std::vector< RobotRaconteur::ServicePathSegment >::size_type n,std::vector< RobotRaconteur::ServicePathSegment >::value_type const &x){ self->insert(pos, n, x); }

SWIGINTERN int
SWIG_AsCharArray(PyObject * obj, char *val, size_t size)
{ 
  char* cptr = 0; size_t csize = 0; int alloc = SWIG_OLDOBJ;
  int res = SWIG_AsCharPtrAndSize(obj, &cptr, &csize, &alloc);
  if (SWIG_IsOK(res)) {
    /* special case of single char conversion when we don't need space for NUL */
    if (size == 1 && csize == 2 && cptr && !cptr[1]) --csize;
    if (csize <= size) {
      if (val) {
	if (csize) memcpy(val, cptr, csize*sizeof(char));
	if (csize < size) memset(val + csize, 0, (size - csize)*sizeof(char));
      }
      if (alloc == SWIG_NEWOBJ) {
	delete[] cptr;
	res = SWIG_DelNewMask(res);
      }      
      return res;
    }
    if (alloc == SWIG_NEWOBJ) delete[] cptr;
  }
  return SWIG_TypeError;
}

SWIGINTERN RobotRaconteur::NodeID RobotRaconteur_NodeID_FromByteArray(char const bytes[16]){
		boost::array<uint8_t,16> b = {};
		memcpy(&b[0],bytes,16);

		NodeID n(b);
		return n;
	}

  namespace swig {
    template <>  struct traits< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > {
      typedef pointer_category category;
      static const char* type_name() { return"boost::intrusive_ptr< RobotRaconteur::MessageEntry >"; }
    };
  }


      namespace swig {
	template <>  struct traits<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >, std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "boost::intrusive_ptr< RobotRaconteur::MessageEntry >" "," "std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg__iterator(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____nonzero__(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____bool__(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____len__(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *self){
      return self->size();
    }
SWIGINTERN std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > *std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____getslice__(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type i,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____setslice____SWIG_0(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type i,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >());
    }
SWIGINTERN void std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____setslice____SWIG_1(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type i,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type j,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____delslice__(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type i,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____delitem____SWIG_0(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > *std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____getitem____SWIG_0(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >::difference_type id = i;
      std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____setitem____SWIG_0(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *self,PySliceObject *slice,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >::difference_type id = i;
      std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____setitem____SWIG_1(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >::difference_type id = i;
      std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____delitem____SWIG_1(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >::difference_type id = i;
      std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type const &std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____getitem____SWIG_1(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____setitem____SWIG_2(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type i,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg__pop(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg__append(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg__erase__SWIG_0(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg__erase__SWIG_1(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator first,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg__insert__SWIG_0(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator pos,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg__insert__SWIG_1(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator pos,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type n,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type const &x){ self->insert(pos, n, x); }

  namespace swig {
    template <>  struct traits< boost::intrusive_ptr< RobotRaconteur::MessageElement > > {
      typedef pointer_category category;
      static const char* type_name() { return"boost::intrusive_ptr< RobotRaconteur::MessageElement >"; }
    };
  }


      namespace swig {
	template <>  struct traits<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >, std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "boost::intrusive_ptr< RobotRaconteur::MessageElement >" "," "std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg__iterator(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____nonzero__(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____bool__(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____len__(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *self){
      return self->size();
    }
SWIGINTERN std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____getslice__(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type i,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____setslice____SWIG_0(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type i,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >());
    }
SWIGINTERN void std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____setslice____SWIG_1(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type i,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type j,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____delslice__(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type i,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____delitem____SWIG_0(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____getitem____SWIG_0(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >::difference_type id = i;
      std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____setitem____SWIG_0(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *self,PySliceObject *slice,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >::difference_type id = i;
      std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____setitem____SWIG_1(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >::difference_type id = i;
      std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____delitem____SWIG_1(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >::difference_type id = i;
      std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type const &std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____getitem____SWIG_1(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____setitem____SWIG_2(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type i,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg__pop(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg__append(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg__erase__SWIG_0(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg__erase__SWIG_1(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator first,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg__insert__SWIG_0(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator pos,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg__insert__SWIG_1(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *self,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator pos,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type n,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type const &x){ self->insert(pos, n, x); }

SWIGINTERN int
SWIG_AsVal_unsigned_SS_short (PyObject * obj, unsigned short *val)
{
  unsigned long v;
  int res = SWIG_AsVal_unsigned_SS_long (obj, &v);
  if (SWIG_IsOK(res)) {
    if ((v > USHRT_MAX)) {
      return SWIG_OverflowError;
    } else {
      if (val) *val = static_cast< unsigned short >(v);
    }
  }  
  return res;
}


SWIGINTERNINLINE PyObject *
SWIG_From_unsigned_SS_short  (unsigned short value)
{    
  return SWIG_From_unsigned_SS_long  (value);
}


		SWIGINTERN int
		SWIG_AsPtr_MessageStringPtr (PyObject * obj, RobotRaconteur::MessageStringPtr **val)
		{
			std::string temp1;
			int res = SWIG_AsVal_std_string (obj, &temp1);
			if (!SWIG_IsOK(res))
			{
				return res;
			}
			*val = new MessageStringPtr(temp1);
			return SWIG_NEWOBJ;
		}
	


class MessageElementDataUtil
{
public:
	static boost::intrusive_ptr<RobotRaconteur::RRBaseArray> ToRRBaseArray(const boost::intrusive_ptr<RobotRaconteur::MessageElementData>& m)
	{
		return boost::dynamic_pointer_cast<RRBaseArray>(m);
	}

	static boost::intrusive_ptr<RobotRaconteur::MessageElementNestedElementList> ToMessageElementNestedElementList(const boost::intrusive_ptr<RobotRaconteur::MessageElementData>& m)
	{
		return boost::dynamic_pointer_cast<MessageElementNestedElementList>(m);
	}
















































































































































































































































































































};



boost::intrusive_ptr<RobotRaconteur::Message> MessageFromBytes(const uint8_t* bytes, size_t bytes_len)
{
	using namespace RobotRaconteur;
	ArrayBinaryReader r(bytes, 0, bytes_len);
	r.Seek(8);
	uint16_t ver = r.ReadNumber<uint16_t>();
	r.Seek(0);
	boost::intrusive_ptr<Message> m = CreateMessage();
	switch (ver)
	{
	case 2:
		m->Read(r);
		break;
	case 4:
		m->Read4(r);
		break;
	default:
		throw InvalidArgumentException("Invalid message version");
	}
	return m;
}
size_t MessageLengthFromBytes(const uint8_t* bytes, size_t bytes_len)
{
	using namespace RobotRaconteur;
	ArrayBinaryReader r(bytes, 0, bytes_len);
	std::string magic = r.ReadString8(4).str().to_string();
	if (magic != "RRAC")
		throw ProtocolException("Invalid message magic");

	return r.ReadNumber<uint32_t>();
}
PyObject* MessageToBytes(boost::intrusive_ptr<RobotRaconteur::Message> m, uint32_t ver = 2)
{
	using namespace RobotRaconteur;
	if (m==NULL)
	{
		throw std::invalid_argument("m must not be null");
	}

	switch (ver)
	{
	case 2:
	{
		size_t len = m->ComputeSize();
		boost::shared_array<uint8_t> buf(new uint8_t[len]);
		ArrayBinaryWriter w(buf.get(), 0, len);
		m->Write(w);
		return PyByteArray_FromStringAndSize((char*)buf.get(),len);
	}
	case 4:
	{
		size_t len = m->ComputeSize4();
		boost::shared_array<uint8_t> buf(new uint8_t[len]);
		ArrayBinaryWriter w(buf.get(), 0, len);
		m->Write4(w);
		return PyByteArray_FromStringAndSize((char*)buf.get(),len);
	}
	default:
		throw InvalidArgumentException("Invalid message version");
	}
}

boost::intrusive_ptr<RobotRaconteur::MessageElement> MessageElementFromBytes(const uint8_t* bytes, size_t bytes_len)
{
	using namespace RobotRaconteur;
	ArrayBinaryReader r(bytes, 0, bytes_len);
	boost::intrusive_ptr<MessageElement> m = CreateMessageElement();
	m->Read(r);
	return m;
}
size_t MessageElementLengthFromBytes(const uint8_t* bytes, size_t bytes_len)
{
	using namespace RobotRaconteur;
	ArrayBinaryReader r(bytes, 0, bytes_len);
	return r.ReadNumber<uint32_t>();
}
PyObject* MessageElementToBytes(const boost::intrusive_ptr<RobotRaconteur::MessageElement>& m, uint32_t ver = 2)
{
	using namespace RobotRaconteur;
	if (m==NULL)
	{
		throw std::invalid_argument("m must not be null");
	}
	m->UpdateData();
	size_t len = m->ComputeSize();
	boost::shared_array<uint8_t> buf(new uint8_t[len]);
	ArrayBinaryWriter w(buf.get(), 0, len);
	m->Write(w);
	return PyByteArray_FromStringAndSize((char*)buf.get(),len);

}



  namespace swig {
    template <>  struct traits< boost::shared_ptr< RobotRaconteur::TypeDefinition > > {
      typedef pointer_category category;
      static const char* type_name() { return"boost::shared_ptr< RobotRaconteur::TypeDefinition >"; }
    };
  }


      namespace swig {
	template <>  struct traits<std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >, std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "boost::shared_ptr< RobotRaconteur::TypeDefinition >" "," "std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg__iterator(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____nonzero__(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____bool__(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____len__(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *self){
      return self->size();
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____getslice__(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____setslice____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >());
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____setslice____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type j,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____delslice__(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____delitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____getitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____setitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *self,PySliceObject *slice,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____setitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____delitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type const &std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____getitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *self,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____setitem____SWIG_2(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg__pop(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg__append(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg__erase__SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg__erase__SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator first,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg__insert__SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator pos,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg__insert__SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator pos,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type n,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type const &x){ self->insert(pos, n, x); }

  namespace swig {
    template <>  struct traits< boost::shared_ptr< RobotRaconteur::MemberDefinition > > {
      typedef pointer_category category;
      static const char* type_name() { return"boost::shared_ptr< RobotRaconteur::MemberDefinition >"; }
    };
  }


      namespace swig {
	template <>  struct traits<std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >, std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "boost::shared_ptr< RobotRaconteur::MemberDefinition >" "," "std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg__iterator(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____nonzero__(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____bool__(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____len__(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *self){
      return self->size();
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____getslice__(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____setslice____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >());
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____setslice____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type j,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____delslice__(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____delitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____getitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____setitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *self,PySliceObject *slice,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____setitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____delitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type const &std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____getitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *self,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____setitem____SWIG_2(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg__pop(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg__append(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg__erase__SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg__erase__SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator first,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg__insert__SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator pos,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg__insert__SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator pos,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type n,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type const &x){ self->insert(pos, n, x); }

  namespace swig {
    template <>  struct traits< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > {
      typedef pointer_category category;
      static const char* type_name() { return"boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >"; }
    };
  }


      namespace swig {
	template <>  struct traits<std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >, std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >" "," "std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg__iterator(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____nonzero__(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____bool__(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____len__(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *self){
      return self->size();
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____getslice__(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____setslice____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >());
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____setslice____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type j,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____delslice__(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____delitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____getitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____setitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *self,PySliceObject *slice,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____setitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____delitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type const &std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____getitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____setitem____SWIG_2(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg__pop(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg__append(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg__erase__SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg__erase__SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator first,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg__insert__SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator pos,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg__insert__SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator pos,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type n,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type const &x){ self->insert(pos, n, x); }

  namespace swig {
    template <>  struct traits< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > {
      typedef pointer_category category;
      static const char* type_name() { return"boost::shared_ptr< RobotRaconteur::ConstantDefinition >"; }
    };
  }


      namespace swig {
	template <>  struct traits<std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >, std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "boost::shared_ptr< RobotRaconteur::ConstantDefinition >" "," "std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg__iterator(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____nonzero__(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____bool__(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____len__(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *self){
      return self->size();
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____getslice__(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____setslice____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >());
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____setslice____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type j,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____delslice__(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____delitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____getitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____setitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *self,PySliceObject *slice,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____setitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____delitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type const &std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____getitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *self,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____setitem____SWIG_2(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg__pop(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg__append(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg__erase__SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg__erase__SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator first,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg__insert__SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator pos,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg__insert__SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator pos,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type n,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type const &x){ self->insert(pos, n, x); }

  namespace swig {
    template <>  struct traits< boost::shared_ptr< RobotRaconteur::EnumDefinition > > {
      typedef pointer_category category;
      static const char* type_name() { return"boost::shared_ptr< RobotRaconteur::EnumDefinition >"; }
    };
  }


      namespace swig {
	template <>  struct traits<std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >, std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "boost::shared_ptr< RobotRaconteur::EnumDefinition >" "," "std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg__iterator(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____nonzero__(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____bool__(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____len__(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *self){
      return self->size();
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____getslice__(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____setslice____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >());
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____setslice____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type j,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____delslice__(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____delitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____getitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____setitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *self,PySliceObject *slice,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____setitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____delitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type const &std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____getitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *self,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____setitem____SWIG_2(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg__pop(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg__append(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg__erase__SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg__erase__SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator first,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg__insert__SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator pos,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg__insert__SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator pos,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type n,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type const &x){ self->insert(pos, n, x); }

  namespace swig {
    template <>  struct traits< RobotRaconteur::ConstantDefinition_StructField > {
      typedef pointer_category category;
      static const char* type_name() { return"RobotRaconteur::ConstantDefinition_StructField"; }
    };
  }


      namespace swig {
	template <>  struct traits<std::vector< RobotRaconteur::ConstantDefinition_StructField, std::allocator< RobotRaconteur::ConstantDefinition_StructField > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "RobotRaconteur::ConstantDefinition_StructField" "," "std::allocator< RobotRaconteur::ConstantDefinition_StructField >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg__iterator(std::vector< RobotRaconteur::ConstantDefinition_StructField > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____nonzero__(std::vector< RobotRaconteur::ConstantDefinition_StructField > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____bool__(std::vector< RobotRaconteur::ConstantDefinition_StructField > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____len__(std::vector< RobotRaconteur::ConstantDefinition_StructField > const *self){
      return self->size();
    }
SWIGINTERN std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > *std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____getslice__(std::vector< RobotRaconteur::ConstantDefinition_StructField > *self,std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type i,std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____setslice____SWIG_0(std::vector< RobotRaconteur::ConstantDefinition_StructField > *self,std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type i,std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >());
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____setslice____SWIG_1(std::vector< RobotRaconteur::ConstantDefinition_StructField > *self,std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type i,std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type j,std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____delslice__(std::vector< RobotRaconteur::ConstantDefinition_StructField > *self,std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type i,std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____delitem____SWIG_0(std::vector< RobotRaconteur::ConstantDefinition_StructField > *self,std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > *std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____getitem____SWIG_0(std::vector< RobotRaconteur::ConstantDefinition_StructField > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >::difference_type id = i;
      std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____setitem____SWIG_0(std::vector< RobotRaconteur::ConstantDefinition_StructField > *self,PySliceObject *slice,std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >::difference_type id = i;
      std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____setitem____SWIG_1(std::vector< RobotRaconteur::ConstantDefinition_StructField > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >::difference_type id = i;
      std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____delitem____SWIG_1(std::vector< RobotRaconteur::ConstantDefinition_StructField > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >::difference_type id = i;
      std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____getitem____SWIG_1(std::vector< RobotRaconteur::ConstantDefinition_StructField > const *self,std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____setitem____SWIG_2(std::vector< RobotRaconteur::ConstantDefinition_StructField > *self,std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type i,std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg__pop(std::vector< RobotRaconteur::ConstantDefinition_StructField > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg__append(std::vector< RobotRaconteur::ConstantDefinition_StructField > *self,std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg__erase__SWIG_0(std::vector< RobotRaconteur::ConstantDefinition_StructField > *self,std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg__erase__SWIG_1(std::vector< RobotRaconteur::ConstantDefinition_StructField > *self,std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator first,std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg__insert__SWIG_0(std::vector< RobotRaconteur::ConstantDefinition_StructField > *self,std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator pos,std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg__insert__SWIG_1(std::vector< RobotRaconteur::ConstantDefinition_StructField > *self,std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator pos,std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type n,std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &x){ self->insert(pos, n, x); }

  namespace swig {
    template <>  struct traits< RobotRaconteur::EnumDefinitionValue > {
      typedef pointer_category category;
      static const char* type_name() { return"RobotRaconteur::EnumDefinitionValue"; }
    };
  }


      namespace swig {
	template <>  struct traits<std::vector< RobotRaconteur::EnumDefinitionValue, std::allocator< RobotRaconteur::EnumDefinitionValue > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "RobotRaconteur::EnumDefinitionValue" "," "std::allocator< RobotRaconteur::EnumDefinitionValue >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg__iterator(std::vector< RobotRaconteur::EnumDefinitionValue > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____nonzero__(std::vector< RobotRaconteur::EnumDefinitionValue > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____bool__(std::vector< RobotRaconteur::EnumDefinitionValue > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< RobotRaconteur::EnumDefinitionValue >::size_type std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____len__(std::vector< RobotRaconteur::EnumDefinitionValue > const *self){
      return self->size();
    }
SWIGINTERN std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > *std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____getslice__(std::vector< RobotRaconteur::EnumDefinitionValue > *self,std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type i,std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____setslice____SWIG_0(std::vector< RobotRaconteur::EnumDefinitionValue > *self,std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type i,std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >());
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____setslice____SWIG_1(std::vector< RobotRaconteur::EnumDefinitionValue > *self,std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type i,std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type j,std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____delslice__(std::vector< RobotRaconteur::EnumDefinitionValue > *self,std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type i,std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____delitem____SWIG_0(std::vector< RobotRaconteur::EnumDefinitionValue > *self,std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > *std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____getitem____SWIG_0(std::vector< RobotRaconteur::EnumDefinitionValue > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >::difference_type id = i;
      std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____setitem____SWIG_0(std::vector< RobotRaconteur::EnumDefinitionValue > *self,PySliceObject *slice,std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >::difference_type id = i;
      std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____setitem____SWIG_1(std::vector< RobotRaconteur::EnumDefinitionValue > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >::difference_type id = i;
      std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____delitem____SWIG_1(std::vector< RobotRaconteur::EnumDefinitionValue > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >::difference_type id = i;
      std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____getitem____SWIG_1(std::vector< RobotRaconteur::EnumDefinitionValue > const *self,std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____setitem____SWIG_2(std::vector< RobotRaconteur::EnumDefinitionValue > *self,std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type i,std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< RobotRaconteur::EnumDefinitionValue >::value_type std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg__pop(std::vector< RobotRaconteur::EnumDefinitionValue > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg__append(std::vector< RobotRaconteur::EnumDefinitionValue > *self,std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< RobotRaconteur::EnumDefinitionValue >::iterator std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg__erase__SWIG_0(std::vector< RobotRaconteur::EnumDefinitionValue > *self,std::vector< RobotRaconteur::EnumDefinitionValue >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< RobotRaconteur::EnumDefinitionValue >::iterator std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg__erase__SWIG_1(std::vector< RobotRaconteur::EnumDefinitionValue > *self,std::vector< RobotRaconteur::EnumDefinitionValue >::iterator first,std::vector< RobotRaconteur::EnumDefinitionValue >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< RobotRaconteur::EnumDefinitionValue >::iterator std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg__insert__SWIG_0(std::vector< RobotRaconteur::EnumDefinitionValue > *self,std::vector< RobotRaconteur::EnumDefinitionValue >::iterator pos,std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg__insert__SWIG_1(std::vector< RobotRaconteur::EnumDefinitionValue > *self,std::vector< RobotRaconteur::EnumDefinitionValue >::iterator pos,std::vector< RobotRaconteur::EnumDefinitionValue >::size_type n,std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &x){ self->insert(pos, n, x); }

  namespace swig {
    template <>  struct traits< boost::shared_ptr< RobotRaconteur::UsingDefinition > > {
      typedef pointer_category category;
      static const char* type_name() { return"boost::shared_ptr< RobotRaconteur::UsingDefinition >"; }
    };
  }


      namespace swig {
	template <>  struct traits<std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >, std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "boost::shared_ptr< RobotRaconteur::UsingDefinition >" "," "std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg__iterator(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____nonzero__(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____bool__(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____len__(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *self){
      return self->size();
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____getslice__(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____setslice____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >());
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____setslice____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type j,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____delslice__(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____delitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____getitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____setitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *self,PySliceObject *slice,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____setitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____delitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type const &std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____getitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *self,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____setitem____SWIG_2(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg__pop(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg__append(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg__erase__SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg__erase__SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator first,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg__insert__SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator pos,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg__insert__SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator pos,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type n,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type const &x){ self->insert(pos, n, x); }

  namespace swig {
    template <>  struct traits< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > {
      typedef pointer_category category;
      static const char* type_name() { return"boost::shared_ptr< RobotRaconteur::ExceptionDefinition >"; }
    };
  }


      namespace swig {
	template <>  struct traits<std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >, std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "boost::shared_ptr< RobotRaconteur::ExceptionDefinition >" "," "std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg__iterator(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____nonzero__(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____bool__(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____len__(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *self){
      return self->size();
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____getslice__(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____setslice____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >());
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____setslice____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type j,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____delslice__(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____delitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____getitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____setitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *self,PySliceObject *slice,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____setitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____delitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type const &std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____getitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *self,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____setitem____SWIG_2(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg__pop(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg__append(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg__erase__SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg__erase__SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator first,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg__insert__SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator pos,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg__insert__SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator pos,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type n,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type const &x){ self->insert(pos, n, x); }

  namespace swig {
    template <>  struct traits< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > {
      typedef pointer_category category;
      static const char* type_name() { return"boost::shared_ptr< RobotRaconteur::ServiceDefinition >"; }
    };
  }


      namespace swig {
	template <>  struct traits<std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >, std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "boost::shared_ptr< RobotRaconteur::ServiceDefinition >" "," "std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg__iterator(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____nonzero__(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____bool__(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____len__(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *self){
      return self->size();
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____getslice__(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____setslice____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >());
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____setslice____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type j,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____delslice__(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____delitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____getitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____setitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *self,PySliceObject *slice,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____setitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____delitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type const &std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____getitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____setitem____SWIG_2(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg__pop(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg__append(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg__erase__SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg__erase__SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator first,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg__insert__SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator pos,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg__insert__SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator pos,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type n,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type const &x){ self->insert(pos, n, x); }
SWIGINTERN boost::shared_ptr< RobotRaconteur::ServiceDefinition > RobotRaconteur_ServiceEntryDefinition_GetServiceDefinition(RobotRaconteur::ServiceEntryDefinition *self){
		return self->ServiceDefinition_.lock();
	}
SWIGINTERN void RobotRaconteur_ServiceEntryDefinition_SetServiceDefinition(RobotRaconteur::ServiceEntryDefinition *self,boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &value){
		self->ServiceDefinition_=value;
	}
SWIGINTERN boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > RobotRaconteur_MemberDefinition_GetServiceEntry(RobotRaconteur::MemberDefinition *self){
		return self->ServiceEntry.lock();
	}
SWIGINTERN void RobotRaconteur_MemberDefinition_SetServiceEntry(RobotRaconteur::MemberDefinition *self,boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &value){
		self->ServiceEntry=value;
	}

SWIGINTERN int
SWIG_AsVal_bool (PyObject *obj, bool *val)
{
  int r;
  if (!PyBool_Check(obj))
    return SWIG_ERROR;
  r = PyObject_IsTrue(obj);
  if (r == -1)
    return SWIG_ERROR;
  if (val) *val = r ? true : false;
  return SWIG_OK;
}

SWIGINTERN boost::shared_ptr< RobotRaconteur::MemberDefinition > RobotRaconteur_TypeDefinition_GetMember(RobotRaconteur::TypeDefinition *self){
		return self->member.lock();
	}
SWIGINTERN void RobotRaconteur_TypeDefinition_SetMember(RobotRaconteur::TypeDefinition *self,boost::shared_ptr< RobotRaconteur::MemberDefinition > const &value){
		self->member=value;
	}
SWIGINTERN boost::shared_ptr< RobotRaconteur::ServiceDefinition > RobotRaconteur_UsingDefinition_GetService(RobotRaconteur::UsingDefinition *self){
			return self->service.lock();
		}
SWIGINTERN void RobotRaconteur_UsingDefinition_SetService(RobotRaconteur::UsingDefinition *self,boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &value){
			self->service=value;
		}
SWIGINTERN boost::shared_ptr< RobotRaconteur::ServiceDefinition > RobotRaconteur_ConstantDefinition_GetService(RobotRaconteur::ConstantDefinition *self){
			return self->service.lock();
		}
SWIGINTERN void RobotRaconteur_ConstantDefinition_SetService(RobotRaconteur::ConstantDefinition *self,boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &value){
			self->service=value;
		}
SWIGINTERN boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > RobotRaconteur_ConstantDefinition_GetServiceEntry(RobotRaconteur::ConstantDefinition *self){
			return self->service_entry.lock();
		}
SWIGINTERN void RobotRaconteur_ConstantDefinition_SetServiceEntry(RobotRaconteur::ConstantDefinition *self,boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &value){
			self->service_entry=value;
		}
SWIGINTERN boost::shared_ptr< RobotRaconteur::ServiceDefinition > RobotRaconteur_EnumDefinition_GetService(RobotRaconteur::EnumDefinition *self){
			return self->service.lock();
		}
SWIGINTERN void RobotRaconteur_EnumDefinition_SetService(RobotRaconteur::EnumDefinition *self,boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &value){
			self->service=value;
		}
SWIGINTERN boost::shared_ptr< RobotRaconteur::ServiceDefinition > RobotRaconteur_ExceptionDefinition_GetService(RobotRaconteur::ExceptionDefinition *self){
			return self->service.lock();
		}
SWIGINTERN void RobotRaconteur_ExceptionDefinition_SetService(RobotRaconteur::ExceptionDefinition *self,boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &value){
			self->service=value;
		}


class MemberDefinitionUtil
{
public:
static boost::shared_ptr<RobotRaconteur::PropertyDefinition> ToProperty(const boost::shared_ptr<RobotRaconteur::MemberDefinition>& t)
{
return boost::dynamic_pointer_cast<RobotRaconteur::PropertyDefinition>(t);
}

static boost::shared_ptr<RobotRaconteur::FunctionDefinition> ToFunction(const boost::shared_ptr<RobotRaconteur::MemberDefinition>& t)
{
return boost::dynamic_pointer_cast<RobotRaconteur::FunctionDefinition>(t);
}

static boost::shared_ptr<RobotRaconteur::ObjRefDefinition> ToObjRef(const boost::shared_ptr<RobotRaconteur::MemberDefinition>& t)
{
return boost::dynamic_pointer_cast<RobotRaconteur::ObjRefDefinition>(t);
}

static boost::shared_ptr<RobotRaconteur::EventDefinition> ToEvent(const boost::shared_ptr<RobotRaconteur::MemberDefinition>& t)
{
return boost::dynamic_pointer_cast<RobotRaconteur::EventDefinition>(t);
}

static boost::shared_ptr<RobotRaconteur::PipeDefinition> ToPipe(const boost::shared_ptr<RobotRaconteur::MemberDefinition>& t)
{
return boost::dynamic_pointer_cast<RobotRaconteur::PipeDefinition>(t);
}

static boost::shared_ptr<RobotRaconteur::CallbackDefinition> ToCallback(const boost::shared_ptr<RobotRaconteur::MemberDefinition>& t)
{
return boost::dynamic_pointer_cast<RobotRaconteur::CallbackDefinition>(t);
}

static boost::shared_ptr<RobotRaconteur::WireDefinition> ToWire(const boost::shared_ptr<RobotRaconteur::MemberDefinition>& t)
{
return boost::dynamic_pointer_cast<RobotRaconteur::WireDefinition>(t);
}

static boost::shared_ptr<RobotRaconteur::MemoryDefinition> ToMemory(const boost::shared_ptr<RobotRaconteur::MemberDefinition>& t)
{
return boost::dynamic_pointer_cast<RobotRaconteur::MemoryDefinition>(t);
}

};

SWIGINTERN std::vector< std::string,std::allocator< std::string > > RobotRaconteur_TcpTransport_GetLocalAdapterIPAddresses(){
		std::vector<std::string> o;
		std::vector<boost::asio::ip::address> addr;
		RobotRaconteur::TcpTransport::GetLocalAdapterIPAddresses(addr);
		for (size_t i=0; i<addr.size(); i++)
		{
			o.push_back(boost::lexical_cast<std::string>(addr[i]));
		}
		return o;
	}
SWIGINTERN RobotRaconteur::WallTimer *new_RobotRaconteur_WallTimer(boost::posix_time::time_duration const &period,bool oneshot,boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &node,RobotRaconteur::AsyncTimerEventReturnDirector *handler,int32_t id){
		boost::shared_ptr<AsyncTimerEventReturnDirector> sphandler(handler,boost::bind(&ReleaseDirector<AsyncTimerEventReturnDirector>,RR_BOOST_PLACEHOLDERS(_1),id));
		return new WallTimer(period,boost::bind(&TimerHandlerFunc,RR_BOOST_PLACEHOLDERS(_1),sphandler),oneshot,node);
	}
SWIGINTERN boost::shared_ptr< RobotRaconteur::WrappedServiceStub > RobotRaconteur_WrappedServiceStub_FindObjRef__SWIG_0(RobotRaconteur::WrappedServiceStub *self,std::string const &n){
		return rr_cast<WrappedServiceStub>(self->FindObjRef(n));
	}
SWIGINTERN boost::shared_ptr< RobotRaconteur::WrappedServiceStub > RobotRaconteur_WrappedServiceStub_FindObjRef__SWIG_1(RobotRaconteur::WrappedServiceStub *self,std::string const &n,std::string const &i){
		return rr_cast<WrappedServiceStub>(self->FindObjRef(n,i));
	}
SWIGINTERN boost::shared_ptr< RobotRaconteur::WrappedServiceStub > RobotRaconteur_WrappedServiceStub_FindObjRefTyped__SWIG_0(RobotRaconteur::WrappedServiceStub *self,std::string const &n,std::string const &objecttype){
		return rr_cast<WrappedServiceStub>(self->FindObjRefTyped(n,objecttype));
	}
SWIGINTERN boost::shared_ptr< RobotRaconteur::WrappedServiceStub > RobotRaconteur_WrappedServiceStub_FindObjRefTyped__SWIG_1(RobotRaconteur::WrappedServiceStub *self,std::string const &n,std::string const &i,std::string const &objecttype){
		return rr_cast<WrappedServiceStub>(self->FindObjRefTyped(n,i,objecttype));
	}

class ServerServiceListenerDirector
{
public:

	int32_t objectheapid;

	ServerServiceListenerDirector()
	{
		objectheapid=0;
	}

	void OuterCallback(const boost::shared_ptr<ServerContext>& c,ServerServiceListenerEventType code,const boost::shared_ptr<void>& p)
	{
		if (code==ServerServiceListenerEventType_ClientConnected || code==ServerServiceListenerEventType_ClientDisconnected)
		{
			boost::shared_ptr<uint32_t> p2=boost::static_pointer_cast<uint32_t>(p);

			DIRECTOR_CALL2(Callback(code,*p2));
		}
		else
		{

			DIRECTOR_CALL2(Callback(code,0));
		}
	}
	virtual void Callback(int32_t code, uint32_t endpoint) {};



	virtual ~ServerServiceListenerDirector()
	{

	}

};

SWIGINTERN void RobotRaconteur_ServerContext_AddServerServiceListener(RobotRaconteur::ServerContext *self,ServerServiceListenerDirector *listener){
		boost::shared_ptr<ServerServiceListenerDirector> listener2=boost::shared_ptr<ServerServiceListenerDirector>(listener,boost::bind(&ReleaseDirector<ServerServiceListenerDirector>, RR_BOOST_PLACEHOLDERS(_1), listener->objectheapid));
		self->ServerServiceListener.connect(boost::bind(&ServerServiceListenerDirector::OuterCallback,listener2,RR_BOOST_PLACEHOLDERS(_1),RR_BOOST_PLACEHOLDERS(_2),RR_BOOST_PLACEHOLDERS(_3)));

	}
SWIGINTERN void RobotRaconteur_ServerContext_SetServiceAttributes(RobotRaconteur::ServerContext *self,boost::intrusive_ptr< RobotRaconteur::MessageElement > const &attributes){
		boost::intrusive_ptr<RRMap<std::string,RRValue> > mmap=rr_cast<RRMap<std::string,RRValue> >(RobotRaconteur::detail::packing::UnpackMapType<std::string,RRValue>(attributes->CastData<MessageElementNestedElementList >(),NULL));
		self->SetAttributes(mmap->GetStorageContainer());

	}
SWIGINTERN uint32_t RobotRaconteur_ServerEndpoint_GetCurrentEndpoint(){
			return ServerEndpoint::GetCurrentEndpoint()->GetLocalEndpoint();
		}
SWIGINTERN boost::shared_ptr< RobotRaconteur::AuthenticatedUser > RobotRaconteur_UserAuthenticator_AuthenticateUser(RobotRaconteur::UserAuthenticator *self,std::string const &username,boost::intrusive_ptr< RobotRaconteur::MessageElement > const &credentials,boost::shared_ptr< RobotRaconteur::ServerContext > const &context){
		if (!context) throw InvalidArgumentException("Context cannot be null");
		if (!credentials) throw InvalidArgumentException("Credentials cannot be null");
		boost::intrusive_ptr<RRMap<std::string,RRValue> > r=rr_cast<RRMap<std::string,RRValue> >(context->GetNode()->UnpackMapType<std::string,RRValue>(credentials->CastData<MessageElementNestedElementList >()));
		return self->AuthenticateUser(username,r->GetStorageContainer(),context,RR_SHARED_PTR<RobotRaconteur::ITransportConnection>());
	}

  namespace swig {
    template <>  struct traits< RobotRaconteur::ServiceInfo2Wrapped > {
      typedef pointer_category category;
      static const char* type_name() { return"RobotRaconteur::ServiceInfo2Wrapped"; }
    };
  }


      namespace swig {
	template <>  struct traits<std::vector< RobotRaconteur::ServiceInfo2Wrapped, std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "RobotRaconteur::ServiceInfo2Wrapped" "," "std::allocator< RobotRaconteur::ServiceInfo2Wrapped >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg__iterator(std::vector< RobotRaconteur::ServiceInfo2Wrapped > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____nonzero__(std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____bool__(std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____len__(std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *self){
      return self->size();
    }
SWIGINTERN std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > *std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____getslice__(std::vector< RobotRaconteur::ServiceInfo2Wrapped > *self,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type i,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____setslice____SWIG_0(std::vector< RobotRaconteur::ServiceInfo2Wrapped > *self,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type i,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >());
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____setslice____SWIG_1(std::vector< RobotRaconteur::ServiceInfo2Wrapped > *self,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type i,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type j,std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____delslice__(std::vector< RobotRaconteur::ServiceInfo2Wrapped > *self,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type i,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____delitem____SWIG_0(std::vector< RobotRaconteur::ServiceInfo2Wrapped > *self,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > *std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____getitem____SWIG_0(std::vector< RobotRaconteur::ServiceInfo2Wrapped > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >::difference_type id = i;
      std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____setitem____SWIG_0(std::vector< RobotRaconteur::ServiceInfo2Wrapped > *self,PySliceObject *slice,std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >::difference_type id = i;
      std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____setitem____SWIG_1(std::vector< RobotRaconteur::ServiceInfo2Wrapped > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >::difference_type id = i;
      std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____delitem____SWIG_1(std::vector< RobotRaconteur::ServiceInfo2Wrapped > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >::difference_type id = i;
      std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____getitem____SWIG_1(std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *self,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____setitem____SWIG_2(std::vector< RobotRaconteur::ServiceInfo2Wrapped > *self,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type i,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg__pop(std::vector< RobotRaconteur::ServiceInfo2Wrapped > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg__append(std::vector< RobotRaconteur::ServiceInfo2Wrapped > *self,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg__erase__SWIG_0(std::vector< RobotRaconteur::ServiceInfo2Wrapped > *self,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg__erase__SWIG_1(std::vector< RobotRaconteur::ServiceInfo2Wrapped > *self,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator first,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg__insert__SWIG_0(std::vector< RobotRaconteur::ServiceInfo2Wrapped > *self,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator pos,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg__insert__SWIG_1(std::vector< RobotRaconteur::ServiceInfo2Wrapped > *self,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator pos,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type n,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &x){ self->insert(pos, n, x); }

  namespace swig {
    template <>  struct traits< RobotRaconteur::NodeInfo2 > {
      typedef pointer_category category;
      static const char* type_name() { return"RobotRaconteur::NodeInfo2"; }
    };
  }


      namespace swig {
	template <>  struct traits<std::vector< RobotRaconteur::NodeInfo2, std::allocator< RobotRaconteur::NodeInfo2 > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "RobotRaconteur::NodeInfo2" "," "std::allocator< RobotRaconteur::NodeInfo2 >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_RobotRaconteur_NodeInfo2_Sg__iterator(std::vector< RobotRaconteur::NodeInfo2 > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____nonzero__(std::vector< RobotRaconteur::NodeInfo2 > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____bool__(std::vector< RobotRaconteur::NodeInfo2 > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< RobotRaconteur::NodeInfo2 >::size_type std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____len__(std::vector< RobotRaconteur::NodeInfo2 > const *self){
      return self->size();
    }
SWIGINTERN std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > *std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____getslice__(std::vector< RobotRaconteur::NodeInfo2 > *self,std::vector< RobotRaconteur::NodeInfo2 >::difference_type i,std::vector< RobotRaconteur::NodeInfo2 >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____setslice____SWIG_0(std::vector< RobotRaconteur::NodeInfo2 > *self,std::vector< RobotRaconteur::NodeInfo2 >::difference_type i,std::vector< RobotRaconteur::NodeInfo2 >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >());
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____setslice____SWIG_1(std::vector< RobotRaconteur::NodeInfo2 > *self,std::vector< RobotRaconteur::NodeInfo2 >::difference_type i,std::vector< RobotRaconteur::NodeInfo2 >::difference_type j,std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____delslice__(std::vector< RobotRaconteur::NodeInfo2 > *self,std::vector< RobotRaconteur::NodeInfo2 >::difference_type i,std::vector< RobotRaconteur::NodeInfo2 >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____delitem____SWIG_0(std::vector< RobotRaconteur::NodeInfo2 > *self,std::vector< RobotRaconteur::NodeInfo2 >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > *std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____getitem____SWIG_0(std::vector< RobotRaconteur::NodeInfo2 > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >::difference_type id = i;
      std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____setitem____SWIG_0(std::vector< RobotRaconteur::NodeInfo2 > *self,PySliceObject *slice,std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >::difference_type id = i;
      std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____setitem____SWIG_1(std::vector< RobotRaconteur::NodeInfo2 > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >::difference_type id = i;
      std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____delitem____SWIG_1(std::vector< RobotRaconteur::NodeInfo2 > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >::difference_type id = i;
      std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< RobotRaconteur::NodeInfo2 >::value_type const &std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____getitem____SWIG_1(std::vector< RobotRaconteur::NodeInfo2 > const *self,std::vector< RobotRaconteur::NodeInfo2 >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____setitem____SWIG_2(std::vector< RobotRaconteur::NodeInfo2 > *self,std::vector< RobotRaconteur::NodeInfo2 >::difference_type i,std::vector< RobotRaconteur::NodeInfo2 >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< RobotRaconteur::NodeInfo2 >::value_type std_vector_Sl_RobotRaconteur_NodeInfo2_Sg__pop(std::vector< RobotRaconteur::NodeInfo2 > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_NodeInfo2_Sg__append(std::vector< RobotRaconteur::NodeInfo2 > *self,std::vector< RobotRaconteur::NodeInfo2 >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< RobotRaconteur::NodeInfo2 >::iterator std_vector_Sl_RobotRaconteur_NodeInfo2_Sg__erase__SWIG_0(std::vector< RobotRaconteur::NodeInfo2 > *self,std::vector< RobotRaconteur::NodeInfo2 >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< RobotRaconteur::NodeInfo2 >::iterator std_vector_Sl_RobotRaconteur_NodeInfo2_Sg__erase__SWIG_1(std::vector< RobotRaconteur::NodeInfo2 > *self,std::vector< RobotRaconteur::NodeInfo2 >::iterator first,std::vector< RobotRaconteur::NodeInfo2 >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< RobotRaconteur::NodeInfo2 >::iterator std_vector_Sl_RobotRaconteur_NodeInfo2_Sg__insert__SWIG_0(std::vector< RobotRaconteur::NodeInfo2 > *self,std::vector< RobotRaconteur::NodeInfo2 >::iterator pos,std::vector< RobotRaconteur::NodeInfo2 >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_RobotRaconteur_NodeInfo2_Sg__insert__SWIG_1(std::vector< RobotRaconteur::NodeInfo2 > *self,std::vector< RobotRaconteur::NodeInfo2 >::iterator pos,std::vector< RobotRaconteur::NodeInfo2 >::size_type n,std::vector< RobotRaconteur::NodeInfo2 >::value_type const &x){ self->insert(pos, n, x); }

  namespace swig {
    template <>  struct traits< RobotRaconteur::ServiceSubscriptionClientID > {
      typedef pointer_category category;
      static const char* type_name() { return"RobotRaconteur::ServiceSubscriptionClientID"; }
    };
  }


      namespace swig {
	template <>  struct traits<std::pair< RobotRaconteur::ServiceSubscriptionClientID, RobotRaconteur::ServiceInfo2Wrapped > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::pair<" "RobotRaconteur::ServiceSubscriptionClientID" "," "RobotRaconteur::ServiceInfo2Wrapped" " >";
	  }
	};
      }
    

      namespace swig {
	template <>  struct traits<std::map< RobotRaconteur::ServiceSubscriptionClientID, RobotRaconteur::ServiceInfo2Wrapped, std::less< RobotRaconteur::ServiceSubscriptionClientID >, std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::map<" "RobotRaconteur::ServiceSubscriptionClientID" "," "RobotRaconteur::ServiceInfo2Wrapped" "," "std::less< RobotRaconteur::ServiceSubscriptionClientID >" "," "std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg__iterator(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg____nonzero__(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg____bool__(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const *self){
      return !(self->empty());
    }
SWIGINTERN std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::size_type std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg____len__(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const *self){
      return self->size();
    }
SWIGINTERN std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::mapped_type const &std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg____getitem__(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *self,std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &key){
      std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > >::const_iterator i = self->find(key);
      if (i != self->end())
	return i->second;
      else
	throw std::out_of_range("key not found");
    }
SWIGINTERN void std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg____delitem__(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *self,std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &key){
      std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > >::iterator i = self->find(key);
      if (i != self->end())
	self->erase(i);
      else
	throw std::out_of_range("key not found");
    }
SWIGINTERN bool std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg__has_key(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const *self,std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &key){
      std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > >::const_iterator i = self->find(key);
      return i != self->end();
    }
SWIGINTERN PyObject *std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg__keys(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *self){
      std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > >::size_type size = self->size();
      Py_ssize_t pysize = (size <= (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > >::size_type) INT_MAX) ? (Py_ssize_t) size : -1;
      SWIG_PYTHON_THREAD_BEGIN_BLOCK;
      if (pysize < 0) {
	PyErr_SetString(PyExc_OverflowError, "map size not valid in python");
	SWIG_PYTHON_THREAD_END_BLOCK;
	return NULL;
      }
      PyObject* keyList = PyList_New(pysize);
      std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > >::const_iterator i = self->begin();
      for (Py_ssize_t j = 0; j < pysize; ++i, ++j) {
	PyList_SET_ITEM(keyList, j, swig::from(i->first));
      }
      SWIG_PYTHON_THREAD_END_BLOCK;
      return keyList;
    }
SWIGINTERN PyObject *std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg__values(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *self){
      std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > >::size_type size = self->size();
      Py_ssize_t pysize = (size <= (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > >::size_type) INT_MAX) ? (Py_ssize_t) size : -1;
      SWIG_PYTHON_THREAD_BEGIN_BLOCK;
      if (pysize < 0) {
	PyErr_SetString(PyExc_OverflowError, "map size not valid in python");
	SWIG_PYTHON_THREAD_END_BLOCK;
	return NULL;
      }
      PyObject* valList = PyList_New(pysize);
      std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > >::const_iterator i = self->begin();
      for (Py_ssize_t j = 0; j < pysize; ++i, ++j) {
	PyList_SET_ITEM(valList, j, swig::from(i->second));
      }
      SWIG_PYTHON_THREAD_END_BLOCK;
      return valList;
    }
SWIGINTERN PyObject *std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg__items(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *self){
      std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > >::size_type size = self->size();
      Py_ssize_t pysize = (size <= (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > >::size_type) INT_MAX) ? (Py_ssize_t) size : -1;
      SWIG_PYTHON_THREAD_BEGIN_BLOCK;
      if (pysize < 0) {
	PyErr_SetString(PyExc_OverflowError, "map size not valid in python");
	SWIG_PYTHON_THREAD_END_BLOCK;
	return NULL;
      }    
      PyObject* itemList = PyList_New(pysize);
      std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > >::const_iterator i = self->begin();
      for (Py_ssize_t j = 0; j < pysize; ++i, ++j) {
	PyList_SET_ITEM(itemList, j, swig::from(*i));
      }
      SWIG_PYTHON_THREAD_END_BLOCK;
      return itemList;
    }
SWIGINTERN bool std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg____contains__(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *self,std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &key){
      return self->find(key) != self->end();
    }
SWIGINTERN swig::SwigPyIterator *std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg__key_iterator(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *self,PyObject **PYTHON_SELF){
      return swig::make_output_key_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN swig::SwigPyIterator *std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg__value_iterator(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *self,PyObject **PYTHON_SELF){
      return swig::make_output_value_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN void std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg____setitem____SWIG_0(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *self,std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &key){
      self->erase(key);
    }
SWIGINTERN void std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg____setitem____SWIG_1(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *self,std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &key,std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::mapped_type const &x){
      (*self)[key] = x;
    }
SWIGINTERN PyObject *std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg__asdict(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *self){
      return swig::traits_from< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > > >::asdict(*self);
    }
SWIGINTERN void std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg__erase__SWIG_1(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *self,std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator position){ self->erase(position); }
SWIGINTERN void std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg__erase__SWIG_2(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *self,std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator first,std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator last){ self->erase(first, last); }

  namespace swig {
    template <>  struct traits< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > {
      typedef pointer_category category;
      static const char* type_name() { return"boost::shared_ptr< RobotRaconteur::WrappedServiceStub >"; }
    };
  }


      namespace swig {
	template <>  struct traits<std::pair< RobotRaconteur::ServiceSubscriptionClientID, boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::pair<" "RobotRaconteur::ServiceSubscriptionClientID" "," "boost::shared_ptr< RobotRaconteur::WrappedServiceStub >" " >";
	  }
	};
      }
    

      namespace swig {
	template <>  struct traits<std::map< RobotRaconteur::ServiceSubscriptionClientID, boost::shared_ptr< RobotRaconteur::WrappedServiceStub >, std::less< RobotRaconteur::ServiceSubscriptionClientID >, std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::map<" "RobotRaconteur::ServiceSubscriptionClientID" "," "boost::shared_ptr< RobotRaconteur::WrappedServiceStub >" "," "std::less< RobotRaconteur::ServiceSubscriptionClientID >" "," "std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg__iterator(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg____nonzero__(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg____bool__(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const *self){
      return !(self->empty());
    }
SWIGINTERN std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::size_type std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg____len__(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const *self){
      return self->size();
    }
SWIGINTERN std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::mapped_type const &std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg____getitem__(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *self,std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &key){
      std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > >::const_iterator i = self->find(key);
      if (i != self->end())
	return i->second;
      else
	throw std::out_of_range("key not found");
    }
SWIGINTERN void std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg____delitem__(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *self,std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &key){
      std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > >::iterator i = self->find(key);
      if (i != self->end())
	self->erase(i);
      else
	throw std::out_of_range("key not found");
    }
SWIGINTERN bool std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg__has_key(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const *self,std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &key){
      std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > >::const_iterator i = self->find(key);
      return i != self->end();
    }
SWIGINTERN PyObject *std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg__keys(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *self){
      std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > >::size_type size = self->size();
      Py_ssize_t pysize = (size <= (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > >::size_type) INT_MAX) ? (Py_ssize_t) size : -1;
      SWIG_PYTHON_THREAD_BEGIN_BLOCK;
      if (pysize < 0) {
	PyErr_SetString(PyExc_OverflowError, "map size not valid in python");
	SWIG_PYTHON_THREAD_END_BLOCK;
	return NULL;
      }
      PyObject* keyList = PyList_New(pysize);
      std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > >::const_iterator i = self->begin();
      for (Py_ssize_t j = 0; j < pysize; ++i, ++j) {
	PyList_SET_ITEM(keyList, j, swig::from(i->first));
      }
      SWIG_PYTHON_THREAD_END_BLOCK;
      return keyList;
    }
SWIGINTERN PyObject *std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg__values(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *self){
      std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > >::size_type size = self->size();
      Py_ssize_t pysize = (size <= (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > >::size_type) INT_MAX) ? (Py_ssize_t) size : -1;
      SWIG_PYTHON_THREAD_BEGIN_BLOCK;
      if (pysize < 0) {
	PyErr_SetString(PyExc_OverflowError, "map size not valid in python");
	SWIG_PYTHON_THREAD_END_BLOCK;
	return NULL;
      }
      PyObject* valList = PyList_New(pysize);
      std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > >::const_iterator i = self->begin();
      for (Py_ssize_t j = 0; j < pysize; ++i, ++j) {
	PyList_SET_ITEM(valList, j, swig::from(i->second));
      }
      SWIG_PYTHON_THREAD_END_BLOCK;
      return valList;
    }
SWIGINTERN PyObject *std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg__items(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *self){
      std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > >::size_type size = self->size();
      Py_ssize_t pysize = (size <= (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > >::size_type) INT_MAX) ? (Py_ssize_t) size : -1;
      SWIG_PYTHON_THREAD_BEGIN_BLOCK;
      if (pysize < 0) {
	PyErr_SetString(PyExc_OverflowError, "map size not valid in python");
	SWIG_PYTHON_THREAD_END_BLOCK;
	return NULL;
      }    
      PyObject* itemList = PyList_New(pysize);
      std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > >::const_iterator i = self->begin();
      for (Py_ssize_t j = 0; j < pysize; ++i, ++j) {
	PyList_SET_ITEM(itemList, j, swig::from(*i));
      }
      SWIG_PYTHON_THREAD_END_BLOCK;
      return itemList;
    }
SWIGINTERN bool std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg____contains__(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *self,std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &key){
      return self->find(key) != self->end();
    }
SWIGINTERN swig::SwigPyIterator *std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg__key_iterator(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *self,PyObject **PYTHON_SELF){
      return swig::make_output_key_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN swig::SwigPyIterator *std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg__value_iterator(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *self,PyObject **PYTHON_SELF){
      return swig::make_output_value_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN void std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg____setitem____SWIG_0(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *self,std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &key){
      self->erase(key);
    }
SWIGINTERN void std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg____setitem____SWIG_1(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *self,std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &key,std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::mapped_type const &x){
      (*self)[key] = x;
    }
SWIGINTERN PyObject *std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg__asdict(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *self){
      return swig::traits_from< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > > >::asdict(*self);
    }
SWIGINTERN void std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg__erase__SWIG_1(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *self,std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator position){ self->erase(position); }
SWIGINTERN void std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg__erase__SWIG_2(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *self,std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator first,std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator last){ self->erase(first, last); }

  namespace swig {
    template <>  struct traits< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > {
      typedef pointer_category category;
      static const char* type_name() { return"boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >"; }
    };
  }


      namespace swig {
	template <>  struct traits<std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >, std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >" "," "std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg__iterator(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____nonzero__(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____bool__(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____len__(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *self){
      return self->size();
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____getslice__(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *self,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____setslice____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *self,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >());
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____setslice____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *self,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type j,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____delslice__(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *self,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____delitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *self,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > *std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____getitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____setitem____SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *self,PySliceObject *slice,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____setitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____delitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >::difference_type id = i;
      std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type const &std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____getitem____SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *self,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____setitem____SWIG_2(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *self,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type i,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg__pop(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg__append(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *self,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg__erase__SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *self,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg__erase__SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *self,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator first,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg__insert__SWIG_0(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *self,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator pos,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg__insert__SWIG_1(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *self,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator pos,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type n,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type const &x){ self->insert(pos, n, x); }

      namespace swig {
	template <>  struct traits<std::vector< RobotRaconteur::ServiceSubscriptionClientID, std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "RobotRaconteur::ServiceSubscriptionClientID" "," "std::allocator< RobotRaconteur::ServiceSubscriptionClientID >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg__iterator(std::vector< RobotRaconteur::ServiceSubscriptionClientID > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____nonzero__(std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____bool__(std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____len__(std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *self){
      return self->size();
    }
SWIGINTERN std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > *std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____getslice__(std::vector< RobotRaconteur::ServiceSubscriptionClientID > *self,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type i,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____setslice____SWIG_0(std::vector< RobotRaconteur::ServiceSubscriptionClientID > *self,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type i,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >());
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____setslice____SWIG_1(std::vector< RobotRaconteur::ServiceSubscriptionClientID > *self,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type i,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type j,std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____delslice__(std::vector< RobotRaconteur::ServiceSubscriptionClientID > *self,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type i,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____delitem____SWIG_0(std::vector< RobotRaconteur::ServiceSubscriptionClientID > *self,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > *std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____getitem____SWIG_0(std::vector< RobotRaconteur::ServiceSubscriptionClientID > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >::difference_type id = i;
      std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____setitem____SWIG_0(std::vector< RobotRaconteur::ServiceSubscriptionClientID > *self,PySliceObject *slice,std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >::difference_type id = i;
      std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____setitem____SWIG_1(std::vector< RobotRaconteur::ServiceSubscriptionClientID > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >::difference_type id = i;
      std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____delitem____SWIG_1(std::vector< RobotRaconteur::ServiceSubscriptionClientID > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >::difference_type id = i;
      std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____getitem____SWIG_1(std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *self,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____setitem____SWIG_2(std::vector< RobotRaconteur::ServiceSubscriptionClientID > *self,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type i,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg__pop(std::vector< RobotRaconteur::ServiceSubscriptionClientID > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg__append(std::vector< RobotRaconteur::ServiceSubscriptionClientID > *self,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg__erase__SWIG_0(std::vector< RobotRaconteur::ServiceSubscriptionClientID > *self,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg__erase__SWIG_1(std::vector< RobotRaconteur::ServiceSubscriptionClientID > *self,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator first,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg__insert__SWIG_0(std::vector< RobotRaconteur::ServiceSubscriptionClientID > *self,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator pos,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg__insert__SWIG_1(std::vector< RobotRaconteur::ServiceSubscriptionClientID > *self,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator pos,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type n,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &x){ self->insert(pos, n, x); }

  namespace swig {
    template <>  struct traits< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > {
      typedef pointer_category category;
      static const char* type_name() { return"RobotRaconteur::WrappedServiceSubscriptionManagerDetails"; }
    };
  }


      namespace swig {
	template <>  struct traits<std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails, std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "RobotRaconteur::WrappedServiceSubscriptionManagerDetails" "," "std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg__iterator(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____nonzero__(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____bool__(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____len__(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *self){
      return self->size();
    }
SWIGINTERN std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > *std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____getslice__(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *self,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type i,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____setslice____SWIG_0(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *self,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type i,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >());
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____setslice____SWIG_1(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *self,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type i,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type j,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____delslice__(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *self,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type i,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____delitem____SWIG_0(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *self,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > *std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____getitem____SWIG_0(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >::difference_type id = i;
      std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____setitem____SWIG_0(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *self,PySliceObject *slice,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >::difference_type id = i;
      std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____setitem____SWIG_1(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >::difference_type id = i;
      std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____delitem____SWIG_1(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >::difference_type id = i;
      std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____getitem____SWIG_1(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *self,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____setitem____SWIG_2(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *self,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type i,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg__pop(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg__append(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *self,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg__erase__SWIG_0(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *self,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg__erase__SWIG_1(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *self,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator first,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg__insert__SWIG_0(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *self,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator pos,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg__insert__SWIG_1(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *self,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator pos,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type n,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &x){ self->insert(pos, n, x); }

  namespace swig {
    template <>  struct traits< RobotRaconteur::ServiceSubscriptionFilterAttribute > {
      typedef pointer_category category;
      static const char* type_name() { return"RobotRaconteur::ServiceSubscriptionFilterAttribute"; }
    };
  }


      namespace swig {
	template <>  struct traits<std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute, std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::vector<" "RobotRaconteur::ServiceSubscriptionFilterAttribute" "," "std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__iterator(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____nonzero__(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____bool__(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *self){
      return !(self->empty());
    }
SWIGINTERN std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____len__(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *self){
      return self->size();
    }
SWIGINTERN std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > *std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____getslice__(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type i,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type j){
      return swig::getslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____setslice____SWIG_0(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type i,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type j){
      swig::setslice(self, i, j, 1, std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >());
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____setslice____SWIG_1(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type i,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type j,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > const &v){
      swig::setslice(self, i, j, 1, v);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____delslice__(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type i,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type j){
      swig::delslice(self, i, j, 1);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____delitem____SWIG_0(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type i){
      swig::erase(self, swig::getpos(self, i));
    }
SWIGINTERN std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > *std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____getitem____SWIG_0(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return NULL;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >::difference_type id = i;
      std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >::difference_type jd = j;
      return swig::getslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____setitem____SWIG_0(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,PySliceObject *slice,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > const &v){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >::difference_type id = i;
      std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >::difference_type jd = j;
      swig::setslice(self, id, jd, step, v);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____setitem____SWIG_1(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >::difference_type id = i;
      std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____delitem____SWIG_1(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,PySliceObject *slice){
      Py_ssize_t i, j, step;
      if( !PySlice_Check(slice) ) {
        SWIG_Error(SWIG_TypeError, "Slice object expected.");
        return;
      }
      PySlice_GetIndices(SWIGPY_SLICE_ARG(slice), (Py_ssize_t)self->size(), &i, &j, &step);
      std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >::difference_type id = i;
      std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >::difference_type jd = j;
      swig::delslice(self, id, jd, step);
    }
SWIGINTERN std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____getitem____SWIG_1(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *self,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type i){
      return *(swig::cgetpos(self, i));
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____setitem____SWIG_2(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type i,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &x){
      *(swig::getpos(self,i)) = x;
    }
SWIGINTERN std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__pop(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *self){
      if (self->size() == 0)
	throw std::out_of_range("pop from empty container");
      std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >::value_type x = self->back();
      self->pop_back();
      return x;
    }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__append(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &x){
      self->push_back(x);
    }
SWIGINTERN std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__erase__SWIG_0(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator pos){ return self->erase(pos); }
SWIGINTERN std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__erase__SWIG_1(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator first,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator last){ return self->erase(first, last); }
SWIGINTERN std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__insert__SWIG_0(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator pos,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &x){ return self->insert(pos, x); }
SWIGINTERN void std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__insert__SWIG_1(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator pos,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type n,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &x){ self->insert(pos, n, x); }

      namespace swig {
	template <>  struct traits<std::pair< std::string, RobotRaconteur::ServiceSubscriptionFilterAttribute > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::pair<" "std::string" "," "RobotRaconteur::ServiceSubscriptionFilterAttribute" " >";
	  }
	};
      }
    

      namespace swig {
	template <>  struct traits<std::map< std::string, RobotRaconteur::ServiceSubscriptionFilterAttribute, std::less< std::string >, std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::map<" "std::string" "," "RobotRaconteur::ServiceSubscriptionFilterAttribute" "," "std::less< std::string >" "," "std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__iterator(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____nonzero__(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____bool__(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const *self){
      return !(self->empty());
    }
SWIGINTERN std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____len__(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const *self){
      return self->size();
    }
SWIGINTERN std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::mapped_type const &std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____getitem__(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &key){
      std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > >::const_iterator i = self->find(key);
      if (i != self->end())
	return i->second;
      else
	throw std::out_of_range("key not found");
    }
SWIGINTERN void std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____delitem__(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &key){
      std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > >::iterator i = self->find(key);
      if (i != self->end())
	self->erase(i);
      else
	throw std::out_of_range("key not found");
    }
SWIGINTERN bool std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__has_key(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const *self,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &key){
      std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > >::const_iterator i = self->find(key);
      return i != self->end();
    }
SWIGINTERN PyObject *std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__keys(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *self){
      std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > >::size_type size = self->size();
      Py_ssize_t pysize = (size <= (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > >::size_type) INT_MAX) ? (Py_ssize_t) size : -1;
      SWIG_PYTHON_THREAD_BEGIN_BLOCK;
      if (pysize < 0) {
	PyErr_SetString(PyExc_OverflowError, "map size not valid in python");
	SWIG_PYTHON_THREAD_END_BLOCK;
	return NULL;
      }
      PyObject* keyList = PyList_New(pysize);
      std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > >::const_iterator i = self->begin();
      for (Py_ssize_t j = 0; j < pysize; ++i, ++j) {
	PyList_SET_ITEM(keyList, j, swig::from(i->first));
      }
      SWIG_PYTHON_THREAD_END_BLOCK;
      return keyList;
    }
SWIGINTERN PyObject *std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__values(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *self){
      std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > >::size_type size = self->size();
      Py_ssize_t pysize = (size <= (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > >::size_type) INT_MAX) ? (Py_ssize_t) size : -1;
      SWIG_PYTHON_THREAD_BEGIN_BLOCK;
      if (pysize < 0) {
	PyErr_SetString(PyExc_OverflowError, "map size not valid in python");
	SWIG_PYTHON_THREAD_END_BLOCK;
	return NULL;
      }
      PyObject* valList = PyList_New(pysize);
      std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > >::const_iterator i = self->begin();
      for (Py_ssize_t j = 0; j < pysize; ++i, ++j) {
	PyList_SET_ITEM(valList, j, swig::from(i->second));
      }
      SWIG_PYTHON_THREAD_END_BLOCK;
      return valList;
    }
SWIGINTERN PyObject *std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__items(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *self){
      std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > >::size_type size = self->size();
      Py_ssize_t pysize = (size <= (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > >::size_type) INT_MAX) ? (Py_ssize_t) size : -1;
      SWIG_PYTHON_THREAD_BEGIN_BLOCK;
      if (pysize < 0) {
	PyErr_SetString(PyExc_OverflowError, "map size not valid in python");
	SWIG_PYTHON_THREAD_END_BLOCK;
	return NULL;
      }    
      PyObject* itemList = PyList_New(pysize);
      std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > >::const_iterator i = self->begin();
      for (Py_ssize_t j = 0; j < pysize; ++i, ++j) {
	PyList_SET_ITEM(itemList, j, swig::from(*i));
      }
      SWIG_PYTHON_THREAD_END_BLOCK;
      return itemList;
    }
SWIGINTERN bool std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____contains__(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &key){
      return self->find(key) != self->end();
    }
SWIGINTERN swig::SwigPyIterator *std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__key_iterator(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,PyObject **PYTHON_SELF){
      return swig::make_output_key_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN swig::SwigPyIterator *std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__value_iterator(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,PyObject **PYTHON_SELF){
      return swig::make_output_value_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN void std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____setitem____SWIG_0(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &key){
      self->erase(key);
    }
SWIGINTERN void std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____setitem____SWIG_1(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &key,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::mapped_type const &x){
      (*self)[key] = x;
    }
SWIGINTERN PyObject *std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__asdict(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *self){
      return swig::traits_from< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > > >::asdict(*self);
    }
SWIGINTERN void std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__erase__SWIG_1(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator position){ self->erase(position); }
SWIGINTERN void std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__erase__SWIG_2(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *self,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator first,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator last){ self->erase(first, last); }

SWIGINTERN int
SWIG_AsVal_char (PyObject * obj, char *val)
{    
  int res = SWIG_AsCharArray(obj, val, 1);
  if (!SWIG_IsOK(res)) {
    long v;
    res = SWIG_AddCast(SWIG_AsVal_long (obj, &v));
    if (SWIG_IsOK(res)) {
      if ((CHAR_MIN <= v) && (v <= CHAR_MAX)) {
	if (val) *val = static_cast< char >(v);
      } else {
	res = SWIG_OverflowError;
      }
    }
  }
  return res;
}


SWIGINTERNINLINE PyObject *
SWIG_From_char  (char c) 
{ 
  return SWIG_FromCharPtrAndSize(&c,1);
}


  namespace swig {
    template <>  struct traits< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > {
      typedef pointer_category category;
      static const char* type_name() { return"RobotRaconteur::ServiceSubscriptionFilterAttributeGroup"; }
    };
  }


      namespace swig {
	template <>  struct traits<std::pair< std::string, RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::pair<" "std::string" "," "RobotRaconteur::ServiceSubscriptionFilterAttributeGroup" " >";
	  }
	};
      }
    

      namespace swig {
	template <>  struct traits<std::map< std::string, RobotRaconteur::ServiceSubscriptionFilterAttributeGroup, std::less< std::string >, std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > > > {
	  typedef pointer_category category;
	  static const char* type_name() {
	    return "std::map<" "std::string" "," "RobotRaconteur::ServiceSubscriptionFilterAttributeGroup" "," "std::less< std::string >" "," "std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > >" " >";
	  }
	};
      }
    
SWIGINTERN swig::SwigPyIterator *std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg__iterator(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *self,PyObject **PYTHON_SELF){
      return swig::make_output_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN bool std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg____nonzero__(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const *self){
      return !(self->empty());
    }
SWIGINTERN bool std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg____bool__(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const *self){
      return !(self->empty());
    }
SWIGINTERN std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::size_type std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg____len__(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const *self){
      return self->size();
    }
SWIGINTERN std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::mapped_type const &std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg____getitem__(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *self,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &key){
      std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > >::const_iterator i = self->find(key);
      if (i != self->end())
	return i->second;
      else
	throw std::out_of_range("key not found");
    }
SWIGINTERN void std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg____delitem__(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *self,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &key){
      std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > >::iterator i = self->find(key);
      if (i != self->end())
	self->erase(i);
      else
	throw std::out_of_range("key not found");
    }
SWIGINTERN bool std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg__has_key(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const *self,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &key){
      std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > >::const_iterator i = self->find(key);
      return i != self->end();
    }
SWIGINTERN PyObject *std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg__keys(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *self){
      std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > >::size_type size = self->size();
      Py_ssize_t pysize = (size <= (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > >::size_type) INT_MAX) ? (Py_ssize_t) size : -1;
      SWIG_PYTHON_THREAD_BEGIN_BLOCK;
      if (pysize < 0) {
	PyErr_SetString(PyExc_OverflowError, "map size not valid in python");
	SWIG_PYTHON_THREAD_END_BLOCK;
	return NULL;
      }
      PyObject* keyList = PyList_New(pysize);
      std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > >::const_iterator i = self->begin();
      for (Py_ssize_t j = 0; j < pysize; ++i, ++j) {
	PyList_SET_ITEM(keyList, j, swig::from(i->first));
      }
      SWIG_PYTHON_THREAD_END_BLOCK;
      return keyList;
    }
SWIGINTERN PyObject *std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg__values(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *self){
      std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > >::size_type size = self->size();
      Py_ssize_t pysize = (size <= (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > >::size_type) INT_MAX) ? (Py_ssize_t) size : -1;
      SWIG_PYTHON_THREAD_BEGIN_BLOCK;
      if (pysize < 0) {
	PyErr_SetString(PyExc_OverflowError, "map size not valid in python");
	SWIG_PYTHON_THREAD_END_BLOCK;
	return NULL;
      }
      PyObject* valList = PyList_New(pysize);
      std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > >::const_iterator i = self->begin();
      for (Py_ssize_t j = 0; j < pysize; ++i, ++j) {
	PyList_SET_ITEM(valList, j, swig::from(i->second));
      }
      SWIG_PYTHON_THREAD_END_BLOCK;
      return valList;
    }
SWIGINTERN PyObject *std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg__items(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *self){
      std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > >::size_type size = self->size();
      Py_ssize_t pysize = (size <= (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > >::size_type) INT_MAX) ? (Py_ssize_t) size : -1;
      SWIG_PYTHON_THREAD_BEGIN_BLOCK;
      if (pysize < 0) {
	PyErr_SetString(PyExc_OverflowError, "map size not valid in python");
	SWIG_PYTHON_THREAD_END_BLOCK;
	return NULL;
      }    
      PyObject* itemList = PyList_New(pysize);
      std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > >::const_iterator i = self->begin();
      for (Py_ssize_t j = 0; j < pysize; ++i, ++j) {
	PyList_SET_ITEM(itemList, j, swig::from(*i));
      }
      SWIG_PYTHON_THREAD_END_BLOCK;
      return itemList;
    }
SWIGINTERN bool std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg____contains__(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *self,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &key){
      return self->find(key) != self->end();
    }
SWIGINTERN swig::SwigPyIterator *std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg__key_iterator(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *self,PyObject **PYTHON_SELF){
      return swig::make_output_key_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN swig::SwigPyIterator *std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg__value_iterator(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *self,PyObject **PYTHON_SELF){
      return swig::make_output_value_iterator(self->begin(), self->begin(), self->end(), *PYTHON_SELF);
    }
SWIGINTERN void std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg____setitem____SWIG_0(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *self,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &key){
      self->erase(key);
    }
SWIGINTERN void std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg____setitem____SWIG_1(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *self,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &key,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::mapped_type const &x){
      (*self)[key] = x;
    }
SWIGINTERN PyObject *std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg__asdict(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *self){
      return swig::traits_from< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > > >::asdict(*self);
    }
SWIGINTERN void std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg__erase__SWIG_1(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *self,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator position){ self->erase(position); }
SWIGINTERN void std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg__erase__SWIG_2(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *self,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator first,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator last){ self->erase(first, last); }
SWIGINTERN boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > RobotRaconteur_RRLogRecord_GetNode(RobotRaconteur::RRLogRecord *self){
                return self->Node.lock();
            }
SWIGINTERN void RobotRaconteur_RRLogRecord_SetNode(RobotRaconteur::RRLogRecord *self,boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &node){
                self->Node = node;
            }
SWIGINTERN std::string RobotRaconteur_RRLogRecord_ToString(RobotRaconteur::RRLogRecord *self){
                std::stringstream ss;
                ss << *self;
                return ss.str();
            }

class ClientServiceListenerDirector
{
public:

	int32_t objectheapid;

	ClientServiceListenerDirector()
	{
		objectheapid=0;
	}

	void OuterCallback(const boost::shared_ptr<RobotRaconteur::ClientContext>& c,ClientServiceListenerEventType code,const boost::shared_ptr<void>& p)
	{
		if (code == ClientServiceListenerEventType_ServicePathReleased)
		{
			std::string p1 = *RR_STATIC_POINTER_CAST<std::string>(p);
			DIRECTOR_CALL2(Callback2(code,p1));
		}
		else
		{
			DIRECTOR_CALL2(Callback(code));
		}
	}
	virtual void Callback(int32_t code) {};
	virtual void Callback2(int32_t code, const std::string& p) {};

	virtual ~ClientServiceListenerDirector()
	{

	}

};

SWIGINTERN boost::shared_ptr< RobotRaconteur::WrappedServiceStub > RobotRaconteur_RobotRaconteurNode_ConnectService__SWIG_0(RobotRaconteur::RobotRaconteurNode *self,std::string const &url,std::string const &username="",boost::intrusive_ptr< RobotRaconteur::MessageElementData > credentials=boost::intrusive_ptr< RobotRaconteur::MessageElementData >(),ClientServiceListenerDirector *listener=0,std::string const &objecttype=""){
		boost::shared_ptr<ClientServiceListenerDirector> listenerptr;
		if (listener)
		{
			listenerptr=boost::shared_ptr<ClientServiceListenerDirector>(listener,boost::bind(&ReleaseDirector<ClientServiceListenerDirector>,RR_BOOST_PLACEHOLDERS(_1),listener->objectheapid));
		}

		boost::intrusive_ptr<RRMap<std::string,RRValue> > credentials2;
		if (credentials) credentials2=rr_cast<RRMap<std::string,RRValue> >(self->UnpackMapType<std::string,RRValue>(rr_cast<MessageElementNestedElementList>(credentials)));
		boost::shared_ptr<WrappedServiceStub> stub;
		if (listener==0)
		{
			stub=rr_cast<WrappedServiceStub>(self->ConnectService(url,username,credentials2,RR_NULL_FN,objecttype));
		}
		else
		{

			stub=rr_cast<WrappedServiceStub>(self->ConnectService(url,username,credentials2,boost::bind(&ClientServiceListenerDirector::OuterCallback,listenerptr,RR_BOOST_PLACEHOLDERS(_1),RR_BOOST_PLACEHOLDERS(_2),RR_BOOST_PLACEHOLDERS(_3)),objecttype));
		}

		return stub;

	}
SWIGINTERN boost::shared_ptr< RobotRaconteur::WrappedServiceStub > RobotRaconteur_RobotRaconteurNode_ConnectService__SWIG_5(RobotRaconteur::RobotRaconteurNode *self,std::vector< std::string,std::allocator< std::string > > const &url,std::string const &username="",boost::intrusive_ptr< RobotRaconteur::MessageElementData > credentials=boost::intrusive_ptr< RobotRaconteur::MessageElementData >(),ClientServiceListenerDirector *listener=0,std::string const &objecttype=""){

		boost::shared_ptr<ClientServiceListenerDirector> listenerptr;
		if (listener)
		{
			listenerptr=boost::shared_ptr<ClientServiceListenerDirector>(listener,boost::bind(&ReleaseDirector<ClientServiceListenerDirector>,RR_BOOST_PLACEHOLDERS(_1),listener->objectheapid));
		}

		boost::intrusive_ptr<RRMap<std::string,RRValue> > credentials2;
		if (credentials) credentials2=rr_cast<RRMap<std::string,RRValue> >(self->UnpackMapType<std::string,RRValue>(rr_cast<MessageElementNestedElementList >(credentials)));
		boost::shared_ptr<WrappedServiceStub> stub;
		if (listener==0)
		{
			stub=rr_cast<WrappedServiceStub>(self->ConnectService(url,username,credentials2,RR_NULL_FN,objecttype));
		}
		else
		{

			stub=rr_cast<WrappedServiceStub>(self->ConnectService(url,username,credentials2,boost::bind(&ClientServiceListenerDirector::OuterCallback,listenerptr,RR_BOOST_PLACEHOLDERS(_1),RR_BOOST_PLACEHOLDERS(_2),RR_BOOST_PLACEHOLDERS(_3)),objecttype));
		}
		return stub;

	}
SWIGINTERN void RobotRaconteur_RobotRaconteurNode_AsyncConnectService__SWIG_0(RobotRaconteur::RobotRaconteurNode *self,std::string const &url,std::string const &username,boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &credentials,ClientServiceListenerDirector *listener,std::string const &objecttype,int32_t timeout,RobotRaconteur::AsyncStubReturnDirector *handler,int32_t id){

		boost::shared_ptr<AsyncStubReturnDirector> sphandler(handler,boost::bind(&ReleaseDirector<AsyncStubReturnDirector>,RR_BOOST_PLACEHOLDERS(_1),id));

		boost::intrusive_ptr<RRMap<std::string,RRValue> > credentials2;
		if (credentials) credentials2=rr_cast<RRMap<std::string,RRValue> >(self->UnpackMapType<std::string,RRValue>(rr_cast<MessageElementNestedElementList >(credentials)));

		if (listener==0)
		{
			self->AsyncConnectService(url,username,credentials2,RR_NULL_FN,objecttype,boost::bind(&AsyncStubReturn_handler,RR_BOOST_PLACEHOLDERS(_1),RR_BOOST_PLACEHOLDERS(_2),sphandler),timeout);
		}
		else
		{
			boost::shared_ptr<ClientServiceListenerDirector> listenerptr=boost::shared_ptr<ClientServiceListenerDirector>(listener);
			self->AsyncConnectService(url,username,credentials2,boost::bind(&ClientServiceListenerDirector::OuterCallback,listenerptr,RR_BOOST_PLACEHOLDERS(_1),RR_BOOST_PLACEHOLDERS(_2),RR_BOOST_PLACEHOLDERS(_3)),objecttype,boost::bind(&AsyncStubReturn_handler,RR_BOOST_PLACEHOLDERS(_1),RR_BOOST_PLACEHOLDERS(_2),sphandler),timeout);
		}
	}
SWIGINTERN void RobotRaconteur_RobotRaconteurNode_AsyncConnectService__SWIG_1(RobotRaconteur::RobotRaconteurNode *self,std::vector< std::string,std::allocator< std::string > > const &url,std::string const &username,boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &credentials,ClientServiceListenerDirector *listener,std::string const &objecttype,int32_t timeout,RobotRaconteur::AsyncStubReturnDirector *handler,int32_t id){

		boost::shared_ptr<AsyncStubReturnDirector> sphandler(handler,boost::bind(&ReleaseDirector<AsyncStubReturnDirector>,RR_BOOST_PLACEHOLDERS(_1),id));

		boost::intrusive_ptr<RRMap<std::string,RRValue> > credentials2;
		if (credentials) credentials2=rr_cast<RRMap<std::string,RRValue> >(self->UnpackMapType<std::string,RRValue>(rr_cast<MessageElementNestedElementList >(credentials)));

		if (listener==0)
		{
			self->AsyncConnectService(url,username,credentials2,RR_NULL_FN,objecttype,boost::bind(&AsyncStubReturn_handler,RR_BOOST_PLACEHOLDERS(_1),RR_BOOST_PLACEHOLDERS(_2),sphandler),timeout);
		}
		else
		{
			boost::shared_ptr<ClientServiceListenerDirector> listenerptr=boost::shared_ptr<ClientServiceListenerDirector>(listener);
			self->AsyncConnectService(url,username,credentials2,boost::bind(&ClientServiceListenerDirector::OuterCallback,listenerptr,RR_BOOST_PLACEHOLDERS(_1),RR_BOOST_PLACEHOLDERS(_2),RR_BOOST_PLACEHOLDERS(_3)),objecttype,boost::bind(&AsyncStubReturn_handler,RR_BOOST_PLACEHOLDERS(_1),RR_BOOST_PLACEHOLDERS(_2),sphandler),timeout);
		}
	}
SWIGINTERN void RobotRaconteur_RobotRaconteurNode_DisconnectService(RobotRaconteur::RobotRaconteurNode *self,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &obj){
		self->DisconnectService(obj);
	}
SWIGINTERN void RobotRaconteur_RobotRaconteurNode_AsyncDisconnectService(RobotRaconteur::RobotRaconteurNode *self,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &obj,RobotRaconteur::AsyncVoidNoErrReturnDirector *handler,int32_t id){
		boost::shared_ptr<AsyncVoidNoErrReturnDirector> sphandler(handler,boost::bind(&ReleaseDirector<AsyncVoidNoErrReturnDirector>,RR_BOOST_PLACEHOLDERS(_1),id));
		self->AsyncDisconnectService(obj,boost::bind(&AsyncVoidNoErrReturn_handler,sphandler));
	}
SWIGINTERN void RobotRaconteur_RobotRaconteurNode_RegisterServiceType__SWIG_0(RobotRaconteur::RobotRaconteurNode *self,std::string const &def){
		boost::shared_ptr<RobotRaconteur::ServiceDefinition> def2=boost::make_shared<ServiceDefinition>();
		def2->FromString(def);
		std::vector<boost::shared_ptr<ServiceDefinition> > defs;
		std::vector<std::string> names=self->GetRegisteredServiceTypes();
		for (std::vector<std::string>::iterator e=names.begin(); e!=names.end(); ++e)
		{
			if ((*e)!="RobotRaconteurServiceIndex")
			defs.push_back(self->GetServiceType(*e)->ServiceDef());
		}
		defs.push_back(def2);

		VerifyServiceDefinitions(defs);

		self->RegisterServiceType(boost::make_shared<WrappedServiceFactory>(def2));
	}
SWIGINTERN void RobotRaconteur_RobotRaconteurNode_RegisterServiceTypes__SWIG_0(RobotRaconteur::RobotRaconteurNode *self,std::vector< std::string,std::allocator< std::string > > const &defs_str){

		std::vector<boost::shared_ptr<ServiceDefinition> > defs;
		std::vector<boost::shared_ptr<ServiceDefinition> > defs2;
		BOOST_FOREACH(const std::string& def_str, defs_str)
		{
			boost::shared_ptr<RobotRaconteur::ServiceDefinition> def2=boost::make_shared<ServiceDefinition>();
			def2->FromString(def_str);
			defs.push_back(def2);
			defs2.push_back(def2);
		}
		std::vector<std::string> names=self->GetRegisteredServiceTypes();
		for (std::vector<std::string>::iterator e=names.begin(); e!=names.end(); ++e)
		{
			if ((*e)!="RobotRaconteurServiceIndex")
			defs2.push_back(self->GetServiceType(*e)->ServiceDef());
		}

		VerifyServiceDefinitions(defs2);

		BOOST_FOREACH(const boost::shared_ptr<RobotRaconteur::ServiceDefinition>& def, defs)
		{
			self->RegisterServiceType(boost::make_shared<WrappedServiceFactory>(def));
		}
	}
SWIGINTERN void RobotRaconteur_RobotRaconteurNode_RegisterServiceType__SWIG_1(RobotRaconteur::RobotRaconteurNode *self,boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &def){

		std::vector<boost::shared_ptr<ServiceDefinition> > defs;
		std::vector<std::string> names=self->GetRegisteredServiceTypes();
		for (std::vector<std::string>::iterator e=names.begin(); e!=names.end(); ++e)
		{
			defs.push_back(self->GetServiceType(*e)->ServiceDef());
		}
		defs.push_back(def);

		VerifyServiceDefinitions(defs);

		self->RegisterServiceType(boost::make_shared<WrappedServiceFactory>(def));
	}
SWIGINTERN void RobotRaconteur_RobotRaconteurNode_RegisterServiceTypes__SWIG_1(RobotRaconteur::RobotRaconteurNode *self,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > const &defs){

		std::vector<boost::shared_ptr<ServiceDefinition> > defs2;
		std::vector<std::string> names=self->GetRegisteredServiceTypes();
		for (std::vector<std::string>::iterator e=names.begin(); e!=names.end(); ++e)
		{
			defs2.push_back(self->GetServiceType(*e)->ServiceDef());
		}
		BOOST_FOREACH(const boost::shared_ptr<RobotRaconteur::ServiceDefinition>& def, defs)
		{
			defs2.push_back(def);
		}

		VerifyServiceDefinitions(defs2);

		BOOST_FOREACH(const boost::shared_ptr<RobotRaconteur::ServiceDefinition>& def, defs)
		{
			self->RegisterServiceType(boost::make_shared<WrappedServiceFactory>(def));
		}
	}
SWIGINTERN boost::shared_ptr< RobotRaconteur::ServiceDefinition > RobotRaconteur_RobotRaconteurNode_GetServiceType(RobotRaconteur::RobotRaconteurNode *self,std::string const &servicename){
		return rr_cast<WrappedServiceFactory>(self->GetServiceType(servicename))->ServiceDef();
	}
SWIGINTERN boost::shared_ptr< RobotRaconteur::ServiceDefinition > RobotRaconteur_RobotRaconteurNode_GetPulledServiceType(RobotRaconteur::RobotRaconteurNode *self,boost::shared_ptr< RobotRaconteur::RRObject > const &obj,std::string const &servicename){
		return rr_cast<WrappedServiceFactory>(self->GetPulledServiceType(obj,servicename))->ServiceDef();
	}
SWIGINTERN std::string RobotRaconteur_RobotRaconteurNode_RequestObjectLock(RobotRaconteur::RobotRaconteurNode *self,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &obj,RobotRaconteur::RobotRaconteurObjectLockFlags flags){
		return self->RequestObjectLock(obj,flags);
	}
SWIGINTERN std::string RobotRaconteur_RobotRaconteurNode_ReleaseObjectLock(RobotRaconteur::RobotRaconteurNode *self,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &obj){
		return self->ReleaseObjectLock(obj);
	}
SWIGINTERN void RobotRaconteur_RobotRaconteurNode_MonitorEnter__SWIG_0(RobotRaconteur::RobotRaconteurNode *self,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &obj,int32_t timeout=-1){
		self->MonitorEnter(obj,timeout);
	}
SWIGINTERN void RobotRaconteur_RobotRaconteurNode_MonitorExit(RobotRaconteur::RobotRaconteurNode *self,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &obj){
		self->MonitorExit(obj);
	}
SWIGINTERN void RobotRaconteur_RobotRaconteurNode_AsyncRequestObjectLock(RobotRaconteur::RobotRaconteurNode *self,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &obj,RobotRaconteur::RobotRaconteurObjectLockFlags flags,int32_t timeout,RobotRaconteur::AsyncStringReturnDirector *handler,int32_t id){
		boost::shared_ptr<AsyncStringReturnDirector> sphandler(handler,boost::bind(&ReleaseDirector<AsyncStringReturnDirector>,RR_BOOST_PLACEHOLDERS(_1),id));
		return self->AsyncRequestObjectLock(obj,flags,boost::bind(&AsyncStringReturn_handler,RR_BOOST_PLACEHOLDERS(_1),RR_BOOST_PLACEHOLDERS(_2),sphandler),timeout);
	}
SWIGINTERN void RobotRaconteur_RobotRaconteurNode_AsyncReleaseObjectLock(RobotRaconteur::RobotRaconteurNode *self,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &obj,int32_t timeout,RobotRaconteur::AsyncStringReturnDirector *handler,int32_t id){
		boost::shared_ptr<AsyncStringReturnDirector> sphandler(handler,boost::bind(&ReleaseDirector<AsyncStringReturnDirector>,RR_BOOST_PLACEHOLDERS(_1),id));
		return self->AsyncReleaseObjectLock(obj,boost::bind(&AsyncStringReturn_handler,RR_BOOST_PLACEHOLDERS(_1),RR_BOOST_PLACEHOLDERS(_2),sphandler),timeout);
	}
SWIGINTERN boost::intrusive_ptr< RobotRaconteur::MessageElement > RobotRaconteur_RobotRaconteurNode_GetServiceAttributes(RobotRaconteur::RobotRaconteurNode *self,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &obj){
			boost::intrusive_ptr<RRMap<std::string,RRValue> > map=AllocateEmptyRRMap<std::string,RRValue>();
			map->GetStorageContainer()=self->GetServiceAttributes(obj);
			boost::intrusive_ptr<MessageElementNestedElementList > mmap=self->PackMapType<std::string,RRValue>(map);
			return CreateMessageElement("value",mmap);
		}
SWIGINTERN RobotRaconteur::NodeID RobotRaconteur_RobotRaconteurNode_GetServiceNodeID(RobotRaconteur::RobotRaconteurNode *self,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &obj){
		return self->GetServiceNodeID(obj);
	}
SWIGINTERN std::string RobotRaconteur_RobotRaconteurNode_GetServiceNodeName(RobotRaconteur::RobotRaconteurNode *self,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &obj){
		return self->GetServiceNodeName(obj);
	}
SWIGINTERN std::string RobotRaconteur_RobotRaconteurNode_GetServiceName(RobotRaconteur::RobotRaconteurNode *self,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &obj){
		return self->GetServiceName(obj);
	}
SWIGINTERN std::string RobotRaconteur_RobotRaconteurNode_GetObjectServicePath(RobotRaconteur::RobotRaconteurNode *self,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &obj){
		return self->GetObjectServicePath(obj);
	}
SWIGINTERN std::string RobotRaconteur_RobotRaconteurNode_GetObjectType(RobotRaconteur::RobotRaconteurNode *self,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &obj){
		return self->GetObjectType(obj);
	}
SWIGINTERN void RobotRaconteur_RobotRaconteurNode_AsyncFindObjectType__SWIG_0(RobotRaconteur::RobotRaconteurNode *self,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &obj,std::string const &n,int32_t timeout,RobotRaconteur::AsyncStringReturnDirector *handler,int32_t id){
		boost::shared_ptr<AsyncStringReturnDirector> sphandler(handler,boost::bind(&ReleaseDirector<AsyncStringReturnDirector>,RR_BOOST_PLACEHOLDERS(_1),id));
		return self->AsyncFindObjectType(obj,n,boost::bind(&AsyncStringReturn_handler,RR_BOOST_PLACEHOLDERS(_1),RR_BOOST_PLACEHOLDERS(_2),sphandler),timeout);
	}
SWIGINTERN void RobotRaconteur_RobotRaconteurNode_AsyncFindObjectType__SWIG_1(RobotRaconteur::RobotRaconteurNode *self,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &obj,std::string const &n,std::string const &i,int32_t timeout,RobotRaconteur::AsyncStringReturnDirector *handler,int32_t id){
		boost::shared_ptr<AsyncStringReturnDirector> sphandler(handler,boost::bind(&ReleaseDirector<AsyncStringReturnDirector>,RR_BOOST_PLACEHOLDERS(_1),id));
		return self->AsyncFindObjectType(obj,n,i,boost::bind(&AsyncStringReturn_handler,RR_BOOST_PLACEHOLDERS(_1),RR_BOOST_PLACEHOLDERS(_2),sphandler),timeout);
	}
SWIGINTERN void RobotRaconteur_RobotRaconteurNode_SetExceptionHandler(RobotRaconteur::RobotRaconteurNode *self,RobotRaconteur::AsyncVoidReturnDirector *handler,int32_t id){
		RR_SHARED_PTR<AsyncVoidReturnDirector> sphandler(handler,boost::bind(&ReleaseDirector<AsyncVoidReturnDirector>,RR_BOOST_PLACEHOLDERS(_1),id));
		self->SetExceptionHandler(boost::bind(&WrappedExceptionHandler,RR_BOOST_PLACEHOLDERS(_1),sphandler));
	}
SWIGINTERN void RobotRaconteur_RobotRaconteurNode_ClearExceptionHandler(RobotRaconteur::RobotRaconteurNode *self){
		self->SetExceptionHandler(RR_NULL_FN);
	}
SWIGINTERN boost::shared_ptr< RobotRaconteur::Timer > RobotRaconteur_RobotRaconteurNode_CreateTimer(RobotRaconteur::RobotRaconteurNode *self,boost::posix_time::time_duration const &period,bool oneshot,RobotRaconteur::AsyncTimerEventReturnDirector *handler,int32_t id){
		boost::shared_ptr<AsyncTimerEventReturnDirector> sphandler(handler,boost::bind(&ReleaseDirector<AsyncTimerEventReturnDirector>,RR_BOOST_PLACEHOLDERS(_1),id));
		return self->CreateTimer(period,boost::bind(&TimerHandlerFunc,RR_BOOST_PLACEHOLDERS(_1),sphandler),oneshot);
	}
SWIGINTERN void RobotRaconteur_RobotRaconteurNode_PostToThreadPool(RobotRaconteur::RobotRaconteurNode *self,RobotRaconteur::AsyncVoidNoErrReturnDirector *handler,int32_t id){
		boost::shared_ptr<AsyncVoidNoErrReturnDirector> sphandler(handler,boost::bind(&ReleaseDirector<AsyncVoidNoErrReturnDirector>,RR_BOOST_PLACEHOLDERS(_1),id));
		self->GetThreadPool()->Post(boost::bind(&AsyncVoidNoErrReturn_handler,sphandler));
	}
SWIGINTERN void RobotRaconteur_RobotRaconteurNode_Init__SWIG_0(RobotRaconteur::RobotRaconteurNode *self,size_t thread_count=20){
    
    self->SetDynamicServiceFactory(RR_MAKE_SHARED<RobotRaconteur::WrappedDynamicServiceFactory>());
    self->SetThreadPoolFactory(RR_MAKE_SHARED<RobotRaconteur::PythonThreadPoolFactory>());

    if (thread_count!=20)
    {
        boost::shared_ptr<RobotRaconteur::ThreadPool> thread_pool=self->GetThreadPoolFactory()->NewThreadPool(self->shared_from_this());
        thread_pool->SetThreadPoolCount(thread_count);
        self->SetThreadPool(thread_pool);
        self->GetThreadPool();
    }
    

    self->Init();

    }
SWIGINTERN RobotRaconteur::RobotRaconteurNodeSetup *new_RobotRaconteur_RobotRaconteurNodeSetup__SWIG_0(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &node,std::string const &node_name,uint16_t tcp_port,uint32_t flags){
				std::vector<RR_SHARED_PTR<ServiceFactory> > s;
				RobotRaconteurNodeSetup* n = new RobotRaconteurNodeSetup(node, s, node_name, tcp_port, flags);
				return n;
			}
SWIGINTERN RobotRaconteur::RobotRaconteurNodeSetup *new_RobotRaconteur_RobotRaconteurNodeSetup__SWIG_1(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &node,boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > const &config){
				std::vector<RR_SHARED_PTR<ServiceFactory> > s;
				RobotRaconteurNodeSetup* n = new RobotRaconteurNodeSetup(node, s, config);
				return n;
			}
SWIGINTERN RobotRaconteur::RobotRaconteurNodeSetup *new_RobotRaconteur_RobotRaconteurNodeSetup__SWIG_2(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &node,std::string const &node_name,uint16_t tcp_port,uint32_t flags,uint32_t allowed_overrides,std::vector< std::string,std::allocator< std::string > > const &args){
				std::vector<RR_SHARED_PTR<ServiceFactory> > s;
				RobotRaconteurNodeSetup* n = new RobotRaconteurNodeSetup(node, s, node_name, tcp_port, flags, allowed_overrides, args);
				return n;
			}


/* ---------------------------------------------------
 * C++ director class methods
 * --------------------------------------------------- */

#include "RobotRaconteurPythonPYTHON_wrap.h"

SwigDirector_AsyncRequestDirector::SwigDirector_AsyncRequestDirector(PyObject *self): RobotRaconteur::AsyncRequestDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::AsyncRequestDirector *)this, this); 
}




SwigDirector_AsyncRequestDirector::~SwigDirector_AsyncRequestDirector() {
}

void SwigDirector_AsyncRequestDirector::handler(boost::intrusive_ptr< RobotRaconteur::MessageElement > ret, RobotRaconteur::HandlerErrorInfo &error) {
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    // intrusive_ptr by value directorin
    
    if(*(&ret)) {
      intrusive_ptr_add_ref((&ret)->get());
      smartarg1 = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&ret)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
    }
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
    
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(&error), SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0 );
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call AsyncRequestDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "handler";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("handler");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_AsyncVoidReturnDirector::SwigDirector_AsyncVoidReturnDirector(PyObject *self): RobotRaconteur::AsyncVoidReturnDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::AsyncVoidReturnDirector *)this, this); 
}




SwigDirector_AsyncVoidReturnDirector::~SwigDirector_AsyncVoidReturnDirector() {
}

void SwigDirector_AsyncVoidReturnDirector::handler(RobotRaconteur::HandlerErrorInfo &error) {
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(&error), SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0 );
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call AsyncVoidReturnDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "handler";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("handler");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_AsyncVoidNoErrReturnDirector::SwigDirector_AsyncVoidNoErrReturnDirector(PyObject *self): RobotRaconteur::AsyncVoidNoErrReturnDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::AsyncVoidNoErrReturnDirector *)this, this); 
}




SwigDirector_AsyncVoidNoErrReturnDirector::~SwigDirector_AsyncVoidNoErrReturnDirector() {
}

void SwigDirector_AsyncVoidNoErrReturnDirector::handler() {
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call AsyncVoidNoErrReturnDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "handler";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject args = PyTuple_New(0);
    swig::SwigVar_PyObject result = PyObject_Call(method, (PyObject *) args, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("handler");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_AsyncStringReturnDirector::SwigDirector_AsyncStringReturnDirector(PyObject *self): RobotRaconteur::AsyncStringReturnDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::AsyncStringReturnDirector *)this, this); 
}




SwigDirector_AsyncStringReturnDirector::~SwigDirector_AsyncStringReturnDirector() {
}

void SwigDirector_AsyncStringReturnDirector::handler(std::string const &ret, RobotRaconteur::HandlerErrorInfo &error) {
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_std_string(static_cast< std::string >(ret));
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(&error), SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0 );
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call AsyncStringReturnDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "handler";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("handler");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_AsyncUInt32ReturnDirector::SwigDirector_AsyncUInt32ReturnDirector(PyObject *self): RobotRaconteur::AsyncUInt32ReturnDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::AsyncUInt32ReturnDirector *)this, this); 
}




SwigDirector_AsyncUInt32ReturnDirector::~SwigDirector_AsyncUInt32ReturnDirector() {
}

void SwigDirector_AsyncUInt32ReturnDirector::handler(uint32_t ret, RobotRaconteur::HandlerErrorInfo &error) {
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(ret));
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(&error), SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0 );
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call AsyncUInt32ReturnDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "handler";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("handler");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_AsyncTimerEventReturnDirector::SwigDirector_AsyncTimerEventReturnDirector(PyObject *self): RobotRaconteur::AsyncTimerEventReturnDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::AsyncTimerEventReturnDirector *)this, this); 
}




SwigDirector_AsyncTimerEventReturnDirector::~SwigDirector_AsyncTimerEventReturnDirector() {
}

void SwigDirector_AsyncTimerEventReturnDirector::handler(RobotRaconteur::TimerEvent const &ret, RobotRaconteur::HandlerErrorInfo &error) {
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(&ret), SWIGTYPE_p_RobotRaconteur__TimerEvent,  0 );
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(&error), SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0 );
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call AsyncTimerEventReturnDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "handler";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("handler");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_WrappedPipeEndpointDirector::SwigDirector_WrappedPipeEndpointDirector(PyObject *self): RobotRaconteur::WrappedPipeEndpointDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedPipeEndpointDirector *)this, this); 
}




SwigDirector_WrappedPipeEndpointDirector::~SwigDirector_WrappedPipeEndpointDirector() {
}

void SwigDirector_WrappedPipeEndpointDirector::PipeEndpointClosedCallback() {
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedPipeEndpointDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "PipeEndpointClosedCallback";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject args = PyTuple_New(0);
    swig::SwigVar_PyObject result = PyObject_Call(method, (PyObject *) args, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("PipeEndpointClosedCallback");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


void SwigDirector_WrappedPipeEndpointDirector::PacketReceivedEvent() {
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedPipeEndpointDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 1;
    const char *const swig_method_name = "PacketReceivedEvent";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject args = PyTuple_New(0);
    swig::SwigVar_PyObject result = PyObject_Call(method, (PyObject *) args, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("PacketReceivedEvent");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


void SwigDirector_WrappedPipeEndpointDirector::PacketAckReceivedEvent(uint32_t packetnum) {
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(packetnum));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedPipeEndpointDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 2;
    const char *const swig_method_name = "PacketAckReceivedEvent";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("PacketAckReceivedEvent");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_AsyncPipeEndpointReturnDirector::SwigDirector_AsyncPipeEndpointReturnDirector(PyObject *self): RobotRaconteur::AsyncPipeEndpointReturnDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::AsyncPipeEndpointReturnDirector *)this, this); 
}




SwigDirector_AsyncPipeEndpointReturnDirector::~SwigDirector_AsyncPipeEndpointReturnDirector() {
}

void SwigDirector_AsyncPipeEndpointReturnDirector::handler(boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > const &ep, RobotRaconteur::HandlerErrorInfo &error) {
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    smartarg1 = ep ? new boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint >(ep) : 0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, SWIG_POINTER_OWN |  0 );
    
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(&error), SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0 );
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call AsyncPipeEndpointReturnDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "handler";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("handler");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_WrappedPipeServerConnectDirector::SwigDirector_WrappedPipeServerConnectDirector(PyObject *self): RobotRaconteur::WrappedPipeServerConnectDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedPipeServerConnectDirector *)this, this); 
}




SwigDirector_WrappedPipeServerConnectDirector::~SwigDirector_WrappedPipeServerConnectDirector() {
}

void SwigDirector_WrappedPipeServerConnectDirector::PipeConnectCallback(boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > const &pipeendpoint) {
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    smartarg1 = pipeendpoint ? new boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint >(pipeendpoint) : 0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, SWIG_POINTER_OWN |  0 );
    
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedPipeServerConnectDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "PipeConnectCallback";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("PipeConnectCallback");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_WrappedPipeBroadcasterPredicateDirector::SwigDirector_WrappedPipeBroadcasterPredicateDirector(PyObject *self): RobotRaconteur::WrappedPipeBroadcasterPredicateDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedPipeBroadcasterPredicateDirector *)this, this); 
}




bool SwigDirector_WrappedPipeBroadcasterPredicateDirector::Predicate(uint32_t client_endpoint, int32_t index) {
  bool c_result = SwigValueInit< bool >() ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(client_endpoint));
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_From_int(static_cast< int >(index));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedPipeBroadcasterPredicateDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "Predicate";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Predicate");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    bool swig_val;
    int swig_res = SWIG_AsVal_bool(result, &swig_val);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""bool""'");
    }
    c_result = static_cast< bool >(swig_val);
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (bool) c_result;
}


SwigDirector_WrappedPipeBroadcasterPredicateDirector::~SwigDirector_WrappedPipeBroadcasterPredicateDirector() {
}

SwigDirector_WrappedWireConnectionDirector::SwigDirector_WrappedWireConnectionDirector(PyObject *self): RobotRaconteur::WrappedWireConnectionDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedWireConnectionDirector *)this, this); 
}




SwigDirector_WrappedWireConnectionDirector::~SwigDirector_WrappedWireConnectionDirector() {
}

void SwigDirector_WrappedWireConnectionDirector::WireValueChanged(boost::intrusive_ptr< RobotRaconteur::MessageElement > value, RobotRaconteur::TimeSpec const &time) {
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    // intrusive_ptr by value directorin
    
    if(*(&value)) {
      intrusive_ptr_add_ref((&value)->get());
      smartarg1 = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&value)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
    }
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
    
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(&time), SWIGTYPE_p_RobotRaconteur__TimeSpec,  0 );
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedWireConnectionDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "WireValueChanged";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("WireValueChanged");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


void SwigDirector_WrappedWireConnectionDirector::WireConnectionClosedCallback() {
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedWireConnectionDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 1;
    const char *const swig_method_name = "WireConnectionClosedCallback";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject args = PyTuple_New(0);
    swig::SwigVar_PyObject result = PyObject_Call(method, (PyObject *) args, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("WireConnectionClosedCallback");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_AsyncWireConnectionReturnDirector::SwigDirector_AsyncWireConnectionReturnDirector(PyObject *self): RobotRaconteur::AsyncWireConnectionReturnDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::AsyncWireConnectionReturnDirector *)this, this); 
}




SwigDirector_AsyncWireConnectionReturnDirector::~SwigDirector_AsyncWireConnectionReturnDirector() {
}

void SwigDirector_AsyncWireConnectionReturnDirector::handler(boost::shared_ptr< RobotRaconteur::WrappedWireConnection > const &ep, RobotRaconteur::HandlerErrorInfo &error) {
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    smartarg1 = ep ? new boost::shared_ptr<  RobotRaconteur::WrappedWireConnection >(ep) : 0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, SWIG_POINTER_OWN |  0 );
    
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(&error), SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0 );
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call AsyncWireConnectionReturnDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "handler";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("handler");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_AsyncWirePeekReturnDirector::SwigDirector_AsyncWirePeekReturnDirector(PyObject *self): RobotRaconteur::AsyncWirePeekReturnDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::AsyncWirePeekReturnDirector *)this, this); 
}




SwigDirector_AsyncWirePeekReturnDirector::~SwigDirector_AsyncWirePeekReturnDirector() {
}

void SwigDirector_AsyncWirePeekReturnDirector::handler(boost::intrusive_ptr< RobotRaconteur::MessageElement > value, RobotRaconteur::TimeSpec const &ts, RobotRaconteur::HandlerErrorInfo &error) {
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    // intrusive_ptr by value directorin
    
    if(*(&value)) {
      intrusive_ptr_add_ref((&value)->get());
      smartarg1 = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&value)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
    }
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
    
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(&ts), SWIGTYPE_p_RobotRaconteur__TimeSpec,  0 );
    swig::SwigVar_PyObject obj2;
    obj2 = SWIG_NewPointerObj(SWIG_as_voidptr(&error), SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0 );
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call AsyncWirePeekReturnDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "handler";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("handler");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_WrappedWireServerConnectDirector::SwigDirector_WrappedWireServerConnectDirector(PyObject *self): RobotRaconteur::WrappedWireServerConnectDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedWireServerConnectDirector *)this, this); 
}




SwigDirector_WrappedWireServerConnectDirector::~SwigDirector_WrappedWireServerConnectDirector() {
}

void SwigDirector_WrappedWireServerConnectDirector::WireConnectCallback(boost::shared_ptr< RobotRaconteur::WrappedWireConnection > const &c) {
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    smartarg1 = c ? new boost::shared_ptr<  RobotRaconteur::WrappedWireConnection >(c) : 0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, SWIG_POINTER_OWN |  0 );
    
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedWireServerConnectDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "WireConnectCallback";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("WireConnectCallback");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_WrappedWireServerPeekValueDirector::SwigDirector_WrappedWireServerPeekValueDirector(PyObject *self): RobotRaconteur::WrappedWireServerPeekValueDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedWireServerPeekValueDirector *)this, this); 
}




SwigDirector_WrappedWireServerPeekValueDirector::~SwigDirector_WrappedWireServerPeekValueDirector() {
}

boost::intrusive_ptr< RobotRaconteur::MessageElement > SwigDirector_WrappedWireServerPeekValueDirector::PeekValue(uint32_t const &ep) {
  void *argp ;
  int swig_res = 0 ;
  int newmem = 0 ;
  
  boost::intrusive_ptr< RobotRaconteur::MessageElement > c_result;
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(ep));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedWireServerPeekValueDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "PeekValue";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("PeekValue");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    
    // intrusive_ptr by value directorout
    //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&result;
    //if (smartarg) {
    //	result = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
    //}
    
    swig_res = SWIG_ConvertPtrAndOwn(result, &argp, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""boost::intrusive_ptr< RobotRaconteur::MessageElement >""'");
    }
    if (argp) c_result = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp)->get(),true);
    if (newmem & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp);
    
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (boost::intrusive_ptr< RobotRaconteur::MessageElement >) c_result;
}


SwigDirector_WrappedWireServerPokeValueDirector::SwigDirector_WrappedWireServerPokeValueDirector(PyObject *self): RobotRaconteur::WrappedWireServerPokeValueDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedWireServerPokeValueDirector *)this, this); 
}




SwigDirector_WrappedWireServerPokeValueDirector::~SwigDirector_WrappedWireServerPokeValueDirector() {
}

void SwigDirector_WrappedWireServerPokeValueDirector::PokeValue(boost::intrusive_ptr< RobotRaconteur::MessageElement > value, RobotRaconteur::TimeSpec const &ts, uint32_t const &ep) {
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    // intrusive_ptr by value directorin
    
    if(*(&value)) {
      intrusive_ptr_add_ref((&value)->get());
      smartarg1 = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&value)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
    }
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
    
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(&ts), SWIGTYPE_p_RobotRaconteur__TimeSpec,  0 );
    swig::SwigVar_PyObject obj2;
    obj2 = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(ep));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedWireServerPokeValueDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "PokeValue";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("PokeValue");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_WrappedWireBroadcasterPredicateDirector::SwigDirector_WrappedWireBroadcasterPredicateDirector(PyObject *self): RobotRaconteur::WrappedWireBroadcasterPredicateDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedWireBroadcasterPredicateDirector *)this, this); 
}




bool SwigDirector_WrappedWireBroadcasterPredicateDirector::Predicate(uint32_t client_endpoint) {
  bool c_result = SwigValueInit< bool >() ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(client_endpoint));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedWireBroadcasterPredicateDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "Predicate";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Predicate");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    bool swig_val;
    int swig_res = SWIG_AsVal_bool(result, &swig_val);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""bool""'");
    }
    c_result = static_cast< bool >(swig_val);
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (bool) c_result;
}


SwigDirector_WrappedWireBroadcasterPredicateDirector::~SwigDirector_WrappedWireBroadcasterPredicateDirector() {
}

SwigDirector_WrappedPodArrayMemoryClientBuffer::SwigDirector_WrappedPodArrayMemoryClientBuffer(PyObject *self): RobotRaconteur::WrappedPodArrayMemoryClientBuffer(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedPodArrayMemoryClientBuffer *)this, this); 
}




void SwigDirector_WrappedPodArrayMemoryClientBuffer::UnpackReadResult(boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > res, uint64_t bufferpos, uint64_t count) {
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartarg1 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    // intrusive_ptr by value directorin
    
    if(*(&res)) {
      intrusive_ptr_add_ref((&res)->get());
      smartarg1 = new boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >((&res)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
    }
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, SWIG_POINTER_OWN);
    
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(bufferpos));
    swig::SwigVar_PyObject obj2;
    obj2 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(count));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedPodArrayMemoryClientBuffer.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "UnpackReadResult";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("UnpackReadResult");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > SwigDirector_WrappedPodArrayMemoryClientBuffer::PackWriteRequest(uint64_t bufferpos, uint64_t count) {
  void *argp ;
  int swig_res = 0 ;
  int newmem = 0 ;
  
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > c_result;
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(bufferpos));
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(count));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedPodArrayMemoryClientBuffer.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 1;
    const char *const swig_method_name = "PackWriteRequest";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("PackWriteRequest");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    
    // intrusive_ptr by value directorout
    //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >**)&result;
    //if (smartarg) {
    //	result = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(smartarg->get(), true);
    //}
    
    swig_res = SWIG_ConvertPtrAndOwn(result, &argp, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t,  0 , &newmem);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >""'");
    }
    if (argp) c_result = boost::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp)->get(),true);
    if (newmem & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp);
    
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >) c_result;
}


uint64_t SwigDirector_WrappedPodArrayMemoryClientBuffer::GetBufferLength() {
  uint64_t c_result = SwigValueInit< uint64_t >() ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedPodArrayMemoryClientBuffer.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 2;
    const char *const swig_method_name = "GetBufferLength";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject args = PyTuple_New(0);
    swig::SwigVar_PyObject result = PyObject_Call(method, (PyObject *) args, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("GetBufferLength");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    unsigned long swig_val;
    int swig_res = SWIG_AsVal_unsigned_SS_long(result, &swig_val);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""uint64_t""'");
    }
    c_result = static_cast< uint64_t >(swig_val);
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (uint64_t) c_result;
}


SwigDirector_WrappedPodArrayMemoryClientBuffer::~SwigDirector_WrappedPodArrayMemoryClientBuffer() {
}

SwigDirector_WrappedPodMultiDimArrayMemoryClientBuffer::SwigDirector_WrappedPodMultiDimArrayMemoryClientBuffer(PyObject *self): RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer *)this, this); 
}




void SwigDirector_WrappedPodMultiDimArrayMemoryClientBuffer::UnpackReadResult(boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > res, std::vector< uint64_t, std::allocator< uint64_t > > const &bufferpos, std::vector< uint64_t, std::allocator< uint64_t > > const &count) {
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartarg1 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    // intrusive_ptr by value directorin
    
    if(*(&res)) {
      intrusive_ptr_add_ref((&res)->get());
      smartarg1 = new boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >((&res)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
    }
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, SWIG_POINTER_OWN);
    
    swig::SwigVar_PyObject obj1;
    obj1 = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(bufferpos));
    swig::SwigVar_PyObject obj2;
    obj2 = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(count));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedPodMultiDimArrayMemoryClientBuffer.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "UnpackReadResult";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("UnpackReadResult");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > SwigDirector_WrappedPodMultiDimArrayMemoryClientBuffer::PackWriteRequest(std::vector< uint64_t, std::allocator< uint64_t > > const &bufferpos, std::vector< uint64_t, std::allocator< uint64_t > > const &count) {
  void *argp ;
  int swig_res = 0 ;
  int newmem = 0 ;
  
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > c_result;
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(bufferpos));
    swig::SwigVar_PyObject obj1;
    obj1 = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(count));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedPodMultiDimArrayMemoryClientBuffer.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 1;
    const char *const swig_method_name = "PackWriteRequest";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("PackWriteRequest");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    
    // intrusive_ptr by value directorout
    //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >**)&result;
    //if (smartarg) {
    //	result = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(smartarg->get(), true);
    //}
    
    swig_res = SWIG_ConvertPtrAndOwn(result, &argp, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t,  0 , &newmem);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >""'");
    }
    if (argp) c_result = boost::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp)->get(),true);
    if (newmem & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp);
    
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >) c_result;
}


SwigDirector_WrappedPodMultiDimArrayMemoryClientBuffer::~SwigDirector_WrappedPodMultiDimArrayMemoryClientBuffer() {
}

SwigDirector_WrappedArrayMemoryDirector::SwigDirector_WrappedArrayMemoryDirector(PyObject *self): RobotRaconteur::WrappedArrayMemoryDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedArrayMemoryDirector *)this, this); 
}




SwigDirector_WrappedArrayMemoryDirector::~SwigDirector_WrappedArrayMemoryDirector() {
}

uint64_t SwigDirector_WrappedArrayMemoryDirector::Length() {
  uint64_t c_result = SwigValueInit< uint64_t >() ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedArrayMemoryDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "Length";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject args = PyTuple_New(0);
    swig::SwigVar_PyObject result = PyObject_Call(method, (PyObject *) args, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Length");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    unsigned long swig_val;
    int swig_res = SWIG_AsVal_unsigned_SS_long(result, &swig_val);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""uint64_t""'");
    }
    c_result = static_cast< uint64_t >(swig_val);
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (uint64_t) c_result;
}


void SwigDirector_WrappedArrayMemoryDirector::Read(uint64_t memorypos, boost::intrusive_ptr< RobotRaconteur::RRBaseArray > buffer, uint64_t bufferpos, uint64_t count) {
  boost::shared_ptr< RobotRaconteur::RRBaseArray > *smartarg2 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(memorypos));
    swig::SwigVar_PyObject obj1;
    
    // intrusive_ptr by value directorin
    
    if(*(&buffer)) {
      intrusive_ptr_add_ref((&buffer)->get());
      smartarg2 = new boost::shared_ptr<  RobotRaconteur::RRBaseArray >((&buffer)->get(),SWIG_intrusive_deleter<  RobotRaconteur::RRBaseArray >());
    }
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg2), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t, SWIG_POINTER_OWN);
    
    swig::SwigVar_PyObject obj2;
    obj2 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(bufferpos));
    swig::SwigVar_PyObject obj3;
    obj3 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(count));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedArrayMemoryDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 1;
    const char *const swig_method_name = "Read";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2,(PyObject *)obj3, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Read");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2,(PyObject *)obj3, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


void SwigDirector_WrappedArrayMemoryDirector::Write(uint64_t memorypos, boost::intrusive_ptr< RobotRaconteur::RRBaseArray > buffer, uint64_t bufferpos, uint64_t count) {
  boost::shared_ptr< RobotRaconteur::RRBaseArray > *smartarg2 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(memorypos));
    swig::SwigVar_PyObject obj1;
    
    // intrusive_ptr by value directorin
    
    if(*(&buffer)) {
      intrusive_ptr_add_ref((&buffer)->get());
      smartarg2 = new boost::shared_ptr<  RobotRaconteur::RRBaseArray >((&buffer)->get(),SWIG_intrusive_deleter<  RobotRaconteur::RRBaseArray >());
    }
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg2), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t, SWIG_POINTER_OWN);
    
    swig::SwigVar_PyObject obj2;
    obj2 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(bufferpos));
    swig::SwigVar_PyObject obj3;
    obj3 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(count));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedArrayMemoryDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 2;
    const char *const swig_method_name = "Write";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2,(PyObject *)obj3, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Write");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2,(PyObject *)obj3, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_WrappedMultiDimArrayMemoryDirector::SwigDirector_WrappedMultiDimArrayMemoryDirector(PyObject *self): RobotRaconteur::WrappedMultiDimArrayMemoryDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedMultiDimArrayMemoryDirector *)this, this); 
}




SwigDirector_WrappedMultiDimArrayMemoryDirector::~SwigDirector_WrappedMultiDimArrayMemoryDirector() {
}

std::vector< uint64_t, std::allocator< uint64_t > > SwigDirector_WrappedMultiDimArrayMemoryDirector::Dimensions() {
  std::vector< uint64_t,std::allocator< uint64_t > > c_result;
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedMultiDimArrayMemoryDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "Dimensions";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject args = PyTuple_New(0);
    swig::SwigVar_PyObject result = PyObject_Call(method, (PyObject *) args, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Dimensions");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    std::vector< unsigned long,std::allocator< unsigned long > > *swig_optr = 0;
    int swig_ores = swig::asptr(result, &swig_optr);
    if (!SWIG_IsOK(swig_ores) || !swig_optr) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError((swig_optr ? swig_ores : SWIG_TypeError))), "in output value of type '""std::vector< uint64_t,std::allocator< uint64_t > >""'");
    }
    c_result = *swig_optr;
    if (SWIG_IsNewObj(swig_ores)) delete swig_optr;
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (std::vector< uint64_t,std::allocator< uint64_t > >) c_result;
}


uint64_t SwigDirector_WrappedMultiDimArrayMemoryDirector::DimCount() {
  uint64_t c_result = SwigValueInit< uint64_t >() ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedMultiDimArrayMemoryDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 1;
    const char *const swig_method_name = "DimCount";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject args = PyTuple_New(0);
    swig::SwigVar_PyObject result = PyObject_Call(method, (PyObject *) args, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("DimCount");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    unsigned long swig_val;
    int swig_res = SWIG_AsVal_unsigned_SS_long(result, &swig_val);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""uint64_t""'");
    }
    c_result = static_cast< uint64_t >(swig_val);
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (uint64_t) c_result;
}


void SwigDirector_WrappedMultiDimArrayMemoryDirector::Read(RobotRaconteur::WrappedMultiDimArrayMemoryParams *p) {
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(p), SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryParams,  0 );
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedMultiDimArrayMemoryDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 2;
    const char *const swig_method_name = "Read";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Read");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


void SwigDirector_WrappedMultiDimArrayMemoryDirector::Write(RobotRaconteur::WrappedMultiDimArrayMemoryParams *p) {
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(p), SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryParams,  0 );
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedMultiDimArrayMemoryDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 3;
    const char *const swig_method_name = "Write";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Write");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_WrappedPodArrayMemoryDirector::SwigDirector_WrappedPodArrayMemoryDirector(PyObject *self): RobotRaconteur::WrappedPodArrayMemoryDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedPodArrayMemoryDirector *)this, this); 
}




SwigDirector_WrappedPodArrayMemoryDirector::~SwigDirector_WrappedPodArrayMemoryDirector() {
}

uint64_t SwigDirector_WrappedPodArrayMemoryDirector::Length() {
  uint64_t c_result = SwigValueInit< uint64_t >() ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedPodArrayMemoryDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "Length";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject args = PyTuple_New(0);
    swig::SwigVar_PyObject result = PyObject_Call(method, (PyObject *) args, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Length");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    unsigned long swig_val;
    int swig_res = SWIG_AsVal_unsigned_SS_long(result, &swig_val);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""uint64_t""'");
    }
    c_result = static_cast< uint64_t >(swig_val);
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (uint64_t) c_result;
}


boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > SwigDirector_WrappedPodArrayMemoryDirector::Read(uint64_t memorypos, uint64_t bufferpos, uint64_t count) {
  void *argp ;
  int swig_res = 0 ;
  int newmem = 0 ;
  
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > c_result;
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(memorypos));
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(bufferpos));
    swig::SwigVar_PyObject obj2;
    obj2 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(count));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedPodArrayMemoryDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 1;
    const char *const swig_method_name = "Read";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Read");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    
    // intrusive_ptr by value directorout
    //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >**)&result;
    //if (smartarg) {
    //	result = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(smartarg->get(), true);
    //}
    
    swig_res = SWIG_ConvertPtrAndOwn(result, &argp, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t,  0 , &newmem);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >""'");
    }
    if (argp) c_result = boost::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp)->get(),true);
    if (newmem & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp);
    
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >) c_result;
}


void SwigDirector_WrappedPodArrayMemoryDirector::Write(uint64_t memorypos, boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > buffer, uint64_t bufferpos, uint64_t count) {
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartarg2 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(memorypos));
    swig::SwigVar_PyObject obj1;
    
    // intrusive_ptr by value directorin
    
    if(*(&buffer)) {
      intrusive_ptr_add_ref((&buffer)->get());
      smartarg2 = new boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >((&buffer)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
    }
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg2), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, SWIG_POINTER_OWN);
    
    swig::SwigVar_PyObject obj2;
    obj2 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(bufferpos));
    swig::SwigVar_PyObject obj3;
    obj3 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(count));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedPodArrayMemoryDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 2;
    const char *const swig_method_name = "Write";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2,(PyObject *)obj3, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Write");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2,(PyObject *)obj3, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_WrappedPodMultiDimArrayMemoryDirector::SwigDirector_WrappedPodMultiDimArrayMemoryDirector(PyObject *self): RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *)this, this); 
}




SwigDirector_WrappedPodMultiDimArrayMemoryDirector::~SwigDirector_WrappedPodMultiDimArrayMemoryDirector() {
}

std::vector< uint64_t, std::allocator< uint64_t > > SwigDirector_WrappedPodMultiDimArrayMemoryDirector::Dimensions() {
  std::vector< uint64_t,std::allocator< uint64_t > > c_result;
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedPodMultiDimArrayMemoryDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "Dimensions";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject args = PyTuple_New(0);
    swig::SwigVar_PyObject result = PyObject_Call(method, (PyObject *) args, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Dimensions");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    std::vector< unsigned long,std::allocator< unsigned long > > *swig_optr = 0;
    int swig_ores = swig::asptr(result, &swig_optr);
    if (!SWIG_IsOK(swig_ores) || !swig_optr) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError((swig_optr ? swig_ores : SWIG_TypeError))), "in output value of type '""std::vector< uint64_t,std::allocator< uint64_t > >""'");
    }
    c_result = *swig_optr;
    if (SWIG_IsNewObj(swig_ores)) delete swig_optr;
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (std::vector< uint64_t,std::allocator< uint64_t > >) c_result;
}


uint64_t SwigDirector_WrappedPodMultiDimArrayMemoryDirector::DimCount() {
  uint64_t c_result = SwigValueInit< uint64_t >() ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedPodMultiDimArrayMemoryDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 1;
    const char *const swig_method_name = "DimCount";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject args = PyTuple_New(0);
    swig::SwigVar_PyObject result = PyObject_Call(method, (PyObject *) args, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("DimCount");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    unsigned long swig_val;
    int swig_res = SWIG_AsVal_unsigned_SS_long(result, &swig_val);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""uint64_t""'");
    }
    c_result = static_cast< uint64_t >(swig_val);
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (uint64_t) c_result;
}


boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > SwigDirector_WrappedPodMultiDimArrayMemoryDirector::Read(std::vector< uint64_t, std::allocator< uint64_t > > const &memorypos, std::vector< uint64_t, std::allocator< uint64_t > > const &bufferpos, std::vector< uint64_t, std::allocator< uint64_t > > const &count) {
  void *argp ;
  int swig_res = 0 ;
  int newmem = 0 ;
  
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > c_result;
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(memorypos));
    swig::SwigVar_PyObject obj1;
    obj1 = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(bufferpos));
    swig::SwigVar_PyObject obj2;
    obj2 = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(count));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedPodMultiDimArrayMemoryDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 2;
    const char *const swig_method_name = "Read";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Read");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    
    // intrusive_ptr by value directorout
    //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >**)&result;
    //if (smartarg) {
    //	result = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(smartarg->get(), true);
    //}
    
    swig_res = SWIG_ConvertPtrAndOwn(result, &argp, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t,  0 , &newmem);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >""'");
    }
    if (argp) c_result = boost::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp)->get(),true);
    if (newmem & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp);
    
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >) c_result;
}


void SwigDirector_WrappedPodMultiDimArrayMemoryDirector::Write(std::vector< uint64_t, std::allocator< uint64_t > > const &memorypos, boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > buffer, std::vector< uint64_t, std::allocator< uint64_t > > const &bufferpos, std::vector< uint64_t, std::allocator< uint64_t > > const &count) {
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartarg2 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(memorypos));
    swig::SwigVar_PyObject obj1;
    
    // intrusive_ptr by value directorin
    
    if(*(&buffer)) {
      intrusive_ptr_add_ref((&buffer)->get());
      smartarg2 = new boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >((&buffer)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
    }
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg2), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, SWIG_POINTER_OWN);
    
    swig::SwigVar_PyObject obj2;
    obj2 = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(bufferpos));
    swig::SwigVar_PyObject obj3;
    obj3 = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(count));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedPodMultiDimArrayMemoryDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 3;
    const char *const swig_method_name = "Write";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2,(PyObject *)obj3, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Write");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2,(PyObject *)obj3, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_WrappedNamedArrayMemoryClientBuffer::SwigDirector_WrappedNamedArrayMemoryClientBuffer(PyObject *self): RobotRaconteur::WrappedNamedArrayMemoryClientBuffer(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *)this, this); 
}




void SwigDirector_WrappedNamedArrayMemoryClientBuffer::UnpackReadResult(boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > res, uint64_t bufferpos, uint64_t count) {
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartarg1 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    // intrusive_ptr by value directorin
    
    if(*(&res)) {
      intrusive_ptr_add_ref((&res)->get());
      smartarg1 = new boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >((&res)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
    }
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, SWIG_POINTER_OWN);
    
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(bufferpos));
    swig::SwigVar_PyObject obj2;
    obj2 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(count));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedNamedArrayMemoryClientBuffer.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "UnpackReadResult";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("UnpackReadResult");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > SwigDirector_WrappedNamedArrayMemoryClientBuffer::PackWriteRequest(uint64_t bufferpos, uint64_t count) {
  void *argp ;
  int swig_res = 0 ;
  int newmem = 0 ;
  
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > c_result;
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(bufferpos));
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(count));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedNamedArrayMemoryClientBuffer.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 1;
    const char *const swig_method_name = "PackWriteRequest";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("PackWriteRequest");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    
    // intrusive_ptr by value directorout
    //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >**)&result;
    //if (smartarg) {
    //	result = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(smartarg->get(), true);
    //}
    
    swig_res = SWIG_ConvertPtrAndOwn(result, &argp, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t,  0 , &newmem);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >""'");
    }
    if (argp) c_result = boost::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp)->get(),true);
    if (newmem & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp);
    
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >) c_result;
}


uint64_t SwigDirector_WrappedNamedArrayMemoryClientBuffer::GetBufferLength() {
  uint64_t c_result = SwigValueInit< uint64_t >() ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedNamedArrayMemoryClientBuffer.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 2;
    const char *const swig_method_name = "GetBufferLength";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject args = PyTuple_New(0);
    swig::SwigVar_PyObject result = PyObject_Call(method, (PyObject *) args, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("GetBufferLength");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    unsigned long swig_val;
    int swig_res = SWIG_AsVal_unsigned_SS_long(result, &swig_val);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""uint64_t""'");
    }
    c_result = static_cast< uint64_t >(swig_val);
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (uint64_t) c_result;
}


SwigDirector_WrappedNamedArrayMemoryClientBuffer::~SwigDirector_WrappedNamedArrayMemoryClientBuffer() {
}

SwigDirector_WrappedNamedMultiDimArrayMemoryClientBuffer::SwigDirector_WrappedNamedMultiDimArrayMemoryClientBuffer(PyObject *self): RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer *)this, this); 
}




void SwigDirector_WrappedNamedMultiDimArrayMemoryClientBuffer::UnpackReadResult(boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > res, std::vector< uint64_t, std::allocator< uint64_t > > const &bufferpos, std::vector< uint64_t, std::allocator< uint64_t > > const &count) {
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartarg1 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    // intrusive_ptr by value directorin
    
    if(*(&res)) {
      intrusive_ptr_add_ref((&res)->get());
      smartarg1 = new boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >((&res)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
    }
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, SWIG_POINTER_OWN);
    
    swig::SwigVar_PyObject obj1;
    obj1 = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(bufferpos));
    swig::SwigVar_PyObject obj2;
    obj2 = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(count));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedNamedMultiDimArrayMemoryClientBuffer.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "UnpackReadResult";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("UnpackReadResult");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > SwigDirector_WrappedNamedMultiDimArrayMemoryClientBuffer::PackWriteRequest(std::vector< uint64_t, std::allocator< uint64_t > > const &bufferpos, std::vector< uint64_t, std::allocator< uint64_t > > const &count) {
  void *argp ;
  int swig_res = 0 ;
  int newmem = 0 ;
  
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > c_result;
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(bufferpos));
    swig::SwigVar_PyObject obj1;
    obj1 = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(count));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedNamedMultiDimArrayMemoryClientBuffer.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 1;
    const char *const swig_method_name = "PackWriteRequest";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("PackWriteRequest");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    
    // intrusive_ptr by value directorout
    //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >**)&result;
    //if (smartarg) {
    //	result = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(smartarg->get(), true);
    //}
    
    swig_res = SWIG_ConvertPtrAndOwn(result, &argp, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t,  0 , &newmem);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >""'");
    }
    if (argp) c_result = boost::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp)->get(),true);
    if (newmem & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp);
    
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >) c_result;
}


SwigDirector_WrappedNamedMultiDimArrayMemoryClientBuffer::~SwigDirector_WrappedNamedMultiDimArrayMemoryClientBuffer() {
}

SwigDirector_WrappedNamedArrayMemoryDirector::SwigDirector_WrappedNamedArrayMemoryDirector(PyObject *self): RobotRaconteur::WrappedNamedArrayMemoryDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedNamedArrayMemoryDirector *)this, this); 
}




SwigDirector_WrappedNamedArrayMemoryDirector::~SwigDirector_WrappedNamedArrayMemoryDirector() {
}

uint64_t SwigDirector_WrappedNamedArrayMemoryDirector::Length() {
  uint64_t c_result = SwigValueInit< uint64_t >() ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedNamedArrayMemoryDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "Length";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject args = PyTuple_New(0);
    swig::SwigVar_PyObject result = PyObject_Call(method, (PyObject *) args, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Length");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    unsigned long swig_val;
    int swig_res = SWIG_AsVal_unsigned_SS_long(result, &swig_val);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""uint64_t""'");
    }
    c_result = static_cast< uint64_t >(swig_val);
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (uint64_t) c_result;
}


boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > SwigDirector_WrappedNamedArrayMemoryDirector::Read(uint64_t memorypos, uint64_t bufferpos, uint64_t count) {
  void *argp ;
  int swig_res = 0 ;
  int newmem = 0 ;
  
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > c_result;
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(memorypos));
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(bufferpos));
    swig::SwigVar_PyObject obj2;
    obj2 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(count));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedNamedArrayMemoryDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 1;
    const char *const swig_method_name = "Read";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Read");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    
    // intrusive_ptr by value directorout
    //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >**)&result;
    //if (smartarg) {
    //	result = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(smartarg->get(), true);
    //}
    
    swig_res = SWIG_ConvertPtrAndOwn(result, &argp, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t,  0 , &newmem);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >""'");
    }
    if (argp) c_result = boost::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp)->get(),true);
    if (newmem & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp);
    
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >) c_result;
}


void SwigDirector_WrappedNamedArrayMemoryDirector::Write(uint64_t memorypos, boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > buffer, uint64_t bufferpos, uint64_t count) {
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartarg2 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(memorypos));
    swig::SwigVar_PyObject obj1;
    
    // intrusive_ptr by value directorin
    
    if(*(&buffer)) {
      intrusive_ptr_add_ref((&buffer)->get());
      smartarg2 = new boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >((&buffer)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
    }
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg2), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, SWIG_POINTER_OWN);
    
    swig::SwigVar_PyObject obj2;
    obj2 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(bufferpos));
    swig::SwigVar_PyObject obj3;
    obj3 = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(count));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedNamedArrayMemoryDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 2;
    const char *const swig_method_name = "Write";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2,(PyObject *)obj3, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Write");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2,(PyObject *)obj3, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_WrappedNamedMultiDimArrayMemoryDirector::SwigDirector_WrappedNamedMultiDimArrayMemoryDirector(PyObject *self): RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *)this, this); 
}




SwigDirector_WrappedNamedMultiDimArrayMemoryDirector::~SwigDirector_WrappedNamedMultiDimArrayMemoryDirector() {
}

std::vector< uint64_t, std::allocator< uint64_t > > SwigDirector_WrappedNamedMultiDimArrayMemoryDirector::Dimensions() {
  std::vector< uint64_t,std::allocator< uint64_t > > c_result;
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedNamedMultiDimArrayMemoryDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "Dimensions";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject args = PyTuple_New(0);
    swig::SwigVar_PyObject result = PyObject_Call(method, (PyObject *) args, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Dimensions");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    std::vector< unsigned long,std::allocator< unsigned long > > *swig_optr = 0;
    int swig_ores = swig::asptr(result, &swig_optr);
    if (!SWIG_IsOK(swig_ores) || !swig_optr) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError((swig_optr ? swig_ores : SWIG_TypeError))), "in output value of type '""std::vector< uint64_t,std::allocator< uint64_t > >""'");
    }
    c_result = *swig_optr;
    if (SWIG_IsNewObj(swig_ores)) delete swig_optr;
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (std::vector< uint64_t,std::allocator< uint64_t > >) c_result;
}


uint64_t SwigDirector_WrappedNamedMultiDimArrayMemoryDirector::DimCount() {
  uint64_t c_result = SwigValueInit< uint64_t >() ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedNamedMultiDimArrayMemoryDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 1;
    const char *const swig_method_name = "DimCount";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject args = PyTuple_New(0);
    swig::SwigVar_PyObject result = PyObject_Call(method, (PyObject *) args, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("DimCount");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    unsigned long swig_val;
    int swig_res = SWIG_AsVal_unsigned_SS_long(result, &swig_val);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""uint64_t""'");
    }
    c_result = static_cast< uint64_t >(swig_val);
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (uint64_t) c_result;
}


boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > SwigDirector_WrappedNamedMultiDimArrayMemoryDirector::Read(std::vector< uint64_t, std::allocator< uint64_t > > const &memorypos, std::vector< uint64_t, std::allocator< uint64_t > > const &bufferpos, std::vector< uint64_t, std::allocator< uint64_t > > const &count) {
  void *argp ;
  int swig_res = 0 ;
  int newmem = 0 ;
  
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > c_result;
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(memorypos));
    swig::SwigVar_PyObject obj1;
    obj1 = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(bufferpos));
    swig::SwigVar_PyObject obj2;
    obj2 = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(count));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedNamedMultiDimArrayMemoryDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 2;
    const char *const swig_method_name = "Read";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Read");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    
    // intrusive_ptr by value directorout
    //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >**)&result;
    //if (smartarg) {
    //	result = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(smartarg->get(), true);
    //}
    
    swig_res = SWIG_ConvertPtrAndOwn(result, &argp, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t,  0 , &newmem);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >""'");
    }
    if (argp) c_result = boost::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp)->get(),true);
    if (newmem & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp);
    
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >) c_result;
}


void SwigDirector_WrappedNamedMultiDimArrayMemoryDirector::Write(std::vector< uint64_t, std::allocator< uint64_t > > const &memorypos, boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > buffer, std::vector< uint64_t, std::allocator< uint64_t > > const &bufferpos, std::vector< uint64_t, std::allocator< uint64_t > > const &count) {
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartarg2 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(memorypos));
    swig::SwigVar_PyObject obj1;
    
    // intrusive_ptr by value directorin
    
    if(*(&buffer)) {
      intrusive_ptr_add_ref((&buffer)->get());
      smartarg2 = new boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >((&buffer)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
    }
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg2), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, SWIG_POINTER_OWN);
    
    swig::SwigVar_PyObject obj2;
    obj2 = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(bufferpos));
    swig::SwigVar_PyObject obj3;
    obj3 = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(count));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedNamedMultiDimArrayMemoryDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 3;
    const char *const swig_method_name = "Write";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2,(PyObject *)obj3, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Write");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2,(PyObject *)obj3, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_AsyncGeneratorClientReturnDirector::SwigDirector_AsyncGeneratorClientReturnDirector(PyObject *self): RobotRaconteur::AsyncGeneratorClientReturnDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::AsyncGeneratorClientReturnDirector *)this, this); 
}




SwigDirector_AsyncGeneratorClientReturnDirector::~SwigDirector_AsyncGeneratorClientReturnDirector() {
}

void SwigDirector_AsyncGeneratorClientReturnDirector::handler(boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > const &ret, RobotRaconteur::HandlerErrorInfo &error) {
  boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > *smartarg1 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    smartarg1 = ret ? new boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient >(ret) : 0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedGeneratorClient_t, SWIG_POINTER_OWN |  0 );
    
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(&error), SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0 );
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call AsyncGeneratorClientReturnDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "handler";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("handler");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_WrappedGeneratorServerDirector::SwigDirector_WrappedGeneratorServerDirector(PyObject *self): RobotRaconteur::WrappedGeneratorServerDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedGeneratorServerDirector *)this, this); 
}




SwigDirector_WrappedGeneratorServerDirector::~SwigDirector_WrappedGeneratorServerDirector() {
}

boost::intrusive_ptr< RobotRaconteur::MessageElement > SwigDirector_WrappedGeneratorServerDirector::Next(boost::intrusive_ptr< RobotRaconteur::MessageElement > m, boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > const &async_adaptor) {
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > *smartarg2 = 0 ;
  void *argp ;
  int swig_res = 0 ;
  int newmem = 0 ;
  
  boost::intrusive_ptr< RobotRaconteur::MessageElement > c_result;
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    // intrusive_ptr by value directorin
    
    if(*(&m)) {
      intrusive_ptr_add_ref((&m)->get());
      smartarg1 = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&m)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
    }
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
    
    swig::SwigVar_PyObject obj1;
    
    smartarg2 = async_adaptor ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSkelAsyncAdapter >(async_adaptor) : 0;
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg2), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t, SWIG_POINTER_OWN |  0 );
    
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedGeneratorServerDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "Next";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Next");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    
    // intrusive_ptr by value directorout
    //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&result;
    //if (smartarg) {
    //	result = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
    //}
    
    swig_res = SWIG_ConvertPtrAndOwn(result, &argp, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""boost::intrusive_ptr< RobotRaconteur::MessageElement >""'");
    }
    if (argp) c_result = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp)->get(),true);
    if (newmem & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp);
    
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (boost::intrusive_ptr< RobotRaconteur::MessageElement >) c_result;
}


void SwigDirector_WrappedGeneratorServerDirector::Abort(boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > const &async_adapter) {
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > *smartarg1 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    smartarg1 = async_adapter ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSkelAsyncAdapter >(async_adapter) : 0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t, SWIG_POINTER_OWN |  0 );
    
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedGeneratorServerDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 1;
    const char *const swig_method_name = "Abort";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Abort");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


void SwigDirector_WrappedGeneratorServerDirector::Close(boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > const &async_adapter) {
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > *smartarg1 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    smartarg1 = async_adapter ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSkelAsyncAdapter >(async_adapter) : 0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t, SWIG_POINTER_OWN |  0 );
    
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedGeneratorServerDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 2;
    const char *const swig_method_name = "Close";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Close");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_WrappedServiceStubDirector::SwigDirector_WrappedServiceStubDirector(PyObject *self): RobotRaconteur::WrappedServiceStubDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedServiceStubDirector *)this, this); 
}




SwigDirector_WrappedServiceStubDirector::~SwigDirector_WrappedServiceStubDirector() {
}

void SwigDirector_WrappedServiceStubDirector::DispatchEvent(std::string const &EventName, std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >, std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > args) {
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_std_string(static_cast< std::string >(EventName));
    swig::SwigVar_PyObject obj1;
    
    //std::vector<boost::intrusive_ptr<T> > workaround
    
    obj1 = SWIG_NewPointerObj((void*)(new std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >((const std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > &)args)), SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, SWIG_POINTER_OWN | 0);
    
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedServiceStubDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "DispatchEvent";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("DispatchEvent");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


boost::intrusive_ptr< RobotRaconteur::MessageElement > SwigDirector_WrappedServiceStubDirector::CallbackCall(std::string const &CallbackName, std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >, std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > args) {
  void *argp ;
  int swig_res = 0 ;
  int newmem = 0 ;
  
  boost::intrusive_ptr< RobotRaconteur::MessageElement > c_result;
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_std_string(static_cast< std::string >(CallbackName));
    swig::SwigVar_PyObject obj1;
    
    //std::vector<boost::intrusive_ptr<T> > workaround
    
    obj1 = SWIG_NewPointerObj((void*)(new std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >((const std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > &)args)), SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, SWIG_POINTER_OWN | 0);
    
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedServiceStubDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 1;
    const char *const swig_method_name = "CallbackCall";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("CallbackCall");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    
    // intrusive_ptr by value directorout
    //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&result;
    //if (smartarg) {
    //	result = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
    //}
    
    swig_res = SWIG_ConvertPtrAndOwn(result, &argp, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""boost::intrusive_ptr< RobotRaconteur::MessageElement >""'");
    }
    if (argp) c_result = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp)->get(),true);
    if (newmem & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp);
    
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (boost::intrusive_ptr< RobotRaconteur::MessageElement >) c_result;
}


SwigDirector_AsyncStubReturnDirector::SwigDirector_AsyncStubReturnDirector(PyObject *self): RobotRaconteur::AsyncStubReturnDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::AsyncStubReturnDirector *)this, this); 
}




SwigDirector_AsyncStubReturnDirector::~SwigDirector_AsyncStubReturnDirector() {
}

void SwigDirector_AsyncStubReturnDirector::handler(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &stub, RobotRaconteur::HandlerErrorInfo &error) {
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    smartarg1 = stub ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(stub) : 0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN |  0 );
    
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(&error), SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0 );
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call AsyncStubReturnDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "handler";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("handler");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_WrappedServiceSkelDirector::SwigDirector_WrappedServiceSkelDirector(PyObject *self): RobotRaconteur::WrappedServiceSkelDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedServiceSkelDirector *)this, this); 
}




SwigDirector_WrappedServiceSkelDirector::~SwigDirector_WrappedServiceSkelDirector() {
}

void SwigDirector_WrappedServiceSkelDirector::Init(boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > const &skel) {
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > *smartarg1 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    smartarg1 = skel ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel >(skel) : 0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t, SWIG_POINTER_OWN |  0 );
    
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedServiceSkelDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "Init";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Init");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


boost::intrusive_ptr< RobotRaconteur::MessageElement > SwigDirector_WrappedServiceSkelDirector::CallGetProperty(std::string const &name, boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > const &async_adaptor) {
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > *smartarg2 = 0 ;
  void *argp ;
  int swig_res = 0 ;
  int newmem = 0 ;
  
  boost::intrusive_ptr< RobotRaconteur::MessageElement > c_result;
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_std_string(static_cast< std::string >(name));
    swig::SwigVar_PyObject obj1;
    
    smartarg2 = async_adaptor ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSkelAsyncAdapter >(async_adaptor) : 0;
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg2), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t, SWIG_POINTER_OWN |  0 );
    
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedServiceSkelDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 1;
    const char *const swig_method_name = "_CallGetProperty";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("_CallGetProperty");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    
    // intrusive_ptr by value directorout
    //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&result;
    //if (smartarg) {
    //	result = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
    //}
    
    swig_res = SWIG_ConvertPtrAndOwn(result, &argp, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""boost::intrusive_ptr< RobotRaconteur::MessageElement >""'");
    }
    if (argp) c_result = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp)->get(),true);
    if (newmem & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp);
    
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (boost::intrusive_ptr< RobotRaconteur::MessageElement >) c_result;
}


void SwigDirector_WrappedServiceSkelDirector::CallSetProperty(std::string const &name, boost::intrusive_ptr< RobotRaconteur::MessageElement > m, boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > const &async_adaptor) {
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > *smartarg3 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_std_string(static_cast< std::string >(name));
    swig::SwigVar_PyObject obj1;
    
    // intrusive_ptr by value directorin
    
    if(*(&m)) {
      intrusive_ptr_add_ref((&m)->get());
      smartarg2 = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&m)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
    }
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg2), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
    
    swig::SwigVar_PyObject obj2;
    
    smartarg3 = async_adaptor ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSkelAsyncAdapter >(async_adaptor) : 0;
    obj2 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg3), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t, SWIG_POINTER_OWN |  0 );
    
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedServiceSkelDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 2;
    const char *const swig_method_name = "_CallSetProperty";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("_CallSetProperty");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


boost::intrusive_ptr< RobotRaconteur::MessageElement > SwigDirector_WrappedServiceSkelDirector::CallFunction(std::string const &name, std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >, std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &m, boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > const &async_adaptor) {
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > *smartarg3 = 0 ;
  void *argp ;
  int swig_res = 0 ;
  int newmem = 0 ;
  
  boost::intrusive_ptr< RobotRaconteur::MessageElement > c_result;
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_std_string(static_cast< std::string >(name));
    swig::SwigVar_PyObject obj1;
    
    //std::vector<boost::intrusive_ptr<T> > workaround
    
    obj1 = SWIG_NewPointerObj((void*)(new std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >(*(const std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *)&m)), SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, SWIG_POINTER_OWN | 0);
    
    swig::SwigVar_PyObject obj2;
    
    smartarg3 = async_adaptor ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSkelAsyncAdapter >(async_adaptor) : 0;
    obj2 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg3), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t, SWIG_POINTER_OWN |  0 );
    
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedServiceSkelDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 3;
    const char *const swig_method_name = "_CallFunction";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("_CallFunction");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    
    // intrusive_ptr by value directorout
    //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&result;
    //if (smartarg) {
    //	result = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
    //}
    
    swig_res = SWIG_ConvertPtrAndOwn(result, &argp, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""boost::intrusive_ptr< RobotRaconteur::MessageElement >""'");
    }
    if (argp) c_result = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp)->get(),true);
    if (newmem & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp);
    
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (boost::intrusive_ptr< RobotRaconteur::MessageElement >) c_result;
}


boost::shared_ptr< RobotRaconteur::WrappedRRObject > SwigDirector_WrappedServiceSkelDirector::GetSubObj(std::string const &name, std::string const &index) {
  void *swig_argp ;
  int swig_res = 0 ;
  
  boost::shared_ptr< RobotRaconteur::WrappedRRObject > c_result;
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_std_string(static_cast< std::string >(name));
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_From_std_string(static_cast< std::string >(index));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedServiceSkelDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 4;
    const char *const swig_method_name = "_GetSubObj";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("_GetSubObj");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    int newmem = 0;
    swig_res = SWIG_ConvertPtrAndOwn(result, &swig_argp, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedRRObject_t,  0 , &newmem);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""boost::shared_ptr< RobotRaconteur::WrappedRRObject >""'");
    }
    if (swig_argp) {
      c_result = *(reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedRRObject > * >(swig_argp));
      if (newmem & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedRRObject > * >(swig_argp);
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (boost::shared_ptr< RobotRaconteur::WrappedRRObject >) c_result;
}


RobotRaconteur::WrappedArrayMemoryDirector *SwigDirector_WrappedServiceSkelDirector::GetArrayMemory(std::string const &name) {
  RobotRaconteur::WrappedArrayMemoryDirector *c_result = 0 ;
  void *swig_argp ;
  int swig_res ;
  swig_owntype own ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_std_string(static_cast< std::string >(name));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedServiceSkelDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 5;
    const char *const swig_method_name = "_GetArrayMemory";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("_GetArrayMemory");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    swig_res = SWIG_ConvertPtrAndOwn(result, &swig_argp, SWIGTYPE_p_RobotRaconteur__WrappedArrayMemoryDirector,  0  | SWIG_POINTER_DISOWN, &own);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""RobotRaconteur::WrappedArrayMemoryDirector *""'");
    }
    c_result = reinterpret_cast< RobotRaconteur::WrappedArrayMemoryDirector * >(swig_argp);
    swig_acquire_ownership_obj(SWIG_as_voidptr(c_result), own /* & TODO: SWIG_POINTER_OWN */);
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (RobotRaconteur::WrappedArrayMemoryDirector *) c_result;
}


RobotRaconteur::WrappedMultiDimArrayMemoryDirector *SwigDirector_WrappedServiceSkelDirector::GetMultiDimArrayMemory(std::string const &name) {
  RobotRaconteur::WrappedMultiDimArrayMemoryDirector *c_result = 0 ;
  void *swig_argp ;
  int swig_res ;
  swig_owntype own ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_std_string(static_cast< std::string >(name));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedServiceSkelDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 6;
    const char *const swig_method_name = "_GetMultiDimArrayMemory";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("_GetMultiDimArrayMemory");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    swig_res = SWIG_ConvertPtrAndOwn(result, &swig_argp, SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryDirector,  0  | SWIG_POINTER_DISOWN, &own);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""RobotRaconteur::WrappedMultiDimArrayMemoryDirector *""'");
    }
    c_result = reinterpret_cast< RobotRaconteur::WrappedMultiDimArrayMemoryDirector * >(swig_argp);
    swig_acquire_ownership_obj(SWIG_as_voidptr(c_result), own /* & TODO: SWIG_POINTER_OWN */);
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (RobotRaconteur::WrappedMultiDimArrayMemoryDirector *) c_result;
}


RobotRaconteur::WrappedPodArrayMemoryDirector *SwigDirector_WrappedServiceSkelDirector::GetPodArrayMemory(std::string const &name) {
  RobotRaconteur::WrappedPodArrayMemoryDirector *c_result = 0 ;
  void *swig_argp ;
  int swig_res ;
  swig_owntype own ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_std_string(static_cast< std::string >(name));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedServiceSkelDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 7;
    const char *const swig_method_name = "_GetPodArrayMemory";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("_GetPodArrayMemory");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    swig_res = SWIG_ConvertPtrAndOwn(result, &swig_argp, SWIGTYPE_p_RobotRaconteur__WrappedPodArrayMemoryDirector,  0  | SWIG_POINTER_DISOWN, &own);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""RobotRaconteur::WrappedPodArrayMemoryDirector *""'");
    }
    c_result = reinterpret_cast< RobotRaconteur::WrappedPodArrayMemoryDirector * >(swig_argp);
    swig_acquire_ownership_obj(SWIG_as_voidptr(c_result), own /* & TODO: SWIG_POINTER_OWN */);
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (RobotRaconteur::WrappedPodArrayMemoryDirector *) c_result;
}


RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *SwigDirector_WrappedServiceSkelDirector::GetPodMultiDimArrayMemory(std::string const &name) {
  RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *c_result = 0 ;
  void *swig_argp ;
  int swig_res ;
  swig_owntype own ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_std_string(static_cast< std::string >(name));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedServiceSkelDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 8;
    const char *const swig_method_name = "_GetPodMultiDimArrayMemory";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("_GetPodMultiDimArrayMemory");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    swig_res = SWIG_ConvertPtrAndOwn(result, &swig_argp, SWIGTYPE_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryDirector,  0  | SWIG_POINTER_DISOWN, &own);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *""'");
    }
    c_result = reinterpret_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector * >(swig_argp);
    swig_acquire_ownership_obj(SWIG_as_voidptr(c_result), own /* & TODO: SWIG_POINTER_OWN */);
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *) c_result;
}


RobotRaconteur::WrappedNamedArrayMemoryDirector *SwigDirector_WrappedServiceSkelDirector::GetNamedArrayMemory(std::string const &name) {
  RobotRaconteur::WrappedNamedArrayMemoryDirector *c_result = 0 ;
  void *swig_argp ;
  int swig_res ;
  swig_owntype own ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_std_string(static_cast< std::string >(name));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedServiceSkelDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 9;
    const char *const swig_method_name = "_GetNamedArrayMemory";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("_GetNamedArrayMemory");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    swig_res = SWIG_ConvertPtrAndOwn(result, &swig_argp, SWIGTYPE_p_RobotRaconteur__WrappedNamedArrayMemoryDirector,  0  | SWIG_POINTER_DISOWN, &own);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""RobotRaconteur::WrappedNamedArrayMemoryDirector *""'");
    }
    c_result = reinterpret_cast< RobotRaconteur::WrappedNamedArrayMemoryDirector * >(swig_argp);
    swig_acquire_ownership_obj(SWIG_as_voidptr(c_result), own /* & TODO: SWIG_POINTER_OWN */);
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (RobotRaconteur::WrappedNamedArrayMemoryDirector *) c_result;
}


RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *SwigDirector_WrappedServiceSkelDirector::GetNamedMultiDimArrayMemory(std::string const &name) {
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *c_result = 0 ;
  void *swig_argp ;
  int swig_res ;
  swig_owntype own ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_std_string(static_cast< std::string >(name));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedServiceSkelDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 10;
    const char *const swig_method_name = "_GetNamedMultiDimArrayMemory";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("_GetNamedMultiDimArrayMemory");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    swig_res = SWIG_ConvertPtrAndOwn(result, &swig_argp, SWIGTYPE_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryDirector,  0  | SWIG_POINTER_DISOWN, &own);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *""'");
    }
    c_result = reinterpret_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector * >(swig_argp);
    swig_acquire_ownership_obj(SWIG_as_voidptr(c_result), own /* & TODO: SWIG_POINTER_OWN */);
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *) c_result;
}


void SwigDirector_WrappedServiceSkelDirector::MonitorEnter(int32_t timeout) {
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_int(static_cast< int >(timeout));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedServiceSkelDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 11;
    const char *const swig_method_name = "MonitorEnter";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("MonitorEnter");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


void SwigDirector_WrappedServiceSkelDirector::MonitorExit() {
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedServiceSkelDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 12;
    const char *const swig_method_name = "MonitorExit";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject args = PyTuple_New(0);
    swig::SwigVar_PyObject result = PyObject_Call(method, (PyObject *) args, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("MonitorExit");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


void SwigDirector_WrappedServiceSkelDirector::ReleaseCastObject() {
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedServiceSkelDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 13;
    const char *const swig_method_name = "ReleaseCastObject";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject args = PyTuple_New(0);
    swig::SwigVar_PyObject result = PyObject_Call(method, (PyObject *) args, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("ReleaseCastObject");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_ServerServiceListenerDirector::SwigDirector_ServerServiceListenerDirector(PyObject *self): ServerServiceListenerDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((ServerServiceListenerDirector *)this, this); 
}




void SwigDirector_ServerServiceListenerDirector::Callback(int32_t code, uint32_t endpoint) {
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_int(static_cast< int >(code));
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(endpoint));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call ServerServiceListenerDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "Callback";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Callback");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_ServerServiceListenerDirector::~SwigDirector_ServerServiceListenerDirector() {
}

SwigDirector_WrappedUserAuthenticatorDirector::SwigDirector_WrappedUserAuthenticatorDirector(PyObject *self): RobotRaconteur::WrappedUserAuthenticatorDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedUserAuthenticatorDirector *)this, this); 
}




SwigDirector_WrappedUserAuthenticatorDirector::~SwigDirector_WrappedUserAuthenticatorDirector() {
}

boost::shared_ptr< RobotRaconteur::AuthenticatedUser > SwigDirector_WrappedUserAuthenticatorDirector::AuthenticateUser(std::string const &username, boost::intrusive_ptr< RobotRaconteur::MessageElement > credentials, boost::shared_ptr< RobotRaconteur::ServerContext > const &context) {
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg2 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *smartarg3 = 0 ;
  void *swig_argp ;
  int swig_res = 0 ;
  
  boost::shared_ptr< RobotRaconteur::AuthenticatedUser > c_result;
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_std_string(static_cast< std::string >(username));
    swig::SwigVar_PyObject obj1;
    
    // intrusive_ptr by value directorin
    
    if(*(&credentials)) {
      intrusive_ptr_add_ref((&credentials)->get());
      smartarg2 = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&credentials)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
    }
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg2), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
    
    swig::SwigVar_PyObject obj2;
    
    smartarg3 = context ? new boost::shared_ptr<  RobotRaconteur::ServerContext >(context) : 0;
    obj2 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg3), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, SWIG_POINTER_OWN |  0 );
    
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedUserAuthenticatorDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "AuthenticateUser";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("AuthenticateUser");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    int newmem = 0;
    swig_res = SWIG_ConvertPtrAndOwn(result, &swig_argp, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__AuthenticatedUser_t,  0 , &newmem);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""boost::shared_ptr< RobotRaconteur::AuthenticatedUser >""'");
    }
    if (swig_argp) {
      c_result = *(reinterpret_cast< boost::shared_ptr< RobotRaconteur::AuthenticatedUser > * >(swig_argp));
      if (newmem & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::AuthenticatedUser > * >(swig_argp);
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (boost::shared_ptr< RobotRaconteur::AuthenticatedUser >) c_result;
}


SwigDirector_AsyncServiceInfo2VectorReturnDirector::SwigDirector_AsyncServiceInfo2VectorReturnDirector(PyObject *self): RobotRaconteur::AsyncServiceInfo2VectorReturnDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::AsyncServiceInfo2VectorReturnDirector *)this, this); 
}




SwigDirector_AsyncServiceInfo2VectorReturnDirector::~SwigDirector_AsyncServiceInfo2VectorReturnDirector() {
}

void SwigDirector_AsyncServiceInfo2VectorReturnDirector::handler(std::vector< RobotRaconteur::ServiceInfo2Wrapped, std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > const &ret) {
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(&ret), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t,  0 );
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call AsyncServiceInfo2VectorReturnDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "handler";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("handler");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_AsyncNodeInfo2VectorReturnDirector::SwigDirector_AsyncNodeInfo2VectorReturnDirector(PyObject *self): RobotRaconteur::AsyncNodeInfo2VectorReturnDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::AsyncNodeInfo2VectorReturnDirector *)this, this); 
}




SwigDirector_AsyncNodeInfo2VectorReturnDirector::~SwigDirector_AsyncNodeInfo2VectorReturnDirector() {
}

void SwigDirector_AsyncNodeInfo2VectorReturnDirector::handler(std::vector< RobotRaconteur::NodeInfo2, std::allocator< RobotRaconteur::NodeInfo2 > > const &ret) {
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(&ret), SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t,  0 );
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call AsyncNodeInfo2VectorReturnDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "handler";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("handler");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_WrappedServiceSubscriptionFilterPredicateDirector::SwigDirector_WrappedServiceSubscriptionFilterPredicateDirector(PyObject *self): RobotRaconteur::WrappedServiceSubscriptionFilterPredicateDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedServiceSubscriptionFilterPredicateDirector *)this, this); 
}




bool SwigDirector_WrappedServiceSubscriptionFilterPredicateDirector::Predicate(RobotRaconteur::ServiceInfo2Wrapped const &info) {
  bool c_result = SwigValueInit< bool >() ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(&info), SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped,  0 );
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedServiceSubscriptionFilterPredicateDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "Predicate";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Predicate");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
    bool swig_val;
    int swig_res = SWIG_AsVal_bool(result, &swig_val);
    if (!SWIG_IsOK(swig_res)) {
      Swig::DirectorTypeMismatchException::raise(SWIG_ErrorType(SWIG_ArgError(swig_res)), "in output value of type '""bool""'");
    }
    c_result = static_cast< bool >(swig_val);
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
  return (bool) c_result;
}


SwigDirector_WrappedServiceSubscriptionFilterPredicateDirector::~SwigDirector_WrappedServiceSubscriptionFilterPredicateDirector() {
}

SwigDirector_WrappedServiceInfo2SubscriptionDirector::SwigDirector_WrappedServiceInfo2SubscriptionDirector(PyObject *self): RobotRaconteur::WrappedServiceInfo2SubscriptionDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedServiceInfo2SubscriptionDirector *)this, this); 
}




void SwigDirector_WrappedServiceInfo2SubscriptionDirector::ServiceDetected(boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > const &subscription, RobotRaconteur::ServiceSubscriptionClientID const &id, RobotRaconteur::ServiceInfo2Wrapped const &service) {
  boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > *smartarg1 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    smartarg1 = subscription ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceInfo2Subscription >(subscription) : 0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceInfo2Subscription_t, SWIG_POINTER_OWN |  0 );
    
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(&id), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0 );
    swig::SwigVar_PyObject obj2;
    obj2 = SWIG_NewPointerObj(SWIG_as_voidptr(&service), SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped,  0 );
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedServiceInfo2SubscriptionDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "ServiceDetected";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("ServiceDetected");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


void SwigDirector_WrappedServiceInfo2SubscriptionDirector::ServiceLost(boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > const &subscription, RobotRaconteur::ServiceSubscriptionClientID const &id, RobotRaconteur::ServiceInfo2Wrapped const &service) {
  boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > *smartarg1 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    smartarg1 = subscription ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceInfo2Subscription >(subscription) : 0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceInfo2Subscription_t, SWIG_POINTER_OWN |  0 );
    
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(&id), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0 );
    swig::SwigVar_PyObject obj2;
    obj2 = SWIG_NewPointerObj(SWIG_as_voidptr(&service), SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped,  0 );
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedServiceInfo2SubscriptionDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 1;
    const char *const swig_method_name = "ServiceLost";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("ServiceLost");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_WrappedServiceInfo2SubscriptionDirector::~SwigDirector_WrappedServiceInfo2SubscriptionDirector() {
}

SwigDirector_WrappedServiceSubscriptionDirector::SwigDirector_WrappedServiceSubscriptionDirector(PyObject *self): RobotRaconteur::WrappedServiceSubscriptionDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedServiceSubscriptionDirector *)this, this); 
}




void SwigDirector_WrappedServiceSubscriptionDirector::ClientConnected(boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > const &subscription, RobotRaconteur::ServiceSubscriptionClientID const &id, boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &silent) {
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg3 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    smartarg1 = subscription ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription >(subscription) : 0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, SWIG_POINTER_OWN |  0 );
    
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(&id), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0 );
    swig::SwigVar_PyObject obj2;
    
    smartarg3 = silent ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(silent) : 0;
    obj2 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg3), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN |  0 );
    
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedServiceSubscriptionDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "ClientConnected";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("ClientConnected");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


void SwigDirector_WrappedServiceSubscriptionDirector::ClientDisconnected(boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > const &subscription, RobotRaconteur::ServiceSubscriptionClientID const &id, boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &silent) {
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg3 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    smartarg1 = subscription ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription >(subscription) : 0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, SWIG_POINTER_OWN |  0 );
    
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(&id), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0 );
    swig::SwigVar_PyObject obj2;
    
    smartarg3 = silent ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(silent) : 0;
    obj2 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg3), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN |  0 );
    
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedServiceSubscriptionDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 1;
    const char *const swig_method_name = "ClientDisconnected";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("ClientDisconnected");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


void SwigDirector_WrappedServiceSubscriptionDirector::ClientConnectFailed(boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > const &subscription, RobotRaconteur::ServiceSubscriptionClientID const &id, std::vector< std::string, std::allocator< std::string > > const &url, RobotRaconteur::HandlerErrorInfo &error) {
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    smartarg1 = subscription ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription >(subscription) : 0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, SWIG_POINTER_OWN |  0 );
    
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(&id), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0 );
    swig::SwigVar_PyObject obj2;
    obj2 = swig::from(static_cast< std::vector< std::string,std::allocator< std::string > > >(url));
    swig::SwigVar_PyObject obj3;
    obj3 = SWIG_NewPointerObj(SWIG_as_voidptr(&error), SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0 );
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedServiceSubscriptionDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 2;
    const char *const swig_method_name = "ClientConnectFailed";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2,(PyObject *)obj3, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("ClientConnectFailed");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2,(PyObject *)obj3, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_WrappedServiceSubscriptionDirector::~SwigDirector_WrappedServiceSubscriptionDirector() {
}

SwigDirector_WrappedWireSubscriptionDirector::SwigDirector_WrappedWireSubscriptionDirector(PyObject *self): RobotRaconteur::WrappedWireSubscriptionDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedWireSubscriptionDirector *)this, this); 
}




void SwigDirector_WrappedWireSubscriptionDirector::WireValueChanged(boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > const &wire_subscription, RobotRaconteur::WrappedService_typed_packet &value, RobotRaconteur::TimeSpec const &time) {
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > *smartarg1 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    smartarg1 = wire_subscription ? new boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription >(wire_subscription) : 0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, SWIG_POINTER_OWN |  0 );
    
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_NewPointerObj(SWIG_as_voidptr(&value), SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet,  0 );
    swig::SwigVar_PyObject obj2;
    obj2 = SWIG_NewPointerObj(SWIG_as_voidptr(&time), SWIGTYPE_p_RobotRaconteur__TimeSpec,  0 );
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedWireSubscriptionDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "WireValueChanged";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("WireValueChanged");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1,(PyObject *)obj2, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_WrappedWireSubscriptionDirector::~SwigDirector_WrappedWireSubscriptionDirector() {
}

SwigDirector_WrappedPipeSubscriptionDirector::SwigDirector_WrappedPipeSubscriptionDirector(PyObject *self): RobotRaconteur::WrappedPipeSubscriptionDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::WrappedPipeSubscriptionDirector *)this, this); 
}




void SwigDirector_WrappedPipeSubscriptionDirector::PipePacketReceived(boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > const &pipe_subscription) {
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > *smartarg1 = 0 ;
  
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    
    smartarg1 = pipe_subscription ? new boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription >(pipe_subscription) : 0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(smartarg1), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t, SWIG_POINTER_OWN |  0 );
    
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call WrappedPipeSubscriptionDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "PipePacketReceived";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("PipePacketReceived");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_WrappedPipeSubscriptionDirector::~SwigDirector_WrappedPipeSubscriptionDirector() {
}

SwigDirector_UserLogRecordHandlerDirector::SwigDirector_UserLogRecordHandlerDirector(PyObject *self): RobotRaconteur::UserLogRecordHandlerDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((RobotRaconteur::UserLogRecordHandlerDirector *)this, this); 
}




void SwigDirector_UserLogRecordHandlerDirector::HandleLogRecord(RobotRaconteur::RRLogRecord const &record) {
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_NewPointerObj(SWIG_as_voidptr(&record), SWIGTYPE_p_RobotRaconteur__RRLogRecord,  0 );
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call UserLogRecordHandlerDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "HandleLogRecord";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("HandleLogRecord");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_UserLogRecordHandlerDirector::~SwigDirector_UserLogRecordHandlerDirector() {
}

SwigDirector_ClientServiceListenerDirector::SwigDirector_ClientServiceListenerDirector(PyObject *self): ClientServiceListenerDirector(), Swig::Director(self) {
  SWIG_DIRECTOR_RGTR((ClientServiceListenerDirector *)this, this); 
}




void SwigDirector_ClientServiceListenerDirector::Callback(int32_t code) {
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_int(static_cast< int >(code));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call ClientServiceListenerDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 0;
    const char *const swig_method_name = "Callback";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Callback");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


void SwigDirector_ClientServiceListenerDirector::Callback2(int32_t code, std::string const &p) {
  SWIG_PYTHON_THREAD_BEGIN_BLOCK;
  {
    swig::SwigVar_PyObject obj0;
    obj0 = SWIG_From_int(static_cast< int >(code));
    swig::SwigVar_PyObject obj1;
    obj1 = SWIG_From_std_string(static_cast< std::string >(p));
    if (!swig_get_self()) {
      Swig::DirectorException::raise("'self' uninitialized, maybe you forgot to call ClientServiceListenerDirector.__init__.");
    }
#if defined(SWIG_PYTHON_DIRECTOR_VTABLE)
    const size_t swig_method_index = 1;
    const char *const swig_method_name = "Callback2";
    PyObject *method = swig_get_method(swig_method_index, swig_method_name);
    swig::SwigVar_PyObject result = PyObject_CallFunctionObjArgs(method ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#else
    swig::SwigVar_PyObject swig_method_name = SWIG_Python_str_FromChar("Callback2");
    swig::SwigVar_PyObject result = PyObject_CallMethodObjArgs(swig_get_self(), (PyObject *) swig_method_name ,(PyObject *)obj0,(PyObject *)obj1, NULL);
#endif
    if (!result) {
      PyObject *error = PyErr_Occurred();
      {
        if (error != NULL) {
          ThrowPythonError();
        }
      }
    }
  }
  SWIG_PYTHON_THREAD_END_BLOCK;
}


SwigDirector_ClientServiceListenerDirector::~SwigDirector_ClientServiceListenerDirector() {
}

#ifdef __cplusplus
extern "C" {
#endif
SWIGINTERN PyObject *_wrap_delete_SwigPyIterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  swig::SwigPyIterator *arg1 = (swig::SwigPyIterator *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_SwigPyIterator" "', argument " "1"" of type '" "swig::SwigPyIterator *""'"); 
  }
  arg1 = reinterpret_cast< swig::SwigPyIterator * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    delete arg1;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_SwigPyIterator_value(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  swig::SwigPyIterator *arg1 = (swig::SwigPyIterator *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_swig__SwigPyIterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "SwigPyIterator_value" "', argument " "1"" of type '" "swig::SwigPyIterator const *""'"); 
  }
  arg1 = reinterpret_cast< swig::SwigPyIterator * >(argp1);
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)((swig::SwigPyIterator const *)arg1)->value();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  } catch(swig::stop_iteration &_e) {
    {
      (void)_e;
      SWIG_SetErrorObj(PyExc_StopIteration, SWIG_Py_Void());
      SWIG_fail;
    }
  }
  resultobj = result;
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_SwigPyIterator_incr__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  swig::SwigPyIterator *arg1 = (swig::SwigPyIterator *) 0 ;
  size_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  swig::SwigPyIterator *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_swig__SwigPyIterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "SwigPyIterator_incr" "', argument " "1"" of type '" "swig::SwigPyIterator *""'"); 
  }
  arg1 = reinterpret_cast< swig::SwigPyIterator * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "SwigPyIterator_incr" "', argument " "2"" of type '" "size_t""'");
  } 
  arg2 = static_cast< size_t >(val2);
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)(arg1)->incr(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  } catch(swig::stop_iteration &_e) {
    {
      (void)_e;
      SWIG_SetErrorObj(PyExc_StopIteration, SWIG_Py_Void());
      SWIG_fail;
    }
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_SwigPyIterator_incr__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  swig::SwigPyIterator *arg1 = (swig::SwigPyIterator *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_swig__SwigPyIterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "SwigPyIterator_incr" "', argument " "1"" of type '" "swig::SwigPyIterator *""'"); 
  }
  arg1 = reinterpret_cast< swig::SwigPyIterator * >(argp1);
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)(arg1)->incr();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  } catch(swig::stop_iteration &_e) {
    {
      (void)_e;
      SWIG_SetErrorObj(PyExc_StopIteration, SWIG_Py_Void());
      SWIG_fail;
    }
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_SwigPyIterator_incr(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "SwigPyIterator_incr", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    void *vptr = 0;
    int res = SWIG_ConvertPtr(argv[0], &vptr, SWIGTYPE_p_swig__SwigPyIterator, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_SwigPyIterator_incr__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    void *vptr = 0;
    int res = SWIG_ConvertPtr(argv[0], &vptr, SWIGTYPE_p_swig__SwigPyIterator, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_SwigPyIterator_incr__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'SwigPyIterator_incr'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    swig::SwigPyIterator::incr(size_t)\n"
    "    swig::SwigPyIterator::incr()\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_SwigPyIterator_decr__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  swig::SwigPyIterator *arg1 = (swig::SwigPyIterator *) 0 ;
  size_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  swig::SwigPyIterator *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_swig__SwigPyIterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "SwigPyIterator_decr" "', argument " "1"" of type '" "swig::SwigPyIterator *""'"); 
  }
  arg1 = reinterpret_cast< swig::SwigPyIterator * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "SwigPyIterator_decr" "', argument " "2"" of type '" "size_t""'");
  } 
  arg2 = static_cast< size_t >(val2);
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)(arg1)->decr(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  } catch(swig::stop_iteration &_e) {
    {
      (void)_e;
      SWIG_SetErrorObj(PyExc_StopIteration, SWIG_Py_Void());
      SWIG_fail;
    }
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_SwigPyIterator_decr__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  swig::SwigPyIterator *arg1 = (swig::SwigPyIterator *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_swig__SwigPyIterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "SwigPyIterator_decr" "', argument " "1"" of type '" "swig::SwigPyIterator *""'"); 
  }
  arg1 = reinterpret_cast< swig::SwigPyIterator * >(argp1);
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)(arg1)->decr();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  } catch(swig::stop_iteration &_e) {
    {
      (void)_e;
      SWIG_SetErrorObj(PyExc_StopIteration, SWIG_Py_Void());
      SWIG_fail;
    }
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_SwigPyIterator_decr(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "SwigPyIterator_decr", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    void *vptr = 0;
    int res = SWIG_ConvertPtr(argv[0], &vptr, SWIGTYPE_p_swig__SwigPyIterator, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_SwigPyIterator_decr__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    void *vptr = 0;
    int res = SWIG_ConvertPtr(argv[0], &vptr, SWIGTYPE_p_swig__SwigPyIterator, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_SwigPyIterator_decr__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'SwigPyIterator_decr'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    swig::SwigPyIterator::decr(size_t)\n"
    "    swig::SwigPyIterator::decr()\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_SwigPyIterator_distance(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  swig::SwigPyIterator *arg1 = (swig::SwigPyIterator *) 0 ;
  swig::SwigPyIterator *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  ptrdiff_t result;
  
  if (!SWIG_Python_UnpackTuple(args, "SwigPyIterator_distance", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_swig__SwigPyIterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "SwigPyIterator_distance" "', argument " "1"" of type '" "swig::SwigPyIterator const *""'"); 
  }
  arg1 = reinterpret_cast< swig::SwigPyIterator * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_swig__SwigPyIterator,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "SwigPyIterator_distance" "', argument " "2"" of type '" "swig::SwigPyIterator const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "SwigPyIterator_distance" "', argument " "2"" of type '" "swig::SwigPyIterator const &""'"); 
  }
  arg2 = reinterpret_cast< swig::SwigPyIterator * >(argp2);
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((swig::SwigPyIterator const *)arg1)->distance((swig::SwigPyIterator const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  } catch(std::invalid_argument &_e) {
    SWIG_Python_Raise(SWIG_NewPointerObj((new std::invalid_argument(static_cast< const std::invalid_argument& >(_e))),SWIGTYPE_p_std__invalid_argument,SWIG_POINTER_OWN), "std::invalid_argument", SWIGTYPE_p_std__invalid_argument); SWIG_fail;
  }
  resultobj = SWIG_From_ptrdiff_t(static_cast< ptrdiff_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_SwigPyIterator_equal(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  swig::SwigPyIterator *arg1 = (swig::SwigPyIterator *) 0 ;
  swig::SwigPyIterator *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "SwigPyIterator_equal", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_swig__SwigPyIterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "SwigPyIterator_equal" "', argument " "1"" of type '" "swig::SwigPyIterator const *""'"); 
  }
  arg1 = reinterpret_cast< swig::SwigPyIterator * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_swig__SwigPyIterator,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "SwigPyIterator_equal" "', argument " "2"" of type '" "swig::SwigPyIterator const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "SwigPyIterator_equal" "', argument " "2"" of type '" "swig::SwigPyIterator const &""'"); 
  }
  arg2 = reinterpret_cast< swig::SwigPyIterator * >(argp2);
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((swig::SwigPyIterator const *)arg1)->equal((swig::SwigPyIterator const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  } catch(std::invalid_argument &_e) {
    SWIG_Python_Raise(SWIG_NewPointerObj((new std::invalid_argument(static_cast< const std::invalid_argument& >(_e))),SWIGTYPE_p_std__invalid_argument,SWIG_POINTER_OWN), "std::invalid_argument", SWIGTYPE_p_std__invalid_argument); SWIG_fail;
  }
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_SwigPyIterator_copy(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  swig::SwigPyIterator *arg1 = (swig::SwigPyIterator *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_swig__SwigPyIterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "SwigPyIterator_copy" "', argument " "1"" of type '" "swig::SwigPyIterator const *""'"); 
  }
  arg1 = reinterpret_cast< swig::SwigPyIterator * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (swig::SwigPyIterator *)((swig::SwigPyIterator const *)arg1)->copy();
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_SwigPyIterator_next(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  swig::SwigPyIterator *arg1 = (swig::SwigPyIterator *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_swig__SwigPyIterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "SwigPyIterator_next" "', argument " "1"" of type '" "swig::SwigPyIterator *""'"); 
  }
  arg1 = reinterpret_cast< swig::SwigPyIterator * >(argp1);
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)(arg1)->next();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  } catch(swig::stop_iteration &_e) {
    {
      (void)_e;
      SWIG_SetErrorObj(PyExc_StopIteration, SWIG_Py_Void());
      SWIG_fail;
    }
  }
  resultobj = result;
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_SwigPyIterator___next__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  swig::SwigPyIterator *arg1 = (swig::SwigPyIterator *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_swig__SwigPyIterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "SwigPyIterator___next__" "', argument " "1"" of type '" "swig::SwigPyIterator *""'"); 
  }
  arg1 = reinterpret_cast< swig::SwigPyIterator * >(argp1);
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)(arg1)->__next__();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  } catch(swig::stop_iteration &_e) {
    {
      (void)_e;
      SWIG_SetErrorObj(PyExc_StopIteration, SWIG_Py_Void());
      SWIG_fail;
    }
  }
  resultobj = result;
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_SwigPyIterator_previous(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  swig::SwigPyIterator *arg1 = (swig::SwigPyIterator *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_swig__SwigPyIterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "SwigPyIterator_previous" "', argument " "1"" of type '" "swig::SwigPyIterator *""'"); 
  }
  arg1 = reinterpret_cast< swig::SwigPyIterator * >(argp1);
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)(arg1)->previous();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  } catch(swig::stop_iteration &_e) {
    {
      (void)_e;
      SWIG_SetErrorObj(PyExc_StopIteration, SWIG_Py_Void());
      SWIG_fail;
    }
  }
  resultobj = result;
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_SwigPyIterator_advance(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  swig::SwigPyIterator *arg1 = (swig::SwigPyIterator *) 0 ;
  ptrdiff_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  swig::SwigPyIterator *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "SwigPyIterator_advance", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_swig__SwigPyIterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "SwigPyIterator_advance" "', argument " "1"" of type '" "swig::SwigPyIterator *""'"); 
  }
  arg1 = reinterpret_cast< swig::SwigPyIterator * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "SwigPyIterator_advance" "', argument " "2"" of type '" "ptrdiff_t""'");
  } 
  arg2 = static_cast< ptrdiff_t >(val2);
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)(arg1)->advance(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  } catch(swig::stop_iteration &_e) {
    {
      (void)_e;
      SWIG_SetErrorObj(PyExc_StopIteration, SWIG_Py_Void());
      SWIG_fail;
    }
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_SwigPyIterator___eq__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  swig::SwigPyIterator *arg1 = (swig::SwigPyIterator *) 0 ;
  swig::SwigPyIterator *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "SwigPyIterator___eq__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_swig__SwigPyIterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "SwigPyIterator___eq__" "', argument " "1"" of type '" "swig::SwigPyIterator const *""'"); 
  }
  arg1 = reinterpret_cast< swig::SwigPyIterator * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_swig__SwigPyIterator,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "SwigPyIterator___eq__" "', argument " "2"" of type '" "swig::SwigPyIterator const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "SwigPyIterator___eq__" "', argument " "2"" of type '" "swig::SwigPyIterator const &""'"); 
  }
  arg2 = reinterpret_cast< swig::SwigPyIterator * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (bool)((swig::SwigPyIterator const *)arg1)->operator ==((swig::SwigPyIterator const &)*arg2);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  PyErr_Clear();
  Py_INCREF(Py_NotImplemented);
  return Py_NotImplemented;
}


SWIGINTERN PyObject *_wrap_SwigPyIterator___ne__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  swig::SwigPyIterator *arg1 = (swig::SwigPyIterator *) 0 ;
  swig::SwigPyIterator *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "SwigPyIterator___ne__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_swig__SwigPyIterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "SwigPyIterator___ne__" "', argument " "1"" of type '" "swig::SwigPyIterator const *""'"); 
  }
  arg1 = reinterpret_cast< swig::SwigPyIterator * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_swig__SwigPyIterator,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "SwigPyIterator___ne__" "', argument " "2"" of type '" "swig::SwigPyIterator const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "SwigPyIterator___ne__" "', argument " "2"" of type '" "swig::SwigPyIterator const &""'"); 
  }
  arg2 = reinterpret_cast< swig::SwigPyIterator * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (bool)((swig::SwigPyIterator const *)arg1)->operator !=((swig::SwigPyIterator const &)*arg2);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  PyErr_Clear();
  Py_INCREF(Py_NotImplemented);
  return Py_NotImplemented;
}


SWIGINTERN PyObject *_wrap_SwigPyIterator___iadd__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  swig::SwigPyIterator *arg1 = (swig::SwigPyIterator *) 0 ;
  ptrdiff_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  swig::SwigPyIterator *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "SwigPyIterator___iadd__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "SwigPyIterator___iadd__" "', argument " "1"" of type '" "swig::SwigPyIterator *""'"); 
  }
  arg1 = reinterpret_cast< swig::SwigPyIterator * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "SwigPyIterator___iadd__" "', argument " "2"" of type '" "ptrdiff_t""'");
  } 
  arg2 = static_cast< ptrdiff_t >(val2);
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *) &(arg1)->operator +=(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  } catch(swig::stop_iteration &_e) {
    {
      (void)_e;
      SWIG_SetErrorObj(PyExc_StopIteration, SWIG_Py_Void());
      SWIG_fail;
    }
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_SwigPyIterator___isub__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  swig::SwigPyIterator *arg1 = (swig::SwigPyIterator *) 0 ;
  ptrdiff_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  swig::SwigPyIterator *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "SwigPyIterator___isub__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "SwigPyIterator___isub__" "', argument " "1"" of type '" "swig::SwigPyIterator *""'"); 
  }
  arg1 = reinterpret_cast< swig::SwigPyIterator * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "SwigPyIterator___isub__" "', argument " "2"" of type '" "ptrdiff_t""'");
  } 
  arg2 = static_cast< ptrdiff_t >(val2);
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *) &(arg1)->operator -=(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  } catch(swig::stop_iteration &_e) {
    {
      (void)_e;
      SWIG_SetErrorObj(PyExc_StopIteration, SWIG_Py_Void());
      SWIG_fail;
    }
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_SwigPyIterator___add__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  swig::SwigPyIterator *arg1 = (swig::SwigPyIterator *) 0 ;
  ptrdiff_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  swig::SwigPyIterator *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "SwigPyIterator___add__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_swig__SwigPyIterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "SwigPyIterator___add__" "', argument " "1"" of type '" "swig::SwigPyIterator const *""'"); 
  }
  arg1 = reinterpret_cast< swig::SwigPyIterator * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "SwigPyIterator___add__" "', argument " "2"" of type '" "ptrdiff_t""'");
  } 
  arg2 = static_cast< ptrdiff_t >(val2);
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)((swig::SwigPyIterator const *)arg1)->operator +(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  } catch(swig::stop_iteration &_e) {
    {
      (void)_e;
      SWIG_SetErrorObj(PyExc_StopIteration, SWIG_Py_Void());
      SWIG_fail;
    }
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  PyErr_Clear();
  Py_INCREF(Py_NotImplemented);
  return Py_NotImplemented;
}


SWIGINTERN PyObject *_wrap_SwigPyIterator___sub____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  swig::SwigPyIterator *arg1 = (swig::SwigPyIterator *) 0 ;
  ptrdiff_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  swig::SwigPyIterator *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_swig__SwigPyIterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "SwigPyIterator___sub__" "', argument " "1"" of type '" "swig::SwigPyIterator const *""'"); 
  }
  arg1 = reinterpret_cast< swig::SwigPyIterator * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "SwigPyIterator___sub__" "', argument " "2"" of type '" "ptrdiff_t""'");
  } 
  arg2 = static_cast< ptrdiff_t >(val2);
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)((swig::SwigPyIterator const *)arg1)->operator -(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  } catch(swig::stop_iteration &_e) {
    {
      (void)_e;
      SWIG_SetErrorObj(PyExc_StopIteration, SWIG_Py_Void());
      SWIG_fail;
    }
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  PyErr_Clear();
  Py_INCREF(Py_NotImplemented);
  return Py_NotImplemented;
}


SWIGINTERN PyObject *_wrap_SwigPyIterator___sub____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  swig::SwigPyIterator *arg1 = (swig::SwigPyIterator *) 0 ;
  swig::SwigPyIterator *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  ptrdiff_t result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_swig__SwigPyIterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "SwigPyIterator___sub__" "', argument " "1"" of type '" "swig::SwigPyIterator const *""'"); 
  }
  arg1 = reinterpret_cast< swig::SwigPyIterator * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_swig__SwigPyIterator,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "SwigPyIterator___sub__" "', argument " "2"" of type '" "swig::SwigPyIterator const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "SwigPyIterator___sub__" "', argument " "2"" of type '" "swig::SwigPyIterator const &""'"); 
  }
  arg2 = reinterpret_cast< swig::SwigPyIterator * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = ((swig::SwigPyIterator const *)arg1)->operator -((swig::SwigPyIterator const &)*arg2);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_ptrdiff_t(static_cast< ptrdiff_t >(result));
  return resultobj;
fail:
  PyErr_Clear();
  Py_INCREF(Py_NotImplemented);
  return Py_NotImplemented;
}


SWIGINTERN PyObject *_wrap_SwigPyIterator___sub__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "SwigPyIterator___sub__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    void *vptr = 0;
    int res = SWIG_ConvertPtr(argv[0], &vptr, SWIGTYPE_p_swig__SwigPyIterator, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_NO_NULL | 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_SwigPyIterator___sub____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    void *vptr = 0;
    int res = SWIG_ConvertPtr(argv[0], &vptr, SWIGTYPE_p_swig__SwigPyIterator, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_SwigPyIterator___sub____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  Py_INCREF(Py_NotImplemented);
  return Py_NotImplemented;
}


SWIGINTERN PyObject *SwigPyIterator_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_swig__SwigPyIterator, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_new_FilesystemPath(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  PyObject *swig_obj[1] ;
  boost::filesystem::path *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_FilesystemPath" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_FilesystemPath" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::filesystem::path *)new boost::filesystem::path((std::string const &)*arg1);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_boost__filesystem__path, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_FilesystemPath_string(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::filesystem::path *arg1 = (boost::filesystem::path *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_boost__filesystem__path, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "FilesystemPath_string" "', argument " "1"" of type '" "boost::filesystem::path *""'"); 
  }
  arg1 = reinterpret_cast< boost::filesystem::path * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (arg1)->string();
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_FilesystemPath(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::filesystem::path *arg1 = (boost::filesystem::path *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_boost__filesystem__path, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_FilesystemPath" "', argument " "1"" of type '" "boost::filesystem::path *""'"); 
  }
  arg1 = reinterpret_cast< boost::filesystem::path * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *FilesystemPath_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__filesystem__path, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *FilesystemPath_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN int Swig_var_MessageFlags_ROUTING_INFO_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageFlags_ROUTING_INFO is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageFlags_ROUTING_INFO_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageFlags_ROUTING_INFO));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageFlags_ENDPOINT_INFO_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageFlags_ENDPOINT_INFO is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageFlags_ENDPOINT_INFO_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageFlags_ENDPOINT_INFO));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageFlags_PRIORITY_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageFlags_PRIORITY is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageFlags_PRIORITY_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageFlags_PRIORITY));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageFlags_UNRELIABLE_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageFlags_UNRELIABLE is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageFlags_UNRELIABLE_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageFlags_UNRELIABLE));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageFlags_META_INFO_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageFlags_META_INFO is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageFlags_META_INFO_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageFlags_META_INFO));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageFlags_STRING_TABLE_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageFlags_STRING_TABLE is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageFlags_STRING_TABLE_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageFlags_STRING_TABLE));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageFlags_MULTIPLE_ENTRIES_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageFlags_MULTIPLE_ENTRIES is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageFlags_MULTIPLE_ENTRIES_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageFlags_MULTIPLE_ENTRIES));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageFlags_EXTENDED_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageFlags_EXTENDED is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageFlags_EXTENDED_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageFlags_EXTENDED));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageFlags_Version2Compat_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageFlags_Version2Compat is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageFlags_Version2Compat_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageFlags_Version2Compat));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageEntryFlags_SERVICE_PATH_STR_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageEntryFlags_SERVICE_PATH_STR is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageEntryFlags_SERVICE_PATH_STR_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageEntryFlags_SERVICE_PATH_STR));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageEntryFlags_SERVICE_PATH_CODE_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageEntryFlags_SERVICE_PATH_CODE is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageEntryFlags_SERVICE_PATH_CODE_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageEntryFlags_SERVICE_PATH_CODE));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageEntryFlags_MEMBER_NAME_STR_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageEntryFlags_MEMBER_NAME_STR is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageEntryFlags_MEMBER_NAME_STR_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageEntryFlags_MEMBER_NAME_STR));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageEntryFlags_MEMBER_NAME_CODE_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageEntryFlags_MEMBER_NAME_CODE is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageEntryFlags_MEMBER_NAME_CODE_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageEntryFlags_MEMBER_NAME_CODE));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageEntryFlags_REQUEST_ID_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageEntryFlags_REQUEST_ID is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageEntryFlags_REQUEST_ID_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageEntryFlags_REQUEST_ID));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageEntryFlags_ERROR_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageEntryFlags_ERROR is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageEntryFlags_ERROR_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageEntryFlags_ERROR));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageEntryFlags_META_INFO_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageEntryFlags_META_INFO is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageEntryFlags_META_INFO_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageEntryFlags_META_INFO));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageEntryFlags_EXTENDED_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageEntryFlags_EXTENDED is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageEntryFlags_EXTENDED_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageEntryFlags_EXTENDED));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageEntryFlags_Version2Compat_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageEntryFlags_Version2Compat is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageEntryFlags_Version2Compat_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageEntryFlags_Version2Compat));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageElementFlags_ELEMENT_NAME_STR_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageElementFlags_ELEMENT_NAME_STR is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageElementFlags_ELEMENT_NAME_STR_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageElementFlags_ELEMENT_NAME_STR));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageElementFlags_ELEMENT_NAME_CODE_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageElementFlags_ELEMENT_NAME_CODE is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageElementFlags_ELEMENT_NAME_CODE_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageElementFlags_ELEMENT_NAME_CODE));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageElementFlags_ELEMENT_NUMBER_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageElementFlags_ELEMENT_NUMBER is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageElementFlags_ELEMENT_NUMBER_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageElementFlags_ELEMENT_NUMBER));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageElementFlags_ELEMENT_TYPE_NAME_STR_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageElementFlags_ELEMENT_TYPE_NAME_STR is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageElementFlags_ELEMENT_TYPE_NAME_STR_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageElementFlags_ELEMENT_TYPE_NAME_STR));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageElementFlags_ELEMENT_TYPE_NAME_CODE_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageElementFlags_ELEMENT_TYPE_NAME_CODE is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageElementFlags_ELEMENT_TYPE_NAME_CODE_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageElementFlags_ELEMENT_TYPE_NAME_CODE));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageElementFlags_META_INFO_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageElementFlags_META_INFO is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageElementFlags_META_INFO_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageElementFlags_META_INFO));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageElementFlags_EXTENDED_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageElementFlags_EXTENDED is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageElementFlags_EXTENDED_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageElementFlags_EXTENDED));
  return pyobj;
}


SWIGINTERN int Swig_var_MessageElementFlags_Version2Compat_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable MessageElementFlags_Version2Compat is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_MessageElementFlags_Version2Compat_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(RobotRaconteur::MessageElementFlags_Version2Compat));
  return pyobj;
}


SWIGINTERN int Swig_var_TranspartCapabilityCode_PAGE_MASK_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable TranspartCapabilityCode_PAGE_MASK is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_TranspartCapabilityCode_PAGE_MASK_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(RobotRaconteur::TranspartCapabilityCode_PAGE_MASK));
  return pyobj;
}


SWIGINTERN int Swig_var_TransportCapabilityCode_MESSAGE2_BASIC_PAGE_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable TransportCapabilityCode_MESSAGE2_BASIC_PAGE is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_TransportCapabilityCode_MESSAGE2_BASIC_PAGE_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(RobotRaconteur::TransportCapabilityCode_MESSAGE2_BASIC_PAGE));
  return pyobj;
}


SWIGINTERN int Swig_var_TransportCapabilityCode_MESSAGE2_BASIC_ENABLE_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable TransportCapabilityCode_MESSAGE2_BASIC_ENABLE is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_TransportCapabilityCode_MESSAGE2_BASIC_ENABLE_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(RobotRaconteur::TransportCapabilityCode_MESSAGE2_BASIC_ENABLE));
  return pyobj;
}


SWIGINTERN int Swig_var_TransportCapabilityCode_MESSAGE2_BASIC_CONNECTCOMBINED_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable TransportCapabilityCode_MESSAGE2_BASIC_CONNECTCOMBINED is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_TransportCapabilityCode_MESSAGE2_BASIC_CONNECTCOMBINED_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(RobotRaconteur::TransportCapabilityCode_MESSAGE2_BASIC_CONNECTCOMBINED));
  return pyobj;
}


SWIGINTERN int Swig_var_TransportCapabilityCode_MESSAGE4_BASIC_PAGE_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable TransportCapabilityCode_MESSAGE4_BASIC_PAGE is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_TransportCapabilityCode_MESSAGE4_BASIC_PAGE_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(RobotRaconteur::TransportCapabilityCode_MESSAGE4_BASIC_PAGE));
  return pyobj;
}


SWIGINTERN int Swig_var_TransportCapabilityCode_MESSAGE4_BASIC_ENABLE_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable TransportCapabilityCode_MESSAGE4_BASIC_ENABLE is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_TransportCapabilityCode_MESSAGE4_BASIC_ENABLE_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(RobotRaconteur::TransportCapabilityCode_MESSAGE4_BASIC_ENABLE));
  return pyobj;
}


SWIGINTERN int Swig_var_TransportCapabilityCode_MESSAGE4_BASIC_CONNECTCOMBINED_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable TransportCapabilityCode_MESSAGE4_BASIC_CONNECTCOMBINED is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_TransportCapabilityCode_MESSAGE4_BASIC_CONNECTCOMBINED_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(RobotRaconteur::TransportCapabilityCode_MESSAGE4_BASIC_CONNECTCOMBINED));
  return pyobj;
}


SWIGINTERN int Swig_var_TransportCapabilityCode_MESSAGE4_STRINGTABLE_PAGE_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable TransportCapabilityCode_MESSAGE4_STRINGTABLE_PAGE is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_TransportCapabilityCode_MESSAGE4_STRINGTABLE_PAGE_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(RobotRaconteur::TransportCapabilityCode_MESSAGE4_STRINGTABLE_PAGE));
  return pyobj;
}


SWIGINTERN int Swig_var_TransportCapabilityCode_MESSAGE4_STRINGTABLE_ENABLE_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable TransportCapabilityCode_MESSAGE4_STRINGTABLE_ENABLE is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_TransportCapabilityCode_MESSAGE4_STRINGTABLE_ENABLE_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(RobotRaconteur::TransportCapabilityCode_MESSAGE4_STRINGTABLE_ENABLE));
  return pyobj;
}


SWIGINTERN int Swig_var_TransportCapabilityCode_MESSAGE4_STRINGTABLE_MESSAGE_LOCAL_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable TransportCapabilityCode_MESSAGE4_STRINGTABLE_MESSAGE_LOCAL is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_TransportCapabilityCode_MESSAGE4_STRINGTABLE_MESSAGE_LOCAL_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(RobotRaconteur::TransportCapabilityCode_MESSAGE4_STRINGTABLE_MESSAGE_LOCAL));
  return pyobj;
}


SWIGINTERN int Swig_var_TransportCapabilityCode_MESSAGE4_STRINGTABLE_STANDARD_TABLE_set(PyObject *) {
  SWIG_Error(SWIG_AttributeError,"Variable TransportCapabilityCode_MESSAGE4_STRINGTABLE_STANDARD_TABLE is read-only.");
  return 1;
}


SWIGINTERN PyObject *Swig_var_TransportCapabilityCode_MESSAGE4_STRINGTABLE_STANDARD_TABLE_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(RobotRaconteur::TransportCapabilityCode_MESSAGE4_STRINGTABLE_STANDARD_TABLE));
  return pyobj;
}


SWIGINTERN PyObject *_wrap_vectorstring_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_iterator" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_std_string_Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring___nonzero__" "', argument " "1"" of type '" "std::vector< std::string > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_std_string_Sg____nonzero__((std::vector< std::string > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring___bool__" "', argument " "1"" of type '" "std::vector< std::string > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_std_string_Sg____bool__((std::vector< std::string > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring___len__" "', argument " "1"" of type '" "std::vector< std::string > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_std_string_Sg____len__((std::vector< std::string > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  std::vector< std::string >::difference_type arg2 ;
  std::vector< std::string >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< std::string,std::allocator< std::string > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorstring___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring___getslice__" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorstring___getslice__" "', argument " "2"" of type '" "std::vector< std::string >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< std::string >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorstring___getslice__" "', argument " "3"" of type '" "std::vector< std::string >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< std::string >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< std::string,std::allocator< std::string > > *)std_vector_Sl_std_string_Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  std::vector< std::string >::difference_type arg2 ;
  std::vector< std::string >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring___setslice__" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorstring___setslice__" "', argument " "2"" of type '" "std::vector< std::string >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< std::string >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorstring___setslice__" "', argument " "3"" of type '" "std::vector< std::string >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< std::string >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_std_string_Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  std::vector< std::string >::difference_type arg2 ;
  std::vector< std::string >::difference_type arg3 ;
  std::vector< std::string,std::allocator< std::string > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring___setslice__" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorstring___setslice__" "', argument " "2"" of type '" "std::vector< std::string >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< std::string >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorstring___setslice__" "', argument " "3"" of type '" "std::vector< std::string >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< std::string >::difference_type >(val3);
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorstring___setslice__" "', argument " "4"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorstring___setslice__" "', argument " "4"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_std_string_Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< std::string,std::allocator< std::string > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorstring___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< std::string,std::allocator< std::string > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vectorstring___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< std::string,std::allocator< std::string > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< std::string,std::allocator< std::string > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorstring___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorstring___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< std::string >::__setslice__(std::vector< std::string >::difference_type,std::vector< std::string >::difference_type)\n"
    "    std::vector< std::string >::__setslice__(std::vector< std::string >::difference_type,std::vector< std::string >::difference_type,std::vector< std::string,std::allocator< std::string > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorstring___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  std::vector< std::string >::difference_type arg2 ;
  std::vector< std::string >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorstring___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring___delslice__" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorstring___delslice__" "', argument " "2"" of type '" "std::vector< std::string >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< std::string >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorstring___delslice__" "', argument " "3"" of type '" "std::vector< std::string >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< std::string >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_std_string_Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  std::vector< std::string >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring___delitem__" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorstring___delitem__" "', argument " "2"" of type '" "std::vector< std::string >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< std::string >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_std_string_Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< std::string,std::allocator< std::string > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring___getitem__" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorstring___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< std::string,std::allocator< std::string > > *)std_vector_Sl_std_string_Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring___setitem__" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorstring___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorstring___setitem__" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorstring___setitem__" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_std_string_Sg____setitem____SWIG_0(arg1,arg2,(std::vector< std::string,std::allocator< std::string > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring___setitem__" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorstring___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_std_string_Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring___delitem__" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorstring___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_std_string_Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorstring___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< std::string,std::allocator< std::string > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorstring___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< std::string,std::allocator< std::string > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorstring___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorstring___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< std::string >::__delitem__(std::vector< std::string >::difference_type)\n"
    "    std::vector< std::string >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorstring___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  std::vector< std::string >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< std::string >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring___getitem__" "', argument " "1"" of type '" "std::vector< std::string > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorstring___getitem__" "', argument " "2"" of type '" "std::vector< std::string >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< std::string >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< std::string >::value_type *) &std_vector_Sl_std_string_Sg____getitem____SWIG_1((std::vector< std::string > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  (void)swig::container_owner<swig::traits<std::vector< std::string >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorstring___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< std::string,std::allocator< std::string > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorstring___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< std::string,std::allocator< std::string > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorstring___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorstring___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< std::string >::__getitem__(PySliceObject *)\n"
    "    std::vector< std::string >::__getitem__(std::vector< std::string >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorstring___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  std::vector< std::string >::difference_type arg2 ;
  std::vector< std::string >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring___setitem__" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorstring___setitem__" "', argument " "2"" of type '" "std::vector< std::string >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< std::string >::difference_type >(val2);
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorstring___setitem__" "', argument " "3"" of type '" "std::vector< std::string >::value_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorstring___setitem__" "', argument " "3"" of type '" "std::vector< std::string >::value_type const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_std_string_Sg____setitem____SWIG_2(arg1,arg2,(std::string const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorstring___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< std::string,std::allocator< std::string > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorstring___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< std::string,std::allocator< std::string > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< std::string,std::allocator< std::string > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorstring___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< std::string,std::allocator< std::string > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorstring___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorstring___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< std::string >::__setitem__(PySliceObject *,std::vector< std::string,std::allocator< std::string > > const &)\n"
    "    std::vector< std::string >::__setitem__(PySliceObject *)\n"
    "    std::vector< std::string >::__setitem__(std::vector< std::string >::difference_type,std::vector< std::string >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorstring_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_pop" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = std_vector_Sl_std_string_Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  std::vector< std::string >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorstring_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_append" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorstring_append" "', argument " "2"" of type '" "std::vector< std::string >::value_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorstring_append" "', argument " "2"" of type '" "std::vector< std::string >::value_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_std_string_Sg__append(arg1,(std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorstring__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< std::string > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< std::string > *)new std::vector< std::string >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorstring__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< std::string > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vectorstring" "', argument " "1"" of type '" "std::vector< std::string > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vectorstring" "', argument " "1"" of type '" "std::vector< std::string > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< std::string > *)new std::vector< std::string >((std::vector< std::string > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_empty" "', argument " "1"" of type '" "std::vector< std::string > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< std::string > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_size" "', argument " "1"" of type '" "std::vector< std::string > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< std::string > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  std::vector< std::string > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorstring_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_swap" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorstring_swap" "', argument " "2"" of type '" "std::vector< std::string > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorstring_swap" "', argument " "2"" of type '" "std::vector< std::string > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< std::string > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_begin" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< std::string >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_end" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< std::string >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_rbegin" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< std::string >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_rend" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< std::string >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_clear" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< std::string > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_get_allocator" "', argument " "1"" of type '" "std::vector< std::string > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< std::string > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< std::string >::allocator_type(static_cast< const std::vector< std::string >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_std__string_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorstring__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< std::string >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< std::string > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorstring" "', argument " "1"" of type '" "std::vector< std::string >::size_type""'");
  } 
  arg1 = static_cast< std::vector< std::string >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< std::string > *)new std::vector< std::string >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_pop_back" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  std::vector< std::string >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_resize" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorstring_resize" "', argument " "2"" of type '" "std::vector< std::string >::size_type""'");
  } 
  arg2 = static_cast< std::vector< std::string >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  std::vector< std::string >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< std::string >::iterator result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_erase" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorstring_erase" "', argument " "2"" of type '" "std::vector< std::string >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< std::string >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< std::string >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorstring_erase" "', argument " "2"" of type '" "std::vector< std::string >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_std_string_Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< std::string >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  std::vector< std::string >::iterator arg2 ;
  std::vector< std::string >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  std::vector< std::string >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_erase" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorstring_erase" "', argument " "2"" of type '" "std::vector< std::string >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< std::string >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< std::string >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorstring_erase" "', argument " "2"" of type '" "std::vector< std::string >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorstring_erase" "', argument " "3"" of type '" "std::vector< std::string >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< std::string >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< std::string >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorstring_erase" "', argument " "3"" of type '" "std::vector< std::string >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_std_string_Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< std::string >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorstring_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< std::string,std::allocator< std::string > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< std::string >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vectorstring_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< std::string,std::allocator< std::string > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< std::string >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< std::string >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vectorstring_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorstring_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< std::string >::erase(std::vector< std::string >::iterator)\n"
    "    std::vector< std::string >::erase(std::vector< std::string >::iterator,std::vector< std::string >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vectorstring__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< std::string >::size_type arg1 ;
  std::vector< std::string >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  std::vector< std::string > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorstring" "', argument " "1"" of type '" "std::vector< std::string >::size_type""'");
  } 
  arg1 = static_cast< std::vector< std::string >::size_type >(val1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_vectorstring" "', argument " "2"" of type '" "std::vector< std::string >::value_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vectorstring" "', argument " "2"" of type '" "std::vector< std::string >::value_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< std::string > *)new std::vector< std::string >(arg1,(std::vector< std::string >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorstring(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vectorstring", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vectorstring__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vectorstring__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< std::string,std::allocator< std::string > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vectorstring__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_vectorstring__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vectorstring'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< std::string >::vector()\n"
    "    std::vector< std::string >::vector(std::vector< std::string > const &)\n"
    "    std::vector< std::string >::vector(std::vector< std::string >::size_type)\n"
    "    std::vector< std::string >::vector(std::vector< std::string >::size_type,std::vector< std::string >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorstring_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  std::vector< std::string >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorstring_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_push_back" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorstring_push_back" "', argument " "2"" of type '" "std::vector< std::string >::value_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorstring_push_back" "', argument " "2"" of type '" "std::vector< std::string >::value_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< std::string >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_front" "', argument " "1"" of type '" "std::vector< std::string > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< std::string >::value_type *) &((std::vector< std::string > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  (void)swig::container_owner<swig::traits<std::vector< std::string >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_back" "', argument " "1"" of type '" "std::vector< std::string > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< std::string >::value_type *) &((std::vector< std::string > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  (void)swig::container_owner<swig::traits<std::vector< std::string >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  std::vector< std::string >::size_type arg2 ;
  std::vector< std::string >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorstring_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_assign" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorstring_assign" "', argument " "2"" of type '" "std::vector< std::string >::size_type""'");
  } 
  arg2 = static_cast< std::vector< std::string >::size_type >(val2);
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorstring_assign" "', argument " "3"" of type '" "std::vector< std::string >::value_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorstring_assign" "', argument " "3"" of type '" "std::vector< std::string >::value_type const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< std::string >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  std::vector< std::string >::size_type arg2 ;
  std::vector< std::string >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_resize" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorstring_resize" "', argument " "2"" of type '" "std::vector< std::string >::size_type""'");
  } 
  arg2 = static_cast< std::vector< std::string >::size_type >(val2);
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorstring_resize" "', argument " "3"" of type '" "std::vector< std::string >::value_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorstring_resize" "', argument " "3"" of type '" "std::vector< std::string >::value_type const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< std::string >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorstring_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< std::string,std::allocator< std::string > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorstring_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< std::string,std::allocator< std::string > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorstring_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorstring_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< std::string >::resize(std::vector< std::string >::size_type)\n"
    "    std::vector< std::string >::resize(std::vector< std::string >::size_type,std::vector< std::string >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorstring_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  std::vector< std::string >::iterator arg2 ;
  std::vector< std::string >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  int res3 = SWIG_OLDOBJ ;
  std::vector< std::string >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_insert" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorstring_insert" "', argument " "2"" of type '" "std::vector< std::string >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< std::string >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< std::string >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorstring_insert" "', argument " "2"" of type '" "std::vector< std::string >::iterator""'");
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorstring_insert" "', argument " "3"" of type '" "std::vector< std::string >::value_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorstring_insert" "', argument " "3"" of type '" "std::vector< std::string >::value_type const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_std_string_Sg__insert__SWIG_0(arg1,arg2,(std::string const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< std::string >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  std::vector< std::string >::iterator arg2 ;
  std::vector< std::string >::size_type arg3 ;
  std::vector< std::string >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_insert" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorstring_insert" "', argument " "2"" of type '" "std::vector< std::string >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< std::string >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< std::string >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorstring_insert" "', argument " "2"" of type '" "std::vector< std::string >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorstring_insert" "', argument " "3"" of type '" "std::vector< std::string >::size_type""'");
  } 
  arg3 = static_cast< std::vector< std::string >::size_type >(val3);
  {
    std::string *ptr = (std::string *)0;
    res4 = SWIG_AsPtr_std_string(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorstring_insert" "', argument " "4"" of type '" "std::vector< std::string >::value_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorstring_insert" "', argument " "4"" of type '" "std::vector< std::string >::value_type const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_std_string_Sg__insert__SWIG_1(arg1,arg2,arg3,(std::string const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorstring_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< std::string,std::allocator< std::string > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< std::string >::iterator > *>(iter) != 0));
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorstring_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< std::string,std::allocator< std::string > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< std::string >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = SWIG_AsPtr_std_string(argv[3], (std::string**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorstring_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorstring_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< std::string >::insert(std::vector< std::string >::iterator,std::vector< std::string >::value_type const &)\n"
    "    std::vector< std::string >::insert(std::vector< std::string >::iterator,std::vector< std::string >::size_type,std::vector< std::string >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorstring_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  std::vector< std::string >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorstring_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_reserve" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorstring_reserve" "', argument " "2"" of type '" "std::vector< std::string >::size_type""'");
  } 
  arg2 = static_cast< std::vector< std::string >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorstring_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorstring_capacity" "', argument " "1"" of type '" "std::vector< std::string > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< std::string > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vectorstring(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string > *arg1 = (std::vector< std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vectorstring" "', argument " "1"" of type '" "std::vector< std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vectorstring_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vectorstring_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_vectorint32_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_iterator" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_int32_t_Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32___nonzero__" "', argument " "1"" of type '" "std::vector< int32_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_int32_t_Sg____nonzero__((std::vector< int > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32___bool__" "', argument " "1"" of type '" "std::vector< int32_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_int32_t_Sg____bool__((std::vector< int > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< int >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32___len__" "', argument " "1"" of type '" "std::vector< int32_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_int32_t_Sg____len__((std::vector< int > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  std::vector< int >::difference_type arg2 ;
  std::vector< int >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< int,std::allocator< int > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorint32___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32___getslice__" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorint32___getslice__" "', argument " "2"" of type '" "std::vector< int >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< int >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorint32___getslice__" "', argument " "3"" of type '" "std::vector< int >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< int >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< int,std::allocator< int > > *)std_vector_Sl_int32_t_Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  std::vector< int >::difference_type arg2 ;
  std::vector< int >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32___setslice__" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorint32___setslice__" "', argument " "2"" of type '" "std::vector< int >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< int >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorint32___setslice__" "', argument " "3"" of type '" "std::vector< int >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< int >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_int32_t_Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  std::vector< int >::difference_type arg2 ;
  std::vector< int >::difference_type arg3 ;
  std::vector< int,std::allocator< int > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32___setslice__" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorint32___setslice__" "', argument " "2"" of type '" "std::vector< int >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< int >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorint32___setslice__" "', argument " "3"" of type '" "std::vector< int >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< int >::difference_type >(val3);
  {
    std::vector< int,std::allocator< int > > *ptr = (std::vector< int,std::allocator< int > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorint32___setslice__" "', argument " "4"" of type '" "std::vector< int,std::allocator< int > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorint32___setslice__" "', argument " "4"" of type '" "std::vector< int,std::allocator< int > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_int32_t_Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< int,std::allocator< int > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorint32___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< int,std::allocator< int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vectorint32___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< int,std::allocator< int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< int,std::allocator< int > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorint32___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorint32___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< int32_t >::__setslice__(std::vector< int >::difference_type,std::vector< int >::difference_type)\n"
    "    std::vector< int32_t >::__setslice__(std::vector< int >::difference_type,std::vector< int >::difference_type,std::vector< int,std::allocator< int > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorint32___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  std::vector< int >::difference_type arg2 ;
  std::vector< int >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorint32___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32___delslice__" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorint32___delslice__" "', argument " "2"" of type '" "std::vector< int >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< int >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorint32___delslice__" "', argument " "3"" of type '" "std::vector< int >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< int >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_int32_t_Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  std::vector< int >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32___delitem__" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorint32___delitem__" "', argument " "2"" of type '" "std::vector< int >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< int >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_int32_t_Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< int,std::allocator< int > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32___getitem__" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorint32___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< int,std::allocator< int > > *)std_vector_Sl_int32_t_Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< int,std::allocator< int > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32___setitem__" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorint32___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< int,std::allocator< int > > *ptr = (std::vector< int,std::allocator< int > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorint32___setitem__" "', argument " "3"" of type '" "std::vector< int,std::allocator< int > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorint32___setitem__" "', argument " "3"" of type '" "std::vector< int,std::allocator< int > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_int32_t_Sg____setitem____SWIG_0(arg1,arg2,(std::vector< int,std::allocator< int > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32___setitem__" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorint32___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_int32_t_Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32___delitem__" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorint32___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_int32_t_Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorint32___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< int,std::allocator< int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorint32___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< int,std::allocator< int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorint32___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorint32___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< int32_t >::__delitem__(std::vector< int >::difference_type)\n"
    "    std::vector< int32_t >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorint32___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  std::vector< int >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< int >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32___getitem__" "', argument " "1"" of type '" "std::vector< int32_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorint32___getitem__" "', argument " "2"" of type '" "std::vector< int >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< int >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< int >::value_type *) &std_vector_Sl_int32_t_Sg____getitem____SWIG_1((std::vector< int > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(*result));
  (void)swig::container_owner<swig::traits<std::vector< int >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorint32___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< int,std::allocator< int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorint32___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< int,std::allocator< int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorint32___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorint32___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< int32_t >::__getitem__(PySliceObject *)\n"
    "    std::vector< int32_t >::__getitem__(std::vector< int >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorint32___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  std::vector< int >::difference_type arg2 ;
  std::vector< int >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< int >::value_type temp3 ;
  int val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32___setitem__" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorint32___setitem__" "', argument " "2"" of type '" "std::vector< int >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< int >::difference_type >(val2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorint32___setitem__" "', argument " "3"" of type '" "std::vector< int >::value_type""'");
  } 
  temp3 = static_cast< std::vector< int >::value_type >(val3);
  arg3 = &temp3;
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_int32_t_Sg____setitem____SWIG_2(arg1,arg2,(int const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorint32___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< int,std::allocator< int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorint32___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< int,std::allocator< int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< int,std::allocator< int > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorint32___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< int,std::allocator< int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_int(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vectorint32___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorint32___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< int32_t >::__setitem__(PySliceObject *,std::vector< int,std::allocator< int > > const &)\n"
    "    std::vector< int32_t >::__setitem__(PySliceObject *)\n"
    "    std::vector< int32_t >::__setitem__(std::vector< int >::difference_type,std::vector< int >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorint32_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< int >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_pop" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< int >::value_type)std_vector_Sl_int32_t_Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  std::vector< int >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< int >::value_type temp2 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorint32_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_append" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorint32_append" "', argument " "2"" of type '" "std::vector< int >::value_type""'");
  } 
  temp2 = static_cast< std::vector< int >::value_type >(val2);
  arg2 = &temp2;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_int32_t_Sg__append(arg1,(int const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorint32__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< int32_t > *)new std::vector< int32_t >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorint32__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< int32_t > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< int,std::allocator< int > > *ptr = (std::vector< int,std::allocator< int > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vectorint32" "', argument " "1"" of type '" "std::vector< int32_t > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vectorint32" "', argument " "1"" of type '" "std::vector< int32_t > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< int32_t > *)new std::vector< int32_t >((std::vector< int32_t > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_empty" "', argument " "1"" of type '" "std::vector< int32_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< int32_t > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< int >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_size" "', argument " "1"" of type '" "std::vector< int32_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< int32_t > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  std::vector< int32_t > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorint32_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_swap" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorint32_swap" "', argument " "2"" of type '" "std::vector< int32_t > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorint32_swap" "', argument " "2"" of type '" "std::vector< int32_t > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< int32_t > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::vector< int >::iterator > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_begin" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< int >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::vector< int >::iterator > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_end" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< int >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::vector< int >::reverse_iterator > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_rbegin" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< int >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::vector< int >::reverse_iterator > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_rend" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< int >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_clear" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< int > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_get_allocator" "', argument " "1"" of type '" "std::vector< int32_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< int32_t > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< int >::allocator_type(static_cast< const std::vector< int >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_int_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorint32__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< int32_t > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorint32" "', argument " "1"" of type '" "std::vector< int >::size_type""'");
  } 
  arg1 = static_cast< std::vector< int >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< int32_t > *)new std::vector< int32_t >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_pop_back" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  std::vector< int >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_resize" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorint32_resize" "', argument " "2"" of type '" "std::vector< int >::size_type""'");
  } 
  arg2 = static_cast< std::vector< int >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  SwigValueWrapper< std::vector< int >::iterator > arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  SwigValueWrapper< std::vector< int >::iterator > result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_erase" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorint32_erase" "', argument " "2"" of type '" "std::vector< int >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< int >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< int >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorint32_erase" "', argument " "2"" of type '" "std::vector< int >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_int32_t_Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< int >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  SwigValueWrapper< std::vector< int >::iterator > arg2 ;
  SwigValueWrapper< std::vector< int >::iterator > arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  SwigValueWrapper< std::vector< int >::iterator > result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_erase" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorint32_erase" "', argument " "2"" of type '" "std::vector< int >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< int >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< int >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorint32_erase" "', argument " "2"" of type '" "std::vector< int >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorint32_erase" "', argument " "3"" of type '" "std::vector< int >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< int >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< int >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorint32_erase" "', argument " "3"" of type '" "std::vector< int >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_int32_t_Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< int >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorint32_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< int,std::allocator< int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< int >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vectorint32_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< int,std::allocator< int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< int >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< int >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vectorint32_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorint32_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< int32_t >::erase(std::vector< int >::iterator)\n"
    "    std::vector< int32_t >::erase(std::vector< int >::iterator,std::vector< int >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vectorint32__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int >::size_type arg1 ;
  std::vector< int >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< int >::value_type temp2 ;
  int val2 ;
  int ecode2 = 0 ;
  std::vector< int32_t > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorint32" "', argument " "1"" of type '" "std::vector< int >::size_type""'");
  } 
  arg1 = static_cast< std::vector< int >::size_type >(val1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "new_vectorint32" "', argument " "2"" of type '" "std::vector< int >::value_type""'");
  } 
  temp2 = static_cast< std::vector< int >::value_type >(val2);
  arg2 = &temp2;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< int32_t > *)new std::vector< int32_t >(arg1,(std::vector< int >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorint32(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vectorint32", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vectorint32__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vectorint32__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< int,std::allocator< int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vectorint32__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_new_vectorint32__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vectorint32'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< int32_t >::vector()\n"
    "    std::vector< int32_t >::vector(std::vector< int32_t > const &)\n"
    "    std::vector< int32_t >::vector(std::vector< int >::size_type)\n"
    "    std::vector< int32_t >::vector(std::vector< int >::size_type,std::vector< int >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorint32_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  std::vector< int >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< int >::value_type temp2 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorint32_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_push_back" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorint32_push_back" "', argument " "2"" of type '" "std::vector< int >::value_type""'");
  } 
  temp2 = static_cast< std::vector< int >::value_type >(val2);
  arg2 = &temp2;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< int >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< int >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_front" "', argument " "1"" of type '" "std::vector< int32_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< int >::value_type *) &((std::vector< int32_t > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(*result));
  (void)swig::container_owner<swig::traits<std::vector< int >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< int >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_back" "', argument " "1"" of type '" "std::vector< int32_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< int >::value_type *) &((std::vector< int32_t > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(*result));
  (void)swig::container_owner<swig::traits<std::vector< int >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  std::vector< int >::size_type arg2 ;
  std::vector< int >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  std::vector< int >::value_type temp3 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorint32_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_assign" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorint32_assign" "', argument " "2"" of type '" "std::vector< int >::size_type""'");
  } 
  arg2 = static_cast< std::vector< int >::size_type >(val2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorint32_assign" "', argument " "3"" of type '" "std::vector< int >::value_type""'");
  } 
  temp3 = static_cast< std::vector< int >::value_type >(val3);
  arg3 = &temp3;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< int >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  std::vector< int >::size_type arg2 ;
  std::vector< int >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  std::vector< int >::value_type temp3 ;
  int val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_resize" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorint32_resize" "', argument " "2"" of type '" "std::vector< int >::size_type""'");
  } 
  arg2 = static_cast< std::vector< int >::size_type >(val2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorint32_resize" "', argument " "3"" of type '" "std::vector< int >::value_type""'");
  } 
  temp3 = static_cast< std::vector< int >::value_type >(val3);
  arg3 = &temp3;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< int >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorint32_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< int,std::allocator< int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorint32_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< int,std::allocator< int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_int(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vectorint32_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorint32_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< int32_t >::resize(std::vector< int >::size_type)\n"
    "    std::vector< int32_t >::resize(std::vector< int >::size_type,std::vector< int >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorint32_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  SwigValueWrapper< std::vector< int >::iterator > arg2 ;
  std::vector< int >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< int >::value_type temp3 ;
  int val3 ;
  int ecode3 = 0 ;
  SwigValueWrapper< std::vector< int >::iterator > result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_insert" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorint32_insert" "', argument " "2"" of type '" "std::vector< int >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< int >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< int >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorint32_insert" "', argument " "2"" of type '" "std::vector< int >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorint32_insert" "', argument " "3"" of type '" "std::vector< int >::value_type""'");
  } 
  temp3 = static_cast< std::vector< int >::value_type >(val3);
  arg3 = &temp3;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_int32_t_Sg__insert__SWIG_0(arg1,arg2,(int const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< int >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  SwigValueWrapper< std::vector< int >::iterator > arg2 ;
  std::vector< int >::size_type arg3 ;
  std::vector< int >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  std::vector< int >::value_type temp4 ;
  int val4 ;
  int ecode4 = 0 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_insert" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorint32_insert" "', argument " "2"" of type '" "std::vector< int >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< int >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< int >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorint32_insert" "', argument " "2"" of type '" "std::vector< int >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorint32_insert" "', argument " "3"" of type '" "std::vector< int >::size_type""'");
  } 
  arg3 = static_cast< std::vector< int >::size_type >(val3);
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "vectorint32_insert" "', argument " "4"" of type '" "std::vector< int >::value_type""'");
  } 
  temp4 = static_cast< std::vector< int >::value_type >(val4);
  arg4 = &temp4;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_int32_t_Sg__insert__SWIG_1(arg1,arg2,arg3,(int const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorint32_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< int,std::allocator< int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< int >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_int(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vectorint32_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< int,std::allocator< int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< int >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          {
            int res = SWIG_AsVal_int(argv[3], NULL);
            _v = SWIG_CheckState(res);
          }
          if (_v) {
            return _wrap_vectorint32_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorint32_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< int32_t >::insert(std::vector< int >::iterator,std::vector< int >::value_type const &)\n"
    "    std::vector< int32_t >::insert(std::vector< int >::iterator,std::vector< int >::size_type,std::vector< int >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorint32_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  std::vector< int >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorint32_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_reserve" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorint32_reserve" "', argument " "2"" of type '" "std::vector< int >::size_type""'");
  } 
  arg2 = static_cast< std::vector< int >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorint32_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< int >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorint32_capacity" "', argument " "1"" of type '" "std::vector< int32_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< int32_t > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vectorint32(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int32_t > *arg1 = (std::vector< int32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vectorint32" "', argument " "1"" of type '" "std::vector< int32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vectorint32_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vectorint32_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_vectoruint32_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_iterator" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_uint32_t_Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32___nonzero__" "', argument " "1"" of type '" "std::vector< uint32_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_uint32_t_Sg____nonzero__((std::vector< unsigned int > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32___bool__" "', argument " "1"" of type '" "std::vector< uint32_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_uint32_t_Sg____bool__((std::vector< unsigned int > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< unsigned int >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32___len__" "', argument " "1"" of type '" "std::vector< uint32_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_uint32_t_Sg____len__((std::vector< unsigned int > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  std::vector< unsigned int >::difference_type arg2 ;
  std::vector< unsigned int >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< unsigned int,std::allocator< unsigned int > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectoruint32___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32___getslice__" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectoruint32___getslice__" "', argument " "2"" of type '" "std::vector< unsigned int >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned int >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectoruint32___getslice__" "', argument " "3"" of type '" "std::vector< unsigned int >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< unsigned int >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< unsigned int,std::allocator< unsigned int > > *)std_vector_Sl_uint32_t_Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  std::vector< unsigned int >::difference_type arg2 ;
  std::vector< unsigned int >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32___setslice__" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectoruint32___setslice__" "', argument " "2"" of type '" "std::vector< unsigned int >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned int >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectoruint32___setslice__" "', argument " "3"" of type '" "std::vector< unsigned int >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< unsigned int >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint32_t_Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  std::vector< unsigned int >::difference_type arg2 ;
  std::vector< unsigned int >::difference_type arg3 ;
  std::vector< unsigned int,std::allocator< unsigned int > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32___setslice__" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectoruint32___setslice__" "', argument " "2"" of type '" "std::vector< unsigned int >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned int >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectoruint32___setslice__" "', argument " "3"" of type '" "std::vector< unsigned int >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< unsigned int >::difference_type >(val3);
  {
    std::vector< unsigned int,std::allocator< unsigned int > > *ptr = (std::vector< unsigned int,std::allocator< unsigned int > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectoruint32___setslice__" "', argument " "4"" of type '" "std::vector< unsigned int,std::allocator< unsigned int > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectoruint32___setslice__" "', argument " "4"" of type '" "std::vector< unsigned int,std::allocator< unsigned int > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint32_t_Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< unsigned int,std::allocator< unsigned int > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectoruint32___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned int,std::allocator< unsigned int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vectoruint32___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned int,std::allocator< unsigned int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< unsigned int,std::allocator< unsigned int > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectoruint32___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectoruint32___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint32_t >::__setslice__(std::vector< unsigned int >::difference_type,std::vector< unsigned int >::difference_type)\n"
    "    std::vector< uint32_t >::__setslice__(std::vector< unsigned int >::difference_type,std::vector< unsigned int >::difference_type,std::vector< unsigned int,std::allocator< unsigned int > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectoruint32___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  std::vector< unsigned int >::difference_type arg2 ;
  std::vector< unsigned int >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectoruint32___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32___delslice__" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectoruint32___delslice__" "', argument " "2"" of type '" "std::vector< unsigned int >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned int >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectoruint32___delslice__" "', argument " "3"" of type '" "std::vector< unsigned int >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< unsigned int >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint32_t_Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  std::vector< unsigned int >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32___delitem__" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectoruint32___delitem__" "', argument " "2"" of type '" "std::vector< unsigned int >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned int >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint32_t_Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< unsigned int,std::allocator< unsigned int > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32___getitem__" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectoruint32___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< unsigned int,std::allocator< unsigned int > > *)std_vector_Sl_uint32_t_Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< unsigned int,std::allocator< unsigned int > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32___setitem__" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectoruint32___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< unsigned int,std::allocator< unsigned int > > *ptr = (std::vector< unsigned int,std::allocator< unsigned int > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectoruint32___setitem__" "', argument " "3"" of type '" "std::vector< unsigned int,std::allocator< unsigned int > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectoruint32___setitem__" "', argument " "3"" of type '" "std::vector< unsigned int,std::allocator< unsigned int > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint32_t_Sg____setitem____SWIG_0(arg1,arg2,(std::vector< unsigned int,std::allocator< unsigned int > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32___setitem__" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectoruint32___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint32_t_Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32___delitem__" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectoruint32___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint32_t_Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectoruint32___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned int,std::allocator< unsigned int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectoruint32___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned int,std::allocator< unsigned int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectoruint32___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectoruint32___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint32_t >::__delitem__(std::vector< unsigned int >::difference_type)\n"
    "    std::vector< uint32_t >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectoruint32___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  std::vector< unsigned int >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< unsigned int >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32___getitem__" "', argument " "1"" of type '" "std::vector< uint32_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectoruint32___getitem__" "', argument " "2"" of type '" "std::vector< unsigned int >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned int >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< unsigned int >::value_type *) &std_vector_Sl_uint32_t_Sg____getitem____SWIG_1((std::vector< unsigned int > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(*result));
  (void)swig::container_owner<swig::traits<std::vector< unsigned int >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectoruint32___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned int,std::allocator< unsigned int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectoruint32___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned int,std::allocator< unsigned int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectoruint32___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectoruint32___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint32_t >::__getitem__(PySliceObject *)\n"
    "    std::vector< uint32_t >::__getitem__(std::vector< unsigned int >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectoruint32___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  std::vector< unsigned int >::difference_type arg2 ;
  std::vector< unsigned int >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< unsigned int >::value_type temp3 ;
  unsigned int val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32___setitem__" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectoruint32___setitem__" "', argument " "2"" of type '" "std::vector< unsigned int >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned int >::difference_type >(val2);
  ecode3 = SWIG_AsVal_unsigned_SS_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectoruint32___setitem__" "', argument " "3"" of type '" "std::vector< unsigned int >::value_type""'");
  } 
  temp3 = static_cast< std::vector< unsigned int >::value_type >(val3);
  arg3 = &temp3;
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint32_t_Sg____setitem____SWIG_2(arg1,arg2,(unsigned int const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectoruint32___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned int,std::allocator< unsigned int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectoruint32___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned int,std::allocator< unsigned int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< unsigned int,std::allocator< unsigned int > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectoruint32___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned int,std::allocator< unsigned int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_unsigned_SS_int(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vectoruint32___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectoruint32___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint32_t >::__setitem__(PySliceObject *,std::vector< unsigned int,std::allocator< unsigned int > > const &)\n"
    "    std::vector< uint32_t >::__setitem__(PySliceObject *)\n"
    "    std::vector< uint32_t >::__setitem__(std::vector< unsigned int >::difference_type,std::vector< unsigned int >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectoruint32_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< unsigned int >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_pop" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< unsigned int >::value_type)std_vector_Sl_uint32_t_Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  std::vector< unsigned int >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< unsigned int >::value_type temp2 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectoruint32_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_append" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectoruint32_append" "', argument " "2"" of type '" "std::vector< unsigned int >::value_type""'");
  } 
  temp2 = static_cast< std::vector< unsigned int >::value_type >(val2);
  arg2 = &temp2;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_uint32_t_Sg__append(arg1,(unsigned int const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectoruint32__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< uint32_t > *)new std::vector< uint32_t >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectoruint32__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< uint32_t > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< unsigned int,std::allocator< unsigned int > > *ptr = (std::vector< unsigned int,std::allocator< unsigned int > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vectoruint32" "', argument " "1"" of type '" "std::vector< uint32_t > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vectoruint32" "', argument " "1"" of type '" "std::vector< uint32_t > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< uint32_t > *)new std::vector< uint32_t >((std::vector< uint32_t > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_empty" "', argument " "1"" of type '" "std::vector< uint32_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< uint32_t > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< unsigned int >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_size" "', argument " "1"" of type '" "std::vector< uint32_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< uint32_t > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  std::vector< uint32_t > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectoruint32_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_swap" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectoruint32_swap" "', argument " "2"" of type '" "std::vector< uint32_t > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectoruint32_swap" "', argument " "2"" of type '" "std::vector< uint32_t > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< uint32_t > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::vector< unsigned int >::iterator > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_begin" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< unsigned int >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::vector< unsigned int >::iterator > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_end" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< unsigned int >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::vector< unsigned int >::reverse_iterator > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_rbegin" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< unsigned int >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::vector< unsigned int >::reverse_iterator > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_rend" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< unsigned int >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_clear" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< unsigned int > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_get_allocator" "', argument " "1"" of type '" "std::vector< uint32_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< uint32_t > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< unsigned int >::allocator_type(static_cast< const std::vector< unsigned int >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_unsigned_int_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectoruint32__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< unsigned int >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< uint32_t > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectoruint32" "', argument " "1"" of type '" "std::vector< unsigned int >::size_type""'");
  } 
  arg1 = static_cast< std::vector< unsigned int >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< uint32_t > *)new std::vector< uint32_t >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_pop_back" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  std::vector< unsigned int >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_resize" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectoruint32_resize" "', argument " "2"" of type '" "std::vector< unsigned int >::size_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned int >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  SwigValueWrapper< std::vector< unsigned int >::iterator > arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  SwigValueWrapper< std::vector< unsigned int >::iterator > result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_erase" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectoruint32_erase" "', argument " "2"" of type '" "std::vector< unsigned int >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< unsigned int >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned int >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectoruint32_erase" "', argument " "2"" of type '" "std::vector< unsigned int >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_uint32_t_Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< unsigned int >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  SwigValueWrapper< std::vector< unsigned int >::iterator > arg2 ;
  SwigValueWrapper< std::vector< unsigned int >::iterator > arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  SwigValueWrapper< std::vector< unsigned int >::iterator > result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_erase" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectoruint32_erase" "', argument " "2"" of type '" "std::vector< unsigned int >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< unsigned int >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned int >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectoruint32_erase" "', argument " "2"" of type '" "std::vector< unsigned int >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectoruint32_erase" "', argument " "3"" of type '" "std::vector< unsigned int >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< unsigned int >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned int >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectoruint32_erase" "', argument " "3"" of type '" "std::vector< unsigned int >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_uint32_t_Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< unsigned int >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectoruint32_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned int,std::allocator< unsigned int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned int >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vectoruint32_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned int,std::allocator< unsigned int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned int >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned int >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vectoruint32_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectoruint32_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint32_t >::erase(std::vector< unsigned int >::iterator)\n"
    "    std::vector< uint32_t >::erase(std::vector< unsigned int >::iterator,std::vector< unsigned int >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vectoruint32__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< unsigned int >::size_type arg1 ;
  std::vector< unsigned int >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< unsigned int >::value_type temp2 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  std::vector< uint32_t > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectoruint32" "', argument " "1"" of type '" "std::vector< unsigned int >::size_type""'");
  } 
  arg1 = static_cast< std::vector< unsigned int >::size_type >(val1);
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "new_vectoruint32" "', argument " "2"" of type '" "std::vector< unsigned int >::value_type""'");
  } 
  temp2 = static_cast< std::vector< unsigned int >::value_type >(val2);
  arg2 = &temp2;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< uint32_t > *)new std::vector< uint32_t >(arg1,(std::vector< unsigned int >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectoruint32(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vectoruint32", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vectoruint32__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vectoruint32__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned int,std::allocator< unsigned int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vectoruint32__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      {
        int res = SWIG_AsVal_unsigned_SS_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_new_vectoruint32__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vectoruint32'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint32_t >::vector()\n"
    "    std::vector< uint32_t >::vector(std::vector< uint32_t > const &)\n"
    "    std::vector< uint32_t >::vector(std::vector< unsigned int >::size_type)\n"
    "    std::vector< uint32_t >::vector(std::vector< unsigned int >::size_type,std::vector< unsigned int >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectoruint32_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  std::vector< unsigned int >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< unsigned int >::value_type temp2 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectoruint32_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_push_back" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectoruint32_push_back" "', argument " "2"" of type '" "std::vector< unsigned int >::value_type""'");
  } 
  temp2 = static_cast< std::vector< unsigned int >::value_type >(val2);
  arg2 = &temp2;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< unsigned int >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< unsigned int >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_front" "', argument " "1"" of type '" "std::vector< uint32_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< unsigned int >::value_type *) &((std::vector< uint32_t > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(*result));
  (void)swig::container_owner<swig::traits<std::vector< unsigned int >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< unsigned int >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_back" "', argument " "1"" of type '" "std::vector< uint32_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< unsigned int >::value_type *) &((std::vector< uint32_t > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(*result));
  (void)swig::container_owner<swig::traits<std::vector< unsigned int >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  std::vector< unsigned int >::size_type arg2 ;
  std::vector< unsigned int >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  std::vector< unsigned int >::value_type temp3 ;
  unsigned int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectoruint32_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_assign" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectoruint32_assign" "', argument " "2"" of type '" "std::vector< unsigned int >::size_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned int >::size_type >(val2);
  ecode3 = SWIG_AsVal_unsigned_SS_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectoruint32_assign" "', argument " "3"" of type '" "std::vector< unsigned int >::value_type""'");
  } 
  temp3 = static_cast< std::vector< unsigned int >::value_type >(val3);
  arg3 = &temp3;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< unsigned int >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  std::vector< unsigned int >::size_type arg2 ;
  std::vector< unsigned int >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  std::vector< unsigned int >::value_type temp3 ;
  unsigned int val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_resize" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectoruint32_resize" "', argument " "2"" of type '" "std::vector< unsigned int >::size_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned int >::size_type >(val2);
  ecode3 = SWIG_AsVal_unsigned_SS_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectoruint32_resize" "', argument " "3"" of type '" "std::vector< unsigned int >::value_type""'");
  } 
  temp3 = static_cast< std::vector< unsigned int >::value_type >(val3);
  arg3 = &temp3;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< unsigned int >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectoruint32_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned int,std::allocator< unsigned int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectoruint32_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned int,std::allocator< unsigned int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_unsigned_SS_int(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vectoruint32_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectoruint32_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint32_t >::resize(std::vector< unsigned int >::size_type)\n"
    "    std::vector< uint32_t >::resize(std::vector< unsigned int >::size_type,std::vector< unsigned int >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectoruint32_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  SwigValueWrapper< std::vector< unsigned int >::iterator > arg2 ;
  std::vector< unsigned int >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< unsigned int >::value_type temp3 ;
  unsigned int val3 ;
  int ecode3 = 0 ;
  SwigValueWrapper< std::vector< unsigned int >::iterator > result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_insert" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectoruint32_insert" "', argument " "2"" of type '" "std::vector< unsigned int >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< unsigned int >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned int >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectoruint32_insert" "', argument " "2"" of type '" "std::vector< unsigned int >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_unsigned_SS_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectoruint32_insert" "', argument " "3"" of type '" "std::vector< unsigned int >::value_type""'");
  } 
  temp3 = static_cast< std::vector< unsigned int >::value_type >(val3);
  arg3 = &temp3;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_uint32_t_Sg__insert__SWIG_0(arg1,arg2,(unsigned int const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< unsigned int >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  SwigValueWrapper< std::vector< unsigned int >::iterator > arg2 ;
  std::vector< unsigned int >::size_type arg3 ;
  std::vector< unsigned int >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  std::vector< unsigned int >::value_type temp4 ;
  unsigned int val4 ;
  int ecode4 = 0 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_insert" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectoruint32_insert" "', argument " "2"" of type '" "std::vector< unsigned int >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< unsigned int >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned int >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectoruint32_insert" "', argument " "2"" of type '" "std::vector< unsigned int >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectoruint32_insert" "', argument " "3"" of type '" "std::vector< unsigned int >::size_type""'");
  } 
  arg3 = static_cast< std::vector< unsigned int >::size_type >(val3);
  ecode4 = SWIG_AsVal_unsigned_SS_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "vectoruint32_insert" "', argument " "4"" of type '" "std::vector< unsigned int >::value_type""'");
  } 
  temp4 = static_cast< std::vector< unsigned int >::value_type >(val4);
  arg4 = &temp4;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_uint32_t_Sg__insert__SWIG_1(arg1,arg2,arg3,(unsigned int const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectoruint32_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned int,std::allocator< unsigned int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned int >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_unsigned_SS_int(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vectoruint32_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned int,std::allocator< unsigned int > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned int >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          {
            int res = SWIG_AsVal_unsigned_SS_int(argv[3], NULL);
            _v = SWIG_CheckState(res);
          }
          if (_v) {
            return _wrap_vectoruint32_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectoruint32_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint32_t >::insert(std::vector< unsigned int >::iterator,std::vector< unsigned int >::value_type const &)\n"
    "    std::vector< uint32_t >::insert(std::vector< unsigned int >::iterator,std::vector< unsigned int >::size_type,std::vector< unsigned int >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectoruint32_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  std::vector< unsigned int >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectoruint32_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_reserve" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectoruint32_reserve" "', argument " "2"" of type '" "std::vector< unsigned int >::size_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned int >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectoruint32_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< unsigned int >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectoruint32_capacity" "', argument " "1"" of type '" "std::vector< uint32_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< uint32_t > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vectoruint32(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t > *arg1 = (std::vector< uint32_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vectoruint32" "', argument " "1"" of type '" "std::vector< uint32_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint32_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vectoruint32_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vectoruint32_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_vector_int8_t_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_iterator" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_int8_t_Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t___nonzero__" "', argument " "1"" of type '" "std::vector< int8_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_int8_t_Sg____nonzero__((std::vector< signed char > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t___bool__" "', argument " "1"" of type '" "std::vector< int8_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_int8_t_Sg____bool__((std::vector< signed char > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< signed char >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t___len__" "', argument " "1"" of type '" "std::vector< int8_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_int8_t_Sg____len__((std::vector< signed char > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  std::vector< signed char >::difference_type arg2 ;
  std::vector< signed char >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< signed char,std::allocator< signed char > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_int8_t___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t___getslice__" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_int8_t___getslice__" "', argument " "2"" of type '" "std::vector< signed char >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< signed char >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_int8_t___getslice__" "', argument " "3"" of type '" "std::vector< signed char >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< signed char >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< signed char,std::allocator< signed char > > *)std_vector_Sl_int8_t_Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  std::vector< signed char >::difference_type arg2 ;
  std::vector< signed char >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t___setslice__" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_int8_t___setslice__" "', argument " "2"" of type '" "std::vector< signed char >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< signed char >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_int8_t___setslice__" "', argument " "3"" of type '" "std::vector< signed char >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< signed char >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_int8_t_Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  std::vector< signed char >::difference_type arg2 ;
  std::vector< signed char >::difference_type arg3 ;
  std::vector< signed char,std::allocator< signed char > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t___setslice__" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_int8_t___setslice__" "', argument " "2"" of type '" "std::vector< signed char >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< signed char >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_int8_t___setslice__" "', argument " "3"" of type '" "std::vector< signed char >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< signed char >::difference_type >(val3);
  {
    std::vector< signed char,std::allocator< signed char > > *ptr = (std::vector< signed char,std::allocator< signed char > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vector_int8_t___setslice__" "', argument " "4"" of type '" "std::vector< signed char,std::allocator< signed char > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_int8_t___setslice__" "', argument " "4"" of type '" "std::vector< signed char,std::allocator< signed char > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_int8_t_Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< signed char,std::allocator< signed char > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_int8_t___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< signed char,std::allocator< signed char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vector_int8_t___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< signed char,std::allocator< signed char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< signed char,std::allocator< signed char > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vector_int8_t___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_int8_t___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< int8_t >::__setslice__(std::vector< signed char >::difference_type,std::vector< signed char >::difference_type)\n"
    "    std::vector< int8_t >::__setslice__(std::vector< signed char >::difference_type,std::vector< signed char >::difference_type,std::vector< signed char,std::allocator< signed char > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_int8_t___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  std::vector< signed char >::difference_type arg2 ;
  std::vector< signed char >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_int8_t___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t___delslice__" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_int8_t___delslice__" "', argument " "2"" of type '" "std::vector< signed char >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< signed char >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_int8_t___delslice__" "', argument " "3"" of type '" "std::vector< signed char >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< signed char >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_int8_t_Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  std::vector< signed char >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t___delitem__" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_int8_t___delitem__" "', argument " "2"" of type '" "std::vector< signed char >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< signed char >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_int8_t_Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< signed char,std::allocator< signed char > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t___getitem__" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_int8_t___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< signed char,std::allocator< signed char > > *)std_vector_Sl_int8_t_Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< signed char,std::allocator< signed char > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t___setitem__" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_int8_t___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< signed char,std::allocator< signed char > > *ptr = (std::vector< signed char,std::allocator< signed char > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_int8_t___setitem__" "', argument " "3"" of type '" "std::vector< signed char,std::allocator< signed char > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_int8_t___setitem__" "', argument " "3"" of type '" "std::vector< signed char,std::allocator< signed char > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_int8_t_Sg____setitem____SWIG_0(arg1,arg2,(std::vector< signed char,std::allocator< signed char > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t___setitem__" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_int8_t___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_int8_t_Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t___delitem__" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_int8_t___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_int8_t_Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_int8_t___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< signed char,std::allocator< signed char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_int8_t___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< signed char,std::allocator< signed char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_int8_t___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_int8_t___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< int8_t >::__delitem__(std::vector< signed char >::difference_type)\n"
    "    std::vector< int8_t >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_int8_t___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  std::vector< signed char >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< signed char >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t___getitem__" "', argument " "1"" of type '" "std::vector< int8_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_int8_t___getitem__" "', argument " "2"" of type '" "std::vector< signed char >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< signed char >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< signed char >::value_type *) &std_vector_Sl_int8_t_Sg____getitem____SWIG_1((std::vector< signed char > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_signed_SS_char(static_cast< signed char >(*result));
  (void)swig::container_owner<swig::traits<std::vector< signed char >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_int8_t___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< signed char,std::allocator< signed char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_int8_t___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< signed char,std::allocator< signed char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_int8_t___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_int8_t___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< int8_t >::__getitem__(PySliceObject *)\n"
    "    std::vector< int8_t >::__getitem__(std::vector< signed char >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_int8_t___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  std::vector< signed char >::difference_type arg2 ;
  std::vector< signed char >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< signed char >::value_type temp3 ;
  signed char val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t___setitem__" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_int8_t___setitem__" "', argument " "2"" of type '" "std::vector< signed char >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< signed char >::difference_type >(val2);
  ecode3 = SWIG_AsVal_signed_SS_char(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_int8_t___setitem__" "', argument " "3"" of type '" "std::vector< signed char >::value_type""'");
  } 
  temp3 = static_cast< std::vector< signed char >::value_type >(val3);
  arg3 = &temp3;
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_int8_t_Sg____setitem____SWIG_2(arg1,arg2,(signed char const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_int8_t___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< signed char,std::allocator< signed char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_int8_t___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< signed char,std::allocator< signed char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< signed char,std::allocator< signed char > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vector_int8_t___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< signed char,std::allocator< signed char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_signed_SS_char(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vector_int8_t___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_int8_t___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< int8_t >::__setitem__(PySliceObject *,std::vector< signed char,std::allocator< signed char > > const &)\n"
    "    std::vector< int8_t >::__setitem__(PySliceObject *)\n"
    "    std::vector< int8_t >::__setitem__(std::vector< signed char >::difference_type,std::vector< signed char >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< signed char >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_pop" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< signed char >::value_type)std_vector_Sl_int8_t_Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_signed_SS_char(static_cast< signed char >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  std::vector< signed char >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< signed char >::value_type temp2 ;
  signed char val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_int8_t_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_append" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  ecode2 = SWIG_AsVal_signed_SS_char(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_int8_t_append" "', argument " "2"" of type '" "std::vector< signed char >::value_type""'");
  } 
  temp2 = static_cast< std::vector< signed char >::value_type >(val2);
  arg2 = &temp2;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_int8_t_Sg__append(arg1,(signed char const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_int8_t__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< int8_t > *)new std::vector< int8_t >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_int8_t__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< int8_t > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< signed char,std::allocator< signed char > > *ptr = (std::vector< signed char,std::allocator< signed char > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vector_int8_t" "', argument " "1"" of type '" "std::vector< int8_t > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vector_int8_t" "', argument " "1"" of type '" "std::vector< int8_t > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< int8_t > *)new std::vector< int8_t >((std::vector< int8_t > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_empty" "', argument " "1"" of type '" "std::vector< int8_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< int8_t > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< signed char >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_size" "', argument " "1"" of type '" "std::vector< int8_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< int8_t > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  std::vector< int8_t > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_int8_t_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_swap" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vector_int8_t_swap" "', argument " "2"" of type '" "std::vector< int8_t > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_int8_t_swap" "', argument " "2"" of type '" "std::vector< int8_t > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< int8_t > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::vector< signed char >::iterator > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_begin" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< signed char >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::vector< signed char >::iterator > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_end" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< signed char >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::vector< signed char >::reverse_iterator > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_rbegin" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< signed char >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::vector< signed char >::reverse_iterator > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_rend" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< signed char >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_clear" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< signed char > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_get_allocator" "', argument " "1"" of type '" "std::vector< int8_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< int8_t > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< signed char >::allocator_type(static_cast< const std::vector< signed char >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_signed_char_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_int8_t__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< signed char >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< int8_t > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vector_int8_t" "', argument " "1"" of type '" "std::vector< signed char >::size_type""'");
  } 
  arg1 = static_cast< std::vector< signed char >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< int8_t > *)new std::vector< int8_t >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_pop_back" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  std::vector< signed char >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_resize" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_int8_t_resize" "', argument " "2"" of type '" "std::vector< signed char >::size_type""'");
  } 
  arg2 = static_cast< std::vector< signed char >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  SwigValueWrapper< std::vector< signed char >::iterator > arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  SwigValueWrapper< std::vector< signed char >::iterator > result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_erase" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_int8_t_erase" "', argument " "2"" of type '" "std::vector< signed char >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< signed char >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< signed char >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_int8_t_erase" "', argument " "2"" of type '" "std::vector< signed char >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_int8_t_Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< signed char >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  SwigValueWrapper< std::vector< signed char >::iterator > arg2 ;
  SwigValueWrapper< std::vector< signed char >::iterator > arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  SwigValueWrapper< std::vector< signed char >::iterator > result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_erase" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_int8_t_erase" "', argument " "2"" of type '" "std::vector< signed char >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< signed char >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< signed char >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_int8_t_erase" "', argument " "2"" of type '" "std::vector< signed char >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_int8_t_erase" "', argument " "3"" of type '" "std::vector< signed char >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< signed char >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< signed char >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_int8_t_erase" "', argument " "3"" of type '" "std::vector< signed char >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_int8_t_Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< signed char >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_int8_t_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< signed char,std::allocator< signed char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< signed char >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vector_int8_t_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< signed char,std::allocator< signed char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< signed char >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< signed char >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vector_int8_t_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_int8_t_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< int8_t >::erase(std::vector< signed char >::iterator)\n"
    "    std::vector< int8_t >::erase(std::vector< signed char >::iterator,std::vector< signed char >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vector_int8_t__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< signed char >::size_type arg1 ;
  std::vector< signed char >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< signed char >::value_type temp2 ;
  signed char val2 ;
  int ecode2 = 0 ;
  std::vector< int8_t > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vector_int8_t" "', argument " "1"" of type '" "std::vector< signed char >::size_type""'");
  } 
  arg1 = static_cast< std::vector< signed char >::size_type >(val1);
  ecode2 = SWIG_AsVal_signed_SS_char(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "new_vector_int8_t" "', argument " "2"" of type '" "std::vector< signed char >::value_type""'");
  } 
  temp2 = static_cast< std::vector< signed char >::value_type >(val2);
  arg2 = &temp2;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< int8_t > *)new std::vector< int8_t >(arg1,(std::vector< signed char >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_int8_t(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vector_int8_t", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vector_int8_t__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vector_int8_t__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< signed char,std::allocator< signed char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vector_int8_t__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      {
        int res = SWIG_AsVal_signed_SS_char(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_new_vector_int8_t__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vector_int8_t'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< int8_t >::vector()\n"
    "    std::vector< int8_t >::vector(std::vector< int8_t > const &)\n"
    "    std::vector< int8_t >::vector(std::vector< signed char >::size_type)\n"
    "    std::vector< int8_t >::vector(std::vector< signed char >::size_type,std::vector< signed char >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  std::vector< signed char >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< signed char >::value_type temp2 ;
  signed char val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_int8_t_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_push_back" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  ecode2 = SWIG_AsVal_signed_SS_char(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_int8_t_push_back" "', argument " "2"" of type '" "std::vector< signed char >::value_type""'");
  } 
  temp2 = static_cast< std::vector< signed char >::value_type >(val2);
  arg2 = &temp2;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< signed char >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< signed char >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_front" "', argument " "1"" of type '" "std::vector< int8_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< signed char >::value_type *) &((std::vector< int8_t > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_signed_SS_char(static_cast< signed char >(*result));
  (void)swig::container_owner<swig::traits<std::vector< signed char >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< signed char >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_back" "', argument " "1"" of type '" "std::vector< int8_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< signed char >::value_type *) &((std::vector< int8_t > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_signed_SS_char(static_cast< signed char >(*result));
  (void)swig::container_owner<swig::traits<std::vector< signed char >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  std::vector< signed char >::size_type arg2 ;
  std::vector< signed char >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  std::vector< signed char >::value_type temp3 ;
  signed char val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_int8_t_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_assign" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_int8_t_assign" "', argument " "2"" of type '" "std::vector< signed char >::size_type""'");
  } 
  arg2 = static_cast< std::vector< signed char >::size_type >(val2);
  ecode3 = SWIG_AsVal_signed_SS_char(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_int8_t_assign" "', argument " "3"" of type '" "std::vector< signed char >::value_type""'");
  } 
  temp3 = static_cast< std::vector< signed char >::value_type >(val3);
  arg3 = &temp3;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< signed char >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  std::vector< signed char >::size_type arg2 ;
  std::vector< signed char >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  std::vector< signed char >::value_type temp3 ;
  signed char val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_resize" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_int8_t_resize" "', argument " "2"" of type '" "std::vector< signed char >::size_type""'");
  } 
  arg2 = static_cast< std::vector< signed char >::size_type >(val2);
  ecode3 = SWIG_AsVal_signed_SS_char(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_int8_t_resize" "', argument " "3"" of type '" "std::vector< signed char >::value_type""'");
  } 
  temp3 = static_cast< std::vector< signed char >::value_type >(val3);
  arg3 = &temp3;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< signed char >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_int8_t_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< signed char,std::allocator< signed char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_int8_t_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< signed char,std::allocator< signed char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_signed_SS_char(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vector_int8_t_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_int8_t_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< int8_t >::resize(std::vector< signed char >::size_type)\n"
    "    std::vector< int8_t >::resize(std::vector< signed char >::size_type,std::vector< signed char >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  SwigValueWrapper< std::vector< signed char >::iterator > arg2 ;
  std::vector< signed char >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< signed char >::value_type temp3 ;
  signed char val3 ;
  int ecode3 = 0 ;
  SwigValueWrapper< std::vector< signed char >::iterator > result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_insert" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_int8_t_insert" "', argument " "2"" of type '" "std::vector< signed char >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< signed char >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< signed char >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_int8_t_insert" "', argument " "2"" of type '" "std::vector< signed char >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_signed_SS_char(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_int8_t_insert" "', argument " "3"" of type '" "std::vector< signed char >::value_type""'");
  } 
  temp3 = static_cast< std::vector< signed char >::value_type >(val3);
  arg3 = &temp3;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_int8_t_Sg__insert__SWIG_0(arg1,arg2,(signed char const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< signed char >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  SwigValueWrapper< std::vector< signed char >::iterator > arg2 ;
  std::vector< signed char >::size_type arg3 ;
  std::vector< signed char >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  std::vector< signed char >::value_type temp4 ;
  signed char val4 ;
  int ecode4 = 0 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_insert" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_int8_t_insert" "', argument " "2"" of type '" "std::vector< signed char >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< signed char >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< signed char >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_int8_t_insert" "', argument " "2"" of type '" "std::vector< signed char >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_int8_t_insert" "', argument " "3"" of type '" "std::vector< signed char >::size_type""'");
  } 
  arg3 = static_cast< std::vector< signed char >::size_type >(val3);
  ecode4 = SWIG_AsVal_signed_SS_char(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "vector_int8_t_insert" "', argument " "4"" of type '" "std::vector< signed char >::value_type""'");
  } 
  temp4 = static_cast< std::vector< signed char >::value_type >(val4);
  arg4 = &temp4;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_int8_t_Sg__insert__SWIG_1(arg1,arg2,arg3,(signed char const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_int8_t_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< signed char,std::allocator< signed char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< signed char >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_signed_SS_char(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vector_int8_t_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< signed char,std::allocator< signed char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< signed char >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          {
            int res = SWIG_AsVal_signed_SS_char(argv[3], NULL);
            _v = SWIG_CheckState(res);
          }
          if (_v) {
            return _wrap_vector_int8_t_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_int8_t_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< int8_t >::insert(std::vector< signed char >::iterator,std::vector< signed char >::value_type const &)\n"
    "    std::vector< int8_t >::insert(std::vector< signed char >::iterator,std::vector< signed char >::size_type,std::vector< signed char >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  std::vector< signed char >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_int8_t_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_reserve" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_int8_t_reserve" "', argument " "2"" of type '" "std::vector< signed char >::size_type""'");
  } 
  arg2 = static_cast< std::vector< signed char >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_int8_t_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< signed char >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_int8_t_capacity" "', argument " "1"" of type '" "std::vector< int8_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< int8_t > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vector_int8_t(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< int8_t > *arg1 = (std::vector< int8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vector_int8_t" "', argument " "1"" of type '" "std::vector< int8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< int8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vector_int8_t_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vector_int8_t_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_vector_uint8_t_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_iterator" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_uint8_t_Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t___nonzero__" "', argument " "1"" of type '" "std::vector< uint8_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_uint8_t_Sg____nonzero__((std::vector< unsigned char > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t___bool__" "', argument " "1"" of type '" "std::vector< uint8_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_uint8_t_Sg____bool__((std::vector< unsigned char > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< unsigned char >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t___len__" "', argument " "1"" of type '" "std::vector< uint8_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_uint8_t_Sg____len__((std::vector< unsigned char > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  std::vector< unsigned char >::difference_type arg2 ;
  std::vector< unsigned char >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< unsigned char,std::allocator< unsigned char > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_uint8_t___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t___getslice__" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint8_t___getslice__" "', argument " "2"" of type '" "std::vector< unsigned char >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned char >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_uint8_t___getslice__" "', argument " "3"" of type '" "std::vector< unsigned char >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< unsigned char >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< unsigned char,std::allocator< unsigned char > > *)std_vector_Sl_uint8_t_Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  std::vector< unsigned char >::difference_type arg2 ;
  std::vector< unsigned char >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t___setslice__" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint8_t___setslice__" "', argument " "2"" of type '" "std::vector< unsigned char >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned char >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_uint8_t___setslice__" "', argument " "3"" of type '" "std::vector< unsigned char >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< unsigned char >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint8_t_Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  std::vector< unsigned char >::difference_type arg2 ;
  std::vector< unsigned char >::difference_type arg3 ;
  std::vector< unsigned char,std::allocator< unsigned char > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t___setslice__" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint8_t___setslice__" "', argument " "2"" of type '" "std::vector< unsigned char >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned char >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_uint8_t___setslice__" "', argument " "3"" of type '" "std::vector< unsigned char >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< unsigned char >::difference_type >(val3);
  {
    std::vector< unsigned char,std::allocator< unsigned char > > *ptr = (std::vector< unsigned char,std::allocator< unsigned char > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vector_uint8_t___setslice__" "', argument " "4"" of type '" "std::vector< unsigned char,std::allocator< unsigned char > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_uint8_t___setslice__" "', argument " "4"" of type '" "std::vector< unsigned char,std::allocator< unsigned char > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint8_t_Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< unsigned char,std::allocator< unsigned char > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_uint8_t___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned char,std::allocator< unsigned char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vector_uint8_t___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned char,std::allocator< unsigned char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< unsigned char,std::allocator< unsigned char > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vector_uint8_t___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_uint8_t___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint8_t >::__setslice__(std::vector< unsigned char >::difference_type,std::vector< unsigned char >::difference_type)\n"
    "    std::vector< uint8_t >::__setslice__(std::vector< unsigned char >::difference_type,std::vector< unsigned char >::difference_type,std::vector< unsigned char,std::allocator< unsigned char > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  std::vector< unsigned char >::difference_type arg2 ;
  std::vector< unsigned char >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_uint8_t___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t___delslice__" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint8_t___delslice__" "', argument " "2"" of type '" "std::vector< unsigned char >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned char >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_uint8_t___delslice__" "', argument " "3"" of type '" "std::vector< unsigned char >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< unsigned char >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint8_t_Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  std::vector< unsigned char >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t___delitem__" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint8_t___delitem__" "', argument " "2"" of type '" "std::vector< unsigned char >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned char >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint8_t_Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< unsigned char,std::allocator< unsigned char > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t___getitem__" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint8_t___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< unsigned char,std::allocator< unsigned char > > *)std_vector_Sl_uint8_t_Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< unsigned char,std::allocator< unsigned char > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t___setitem__" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint8_t___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< unsigned char,std::allocator< unsigned char > > *ptr = (std::vector< unsigned char,std::allocator< unsigned char > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_uint8_t___setitem__" "', argument " "3"" of type '" "std::vector< unsigned char,std::allocator< unsigned char > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_uint8_t___setitem__" "', argument " "3"" of type '" "std::vector< unsigned char,std::allocator< unsigned char > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint8_t_Sg____setitem____SWIG_0(arg1,arg2,(std::vector< unsigned char,std::allocator< unsigned char > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t___setitem__" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint8_t___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint8_t_Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t___delitem__" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint8_t___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint8_t_Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_uint8_t___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned char,std::allocator< unsigned char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_uint8_t___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned char,std::allocator< unsigned char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_uint8_t___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_uint8_t___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint8_t >::__delitem__(std::vector< unsigned char >::difference_type)\n"
    "    std::vector< uint8_t >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  std::vector< unsigned char >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< unsigned char >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t___getitem__" "', argument " "1"" of type '" "std::vector< uint8_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint8_t___getitem__" "', argument " "2"" of type '" "std::vector< unsigned char >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned char >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< unsigned char >::value_type *) &std_vector_Sl_uint8_t_Sg____getitem____SWIG_1((std::vector< unsigned char > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(*result));
  (void)swig::container_owner<swig::traits<std::vector< unsigned char >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_uint8_t___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned char,std::allocator< unsigned char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_uint8_t___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned char,std::allocator< unsigned char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_uint8_t___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_uint8_t___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint8_t >::__getitem__(PySliceObject *)\n"
    "    std::vector< uint8_t >::__getitem__(std::vector< unsigned char >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  std::vector< unsigned char >::difference_type arg2 ;
  std::vector< unsigned char >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< unsigned char >::value_type temp3 ;
  unsigned char val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t___setitem__" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint8_t___setitem__" "', argument " "2"" of type '" "std::vector< unsigned char >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned char >::difference_type >(val2);
  ecode3 = SWIG_AsVal_unsigned_SS_char(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_uint8_t___setitem__" "', argument " "3"" of type '" "std::vector< unsigned char >::value_type""'");
  } 
  temp3 = static_cast< std::vector< unsigned char >::value_type >(val3);
  arg3 = &temp3;
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint8_t_Sg____setitem____SWIG_2(arg1,arg2,(unsigned char const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_uint8_t___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned char,std::allocator< unsigned char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_uint8_t___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned char,std::allocator< unsigned char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< unsigned char,std::allocator< unsigned char > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vector_uint8_t___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned char,std::allocator< unsigned char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_unsigned_SS_char(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vector_uint8_t___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_uint8_t___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint8_t >::__setitem__(PySliceObject *,std::vector< unsigned char,std::allocator< unsigned char > > const &)\n"
    "    std::vector< uint8_t >::__setitem__(PySliceObject *)\n"
    "    std::vector< uint8_t >::__setitem__(std::vector< unsigned char >::difference_type,std::vector< unsigned char >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< unsigned char >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_pop" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< unsigned char >::value_type)std_vector_Sl_uint8_t_Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  std::vector< unsigned char >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< unsigned char >::value_type temp2 ;
  unsigned char val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_uint8_t_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_append" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_char(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint8_t_append" "', argument " "2"" of type '" "std::vector< unsigned char >::value_type""'");
  } 
  temp2 = static_cast< std::vector< unsigned char >::value_type >(val2);
  arg2 = &temp2;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_uint8_t_Sg__append(arg1,(unsigned char const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_uint8_t__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< uint8_t > *)new std::vector< uint8_t >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_uint8_t__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< uint8_t > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< unsigned char,std::allocator< unsigned char > > *ptr = (std::vector< unsigned char,std::allocator< unsigned char > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vector_uint8_t" "', argument " "1"" of type '" "std::vector< uint8_t > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vector_uint8_t" "', argument " "1"" of type '" "std::vector< uint8_t > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< uint8_t > *)new std::vector< uint8_t >((std::vector< uint8_t > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_empty" "', argument " "1"" of type '" "std::vector< uint8_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< uint8_t > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< unsigned char >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_size" "', argument " "1"" of type '" "std::vector< uint8_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< uint8_t > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  std::vector< uint8_t > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_uint8_t_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_swap" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vector_uint8_t_swap" "', argument " "2"" of type '" "std::vector< uint8_t > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_uint8_t_swap" "', argument " "2"" of type '" "std::vector< uint8_t > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< uint8_t > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::vector< unsigned char >::iterator > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_begin" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< unsigned char >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::vector< unsigned char >::iterator > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_end" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< unsigned char >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::vector< unsigned char >::reverse_iterator > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_rbegin" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< unsigned char >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::vector< unsigned char >::reverse_iterator > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_rend" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< unsigned char >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_clear" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< unsigned char > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_get_allocator" "', argument " "1"" of type '" "std::vector< uint8_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< uint8_t > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< unsigned char >::allocator_type(static_cast< const std::vector< unsigned char >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_unsigned_char_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_uint8_t__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< unsigned char >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< uint8_t > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vector_uint8_t" "', argument " "1"" of type '" "std::vector< unsigned char >::size_type""'");
  } 
  arg1 = static_cast< std::vector< unsigned char >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< uint8_t > *)new std::vector< uint8_t >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_pop_back" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  std::vector< unsigned char >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_resize" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint8_t_resize" "', argument " "2"" of type '" "std::vector< unsigned char >::size_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned char >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  SwigValueWrapper< std::vector< unsigned char >::iterator > arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  SwigValueWrapper< std::vector< unsigned char >::iterator > result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_erase" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint8_t_erase" "', argument " "2"" of type '" "std::vector< unsigned char >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< unsigned char >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned char >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint8_t_erase" "', argument " "2"" of type '" "std::vector< unsigned char >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_uint8_t_Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< unsigned char >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  SwigValueWrapper< std::vector< unsigned char >::iterator > arg2 ;
  SwigValueWrapper< std::vector< unsigned char >::iterator > arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  SwigValueWrapper< std::vector< unsigned char >::iterator > result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_erase" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint8_t_erase" "', argument " "2"" of type '" "std::vector< unsigned char >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< unsigned char >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned char >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint8_t_erase" "', argument " "2"" of type '" "std::vector< unsigned char >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint8_t_erase" "', argument " "3"" of type '" "std::vector< unsigned char >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< unsigned char >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned char >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint8_t_erase" "', argument " "3"" of type '" "std::vector< unsigned char >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_uint8_t_Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< unsigned char >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_uint8_t_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned char,std::allocator< unsigned char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned char >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vector_uint8_t_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned char,std::allocator< unsigned char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned char >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned char >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vector_uint8_t_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_uint8_t_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint8_t >::erase(std::vector< unsigned char >::iterator)\n"
    "    std::vector< uint8_t >::erase(std::vector< unsigned char >::iterator,std::vector< unsigned char >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vector_uint8_t__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< unsigned char >::size_type arg1 ;
  std::vector< unsigned char >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< unsigned char >::value_type temp2 ;
  unsigned char val2 ;
  int ecode2 = 0 ;
  std::vector< uint8_t > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vector_uint8_t" "', argument " "1"" of type '" "std::vector< unsigned char >::size_type""'");
  } 
  arg1 = static_cast< std::vector< unsigned char >::size_type >(val1);
  ecode2 = SWIG_AsVal_unsigned_SS_char(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "new_vector_uint8_t" "', argument " "2"" of type '" "std::vector< unsigned char >::value_type""'");
  } 
  temp2 = static_cast< std::vector< unsigned char >::value_type >(val2);
  arg2 = &temp2;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< uint8_t > *)new std::vector< uint8_t >(arg1,(std::vector< unsigned char >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_uint8_t(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vector_uint8_t", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vector_uint8_t__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vector_uint8_t__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned char,std::allocator< unsigned char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vector_uint8_t__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      {
        int res = SWIG_AsVal_unsigned_SS_char(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_new_vector_uint8_t__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vector_uint8_t'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint8_t >::vector()\n"
    "    std::vector< uint8_t >::vector(std::vector< uint8_t > const &)\n"
    "    std::vector< uint8_t >::vector(std::vector< unsigned char >::size_type)\n"
    "    std::vector< uint8_t >::vector(std::vector< unsigned char >::size_type,std::vector< unsigned char >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  std::vector< unsigned char >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< unsigned char >::value_type temp2 ;
  unsigned char val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_uint8_t_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_push_back" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_char(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint8_t_push_back" "', argument " "2"" of type '" "std::vector< unsigned char >::value_type""'");
  } 
  temp2 = static_cast< std::vector< unsigned char >::value_type >(val2);
  arg2 = &temp2;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< unsigned char >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< unsigned char >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_front" "', argument " "1"" of type '" "std::vector< uint8_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< unsigned char >::value_type *) &((std::vector< uint8_t > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(*result));
  (void)swig::container_owner<swig::traits<std::vector< unsigned char >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< unsigned char >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_back" "', argument " "1"" of type '" "std::vector< uint8_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< unsigned char >::value_type *) &((std::vector< uint8_t > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(*result));
  (void)swig::container_owner<swig::traits<std::vector< unsigned char >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  std::vector< unsigned char >::size_type arg2 ;
  std::vector< unsigned char >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  std::vector< unsigned char >::value_type temp3 ;
  unsigned char val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_uint8_t_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_assign" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint8_t_assign" "', argument " "2"" of type '" "std::vector< unsigned char >::size_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned char >::size_type >(val2);
  ecode3 = SWIG_AsVal_unsigned_SS_char(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_uint8_t_assign" "', argument " "3"" of type '" "std::vector< unsigned char >::value_type""'");
  } 
  temp3 = static_cast< std::vector< unsigned char >::value_type >(val3);
  arg3 = &temp3;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< unsigned char >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  std::vector< unsigned char >::size_type arg2 ;
  std::vector< unsigned char >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  std::vector< unsigned char >::value_type temp3 ;
  unsigned char val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_resize" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint8_t_resize" "', argument " "2"" of type '" "std::vector< unsigned char >::size_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned char >::size_type >(val2);
  ecode3 = SWIG_AsVal_unsigned_SS_char(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_uint8_t_resize" "', argument " "3"" of type '" "std::vector< unsigned char >::value_type""'");
  } 
  temp3 = static_cast< std::vector< unsigned char >::value_type >(val3);
  arg3 = &temp3;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< unsigned char >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_uint8_t_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned char,std::allocator< unsigned char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_uint8_t_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned char,std::allocator< unsigned char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_unsigned_SS_char(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vector_uint8_t_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_uint8_t_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint8_t >::resize(std::vector< unsigned char >::size_type)\n"
    "    std::vector< uint8_t >::resize(std::vector< unsigned char >::size_type,std::vector< unsigned char >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  SwigValueWrapper< std::vector< unsigned char >::iterator > arg2 ;
  std::vector< unsigned char >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< unsigned char >::value_type temp3 ;
  unsigned char val3 ;
  int ecode3 = 0 ;
  SwigValueWrapper< std::vector< unsigned char >::iterator > result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_insert" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint8_t_insert" "', argument " "2"" of type '" "std::vector< unsigned char >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< unsigned char >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned char >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint8_t_insert" "', argument " "2"" of type '" "std::vector< unsigned char >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_unsigned_SS_char(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_uint8_t_insert" "', argument " "3"" of type '" "std::vector< unsigned char >::value_type""'");
  } 
  temp3 = static_cast< std::vector< unsigned char >::value_type >(val3);
  arg3 = &temp3;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_uint8_t_Sg__insert__SWIG_0(arg1,arg2,(unsigned char const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< unsigned char >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  SwigValueWrapper< std::vector< unsigned char >::iterator > arg2 ;
  std::vector< unsigned char >::size_type arg3 ;
  std::vector< unsigned char >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  std::vector< unsigned char >::value_type temp4 ;
  unsigned char val4 ;
  int ecode4 = 0 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_insert" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint8_t_insert" "', argument " "2"" of type '" "std::vector< unsigned char >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< unsigned char >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned char >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint8_t_insert" "', argument " "2"" of type '" "std::vector< unsigned char >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_uint8_t_insert" "', argument " "3"" of type '" "std::vector< unsigned char >::size_type""'");
  } 
  arg3 = static_cast< std::vector< unsigned char >::size_type >(val3);
  ecode4 = SWIG_AsVal_unsigned_SS_char(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "vector_uint8_t_insert" "', argument " "4"" of type '" "std::vector< unsigned char >::value_type""'");
  } 
  temp4 = static_cast< std::vector< unsigned char >::value_type >(val4);
  arg4 = &temp4;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_uint8_t_Sg__insert__SWIG_1(arg1,arg2,arg3,(unsigned char const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_uint8_t_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned char,std::allocator< unsigned char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned char >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_unsigned_SS_char(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vector_uint8_t_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned char,std::allocator< unsigned char > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned char >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          {
            int res = SWIG_AsVal_unsigned_SS_char(argv[3], NULL);
            _v = SWIG_CheckState(res);
          }
          if (_v) {
            return _wrap_vector_uint8_t_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_uint8_t_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint8_t >::insert(std::vector< unsigned char >::iterator,std::vector< unsigned char >::value_type const &)\n"
    "    std::vector< uint8_t >::insert(std::vector< unsigned char >::iterator,std::vector< unsigned char >::size_type,std::vector< unsigned char >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  std::vector< unsigned char >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_uint8_t_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_reserve" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint8_t_reserve" "', argument " "2"" of type '" "std::vector< unsigned char >::size_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned char >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint8_t_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< unsigned char >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint8_t_capacity" "', argument " "1"" of type '" "std::vector< uint8_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< uint8_t > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vector_uint8_t(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint8_t > *arg1 = (std::vector< uint8_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vector_uint8_t" "', argument " "1"" of type '" "std::vector< uint8_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint8_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vector_uint8_t_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vector_uint8_t_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_vector_uint64_t_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_iterator" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_uint64_t_Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t___nonzero__" "', argument " "1"" of type '" "std::vector< uint64_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_uint64_t_Sg____nonzero__((std::vector< unsigned long > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t___bool__" "', argument " "1"" of type '" "std::vector< uint64_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_uint64_t_Sg____bool__((std::vector< unsigned long > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< unsigned long >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t___len__" "', argument " "1"" of type '" "std::vector< uint64_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_uint64_t_Sg____len__((std::vector< unsigned long > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  std::vector< unsigned long >::difference_type arg2 ;
  std::vector< unsigned long >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< unsigned long,std::allocator< unsigned long > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_uint64_t___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t___getslice__" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint64_t___getslice__" "', argument " "2"" of type '" "std::vector< unsigned long >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned long >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_uint64_t___getslice__" "', argument " "3"" of type '" "std::vector< unsigned long >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< unsigned long >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< unsigned long,std::allocator< unsigned long > > *)std_vector_Sl_uint64_t_Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  std::vector< unsigned long >::difference_type arg2 ;
  std::vector< unsigned long >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t___setslice__" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint64_t___setslice__" "', argument " "2"" of type '" "std::vector< unsigned long >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned long >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_uint64_t___setslice__" "', argument " "3"" of type '" "std::vector< unsigned long >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< unsigned long >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint64_t_Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  std::vector< unsigned long >::difference_type arg2 ;
  std::vector< unsigned long >::difference_type arg3 ;
  std::vector< unsigned long,std::allocator< unsigned long > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t___setslice__" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint64_t___setslice__" "', argument " "2"" of type '" "std::vector< unsigned long >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned long >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_uint64_t___setslice__" "', argument " "3"" of type '" "std::vector< unsigned long >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< unsigned long >::difference_type >(val3);
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vector_uint64_t___setslice__" "', argument " "4"" of type '" "std::vector< unsigned long,std::allocator< unsigned long > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_uint64_t___setslice__" "', argument " "4"" of type '" "std::vector< unsigned long,std::allocator< unsigned long > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint64_t_Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< unsigned long,std::allocator< unsigned long > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_uint64_t___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned long,std::allocator< unsigned long > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vector_uint64_t___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned long,std::allocator< unsigned long > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< unsigned long,std::allocator< unsigned long > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vector_uint64_t___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_uint64_t___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint64_t >::__setslice__(std::vector< unsigned long >::difference_type,std::vector< unsigned long >::difference_type)\n"
    "    std::vector< uint64_t >::__setslice__(std::vector< unsigned long >::difference_type,std::vector< unsigned long >::difference_type,std::vector< unsigned long,std::allocator< unsigned long > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  std::vector< unsigned long >::difference_type arg2 ;
  std::vector< unsigned long >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_uint64_t___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t___delslice__" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint64_t___delslice__" "', argument " "2"" of type '" "std::vector< unsigned long >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned long >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_uint64_t___delslice__" "', argument " "3"" of type '" "std::vector< unsigned long >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< unsigned long >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint64_t_Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  std::vector< unsigned long >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t___delitem__" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint64_t___delitem__" "', argument " "2"" of type '" "std::vector< unsigned long >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned long >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint64_t_Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< unsigned long,std::allocator< unsigned long > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t___getitem__" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint64_t___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< unsigned long,std::allocator< unsigned long > > *)std_vector_Sl_uint64_t_Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< unsigned long,std::allocator< unsigned long > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t___setitem__" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint64_t___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_uint64_t___setitem__" "', argument " "3"" of type '" "std::vector< unsigned long,std::allocator< unsigned long > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_uint64_t___setitem__" "', argument " "3"" of type '" "std::vector< unsigned long,std::allocator< unsigned long > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint64_t_Sg____setitem____SWIG_0(arg1,arg2,(std::vector< unsigned long,std::allocator< unsigned long > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t___setitem__" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint64_t___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint64_t_Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t___delitem__" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint64_t___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint64_t_Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_uint64_t___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned long,std::allocator< unsigned long > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_uint64_t___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned long,std::allocator< unsigned long > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_uint64_t___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_uint64_t___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint64_t >::__delitem__(std::vector< unsigned long >::difference_type)\n"
    "    std::vector< uint64_t >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  std::vector< unsigned long >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< unsigned long >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t___getitem__" "', argument " "1"" of type '" "std::vector< uint64_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint64_t___getitem__" "', argument " "2"" of type '" "std::vector< unsigned long >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned long >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< unsigned long >::value_type *) &std_vector_Sl_uint64_t_Sg____getitem____SWIG_1((std::vector< unsigned long > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(*result));
  (void)swig::container_owner<swig::traits<std::vector< unsigned long >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_uint64_t___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned long,std::allocator< unsigned long > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_uint64_t___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned long,std::allocator< unsigned long > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_uint64_t___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_uint64_t___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint64_t >::__getitem__(PySliceObject *)\n"
    "    std::vector< uint64_t >::__getitem__(std::vector< unsigned long >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  std::vector< unsigned long >::difference_type arg2 ;
  std::vector< unsigned long >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< unsigned long >::value_type temp3 ;
  unsigned long val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t___setitem__" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint64_t___setitem__" "', argument " "2"" of type '" "std::vector< unsigned long >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned long >::difference_type >(val2);
  ecode3 = SWIG_AsVal_unsigned_SS_long(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_uint64_t___setitem__" "', argument " "3"" of type '" "std::vector< unsigned long >::value_type""'");
  } 
  temp3 = static_cast< std::vector< unsigned long >::value_type >(val3);
  arg3 = &temp3;
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_uint64_t_Sg____setitem____SWIG_2(arg1,arg2,(unsigned long const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_uint64_t___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned long,std::allocator< unsigned long > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_uint64_t___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned long,std::allocator< unsigned long > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< unsigned long,std::allocator< unsigned long > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vector_uint64_t___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned long,std::allocator< unsigned long > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_unsigned_SS_long(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vector_uint64_t___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_uint64_t___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint64_t >::__setitem__(PySliceObject *,std::vector< unsigned long,std::allocator< unsigned long > > const &)\n"
    "    std::vector< uint64_t >::__setitem__(PySliceObject *)\n"
    "    std::vector< uint64_t >::__setitem__(std::vector< unsigned long >::difference_type,std::vector< unsigned long >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< unsigned long >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_pop" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< unsigned long >::value_type)std_vector_Sl_uint64_t_Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  std::vector< unsigned long >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< unsigned long >::value_type temp2 ;
  unsigned long val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_uint64_t_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_append" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_long(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint64_t_append" "', argument " "2"" of type '" "std::vector< unsigned long >::value_type""'");
  } 
  temp2 = static_cast< std::vector< unsigned long >::value_type >(val2);
  arg2 = &temp2;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_uint64_t_Sg__append(arg1,(unsigned long const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_uint64_t__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< uint64_t > *)new std::vector< uint64_t >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_uint64_t__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< uint64_t > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vector_uint64_t" "', argument " "1"" of type '" "std::vector< uint64_t > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vector_uint64_t" "', argument " "1"" of type '" "std::vector< uint64_t > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< uint64_t > *)new std::vector< uint64_t >((std::vector< uint64_t > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_empty" "', argument " "1"" of type '" "std::vector< uint64_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< uint64_t > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< unsigned long >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_size" "', argument " "1"" of type '" "std::vector< uint64_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< uint64_t > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  std::vector< uint64_t > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_uint64_t_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_swap" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vector_uint64_t_swap" "', argument " "2"" of type '" "std::vector< uint64_t > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_uint64_t_swap" "', argument " "2"" of type '" "std::vector< uint64_t > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< uint64_t > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::vector< unsigned long >::iterator > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_begin" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< unsigned long >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::vector< unsigned long >::iterator > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_end" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< unsigned long >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::vector< unsigned long >::reverse_iterator > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_rbegin" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< unsigned long >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::vector< unsigned long >::reverse_iterator > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_rend" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< unsigned long >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_clear" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< unsigned long > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_get_allocator" "', argument " "1"" of type '" "std::vector< uint64_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< uint64_t > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< unsigned long >::allocator_type(static_cast< const std::vector< unsigned long >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_unsigned_long_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_uint64_t__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< unsigned long >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< uint64_t > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vector_uint64_t" "', argument " "1"" of type '" "std::vector< unsigned long >::size_type""'");
  } 
  arg1 = static_cast< std::vector< unsigned long >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< uint64_t > *)new std::vector< uint64_t >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_pop_back" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  std::vector< unsigned long >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_resize" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint64_t_resize" "', argument " "2"" of type '" "std::vector< unsigned long >::size_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned long >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  SwigValueWrapper< std::vector< unsigned long >::iterator > arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  SwigValueWrapper< std::vector< unsigned long >::iterator > result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_erase" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint64_t_erase" "', argument " "2"" of type '" "std::vector< unsigned long >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< unsigned long >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned long >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint64_t_erase" "', argument " "2"" of type '" "std::vector< unsigned long >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_uint64_t_Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< unsigned long >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  SwigValueWrapper< std::vector< unsigned long >::iterator > arg2 ;
  SwigValueWrapper< std::vector< unsigned long >::iterator > arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  SwigValueWrapper< std::vector< unsigned long >::iterator > result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_erase" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint64_t_erase" "', argument " "2"" of type '" "std::vector< unsigned long >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< unsigned long >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned long >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint64_t_erase" "', argument " "2"" of type '" "std::vector< unsigned long >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint64_t_erase" "', argument " "3"" of type '" "std::vector< unsigned long >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< unsigned long >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned long >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint64_t_erase" "', argument " "3"" of type '" "std::vector< unsigned long >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_uint64_t_Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< unsigned long >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_uint64_t_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned long,std::allocator< unsigned long > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned long >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vector_uint64_t_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned long,std::allocator< unsigned long > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned long >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned long >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vector_uint64_t_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_uint64_t_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint64_t >::erase(std::vector< unsigned long >::iterator)\n"
    "    std::vector< uint64_t >::erase(std::vector< unsigned long >::iterator,std::vector< unsigned long >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vector_uint64_t__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< unsigned long >::size_type arg1 ;
  std::vector< unsigned long >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< unsigned long >::value_type temp2 ;
  unsigned long val2 ;
  int ecode2 = 0 ;
  std::vector< uint64_t > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vector_uint64_t" "', argument " "1"" of type '" "std::vector< unsigned long >::size_type""'");
  } 
  arg1 = static_cast< std::vector< unsigned long >::size_type >(val1);
  ecode2 = SWIG_AsVal_unsigned_SS_long(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "new_vector_uint64_t" "', argument " "2"" of type '" "std::vector< unsigned long >::value_type""'");
  } 
  temp2 = static_cast< std::vector< unsigned long >::value_type >(val2);
  arg2 = &temp2;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< uint64_t > *)new std::vector< uint64_t >(arg1,(std::vector< unsigned long >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_uint64_t(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vector_uint64_t", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vector_uint64_t__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vector_uint64_t__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned long,std::allocator< unsigned long > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vector_uint64_t__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      {
        int res = SWIG_AsVal_unsigned_SS_long(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_new_vector_uint64_t__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vector_uint64_t'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint64_t >::vector()\n"
    "    std::vector< uint64_t >::vector(std::vector< uint64_t > const &)\n"
    "    std::vector< uint64_t >::vector(std::vector< unsigned long >::size_type)\n"
    "    std::vector< uint64_t >::vector(std::vector< unsigned long >::size_type,std::vector< unsigned long >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  std::vector< unsigned long >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< unsigned long >::value_type temp2 ;
  unsigned long val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_uint64_t_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_push_back" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_long(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint64_t_push_back" "', argument " "2"" of type '" "std::vector< unsigned long >::value_type""'");
  } 
  temp2 = static_cast< std::vector< unsigned long >::value_type >(val2);
  arg2 = &temp2;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< unsigned long >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< unsigned long >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_front" "', argument " "1"" of type '" "std::vector< uint64_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< unsigned long >::value_type *) &((std::vector< uint64_t > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(*result));
  (void)swig::container_owner<swig::traits<std::vector< unsigned long >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< unsigned long >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_back" "', argument " "1"" of type '" "std::vector< uint64_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< unsigned long >::value_type *) &((std::vector< uint64_t > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(*result));
  (void)swig::container_owner<swig::traits<std::vector< unsigned long >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  std::vector< unsigned long >::size_type arg2 ;
  std::vector< unsigned long >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  std::vector< unsigned long >::value_type temp3 ;
  unsigned long val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_uint64_t_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_assign" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint64_t_assign" "', argument " "2"" of type '" "std::vector< unsigned long >::size_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned long >::size_type >(val2);
  ecode3 = SWIG_AsVal_unsigned_SS_long(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_uint64_t_assign" "', argument " "3"" of type '" "std::vector< unsigned long >::value_type""'");
  } 
  temp3 = static_cast< std::vector< unsigned long >::value_type >(val3);
  arg3 = &temp3;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< unsigned long >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  std::vector< unsigned long >::size_type arg2 ;
  std::vector< unsigned long >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  std::vector< unsigned long >::value_type temp3 ;
  unsigned long val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_resize" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint64_t_resize" "', argument " "2"" of type '" "std::vector< unsigned long >::size_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned long >::size_type >(val2);
  ecode3 = SWIG_AsVal_unsigned_SS_long(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_uint64_t_resize" "', argument " "3"" of type '" "std::vector< unsigned long >::value_type""'");
  } 
  temp3 = static_cast< std::vector< unsigned long >::value_type >(val3);
  arg3 = &temp3;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< unsigned long >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_uint64_t_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned long,std::allocator< unsigned long > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_uint64_t_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned long,std::allocator< unsigned long > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_unsigned_SS_long(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vector_uint64_t_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_uint64_t_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint64_t >::resize(std::vector< unsigned long >::size_type)\n"
    "    std::vector< uint64_t >::resize(std::vector< unsigned long >::size_type,std::vector< unsigned long >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  SwigValueWrapper< std::vector< unsigned long >::iterator > arg2 ;
  std::vector< unsigned long >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< unsigned long >::value_type temp3 ;
  unsigned long val3 ;
  int ecode3 = 0 ;
  SwigValueWrapper< std::vector< unsigned long >::iterator > result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_insert" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint64_t_insert" "', argument " "2"" of type '" "std::vector< unsigned long >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< unsigned long >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned long >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint64_t_insert" "', argument " "2"" of type '" "std::vector< unsigned long >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_unsigned_SS_long(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_uint64_t_insert" "', argument " "3"" of type '" "std::vector< unsigned long >::value_type""'");
  } 
  temp3 = static_cast< std::vector< unsigned long >::value_type >(val3);
  arg3 = &temp3;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_uint64_t_Sg__insert__SWIG_0(arg1,arg2,(unsigned long const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< unsigned long >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  SwigValueWrapper< std::vector< unsigned long >::iterator > arg2 ;
  std::vector< unsigned long >::size_type arg3 ;
  std::vector< unsigned long >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  std::vector< unsigned long >::value_type temp4 ;
  unsigned long val4 ;
  int ecode4 = 0 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_insert" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint64_t_insert" "', argument " "2"" of type '" "std::vector< unsigned long >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< unsigned long >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned long >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_uint64_t_insert" "', argument " "2"" of type '" "std::vector< unsigned long >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_uint64_t_insert" "', argument " "3"" of type '" "std::vector< unsigned long >::size_type""'");
  } 
  arg3 = static_cast< std::vector< unsigned long >::size_type >(val3);
  ecode4 = SWIG_AsVal_unsigned_SS_long(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "vector_uint64_t_insert" "', argument " "4"" of type '" "std::vector< unsigned long >::value_type""'");
  } 
  temp4 = static_cast< std::vector< unsigned long >::value_type >(val4);
  arg4 = &temp4;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_uint64_t_Sg__insert__SWIG_1(arg1,arg2,arg3,(unsigned long const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_uint64_t_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned long,std::allocator< unsigned long > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned long >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_unsigned_SS_long(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vector_uint64_t_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< unsigned long,std::allocator< unsigned long > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< unsigned long >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          {
            int res = SWIG_AsVal_unsigned_SS_long(argv[3], NULL);
            _v = SWIG_CheckState(res);
          }
          if (_v) {
            return _wrap_vector_uint64_t_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_uint64_t_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< uint64_t >::insert(std::vector< unsigned long >::iterator,std::vector< unsigned long >::value_type const &)\n"
    "    std::vector< uint64_t >::insert(std::vector< unsigned long >::iterator,std::vector< unsigned long >::size_type,std::vector< unsigned long >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  std::vector< unsigned long >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_uint64_t_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_reserve" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_uint64_t_reserve" "', argument " "2"" of type '" "std::vector< unsigned long >::size_type""'");
  } 
  arg2 = static_cast< std::vector< unsigned long >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_uint64_t_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< unsigned long >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_uint64_t_capacity" "', argument " "1"" of type '" "std::vector< uint64_t > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< uint64_t > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vector_uint64_t(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint64_t > *arg1 = (std::vector< uint64_t > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vector_uint64_t" "', argument " "1"" of type '" "std::vector< uint64_t > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< uint64_t > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vector_uint64_t_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vector_uint64_t_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_new_map_strstr__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::less< std::string > *arg1 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::map< std::string,std::string > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_std__lessT_std__string_t,  0  | 0);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_map_strstr" "', argument " "1"" of type '" "std::less< std::string > const &""'"); 
  }
  if (!argp1) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_map_strstr" "', argument " "1"" of type '" "std::less< std::string > const &""'"); 
  }
  arg1 = reinterpret_cast< std::less< std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::map< std::string,std::string > *)new std::map< std::string,std::string >((std::less< std::string > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_iterator" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_map_Sl_std_string_Sc_std_string_Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr___nonzero__" "', argument " "1"" of type '" "std::map< std::string,std::string > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_map_Sl_std_string_Sc_std_string_Sg____nonzero__((std::map< std::string,std::string > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr___bool__" "', argument " "1"" of type '" "std::map< std::string,std::string > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_map_Sl_std_string_Sc_std_string_Sg____bool__((std::map< std::string,std::string > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< std::string,std::string >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr___len__" "', argument " "1"" of type '" "std::map< std::string,std::string > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_map_Sl_std_string_Sc_std_string_Sg____len__((std::map< std::string,std::string > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr___getitem__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  std::map< std::string,std::string >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  std::map< std::string,std::string >::mapped_type *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "map_strstr___getitem__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr___getitem__" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_strstr___getitem__" "', argument " "2"" of type '" "std::map< std::string,std::string >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_strstr___getitem__" "', argument " "2"" of type '" "std::map< std::string,std::string >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::map< std::string,std::string >::mapped_type *) &std_map_Sl_std_string_Sc_std_string_Sg____getitem__(arg1,(std::string const &)*arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr___delitem__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  std::map< std::string,std::string >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "map_strstr___delitem__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr___delitem__" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_strstr___delitem__" "', argument " "2"" of type '" "std::map< std::string,std::string >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_strstr___delitem__" "', argument " "2"" of type '" "std::map< std::string,std::string >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_map_Sl_std_string_Sc_std_string_Sg____delitem__(arg1,(std::string const &)*arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr_has_key(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  std::map< std::string,std::string >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_strstr_has_key", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_has_key" "', argument " "1"" of type '" "std::map< std::string,std::string > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_strstr_has_key" "', argument " "2"" of type '" "std::map< std::string,std::string >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_strstr_has_key" "', argument " "2"" of type '" "std::map< std::string,std::string >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_map_Sl_std_string_Sc_std_string_Sg__has_key((std::map< std::string,std::string > const *)arg1,(std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr_keys(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_keys" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)std_map_Sl_std_string_Sc_std_string_Sg__keys(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr_values(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_values" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)std_map_Sl_std_string_Sc_std_string_Sg__values(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr_items(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_items" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)std_map_Sl_std_string_Sc_std_string_Sg__items(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr___contains__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  std::map< std::string,std::string >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_strstr___contains__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr___contains__" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_strstr___contains__" "', argument " "2"" of type '" "std::map< std::string,std::string >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_strstr___contains__" "', argument " "2"" of type '" "std::map< std::string,std::string >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_map_Sl_std_string_Sc_std_string_Sg____contains__(arg1,(std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr_key_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_key_iterator" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_map_Sl_std_string_Sc_std_string_Sg__key_iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr_value_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_value_iterator" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_map_Sl_std_string_Sc_std_string_Sg__value_iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  std::map< std::string,std::string >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr___setitem__" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_strstr___setitem__" "', argument " "2"" of type '" "std::map< std::string,std::string >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_strstr___setitem__" "', argument " "2"" of type '" "std::map< std::string,std::string >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_map_Sl_std_string_Sc_std_string_Sg____setitem____SWIG_0(arg1,(std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  std::map< std::string,std::string >::key_type *arg2 = 0 ;
  std::map< std::string,std::string >::mapped_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr___setitem__" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_strstr___setitem__" "', argument " "2"" of type '" "std::map< std::string,std::string >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_strstr___setitem__" "', argument " "2"" of type '" "std::map< std::string,std::string >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "map_strstr___setitem__" "', argument " "3"" of type '" "std::map< std::string,std::string >::mapped_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_strstr___setitem__" "', argument " "3"" of type '" "std::map< std::string,std::string >::mapped_type const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_map_Sl_std_string_Sc_std_string_Sg____setitem____SWIG_1(arg1,(std::string const &)*arg2,(std::string const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "map_strstr___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_map_strstr___setitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_map_strstr___setitem____SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'map_strstr___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::map< std::string,std::string >::__setitem__(std::map< std::string,std::string >::key_type const &)\n"
    "    std::map< std::string,std::string >::__setitem__(std::map< std::string,std::string >::key_type const &,std::map< std::string,std::string >::mapped_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_map_strstr_asdict(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_asdict" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)std_map_Sl_std_string_Sc_std_string_Sg__asdict(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_map_strstr__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::map< std::string,std::string > *)new std::map< std::string,std::string >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_map_strstr__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::map< std::string,std::string > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > *ptr = (std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_map_strstr" "', argument " "1"" of type '" "std::map< std::string,std::string > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_map_strstr" "', argument " "1"" of type '" "std::map< std::string,std::string > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::map< std::string,std::string > *)new std::map< std::string,std::string >((std::map< std::string,std::string > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_map_strstr(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[2] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_map_strstr", 0, 1, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_map_strstr__SWIG_1(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_std__lessT_std__string_t, SWIG_POINTER_NO_NULL | 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_map_strstr__SWIG_0(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_map_strstr__SWIG_2(self, argc, argv);
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_map_strstr'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::map< std::string,std::string >::map(std::less< std::string > const &)\n"
    "    std::map< std::string,std::string >::map()\n"
    "    std::map< std::string,std::string >::map(std::map< std::string,std::string > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_map_strstr_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_empty" "', argument " "1"" of type '" "std::map< std::string,std::string > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::map< std::string,std::string > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< std::string,std::string >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_size" "', argument " "1"" of type '" "std::map< std::string,std::string > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::map< std::string,std::string > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  std::map< std::string,std::string > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "map_strstr_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_swap" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_strstr_swap" "', argument " "2"" of type '" "std::map< std::string,std::string > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_strstr_swap" "', argument " "2"" of type '" "std::map< std::string,std::string > &""'"); 
  }
  arg2 = reinterpret_cast< std::map< std::string,std::string > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< std::string,std::string >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_begin" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< std::string,std::string >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< std::string,std::string >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_end" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< std::string,std::string >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< std::string,std::string >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_rbegin" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< std::string,std::string >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< std::string,std::string >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_rend" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< std::string,std::string >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_clear" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< std::pair< std::string const,std::string > > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_get_allocator" "', argument " "1"" of type '" "std::map< std::string,std::string > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::map< std::string,std::string > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::map< std::string,std::string >::allocator_type(static_cast< const std::map< std::string,std::string >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_std__pairT_std__string_const_std__string_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  std::map< std::string,std::string >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  std::map< std::string,std::string >::size_type result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_erase" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_strstr_erase" "', argument " "2"" of type '" "std::map< std::string,std::string >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_strstr_erase" "', argument " "2"" of type '" "std::map< std::string,std::string >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->erase((std::map< std::string,std::string >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr_count(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  std::map< std::string,std::string >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  std::map< std::string,std::string >::size_type result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_strstr_count", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_count" "', argument " "1"" of type '" "std::map< std::string,std::string > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_strstr_count" "', argument " "2"" of type '" "std::map< std::string,std::string >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_strstr_count" "', argument " "2"" of type '" "std::map< std::string,std::string >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::map< std::string,std::string > const *)arg1)->count((std::map< std::string,std::string >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  std::map< std::string,std::string >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_erase" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_strstr_erase" "', argument " "2"" of type '" "std::map< std::string,std::string >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::map< std::string,std::string >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::map< std::string,std::string >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_strstr_erase" "', argument " "2"" of type '" "std::map< std::string,std::string >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_map_Sl_std_string_Sc_std_string_Sg__erase__SWIG_1(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr_erase__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  std::map< std::string,std::string >::iterator arg2 ;
  std::map< std::string,std::string >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_erase" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_strstr_erase" "', argument " "2"" of type '" "std::map< std::string,std::string >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::map< std::string,std::string >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::map< std::string,std::string >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_strstr_erase" "', argument " "2"" of type '" "std::map< std::string,std::string >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_strstr_erase" "', argument " "3"" of type '" "std::map< std::string,std::string >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::map< std::string,std::string >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::map< std::string,std::string >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_strstr_erase" "', argument " "3"" of type '" "std::map< std::string,std::string >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_map_Sl_std_string_Sc_std_string_Sg__erase__SWIG_2(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "map_strstr_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::map< std::string,std::string >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_map_strstr_erase__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_map_strstr_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::map< std::string,std::string >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::map< std::string,std::string >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_map_strstr_erase__SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'map_strstr_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::map< std::string,std::string >::erase(std::map< std::string,std::string >::key_type const &)\n"
    "    std::map< std::string,std::string >::erase(std::map< std::string,std::string >::iterator)\n"
    "    std::map< std::string,std::string >::erase(std::map< std::string,std::string >::iterator,std::map< std::string,std::string >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_map_strstr_find(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  std::map< std::string,std::string >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  std::map< std::string,std::string >::iterator result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_strstr_find", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_find" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_strstr_find" "', argument " "2"" of type '" "std::map< std::string,std::string >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_strstr_find" "', argument " "2"" of type '" "std::map< std::string,std::string >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->find((std::map< std::string,std::string >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< std::string,std::string >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr_lower_bound(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  std::map< std::string,std::string >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  std::map< std::string,std::string >::iterator result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_strstr_lower_bound", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_lower_bound" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_strstr_lower_bound" "', argument " "2"" of type '" "std::map< std::string,std::string >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_strstr_lower_bound" "', argument " "2"" of type '" "std::map< std::string,std::string >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->lower_bound((std::map< std::string,std::string >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< std::string,std::string >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_strstr_upper_bound(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  std::map< std::string,std::string >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  std::map< std::string,std::string >::iterator result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_strstr_upper_bound", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_strstr_upper_bound" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_strstr_upper_bound" "', argument " "2"" of type '" "std::map< std::string,std::string >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_strstr_upper_bound" "', argument " "2"" of type '" "std::map< std::string,std::string >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->upper_bound((std::map< std::string,std::string >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< std::string,std::string >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_map_strstr(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,std::string > *arg1 = (std::map< std::string,std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_map_strstr" "', argument " "1"" of type '" "std::map< std::string,std::string > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *map_strstr_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *map_strstr_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_GetRRDataTypeString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::DataTypes arg1 ;
  int val1 ;
  int ecode1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  ecode1 = SWIG_AsVal_int(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "GetRRDataTypeString" "', argument " "1"" of type '" "RobotRaconteur::DataTypes""'");
  } 
  arg1 = static_cast< RobotRaconteur::DataTypes >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::GetRRDataTypeString(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_IsTypeRRArray(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::DataTypes arg1 ;
  int val1 ;
  int ecode1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  ecode1 = SWIG_AsVal_int(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "IsTypeRRArray" "', argument " "1"" of type '" "RobotRaconteur::DataTypes""'");
  } 
  arg1 = static_cast< RobotRaconteur::DataTypes >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)RobotRaconteur::IsTypeRRArray(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_IsTypeNumeric(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::DataTypes arg1 ;
  int val1 ;
  int ecode1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  ecode1 = SWIG_AsVal_int(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "IsTypeNumeric" "', argument " "1"" of type '" "RobotRaconteur::DataTypes""'");
  } 
  arg1 = static_cast< RobotRaconteur::DataTypes >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)RobotRaconteur::IsTypeNumeric(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_RRObject(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRObject *arg1 = (RobotRaconteur::RRObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRObject > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RRObject > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRObject_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_RRObject" "', argument " "1"" of type '" "RobotRaconteur::RRObject *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRObject > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRObject > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RRObject * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRObject > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RRObject * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *RRObject_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRObject_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_delete_RRValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRValue *arg1 = (RobotRaconteur::RRValue *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRValue > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RRValue > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRValue > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::RRValue *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRValue_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_RRValue" "', argument " "1"" of type '" "RobotRaconteur::RRValue *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRValue > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRValue > * >(argp1);
    arg1 = const_cast< RobotRaconteur::RRValue * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRValue > * >(argp1);
    arg1 = const_cast< RobotRaconteur::RRValue * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *RRValue_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRValue_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_MessageElementData_GetTypeString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElementData *arg1 = (RobotRaconteur::MessageElementData *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElementData > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElementData > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MessageStringPtr result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElementData *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElementData_GetTypeString" "', argument " "1"" of type '" "RobotRaconteur::MessageElementData *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementData > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementData > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementData * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementData > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementData * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetTypeString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_MessageStringPtr(static_cast< MessageStringPtr >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElementData_GetTypeID(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElementData *arg1 = (RobotRaconteur::MessageElementData *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElementData > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElementData > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::DataTypes result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElementData *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElementData_GetTypeID" "', argument " "1"" of type '" "RobotRaconteur::MessageElementData *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementData > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementData > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementData * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementData > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementData * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::DataTypes)(arg1)->GetTypeID();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_MessageElementData(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElementData *arg1 = (RobotRaconteur::MessageElementData *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElementData > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElementData > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElementData *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_MessageElementData" "', argument " "1"" of type '" "RobotRaconteur::MessageElementData *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementData > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementData > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementData * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementData > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementData * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *MessageElementData_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_RRBaseArray_GetTypeString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRBaseArray *arg1 = (RobotRaconteur::RRBaseArray *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRBaseArray > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RRBaseArray > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MessageStringPtr result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::RRBaseArray *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRBaseArray_GetTypeString" "', argument " "1"" of type '" "RobotRaconteur::RRBaseArray *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRBaseArray > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRBaseArray > * >(argp1);
    arg1 = const_cast< RobotRaconteur::RRBaseArray * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRBaseArray > * >(argp1);
    arg1 = const_cast< RobotRaconteur::RRBaseArray * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetTypeString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_MessageStringPtr(static_cast< MessageStringPtr >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRBaseArray_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRBaseArray *arg1 = (RobotRaconteur::RRBaseArray *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRBaseArray > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RRBaseArray > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  size_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::RRBaseArray *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRBaseArray_size" "', argument " "1"" of type '" "RobotRaconteur::RRBaseArray *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRBaseArray > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRBaseArray > * >(argp1);
    arg1 = const_cast< RobotRaconteur::RRBaseArray * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRBaseArray > * >(argp1);
    arg1 = const_cast< RobotRaconteur::RRBaseArray * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRBaseArray_void_ptr(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRBaseArray *arg1 = (RobotRaconteur::RRBaseArray *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRBaseArray > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RRBaseArray > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  void *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::RRBaseArray *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRBaseArray_void_ptr" "', argument " "1"" of type '" "RobotRaconteur::RRBaseArray *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRBaseArray > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRBaseArray > * >(argp1);
    arg1 = const_cast< RobotRaconteur::RRBaseArray * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRBaseArray > * >(argp1);
    arg1 = const_cast< RobotRaconteur::RRBaseArray * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (void *)(arg1)->void_ptr();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj=PyLong_FromVoidPtr(result);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRBaseArray_ElementSize(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRBaseArray *arg1 = (RobotRaconteur::RRBaseArray *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRBaseArray > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RRBaseArray > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  size_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::RRBaseArray *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRBaseArray_ElementSize" "', argument " "1"" of type '" "RobotRaconteur::RRBaseArray *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRBaseArray > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRBaseArray > * >(argp1);
    arg1 = const_cast< RobotRaconteur::RRBaseArray * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRBaseArray > * >(argp1);
    arg1 = const_cast< RobotRaconteur::RRBaseArray * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ElementSize();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRBaseArray_GetTypeID(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRBaseArray *arg1 = (RobotRaconteur::RRBaseArray *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRBaseArray > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RRBaseArray > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::DataTypes result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::RRBaseArray *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRBaseArray_GetTypeID" "', argument " "1"" of type '" "RobotRaconteur::RRBaseArray *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRBaseArray > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRBaseArray > * >(argp1);
    arg1 = const_cast< RobotRaconteur::RRBaseArray * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRBaseArray > * >(argp1);
    arg1 = const_cast< RobotRaconteur::RRBaseArray * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::DataTypes)(arg1)->GetTypeID();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_RRBaseArray(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRBaseArray *arg1 = (RobotRaconteur::RRBaseArray *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRBaseArray > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RRBaseArray > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::RRBaseArray *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_RRBaseArray" "', argument " "1"" of type '" "RobotRaconteur::RRBaseArray *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRBaseArray > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRBaseArray > * >(argp1);
    arg1 = const_cast< RobotRaconteur::RRBaseArray * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRBaseArray > * >(argp1);
    arg1 = const_cast< RobotRaconteur::RRBaseArray * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *RRBaseArray_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_AllocateRRArrayByType(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::DataTypes arg1 ;
  size_t arg2 ;
  int val1 ;
  int ecode1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::RRBaseArray > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::RRBaseArray > result;
  
  if (!SWIG_Python_UnpackTuple(args, "AllocateRRArrayByType", 2, 2, swig_obj)) SWIG_fail;
  ecode1 = SWIG_AsVal_int(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "AllocateRRArrayByType" "', argument " "1"" of type '" "RobotRaconteur::DataTypes""'");
  } 
  arg1 = static_cast< RobotRaconteur::DataTypes >(val1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "AllocateRRArrayByType" "', argument " "2"" of type '" "size_t""'");
  } 
  arg2 = static_cast< size_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::AllocateRRArrayByType(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::RRBaseArray >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::RRBaseArray >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::RRBaseArray >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRMultiDimArrayUntyped_Dims_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRMultiDimArrayUntyped *arg1 = (RobotRaconteur::RRMultiDimArrayUntyped *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::RRBaseArray > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::RRBaseArray > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::RRBaseArray > temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RRMultiDimArrayUntyped_Dims_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRMultiDimArrayUntyped_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRMultiDimArrayUntyped_Dims_set" "', argument " "1"" of type '" "RobotRaconteur::RRMultiDimArrayUntyped *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRMultiDimArrayUntyped > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRMultiDimArrayUntyped > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RRMultiDimArrayUntyped * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRMultiDimArrayUntyped > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RRMultiDimArrayUntyped * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::RRBaseArray >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RRMultiDimArrayUntyped_Dims_set" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::RRBaseArray > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::RRBaseArray >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRBaseArray >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRBaseArray >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Dims = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRMultiDimArrayUntyped_Dims_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRMultiDimArrayUntyped *arg1 = (RobotRaconteur::RRMultiDimArrayUntyped *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::RRBaseArray > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::RRBaseArray > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRMultiDimArrayUntyped_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRMultiDimArrayUntyped_Dims_get" "', argument " "1"" of type '" "RobotRaconteur::RRMultiDimArrayUntyped *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRMultiDimArrayUntyped > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRMultiDimArrayUntyped > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RRMultiDimArrayUntyped * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRMultiDimArrayUntyped > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RRMultiDimArrayUntyped * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::intrusive_ptr< RobotRaconteur::RRBaseArray > *) & ((arg1)->Dims);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  
  //if (*result) {
  //  intrusive_ptr_add_ref(result->get());
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray >(result->get(), SWIG_intrusive_deleter<  RobotRaconteur::RRBaseArray >());
  //} else {
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray > **)&resultobj = 0;
  //}
  
  if(*(result)) {
    intrusive_ptr_add_ref(result->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::RRBaseArray >(result->get(),SWIG_intrusive_deleter<  RobotRaconteur::RRBaseArray >());
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRMultiDimArrayUntyped_Array_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRMultiDimArrayUntyped *arg1 = (RobotRaconteur::RRMultiDimArrayUntyped *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::RRBaseArray > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::RRBaseArray > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::RRBaseArray > temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RRMultiDimArrayUntyped_Array_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRMultiDimArrayUntyped_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRMultiDimArrayUntyped_Array_set" "', argument " "1"" of type '" "RobotRaconteur::RRMultiDimArrayUntyped *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRMultiDimArrayUntyped > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRMultiDimArrayUntyped > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RRMultiDimArrayUntyped * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRMultiDimArrayUntyped > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RRMultiDimArrayUntyped * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::RRBaseArray >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RRMultiDimArrayUntyped_Array_set" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::RRBaseArray > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::RRBaseArray >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRBaseArray >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRBaseArray >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Array = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRMultiDimArrayUntyped_Array_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRMultiDimArrayUntyped *arg1 = (RobotRaconteur::RRMultiDimArrayUntyped *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::RRBaseArray > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::RRBaseArray > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRMultiDimArrayUntyped_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRMultiDimArrayUntyped_Array_get" "', argument " "1"" of type '" "RobotRaconteur::RRMultiDimArrayUntyped *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRMultiDimArrayUntyped > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRMultiDimArrayUntyped > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RRMultiDimArrayUntyped * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRMultiDimArrayUntyped > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RRMultiDimArrayUntyped * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::intrusive_ptr< RobotRaconteur::RRBaseArray > *) & ((arg1)->Array);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  
  //if (*result) {
  //  intrusive_ptr_add_ref(result->get());
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray >(result->get(), SWIG_intrusive_deleter<  RobotRaconteur::RRBaseArray >());
  //} else {
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray > **)&resultobj = 0;
  //}
  
  if(*(result)) {
    intrusive_ptr_add_ref(result->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::RRBaseArray >(result->get(),SWIG_intrusive_deleter<  RobotRaconteur::RRBaseArray >());
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_RRMultiDimArrayUntyped(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRMultiDimArrayUntyped *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_RRMultiDimArrayUntyped", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::RRMultiDimArrayUntyped *)new RobotRaconteur::RRMultiDimArrayUntyped();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::RRMultiDimArrayUntyped > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::RRMultiDimArrayUntyped >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRMultiDimArrayUntyped_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_RRMultiDimArrayUntyped(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRMultiDimArrayUntyped *arg1 = (RobotRaconteur::RRMultiDimArrayUntyped *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRMultiDimArrayUntyped_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_RRMultiDimArrayUntyped" "', argument " "1"" of type '" "RobotRaconteur::RRMultiDimArrayUntyped *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRMultiDimArrayUntyped > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRMultiDimArrayUntyped > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RRMultiDimArrayUntyped * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RRMultiDimArrayUntyped > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RRMultiDimArrayUntyped * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *RRMultiDimArrayUntyped_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRMultiDimArrayUntyped_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *RRMultiDimArrayUntyped_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_new_ServicePathSegment__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServicePathSegment *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::ServicePathSegment *)new RobotRaconteur::ServicePathSegment();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServicePathSegment, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ServicePathSegment__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  RobotRaconteur::ServicePathSegment *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_ServicePathSegment" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_ServicePathSegment" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::ServicePathSegment *)new RobotRaconteur::ServicePathSegment((std::string const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServicePathSegment, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ServicePathSegment__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  std::string *arg2 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  int res2 = SWIG_OLDOBJ ;
  RobotRaconteur::ServicePathSegment *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_ServicePathSegment" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_ServicePathSegment" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_ServicePathSegment" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_ServicePathSegment" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::ServicePathSegment *)new RobotRaconteur::ServicePathSegment((std::string const &)*arg1,(std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServicePathSegment, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ServicePathSegment(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_ServicePathSegment", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_ServicePathSegment__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    int res = SWIG_AsPtr_std_string(argv[0], (std::string**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_ServicePathSegment__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_AsPtr_std_string(argv[0], (std::string**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_ServicePathSegment__SWIG_2(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_ServicePathSegment'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::ServicePathSegment::ServicePathSegment()\n"
    "    RobotRaconteur::ServicePathSegment::ServicePathSegment(std::string const &)\n"
    "    RobotRaconteur::ServicePathSegment::ServicePathSegment(std::string const &,std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_ServicePathSegment_name_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServicePathSegment *arg1 = (RobotRaconteur::ServicePathSegment *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServicePathSegment_name_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServicePathSegment, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegment_name_set" "', argument " "1"" of type '" "RobotRaconteur::ServicePathSegment *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServicePathSegment * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServicePathSegment_name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServicePathSegment_name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->name = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegment_name_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServicePathSegment *arg1 = (RobotRaconteur::ServicePathSegment *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServicePathSegment, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegment_name_get" "', argument " "1"" of type '" "RobotRaconteur::ServicePathSegment *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServicePathSegment * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->name);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegment_index_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServicePathSegment *arg1 = (RobotRaconteur::ServicePathSegment *) 0 ;
  boost::optional< std::string > *arg2 = (boost::optional< std::string > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServicePathSegment_index_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServicePathSegment, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegment_index_set" "', argument " "1"" of type '" "RobotRaconteur::ServicePathSegment *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServicePathSegment * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_boost__optionalT_std__string_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServicePathSegment_index_set" "', argument " "2"" of type '" "boost::optional< std::string > *""'"); 
  }
  arg2 = reinterpret_cast< boost::optional< std::string > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->index = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegment_index_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServicePathSegment *arg1 = (RobotRaconteur::ServicePathSegment *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::optional< std::string > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServicePathSegment, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegment_index_get" "', argument " "1"" of type '" "RobotRaconteur::ServicePathSegment *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServicePathSegment * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::optional< std::string > *)& ((arg1)->index);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_boost__optionalT_std__string_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_ServicePathSegment(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServicePathSegment *arg1 = (RobotRaconteur::ServicePathSegment *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServicePathSegment, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_ServicePathSegment" "', argument " "1"" of type '" "RobotRaconteur::ServicePathSegment *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServicePathSegment * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *ServicePathSegment_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__ServicePathSegment, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *ServicePathSegment_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_ServicePathSegments_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_iterator" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments___nonzero__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____nonzero__((std::vector< RobotRaconteur::ServicePathSegment > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments___bool__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____bool__((std::vector< RobotRaconteur::ServicePathSegment > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServicePathSegment >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments___len__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____len__((std::vector< RobotRaconteur::ServicePathSegment > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  std::vector< RobotRaconteur::ServicePathSegment >::difference_type arg2 ;
  std::vector< RobotRaconteur::ServicePathSegment >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServicePathSegments___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments___getslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ServicePathSegments___getslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServicePathSegment >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "ServicePathSegments___getslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ServicePathSegment >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > *)std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  std::vector< RobotRaconteur::ServicePathSegment >::difference_type arg2 ;
  std::vector< RobotRaconteur::ServicePathSegment >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments___setslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ServicePathSegments___setslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServicePathSegment >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "ServicePathSegments___setslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ServicePathSegment >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  std::vector< RobotRaconteur::ServicePathSegment >::difference_type arg2 ;
  std::vector< RobotRaconteur::ServicePathSegment >::difference_type arg3 ;
  std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments___setslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ServicePathSegments___setslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServicePathSegment >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "ServicePathSegments___setslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ServicePathSegment >::difference_type >(val3);
  {
    std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > *ptr = (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "ServicePathSegments___setslice__" "', argument " "4"" of type '" "std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServicePathSegments___setslice__" "', argument " "4"" of type '" "std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "ServicePathSegments___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_ServicePathSegments___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_ServicePathSegments___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'ServicePathSegments___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServicePathSegment >::__setslice__(std::vector< RobotRaconteur::ServicePathSegment >::difference_type,std::vector< RobotRaconteur::ServicePathSegment >::difference_type)\n"
    "    std::vector< RobotRaconteur::ServicePathSegment >::__setslice__(std::vector< RobotRaconteur::ServicePathSegment >::difference_type,std::vector< RobotRaconteur::ServicePathSegment >::difference_type,std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  std::vector< RobotRaconteur::ServicePathSegment >::difference_type arg2 ;
  std::vector< RobotRaconteur::ServicePathSegment >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServicePathSegments___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments___delslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ServicePathSegments___delslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServicePathSegment >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "ServicePathSegments___delslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ServicePathSegment >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  std::vector< RobotRaconteur::ServicePathSegment >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments___delitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ServicePathSegments___delitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServicePathSegment >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments___getitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "ServicePathSegments___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > *)std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "ServicePathSegments___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > *ptr = (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "ServicePathSegments___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServicePathSegments___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____setitem____SWIG_0(arg1,arg2,(std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "ServicePathSegments___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments___delitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "ServicePathSegments___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "ServicePathSegments___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_ServicePathSegments___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_ServicePathSegments___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'ServicePathSegments___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServicePathSegment >::__delitem__(std::vector< RobotRaconteur::ServicePathSegment >::difference_type)\n"
    "    std::vector< RobotRaconteur::ServicePathSegment >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  std::vector< RobotRaconteur::ServicePathSegment >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< RobotRaconteur::ServicePathSegment >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments___getitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ServicePathSegments___getitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServicePathSegment >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::ServicePathSegment >::value_type *) &std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____getitem____SWIG_1((std::vector< RobotRaconteur::ServicePathSegment > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServicePathSegment, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::ServicePathSegment >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "ServicePathSegments___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_ServicePathSegments___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_ServicePathSegments___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'ServicePathSegments___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServicePathSegment >::__getitem__(PySliceObject *)\n"
    "    std::vector< RobotRaconteur::ServicePathSegment >::__getitem__(std::vector< RobotRaconteur::ServicePathSegment >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  std::vector< RobotRaconteur::ServicePathSegment >::difference_type arg2 ;
  std::vector< RobotRaconteur::ServicePathSegment >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ServicePathSegments___setitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServicePathSegment >::difference_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServicePathSegment,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "ServicePathSegments___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServicePathSegments___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment >::value_type * >(argp3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg____setitem____SWIG_2(arg1,arg2,(RobotRaconteur::ServicePathSegment const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "ServicePathSegments___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_ServicePathSegments___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_ServicePathSegments___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__ServicePathSegment, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_ServicePathSegments___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'ServicePathSegments___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServicePathSegment >::__setitem__(PySliceObject *,std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > const &)\n"
    "    std::vector< RobotRaconteur::ServicePathSegment >::__setitem__(PySliceObject *)\n"
    "    std::vector< RobotRaconteur::ServicePathSegment >::__setitem__(std::vector< RobotRaconteur::ServicePathSegment >::difference_type,std::vector< RobotRaconteur::ServicePathSegment >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServicePathSegment >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_pop" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< RobotRaconteur::ServicePathSegment >::value_type(static_cast< const std::vector< RobotRaconteur::ServicePathSegment >::value_type& >(result))), SWIGTYPE_p_RobotRaconteur__ServicePathSegment, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  std::vector< RobotRaconteur::ServicePathSegment >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServicePathSegments_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_append" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServicePathSegment,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServicePathSegments_append" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServicePathSegments_append" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg__append(arg1,(RobotRaconteur::ServicePathSegment const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ServicePathSegments__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServicePathSegment > *)new std::vector< RobotRaconteur::ServicePathSegment >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ServicePathSegments__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< RobotRaconteur::ServicePathSegment > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > *ptr = (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_ServicePathSegments" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_ServicePathSegments" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServicePathSegment > *)new std::vector< RobotRaconteur::ServicePathSegment >((std::vector< RobotRaconteur::ServicePathSegment > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_empty" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< RobotRaconteur::ServicePathSegment > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServicePathSegment >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_size" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::ServicePathSegment > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  std::vector< RobotRaconteur::ServicePathSegment > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServicePathSegments_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_swap" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServicePathSegments_swap" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServicePathSegments_swap" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServicePathSegment >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_begin" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServicePathSegment >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServicePathSegment >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_end" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServicePathSegment >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServicePathSegment >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_rbegin" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServicePathSegment >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServicePathSegment >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_rend" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServicePathSegment >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_clear" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< RobotRaconteur::ServicePathSegment > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_get_allocator" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::ServicePathSegment > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< RobotRaconteur::ServicePathSegment >::allocator_type(static_cast< const std::vector< RobotRaconteur::ServicePathSegment >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_RobotRaconteur__ServicePathSegment_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ServicePathSegments__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< RobotRaconteur::ServicePathSegment > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_ServicePathSegments" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::size_type""'");
  } 
  arg1 = static_cast< std::vector< RobotRaconteur::ServicePathSegment >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServicePathSegment > *)new std::vector< RobotRaconteur::ServicePathSegment >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_pop_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  std::vector< RobotRaconteur::ServicePathSegment >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_resize" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ServicePathSegments_resize" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServicePathSegment >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  std::vector< RobotRaconteur::ServicePathSegment >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< RobotRaconteur::ServicePathSegment >::iterator result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_erase" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "ServicePathSegments_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServicePathSegment >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServicePathSegment >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "ServicePathSegments_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServicePathSegment >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  std::vector< RobotRaconteur::ServicePathSegment >::iterator arg2 ;
  std::vector< RobotRaconteur::ServicePathSegment >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  std::vector< RobotRaconteur::ServicePathSegment >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_erase" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "ServicePathSegments_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServicePathSegment >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServicePathSegment >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "ServicePathSegments_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "ServicePathSegments_erase" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServicePathSegment >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServicePathSegment >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "ServicePathSegments_erase" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServicePathSegment >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "ServicePathSegments_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServicePathSegment >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_ServicePathSegments_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServicePathSegment >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServicePathSegment >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_ServicePathSegments_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'ServicePathSegments_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServicePathSegment >::erase(std::vector< RobotRaconteur::ServicePathSegment >::iterator)\n"
    "    std::vector< RobotRaconteur::ServicePathSegment >::erase(std::vector< RobotRaconteur::ServicePathSegment >::iterator,std::vector< RobotRaconteur::ServicePathSegment >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_ServicePathSegments__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment >::size_type arg1 ;
  std::vector< RobotRaconteur::ServicePathSegment >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  std::vector< RobotRaconteur::ServicePathSegment > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_ServicePathSegments" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::size_type""'");
  } 
  arg1 = static_cast< std::vector< RobotRaconteur::ServicePathSegment >::size_type >(val1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServicePathSegment,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_ServicePathSegments" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_ServicePathSegments" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServicePathSegment > *)new std::vector< RobotRaconteur::ServicePathSegment >(arg1,(std::vector< RobotRaconteur::ServicePathSegment >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ServicePathSegments(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_ServicePathSegments", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_ServicePathSegments__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_ServicePathSegments__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_ServicePathSegments__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_RobotRaconteur__ServicePathSegment, SWIG_POINTER_NO_NULL | 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_ServicePathSegments__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_ServicePathSegments'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServicePathSegment >::vector()\n"
    "    std::vector< RobotRaconteur::ServicePathSegment >::vector(std::vector< RobotRaconteur::ServicePathSegment > const &)\n"
    "    std::vector< RobotRaconteur::ServicePathSegment >::vector(std::vector< RobotRaconteur::ServicePathSegment >::size_type)\n"
    "    std::vector< RobotRaconteur::ServicePathSegment >::vector(std::vector< RobotRaconteur::ServicePathSegment >::size_type,std::vector< RobotRaconteur::ServicePathSegment >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  std::vector< RobotRaconteur::ServicePathSegment >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServicePathSegments_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_push_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServicePathSegment,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServicePathSegments_push_back" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServicePathSegments_push_back" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< RobotRaconteur::ServicePathSegment >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServicePathSegment >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_front" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServicePathSegment >::value_type *) &((std::vector< RobotRaconteur::ServicePathSegment > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServicePathSegment, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::ServicePathSegment >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServicePathSegment >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServicePathSegment >::value_type *) &((std::vector< RobotRaconteur::ServicePathSegment > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServicePathSegment, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::ServicePathSegment >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  std::vector< RobotRaconteur::ServicePathSegment >::size_type arg2 ;
  std::vector< RobotRaconteur::ServicePathSegment >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServicePathSegments_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_assign" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ServicePathSegments_assign" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServicePathSegment >::size_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServicePathSegment,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "ServicePathSegments_assign" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServicePathSegments_assign" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< RobotRaconteur::ServicePathSegment >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  std::vector< RobotRaconteur::ServicePathSegment >::size_type arg2 ;
  std::vector< RobotRaconteur::ServicePathSegment >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_resize" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ServicePathSegments_resize" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServicePathSegment >::size_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServicePathSegment,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "ServicePathSegments_resize" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServicePathSegments_resize" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< RobotRaconteur::ServicePathSegment >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "ServicePathSegments_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_ServicePathSegments_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__ServicePathSegment, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_ServicePathSegments_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'ServicePathSegments_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServicePathSegment >::resize(std::vector< RobotRaconteur::ServicePathSegment >::size_type)\n"
    "    std::vector< RobotRaconteur::ServicePathSegment >::resize(std::vector< RobotRaconteur::ServicePathSegment >::size_type,std::vector< RobotRaconteur::ServicePathSegment >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  std::vector< RobotRaconteur::ServicePathSegment >::iterator arg2 ;
  std::vector< RobotRaconteur::ServicePathSegment >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  std::vector< RobotRaconteur::ServicePathSegment >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_insert" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "ServicePathSegments_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServicePathSegment >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServicePathSegment >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "ServicePathSegments_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServicePathSegment,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "ServicePathSegments_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServicePathSegments_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg__insert__SWIG_0(arg1,arg2,(RobotRaconteur::ServicePathSegment const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServicePathSegment >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  std::vector< RobotRaconteur::ServicePathSegment >::iterator arg2 ;
  std::vector< RobotRaconteur::ServicePathSegment >::size_type arg3 ;
  std::vector< RobotRaconteur::ServicePathSegment >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  void *argp4 = 0 ;
  int res4 = 0 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_insert" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "ServicePathSegments_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServicePathSegment >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServicePathSegment >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "ServicePathSegments_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "ServicePathSegments_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::size_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ServicePathSegment >::size_type >(val3);
  res4 = SWIG_ConvertPtr(swig_obj[3], &argp4, SWIGTYPE_p_RobotRaconteur__ServicePathSegment,  0  | 0);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "ServicePathSegments_insert" "', argument " "4"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::value_type const &""'"); 
  }
  if (!argp4) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServicePathSegments_insert" "', argument " "4"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::value_type const &""'"); 
  }
  arg4 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment >::value_type * >(argp4);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_RobotRaconteur_ServicePathSegment_Sg__insert__SWIG_1(arg1,arg2,arg3,(RobotRaconteur::ServicePathSegment const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "ServicePathSegments_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServicePathSegment >::iterator > *>(iter) != 0));
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__ServicePathSegment, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_ServicePathSegments_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServicePathSegment >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_RobotRaconteur__ServicePathSegment, SWIG_POINTER_NO_NULL | 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_ServicePathSegments_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'ServicePathSegments_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServicePathSegment >::insert(std::vector< RobotRaconteur::ServicePathSegment >::iterator,std::vector< RobotRaconteur::ServicePathSegment >::value_type const &)\n"
    "    std::vector< RobotRaconteur::ServicePathSegment >::insert(std::vector< RobotRaconteur::ServicePathSegment >::iterator,std::vector< RobotRaconteur::ServicePathSegment >::size_type,std::vector< RobotRaconteur::ServicePathSegment >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  std::vector< RobotRaconteur::ServicePathSegment >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServicePathSegments_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_reserve" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ServicePathSegments_reserve" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServicePathSegment >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServicePathSegment >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServicePathSegments_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServicePathSegment >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServicePathSegments_capacity" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::ServicePathSegment > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_ServicePathSegments(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment > *arg1 = (std::vector< RobotRaconteur::ServicePathSegment > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_ServicePathSegments" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServicePathSegment > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *ServicePathSegments_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *ServicePathSegments_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_EncodeServicePathIndex(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EncodeServicePathIndex" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "EncodeServicePathIndex" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::EncodeServicePathIndex((std::string const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_DecodeServicePathIndex(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "DecodeServicePathIndex" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "DecodeServicePathIndex" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::DecodeServicePathIndex((std::string const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ParseServicePath(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ParseServicePath" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ParseServicePath" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::ParseServicePath((std::string const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > >(result));
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_BuildServicePath(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > *ptr = (std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "BuildServicePath" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "BuildServicePath" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::BuildServicePath((std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_IsStringName(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::string_ref arg1 ;
  void *argp1 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_boost__string_ref,  0  | 0);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "IsStringName" "', argument " "1"" of type '" "boost::string_ref""'"); 
    }  
    if (!argp1) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "IsStringName" "', argument " "1"" of type '" "boost::string_ref""'");
    } else {
      boost::string_ref * temp = reinterpret_cast< boost::string_ref * >(argp1);
      arg1 = *temp;
      if (SWIG_IsNewObj(res1)) delete temp;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)RobotRaconteur::IsStringName(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_IsStringScopedName(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::string_ref arg1 ;
  void *argp1 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_boost__string_ref,  0  | 0);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "IsStringScopedName" "', argument " "1"" of type '" "boost::string_ref""'"); 
    }  
    if (!argp1) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "IsStringScopedName" "', argument " "1"" of type '" "boost::string_ref""'");
    } else {
      boost::string_ref * temp = reinterpret_cast< boost::string_ref * >(argp1);
      arg1 = *temp;
      if (SWIG_IsNewObj(res1)) delete temp;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)RobotRaconteur::IsStringScopedName(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_IsStringUUID(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::string_ref arg1 ;
  void *argp1 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_boost__string_ref,  0  | 0);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "IsStringUUID" "', argument " "1"" of type '" "boost::string_ref""'"); 
    }  
    if (!argp1) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "IsStringUUID" "', argument " "1"" of type '" "boost::string_ref""'");
    } else {
      boost::string_ref * temp = reinterpret_cast< boost::string_ref * >(argp1);
      arg1 = *temp;
      if (SWIG_IsNewObj(res1)) delete temp;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)RobotRaconteur::IsStringUUID(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_IsStringIdentifier(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::string_ref arg1 ;
  void *argp1 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_boost__string_ref,  0  | 0);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "IsStringIdentifier" "', argument " "1"" of type '" "boost::string_ref""'"); 
    }  
    if (!argp1) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "IsStringIdentifier" "', argument " "1"" of type '" "boost::string_ref""'");
    } else {
      boost::string_ref * temp = reinterpret_cast< boost::string_ref * >(argp1);
      arg1 = *temp;
      if (SWIG_IsNewObj(res1)) delete temp;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)RobotRaconteur::IsStringIdentifier(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MultiDimArray_CalculateCopyIndicesIter_Next(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter *arg1 = (RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter *) 0 ;
  uint32_t *arg2 = 0 ;
  uint32_t *arg3 = 0 ;
  uint32_t *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter > *smartarg1 = 0 ;
  uint32_t temp2 ;
  int res2 = SWIG_TMPOBJ ;
  uint32_t temp3 ;
  int res3 = SWIG_TMPOBJ ;
  uint32_t temp4 ;
  int res4 = SWIG_TMPOBJ ;
  PyObject *swig_obj[1] ;
  bool result;
  
  arg2 = &temp2;
  arg3 = &temp3;
  arg4 = &temp4;
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__detail__MultiDimArray_CalculateCopyIndicesIter_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MultiDimArray_CalculateCopyIndicesIter_Next" "', argument " "1"" of type '" "RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->Next(*arg2,*arg3,*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  if (SWIG_IsTmpObj(res2)) {
    resultobj = SWIG_Python_AppendOutput(resultobj, SWIG_From_unsigned_SS_int((*arg2)));
  } else {
    int new_flags = SWIG_IsNewObj(res2) ? (SWIG_POINTER_OWN |  0 ) :  0 ;
    resultobj = SWIG_Python_AppendOutput(resultobj, SWIG_NewPointerObj((void*)(arg2), SWIGTYPE_p_unsigned_int, new_flags));
  }
  if (SWIG_IsTmpObj(res3)) {
    resultobj = SWIG_Python_AppendOutput(resultobj, SWIG_From_unsigned_SS_int((*arg3)));
  } else {
    int new_flags = SWIG_IsNewObj(res3) ? (SWIG_POINTER_OWN |  0 ) :  0 ;
    resultobj = SWIG_Python_AppendOutput(resultobj, SWIG_NewPointerObj((void*)(arg3), SWIGTYPE_p_unsigned_int, new_flags));
  }
  if (SWIG_IsTmpObj(res4)) {
    resultobj = SWIG_Python_AppendOutput(resultobj, SWIG_From_unsigned_SS_int((*arg4)));
  } else {
    int new_flags = SWIG_IsNewObj(res4) ? (SWIG_POINTER_OWN |  0 ) :  0 ;
    resultobj = SWIG_Python_AppendOutput(resultobj, SWIG_NewPointerObj((void*)(arg4), SWIGTYPE_p_unsigned_int, new_flags));
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_MultiDimArray_CalculateCopyIndicesIter(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter *arg1 = (RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__detail__MultiDimArray_CalculateCopyIndicesIter_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_MultiDimArray_CalculateCopyIndicesIter" "', argument " "1"" of type '" "RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *MultiDimArray_CalculateCopyIndicesIter_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__detail__MultiDimArray_CalculateCopyIndicesIter_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_MultiDimArray_CalculateCopyIndicesBeginIter(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< uint32_t,std::allocator< uint32_t > > *arg1 = 0 ;
  std::vector< uint32_t,std::allocator< uint32_t > > *arg2 = 0 ;
  std::vector< uint32_t,std::allocator< uint32_t > > *arg3 = 0 ;
  std::vector< uint32_t,std::allocator< uint32_t > > *arg4 = 0 ;
  std::vector< uint32_t,std::allocator< uint32_t > > *arg5 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  int res4 = SWIG_OLDOBJ ;
  int res5 = SWIG_OLDOBJ ;
  PyObject *swig_obj[5] ;
  boost::shared_ptr< RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter > result;
  
  if (!SWIG_Python_UnpackTuple(args, "MultiDimArray_CalculateCopyIndicesBeginIter", 5, 5, swig_obj)) SWIG_fail;
  {
    std::vector< unsigned int,std::allocator< unsigned int > > *ptr = (std::vector< unsigned int,std::allocator< unsigned int > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MultiDimArray_CalculateCopyIndicesBeginIter" "', argument " "1"" of type '" "std::vector< uint32_t,std::allocator< uint32_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MultiDimArray_CalculateCopyIndicesBeginIter" "', argument " "1"" of type '" "std::vector< uint32_t,std::allocator< uint32_t > > const &""'"); 
    }
    arg1 = ptr;
  }
  {
    std::vector< unsigned int,std::allocator< unsigned int > > *ptr = (std::vector< unsigned int,std::allocator< unsigned int > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MultiDimArray_CalculateCopyIndicesBeginIter" "', argument " "2"" of type '" "std::vector< uint32_t,std::allocator< uint32_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MultiDimArray_CalculateCopyIndicesBeginIter" "', argument " "2"" of type '" "std::vector< uint32_t,std::allocator< uint32_t > > const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::vector< unsigned int,std::allocator< unsigned int > > *ptr = (std::vector< unsigned int,std::allocator< unsigned int > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "MultiDimArray_CalculateCopyIndicesBeginIter" "', argument " "3"" of type '" "std::vector< uint32_t,std::allocator< uint32_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MultiDimArray_CalculateCopyIndicesBeginIter" "', argument " "3"" of type '" "std::vector< uint32_t,std::allocator< uint32_t > > const &""'"); 
    }
    arg3 = ptr;
  }
  {
    std::vector< unsigned int,std::allocator< unsigned int > > *ptr = (std::vector< unsigned int,std::allocator< unsigned int > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "MultiDimArray_CalculateCopyIndicesBeginIter" "', argument " "4"" of type '" "std::vector< uint32_t,std::allocator< uint32_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MultiDimArray_CalculateCopyIndicesBeginIter" "', argument " "4"" of type '" "std::vector< uint32_t,std::allocator< uint32_t > > const &""'"); 
    }
    arg4 = ptr;
  }
  {
    std::vector< unsigned int,std::allocator< unsigned int > > *ptr = (std::vector< unsigned int,std::allocator< unsigned int > > *)0;
    res5 = swig::asptr(swig_obj[4], &ptr);
    if (!SWIG_IsOK(res5)) {
      SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "MultiDimArray_CalculateCopyIndicesBeginIter" "', argument " "5"" of type '" "std::vector< uint32_t,std::allocator< uint32_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MultiDimArray_CalculateCopyIndicesBeginIter" "', argument " "5"" of type '" "std::vector< uint32_t,std::allocator< uint32_t > > const &""'"); 
    }
    arg5 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesBeginIter((std::vector< unsigned int,std::allocator< unsigned int > > const &)*arg1,(std::vector< unsigned int,std::allocator< unsigned int > > const &)*arg2,(std::vector< unsigned int,std::allocator< unsigned int > > const &)*arg3,(std::vector< unsigned int,std::allocator< unsigned int > > const &)*arg4,(std::vector< unsigned int,std::allocator< unsigned int > > const &)*arg5);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__detail__MultiDimArray_CalculateCopyIndicesIter_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_NodeID__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeID *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::NodeID *)new RobotRaconteur::NodeID();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__NodeID, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeID_ToString__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeID *arg1 = (RobotRaconteur::NodeID *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::string result;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeID, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeID_ToString" "', argument " "1"" of type '" "RobotRaconteur::NodeID const *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeID * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((RobotRaconteur::NodeID const *)arg1)->ToString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeID_ToString__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeID *arg1 = (RobotRaconteur::NodeID *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  std::string result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeID, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeID_ToString" "', argument " "1"" of type '" "RobotRaconteur::NodeID const *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeID * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "NodeID_ToString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "NodeID_ToString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((RobotRaconteur::NodeID const *)arg1)->ToString((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeID_ToString(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "NodeID_ToString", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    void *vptr = 0;
    int res = SWIG_ConvertPtr(argv[0], &vptr, SWIGTYPE_p_RobotRaconteur__NodeID, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_NodeID_ToString__SWIG_0(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    void *vptr = 0;
    int res = SWIG_ConvertPtr(argv[0], &vptr, SWIGTYPE_p_RobotRaconteur__NodeID, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_NodeID_ToString__SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'NodeID_ToString'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::NodeID::ToString() const\n"
    "    RobotRaconteur::NodeID::ToString(std::string const &) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_NodeID_NewUniqueID(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeID result;
  
  if (!SWIG_Python_UnpackTuple(args, "NodeID_NewUniqueID", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::NodeID::NewUniqueID();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::NodeID(static_cast< const RobotRaconteur::NodeID& >(result))), SWIGTYPE_p_RobotRaconteur__NodeID, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeID___eq__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeID *arg1 = (RobotRaconteur::NodeID *) 0 ;
  RobotRaconteur::NodeID *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "NodeID___eq__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeID, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeID___eq__" "', argument " "1"" of type '" "RobotRaconteur::NodeID const *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeID * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__NodeID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "NodeID___eq__" "', argument " "2"" of type '" "RobotRaconteur::NodeID const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "NodeID___eq__" "', argument " "2"" of type '" "RobotRaconteur::NodeID const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::NodeID * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((RobotRaconteur::NodeID const *)arg1)->operator ==((RobotRaconteur::NodeID const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  PyErr_Clear();
  Py_INCREF(Py_NotImplemented);
  return Py_NotImplemented;
}


SWIGINTERN PyObject *_wrap_NodeID___ne__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeID *arg1 = (RobotRaconteur::NodeID *) 0 ;
  RobotRaconteur::NodeID *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "NodeID___ne__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeID, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeID___ne__" "', argument " "1"" of type '" "RobotRaconteur::NodeID const *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeID * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__NodeID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "NodeID___ne__" "', argument " "2"" of type '" "RobotRaconteur::NodeID const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "NodeID___ne__" "', argument " "2"" of type '" "RobotRaconteur::NodeID const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::NodeID * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((RobotRaconteur::NodeID const *)arg1)->operator !=((RobotRaconteur::NodeID const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  PyErr_Clear();
  Py_INCREF(Py_NotImplemented);
  return Py_NotImplemented;
}


SWIGINTERN PyObject *_wrap_NodeID___lt__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeID *arg1 = (RobotRaconteur::NodeID *) 0 ;
  RobotRaconteur::NodeID *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "NodeID___lt__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeID, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeID___lt__" "', argument " "1"" of type '" "RobotRaconteur::NodeID const *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeID * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__NodeID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "NodeID___lt__" "', argument " "2"" of type '" "RobotRaconteur::NodeID const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "NodeID___lt__" "', argument " "2"" of type '" "RobotRaconteur::NodeID const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::NodeID * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((RobotRaconteur::NodeID const *)arg1)->operator <((RobotRaconteur::NodeID const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  PyErr_Clear();
  Py_INCREF(Py_NotImplemented);
  return Py_NotImplemented;
}


SWIGINTERN PyObject *_wrap_NodeID_IsAnyNode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeID *arg1 = (RobotRaconteur::NodeID *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeID, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeID_IsAnyNode" "', argument " "1"" of type '" "RobotRaconteur::NodeID *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeID * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->IsAnyNode();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeID_GetAny(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeID result;
  
  if (!SWIG_Python_UnpackTuple(args, "NodeID_GetAny", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::NodeID::GetAny();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::NodeID(static_cast< const RobotRaconteur::NodeID& >(result))), SWIGTYPE_p_RobotRaconteur__NodeID, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_NodeID__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::array< uint8_t,16 > arg1 ;
  void *argp1 ;
  int res1 = 0 ;
  RobotRaconteur::NodeID *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_boost__arrayT_unsigned_char_16_t,  0  | 0);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_NodeID" "', argument " "1"" of type '" "boost::array< uint8_t,16 >""'"); 
    }  
    if (!argp1) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_NodeID" "', argument " "1"" of type '" "boost::array< uint8_t,16 >""'");
    } else {
      boost::array< uint8_t,16 > * temp = reinterpret_cast< boost::array< uint8_t,16 > * >(argp1);
      arg1 = *temp;
      if (SWIG_IsNewObj(res1)) delete temp;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::NodeID *)new RobotRaconteur::NodeID(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__NodeID, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_NodeID__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  RobotRaconteur::NodeID *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_NodeID" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_NodeID" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::NodeID *)new RobotRaconteur::NodeID((std::string const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__NodeID, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_NodeID__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeID *arg1 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  RobotRaconteur::NodeID *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_RobotRaconteur__NodeID,  0  | 0);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_NodeID" "', argument " "1"" of type '" "RobotRaconteur::NodeID const &""'"); 
  }
  if (!argp1) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_NodeID" "', argument " "1"" of type '" "RobotRaconteur::NodeID const &""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeID * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::NodeID *)new RobotRaconteur::NodeID((RobotRaconteur::NodeID const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__NodeID, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_NodeID(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[2] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_NodeID", 0, 1, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_NodeID__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__arrayT_unsigned_char_16_t, SWIG_POINTER_NO_NULL | 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_NodeID__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_RobotRaconteur__NodeID, SWIG_POINTER_NO_NULL | 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_NodeID__SWIG_3(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = SWIG_AsPtr_std_string(argv[0], (std::string**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_NodeID__SWIG_2(self, argc, argv);
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_NodeID'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::NodeID::NodeID()\n"
    "    RobotRaconteur::NodeID::NodeID(boost::array< uint8_t,16 >)\n"
    "    RobotRaconteur::NodeID::NodeID(std::string const &)\n"
    "    RobotRaconteur::NodeID::NodeID(RobotRaconteur::NodeID const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_NodeID_ToByteArray(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeID *arg1 = (RobotRaconteur::NodeID *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::array< uint8_t,16 > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeID, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeID_ToByteArray" "', argument " "1"" of type '" "RobotRaconteur::NodeID *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeID * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ToByteArray();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  resultobj=PyByteArray_FromStringAndSize((const char*)&result[0],16);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeID_FromByteArray(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  char *arg1 ;
  char temp1[16] ;
  int res1 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::NodeID result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_AsCharArray(swig_obj[0], temp1, 16);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeID_FromByteArray" "', argument " "1"" of type '" "char const [16]""'");
  }
  arg1 = reinterpret_cast< char * >(temp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_NodeID_FromByteArray((char const (*))arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::NodeID(static_cast< const RobotRaconteur::NodeID& >(result))), SWIGTYPE_p_RobotRaconteur__NodeID, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_NodeID(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeID *arg1 = (RobotRaconteur::NodeID *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeID, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_NodeID" "', argument " "1"" of type '" "RobotRaconteur::NodeID *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeID * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *NodeID_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__NodeID, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *NodeID_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_vectorptr_messageentry_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_iterator" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry___nonzero__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____nonzero__((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry___bool__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____bool__((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry___len__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____len__((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type arg2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_messageentry___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry___getslice__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_messageentry___getslice__" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_messageentry___getslice__" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > *)std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type arg2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry___setslice__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_messageentry___setslice__" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_messageentry___setslice__" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type arg2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type arg3 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry___setslice__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_messageentry___setslice__" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_messageentry___setslice__" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type >(val3);
  {
    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > *ptr = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorptr_messageentry___setslice__" "', argument " "4"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_messageentry___setslice__" "', argument " "4"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_messageentry___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vectorptr_messageentry___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorptr_messageentry___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_messageentry___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::__setslice__(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type)\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::__setslice__(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type arg2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_messageentry___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry___delslice__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_messageentry___delslice__" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_messageentry___delslice__" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry___delitem__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_messageentry___delitem__" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry___getitem__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageentry___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > *)std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry___setitem__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageentry___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > *ptr = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_messageentry___setitem__" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_messageentry___setitem__" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____setitem____SWIG_0(arg1,arg2,(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry___setitem__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageentry___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry___delitem__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageentry___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_messageentry___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_messageentry___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_messageentry___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_messageentry___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::__delitem__(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type)\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartresult = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry___getitem__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_messageentry___getitem__" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type *) &std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____getitem____SWIG_1((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (*result) {
  //  intrusive_ptr_add_ref(result->get());
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry >(result->get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageEntry >());
  //} else {
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&resultobj = 0;
  //}
  
  if(*(result)) {
    intrusive_ptr_add_ref(result->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageEntry >(result->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageEntry >());
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, SWIG_POINTER_OWN);
  
  (void)swig::container_owner<swig::traits<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_messageentry___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_messageentry___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_messageentry___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_messageentry___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::__getitem__(PySliceObject *)\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::__getitem__(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type arg2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type tempnull3 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type temp3 ;
  int newmem3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry___setitem__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_messageentry___setitem__" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type >(val2);
  
  // intrusive_ptr by reference
  //if ( swig_obj[2] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry >**)&swig_obj[2];
  //	temp3 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageEntry >(smartarg->get(), true);
  //	arg3 = &temp3;
  //} else {
  //  arg3 = &tempnull3;
  //}
  
  res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t,  0 , &newmem3);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_messageentry___setitem__" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type const &""'");
  }
  if (argp3) {
    temp3 = boost::intrusive_ptr<  RobotRaconteur::MessageEntry >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageEntry >* >(argp3)->get(),true);
    if (newmem3 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageEntry >* >(argp3);
    arg3 = &temp3;
  }
  else {
    arg3 = &tempnull3;
  }
  
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg____setitem____SWIG_2(arg1,arg2,(boost::intrusive_ptr< RobotRaconteur::MessageEntry > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_messageentry___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_messageentry___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_messageentry___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_messageentry___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_messageentry___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::__setitem__(PySliceObject *,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > const &)\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::__setitem__(PySliceObject *)\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::__setitem__(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::difference_type,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartresult = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_pop" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageEntry >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageEntry >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageEntry >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type tempnull2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_messageentry_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_append" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageEntry >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_messageentry_append" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageEntry >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageEntry >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageEntry >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg__append(arg1,(boost::intrusive_ptr< RobotRaconteur::MessageEntry > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_messageentry__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *)new std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_messageentry__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > *ptr = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vectorptr_messageentry" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vectorptr_messageentry" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *)new std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_empty" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_size" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_messageentry_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_swap" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_messageentry_swap" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_messageentry_swap" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_begin" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_end" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_rbegin" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_rend" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_clear" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_get_allocator" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::allocator_type(static_cast< const std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_messageentry__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorptr_messageentry" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type""'");
  } 
  arg1 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *)new std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_pop_back" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_resize" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_messageentry_resize" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_erase" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageentry_erase" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageentry_erase" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator arg2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_erase" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageentry_erase" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageentry_erase" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageentry_erase" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageentry_erase" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_messageentry_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vectorptr_messageentry_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vectorptr_messageentry_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_messageentry_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::erase(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator)\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::erase(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_messageentry__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type arg1 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type tempnull2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type temp2 ;
  int newmem2 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorptr_messageentry" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type""'");
  } 
  arg1 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type >(val1);
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageEntry >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_vectorptr_messageentry" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageEntry >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageEntry >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageEntry >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *)new std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >(arg1,(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_messageentry(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vectorptr_messageentry", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vectorptr_messageentry__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vectorptr_messageentry__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vectorptr_messageentry__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_vectorptr_messageentry__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vectorptr_messageentry'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::vector()\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::vector(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const &)\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::vector(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type)\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::vector(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type tempnull2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_messageentry_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_push_back" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageEntry >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_messageentry_push_back" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageEntry >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageEntry >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageEntry >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartresult = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_front" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type *) &((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (*result) {
  //  intrusive_ptr_add_ref(result->get());
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry >(result->get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageEntry >());
  //} else {
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&resultobj = 0;
  //}
  
  if(*(result)) {
    intrusive_ptr_add_ref(result->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageEntry >(result->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageEntry >());
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, SWIG_POINTER_OWN);
  
  (void)swig::container_owner<swig::traits<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartresult = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_back" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type *) &((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (*result) {
  //  intrusive_ptr_add_ref(result->get());
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry >(result->get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageEntry >());
  //} else {
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&resultobj = 0;
  //}
  
  if(*(result)) {
    intrusive_ptr_add_ref(result->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageEntry >(result->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageEntry >());
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, SWIG_POINTER_OWN);
  
  (void)swig::container_owner<swig::traits<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type arg2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type tempnull3 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type temp3 ;
  int newmem3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_messageentry_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_assign" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_messageentry_assign" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type >(val2);
  
  // intrusive_ptr by reference
  //if ( swig_obj[2] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry >**)&swig_obj[2];
  //	temp3 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageEntry >(smartarg->get(), true);
  //	arg3 = &temp3;
  //} else {
  //  arg3 = &tempnull3;
  //}
  
  res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t,  0 , &newmem3);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_messageentry_assign" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type const &""'");
  }
  if (argp3) {
    temp3 = boost::intrusive_ptr<  RobotRaconteur::MessageEntry >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageEntry >* >(argp3)->get(),true);
    if (newmem3 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageEntry >* >(argp3);
    arg3 = &temp3;
  }
  else {
    arg3 = &tempnull3;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type arg2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type tempnull3 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type temp3 ;
  int newmem3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_resize" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_messageentry_resize" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type >(val2);
  
  // intrusive_ptr by reference
  //if ( swig_obj[2] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry >**)&swig_obj[2];
  //	temp3 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageEntry >(smartarg->get(), true);
  //	arg3 = &temp3;
  //} else {
  //  arg3 = &tempnull3;
  //}
  
  res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t,  0 , &newmem3);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_messageentry_resize" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type const &""'");
  }
  if (argp3) {
    temp3 = boost::intrusive_ptr<  RobotRaconteur::MessageEntry >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageEntry >* >(argp3)->get(),true);
    if (newmem3 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageEntry >* >(argp3);
    arg3 = &temp3;
  }
  else {
    arg3 = &tempnull3;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_messageentry_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_messageentry_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_messageentry_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_messageentry_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::resize(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type)\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::resize(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator arg2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type tempnull3 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type temp3 ;
  int newmem3 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_insert" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageentry_insert" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageentry_insert" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator""'");
    }
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[2] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry >**)&swig_obj[2];
  //	temp3 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageEntry >(smartarg->get(), true);
  //	arg3 = &temp3;
  //} else {
  //  arg3 = &tempnull3;
  //}
  
  res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t,  0 , &newmem3);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_messageentry_insert" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type const &""'");
  }
  if (argp3) {
    temp3 = boost::intrusive_ptr<  RobotRaconteur::MessageEntry >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageEntry >* >(argp3)->get(),true);
    if (newmem3 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageEntry >* >(argp3);
    arg3 = &temp3;
  }
  else {
    arg3 = &tempnull3;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg__insert__SWIG_0(arg1,arg2,(boost::intrusive_ptr< RobotRaconteur::MessageEntry > const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator arg2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type arg3 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  void *argp4 ;
  int res4 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type tempnull4 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type temp4 ;
  int newmem4 = 0 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_insert" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageentry_insert" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageentry_insert" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_messageentry_insert" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type""'");
  } 
  arg3 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type >(val3);
  
  // intrusive_ptr by reference
  //if ( swig_obj[3] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry >**)&swig_obj[3];
  //	temp4 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageEntry >(smartarg->get(), true);
  //	arg4 = &temp4;
  //} else {
  //  arg4 = &tempnull4;
  //}
  
  res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t,  0 , &newmem4);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorptr_messageentry_insert" "', argument " "4"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type const &""'");
  }
  if (argp4) {
    temp4 = boost::intrusive_ptr<  RobotRaconteur::MessageEntry >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageEntry >* >(argp4)->get(),true);
    if (newmem4 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageEntry >* >(argp4);
    arg4 = &temp4;
  }
  else {
    arg4 = &tempnull4;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageEntry_Sg__Sg__insert__SWIG_1(arg1,arg2,arg3,(boost::intrusive_ptr< RobotRaconteur::MessageEntry > const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_messageentry_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator > *>(iter) != 0));
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_messageentry_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorptr_messageentry_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_messageentry_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::insert(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type const &)\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::insert(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::iterator,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_messageentry_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_reserve" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_messageentry_reserve" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageentry_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageentry_capacity" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vectorptr_messageentry(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vectorptr_messageentry" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vectorptr_messageentry_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vectorptr_messageentry_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_vectorptr_messageelement_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_iterator" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement___nonzero__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____nonzero__((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement___bool__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____bool__((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement___len__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____len__((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type arg2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_messageelement___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement___getslice__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_messageelement___getslice__" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_messageelement___getslice__" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *)std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type arg2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement___setslice__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_messageelement___setslice__" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_messageelement___setslice__" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type arg2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type arg3 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement___setslice__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_messageelement___setslice__" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_messageelement___setslice__" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type >(val3);
  {
    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *ptr = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorptr_messageelement___setslice__" "', argument " "4"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_messageelement___setslice__" "', argument " "4"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_messageelement___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vectorptr_messageelement___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorptr_messageelement___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_messageelement___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::__setslice__(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type)\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::__setslice__(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type arg2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_messageelement___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement___delslice__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_messageelement___delslice__" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_messageelement___delslice__" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement___delitem__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_messageelement___delitem__" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement___getitem__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageelement___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *)std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement___setitem__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageelement___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *ptr = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_messageelement___setitem__" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_messageelement___setitem__" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____setitem____SWIG_0(arg1,arg2,(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement___setitem__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageelement___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement___delitem__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageelement___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_messageelement___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_messageelement___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_messageelement___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_messageelement___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::__delitem__(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type)\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement___getitem__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_messageelement___getitem__" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type *) &std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____getitem____SWIG_1((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (*result) {
  //  intrusive_ptr_add_ref(result->get());
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result->get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(result)) {
    intrusive_ptr_add_ref(result->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >(result->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  (void)swig::container_owner<swig::traits<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_messageelement___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_messageelement___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_messageelement___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_messageelement___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::__getitem__(PySliceObject *)\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::__getitem__(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type arg2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type tempnull3 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type temp3 ;
  int newmem3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement___setitem__" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_messageelement___setitem__" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type >(val2);
  
  // intrusive_ptr by reference
  //if ( swig_obj[2] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[2];
  //	temp3 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg3 = &temp3;
  //} else {
  //  arg3 = &tempnull3;
  //}
  
  res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem3);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_messageelement___setitem__" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type const &""'");
  }
  if (argp3) {
    temp3 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp3)->get(),true);
    if (newmem3 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp3);
    arg3 = &temp3;
  }
  else {
    arg3 = &tempnull3;
  }
  
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg____setitem____SWIG_2(arg1,arg2,(boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_messageelement___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_messageelement___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_messageelement___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_messageelement___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_messageelement___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::__setitem__(PySliceObject *,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &)\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::__setitem__(PySliceObject *)\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::__setitem__(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::difference_type,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_pop" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type tempnull2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_messageelement_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_append" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_messageelement_append" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg__append(arg1,(boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_messageelement__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *)new std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_messageelement__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *ptr = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vectorptr_messageelement" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vectorptr_messageelement" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *)new std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_empty" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_size" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_messageelement_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_swap" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_messageelement_swap" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_messageelement_swap" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_begin" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_end" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_rbegin" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_rend" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_clear" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_get_allocator" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::allocator_type(static_cast< const std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_messageelement__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorptr_messageelement" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type""'");
  } 
  arg1 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *)new std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_pop_back" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_resize" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_messageelement_resize" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_erase" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageelement_erase" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageelement_erase" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator arg2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_erase" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageelement_erase" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageelement_erase" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageelement_erase" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageelement_erase" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_messageelement_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vectorptr_messageelement_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vectorptr_messageelement_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_messageelement_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::erase(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator)\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::erase(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_messageelement__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type arg1 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type tempnull2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type temp2 ;
  int newmem2 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorptr_messageelement" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type""'");
  } 
  arg1 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type >(val1);
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_vectorptr_messageelement" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *)new std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >(arg1,(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_messageelement(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vectorptr_messageelement", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vectorptr_messageelement__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vectorptr_messageelement__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vectorptr_messageelement__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_vectorptr_messageelement__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vectorptr_messageelement'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::vector()\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::vector(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const &)\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::vector(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type)\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::vector(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type tempnull2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_messageelement_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_push_back" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_messageelement_push_back" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_front" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type *) &((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (*result) {
  //  intrusive_ptr_add_ref(result->get());
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result->get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(result)) {
    intrusive_ptr_add_ref(result->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >(result->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  (void)swig::container_owner<swig::traits<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_back" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type *) &((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (*result) {
  //  intrusive_ptr_add_ref(result->get());
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result->get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(result)) {
    intrusive_ptr_add_ref(result->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >(result->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  (void)swig::container_owner<swig::traits<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type arg2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type tempnull3 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type temp3 ;
  int newmem3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_messageelement_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_assign" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_messageelement_assign" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type >(val2);
  
  // intrusive_ptr by reference
  //if ( swig_obj[2] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[2];
  //	temp3 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg3 = &temp3;
  //} else {
  //  arg3 = &tempnull3;
  //}
  
  res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem3);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_messageelement_assign" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type const &""'");
  }
  if (argp3) {
    temp3 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp3)->get(),true);
    if (newmem3 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp3);
    arg3 = &temp3;
  }
  else {
    arg3 = &tempnull3;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type arg2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type tempnull3 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type temp3 ;
  int newmem3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_resize" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_messageelement_resize" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type >(val2);
  
  // intrusive_ptr by reference
  //if ( swig_obj[2] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[2];
  //	temp3 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg3 = &temp3;
  //} else {
  //  arg3 = &tempnull3;
  //}
  
  res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem3);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_messageelement_resize" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type const &""'");
  }
  if (argp3) {
    temp3 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp3)->get(),true);
    if (newmem3 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp3);
    arg3 = &temp3;
  }
  else {
    arg3 = &tempnull3;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_messageelement_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_messageelement_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_messageelement_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_messageelement_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::resize(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type)\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::resize(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator arg2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type tempnull3 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type temp3 ;
  int newmem3 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_insert" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageelement_insert" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageelement_insert" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator""'");
    }
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[2] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[2];
  //	temp3 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg3 = &temp3;
  //} else {
  //  arg3 = &tempnull3;
  //}
  
  res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem3);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_messageelement_insert" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type const &""'");
  }
  if (argp3) {
    temp3 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp3)->get(),true);
    if (newmem3 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp3);
    arg3 = &temp3;
  }
  else {
    arg3 = &tempnull3;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg__insert__SWIG_0(arg1,arg2,(boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator arg2 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type arg3 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  void *argp4 ;
  int res4 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type tempnull4 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type temp4 ;
  int newmem4 = 0 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_insert" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageelement_insert" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_messageelement_insert" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_messageelement_insert" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type""'");
  } 
  arg3 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type >(val3);
  
  // intrusive_ptr by reference
  //if ( swig_obj[3] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[3];
  //	temp4 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg4 = &temp4;
  //} else {
  //  arg4 = &tempnull4;
  //}
  
  res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem4);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorptr_messageelement_insert" "', argument " "4"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type const &""'");
  }
  if (argp4) {
    temp4 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp4)->get(),true);
    if (newmem4 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp4);
    arg4 = &temp4;
  }
  else {
    arg4 = &tempnull4;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_boost_intrusive_ptr_Sl_RobotRaconteur_MessageElement_Sg__Sg__insert__SWIG_1(arg1,arg2,arg3,(boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_messageelement_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator > *>(iter) != 0));
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_messageelement_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorptr_messageelement_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_messageelement_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::insert(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type const &)\n"
    "    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::insert(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::iterator,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type,std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_messageelement_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_reserve" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_messageelement_reserve" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_messageelement_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_messageelement_capacity" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vectorptr_messageelement(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *arg1 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vectorptr_messageelement" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vectorptr_messageelement_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vectorptr_messageelement_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_Message_header_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::Message *arg1 = (RobotRaconteur::Message *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageHeader > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::Message > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::Message > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageHeader > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageHeader > temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "Message_header_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::Message > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::Message *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Message_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Message_header_set" "', argument " "1"" of type '" "RobotRaconteur::Message *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    arg1 = const_cast< RobotRaconteur::Message * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    arg1 = const_cast< RobotRaconteur::Message * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageHeader >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Message_header_set" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageHeader > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageHeader >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageHeader >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageHeader >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->header = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_Message_header_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::Message *arg1 = (RobotRaconteur::Message *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::Message > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::Message > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageHeader > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::Message > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::Message *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Message_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Message_header_get" "', argument " "1"" of type '" "RobotRaconteur::Message *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    arg1 = const_cast< RobotRaconteur::Message * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    arg1 = const_cast< RobotRaconteur::Message * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::intrusive_ptr< RobotRaconteur::MessageHeader > *) & ((arg1)->header);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  
  //if (*result) {
  //  intrusive_ptr_add_ref(result->get());
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader >(result->get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageHeader >());
  //} else {
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&resultobj = 0;
  //}
  
  if(*(result)) {
    intrusive_ptr_add_ref(result->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageHeader >(result->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageHeader >());
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_Message_entries_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::Message *arg1 = (RobotRaconteur::Message *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > *arg2 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::Message > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::Message > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "Message_entries_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::Message > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::Message *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Message_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Message_entries_set" "', argument " "1"" of type '" "RobotRaconteur::Message *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    arg1 = const_cast< RobotRaconteur::Message * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    arg1 = const_cast< RobotRaconteur::Message * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Message_entries_set" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->entries = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_Message_entries_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::Message *arg1 = (RobotRaconteur::Message *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::Message > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::Message > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::Message > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::Message *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Message_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Message_entries_get" "', argument " "1"" of type '" "RobotRaconteur::Message *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    arg1 = const_cast< RobotRaconteur::Message * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    arg1 = const_cast< RobotRaconteur::Message * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > *)& ((arg1)->entries);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_Message(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::Message *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_Message", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::Message *)new RobotRaconteur::Message();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    //#if (SWIG_POINTER_NEW)
    //if (result) {
    //  intrusive_ptr_add_ref(result);
    //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::Message > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::Message >(result, SWIG_intrusive_deleter<  RobotRaconteur::Message >());
    //} else {
    //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::Message > **)&resultobj = 0;
    //}
    //#else
    //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::Message > **)&resultobj = result ? new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::Message >(result SWIG_NO_NULL_DELETER_0) : 0;
    //#endif
    if (SWIG_POINTER_NEW) {
      if (result) {
        intrusive_ptr_add_ref(result);
      }
      boost::shared_ptr<  RobotRaconteur::Message > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::Message >(result, SWIG_intrusive_deleter<  RobotRaconteur::Message >()) : 0;
      resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Message_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
    } else {
      boost::shared_ptr<  RobotRaconteur::Message > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::Message >(result SWIG_NO_NULL_DELETER_0) : 0;
      resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Message_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
    }
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_Message_ComputeSize(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::Message *arg1 = (RobotRaconteur::Message *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::Message > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::Message > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::Message > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::Message *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Message_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Message_ComputeSize" "', argument " "1"" of type '" "RobotRaconteur::Message *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    arg1 = const_cast< RobotRaconteur::Message * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    arg1 = const_cast< RobotRaconteur::Message * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (uint32_t)(arg1)->ComputeSize();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_Message_FindEntry(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::Message *arg1 = (RobotRaconteur::Message *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::Message > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::Message > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageEntry > result;
  
  if (!SWIG_Python_UnpackTuple(args, "Message_FindEntry", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::Message > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::Message *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Message_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Message_FindEntry" "', argument " "1"" of type '" "RobotRaconteur::Message *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    arg1 = const_cast< RobotRaconteur::Message * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    arg1 = const_cast< RobotRaconteur::Message * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Message_FindEntry" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "Message_FindEntry" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->FindEntry((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageEntry >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageEntry >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageEntry >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, SWIG_POINTER_OWN);
  
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_Message_AddEntry(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::Message *arg1 = (RobotRaconteur::Message *) 0 ;
  RobotRaconteur::MessageEntryType arg2 ;
  std::string *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::Message > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::Message > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  PyObject *swig_obj[3] ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageEntry > result;
  
  if (!SWIG_Python_UnpackTuple(args, "Message_AddEntry", 3, 3, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::Message > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::Message *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Message_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Message_AddEntry" "', argument " "1"" of type '" "RobotRaconteur::Message *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    arg1 = const_cast< RobotRaconteur::Message * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    arg1 = const_cast< RobotRaconteur::Message * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Message_AddEntry" "', argument " "2"" of type '" "RobotRaconteur::MessageEntryType""'");
  } 
  arg2 = static_cast< RobotRaconteur::MessageEntryType >(val2);
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "Message_AddEntry" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "Message_AddEntry" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->AddEntry(arg2,(std::string const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageEntry >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageEntry >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageEntry >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, SWIG_POINTER_OWN);
  
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_Message(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::Message *arg1 = (RobotRaconteur::Message *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::Message > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::Message > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::Message > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::Message *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Message_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Message" "', argument " "1"" of type '" "RobotRaconteur::Message *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    arg1 = const_cast< RobotRaconteur::Message * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Message > * >(argp1);
    arg1 = const_cast< RobotRaconteur::Message * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *Message_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Message_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *Message_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_MessageHeader_HeaderSize_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  uint16_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  unsigned short val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageHeader_HeaderSize_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_HeaderSize_set" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_unsigned_SS_short(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageHeader_HeaderSize_set" "', argument " "2"" of type '" "uint16_t""'");
  } 
  arg2 = static_cast< uint16_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->HeaderSize = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_HeaderSize_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  uint16_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_HeaderSize_get" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint16_t) ((arg1)->HeaderSize);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_short(static_cast< unsigned short >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_MessageFlags_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  uint8_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  unsigned char val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageHeader_MessageFlags_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_MessageFlags_set" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_unsigned_SS_char(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageHeader_MessageFlags_set" "', argument " "2"" of type '" "uint8_t""'");
  } 
  arg2 = static_cast< uint8_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->MessageFlags = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_MessageFlags_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  uint8_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_MessageFlags_get" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint8_t) ((arg1)->MessageFlags);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_SenderEndpoint_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageHeader_SenderEndpoint_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_SenderEndpoint_set" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageHeader_SenderEndpoint_set" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->SenderEndpoint = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_SenderEndpoint_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_SenderEndpoint_get" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint32_t) ((arg1)->SenderEndpoint);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_ReceiverEndpoint_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageHeader_ReceiverEndpoint_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_ReceiverEndpoint_set" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageHeader_ReceiverEndpoint_set" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ReceiverEndpoint = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_ReceiverEndpoint_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_ReceiverEndpoint_get" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint32_t) ((arg1)->ReceiverEndpoint);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_SenderNodeName_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  RobotRaconteur::MessageStringPtr *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageHeader_SenderNodeName_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_SenderNodeName_set" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    MessageStringPtr *ptr = (MessageStringPtr *)0;
    res2 = SWIG_AsPtr_MessageStringPtr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MessageHeader_SenderNodeName_set" "', argument " "2"" of type '" "RobotRaconteur::MessageStringPtr const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MessageHeader_SenderNodeName_set" "', argument " "2"" of type '" "RobotRaconteur::MessageStringPtr const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->SenderNodeName = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_SenderNodeName_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MessageStringPtr *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_SenderNodeName_get" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::MessageStringPtr *) & ((arg1)->SenderNodeName);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_MessageStringPtr(static_cast< MessageStringPtr >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_ReceiverNodeName_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  RobotRaconteur::MessageStringPtr *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageHeader_ReceiverNodeName_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_ReceiverNodeName_set" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    MessageStringPtr *ptr = (MessageStringPtr *)0;
    res2 = SWIG_AsPtr_MessageStringPtr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MessageHeader_ReceiverNodeName_set" "', argument " "2"" of type '" "RobotRaconteur::MessageStringPtr const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MessageHeader_ReceiverNodeName_set" "', argument " "2"" of type '" "RobotRaconteur::MessageStringPtr const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ReceiverNodeName = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_ReceiverNodeName_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MessageStringPtr *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_ReceiverNodeName_get" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::MessageStringPtr *) & ((arg1)->ReceiverNodeName);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_MessageStringPtr(static_cast< MessageStringPtr >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_SenderNodeID_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  RobotRaconteur::NodeID *arg2 = (RobotRaconteur::NodeID *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageHeader_SenderNodeID_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_SenderNodeID_set" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__NodeID, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MessageHeader_SenderNodeID_set" "', argument " "2"" of type '" "RobotRaconteur::NodeID *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::NodeID * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->SenderNodeID = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_SenderNodeID_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::NodeID *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_SenderNodeID_get" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::NodeID *)& ((arg1)->SenderNodeID);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__NodeID, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_ReceiverNodeID_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  RobotRaconteur::NodeID *arg2 = (RobotRaconteur::NodeID *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageHeader_ReceiverNodeID_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_ReceiverNodeID_set" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__NodeID, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MessageHeader_ReceiverNodeID_set" "', argument " "2"" of type '" "RobotRaconteur::NodeID *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::NodeID * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ReceiverNodeID = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_ReceiverNodeID_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::NodeID *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_ReceiverNodeID_get" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::NodeID *)& ((arg1)->ReceiverNodeID);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__NodeID, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_MetaData_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  RobotRaconteur::MessageStringPtr *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageHeader_MetaData_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_MetaData_set" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    MessageStringPtr *ptr = (MessageStringPtr *)0;
    res2 = SWIG_AsPtr_MessageStringPtr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MessageHeader_MetaData_set" "', argument " "2"" of type '" "RobotRaconteur::MessageStringPtr const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MessageHeader_MetaData_set" "', argument " "2"" of type '" "RobotRaconteur::MessageStringPtr const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->MetaData = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_MetaData_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MessageStringPtr *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_MetaData_get" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::MessageStringPtr *) & ((arg1)->MetaData);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_MessageStringPtr(static_cast< MessageStringPtr >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_EntryCount_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  uint16_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  unsigned short val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageHeader_EntryCount_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_EntryCount_set" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_unsigned_SS_short(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageHeader_EntryCount_set" "', argument " "2"" of type '" "uint16_t""'");
  } 
  arg2 = static_cast< uint16_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->EntryCount = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_EntryCount_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  uint16_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_EntryCount_get" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint16_t) ((arg1)->EntryCount);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_short(static_cast< unsigned short >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_MessageID_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  uint16_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  unsigned short val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageHeader_MessageID_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_MessageID_set" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_unsigned_SS_short(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageHeader_MessageID_set" "', argument " "2"" of type '" "uint16_t""'");
  } 
  arg2 = static_cast< uint16_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->MessageID = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_MessageID_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  uint16_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_MessageID_get" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint16_t) ((arg1)->MessageID);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_short(static_cast< unsigned short >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_MessageResID_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  uint16_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  unsigned short val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageHeader_MessageResID_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_MessageResID_set" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_unsigned_SS_short(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageHeader_MessageResID_set" "', argument " "2"" of type '" "uint16_t""'");
  } 
  arg2 = static_cast< uint16_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->MessageResID = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_MessageResID_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  uint16_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_MessageResID_get" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint16_t) ((arg1)->MessageResID);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_short(static_cast< unsigned short >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_MessageSize_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageHeader_MessageSize_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_MessageSize_set" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageHeader_MessageSize_set" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->MessageSize = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_MessageSize_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_MessageSize_get" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint32_t) ((arg1)->MessageSize);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_Priority_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  uint16_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  unsigned short val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageHeader_Priority_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_Priority_set" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_unsigned_SS_short(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageHeader_Priority_set" "', argument " "2"" of type '" "uint16_t""'");
  } 
  arg2 = static_cast< uint16_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Priority = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_Priority_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  uint16_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_Priority_get" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint16_t) ((arg1)->Priority);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_short(static_cast< unsigned short >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_Extended_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  std::vector< uint8_t,std::allocator< uint8_t > > *arg2 = (std::vector< uint8_t,std::allocator< uint8_t > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageHeader_Extended_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_Extended_set" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MessageHeader_Extended_set" "', argument " "2"" of type '" "std::vector< uint8_t,std::allocator< uint8_t > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< uint8_t,std::allocator< uint8_t > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Extended = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_Extended_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< uint8_t,std::allocator< uint8_t > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_Extended_get" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< uint8_t,std::allocator< uint8_t > > *)& ((arg1)->Extended);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_ComputeSize(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  uint16_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_ComputeSize" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (uint16_t)(arg1)->ComputeSize();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_short(static_cast< unsigned short >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageHeader_UpdateHeader(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  uint32_t arg2 ;
  uint16_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  unsigned short val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageHeader_UpdateHeader", 3, 3, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageHeader_UpdateHeader" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageHeader_UpdateHeader" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  ecode3 = SWIG_AsVal_unsigned_SS_short(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "MessageHeader_UpdateHeader" "', argument " "3"" of type '" "uint16_t""'");
  } 
  arg3 = static_cast< uint16_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->UpdateHeader(arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_MessageHeader(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_MessageHeader", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::MessageHeader *)new RobotRaconteur::MessageHeader();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    //#if (SWIG_POINTER_NEW)
    //if (result) {
    //  intrusive_ptr_add_ref(result);
    //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader >(result, SWIG_intrusive_deleter<  RobotRaconteur::MessageHeader >());
    //} else {
    //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&resultobj = 0;
    //}
    //#else
    //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&resultobj = result ? new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader >(result SWIG_NO_NULL_DELETER_0) : 0;
    //#endif
    if (SWIG_POINTER_NEW) {
      if (result) {
        intrusive_ptr_add_ref(result);
      }
      boost::shared_ptr<  RobotRaconteur::MessageHeader > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::MessageHeader >(result, SWIG_intrusive_deleter<  RobotRaconteur::MessageHeader >()) : 0;
      resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
    } else {
      boost::shared_ptr<  RobotRaconteur::MessageHeader > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::MessageHeader >(result SWIG_NO_NULL_DELETER_0) : 0;
      resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
    }
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_MessageHeader(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageHeader *arg1 = (RobotRaconteur::MessageHeader *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageHeader > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageHeader > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageHeader *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_MessageHeader" "', argument " "1"" of type '" "RobotRaconteur::MessageHeader *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageHeader > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageHeader * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *MessageHeader_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *MessageHeader_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_MessageEntry_EntrySize_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageEntry_EntrySize_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_EntrySize_set" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageEntry_EntrySize_set" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->EntrySize = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageEntry_EntrySize_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_EntrySize_get" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint32_t) ((arg1)->EntrySize);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageEntry_EntryFlags_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  uint8_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  unsigned char val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageEntry_EntryFlags_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_EntryFlags_set" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_unsigned_SS_char(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageEntry_EntryFlags_set" "', argument " "2"" of type '" "uint8_t""'");
  } 
  arg2 = static_cast< uint8_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->EntryFlags = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageEntry_EntryFlags_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  uint8_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_EntryFlags_get" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint8_t) ((arg1)->EntryFlags);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageEntry_EntryType_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  RobotRaconteur::MessageEntryType arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageEntry_EntryType_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_EntryType_set" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageEntry_EntryType_set" "', argument " "2"" of type '" "RobotRaconteur::MessageEntryType""'");
  } 
  arg2 = static_cast< RobotRaconteur::MessageEntryType >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->EntryType = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageEntry_EntryType_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MessageEntryType result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_EntryType_get" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::MessageEntryType) ((arg1)->EntryType);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageEntry_ServicePath_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  RobotRaconteur::MessageStringPtr *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageEntry_ServicePath_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_ServicePath_set" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    MessageStringPtr *ptr = (MessageStringPtr *)0;
    res2 = SWIG_AsPtr_MessageStringPtr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MessageEntry_ServicePath_set" "', argument " "2"" of type '" "RobotRaconteur::MessageStringPtr const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MessageEntry_ServicePath_set" "', argument " "2"" of type '" "RobotRaconteur::MessageStringPtr const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ServicePath = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageEntry_ServicePath_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MessageStringPtr *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_ServicePath_get" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::MessageStringPtr *) & ((arg1)->ServicePath);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_MessageStringPtr(static_cast< MessageStringPtr >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageEntry_MemberName_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  RobotRaconteur::MessageStringPtr *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageEntry_MemberName_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_MemberName_set" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    MessageStringPtr *ptr = (MessageStringPtr *)0;
    res2 = SWIG_AsPtr_MessageStringPtr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MessageEntry_MemberName_set" "', argument " "2"" of type '" "RobotRaconteur::MessageStringPtr const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MessageEntry_MemberName_set" "', argument " "2"" of type '" "RobotRaconteur::MessageStringPtr const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->MemberName = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageEntry_MemberName_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MessageStringPtr *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_MemberName_get" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::MessageStringPtr *) & ((arg1)->MemberName);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_MessageStringPtr(static_cast< MessageStringPtr >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageEntry_MemberNameCode_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageEntry_MemberNameCode_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_MemberNameCode_set" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageEntry_MemberNameCode_set" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->MemberNameCode = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageEntry_MemberNameCode_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_MemberNameCode_get" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint32_t) ((arg1)->MemberNameCode);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageEntry_RequestID_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageEntry_RequestID_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_RequestID_set" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageEntry_RequestID_set" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->RequestID = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageEntry_RequestID_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_RequestID_get" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint32_t) ((arg1)->RequestID);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageEntry_Error_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  RobotRaconteur::MessageErrorType arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageEntry_Error_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_Error_set" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageEntry_Error_set" "', argument " "2"" of type '" "RobotRaconteur::MessageErrorType""'");
  } 
  arg2 = static_cast< RobotRaconteur::MessageErrorType >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Error = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageEntry_Error_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MessageErrorType result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_Error_get" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::MessageErrorType) ((arg1)->Error);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageEntry_MetaData_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  RobotRaconteur::MessageStringPtr *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageEntry_MetaData_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_MetaData_set" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    MessageStringPtr *ptr = (MessageStringPtr *)0;
    res2 = SWIG_AsPtr_MessageStringPtr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MessageEntry_MetaData_set" "', argument " "2"" of type '" "RobotRaconteur::MessageStringPtr const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MessageEntry_MetaData_set" "', argument " "2"" of type '" "RobotRaconteur::MessageStringPtr const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->MetaData = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageEntry_MetaData_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MessageStringPtr *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_MetaData_get" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::MessageStringPtr *) & ((arg1)->MetaData);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_MessageStringPtr(static_cast< MessageStringPtr >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageEntry_Extended_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  std::vector< uint8_t,std::allocator< uint8_t > > *arg2 = (std::vector< uint8_t,std::allocator< uint8_t > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageEntry_Extended_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_Extended_set" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MessageEntry_Extended_set" "', argument " "2"" of type '" "std::vector< uint8_t,std::allocator< uint8_t > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< uint8_t,std::allocator< uint8_t > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Extended = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageEntry_Extended_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< uint8_t,std::allocator< uint8_t > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_Extended_get" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< uint8_t,std::allocator< uint8_t > > *)& ((arg1)->Extended);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageEntry_elements_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *arg2 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageEntry_elements_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_elements_set" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MessageEntry_elements_set" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->elements = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageEntry_elements_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_elements_get" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *)& ((arg1)->elements);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_MessageEntry__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::MessageEntry *)new RobotRaconteur::MessageEntry();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    //#if (SWIG_POINTER_NEW)
    //if (result) {
    //  intrusive_ptr_add_ref(result);
    //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry >(result, SWIG_intrusive_deleter<  RobotRaconteur::MessageEntry >());
    //} else {
    //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&resultobj = 0;
    //}
    //#else
    //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&resultobj = result ? new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry >(result SWIG_NO_NULL_DELETER_0) : 0;
    //#endif
    if (SWIG_POINTER_NEW) {
      if (result) {
        intrusive_ptr_add_ref(result);
      }
      boost::shared_ptr<  RobotRaconteur::MessageEntry > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::MessageEntry >(result, SWIG_intrusive_deleter<  RobotRaconteur::MessageEntry >()) : 0;
      resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
    } else {
      boost::shared_ptr<  RobotRaconteur::MessageEntry > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::MessageEntry >(result SWIG_NO_NULL_DELETER_0) : 0;
      resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
    }
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_MessageEntry__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntryType arg1 ;
  std::string *arg2 = 0 ;
  int val1 ;
  int ecode1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  RobotRaconteur::MessageEntry *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_int(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_MessageEntry" "', argument " "1"" of type '" "RobotRaconteur::MessageEntryType""'");
  } 
  arg1 = static_cast< RobotRaconteur::MessageEntryType >(val1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_MessageEntry" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_MessageEntry" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::MessageEntry *)new RobotRaconteur::MessageEntry(arg1,(std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    //#if (SWIG_POINTER_NEW)
    //if (result) {
    //  intrusive_ptr_add_ref(result);
    //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry >(result, SWIG_intrusive_deleter<  RobotRaconteur::MessageEntry >());
    //} else {
    //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&resultobj = 0;
    //}
    //#else
    //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&resultobj = result ? new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry >(result SWIG_NO_NULL_DELETER_0) : 0;
    //#endif
    if (SWIG_POINTER_NEW) {
      if (result) {
        intrusive_ptr_add_ref(result);
      }
      boost::shared_ptr<  RobotRaconteur::MessageEntry > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::MessageEntry >(result, SWIG_intrusive_deleter<  RobotRaconteur::MessageEntry >()) : 0;
      resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
    } else {
      boost::shared_ptr<  RobotRaconteur::MessageEntry > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::MessageEntry >(result SWIG_NO_NULL_DELETER_0) : 0;
      resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
    }
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_MessageEntry(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_MessageEntry", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_MessageEntry__SWIG_0(self, argc, argv);
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_int(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_MessageEntry__SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_MessageEntry'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::MessageEntry::MessageEntry()\n"
    "    RobotRaconteur::MessageEntry::MessageEntry(RobotRaconteur::MessageEntryType,std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_MessageEntry_ComputeSize(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_ComputeSize" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (uint32_t)(arg1)->ComputeSize();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageEntry_FindElement(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageEntry_FindElement", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_FindElement" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MessageEntry_FindElement" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MessageEntry_FindElement" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->FindElement((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageEntry_AddElement(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageEntry_AddElement", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageEntry_AddElement" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MessageEntry_AddElement" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->AddElement((boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_MessageEntry(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageEntry *arg1 = (RobotRaconteur::MessageEntry *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageEntry > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageEntry > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageEntry *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_MessageEntry" "', argument " "1"" of type '" "RobotRaconteur::MessageEntry *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageEntry > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageEntry * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *MessageEntry_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *MessageEntry_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_MessageElement_ElementSize_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageElement_ElementSize_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_ElementSize_set" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageElement_ElementSize_set" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ElementSize = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_ElementSize_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_ElementSize_get" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint32_t) ((arg1)->ElementSize);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_ElementFlags_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  uint8_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  unsigned char val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageElement_ElementFlags_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_ElementFlags_set" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_unsigned_SS_char(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageElement_ElementFlags_set" "', argument " "2"" of type '" "uint8_t""'");
  } 
  arg2 = static_cast< uint8_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ElementFlags = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_ElementFlags_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  uint8_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_ElementFlags_get" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint8_t) ((arg1)->ElementFlags);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_char(static_cast< unsigned char >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_ElementName_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  RobotRaconteur::MessageStringPtr *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageElement_ElementName_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_ElementName_set" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    MessageStringPtr *ptr = (MessageStringPtr *)0;
    res2 = SWIG_AsPtr_MessageStringPtr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MessageElement_ElementName_set" "', argument " "2"" of type '" "RobotRaconteur::MessageStringPtr const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MessageElement_ElementName_set" "', argument " "2"" of type '" "RobotRaconteur::MessageStringPtr const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ElementName = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_ElementName_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MessageStringPtr *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_ElementName_get" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::MessageStringPtr *) & ((arg1)->ElementName);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_MessageStringPtr(static_cast< MessageStringPtr >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_ElementNameCode_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageElement_ElementNameCode_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_ElementNameCode_set" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageElement_ElementNameCode_set" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ElementNameCode = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_ElementNameCode_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_ElementNameCode_get" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint32_t) ((arg1)->ElementNameCode);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_ElementNumber_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageElement_ElementNumber_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_ElementNumber_set" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageElement_ElementNumber_set" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ElementNumber = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_ElementNumber_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_ElementNumber_get" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (int32_t) ((arg1)->ElementNumber);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_ElementType_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  RobotRaconteur::DataTypes arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageElement_ElementType_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_ElementType_set" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageElement_ElementType_set" "', argument " "2"" of type '" "RobotRaconteur::DataTypes""'");
  } 
  arg2 = static_cast< RobotRaconteur::DataTypes >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ElementType = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_ElementType_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::DataTypes result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_ElementType_get" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::DataTypes) ((arg1)->ElementType);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_ElementTypeName_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  RobotRaconteur::MessageStringPtr *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageElement_ElementTypeName_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_ElementTypeName_set" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    MessageStringPtr *ptr = (MessageStringPtr *)0;
    res2 = SWIG_AsPtr_MessageStringPtr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MessageElement_ElementTypeName_set" "', argument " "2"" of type '" "RobotRaconteur::MessageStringPtr const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MessageElement_ElementTypeName_set" "', argument " "2"" of type '" "RobotRaconteur::MessageStringPtr const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ElementTypeName = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_ElementTypeName_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MessageStringPtr *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_ElementTypeName_get" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::MessageStringPtr *) & ((arg1)->ElementTypeName);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_MessageStringPtr(static_cast< MessageStringPtr >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_ElementTypeNameCode_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageElement_ElementTypeNameCode_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_ElementTypeNameCode_set" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageElement_ElementTypeNameCode_set" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ElementTypeNameCode = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_ElementTypeNameCode_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_ElementTypeNameCode_get" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint32_t) ((arg1)->ElementTypeNameCode);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_MetaData_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  RobotRaconteur::MessageStringPtr *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageElement_MetaData_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_MetaData_set" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    MessageStringPtr *ptr = (MessageStringPtr *)0;
    res2 = SWIG_AsPtr_MessageStringPtr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MessageElement_MetaData_set" "', argument " "2"" of type '" "RobotRaconteur::MessageStringPtr const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MessageElement_MetaData_set" "', argument " "2"" of type '" "RobotRaconteur::MessageStringPtr const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->MetaData = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_MetaData_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MessageStringPtr *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_MetaData_get" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::MessageStringPtr *) & ((arg1)->MetaData);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_MessageStringPtr(static_cast< MessageStringPtr >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_Extended_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  std::vector< uint8_t,std::allocator< uint8_t > > *arg2 = (std::vector< uint8_t,std::allocator< uint8_t > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageElement_Extended_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_Extended_set" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MessageElement_Extended_set" "', argument " "2"" of type '" "std::vector< uint8_t,std::allocator< uint8_t > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< uint8_t,std::allocator< uint8_t > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Extended = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_Extended_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< uint8_t,std::allocator< uint8_t > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_Extended_get" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< uint8_t,std::allocator< uint8_t > > *)& ((arg1)->Extended);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_DataCount_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageElement_DataCount_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_DataCount_set" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageElement_DataCount_set" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->DataCount = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_DataCount_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_DataCount_get" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint32_t) ((arg1)->DataCount);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_MessageElement(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_MessageElement", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::MessageElement *)new RobotRaconteur::MessageElement();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    //#if (SWIG_POINTER_NEW)
    //if (result) {
    //  intrusive_ptr_add_ref(result);
    //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result, SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
    //} else {
    //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
    //}
    //#else
    //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = result ? new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result SWIG_NO_NULL_DELETER_0) : 0;
    //#endif
    if (SWIG_POINTER_NEW) {
      if (result) {
        intrusive_ptr_add_ref(result);
      }
      boost::shared_ptr<  RobotRaconteur::MessageElement > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::MessageElement >(result, SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >()) : 0;
      resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
    } else {
      boost::shared_ptr<  RobotRaconteur::MessageElement > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::MessageElement >(result SWIG_NO_NULL_DELETER_0) : 0;
      resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
    }
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement__GetData(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MessageElementData > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement__GetData" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetData();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElementData >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElementData >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElementData >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement__SetData(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageElement__SetData", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement__SetData" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MessageElement__SetData" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetData((boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_ComputeSize(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_ComputeSize" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (uint32_t)(arg1)->ComputeSize();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_UpdateData(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_UpdateData" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->UpdateData();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_FindElement(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *arg1 = 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageElement_FindElement", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t,  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_FindElement" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > &""'"); 
  }
  if (!argp1) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MessageElement_FindElement" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > &""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MessageElement_FindElement" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MessageElement_FindElement" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::MessageElement::FindElement(*arg1,(std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElement_ContainsElement(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *arg1 = 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageElement_ContainsElement", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t,  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElement_ContainsElement" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > &""'"); 
  }
  if (!argp1) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MessageElement_ContainsElement" "', argument " "1"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > &""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MessageElement_ContainsElement" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MessageElement_ContainsElement" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)RobotRaconteur::MessageElement::ContainsElement(*arg1,(std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_MessageElement(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElement *arg1 = (RobotRaconteur::MessageElement *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElement *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_MessageElement" "', argument " "1"" of type '" "RobotRaconteur::MessageElement *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElement > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElement * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *MessageElement_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *MessageElement_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_MessageElementNestedElementList_Type_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElementNestedElementList *arg1 = (RobotRaconteur::MessageElementNestedElementList *) 0 ;
  RobotRaconteur::DataTypes arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageElementNestedElementList_Type_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElementNestedElementList *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElementNestedElementList_Type_set" "', argument " "1"" of type '" "RobotRaconteur::MessageElementNestedElementList *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementNestedElementList * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementNestedElementList * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageElementNestedElementList_Type_set" "', argument " "2"" of type '" "RobotRaconteur::DataTypes""'");
  } 
  arg2 = static_cast< RobotRaconteur::DataTypes >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Type = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElementNestedElementList_Type_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElementNestedElementList *arg1 = (RobotRaconteur::MessageElementNestedElementList *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::DataTypes result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElementNestedElementList *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElementNestedElementList_Type_get" "', argument " "1"" of type '" "RobotRaconteur::MessageElementNestedElementList *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementNestedElementList * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementNestedElementList * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::DataTypes) ((arg1)->Type);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElementNestedElementList_TypeName_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElementNestedElementList *arg1 = (RobotRaconteur::MessageElementNestedElementList *) 0 ;
  RobotRaconteur::MessageStringPtr *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageElementNestedElementList_TypeName_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElementNestedElementList *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElementNestedElementList_TypeName_set" "', argument " "1"" of type '" "RobotRaconteur::MessageElementNestedElementList *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementNestedElementList * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementNestedElementList * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    MessageStringPtr *ptr = (MessageStringPtr *)0;
    res2 = SWIG_AsPtr_MessageStringPtr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MessageElementNestedElementList_TypeName_set" "', argument " "2"" of type '" "RobotRaconteur::MessageStringPtr const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MessageElementNestedElementList_TypeName_set" "', argument " "2"" of type '" "RobotRaconteur::MessageStringPtr const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->TypeName = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElementNestedElementList_TypeName_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElementNestedElementList *arg1 = (RobotRaconteur::MessageElementNestedElementList *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MessageStringPtr *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElementNestedElementList *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElementNestedElementList_TypeName_get" "', argument " "1"" of type '" "RobotRaconteur::MessageElementNestedElementList *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementNestedElementList * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementNestedElementList * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::MessageStringPtr *) & ((arg1)->TypeName);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_MessageStringPtr(static_cast< MessageStringPtr >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElementNestedElementList_Elements_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElementNestedElementList *arg1 = (RobotRaconteur::MessageElementNestedElementList *) 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *arg2 = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MessageElementNestedElementList_Elements_set", 2, 2, swig_obj)) SWIG_fail;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElementNestedElementList *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElementNestedElementList_Elements_set" "', argument " "1"" of type '" "RobotRaconteur::MessageElementNestedElementList *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementNestedElementList * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementNestedElementList * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MessageElementNestedElementList_Elements_set" "', argument " "2"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Elements = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElementNestedElementList_Elements_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElementNestedElementList *arg1 = (RobotRaconteur::MessageElementNestedElementList *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElementNestedElementList *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElementNestedElementList_Elements_get" "', argument " "1"" of type '" "RobotRaconteur::MessageElementNestedElementList *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementNestedElementList * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementNestedElementList * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *)& ((arg1)->Elements);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_MessageElementNestedElementList(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::DataTypes arg1 ;
  std::string *arg2 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *arg3 = 0 ;
  int val1 ;
  int ecode1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  PyObject *swig_obj[3] ;
  RobotRaconteur::MessageElementNestedElementList *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_MessageElementNestedElementList", 3, 3, swig_obj)) SWIG_fail;
  ecode1 = SWIG_AsVal_int(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_MessageElementNestedElementList" "', argument " "1"" of type '" "RobotRaconteur::DataTypes""'");
  } 
  arg1 = static_cast< RobotRaconteur::DataTypes >(val1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_MessageElementNestedElementList" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_MessageElementNestedElementList" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *ptr = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "new_MessageElementNestedElementList" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_MessageElementNestedElementList" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::MessageElementNestedElementList *)new RobotRaconteur::MessageElementNestedElementList(arg1,(std::string const &)*arg2,(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    //#if (SWIG_POINTER_NEW)
    //if (result) {
    //  intrusive_ptr_add_ref(result);
    //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >(result, SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
    //} else {
    //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&resultobj = 0;
    //}
    //#else
    //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&resultobj = result ? new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >(result SWIG_NO_NULL_DELETER_0) : 0;
    //#endif
    if (SWIG_POINTER_NEW) {
      if (result) {
        intrusive_ptr_add_ref(result);
      }
      boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >(result, SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >()) : 0;
      resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
    } else {
      boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >(result SWIG_NO_NULL_DELETER_0) : 0;
      resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
    }
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElementNestedElementList_GetTypeString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElementNestedElementList *arg1 = (RobotRaconteur::MessageElementNestedElementList *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MessageStringPtr result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElementNestedElementList *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElementNestedElementList_GetTypeString" "', argument " "1"" of type '" "RobotRaconteur::MessageElementNestedElementList *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementNestedElementList * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementNestedElementList * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetTypeString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_MessageStringPtr(static_cast< MessageStringPtr >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElementNestedElementList_GetTypeID(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElementNestedElementList *arg1 = (RobotRaconteur::MessageElementNestedElementList *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::DataTypes result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElementNestedElementList *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElementNestedElementList_GetTypeID" "', argument " "1"" of type '" "RobotRaconteur::MessageElementNestedElementList *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementNestedElementList * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementNestedElementList * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::DataTypes)(arg1)->GetTypeID();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_MessageElementNestedElementList(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MessageElementNestedElementList *arg1 = (RobotRaconteur::MessageElementNestedElementList *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartarg1 = 0 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // plain pointer
  // smartarg1 = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&swig_obj[0];
  // arg1 = (RobotRaconteur::MessageElementNestedElementList *)(smartarg1 ? smartarg1->get() : 0);
  
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, 0 |  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_MessageElementNestedElementList" "', argument " "1"" of type '" "RobotRaconteur::MessageElementNestedElementList *""'");
  }
  if (newmem1 & SWIG_CAST_NEW_MEMORY) {
    tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementNestedElementList * >(tempshared1.get());
  } else {
    smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > * >(argp1);
    arg1 = const_cast< RobotRaconteur::MessageElementNestedElementList * >((smartarg1 ? smartarg1->get() : 0));
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *MessageElementNestedElementList_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *MessageElementNestedElementList_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_MessageElementDataUtil_ToRRBaseArray(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > tempnull1 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > temp1 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::RRBaseArray > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::RRBaseArray > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // intrusive_ptr by reference
  //if ( swig_obj[0] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[0];
  //	temp1 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //	arg1 = &temp1;
  //} else {
  //  arg1 = &tempnull1;
  //}
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElementDataUtil_ToRRBaseArray" "', argument " "1"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &""'");
  }
  if (argp1) {
    temp1 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp1)->get(),true);
    if (newmem1 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp1);
    arg1 = &temp1;
  }
  else {
    arg1 = &tempnull1;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = MessageElementDataUtil::ToRRBaseArray((boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::RRBaseArray >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::RRBaseArray >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::RRBaseArray >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElementDataUtil_ToMessageElementNestedElementList(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > tempnull1 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > temp1 ;
  int newmem1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  
  // intrusive_ptr by reference
  //if ( swig_obj[0] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[0];
  //	temp1 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //	arg1 = &temp1;
  //} else {
  //  arg1 = &tempnull1;
  //}
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElementDataUtil_ToMessageElementNestedElementList" "', argument " "1"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &""'");
  }
  if (argp1) {
    temp1 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp1)->get(),true);
    if (newmem1 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp1);
    arg1 = &temp1;
  }
  else {
    arg1 = &tempnull1;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = MessageElementDataUtil::ToMessageElementNestedElementList((boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_MessageElementDataUtil(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  MessageElementDataUtil *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_MessageElementDataUtil", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (MessageElementDataUtil *)new MessageElementDataUtil();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_MessageElementDataUtil, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_MessageElementDataUtil(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  MessageElementDataUtil *arg1 = (MessageElementDataUtil *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_MessageElementDataUtil, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_MessageElementDataUtil" "', argument " "1"" of type '" "MessageElementDataUtil *""'"); 
  }
  arg1 = reinterpret_cast< MessageElementDataUtil * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *MessageElementDataUtil_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_MessageElementDataUtil, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *MessageElementDataUtil_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_MessageFromBytes(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  uint8_t *arg1 = (uint8_t *) 0 ;
  size_t arg2 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::Message > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::Message > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int res; Py_ssize_t size = 0; const void *buf = 0;
    Py_buffer view;
    res = PyObject_GetBuffer(swig_obj[0], &view, PyBUF_CONTIG_RO);
    size = view.len;
    buf = view.buf;
    PyBuffer_Release(&view);
    if (res < 0) {
      PyErr_Clear();
      SWIG_exception_fail(SWIG_ArgError(res), "in method '" "MessageFromBytes" "', argument " "1"" of type '" "(const uint8_t* bytes, size_t bytes_len)""'");
    }
    arg1 = (uint8_t *) buf;
    arg2 = (size_t) (size / sizeof(uint8_t const));
  }
  
  try {
    result = MessageFromBytes((unsigned char const *)arg1,arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::Message > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::Message >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::Message >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::Message > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::Message >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::Message >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Message_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageLengthFromBytes(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  uint8_t *arg1 = (uint8_t *) 0 ;
  size_t arg2 ;
  PyObject *swig_obj[1] ;
  size_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int res; Py_ssize_t size = 0; const void *buf = 0;
    Py_buffer view;
    res = PyObject_GetBuffer(swig_obj[0], &view, PyBUF_CONTIG_RO);
    size = view.len;
    buf = view.buf;
    PyBuffer_Release(&view);
    if (res < 0) {
      PyErr_Clear();
      SWIG_exception_fail(SWIG_ArgError(res), "in method '" "MessageLengthFromBytes" "', argument " "1"" of type '" "(const uint8_t* bytes, size_t bytes_len)""'");
    }
    arg1 = (uint8_t *) buf;
    arg2 = (size_t) (size / sizeof(uint8_t const));
  }
  
  try {
    result = MessageLengthFromBytes((unsigned char const *)arg1,arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageToBytes__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::intrusive_ptr< RobotRaconteur::Message > arg1 ;
  uint32_t arg2 ;
  void *argp1 ;
  int res1 = 0 ;
  int newmem1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::Message >**)&swig_obj[0];
  //if (smartarg) {
  //	arg1 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::Message >(smartarg->get(), true);
  //}
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Message_t,  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageToBytes" "', argument " "1"" of type '" "boost::intrusive_ptr< RobotRaconteur::Message >""'");
  }
  if (argp1) arg1 = boost::intrusive_ptr<  RobotRaconteur::Message >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::Message >* >(argp1)->get(),true);
  if (newmem1 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::Message >* >(argp1);
  
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageToBytes" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  
  try {
    result = (PyObject *)MessageToBytes(arg1,arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageToBytes__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::intrusive_ptr< RobotRaconteur::Message > arg1 ;
  void *argp1 ;
  int res1 = 0 ;
  int newmem1 = 0 ;
  PyObject *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::Message >**)&swig_obj[0];
  //if (smartarg) {
  //	arg1 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::Message >(smartarg->get(), true);
  //}
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Message_t,  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageToBytes" "', argument " "1"" of type '" "boost::intrusive_ptr< RobotRaconteur::Message >""'");
  }
  if (argp1) arg1 = boost::intrusive_ptr<  RobotRaconteur::Message >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::Message >* >(argp1)->get(),true);
  if (newmem1 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::Message >* >(argp1);
  
  
  try {
    result = (PyObject *)MessageToBytes(arg1);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageToBytes(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "MessageToBytes", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Message_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_MessageToBytes__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Message_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_unsigned_SS_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_MessageToBytes__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'MessageToBytes'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    MessageToBytes(boost::intrusive_ptr< RobotRaconteur::Message >,uint32_t)\n"
    "    MessageToBytes(boost::intrusive_ptr< RobotRaconteur::Message >)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_MessageElementFromBytes(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  uint8_t *arg1 = (uint8_t *) 0 ;
  size_t arg2 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int res; Py_ssize_t size = 0; const void *buf = 0;
    Py_buffer view;
    res = PyObject_GetBuffer(swig_obj[0], &view, PyBUF_CONTIG_RO);
    size = view.len;
    buf = view.buf;
    PyBuffer_Release(&view);
    if (res < 0) {
      PyErr_Clear();
      SWIG_exception_fail(SWIG_ArgError(res), "in method '" "MessageElementFromBytes" "', argument " "1"" of type '" "(const uint8_t* bytes, size_t bytes_len)""'");
    }
    arg1 = (uint8_t *) buf;
    arg2 = (size_t) (size / sizeof(uint8_t const));
  }
  
  try {
    result = MessageElementFromBytes((unsigned char const *)arg1,arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElementLengthFromBytes(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  uint8_t *arg1 = (uint8_t *) 0 ;
  size_t arg2 ;
  PyObject *swig_obj[1] ;
  size_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int res; Py_ssize_t size = 0; const void *buf = 0;
    Py_buffer view;
    res = PyObject_GetBuffer(swig_obj[0], &view, PyBUF_CONTIG_RO);
    size = view.len;
    buf = view.buf;
    PyBuffer_Release(&view);
    if (res < 0) {
      PyErr_Clear();
      SWIG_exception_fail(SWIG_ArgError(res), "in method '" "MessageElementLengthFromBytes" "', argument " "1"" of type '" "(const uint8_t* bytes, size_t bytes_len)""'");
    }
    arg1 = (uint8_t *) buf;
    arg2 = (size_t) (size / sizeof(uint8_t const));
  }
  
  try {
    result = MessageElementLengthFromBytes((unsigned char const *)arg1,arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElementToBytes__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg1 = 0 ;
  uint32_t arg2 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull1 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp1 ;
  int newmem1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  
  // intrusive_ptr by reference
  //if ( swig_obj[0] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[0];
  //	temp1 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg1 = &temp1;
  //} else {
  //  arg1 = &tempnull1;
  //}
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElementToBytes" "', argument " "1"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp1) {
    temp1 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp1)->get(),true);
    if (newmem1 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp1);
    arg1 = &temp1;
  }
  else {
    arg1 = &tempnull1;
  }
  
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "MessageElementToBytes" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  
  try {
    result = (PyObject *)MessageElementToBytes((boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg1,arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElementToBytes__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull1 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp1 ;
  int newmem1 = 0 ;
  PyObject *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  
  // intrusive_ptr by reference
  //if ( swig_obj[0] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[0];
  //	temp1 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg1 = &temp1;
  //} else {
  //  arg1 = &tempnull1;
  //}
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MessageElementToBytes" "', argument " "1"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp1) {
    temp1 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp1)->get(),true);
    if (newmem1 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp1);
    arg1 = &temp1;
  }
  else {
    arg1 = &tempnull1;
  }
  
  
  try {
    result = (PyObject *)MessageElementToBytes((boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg1);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MessageElementToBytes(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "MessageElementToBytes", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_MessageElementToBytes__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_unsigned_SS_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_MessageElementToBytes__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'MessageElementToBytes'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    MessageElementToBytes(boost::intrusive_ptr< RobotRaconteur::MessageElement > const &,uint32_t)\n"
    "    MessageElementToBytes(boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_HandlerErrorInfo_error_code_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HandlerErrorInfo *arg1 = (RobotRaconteur::HandlerErrorInfo *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "HandlerErrorInfo_error_code_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "HandlerErrorInfo_error_code_set" "', argument " "1"" of type '" "RobotRaconteur::HandlerErrorInfo *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "HandlerErrorInfo_error_code_set" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->error_code = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_HandlerErrorInfo_error_code_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HandlerErrorInfo *arg1 = (RobotRaconteur::HandlerErrorInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "HandlerErrorInfo_error_code_get" "', argument " "1"" of type '" "RobotRaconteur::HandlerErrorInfo *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint32_t) ((arg1)->error_code);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_HandlerErrorInfo_errorname_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HandlerErrorInfo *arg1 = (RobotRaconteur::HandlerErrorInfo *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "HandlerErrorInfo_errorname_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "HandlerErrorInfo_errorname_set" "', argument " "1"" of type '" "RobotRaconteur::HandlerErrorInfo *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "HandlerErrorInfo_errorname_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "HandlerErrorInfo_errorname_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->errorname = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_HandlerErrorInfo_errorname_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HandlerErrorInfo *arg1 = (RobotRaconteur::HandlerErrorInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "HandlerErrorInfo_errorname_get" "', argument " "1"" of type '" "RobotRaconteur::HandlerErrorInfo *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->errorname);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_HandlerErrorInfo_errormessage_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HandlerErrorInfo *arg1 = (RobotRaconteur::HandlerErrorInfo *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "HandlerErrorInfo_errormessage_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "HandlerErrorInfo_errormessage_set" "', argument " "1"" of type '" "RobotRaconteur::HandlerErrorInfo *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "HandlerErrorInfo_errormessage_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "HandlerErrorInfo_errormessage_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->errormessage = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_HandlerErrorInfo_errormessage_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HandlerErrorInfo *arg1 = (RobotRaconteur::HandlerErrorInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "HandlerErrorInfo_errormessage_get" "', argument " "1"" of type '" "RobotRaconteur::HandlerErrorInfo *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->errormessage);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_HandlerErrorInfo_errorsubname_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HandlerErrorInfo *arg1 = (RobotRaconteur::HandlerErrorInfo *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "HandlerErrorInfo_errorsubname_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "HandlerErrorInfo_errorsubname_set" "', argument " "1"" of type '" "RobotRaconteur::HandlerErrorInfo *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "HandlerErrorInfo_errorsubname_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "HandlerErrorInfo_errorsubname_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->errorsubname = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_HandlerErrorInfo_errorsubname_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HandlerErrorInfo *arg1 = (RobotRaconteur::HandlerErrorInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "HandlerErrorInfo_errorsubname_get" "', argument " "1"" of type '" "RobotRaconteur::HandlerErrorInfo *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->errorsubname);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_HandlerErrorInfo_param__set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HandlerErrorInfo *arg1 = (RobotRaconteur::HandlerErrorInfo *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "HandlerErrorInfo_param__set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "HandlerErrorInfo_param__set" "', argument " "1"" of type '" "RobotRaconteur::HandlerErrorInfo *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp1);
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "HandlerErrorInfo_param__set" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->param_ = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_HandlerErrorInfo_param__get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HandlerErrorInfo *arg1 = (RobotRaconteur::HandlerErrorInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "HandlerErrorInfo_param__get" "', argument " "1"" of type '" "RobotRaconteur::HandlerErrorInfo *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::intrusive_ptr< RobotRaconteur::MessageElement > *) & ((arg1)->param_);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  
  //if (*result) {
  //  intrusive_ptr_add_ref(result->get());
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result->get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(result)) {
    intrusive_ptr_add_ref(result->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >(result->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_HandlerErrorInfo(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HandlerErrorInfo *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_HandlerErrorInfo", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::HandlerErrorInfo *)new RobotRaconteur::HandlerErrorInfo();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_HandlerErrorInfo(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HandlerErrorInfo *arg1 = (RobotRaconteur::HandlerErrorInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_HandlerErrorInfo" "', argument " "1"" of type '" "RobotRaconteur::HandlerErrorInfo *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *HandlerErrorInfo_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *HandlerErrorInfo_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_AsyncRequestDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncRequestDirector *arg1 = (RobotRaconteur::AsyncRequestDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncRequestDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_AsyncRequestDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncRequestDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncRequestDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_AsyncRequestDirector_handler(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncRequestDirector *arg1 = (RobotRaconteur::AsyncRequestDirector *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > arg2 ;
  RobotRaconteur::HandlerErrorInfo *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  int newmem2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  PyObject *swig_obj[3] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "AsyncRequestDirector_handler", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncRequestDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AsyncRequestDirector_handler" "', argument " "1"" of type '" "RobotRaconteur::AsyncRequestDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncRequestDirector * >(argp1);
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //if (smartarg) {
  //	arg2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "AsyncRequestDirector_handler" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement >""'");
  }
  if (argp2) arg2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
  if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
  
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "AsyncRequestDirector_handler" "', argument " "3"" of type '" "RobotRaconteur::HandlerErrorInfo &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "AsyncRequestDirector_handler" "', argument " "3"" of type '" "RobotRaconteur::HandlerErrorInfo &""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp3);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::AsyncRequestDirector::handler");
      } else {
        (arg1)->handler(arg2,*arg3);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_AsyncRequestDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::AsyncRequestDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::AsyncRequestDirector *)new SwigDirector_AsyncRequestDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__AsyncRequestDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_AsyncRequestDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncRequestDirector *arg1 = (RobotRaconteur::AsyncRequestDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncRequestDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_AsyncRequestDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncRequestDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncRequestDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *AsyncRequestDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__AsyncRequestDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *AsyncRequestDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_AsyncVoidReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncVoidReturnDirector *arg1 = (RobotRaconteur::AsyncVoidReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncVoidReturnDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_AsyncVoidReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncVoidReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncVoidReturnDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_AsyncVoidReturnDirector_handler(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncVoidReturnDirector *arg1 = (RobotRaconteur::AsyncVoidReturnDirector *) 0 ;
  RobotRaconteur::HandlerErrorInfo *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "AsyncVoidReturnDirector_handler", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncVoidReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AsyncVoidReturnDirector_handler" "', argument " "1"" of type '" "RobotRaconteur::AsyncVoidReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncVoidReturnDirector * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "AsyncVoidReturnDirector_handler" "', argument " "2"" of type '" "RobotRaconteur::HandlerErrorInfo &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "AsyncVoidReturnDirector_handler" "', argument " "2"" of type '" "RobotRaconteur::HandlerErrorInfo &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp2);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::AsyncVoidReturnDirector::handler");
      } else {
        (arg1)->handler(*arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_AsyncVoidReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::AsyncVoidReturnDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::AsyncVoidReturnDirector *)new SwigDirector_AsyncVoidReturnDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__AsyncVoidReturnDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_AsyncVoidReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncVoidReturnDirector *arg1 = (RobotRaconteur::AsyncVoidReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncVoidReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_AsyncVoidReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncVoidReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncVoidReturnDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *AsyncVoidReturnDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__AsyncVoidReturnDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *AsyncVoidReturnDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_AsyncVoidNoErrReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncVoidNoErrReturnDirector *arg1 = (RobotRaconteur::AsyncVoidNoErrReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncVoidNoErrReturnDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_AsyncVoidNoErrReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncVoidNoErrReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncVoidNoErrReturnDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_AsyncVoidNoErrReturnDirector_handler(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncVoidNoErrReturnDirector *arg1 = (RobotRaconteur::AsyncVoidNoErrReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncVoidNoErrReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AsyncVoidNoErrReturnDirector_handler" "', argument " "1"" of type '" "RobotRaconteur::AsyncVoidNoErrReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncVoidNoErrReturnDirector * >(argp1);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::AsyncVoidNoErrReturnDirector::handler");
      } else {
        (arg1)->handler();
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_AsyncVoidNoErrReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::AsyncVoidNoErrReturnDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::AsyncVoidNoErrReturnDirector *)new SwigDirector_AsyncVoidNoErrReturnDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__AsyncVoidNoErrReturnDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_AsyncVoidNoErrReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncVoidNoErrReturnDirector *arg1 = (RobotRaconteur::AsyncVoidNoErrReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncVoidNoErrReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_AsyncVoidNoErrReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncVoidNoErrReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncVoidNoErrReturnDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *AsyncVoidNoErrReturnDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__AsyncVoidNoErrReturnDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *AsyncVoidNoErrReturnDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_AsyncStringReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncStringReturnDirector *arg1 = (RobotRaconteur::AsyncStringReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncStringReturnDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_AsyncStringReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncStringReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncStringReturnDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_AsyncStringReturnDirector_handler(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncStringReturnDirector *arg1 = (RobotRaconteur::AsyncStringReturnDirector *) 0 ;
  std::string *arg2 = 0 ;
  RobotRaconteur::HandlerErrorInfo *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  PyObject *swig_obj[3] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "AsyncStringReturnDirector_handler", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncStringReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AsyncStringReturnDirector_handler" "', argument " "1"" of type '" "RobotRaconteur::AsyncStringReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncStringReturnDirector * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "AsyncStringReturnDirector_handler" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "AsyncStringReturnDirector_handler" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "AsyncStringReturnDirector_handler" "', argument " "3"" of type '" "RobotRaconteur::HandlerErrorInfo &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "AsyncStringReturnDirector_handler" "', argument " "3"" of type '" "RobotRaconteur::HandlerErrorInfo &""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp3);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::AsyncStringReturnDirector::handler");
      } else {
        (arg1)->handler((std::string const &)*arg2,*arg3);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_AsyncStringReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::AsyncStringReturnDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::AsyncStringReturnDirector *)new SwigDirector_AsyncStringReturnDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__AsyncStringReturnDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_AsyncStringReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncStringReturnDirector *arg1 = (RobotRaconteur::AsyncStringReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncStringReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_AsyncStringReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncStringReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncStringReturnDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *AsyncStringReturnDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__AsyncStringReturnDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *AsyncStringReturnDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_AsyncUInt32ReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncUInt32ReturnDirector *arg1 = (RobotRaconteur::AsyncUInt32ReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncUInt32ReturnDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_AsyncUInt32ReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncUInt32ReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncUInt32ReturnDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_AsyncUInt32ReturnDirector_handler(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncUInt32ReturnDirector *arg1 = (RobotRaconteur::AsyncUInt32ReturnDirector *) 0 ;
  uint32_t arg2 ;
  RobotRaconteur::HandlerErrorInfo *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  PyObject *swig_obj[3] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "AsyncUInt32ReturnDirector_handler", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncUInt32ReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AsyncUInt32ReturnDirector_handler" "', argument " "1"" of type '" "RobotRaconteur::AsyncUInt32ReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncUInt32ReturnDirector * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "AsyncUInt32ReturnDirector_handler" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "AsyncUInt32ReturnDirector_handler" "', argument " "3"" of type '" "RobotRaconteur::HandlerErrorInfo &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "AsyncUInt32ReturnDirector_handler" "', argument " "3"" of type '" "RobotRaconteur::HandlerErrorInfo &""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp3);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::AsyncUInt32ReturnDirector::handler");
      } else {
        (arg1)->handler(arg2,*arg3);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_AsyncUInt32ReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::AsyncUInt32ReturnDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::AsyncUInt32ReturnDirector *)new SwigDirector_AsyncUInt32ReturnDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__AsyncUInt32ReturnDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_AsyncUInt32ReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncUInt32ReturnDirector *arg1 = (RobotRaconteur::AsyncUInt32ReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncUInt32ReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_AsyncUInt32ReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncUInt32ReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncUInt32ReturnDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *AsyncUInt32ReturnDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__AsyncUInt32ReturnDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *AsyncUInt32ReturnDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_iterator" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition___nonzero__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____nonzero__((std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition___bool__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____bool__((std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition___len__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____len__((std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_typedefinition___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition___getslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_typedefinition___getslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_typedefinition___getslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition___setslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_typedefinition___setslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_typedefinition___setslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type arg3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition___setslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_typedefinition___setslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_typedefinition___setslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type >(val3);
  {
    std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorptr_typedefinition___setslice__" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_typedefinition___setslice__" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_typedefinition___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vectorptr_typedefinition___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorptr_typedefinition___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_typedefinition___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::__setslice__(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::__setslice__(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_typedefinition___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition___delslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_typedefinition___delslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_typedefinition___delslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition___delitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_typedefinition___delitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition___getitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_typedefinition___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_typedefinition___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_typedefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_typedefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____setitem____SWIG_0(arg1,arg2,(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_typedefinition___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition___delitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_typedefinition___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_typedefinition___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_typedefinition___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_typedefinition___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_typedefinition___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::__delitem__(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition___getitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_typedefinition___getitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type *) &std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____getitem____SWIG_1((std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::TypeDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::TypeDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_typedefinition___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_typedefinition___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_typedefinition___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_typedefinition___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::__getitem__(PySliceObject *)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::__getitem__(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type tempshared3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_typedefinition___setitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_typedefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg____setitem____SWIG_2(arg1,arg2,(boost::shared_ptr< RobotRaconteur::TypeDefinition > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_typedefinition___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_typedefinition___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_typedefinition___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_typedefinition___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_typedefinition___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::__setitem__(PySliceObject *,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::__setitem__(PySliceObject *)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::__setitem__(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_pop" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::TypeDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::TypeDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_typedefinition_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_append" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_typedefinition_append" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg__append(arg1,(boost::shared_ptr< RobotRaconteur::TypeDefinition > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_typedefinition__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_typedefinition__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vectorptr_typedefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vectorptr_typedefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >((std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_empty" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_size" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_typedefinition_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_swap" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_typedefinition_swap" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_typedefinition_swap" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_begin" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_end" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_rbegin" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_rend" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_clear" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_get_allocator" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::allocator_type(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_typedefinition__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorptr_typedefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type""'");
  } 
  arg1 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_pop_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_resize" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_typedefinition_resize" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_erase" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_typedefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_typedefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_erase" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_typedefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_typedefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_typedefinition_erase" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_typedefinition_erase" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_typedefinition_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vectorptr_typedefinition_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vectorptr_typedefinition_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_typedefinition_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::erase(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::erase(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_typedefinition__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type arg1 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type tempshared2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorptr_typedefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type""'");
  } 
  arg1 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type >(val1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_vectorptr_typedefinition" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >(arg1,(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_typedefinition(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vectorptr_typedefinition", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vectorptr_typedefinition__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vectorptr_typedefinition__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vectorptr_typedefinition__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_vectorptr_typedefinition__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vectorptr_typedefinition'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::vector()\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_typedefinition_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_push_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_typedefinition_push_back" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_front" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type *) &((std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::TypeDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::TypeDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type *) &((std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::TypeDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::TypeDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type tempshared3 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_typedefinition_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_assign" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_typedefinition_assign" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_typedefinition_assign" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type tempshared3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_resize" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_typedefinition_resize" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_typedefinition_resize" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_typedefinition_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_typedefinition_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_typedefinition_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_typedefinition_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::resize(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::resize(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type tempshared3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_insert" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_typedefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_typedefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator""'");
    }
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_typedefinition_insert" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg__insert__SWIG_0(arg1,arg2,(boost::shared_ptr< RobotRaconteur::TypeDefinition > const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type arg3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  void *argp4 ;
  int res4 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type tempshared4 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_insert" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_typedefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_typedefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_typedefinition_insert" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type >(val3);
  {
    int newmem = 0;
    res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorptr_typedefinition_insert" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp4) tempshared4 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp4);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp4);
      arg4 = &tempshared4;
    } else {
      arg4 = (argp4) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type * >(argp4) : &tempshared4;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_TypeDefinition_Sg__Sg__insert__SWIG_1(arg1,arg2,arg3,(boost::shared_ptr< RobotRaconteur::TypeDefinition > const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_typedefinition_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_typedefinition_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorptr_typedefinition_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_typedefinition_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::insert(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::insert(std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_typedefinition_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_reserve" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_typedefinition_reserve" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_typedefinition_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_typedefinition_capacity" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vectorptr_typedefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vectorptr_typedefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vectorptr_typedefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vectorptr_typedefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_iterator" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition___nonzero__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____nonzero__((std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition___bool__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____bool__((std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition___len__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____len__((std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_memberdefinition___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition___getslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_memberdefinition___getslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_memberdefinition___getslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition___setslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_memberdefinition___setslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_memberdefinition___setslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type arg3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition___setslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_memberdefinition___setslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_memberdefinition___setslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type >(val3);
  {
    std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorptr_memberdefinition___setslice__" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_memberdefinition___setslice__" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_memberdefinition___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vectorptr_memberdefinition___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorptr_memberdefinition___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_memberdefinition___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::__setslice__(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::__setslice__(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_memberdefinition___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition___delslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_memberdefinition___delslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_memberdefinition___delslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition___delitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_memberdefinition___delitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition___getitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_memberdefinition___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_memberdefinition___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_memberdefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_memberdefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____setitem____SWIG_0(arg1,arg2,(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_memberdefinition___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition___delitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_memberdefinition___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_memberdefinition___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_memberdefinition___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_memberdefinition___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_memberdefinition___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::__delitem__(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition___getitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_memberdefinition___getitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type *) &std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____getitem____SWIG_1((std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::MemberDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::MemberDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_memberdefinition___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_memberdefinition___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_memberdefinition___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_memberdefinition___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::__getitem__(PySliceObject *)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::__getitem__(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type tempshared3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_memberdefinition___setitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_memberdefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg____setitem____SWIG_2(arg1,arg2,(boost::shared_ptr< RobotRaconteur::MemberDefinition > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_memberdefinition___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_memberdefinition___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_memberdefinition___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_memberdefinition___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_memberdefinition___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::__setitem__(PySliceObject *,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::__setitem__(PySliceObject *)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::__setitem__(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_pop" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::MemberDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::MemberDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_memberdefinition_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_append" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_memberdefinition_append" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg__append(arg1,(boost::shared_ptr< RobotRaconteur::MemberDefinition > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_memberdefinition__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_memberdefinition__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vectorptr_memberdefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vectorptr_memberdefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >((std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_empty" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_size" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_memberdefinition_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_swap" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_memberdefinition_swap" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_memberdefinition_swap" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_begin" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_end" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_rbegin" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_rend" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_clear" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_get_allocator" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::allocator_type(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_memberdefinition__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorptr_memberdefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type""'");
  } 
  arg1 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_pop_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_resize" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_memberdefinition_resize" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_erase" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_memberdefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_memberdefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_erase" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_memberdefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_memberdefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_memberdefinition_erase" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_memberdefinition_erase" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_memberdefinition_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vectorptr_memberdefinition_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vectorptr_memberdefinition_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_memberdefinition_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::erase(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::erase(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_memberdefinition__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type arg1 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type tempshared2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorptr_memberdefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type""'");
  } 
  arg1 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type >(val1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_vectorptr_memberdefinition" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >(arg1,(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_memberdefinition(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vectorptr_memberdefinition", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vectorptr_memberdefinition__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vectorptr_memberdefinition__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vectorptr_memberdefinition__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_vectorptr_memberdefinition__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vectorptr_memberdefinition'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::vector()\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_memberdefinition_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_push_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_memberdefinition_push_back" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_front" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type *) &((std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::MemberDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::MemberDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type *) &((std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::MemberDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::MemberDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type tempshared3 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_memberdefinition_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_assign" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_memberdefinition_assign" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_memberdefinition_assign" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type tempshared3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_resize" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_memberdefinition_resize" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_memberdefinition_resize" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_memberdefinition_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_memberdefinition_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_memberdefinition_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_memberdefinition_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::resize(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::resize(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type tempshared3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_insert" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_memberdefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_memberdefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator""'");
    }
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_memberdefinition_insert" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg__insert__SWIG_0(arg1,arg2,(boost::shared_ptr< RobotRaconteur::MemberDefinition > const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type arg3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  void *argp4 ;
  int res4 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type tempshared4 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_insert" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_memberdefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_memberdefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_memberdefinition_insert" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type >(val3);
  {
    int newmem = 0;
    res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorptr_memberdefinition_insert" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp4) tempshared4 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp4);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp4);
      arg4 = &tempshared4;
    } else {
      arg4 = (argp4) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type * >(argp4) : &tempshared4;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_MemberDefinition_Sg__Sg__insert__SWIG_1(arg1,arg2,arg3,(boost::shared_ptr< RobotRaconteur::MemberDefinition > const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_memberdefinition_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_memberdefinition_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorptr_memberdefinition_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_memberdefinition_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::insert(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::insert(std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_memberdefinition_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_reserve" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_memberdefinition_reserve" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_memberdefinition_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_memberdefinition_capacity" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vectorptr_memberdefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vectorptr_memberdefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vectorptr_memberdefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vectorptr_memberdefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_iterator" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition___nonzero__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____nonzero__((std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition___bool__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____bool__((std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition___len__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____len__((std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_serviceentrydefinition___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition___getslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_serviceentrydefinition___getslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_serviceentrydefinition___getslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition___setslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_serviceentrydefinition___setslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_serviceentrydefinition___setslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type arg3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition___setslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_serviceentrydefinition___setslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_serviceentrydefinition___setslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type >(val3);
  {
    std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorptr_serviceentrydefinition___setslice__" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_serviceentrydefinition___setslice__" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_serviceentrydefinition___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vectorptr_serviceentrydefinition___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorptr_serviceentrydefinition___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_serviceentrydefinition___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::__setslice__(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::__setslice__(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_serviceentrydefinition___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition___delslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_serviceentrydefinition___delslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_serviceentrydefinition___delslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition___delitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_serviceentrydefinition___delitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition___getitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_serviceentrydefinition___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_serviceentrydefinition___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_serviceentrydefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_serviceentrydefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____setitem____SWIG_0(arg1,arg2,(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_serviceentrydefinition___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition___delitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_serviceentrydefinition___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_serviceentrydefinition___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_serviceentrydefinition___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_serviceentrydefinition___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_serviceentrydefinition___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::__delitem__(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition___getitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_serviceentrydefinition___getitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type *) &std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____getitem____SWIG_1((std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_serviceentrydefinition___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_serviceentrydefinition___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_serviceentrydefinition___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_serviceentrydefinition___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::__getitem__(PySliceObject *)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::__getitem__(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type tempshared3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_serviceentrydefinition___setitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_serviceentrydefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg____setitem____SWIG_2(arg1,arg2,(boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_serviceentrydefinition___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_serviceentrydefinition___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_serviceentrydefinition___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_serviceentrydefinition___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_serviceentrydefinition___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::__setitem__(PySliceObject *,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::__setitem__(PySliceObject *)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::__setitem__(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_pop" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_serviceentrydefinition_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_append" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_serviceentrydefinition_append" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg__append(arg1,(boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_serviceentrydefinition__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_serviceentrydefinition__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vectorptr_serviceentrydefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vectorptr_serviceentrydefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >((std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_empty" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_size" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_serviceentrydefinition_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_swap" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_serviceentrydefinition_swap" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_serviceentrydefinition_swap" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_begin" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_end" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_rbegin" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_rend" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_clear" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_get_allocator" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::allocator_type(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_serviceentrydefinition__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorptr_serviceentrydefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type""'");
  } 
  arg1 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_pop_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_resize" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_serviceentrydefinition_resize" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_erase" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_serviceentrydefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_serviceentrydefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_erase" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_serviceentrydefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_serviceentrydefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_serviceentrydefinition_erase" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_serviceentrydefinition_erase" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_serviceentrydefinition_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vectorptr_serviceentrydefinition_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vectorptr_serviceentrydefinition_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_serviceentrydefinition_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::erase(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::erase(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_serviceentrydefinition__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type arg1 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type tempshared2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorptr_serviceentrydefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type""'");
  } 
  arg1 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type >(val1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_vectorptr_serviceentrydefinition" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >(arg1,(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_serviceentrydefinition(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vectorptr_serviceentrydefinition", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vectorptr_serviceentrydefinition__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vectorptr_serviceentrydefinition__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vectorptr_serviceentrydefinition__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_vectorptr_serviceentrydefinition__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vectorptr_serviceentrydefinition'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::vector()\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_serviceentrydefinition_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_push_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_serviceentrydefinition_push_back" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_front" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type *) &((std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type *) &((std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type tempshared3 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_serviceentrydefinition_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_assign" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_serviceentrydefinition_assign" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_serviceentrydefinition_assign" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type tempshared3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_resize" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_serviceentrydefinition_resize" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_serviceentrydefinition_resize" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_serviceentrydefinition_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_serviceentrydefinition_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_serviceentrydefinition_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_serviceentrydefinition_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::resize(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::resize(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type tempshared3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_insert" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_serviceentrydefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_serviceentrydefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator""'");
    }
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_serviceentrydefinition_insert" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg__insert__SWIG_0(arg1,arg2,(boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type arg3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  void *argp4 ;
  int res4 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type tempshared4 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_insert" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_serviceentrydefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_serviceentrydefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_serviceentrydefinition_insert" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type >(val3);
  {
    int newmem = 0;
    res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorptr_serviceentrydefinition_insert" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp4) tempshared4 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp4);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp4);
      arg4 = &tempshared4;
    } else {
      arg4 = (argp4) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type * >(argp4) : &tempshared4;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceEntryDefinition_Sg__Sg__insert__SWIG_1(arg1,arg2,arg3,(boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_serviceentrydefinition_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_serviceentrydefinition_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorptr_serviceentrydefinition_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_serviceentrydefinition_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::insert(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::insert(std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_serviceentrydefinition_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_reserve" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_serviceentrydefinition_reserve" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_serviceentrydefinition_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_serviceentrydefinition_capacity" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vectorptr_serviceentrydefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vectorptr_serviceentrydefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vectorptr_serviceentrydefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vectorptr_serviceentrydefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_iterator" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition___nonzero__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____nonzero__((std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition___bool__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____bool__((std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition___len__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____len__((std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_constantdefinition___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition___getslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_constantdefinition___getslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_constantdefinition___getslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition___setslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_constantdefinition___setslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_constantdefinition___setslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type arg3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition___setslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_constantdefinition___setslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_constantdefinition___setslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type >(val3);
  {
    std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorptr_constantdefinition___setslice__" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_constantdefinition___setslice__" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_constantdefinition___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vectorptr_constantdefinition___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorptr_constantdefinition___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_constantdefinition___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::__setslice__(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::__setslice__(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_constantdefinition___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition___delslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_constantdefinition___delslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_constantdefinition___delslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition___delitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_constantdefinition___delitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition___getitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_constantdefinition___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_constantdefinition___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_constantdefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_constantdefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____setitem____SWIG_0(arg1,arg2,(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_constantdefinition___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition___delitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_constantdefinition___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_constantdefinition___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_constantdefinition___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_constantdefinition___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_constantdefinition___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::__delitem__(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition___getitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_constantdefinition___getitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type *) &std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____getitem____SWIG_1((std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ConstantDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::ConstantDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_constantdefinition___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_constantdefinition___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_constantdefinition___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_constantdefinition___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::__getitem__(PySliceObject *)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::__getitem__(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type tempshared3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_constantdefinition___setitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_constantdefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg____setitem____SWIG_2(arg1,arg2,(boost::shared_ptr< RobotRaconteur::ConstantDefinition > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_constantdefinition___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_constantdefinition___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_constantdefinition___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_constantdefinition___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_constantdefinition___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::__setitem__(PySliceObject *,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::__setitem__(PySliceObject *)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::__setitem__(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_pop" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ConstantDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ConstantDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_constantdefinition_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_append" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_constantdefinition_append" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg__append(arg1,(boost::shared_ptr< RobotRaconteur::ConstantDefinition > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_constantdefinition__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_constantdefinition__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vectorptr_constantdefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vectorptr_constantdefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >((std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_empty" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_size" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_constantdefinition_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_swap" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_constantdefinition_swap" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_constantdefinition_swap" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_begin" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_end" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_rbegin" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_rend" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_clear" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_get_allocator" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::allocator_type(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_constantdefinition__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorptr_constantdefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type""'");
  } 
  arg1 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_pop_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_resize" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_constantdefinition_resize" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_erase" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_constantdefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_constantdefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_erase" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_constantdefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_constantdefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_constantdefinition_erase" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_constantdefinition_erase" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_constantdefinition_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vectorptr_constantdefinition_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vectorptr_constantdefinition_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_constantdefinition_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::erase(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::erase(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_constantdefinition__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type arg1 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type tempshared2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorptr_constantdefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type""'");
  } 
  arg1 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type >(val1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_vectorptr_constantdefinition" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >(arg1,(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_constantdefinition(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vectorptr_constantdefinition", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vectorptr_constantdefinition__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vectorptr_constantdefinition__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vectorptr_constantdefinition__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_vectorptr_constantdefinition__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vectorptr_constantdefinition'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::vector()\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_constantdefinition_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_push_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_constantdefinition_push_back" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_front" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type *) &((std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ConstantDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::ConstantDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type *) &((std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ConstantDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::ConstantDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type tempshared3 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_constantdefinition_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_assign" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_constantdefinition_assign" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_constantdefinition_assign" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type tempshared3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_resize" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_constantdefinition_resize" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_constantdefinition_resize" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_constantdefinition_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_constantdefinition_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_constantdefinition_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_constantdefinition_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::resize(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::resize(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type tempshared3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_insert" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_constantdefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_constantdefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator""'");
    }
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_constantdefinition_insert" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg__insert__SWIG_0(arg1,arg2,(boost::shared_ptr< RobotRaconteur::ConstantDefinition > const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type arg3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  void *argp4 ;
  int res4 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type tempshared4 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_insert" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_constantdefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_constantdefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_constantdefinition_insert" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type >(val3);
  {
    int newmem = 0;
    res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorptr_constantdefinition_insert" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp4) tempshared4 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp4);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp4);
      arg4 = &tempshared4;
    } else {
      arg4 = (argp4) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type * >(argp4) : &tempshared4;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ConstantDefinition_Sg__Sg__insert__SWIG_1(arg1,arg2,arg3,(boost::shared_ptr< RobotRaconteur::ConstantDefinition > const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_constantdefinition_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_constantdefinition_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorptr_constantdefinition_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_constantdefinition_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::insert(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::insert(std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_constantdefinition_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_reserve" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_constantdefinition_reserve" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_constantdefinition_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_constantdefinition_capacity" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vectorptr_constantdefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vectorptr_constantdefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vectorptr_constantdefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vectorptr_constantdefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_iterator" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition___nonzero__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____nonzero__((std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition___bool__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____bool__((std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition___len__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____len__((std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_enumdefinition___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition___getslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_enumdefinition___getslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_enumdefinition___getslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition___setslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_enumdefinition___setslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_enumdefinition___setslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type arg3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition___setslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_enumdefinition___setslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_enumdefinition___setslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type >(val3);
  {
    std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorptr_enumdefinition___setslice__" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_enumdefinition___setslice__" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_enumdefinition___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vectorptr_enumdefinition___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorptr_enumdefinition___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_enumdefinition___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::__setslice__(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::__setslice__(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_enumdefinition___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition___delslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_enumdefinition___delslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_enumdefinition___delslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition___delitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_enumdefinition___delitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition___getitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_enumdefinition___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_enumdefinition___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_enumdefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_enumdefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____setitem____SWIG_0(arg1,arg2,(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_enumdefinition___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition___delitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_enumdefinition___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_enumdefinition___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_enumdefinition___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_enumdefinition___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_enumdefinition___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::__delitem__(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition___getitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_enumdefinition___getitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type *) &std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____getitem____SWIG_1((std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::EnumDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::EnumDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_enumdefinition___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_enumdefinition___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_enumdefinition___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_enumdefinition___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::__getitem__(PySliceObject *)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::__getitem__(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type tempshared3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_enumdefinition___setitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_enumdefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg____setitem____SWIG_2(arg1,arg2,(boost::shared_ptr< RobotRaconteur::EnumDefinition > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_enumdefinition___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_enumdefinition___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_enumdefinition___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_enumdefinition___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_enumdefinition___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::__setitem__(PySliceObject *,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::__setitem__(PySliceObject *)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::__setitem__(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_pop" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::EnumDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::EnumDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_enumdefinition_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_append" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_enumdefinition_append" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg__append(arg1,(boost::shared_ptr< RobotRaconteur::EnumDefinition > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_enumdefinition__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_enumdefinition__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vectorptr_enumdefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vectorptr_enumdefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >((std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_empty" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_size" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_enumdefinition_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_swap" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_enumdefinition_swap" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_enumdefinition_swap" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_begin" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_end" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_rbegin" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_rend" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_clear" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_get_allocator" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::allocator_type(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_enumdefinition__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorptr_enumdefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type""'");
  } 
  arg1 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_pop_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_resize" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_enumdefinition_resize" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_erase" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_enumdefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_enumdefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_erase" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_enumdefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_enumdefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_enumdefinition_erase" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_enumdefinition_erase" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_enumdefinition_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vectorptr_enumdefinition_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vectorptr_enumdefinition_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_enumdefinition_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::erase(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::erase(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_enumdefinition__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type arg1 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type tempshared2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorptr_enumdefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type""'");
  } 
  arg1 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type >(val1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_vectorptr_enumdefinition" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >(arg1,(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_enumdefinition(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vectorptr_enumdefinition", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vectorptr_enumdefinition__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vectorptr_enumdefinition__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vectorptr_enumdefinition__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_vectorptr_enumdefinition__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vectorptr_enumdefinition'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::vector()\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_enumdefinition_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_push_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_enumdefinition_push_back" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_front" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type *) &((std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::EnumDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::EnumDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type *) &((std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::EnumDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::EnumDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type tempshared3 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_enumdefinition_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_assign" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_enumdefinition_assign" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_enumdefinition_assign" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type tempshared3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_resize" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_enumdefinition_resize" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_enumdefinition_resize" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_enumdefinition_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_enumdefinition_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_enumdefinition_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_enumdefinition_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::resize(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::resize(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type tempshared3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_insert" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_enumdefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_enumdefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator""'");
    }
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_enumdefinition_insert" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg__insert__SWIG_0(arg1,arg2,(boost::shared_ptr< RobotRaconteur::EnumDefinition > const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type arg3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  void *argp4 ;
  int res4 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type tempshared4 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_insert" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_enumdefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_enumdefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_enumdefinition_insert" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type >(val3);
  {
    int newmem = 0;
    res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorptr_enumdefinition_insert" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp4) tempshared4 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp4);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp4);
      arg4 = &tempshared4;
    } else {
      arg4 = (argp4) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type * >(argp4) : &tempshared4;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_EnumDefinition_Sg__Sg__insert__SWIG_1(arg1,arg2,arg3,(boost::shared_ptr< RobotRaconteur::EnumDefinition > const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_enumdefinition_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_enumdefinition_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorptr_enumdefinition_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_enumdefinition_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::insert(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::insert(std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_enumdefinition_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_reserve" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_enumdefinition_reserve" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_enumdefinition_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_enumdefinition_capacity" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vectorptr_enumdefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vectorptr_enumdefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vectorptr_enumdefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vectorptr_enumdefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_iterator" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield___nonzero__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____nonzero__((std::vector< RobotRaconteur::ConstantDefinition_StructField > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield___bool__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____bool__((std::vector< RobotRaconteur::ConstantDefinition_StructField > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield___len__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____len__((std::vector< RobotRaconteur::ConstantDefinition_StructField > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type arg2 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_constantdefinition_structfield___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield___getslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_constantdefinition_structfield___getslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_constantdefinition_structfield___getslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > *)std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type arg2 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield___setslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_constantdefinition_structfield___setslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_constantdefinition_structfield___setslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type arg2 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type arg3 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield___setslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_constantdefinition_structfield___setslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_constantdefinition_structfield___setslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type >(val3);
  {
    std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > *ptr = (std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vector_constantdefinition_structfield___setslice__" "', argument " "4"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_constantdefinition_structfield___setslice__" "', argument " "4"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_constantdefinition_structfield___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vector_constantdefinition_structfield___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vector_constantdefinition_structfield___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_constantdefinition_structfield___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ConstantDefinition_StructField >::__setslice__(std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type,std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type)\n"
    "    std::vector< RobotRaconteur::ConstantDefinition_StructField >::__setslice__(std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type,std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type,std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type arg2 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_constantdefinition_structfield___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield___delslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_constantdefinition_structfield___delslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_constantdefinition_structfield___delslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield___delitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_constantdefinition_structfield___delitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield___getitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_constantdefinition_structfield___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > *)std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_constantdefinition_structfield___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > *ptr = (std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_constantdefinition_structfield___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_constantdefinition_structfield___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____setitem____SWIG_0(arg1,arg2,(std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_constantdefinition_structfield___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield___delitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_constantdefinition_structfield___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_constantdefinition_structfield___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_constantdefinition_structfield___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_constantdefinition_structfield___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_constantdefinition_structfield___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ConstantDefinition_StructField >::__delitem__(std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type)\n"
    "    std::vector< RobotRaconteur::ConstantDefinition_StructField >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield___getitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_constantdefinition_structfield___getitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type *) &std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____getitem____SWIG_1((std::vector< RobotRaconteur::ConstantDefinition_StructField > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_constantdefinition_structfield___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_constantdefinition_structfield___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_constantdefinition_structfield___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_constantdefinition_structfield___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ConstantDefinition_StructField >::__getitem__(PySliceObject *)\n"
    "    std::vector< RobotRaconteur::ConstantDefinition_StructField >::__getitem__(std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type arg2 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_constantdefinition_structfield___setitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_constantdefinition_structfield___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_constantdefinition_structfield___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type * >(argp3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg____setitem____SWIG_2(arg1,arg2,(RobotRaconteur::ConstantDefinition_StructField const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_constantdefinition_structfield___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_constantdefinition_structfield___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vector_constantdefinition_structfield___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vector_constantdefinition_structfield___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_constantdefinition_structfield___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ConstantDefinition_StructField >::__setitem__(PySliceObject *,std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > const &)\n"
    "    std::vector< RobotRaconteur::ConstantDefinition_StructField >::__setitem__(PySliceObject *)\n"
    "    std::vector< RobotRaconteur::ConstantDefinition_StructField >::__setitem__(std::vector< RobotRaconteur::ConstantDefinition_StructField >::difference_type,std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_pop" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type(static_cast< const std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type& >(result))), SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_constantdefinition_structfield_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_append" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vector_constantdefinition_structfield_append" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_constantdefinition_structfield_append" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg__append(arg1,(RobotRaconteur::ConstantDefinition_StructField const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_constantdefinition_structfield__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *)new std::vector< RobotRaconteur::ConstantDefinition_StructField >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_constantdefinition_structfield__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > *ptr = (std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vector_constantdefinition_structfield" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vector_constantdefinition_structfield" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *)new std::vector< RobotRaconteur::ConstantDefinition_StructField >((std::vector< RobotRaconteur::ConstantDefinition_StructField > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_empty" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< RobotRaconteur::ConstantDefinition_StructField > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_size" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::ConstantDefinition_StructField > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_constantdefinition_structfield_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_swap" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vector_constantdefinition_structfield_swap" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_constantdefinition_structfield_swap" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_begin" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_end" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_rbegin" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ConstantDefinition_StructField >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_rend" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ConstantDefinition_StructField >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_clear" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< RobotRaconteur::ConstantDefinition_StructField > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_get_allocator" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::ConstantDefinition_StructField > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< RobotRaconteur::ConstantDefinition_StructField >::allocator_type(static_cast< const std::vector< RobotRaconteur::ConstantDefinition_StructField >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_constantdefinition_structfield__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vector_constantdefinition_structfield" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type""'");
  } 
  arg1 = static_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *)new std::vector< RobotRaconteur::ConstantDefinition_StructField >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_pop_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_resize" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_constantdefinition_structfield_resize" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_erase" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_constantdefinition_structfield_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_constantdefinition_structfield_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator arg2 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_erase" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_constantdefinition_structfield_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_constantdefinition_structfield_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_constantdefinition_structfield_erase" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_constantdefinition_structfield_erase" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_constantdefinition_structfield_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vector_constantdefinition_structfield_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vector_constantdefinition_structfield_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_constantdefinition_structfield_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ConstantDefinition_StructField >::erase(std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator)\n"
    "    std::vector< RobotRaconteur::ConstantDefinition_StructField >::erase(std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator,std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vector_constantdefinition_structfield__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type arg1 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vector_constantdefinition_structfield" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type""'");
  } 
  arg1 = static_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type >(val1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_vector_constantdefinition_structfield" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vector_constantdefinition_structfield" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *)new std::vector< RobotRaconteur::ConstantDefinition_StructField >(arg1,(std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_constantdefinition_structfield(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vector_constantdefinition_structfield", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vector_constantdefinition_structfield__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vector_constantdefinition_structfield__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vector_constantdefinition_structfield__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField, SWIG_POINTER_NO_NULL | 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_vector_constantdefinition_structfield__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vector_constantdefinition_structfield'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ConstantDefinition_StructField >::vector()\n"
    "    std::vector< RobotRaconteur::ConstantDefinition_StructField >::vector(std::vector< RobotRaconteur::ConstantDefinition_StructField > const &)\n"
    "    std::vector< RobotRaconteur::ConstantDefinition_StructField >::vector(std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type)\n"
    "    std::vector< RobotRaconteur::ConstantDefinition_StructField >::vector(std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type,std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_constantdefinition_structfield_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_push_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vector_constantdefinition_structfield_push_back" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_constantdefinition_structfield_push_back" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_front" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type *) &((std::vector< RobotRaconteur::ConstantDefinition_StructField > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type *) &((std::vector< RobotRaconteur::ConstantDefinition_StructField > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type arg2 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_constantdefinition_structfield_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_assign" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_constantdefinition_structfield_assign" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_constantdefinition_structfield_assign" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_constantdefinition_structfield_assign" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type arg2 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_resize" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_constantdefinition_structfield_resize" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_constantdefinition_structfield_resize" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_constantdefinition_structfield_resize" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_constantdefinition_structfield_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_constantdefinition_structfield_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vector_constantdefinition_structfield_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_constantdefinition_structfield_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ConstantDefinition_StructField >::resize(std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type)\n"
    "    std::vector< RobotRaconteur::ConstantDefinition_StructField >::resize(std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type,std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator arg2 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_insert" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_constantdefinition_structfield_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_constantdefinition_structfield_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_constantdefinition_structfield_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_constantdefinition_structfield_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg__insert__SWIG_0(arg1,arg2,(RobotRaconteur::ConstantDefinition_StructField const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator arg2 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type arg3 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  void *argp4 = 0 ;
  int res4 = 0 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_insert" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_constantdefinition_structfield_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_constantdefinition_structfield_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_constantdefinition_structfield_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type >(val3);
  res4 = SWIG_ConvertPtr(swig_obj[3], &argp4, SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField,  0  | 0);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vector_constantdefinition_structfield_insert" "', argument " "4"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &""'"); 
  }
  if (!argp4) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_constantdefinition_structfield_insert" "', argument " "4"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &""'"); 
  }
  arg4 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type * >(argp4);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_RobotRaconteur_ConstantDefinition_StructField_Sg__insert__SWIG_1(arg1,arg2,arg3,(RobotRaconteur::ConstantDefinition_StructField const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_constantdefinition_structfield_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator > *>(iter) != 0));
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vector_constantdefinition_structfield_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField, SWIG_POINTER_NO_NULL | 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vector_constantdefinition_structfield_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_constantdefinition_structfield_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ConstantDefinition_StructField >::insert(std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator,std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &)\n"
    "    std::vector< RobotRaconteur::ConstantDefinition_StructField >::insert(std::vector< RobotRaconteur::ConstantDefinition_StructField >::iterator,std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type,std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_constantdefinition_structfield_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_reserve" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_constantdefinition_structfield_reserve" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_constantdefinition_structfield_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_constantdefinition_structfield_capacity" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::ConstantDefinition_StructField > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vector_constantdefinition_structfield(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ConstantDefinition_StructField > *arg1 = (std::vector< RobotRaconteur::ConstantDefinition_StructField > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vector_constantdefinition_structfield" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ConstantDefinition_StructField > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vector_constantdefinition_structfield_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vector_constantdefinition_structfield_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_iterator" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues___nonzero__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____nonzero__((std::vector< RobotRaconteur::EnumDefinitionValue > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues___bool__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____bool__((std::vector< RobotRaconteur::EnumDefinitionValue > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues___len__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____len__((std::vector< RobotRaconteur::EnumDefinitionValue > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type arg2 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_enumdefinitionvalues___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues___getslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_enumdefinitionvalues___getslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_enumdefinitionvalues___getslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > *)std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type arg2 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues___setslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_enumdefinitionvalues___setslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_enumdefinitionvalues___setslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type arg2 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type arg3 ;
  std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues___setslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_enumdefinitionvalues___setslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_enumdefinitionvalues___setslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type >(val3);
  {
    std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > *ptr = (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vector_enumdefinitionvalues___setslice__" "', argument " "4"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_enumdefinitionvalues___setslice__" "', argument " "4"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_enumdefinitionvalues___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vector_enumdefinitionvalues___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vector_enumdefinitionvalues___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_enumdefinitionvalues___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::EnumDefinitionValue >::__setslice__(std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type,std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type)\n"
    "    std::vector< RobotRaconteur::EnumDefinitionValue >::__setslice__(std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type,std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type,std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type arg2 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_enumdefinitionvalues___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues___delslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_enumdefinitionvalues___delslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_enumdefinitionvalues___delslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues___delitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_enumdefinitionvalues___delitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues___getitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_enumdefinitionvalues___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > *)std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_enumdefinitionvalues___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > *ptr = (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_enumdefinitionvalues___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_enumdefinitionvalues___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____setitem____SWIG_0(arg1,arg2,(std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_enumdefinitionvalues___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues___delitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_enumdefinitionvalues___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_enumdefinitionvalues___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_enumdefinitionvalues___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_enumdefinitionvalues___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_enumdefinitionvalues___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::EnumDefinitionValue >::__delitem__(std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type)\n"
    "    std::vector< RobotRaconteur::EnumDefinitionValue >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues___getitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_enumdefinitionvalues___getitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::EnumDefinitionValue >::value_type *) &std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____getitem____SWIG_1((std::vector< RobotRaconteur::EnumDefinitionValue > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::EnumDefinitionValue >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_enumdefinitionvalues___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_enumdefinitionvalues___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_enumdefinitionvalues___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_enumdefinitionvalues___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::EnumDefinitionValue >::__getitem__(PySliceObject *)\n"
    "    std::vector< RobotRaconteur::EnumDefinitionValue >::__getitem__(std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type arg2 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_enumdefinitionvalues___setitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_enumdefinitionvalues___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_enumdefinitionvalues___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::value_type * >(argp3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg____setitem____SWIG_2(arg1,arg2,(RobotRaconteur::EnumDefinitionValue const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_enumdefinitionvalues___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_enumdefinitionvalues___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vector_enumdefinitionvalues___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vector_enumdefinitionvalues___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_enumdefinitionvalues___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::EnumDefinitionValue >::__setitem__(PySliceObject *,std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > const &)\n"
    "    std::vector< RobotRaconteur::EnumDefinitionValue >::__setitem__(PySliceObject *)\n"
    "    std::vector< RobotRaconteur::EnumDefinitionValue >::__setitem__(std::vector< RobotRaconteur::EnumDefinitionValue >::difference_type,std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_pop" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< RobotRaconteur::EnumDefinitionValue >::value_type(static_cast< const std::vector< RobotRaconteur::EnumDefinitionValue >::value_type& >(result))), SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_enumdefinitionvalues_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_append" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vector_enumdefinitionvalues_append" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_enumdefinitionvalues_append" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg__append(arg1,(RobotRaconteur::EnumDefinitionValue const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_enumdefinitionvalues__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::EnumDefinitionValue > *)new std::vector< RobotRaconteur::EnumDefinitionValue >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_enumdefinitionvalues__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< RobotRaconteur::EnumDefinitionValue > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > *ptr = (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vector_enumdefinitionvalues" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vector_enumdefinitionvalues" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::EnumDefinitionValue > *)new std::vector< RobotRaconteur::EnumDefinitionValue >((std::vector< RobotRaconteur::EnumDefinitionValue > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_empty" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< RobotRaconteur::EnumDefinitionValue > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_size" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::EnumDefinitionValue > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_enumdefinitionvalues_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_swap" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vector_enumdefinitionvalues_swap" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_enumdefinitionvalues_swap" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_begin" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::EnumDefinitionValue >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_end" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::EnumDefinitionValue >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_rbegin" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::EnumDefinitionValue >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_rend" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::EnumDefinitionValue >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_clear" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< RobotRaconteur::EnumDefinitionValue > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_get_allocator" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::EnumDefinitionValue > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< RobotRaconteur::EnumDefinitionValue >::allocator_type(static_cast< const std::vector< RobotRaconteur::EnumDefinitionValue >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_enumdefinitionvalues__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vector_enumdefinitionvalues" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::size_type""'");
  } 
  arg1 = static_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::EnumDefinitionValue > *)new std::vector< RobotRaconteur::EnumDefinitionValue >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_pop_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_resize" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_enumdefinitionvalues_resize" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::iterator result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_erase" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_enumdefinitionvalues_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::EnumDefinitionValue >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::EnumDefinitionValue >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_enumdefinitionvalues_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::EnumDefinitionValue >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::iterator arg2 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_erase" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_enumdefinitionvalues_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::EnumDefinitionValue >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::EnumDefinitionValue >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_enumdefinitionvalues_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_enumdefinitionvalues_erase" "', argument " "3"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::EnumDefinitionValue >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::EnumDefinitionValue >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_enumdefinitionvalues_erase" "', argument " "3"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::EnumDefinitionValue >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_enumdefinitionvalues_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::EnumDefinitionValue >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vector_enumdefinitionvalues_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::EnumDefinitionValue >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::EnumDefinitionValue >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vector_enumdefinitionvalues_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_enumdefinitionvalues_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::EnumDefinitionValue >::erase(std::vector< RobotRaconteur::EnumDefinitionValue >::iterator)\n"
    "    std::vector< RobotRaconteur::EnumDefinitionValue >::erase(std::vector< RobotRaconteur::EnumDefinitionValue >::iterator,std::vector< RobotRaconteur::EnumDefinitionValue >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vector_enumdefinitionvalues__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue >::size_type arg1 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vector_enumdefinitionvalues" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::size_type""'");
  } 
  arg1 = static_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::size_type >(val1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_vector_enumdefinitionvalues" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vector_enumdefinitionvalues" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::EnumDefinitionValue > *)new std::vector< RobotRaconteur::EnumDefinitionValue >(arg1,(std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_enumdefinitionvalues(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vector_enumdefinitionvalues", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vector_enumdefinitionvalues__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vector_enumdefinitionvalues__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vector_enumdefinitionvalues__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue, SWIG_POINTER_NO_NULL | 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_vector_enumdefinitionvalues__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vector_enumdefinitionvalues'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::EnumDefinitionValue >::vector()\n"
    "    std::vector< RobotRaconteur::EnumDefinitionValue >::vector(std::vector< RobotRaconteur::EnumDefinitionValue > const &)\n"
    "    std::vector< RobotRaconteur::EnumDefinitionValue >::vector(std::vector< RobotRaconteur::EnumDefinitionValue >::size_type)\n"
    "    std::vector< RobotRaconteur::EnumDefinitionValue >::vector(std::vector< RobotRaconteur::EnumDefinitionValue >::size_type,std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_enumdefinitionvalues_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_push_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vector_enumdefinitionvalues_push_back" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_enumdefinitionvalues_push_back" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_front" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::EnumDefinitionValue >::value_type *) &((std::vector< RobotRaconteur::EnumDefinitionValue > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::EnumDefinitionValue >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::EnumDefinitionValue >::value_type *) &((std::vector< RobotRaconteur::EnumDefinitionValue > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::EnumDefinitionValue >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::size_type arg2 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_enumdefinitionvalues_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_assign" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_enumdefinitionvalues_assign" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::size_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_enumdefinitionvalues_assign" "', argument " "3"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_enumdefinitionvalues_assign" "', argument " "3"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::size_type arg2 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_resize" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_enumdefinitionvalues_resize" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::size_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_enumdefinitionvalues_resize" "', argument " "3"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_enumdefinitionvalues_resize" "', argument " "3"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_enumdefinitionvalues_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_enumdefinitionvalues_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vector_enumdefinitionvalues_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_enumdefinitionvalues_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::EnumDefinitionValue >::resize(std::vector< RobotRaconteur::EnumDefinitionValue >::size_type)\n"
    "    std::vector< RobotRaconteur::EnumDefinitionValue >::resize(std::vector< RobotRaconteur::EnumDefinitionValue >::size_type,std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::iterator arg2 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_insert" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_enumdefinitionvalues_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::EnumDefinitionValue >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::EnumDefinitionValue >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_enumdefinitionvalues_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_enumdefinitionvalues_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_enumdefinitionvalues_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg__insert__SWIG_0(arg1,arg2,(RobotRaconteur::EnumDefinitionValue const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::EnumDefinitionValue >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::iterator arg2 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::size_type arg3 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  void *argp4 = 0 ;
  int res4 = 0 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_insert" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_enumdefinitionvalues_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::EnumDefinitionValue >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::EnumDefinitionValue >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_enumdefinitionvalues_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_enumdefinitionvalues_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::size_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::size_type >(val3);
  res4 = SWIG_ConvertPtr(swig_obj[3], &argp4, SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue,  0  | 0);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vector_enumdefinitionvalues_insert" "', argument " "4"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &""'"); 
  }
  if (!argp4) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_enumdefinitionvalues_insert" "', argument " "4"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &""'"); 
  }
  arg4 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::value_type * >(argp4);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_RobotRaconteur_EnumDefinitionValue_Sg__insert__SWIG_1(arg1,arg2,arg3,(RobotRaconteur::EnumDefinitionValue const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_enumdefinitionvalues_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::EnumDefinitionValue >::iterator > *>(iter) != 0));
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vector_enumdefinitionvalues_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::EnumDefinitionValue >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue, SWIG_POINTER_NO_NULL | 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vector_enumdefinitionvalues_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_enumdefinitionvalues_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::EnumDefinitionValue >::insert(std::vector< RobotRaconteur::EnumDefinitionValue >::iterator,std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &)\n"
    "    std::vector< RobotRaconteur::EnumDefinitionValue >::insert(std::vector< RobotRaconteur::EnumDefinitionValue >::iterator,std::vector< RobotRaconteur::EnumDefinitionValue >::size_type,std::vector< RobotRaconteur::EnumDefinitionValue >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_enumdefinitionvalues_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_reserve" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_enumdefinitionvalues_reserve" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::EnumDefinitionValue >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_enumdefinitionvalues_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::EnumDefinitionValue >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_enumdefinitionvalues_capacity" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::EnumDefinitionValue > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vector_enumdefinitionvalues(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::EnumDefinitionValue > *arg1 = (std::vector< RobotRaconteur::EnumDefinitionValue > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vector_enumdefinitionvalues" "', argument " "1"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vector_enumdefinitionvalues_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vector_enumdefinitionvalues_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_vector_usingdefinition_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_iterator" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition___nonzero__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____nonzero__((std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition___bool__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____bool__((std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition___len__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____len__((std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_usingdefinition___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition___getslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_usingdefinition___getslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_usingdefinition___getslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition___setslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_usingdefinition___setslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_usingdefinition___setslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type arg3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition___setslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_usingdefinition___setslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_usingdefinition___setslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type >(val3);
  {
    std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vector_usingdefinition___setslice__" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_usingdefinition___setslice__" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_usingdefinition___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vector_usingdefinition___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vector_usingdefinition___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_usingdefinition___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::__setslice__(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::__setslice__(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_usingdefinition___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition___delslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_usingdefinition___delslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_usingdefinition___delslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition___delitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_usingdefinition___delitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition___getitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_usingdefinition___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_usingdefinition___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_usingdefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_usingdefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____setitem____SWIG_0(arg1,arg2,(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_usingdefinition___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition___delitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_usingdefinition___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_usingdefinition___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_usingdefinition___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_usingdefinition___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_usingdefinition___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::__delitem__(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition___getitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_usingdefinition___getitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type *) &std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____getitem____SWIG_1((std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::UsingDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::UsingDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_usingdefinition___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_usingdefinition___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_usingdefinition___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_usingdefinition___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::__getitem__(PySliceObject *)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::__getitem__(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type tempshared3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_usingdefinition___setitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_usingdefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg____setitem____SWIG_2(arg1,arg2,(boost::shared_ptr< RobotRaconteur::UsingDefinition > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_usingdefinition___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_usingdefinition___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vector_usingdefinition___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vector_usingdefinition___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_usingdefinition___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::__setitem__(PySliceObject *,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::__setitem__(PySliceObject *)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::__setitem__(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_pop" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::UsingDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::UsingDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_usingdefinition_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_append" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vector_usingdefinition_append" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg__append(arg1,(boost::shared_ptr< RobotRaconteur::UsingDefinition > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_usingdefinition__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_usingdefinition__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vector_usingdefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vector_usingdefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >((std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_empty" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_size" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_usingdefinition_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_swap" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vector_usingdefinition_swap" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_usingdefinition_swap" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_begin" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_end" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_rbegin" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_rend" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_clear" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_get_allocator" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::allocator_type(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_usingdefinition__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vector_usingdefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type""'");
  } 
  arg1 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_pop_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_resize" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_usingdefinition_resize" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_erase" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_usingdefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_usingdefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_erase" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_usingdefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_usingdefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_usingdefinition_erase" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_usingdefinition_erase" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_usingdefinition_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vector_usingdefinition_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vector_usingdefinition_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_usingdefinition_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::erase(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::erase(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vector_usingdefinition__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type arg1 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type tempshared2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vector_usingdefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type""'");
  } 
  arg1 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type >(val1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_vector_usingdefinition" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >(arg1,(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_usingdefinition(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vector_usingdefinition", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vector_usingdefinition__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vector_usingdefinition__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vector_usingdefinition__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_vector_usingdefinition__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vector_usingdefinition'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::vector()\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_usingdefinition_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_push_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vector_usingdefinition_push_back" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_front" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type *) &((std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::UsingDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::UsingDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type *) &((std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::UsingDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::UsingDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type tempshared3 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_usingdefinition_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_assign" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_usingdefinition_assign" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_usingdefinition_assign" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type tempshared3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_resize" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_usingdefinition_resize" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_usingdefinition_resize" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_usingdefinition_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_usingdefinition_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vector_usingdefinition_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_usingdefinition_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::resize(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::resize(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type tempshared3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_insert" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_usingdefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_usingdefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator""'");
    }
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_usingdefinition_insert" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg__insert__SWIG_0(arg1,arg2,(boost::shared_ptr< RobotRaconteur::UsingDefinition > const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type arg3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  void *argp4 ;
  int res4 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type tempshared4 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_insert" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_usingdefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_usingdefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_usingdefinition_insert" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type >(val3);
  {
    int newmem = 0;
    res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vector_usingdefinition_insert" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp4) tempshared4 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp4);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp4);
      arg4 = &tempshared4;
    } else {
      arg4 = (argp4) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type * >(argp4) : &tempshared4;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_UsingDefinition_Sg__Sg__insert__SWIG_1(arg1,arg2,arg3,(boost::shared_ptr< RobotRaconteur::UsingDefinition > const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_usingdefinition_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vector_usingdefinition_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vector_usingdefinition_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_usingdefinition_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::insert(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::insert(std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_usingdefinition_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_reserve" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_usingdefinition_reserve" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_usingdefinition_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_usingdefinition_capacity" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vector_usingdefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vector_usingdefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vector_usingdefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vector_usingdefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_iterator" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition___nonzero__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____nonzero__((std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition___bool__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____bool__((std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition___len__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____len__((std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_exceptiondefinition___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition___getslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_exceptiondefinition___getslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_exceptiondefinition___getslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition___setslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_exceptiondefinition___setslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_exceptiondefinition___setslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type arg3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition___setslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_exceptiondefinition___setslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_exceptiondefinition___setslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type >(val3);
  {
    std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorptr_exceptiondefinition___setslice__" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_exceptiondefinition___setslice__" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_exceptiondefinition___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vectorptr_exceptiondefinition___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorptr_exceptiondefinition___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_exceptiondefinition___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::__setslice__(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::__setslice__(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_exceptiondefinition___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition___delslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_exceptiondefinition___delslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_exceptiondefinition___delslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition___delitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_exceptiondefinition___delitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition___getitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_exceptiondefinition___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_exceptiondefinition___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_exceptiondefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_exceptiondefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____setitem____SWIG_0(arg1,arg2,(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_exceptiondefinition___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition___delitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_exceptiondefinition___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_exceptiondefinition___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_exceptiondefinition___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_exceptiondefinition___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_exceptiondefinition___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::__delitem__(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition___getitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_exceptiondefinition___getitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type *) &std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____getitem____SWIG_1((std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::ExceptionDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_exceptiondefinition___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_exceptiondefinition___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_exceptiondefinition___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_exceptiondefinition___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::__getitem__(PySliceObject *)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::__getitem__(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type tempshared3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_exceptiondefinition___setitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_exceptiondefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg____setitem____SWIG_2(arg1,arg2,(boost::shared_ptr< RobotRaconteur::ExceptionDefinition > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_exceptiondefinition___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_exceptiondefinition___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_exceptiondefinition___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_exceptiondefinition___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_exceptiondefinition___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::__setitem__(PySliceObject *,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::__setitem__(PySliceObject *)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::__setitem__(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_pop" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ExceptionDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_exceptiondefinition_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_append" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_exceptiondefinition_append" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg__append(arg1,(boost::shared_ptr< RobotRaconteur::ExceptionDefinition > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_exceptiondefinition__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_exceptiondefinition__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vectorptr_exceptiondefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vectorptr_exceptiondefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >((std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_empty" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_size" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_exceptiondefinition_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_swap" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_exceptiondefinition_swap" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_exceptiondefinition_swap" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_begin" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_end" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_rbegin" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_rend" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_clear" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_get_allocator" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::allocator_type(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_exceptiondefinition__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorptr_exceptiondefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type""'");
  } 
  arg1 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_pop_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_resize" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_exceptiondefinition_resize" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_erase" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_exceptiondefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_exceptiondefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_erase" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_exceptiondefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_exceptiondefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_exceptiondefinition_erase" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_exceptiondefinition_erase" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_exceptiondefinition_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vectorptr_exceptiondefinition_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vectorptr_exceptiondefinition_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_exceptiondefinition_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::erase(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::erase(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_exceptiondefinition__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type arg1 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type tempshared2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorptr_exceptiondefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type""'");
  } 
  arg1 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type >(val1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_vectorptr_exceptiondefinition" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >(arg1,(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_exceptiondefinition(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vectorptr_exceptiondefinition", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vectorptr_exceptiondefinition__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vectorptr_exceptiondefinition__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vectorptr_exceptiondefinition__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_vectorptr_exceptiondefinition__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vectorptr_exceptiondefinition'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::vector()\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_exceptiondefinition_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_push_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_exceptiondefinition_push_back" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_front" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type *) &((std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::ExceptionDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type *) &((std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::ExceptionDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type tempshared3 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_exceptiondefinition_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_assign" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_exceptiondefinition_assign" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_exceptiondefinition_assign" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type tempshared3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_resize" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_exceptiondefinition_resize" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_exceptiondefinition_resize" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_exceptiondefinition_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_exceptiondefinition_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_exceptiondefinition_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_exceptiondefinition_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::resize(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::resize(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type tempshared3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_insert" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_exceptiondefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_exceptiondefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator""'");
    }
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_exceptiondefinition_insert" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg__insert__SWIG_0(arg1,arg2,(boost::shared_ptr< RobotRaconteur::ExceptionDefinition > const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type arg3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  void *argp4 ;
  int res4 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type tempshared4 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_insert" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_exceptiondefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_exceptiondefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_exceptiondefinition_insert" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type >(val3);
  {
    int newmem = 0;
    res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorptr_exceptiondefinition_insert" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp4) tempshared4 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp4);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp4);
      arg4 = &tempshared4;
    } else {
      arg4 = (argp4) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type * >(argp4) : &tempshared4;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ExceptionDefinition_Sg__Sg__insert__SWIG_1(arg1,arg2,arg3,(boost::shared_ptr< RobotRaconteur::ExceptionDefinition > const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_exceptiondefinition_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_exceptiondefinition_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorptr_exceptiondefinition_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_exceptiondefinition_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::insert(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::insert(std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_exceptiondefinition_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_reserve" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_exceptiondefinition_reserve" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_exceptiondefinition_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_exceptiondefinition_capacity" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vectorptr_exceptiondefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vectorptr_exceptiondefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vectorptr_exceptiondefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vectorptr_exceptiondefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_iterator" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition___nonzero__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____nonzero__((std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition___bool__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____bool__((std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition___len__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____len__((std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_servicedefinition___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition___getslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_servicedefinition___getslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_servicedefinition___getslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition___setslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_servicedefinition___setslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_servicedefinition___setslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type arg3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition___setslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_servicedefinition___setslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_servicedefinition___setslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type >(val3);
  {
    std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorptr_servicedefinition___setslice__" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_servicedefinition___setslice__" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_servicedefinition___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vectorptr_servicedefinition___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorptr_servicedefinition___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_servicedefinition___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::__setslice__(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::__setslice__(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_servicedefinition___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition___delslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_servicedefinition___delslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_servicedefinition___delslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition___delitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_servicedefinition___delitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition___getitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_servicedefinition___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_servicedefinition___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_servicedefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_servicedefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____setitem____SWIG_0(arg1,arg2,(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_servicedefinition___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition___delitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_servicedefinition___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_servicedefinition___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_servicedefinition___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_servicedefinition___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_servicedefinition___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::__delitem__(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition___getitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_servicedefinition___getitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type *) &std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____getitem____SWIG_1((std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServiceDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::ServiceDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_servicedefinition___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_servicedefinition___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_servicedefinition___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_servicedefinition___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::__getitem__(PySliceObject *)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::__getitem__(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type tempshared3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_servicedefinition___setitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_servicedefinition___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg____setitem____SWIG_2(arg1,arg2,(boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_servicedefinition___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_servicedefinition___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_servicedefinition___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_servicedefinition___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_servicedefinition___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::__setitem__(PySliceObject *,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::__setitem__(PySliceObject *)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::__setitem__(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_pop" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServiceDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ServiceDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_servicedefinition_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_append" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_servicedefinition_append" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg__append(arg1,(boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_servicedefinition__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_servicedefinition__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vectorptr_servicedefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vectorptr_servicedefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >((std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_empty" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_size" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_servicedefinition_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_swap" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_servicedefinition_swap" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_servicedefinition_swap" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_begin" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_end" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_rbegin" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_rend" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_clear" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_get_allocator" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::allocator_type(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_servicedefinition__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorptr_servicedefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type""'");
  } 
  arg1 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_pop_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_resize" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_servicedefinition_resize" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_erase" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_servicedefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_servicedefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_erase" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_servicedefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_servicedefinition_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_servicedefinition_erase" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_servicedefinition_erase" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_servicedefinition_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vectorptr_servicedefinition_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vectorptr_servicedefinition_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_servicedefinition_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::erase(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::erase(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_servicedefinition__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type arg1 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type tempshared2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorptr_servicedefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type""'");
  } 
  arg1 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type >(val1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_vectorptr_servicedefinition" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *)new std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >(arg1,(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_servicedefinition(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vectorptr_servicedefinition", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vectorptr_servicedefinition__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vectorptr_servicedefinition__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vectorptr_servicedefinition__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_vectorptr_servicedefinition__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vectorptr_servicedefinition'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::vector()\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_servicedefinition_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_push_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_servicedefinition_push_back" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_front" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type *) &((std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServiceDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::ServiceDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type *) &((std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServiceDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::ServiceDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type tempshared3 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_servicedefinition_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_assign" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_servicedefinition_assign" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_servicedefinition_assign" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type tempshared3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_resize" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_servicedefinition_resize" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_servicedefinition_resize" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_servicedefinition_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_servicedefinition_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_servicedefinition_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_servicedefinition_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::resize(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::resize(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type tempshared3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_insert" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_servicedefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_servicedefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator""'");
    }
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_servicedefinition_insert" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg__insert__SWIG_0(arg1,arg2,(boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type arg3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  void *argp4 ;
  int res4 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type tempshared4 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_insert" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_servicedefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_servicedefinition_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_servicedefinition_insert" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type >(val3);
  {
    int newmem = 0;
    res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorptr_servicedefinition_insert" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp4) tempshared4 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp4);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp4);
      arg4 = &tempshared4;
    } else {
      arg4 = (argp4) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type * >(argp4) : &tempshared4;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_ServiceDefinition_Sg__Sg__insert__SWIG_1(arg1,arg2,arg3,(boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_servicedefinition_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_servicedefinition_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorptr_servicedefinition_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_servicedefinition_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::insert(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::insert(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_servicedefinition_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_reserve" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_servicedefinition_reserve" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_servicedefinition_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_servicedefinition_capacity" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vectorptr_servicedefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vectorptr_servicedefinition" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vectorptr_servicedefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vectorptr_servicedefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_ServiceDefinitionParseInfo_ServiceName_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinitionParseInfo *arg1 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceDefinitionParseInfo_ServiceName_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinitionParseInfo_ServiceName_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceDefinitionParseInfo_ServiceName_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceDefinitionParseInfo_ServiceName_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ServiceName = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinitionParseInfo_ServiceName_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinitionParseInfo *arg1 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinitionParseInfo_ServiceName_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->ServiceName);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinitionParseInfo_RobDefFilePath_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinitionParseInfo *arg1 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceDefinitionParseInfo_RobDefFilePath_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinitionParseInfo_RobDefFilePath_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceDefinitionParseInfo_RobDefFilePath_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceDefinitionParseInfo_RobDefFilePath_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->RobDefFilePath = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinitionParseInfo_RobDefFilePath_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinitionParseInfo *arg1 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinitionParseInfo_RobDefFilePath_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->RobDefFilePath);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinitionParseInfo_Line_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinitionParseInfo *arg1 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceDefinitionParseInfo_Line_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinitionParseInfo_Line_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceDefinitionParseInfo_Line_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceDefinitionParseInfo_Line_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Line = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinitionParseInfo_Line_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinitionParseInfo *arg1 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinitionParseInfo_Line_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->Line);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinitionParseInfo_LineNumber_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinitionParseInfo *arg1 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceDefinitionParseInfo_LineNumber_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinitionParseInfo_LineNumber_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ServiceDefinitionParseInfo_LineNumber_set" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->LineNumber = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinitionParseInfo_LineNumber_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinitionParseInfo *arg1 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinitionParseInfo_LineNumber_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (int32_t) ((arg1)->LineNumber);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ServiceDefinitionParseInfo(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinitionParseInfo *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_ServiceDefinitionParseInfo", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::ServiceDefinitionParseInfo *)new RobotRaconteur::ServiceDefinitionParseInfo();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinitionParseInfo_Reset(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinitionParseInfo *arg1 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinitionParseInfo_Reset" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Reset();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_ServiceDefinitionParseInfo(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinitionParseInfo *arg1 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_ServiceDefinitionParseInfo" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *ServiceDefinitionParseInfo_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *ServiceDefinitionParseInfo_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_new_RobotRaconteurVersion__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurVersion *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::RobotRaconteurVersion *)new RobotRaconteur::RobotRaconteurVersion();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_RobotRaconteurVersion__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  uint32_t arg1 ;
  uint32_t arg2 ;
  uint32_t arg3 ;
  uint32_t arg4 ;
  unsigned int val1 ;
  int ecode1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  unsigned int val3 ;
  int ecode3 = 0 ;
  unsigned int val4 ;
  int ecode4 = 0 ;
  RobotRaconteur::RobotRaconteurVersion *result = 0 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  ecode1 = SWIG_AsVal_unsigned_SS_int(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_RobotRaconteurVersion" "', argument " "1"" of type '" "uint32_t""'");
  } 
  arg1 = static_cast< uint32_t >(val1);
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "new_RobotRaconteurVersion" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  ecode3 = SWIG_AsVal_unsigned_SS_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "new_RobotRaconteurVersion" "', argument " "3"" of type '" "uint32_t""'");
  } 
  arg3 = static_cast< uint32_t >(val3);
  ecode4 = SWIG_AsVal_unsigned_SS_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "new_RobotRaconteurVersion" "', argument " "4"" of type '" "uint32_t""'");
  } 
  arg4 = static_cast< uint32_t >(val4);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::RobotRaconteurVersion *)new RobotRaconteur::RobotRaconteurVersion(arg1,arg2,arg3,arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_RobotRaconteurVersion__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  uint32_t arg1 ;
  uint32_t arg2 ;
  uint32_t arg3 ;
  unsigned int val1 ;
  int ecode1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  unsigned int val3 ;
  int ecode3 = 0 ;
  RobotRaconteur::RobotRaconteurVersion *result = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  ecode1 = SWIG_AsVal_unsigned_SS_int(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_RobotRaconteurVersion" "', argument " "1"" of type '" "uint32_t""'");
  } 
  arg1 = static_cast< uint32_t >(val1);
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "new_RobotRaconteurVersion" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  ecode3 = SWIG_AsVal_unsigned_SS_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "new_RobotRaconteurVersion" "', argument " "3"" of type '" "uint32_t""'");
  } 
  arg3 = static_cast< uint32_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::RobotRaconteurVersion *)new RobotRaconteur::RobotRaconteurVersion(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_RobotRaconteurVersion__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  uint32_t arg1 ;
  uint32_t arg2 ;
  unsigned int val1 ;
  int ecode1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  RobotRaconteur::RobotRaconteurVersion *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_unsigned_SS_int(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_RobotRaconteurVersion" "', argument " "1"" of type '" "uint32_t""'");
  } 
  arg1 = static_cast< uint32_t >(val1);
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "new_RobotRaconteurVersion" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::RobotRaconteurVersion *)new RobotRaconteur::RobotRaconteurVersion(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_RobotRaconteurVersion__SWIG_4(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  RobotRaconteur::RobotRaconteurVersion *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_RobotRaconteurVersion" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_RobotRaconteurVersion" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::RobotRaconteurVersion *)new RobotRaconteur::RobotRaconteurVersion((std::string const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_RobotRaconteurVersion(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_RobotRaconteurVersion", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_RobotRaconteurVersion__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    int res = SWIG_AsPtr_std_string(argv[0], (std::string**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_RobotRaconteurVersion__SWIG_4(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_unsigned_SS_int(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      {
        int res = SWIG_AsVal_unsigned_SS_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_new_RobotRaconteurVersion__SWIG_3(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    {
      int res = SWIG_AsVal_unsigned_SS_int(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      {
        int res = SWIG_AsVal_unsigned_SS_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_unsigned_SS_int(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_new_RobotRaconteurVersion__SWIG_2(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    {
      int res = SWIG_AsVal_unsigned_SS_int(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      {
        int res = SWIG_AsVal_unsigned_SS_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_unsigned_SS_int(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          {
            int res = SWIG_AsVal_unsigned_SS_int(argv[3], NULL);
            _v = SWIG_CheckState(res);
          }
          if (_v) {
            return _wrap_new_RobotRaconteurVersion__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_RobotRaconteurVersion'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::RobotRaconteurVersion::RobotRaconteurVersion()\n"
    "    RobotRaconteur::RobotRaconteurVersion::RobotRaconteurVersion(uint32_t,uint32_t,uint32_t,uint32_t)\n"
    "    RobotRaconteur::RobotRaconteurVersion::RobotRaconteurVersion(uint32_t,uint32_t,uint32_t)\n"
    "    RobotRaconteur::RobotRaconteurVersion::RobotRaconteurVersion(uint32_t,uint32_t)\n"
    "    RobotRaconteur::RobotRaconteurVersion::RobotRaconteurVersion(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurVersion_ToString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurVersion *arg1 = (RobotRaconteur::RobotRaconteurVersion *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurVersion_ToString" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion const *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((RobotRaconteur::RobotRaconteurVersion const *)arg1)->ToString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurVersion_FromString__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurVersion *arg1 = (RobotRaconteur::RobotRaconteurVersion *) 0 ;
  std::string *arg2 = 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg3 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurVersion_FromString" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurVersion_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurVersion_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "RobotRaconteurVersion_FromString" "', argument " "3"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo const *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2,(RobotRaconteur::ServiceDefinitionParseInfo const *)arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurVersion_FromString__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurVersion *arg1 = (RobotRaconteur::RobotRaconteurVersion *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurVersion_FromString" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurVersion_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurVersion_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurVersion_FromString(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "RobotRaconteurVersion_FromString", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    void *vptr = 0;
    int res = SWIG_ConvertPtr(argv[0], &vptr, SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_RobotRaconteurVersion_FromString__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    void *vptr = 0;
    int res = SWIG_ConvertPtr(argv[0], &vptr, SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        void *vptr = 0;
        int res = SWIG_ConvertPtr(argv[2], &vptr, SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_RobotRaconteurVersion_FromString__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'RobotRaconteurVersion_FromString'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::RobotRaconteurVersion::FromString(std::string const &,RobotRaconteur::ServiceDefinitionParseInfo const *)\n"
    "    RobotRaconteur::RobotRaconteurVersion::FromString(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurVersion___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurVersion *arg1 = (RobotRaconteur::RobotRaconteurVersion *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurVersion___nonzero__" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion const *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((RobotRaconteur::RobotRaconteurVersion const *)arg1)->operator bool();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurVersion_major_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurVersion *arg1 = (RobotRaconteur::RobotRaconteurVersion *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurVersion_major_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurVersion_major_set" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "RobotRaconteurVersion_major_set" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->major = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurVersion_major_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurVersion *arg1 = (RobotRaconteur::RobotRaconteurVersion *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurVersion_major_get" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint32_t) ((arg1)->major);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurVersion_minor_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurVersion *arg1 = (RobotRaconteur::RobotRaconteurVersion *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurVersion_minor_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurVersion_minor_set" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "RobotRaconteurVersion_minor_set" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->minor = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurVersion_minor_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurVersion *arg1 = (RobotRaconteur::RobotRaconteurVersion *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurVersion_minor_get" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint32_t) ((arg1)->minor);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurVersion_patch_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurVersion *arg1 = (RobotRaconteur::RobotRaconteurVersion *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurVersion_patch_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurVersion_patch_set" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "RobotRaconteurVersion_patch_set" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->patch = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurVersion_patch_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurVersion *arg1 = (RobotRaconteur::RobotRaconteurVersion *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurVersion_patch_get" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint32_t) ((arg1)->patch);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurVersion_tweak_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurVersion *arg1 = (RobotRaconteur::RobotRaconteurVersion *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurVersion_tweak_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurVersion_tweak_set" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "RobotRaconteurVersion_tweak_set" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->tweak = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurVersion_tweak_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurVersion *arg1 = (RobotRaconteur::RobotRaconteurVersion *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurVersion_tweak_get" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint32_t) ((arg1)->tweak);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurVersion_ParseInfo_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurVersion *arg1 = (RobotRaconteur::RobotRaconteurVersion *) 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg2 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurVersion_ParseInfo_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurVersion_ParseInfo_set" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurVersion_ParseInfo_set" "', argument " "2"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ParseInfo = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurVersion_ParseInfo_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurVersion *arg1 = (RobotRaconteur::RobotRaconteurVersion *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::ServiceDefinitionParseInfo *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurVersion_ParseInfo_get" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::ServiceDefinitionParseInfo *)& ((arg1)->ParseInfo);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_RobotRaconteurVersion(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurVersion *arg1 = (RobotRaconteur::RobotRaconteurVersion *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_RobotRaconteurVersion" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *RobotRaconteurVersion_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *RobotRaconteurVersion_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_RobotRaconteurVersion_eq(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurVersion *arg1 = 0 ;
  RobotRaconteur::RobotRaconteurVersion *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurVersion_eq", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion,  0  | 0);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurVersion_eq" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  if (!argp1) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurVersion_eq" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurVersion_eq" "', argument " "2"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurVersion_eq" "', argument " "2"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)operator ==((RobotRaconteur::RobotRaconteurVersion const &)*arg1,(RobotRaconteur::RobotRaconteurVersion const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurVersion_ne(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurVersion *arg1 = 0 ;
  RobotRaconteur::RobotRaconteurVersion *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurVersion_ne", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion,  0  | 0);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurVersion_ne" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  if (!argp1) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurVersion_ne" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurVersion_ne" "', argument " "2"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurVersion_ne" "', argument " "2"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)operator !=((RobotRaconteur::RobotRaconteurVersion const &)*arg1,(RobotRaconteur::RobotRaconteurVersion const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurVersion_gt(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurVersion *arg1 = 0 ;
  RobotRaconteur::RobotRaconteurVersion *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurVersion_gt", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion,  0  | 0);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurVersion_gt" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  if (!argp1) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurVersion_gt" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurVersion_gt" "', argument " "2"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurVersion_gt" "', argument " "2"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)operator >((RobotRaconteur::RobotRaconteurVersion const &)*arg1,(RobotRaconteur::RobotRaconteurVersion const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurVersion_ge(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurVersion *arg1 = 0 ;
  RobotRaconteur::RobotRaconteurVersion *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurVersion_ge", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion,  0  | 0);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurVersion_ge" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  if (!argp1) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurVersion_ge" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurVersion_ge" "', argument " "2"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurVersion_ge" "', argument " "2"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)operator >=((RobotRaconteur::RobotRaconteurVersion const &)*arg1,(RobotRaconteur::RobotRaconteurVersion const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurVersion_lt(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurVersion *arg1 = 0 ;
  RobotRaconteur::RobotRaconteurVersion *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurVersion_lt", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion,  0  | 0);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurVersion_lt" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  if (!argp1) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurVersion_lt" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurVersion_lt" "', argument " "2"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurVersion_lt" "', argument " "2"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)operator <((RobotRaconteur::RobotRaconteurVersion const &)*arg1,(RobotRaconteur::RobotRaconteurVersion const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurVersion_le(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurVersion *arg1 = 0 ;
  RobotRaconteur::RobotRaconteurVersion *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurVersion_le", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion,  0  | 0);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurVersion_le" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  if (!argp1) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurVersion_le" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurVersion_le" "', argument " "2"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurVersion_le" "', argument " "2"" of type '" "RobotRaconteur::RobotRaconteurVersion const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)operator <=((RobotRaconteur::RobotRaconteurVersion const &)*arg1,(RobotRaconteur::RobotRaconteurVersion const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_Name_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceDefinition_Name_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_Name_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceDefinition_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceDefinition_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Name = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_Name_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_Name_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->Name);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_NamedArrays_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *arg2 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceDefinition_NamedArrays_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_NamedArrays_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceDefinition_NamedArrays_set" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->NamedArrays = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_NamedArrays_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_NamedArrays_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *)& ((arg1)->NamedArrays);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_Pods_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *arg2 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceDefinition_Pods_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_Pods_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceDefinition_Pods_set" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Pods = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_Pods_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_Pods_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *)& ((arg1)->Pods);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_Structures_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *arg2 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceDefinition_Structures_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_Structures_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceDefinition_Structures_set" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Structures = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_Structures_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_Structures_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *)& ((arg1)->Structures);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_Objects_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *arg2 = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceDefinition_Objects_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_Objects_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceDefinition_Objects_set" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Objects = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_Objects_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_Objects_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *)& ((arg1)->Objects);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_Options_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = (std::vector< std::string,std::allocator< std::string > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceDefinition_Options_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_Options_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceDefinition_Options_set" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< std::string,std::allocator< std::string > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Options = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_Options_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_Options_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< std::string,std::allocator< std::string > > *)& ((arg1)->Options);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_Imports_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = (std::vector< std::string,std::allocator< std::string > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceDefinition_Imports_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_Imports_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceDefinition_Imports_set" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< std::string,std::allocator< std::string > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Imports = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_Imports_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_Imports_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< std::string,std::allocator< std::string > > *)& ((arg1)->Imports);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_Exceptions_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > *arg2 = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceDefinition_Exceptions_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_Exceptions_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceDefinition_Exceptions_set" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Exceptions = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_Exceptions_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_Exceptions_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > *)& ((arg1)->Exceptions);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_Using_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > *arg2 = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceDefinition_Using_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_Using_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceDefinition_Using_set" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Using = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_Using_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_Using_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > *)& ((arg1)->Using);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_Constants_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *arg2 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceDefinition_Constants_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_Constants_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceDefinition_Constants_set" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Constants = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_Constants_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_Constants_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *)& ((arg1)->Constants);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_Enums_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > *arg2 = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceDefinition_Enums_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_Enums_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceDefinition_Enums_set" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Enums = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_Enums_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_Enums_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > *)& ((arg1)->Enums);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_StdVer_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  RobotRaconteur::RobotRaconteurVersion *arg2 = (RobotRaconteur::RobotRaconteurVersion *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceDefinition_StdVer_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_StdVer_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceDefinition_StdVer_set" "', argument " "2"" of type '" "RobotRaconteur::RobotRaconteurVersion *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::RobotRaconteurVersion * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->StdVer = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_StdVer_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::RobotRaconteurVersion *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_StdVer_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::RobotRaconteurVersion *)& ((arg1)->StdVer);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__RobotRaconteurVersion, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_ParseInfo_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg2 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceDefinition_ParseInfo_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_ParseInfo_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceDefinition_ParseInfo_set" "', argument " "2"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ParseInfo = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_ParseInfo_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::ServiceDefinitionParseInfo *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_ParseInfo_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::ServiceDefinitionParseInfo *)& ((arg1)->ParseInfo);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_DocString_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceDefinition_DocString_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_DocString_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceDefinition_DocString_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceDefinition_DocString_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->DocString = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_DocString_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_DocString_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->DocString);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_ToString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_ToString" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ToString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_FromString__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  std::string *arg2 = 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg3 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "ServiceDefinition_FromString" "', argument " "3"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo const *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2,(RobotRaconteur::ServiceDefinitionParseInfo const *)arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_FromString__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_FromString(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "ServiceDefinition_FromString", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_ServiceDefinition_FromString__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        void *vptr = 0;
        int res = SWIG_ConvertPtr(argv[2], &vptr, SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_ServiceDefinition_FromString__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'ServiceDefinition_FromString'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::ServiceDefinition::FromString(std::string const &,RobotRaconteur::ServiceDefinitionParseInfo const *)\n"
    "    RobotRaconteur::ServiceDefinition::FromString(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_ServiceDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_ServiceDefinition", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::ServiceDefinition *)new RobotRaconteur::ServiceDefinition();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServiceDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ServiceDefinition >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceDefinition_Reset(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceDefinition_Reset" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Reset();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_ServiceDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceDefinition *arg1 = (RobotRaconteur::ServiceDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_ServiceDefinition" "', argument " "1"" of type '" "RobotRaconteur::ServiceDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *ServiceDefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *ServiceDefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_ServiceEntryDefinition_Name_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceEntryDefinition *arg1 = (RobotRaconteur::ServiceEntryDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceEntryDefinition_Name_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceEntryDefinition_Name_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceEntryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceEntryDefinition_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceEntryDefinition_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Name = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceEntryDefinition_Name_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceEntryDefinition *arg1 = (RobotRaconteur::ServiceEntryDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceEntryDefinition_Name_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceEntryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->Name);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceEntryDefinition_Members_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceEntryDefinition *arg1 = (RobotRaconteur::ServiceEntryDefinition *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > *arg2 = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceEntryDefinition_Members_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceEntryDefinition_Members_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceEntryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceEntryDefinition_Members_set" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Members = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceEntryDefinition_Members_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceEntryDefinition *arg1 = (RobotRaconteur::ServiceEntryDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceEntryDefinition_Members_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceEntryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > *)& ((arg1)->Members);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceEntryDefinition_EntryType_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceEntryDefinition *arg1 = (RobotRaconteur::ServiceEntryDefinition *) 0 ;
  RobotRaconteur::DataTypes arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceEntryDefinition_EntryType_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceEntryDefinition_EntryType_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceEntryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ServiceEntryDefinition_EntryType_set" "', argument " "2"" of type '" "RobotRaconteur::DataTypes""'");
  } 
  arg2 = static_cast< RobotRaconteur::DataTypes >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->EntryType = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceEntryDefinition_EntryType_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceEntryDefinition *arg1 = (RobotRaconteur::ServiceEntryDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::DataTypes result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceEntryDefinition_EntryType_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceEntryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::DataTypes) ((arg1)->EntryType);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceEntryDefinition_Implements_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceEntryDefinition *arg1 = (RobotRaconteur::ServiceEntryDefinition *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = (std::vector< std::string,std::allocator< std::string > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceEntryDefinition_Implements_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceEntryDefinition_Implements_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceEntryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceEntryDefinition_Implements_set" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< std::string,std::allocator< std::string > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Implements = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceEntryDefinition_Implements_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceEntryDefinition *arg1 = (RobotRaconteur::ServiceEntryDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceEntryDefinition_Implements_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceEntryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< std::string,std::allocator< std::string > > *)& ((arg1)->Implements);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceEntryDefinition_Options_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceEntryDefinition *arg1 = (RobotRaconteur::ServiceEntryDefinition *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = (std::vector< std::string,std::allocator< std::string > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceEntryDefinition_Options_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceEntryDefinition_Options_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceEntryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceEntryDefinition_Options_set" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< std::string,std::allocator< std::string > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Options = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceEntryDefinition_Options_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceEntryDefinition *arg1 = (RobotRaconteur::ServiceEntryDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceEntryDefinition_Options_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceEntryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< std::string,std::allocator< std::string > > *)& ((arg1)->Options);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceEntryDefinition_Constants_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceEntryDefinition *arg1 = (RobotRaconteur::ServiceEntryDefinition *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *arg2 = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceEntryDefinition_Constants_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceEntryDefinition_Constants_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceEntryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceEntryDefinition_Constants_set" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Constants = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceEntryDefinition_Constants_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceEntryDefinition *arg1 = (RobotRaconteur::ServiceEntryDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceEntryDefinition_Constants_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceEntryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *)& ((arg1)->Constants);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceEntryDefinition_ParseInfo_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceEntryDefinition *arg1 = (RobotRaconteur::ServiceEntryDefinition *) 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg2 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceEntryDefinition_ParseInfo_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceEntryDefinition_ParseInfo_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceEntryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceEntryDefinition_ParseInfo_set" "', argument " "2"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ParseInfo = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceEntryDefinition_ParseInfo_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceEntryDefinition *arg1 = (RobotRaconteur::ServiceEntryDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::ServiceDefinitionParseInfo *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceEntryDefinition_ParseInfo_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceEntryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::ServiceDefinitionParseInfo *)& ((arg1)->ParseInfo);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceEntryDefinition_DocString_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceEntryDefinition *arg1 = (RobotRaconteur::ServiceEntryDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceEntryDefinition_DocString_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceEntryDefinition_DocString_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceEntryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceEntryDefinition_DocString_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceEntryDefinition_DocString_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->DocString = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceEntryDefinition_DocString_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceEntryDefinition *arg1 = (RobotRaconteur::ServiceEntryDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceEntryDefinition_DocString_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceEntryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->DocString);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ServiceEntryDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::ServiceEntryDefinition *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_ServiceEntryDefinition" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::ServiceEntryDefinition *)new RobotRaconteur::ServiceEntryDefinition((boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceEntryDefinition_ToString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceEntryDefinition *arg1 = (RobotRaconteur::ServiceEntryDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceEntryDefinition_ToString" "', argument " "1"" of type '" "RobotRaconteur::ServiceEntryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ToString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceEntryDefinition_FromString__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceEntryDefinition *arg1 = (RobotRaconteur::ServiceEntryDefinition *) 0 ;
  std::string *arg2 = 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg3 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceEntryDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::ServiceEntryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceEntryDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceEntryDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "ServiceEntryDefinition_FromString" "', argument " "3"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo const *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2,(RobotRaconteur::ServiceDefinitionParseInfo const *)arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceEntryDefinition_FromString__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceEntryDefinition *arg1 = (RobotRaconteur::ServiceEntryDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceEntryDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::ServiceEntryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceEntryDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceEntryDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceEntryDefinition_FromString(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "ServiceEntryDefinition_FromString", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_ServiceEntryDefinition_FromString__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        void *vptr = 0;
        int res = SWIG_ConvertPtr(argv[2], &vptr, SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_ServiceEntryDefinition_FromString__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'ServiceEntryDefinition_FromString'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::ServiceEntryDefinition::FromString(std::string const &,RobotRaconteur::ServiceDefinitionParseInfo const *)\n"
    "    RobotRaconteur::ServiceEntryDefinition::FromString(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_ServiceEntryDefinition_GetServiceDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceEntryDefinition *arg1 = (RobotRaconteur::ServiceEntryDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceEntryDefinition_GetServiceDefinition" "', argument " "1"" of type '" "RobotRaconteur::ServiceEntryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_ServiceEntryDefinition_GetServiceDefinition(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServiceDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ServiceDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceEntryDefinition_SetServiceDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceEntryDefinition *arg1 = (RobotRaconteur::ServiceEntryDefinition *) 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceEntryDefinition_SetServiceDefinition", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceEntryDefinition_SetServiceDefinition" "', argument " "1"" of type '" "RobotRaconteur::ServiceEntryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceEntryDefinition_SetServiceDefinition" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur_ServiceEntryDefinition_SetServiceDefinition(arg1,(boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceEntryDefinition_Reset(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceEntryDefinition *arg1 = (RobotRaconteur::ServiceEntryDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceEntryDefinition_Reset" "', argument " "1"" of type '" "RobotRaconteur::ServiceEntryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Reset();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_ServiceEntryDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceEntryDefinition *arg1 = (RobotRaconteur::ServiceEntryDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_ServiceEntryDefinition" "', argument " "1"" of type '" "RobotRaconteur::ServiceEntryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceEntryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *ServiceEntryDefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *ServiceEntryDefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_MemberDefinition_Name_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MemberDefinition *arg1 = (RobotRaconteur::MemberDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MemberDefinition_Name_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemberDefinition_Name_set" "', argument " "1"" of type '" "RobotRaconteur::MemberDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MemberDefinition_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MemberDefinition_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Name = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemberDefinition_Name_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MemberDefinition *arg1 = (RobotRaconteur::MemberDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemberDefinition_Name_get" "', argument " "1"" of type '" "RobotRaconteur::MemberDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->Name);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemberDefinition_DocString_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MemberDefinition *arg1 = (RobotRaconteur::MemberDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MemberDefinition_DocString_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemberDefinition_DocString_set" "', argument " "1"" of type '" "RobotRaconteur::MemberDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MemberDefinition_DocString_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MemberDefinition_DocString_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->DocString = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemberDefinition_DocString_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MemberDefinition *arg1 = (RobotRaconteur::MemberDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemberDefinition_DocString_get" "', argument " "1"" of type '" "RobotRaconteur::MemberDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->DocString);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_MemberDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MemberDefinition *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_MemberDefinition" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::MemberDefinition *)new RobotRaconteur::MemberDefinition((boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::MemberDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::MemberDefinition >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemberDefinition_NoLock(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MemberDefinition *arg1 = (RobotRaconteur::MemberDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MemberDefinition_NoLock result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemberDefinition_NoLock" "', argument " "1"" of type '" "RobotRaconteur::MemberDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::MemberDefinition_NoLock)(arg1)->NoLock();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemberDefinition_ToString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MemberDefinition *arg1 = (RobotRaconteur::MemberDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemberDefinition_ToString" "', argument " "1"" of type '" "RobotRaconteur::MemberDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ToString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemberDefinition_GetServiceEntry(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MemberDefinition *arg1 = (RobotRaconteur::MemberDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemberDefinition_GetServiceEntry" "', argument " "1"" of type '" "RobotRaconteur::MemberDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_MemberDefinition_GetServiceEntry(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemberDefinition_SetServiceEntry(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MemberDefinition *arg1 = (RobotRaconteur::MemberDefinition *) 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MemberDefinition_SetServiceEntry", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemberDefinition_SetServiceEntry" "', argument " "1"" of type '" "RobotRaconteur::MemberDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MemberDefinition_SetServiceEntry" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur_MemberDefinition_SetServiceEntry(arg1,(boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemberDefinition_Modifiers_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MemberDefinition *arg1 = (RobotRaconteur::MemberDefinition *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = (std::vector< std::string,std::allocator< std::string > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MemberDefinition_Modifiers_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemberDefinition_Modifiers_set" "', argument " "1"" of type '" "RobotRaconteur::MemberDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MemberDefinition_Modifiers_set" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< std::string,std::allocator< std::string > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Modifiers = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemberDefinition_Modifiers_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MemberDefinition *arg1 = (RobotRaconteur::MemberDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemberDefinition_Modifiers_get" "', argument " "1"" of type '" "RobotRaconteur::MemberDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< std::string,std::allocator< std::string > > *)& ((arg1)->Modifiers);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemberDefinition_ParseInfo_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MemberDefinition *arg1 = (RobotRaconteur::MemberDefinition *) 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg2 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MemberDefinition_ParseInfo_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemberDefinition_ParseInfo_set" "', argument " "1"" of type '" "RobotRaconteur::MemberDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MemberDefinition_ParseInfo_set" "', argument " "2"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ParseInfo = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemberDefinition_ParseInfo_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MemberDefinition *arg1 = (RobotRaconteur::MemberDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::ServiceDefinitionParseInfo *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemberDefinition_ParseInfo_get" "', argument " "1"" of type '" "RobotRaconteur::MemberDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::ServiceDefinitionParseInfo *)& ((arg1)->ParseInfo);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemberDefinition_Reset(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MemberDefinition *arg1 = (RobotRaconteur::MemberDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemberDefinition_Reset" "', argument " "1"" of type '" "RobotRaconteur::MemberDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Reset();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_MemberDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MemberDefinition *arg1 = (RobotRaconteur::MemberDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_MemberDefinition" "', argument " "1"" of type '" "RobotRaconteur::MemberDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemberDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *MemberDefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *MemberDefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_PropertyDefinition_Type_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::PropertyDefinition *arg1 = (RobotRaconteur::PropertyDefinition *) 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::PropertyDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::PropertyDefinition > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "PropertyDefinition_Type_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "PropertyDefinition_Type_set" "', argument " "1"" of type '" "RobotRaconteur::PropertyDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PropertyDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PropertyDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PropertyDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PropertyDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PropertyDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "PropertyDefinition_Type_set" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::TypeDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2) : &tempshared2;
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Type = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_PropertyDefinition_Type_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::PropertyDefinition *arg1 = (RobotRaconteur::PropertyDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::PropertyDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::PropertyDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "PropertyDefinition_Type_get" "', argument " "1"" of type '" "RobotRaconteur::PropertyDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PropertyDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PropertyDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PropertyDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PropertyDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PropertyDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::shared_ptr< RobotRaconteur::TypeDefinition > *) & ((arg1)->Type);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  {
    boost::shared_ptr<  RobotRaconteur::TypeDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::TypeDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_PropertyDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::PropertyDefinition *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_PropertyDefinition" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::PropertyDefinition *)new RobotRaconteur::PropertyDefinition((boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::PropertyDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::PropertyDefinition >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_PropertyDefinition_ToString__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::PropertyDefinition *arg1 = (RobotRaconteur::PropertyDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::PropertyDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::PropertyDefinition > *smartarg1 = 0 ;
  std::string result;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "PropertyDefinition_ToString" "', argument " "1"" of type '" "RobotRaconteur::PropertyDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PropertyDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PropertyDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PropertyDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PropertyDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PropertyDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ToString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_PropertyDefinition_ToString__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::PropertyDefinition *arg1 = (RobotRaconteur::PropertyDefinition *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::PropertyDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::PropertyDefinition > *smartarg1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  std::string result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "PropertyDefinition_ToString" "', argument " "1"" of type '" "RobotRaconteur::PropertyDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PropertyDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PropertyDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PropertyDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PropertyDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PropertyDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "PropertyDefinition_ToString" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ToString(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_PropertyDefinition_ToString(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "PropertyDefinition_ToString", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_PropertyDefinition_ToString__SWIG_0(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_bool(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_PropertyDefinition_ToString__SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'PropertyDefinition_ToString'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::PropertyDefinition::ToString()\n"
    "    RobotRaconteur::PropertyDefinition::ToString(bool)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_PropertyDefinition_FromString__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::PropertyDefinition *arg1 = (RobotRaconteur::PropertyDefinition *) 0 ;
  std::string *arg2 = 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg3 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::PropertyDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::PropertyDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "PropertyDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::PropertyDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PropertyDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PropertyDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PropertyDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PropertyDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PropertyDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "PropertyDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "PropertyDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "PropertyDefinition_FromString" "', argument " "3"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo const *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2,(RobotRaconteur::ServiceDefinitionParseInfo const *)arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_PropertyDefinition_FromString__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::PropertyDefinition *arg1 = (RobotRaconteur::PropertyDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::PropertyDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::PropertyDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "PropertyDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::PropertyDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PropertyDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PropertyDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PropertyDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PropertyDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PropertyDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "PropertyDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "PropertyDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_PropertyDefinition_FromString(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "PropertyDefinition_FromString", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_PropertyDefinition_FromString__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        void *vptr = 0;
        int res = SWIG_ConvertPtr(argv[2], &vptr, SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_PropertyDefinition_FromString__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'PropertyDefinition_FromString'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::PropertyDefinition::FromString(std::string const &,RobotRaconteur::ServiceDefinitionParseInfo const *)\n"
    "    RobotRaconteur::PropertyDefinition::FromString(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_delete_PropertyDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::PropertyDefinition *arg1 = (RobotRaconteur::PropertyDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::PropertyDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::PropertyDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_PropertyDefinition" "', argument " "1"" of type '" "RobotRaconteur::PropertyDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PropertyDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PropertyDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PropertyDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PropertyDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PropertyDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *PropertyDefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *PropertyDefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_FunctionDefinition_ReturnType_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::FunctionDefinition *arg1 = (RobotRaconteur::FunctionDefinition *) 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::FunctionDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::FunctionDefinition > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "FunctionDefinition_ReturnType_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "FunctionDefinition_ReturnType_set" "', argument " "1"" of type '" "RobotRaconteur::FunctionDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FunctionDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FunctionDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "FunctionDefinition_ReturnType_set" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::TypeDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2) : &tempshared2;
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ReturnType = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_FunctionDefinition_ReturnType_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::FunctionDefinition *arg1 = (RobotRaconteur::FunctionDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::FunctionDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::FunctionDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "FunctionDefinition_ReturnType_get" "', argument " "1"" of type '" "RobotRaconteur::FunctionDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FunctionDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FunctionDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::shared_ptr< RobotRaconteur::TypeDefinition > *) & ((arg1)->ReturnType);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  {
    boost::shared_ptr<  RobotRaconteur::TypeDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::TypeDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_FunctionDefinition_Parameters_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::FunctionDefinition *arg1 = (RobotRaconteur::FunctionDefinition *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *arg2 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::FunctionDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::FunctionDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "FunctionDefinition_Parameters_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "FunctionDefinition_Parameters_set" "', argument " "1"" of type '" "RobotRaconteur::FunctionDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FunctionDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FunctionDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "FunctionDefinition_Parameters_set" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Parameters = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_FunctionDefinition_Parameters_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::FunctionDefinition *arg1 = (RobotRaconteur::FunctionDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::FunctionDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::FunctionDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "FunctionDefinition_Parameters_get" "', argument " "1"" of type '" "RobotRaconteur::FunctionDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FunctionDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FunctionDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *)& ((arg1)->Parameters);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_FunctionDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::FunctionDefinition *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_FunctionDefinition" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::FunctionDefinition *)new RobotRaconteur::FunctionDefinition((boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::FunctionDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::FunctionDefinition >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_FunctionDefinition_ToString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::FunctionDefinition *arg1 = (RobotRaconteur::FunctionDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::FunctionDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::FunctionDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "FunctionDefinition_ToString" "', argument " "1"" of type '" "RobotRaconteur::FunctionDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FunctionDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FunctionDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ToString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_FunctionDefinition_FromString__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::FunctionDefinition *arg1 = (RobotRaconteur::FunctionDefinition *) 0 ;
  std::string *arg2 = 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg3 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::FunctionDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::FunctionDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "FunctionDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::FunctionDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FunctionDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FunctionDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "FunctionDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "FunctionDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "FunctionDefinition_FromString" "', argument " "3"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo const *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2,(RobotRaconteur::ServiceDefinitionParseInfo const *)arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_FunctionDefinition_FromString__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::FunctionDefinition *arg1 = (RobotRaconteur::FunctionDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::FunctionDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::FunctionDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "FunctionDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::FunctionDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FunctionDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FunctionDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "FunctionDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "FunctionDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_FunctionDefinition_FromString(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "FunctionDefinition_FromString", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_FunctionDefinition_FromString__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        void *vptr = 0;
        int res = SWIG_ConvertPtr(argv[2], &vptr, SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_FunctionDefinition_FromString__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'FunctionDefinition_FromString'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::FunctionDefinition::FromString(std::string const &,RobotRaconteur::ServiceDefinitionParseInfo const *)\n"
    "    RobotRaconteur::FunctionDefinition::FromString(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_FunctionDefinition_IsGenerator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::FunctionDefinition *arg1 = (RobotRaconteur::FunctionDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::FunctionDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::FunctionDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "FunctionDefinition_IsGenerator" "', argument " "1"" of type '" "RobotRaconteur::FunctionDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FunctionDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FunctionDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->IsGenerator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_FunctionDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::FunctionDefinition *arg1 = (RobotRaconteur::FunctionDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::FunctionDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::FunctionDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_FunctionDefinition" "', argument " "1"" of type '" "RobotRaconteur::FunctionDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FunctionDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FunctionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FunctionDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *FunctionDefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *FunctionDefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_EventDefinition_Parameters_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EventDefinition *arg1 = (RobotRaconteur::EventDefinition *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *arg2 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::EventDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::EventDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "EventDefinition_Parameters_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EventDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EventDefinition_Parameters_set" "', argument " "1"" of type '" "RobotRaconteur::EventDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EventDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EventDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EventDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EventDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EventDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "EventDefinition_Parameters_set" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Parameters = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_EventDefinition_Parameters_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EventDefinition *arg1 = (RobotRaconteur::EventDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::EventDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::EventDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EventDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EventDefinition_Parameters_get" "', argument " "1"" of type '" "RobotRaconteur::EventDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EventDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EventDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EventDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EventDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EventDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *)& ((arg1)->Parameters);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_EventDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::EventDefinition *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_EventDefinition" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::EventDefinition *)new RobotRaconteur::EventDefinition((boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::EventDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::EventDefinition >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EventDefinition_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_EventDefinition_ToString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EventDefinition *arg1 = (RobotRaconteur::EventDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::EventDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::EventDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EventDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EventDefinition_ToString" "', argument " "1"" of type '" "RobotRaconteur::EventDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EventDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EventDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EventDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EventDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EventDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ToString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_EventDefinition_FromString__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::EventDefinition *arg1 = (RobotRaconteur::EventDefinition *) 0 ;
  std::string *arg2 = 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg3 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::EventDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::EventDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EventDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EventDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::EventDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EventDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EventDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EventDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EventDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EventDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "EventDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "EventDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "EventDefinition_FromString" "', argument " "3"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo const *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2,(RobotRaconteur::ServiceDefinitionParseInfo const *)arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_EventDefinition_FromString__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::EventDefinition *arg1 = (RobotRaconteur::EventDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::EventDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::EventDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EventDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EventDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::EventDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EventDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EventDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EventDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EventDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EventDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "EventDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "EventDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_EventDefinition_FromString(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "EventDefinition_FromString", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EventDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_EventDefinition_FromString__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EventDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        void *vptr = 0;
        int res = SWIG_ConvertPtr(argv[2], &vptr, SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_EventDefinition_FromString__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'EventDefinition_FromString'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::EventDefinition::FromString(std::string const &,RobotRaconteur::ServiceDefinitionParseInfo const *)\n"
    "    RobotRaconteur::EventDefinition::FromString(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_delete_EventDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EventDefinition *arg1 = (RobotRaconteur::EventDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::EventDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::EventDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EventDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_EventDefinition" "', argument " "1"" of type '" "RobotRaconteur::EventDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EventDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EventDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EventDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EventDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EventDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *EventDefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EventDefinition_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *EventDefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_ObjRefDefinition_ObjectType_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ObjRefDefinition *arg1 = (RobotRaconteur::ObjRefDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ObjRefDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ObjRefDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ObjRefDefinition_ObjectType_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ObjRefDefinition_ObjectType_set" "', argument " "1"" of type '" "RobotRaconteur::ObjRefDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ObjRefDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ObjRefDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ObjRefDefinition_ObjectType_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ObjRefDefinition_ObjectType_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ObjectType = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ObjRefDefinition_ObjectType_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ObjRefDefinition *arg1 = (RobotRaconteur::ObjRefDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ObjRefDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ObjRefDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ObjRefDefinition_ObjectType_get" "', argument " "1"" of type '" "RobotRaconteur::ObjRefDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ObjRefDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ObjRefDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->ObjectType);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ObjRefDefinition_ArrayType_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ObjRefDefinition *arg1 = (RobotRaconteur::ObjRefDefinition *) 0 ;
  RobotRaconteur::DataTypes_ArrayTypes arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ObjRefDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ObjRefDefinition > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ObjRefDefinition_ArrayType_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ObjRefDefinition_ArrayType_set" "', argument " "1"" of type '" "RobotRaconteur::ObjRefDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ObjRefDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ObjRefDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ObjRefDefinition_ArrayType_set" "', argument " "2"" of type '" "RobotRaconteur::DataTypes_ArrayTypes""'");
  } 
  arg2 = static_cast< RobotRaconteur::DataTypes_ArrayTypes >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ArrayType = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ObjRefDefinition_ArrayType_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ObjRefDefinition *arg1 = (RobotRaconteur::ObjRefDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ObjRefDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ObjRefDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::DataTypes_ArrayTypes result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ObjRefDefinition_ArrayType_get" "', argument " "1"" of type '" "RobotRaconteur::ObjRefDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ObjRefDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ObjRefDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::DataTypes_ArrayTypes) ((arg1)->ArrayType);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ObjRefDefinition_ContainerType_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ObjRefDefinition *arg1 = (RobotRaconteur::ObjRefDefinition *) 0 ;
  RobotRaconteur::DataTypes_ContainerTypes arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ObjRefDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ObjRefDefinition > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ObjRefDefinition_ContainerType_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ObjRefDefinition_ContainerType_set" "', argument " "1"" of type '" "RobotRaconteur::ObjRefDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ObjRefDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ObjRefDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ObjRefDefinition_ContainerType_set" "', argument " "2"" of type '" "RobotRaconteur::DataTypes_ContainerTypes""'");
  } 
  arg2 = static_cast< RobotRaconteur::DataTypes_ContainerTypes >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ContainerType = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ObjRefDefinition_ContainerType_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ObjRefDefinition *arg1 = (RobotRaconteur::ObjRefDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ObjRefDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ObjRefDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::DataTypes_ContainerTypes result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ObjRefDefinition_ContainerType_get" "', argument " "1"" of type '" "RobotRaconteur::ObjRefDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ObjRefDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ObjRefDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::DataTypes_ContainerTypes) ((arg1)->ContainerType);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ObjRefDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::ObjRefDefinition *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_ObjRefDefinition" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::ObjRefDefinition *)new RobotRaconteur::ObjRefDefinition((boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ObjRefDefinition >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ObjRefDefinition_ToString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ObjRefDefinition *arg1 = (RobotRaconteur::ObjRefDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ObjRefDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ObjRefDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ObjRefDefinition_ToString" "', argument " "1"" of type '" "RobotRaconteur::ObjRefDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ObjRefDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ObjRefDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ToString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ObjRefDefinition_FromString__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ObjRefDefinition *arg1 = (RobotRaconteur::ObjRefDefinition *) 0 ;
  std::string *arg2 = 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg3 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ObjRefDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ObjRefDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ObjRefDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::ObjRefDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ObjRefDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ObjRefDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ObjRefDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ObjRefDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "ObjRefDefinition_FromString" "', argument " "3"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo const *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2,(RobotRaconteur::ServiceDefinitionParseInfo const *)arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ObjRefDefinition_FromString__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ObjRefDefinition *arg1 = (RobotRaconteur::ObjRefDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ObjRefDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ObjRefDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ObjRefDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::ObjRefDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ObjRefDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ObjRefDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ObjRefDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ObjRefDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ObjRefDefinition_FromString(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "ObjRefDefinition_FromString", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_ObjRefDefinition_FromString__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        void *vptr = 0;
        int res = SWIG_ConvertPtr(argv[2], &vptr, SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_ObjRefDefinition_FromString__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'ObjRefDefinition_FromString'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::ObjRefDefinition::FromString(std::string const &,RobotRaconteur::ServiceDefinitionParseInfo const *)\n"
    "    RobotRaconteur::ObjRefDefinition::FromString(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_delete_ObjRefDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ObjRefDefinition *arg1 = (RobotRaconteur::ObjRefDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ObjRefDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ObjRefDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_ObjRefDefinition" "', argument " "1"" of type '" "RobotRaconteur::ObjRefDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ObjRefDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ObjRefDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *ObjRefDefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *ObjRefDefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_PipeDefinition_Type_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::PipeDefinition *arg1 = (RobotRaconteur::PipeDefinition *) 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::PipeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::PipeDefinition > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "PipeDefinition_Type_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PipeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "PipeDefinition_Type_set" "', argument " "1"" of type '" "RobotRaconteur::PipeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PipeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PipeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PipeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PipeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PipeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "PipeDefinition_Type_set" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::TypeDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2) : &tempshared2;
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Type = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_PipeDefinition_Type_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::PipeDefinition *arg1 = (RobotRaconteur::PipeDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::PipeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::PipeDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PipeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "PipeDefinition_Type_get" "', argument " "1"" of type '" "RobotRaconteur::PipeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PipeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PipeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PipeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PipeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PipeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::shared_ptr< RobotRaconteur::TypeDefinition > *) & ((arg1)->Type);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  {
    boost::shared_ptr<  RobotRaconteur::TypeDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::TypeDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_PipeDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::PipeDefinition *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_PipeDefinition" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::PipeDefinition *)new RobotRaconteur::PipeDefinition((boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::PipeDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::PipeDefinition >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PipeDefinition_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_PipeDefinition_ToString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::PipeDefinition *arg1 = (RobotRaconteur::PipeDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::PipeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::PipeDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PipeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "PipeDefinition_ToString" "', argument " "1"" of type '" "RobotRaconteur::PipeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PipeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PipeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PipeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PipeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PipeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ToString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_PipeDefinition_FromString__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::PipeDefinition *arg1 = (RobotRaconteur::PipeDefinition *) 0 ;
  std::string *arg2 = 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg3 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::PipeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::PipeDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PipeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "PipeDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::PipeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PipeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PipeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PipeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PipeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PipeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "PipeDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "PipeDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "PipeDefinition_FromString" "', argument " "3"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo const *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2,(RobotRaconteur::ServiceDefinitionParseInfo const *)arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_PipeDefinition_FromString__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::PipeDefinition *arg1 = (RobotRaconteur::PipeDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::PipeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::PipeDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PipeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "PipeDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::PipeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PipeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PipeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PipeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PipeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PipeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "PipeDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "PipeDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_PipeDefinition_FromString(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "PipeDefinition_FromString", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PipeDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_PipeDefinition_FromString__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PipeDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        void *vptr = 0;
        int res = SWIG_ConvertPtr(argv[2], &vptr, SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_PipeDefinition_FromString__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'PipeDefinition_FromString'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::PipeDefinition::FromString(std::string const &,RobotRaconteur::ServiceDefinitionParseInfo const *)\n"
    "    RobotRaconteur::PipeDefinition::FromString(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_delete_PipeDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::PipeDefinition *arg1 = (RobotRaconteur::PipeDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::PipeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::PipeDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PipeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_PipeDefinition" "', argument " "1"" of type '" "RobotRaconteur::PipeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PipeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PipeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PipeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PipeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PipeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *PipeDefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PipeDefinition_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *PipeDefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_CallbackDefinition_ReturnType_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::CallbackDefinition *arg1 = (RobotRaconteur::CallbackDefinition *) 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::CallbackDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::CallbackDefinition > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "CallbackDefinition_ReturnType_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CallbackDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "CallbackDefinition_ReturnType_set" "', argument " "1"" of type '" "RobotRaconteur::CallbackDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CallbackDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CallbackDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "CallbackDefinition_ReturnType_set" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::TypeDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2) : &tempshared2;
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ReturnType = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_CallbackDefinition_ReturnType_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::CallbackDefinition *arg1 = (RobotRaconteur::CallbackDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::CallbackDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::CallbackDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CallbackDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "CallbackDefinition_ReturnType_get" "', argument " "1"" of type '" "RobotRaconteur::CallbackDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CallbackDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CallbackDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::shared_ptr< RobotRaconteur::TypeDefinition > *) & ((arg1)->ReturnType);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  {
    boost::shared_ptr<  RobotRaconteur::TypeDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::TypeDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_CallbackDefinition_Parameters_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::CallbackDefinition *arg1 = (RobotRaconteur::CallbackDefinition *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *arg2 = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::CallbackDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::CallbackDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "CallbackDefinition_Parameters_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CallbackDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "CallbackDefinition_Parameters_set" "', argument " "1"" of type '" "RobotRaconteur::CallbackDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CallbackDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CallbackDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "CallbackDefinition_Parameters_set" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Parameters = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_CallbackDefinition_Parameters_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::CallbackDefinition *arg1 = (RobotRaconteur::CallbackDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::CallbackDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::CallbackDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CallbackDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "CallbackDefinition_Parameters_get" "', argument " "1"" of type '" "RobotRaconteur::CallbackDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CallbackDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CallbackDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *)& ((arg1)->Parameters);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_CallbackDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::CallbackDefinition *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_CallbackDefinition" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::CallbackDefinition *)new RobotRaconteur::CallbackDefinition((boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::CallbackDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::CallbackDefinition >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CallbackDefinition_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_CallbackDefinition_ToString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::CallbackDefinition *arg1 = (RobotRaconteur::CallbackDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::CallbackDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::CallbackDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CallbackDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "CallbackDefinition_ToString" "', argument " "1"" of type '" "RobotRaconteur::CallbackDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CallbackDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CallbackDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ToString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_CallbackDefinition_FromString__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::CallbackDefinition *arg1 = (RobotRaconteur::CallbackDefinition *) 0 ;
  std::string *arg2 = 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg3 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::CallbackDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::CallbackDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CallbackDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "CallbackDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::CallbackDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CallbackDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CallbackDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "CallbackDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CallbackDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "CallbackDefinition_FromString" "', argument " "3"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo const *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2,(RobotRaconteur::ServiceDefinitionParseInfo const *)arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_CallbackDefinition_FromString__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::CallbackDefinition *arg1 = (RobotRaconteur::CallbackDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::CallbackDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::CallbackDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CallbackDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "CallbackDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::CallbackDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CallbackDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CallbackDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "CallbackDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CallbackDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_CallbackDefinition_FromString(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "CallbackDefinition_FromString", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CallbackDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_CallbackDefinition_FromString__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CallbackDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        void *vptr = 0;
        int res = SWIG_ConvertPtr(argv[2], &vptr, SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_CallbackDefinition_FromString__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'CallbackDefinition_FromString'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::CallbackDefinition::FromString(std::string const &,RobotRaconteur::ServiceDefinitionParseInfo const *)\n"
    "    RobotRaconteur::CallbackDefinition::FromString(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_delete_CallbackDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::CallbackDefinition *arg1 = (RobotRaconteur::CallbackDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::CallbackDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::CallbackDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CallbackDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_CallbackDefinition" "', argument " "1"" of type '" "RobotRaconteur::CallbackDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CallbackDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CallbackDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CallbackDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *CallbackDefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CallbackDefinition_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *CallbackDefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WireDefinition_Type_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WireDefinition *arg1 = (RobotRaconteur::WireDefinition *) 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WireDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WireDefinition > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WireDefinition_Type_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WireDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WireDefinition_Type_set" "', argument " "1"" of type '" "RobotRaconteur::WireDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WireDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WireDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WireDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WireDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WireDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WireDefinition_Type_set" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::TypeDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2) : &tempshared2;
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Type = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WireDefinition_Type_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WireDefinition *arg1 = (RobotRaconteur::WireDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WireDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WireDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WireDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WireDefinition_Type_get" "', argument " "1"" of type '" "RobotRaconteur::WireDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WireDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WireDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WireDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WireDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WireDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::shared_ptr< RobotRaconteur::TypeDefinition > *) & ((arg1)->Type);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  {
    boost::shared_ptr<  RobotRaconteur::TypeDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::TypeDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WireDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WireDefinition *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_WireDefinition" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WireDefinition *)new RobotRaconteur::WireDefinition((boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WireDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WireDefinition >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WireDefinition_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WireDefinition_ToString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WireDefinition *arg1 = (RobotRaconteur::WireDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WireDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WireDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WireDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WireDefinition_ToString" "', argument " "1"" of type '" "RobotRaconteur::WireDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WireDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WireDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WireDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WireDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WireDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ToString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WireDefinition_FromString__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WireDefinition *arg1 = (RobotRaconteur::WireDefinition *) 0 ;
  std::string *arg2 = 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg3 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WireDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WireDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WireDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WireDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::WireDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WireDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WireDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WireDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WireDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WireDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WireDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WireDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WireDefinition_FromString" "', argument " "3"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo const *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2,(RobotRaconteur::ServiceDefinitionParseInfo const *)arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WireDefinition_FromString__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WireDefinition *arg1 = (RobotRaconteur::WireDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WireDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WireDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WireDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WireDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::WireDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WireDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WireDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WireDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WireDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WireDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WireDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WireDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WireDefinition_FromString(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WireDefinition_FromString", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WireDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_WireDefinition_FromString__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WireDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        void *vptr = 0;
        int res = SWIG_ConvertPtr(argv[2], &vptr, SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_WireDefinition_FromString__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WireDefinition_FromString'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WireDefinition::FromString(std::string const &,RobotRaconteur::ServiceDefinitionParseInfo const *)\n"
    "    RobotRaconteur::WireDefinition::FromString(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_delete_WireDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WireDefinition *arg1 = (RobotRaconteur::WireDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WireDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WireDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WireDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WireDefinition" "', argument " "1"" of type '" "RobotRaconteur::WireDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WireDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WireDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WireDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WireDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WireDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WireDefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WireDefinition_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WireDefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_MemoryDefinition_Type_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MemoryDefinition *arg1 = (RobotRaconteur::MemoryDefinition *) 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemoryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MemoryDefinition > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "MemoryDefinition_Type_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemoryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemoryDefinition_Type_set" "', argument " "1"" of type '" "RobotRaconteur::MemoryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemoryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemoryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemoryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemoryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemoryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MemoryDefinition_Type_set" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::TypeDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2) : &tempshared2;
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Type = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemoryDefinition_Type_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MemoryDefinition *arg1 = (RobotRaconteur::MemoryDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemoryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MemoryDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemoryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemoryDefinition_Type_get" "', argument " "1"" of type '" "RobotRaconteur::MemoryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemoryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemoryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemoryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemoryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemoryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::shared_ptr< RobotRaconteur::TypeDefinition > *) & ((arg1)->Type);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  {
    boost::shared_ptr<  RobotRaconteur::TypeDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::TypeDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_MemoryDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MemoryDefinition *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_MemoryDefinition" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::MemoryDefinition *)new RobotRaconteur::MemoryDefinition((boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::MemoryDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::MemoryDefinition >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemoryDefinition_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemoryDefinition_ToString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MemoryDefinition *arg1 = (RobotRaconteur::MemoryDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemoryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MemoryDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemoryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemoryDefinition_ToString" "', argument " "1"" of type '" "RobotRaconteur::MemoryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemoryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemoryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemoryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemoryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemoryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ToString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemoryDefinition_FromString__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::MemoryDefinition *arg1 = (RobotRaconteur::MemoryDefinition *) 0 ;
  std::string *arg2 = 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg3 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemoryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MemoryDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemoryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemoryDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::MemoryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemoryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemoryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemoryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemoryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemoryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MemoryDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MemoryDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "MemoryDefinition_FromString" "', argument " "3"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo const *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2,(RobotRaconteur::ServiceDefinitionParseInfo const *)arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemoryDefinition_FromString__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::MemoryDefinition *arg1 = (RobotRaconteur::MemoryDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemoryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MemoryDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemoryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemoryDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::MemoryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemoryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemoryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemoryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemoryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemoryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "MemoryDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "MemoryDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemoryDefinition_FromString(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "MemoryDefinition_FromString", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemoryDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_MemoryDefinition_FromString__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemoryDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        void *vptr = 0;
        int res = SWIG_ConvertPtr(argv[2], &vptr, SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_MemoryDefinition_FromString__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'MemoryDefinition_FromString'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::MemoryDefinition::FromString(std::string const &,RobotRaconteur::ServiceDefinitionParseInfo const *)\n"
    "    RobotRaconteur::MemoryDefinition::FromString(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_delete_MemoryDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MemoryDefinition *arg1 = (RobotRaconteur::MemoryDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemoryDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MemoryDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemoryDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_MemoryDefinition" "', argument " "1"" of type '" "RobotRaconteur::MemoryDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemoryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemoryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemoryDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MemoryDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MemoryDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *MemoryDefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemoryDefinition_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *MemoryDefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_TypeDefinition_Name_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TypeDefinition_Name_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_Name_set" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "TypeDefinition_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "TypeDefinition_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Name = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_Name_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_Name_get" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->Name);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_Type_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  RobotRaconteur::DataTypes arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TypeDefinition_Type_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_Type_set" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TypeDefinition_Type_set" "', argument " "2"" of type '" "RobotRaconteur::DataTypes""'");
  } 
  arg2 = static_cast< RobotRaconteur::DataTypes >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Type = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_Type_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::DataTypes result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_Type_get" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::DataTypes) ((arg1)->Type);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_TypeString_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TypeDefinition_TypeString_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_TypeString_set" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "TypeDefinition_TypeString_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "TypeDefinition_TypeString_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->TypeString = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_TypeString_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_TypeString_get" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->TypeString);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_ArrayType_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  RobotRaconteur::DataTypes_ArrayTypes arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TypeDefinition_ArrayType_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_ArrayType_set" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TypeDefinition_ArrayType_set" "', argument " "2"" of type '" "RobotRaconteur::DataTypes_ArrayTypes""'");
  } 
  arg2 = static_cast< RobotRaconteur::DataTypes_ArrayTypes >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ArrayType = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_ArrayType_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::DataTypes_ArrayTypes result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_ArrayType_get" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::DataTypes_ArrayTypes) ((arg1)->ArrayType);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_ArrayVarLength_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TypeDefinition_ArrayVarLength_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_ArrayVarLength_set" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TypeDefinition_ArrayVarLength_set" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ArrayVarLength = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_ArrayVarLength_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_ArrayVarLength_get" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (bool) ((arg1)->ArrayVarLength);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_ArrayLength_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  std::vector< int32_t,std::allocator< int32_t > > *arg2 = (std::vector< int32_t,std::allocator< int32_t > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TypeDefinition_ArrayLength_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_ArrayLength_set" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "TypeDefinition_ArrayLength_set" "', argument " "2"" of type '" "std::vector< int32_t,std::allocator< int32_t > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< int32_t,std::allocator< int32_t > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ArrayLength = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_ArrayLength_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< int32_t,std::allocator< int32_t > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_ArrayLength_get" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< int32_t,std::allocator< int32_t > > *)& ((arg1)->ArrayLength);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_int_std__allocatorT_int_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_ContainerType_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  RobotRaconteur::DataTypes_ContainerTypes arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TypeDefinition_ContainerType_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_ContainerType_set" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TypeDefinition_ContainerType_set" "', argument " "2"" of type '" "RobotRaconteur::DataTypes_ContainerTypes""'");
  } 
  arg2 = static_cast< RobotRaconteur::DataTypes_ContainerTypes >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ContainerType = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_ContainerType_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::DataTypes_ContainerTypes result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_ContainerType_get" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::DataTypes_ContainerTypes) ((arg1)->ContainerType);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_TypeDefinition__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::TypeDefinition *)new RobotRaconteur::TypeDefinition();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::TypeDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::TypeDefinition >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_ToString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_ToString" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ToString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_FromString__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  std::string *arg2 = 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg3 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "TypeDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "TypeDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "TypeDefinition_FromString" "', argument " "3"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo const *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2,(RobotRaconteur::ServiceDefinitionParseInfo const *)arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_FromString__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "TypeDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "TypeDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_FromString(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "TypeDefinition_FromString", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_TypeDefinition_FromString__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        void *vptr = 0;
        int res = SWIG_ConvertPtr(argv[2], &vptr, SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_TypeDefinition_FromString__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'TypeDefinition_FromString'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::TypeDefinition::FromString(std::string const &,RobotRaconteur::ServiceDefinitionParseInfo const *)\n"
    "    RobotRaconteur::TypeDefinition::FromString(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_DataTypeFromString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::DataTypes result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_DataTypeFromString" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "TypeDefinition_DataTypeFromString" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::DataTypes)RobotRaconteur::TypeDefinition::DataTypeFromString((std::string const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_StringFromDataType(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::DataTypes arg1 ;
  int val1 ;
  int ecode1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  ecode1 = SWIG_AsVal_int(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "TypeDefinition_StringFromDataType" "', argument " "1"" of type '" "RobotRaconteur::DataTypes""'");
  } 
  arg1 = static_cast< RobotRaconteur::DataTypes >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::TypeDefinition::StringFromDataType(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition__GetMember(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition__GetMember" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_TypeDefinition_GetMember(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::MemberDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::MemberDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition__SetMember(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TypeDefinition__SetMember", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition__SetMember" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "TypeDefinition__SetMember" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::MemberDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur_TypeDefinition_SetMember(arg1,(boost::shared_ptr< RobotRaconteur::MemberDefinition > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_CopyTo(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  RobotRaconteur::TypeDefinition *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition const > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition const > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TypeDefinition_CopyTo", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_CopyTo" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition const *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr< const RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< const RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr< const RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "TypeDefinition_CopyTo" "', argument " "2"" of type '" "RobotRaconteur::TypeDefinition &""'");
    }
    if (!argp2) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "TypeDefinition_CopyTo" "', argument " "2"" of type '" "RobotRaconteur::TypeDefinition &""'"); 
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared2 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp2);
      arg2 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared2.get());
    } else {
      arg2 = const_cast< RobotRaconteur::TypeDefinition * >(reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp2)->get());
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      ((RobotRaconteur::TypeDefinition const *)arg1)->CopyTo(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_Clone(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition const > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition const > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_Clone" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition const *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr< const RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< const RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr< const RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((RobotRaconteur::TypeDefinition const *)arg1)->Clone();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::TypeDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::TypeDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_Rename(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TypeDefinition_Rename", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_Rename" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "TypeDefinition_Rename" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "TypeDefinition_Rename" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Rename((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_RemoveContainers(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_RemoveContainers" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->RemoveContainers();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_RemoveArray(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_RemoveArray" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->RemoveArray();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_QualifyTypeStringWithUsing(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_QualifyTypeStringWithUsing" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->QualifyTypeStringWithUsing();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_UnqualifyTypeStringWithUsing(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_UnqualifyTypeStringWithUsing" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->UnqualifyTypeStringWithUsing();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TypeDefinition_Reset(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TypeDefinition_Reset" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Reset();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_TypeDefinition__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition const > tempshared1 ;
  RobotRaconteur::TypeDefinition *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_TypeDefinition" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition const &""'");
    }
    if (!argp1) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_TypeDefinition" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition const &""'"); 
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr< const RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< const RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(reinterpret_cast< boost::shared_ptr< const RobotRaconteur::TypeDefinition > * >(argp1)->get());
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::TypeDefinition *)new RobotRaconteur::TypeDefinition((RobotRaconteur::TypeDefinition const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::TypeDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::TypeDefinition >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_TypeDefinition(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[2] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_TypeDefinition", 0, 1, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_TypeDefinition__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_TypeDefinition__SWIG_1(self, argc, argv);
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_TypeDefinition'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::TypeDefinition::TypeDefinition()\n"
    "    RobotRaconteur::TypeDefinition::TypeDefinition(RobotRaconteur::TypeDefinition const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_delete_TypeDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TypeDefinition *arg1 = (RobotRaconteur::TypeDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_TypeDefinition" "', argument " "1"" of type '" "RobotRaconteur::TypeDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TypeDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *TypeDefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *TypeDefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_UsingDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::UsingDefinition *arg1 = (RobotRaconteur::UsingDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_UsingDefinition" "', argument " "1"" of type '" "RobotRaconteur::UsingDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_UsingDefinition_QualifiedName_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::UsingDefinition *arg1 = (RobotRaconteur::UsingDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "UsingDefinition_QualifiedName_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "UsingDefinition_QualifiedName_set" "', argument " "1"" of type '" "RobotRaconteur::UsingDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "UsingDefinition_QualifiedName_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "UsingDefinition_QualifiedName_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->QualifiedName = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_UsingDefinition_QualifiedName_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::UsingDefinition *arg1 = (RobotRaconteur::UsingDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "UsingDefinition_QualifiedName_get" "', argument " "1"" of type '" "RobotRaconteur::UsingDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->QualifiedName);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_UsingDefinition_UnqualifiedName_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::UsingDefinition *arg1 = (RobotRaconteur::UsingDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "UsingDefinition_UnqualifiedName_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "UsingDefinition_UnqualifiedName_set" "', argument " "1"" of type '" "RobotRaconteur::UsingDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "UsingDefinition_UnqualifiedName_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "UsingDefinition_UnqualifiedName_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->UnqualifiedName = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_UsingDefinition_UnqualifiedName_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::UsingDefinition *arg1 = (RobotRaconteur::UsingDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "UsingDefinition_UnqualifiedName_get" "', argument " "1"" of type '" "RobotRaconteur::UsingDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->UnqualifiedName);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_UsingDefinition_ParseInfo_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::UsingDefinition *arg1 = (RobotRaconteur::UsingDefinition *) 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg2 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "UsingDefinition_ParseInfo_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "UsingDefinition_ParseInfo_set" "', argument " "1"" of type '" "RobotRaconteur::UsingDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "UsingDefinition_ParseInfo_set" "', argument " "2"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ParseInfo = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_UsingDefinition_ParseInfo_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::UsingDefinition *arg1 = (RobotRaconteur::UsingDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::ServiceDefinitionParseInfo *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "UsingDefinition_ParseInfo_get" "', argument " "1"" of type '" "RobotRaconteur::UsingDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::ServiceDefinitionParseInfo *)& ((arg1)->ParseInfo);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_UsingDefinition__GetService(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::UsingDefinition *arg1 = (RobotRaconteur::UsingDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "UsingDefinition__GetService" "', argument " "1"" of type '" "RobotRaconteur::UsingDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_UsingDefinition_GetService(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServiceDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ServiceDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_UsingDefinition__SetService(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::UsingDefinition *arg1 = (RobotRaconteur::UsingDefinition *) 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "UsingDefinition__SetService", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "UsingDefinition__SetService" "', argument " "1"" of type '" "RobotRaconteur::UsingDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "UsingDefinition__SetService" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur_UsingDefinition_SetService(arg1,(boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_UsingDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::UsingDefinition *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_UsingDefinition" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::UsingDefinition *)new RobotRaconteur::UsingDefinition((boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::UsingDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::UsingDefinition >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_UsingDefinition_ToString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::UsingDefinition *arg1 = (RobotRaconteur::UsingDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "UsingDefinition_ToString" "', argument " "1"" of type '" "RobotRaconteur::UsingDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ToString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_UsingDefinition_FromString__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::UsingDefinition *arg1 = (RobotRaconteur::UsingDefinition *) 0 ;
  std::string *arg2 = 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg3 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "UsingDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::UsingDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "UsingDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "UsingDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "UsingDefinition_FromString" "', argument " "3"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo const *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2,(RobotRaconteur::ServiceDefinitionParseInfo const *)arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_UsingDefinition_FromString__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::UsingDefinition *arg1 = (RobotRaconteur::UsingDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::UsingDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "UsingDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::UsingDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UsingDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UsingDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "UsingDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "UsingDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_UsingDefinition_FromString(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "UsingDefinition_FromString", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_UsingDefinition_FromString__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        void *vptr = 0;
        int res = SWIG_ConvertPtr(argv[2], &vptr, SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_UsingDefinition_FromString__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'UsingDefinition_FromString'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::UsingDefinition::FromString(std::string const &,RobotRaconteur::ServiceDefinitionParseInfo const *)\n"
    "    RobotRaconteur::UsingDefinition::FromString(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *UsingDefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *UsingDefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_ConstantDefinition_StructField_Name_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition_StructField *arg1 = (RobotRaconteur::ConstantDefinition_StructField *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ConstantDefinition_StructField_Name_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition_StructField_Name_set" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition_StructField *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ConstantDefinition_StructField * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ConstantDefinition_StructField_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ConstantDefinition_StructField_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Name = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition_StructField_Name_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition_StructField *arg1 = (RobotRaconteur::ConstantDefinition_StructField *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition_StructField_Name_get" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition_StructField *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ConstantDefinition_StructField * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->Name);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition_StructField_ConstantRefName_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition_StructField *arg1 = (RobotRaconteur::ConstantDefinition_StructField *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ConstantDefinition_StructField_ConstantRefName_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition_StructField_ConstantRefName_set" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition_StructField *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ConstantDefinition_StructField * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ConstantDefinition_StructField_ConstantRefName_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ConstantDefinition_StructField_ConstantRefName_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ConstantRefName = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition_StructField_ConstantRefName_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition_StructField *arg1 = (RobotRaconteur::ConstantDefinition_StructField *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition_StructField_ConstantRefName_get" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition_StructField *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ConstantDefinition_StructField * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->ConstantRefName);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ConstantDefinition_StructField(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition_StructField *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_ConstantDefinition_StructField", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::ConstantDefinition_StructField *)new RobotRaconteur::ConstantDefinition_StructField();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_ConstantDefinition_StructField(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition_StructField *arg1 = (RobotRaconteur::ConstantDefinition_StructField *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_ConstantDefinition_StructField" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition_StructField *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ConstantDefinition_StructField * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *ConstantDefinition_StructField_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__ConstantDefinition_StructField, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *ConstantDefinition_StructField_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_ConstantDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition *arg1 = (RobotRaconteur::ConstantDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_ConstantDefinition" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition_Name_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition *arg1 = (RobotRaconteur::ConstantDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ConstantDefinition_Name_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition_Name_set" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ConstantDefinition_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ConstantDefinition_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Name = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition_Name_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition *arg1 = (RobotRaconteur::ConstantDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition_Name_get" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->Name);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition_DocString_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition *arg1 = (RobotRaconteur::ConstantDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ConstantDefinition_DocString_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition_DocString_set" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ConstantDefinition_DocString_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ConstantDefinition_DocString_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->DocString = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition_DocString_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition *arg1 = (RobotRaconteur::ConstantDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition_DocString_get" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->DocString);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition_Type_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition *arg1 = (RobotRaconteur::ConstantDefinition *) 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ConstantDefinition_Type_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition_Type_set" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ConstantDefinition_Type_set" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::TypeDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2) : &tempshared2;
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Type = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition_Type_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition *arg1 = (RobotRaconteur::ConstantDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition_Type_get" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::shared_ptr< RobotRaconteur::TypeDefinition > *) & ((arg1)->Type);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  {
    boost::shared_ptr<  RobotRaconteur::TypeDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::TypeDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition_Value_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition *arg1 = (RobotRaconteur::ConstantDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ConstantDefinition_Value_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition_Value_set" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ConstantDefinition_Value_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ConstantDefinition_Value_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Value = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition_Value_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition *arg1 = (RobotRaconteur::ConstantDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition_Value_get" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->Value);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition_ParseInfo_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition *arg1 = (RobotRaconteur::ConstantDefinition *) 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg2 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ConstantDefinition_ParseInfo_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition_ParseInfo_set" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ConstantDefinition_ParseInfo_set" "', argument " "2"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ParseInfo = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition_ParseInfo_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition *arg1 = (RobotRaconteur::ConstantDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::ServiceDefinitionParseInfo *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition_ParseInfo_get" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::ServiceDefinitionParseInfo *)& ((arg1)->ParseInfo);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition__GetService(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition *arg1 = (RobotRaconteur::ConstantDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition__GetService" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_ConstantDefinition_GetService(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServiceDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ServiceDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition__SetService(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition *arg1 = (RobotRaconteur::ConstantDefinition *) 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ConstantDefinition__SetService", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition__SetService" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ConstantDefinition__SetService" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur_ConstantDefinition_SetService(arg1,(boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition__GetServiceEntry(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition *arg1 = (RobotRaconteur::ConstantDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition__GetServiceEntry" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_ConstantDefinition_GetServiceEntry(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition__SetServiceEntry(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition *arg1 = (RobotRaconteur::ConstantDefinition *) 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ConstantDefinition__SetServiceEntry", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition__SetServiceEntry" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ConstantDefinition__SetServiceEntry" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur_ConstantDefinition_SetServiceEntry(arg1,(boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ConstantDefinition__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  RobotRaconteur::ConstantDefinition *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_ConstantDefinition" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::ConstantDefinition *)new RobotRaconteur::ConstantDefinition((boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ConstantDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ConstantDefinition >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ConstantDefinition__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  RobotRaconteur::ConstantDefinition *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_ConstantDefinition" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::ConstantDefinition *)new RobotRaconteur::ConstantDefinition((boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ConstantDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ConstantDefinition >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ConstantDefinition(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[2] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_ConstantDefinition", 0, 1, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_ConstantDefinition__SWIG_0(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_ConstantDefinition__SWIG_1(self, argc, argv);
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_ConstantDefinition'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::ConstantDefinition::ConstantDefinition(boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &)\n"
    "    RobotRaconteur::ConstantDefinition::ConstantDefinition(boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition_ToString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition *arg1 = (RobotRaconteur::ConstantDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition_ToString" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ToString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition_FromString__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition *arg1 = (RobotRaconteur::ConstantDefinition *) 0 ;
  std::string *arg2 = 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg3 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ConstantDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ConstantDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "ConstantDefinition_FromString" "', argument " "3"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo const *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2,(RobotRaconteur::ServiceDefinitionParseInfo const *)arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition_FromString__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition *arg1 = (RobotRaconteur::ConstantDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ConstantDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ConstantDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition_FromString(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "ConstantDefinition_FromString", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_ConstantDefinition_FromString__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        void *vptr = 0;
        int res = SWIG_ConvertPtr(argv[2], &vptr, SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_ConstantDefinition_FromString__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'ConstantDefinition_FromString'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::ConstantDefinition::FromString(std::string const &,RobotRaconteur::ServiceDefinitionParseInfo const *)\n"
    "    RobotRaconteur::ConstantDefinition::FromString(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition_Reset(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition *arg1 = (RobotRaconteur::ConstantDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition_Reset" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Reset();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition_VerifyValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition *arg1 = (RobotRaconteur::ConstantDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition_VerifyValue" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->VerifyValue();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition_ValueToString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition *arg1 = (RobotRaconteur::ConstantDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition_ValueToString" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ValueToString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition_ValueToStructFields(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ConstantDefinition *arg1 = (RobotRaconteur::ConstantDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ConstantDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition_ValueToStructFields" "', argument " "1"" of type '" "RobotRaconteur::ConstantDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ConstantDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ConstantDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ValueToStructFields();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition_UnescapeString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition_UnescapeString" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ConstantDefinition_UnescapeString" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::ConstantDefinition::UnescapeString((std::string const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ConstantDefinition_EscapeString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ConstantDefinition_EscapeString" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ConstantDefinition_EscapeString" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::ConstantDefinition::EscapeString((std::string const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *ConstantDefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *ConstantDefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_EnumDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinition *arg1 = (RobotRaconteur::EnumDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_EnumDefinition" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinition_Name_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinition *arg1 = (RobotRaconteur::EnumDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "EnumDefinition_Name_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinition_Name_set" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "EnumDefinition_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "EnumDefinition_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Name = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinition_Name_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinition *arg1 = (RobotRaconteur::EnumDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinition_Name_get" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->Name);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinition_DocString_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinition *arg1 = (RobotRaconteur::EnumDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "EnumDefinition_DocString_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinition_DocString_set" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "EnumDefinition_DocString_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "EnumDefinition_DocString_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->DocString = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinition_DocString_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinition *arg1 = (RobotRaconteur::EnumDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinition_DocString_get" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->DocString);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinition_Values_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinition *arg1 = (RobotRaconteur::EnumDefinition *) 0 ;
  std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > *arg2 = (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "EnumDefinition_Values_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinition_Values_set" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "EnumDefinition_Values_set" "', argument " "2"" of type '" "std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Values = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinition_Values_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinition *arg1 = (RobotRaconteur::EnumDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinition_Values_get" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > *)& ((arg1)->Values);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinition_ParseInfo_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinition *arg1 = (RobotRaconteur::EnumDefinition *) 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg2 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "EnumDefinition_ParseInfo_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinition_ParseInfo_set" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "EnumDefinition_ParseInfo_set" "', argument " "2"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ParseInfo = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinition_ParseInfo_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinition *arg1 = (RobotRaconteur::EnumDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::ServiceDefinitionParseInfo *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinition_ParseInfo_get" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::ServiceDefinitionParseInfo *)& ((arg1)->ParseInfo);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinition__GetService(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinition *arg1 = (RobotRaconteur::EnumDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinition__GetService" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_EnumDefinition_GetService(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServiceDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ServiceDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinition__SetService(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinition *arg1 = (RobotRaconteur::EnumDefinition *) 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "EnumDefinition__SetService", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinition__SetService" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "EnumDefinition__SetService" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur_EnumDefinition_SetService(arg1,(boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_EnumDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::EnumDefinition *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_EnumDefinition" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::EnumDefinition *)new RobotRaconteur::EnumDefinition((boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::EnumDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::EnumDefinition >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinition_ToString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinition *arg1 = (RobotRaconteur::EnumDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinition_ToString" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ToString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinition_FromString__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinition *arg1 = (RobotRaconteur::EnumDefinition *) 0 ;
  std::string *arg2 = 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg3 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "EnumDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "EnumDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "EnumDefinition_FromString" "', argument " "3"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo const *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2,(RobotRaconteur::ServiceDefinitionParseInfo const *)arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinition_FromString__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinition *arg1 = (RobotRaconteur::EnumDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "EnumDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "EnumDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinition_FromString(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "EnumDefinition_FromString", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_EnumDefinition_FromString__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        void *vptr = 0;
        int res = SWIG_ConvertPtr(argv[2], &vptr, SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_EnumDefinition_FromString__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'EnumDefinition_FromString'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::EnumDefinition::FromString(std::string const &,RobotRaconteur::ServiceDefinitionParseInfo const *)\n"
    "    RobotRaconteur::EnumDefinition::FromString(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_EnumDefinition_VerifyValues(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinition *arg1 = (RobotRaconteur::EnumDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinition_VerifyValues" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->VerifyValues();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinition_Reset(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinition *arg1 = (RobotRaconteur::EnumDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::EnumDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinition_Reset" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::EnumDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::EnumDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Reset();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *EnumDefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *EnumDefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_new_EnumDefinitionValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinitionValue *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_EnumDefinitionValue", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::EnumDefinitionValue *)new RobotRaconteur::EnumDefinitionValue();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinitionValue_Name_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinitionValue *arg1 = (RobotRaconteur::EnumDefinitionValue *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "EnumDefinitionValue_Name_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinitionValue_Name_set" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinitionValue *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::EnumDefinitionValue * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "EnumDefinitionValue_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "EnumDefinitionValue_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Name = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinitionValue_Name_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinitionValue *arg1 = (RobotRaconteur::EnumDefinitionValue *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinitionValue_Name_get" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinitionValue *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::EnumDefinitionValue * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->Name);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinitionValue_DocString_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinitionValue *arg1 = (RobotRaconteur::EnumDefinitionValue *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "EnumDefinitionValue_DocString_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinitionValue_DocString_set" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinitionValue *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::EnumDefinitionValue * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "EnumDefinitionValue_DocString_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "EnumDefinitionValue_DocString_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->DocString = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinitionValue_DocString_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinitionValue *arg1 = (RobotRaconteur::EnumDefinitionValue *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinitionValue_DocString_get" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinitionValue *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::EnumDefinitionValue * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->DocString);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinitionValue_Value_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinitionValue *arg1 = (RobotRaconteur::EnumDefinitionValue *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "EnumDefinitionValue_Value_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinitionValue_Value_set" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinitionValue *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::EnumDefinitionValue * >(argp1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "EnumDefinitionValue_Value_set" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Value = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinitionValue_Value_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinitionValue *arg1 = (RobotRaconteur::EnumDefinitionValue *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinitionValue_Value_get" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinitionValue *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::EnumDefinitionValue * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (int32_t) ((arg1)->Value);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinitionValue_ImplicitValue_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinitionValue *arg1 = (RobotRaconteur::EnumDefinitionValue *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "EnumDefinitionValue_ImplicitValue_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinitionValue_ImplicitValue_set" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinitionValue *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::EnumDefinitionValue * >(argp1);
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "EnumDefinitionValue_ImplicitValue_set" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ImplicitValue = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinitionValue_ImplicitValue_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinitionValue *arg1 = (RobotRaconteur::EnumDefinitionValue *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinitionValue_ImplicitValue_get" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinitionValue *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::EnumDefinitionValue * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (bool) ((arg1)->ImplicitValue);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinitionValue_HexValue_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinitionValue *arg1 = (RobotRaconteur::EnumDefinitionValue *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "EnumDefinitionValue_HexValue_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinitionValue_HexValue_set" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinitionValue *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::EnumDefinitionValue * >(argp1);
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "EnumDefinitionValue_HexValue_set" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->HexValue = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_EnumDefinitionValue_HexValue_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinitionValue *arg1 = (RobotRaconteur::EnumDefinitionValue *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "EnumDefinitionValue_HexValue_get" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinitionValue *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::EnumDefinitionValue * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (bool) ((arg1)->HexValue);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_EnumDefinitionValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::EnumDefinitionValue *arg1 = (RobotRaconteur::EnumDefinitionValue *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_EnumDefinitionValue" "', argument " "1"" of type '" "RobotRaconteur::EnumDefinitionValue *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::EnumDefinitionValue * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *EnumDefinitionValue_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__EnumDefinitionValue, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *EnumDefinitionValue_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_ExceptionDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ExceptionDefinition *arg1 = (RobotRaconteur::ExceptionDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_ExceptionDefinition" "', argument " "1"" of type '" "RobotRaconteur::ExceptionDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ExceptionDefinition_Name_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ExceptionDefinition *arg1 = (RobotRaconteur::ExceptionDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ExceptionDefinition_Name_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ExceptionDefinition_Name_set" "', argument " "1"" of type '" "RobotRaconteur::ExceptionDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ExceptionDefinition_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ExceptionDefinition_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Name = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ExceptionDefinition_Name_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ExceptionDefinition *arg1 = (RobotRaconteur::ExceptionDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ExceptionDefinition_Name_get" "', argument " "1"" of type '" "RobotRaconteur::ExceptionDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->Name);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ExceptionDefinition_DocString_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ExceptionDefinition *arg1 = (RobotRaconteur::ExceptionDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ExceptionDefinition_DocString_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ExceptionDefinition_DocString_set" "', argument " "1"" of type '" "RobotRaconteur::ExceptionDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ExceptionDefinition_DocString_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ExceptionDefinition_DocString_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->DocString = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ExceptionDefinition_DocString_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ExceptionDefinition *arg1 = (RobotRaconteur::ExceptionDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ExceptionDefinition_DocString_get" "', argument " "1"" of type '" "RobotRaconteur::ExceptionDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->DocString);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ExceptionDefinition_ParseInfo_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ExceptionDefinition *arg1 = (RobotRaconteur::ExceptionDefinition *) 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg2 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ExceptionDefinition_ParseInfo_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ExceptionDefinition_ParseInfo_set" "', argument " "1"" of type '" "RobotRaconteur::ExceptionDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ExceptionDefinition_ParseInfo_set" "', argument " "2"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ParseInfo = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ExceptionDefinition_ParseInfo_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ExceptionDefinition *arg1 = (RobotRaconteur::ExceptionDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::ServiceDefinitionParseInfo *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ExceptionDefinition_ParseInfo_get" "', argument " "1"" of type '" "RobotRaconteur::ExceptionDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::ServiceDefinitionParseInfo *)& ((arg1)->ParseInfo);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ExceptionDefinition__GetService(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ExceptionDefinition *arg1 = (RobotRaconteur::ExceptionDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ExceptionDefinition__GetService" "', argument " "1"" of type '" "RobotRaconteur::ExceptionDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_ExceptionDefinition_GetService(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServiceDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ServiceDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ExceptionDefinition__SetService(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ExceptionDefinition *arg1 = (RobotRaconteur::ExceptionDefinition *) 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ExceptionDefinition__SetService", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ExceptionDefinition__SetService" "', argument " "1"" of type '" "RobotRaconteur::ExceptionDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ExceptionDefinition__SetService" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur_ExceptionDefinition_SetService(arg1,(boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ExceptionDefinition(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared1 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::ExceptionDefinition *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_ExceptionDefinition" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::ExceptionDefinition *)new RobotRaconteur::ExceptionDefinition((boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ExceptionDefinition >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ExceptionDefinition_ToString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ExceptionDefinition *arg1 = (RobotRaconteur::ExceptionDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ExceptionDefinition_ToString" "', argument " "1"" of type '" "RobotRaconteur::ExceptionDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ToString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ExceptionDefinition_FromString__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ExceptionDefinition *arg1 = (RobotRaconteur::ExceptionDefinition *) 0 ;
  std::string *arg2 = 0 ;
  RobotRaconteur::ServiceDefinitionParseInfo *arg3 = (RobotRaconteur::ServiceDefinitionParseInfo *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ExceptionDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::ExceptionDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ExceptionDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ExceptionDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "ExceptionDefinition_FromString" "', argument " "3"" of type '" "RobotRaconteur::ServiceDefinitionParseInfo const *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::ServiceDefinitionParseInfo * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2,(RobotRaconteur::ServiceDefinitionParseInfo const *)arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ExceptionDefinition_FromString__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ExceptionDefinition *arg1 = (RobotRaconteur::ExceptionDefinition *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ExceptionDefinition_FromString" "', argument " "1"" of type '" "RobotRaconteur::ExceptionDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ExceptionDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ExceptionDefinition_FromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->FromString((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ExceptionDefinition_FromString(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "ExceptionDefinition_FromString", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_ExceptionDefinition_FromString__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        void *vptr = 0;
        int res = SWIG_ConvertPtr(argv[2], &vptr, SWIGTYPE_p_RobotRaconteur__ServiceDefinitionParseInfo, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_ExceptionDefinition_FromString__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'ExceptionDefinition_FromString'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::ExceptionDefinition::FromString(std::string const &,RobotRaconteur::ServiceDefinitionParseInfo const *)\n"
    "    RobotRaconteur::ExceptionDefinition::FromString(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_ExceptionDefinition_Reset(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ExceptionDefinition *arg1 = (RobotRaconteur::ExceptionDefinition *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ExceptionDefinition > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ExceptionDefinition_Reset" "', argument " "1"" of type '" "RobotRaconteur::ExceptionDefinition *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ExceptionDefinition > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ExceptionDefinition * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Reset();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *ExceptionDefinition_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *ExceptionDefinition_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_MemberDefinitionUtil_ToProperty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > tempshared1 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::PropertyDefinition > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemberDefinitionUtil_ToProperty" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::MemberDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = MemberDefinitionUtil::ToProperty((boost::shared_ptr< RobotRaconteur::MemberDefinition > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::PropertyDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::PropertyDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemberDefinitionUtil_ToFunction(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > tempshared1 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::FunctionDefinition > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemberDefinitionUtil_ToFunction" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::MemberDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = MemberDefinitionUtil::ToFunction((boost::shared_ptr< RobotRaconteur::MemberDefinition > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::FunctionDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::FunctionDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemberDefinitionUtil_ToObjRef(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > tempshared1 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::ObjRefDefinition > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemberDefinitionUtil_ToObjRef" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::MemberDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = MemberDefinitionUtil::ToObjRef((boost::shared_ptr< RobotRaconteur::MemberDefinition > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ObjRefDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ObjRefDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemberDefinitionUtil_ToEvent(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > tempshared1 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::EventDefinition > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemberDefinitionUtil_ToEvent" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::MemberDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = MemberDefinitionUtil::ToEvent((boost::shared_ptr< RobotRaconteur::MemberDefinition > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::EventDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::EventDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__EventDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemberDefinitionUtil_ToPipe(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > tempshared1 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::PipeDefinition > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemberDefinitionUtil_ToPipe" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::MemberDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = MemberDefinitionUtil::ToPipe((boost::shared_ptr< RobotRaconteur::MemberDefinition > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::PipeDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::PipeDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PipeDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemberDefinitionUtil_ToCallback(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > tempshared1 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::CallbackDefinition > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemberDefinitionUtil_ToCallback" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::MemberDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = MemberDefinitionUtil::ToCallback((boost::shared_ptr< RobotRaconteur::MemberDefinition > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::CallbackDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::CallbackDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CallbackDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemberDefinitionUtil_ToWire(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > tempshared1 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::WireDefinition > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemberDefinitionUtil_ToWire" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::MemberDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = MemberDefinitionUtil::ToWire((boost::shared_ptr< RobotRaconteur::MemberDefinition > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WireDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WireDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WireDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MemberDefinitionUtil_ToMemory(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MemberDefinition > tempshared1 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MemoryDefinition > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MemberDefinitionUtil_ToMemory" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::MemberDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::MemberDefinition > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = MemberDefinitionUtil::ToMemory((boost::shared_ptr< RobotRaconteur::MemberDefinition > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::MemoryDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::MemoryDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MemoryDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_MemberDefinitionUtil(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  MemberDefinitionUtil *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_MemberDefinitionUtil", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (MemberDefinitionUtil *)new MemberDefinitionUtil();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_MemberDefinitionUtil, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_MemberDefinitionUtil(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  MemberDefinitionUtil *arg1 = (MemberDefinitionUtil *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_MemberDefinitionUtil, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_MemberDefinitionUtil" "', argument " "1"" of type '" "MemberDefinitionUtil *""'"); 
  }
  arg1 = reinterpret_cast< MemberDefinitionUtil * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *MemberDefinitionUtil_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_MemberDefinitionUtil, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *MemberDefinitionUtil_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_Transport_GetUrlSchemeString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::Transport *arg1 = (RobotRaconteur::Transport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::Transport const > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::Transport const > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Transport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Transport_GetUrlSchemeString" "', argument " "1"" of type '" "RobotRaconteur::Transport const *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr< const RobotRaconteur::Transport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< const RobotRaconteur::Transport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Transport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr< const RobotRaconteur::Transport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Transport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((RobotRaconteur::Transport const *)arg1)->GetUrlSchemeString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_Transport_GetCurrentTransportConnectionURL(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::string result;
  
  if (!SWIG_Python_UnpackTuple(args, "Transport_GetCurrentTransportConnectionURL", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::Transport::GetCurrentTransportConnectionURL();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_Transport_GetServerListenUrls(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::Transport *arg1 = (RobotRaconteur::Transport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::Transport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::Transport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Transport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Transport_GetServerListenUrls" "', argument " "1"" of type '" "RobotRaconteur::Transport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Transport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Transport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Transport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Transport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Transport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetServerListenUrls();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< std::string,std::allocator< std::string > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_Transport(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::Transport *arg1 = (RobotRaconteur::Transport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::Transport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::Transport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Transport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Transport" "', argument " "1"" of type '" "RobotRaconteur::Transport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Transport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Transport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Transport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Transport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Transport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *Transport_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Transport_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_new_TcpTransport__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::TcpTransport *)new RobotRaconteur::TcpTransport();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::TcpTransport > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::TcpTransport >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_TcpTransport__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  RobotRaconteur::TcpTransport *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_TcpTransport" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::TcpTransport *)new RobotRaconteur::TcpTransport((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::TcpTransport > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::TcpTransport >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_TcpTransport(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[2] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_TcpTransport", 0, 1, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_TcpTransport__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_TcpTransport__SWIG_1(self, argc, argv);
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_TcpTransport'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::TcpTransport::TcpTransport()\n"
    "    RobotRaconteur::TcpTransport::TcpTransport(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_TcpTransport__GetDefaultReceiveTimeout(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__GetDefaultReceiveTimeout" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (int32_t)(arg1)->GetDefaultReceiveTimeout();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__SetDefaultReceiveTimeout(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TcpTransport__SetDefaultReceiveTimeout", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__SetDefaultReceiveTimeout" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TcpTransport__SetDefaultReceiveTimeout" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetDefaultReceiveTimeout(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__GetDefaultConnectTimeout(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__GetDefaultConnectTimeout" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (int32_t)(arg1)->GetDefaultConnectTimeout();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__SetDefaultConnectTimeout(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TcpTransport__SetDefaultConnectTimeout", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__SetDefaultConnectTimeout" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TcpTransport__SetDefaultConnectTimeout" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetDefaultConnectTimeout(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__GetDefaultHeartbeatPeriod(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__GetDefaultHeartbeatPeriod" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (int32_t)(arg1)->GetDefaultHeartbeatPeriod();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__SetDefaultHeartbeatPeriod(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TcpTransport__SetDefaultHeartbeatPeriod", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__SetDefaultHeartbeatPeriod" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TcpTransport__SetDefaultHeartbeatPeriod" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetDefaultHeartbeatPeriod(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport_GetUrlSchemeString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport const > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport const > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport_GetUrlSchemeString" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport const *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr< const RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< const RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr< const RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((RobotRaconteur::TcpTransport const *)arg1)->GetUrlSchemeString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport_GetServerListenUrls(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport_GetServerListenUrls" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetServerListenUrls();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< std::string,std::allocator< std::string > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport_GetListenPort(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport_GetListenPort" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (int32_t)(arg1)->GetListenPort();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport_StartServer__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  int32_t arg2 ;
  bool arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  bool val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport_StartServer" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TcpTransport_StartServer" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  ecode3 = SWIG_AsVal_bool(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "TcpTransport_StartServer" "', argument " "3"" of type '" "bool""'");
  } 
  arg3 = static_cast< bool >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->StartServer(arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport_StartServer__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport_StartServer" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TcpTransport_StartServer" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->StartServer(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport_StartServer(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "TcpTransport_StartServer", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_TcpTransport_StartServer__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_bool(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_TcpTransport_StartServer__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'TcpTransport_StartServer'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::TcpTransport::StartServer(int32_t,bool)\n"
    "    RobotRaconteur::TcpTransport::StartServer(int32_t)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_TcpTransport_Close(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport_Close" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Close();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport_EnableNodeDiscoveryListening__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport_EnableNodeDiscoveryListening" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TcpTransport_EnableNodeDiscoveryListening" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->EnableNodeDiscoveryListening(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport_EnableNodeDiscoveryListening__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport_EnableNodeDiscoveryListening" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->EnableNodeDiscoveryListening();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport_EnableNodeDiscoveryListening(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "TcpTransport_EnableNodeDiscoveryListening", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_TcpTransport_EnableNodeDiscoveryListening__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_unsigned_SS_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_TcpTransport_EnableNodeDiscoveryListening__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'TcpTransport_EnableNodeDiscoveryListening'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::TcpTransport::EnableNodeDiscoveryListening(uint32_t)\n"
    "    RobotRaconteur::TcpTransport::EnableNodeDiscoveryListening()\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_TcpTransport_DisableNodeDiscoveryListening(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport_DisableNodeDiscoveryListening" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->DisableNodeDiscoveryListening();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport_EnableNodeAnnounce__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport_EnableNodeAnnounce" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TcpTransport_EnableNodeAnnounce" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->EnableNodeAnnounce(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport_EnableNodeAnnounce__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport_EnableNodeAnnounce" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->EnableNodeAnnounce();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport_EnableNodeAnnounce(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "TcpTransport_EnableNodeAnnounce", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_TcpTransport_EnableNodeAnnounce__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_unsigned_SS_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_TcpTransport_EnableNodeAnnounce__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'TcpTransport_EnableNodeAnnounce'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::TcpTransport::EnableNodeAnnounce(uint32_t)\n"
    "    RobotRaconteur::TcpTransport::EnableNodeAnnounce()\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_TcpTransport_DisableNodeAnnounce(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport_DisableNodeAnnounce" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->DisableNodeAnnounce();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__GetNodeAnnouncePeriod(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__GetNodeAnnouncePeriod" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (int32_t)(arg1)->GetNodeAnnouncePeriod();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__SetNodeAnnouncePeriod(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TcpTransport__SetNodeAnnouncePeriod", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__SetNodeAnnouncePeriod" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TcpTransport__SetNodeAnnouncePeriod" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetNodeAnnouncePeriod(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__GetMaxMessageSize(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__GetMaxMessageSize" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (int32_t)(arg1)->GetMaxMessageSize();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__SetMaxMessageSize(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TcpTransport__SetMaxMessageSize", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__SetMaxMessageSize" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TcpTransport__SetMaxMessageSize" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetMaxMessageSize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__GetMaxConnectionCount(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__GetMaxConnectionCount" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (int32_t)(arg1)->GetMaxConnectionCount();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__SetMaxConnectionCount(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TcpTransport__SetMaxConnectionCount", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__SetMaxConnectionCount" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TcpTransport__SetMaxConnectionCount" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetMaxConnectionCount(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__GetRequireTls(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__GetRequireTls" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->GetRequireTls();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__SetRequireTls(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TcpTransport__SetRequireTls", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__SetRequireTls" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TcpTransport__SetRequireTls" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetRequireTls(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport_LoadTlsNodeCertificate(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport_LoadTlsNodeCertificate" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->LoadTlsNodeCertificate();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__IsTlsNodeCertificateLoaded(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__IsTlsNodeCertificateLoaded" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->IsTlsNodeCertificateLoaded();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__IsTransportConnectionSecure__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  bool result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__IsTransportConnectionSecure" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TcpTransport__IsTransportConnectionSecure" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->IsTransportConnectionSecure(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__IsTransportConnectionSecure__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  boost::shared_ptr< RobotRaconteur::RRObject > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRObject > tempshared2 ;
  bool result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__IsTransportConnectionSecure" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRObject_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "TcpTransport__IsTransportConnectionSecure" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::RRObject > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->IsTransportConnectionSecure((boost::shared_ptr< RobotRaconteur::RRObject > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__IsTransportConnectionSecure(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "TcpTransport__IsTransportConnectionSecure", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRObject_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_TcpTransport__IsTransportConnectionSecure__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_unsigned_SS_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_TcpTransport__IsTransportConnectionSecure__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'TcpTransport__IsTransportConnectionSecure'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::TcpTransport::IsTransportConnectionSecure(uint32_t)\n"
    "    RobotRaconteur::TcpTransport::IsTransportConnectionSecure(boost::shared_ptr< RobotRaconteur::RRObject > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_TcpTransport__IsSecurePeerIdentityVerified__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  bool result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__IsSecurePeerIdentityVerified" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TcpTransport__IsSecurePeerIdentityVerified" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->IsSecurePeerIdentityVerified(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__IsSecurePeerIdentityVerified__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  boost::shared_ptr< RobotRaconteur::RRObject > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRObject > tempshared2 ;
  bool result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__IsSecurePeerIdentityVerified" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRObject_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "TcpTransport__IsSecurePeerIdentityVerified" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::RRObject > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->IsSecurePeerIdentityVerified((boost::shared_ptr< RobotRaconteur::RRObject > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__IsSecurePeerIdentityVerified(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "TcpTransport__IsSecurePeerIdentityVerified", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRObject_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_TcpTransport__IsSecurePeerIdentityVerified__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_unsigned_SS_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_TcpTransport__IsSecurePeerIdentityVerified__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'TcpTransport__IsSecurePeerIdentityVerified'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::TcpTransport::IsSecurePeerIdentityVerified(uint32_t)\n"
    "    RobotRaconteur::TcpTransport::IsSecurePeerIdentityVerified(boost::shared_ptr< RobotRaconteur::RRObject > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_TcpTransport__GetSecurePeerIdentity__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  std::string result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__GetSecurePeerIdentity" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TcpTransport__GetSecurePeerIdentity" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetSecurePeerIdentity(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__GetSecurePeerIdentity__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  boost::shared_ptr< RobotRaconteur::RRObject > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRObject > tempshared2 ;
  std::string result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__GetSecurePeerIdentity" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRObject_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "TcpTransport__GetSecurePeerIdentity" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::RRObject > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetSecurePeerIdentity((boost::shared_ptr< RobotRaconteur::RRObject > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__GetSecurePeerIdentity(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "TcpTransport__GetSecurePeerIdentity", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRObject_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_TcpTransport__GetSecurePeerIdentity__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_unsigned_SS_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_TcpTransport__GetSecurePeerIdentity__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'TcpTransport__GetSecurePeerIdentity'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::TcpTransport::GetSecurePeerIdentity(uint32_t)\n"
    "    RobotRaconteur::TcpTransport::GetSecurePeerIdentity(boost::shared_ptr< RobotRaconteur::RRObject > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_TcpTransport_StartServerUsingPortSharer(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport_StartServerUsingPortSharer" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->StartServerUsingPortSharer();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__IsPortSharerRunning(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__IsPortSharerRunning" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->IsPortSharerRunning();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__GetAcceptWebSockets(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__GetAcceptWebSockets" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->GetAcceptWebSockets();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__SetAcceptWebSockets(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TcpTransport__SetAcceptWebSockets", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__SetAcceptWebSockets" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TcpTransport__SetAcceptWebSockets" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetAcceptWebSockets(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport_GetWebSocketAllowedOrigins(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport_GetWebSocketAllowedOrigins" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetWebSocketAllowedOrigins();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< std::string,std::allocator< std::string > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport_AddWebSocketAllowedOrigin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TcpTransport_AddWebSocketAllowedOrigin", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport_AddWebSocketAllowedOrigin" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "TcpTransport_AddWebSocketAllowedOrigin" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "TcpTransport_AddWebSocketAllowedOrigin" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->AddWebSocketAllowedOrigin((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport_RemoveWebSocketAllowedOrigin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TcpTransport_RemoveWebSocketAllowedOrigin", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport_RemoveWebSocketAllowedOrigin" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "TcpTransport_RemoveWebSocketAllowedOrigin" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "TcpTransport_RemoveWebSocketAllowedOrigin" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->RemoveWebSocketAllowedOrigin((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__GetDisableMessage4(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__GetDisableMessage4" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->GetDisableMessage4();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__SetDisableMessage4(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TcpTransport__SetDisableMessage4", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__SetDisableMessage4" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TcpTransport__SetDisableMessage4" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetDisableMessage4(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__GetDisableStringTable(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__GetDisableStringTable" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->GetDisableStringTable();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__SetDisableStringTable(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TcpTransport__SetDisableStringTable", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__SetDisableStringTable" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TcpTransport__SetDisableStringTable" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetDisableStringTable(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__GetDisableAsyncMessageIO(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__GetDisableAsyncMessageIO" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->GetDisableAsyncMessageIO();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport__SetDisableAsyncMessageIO(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TcpTransport__SetDisableAsyncMessageIO", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TcpTransport__SetDisableAsyncMessageIO" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TcpTransport__SetDisableAsyncMessageIO" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetDisableAsyncMessageIO(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TcpTransport_GetLocalAdapterIPAddresses(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string,std::allocator< std::string > > result;
  
  if (!SWIG_Python_UnpackTuple(args, "TcpTransport_GetLocalAdapterIPAddresses", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_TcpTransport_GetLocalAdapterIPAddresses();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< std::string,std::allocator< std::string > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_TcpTransport(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TcpTransport *arg1 = (RobotRaconteur::TcpTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_TcpTransport" "', argument " "1"" of type '" "RobotRaconteur::TcpTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::TcpTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::TcpTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *TcpTransport_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *TcpTransport_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_new_LocalTransport__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  RobotRaconteur::LocalTransport *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::LocalTransport *)new RobotRaconteur::LocalTransport();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::LocalTransport > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::LocalTransport >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_LocalTransport__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  RobotRaconteur::LocalTransport *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_LocalTransport" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::LocalTransport *)new RobotRaconteur::LocalTransport((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::LocalTransport > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::LocalTransport >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_LocalTransport(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[2] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_LocalTransport", 0, 1, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_LocalTransport__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_LocalTransport__SWIG_1(self, argc, argv);
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_LocalTransport'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::LocalTransport::LocalTransport()\n"
    "    RobotRaconteur::LocalTransport::LocalTransport(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_LocalTransport_IsLocalTransportSupported(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "LocalTransport_IsLocalTransportSupported", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)RobotRaconteur::LocalTransport::IsLocalTransportSupported();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_LocalTransport_GetUrlSchemeString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::LocalTransport *arg1 = (RobotRaconteur::LocalTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport const > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport const > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "LocalTransport_GetUrlSchemeString" "', argument " "1"" of type '" "RobotRaconteur::LocalTransport const *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr< const RobotRaconteur::LocalTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< const RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr< const RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((RobotRaconteur::LocalTransport const *)arg1)->GetUrlSchemeString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_LocalTransport_GetServerListenUrls(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::LocalTransport *arg1 = (RobotRaconteur::LocalTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "LocalTransport_GetServerListenUrls" "', argument " "1"" of type '" "RobotRaconteur::LocalTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetServerListenUrls();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< std::string,std::allocator< std::string > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_LocalTransport_StartClientAsNodeName(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::LocalTransport *arg1 = (RobotRaconteur::LocalTransport *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "LocalTransport_StartClientAsNodeName", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "LocalTransport_StartClientAsNodeName" "', argument " "1"" of type '" "RobotRaconteur::LocalTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "LocalTransport_StartClientAsNodeName" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "LocalTransport_StartClientAsNodeName" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->StartClientAsNodeName((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_LocalTransport_StartServerAsNodeName__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::LocalTransport *arg1 = (RobotRaconteur::LocalTransport *) 0 ;
  std::string *arg2 = 0 ;
  bool arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  bool val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "LocalTransport_StartServerAsNodeName" "', argument " "1"" of type '" "RobotRaconteur::LocalTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "LocalTransport_StartServerAsNodeName" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "LocalTransport_StartServerAsNodeName" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  ecode3 = SWIG_AsVal_bool(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "LocalTransport_StartServerAsNodeName" "', argument " "3"" of type '" "bool""'");
  } 
  arg3 = static_cast< bool >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->StartServerAsNodeName((std::string const &)*arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_LocalTransport_StartServerAsNodeName__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::LocalTransport *arg1 = (RobotRaconteur::LocalTransport *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "LocalTransport_StartServerAsNodeName" "', argument " "1"" of type '" "RobotRaconteur::LocalTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "LocalTransport_StartServerAsNodeName" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "LocalTransport_StartServerAsNodeName" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->StartServerAsNodeName((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_LocalTransport_StartServerAsNodeName(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "LocalTransport_StartServerAsNodeName", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_LocalTransport_StartServerAsNodeName__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        {
          int res = SWIG_AsVal_bool(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_LocalTransport_StartServerAsNodeName__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'LocalTransport_StartServerAsNodeName'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::LocalTransport::StartServerAsNodeName(std::string const &,bool)\n"
    "    RobotRaconteur::LocalTransport::StartServerAsNodeName(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_LocalTransport_StartServerAsNodeID__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::LocalTransport *arg1 = (RobotRaconteur::LocalTransport *) 0 ;
  RobotRaconteur::NodeID *arg2 = 0 ;
  bool arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  bool val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "LocalTransport_StartServerAsNodeID" "', argument " "1"" of type '" "RobotRaconteur::LocalTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__NodeID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "LocalTransport_StartServerAsNodeID" "', argument " "2"" of type '" "RobotRaconteur::NodeID const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "LocalTransport_StartServerAsNodeID" "', argument " "2"" of type '" "RobotRaconteur::NodeID const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::NodeID * >(argp2);
  ecode3 = SWIG_AsVal_bool(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "LocalTransport_StartServerAsNodeID" "', argument " "3"" of type '" "bool""'");
  } 
  arg3 = static_cast< bool >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->StartServerAsNodeID((RobotRaconteur::NodeID const &)*arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_LocalTransport_StartServerAsNodeID__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::LocalTransport *arg1 = (RobotRaconteur::LocalTransport *) 0 ;
  RobotRaconteur::NodeID *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "LocalTransport_StartServerAsNodeID" "', argument " "1"" of type '" "RobotRaconteur::LocalTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__NodeID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "LocalTransport_StartServerAsNodeID" "', argument " "2"" of type '" "RobotRaconteur::NodeID const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "LocalTransport_StartServerAsNodeID" "', argument " "2"" of type '" "RobotRaconteur::NodeID const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::NodeID * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->StartServerAsNodeID((RobotRaconteur::NodeID const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_LocalTransport_StartServerAsNodeID(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "LocalTransport_StartServerAsNodeID", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_RobotRaconteur__NodeID, SWIG_POINTER_NO_NULL | 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_LocalTransport_StartServerAsNodeID__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_RobotRaconteur__NodeID, SWIG_POINTER_NO_NULL | 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        {
          int res = SWIG_AsVal_bool(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_LocalTransport_StartServerAsNodeID__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'LocalTransport_StartServerAsNodeID'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::LocalTransport::StartServerAsNodeID(RobotRaconteur::NodeID const &,bool)\n"
    "    RobotRaconteur::LocalTransport::StartServerAsNodeID(RobotRaconteur::NodeID const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_LocalTransport_Close(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::LocalTransport *arg1 = (RobotRaconteur::LocalTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "LocalTransport_Close" "', argument " "1"" of type '" "RobotRaconteur::LocalTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Close();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_LocalTransport__GetMaxMessageSize(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::LocalTransport *arg1 = (RobotRaconteur::LocalTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "LocalTransport__GetMaxMessageSize" "', argument " "1"" of type '" "RobotRaconteur::LocalTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (int32_t)(arg1)->GetMaxMessageSize();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_LocalTransport__SetMaxMessageSize(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::LocalTransport *arg1 = (RobotRaconteur::LocalTransport *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "LocalTransport__SetMaxMessageSize", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "LocalTransport__SetMaxMessageSize" "', argument " "1"" of type '" "RobotRaconteur::LocalTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "LocalTransport__SetMaxMessageSize" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetMaxMessageSize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_LocalTransport__GetDisableMessage4(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::LocalTransport *arg1 = (RobotRaconteur::LocalTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "LocalTransport__GetDisableMessage4" "', argument " "1"" of type '" "RobotRaconteur::LocalTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->GetDisableMessage4();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_LocalTransport__SetDisableMessage4(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::LocalTransport *arg1 = (RobotRaconteur::LocalTransport *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > *smartarg1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "LocalTransport__SetDisableMessage4", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "LocalTransport__SetDisableMessage4" "', argument " "1"" of type '" "RobotRaconteur::LocalTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "LocalTransport__SetDisableMessage4" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetDisableMessage4(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_LocalTransport__GetDisableStringTable(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::LocalTransport *arg1 = (RobotRaconteur::LocalTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "LocalTransport__GetDisableStringTable" "', argument " "1"" of type '" "RobotRaconteur::LocalTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->GetDisableStringTable();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_LocalTransport__SetDisableStringTable(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::LocalTransport *arg1 = (RobotRaconteur::LocalTransport *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > *smartarg1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "LocalTransport__SetDisableStringTable", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "LocalTransport__SetDisableStringTable" "', argument " "1"" of type '" "RobotRaconteur::LocalTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "LocalTransport__SetDisableStringTable" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetDisableStringTable(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_LocalTransport__GetDisableAsyncMessageIO(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::LocalTransport *arg1 = (RobotRaconteur::LocalTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "LocalTransport__GetDisableAsyncMessageIO" "', argument " "1"" of type '" "RobotRaconteur::LocalTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->GetDisableAsyncMessageIO();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_LocalTransport__SetDisableAsyncMessageIO(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::LocalTransport *arg1 = (RobotRaconteur::LocalTransport *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > *smartarg1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "LocalTransport__SetDisableAsyncMessageIO", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "LocalTransport__SetDisableAsyncMessageIO" "', argument " "1"" of type '" "RobotRaconteur::LocalTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "LocalTransport__SetDisableAsyncMessageIO" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetDisableAsyncMessageIO(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_LocalTransport_EnableNodeDiscoveryListening(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::LocalTransport *arg1 = (RobotRaconteur::LocalTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "LocalTransport_EnableNodeDiscoveryListening" "', argument " "1"" of type '" "RobotRaconteur::LocalTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->EnableNodeDiscoveryListening();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_LocalTransport_DisableNodeDiscoveryListening(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::LocalTransport *arg1 = (RobotRaconteur::LocalTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "LocalTransport_DisableNodeDiscoveryListening" "', argument " "1"" of type '" "RobotRaconteur::LocalTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->DisableNodeDiscoveryListening();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_LocalTransport(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::LocalTransport *arg1 = (RobotRaconteur::LocalTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_LocalTransport" "', argument " "1"" of type '" "RobotRaconteur::LocalTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LocalTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LocalTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *LocalTransport_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *LocalTransport_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_new_HardwareTransport__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  RobotRaconteur::HardwareTransport *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::HardwareTransport *)new RobotRaconteur::HardwareTransport();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::HardwareTransport > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::HardwareTransport >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_HardwareTransport__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  RobotRaconteur::HardwareTransport *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_HardwareTransport" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::HardwareTransport *)new RobotRaconteur::HardwareTransport((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::HardwareTransport > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::HardwareTransport >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_HardwareTransport(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[2] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_HardwareTransport", 0, 1, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_HardwareTransport__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_HardwareTransport__SWIG_1(self, argc, argv);
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_HardwareTransport'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::HardwareTransport::HardwareTransport()\n"
    "    RobotRaconteur::HardwareTransport::HardwareTransport(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_HardwareTransport_GetUrlSchemeString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HardwareTransport *arg1 = (RobotRaconteur::HardwareTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport const > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport const > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "HardwareTransport_GetUrlSchemeString" "', argument " "1"" of type '" "RobotRaconteur::HardwareTransport const *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr< const RobotRaconteur::HardwareTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< const RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr< const RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((RobotRaconteur::HardwareTransport const *)arg1)->GetUrlSchemeString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_HardwareTransport_GetServerListenUrls(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HardwareTransport *arg1 = (RobotRaconteur::HardwareTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "HardwareTransport_GetServerListenUrls" "', argument " "1"" of type '" "RobotRaconteur::HardwareTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetServerListenUrls();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< std::string,std::allocator< std::string > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_HardwareTransport_Close(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HardwareTransport *arg1 = (RobotRaconteur::HardwareTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "HardwareTransport_Close" "', argument " "1"" of type '" "RobotRaconteur::HardwareTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Close();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_HardwareTransport__GetMaxMessageSize(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HardwareTransport *arg1 = (RobotRaconteur::HardwareTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "HardwareTransport__GetMaxMessageSize" "', argument " "1"" of type '" "RobotRaconteur::HardwareTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (int32_t)(arg1)->GetMaxMessageSize();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_HardwareTransport__SetMaxMessageSize(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HardwareTransport *arg1 = (RobotRaconteur::HardwareTransport *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "HardwareTransport__SetMaxMessageSize", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "HardwareTransport__SetMaxMessageSize" "', argument " "1"" of type '" "RobotRaconteur::HardwareTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "HardwareTransport__SetMaxMessageSize" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetMaxMessageSize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_HardwareTransport__GetDisableMessage4(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HardwareTransport *arg1 = (RobotRaconteur::HardwareTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "HardwareTransport__GetDisableMessage4" "', argument " "1"" of type '" "RobotRaconteur::HardwareTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->GetDisableMessage4();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_HardwareTransport__SetDisableMessage4(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HardwareTransport *arg1 = (RobotRaconteur::HardwareTransport *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport > *smartarg1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "HardwareTransport__SetDisableMessage4", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "HardwareTransport__SetDisableMessage4" "', argument " "1"" of type '" "RobotRaconteur::HardwareTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "HardwareTransport__SetDisableMessage4" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetDisableMessage4(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_HardwareTransport__GetDisableStringTable(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HardwareTransport *arg1 = (RobotRaconteur::HardwareTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "HardwareTransport__GetDisableStringTable" "', argument " "1"" of type '" "RobotRaconteur::HardwareTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->GetDisableStringTable();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_HardwareTransport__SetDisableStringTable(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HardwareTransport *arg1 = (RobotRaconteur::HardwareTransport *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport > *smartarg1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "HardwareTransport__SetDisableStringTable", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "HardwareTransport__SetDisableStringTable" "', argument " "1"" of type '" "RobotRaconteur::HardwareTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "HardwareTransport__SetDisableStringTable" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetDisableStringTable(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_HardwareTransport__GetDisableAsyncMessageIO(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HardwareTransport *arg1 = (RobotRaconteur::HardwareTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "HardwareTransport__GetDisableAsyncMessageIO" "', argument " "1"" of type '" "RobotRaconteur::HardwareTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->GetDisableAsyncMessageIO();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_HardwareTransport__SetDisableAsyncMessageIO(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HardwareTransport *arg1 = (RobotRaconteur::HardwareTransport *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport > *smartarg1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "HardwareTransport__SetDisableAsyncMessageIO", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "HardwareTransport__SetDisableAsyncMessageIO" "', argument " "1"" of type '" "RobotRaconteur::HardwareTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "HardwareTransport__SetDisableAsyncMessageIO" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetDisableAsyncMessageIO(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_HardwareTransport(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::HardwareTransport *arg1 = (RobotRaconteur::HardwareTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_HardwareTransport" "', argument " "1"" of type '" "RobotRaconteur::HardwareTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::HardwareTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::HardwareTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *HardwareTransport_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *HardwareTransport_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_new_IntraTransport__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  RobotRaconteur::IntraTransport *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::IntraTransport *)new RobotRaconteur::IntraTransport();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::IntraTransport > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::IntraTransport >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__IntraTransport_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_IntraTransport__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  RobotRaconteur::IntraTransport *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_IntraTransport" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::IntraTransport *)new RobotRaconteur::IntraTransport((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::IntraTransport > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::IntraTransport >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__IntraTransport_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_IntraTransport(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[2] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_IntraTransport", 0, 1, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_IntraTransport__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_IntraTransport__SWIG_1(self, argc, argv);
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_IntraTransport'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::IntraTransport::IntraTransport()\n"
    "    RobotRaconteur::IntraTransport::IntraTransport(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_IntraTransport_GetUrlSchemeString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::IntraTransport *arg1 = (RobotRaconteur::IntraTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::IntraTransport const > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::IntraTransport const > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__IntraTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "IntraTransport_GetUrlSchemeString" "', argument " "1"" of type '" "RobotRaconteur::IntraTransport const *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr< const RobotRaconteur::IntraTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< const RobotRaconteur::IntraTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::IntraTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr< const RobotRaconteur::IntraTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::IntraTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((RobotRaconteur::IntraTransport const *)arg1)->GetUrlSchemeString();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_IntraTransport_GetServerListenUrls(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::IntraTransport *arg1 = (RobotRaconteur::IntraTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::IntraTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::IntraTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__IntraTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "IntraTransport_GetServerListenUrls" "', argument " "1"" of type '" "RobotRaconteur::IntraTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::IntraTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::IntraTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::IntraTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::IntraTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::IntraTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetServerListenUrls();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< std::string,std::allocator< std::string > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_IntraTransport_Close(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::IntraTransport *arg1 = (RobotRaconteur::IntraTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::IntraTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::IntraTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__IntraTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "IntraTransport_Close" "', argument " "1"" of type '" "RobotRaconteur::IntraTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::IntraTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::IntraTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::IntraTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::IntraTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::IntraTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Close();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_IntraTransport_StartClient(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::IntraTransport *arg1 = (RobotRaconteur::IntraTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::IntraTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::IntraTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__IntraTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "IntraTransport_StartClient" "', argument " "1"" of type '" "RobotRaconteur::IntraTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::IntraTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::IntraTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::IntraTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::IntraTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::IntraTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->StartClient();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_IntraTransport_StartServer(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::IntraTransport *arg1 = (RobotRaconteur::IntraTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::IntraTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::IntraTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__IntraTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "IntraTransport_StartServer" "', argument " "1"" of type '" "RobotRaconteur::IntraTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::IntraTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::IntraTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::IntraTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::IntraTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::IntraTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->StartServer();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_IntraTransport(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::IntraTransport *arg1 = (RobotRaconteur::IntraTransport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::IntraTransport > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::IntraTransport > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__IntraTransport_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_IntraTransport" "', argument " "1"" of type '" "RobotRaconteur::IntraTransport *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::IntraTransport > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::IntraTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::IntraTransport * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::IntraTransport > * >(argp1);
      arg1 = const_cast< RobotRaconteur::IntraTransport * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *IntraTransport_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__IntraTransport_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *IntraTransport_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_TimerEvent_stopped_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TimerEvent *arg1 = (RobotRaconteur::TimerEvent *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TimerEvent, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TimerEvent_stopped_get" "', argument " "1"" of type '" "RobotRaconteur::TimerEvent *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TimerEvent * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (bool) ((arg1)->stopped);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TimerEvent_last_expected_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TimerEvent *arg1 = (RobotRaconteur::TimerEvent *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::posix_time::ptime result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TimerEvent, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TimerEvent_last_expected_get" "', argument " "1"" of type '" "RobotRaconteur::TimerEvent *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TimerEvent * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result =  ((arg1)->last_expected);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  {
    try {
      resultobj = PyDateTime_FromDateAndTime((int)(&result)->date().year(), (int)(&result)->date().month(), (int)(&result)->date().day(),
        (int)(&result)->time_of_day().hours(), (int)(&result)->time_of_day().minutes(), (int)(&result)->time_of_day().seconds(),
        boost::numeric_cast<int32_t>(((&result)->time_of_day().fractional_seconds() * boost::numeric_cast<int32_t>(pow(10.0,(9-(&result)->time_of_day().num_fractional_digits())))))/1000);
    }
    catch (std::exception&)
    {
      resultobj = SWIG_Py_Void();
    }
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TimerEvent_last_real_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TimerEvent *arg1 = (RobotRaconteur::TimerEvent *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::posix_time::ptime result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TimerEvent, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TimerEvent_last_real_get" "', argument " "1"" of type '" "RobotRaconteur::TimerEvent *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TimerEvent * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result =  ((arg1)->last_real);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  {
    try {
      resultobj = PyDateTime_FromDateAndTime((int)(&result)->date().year(), (int)(&result)->date().month(), (int)(&result)->date().day(),
        (int)(&result)->time_of_day().hours(), (int)(&result)->time_of_day().minutes(), (int)(&result)->time_of_day().seconds(),
        boost::numeric_cast<int32_t>(((&result)->time_of_day().fractional_seconds() * boost::numeric_cast<int32_t>(pow(10.0,(9-(&result)->time_of_day().num_fractional_digits())))))/1000);
    }
    catch (std::exception&)
    {
      resultobj = SWIG_Py_Void();
    }
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TimerEvent_current_expected_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TimerEvent *arg1 = (RobotRaconteur::TimerEvent *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::posix_time::ptime result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TimerEvent, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TimerEvent_current_expected_get" "', argument " "1"" of type '" "RobotRaconteur::TimerEvent *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TimerEvent * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result =  ((arg1)->current_expected);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  {
    try {
      resultobj = PyDateTime_FromDateAndTime((int)(&result)->date().year(), (int)(&result)->date().month(), (int)(&result)->date().day(),
        (int)(&result)->time_of_day().hours(), (int)(&result)->time_of_day().minutes(), (int)(&result)->time_of_day().seconds(),
        boost::numeric_cast<int32_t>(((&result)->time_of_day().fractional_seconds() * boost::numeric_cast<int32_t>(pow(10.0,(9-(&result)->time_of_day().num_fractional_digits())))))/1000);
    }
    catch (std::exception&)
    {
      resultobj = SWIG_Py_Void();
    }
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TimerEvent_current_real_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TimerEvent *arg1 = (RobotRaconteur::TimerEvent *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::posix_time::ptime result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TimerEvent, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TimerEvent_current_real_get" "', argument " "1"" of type '" "RobotRaconteur::TimerEvent *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TimerEvent * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result =  ((arg1)->current_real);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  {
    try {
      resultobj = PyDateTime_FromDateAndTime((int)(&result)->date().year(), (int)(&result)->date().month(), (int)(&result)->date().day(),
        (int)(&result)->time_of_day().hours(), (int)(&result)->time_of_day().minutes(), (int)(&result)->time_of_day().seconds(),
        boost::numeric_cast<int32_t>(((&result)->time_of_day().fractional_seconds() * boost::numeric_cast<int32_t>(pow(10.0,(9-(&result)->time_of_day().num_fractional_digits())))))/1000);
    }
    catch (std::exception&)
    {
      resultobj = SWIG_Py_Void();
    }
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_TimerEvent(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TimerEvent *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_TimerEvent", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::TimerEvent *)new RobotRaconteur::TimerEvent();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__TimerEvent, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_TimerEvent(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TimerEvent *arg1 = (RobotRaconteur::TimerEvent *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TimerEvent, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_TimerEvent" "', argument " "1"" of type '" "RobotRaconteur::TimerEvent *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TimerEvent * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *TimerEvent_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__TimerEvent, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *TimerEvent_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_AsyncTimerEventReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncTimerEventReturnDirector *arg1 = (RobotRaconteur::AsyncTimerEventReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncTimerEventReturnDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_AsyncTimerEventReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncTimerEventReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncTimerEventReturnDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_AsyncTimerEventReturnDirector_handler(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncTimerEventReturnDirector *arg1 = (RobotRaconteur::AsyncTimerEventReturnDirector *) 0 ;
  RobotRaconteur::TimerEvent *arg2 = 0 ;
  RobotRaconteur::HandlerErrorInfo *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  PyObject *swig_obj[3] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "AsyncTimerEventReturnDirector_handler", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncTimerEventReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AsyncTimerEventReturnDirector_handler" "', argument " "1"" of type '" "RobotRaconteur::AsyncTimerEventReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncTimerEventReturnDirector * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__TimerEvent,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "AsyncTimerEventReturnDirector_handler" "', argument " "2"" of type '" "RobotRaconteur::TimerEvent const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "AsyncTimerEventReturnDirector_handler" "', argument " "2"" of type '" "RobotRaconteur::TimerEvent const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::TimerEvent * >(argp2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "AsyncTimerEventReturnDirector_handler" "', argument " "3"" of type '" "RobotRaconteur::HandlerErrorInfo &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "AsyncTimerEventReturnDirector_handler" "', argument " "3"" of type '" "RobotRaconteur::HandlerErrorInfo &""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp3);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::AsyncTimerEventReturnDirector::handler");
      } else {
        (arg1)->handler((RobotRaconteur::TimerEvent const &)*arg2,*arg3);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_AsyncTimerEventReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::AsyncTimerEventReturnDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::AsyncTimerEventReturnDirector *)new SwigDirector_AsyncTimerEventReturnDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__AsyncTimerEventReturnDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_AsyncTimerEventReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncTimerEventReturnDirector *arg1 = (RobotRaconteur::AsyncTimerEventReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncTimerEventReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_AsyncTimerEventReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncTimerEventReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncTimerEventReturnDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *AsyncTimerEventReturnDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__AsyncTimerEventReturnDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *AsyncTimerEventReturnDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_Timer_Start(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::Timer *arg1 = (RobotRaconteur::Timer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::Timer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::Timer > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Timer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Timer_Start" "', argument " "1"" of type '" "RobotRaconteur::Timer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Timer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Timer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Timer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Timer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Timer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Start();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_Timer_Stop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::Timer *arg1 = (RobotRaconteur::Timer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::Timer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::Timer > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Timer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Timer_Stop" "', argument " "1"" of type '" "RobotRaconteur::Timer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Timer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Timer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Timer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Timer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Timer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Stop();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_Timer_TryStop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::Timer *arg1 = (RobotRaconteur::Timer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::Timer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::Timer > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Timer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Timer_TryStop" "', argument " "1"" of type '" "RobotRaconteur::Timer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Timer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Timer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Timer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Timer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Timer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->TryStop();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_Timer_GetPeriod(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::Timer *arg1 = (RobotRaconteur::Timer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::Timer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::Timer > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::posix_time::time_duration result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Timer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Timer_GetPeriod" "', argument " "1"" of type '" "RobotRaconteur::Timer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Timer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Timer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Timer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Timer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Timer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetPeriod();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  resultobj=PyFloat_FromDouble(boost::lexical_cast<double>((&result)->total_microseconds())/1000000.0);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_Timer_SetPeriod(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::Timer *arg1 = (RobotRaconteur::Timer *) 0 ;
  boost::posix_time::time_duration *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::Timer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::Timer > *smartarg1 = 0 ;
  boost::posix_time::time_duration tmpDur2 ;
  int64_t pyinputval22 = 0 ;
  double pyinputval2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "Timer_SetPeriod", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Timer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Timer_SetPeriod" "', argument " "1"" of type '" "RobotRaconteur::Timer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Timer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Timer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Timer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Timer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Timer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  
  
  if (PyInt_Check(swig_obj[1]))
  {
    pyinputval2=boost::lexical_cast<double>(PyInt_AsLong(swig_obj[1]));
  }
  else if (PyLong_Check(swig_obj[1]))
  {
    pyinputval2=boost::lexical_cast<double>(PyLong_AsLong(swig_obj[1]));
  }
  else if (PyFloat_Check(swig_obj[1]))
  {
    pyinputval2=PyFloat_AsDouble(swig_obj[1]);
  }
  else
  {
    PyErr_SetString(PyExc_TypeError,"Input must be an Integer, Long, or Float");
    return NULL;
  }
  
  pyinputval22=boost::lexical_cast<int64_t>(pyinputval2*1000000.0);
  tmpDur2=boost::posix_time::microseconds(pyinputval22);
  arg2=&tmpDur2;
  
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetPeriod((boost::posix_time::time_duration const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_Timer_IsRunning(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::Timer *arg1 = (RobotRaconteur::Timer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::Timer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::Timer > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Timer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Timer_IsRunning" "', argument " "1"" of type '" "RobotRaconteur::Timer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Timer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Timer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Timer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Timer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Timer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->IsRunning();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_Timer(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::Timer *arg1 = (RobotRaconteur::Timer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::Timer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::Timer > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Timer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Timer" "', argument " "1"" of type '" "RobotRaconteur::Timer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Timer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Timer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Timer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Timer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Timer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *Timer_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Timer_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_WallTimer_Start(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WallTimer *arg1 = (RobotRaconteur::WallTimer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WallTimer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WallTimer > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WallTimer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WallTimer_Start" "', argument " "1"" of type '" "RobotRaconteur::WallTimer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallTimer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallTimer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WallTimer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallTimer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WallTimer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Start();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WallTimer_Stop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WallTimer *arg1 = (RobotRaconteur::WallTimer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WallTimer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WallTimer > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WallTimer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WallTimer_Stop" "', argument " "1"" of type '" "RobotRaconteur::WallTimer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallTimer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallTimer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WallTimer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallTimer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WallTimer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Stop();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WallTimer_TryStop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WallTimer *arg1 = (RobotRaconteur::WallTimer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WallTimer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WallTimer > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WallTimer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WallTimer_TryStop" "', argument " "1"" of type '" "RobotRaconteur::WallTimer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallTimer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallTimer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WallTimer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallTimer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WallTimer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->TryStop();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WallTimer_GetPeriod(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WallTimer *arg1 = (RobotRaconteur::WallTimer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WallTimer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WallTimer > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::posix_time::time_duration result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WallTimer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WallTimer_GetPeriod" "', argument " "1"" of type '" "RobotRaconteur::WallTimer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallTimer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallTimer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WallTimer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallTimer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WallTimer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetPeriod();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  resultobj=PyFloat_FromDouble(boost::lexical_cast<double>((&result)->total_microseconds())/1000000.0);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WallTimer_SetPeriod(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WallTimer *arg1 = (RobotRaconteur::WallTimer *) 0 ;
  boost::posix_time::time_duration *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WallTimer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WallTimer > *smartarg1 = 0 ;
  boost::posix_time::time_duration tmpDur2 ;
  int64_t pyinputval22 = 0 ;
  double pyinputval2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WallTimer_SetPeriod", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WallTimer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WallTimer_SetPeriod" "', argument " "1"" of type '" "RobotRaconteur::WallTimer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallTimer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallTimer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WallTimer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallTimer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WallTimer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  
  
  if (PyInt_Check(swig_obj[1]))
  {
    pyinputval2=boost::lexical_cast<double>(PyInt_AsLong(swig_obj[1]));
  }
  else if (PyLong_Check(swig_obj[1]))
  {
    pyinputval2=boost::lexical_cast<double>(PyLong_AsLong(swig_obj[1]));
  }
  else if (PyFloat_Check(swig_obj[1]))
  {
    pyinputval2=PyFloat_AsDouble(swig_obj[1]);
  }
  else
  {
    PyErr_SetString(PyExc_TypeError,"Input must be an Integer, Long, or Float");
    return NULL;
  }
  
  pyinputval22=boost::lexical_cast<int64_t>(pyinputval2*1000000.0);
  tmpDur2=boost::posix_time::microseconds(pyinputval22);
  arg2=&tmpDur2;
  
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetPeriod((boost::posix_time::time_duration const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WallTimer_IsRunning(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WallTimer *arg1 = (RobotRaconteur::WallTimer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WallTimer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WallTimer > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WallTimer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WallTimer_IsRunning" "', argument " "1"" of type '" "RobotRaconteur::WallTimer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallTimer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallTimer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WallTimer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallTimer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WallTimer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->IsRunning();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WallTimer(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WallTimer *arg1 = (RobotRaconteur::WallTimer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WallTimer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WallTimer > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WallTimer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WallTimer" "', argument " "1"" of type '" "RobotRaconteur::WallTimer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallTimer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallTimer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WallTimer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallTimer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WallTimer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WallTimer(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::posix_time::time_duration *arg1 = 0 ;
  bool arg2 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg3 = 0 ;
  RobotRaconteur::AsyncTimerEventReturnDirector *arg4 = (RobotRaconteur::AsyncTimerEventReturnDirector *) 0 ;
  int32_t arg5 ;
  boost::posix_time::time_duration tmpDur1 ;
  int64_t pyinputval21 = 0 ;
  double pyinputval1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared3 ;
  void *argp4 = 0 ;
  int res4 = 0 ;
  int val5 ;
  int ecode5 = 0 ;
  PyObject *swig_obj[5] ;
  RobotRaconteur::WallTimer *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_WallTimer", 5, 5, swig_obj)) SWIG_fail;
  
  
  
  if (PyInt_Check(swig_obj[0]))
  {
    pyinputval1=boost::lexical_cast<double>(PyInt_AsLong(swig_obj[0]));
  }
  else if (PyLong_Check(swig_obj[0]))
  {
    pyinputval1=boost::lexical_cast<double>(PyLong_AsLong(swig_obj[0]));
  }
  else if (PyFloat_Check(swig_obj[0]))
  {
    pyinputval1=PyFloat_AsDouble(swig_obj[0]);
  }
  else
  {
    PyErr_SetString(PyExc_TypeError,"Input must be an Integer, Long, or Float");
    return NULL;
  }
  
  pyinputval21=boost::lexical_cast<int64_t>(pyinputval1*1000000.0);
  tmpDur1=boost::posix_time::microseconds(pyinputval21);
  arg1=&tmpDur1;
  
  
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "new_WallTimer" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "new_WallTimer" "', argument " "3"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp3);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp3) : &tempshared3;
    }
  }
  res4 = SWIG_ConvertPtr(swig_obj[3], &argp4,SWIGTYPE_p_RobotRaconteur__AsyncTimerEventReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "new_WallTimer" "', argument " "4"" of type '" "RobotRaconteur::AsyncTimerEventReturnDirector *""'"); 
  }
  arg4 = reinterpret_cast< RobotRaconteur::AsyncTimerEventReturnDirector * >(argp4);
  ecode5 = SWIG_AsVal_int(swig_obj[4], &val5);
  if (!SWIG_IsOK(ecode5)) {
    SWIG_exception_fail(SWIG_ArgError(ecode5), "in method '" "new_WallTimer" "', argument " "5"" of type '" "int32_t""'");
  } 
  arg5 = static_cast< int32_t >(val5);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WallTimer *)new_RobotRaconteur_WallTimer((boost::posix_time::time_duration const &)*arg1,arg2,(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg3,arg4,arg5);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WallTimer > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WallTimer >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WallTimer_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WallTimer_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WallTimer_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WallTimer_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_Rate_Sleep(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::Rate *arg1 = (RobotRaconteur::Rate *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::Rate > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::Rate > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Rate_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Rate_Sleep" "', argument " "1"" of type '" "RobotRaconteur::Rate *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Rate > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Rate > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Rate * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Rate > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Rate * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Sleep();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_Rate(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::Rate *arg1 = (RobotRaconteur::Rate *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::Rate > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::Rate > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Rate_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Rate" "', argument " "1"" of type '" "RobotRaconteur::Rate *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Rate > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Rate > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Rate * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::Rate > * >(argp1);
      arg1 = const_cast< RobotRaconteur::Rate * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    (void)arg1; delete smartarg1;
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *Rate_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Rate_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_new_WallRate__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  double arg1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > arg2 ;
  double val1 ;
  int ecode1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  RobotRaconteur::WallRate *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_double(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_WallRate" "', argument " "1"" of type '" "double""'");
  } 
  arg1 = static_cast< double >(val1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_WallRate" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode >""'");
    }
    if (argp2) arg2 = *(reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp2));
    if (newmem & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp2);
  }
  
  try {
    result = (RobotRaconteur::WallRate *)new RobotRaconteur::WallRate(arg1,arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WallRate > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WallRate >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WallRate_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WallRate__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  double arg1 ;
  double val1 ;
  int ecode1 = 0 ;
  RobotRaconteur::WallRate *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_double(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_WallRate" "', argument " "1"" of type '" "double""'");
  } 
  arg1 = static_cast< double >(val1);
  
  try {
    result = (RobotRaconteur::WallRate *)new RobotRaconteur::WallRate(arg1);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WallRate > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WallRate >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WallRate_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WallRate(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_WallRate", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_double(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_WallRate__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_double(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_WallRate__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_WallRate'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WallRate::WallRate(double,boost::shared_ptr< RobotRaconteur::RobotRaconteurNode >)\n"
    "    RobotRaconteur::WallRate::WallRate(double)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WallRate_Sleep(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WallRate *arg1 = (RobotRaconteur::WallRate *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WallRate > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WallRate > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WallRate_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WallRate_Sleep" "', argument " "1"" of type '" "RobotRaconteur::WallRate *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallRate > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallRate > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WallRate * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallRate > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WallRate * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Sleep();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WallRate(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WallRate *arg1 = (RobotRaconteur::WallRate *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WallRate > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WallRate > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WallRate_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WallRate" "', argument " "1"" of type '" "RobotRaconteur::WallRate *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallRate > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallRate > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WallRate * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WallRate > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WallRate * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    (void)arg1; delete smartarg1;
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WallRate_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WallRate_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WallRate_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_new_AutoResetEvent(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AutoResetEvent *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_AutoResetEvent", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::AutoResetEvent *)new RobotRaconteur::AutoResetEvent();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::AutoResetEvent > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::AutoResetEvent >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__AutoResetEvent_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_AutoResetEvent(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AutoResetEvent *arg1 = (RobotRaconteur::AutoResetEvent *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::AutoResetEvent > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::AutoResetEvent > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__AutoResetEvent_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_AutoResetEvent" "', argument " "1"" of type '" "RobotRaconteur::AutoResetEvent *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AutoResetEvent > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AutoResetEvent > * >(argp1);
      arg1 = const_cast< RobotRaconteur::AutoResetEvent * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AutoResetEvent > * >(argp1);
      arg1 = const_cast< RobotRaconteur::AutoResetEvent * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_AutoResetEvent_Set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AutoResetEvent *arg1 = (RobotRaconteur::AutoResetEvent *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::AutoResetEvent > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::AutoResetEvent > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__AutoResetEvent_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AutoResetEvent_Set" "', argument " "1"" of type '" "RobotRaconteur::AutoResetEvent *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AutoResetEvent > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AutoResetEvent > * >(argp1);
      arg1 = const_cast< RobotRaconteur::AutoResetEvent * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AutoResetEvent > * >(argp1);
      arg1 = const_cast< RobotRaconteur::AutoResetEvent * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Set();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_AutoResetEvent_Reset(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AutoResetEvent *arg1 = (RobotRaconteur::AutoResetEvent *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::AutoResetEvent > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::AutoResetEvent > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__AutoResetEvent_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AutoResetEvent_Reset" "', argument " "1"" of type '" "RobotRaconteur::AutoResetEvent *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AutoResetEvent > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AutoResetEvent > * >(argp1);
      arg1 = const_cast< RobotRaconteur::AutoResetEvent * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AutoResetEvent > * >(argp1);
      arg1 = const_cast< RobotRaconteur::AutoResetEvent * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Reset();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_AutoResetEvent_WaitOne__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::AutoResetEvent *arg1 = (RobotRaconteur::AutoResetEvent *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::AutoResetEvent > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::AutoResetEvent > *smartarg1 = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__AutoResetEvent_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AutoResetEvent_WaitOne" "', argument " "1"" of type '" "RobotRaconteur::AutoResetEvent *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AutoResetEvent > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AutoResetEvent > * >(argp1);
      arg1 = const_cast< RobotRaconteur::AutoResetEvent * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AutoResetEvent > * >(argp1);
      arg1 = const_cast< RobotRaconteur::AutoResetEvent * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->WaitOne();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_AutoResetEvent_WaitOne__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::AutoResetEvent *arg1 = (RobotRaconteur::AutoResetEvent *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::AutoResetEvent > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::AutoResetEvent > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  bool result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__AutoResetEvent_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AutoResetEvent_WaitOne" "', argument " "1"" of type '" "RobotRaconteur::AutoResetEvent *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AutoResetEvent > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AutoResetEvent > * >(argp1);
      arg1 = const_cast< RobotRaconteur::AutoResetEvent * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AutoResetEvent > * >(argp1);
      arg1 = const_cast< RobotRaconteur::AutoResetEvent * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "AutoResetEvent_WaitOne" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->WaitOne(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_AutoResetEvent_WaitOne(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "AutoResetEvent_WaitOne", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__AutoResetEvent_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_AutoResetEvent_WaitOne__SWIG_0(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__AutoResetEvent_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_AutoResetEvent_WaitOne__SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'AutoResetEvent_WaitOne'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::AutoResetEvent::WaitOne()\n"
    "    RobotRaconteur::AutoResetEvent::WaitOne(int32_t)\n");
  return 0;
}


SWIGINTERN PyObject *AutoResetEvent_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__AutoResetEvent_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *AutoResetEvent_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_TimeSpec_seconds_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TimeSpec *arg1 = (RobotRaconteur::TimeSpec *) 0 ;
  int64_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  long val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TimeSpec_seconds_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TimeSpec, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TimeSpec_seconds_set" "', argument " "1"" of type '" "RobotRaconteur::TimeSpec *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp1);
  ecode2 = SWIG_AsVal_long(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TimeSpec_seconds_set" "', argument " "2"" of type '" "int64_t""'");
  } 
  arg2 = static_cast< int64_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->seconds = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TimeSpec_seconds_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TimeSpec *arg1 = (RobotRaconteur::TimeSpec *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  int64_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TimeSpec, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TimeSpec_seconds_get" "', argument " "1"" of type '" "RobotRaconteur::TimeSpec *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (int64_t) ((arg1)->seconds);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_long(static_cast< long >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TimeSpec_nanoseconds_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TimeSpec *arg1 = (RobotRaconteur::TimeSpec *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TimeSpec_nanoseconds_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TimeSpec, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TimeSpec_nanoseconds_set" "', argument " "1"" of type '" "RobotRaconteur::TimeSpec *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TimeSpec_nanoseconds_set" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->nanoseconds = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TimeSpec_nanoseconds_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TimeSpec *arg1 = (RobotRaconteur::TimeSpec *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TimeSpec, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TimeSpec_nanoseconds_get" "', argument " "1"" of type '" "RobotRaconteur::TimeSpec *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (int32_t) ((arg1)->nanoseconds);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_TimeSpec__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  RobotRaconteur::TimeSpec *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::TimeSpec *)new RobotRaconteur::TimeSpec();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__TimeSpec, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_TimeSpec__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  int64_t arg1 ;
  int32_t arg2 ;
  long val1 ;
  int ecode1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  RobotRaconteur::TimeSpec *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_long(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_TimeSpec" "', argument " "1"" of type '" "int64_t""'");
  } 
  arg1 = static_cast< int64_t >(val1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "new_TimeSpec" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::TimeSpec *)new RobotRaconteur::TimeSpec(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__TimeSpec, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_TimeSpec(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_TimeSpec", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_TimeSpec__SWIG_0(self, argc, argv);
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_long(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_new_TimeSpec__SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_TimeSpec'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::TimeSpec::TimeSpec()\n"
    "    RobotRaconteur::TimeSpec::TimeSpec(int64_t,int32_t)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_TimeSpec___eq__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TimeSpec *arg1 = (RobotRaconteur::TimeSpec *) 0 ;
  RobotRaconteur::TimeSpec *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "TimeSpec___eq__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TimeSpec, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TimeSpec___eq__" "', argument " "1"" of type '" "RobotRaconteur::TimeSpec *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__TimeSpec,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "TimeSpec___eq__" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "TimeSpec___eq__" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->operator ==((RobotRaconteur::TimeSpec const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  PyErr_Clear();
  Py_INCREF(Py_NotImplemented);
  return Py_NotImplemented;
}


SWIGINTERN PyObject *_wrap_TimeSpec___ne__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TimeSpec *arg1 = (RobotRaconteur::TimeSpec *) 0 ;
  RobotRaconteur::TimeSpec *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "TimeSpec___ne__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TimeSpec, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TimeSpec___ne__" "', argument " "1"" of type '" "RobotRaconteur::TimeSpec *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__TimeSpec,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "TimeSpec___ne__" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "TimeSpec___ne__" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->operator !=((RobotRaconteur::TimeSpec const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  PyErr_Clear();
  Py_INCREF(Py_NotImplemented);
  return Py_NotImplemented;
}


SWIGINTERN PyObject *_wrap_TimeSpec___sub__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TimeSpec *arg1 = (RobotRaconteur::TimeSpec *) 0 ;
  RobotRaconteur::TimeSpec *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  RobotRaconteur::TimeSpec result;
  
  if (!SWIG_Python_UnpackTuple(args, "TimeSpec___sub__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TimeSpec, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TimeSpec___sub__" "', argument " "1"" of type '" "RobotRaconteur::TimeSpec *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__TimeSpec,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "TimeSpec___sub__" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "TimeSpec___sub__" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->operator -((RobotRaconteur::TimeSpec const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::TimeSpec(static_cast< const RobotRaconteur::TimeSpec& >(result))), SWIGTYPE_p_RobotRaconteur__TimeSpec, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  PyErr_Clear();
  Py_INCREF(Py_NotImplemented);
  return Py_NotImplemented;
}


SWIGINTERN PyObject *_wrap_TimeSpec___add__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TimeSpec *arg1 = (RobotRaconteur::TimeSpec *) 0 ;
  RobotRaconteur::TimeSpec *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  RobotRaconteur::TimeSpec result;
  
  if (!SWIG_Python_UnpackTuple(args, "TimeSpec___add__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TimeSpec, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TimeSpec___add__" "', argument " "1"" of type '" "RobotRaconteur::TimeSpec *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__TimeSpec,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "TimeSpec___add__" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "TimeSpec___add__" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->operator +((RobotRaconteur::TimeSpec const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::TimeSpec(static_cast< const RobotRaconteur::TimeSpec& >(result))), SWIGTYPE_p_RobotRaconteur__TimeSpec, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  PyErr_Clear();
  Py_INCREF(Py_NotImplemented);
  return Py_NotImplemented;
}


SWIGINTERN PyObject *_wrap_TimeSpec___gt__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TimeSpec *arg1 = (RobotRaconteur::TimeSpec *) 0 ;
  RobotRaconteur::TimeSpec *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "TimeSpec___gt__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TimeSpec, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TimeSpec___gt__" "', argument " "1"" of type '" "RobotRaconteur::TimeSpec *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__TimeSpec,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "TimeSpec___gt__" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "TimeSpec___gt__" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->operator >((RobotRaconteur::TimeSpec const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  PyErr_Clear();
  Py_INCREF(Py_NotImplemented);
  return Py_NotImplemented;
}


SWIGINTERN PyObject *_wrap_TimeSpec___ge__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TimeSpec *arg1 = (RobotRaconteur::TimeSpec *) 0 ;
  RobotRaconteur::TimeSpec *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "TimeSpec___ge__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TimeSpec, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TimeSpec___ge__" "', argument " "1"" of type '" "RobotRaconteur::TimeSpec *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__TimeSpec,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "TimeSpec___ge__" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "TimeSpec___ge__" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->operator >=((RobotRaconteur::TimeSpec const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  PyErr_Clear();
  Py_INCREF(Py_NotImplemented);
  return Py_NotImplemented;
}


SWIGINTERN PyObject *_wrap_TimeSpec___lt__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TimeSpec *arg1 = (RobotRaconteur::TimeSpec *) 0 ;
  RobotRaconteur::TimeSpec *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "TimeSpec___lt__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TimeSpec, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TimeSpec___lt__" "', argument " "1"" of type '" "RobotRaconteur::TimeSpec *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__TimeSpec,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "TimeSpec___lt__" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "TimeSpec___lt__" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->operator <((RobotRaconteur::TimeSpec const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  PyErr_Clear();
  Py_INCREF(Py_NotImplemented);
  return Py_NotImplemented;
}


SWIGINTERN PyObject *_wrap_TimeSpec___le__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TimeSpec *arg1 = (RobotRaconteur::TimeSpec *) 0 ;
  RobotRaconteur::TimeSpec *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "TimeSpec___le__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TimeSpec, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TimeSpec___le__" "', argument " "1"" of type '" "RobotRaconteur::TimeSpec *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__TimeSpec,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "TimeSpec___le__" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "TimeSpec___le__" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->operator <=((RobotRaconteur::TimeSpec const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  PyErr_Clear();
  Py_INCREF(Py_NotImplemented);
  return Py_NotImplemented;
}


SWIGINTERN PyObject *_wrap_TimeSpec_cleanup_nanosecs(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TimeSpec *arg1 = (RobotRaconteur::TimeSpec *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TimeSpec, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TimeSpec_cleanup_nanosecs" "', argument " "1"" of type '" "RobotRaconteur::TimeSpec *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->cleanup_nanosecs();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_TimeSpec(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TimeSpec *arg1 = (RobotRaconteur::TimeSpec *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TimeSpec, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_TimeSpec" "', argument " "1"" of type '" "RobotRaconteur::TimeSpec *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *TimeSpec_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__TimeSpec, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *TimeSpec_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedService_typed_packet_packet_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedService_typed_packet *arg1 = (RobotRaconteur::WrappedService_typed_packet *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedService_typed_packet_packet_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedService_typed_packet_packet_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedService_typed_packet *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedService_typed_packet * >(argp1);
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedService_typed_packet_packet_set" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->packet = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedService_typed_packet_packet_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedService_typed_packet *arg1 = (RobotRaconteur::WrappedService_typed_packet *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedService_typed_packet_packet_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedService_typed_packet *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedService_typed_packet * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::intrusive_ptr< RobotRaconteur::MessageElement > *) & ((arg1)->packet);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  
  //if (*result) {
  //  intrusive_ptr_add_ref(result->get());
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result->get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(result)) {
    intrusive_ptr_add_ref(result->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >(result->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedService_typed_packet_type_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedService_typed_packet *arg1 = (RobotRaconteur::WrappedService_typed_packet *) 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedService_typed_packet_type_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedService_typed_packet_type_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedService_typed_packet *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedService_typed_packet * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedService_typed_packet_type_set" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::TypeDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2) : &tempshared2;
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->type = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedService_typed_packet_type_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedService_typed_packet *arg1 = (RobotRaconteur::WrappedService_typed_packet *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedService_typed_packet_type_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedService_typed_packet *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedService_typed_packet * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::shared_ptr< RobotRaconteur::TypeDefinition > *) & ((arg1)->type);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  {
    boost::shared_ptr<  RobotRaconteur::TypeDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::TypeDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedService_typed_packet_stub_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedService_typed_packet *arg1 = (RobotRaconteur::WrappedService_typed_packet *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedService_typed_packet_stub_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedService_typed_packet_stub_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedService_typed_packet *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedService_typed_packet * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedService_typed_packet_stub_set" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->stub = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedService_typed_packet_stub_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedService_typed_packet *arg1 = (RobotRaconteur::WrappedService_typed_packet *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedService_typed_packet_stub_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedService_typed_packet *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedService_typed_packet * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *) & ((arg1)->stub);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedService_typed_packet_client_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedService_typed_packet *arg1 = (RobotRaconteur::WrappedService_typed_packet *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedService_typed_packet_client_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedService_typed_packet_client_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedService_typed_packet *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedService_typed_packet * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedService_typed_packet_client_set" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->client = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedService_typed_packet_client_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedService_typed_packet *arg1 = (RobotRaconteur::WrappedService_typed_packet *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedService_typed_packet_client_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedService_typed_packet *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedService_typed_packet * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint32_t) ((arg1)->client);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedService_typed_packet(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedService_typed_packet *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_WrappedService_typed_packet", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedService_typed_packet *)new RobotRaconteur::WrappedService_typed_packet();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedService_typed_packet(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedService_typed_packet *arg1 = (RobotRaconteur::WrappedService_typed_packet *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedService_typed_packet" "', argument " "1"" of type '" "RobotRaconteur::WrappedService_typed_packet *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedService_typed_packet * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedService_typed_packet_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedService_typed_packet_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_WrappedPipeEndpointDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpointDirector *arg1 = (RobotRaconteur::WrappedPipeEndpointDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPipeEndpointDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedPipeEndpointDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpointDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPipeEndpointDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpointDirector_PipeEndpointClosedCallback(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpointDirector *arg1 = (RobotRaconteur::WrappedPipeEndpointDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPipeEndpointDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpointDirector_PipeEndpointClosedCallback" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpointDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPipeEndpointDirector * >(argp1);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedPipeEndpointDirector::PipeEndpointClosedCallback");
      } else {
        (arg1)->PipeEndpointClosedCallback();
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpointDirector_PacketReceivedEvent(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpointDirector *arg1 = (RobotRaconteur::WrappedPipeEndpointDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPipeEndpointDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpointDirector_PacketReceivedEvent" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpointDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPipeEndpointDirector * >(argp1);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedPipeEndpointDirector::PacketReceivedEvent");
      } else {
        (arg1)->PacketReceivedEvent();
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpointDirector_PacketAckReceivedEvent(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpointDirector *arg1 = (RobotRaconteur::WrappedPipeEndpointDirector *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeEndpointDirector_PacketAckReceivedEvent", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPipeEndpointDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpointDirector_PacketAckReceivedEvent" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpointDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPipeEndpointDirector * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedPipeEndpointDirector_PacketAckReceivedEvent" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedPipeEndpointDirector::PacketAckReceivedEvent");
      } else {
        (arg1)->PacketAckReceivedEvent(arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedPipeEndpointDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedPipeEndpointDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedPipeEndpointDirector *)new SwigDirector_WrappedPipeEndpointDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedPipeEndpointDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedPipeEndpointDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpointDirector *arg1 = (RobotRaconteur::WrappedPipeEndpointDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPipeEndpointDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedPipeEndpointDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpointDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPipeEndpointDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedPipeEndpointDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedPipeEndpointDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedPipeEndpointDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_AsyncPipeEndpointReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncPipeEndpointReturnDirector *arg1 = (RobotRaconteur::AsyncPipeEndpointReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncPipeEndpointReturnDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_AsyncPipeEndpointReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncPipeEndpointReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncPipeEndpointReturnDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_AsyncPipeEndpointReturnDirector_handler(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncPipeEndpointReturnDirector *arg1 = (RobotRaconteur::AsyncPipeEndpointReturnDirector *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *arg2 = 0 ;
  RobotRaconteur::HandlerErrorInfo *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared2 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  PyObject *swig_obj[3] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "AsyncPipeEndpointReturnDirector_handler", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncPipeEndpointReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AsyncPipeEndpointReturnDirector_handler" "', argument " "1"" of type '" "RobotRaconteur::AsyncPipeEndpointReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncPipeEndpointReturnDirector * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "AsyncPipeEndpointReturnDirector_handler" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > * >(argp2) : &tempshared2;
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "AsyncPipeEndpointReturnDirector_handler" "', argument " "3"" of type '" "RobotRaconteur::HandlerErrorInfo &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "AsyncPipeEndpointReturnDirector_handler" "', argument " "3"" of type '" "RobotRaconteur::HandlerErrorInfo &""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp3);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::AsyncPipeEndpointReturnDirector::handler");
      } else {
        (arg1)->handler((boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > const &)*arg2,*arg3);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_AsyncPipeEndpointReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::AsyncPipeEndpointReturnDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::AsyncPipeEndpointReturnDirector *)new SwigDirector_AsyncPipeEndpointReturnDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__AsyncPipeEndpointReturnDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_AsyncPipeEndpointReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncPipeEndpointReturnDirector *arg1 = (RobotRaconteur::AsyncPipeEndpointReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncPipeEndpointReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_AsyncPipeEndpointReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncPipeEndpointReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncPipeEndpointReturnDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *AsyncPipeEndpointReturnDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__AsyncPipeEndpointReturnDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *AsyncPipeEndpointReturnDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedTryReceivePacketWaitResult_res_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedTryReceivePacketWaitResult *arg1 = (RobotRaconteur::WrappedTryReceivePacketWaitResult *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedTryReceivePacketWaitResult_res_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedTryReceivePacketWaitResult, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedTryReceivePacketWaitResult_res_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedTryReceivePacketWaitResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedTryReceivePacketWaitResult * >(argp1);
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedTryReceivePacketWaitResult_res_set" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->res = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedTryReceivePacketWaitResult_res_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedTryReceivePacketWaitResult *arg1 = (RobotRaconteur::WrappedTryReceivePacketWaitResult *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedTryReceivePacketWaitResult, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedTryReceivePacketWaitResult_res_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedTryReceivePacketWaitResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedTryReceivePacketWaitResult * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (bool) ((arg1)->res);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedTryReceivePacketWaitResult_packet_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedTryReceivePacketWaitResult *arg1 = (RobotRaconteur::WrappedTryReceivePacketWaitResult *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedTryReceivePacketWaitResult_packet_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedTryReceivePacketWaitResult, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedTryReceivePacketWaitResult_packet_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedTryReceivePacketWaitResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedTryReceivePacketWaitResult * >(argp1);
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedTryReceivePacketWaitResult_packet_set" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->packet = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedTryReceivePacketWaitResult_packet_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedTryReceivePacketWaitResult *arg1 = (RobotRaconteur::WrappedTryReceivePacketWaitResult *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedTryReceivePacketWaitResult, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedTryReceivePacketWaitResult_packet_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedTryReceivePacketWaitResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedTryReceivePacketWaitResult * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::intrusive_ptr< RobotRaconteur::MessageElement > *) & ((arg1)->packet);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  
  //if (*result) {
  //  intrusive_ptr_add_ref(result->get());
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result->get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(result)) {
    intrusive_ptr_add_ref(result->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >(result->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedTryReceivePacketWaitResult(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedTryReceivePacketWaitResult *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_WrappedTryReceivePacketWaitResult", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedTryReceivePacketWaitResult *)new RobotRaconteur::WrappedTryReceivePacketWaitResult();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedTryReceivePacketWaitResult, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedTryReceivePacketWaitResult(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedTryReceivePacketWaitResult *arg1 = (RobotRaconteur::WrappedTryReceivePacketWaitResult *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedTryReceivePacketWaitResult, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedTryReceivePacketWaitResult" "', argument " "1"" of type '" "RobotRaconteur::WrappedTryReceivePacketWaitResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedTryReceivePacketWaitResult * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedTryReceivePacketWaitResult_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedTryReceivePacketWaitResult, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedTryReceivePacketWaitResult_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_SendPacket(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  uint32_t result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeEndpoint_SendPacket", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_SendPacket" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPipeEndpoint_SendPacket" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (uint32_t)(arg1)->SendPacket((boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_ReceivePacket(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_ReceivePacket" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->ReceivePacket();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_PeekNextPacket(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_PeekNextPacket" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->PeekNextPacket();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_ReceivePacketWait__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_ReceivePacketWait" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedPipeEndpoint_ReceivePacketWait" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ReceivePacketWait(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_ReceivePacketWait__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_ReceivePacketWait" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ReceivePacketWait();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_ReceivePacketWait(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedPipeEndpoint_ReceivePacketWait", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_WrappedPipeEndpoint_ReceivePacketWait__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_WrappedPipeEndpoint_ReceivePacketWait__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedPipeEndpoint_ReceivePacketWait'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedPipeEndpoint::ReceivePacketWait(int32_t)\n"
    "    RobotRaconteur::WrappedPipeEndpoint::ReceivePacketWait()\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_PeekNextPacketWait__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_PeekNextPacketWait" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedPipeEndpoint_PeekNextPacketWait" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->PeekNextPacketWait(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_PeekNextPacketWait__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_PeekNextPacketWait" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->PeekNextPacketWait();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_PeekNextPacketWait(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedPipeEndpoint_PeekNextPacketWait", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_WrappedPipeEndpoint_PeekNextPacketWait__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_WrappedPipeEndpoint_PeekNextPacketWait__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedPipeEndpoint_PeekNextPacketWait'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedPipeEndpoint::PeekNextPacketWait(int32_t)\n"
    "    RobotRaconteur::WrappedPipeEndpoint::PeekNextPacketWait()\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_TryReceivePacketWait__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  int32_t arg2 ;
  bool arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  bool val3 ;
  int ecode3 = 0 ;
  RobotRaconteur::WrappedTryReceivePacketWaitResult result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_TryReceivePacketWait" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedPipeEndpoint_TryReceivePacketWait" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  ecode3 = SWIG_AsVal_bool(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedPipeEndpoint_TryReceivePacketWait" "', argument " "3"" of type '" "bool""'");
  } 
  arg3 = static_cast< bool >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->TryReceivePacketWait(arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::WrappedTryReceivePacketWaitResult(static_cast< const RobotRaconteur::WrappedTryReceivePacketWaitResult& >(result))), SWIGTYPE_p_RobotRaconteur__WrappedTryReceivePacketWaitResult, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_TryReceivePacketWait__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  RobotRaconteur::WrappedTryReceivePacketWaitResult result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_TryReceivePacketWait" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedPipeEndpoint_TryReceivePacketWait" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->TryReceivePacketWait(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::WrappedTryReceivePacketWaitResult(static_cast< const RobotRaconteur::WrappedTryReceivePacketWaitResult& >(result))), SWIGTYPE_p_RobotRaconteur__WrappedTryReceivePacketWaitResult, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_TryReceivePacketWait__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  RobotRaconteur::WrappedTryReceivePacketWaitResult result;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_TryReceivePacketWait" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->TryReceivePacketWait();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::WrappedTryReceivePacketWaitResult(static_cast< const RobotRaconteur::WrappedTryReceivePacketWaitResult& >(result))), SWIGTYPE_p_RobotRaconteur__WrappedTryReceivePacketWaitResult, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_TryReceivePacketWait(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedPipeEndpoint_TryReceivePacketWait", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_WrappedPipeEndpoint_TryReceivePacketWait__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_WrappedPipeEndpoint_TryReceivePacketWait__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_bool(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_WrappedPipeEndpoint_TryReceivePacketWait__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedPipeEndpoint_TryReceivePacketWait'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedPipeEndpoint::TryReceivePacketWait(int32_t,bool)\n"
    "    RobotRaconteur::WrappedPipeEndpoint::TryReceivePacketWait(int32_t)\n"
    "    RobotRaconteur::WrappedPipeEndpoint::TryReceivePacketWait()\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_Close(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_Close" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Close();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_GetIndex(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_GetIndex" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (int32_t)(arg1)->GetIndex();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_GetEndpoint(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_GetEndpoint" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (uint32_t)(arg1)->GetEndpoint();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_Available(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  size_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_Available" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->Available();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_GetRequestPacketAck(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_GetRequestPacketAck" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (bool)(arg1)->GetRequestPacketAck();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_SetRequestPacketAck(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeEndpoint_SetRequestPacketAck", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_SetRequestPacketAck" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedPipeEndpoint_SetRequestPacketAck" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  
  try {
    (arg1)->SetRequestPacketAck(arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_Type_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeEndpoint_Type_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_Type_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPipeEndpoint_Type_set" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::TypeDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2) : &tempshared2;
    }
  }
  if (arg1) (arg1)->Type = *arg2;
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_Type_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_Type_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  result = (boost::shared_ptr< RobotRaconteur::TypeDefinition > *) & ((arg1)->Type);
  {
    boost::shared_ptr<  RobotRaconteur::TypeDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::TypeDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_SetRRDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  RobotRaconteur::WrappedPipeEndpointDirector *arg2 = (RobotRaconteur::WrappedPipeEndpointDirector *) 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeEndpoint_SetRRDirector", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_SetRRDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__WrappedPipeEndpointDirector, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPipeEndpoint_SetRRDirector" "', argument " "2"" of type '" "RobotRaconteur::WrappedPipeEndpointDirector *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedPipeEndpointDirector * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedPipeEndpoint_SetRRDirector" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    (arg1)->SetRRDirector(arg2,arg3);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_IsUnreliable(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_IsUnreliable" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (bool)(arg1)->IsUnreliable();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_Direction(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MemberDefinition_Direction result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_Direction" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (RobotRaconteur::MemberDefinition_Direction)(arg1)->Direction();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_AsyncSendPacket(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg2 = 0 ;
  RobotRaconteur::AsyncUInt32ReturnDirector *arg3 = (RobotRaconteur::AsyncUInt32ReturnDirector *) 0 ;
  int32_t arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp2 ;
  int newmem2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  int val4 ;
  int ecode4 = 0 ;
  PyObject *swig_obj[4] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeEndpoint_AsyncSendPacket", 4, 4, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_AsyncSendPacket" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPipeEndpoint_AsyncSendPacket" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__AsyncUInt32ReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedPipeEndpoint_AsyncSendPacket" "', argument " "3"" of type '" "RobotRaconteur::AsyncUInt32ReturnDirector *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::AsyncUInt32ReturnDirector * >(argp3);
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedPipeEndpoint_AsyncSendPacket" "', argument " "4"" of type '" "int32_t""'");
  } 
  arg4 = static_cast< int32_t >(val4);
  
  try {
    (arg1)->AsyncSendPacket((boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg2,arg3,arg4);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_AsyncClose(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  int32_t arg2 ;
  RobotRaconteur::AsyncVoidReturnDirector *arg3 = (RobotRaconteur::AsyncVoidReturnDirector *) 0 ;
  int32_t arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  int val4 ;
  int ecode4 = 0 ;
  PyObject *swig_obj[4] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeEndpoint_AsyncClose", 4, 4, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_AsyncClose" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedPipeEndpoint_AsyncClose" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__AsyncVoidReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedPipeEndpoint_AsyncClose" "', argument " "3"" of type '" "RobotRaconteur::AsyncVoidReturnDirector *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::AsyncVoidReturnDirector * >(argp3);
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedPipeEndpoint_AsyncClose" "', argument " "4"" of type '" "int32_t""'");
  } 
  arg4 = static_cast< int32_t >(val4);
  
  try {
    (arg1)->AsyncClose(arg2,arg3,arg4);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_GetNode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_GetNode" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->GetNode();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_GetIgnoreReceived(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_GetIgnoreReceived" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (bool)(arg1)->GetIgnoreReceived();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeEndpoint_SetIgnoreReceived(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeEndpoint_SetIgnoreReceived", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeEndpoint_SetIgnoreReceived" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedPipeEndpoint_SetIgnoreReceived" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  
  try {
    (arg1)->SetIgnoreReceived(arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedPipeEndpoint(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeEndpoint *arg1 = (RobotRaconteur::WrappedPipeEndpoint *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedPipeEndpoint" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    (void)arg1; delete smartarg1;
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedPipeEndpoint_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_WrappedPipeClient_Connect(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeClient *arg1 = (RobotRaconteur::WrappedPipeClient *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeClient > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeClient_Connect", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeClient_Connect" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedPipeClient_Connect" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->Connect(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeClient_AsyncConnect(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeClient *arg1 = (RobotRaconteur::WrappedPipeClient *) 0 ;
  int32_t arg2 ;
  int32_t arg3 ;
  RobotRaconteur::AsyncPipeEndpointReturnDirector *arg4 = (RobotRaconteur::AsyncPipeEndpointReturnDirector *) 0 ;
  int32_t arg5 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeClient > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  void *argp4 = 0 ;
  int res4 = 0 ;
  int val5 ;
  int ecode5 = 0 ;
  PyObject *swig_obj[5] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeClient_AsyncConnect", 5, 5, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeClient_AsyncConnect" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedPipeClient_AsyncConnect" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedPipeClient_AsyncConnect" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  res4 = SWIG_ConvertPtr(swig_obj[3], &argp4,SWIGTYPE_p_RobotRaconteur__AsyncPipeEndpointReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedPipeClient_AsyncConnect" "', argument " "4"" of type '" "RobotRaconteur::AsyncPipeEndpointReturnDirector *""'"); 
  }
  arg4 = reinterpret_cast< RobotRaconteur::AsyncPipeEndpointReturnDirector * >(argp4);
  ecode5 = SWIG_AsVal_int(swig_obj[4], &val5);
  if (!SWIG_IsOK(ecode5)) {
    SWIG_exception_fail(SWIG_ArgError(ecode5), "in method '" "WrappedPipeClient_AsyncConnect" "', argument " "5"" of type '" "int32_t""'");
  } 
  arg5 = static_cast< int32_t >(val5);
  
  try {
    (arg1)->AsyncConnect(arg2,arg3,arg4,arg5);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeClient_GetMemberName(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeClient *arg1 = (RobotRaconteur::WrappedPipeClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeClient_GetMemberName" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->GetMemberName();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeClient_Type_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeClient *arg1 = (RobotRaconteur::WrappedPipeClient *) 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeClient > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeClient_Type_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeClient_Type_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPipeClient_Type_set" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::TypeDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2) : &tempshared2;
    }
  }
  if (arg1) (arg1)->Type = *arg2;
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeClient_Type_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeClient *arg1 = (RobotRaconteur::WrappedPipeClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeClient_Type_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  result = (boost::shared_ptr< RobotRaconteur::TypeDefinition > *) & ((arg1)->Type);
  {
    boost::shared_ptr<  RobotRaconteur::TypeDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::TypeDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeClient_GetNode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeClient *arg1 = (RobotRaconteur::WrappedPipeClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeClient_GetNode" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->GetNode();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeClient_Direction(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeClient *arg1 = (RobotRaconteur::WrappedPipeClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MemberDefinition_Direction result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeClient_Direction" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (RobotRaconteur::MemberDefinition_Direction)(arg1)->Direction();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedPipeClient(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeClient *arg1 = (RobotRaconteur::WrappedPipeClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedPipeClient" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    (void)arg1; delete smartarg1;
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedPipeClient_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeClient_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_delete_WrappedPipeServerConnectDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeServerConnectDirector *arg1 = (RobotRaconteur::WrappedPipeServerConnectDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPipeServerConnectDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedPipeServerConnectDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeServerConnectDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPipeServerConnectDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeServerConnectDirector_PipeConnectCallback(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeServerConnectDirector *arg1 = (RobotRaconteur::WrappedPipeServerConnectDirector *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > tempshared2 ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeServerConnectDirector_PipeConnectCallback", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPipeServerConnectDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeServerConnectDirector_PipeConnectCallback" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeServerConnectDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPipeServerConnectDirector * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPipeServerConnectDirector_PipeConnectCallback" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > * >(argp2) : &tempshared2;
    }
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedPipeServerConnectDirector::PipeConnectCallback");
      } else {
        (arg1)->PipeConnectCallback((boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > const &)*arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedPipeServerConnectDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedPipeServerConnectDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedPipeServerConnectDirector *)new SwigDirector_WrappedPipeServerConnectDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedPipeServerConnectDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedPipeServerConnectDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeServerConnectDirector *arg1 = (RobotRaconteur::WrappedPipeServerConnectDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPipeServerConnectDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedPipeServerConnectDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeServerConnectDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPipeServerConnectDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedPipeServerConnectDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedPipeServerConnectDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedPipeServerConnectDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedPipeServer_GetMemberName(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeServer *arg1 = (RobotRaconteur::WrappedPipeServer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeServer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeServer > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeServer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeServer_GetMemberName" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeServer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeServer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeServer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeServer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetMemberName();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeServer_Type_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeServer *arg1 = (RobotRaconteur::WrappedPipeServer *) 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeServer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeServer > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeServer_Type_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeServer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeServer_Type_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeServer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeServer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeServer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeServer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPipeServer_Type_set" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::TypeDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2) : &tempshared2;
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Type = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeServer_Type_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeServer *arg1 = (RobotRaconteur::WrappedPipeServer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeServer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeServer > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeServer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeServer_Type_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeServer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeServer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeServer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeServer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::shared_ptr< RobotRaconteur::TypeDefinition > *) & ((arg1)->Type);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  {
    boost::shared_ptr<  RobotRaconteur::TypeDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::TypeDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeServer_SetWrappedPipeConnectCallback(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeServer *arg1 = (RobotRaconteur::WrappedPipeServer *) 0 ;
  RobotRaconteur::WrappedPipeServerConnectDirector *arg2 = (RobotRaconteur::WrappedPipeServerConnectDirector *) 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeServer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeServer > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeServer_SetWrappedPipeConnectCallback", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeServer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeServer_SetWrappedPipeConnectCallback" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeServer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeServer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeServer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeServer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__WrappedPipeServerConnectDirector, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPipeServer_SetWrappedPipeConnectCallback" "', argument " "2"" of type '" "RobotRaconteur::WrappedPipeServerConnectDirector *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedPipeServerConnectDirector * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedPipeServer_SetWrappedPipeConnectCallback" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetWrappedPipeConnectCallback(arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeServer_GetNode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeServer *arg1 = (RobotRaconteur::WrappedPipeServer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeServer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeServer > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeServer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeServer_GetNode" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeServer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeServer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeServer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeServer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetNode();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeServer_Direction(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeServer *arg1 = (RobotRaconteur::WrappedPipeServer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeServer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeServer > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MemberDefinition_Direction result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeServer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeServer_Direction" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeServer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeServer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeServer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeServer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::MemberDefinition_Direction)(arg1)->Direction();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedPipeServer(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeServer *arg1 = (RobotRaconteur::WrappedPipeServer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeServer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeServer > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeServer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedPipeServer" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeServer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeServer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeServer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeServer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedPipeServer_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeServer_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_WrappedPipeBroadcasterPredicateDirector_Predicate(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeBroadcasterPredicateDirector *arg1 = (RobotRaconteur::WrappedPipeBroadcasterPredicateDirector *) 0 ;
  uint32_t arg2 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  Swig::Director *director = 0;
  bool upcall = false;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeBroadcasterPredicateDirector_Predicate", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPipeBroadcasterPredicateDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeBroadcasterPredicateDirector_Predicate" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeBroadcasterPredicateDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPipeBroadcasterPredicateDirector * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedPipeBroadcasterPredicateDirector_Predicate" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedPipeBroadcasterPredicateDirector_Predicate" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedPipeBroadcasterPredicateDirector::Predicate");
      } else {
        result = (bool)(arg1)->Predicate(arg2,arg3);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedPipeBroadcasterPredicateDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeBroadcasterPredicateDirector *arg1 = (RobotRaconteur::WrappedPipeBroadcasterPredicateDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPipeBroadcasterPredicateDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedPipeBroadcasterPredicateDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeBroadcasterPredicateDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPipeBroadcasterPredicateDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedPipeBroadcasterPredicateDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedPipeBroadcasterPredicateDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedPipeBroadcasterPredicateDirector *)new SwigDirector_WrappedPipeBroadcasterPredicateDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedPipeBroadcasterPredicateDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedPipeBroadcasterPredicateDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeBroadcasterPredicateDirector *arg1 = (RobotRaconteur::WrappedPipeBroadcasterPredicateDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPipeBroadcasterPredicateDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedPipeBroadcasterPredicateDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeBroadcasterPredicateDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPipeBroadcasterPredicateDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedPipeBroadcasterPredicateDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedPipeBroadcasterPredicateDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedPipeBroadcasterPredicateDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedPipeBroadcaster_Init__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeBroadcaster *arg1 = (RobotRaconteur::WrappedPipeBroadcaster *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeServer > *arg2 = 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeServer > tempshared2 ;
  int val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeBroadcaster_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeBroadcaster_Init" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeBroadcaster *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeBroadcaster * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeBroadcaster * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeServer_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPipeBroadcaster_Init" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedPipeServer > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedPipeServer > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedPipeServer > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedPipeServer > * >(argp2) : &tempshared2;
    }
  }
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedPipeBroadcaster_Init" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Init((boost::shared_ptr< RobotRaconteur::WrappedPipeServer > const &)*arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeBroadcaster_Init__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeBroadcaster *arg1 = (RobotRaconteur::WrappedPipeBroadcaster *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeServer > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeServer > tempshared2 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeBroadcaster_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeBroadcaster_Init" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeBroadcaster *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeBroadcaster * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeBroadcaster * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeServer_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPipeBroadcaster_Init" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedPipeServer > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedPipeServer > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedPipeServer > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedPipeServer > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Init((boost::shared_ptr< RobotRaconteur::WrappedPipeServer > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeBroadcaster_Init(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedPipeBroadcaster_Init", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeBroadcaster_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeServer_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_WrappedPipeBroadcaster_Init__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeBroadcaster_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeServer_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        {
          int res = SWIG_AsVal_int(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_WrappedPipeBroadcaster_Init__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedPipeBroadcaster_Init'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedPipeBroadcaster::Init(boost::shared_ptr< RobotRaconteur::WrappedPipeServer > const &,int32_t)\n"
    "    RobotRaconteur::WrappedPipeBroadcaster::Init(boost::shared_ptr< RobotRaconteur::WrappedPipeServer > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedPipeBroadcaster_SendPacket(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeBroadcaster *arg1 = (RobotRaconteur::WrappedPipeBroadcaster *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeBroadcaster_SendPacket", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeBroadcaster_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeBroadcaster_SendPacket" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeBroadcaster *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeBroadcaster * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeBroadcaster * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPipeBroadcaster_SendPacket" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SendPacket((boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeBroadcaster_AsyncSendPacket(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeBroadcaster *arg1 = (RobotRaconteur::WrappedPipeBroadcaster *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg2 = 0 ;
  RobotRaconteur::AsyncVoidNoErrReturnDirector *arg3 = (RobotRaconteur::AsyncVoidNoErrReturnDirector *) 0 ;
  int32_t arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp2 ;
  int newmem2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  int val4 ;
  int ecode4 = 0 ;
  PyObject *swig_obj[4] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeBroadcaster_AsyncSendPacket", 4, 4, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeBroadcaster_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeBroadcaster_AsyncSendPacket" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeBroadcaster *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeBroadcaster * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeBroadcaster * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPipeBroadcaster_AsyncSendPacket" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__AsyncVoidNoErrReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedPipeBroadcaster_AsyncSendPacket" "', argument " "3"" of type '" "RobotRaconteur::AsyncVoidNoErrReturnDirector *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::AsyncVoidNoErrReturnDirector * >(argp3);
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedPipeBroadcaster_AsyncSendPacket" "', argument " "4"" of type '" "int32_t""'");
  } 
  arg4 = static_cast< int32_t >(val4);
  
  try {
    (arg1)->AsyncSendPacket((boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg2,arg3,arg4);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeBroadcaster_GetActivePipeEndpointCount(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeBroadcaster *arg1 = (RobotRaconteur::WrappedPipeBroadcaster *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  size_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeBroadcaster_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeBroadcaster_GetActivePipeEndpointCount" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeBroadcaster *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeBroadcaster * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeBroadcaster * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->GetActivePipeEndpointCount();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeBroadcaster_SetPredicateDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeBroadcaster *arg1 = (RobotRaconteur::WrappedPipeBroadcaster *) 0 ;
  RobotRaconteur::WrappedPipeBroadcasterPredicateDirector *arg2 = (RobotRaconteur::WrappedPipeBroadcasterPredicateDirector *) 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeBroadcaster_SetPredicateDirector", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeBroadcaster_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeBroadcaster_SetPredicateDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeBroadcaster *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeBroadcaster * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeBroadcaster * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__WrappedPipeBroadcasterPredicateDirector, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPipeBroadcaster_SetPredicateDirector" "', argument " "2"" of type '" "RobotRaconteur::WrappedPipeBroadcasterPredicateDirector *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedPipeBroadcasterPredicateDirector * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedPipeBroadcaster_SetPredicateDirector" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    (arg1)->SetPredicateDirector(arg2,arg3);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeBroadcaster_GetMaxBacklog(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeBroadcaster *arg1 = (RobotRaconteur::WrappedPipeBroadcaster *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeBroadcaster_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeBroadcaster_GetMaxBacklog" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeBroadcaster *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeBroadcaster * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeBroadcaster * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (int32_t)(arg1)->GetMaxBacklog();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeBroadcaster_SetMaxBacklog(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeBroadcaster *arg1 = (RobotRaconteur::WrappedPipeBroadcaster *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeBroadcaster_SetMaxBacklog", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeBroadcaster_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeBroadcaster_SetMaxBacklog" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeBroadcaster *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeBroadcaster * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeBroadcaster * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedPipeBroadcaster_SetMaxBacklog" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    (arg1)->SetMaxBacklog(arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedPipeBroadcaster(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeBroadcaster *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_WrappedPipeBroadcaster", 0, 0, 0)) SWIG_fail;
  
  try {
    result = (RobotRaconteur::WrappedPipeBroadcaster *)new RobotRaconteur::WrappedPipeBroadcaster();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeBroadcaster_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedPipeBroadcaster(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeBroadcaster *arg1 = (RobotRaconteur::WrappedPipeBroadcaster *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeBroadcaster_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedPipeBroadcaster" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeBroadcaster *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeBroadcaster * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeBroadcaster * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    (void)arg1; delete smartarg1;
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedPipeBroadcaster_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeBroadcaster_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedPipeBroadcaster_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_WrappedWireConnectionDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnectionDirector *arg1 = (RobotRaconteur::WrappedWireConnectionDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireConnectionDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedWireConnectionDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnectionDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireConnectionDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnectionDirector_WireValueChanged(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnectionDirector *arg1 = (RobotRaconteur::WrappedWireConnectionDirector *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > arg2 ;
  RobotRaconteur::TimeSpec *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  int newmem2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  PyObject *swig_obj[3] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireConnectionDirector_WireValueChanged", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireConnectionDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnectionDirector_WireValueChanged" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnectionDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireConnectionDirector * >(argp1);
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //if (smartarg) {
  //	arg2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireConnectionDirector_WireValueChanged" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement >""'");
  }
  if (argp2) arg2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
  if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
  
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__TimeSpec,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedWireConnectionDirector_WireValueChanged" "', argument " "3"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedWireConnectionDirector_WireValueChanged" "', argument " "3"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp3);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedWireConnectionDirector::WireValueChanged");
      } else {
        (arg1)->WireValueChanged(arg2,(RobotRaconteur::TimeSpec const &)*arg3);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnectionDirector_WireConnectionClosedCallback(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnectionDirector *arg1 = (RobotRaconteur::WrappedWireConnectionDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireConnectionDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnectionDirector_WireConnectionClosedCallback" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnectionDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireConnectionDirector * >(argp1);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedWireConnectionDirector::WireConnectionClosedCallback");
      } else {
        (arg1)->WireConnectionClosedCallback();
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedWireConnectionDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedWireConnectionDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedWireConnectionDirector *)new SwigDirector_WrappedWireConnectionDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedWireConnectionDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedWireConnectionDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnectionDirector *arg1 = (RobotRaconteur::WrappedWireConnectionDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireConnectionDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedWireConnectionDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnectionDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireConnectionDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedWireConnectionDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedWireConnectionDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedWireConnectionDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_AsyncWireConnectionReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncWireConnectionReturnDirector *arg1 = (RobotRaconteur::AsyncWireConnectionReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncWireConnectionReturnDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_AsyncWireConnectionReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncWireConnectionReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncWireConnectionReturnDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_AsyncWireConnectionReturnDirector_handler(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncWireConnectionReturnDirector *arg1 = (RobotRaconteur::AsyncWireConnectionReturnDirector *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *arg2 = 0 ;
  RobotRaconteur::HandlerErrorInfo *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared2 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  PyObject *swig_obj[3] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "AsyncWireConnectionReturnDirector_handler", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncWireConnectionReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AsyncWireConnectionReturnDirector_handler" "', argument " "1"" of type '" "RobotRaconteur::AsyncWireConnectionReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncWireConnectionReturnDirector * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "AsyncWireConnectionReturnDirector_handler" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedWireConnection > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedWireConnection > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedWireConnection > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedWireConnection > * >(argp2) : &tempshared2;
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "AsyncWireConnectionReturnDirector_handler" "', argument " "3"" of type '" "RobotRaconteur::HandlerErrorInfo &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "AsyncWireConnectionReturnDirector_handler" "', argument " "3"" of type '" "RobotRaconteur::HandlerErrorInfo &""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp3);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::AsyncWireConnectionReturnDirector::handler");
      } else {
        (arg1)->handler((boost::shared_ptr< RobotRaconteur::WrappedWireConnection > const &)*arg2,*arg3);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_AsyncWireConnectionReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::AsyncWireConnectionReturnDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::AsyncWireConnectionReturnDirector *)new SwigDirector_AsyncWireConnectionReturnDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__AsyncWireConnectionReturnDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_AsyncWireConnectionReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncWireConnectionReturnDirector *arg1 = (RobotRaconteur::AsyncWireConnectionReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncWireConnectionReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_AsyncWireConnectionReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncWireConnectionReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncWireConnectionReturnDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *AsyncWireConnectionReturnDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__AsyncWireConnectionReturnDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *AsyncWireConnectionReturnDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_AsyncWirePeekReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncWirePeekReturnDirector *arg1 = (RobotRaconteur::AsyncWirePeekReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncWirePeekReturnDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_AsyncWirePeekReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncWirePeekReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncWirePeekReturnDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_AsyncWirePeekReturnDirector_handler(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncWirePeekReturnDirector *arg1 = (RobotRaconteur::AsyncWirePeekReturnDirector *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > arg2 ;
  RobotRaconteur::TimeSpec *arg3 = 0 ;
  RobotRaconteur::HandlerErrorInfo *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  int newmem2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  void *argp4 = 0 ;
  int res4 = 0 ;
  PyObject *swig_obj[4] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "AsyncWirePeekReturnDirector_handler", 4, 4, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncWirePeekReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AsyncWirePeekReturnDirector_handler" "', argument " "1"" of type '" "RobotRaconteur::AsyncWirePeekReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncWirePeekReturnDirector * >(argp1);
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //if (smartarg) {
  //	arg2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "AsyncWirePeekReturnDirector_handler" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement >""'");
  }
  if (argp2) arg2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
  if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
  
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__TimeSpec,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "AsyncWirePeekReturnDirector_handler" "', argument " "3"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "AsyncWirePeekReturnDirector_handler" "', argument " "3"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp3);
  res4 = SWIG_ConvertPtr(swig_obj[3], &argp4, SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0 );
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "AsyncWirePeekReturnDirector_handler" "', argument " "4"" of type '" "RobotRaconteur::HandlerErrorInfo &""'"); 
  }
  if (!argp4) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "AsyncWirePeekReturnDirector_handler" "', argument " "4"" of type '" "RobotRaconteur::HandlerErrorInfo &""'"); 
  }
  arg4 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp4);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::AsyncWirePeekReturnDirector::handler");
      } else {
        (arg1)->handler(arg2,(RobotRaconteur::TimeSpec const &)*arg3,*arg4);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_AsyncWirePeekReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::AsyncWirePeekReturnDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::AsyncWirePeekReturnDirector *)new SwigDirector_AsyncWirePeekReturnDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__AsyncWirePeekReturnDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_AsyncWirePeekReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncWirePeekReturnDirector *arg1 = (RobotRaconteur::AsyncWirePeekReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncWirePeekReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_AsyncWirePeekReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncWirePeekReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncWirePeekReturnDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *AsyncWirePeekReturnDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__AsyncWirePeekReturnDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *AsyncWirePeekReturnDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_TryGetValueResult_res_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TryGetValueResult *arg1 = (RobotRaconteur::TryGetValueResult *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TryGetValueResult_res_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TryGetValueResult, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TryGetValueResult_res_set" "', argument " "1"" of type '" "RobotRaconteur::TryGetValueResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TryGetValueResult * >(argp1);
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "TryGetValueResult_res_set" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->res = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TryGetValueResult_res_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TryGetValueResult *arg1 = (RobotRaconteur::TryGetValueResult *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TryGetValueResult, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TryGetValueResult_res_get" "', argument " "1"" of type '" "RobotRaconteur::TryGetValueResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TryGetValueResult * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (bool) ((arg1)->res);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TryGetValueResult_value_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TryGetValueResult *arg1 = (RobotRaconteur::TryGetValueResult *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TryGetValueResult_value_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TryGetValueResult, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TryGetValueResult_value_set" "', argument " "1"" of type '" "RobotRaconteur::TryGetValueResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TryGetValueResult * >(argp1);
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "TryGetValueResult_value_set" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->value = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TryGetValueResult_value_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TryGetValueResult *arg1 = (RobotRaconteur::TryGetValueResult *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TryGetValueResult, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TryGetValueResult_value_get" "', argument " "1"" of type '" "RobotRaconteur::TryGetValueResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TryGetValueResult * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::intrusive_ptr< RobotRaconteur::MessageElement > *) & ((arg1)->value);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  
  //if (*result) {
  //  intrusive_ptr_add_ref(result->get());
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result->get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(result)) {
    intrusive_ptr_add_ref(result->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >(result->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TryGetValueResult_ts_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TryGetValueResult *arg1 = (RobotRaconteur::TryGetValueResult *) 0 ;
  RobotRaconteur::TimeSpec *arg2 = (RobotRaconteur::TimeSpec *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "TryGetValueResult_ts_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TryGetValueResult, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TryGetValueResult_ts_set" "', argument " "1"" of type '" "RobotRaconteur::TryGetValueResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TryGetValueResult * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__TimeSpec, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "TryGetValueResult_ts_set" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ts = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_TryGetValueResult_ts_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TryGetValueResult *arg1 = (RobotRaconteur::TryGetValueResult *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::TimeSpec *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TryGetValueResult, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "TryGetValueResult_ts_get" "', argument " "1"" of type '" "RobotRaconteur::TryGetValueResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TryGetValueResult * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::TimeSpec *)& ((arg1)->ts);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__TimeSpec, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_TryGetValueResult(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TryGetValueResult *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_TryGetValueResult", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::TryGetValueResult *)new RobotRaconteur::TryGetValueResult();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__TryGetValueResult, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_TryGetValueResult(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::TryGetValueResult *arg1 = (RobotRaconteur::TryGetValueResult *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__TryGetValueResult, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_TryGetValueResult" "', argument " "1"" of type '" "RobotRaconteur::TryGetValueResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::TryGetValueResult * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *TryGetValueResult_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__TryGetValueResult, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *TryGetValueResult_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedWireConnection_GetInValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_GetInValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetInValue();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_GetOutValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_GetOutValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetOutValue();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_SetOutValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireConnection_SetOutValue", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_SetOutValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireConnection_SetOutValue" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetOutValue((boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_SetRRDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  RobotRaconteur::WrappedWireConnectionDirector *arg2 = (RobotRaconteur::WrappedWireConnectionDirector *) 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireConnection_SetRRDirector", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_SetRRDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__WrappedWireConnectionDirector, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireConnection_SetRRDirector" "', argument " "2"" of type '" "RobotRaconteur::WrappedWireConnectionDirector *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedWireConnectionDirector * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedWireConnection_SetRRDirector" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetRRDirector(arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_Type_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireConnection_Type_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_Type_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireConnection_Type_set" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::TypeDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2) : &tempshared2;
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Type = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_Type_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_Type_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::shared_ptr< RobotRaconteur::TypeDefinition > *) & ((arg1)->Type);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  {
    boost::shared_ptr<  RobotRaconteur::TypeDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::TypeDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_Close(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_Close" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Close();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_GetEndpoint(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_GetEndpoint" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (uint32_t)(arg1)->GetEndpoint();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_GetLastValueReceivedTime(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::TimeSpec result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_GetLastValueReceivedTime" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->GetLastValueReceivedTime();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::TimeSpec(static_cast< const RobotRaconteur::TimeSpec& >(result))), SWIGTYPE_p_RobotRaconteur__TimeSpec, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_GetLastValueSentTime(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::TimeSpec result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_GetLastValueSentTime" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->GetLastValueSentTime();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::TimeSpec(static_cast< const RobotRaconteur::TimeSpec& >(result))), SWIGTYPE_p_RobotRaconteur__TimeSpec, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_GetInValueValid(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_GetInValueValid" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (bool)(arg1)->GetInValueValid();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_GetOutValueValid(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_GetOutValueValid" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (bool)(arg1)->GetOutValueValid();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_TryGetInValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::TryGetValueResult result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_TryGetInValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->TryGetInValue();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::TryGetValueResult(static_cast< const RobotRaconteur::TryGetValueResult& >(result))), SWIGTYPE_p_RobotRaconteur__TryGetValueResult, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_TryGetOutValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::TryGetValueResult result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_TryGetOutValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->TryGetOutValue();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::TryGetValueResult(static_cast< const RobotRaconteur::TryGetValueResult& >(result))), SWIGTYPE_p_RobotRaconteur__TryGetValueResult, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_AsyncClose(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  int32_t arg2 ;
  RobotRaconteur::AsyncVoidReturnDirector *arg3 = (RobotRaconteur::AsyncVoidReturnDirector *) 0 ;
  int32_t arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  int val4 ;
  int ecode4 = 0 ;
  PyObject *swig_obj[4] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireConnection_AsyncClose", 4, 4, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_AsyncClose" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedWireConnection_AsyncClose" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__AsyncVoidReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedWireConnection_AsyncClose" "', argument " "3"" of type '" "RobotRaconteur::AsyncVoidReturnDirector *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::AsyncVoidReturnDirector * >(argp3);
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedWireConnection_AsyncClose" "', argument " "4"" of type '" "int32_t""'");
  } 
  arg4 = static_cast< int32_t >(val4);
  
  try {
    (arg1)->AsyncClose(arg2,arg3,arg4);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_GetNode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_GetNode" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->GetNode();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_GetIgnoreInValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_GetIgnoreInValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (bool)(arg1)->GetIgnoreInValue();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_SetIgnoreInValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireConnection_SetIgnoreInValue", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_SetIgnoreInValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedWireConnection_SetIgnoreInValue" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  
  try {
    (arg1)->SetIgnoreInValue(arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_GetInValueLifespan(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_GetInValueLifespan" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (int32_t)(arg1)->GetInValueLifespan();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_SetInValueLifespan(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireConnection_SetInValueLifespan", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_SetInValueLifespan" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedWireConnection_SetInValueLifespan" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    (arg1)->SetInValueLifespan(arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_GetOutValueLifespan(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_GetOutValueLifespan" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (int32_t)(arg1)->GetOutValueLifespan();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_SetOutValueLifespan(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireConnection_SetOutValueLifespan", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_SetOutValueLifespan" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedWireConnection_SetOutValueLifespan" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    (arg1)->SetOutValueLifespan(arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_WaitInValueValid__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  bool result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_WaitInValueValid" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedWireConnection_WaitInValueValid" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->WaitInValueValid(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_WaitInValueValid__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  bool result;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_WaitInValueValid" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->WaitInValueValid();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_WaitInValueValid(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedWireConnection_WaitInValueValid", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_WrappedWireConnection_WaitInValueValid__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_WrappedWireConnection_WaitInValueValid__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedWireConnection_WaitInValueValid'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedWireConnection::WaitInValueValid(int32_t)\n"
    "    RobotRaconteur::WrappedWireConnection::WaitInValueValid()\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_WaitOutValueValid__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  bool result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_WaitOutValueValid" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedWireConnection_WaitOutValueValid" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->WaitOutValueValid(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_WaitOutValueValid__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  bool result;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_WaitOutValueValid" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->WaitOutValueValid();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_WaitOutValueValid(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedWireConnection_WaitOutValueValid", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_WrappedWireConnection_WaitOutValueValid__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_WrappedWireConnection_WaitOutValueValid__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedWireConnection_WaitOutValueValid'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedWireConnection::WaitOutValueValid(int32_t)\n"
    "    RobotRaconteur::WrappedWireConnection::WaitOutValueValid()\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedWireConnection_Direction(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MemberDefinition_Direction result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireConnection_Direction" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (RobotRaconteur::MemberDefinition_Direction)(arg1)->Direction();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedWireConnection(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireConnection *arg1 = (RobotRaconteur::WrappedWireConnection *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedWireConnection" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireConnection *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireConnection * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    (void)arg1; delete smartarg1;
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedWireConnection_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_WrappedWireClient_Connect(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireClient *arg1 = (RobotRaconteur::WrappedWireClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireClient_Connect" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->Connect();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedWireConnection >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireClient_AsyncConnect(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireClient *arg1 = (RobotRaconteur::WrappedWireClient *) 0 ;
  int32_t arg2 ;
  RobotRaconteur::AsyncWireConnectionReturnDirector *arg3 = (RobotRaconteur::AsyncWireConnectionReturnDirector *) 0 ;
  int32_t arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  int val4 ;
  int ecode4 = 0 ;
  PyObject *swig_obj[4] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireClient_AsyncConnect", 4, 4, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireClient_AsyncConnect" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedWireClient_AsyncConnect" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__AsyncWireConnectionReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedWireClient_AsyncConnect" "', argument " "3"" of type '" "RobotRaconteur::AsyncWireConnectionReturnDirector *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::AsyncWireConnectionReturnDirector * >(argp3);
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedWireClient_AsyncConnect" "', argument " "4"" of type '" "int32_t""'");
  } 
  arg4 = static_cast< int32_t >(val4);
  
  try {
    (arg1)->AsyncConnect(arg2,arg3,arg4);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireClient_GetMemberName(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireClient *arg1 = (RobotRaconteur::WrappedWireClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireClient_GetMemberName" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->GetMemberName();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireClient_PeekInValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireClient *arg1 = (RobotRaconteur::WrappedWireClient *) 0 ;
  RobotRaconteur::TimeSpec *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireClient_PeekInValue", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireClient_PeekInValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__TimeSpec,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireClient_PeekInValue" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedWireClient_PeekInValue" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->PeekInValue(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireClient_PeekOutValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireClient *arg1 = (RobotRaconteur::WrappedWireClient *) 0 ;
  RobotRaconteur::TimeSpec *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireClient_PeekOutValue", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireClient_PeekOutValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__TimeSpec,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireClient_PeekOutValue" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedWireClient_PeekOutValue" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->PeekOutValue(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireClient_PokeOutValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireClient *arg1 = (RobotRaconteur::WrappedWireClient *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireClient_PokeOutValue", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireClient_PokeOutValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireClient_PokeOutValue" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->PokeOutValue((boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireClient_AsyncPeekInValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireClient *arg1 = (RobotRaconteur::WrappedWireClient *) 0 ;
  int32_t arg2 ;
  RobotRaconteur::AsyncWirePeekReturnDirector *arg3 = (RobotRaconteur::AsyncWirePeekReturnDirector *) 0 ;
  int32_t arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  int val4 ;
  int ecode4 = 0 ;
  PyObject *swig_obj[4] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireClient_AsyncPeekInValue", 4, 4, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireClient_AsyncPeekInValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedWireClient_AsyncPeekInValue" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__AsyncWirePeekReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedWireClient_AsyncPeekInValue" "', argument " "3"" of type '" "RobotRaconteur::AsyncWirePeekReturnDirector *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::AsyncWirePeekReturnDirector * >(argp3);
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedWireClient_AsyncPeekInValue" "', argument " "4"" of type '" "int32_t""'");
  } 
  arg4 = static_cast< int32_t >(val4);
  
  try {
    (arg1)->AsyncPeekInValue(arg2,arg3,arg4);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireClient_AsyncPeekOutValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireClient *arg1 = (RobotRaconteur::WrappedWireClient *) 0 ;
  int32_t arg2 ;
  RobotRaconteur::AsyncWirePeekReturnDirector *arg3 = (RobotRaconteur::AsyncWirePeekReturnDirector *) 0 ;
  int32_t arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  int val4 ;
  int ecode4 = 0 ;
  PyObject *swig_obj[4] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireClient_AsyncPeekOutValue", 4, 4, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireClient_AsyncPeekOutValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedWireClient_AsyncPeekOutValue" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__AsyncWirePeekReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedWireClient_AsyncPeekOutValue" "', argument " "3"" of type '" "RobotRaconteur::AsyncWirePeekReturnDirector *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::AsyncWirePeekReturnDirector * >(argp3);
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedWireClient_AsyncPeekOutValue" "', argument " "4"" of type '" "int32_t""'");
  } 
  arg4 = static_cast< int32_t >(val4);
  
  try {
    (arg1)->AsyncPeekOutValue(arg2,arg3,arg4);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireClient_AsyncPokeOutValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireClient *arg1 = (RobotRaconteur::WrappedWireClient *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg2 = 0 ;
  int32_t arg3 ;
  RobotRaconteur::AsyncVoidReturnDirector *arg4 = (RobotRaconteur::AsyncVoidReturnDirector *) 0 ;
  int32_t arg5 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp2 ;
  int newmem2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  void *argp4 = 0 ;
  int res4 = 0 ;
  int val5 ;
  int ecode5 = 0 ;
  PyObject *swig_obj[5] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireClient_AsyncPokeOutValue", 5, 5, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireClient_AsyncPokeOutValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireClient_AsyncPokeOutValue" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedWireClient_AsyncPokeOutValue" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  res4 = SWIG_ConvertPtr(swig_obj[3], &argp4,SWIGTYPE_p_RobotRaconteur__AsyncVoidReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedWireClient_AsyncPokeOutValue" "', argument " "4"" of type '" "RobotRaconteur::AsyncVoidReturnDirector *""'"); 
  }
  arg4 = reinterpret_cast< RobotRaconteur::AsyncVoidReturnDirector * >(argp4);
  ecode5 = SWIG_AsVal_int(swig_obj[4], &val5);
  if (!SWIG_IsOK(ecode5)) {
    SWIG_exception_fail(SWIG_ArgError(ecode5), "in method '" "WrappedWireClient_AsyncPokeOutValue" "', argument " "5"" of type '" "int32_t""'");
  } 
  arg5 = static_cast< int32_t >(val5);
  
  try {
    (arg1)->AsyncPokeOutValue((boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg2,arg3,arg4,arg5);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireClient_Type_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireClient *arg1 = (RobotRaconteur::WrappedWireClient *) 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireClient_Type_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireClient_Type_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireClient_Type_set" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::TypeDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2) : &tempshared2;
    }
  }
  if (arg1) (arg1)->Type = *arg2;
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireClient_Type_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireClient *arg1 = (RobotRaconteur::WrappedWireClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireClient_Type_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  result = (boost::shared_ptr< RobotRaconteur::TypeDefinition > *) & ((arg1)->Type);
  {
    boost::shared_ptr<  RobotRaconteur::TypeDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::TypeDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireClient_GetNode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireClient *arg1 = (RobotRaconteur::WrappedWireClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireClient_GetNode" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->GetNode();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireClient_Direction(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireClient *arg1 = (RobotRaconteur::WrappedWireClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MemberDefinition_Direction result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireClient_Direction" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (RobotRaconteur::MemberDefinition_Direction)(arg1)->Direction();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedWireClient(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireClient *arg1 = (RobotRaconteur::WrappedWireClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedWireClient" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    (void)arg1; delete smartarg1;
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedWireClient_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireClient_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_delete_WrappedWireServerConnectDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireServerConnectDirector *arg1 = (RobotRaconteur::WrappedWireServerConnectDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireServerConnectDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedWireServerConnectDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireServerConnectDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireServerConnectDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireServerConnectDirector_WireConnectCallback(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireServerConnectDirector *arg1 = (RobotRaconteur::WrappedWireServerConnectDirector *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > tempshared2 ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireServerConnectDirector_WireConnectCallback", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireServerConnectDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireServerConnectDirector_WireConnectCallback" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireServerConnectDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireServerConnectDirector * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireServerConnectDirector_WireConnectCallback" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedWireConnection > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedWireConnection > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedWireConnection > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedWireConnection > * >(argp2) : &tempshared2;
    }
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedWireServerConnectDirector::WireConnectCallback");
      } else {
        (arg1)->WireConnectCallback((boost::shared_ptr< RobotRaconteur::WrappedWireConnection > const &)*arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedWireServerConnectDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedWireServerConnectDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedWireServerConnectDirector *)new SwigDirector_WrappedWireServerConnectDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedWireServerConnectDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedWireServerConnectDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireServerConnectDirector *arg1 = (RobotRaconteur::WrappedWireServerConnectDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireServerConnectDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedWireServerConnectDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireServerConnectDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireServerConnectDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedWireServerConnectDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedWireServerConnectDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedWireServerConnectDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_WrappedWireServerPeekValueDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireServerPeekValueDirector *arg1 = (RobotRaconteur::WrappedWireServerPeekValueDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireServerPeekValueDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedWireServerPeekValueDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireServerPeekValueDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireServerPeekValueDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireServerPeekValueDirector_PeekValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireServerPeekValueDirector *arg1 = (RobotRaconteur::WrappedWireServerPeekValueDirector *) 0 ;
  uint32_t *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  uint32_t temp2 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireServerPeekValueDirector_PeekValue", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireServerPeekValueDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireServerPeekValueDirector_PeekValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireServerPeekValueDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireServerPeekValueDirector * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedWireServerPeekValueDirector_PeekValue" "', argument " "2"" of type '" "uint32_t""'");
  } 
  temp2 = static_cast< uint32_t >(val2);
  arg2 = &temp2;
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedWireServerPeekValueDirector::PeekValue");
      } else {
        result = (arg1)->PeekValue((uint32_t const &)*arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedWireServerPeekValueDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedWireServerPeekValueDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedWireServerPeekValueDirector *)new SwigDirector_WrappedWireServerPeekValueDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedWireServerPeekValueDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedWireServerPeekValueDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireServerPeekValueDirector *arg1 = (RobotRaconteur::WrappedWireServerPeekValueDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireServerPeekValueDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedWireServerPeekValueDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireServerPeekValueDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireServerPeekValueDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedWireServerPeekValueDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedWireServerPeekValueDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedWireServerPeekValueDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_WrappedWireServerPokeValueDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireServerPokeValueDirector *arg1 = (RobotRaconteur::WrappedWireServerPokeValueDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireServerPokeValueDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedWireServerPokeValueDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireServerPokeValueDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireServerPokeValueDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireServerPokeValueDirector_PokeValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireServerPokeValueDirector *arg1 = (RobotRaconteur::WrappedWireServerPokeValueDirector *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > arg2 ;
  RobotRaconteur::TimeSpec *arg3 = 0 ;
  uint32_t *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  int newmem2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  uint32_t temp4 ;
  unsigned int val4 ;
  int ecode4 = 0 ;
  PyObject *swig_obj[4] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireServerPokeValueDirector_PokeValue", 4, 4, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireServerPokeValueDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireServerPokeValueDirector_PokeValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireServerPokeValueDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireServerPokeValueDirector * >(argp1);
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //if (smartarg) {
  //	arg2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireServerPokeValueDirector_PokeValue" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement >""'");
  }
  if (argp2) arg2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
  if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
  
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__TimeSpec,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedWireServerPokeValueDirector_PokeValue" "', argument " "3"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedWireServerPokeValueDirector_PokeValue" "', argument " "3"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp3);
  ecode4 = SWIG_AsVal_unsigned_SS_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedWireServerPokeValueDirector_PokeValue" "', argument " "4"" of type '" "uint32_t""'");
  } 
  temp4 = static_cast< uint32_t >(val4);
  arg4 = &temp4;
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedWireServerPokeValueDirector::PokeValue");
      } else {
        (arg1)->PokeValue(arg2,(RobotRaconteur::TimeSpec const &)*arg3,(uint32_t const &)*arg4);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedWireServerPokeValueDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedWireServerPokeValueDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedWireServerPokeValueDirector *)new SwigDirector_WrappedWireServerPokeValueDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedWireServerPokeValueDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedWireServerPokeValueDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireServerPokeValueDirector *arg1 = (RobotRaconteur::WrappedWireServerPokeValueDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireServerPokeValueDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedWireServerPokeValueDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireServerPokeValueDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireServerPokeValueDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedWireServerPokeValueDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedWireServerPokeValueDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedWireServerPokeValueDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedWireServer_GetMemberName(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireServer *arg1 = (RobotRaconteur::WrappedWireServer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireServer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireServer_GetMemberName" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireServer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireServer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireServer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetMemberName();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireServer_Type_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireServer *arg1 = (RobotRaconteur::WrappedWireServer *) 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireServer_Type_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireServer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireServer_Type_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireServer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireServer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireServer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireServer_Type_set" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::TypeDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2) : &tempshared2;
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Type = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireServer_Type_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireServer *arg1 = (RobotRaconteur::WrappedWireServer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireServer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireServer_Type_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireServer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireServer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireServer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::shared_ptr< RobotRaconteur::TypeDefinition > *) & ((arg1)->Type);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  {
    boost::shared_ptr<  RobotRaconteur::TypeDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::TypeDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireServer_SetWrappedWireConnectCallback(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireServer *arg1 = (RobotRaconteur::WrappedWireServer *) 0 ;
  RobotRaconteur::WrappedWireServerConnectDirector *arg2 = (RobotRaconteur::WrappedWireServerConnectDirector *) 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireServer_SetWrappedWireConnectCallback", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireServer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireServer_SetWrappedWireConnectCallback" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireServer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireServer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireServer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__WrappedWireServerConnectDirector, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireServer_SetWrappedWireConnectCallback" "', argument " "2"" of type '" "RobotRaconteur::WrappedWireServerConnectDirector *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedWireServerConnectDirector * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedWireServer_SetWrappedWireConnectCallback" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetWrappedWireConnectCallback(arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireServer_GetNode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireServer *arg1 = (RobotRaconteur::WrappedWireServer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireServer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireServer_GetNode" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireServer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireServer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireServer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetNode();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireServer_SetPeekInValueCallback(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireServer *arg1 = (RobotRaconteur::WrappedWireServer *) 0 ;
  RobotRaconteur::WrappedWireServerPeekValueDirector *arg2 = (RobotRaconteur::WrappedWireServerPeekValueDirector *) 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireServer_SetPeekInValueCallback", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireServer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireServer_SetPeekInValueCallback" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireServer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireServer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireServer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__WrappedWireServerPeekValueDirector, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireServer_SetPeekInValueCallback" "', argument " "2"" of type '" "RobotRaconteur::WrappedWireServerPeekValueDirector *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedWireServerPeekValueDirector * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedWireServer_SetPeekInValueCallback" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetPeekInValueCallback(arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireServer_SetPeekOutValueCallback(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireServer *arg1 = (RobotRaconteur::WrappedWireServer *) 0 ;
  RobotRaconteur::WrappedWireServerPeekValueDirector *arg2 = (RobotRaconteur::WrappedWireServerPeekValueDirector *) 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireServer_SetPeekOutValueCallback", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireServer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireServer_SetPeekOutValueCallback" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireServer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireServer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireServer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__WrappedWireServerPeekValueDirector, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireServer_SetPeekOutValueCallback" "', argument " "2"" of type '" "RobotRaconteur::WrappedWireServerPeekValueDirector *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedWireServerPeekValueDirector * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedWireServer_SetPeekOutValueCallback" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetPeekOutValueCallback(arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireServer_SetPokeOutValueCallback(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireServer *arg1 = (RobotRaconteur::WrappedWireServer *) 0 ;
  RobotRaconteur::WrappedWireServerPokeValueDirector *arg2 = (RobotRaconteur::WrappedWireServerPokeValueDirector *) 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireServer_SetPokeOutValueCallback", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireServer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireServer_SetPokeOutValueCallback" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireServer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireServer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireServer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__WrappedWireServerPokeValueDirector, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireServer_SetPokeOutValueCallback" "', argument " "2"" of type '" "RobotRaconteur::WrappedWireServerPokeValueDirector *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedWireServerPokeValueDirector * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedWireServer_SetPokeOutValueCallback" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetPokeOutValueCallback(arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireServer_Direction(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireServer *arg1 = (RobotRaconteur::WrappedWireServer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MemberDefinition_Direction result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireServer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireServer_Direction" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireServer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireServer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireServer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::MemberDefinition_Direction)(arg1)->Direction();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedWireServer(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireServer *arg1 = (RobotRaconteur::WrappedWireServer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireServer_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedWireServer" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireServer *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireServer * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireServer > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireServer * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedWireServer_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireServer_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_WrappedWireBroadcasterPredicateDirector_Predicate(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireBroadcasterPredicateDirector *arg1 = (RobotRaconteur::WrappedWireBroadcasterPredicateDirector *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireBroadcasterPredicateDirector_Predicate", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireBroadcasterPredicateDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireBroadcasterPredicateDirector_Predicate" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireBroadcasterPredicateDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireBroadcasterPredicateDirector * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedWireBroadcasterPredicateDirector_Predicate" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedWireBroadcasterPredicateDirector::Predicate");
      } else {
        result = (bool)(arg1)->Predicate(arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedWireBroadcasterPredicateDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireBroadcasterPredicateDirector *arg1 = (RobotRaconteur::WrappedWireBroadcasterPredicateDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireBroadcasterPredicateDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedWireBroadcasterPredicateDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireBroadcasterPredicateDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireBroadcasterPredicateDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedWireBroadcasterPredicateDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedWireBroadcasterPredicateDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedWireBroadcasterPredicateDirector *)new SwigDirector_WrappedWireBroadcasterPredicateDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedWireBroadcasterPredicateDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedWireBroadcasterPredicateDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireBroadcasterPredicateDirector *arg1 = (RobotRaconteur::WrappedWireBroadcasterPredicateDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireBroadcasterPredicateDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedWireBroadcasterPredicateDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireBroadcasterPredicateDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireBroadcasterPredicateDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedWireBroadcasterPredicateDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedWireBroadcasterPredicateDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedWireBroadcasterPredicateDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedWireBroadcaster_Init(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireBroadcaster *arg1 = (RobotRaconteur::WrappedWireBroadcaster *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireBroadcaster_Init", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireBroadcaster_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireBroadcaster_Init" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireBroadcaster *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireBroadcaster * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireBroadcaster * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireServer_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireBroadcaster_Init" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedWireServer > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedWireServer > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedWireServer > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedWireServer > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Init((boost::shared_ptr< RobotRaconteur::WrappedWireServer > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireBroadcaster_SetOutValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireBroadcaster *arg1 = (RobotRaconteur::WrappedWireBroadcaster *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireBroadcaster_SetOutValue", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireBroadcaster_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireBroadcaster_SetOutValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireBroadcaster *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireBroadcaster * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireBroadcaster * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireBroadcaster_SetOutValue" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetOutValue((boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireBroadcaster_GetActiveWireConnectionCount(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireBroadcaster *arg1 = (RobotRaconteur::WrappedWireBroadcaster *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  size_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireBroadcaster_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireBroadcaster_GetActiveWireConnectionCount" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireBroadcaster *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireBroadcaster * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireBroadcaster * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetActiveWireConnectionCount();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireBroadcaster_SetPredicateDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireBroadcaster *arg1 = (RobotRaconteur::WrappedWireBroadcaster *) 0 ;
  RobotRaconteur::WrappedWireBroadcasterPredicateDirector *arg2 = (RobotRaconteur::WrappedWireBroadcasterPredicateDirector *) 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireBroadcaster_SetPredicateDirector", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireBroadcaster_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireBroadcaster_SetPredicateDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireBroadcaster *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireBroadcaster * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireBroadcaster * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__WrappedWireBroadcasterPredicateDirector, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireBroadcaster_SetPredicateDirector" "', argument " "2"" of type '" "RobotRaconteur::WrappedWireBroadcasterPredicateDirector *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedWireBroadcasterPredicateDirector * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedWireBroadcaster_SetPredicateDirector" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetPredicateDirector(arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireBroadcaster_GetOutValueLifespan(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireBroadcaster *arg1 = (RobotRaconteur::WrappedWireBroadcaster *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireBroadcaster_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireBroadcaster_GetOutValueLifespan" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireBroadcaster *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireBroadcaster * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireBroadcaster * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (int32_t)(arg1)->GetOutValueLifespan();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireBroadcaster_SetOutValueLifespan(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireBroadcaster *arg1 = (RobotRaconteur::WrappedWireBroadcaster *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireBroadcaster_SetOutValueLifespan", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireBroadcaster_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireBroadcaster_SetOutValueLifespan" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireBroadcaster *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireBroadcaster * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireBroadcaster * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedWireBroadcaster_SetOutValueLifespan" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetOutValueLifespan(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireBroadcaster_SetPeekInValueCallback(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireBroadcaster *arg1 = (RobotRaconteur::WrappedWireBroadcaster *) 0 ;
  RobotRaconteur::WrappedWireServerPeekValueDirector *arg2 = (RobotRaconteur::WrappedWireServerPeekValueDirector *) 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireBroadcaster_SetPeekInValueCallback", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireBroadcaster_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireBroadcaster_SetPeekInValueCallback" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireBroadcaster *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireBroadcaster * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireBroadcaster * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__WrappedWireServerPeekValueDirector, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireBroadcaster_SetPeekInValueCallback" "', argument " "2"" of type '" "RobotRaconteur::WrappedWireServerPeekValueDirector *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedWireServerPeekValueDirector * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedWireBroadcaster_SetPeekInValueCallback" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetPeekInValueCallback(arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedWireBroadcaster(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireBroadcaster *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_WrappedWireBroadcaster", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedWireBroadcaster *)new RobotRaconteur::WrappedWireBroadcaster();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireBroadcaster_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedWireBroadcaster(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireBroadcaster *arg1 = (RobotRaconteur::WrappedWireBroadcaster *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireBroadcaster_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedWireBroadcaster" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireBroadcaster *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireBroadcaster * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireBroadcaster > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireBroadcaster * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedWireBroadcaster_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireBroadcaster_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedWireBroadcaster_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedWireUnicastReceiver_Init(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireUnicastReceiver *arg1 = (RobotRaconteur::WrappedWireUnicastReceiver *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireUnicastReceiver > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireUnicastReceiver > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireUnicastReceiver_Init", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireUnicastReceiver_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireUnicastReceiver_Init" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireUnicastReceiver *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireUnicastReceiver > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireUnicastReceiver > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireUnicastReceiver * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireUnicastReceiver > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireUnicastReceiver * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireServer_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireUnicastReceiver_Init" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedWireServer > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedWireServer > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedWireServer > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedWireServer > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Init((boost::shared_ptr< RobotRaconteur::WrappedWireServer > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireUnicastReceiver_GetInValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireUnicastReceiver *arg1 = (RobotRaconteur::WrappedWireUnicastReceiver *) 0 ;
  RobotRaconteur::TimeSpec *arg2 = 0 ;
  uint32_t *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireUnicastReceiver > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireUnicastReceiver > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  uint32_t temp3 ;
  int res3 = SWIG_TMPOBJ ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  arg3 = &temp3;
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireUnicastReceiver_GetInValue", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireUnicastReceiver_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireUnicastReceiver_GetInValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireUnicastReceiver *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireUnicastReceiver > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireUnicastReceiver > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireUnicastReceiver * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireUnicastReceiver > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireUnicastReceiver * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__TimeSpec,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireUnicastReceiver_GetInValue" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedWireUnicastReceiver_GetInValue" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetInValue(*arg2,*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  if (SWIG_IsTmpObj(res3)) {
    resultobj = SWIG_Python_AppendOutput(resultobj, SWIG_From_unsigned_SS_int((*arg3)));
  } else {
    int new_flags = SWIG_IsNewObj(res3) ? (SWIG_POINTER_OWN |  0 ) :  0 ;
    resultobj = SWIG_Python_AppendOutput(resultobj, SWIG_NewPointerObj((void*)(arg3), SWIGTYPE_p_unsigned_int, new_flags));
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireUnicastReceiver_AddInValueChangedListener(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireUnicastReceiver *arg1 = (RobotRaconteur::WrappedWireUnicastReceiver *) 0 ;
  RobotRaconteur::WrappedWireServerPokeValueDirector *arg2 = (RobotRaconteur::WrappedWireServerPokeValueDirector *) 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireUnicastReceiver > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireUnicastReceiver > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireUnicastReceiver_AddInValueChangedListener", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireUnicastReceiver_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireUnicastReceiver_AddInValueChangedListener" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireUnicastReceiver *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireUnicastReceiver > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireUnicastReceiver > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireUnicastReceiver * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireUnicastReceiver > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireUnicastReceiver * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__WrappedWireServerPokeValueDirector, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireUnicastReceiver_AddInValueChangedListener" "', argument " "2"" of type '" "RobotRaconteur::WrappedWireServerPokeValueDirector *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedWireServerPokeValueDirector * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedWireUnicastReceiver_AddInValueChangedListener" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->AddInValueChangedListener(arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireUnicastReceiver_GetInValueLifespan(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireUnicastReceiver *arg1 = (RobotRaconteur::WrappedWireUnicastReceiver *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireUnicastReceiver > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireUnicastReceiver > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireUnicastReceiver_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireUnicastReceiver_GetInValueLifespan" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireUnicastReceiver *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireUnicastReceiver > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireUnicastReceiver > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireUnicastReceiver * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireUnicastReceiver > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireUnicastReceiver * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (int32_t)(arg1)->GetInValueLifespan();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireUnicastReceiver_SetInValueLifespan(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireUnicastReceiver *arg1 = (RobotRaconteur::WrappedWireUnicastReceiver *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireUnicastReceiver > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireUnicastReceiver > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireUnicastReceiver_SetInValueLifespan", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireUnicastReceiver_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireUnicastReceiver_SetInValueLifespan" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireUnicastReceiver *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireUnicastReceiver > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireUnicastReceiver > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireUnicastReceiver * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireUnicastReceiver > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireUnicastReceiver * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedWireUnicastReceiver_SetInValueLifespan" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetInValueLifespan(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireUnicastReceiver_TryGetInValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireUnicastReceiver *arg1 = (RobotRaconteur::WrappedWireUnicastReceiver *) 0 ;
  RobotRaconteur::WrappedService_typed_packet *arg2 = 0 ;
  RobotRaconteur::TimeSpec *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireUnicastReceiver > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireUnicastReceiver > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  PyObject *swig_obj[3] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireUnicastReceiver_TryGetInValue", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireUnicastReceiver_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireUnicastReceiver_TryGetInValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireUnicastReceiver *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireUnicastReceiver > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireUnicastReceiver > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireUnicastReceiver * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireUnicastReceiver > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireUnicastReceiver * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireUnicastReceiver_TryGetInValue" "', argument " "2"" of type '" "RobotRaconteur::WrappedService_typed_packet &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedWireUnicastReceiver_TryGetInValue" "', argument " "2"" of type '" "RobotRaconteur::WrappedService_typed_packet &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedService_typed_packet * >(argp2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__TimeSpec,  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedWireUnicastReceiver_TryGetInValue" "', argument " "3"" of type '" "RobotRaconteur::TimeSpec &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedWireUnicastReceiver_TryGetInValue" "', argument " "3"" of type '" "RobotRaconteur::TimeSpec &""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->TryGetInValue(*arg2,*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedWireUnicastReceiver(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireUnicastReceiver *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_WrappedWireUnicastReceiver", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedWireUnicastReceiver *)new RobotRaconteur::WrappedWireUnicastReceiver();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedWireUnicastReceiver > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedWireUnicastReceiver >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireUnicastReceiver_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedWireUnicastReceiver(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireUnicastReceiver *arg1 = (RobotRaconteur::WrappedWireUnicastReceiver *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireUnicastReceiver > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireUnicastReceiver > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireUnicastReceiver_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedWireUnicastReceiver" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireUnicastReceiver *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireUnicastReceiver > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireUnicastReceiver > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireUnicastReceiver * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireUnicastReceiver > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireUnicastReceiver * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedWireUnicastReceiver_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireUnicastReceiver_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedWireUnicastReceiver_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_ArrayMemoryBase_Length(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ArrayMemoryBase *arg1 = (RobotRaconteur::ArrayMemoryBase *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  uint64_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ArrayMemoryBase_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ArrayMemoryBase_Length" "', argument " "1"" of type '" "RobotRaconteur::ArrayMemoryBase *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ArrayMemoryBase > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ArrayMemoryBase > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ArrayMemoryBase * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ArrayMemoryBase > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ArrayMemoryBase * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (uint64_t)(arg1)->Length();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ArrayMemoryBase_ElementTypeID(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ArrayMemoryBase *arg1 = (RobotRaconteur::ArrayMemoryBase *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::DataTypes result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ArrayMemoryBase_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ArrayMemoryBase_ElementTypeID" "', argument " "1"" of type '" "RobotRaconteur::ArrayMemoryBase *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ArrayMemoryBase > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ArrayMemoryBase > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ArrayMemoryBase * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ArrayMemoryBase > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ArrayMemoryBase * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::DataTypes)(arg1)->ElementTypeID();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_ArrayMemoryBase(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ArrayMemoryBase *arg1 = (RobotRaconteur::ArrayMemoryBase *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ArrayMemoryBase_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_ArrayMemoryBase" "', argument " "1"" of type '" "RobotRaconteur::ArrayMemoryBase *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ArrayMemoryBase > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ArrayMemoryBase > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ArrayMemoryBase * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ArrayMemoryBase > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ArrayMemoryBase * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    (void)arg1; delete smartarg1;
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *ArrayMemoryBase_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ArrayMemoryBase_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_MultiDimArrayMemoryBase_Dimensions(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MultiDimArrayMemoryBase *arg1 = (RobotRaconteur::MultiDimArrayMemoryBase *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< uint64_t,std::allocator< uint64_t > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MultiDimArrayMemoryBase_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MultiDimArrayMemoryBase_Dimensions" "', argument " "1"" of type '" "RobotRaconteur::MultiDimArrayMemoryBase *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MultiDimArrayMemoryBase > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MultiDimArrayMemoryBase > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MultiDimArrayMemoryBase * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MultiDimArrayMemoryBase > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MultiDimArrayMemoryBase * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->Dimensions();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MultiDimArrayMemoryBase_DimCount(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MultiDimArrayMemoryBase *arg1 = (RobotRaconteur::MultiDimArrayMemoryBase *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  uint64_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MultiDimArrayMemoryBase_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MultiDimArrayMemoryBase_DimCount" "', argument " "1"" of type '" "RobotRaconteur::MultiDimArrayMemoryBase *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MultiDimArrayMemoryBase > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MultiDimArrayMemoryBase > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MultiDimArrayMemoryBase * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MultiDimArrayMemoryBase > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MultiDimArrayMemoryBase * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (uint64_t)(arg1)->DimCount();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_MultiDimArrayMemoryBase_ElementTypeID(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MultiDimArrayMemoryBase *arg1 = (RobotRaconteur::MultiDimArrayMemoryBase *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::DataTypes result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MultiDimArrayMemoryBase_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "MultiDimArrayMemoryBase_ElementTypeID" "', argument " "1"" of type '" "RobotRaconteur::MultiDimArrayMemoryBase *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MultiDimArrayMemoryBase > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MultiDimArrayMemoryBase > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MultiDimArrayMemoryBase * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MultiDimArrayMemoryBase > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MultiDimArrayMemoryBase * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::DataTypes)(arg1)->ElementTypeID();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_MultiDimArrayMemoryBase(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::MultiDimArrayMemoryBase *arg1 = (RobotRaconteur::MultiDimArrayMemoryBase *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MultiDimArrayMemoryBase_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_MultiDimArrayMemoryBase" "', argument " "1"" of type '" "RobotRaconteur::MultiDimArrayMemoryBase *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MultiDimArrayMemoryBase > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MultiDimArrayMemoryBase > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MultiDimArrayMemoryBase * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::MultiDimArrayMemoryBase > * >(argp1);
      arg1 = const_cast< RobotRaconteur::MultiDimArrayMemoryBase * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    (void)arg1; delete smartarg1;
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *MultiDimArrayMemoryBase_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MultiDimArrayMemoryBase_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_WrappedArrayMemoryClientUtil_Read(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > *arg1 = 0 ;
  uint64_t arg2 ;
  uint64_t arg3 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > tempshared1 ;
  unsigned long val2 ;
  int ecode2 = 0 ;
  unsigned long val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  boost::shared_ptr< RobotRaconteur::RRBaseArray > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::RRBaseArray > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedArrayMemoryClientUtil_Read", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ArrayMemoryBase_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedArrayMemoryClientUtil_Read" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > * >(argp1) : &tempshared1;
    }
  }
  ecode2 = SWIG_AsVal_unsigned_SS_long(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedArrayMemoryClientUtil_Read" "', argument " "2"" of type '" "uint64_t""'");
  } 
  arg2 = static_cast< uint64_t >(val2);
  ecode3 = SWIG_AsVal_unsigned_SS_long(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedArrayMemoryClientUtil_Read" "', argument " "3"" of type '" "uint64_t""'");
  } 
  arg3 = static_cast< uint64_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::WrappedArrayMemoryClientUtil::Read((boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > const &)*arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::RRBaseArray >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::RRBaseArray >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::RRBaseArray >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedArrayMemoryClientUtil_Write(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > *arg1 = 0 ;
  uint64_t arg2 ;
  boost::intrusive_ptr< RobotRaconteur::RRBaseArray > *arg3 = 0 ;
  uint64_t arg4 ;
  uint64_t arg5 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > tempshared1 ;
  unsigned long val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::RRBaseArray > tempnull3 ;
  boost::intrusive_ptr< RobotRaconteur::RRBaseArray > temp3 ;
  int newmem3 = 0 ;
  unsigned long val4 ;
  int ecode4 = 0 ;
  unsigned long val5 ;
  int ecode5 = 0 ;
  PyObject *swig_obj[5] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedArrayMemoryClientUtil_Write", 5, 5, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ArrayMemoryBase_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedArrayMemoryClientUtil_Write" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > * >(argp1) : &tempshared1;
    }
  }
  ecode2 = SWIG_AsVal_unsigned_SS_long(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedArrayMemoryClientUtil_Write" "', argument " "2"" of type '" "uint64_t""'");
  } 
  arg2 = static_cast< uint64_t >(val2);
  
  // intrusive_ptr by reference
  //if ( swig_obj[2] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray >**)&swig_obj[2];
  //	temp3 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::RRBaseArray >(smartarg->get(), true);
  //	arg3 = &temp3;
  //} else {
  //  arg3 = &tempnull3;
  //}
  
  res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t,  0 , &newmem3);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedArrayMemoryClientUtil_Write" "', argument " "3"" of type '" "boost::intrusive_ptr< RobotRaconteur::RRBaseArray > const &""'");
  }
  if (argp3) {
    temp3 = boost::intrusive_ptr<  RobotRaconteur::RRBaseArray >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRBaseArray >* >(argp3)->get(),true);
    if (newmem3 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRBaseArray >* >(argp3);
    arg3 = &temp3;
  }
  else {
    arg3 = &tempnull3;
  }
  
  ecode4 = SWIG_AsVal_unsigned_SS_long(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedArrayMemoryClientUtil_Write" "', argument " "4"" of type '" "uint64_t""'");
  } 
  arg4 = static_cast< uint64_t >(val4);
  ecode5 = SWIG_AsVal_unsigned_SS_long(swig_obj[4], &val5);
  if (!SWIG_IsOK(ecode5)) {
    SWIG_exception_fail(SWIG_ArgError(ecode5), "in method '" "WrappedArrayMemoryClientUtil_Write" "', argument " "5"" of type '" "uint64_t""'");
  } 
  arg5 = static_cast< uint64_t >(val5);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur::WrappedArrayMemoryClientUtil::Write((boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > const &)*arg1,arg2,(boost::intrusive_ptr< RobotRaconteur::RRBaseArray > const &)*arg3,arg4,arg5);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedArrayMemoryClientUtil_Direction(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > tempshared1 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MemberDefinition_Direction result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ArrayMemoryBase_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedArrayMemoryClientUtil_Direction" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::MemberDefinition_Direction)RobotRaconteur::WrappedArrayMemoryClientUtil::Direction((boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedArrayMemoryClientUtil(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedArrayMemoryClientUtil *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_WrappedArrayMemoryClientUtil", 0, 0, 0)) SWIG_fail;
  
  try {
    result = (RobotRaconteur::WrappedArrayMemoryClientUtil *)new RobotRaconteur::WrappedArrayMemoryClientUtil();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedArrayMemoryClientUtil, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedArrayMemoryClientUtil(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedArrayMemoryClientUtil *arg1 = (RobotRaconteur::WrappedArrayMemoryClientUtil *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedArrayMemoryClientUtil, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedArrayMemoryClientUtil" "', argument " "1"" of type '" "RobotRaconteur::WrappedArrayMemoryClientUtil *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedArrayMemoryClientUtil * >(argp1);
  
  try {
    delete arg1;
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedArrayMemoryClientUtil_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedArrayMemoryClientUtil, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedArrayMemoryClientUtil_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedMultiDimArrayMemoryClientUtil_Read(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > *arg1 = 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > arg2 ;
  std::vector< uint64_t,std::allocator< uint64_t > > arg3 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > tempshared1 ;
  PyObject *swig_obj[3] ;
  boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedMultiDimArrayMemoryClientUtil_Read", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MultiDimArrayMemoryBase_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedMultiDimArrayMemoryClientUtil_Read" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > * >(argp1) : &tempshared1;
    }
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    int res = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res) || !ptr) {
      SWIG_exception_fail(SWIG_ArgError((ptr ? res : SWIG_TypeError)), "in method '" "WrappedMultiDimArrayMemoryClientUtil_Read" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > >""'"); 
    }
    arg2 = *ptr;
    if (SWIG_IsNewObj(res)) delete ptr;
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    int res = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res) || !ptr) {
      SWIG_exception_fail(SWIG_ArgError((ptr ? res : SWIG_TypeError)), "in method '" "WrappedMultiDimArrayMemoryClientUtil_Read" "', argument " "3"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > >""'"); 
    }
    arg3 = *ptr;
    if (SWIG_IsNewObj(res)) delete ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::WrappedMultiDimArrayMemoryClientUtil::Read((boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > const &)*arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::RRMultiDimArrayUntyped > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::RRMultiDimArrayUntyped >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRMultiDimArrayUntyped_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedMultiDimArrayMemoryClientUtil_Write(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > *arg1 = 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > arg2 ;
  boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > *arg3 = 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > arg4 ;
  std::vector< uint64_t,std::allocator< uint64_t > > arg5 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > tempshared1 ;
  void *argp3 ;
  int res3 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > tempshared3 ;
  PyObject *swig_obj[5] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedMultiDimArrayMemoryClientUtil_Write", 5, 5, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MultiDimArrayMemoryBase_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedMultiDimArrayMemoryClientUtil_Write" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > * >(argp1) : &tempshared1;
    }
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    int res = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res) || !ptr) {
      SWIG_exception_fail(SWIG_ArgError((ptr ? res : SWIG_TypeError)), "in method '" "WrappedMultiDimArrayMemoryClientUtil_Write" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > >""'"); 
    }
    arg2 = *ptr;
    if (SWIG_IsNewObj(res)) delete ptr;
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRMultiDimArrayUntyped_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedMultiDimArrayMemoryClientUtil_Write" "', argument " "3"" of type '" "boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > * >(argp3);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > * >(argp3) : &tempshared3;
    }
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    int res = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res) || !ptr) {
      SWIG_exception_fail(SWIG_ArgError((ptr ? res : SWIG_TypeError)), "in method '" "WrappedMultiDimArrayMemoryClientUtil_Write" "', argument " "4"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > >""'"); 
    }
    arg4 = *ptr;
    if (SWIG_IsNewObj(res)) delete ptr;
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    int res = swig::asptr(swig_obj[4], &ptr);
    if (!SWIG_IsOK(res) || !ptr) {
      SWIG_exception_fail(SWIG_ArgError((ptr ? res : SWIG_TypeError)), "in method '" "WrappedMultiDimArrayMemoryClientUtil_Write" "', argument " "5"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > >""'"); 
    }
    arg5 = *ptr;
    if (SWIG_IsNewObj(res)) delete ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur::WrappedMultiDimArrayMemoryClientUtil::Write((boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > const &)*arg1,arg2,(boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > const &)*arg3,arg4,arg5);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedMultiDimArrayMemoryClientUtil_Direction(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > tempshared1 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MemberDefinition_Direction result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MultiDimArrayMemoryBase_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedMultiDimArrayMemoryClientUtil_Direction" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::MemberDefinition_Direction)RobotRaconteur::WrappedMultiDimArrayMemoryClientUtil::Direction((boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedMultiDimArrayMemoryClientUtil(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedMultiDimArrayMemoryClientUtil *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_WrappedMultiDimArrayMemoryClientUtil", 0, 0, 0)) SWIG_fail;
  
  try {
    result = (RobotRaconteur::WrappedMultiDimArrayMemoryClientUtil *)new RobotRaconteur::WrappedMultiDimArrayMemoryClientUtil();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryClientUtil, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedMultiDimArrayMemoryClientUtil(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedMultiDimArrayMemoryClientUtil *arg1 = (RobotRaconteur::WrappedMultiDimArrayMemoryClientUtil *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryClientUtil, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedMultiDimArrayMemoryClientUtil" "', argument " "1"" of type '" "RobotRaconteur::WrappedMultiDimArrayMemoryClientUtil *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedMultiDimArrayMemoryClientUtil * >(argp1);
  
  try {
    delete arg1;
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedMultiDimArrayMemoryClientUtil_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryClientUtil, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedMultiDimArrayMemoryClientUtil_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedPodArrayMemoryClientBuffer_UnpackReadResult(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodArrayMemoryClientBuffer *arg1 = (RobotRaconteur::WrappedPodArrayMemoryClientBuffer *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > arg2 ;
  uint64_t arg3 ;
  uint64_t arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  int newmem2 = 0 ;
  unsigned long val3 ;
  int ecode3 = 0 ;
  unsigned long val4 ;
  int ecode4 = 0 ;
  PyObject *swig_obj[4] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPodArrayMemoryClientBuffer_UnpackReadResult", 4, 4, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodArrayMemoryClientBuffer, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodArrayMemoryClientBuffer_UnpackReadResult" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodArrayMemoryClientBuffer *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodArrayMemoryClientBuffer * >(argp1);
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >**)&swig_obj[1];
  //if (smartarg) {
  //	arg2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(smartarg->get(), true);
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPodArrayMemoryClientBuffer_UnpackReadResult" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >""'");
  }
  if (argp2) arg2 = boost::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp2)->get(),true);
  if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp2);
  
  ecode3 = SWIG_AsVal_unsigned_SS_long(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedPodArrayMemoryClientBuffer_UnpackReadResult" "', argument " "3"" of type '" "uint64_t""'");
  } 
  arg3 = static_cast< uint64_t >(val3);
  ecode4 = SWIG_AsVal_unsigned_SS_long(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedPodArrayMemoryClientBuffer_UnpackReadResult" "', argument " "4"" of type '" "uint64_t""'");
  } 
  arg4 = static_cast< uint64_t >(val4);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedPodArrayMemoryClientBuffer::UnpackReadResult");
      } else {
        (arg1)->UnpackReadResult(arg2,arg3,arg4);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPodArrayMemoryClientBuffer_PackWriteRequest(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodArrayMemoryClientBuffer *arg1 = (RobotRaconteur::WrappedPodArrayMemoryClientBuffer *) 0 ;
  uint64_t arg2 ;
  uint64_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  unsigned long val2 ;
  int ecode2 = 0 ;
  unsigned long val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  Swig::Director *director = 0;
  bool upcall = false;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPodArrayMemoryClientBuffer_PackWriteRequest", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodArrayMemoryClientBuffer, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodArrayMemoryClientBuffer_PackWriteRequest" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodArrayMemoryClientBuffer *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodArrayMemoryClientBuffer * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_long(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedPodArrayMemoryClientBuffer_PackWriteRequest" "', argument " "2"" of type '" "uint64_t""'");
  } 
  arg2 = static_cast< uint64_t >(val2);
  ecode3 = SWIG_AsVal_unsigned_SS_long(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedPodArrayMemoryClientBuffer_PackWriteRequest" "', argument " "3"" of type '" "uint64_t""'");
  } 
  arg3 = static_cast< uint64_t >(val3);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedPodArrayMemoryClientBuffer::PackWriteRequest");
      } else {
        result = (arg1)->PackWriteRequest(arg2,arg3);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPodArrayMemoryClientBuffer_GetBufferLength(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodArrayMemoryClientBuffer *arg1 = (RobotRaconteur::WrappedPodArrayMemoryClientBuffer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  Swig::Director *director = 0;
  bool upcall = false;
  uint64_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodArrayMemoryClientBuffer, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodArrayMemoryClientBuffer_GetBufferLength" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodArrayMemoryClientBuffer *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodArrayMemoryClientBuffer * >(argp1);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedPodArrayMemoryClientBuffer::GetBufferLength");
      } else {
        result = (uint64_t)(arg1)->GetBufferLength();
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedPodArrayMemoryClientBuffer(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodArrayMemoryClientBuffer *arg1 = (RobotRaconteur::WrappedPodArrayMemoryClientBuffer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodArrayMemoryClientBuffer, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedPodArrayMemoryClientBuffer" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodArrayMemoryClientBuffer *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodArrayMemoryClientBuffer * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedPodArrayMemoryClientBuffer(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedPodArrayMemoryClientBuffer *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedPodArrayMemoryClientBuffer *)new SwigDirector_WrappedPodArrayMemoryClientBuffer(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedPodArrayMemoryClientBuffer, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedPodArrayMemoryClientBuffer(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodArrayMemoryClientBuffer *arg1 = (RobotRaconteur::WrappedPodArrayMemoryClientBuffer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodArrayMemoryClientBuffer, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedPodArrayMemoryClientBuffer" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodArrayMemoryClientBuffer *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodArrayMemoryClientBuffer * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedPodArrayMemoryClientBuffer_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedPodArrayMemoryClientBuffer, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedPodArrayMemoryClientBuffer_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedPodArrayMemoryClient_Length(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodArrayMemoryClient *arg1 = (RobotRaconteur::WrappedPodArrayMemoryClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPodArrayMemoryClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPodArrayMemoryClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  uint64_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPodArrayMemoryClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodArrayMemoryClient_Length" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodArrayMemoryClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodArrayMemoryClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPodArrayMemoryClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPodArrayMemoryClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (uint64_t)(arg1)->Length();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPodArrayMemoryClient_Direction(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodArrayMemoryClient *arg1 = (RobotRaconteur::WrappedPodArrayMemoryClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPodArrayMemoryClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPodArrayMemoryClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MemberDefinition_Direction result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPodArrayMemoryClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodArrayMemoryClient_Direction" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodArrayMemoryClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodArrayMemoryClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPodArrayMemoryClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPodArrayMemoryClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::MemberDefinition_Direction)(arg1)->Direction();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPodArrayMemoryClient_Read(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodArrayMemoryClient *arg1 = (RobotRaconteur::WrappedPodArrayMemoryClient *) 0 ;
  uint64_t arg2 ;
  RobotRaconteur::WrappedPodArrayMemoryClientBuffer *arg3 = (RobotRaconteur::WrappedPodArrayMemoryClientBuffer *) 0 ;
  uint64_t arg4 ;
  uint64_t arg5 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPodArrayMemoryClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPodArrayMemoryClient > *smartarg1 = 0 ;
  unsigned long val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  unsigned long val4 ;
  int ecode4 = 0 ;
  unsigned long val5 ;
  int ecode5 = 0 ;
  PyObject *swig_obj[5] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPodArrayMemoryClient_Read", 5, 5, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPodArrayMemoryClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodArrayMemoryClient_Read" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodArrayMemoryClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodArrayMemoryClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPodArrayMemoryClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPodArrayMemoryClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_unsigned_SS_long(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedPodArrayMemoryClient_Read" "', argument " "2"" of type '" "uint64_t""'");
  } 
  arg2 = static_cast< uint64_t >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__WrappedPodArrayMemoryClientBuffer, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedPodArrayMemoryClient_Read" "', argument " "3"" of type '" "RobotRaconteur::WrappedPodArrayMemoryClientBuffer *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::WrappedPodArrayMemoryClientBuffer * >(argp3);
  ecode4 = SWIG_AsVal_unsigned_SS_long(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedPodArrayMemoryClient_Read" "', argument " "4"" of type '" "uint64_t""'");
  } 
  arg4 = static_cast< uint64_t >(val4);
  ecode5 = SWIG_AsVal_unsigned_SS_long(swig_obj[4], &val5);
  if (!SWIG_IsOK(ecode5)) {
    SWIG_exception_fail(SWIG_ArgError(ecode5), "in method '" "WrappedPodArrayMemoryClient_Read" "', argument " "5"" of type '" "uint64_t""'");
  } 
  arg5 = static_cast< uint64_t >(val5);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Read(arg2,arg3,arg4,arg5);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPodArrayMemoryClient_Write(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodArrayMemoryClient *arg1 = (RobotRaconteur::WrappedPodArrayMemoryClient *) 0 ;
  uint64_t arg2 ;
  RobotRaconteur::WrappedPodArrayMemoryClientBuffer *arg3 = (RobotRaconteur::WrappedPodArrayMemoryClientBuffer *) 0 ;
  uint64_t arg4 ;
  uint64_t arg5 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPodArrayMemoryClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPodArrayMemoryClient > *smartarg1 = 0 ;
  unsigned long val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  unsigned long val4 ;
  int ecode4 = 0 ;
  unsigned long val5 ;
  int ecode5 = 0 ;
  PyObject *swig_obj[5] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPodArrayMemoryClient_Write", 5, 5, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPodArrayMemoryClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodArrayMemoryClient_Write" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodArrayMemoryClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodArrayMemoryClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPodArrayMemoryClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPodArrayMemoryClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_unsigned_SS_long(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedPodArrayMemoryClient_Write" "', argument " "2"" of type '" "uint64_t""'");
  } 
  arg2 = static_cast< uint64_t >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__WrappedPodArrayMemoryClientBuffer, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedPodArrayMemoryClient_Write" "', argument " "3"" of type '" "RobotRaconteur::WrappedPodArrayMemoryClientBuffer *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::WrappedPodArrayMemoryClientBuffer * >(argp3);
  ecode4 = SWIG_AsVal_unsigned_SS_long(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedPodArrayMemoryClient_Write" "', argument " "4"" of type '" "uint64_t""'");
  } 
  arg4 = static_cast< uint64_t >(val4);
  ecode5 = SWIG_AsVal_unsigned_SS_long(swig_obj[4], &val5);
  if (!SWIG_IsOK(ecode5)) {
    SWIG_exception_fail(SWIG_ArgError(ecode5), "in method '" "WrappedPodArrayMemoryClient_Write" "', argument " "5"" of type '" "uint64_t""'");
  } 
  arg5 = static_cast< uint64_t >(val5);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Write(arg2,arg3,arg4,arg5);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedPodArrayMemoryClient(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodArrayMemoryClient *arg1 = (RobotRaconteur::WrappedPodArrayMemoryClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPodArrayMemoryClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPodArrayMemoryClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPodArrayMemoryClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedPodArrayMemoryClient" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodArrayMemoryClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodArrayMemoryClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPodArrayMemoryClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPodArrayMemoryClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    (void)arg1; delete smartarg1;
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedPodArrayMemoryClient_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPodArrayMemoryClient_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_WrappedPodMultiDimArrayMemoryClientBuffer_UnpackReadResult(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer *arg1 = (RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > arg2 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg3 = 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  int newmem2 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  int res4 = SWIG_OLDOBJ ;
  PyObject *swig_obj[4] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPodMultiDimArrayMemoryClientBuffer_UnpackReadResult", 4, 4, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryClientBuffer, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodMultiDimArrayMemoryClientBuffer_UnpackReadResult" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer * >(argp1);
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >**)&swig_obj[1];
  //if (smartarg) {
  //	arg2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(smartarg->get(), true);
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPodMultiDimArrayMemoryClientBuffer_UnpackReadResult" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >""'");
  }
  if (argp2) arg2 = boost::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp2)->get(),true);
  if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp2);
  
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedPodMultiDimArrayMemoryClientBuffer_UnpackReadResult" "', argument " "3"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedPodMultiDimArrayMemoryClientBuffer_UnpackReadResult" "', argument " "3"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg3 = ptr;
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedPodMultiDimArrayMemoryClientBuffer_UnpackReadResult" "', argument " "4"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedPodMultiDimArrayMemoryClientBuffer_UnpackReadResult" "', argument " "4"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg4 = ptr;
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer::UnpackReadResult");
      } else {
        (arg1)->UnpackReadResult(arg2,(std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg3,(std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg4);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPodMultiDimArrayMemoryClientBuffer_PackWriteRequest(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer *arg1 = (RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer *) 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg2 = 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  PyObject *swig_obj[3] ;
  Swig::Director *director = 0;
  bool upcall = false;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPodMultiDimArrayMemoryClientBuffer_PackWriteRequest", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryClientBuffer, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodMultiDimArrayMemoryClientBuffer_PackWriteRequest" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer * >(argp1);
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPodMultiDimArrayMemoryClientBuffer_PackWriteRequest" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedPodMultiDimArrayMemoryClientBuffer_PackWriteRequest" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedPodMultiDimArrayMemoryClientBuffer_PackWriteRequest" "', argument " "3"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedPodMultiDimArrayMemoryClientBuffer_PackWriteRequest" "', argument " "3"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg3 = ptr;
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer::PackWriteRequest");
      } else {
        result = (arg1)->PackWriteRequest((std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg2,(std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg3);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, SWIG_POINTER_OWN);
  
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedPodMultiDimArrayMemoryClientBuffer(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer *arg1 = (RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryClientBuffer, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedPodMultiDimArrayMemoryClientBuffer" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedPodMultiDimArrayMemoryClientBuffer(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer *)new SwigDirector_WrappedPodMultiDimArrayMemoryClientBuffer(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryClientBuffer, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedPodMultiDimArrayMemoryClientBuffer(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer *arg1 = (RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryClientBuffer, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedPodMultiDimArrayMemoryClientBuffer" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedPodMultiDimArrayMemoryClientBuffer_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryClientBuffer, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedPodMultiDimArrayMemoryClientBuffer_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedPodMultiDimArrayMemoryClient_Dimensions(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient *arg1 = (RobotRaconteur::WrappedPodMultiDimArrayMemoryClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< uint64_t,std::allocator< uint64_t > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPodMultiDimArrayMemoryClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodMultiDimArrayMemoryClient_Dimensions" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodMultiDimArrayMemoryClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->Dimensions();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPodMultiDimArrayMemoryClient_DimCount(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient *arg1 = (RobotRaconteur::WrappedPodMultiDimArrayMemoryClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  uint64_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPodMultiDimArrayMemoryClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodMultiDimArrayMemoryClient_DimCount" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodMultiDimArrayMemoryClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (uint64_t)(arg1)->DimCount();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPodMultiDimArrayMemoryClient_Direction(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient *arg1 = (RobotRaconteur::WrappedPodMultiDimArrayMemoryClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MemberDefinition_Direction result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPodMultiDimArrayMemoryClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodMultiDimArrayMemoryClient_Direction" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodMultiDimArrayMemoryClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::MemberDefinition_Direction)(arg1)->Direction();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPodMultiDimArrayMemoryClient_Read(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient *arg1 = (RobotRaconteur::WrappedPodMultiDimArrayMemoryClient *) 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg2 = 0 ;
  RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer *arg3 = (RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer *) 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg4 = 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg5 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  int res5 = SWIG_OLDOBJ ;
  PyObject *swig_obj[5] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPodMultiDimArrayMemoryClient_Read", 5, 5, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPodMultiDimArrayMemoryClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodMultiDimArrayMemoryClient_Read" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodMultiDimArrayMemoryClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPodMultiDimArrayMemoryClient_Read" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedPodMultiDimArrayMemoryClient_Read" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryClientBuffer, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedPodMultiDimArrayMemoryClient_Read" "', argument " "3"" of type '" "RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer * >(argp3);
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedPodMultiDimArrayMemoryClient_Read" "', argument " "4"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedPodMultiDimArrayMemoryClient_Read" "', argument " "4"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg4 = ptr;
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res5 = swig::asptr(swig_obj[4], &ptr);
    if (!SWIG_IsOK(res5)) {
      SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "WrappedPodMultiDimArrayMemoryClient_Read" "', argument " "5"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedPodMultiDimArrayMemoryClient_Read" "', argument " "5"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg5 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Read((std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg2,arg3,(std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg4,(std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg5);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res4)) delete arg4;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res4)) delete arg4;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPodMultiDimArrayMemoryClient_Write(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient *arg1 = (RobotRaconteur::WrappedPodMultiDimArrayMemoryClient *) 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg2 = 0 ;
  RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer *arg3 = (RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer *) 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg4 = 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg5 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  int res5 = SWIG_OLDOBJ ;
  PyObject *swig_obj[5] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPodMultiDimArrayMemoryClient_Write", 5, 5, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPodMultiDimArrayMemoryClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodMultiDimArrayMemoryClient_Write" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodMultiDimArrayMemoryClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPodMultiDimArrayMemoryClient_Write" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedPodMultiDimArrayMemoryClient_Write" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryClientBuffer, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedPodMultiDimArrayMemoryClient_Write" "', argument " "3"" of type '" "RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer * >(argp3);
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedPodMultiDimArrayMemoryClient_Write" "', argument " "4"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedPodMultiDimArrayMemoryClient_Write" "', argument " "4"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg4 = ptr;
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res5 = swig::asptr(swig_obj[4], &ptr);
    if (!SWIG_IsOK(res5)) {
      SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "WrappedPodMultiDimArrayMemoryClient_Write" "', argument " "5"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedPodMultiDimArrayMemoryClient_Write" "', argument " "5"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg5 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Write((std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg2,arg3,(std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg4,(std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg5);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res4)) delete arg4;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res4)) delete arg4;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedPodMultiDimArrayMemoryClient(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient *arg1 = (RobotRaconteur::WrappedPodMultiDimArrayMemoryClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPodMultiDimArrayMemoryClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedPodMultiDimArrayMemoryClient" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodMultiDimArrayMemoryClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    (void)arg1; delete smartarg1;
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedPodMultiDimArrayMemoryClient_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPodMultiDimArrayMemoryClient_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_delete_WrappedArrayMemoryDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedArrayMemoryDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedArrayMemoryDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedArrayMemoryDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedArrayMemoryDirector_Length(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  Swig::Director *director = 0;
  bool upcall = false;
  uint64_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedArrayMemoryDirector_Length" "', argument " "1"" of type '" "RobotRaconteur::WrappedArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedArrayMemoryDirector * >(argp1);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedArrayMemoryDirector::Length");
      } else {
        result = (uint64_t)(arg1)->Length();
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedArrayMemoryDirector_Read(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedArrayMemoryDirector *) 0 ;
  uint64_t arg2 ;
  boost::intrusive_ptr< RobotRaconteur::RRBaseArray > arg3 ;
  uint64_t arg4 ;
  uint64_t arg5 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  unsigned long val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  int newmem3 = 0 ;
  unsigned long val4 ;
  int ecode4 = 0 ;
  unsigned long val5 ;
  int ecode5 = 0 ;
  PyObject *swig_obj[5] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedArrayMemoryDirector_Read", 5, 5, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedArrayMemoryDirector_Read" "', argument " "1"" of type '" "RobotRaconteur::WrappedArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedArrayMemoryDirector * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_long(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedArrayMemoryDirector_Read" "', argument " "2"" of type '" "uint64_t""'");
  } 
  arg2 = static_cast< uint64_t >(val2);
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray >**)&swig_obj[2];
  //if (smartarg) {
  //	arg3 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::RRBaseArray >(smartarg->get(), true);
  //}
  
  res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t,  0 , &newmem3);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedArrayMemoryDirector_Read" "', argument " "3"" of type '" "boost::intrusive_ptr< RobotRaconteur::RRBaseArray >""'");
  }
  if (argp3) arg3 = boost::intrusive_ptr<  RobotRaconteur::RRBaseArray >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRBaseArray >* >(argp3)->get(),true);
  if (newmem3 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRBaseArray >* >(argp3);
  
  ecode4 = SWIG_AsVal_unsigned_SS_long(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedArrayMemoryDirector_Read" "', argument " "4"" of type '" "uint64_t""'");
  } 
  arg4 = static_cast< uint64_t >(val4);
  ecode5 = SWIG_AsVal_unsigned_SS_long(swig_obj[4], &val5);
  if (!SWIG_IsOK(ecode5)) {
    SWIG_exception_fail(SWIG_ArgError(ecode5), "in method '" "WrappedArrayMemoryDirector_Read" "', argument " "5"" of type '" "uint64_t""'");
  } 
  arg5 = static_cast< uint64_t >(val5);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedArrayMemoryDirector::Read");
      } else {
        (arg1)->Read(arg2,arg3,arg4,arg5);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedArrayMemoryDirector_Write(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedArrayMemoryDirector *) 0 ;
  uint64_t arg2 ;
  boost::intrusive_ptr< RobotRaconteur::RRBaseArray > arg3 ;
  uint64_t arg4 ;
  uint64_t arg5 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  unsigned long val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  int newmem3 = 0 ;
  unsigned long val4 ;
  int ecode4 = 0 ;
  unsigned long val5 ;
  int ecode5 = 0 ;
  PyObject *swig_obj[5] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedArrayMemoryDirector_Write", 5, 5, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedArrayMemoryDirector_Write" "', argument " "1"" of type '" "RobotRaconteur::WrappedArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedArrayMemoryDirector * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_long(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedArrayMemoryDirector_Write" "', argument " "2"" of type '" "uint64_t""'");
  } 
  arg2 = static_cast< uint64_t >(val2);
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray >**)&swig_obj[2];
  //if (smartarg) {
  //	arg3 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::RRBaseArray >(smartarg->get(), true);
  //}
  
  res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t,  0 , &newmem3);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedArrayMemoryDirector_Write" "', argument " "3"" of type '" "boost::intrusive_ptr< RobotRaconteur::RRBaseArray >""'");
  }
  if (argp3) arg3 = boost::intrusive_ptr<  RobotRaconteur::RRBaseArray >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRBaseArray >* >(argp3)->get(),true);
  if (newmem3 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRBaseArray >* >(argp3);
  
  ecode4 = SWIG_AsVal_unsigned_SS_long(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedArrayMemoryDirector_Write" "', argument " "4"" of type '" "uint64_t""'");
  } 
  arg4 = static_cast< uint64_t >(val4);
  ecode5 = SWIG_AsVal_unsigned_SS_long(swig_obj[4], &val5);
  if (!SWIG_IsOK(ecode5)) {
    SWIG_exception_fail(SWIG_ArgError(ecode5), "in method '" "WrappedArrayMemoryDirector_Write" "', argument " "5"" of type '" "uint64_t""'");
  } 
  arg5 = static_cast< uint64_t >(val5);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedArrayMemoryDirector::Write");
      } else {
        (arg1)->Write(arg2,arg3,arg4,arg5);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedArrayMemoryDirector_objectheapid_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedArrayMemoryDirector *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedArrayMemoryDirector_objectheapid_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedArrayMemoryDirector_objectheapid_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedArrayMemoryDirector * >(argp1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedArrayMemoryDirector_objectheapid_set" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->objectheapid = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedArrayMemoryDirector_objectheapid_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedArrayMemoryDirector_objectheapid_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedArrayMemoryDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (int32_t) ((arg1)->objectheapid);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedArrayMemoryDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedArrayMemoryDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedArrayMemoryDirector *)new SwigDirector_WrappedArrayMemoryDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedArrayMemoryDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedArrayMemoryDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedArrayMemoryDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedArrayMemoryDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedArrayMemoryDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedArrayMemoryDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedArrayMemoryDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedMultiDimArrayMemoryParams_memorypos_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedMultiDimArrayMemoryParams *arg1 = (RobotRaconteur::WrappedMultiDimArrayMemoryParams *) 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg2 = (std::vector< uint64_t,std::allocator< uint64_t > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedMultiDimArrayMemoryParams_memorypos_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryParams, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedMultiDimArrayMemoryParams_memorypos_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedMultiDimArrayMemoryParams *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedMultiDimArrayMemoryParams * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedMultiDimArrayMemoryParams_memorypos_set" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< uint64_t,std::allocator< uint64_t > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->memorypos = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedMultiDimArrayMemoryParams_memorypos_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedMultiDimArrayMemoryParams *arg1 = (RobotRaconteur::WrappedMultiDimArrayMemoryParams *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< uint64_t,std::allocator< uint64_t > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryParams, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedMultiDimArrayMemoryParams_memorypos_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedMultiDimArrayMemoryParams *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedMultiDimArrayMemoryParams * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< uint64_t,std::allocator< uint64_t > > *)& ((arg1)->memorypos);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedMultiDimArrayMemoryParams_buffer_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedMultiDimArrayMemoryParams *arg1 = (RobotRaconteur::WrappedMultiDimArrayMemoryParams *) 0 ;
  boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedMultiDimArrayMemoryParams_buffer_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryParams, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedMultiDimArrayMemoryParams_buffer_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedMultiDimArrayMemoryParams *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedMultiDimArrayMemoryParams * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRMultiDimArrayUntyped_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedMultiDimArrayMemoryParams_buffer_set" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > * >(argp2) : &tempshared2;
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->buffer = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedMultiDimArrayMemoryParams_buffer_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedMultiDimArrayMemoryParams *arg1 = (RobotRaconteur::WrappedMultiDimArrayMemoryParams *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryParams, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedMultiDimArrayMemoryParams_buffer_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedMultiDimArrayMemoryParams *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedMultiDimArrayMemoryParams * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > *) & ((arg1)->buffer);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  {
    boost::shared_ptr<  RobotRaconteur::RRMultiDimArrayUntyped > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::RRMultiDimArrayUntyped >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRMultiDimArrayUntyped_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedMultiDimArrayMemoryParams_bufferpos_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedMultiDimArrayMemoryParams *arg1 = (RobotRaconteur::WrappedMultiDimArrayMemoryParams *) 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg2 = (std::vector< uint64_t,std::allocator< uint64_t > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedMultiDimArrayMemoryParams_bufferpos_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryParams, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedMultiDimArrayMemoryParams_bufferpos_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedMultiDimArrayMemoryParams *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedMultiDimArrayMemoryParams * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedMultiDimArrayMemoryParams_bufferpos_set" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< uint64_t,std::allocator< uint64_t > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->bufferpos = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedMultiDimArrayMemoryParams_bufferpos_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedMultiDimArrayMemoryParams *arg1 = (RobotRaconteur::WrappedMultiDimArrayMemoryParams *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< uint64_t,std::allocator< uint64_t > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryParams, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedMultiDimArrayMemoryParams_bufferpos_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedMultiDimArrayMemoryParams *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedMultiDimArrayMemoryParams * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< uint64_t,std::allocator< uint64_t > > *)& ((arg1)->bufferpos);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedMultiDimArrayMemoryParams_count_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedMultiDimArrayMemoryParams *arg1 = (RobotRaconteur::WrappedMultiDimArrayMemoryParams *) 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg2 = (std::vector< uint64_t,std::allocator< uint64_t > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedMultiDimArrayMemoryParams_count_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryParams, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedMultiDimArrayMemoryParams_count_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedMultiDimArrayMemoryParams *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedMultiDimArrayMemoryParams * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedMultiDimArrayMemoryParams_count_set" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< uint64_t,std::allocator< uint64_t > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->count = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedMultiDimArrayMemoryParams_count_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedMultiDimArrayMemoryParams *arg1 = (RobotRaconteur::WrappedMultiDimArrayMemoryParams *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< uint64_t,std::allocator< uint64_t > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryParams, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedMultiDimArrayMemoryParams_count_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedMultiDimArrayMemoryParams *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedMultiDimArrayMemoryParams * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< uint64_t,std::allocator< uint64_t > > *)& ((arg1)->count);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedMultiDimArrayMemoryParams(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedMultiDimArrayMemoryParams *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_WrappedMultiDimArrayMemoryParams", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedMultiDimArrayMemoryParams *)new RobotRaconteur::WrappedMultiDimArrayMemoryParams();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryParams, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedMultiDimArrayMemoryParams(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedMultiDimArrayMemoryParams *arg1 = (RobotRaconteur::WrappedMultiDimArrayMemoryParams *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryParams, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedMultiDimArrayMemoryParams" "', argument " "1"" of type '" "RobotRaconteur::WrappedMultiDimArrayMemoryParams *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedMultiDimArrayMemoryParams * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedMultiDimArrayMemoryParams_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryParams, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedMultiDimArrayMemoryParams_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_WrappedMultiDimArrayMemoryDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedMultiDimArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedMultiDimArrayMemoryDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedMultiDimArrayMemoryDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedMultiDimArrayMemoryDirector_Dimensions(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedMultiDimArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  Swig::Director *director = 0;
  bool upcall = false;
  std::vector< uint64_t,std::allocator< uint64_t > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedMultiDimArrayMemoryDirector_Dimensions" "', argument " "1"" of type '" "RobotRaconteur::WrappedMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedMultiDimArrayMemoryDirector * >(argp1);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedMultiDimArrayMemoryDirector::Dimensions");
      } else {
        result = (arg1)->Dimensions();
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedMultiDimArrayMemoryDirector_DimCount(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedMultiDimArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  Swig::Director *director = 0;
  bool upcall = false;
  uint64_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedMultiDimArrayMemoryDirector_DimCount" "', argument " "1"" of type '" "RobotRaconteur::WrappedMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedMultiDimArrayMemoryDirector * >(argp1);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedMultiDimArrayMemoryDirector::DimCount");
      } else {
        result = (uint64_t)(arg1)->DimCount();
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedMultiDimArrayMemoryDirector_Read(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedMultiDimArrayMemoryDirector *) 0 ;
  RobotRaconteur::WrappedMultiDimArrayMemoryParams *arg2 = (RobotRaconteur::WrappedMultiDimArrayMemoryParams *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedMultiDimArrayMemoryDirector_Read", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedMultiDimArrayMemoryDirector_Read" "', argument " "1"" of type '" "RobotRaconteur::WrappedMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedMultiDimArrayMemoryDirector * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryParams, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedMultiDimArrayMemoryDirector_Read" "', argument " "2"" of type '" "RobotRaconteur::WrappedMultiDimArrayMemoryParams *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedMultiDimArrayMemoryParams * >(argp2);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedMultiDimArrayMemoryDirector::Read");
      } else {
        (arg1)->Read(arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedMultiDimArrayMemoryDirector_Write(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedMultiDimArrayMemoryDirector *) 0 ;
  RobotRaconteur::WrappedMultiDimArrayMemoryParams *arg2 = (RobotRaconteur::WrappedMultiDimArrayMemoryParams *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedMultiDimArrayMemoryDirector_Write", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedMultiDimArrayMemoryDirector_Write" "', argument " "1"" of type '" "RobotRaconteur::WrappedMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedMultiDimArrayMemoryDirector * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryParams, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedMultiDimArrayMemoryDirector_Write" "', argument " "2"" of type '" "RobotRaconteur::WrappedMultiDimArrayMemoryParams *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedMultiDimArrayMemoryParams * >(argp2);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedMultiDimArrayMemoryDirector::Write");
      } else {
        (arg1)->Write(arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedMultiDimArrayMemoryDirector_objectheapid_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedMultiDimArrayMemoryDirector *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedMultiDimArrayMemoryDirector_objectheapid_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedMultiDimArrayMemoryDirector_objectheapid_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedMultiDimArrayMemoryDirector * >(argp1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedMultiDimArrayMemoryDirector_objectheapid_set" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->objectheapid = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedMultiDimArrayMemoryDirector_objectheapid_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedMultiDimArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedMultiDimArrayMemoryDirector_objectheapid_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedMultiDimArrayMemoryDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (int32_t) ((arg1)->objectheapid);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedMultiDimArrayMemoryDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedMultiDimArrayMemoryDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedMultiDimArrayMemoryDirector *)new SwigDirector_WrappedMultiDimArrayMemoryDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedMultiDimArrayMemoryDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedMultiDimArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedMultiDimArrayMemoryDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedMultiDimArrayMemoryDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedMultiDimArrayMemoryDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedMultiDimArrayMemoryDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_new_WrappedPodArrayMemoryDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedPodArrayMemoryDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedPodArrayMemoryDirector *)new SwigDirector_WrappedPodArrayMemoryDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedPodArrayMemoryDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedPodArrayMemoryDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedPodArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodArrayMemoryDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedPodArrayMemoryDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodArrayMemoryDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPodArrayMemoryDirector_Length(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedPodArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  Swig::Director *director = 0;
  bool upcall = false;
  uint64_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodArrayMemoryDirector_Length" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodArrayMemoryDirector * >(argp1);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedPodArrayMemoryDirector::Length");
      } else {
        result = (uint64_t)(arg1)->Length();
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPodArrayMemoryDirector_Read(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedPodArrayMemoryDirector *) 0 ;
  uint64_t arg2 ;
  uint64_t arg3 ;
  uint64_t arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  unsigned long val2 ;
  int ecode2 = 0 ;
  unsigned long val3 ;
  int ecode3 = 0 ;
  unsigned long val4 ;
  int ecode4 = 0 ;
  PyObject *swig_obj[4] ;
  Swig::Director *director = 0;
  bool upcall = false;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPodArrayMemoryDirector_Read", 4, 4, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodArrayMemoryDirector_Read" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodArrayMemoryDirector * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_long(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedPodArrayMemoryDirector_Read" "', argument " "2"" of type '" "uint64_t""'");
  } 
  arg2 = static_cast< uint64_t >(val2);
  ecode3 = SWIG_AsVal_unsigned_SS_long(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedPodArrayMemoryDirector_Read" "', argument " "3"" of type '" "uint64_t""'");
  } 
  arg3 = static_cast< uint64_t >(val3);
  ecode4 = SWIG_AsVal_unsigned_SS_long(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedPodArrayMemoryDirector_Read" "', argument " "4"" of type '" "uint64_t""'");
  } 
  arg4 = static_cast< uint64_t >(val4);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedPodArrayMemoryDirector::Read");
      } else {
        result = (arg1)->Read(arg2,arg3,arg4);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPodArrayMemoryDirector_Write(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedPodArrayMemoryDirector *) 0 ;
  uint64_t arg2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > arg3 ;
  uint64_t arg4 ;
  uint64_t arg5 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  unsigned long val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  int newmem3 = 0 ;
  unsigned long val4 ;
  int ecode4 = 0 ;
  unsigned long val5 ;
  int ecode5 = 0 ;
  PyObject *swig_obj[5] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPodArrayMemoryDirector_Write", 5, 5, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodArrayMemoryDirector_Write" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodArrayMemoryDirector * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_long(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedPodArrayMemoryDirector_Write" "', argument " "2"" of type '" "uint64_t""'");
  } 
  arg2 = static_cast< uint64_t >(val2);
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >**)&swig_obj[2];
  //if (smartarg) {
  //	arg3 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(smartarg->get(), true);
  //}
  
  res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t,  0 , &newmem3);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedPodArrayMemoryDirector_Write" "', argument " "3"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >""'");
  }
  if (argp3) arg3 = boost::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp3)->get(),true);
  if (newmem3 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp3);
  
  ecode4 = SWIG_AsVal_unsigned_SS_long(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedPodArrayMemoryDirector_Write" "', argument " "4"" of type '" "uint64_t""'");
  } 
  arg4 = static_cast< uint64_t >(val4);
  ecode5 = SWIG_AsVal_unsigned_SS_long(swig_obj[4], &val5);
  if (!SWIG_IsOK(ecode5)) {
    SWIG_exception_fail(SWIG_ArgError(ecode5), "in method '" "WrappedPodArrayMemoryDirector_Write" "', argument " "5"" of type '" "uint64_t""'");
  } 
  arg5 = static_cast< uint64_t >(val5);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedPodArrayMemoryDirector::Write");
      } else {
        (arg1)->Write(arg2,arg3,arg4,arg5);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPodArrayMemoryDirector_objectheapid_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedPodArrayMemoryDirector *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPodArrayMemoryDirector_objectheapid_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodArrayMemoryDirector_objectheapid_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodArrayMemoryDirector * >(argp1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedPodArrayMemoryDirector_objectheapid_set" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->objectheapid = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPodArrayMemoryDirector_objectheapid_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedPodArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodArrayMemoryDirector_objectheapid_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodArrayMemoryDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (int32_t) ((arg1)->objectheapid);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedPodArrayMemoryDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedPodArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedPodArrayMemoryDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodArrayMemoryDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedPodArrayMemoryDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedPodArrayMemoryDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedPodArrayMemoryDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_WrappedPodMultiDimArrayMemoryDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedPodMultiDimArrayMemoryDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPodMultiDimArrayMemoryDirector_Dimensions(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  Swig::Director *director = 0;
  bool upcall = false;
  std::vector< uint64_t,std::allocator< uint64_t > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodMultiDimArrayMemoryDirector_Dimensions" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector * >(argp1);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector::Dimensions");
      } else {
        result = (arg1)->Dimensions();
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPodMultiDimArrayMemoryDirector_DimCount(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  Swig::Director *director = 0;
  bool upcall = false;
  uint64_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodMultiDimArrayMemoryDirector_DimCount" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector * >(argp1);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector::DimCount");
      } else {
        result = (uint64_t)(arg1)->DimCount();
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPodMultiDimArrayMemoryDirector_Read(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *) 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg2 = 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg3 = 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  int res4 = SWIG_OLDOBJ ;
  PyObject *swig_obj[4] ;
  Swig::Director *director = 0;
  bool upcall = false;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPodMultiDimArrayMemoryDirector_Read", 4, 4, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodMultiDimArrayMemoryDirector_Read" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector * >(argp1);
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPodMultiDimArrayMemoryDirector_Read" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedPodMultiDimArrayMemoryDirector_Read" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedPodMultiDimArrayMemoryDirector_Read" "', argument " "3"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedPodMultiDimArrayMemoryDirector_Read" "', argument " "3"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg3 = ptr;
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedPodMultiDimArrayMemoryDirector_Read" "', argument " "4"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedPodMultiDimArrayMemoryDirector_Read" "', argument " "4"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg4 = ptr;
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector::Read");
      } else {
        result = (arg1)->Read((std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg2,(std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg3,(std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg4);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, SWIG_POINTER_OWN);
  
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPodMultiDimArrayMemoryDirector_Write(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *) 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > arg3 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg4 = 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg5 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 ;
  int res3 = 0 ;
  int newmem3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  int res5 = SWIG_OLDOBJ ;
  PyObject *swig_obj[5] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPodMultiDimArrayMemoryDirector_Write", 5, 5, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodMultiDimArrayMemoryDirector_Write" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector * >(argp1);
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPodMultiDimArrayMemoryDirector_Write" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedPodMultiDimArrayMemoryDirector_Write" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg2 = ptr;
  }
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >**)&swig_obj[2];
  //if (smartarg) {
  //	arg3 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(smartarg->get(), true);
  //}
  
  res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t,  0 , &newmem3);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedPodMultiDimArrayMemoryDirector_Write" "', argument " "3"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >""'");
  }
  if (argp3) arg3 = boost::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp3)->get(),true);
  if (newmem3 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp3);
  
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedPodMultiDimArrayMemoryDirector_Write" "', argument " "4"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedPodMultiDimArrayMemoryDirector_Write" "', argument " "4"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg4 = ptr;
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res5 = swig::asptr(swig_obj[4], &ptr);
    if (!SWIG_IsOK(res5)) {
      SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "WrappedPodMultiDimArrayMemoryDirector_Write" "', argument " "5"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedPodMultiDimArrayMemoryDirector_Write" "', argument " "5"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg5 = ptr;
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector::Write");
      } else {
        (arg1)->Write((std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg2,arg3,(std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg4,(std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg5);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res4)) delete arg4;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res4)) delete arg4;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPodMultiDimArrayMemoryDirector_objectheapid_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPodMultiDimArrayMemoryDirector_objectheapid_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodMultiDimArrayMemoryDirector_objectheapid_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector * >(argp1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedPodMultiDimArrayMemoryDirector_objectheapid_set" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->objectheapid = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPodMultiDimArrayMemoryDirector_objectheapid_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPodMultiDimArrayMemoryDirector_objectheapid_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (int32_t) ((arg1)->objectheapid);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedPodMultiDimArrayMemoryDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *)new SwigDirector_WrappedPodMultiDimArrayMemoryDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedPodMultiDimArrayMemoryDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedPodMultiDimArrayMemoryDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedPodMultiDimArrayMemoryDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedPodMultiDimArrayMemoryDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedNamedArrayMemoryClientBuffer_UnpackReadResult(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *arg1 = (RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > arg2 ;
  uint64_t arg3 ;
  uint64_t arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  int newmem2 = 0 ;
  unsigned long val3 ;
  int ecode3 = 0 ;
  unsigned long val4 ;
  int ecode4 = 0 ;
  PyObject *swig_obj[4] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedNamedArrayMemoryClientBuffer_UnpackReadResult", 4, 4, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedArrayMemoryClientBuffer, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedArrayMemoryClientBuffer_UnpackReadResult" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedArrayMemoryClientBuffer * >(argp1);
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >**)&swig_obj[1];
  //if (smartarg) {
  //	arg2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(smartarg->get(), true);
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedNamedArrayMemoryClientBuffer_UnpackReadResult" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >""'");
  }
  if (argp2) arg2 = boost::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp2)->get(),true);
  if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp2);
  
  ecode3 = SWIG_AsVal_unsigned_SS_long(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedNamedArrayMemoryClientBuffer_UnpackReadResult" "', argument " "3"" of type '" "uint64_t""'");
  } 
  arg3 = static_cast< uint64_t >(val3);
  ecode4 = SWIG_AsVal_unsigned_SS_long(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedNamedArrayMemoryClientBuffer_UnpackReadResult" "', argument " "4"" of type '" "uint64_t""'");
  } 
  arg4 = static_cast< uint64_t >(val4);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedNamedArrayMemoryClientBuffer::UnpackReadResult");
      } else {
        (arg1)->UnpackReadResult(arg2,arg3,arg4);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedNamedArrayMemoryClientBuffer_PackWriteRequest(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *arg1 = (RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *) 0 ;
  uint64_t arg2 ;
  uint64_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  unsigned long val2 ;
  int ecode2 = 0 ;
  unsigned long val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  Swig::Director *director = 0;
  bool upcall = false;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedNamedArrayMemoryClientBuffer_PackWriteRequest", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedArrayMemoryClientBuffer, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedArrayMemoryClientBuffer_PackWriteRequest" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedArrayMemoryClientBuffer * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_long(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedNamedArrayMemoryClientBuffer_PackWriteRequest" "', argument " "2"" of type '" "uint64_t""'");
  } 
  arg2 = static_cast< uint64_t >(val2);
  ecode3 = SWIG_AsVal_unsigned_SS_long(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedNamedArrayMemoryClientBuffer_PackWriteRequest" "', argument " "3"" of type '" "uint64_t""'");
  } 
  arg3 = static_cast< uint64_t >(val3);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedNamedArrayMemoryClientBuffer::PackWriteRequest");
      } else {
        result = (arg1)->PackWriteRequest(arg2,arg3);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedNamedArrayMemoryClientBuffer_GetBufferLength(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *arg1 = (RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  Swig::Director *director = 0;
  bool upcall = false;
  uint64_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedArrayMemoryClientBuffer, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedArrayMemoryClientBuffer_GetBufferLength" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedArrayMemoryClientBuffer * >(argp1);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedNamedArrayMemoryClientBuffer::GetBufferLength");
      } else {
        result = (uint64_t)(arg1)->GetBufferLength();
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedNamedArrayMemoryClientBuffer(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *arg1 = (RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedArrayMemoryClientBuffer, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedNamedArrayMemoryClientBuffer" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedArrayMemoryClientBuffer * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedNamedArrayMemoryClientBuffer(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *)new SwigDirector_WrappedNamedArrayMemoryClientBuffer(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedNamedArrayMemoryClientBuffer, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedNamedArrayMemoryClientBuffer(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *arg1 = (RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedArrayMemoryClientBuffer, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedNamedArrayMemoryClientBuffer" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedArrayMemoryClientBuffer * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedNamedArrayMemoryClientBuffer_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedNamedArrayMemoryClientBuffer, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedNamedArrayMemoryClientBuffer_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedNamedArrayMemoryClient_Length(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedArrayMemoryClient *arg1 = (RobotRaconteur::WrappedNamedArrayMemoryClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedArrayMemoryClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedArrayMemoryClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  uint64_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedNamedArrayMemoryClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedArrayMemoryClient_Length" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedArrayMemoryClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedArrayMemoryClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedNamedArrayMemoryClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedNamedArrayMemoryClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (uint64_t)(arg1)->Length();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedNamedArrayMemoryClient_Direction(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedArrayMemoryClient *arg1 = (RobotRaconteur::WrappedNamedArrayMemoryClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedArrayMemoryClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedArrayMemoryClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MemberDefinition_Direction result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedNamedArrayMemoryClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedArrayMemoryClient_Direction" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedArrayMemoryClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedArrayMemoryClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedNamedArrayMemoryClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedNamedArrayMemoryClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::MemberDefinition_Direction)(arg1)->Direction();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedNamedArrayMemoryClient_Read(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedArrayMemoryClient *arg1 = (RobotRaconteur::WrappedNamedArrayMemoryClient *) 0 ;
  uint64_t arg2 ;
  RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *arg3 = (RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *) 0 ;
  uint64_t arg4 ;
  uint64_t arg5 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedArrayMemoryClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedArrayMemoryClient > *smartarg1 = 0 ;
  unsigned long val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  unsigned long val4 ;
  int ecode4 = 0 ;
  unsigned long val5 ;
  int ecode5 = 0 ;
  PyObject *swig_obj[5] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedNamedArrayMemoryClient_Read", 5, 5, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedNamedArrayMemoryClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedArrayMemoryClient_Read" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedArrayMemoryClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedArrayMemoryClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedNamedArrayMemoryClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedNamedArrayMemoryClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_unsigned_SS_long(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedNamedArrayMemoryClient_Read" "', argument " "2"" of type '" "uint64_t""'");
  } 
  arg2 = static_cast< uint64_t >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__WrappedNamedArrayMemoryClientBuffer, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedNamedArrayMemoryClient_Read" "', argument " "3"" of type '" "RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::WrappedNamedArrayMemoryClientBuffer * >(argp3);
  ecode4 = SWIG_AsVal_unsigned_SS_long(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedNamedArrayMemoryClient_Read" "', argument " "4"" of type '" "uint64_t""'");
  } 
  arg4 = static_cast< uint64_t >(val4);
  ecode5 = SWIG_AsVal_unsigned_SS_long(swig_obj[4], &val5);
  if (!SWIG_IsOK(ecode5)) {
    SWIG_exception_fail(SWIG_ArgError(ecode5), "in method '" "WrappedNamedArrayMemoryClient_Read" "', argument " "5"" of type '" "uint64_t""'");
  } 
  arg5 = static_cast< uint64_t >(val5);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Read(arg2,arg3,arg4,arg5);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedNamedArrayMemoryClient_Write(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedArrayMemoryClient *arg1 = (RobotRaconteur::WrappedNamedArrayMemoryClient *) 0 ;
  uint64_t arg2 ;
  RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *arg3 = (RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *) 0 ;
  uint64_t arg4 ;
  uint64_t arg5 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedArrayMemoryClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedArrayMemoryClient > *smartarg1 = 0 ;
  unsigned long val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  unsigned long val4 ;
  int ecode4 = 0 ;
  unsigned long val5 ;
  int ecode5 = 0 ;
  PyObject *swig_obj[5] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedNamedArrayMemoryClient_Write", 5, 5, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedNamedArrayMemoryClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedArrayMemoryClient_Write" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedArrayMemoryClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedArrayMemoryClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedNamedArrayMemoryClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedNamedArrayMemoryClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_unsigned_SS_long(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedNamedArrayMemoryClient_Write" "', argument " "2"" of type '" "uint64_t""'");
  } 
  arg2 = static_cast< uint64_t >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__WrappedNamedArrayMemoryClientBuffer, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedNamedArrayMemoryClient_Write" "', argument " "3"" of type '" "RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::WrappedNamedArrayMemoryClientBuffer * >(argp3);
  ecode4 = SWIG_AsVal_unsigned_SS_long(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedNamedArrayMemoryClient_Write" "', argument " "4"" of type '" "uint64_t""'");
  } 
  arg4 = static_cast< uint64_t >(val4);
  ecode5 = SWIG_AsVal_unsigned_SS_long(swig_obj[4], &val5);
  if (!SWIG_IsOK(ecode5)) {
    SWIG_exception_fail(SWIG_ArgError(ecode5), "in method '" "WrappedNamedArrayMemoryClient_Write" "', argument " "5"" of type '" "uint64_t""'");
  } 
  arg5 = static_cast< uint64_t >(val5);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Write(arg2,arg3,arg4,arg5);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedNamedArrayMemoryClient(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedArrayMemoryClient *arg1 = (RobotRaconteur::WrappedNamedArrayMemoryClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedArrayMemoryClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedArrayMemoryClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedNamedArrayMemoryClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedNamedArrayMemoryClient" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedArrayMemoryClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedArrayMemoryClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedNamedArrayMemoryClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedNamedArrayMemoryClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    (void)arg1; delete smartarg1;
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedNamedArrayMemoryClient_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedNamedArrayMemoryClient_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_WrappedNamedMultiDimArrayMemoryClientBuffer_UnpackReadResult(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer *arg1 = (RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > arg2 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg3 = 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  int newmem2 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  int res4 = SWIG_OLDOBJ ;
  PyObject *swig_obj[4] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedNamedMultiDimArrayMemoryClientBuffer_UnpackReadResult", 4, 4, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClientBuffer, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedMultiDimArrayMemoryClientBuffer_UnpackReadResult" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer * >(argp1);
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >**)&swig_obj[1];
  //if (smartarg) {
  //	arg2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(smartarg->get(), true);
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedNamedMultiDimArrayMemoryClientBuffer_UnpackReadResult" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >""'");
  }
  if (argp2) arg2 = boost::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp2)->get(),true);
  if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp2);
  
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedNamedMultiDimArrayMemoryClientBuffer_UnpackReadResult" "', argument " "3"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedNamedMultiDimArrayMemoryClientBuffer_UnpackReadResult" "', argument " "3"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg3 = ptr;
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedNamedMultiDimArrayMemoryClientBuffer_UnpackReadResult" "', argument " "4"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedNamedMultiDimArrayMemoryClientBuffer_UnpackReadResult" "', argument " "4"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg4 = ptr;
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer::UnpackReadResult");
      } else {
        (arg1)->UnpackReadResult(arg2,(std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg3,(std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg4);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedNamedMultiDimArrayMemoryClientBuffer_PackWriteRequest(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer *arg1 = (RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer *) 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg2 = 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  PyObject *swig_obj[3] ;
  Swig::Director *director = 0;
  bool upcall = false;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedNamedMultiDimArrayMemoryClientBuffer_PackWriteRequest", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClientBuffer, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedMultiDimArrayMemoryClientBuffer_PackWriteRequest" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer * >(argp1);
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedNamedMultiDimArrayMemoryClientBuffer_PackWriteRequest" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedNamedMultiDimArrayMemoryClientBuffer_PackWriteRequest" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedNamedMultiDimArrayMemoryClientBuffer_PackWriteRequest" "', argument " "3"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedNamedMultiDimArrayMemoryClientBuffer_PackWriteRequest" "', argument " "3"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg3 = ptr;
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer::PackWriteRequest");
      } else {
        result = (arg1)->PackWriteRequest((std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg2,(std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg3);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, SWIG_POINTER_OWN);
  
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedNamedMultiDimArrayMemoryClientBuffer(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer *arg1 = (RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClientBuffer, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedNamedMultiDimArrayMemoryClientBuffer" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedNamedMultiDimArrayMemoryClientBuffer(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer *)new SwigDirector_WrappedNamedMultiDimArrayMemoryClientBuffer(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClientBuffer, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedNamedMultiDimArrayMemoryClientBuffer(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer *arg1 = (RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClientBuffer, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedNamedMultiDimArrayMemoryClientBuffer" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedNamedMultiDimArrayMemoryClientBuffer_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClientBuffer, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedNamedMultiDimArrayMemoryClientBuffer_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedNamedMultiDimArrayMemoryClient_Dimensions(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient *arg1 = (RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< uint64_t,std::allocator< uint64_t > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedMultiDimArrayMemoryClient_Dimensions" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->Dimensions();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedNamedMultiDimArrayMemoryClient_DimCount(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient *arg1 = (RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  uint64_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedMultiDimArrayMemoryClient_DimCount" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (uint64_t)(arg1)->DimCount();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedNamedMultiDimArrayMemoryClient_Direction(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient *arg1 = (RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::MemberDefinition_Direction result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedMultiDimArrayMemoryClient_Direction" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::MemberDefinition_Direction)(arg1)->Direction();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedNamedMultiDimArrayMemoryClient_Read(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient *arg1 = (RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient *) 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg2 = 0 ;
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer *arg3 = (RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer *) 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg4 = 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg5 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  int res5 = SWIG_OLDOBJ ;
  PyObject *swig_obj[5] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedNamedMultiDimArrayMemoryClient_Read", 5, 5, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedMultiDimArrayMemoryClient_Read" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedNamedMultiDimArrayMemoryClient_Read" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedNamedMultiDimArrayMemoryClient_Read" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClientBuffer, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedNamedMultiDimArrayMemoryClient_Read" "', argument " "3"" of type '" "RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer * >(argp3);
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedNamedMultiDimArrayMemoryClient_Read" "', argument " "4"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedNamedMultiDimArrayMemoryClient_Read" "', argument " "4"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg4 = ptr;
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res5 = swig::asptr(swig_obj[4], &ptr);
    if (!SWIG_IsOK(res5)) {
      SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "WrappedNamedMultiDimArrayMemoryClient_Read" "', argument " "5"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedNamedMultiDimArrayMemoryClient_Read" "', argument " "5"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg5 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Read((std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg2,arg3,(std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg4,(std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg5);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res4)) delete arg4;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res4)) delete arg4;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedNamedMultiDimArrayMemoryClient_Write(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient *arg1 = (RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient *) 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg2 = 0 ;
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer *arg3 = (RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer *) 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg4 = 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg5 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  int res5 = SWIG_OLDOBJ ;
  PyObject *swig_obj[5] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedNamedMultiDimArrayMemoryClient_Write", 5, 5, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedMultiDimArrayMemoryClient_Write" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedNamedMultiDimArrayMemoryClient_Write" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedNamedMultiDimArrayMemoryClient_Write" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClientBuffer, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedNamedMultiDimArrayMemoryClient_Write" "', argument " "3"" of type '" "RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer * >(argp3);
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedNamedMultiDimArrayMemoryClient_Write" "', argument " "4"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedNamedMultiDimArrayMemoryClient_Write" "', argument " "4"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg4 = ptr;
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res5 = swig::asptr(swig_obj[4], &ptr);
    if (!SWIG_IsOK(res5)) {
      SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "WrappedNamedMultiDimArrayMemoryClient_Write" "', argument " "5"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedNamedMultiDimArrayMemoryClient_Write" "', argument " "5"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg5 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Write((std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg2,arg3,(std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg4,(std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg5);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res4)) delete arg4;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res4)) delete arg4;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedNamedMultiDimArrayMemoryClient(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient *arg1 = (RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedNamedMultiDimArrayMemoryClient" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    (void)arg1; delete smartarg1;
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedNamedMultiDimArrayMemoryClient_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClient_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_new_WrappedNamedArrayMemoryDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedNamedArrayMemoryDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedNamedArrayMemoryDirector *)new SwigDirector_WrappedNamedArrayMemoryDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedNamedArrayMemoryDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedNamedArrayMemoryDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedNamedArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedArrayMemoryDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedNamedArrayMemoryDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedArrayMemoryDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedNamedArrayMemoryDirector_Length(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedNamedArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  Swig::Director *director = 0;
  bool upcall = false;
  uint64_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedArrayMemoryDirector_Length" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedArrayMemoryDirector * >(argp1);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedNamedArrayMemoryDirector::Length");
      } else {
        result = (uint64_t)(arg1)->Length();
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedNamedArrayMemoryDirector_Read(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedNamedArrayMemoryDirector *) 0 ;
  uint64_t arg2 ;
  uint64_t arg3 ;
  uint64_t arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  unsigned long val2 ;
  int ecode2 = 0 ;
  unsigned long val3 ;
  int ecode3 = 0 ;
  unsigned long val4 ;
  int ecode4 = 0 ;
  PyObject *swig_obj[4] ;
  Swig::Director *director = 0;
  bool upcall = false;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedNamedArrayMemoryDirector_Read", 4, 4, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedArrayMemoryDirector_Read" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedArrayMemoryDirector * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_long(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedNamedArrayMemoryDirector_Read" "', argument " "2"" of type '" "uint64_t""'");
  } 
  arg2 = static_cast< uint64_t >(val2);
  ecode3 = SWIG_AsVal_unsigned_SS_long(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedNamedArrayMemoryDirector_Read" "', argument " "3"" of type '" "uint64_t""'");
  } 
  arg3 = static_cast< uint64_t >(val3);
  ecode4 = SWIG_AsVal_unsigned_SS_long(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedNamedArrayMemoryDirector_Read" "', argument " "4"" of type '" "uint64_t""'");
  } 
  arg4 = static_cast< uint64_t >(val4);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedNamedArrayMemoryDirector::Read");
      } else {
        result = (arg1)->Read(arg2,arg3,arg4);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedNamedArrayMemoryDirector_Write(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedNamedArrayMemoryDirector *) 0 ;
  uint64_t arg2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > arg3 ;
  uint64_t arg4 ;
  uint64_t arg5 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  unsigned long val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  int newmem3 = 0 ;
  unsigned long val4 ;
  int ecode4 = 0 ;
  unsigned long val5 ;
  int ecode5 = 0 ;
  PyObject *swig_obj[5] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedNamedArrayMemoryDirector_Write", 5, 5, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedArrayMemoryDirector_Write" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedArrayMemoryDirector * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_long(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedNamedArrayMemoryDirector_Write" "', argument " "2"" of type '" "uint64_t""'");
  } 
  arg2 = static_cast< uint64_t >(val2);
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >**)&swig_obj[2];
  //if (smartarg) {
  //	arg3 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(smartarg->get(), true);
  //}
  
  res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t,  0 , &newmem3);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedNamedArrayMemoryDirector_Write" "', argument " "3"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >""'");
  }
  if (argp3) arg3 = boost::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp3)->get(),true);
  if (newmem3 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp3);
  
  ecode4 = SWIG_AsVal_unsigned_SS_long(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedNamedArrayMemoryDirector_Write" "', argument " "4"" of type '" "uint64_t""'");
  } 
  arg4 = static_cast< uint64_t >(val4);
  ecode5 = SWIG_AsVal_unsigned_SS_long(swig_obj[4], &val5);
  if (!SWIG_IsOK(ecode5)) {
    SWIG_exception_fail(SWIG_ArgError(ecode5), "in method '" "WrappedNamedArrayMemoryDirector_Write" "', argument " "5"" of type '" "uint64_t""'");
  } 
  arg5 = static_cast< uint64_t >(val5);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedNamedArrayMemoryDirector::Write");
      } else {
        (arg1)->Write(arg2,arg3,arg4,arg5);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedNamedArrayMemoryDirector_objectheapid_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedNamedArrayMemoryDirector *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedNamedArrayMemoryDirector_objectheapid_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedArrayMemoryDirector_objectheapid_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedArrayMemoryDirector * >(argp1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedNamedArrayMemoryDirector_objectheapid_set" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->objectheapid = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedNamedArrayMemoryDirector_objectheapid_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedNamedArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedArrayMemoryDirector_objectheapid_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedArrayMemoryDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (int32_t) ((arg1)->objectheapid);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedNamedArrayMemoryDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedNamedArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedNamedArrayMemoryDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedArrayMemoryDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedNamedArrayMemoryDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedNamedArrayMemoryDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedNamedArrayMemoryDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_WrappedNamedMultiDimArrayMemoryDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedNamedMultiDimArrayMemoryDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedNamedMultiDimArrayMemoryDirector_Dimensions(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  Swig::Director *director = 0;
  bool upcall = false;
  std::vector< uint64_t,std::allocator< uint64_t > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedMultiDimArrayMemoryDirector_Dimensions" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector * >(argp1);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector::Dimensions");
      } else {
        result = (arg1)->Dimensions();
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = swig::from(static_cast< std::vector< unsigned long,std::allocator< unsigned long > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedNamedMultiDimArrayMemoryDirector_DimCount(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  Swig::Director *director = 0;
  bool upcall = false;
  uint64_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedMultiDimArrayMemoryDirector_DimCount" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector * >(argp1);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector::DimCount");
      } else {
        result = (uint64_t)(arg1)->DimCount();
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_From_unsigned_SS_long(static_cast< unsigned long >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedNamedMultiDimArrayMemoryDirector_Read(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *) 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg2 = 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg3 = 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  int res4 = SWIG_OLDOBJ ;
  PyObject *swig_obj[4] ;
  Swig::Director *director = 0;
  bool upcall = false;
  boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedNamedMultiDimArrayMemoryDirector_Read", 4, 4, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedMultiDimArrayMemoryDirector_Read" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector * >(argp1);
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedNamedMultiDimArrayMemoryDirector_Read" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedNamedMultiDimArrayMemoryDirector_Read" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedNamedMultiDimArrayMemoryDirector_Read" "', argument " "3"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedNamedMultiDimArrayMemoryDirector_Read" "', argument " "3"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg3 = ptr;
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedNamedMultiDimArrayMemoryDirector_Read" "', argument " "4"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedNamedMultiDimArrayMemoryDirector_Read" "', argument " "4"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg4 = ptr;
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector::Read");
      } else {
        result = (arg1)->Read((std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg2,(std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg3,(std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg4);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElementNestedElementList >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, SWIG_POINTER_OWN);
  
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedNamedMultiDimArrayMemoryDirector_Write(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *) 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList > arg3 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg4 = 0 ;
  std::vector< uint64_t,std::allocator< uint64_t > > *arg5 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 ;
  int res3 = 0 ;
  int newmem3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  int res5 = SWIG_OLDOBJ ;
  PyObject *swig_obj[5] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedNamedMultiDimArrayMemoryDirector_Write", 5, 5, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedMultiDimArrayMemoryDirector_Write" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector * >(argp1);
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedNamedMultiDimArrayMemoryDirector_Write" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedNamedMultiDimArrayMemoryDirector_Write" "', argument " "2"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg2 = ptr;
  }
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementNestedElementList >**)&swig_obj[2];
  //if (smartarg) {
  //	arg3 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(smartarg->get(), true);
  //}
  
  res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t,  0 , &newmem3);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedNamedMultiDimArrayMemoryDirector_Write" "', argument " "3"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementNestedElementList >""'");
  }
  if (argp3) arg3 = boost::intrusive_ptr<  RobotRaconteur::MessageElementNestedElementList >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp3)->get(),true);
  if (newmem3 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList >* >(argp3);
  
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedNamedMultiDimArrayMemoryDirector_Write" "', argument " "4"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedNamedMultiDimArrayMemoryDirector_Write" "', argument " "4"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg4 = ptr;
  }
  {
    std::vector< unsigned long,std::allocator< unsigned long > > *ptr = (std::vector< unsigned long,std::allocator< unsigned long > > *)0;
    res5 = swig::asptr(swig_obj[4], &ptr);
    if (!SWIG_IsOK(res5)) {
      SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "WrappedNamedMultiDimArrayMemoryDirector_Write" "', argument " "5"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedNamedMultiDimArrayMemoryDirector_Write" "', argument " "5"" of type '" "std::vector< uint64_t,std::allocator< uint64_t > > const &""'"); 
    }
    arg5 = ptr;
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector::Write");
      } else {
        (arg1)->Write((std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg2,arg3,(std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg4,(std::vector< uint64_t,std::allocator< uint64_t > > const &)*arg5);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res4)) delete arg4;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res4)) delete arg4;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedNamedMultiDimArrayMemoryDirector_objectheapid_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedNamedMultiDimArrayMemoryDirector_objectheapid_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedMultiDimArrayMemoryDirector_objectheapid_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector * >(argp1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedNamedMultiDimArrayMemoryDirector_objectheapid_set" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->objectheapid = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedNamedMultiDimArrayMemoryDirector_objectheapid_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedNamedMultiDimArrayMemoryDirector_objectheapid_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (int32_t) ((arg1)->objectheapid);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedNamedMultiDimArrayMemoryDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *)new SwigDirector_WrappedNamedMultiDimArrayMemoryDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedNamedMultiDimArrayMemoryDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *arg1 = (RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedNamedMultiDimArrayMemoryDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedNamedMultiDimArrayMemoryDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedNamedMultiDimArrayMemoryDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedGeneratorClient_TryGetNextResult_value_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedGeneratorClient_TryGetNextResult *arg1 = (RobotRaconteur::WrappedGeneratorClient_TryGetNextResult *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedGeneratorClient_TryGetNextResult_value_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedGeneratorClient_TryGetNextResult, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedGeneratorClient_TryGetNextResult_value_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedGeneratorClient_TryGetNextResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedGeneratorClient_TryGetNextResult * >(argp1);
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedGeneratorClient_TryGetNextResult_value_set" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->value = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedGeneratorClient_TryGetNextResult_value_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedGeneratorClient_TryGetNextResult *arg1 = (RobotRaconteur::WrappedGeneratorClient_TryGetNextResult *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedGeneratorClient_TryGetNextResult, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedGeneratorClient_TryGetNextResult_value_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedGeneratorClient_TryGetNextResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedGeneratorClient_TryGetNextResult * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::intrusive_ptr< RobotRaconteur::MessageElement > *) & ((arg1)->value);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  
  //if (*result) {
  //  intrusive_ptr_add_ref(result->get());
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result->get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(result)) {
    intrusive_ptr_add_ref(result->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >(result->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedGeneratorClient_TryGetNextResult_res_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedGeneratorClient_TryGetNextResult *arg1 = (RobotRaconteur::WrappedGeneratorClient_TryGetNextResult *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedGeneratorClient_TryGetNextResult_res_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedGeneratorClient_TryGetNextResult, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedGeneratorClient_TryGetNextResult_res_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedGeneratorClient_TryGetNextResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedGeneratorClient_TryGetNextResult * >(argp1);
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedGeneratorClient_TryGetNextResult_res_set" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->res = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedGeneratorClient_TryGetNextResult_res_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedGeneratorClient_TryGetNextResult *arg1 = (RobotRaconteur::WrappedGeneratorClient_TryGetNextResult *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedGeneratorClient_TryGetNextResult, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedGeneratorClient_TryGetNextResult_res_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedGeneratorClient_TryGetNextResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedGeneratorClient_TryGetNextResult * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (bool) ((arg1)->res);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedGeneratorClient_TryGetNextResult(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedGeneratorClient_TryGetNextResult *arg1 = (RobotRaconteur::WrappedGeneratorClient_TryGetNextResult *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedGeneratorClient_TryGetNextResult, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedGeneratorClient_TryGetNextResult" "', argument " "1"" of type '" "RobotRaconteur::WrappedGeneratorClient_TryGetNextResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedGeneratorClient_TryGetNextResult * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedGeneratorClient_TryGetNextResult_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedGeneratorClient_TryGetNextResult, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_WrappedGeneratorClient_Next(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedGeneratorClient *arg1 = (RobotRaconteur::WrappedGeneratorClient *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedGeneratorClient_Next", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedGeneratorClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedGeneratorClient_Next" "', argument " "1"" of type '" "RobotRaconteur::WrappedGeneratorClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedGeneratorClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedGeneratorClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedGeneratorClient_Next" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->Next((boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedGeneratorClient_TryNext(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedGeneratorClient *arg1 = (RobotRaconteur::WrappedGeneratorClient *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  RobotRaconteur::WrappedGeneratorClient_TryGetNextResult result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedGeneratorClient_TryNext", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedGeneratorClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedGeneratorClient_TryNext" "', argument " "1"" of type '" "RobotRaconteur::WrappedGeneratorClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedGeneratorClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedGeneratorClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedGeneratorClient_TryNext" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->TryNext((boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::WrappedGeneratorClient_TryGetNextResult(static_cast< const RobotRaconteur::WrappedGeneratorClient_TryGetNextResult& >(result))), SWIGTYPE_p_RobotRaconteur__WrappedGeneratorClient_TryGetNextResult, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedGeneratorClient_AsyncNext(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedGeneratorClient *arg1 = (RobotRaconteur::WrappedGeneratorClient *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg2 = 0 ;
  int32_t arg3 ;
  RobotRaconteur::AsyncRequestDirector *arg4 = (RobotRaconteur::AsyncRequestDirector *) 0 ;
  int32_t arg5 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp2 ;
  int newmem2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  void *argp4 = 0 ;
  int res4 = 0 ;
  int val5 ;
  int ecode5 = 0 ;
  PyObject *swig_obj[5] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedGeneratorClient_AsyncNext", 5, 5, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedGeneratorClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedGeneratorClient_AsyncNext" "', argument " "1"" of type '" "RobotRaconteur::WrappedGeneratorClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedGeneratorClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedGeneratorClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedGeneratorClient_AsyncNext" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedGeneratorClient_AsyncNext" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  res4 = SWIG_ConvertPtr(swig_obj[3], &argp4,SWIGTYPE_p_RobotRaconteur__AsyncRequestDirector, 0 |  0 );
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedGeneratorClient_AsyncNext" "', argument " "4"" of type '" "RobotRaconteur::AsyncRequestDirector *""'"); 
  }
  arg4 = reinterpret_cast< RobotRaconteur::AsyncRequestDirector * >(argp4);
  ecode5 = SWIG_AsVal_int(swig_obj[4], &val5);
  if (!SWIG_IsOK(ecode5)) {
    SWIG_exception_fail(SWIG_ArgError(ecode5), "in method '" "WrappedGeneratorClient_AsyncNext" "', argument " "5"" of type '" "int32_t""'");
  } 
  arg5 = static_cast< int32_t >(val5);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->AsyncNext((boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg2,arg3,arg4,arg5);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedGeneratorClient_Abort(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedGeneratorClient *arg1 = (RobotRaconteur::WrappedGeneratorClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedGeneratorClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedGeneratorClient_Abort" "', argument " "1"" of type '" "RobotRaconteur::WrappedGeneratorClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedGeneratorClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedGeneratorClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Abort();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedGeneratorClient_AsyncAbort(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedGeneratorClient *arg1 = (RobotRaconteur::WrappedGeneratorClient *) 0 ;
  int32_t arg2 ;
  RobotRaconteur::AsyncVoidReturnDirector *arg3 = (RobotRaconteur::AsyncVoidReturnDirector *) 0 ;
  int32_t arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  int val4 ;
  int ecode4 = 0 ;
  PyObject *swig_obj[4] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedGeneratorClient_AsyncAbort", 4, 4, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedGeneratorClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedGeneratorClient_AsyncAbort" "', argument " "1"" of type '" "RobotRaconteur::WrappedGeneratorClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedGeneratorClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedGeneratorClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedGeneratorClient_AsyncAbort" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__AsyncVoidReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedGeneratorClient_AsyncAbort" "', argument " "3"" of type '" "RobotRaconteur::AsyncVoidReturnDirector *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::AsyncVoidReturnDirector * >(argp3);
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedGeneratorClient_AsyncAbort" "', argument " "4"" of type '" "int32_t""'");
  } 
  arg4 = static_cast< int32_t >(val4);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->AsyncAbort(arg2,arg3,arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedGeneratorClient_Close(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedGeneratorClient *arg1 = (RobotRaconteur::WrappedGeneratorClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedGeneratorClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedGeneratorClient_Close" "', argument " "1"" of type '" "RobotRaconteur::WrappedGeneratorClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedGeneratorClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedGeneratorClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Close();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedGeneratorClient_AsyncClose(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedGeneratorClient *arg1 = (RobotRaconteur::WrappedGeneratorClient *) 0 ;
  int32_t arg2 ;
  RobotRaconteur::AsyncVoidReturnDirector *arg3 = (RobotRaconteur::AsyncVoidReturnDirector *) 0 ;
  int32_t arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  int val4 ;
  int ecode4 = 0 ;
  PyObject *swig_obj[4] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedGeneratorClient_AsyncClose", 4, 4, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedGeneratorClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedGeneratorClient_AsyncClose" "', argument " "1"" of type '" "RobotRaconteur::WrappedGeneratorClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedGeneratorClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedGeneratorClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedGeneratorClient_AsyncClose" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__AsyncVoidReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedGeneratorClient_AsyncClose" "', argument " "3"" of type '" "RobotRaconteur::AsyncVoidReturnDirector *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::AsyncVoidReturnDirector * >(argp3);
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedGeneratorClient_AsyncClose" "', argument " "4"" of type '" "int32_t""'");
  } 
  arg4 = static_cast< int32_t >(val4);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->AsyncClose(arg2,arg3,arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedGeneratorClient_NextAll(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedGeneratorClient *arg1 = (RobotRaconteur::WrappedGeneratorClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedGeneratorClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedGeneratorClient_NextAll" "', argument " "1"" of type '" "RobotRaconteur::WrappedGeneratorClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedGeneratorClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedGeneratorClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->NextAll();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedGeneratorClient(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedGeneratorClient *arg1 = (RobotRaconteur::WrappedGeneratorClient *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedGeneratorClient_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedGeneratorClient" "', argument " "1"" of type '" "RobotRaconteur::WrappedGeneratorClient *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedGeneratorClient * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedGeneratorClient * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    (void)arg1; delete smartarg1;
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedGeneratorClient_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedGeneratorClient_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_delete_AsyncGeneratorClientReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncGeneratorClientReturnDirector *arg1 = (RobotRaconteur::AsyncGeneratorClientReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncGeneratorClientReturnDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_AsyncGeneratorClientReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncGeneratorClientReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncGeneratorClientReturnDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_AsyncGeneratorClientReturnDirector_handler(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncGeneratorClientReturnDirector *arg1 = (RobotRaconteur::AsyncGeneratorClientReturnDirector *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > *arg2 = 0 ;
  RobotRaconteur::HandlerErrorInfo *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > tempshared2 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  PyObject *swig_obj[3] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "AsyncGeneratorClientReturnDirector_handler", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncGeneratorClientReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AsyncGeneratorClientReturnDirector_handler" "', argument " "1"" of type '" "RobotRaconteur::AsyncGeneratorClientReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncGeneratorClientReturnDirector * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedGeneratorClient_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "AsyncGeneratorClientReturnDirector_handler" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > * >(argp2) : &tempshared2;
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "AsyncGeneratorClientReturnDirector_handler" "', argument " "3"" of type '" "RobotRaconteur::HandlerErrorInfo &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "AsyncGeneratorClientReturnDirector_handler" "', argument " "3"" of type '" "RobotRaconteur::HandlerErrorInfo &""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp3);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::AsyncGeneratorClientReturnDirector::handler");
      } else {
        (arg1)->handler((boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > const &)*arg2,*arg3);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_AsyncGeneratorClientReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::AsyncGeneratorClientReturnDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::AsyncGeneratorClientReturnDirector *)new SwigDirector_AsyncGeneratorClientReturnDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__AsyncGeneratorClientReturnDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_AsyncGeneratorClientReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncGeneratorClientReturnDirector *arg1 = (RobotRaconteur::AsyncGeneratorClientReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncGeneratorClientReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_AsyncGeneratorClientReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncGeneratorClientReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncGeneratorClientReturnDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *AsyncGeneratorClientReturnDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__AsyncGeneratorClientReturnDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *AsyncGeneratorClientReturnDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_new_WrappedGeneratorServerDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedGeneratorServerDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedGeneratorServerDirector *)new SwigDirector_WrappedGeneratorServerDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedGeneratorServerDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedGeneratorServerDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedGeneratorServerDirector *arg1 = (RobotRaconteur::WrappedGeneratorServerDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedGeneratorServerDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedGeneratorServerDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedGeneratorServerDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedGeneratorServerDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedGeneratorServerDirector_Next(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedGeneratorServerDirector *arg1 = (RobotRaconteur::WrappedGeneratorServerDirector *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > arg2 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  int newmem2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > tempshared3 ;
  PyObject *swig_obj[3] ;
  Swig::Director *director = 0;
  bool upcall = false;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedGeneratorServerDirector_Next", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedGeneratorServerDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedGeneratorServerDirector_Next" "', argument " "1"" of type '" "RobotRaconteur::WrappedGeneratorServerDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedGeneratorServerDirector * >(argp1);
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //if (smartarg) {
  //	arg2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedGeneratorServerDirector_Next" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement >""'");
  }
  if (argp2) arg2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
  if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
  
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedGeneratorServerDirector_Next" "', argument " "3"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp3);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp3) : &tempshared3;
    }
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedGeneratorServerDirector::Next");
      } else {
        result = (arg1)->Next(arg2,(boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > const &)*arg3);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedGeneratorServerDirector_Abort(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedGeneratorServerDirector *arg1 = (RobotRaconteur::WrappedGeneratorServerDirector *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > tempshared2 ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedGeneratorServerDirector_Abort", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedGeneratorServerDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedGeneratorServerDirector_Abort" "', argument " "1"" of type '" "RobotRaconteur::WrappedGeneratorServerDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedGeneratorServerDirector * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedGeneratorServerDirector_Abort" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp2) : &tempshared2;
    }
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedGeneratorServerDirector::Abort");
      } else {
        (arg1)->Abort((boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > const &)*arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedGeneratorServerDirector_Close(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedGeneratorServerDirector *arg1 = (RobotRaconteur::WrappedGeneratorServerDirector *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > tempshared2 ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedGeneratorServerDirector_Close", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedGeneratorServerDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedGeneratorServerDirector_Close" "', argument " "1"" of type '" "RobotRaconteur::WrappedGeneratorServerDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedGeneratorServerDirector * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedGeneratorServerDirector_Close" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp2) : &tempshared2;
    }
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedGeneratorServerDirector::Close");
      } else {
        (arg1)->Close((boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > const &)*arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedGeneratorServerDirector_objectheapid_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedGeneratorServerDirector *arg1 = (RobotRaconteur::WrappedGeneratorServerDirector *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedGeneratorServerDirector_objectheapid_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedGeneratorServerDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedGeneratorServerDirector_objectheapid_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedGeneratorServerDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedGeneratorServerDirector * >(argp1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedGeneratorServerDirector_objectheapid_set" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->objectheapid = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedGeneratorServerDirector_objectheapid_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedGeneratorServerDirector *arg1 = (RobotRaconteur::WrappedGeneratorServerDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedGeneratorServerDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedGeneratorServerDirector_objectheapid_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedGeneratorServerDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedGeneratorServerDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (int32_t) ((arg1)->objectheapid);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedGeneratorServerDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedGeneratorServerDirector *arg1 = (RobotRaconteur::WrappedGeneratorServerDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedGeneratorServerDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedGeneratorServerDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedGeneratorServerDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedGeneratorServerDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedGeneratorServerDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedGeneratorServerDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedGeneratorServerDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_WrappedServiceStubDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStubDirector *arg1 = (RobotRaconteur::WrappedServiceStubDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceStubDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedServiceStubDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStubDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceStubDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStubDirector_DispatchEvent(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStubDirector *arg1 = (RobotRaconteur::WrappedServiceStubDirector *) 0 ;
  std::string *arg2 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[3] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceStubDirector_DispatchEvent", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceStubDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStubDirector_DispatchEvent" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStubDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceStubDirector * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStubDirector_DispatchEvent" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStubDirector_DispatchEvent" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *ptr = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *)0;
    int res = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res) || !ptr) {
      SWIG_exception_fail(SWIG_ArgError((ptr ? res : SWIG_TypeError)), "in method '" "WrappedServiceStubDirector_DispatchEvent" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >""'"); 
    }
    arg3 = *ptr;
    if (SWIG_IsNewObj(res)) delete ptr;
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedServiceStubDirector::DispatchEvent");
      } else {
        (arg1)->DispatchEvent((std::string const &)*arg2,arg3);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStubDirector_CallbackCall(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStubDirector *arg1 = (RobotRaconteur::WrappedServiceStubDirector *) 0 ;
  std::string *arg2 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[3] ;
  Swig::Director *director = 0;
  bool upcall = false;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceStubDirector_CallbackCall", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceStubDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStubDirector_CallbackCall" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStubDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceStubDirector * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStubDirector_CallbackCall" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStubDirector_CallbackCall" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *ptr = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *)0;
    int res = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res) || !ptr) {
      SWIG_exception_fail(SWIG_ArgError((ptr ? res : SWIG_TypeError)), "in method '" "WrappedServiceStubDirector_CallbackCall" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > >""'"); 
    }
    arg3 = *ptr;
    if (SWIG_IsNewObj(res)) delete ptr;
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedServiceStubDirector::CallbackCall");
      } else {
        result = (arg1)->CallbackCall((std::string const &)*arg2,arg3);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedServiceStubDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedServiceStubDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedServiceStubDirector *)new SwigDirector_WrappedServiceStubDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedServiceStubDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedServiceStubDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStubDirector *arg1 = (RobotRaconteur::WrappedServiceStubDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceStubDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedServiceStubDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStubDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceStubDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedServiceStubDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedServiceStubDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedServiceStubDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_AsyncStubReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncStubReturnDirector *arg1 = (RobotRaconteur::AsyncStubReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncStubReturnDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_AsyncStubReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncStubReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncStubReturnDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_AsyncStubReturnDirector_handler(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncStubReturnDirector *arg1 = (RobotRaconteur::AsyncStubReturnDirector *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  RobotRaconteur::HandlerErrorInfo *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  PyObject *swig_obj[3] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "AsyncStubReturnDirector_handler", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncStubReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AsyncStubReturnDirector_handler" "', argument " "1"" of type '" "RobotRaconteur::AsyncStubReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncStubReturnDirector * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "AsyncStubReturnDirector_handler" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "AsyncStubReturnDirector_handler" "', argument " "3"" of type '" "RobotRaconteur::HandlerErrorInfo &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "AsyncStubReturnDirector_handler" "', argument " "3"" of type '" "RobotRaconteur::HandlerErrorInfo &""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp3);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::AsyncStubReturnDirector::handler");
      } else {
        (arg1)->handler((boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2,*arg3);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_AsyncStubReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::AsyncStubReturnDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::AsyncStubReturnDirector *)new SwigDirector_AsyncStubReturnDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__AsyncStubReturnDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_AsyncStubReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncStubReturnDirector *arg1 = (RobotRaconteur::AsyncStubReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncStubReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_AsyncStubReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncStubReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncStubReturnDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *AsyncStubReturnDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__AsyncStubReturnDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *AsyncStubReturnDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedDirectorLock_valid(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedDirectorLock *arg1 = (RobotRaconteur::WrappedDirectorLock *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedDirectorLock > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedDirectorLock > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedDirectorLock_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedDirectorLock_valid" "', argument " "1"" of type '" "RobotRaconteur::WrappedDirectorLock *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedDirectorLock > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedDirectorLock > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedDirectorLock * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedDirectorLock > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedDirectorLock * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->valid();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedDirectorLock(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedDirectorLock *arg1 = (RobotRaconteur::WrappedDirectorLock *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedDirectorLock > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedDirectorLock > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedDirectorLock_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedDirectorLock" "', argument " "1"" of type '" "RobotRaconteur::WrappedDirectorLock *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedDirectorLock > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedDirectorLock > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedDirectorLock * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedDirectorLock > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedDirectorLock * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedDirectorLock_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedDirectorLock_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_WrappedServiceStub_PropertyGet(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceStub_PropertyGet", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_PropertyGet" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub_PropertyGet" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_PropertyGet" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->PropertyGet((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_PropertySet(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 ;
  int res3 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull3 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp3 ;
  int newmem3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceStub_PropertySet", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_PropertySet" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub_PropertySet" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_PropertySet" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[2] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[2];
  //	temp3 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg3 = &temp3;
  //} else {
  //  arg3 = &tempnull3;
  //}
  
  res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem3);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceStub_PropertySet" "', argument " "3"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp3) {
    temp3 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp3)->get(),true);
    if (newmem3 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp3);
    arg3 = &temp3;
  }
  else {
    arg3 = &tempnull3;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->PropertySet((std::string const &)*arg2,(boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_FunctionCall(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  PyObject *swig_obj[3] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceStub_FunctionCall", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_FunctionCall" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub_FunctionCall" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_FunctionCall" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *ptr = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceStub_FunctionCall" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_FunctionCall" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->FunctionCall((std::string const &)*arg2,(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_GeneratorFunctionCall(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  PyObject *swig_obj[3] ;
  boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceStub_GeneratorFunctionCall", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_GeneratorFunctionCall" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub_GeneratorFunctionCall" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_GeneratorFunctionCall" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *ptr = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceStub_GeneratorFunctionCall" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_GeneratorFunctionCall" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GeneratorFunctionCall((std::string const &)*arg2,(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedGeneratorClient >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedGeneratorClient_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_async_PropertyGet(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  int32_t arg3 ;
  RobotRaconteur::AsyncRequestDirector *arg4 = (RobotRaconteur::AsyncRequestDirector *) 0 ;
  int32_t arg5 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int val3 ;
  int ecode3 = 0 ;
  void *argp4 = 0 ;
  int res4 = 0 ;
  int val5 ;
  int ecode5 = 0 ;
  PyObject *swig_obj[5] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceStub_async_PropertyGet", 5, 5, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_async_PropertyGet" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub_async_PropertyGet" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_async_PropertyGet" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedServiceStub_async_PropertyGet" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  res4 = SWIG_ConvertPtr(swig_obj[3], &argp4,SWIGTYPE_p_RobotRaconteur__AsyncRequestDirector, 0 |  0 );
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedServiceStub_async_PropertyGet" "', argument " "4"" of type '" "RobotRaconteur::AsyncRequestDirector *""'"); 
  }
  arg4 = reinterpret_cast< RobotRaconteur::AsyncRequestDirector * >(argp4);
  ecode5 = SWIG_AsVal_int(swig_obj[4], &val5);
  if (!SWIG_IsOK(ecode5)) {
    SWIG_exception_fail(SWIG_ArgError(ecode5), "in method '" "WrappedServiceStub_async_PropertyGet" "', argument " "5"" of type '" "int32_t""'");
  } 
  arg5 = static_cast< int32_t >(val5);
  
  try {
    (arg1)->async_PropertyGet((std::string const &)*arg2,arg3,arg4,arg5);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_async_PropertySet(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg3 = 0 ;
  int32_t arg4 ;
  RobotRaconteur::AsyncRequestDirector *arg5 = (RobotRaconteur::AsyncRequestDirector *) 0 ;
  int32_t arg6 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 ;
  int res3 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull3 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp3 ;
  int newmem3 = 0 ;
  int val4 ;
  int ecode4 = 0 ;
  void *argp5 = 0 ;
  int res5 = 0 ;
  int val6 ;
  int ecode6 = 0 ;
  PyObject *swig_obj[6] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceStub_async_PropertySet", 6, 6, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_async_PropertySet" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub_async_PropertySet" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_async_PropertySet" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[2] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[2];
  //	temp3 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg3 = &temp3;
  //} else {
  //  arg3 = &tempnull3;
  //}
  
  res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem3);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceStub_async_PropertySet" "', argument " "3"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp3) {
    temp3 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp3)->get(),true);
    if (newmem3 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp3);
    arg3 = &temp3;
  }
  else {
    arg3 = &tempnull3;
  }
  
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedServiceStub_async_PropertySet" "', argument " "4"" of type '" "int32_t""'");
  } 
  arg4 = static_cast< int32_t >(val4);
  res5 = SWIG_ConvertPtr(swig_obj[4], &argp5,SWIGTYPE_p_RobotRaconteur__AsyncRequestDirector, 0 |  0 );
  if (!SWIG_IsOK(res5)) {
    SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "WrappedServiceStub_async_PropertySet" "', argument " "5"" of type '" "RobotRaconteur::AsyncRequestDirector *""'"); 
  }
  arg5 = reinterpret_cast< RobotRaconteur::AsyncRequestDirector * >(argp5);
  ecode6 = SWIG_AsVal_int(swig_obj[5], &val6);
  if (!SWIG_IsOK(ecode6)) {
    SWIG_exception_fail(SWIG_ArgError(ecode6), "in method '" "WrappedServiceStub_async_PropertySet" "', argument " "6"" of type '" "int32_t""'");
  } 
  arg6 = static_cast< int32_t >(val6);
  
  try {
    (arg1)->async_PropertySet((std::string const &)*arg2,(boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg3,arg4,arg5,arg6);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_async_FunctionCall(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *arg3 = 0 ;
  int32_t arg4 ;
  RobotRaconteur::AsyncRequestDirector *arg5 = (RobotRaconteur::AsyncRequestDirector *) 0 ;
  int32_t arg6 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  int val4 ;
  int ecode4 = 0 ;
  void *argp5 = 0 ;
  int res5 = 0 ;
  int val6 ;
  int ecode6 = 0 ;
  PyObject *swig_obj[6] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceStub_async_FunctionCall", 6, 6, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_async_FunctionCall" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub_async_FunctionCall" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_async_FunctionCall" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *ptr = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceStub_async_FunctionCall" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_async_FunctionCall" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &""'"); 
    }
    arg3 = ptr;
  }
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedServiceStub_async_FunctionCall" "', argument " "4"" of type '" "int32_t""'");
  } 
  arg4 = static_cast< int32_t >(val4);
  res5 = SWIG_ConvertPtr(swig_obj[4], &argp5,SWIGTYPE_p_RobotRaconteur__AsyncRequestDirector, 0 |  0 );
  if (!SWIG_IsOK(res5)) {
    SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "WrappedServiceStub_async_FunctionCall" "', argument " "5"" of type '" "RobotRaconteur::AsyncRequestDirector *""'"); 
  }
  arg5 = reinterpret_cast< RobotRaconteur::AsyncRequestDirector * >(argp5);
  ecode6 = SWIG_AsVal_int(swig_obj[5], &val6);
  if (!SWIG_IsOK(ecode6)) {
    SWIG_exception_fail(SWIG_ArgError(ecode6), "in method '" "WrappedServiceStub_async_FunctionCall" "', argument " "6"" of type '" "int32_t""'");
  } 
  arg6 = static_cast< int32_t >(val6);
  
  try {
    (arg1)->async_FunctionCall((std::string const &)*arg2,(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &)*arg3,arg4,arg5,arg6);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_async_GeneratorFunctionCall(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *arg3 = 0 ;
  int32_t arg4 ;
  RobotRaconteur::AsyncGeneratorClientReturnDirector *arg5 = (RobotRaconteur::AsyncGeneratorClientReturnDirector *) 0 ;
  int32_t arg6 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  int val4 ;
  int ecode4 = 0 ;
  void *argp5 = 0 ;
  int res5 = 0 ;
  int val6 ;
  int ecode6 = 0 ;
  PyObject *swig_obj[6] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceStub_async_GeneratorFunctionCall", 6, 6, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_async_GeneratorFunctionCall" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub_async_GeneratorFunctionCall" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_async_GeneratorFunctionCall" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *ptr = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceStub_async_GeneratorFunctionCall" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_async_GeneratorFunctionCall" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &""'"); 
    }
    arg3 = ptr;
  }
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedServiceStub_async_GeneratorFunctionCall" "', argument " "4"" of type '" "int32_t""'");
  } 
  arg4 = static_cast< int32_t >(val4);
  res5 = SWIG_ConvertPtr(swig_obj[4], &argp5,SWIGTYPE_p_RobotRaconteur__AsyncGeneratorClientReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res5)) {
    SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "WrappedServiceStub_async_GeneratorFunctionCall" "', argument " "5"" of type '" "RobotRaconteur::AsyncGeneratorClientReturnDirector *""'"); 
  }
  arg5 = reinterpret_cast< RobotRaconteur::AsyncGeneratorClientReturnDirector * >(argp5);
  ecode6 = SWIG_AsVal_int(swig_obj[5], &val6);
  if (!SWIG_IsOK(ecode6)) {
    SWIG_exception_fail(SWIG_ArgError(ecode6), "in method '" "WrappedServiceStub_async_GeneratorFunctionCall" "', argument " "6"" of type '" "int32_t""'");
  } 
  arg6 = static_cast< int32_t >(val6);
  
  try {
    (arg1)->async_GeneratorFunctionCall((std::string const &)*arg2,(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &)*arg3,arg4,arg5,arg6);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_async_FindObjRef__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  int32_t arg3 ;
  RobotRaconteur::AsyncStubReturnDirector *arg4 = (RobotRaconteur::AsyncStubReturnDirector *) 0 ;
  int32_t arg5 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int val3 ;
  int ecode3 = 0 ;
  void *argp4 = 0 ;
  int res4 = 0 ;
  int val5 ;
  int ecode5 = 0 ;
  
  if ((nobjs < 5) || (nobjs > 5)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_async_FindObjRef" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub_async_FindObjRef" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_async_FindObjRef" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedServiceStub_async_FindObjRef" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  res4 = SWIG_ConvertPtr(swig_obj[3], &argp4,SWIGTYPE_p_RobotRaconteur__AsyncStubReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedServiceStub_async_FindObjRef" "', argument " "4"" of type '" "RobotRaconteur::AsyncStubReturnDirector *""'"); 
  }
  arg4 = reinterpret_cast< RobotRaconteur::AsyncStubReturnDirector * >(argp4);
  ecode5 = SWIG_AsVal_int(swig_obj[4], &val5);
  if (!SWIG_IsOK(ecode5)) {
    SWIG_exception_fail(SWIG_ArgError(ecode5), "in method '" "WrappedServiceStub_async_FindObjRef" "', argument " "5"" of type '" "int32_t""'");
  } 
  arg5 = static_cast< int32_t >(val5);
  
  try {
    (arg1)->async_FindObjRef((std::string const &)*arg2,arg3,arg4,arg5);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_async_FindObjRef__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  int32_t arg4 ;
  RobotRaconteur::AsyncStubReturnDirector *arg5 = (RobotRaconteur::AsyncStubReturnDirector *) 0 ;
  int32_t arg6 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  int val4 ;
  int ecode4 = 0 ;
  void *argp5 = 0 ;
  int res5 = 0 ;
  int val6 ;
  int ecode6 = 0 ;
  
  if ((nobjs < 6) || (nobjs > 6)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_async_FindObjRef" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub_async_FindObjRef" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_async_FindObjRef" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceStub_async_FindObjRef" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_async_FindObjRef" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedServiceStub_async_FindObjRef" "', argument " "4"" of type '" "int32_t""'");
  } 
  arg4 = static_cast< int32_t >(val4);
  res5 = SWIG_ConvertPtr(swig_obj[4], &argp5,SWIGTYPE_p_RobotRaconteur__AsyncStubReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res5)) {
    SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "WrappedServiceStub_async_FindObjRef" "', argument " "5"" of type '" "RobotRaconteur::AsyncStubReturnDirector *""'"); 
  }
  arg5 = reinterpret_cast< RobotRaconteur::AsyncStubReturnDirector * >(argp5);
  ecode6 = SWIG_AsVal_int(swig_obj[5], &val6);
  if (!SWIG_IsOK(ecode6)) {
    SWIG_exception_fail(SWIG_ArgError(ecode6), "in method '" "WrappedServiceStub_async_FindObjRef" "', argument " "6"" of type '" "int32_t""'");
  } 
  arg6 = static_cast< int32_t >(val6);
  
  try {
    (arg1)->async_FindObjRef((std::string const &)*arg2,(std::string const &)*arg3,arg4,arg5,arg6);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_async_FindObjRef(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[7] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedServiceStub_async_FindObjRef", 0, 6, argv))) SWIG_fail;
  --argc;
  if (argc == 5) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        {
          int res = SWIG_AsVal_int(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          void *vptr = 0;
          int res = SWIG_ConvertPtr(argv[3], &vptr, SWIGTYPE_p_RobotRaconteur__AsyncStubReturnDirector, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            {
              int res = SWIG_AsVal_int(argv[4], NULL);
              _v = SWIG_CheckState(res);
            }
            if (_v) {
              return _wrap_WrappedServiceStub_async_FindObjRef__SWIG_0(self, argc, argv);
            }
          }
        }
      }
    }
  }
  if (argc == 6) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          {
            int res = SWIG_AsVal_int(argv[3], NULL);
            _v = SWIG_CheckState(res);
          }
          if (_v) {
            void *vptr = 0;
            int res = SWIG_ConvertPtr(argv[4], &vptr, SWIGTYPE_p_RobotRaconteur__AsyncStubReturnDirector, 0);
            _v = SWIG_CheckState(res);
            if (_v) {
              {
                int res = SWIG_AsVal_int(argv[5], NULL);
                _v = SWIG_CheckState(res);
              }
              if (_v) {
                return _wrap_WrappedServiceStub_async_FindObjRef__SWIG_1(self, argc, argv);
              }
            }
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedServiceStub_async_FindObjRef'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedServiceStub::async_FindObjRef(std::string const &,int32_t,RobotRaconteur::AsyncStubReturnDirector *,int32_t)\n"
    "    RobotRaconteur::WrappedServiceStub::async_FindObjRef(std::string const &,std::string const &,int32_t,RobotRaconteur::AsyncStubReturnDirector *,int32_t)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_async_FindObjRefTyped__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  int32_t arg4 ;
  RobotRaconteur::AsyncStubReturnDirector *arg5 = (RobotRaconteur::AsyncStubReturnDirector *) 0 ;
  int32_t arg6 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  int val4 ;
  int ecode4 = 0 ;
  void *argp5 = 0 ;
  int res5 = 0 ;
  int val6 ;
  int ecode6 = 0 ;
  
  if ((nobjs < 6) || (nobjs > 6)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_async_FindObjRefTyped" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub_async_FindObjRefTyped" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_async_FindObjRefTyped" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceStub_async_FindObjRefTyped" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_async_FindObjRefTyped" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedServiceStub_async_FindObjRefTyped" "', argument " "4"" of type '" "int32_t""'");
  } 
  arg4 = static_cast< int32_t >(val4);
  res5 = SWIG_ConvertPtr(swig_obj[4], &argp5,SWIGTYPE_p_RobotRaconteur__AsyncStubReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res5)) {
    SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "WrappedServiceStub_async_FindObjRefTyped" "', argument " "5"" of type '" "RobotRaconteur::AsyncStubReturnDirector *""'"); 
  }
  arg5 = reinterpret_cast< RobotRaconteur::AsyncStubReturnDirector * >(argp5);
  ecode6 = SWIG_AsVal_int(swig_obj[5], &val6);
  if (!SWIG_IsOK(ecode6)) {
    SWIG_exception_fail(SWIG_ArgError(ecode6), "in method '" "WrappedServiceStub_async_FindObjRefTyped" "', argument " "6"" of type '" "int32_t""'");
  } 
  arg6 = static_cast< int32_t >(val6);
  
  try {
    (arg1)->async_FindObjRefTyped((std::string const &)*arg2,(std::string const &)*arg3,arg4,arg5,arg6);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_async_FindObjRefTyped__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  std::string *arg4 = 0 ;
  int32_t arg5 ;
  RobotRaconteur::AsyncStubReturnDirector *arg6 = (RobotRaconteur::AsyncStubReturnDirector *) 0 ;
  int32_t arg7 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  int res4 = SWIG_OLDOBJ ;
  int val5 ;
  int ecode5 = 0 ;
  void *argp6 = 0 ;
  int res6 = 0 ;
  int val7 ;
  int ecode7 = 0 ;
  
  if ((nobjs < 7) || (nobjs > 7)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_async_FindObjRefTyped" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub_async_FindObjRefTyped" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_async_FindObjRefTyped" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceStub_async_FindObjRefTyped" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_async_FindObjRefTyped" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res4 = SWIG_AsPtr_std_string(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedServiceStub_async_FindObjRefTyped" "', argument " "4"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_async_FindObjRefTyped" "', argument " "4"" of type '" "std::string const &""'"); 
    }
    arg4 = ptr;
  }
  ecode5 = SWIG_AsVal_int(swig_obj[4], &val5);
  if (!SWIG_IsOK(ecode5)) {
    SWIG_exception_fail(SWIG_ArgError(ecode5), "in method '" "WrappedServiceStub_async_FindObjRefTyped" "', argument " "5"" of type '" "int32_t""'");
  } 
  arg5 = static_cast< int32_t >(val5);
  res6 = SWIG_ConvertPtr(swig_obj[5], &argp6,SWIGTYPE_p_RobotRaconteur__AsyncStubReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res6)) {
    SWIG_exception_fail(SWIG_ArgError(res6), "in method '" "WrappedServiceStub_async_FindObjRefTyped" "', argument " "6"" of type '" "RobotRaconteur::AsyncStubReturnDirector *""'"); 
  }
  arg6 = reinterpret_cast< RobotRaconteur::AsyncStubReturnDirector * >(argp6);
  ecode7 = SWIG_AsVal_int(swig_obj[6], &val7);
  if (!SWIG_IsOK(ecode7)) {
    SWIG_exception_fail(SWIG_ArgError(ecode7), "in method '" "WrappedServiceStub_async_FindObjRefTyped" "', argument " "7"" of type '" "int32_t""'");
  } 
  arg7 = static_cast< int32_t >(val7);
  
  try {
    (arg1)->async_FindObjRefTyped((std::string const &)*arg2,(std::string const &)*arg3,(std::string const &)*arg4,arg5,arg6,arg7);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_async_FindObjRefTyped(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[8] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedServiceStub_async_FindObjRefTyped", 0, 7, argv))) SWIG_fail;
  --argc;
  if (argc == 6) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          {
            int res = SWIG_AsVal_int(argv[3], NULL);
            _v = SWIG_CheckState(res);
          }
          if (_v) {
            void *vptr = 0;
            int res = SWIG_ConvertPtr(argv[4], &vptr, SWIGTYPE_p_RobotRaconteur__AsyncStubReturnDirector, 0);
            _v = SWIG_CheckState(res);
            if (_v) {
              {
                int res = SWIG_AsVal_int(argv[5], NULL);
                _v = SWIG_CheckState(res);
              }
              if (_v) {
                return _wrap_WrappedServiceStub_async_FindObjRefTyped__SWIG_0(self, argc, argv);
              }
            }
          }
        }
      }
    }
  }
  if (argc == 7) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_AsPtr_std_string(argv[3], (std::string**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            {
              int res = SWIG_AsVal_int(argv[4], NULL);
              _v = SWIG_CheckState(res);
            }
            if (_v) {
              void *vptr = 0;
              int res = SWIG_ConvertPtr(argv[5], &vptr, SWIGTYPE_p_RobotRaconteur__AsyncStubReturnDirector, 0);
              _v = SWIG_CheckState(res);
              if (_v) {
                {
                  int res = SWIG_AsVal_int(argv[6], NULL);
                  _v = SWIG_CheckState(res);
                }
                if (_v) {
                  return _wrap_WrappedServiceStub_async_FindObjRefTyped__SWIG_1(self, argc, argv);
                }
              }
            }
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedServiceStub_async_FindObjRefTyped'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedServiceStub::async_FindObjRefTyped(std::string const &,std::string const &,int32_t,RobotRaconteur::AsyncStubReturnDirector *,int32_t)\n"
    "    RobotRaconteur::WrappedServiceStub::async_FindObjRefTyped(std::string const &,std::string const &,std::string const &,int32_t,RobotRaconteur::AsyncStubReturnDirector *,int32_t)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_GetPipe(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeClient > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceStub_GetPipe", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_GetPipe" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub_GetPipe" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_GetPipe" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetPipe((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedPipeClient > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedPipeClient >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeClient_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_GetWire(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::WrappedWireClient > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceStub_GetWire", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_GetWire" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub_GetWire" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_GetWire" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetWire((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedWireClient > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedWireClient >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireClient_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_GetObjectHeapID(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  int result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_GetObjectHeapID" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (int)(arg1)->GetObjectHeapID();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub__FindObjRef__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub__FindObjRef" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub__FindObjRef" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub__FindObjRef" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_WrappedServiceStub_FindObjRef__SWIG_0(arg1,(std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub__FindObjRef__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub__FindObjRef" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub__FindObjRef" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub__FindObjRef" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceStub__FindObjRef" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub__FindObjRef" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_WrappedServiceStub_FindObjRef__SWIG_1(arg1,(std::string const &)*arg2,(std::string const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub__FindObjRef(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedServiceStub__FindObjRef", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_WrappedServiceStub__FindObjRef__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_WrappedServiceStub__FindObjRef__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedServiceStub__FindObjRef'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedServiceStub::FindObjRef(std::string const &)\n"
    "    RobotRaconteur::WrappedServiceStub::FindObjRef(std::string const &,std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub__FindObjRefTyped__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub__FindObjRefTyped" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub__FindObjRefTyped" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub__FindObjRefTyped" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceStub__FindObjRefTyped" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub__FindObjRefTyped" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_WrappedServiceStub_FindObjRefTyped__SWIG_0(arg1,(std::string const &)*arg2,(std::string const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub__FindObjRefTyped__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  std::string *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  int res4 = SWIG_OLDOBJ ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > result;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub__FindObjRefTyped" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub__FindObjRefTyped" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub__FindObjRefTyped" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceStub__FindObjRefTyped" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub__FindObjRefTyped" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res4 = SWIG_AsPtr_std_string(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedServiceStub__FindObjRefTyped" "', argument " "4"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub__FindObjRefTyped" "', argument " "4"" of type '" "std::string const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_WrappedServiceStub_FindObjRefTyped__SWIG_1(arg1,(std::string const &)*arg2,(std::string const &)*arg3,(std::string const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub__FindObjRefTyped(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedServiceStub__FindObjRefTyped", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_WrappedServiceStub__FindObjRefTyped__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_AsPtr_std_string(argv[3], (std::string**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_WrappedServiceStub__FindObjRefTyped__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedServiceStub__FindObjRefTyped'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedServiceStub::FindObjRefTyped(std::string const &,std::string const &)\n"
    "    RobotRaconteur::WrappedServiceStub::FindObjRefTyped(std::string const &,std::string const &,std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_GetArrayMemory(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceStub_GetArrayMemory", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_GetArrayMemory" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub_GetArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_GetArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetArrayMemory((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ArrayMemoryBase > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ArrayMemoryBase >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ArrayMemoryBase_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_GetMultiDimArrayMemory(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceStub_GetMultiDimArrayMemory", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_GetMultiDimArrayMemory" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub_GetMultiDimArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_GetMultiDimArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetMultiDimArrayMemory((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::MultiDimArrayMemoryBase > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::MultiDimArrayMemoryBase >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MultiDimArrayMemoryBase_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_GetPodArrayMemory(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::WrappedPodArrayMemoryClient > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceStub_GetPodArrayMemory", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_GetPodArrayMemory" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub_GetPodArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_GetPodArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetPodArrayMemory((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedPodArrayMemoryClient > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedPodArrayMemoryClient >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPodArrayMemoryClient_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_GetPodMultiDimArrayMemory(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceStub_GetPodMultiDimArrayMemory", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_GetPodMultiDimArrayMemory" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub_GetPodMultiDimArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_GetPodMultiDimArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetPodMultiDimArrayMemory((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedPodMultiDimArrayMemoryClient >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPodMultiDimArrayMemoryClient_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_GetNamedArrayMemory(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedArrayMemoryClient > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceStub_GetNamedArrayMemory", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_GetNamedArrayMemory" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub_GetNamedArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_GetNamedArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetNamedArrayMemory((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedNamedArrayMemoryClient > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedNamedArrayMemoryClient >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedNamedArrayMemoryClient_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_GetNamedMultiDimArrayMemory(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceStub_GetNamedMultiDimArrayMemory", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_GetNamedMultiDimArrayMemory" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub_GetNamedMultiDimArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceStub_GetNamedMultiDimArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetNamedMultiDimArrayMemory((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClient_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_RRClose(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_RRClose" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->RRClose();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_RR_objecttype_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceStub_RR_objecttype_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_RR_objecttype_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub_RR_objecttype_set" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp2) : &tempshared2;
    }
  }
  if (arg1) (arg1)->RR_objecttype = *arg2;
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_RR_objecttype_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_RR_objecttype_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  result = (boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *) & ((arg1)->RR_objecttype);
  {
    boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_SetRRDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  RobotRaconteur::WrappedServiceStubDirector *arg2 = (RobotRaconteur::WrappedServiceStubDirector *) 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceStub_SetRRDirector", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_SetRRDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__WrappedServiceStubDirector, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceStub_SetRRDirector" "', argument " "2"" of type '" "RobotRaconteur::WrappedServiceStubDirector *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedServiceStubDirector * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedServiceStub_SetRRDirector" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    result = (bool)(arg1)->SetRRDirector(arg2,arg3);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_GetRRDirectorLock(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::WrappedDirectorLock > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_GetRRDirectorLock" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->GetRRDirectorLock();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedDirectorLock > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedDirectorLock >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedDirectorLock_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_RRGetNode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_RRGetNode" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->RRGetNode();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_GetPyStub(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_GetPyStub" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)(arg1)->GetPyStub();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceStub_SetPyStub(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  PyObject *arg2 = (PyObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceStub_SetPyStub", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceStub_SetPyStub" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    arg2 = swig_obj[1];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetPyStub(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedServiceStub(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceStub *arg1 = (RobotRaconteur::WrappedServiceStub *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedServiceStub" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceStub *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceStub * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    (void)arg1; delete smartarg1;
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedServiceStub_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_delete_WrappedServiceSkelDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkelDirector *arg1 = (RobotRaconteur::WrappedServiceSkelDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSkelDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedServiceSkelDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkelDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSkelDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkelDirector_Init(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkelDirector *arg1 = (RobotRaconteur::WrappedServiceSkelDirector *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > tempshared2 ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSkelDirector_Init", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSkelDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkelDirector_Init" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkelDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSkelDirector * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSkelDirector_Init" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > * >(argp2) : &tempshared2;
    }
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedServiceSkelDirector::Init");
      } else {
        (arg1)->Init((boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > const &)*arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkelDirector__CallGetProperty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkelDirector *arg1 = (RobotRaconteur::WrappedServiceSkelDirector *) 0 ;
  std::string *arg2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 ;
  int res3 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > tempshared3 ;
  PyObject *swig_obj[3] ;
  Swig::Director *director = 0;
  bool upcall = false;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSkelDirector__CallGetProperty", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSkelDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkelDirector__CallGetProperty" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkelDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSkelDirector * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSkelDirector__CallGetProperty" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSkelDirector__CallGetProperty" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSkelDirector__CallGetProperty" "', argument " "3"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp3);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp3) : &tempshared3;
    }
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedServiceSkelDirector::CallGetProperty");
      } else {
        result = (arg1)->CallGetProperty((std::string const &)*arg2,(boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > const &)*arg3);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkelDirector__CallSetProperty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkelDirector *arg1 = (RobotRaconteur::WrappedServiceSkelDirector *) 0 ;
  std::string *arg2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > arg3 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 ;
  int res3 = 0 ;
  int newmem3 = 0 ;
  void *argp4 ;
  int res4 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > tempshared4 ;
  PyObject *swig_obj[4] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSkelDirector__CallSetProperty", 4, 4, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSkelDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkelDirector__CallSetProperty" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkelDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSkelDirector * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSkelDirector__CallSetProperty" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSkelDirector__CallSetProperty" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[2];
  //if (smartarg) {
  //	arg3 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //}
  
  res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem3);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSkelDirector__CallSetProperty" "', argument " "3"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement >""'");
  }
  if (argp3) arg3 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp3)->get(),true);
  if (newmem3 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp3);
  
  {
    int newmem = 0;
    res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t,  0 , &newmem);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedServiceSkelDirector__CallSetProperty" "', argument " "4"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp4) tempshared4 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp4);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp4);
      arg4 = &tempshared4;
    } else {
      arg4 = (argp4) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp4) : &tempshared4;
    }
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedServiceSkelDirector::CallSetProperty");
      } else {
        (arg1)->CallSetProperty((std::string const &)*arg2,arg3,(boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > const &)*arg4);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkelDirector__CallFunction(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkelDirector *arg1 = (RobotRaconteur::WrappedServiceSkelDirector *) 0 ;
  std::string *arg2 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *arg3 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  void *argp4 ;
  int res4 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > tempshared4 ;
  PyObject *swig_obj[4] ;
  Swig::Director *director = 0;
  bool upcall = false;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSkelDirector__CallFunction", 4, 4, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSkelDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkelDirector__CallFunction" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkelDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSkelDirector * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSkelDirector__CallFunction" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSkelDirector__CallFunction" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *ptr = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSkelDirector__CallFunction" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSkelDirector__CallFunction" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &""'"); 
    }
    arg3 = ptr;
  }
  {
    int newmem = 0;
    res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t,  0 , &newmem);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedServiceSkelDirector__CallFunction" "', argument " "4"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp4) tempshared4 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp4);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp4);
      arg4 = &tempshared4;
    } else {
      arg4 = (argp4) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp4) : &tempshared4;
    }
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedServiceSkelDirector::CallFunction");
      } else {
        result = (arg1)->CallFunction((std::string const &)*arg2,(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &)*arg3,(boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > const &)*arg4);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkelDirector__GetSubObj(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkelDirector *arg1 = (RobotRaconteur::WrappedServiceSkelDirector *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  PyObject *swig_obj[3] ;
  Swig::Director *director = 0;
  bool upcall = false;
  boost::shared_ptr< RobotRaconteur::WrappedRRObject > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSkelDirector__GetSubObj", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSkelDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkelDirector__GetSubObj" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkelDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSkelDirector * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSkelDirector__GetSubObj" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSkelDirector__GetSubObj" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSkelDirector__GetSubObj" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSkelDirector__GetSubObj" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedServiceSkelDirector::GetSubObj");
      } else {
        result = (arg1)->GetSubObj((std::string const &)*arg2,(std::string const &)*arg3);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  {
    boost::shared_ptr<  RobotRaconteur::WrappedRRObject > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedRRObject >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedRRObject_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkelDirector__GetArrayMemory(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkelDirector *arg1 = (RobotRaconteur::WrappedServiceSkelDirector *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  RobotRaconteur::WrappedArrayMemoryDirector *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSkelDirector__GetArrayMemory", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSkelDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkelDirector__GetArrayMemory" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkelDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSkelDirector * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSkelDirector__GetArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSkelDirector__GetArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedServiceSkelDirector::GetArrayMemory");
      } else {
        result = (RobotRaconteur::WrappedArrayMemoryDirector *)(arg1)->GetArrayMemory((std::string const &)*arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  director = SWIG_DIRECTOR_CAST(result);
  if (director) {
    resultobj = director->swig_get_self();
    Py_INCREF(resultobj);
  } else {
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedArrayMemoryDirector, 0 |  0 );
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (director) {
    SWIG_AcquirePtr(resultobj, director->swig_release_ownership(SWIG_as_voidptr(result)));
  }
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkelDirector__GetMultiDimArrayMemory(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkelDirector *arg1 = (RobotRaconteur::WrappedServiceSkelDirector *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  RobotRaconteur::WrappedMultiDimArrayMemoryDirector *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSkelDirector__GetMultiDimArrayMemory", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSkelDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkelDirector__GetMultiDimArrayMemory" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkelDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSkelDirector * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSkelDirector__GetMultiDimArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSkelDirector__GetMultiDimArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedServiceSkelDirector::GetMultiDimArrayMemory");
      } else {
        result = (RobotRaconteur::WrappedMultiDimArrayMemoryDirector *)(arg1)->GetMultiDimArrayMemory((std::string const &)*arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  director = SWIG_DIRECTOR_CAST(result);
  if (director) {
    resultobj = director->swig_get_self();
    Py_INCREF(resultobj);
  } else {
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedMultiDimArrayMemoryDirector, 0 |  0 );
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (director) {
    SWIG_AcquirePtr(resultobj, director->swig_release_ownership(SWIG_as_voidptr(result)));
  }
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkelDirector__GetPodArrayMemory(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkelDirector *arg1 = (RobotRaconteur::WrappedServiceSkelDirector *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  RobotRaconteur::WrappedPodArrayMemoryDirector *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSkelDirector__GetPodArrayMemory", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSkelDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkelDirector__GetPodArrayMemory" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkelDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSkelDirector * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSkelDirector__GetPodArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSkelDirector__GetPodArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedServiceSkelDirector::GetPodArrayMemory");
      } else {
        result = (RobotRaconteur::WrappedPodArrayMemoryDirector *)(arg1)->GetPodArrayMemory((std::string const &)*arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  director = SWIG_DIRECTOR_CAST(result);
  if (director) {
    resultobj = director->swig_get_self();
    Py_INCREF(resultobj);
  } else {
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedPodArrayMemoryDirector, 0 |  0 );
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (director) {
    SWIG_AcquirePtr(resultobj, director->swig_release_ownership(SWIG_as_voidptr(result)));
  }
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkelDirector__GetPodMultiDimArrayMemory(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkelDirector *arg1 = (RobotRaconteur::WrappedServiceSkelDirector *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSkelDirector__GetPodMultiDimArrayMemory", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSkelDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkelDirector__GetPodMultiDimArrayMemory" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkelDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSkelDirector * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSkelDirector__GetPodMultiDimArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSkelDirector__GetPodMultiDimArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedServiceSkelDirector::GetPodMultiDimArrayMemory");
      } else {
        result = (RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *)(arg1)->GetPodMultiDimArrayMemory((std::string const &)*arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  director = SWIG_DIRECTOR_CAST(result);
  if (director) {
    resultobj = director->swig_get_self();
    Py_INCREF(resultobj);
  } else {
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryDirector, 0 |  0 );
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (director) {
    SWIG_AcquirePtr(resultobj, director->swig_release_ownership(SWIG_as_voidptr(result)));
  }
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkelDirector__GetNamedArrayMemory(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkelDirector *arg1 = (RobotRaconteur::WrappedServiceSkelDirector *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  RobotRaconteur::WrappedNamedArrayMemoryDirector *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSkelDirector__GetNamedArrayMemory", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSkelDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkelDirector__GetNamedArrayMemory" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkelDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSkelDirector * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSkelDirector__GetNamedArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSkelDirector__GetNamedArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedServiceSkelDirector::GetNamedArrayMemory");
      } else {
        result = (RobotRaconteur::WrappedNamedArrayMemoryDirector *)(arg1)->GetNamedArrayMemory((std::string const &)*arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  director = SWIG_DIRECTOR_CAST(result);
  if (director) {
    resultobj = director->swig_get_self();
    Py_INCREF(resultobj);
  } else {
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedNamedArrayMemoryDirector, 0 |  0 );
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (director) {
    SWIG_AcquirePtr(resultobj, director->swig_release_ownership(SWIG_as_voidptr(result)));
  }
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkelDirector__GetNamedMultiDimArrayMemory(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkelDirector *arg1 = (RobotRaconteur::WrappedServiceSkelDirector *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSkelDirector__GetNamedMultiDimArrayMemory", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSkelDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkelDirector__GetNamedMultiDimArrayMemory" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkelDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSkelDirector * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSkelDirector__GetNamedMultiDimArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSkelDirector__GetNamedMultiDimArrayMemory" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedServiceSkelDirector::GetNamedMultiDimArrayMemory");
      } else {
        result = (RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *)(arg1)->GetNamedMultiDimArrayMemory((std::string const &)*arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  director = SWIG_DIRECTOR_CAST(result);
  if (director) {
    resultobj = director->swig_get_self();
    Py_INCREF(resultobj);
  } else {
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryDirector, 0 |  0 );
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (director) {
    SWIG_AcquirePtr(resultobj, director->swig_release_ownership(SWIG_as_voidptr(result)));
  }
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkelDirector_MonitorEnter(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkelDirector *arg1 = (RobotRaconteur::WrappedServiceSkelDirector *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSkelDirector_MonitorEnter", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSkelDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkelDirector_MonitorEnter" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkelDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSkelDirector * >(argp1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedServiceSkelDirector_MonitorEnter" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedServiceSkelDirector::MonitorEnter");
      } else {
        (arg1)->MonitorEnter(arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkelDirector_MonitorExit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkelDirector *arg1 = (RobotRaconteur::WrappedServiceSkelDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSkelDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkelDirector_MonitorExit" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkelDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSkelDirector * >(argp1);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedServiceSkelDirector::MonitorExit");
      } else {
        (arg1)->MonitorExit();
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkelDirector_ReleaseCastObject(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkelDirector *arg1 = (RobotRaconteur::WrappedServiceSkelDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSkelDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkelDirector_ReleaseCastObject" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkelDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSkelDirector * >(argp1);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedServiceSkelDirector::ReleaseCastObject");
      } else {
        (arg1)->ReleaseCastObject();
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedServiceSkelDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedServiceSkelDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedServiceSkelDirector *)new SwigDirector_WrappedServiceSkelDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedServiceSkelDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedServiceSkelDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkelDirector *arg1 = (RobotRaconteur::WrappedServiceSkelDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSkelDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedServiceSkelDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkelDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSkelDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedServiceSkelDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedServiceSkelDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedServiceSkelDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_new_WrappedRRObject(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  RobotRaconteur::WrappedServiceSkelDirector *arg2 = (RobotRaconteur::WrappedServiceSkelDirector *) 0 ;
  int32_t arg3 ;
  int res1 = SWIG_OLDOBJ ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  RobotRaconteur::WrappedRRObject *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_WrappedRRObject", 3, 3, swig_obj)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_WrappedRRObject" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedRRObject" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__WrappedServiceSkelDirector, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_WrappedRRObject" "', argument " "2"" of type '" "RobotRaconteur::WrappedServiceSkelDirector *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedServiceSkelDirector * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "new_WrappedRRObject" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedRRObject *)new RobotRaconteur::WrappedRRObject((std::string const &)*arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedRRObject > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedRRObject >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedRRObject_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedRRObject_Type_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedRRObject *arg1 = (RobotRaconteur::WrappedRRObject *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedRRObject > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedRRObject > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedRRObject_Type_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedRRObject_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedRRObject_Type_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedRRObject *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedRRObject > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedRRObject > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedRRObject * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedRRObject > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedRRObject * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedRRObject_Type_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedRRObject_Type_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Type = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedRRObject_Type_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedRRObject *arg1 = (RobotRaconteur::WrappedRRObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedRRObject > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedRRObject > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedRRObject_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedRRObject_Type_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedRRObject *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedRRObject > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedRRObject > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedRRObject * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedRRObject > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedRRObject * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->Type);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedRRObject(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedRRObject *arg1 = (RobotRaconteur::WrappedRRObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedRRObject > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedRRObject > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedRRObject_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedRRObject" "', argument " "1"" of type '" "RobotRaconteur::WrappedRRObject *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedRRObject > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedRRObject > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedRRObject * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedRRObject > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedRRObject * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedRRObject_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedRRObject_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedRRObject_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedServiceSkel_Type_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkel *arg1 = (RobotRaconteur::WrappedServiceSkel *) 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSkel_Type_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkel_Type_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkel *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSkel_Type_set" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp2) : &tempshared2;
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Type = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkel_Type_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkel *arg1 = (RobotRaconteur::WrappedServiceSkel *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkel_Type_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkel *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *) & ((arg1)->Type);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  {
    boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::ServiceEntryDefinition >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkel_castobj_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkel *arg1 = (RobotRaconteur::WrappedServiceSkel *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedRRObject > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedRRObject > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSkel_castobj_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkel_castobj_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkel *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedRRObject_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSkel_castobj_set" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedRRObject > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedRRObject > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedRRObject > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedRRObject > * >(argp2) : &tempshared2;
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->castobj = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkel_castobj_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkel *arg1 = (RobotRaconteur::WrappedServiceSkel *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::WrappedRRObject > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkel_castobj_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkel *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::shared_ptr< RobotRaconteur::WrappedRRObject > *) & ((arg1)->castobj);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  {
    boost::shared_ptr<  RobotRaconteur::WrappedRRObject > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::WrappedRRObject >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedRRObject_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkel_GetPipe(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkel *arg1 = (RobotRaconteur::WrappedServiceSkel *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeServer > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSkel_GetPipe", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkel_GetPipe" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkel *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSkel_GetPipe" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSkel_GetPipe" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetPipe((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedPipeServer > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedPipeServer >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeServer_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkel_GetWire(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkel *arg1 = (RobotRaconteur::WrappedServiceSkel *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::WrappedWireServer > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSkel_GetWire", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkel_GetWire" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkel *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSkel_GetWire" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSkel_GetWire" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetWire((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedWireServer > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedWireServer >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireServer_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkel_WrappedDispatchEvent(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkel *arg1 = (RobotRaconteur::WrappedServiceSkel *) 0 ;
  std::string *arg2 = 0 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSkel_WrappedDispatchEvent", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkel_WrappedDispatchEvent" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkel *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSkel_WrappedDispatchEvent" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSkel_WrappedDispatchEvent" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *ptr = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSkel_WrappedDispatchEvent" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSkel_WrappedDispatchEvent" "', argument " "3"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->WrappedDispatchEvent((std::string const &)*arg2,(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkel_WrappedCallbackCall(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkel *arg1 = (RobotRaconteur::WrappedServiceSkel *) 0 ;
  std::string *arg2 = 0 ;
  uint32_t arg3 ;
  std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  unsigned int val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  PyObject *swig_obj[4] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSkel_WrappedCallbackCall", 4, 4, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkel_WrappedCallbackCall" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkel *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSkel_WrappedCallbackCall" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSkel_WrappedCallbackCall" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  ecode3 = SWIG_AsVal_unsigned_SS_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedServiceSkel_WrappedCallbackCall" "', argument " "3"" of type '" "uint32_t""'");
  } 
  arg3 = static_cast< uint32_t >(val3);
  {
    std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *ptr = (std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedServiceSkel_WrappedCallbackCall" "', argument " "4"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSkel_WrappedCallbackCall" "', argument " "4"" of type '" "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->WrappedCallbackCall((std::string const &)*arg2,arg3,(std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkel_RRGetNode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkel *arg1 = (RobotRaconteur::WrappedServiceSkel *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkel_RRGetNode" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkel *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->RRGetNode();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkel_RegisterGeneratorServer(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkel *arg1 = (RobotRaconteur::WrappedServiceSkel *) 0 ;
  std::string *arg2 = 0 ;
  RobotRaconteur::WrappedGeneratorServerDirector *arg3 = (RobotRaconteur::WrappedGeneratorServerDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  PyObject *swig_obj[3] ;
  int32_t result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSkel_RegisterGeneratorServer", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkel_RegisterGeneratorServer" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkel *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSkel_RegisterGeneratorServer" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSkel_RegisterGeneratorServer" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__WrappedGeneratorServerDirector, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSkel_RegisterGeneratorServer" "', argument " "3"" of type '" "RobotRaconteur::WrappedGeneratorServerDirector *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::WrappedGeneratorServerDirector * >(argp3);
  
  try {
    result = (int32_t)(arg1)->RegisterGeneratorServer((std::string const &)*arg2,arg3);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkel_GetServicePath(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkel *arg1 = (RobotRaconteur::WrappedServiceSkel *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkel_GetServicePath" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkel *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->GetServicePath();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkel_GetContext(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkel *arg1 = (RobotRaconteur::WrappedServiceSkel *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::ServerContext > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkel_GetContext" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkel *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->GetContext();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServerContext > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ServerContext >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedServiceSkel(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkel *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_WrappedServiceSkel", 0, 0, 0)) SWIG_fail;
  
  try {
    result = (RobotRaconteur::WrappedServiceSkel *)new RobotRaconteur::WrappedServiceSkel();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedServiceSkel(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkel *arg1 = (RobotRaconteur::WrappedServiceSkel *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedServiceSkel" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkel *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkel > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkel * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    (void)arg1; delete smartarg1;
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedServiceSkel_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedServiceSkel_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_ServerServiceListenerDirector_objectheapid_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  ServerServiceListenerDirector *arg1 = (ServerServiceListenerDirector *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServerServiceListenerDirector_objectheapid_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_ServerServiceListenerDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServerServiceListenerDirector_objectheapid_set" "', argument " "1"" of type '" "ServerServiceListenerDirector *""'"); 
  }
  arg1 = reinterpret_cast< ServerServiceListenerDirector * >(argp1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ServerServiceListenerDirector_objectheapid_set" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->objectheapid = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServerServiceListenerDirector_objectheapid_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  ServerServiceListenerDirector *arg1 = (ServerServiceListenerDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_ServerServiceListenerDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServerServiceListenerDirector_objectheapid_get" "', argument " "1"" of type '" "ServerServiceListenerDirector *""'"); 
  }
  arg1 = reinterpret_cast< ServerServiceListenerDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (int32_t) ((arg1)->objectheapid);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ServerServiceListenerDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  ServerServiceListenerDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (ServerServiceListenerDirector *)new SwigDirector_ServerServiceListenerDirector(arg1); 
      } else {
        result = (ServerServiceListenerDirector *)new ServerServiceListenerDirector(); 
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_ServerServiceListenerDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServerServiceListenerDirector_Callback(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  ServerServiceListenerDirector *arg1 = (ServerServiceListenerDirector *) 0 ;
  int32_t arg2 ;
  uint32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  unsigned int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "ServerServiceListenerDirector_Callback", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_ServerServiceListenerDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServerServiceListenerDirector_Callback" "', argument " "1"" of type '" "ServerServiceListenerDirector *""'"); 
  }
  arg1 = reinterpret_cast< ServerServiceListenerDirector * >(argp1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ServerServiceListenerDirector_Callback" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  ecode3 = SWIG_AsVal_unsigned_SS_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "ServerServiceListenerDirector_Callback" "', argument " "3"" of type '" "uint32_t""'");
  } 
  arg3 = static_cast< uint32_t >(val3);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        (arg1)->ServerServiceListenerDirector::Callback(arg2,arg3);
      } else {
        (arg1)->Callback(arg2,arg3);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_ServerServiceListenerDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  ServerServiceListenerDirector *arg1 = (ServerServiceListenerDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_ServerServiceListenerDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_ServerServiceListenerDirector" "', argument " "1"" of type '" "ServerServiceListenerDirector *""'"); 
  }
  arg1 = reinterpret_cast< ServerServiceListenerDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_ServerServiceListenerDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  ServerServiceListenerDirector *arg1 = (ServerServiceListenerDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_ServerServiceListenerDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_ServerServiceListenerDirector" "', argument " "1"" of type '" "ServerServiceListenerDirector *""'"); 
  }
  arg1 = reinterpret_cast< ServerServiceListenerDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *ServerServiceListenerDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_ServerServiceListenerDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *ServerServiceListenerDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_ServerContext_GetCurrentServicePath(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::string result;
  
  if (!SWIG_Python_UnpackTuple(args, "ServerContext_GetCurrentServicePath", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::ServerContext::GetCurrentServicePath();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServerContext__ReleaseServicePath__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServerContext *arg1 = (RobotRaconteur::ServerContext *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServerContext__ReleaseServicePath" "', argument " "1"" of type '" "RobotRaconteur::ServerContext *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServerContext__ReleaseServicePath" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServerContext__ReleaseServicePath" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->ReleaseServicePath((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServerContext__ReleaseServicePath__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServerContext *arg1 = (RobotRaconteur::ServerContext *) 0 ;
  std::string *arg2 = 0 ;
  std::vector< uint32_t,std::allocator< uint32_t > > arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServerContext__ReleaseServicePath" "', argument " "1"" of type '" "RobotRaconteur::ServerContext *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServerContext__ReleaseServicePath" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServerContext__ReleaseServicePath" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::vector< unsigned int,std::allocator< unsigned int > > *ptr = (std::vector< unsigned int,std::allocator< unsigned int > > *)0;
    int res = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res) || !ptr) {
      SWIG_exception_fail(SWIG_ArgError((ptr ? res : SWIG_TypeError)), "in method '" "ServerContext__ReleaseServicePath" "', argument " "3"" of type '" "std::vector< uint32_t,std::allocator< uint32_t > >""'"); 
    }
    arg3 = *ptr;
    if (SWIG_IsNewObj(res)) delete ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->ReleaseServicePath((std::string const &)*arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServerContext__ReleaseServicePath(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "ServerContext__ReleaseServicePath", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_ServerContext__ReleaseServicePath__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< unsigned int,std::allocator< unsigned int > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_ServerContext__ReleaseServicePath__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'ServerContext__ReleaseServicePath'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::ServerContext::ReleaseServicePath(std::string const &)\n"
    "    RobotRaconteur::ServerContext::ReleaseServicePath(std::string const &,std::vector< uint32_t,std::allocator< uint32_t > >)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_ServerContext_GetCurrentServerContext(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::ServerContext > result;
  
  if (!SWIG_Python_UnpackTuple(args, "ServerContext_GetCurrentServerContext", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::ServerContext::GetCurrentServerContext();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServerContext > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ServerContext >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServerContext_KickUser(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServerContext *arg1 = (RobotRaconteur::ServerContext *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServerContext_KickUser", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServerContext_KickUser" "', argument " "1"" of type '" "RobotRaconteur::ServerContext *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServerContext_KickUser" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServerContext_KickUser" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->KickUser((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServerContext__AddServerServiceListener(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServerContext *arg1 = (RobotRaconteur::ServerContext *) 0 ;
  ServerServiceListenerDirector *arg2 = (ServerServiceListenerDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServerContext__AddServerServiceListener", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServerContext__AddServerServiceListener" "', argument " "1"" of type '" "RobotRaconteur::ServerContext *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_ServerServiceListenerDirector, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServerContext__AddServerServiceListener" "', argument " "2"" of type '" "ServerServiceListenerDirector *""'"); 
  }
  arg2 = reinterpret_cast< ServerServiceListenerDirector * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur_ServerContext_AddServerServiceListener(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServerContext__SetServiceAttributes(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServerContext *arg1 = (RobotRaconteur::ServerContext *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServerContext__SetServiceAttributes", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServerContext__SetServiceAttributes" "', argument " "1"" of type '" "RobotRaconteur::ServerContext *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServerContext__SetServiceAttributes" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur_ServerContext_SetServiceAttributes(arg1,(boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServerContext_RequestObjectLock(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServerContext *arg1 = (RobotRaconteur::ServerContext *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServerContext_RequestObjectLock", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServerContext_RequestObjectLock" "', argument " "1"" of type '" "RobotRaconteur::ServerContext *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServerContext_RequestObjectLock" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServerContext_RequestObjectLock" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "ServerContext_RequestObjectLock" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServerContext_RequestObjectLock" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->RequestObjectLock((std::string const &)*arg2,(std::string const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServerContext_RequestClientObjectLock(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServerContext *arg1 = (RobotRaconteur::ServerContext *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  uint32_t arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  unsigned int val4 ;
  int ecode4 = 0 ;
  PyObject *swig_obj[4] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServerContext_RequestClientObjectLock", 4, 4, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServerContext_RequestClientObjectLock" "', argument " "1"" of type '" "RobotRaconteur::ServerContext *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServerContext_RequestClientObjectLock" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServerContext_RequestClientObjectLock" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "ServerContext_RequestClientObjectLock" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServerContext_RequestClientObjectLock" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  ecode4 = SWIG_AsVal_unsigned_SS_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "ServerContext_RequestClientObjectLock" "', argument " "4"" of type '" "uint32_t""'");
  } 
  arg4 = static_cast< uint32_t >(val4);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->RequestClientObjectLock((std::string const &)*arg2,(std::string const &)*arg3,arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServerContext_ReleaseObjectLock(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServerContext *arg1 = (RobotRaconteur::ServerContext *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  bool arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  bool val4 ;
  int ecode4 = 0 ;
  PyObject *swig_obj[4] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServerContext_ReleaseObjectLock", 4, 4, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServerContext_ReleaseObjectLock" "', argument " "1"" of type '" "RobotRaconteur::ServerContext *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServerContext_ReleaseObjectLock" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServerContext_ReleaseObjectLock" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "ServerContext_ReleaseObjectLock" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServerContext_ReleaseObjectLock" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  ecode4 = SWIG_AsVal_bool(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "ServerContext_ReleaseObjectLock" "', argument " "4"" of type '" "bool""'");
  } 
  arg4 = static_cast< bool >(val4);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->ReleaseObjectLock((std::string const &)*arg2,(std::string const &)*arg3,arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServerContext_GetObjectLockUsername(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServerContext *arg1 = (RobotRaconteur::ServerContext *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  std::string result;
  
  if (!SWIG_Python_UnpackTuple(args, "ServerContext_GetObjectLockUsername", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServerContext_GetObjectLockUsername" "', argument " "1"" of type '" "RobotRaconteur::ServerContext *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServerContext_GetObjectLockUsername" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServerContext_GetObjectLockUsername" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetObjectLockUsername((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServerContext_GetNode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServerContext *arg1 = (RobotRaconteur::ServerContext *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServerContext_GetNode" "', argument " "1"" of type '" "RobotRaconteur::ServerContext *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetNode();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServerContext_GetExtraImports(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServerContext *arg1 = (RobotRaconteur::ServerContext *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServerContext_GetExtraImports" "', argument " "1"" of type '" "RobotRaconteur::ServerContext *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetExtraImports();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< std::string,std::allocator< std::string > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServerContext_AddExtraImport(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServerContext *arg1 = (RobotRaconteur::ServerContext *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServerContext_AddExtraImport", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServerContext_AddExtraImport" "', argument " "1"" of type '" "RobotRaconteur::ServerContext *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServerContext_AddExtraImport" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServerContext_AddExtraImport" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->AddExtraImport((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServerContext_RemoveExtraImport(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServerContext *arg1 = (RobotRaconteur::ServerContext *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "ServerContext_RemoveExtraImport", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServerContext_RemoveExtraImport" "', argument " "1"" of type '" "RobotRaconteur::ServerContext *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServerContext_RemoveExtraImport" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServerContext_RemoveExtraImport" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->RemoveExtraImport((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServerContext_GetCandidateConnectionURLs(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServerContext *arg1 = (RobotRaconteur::ServerContext *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServerContext_GetCandidateConnectionURLs" "', argument " "1"" of type '" "RobotRaconteur::ServerContext *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetCandidateConnectionURLs();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< std::string,std::allocator< std::string > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServerContext_PrintCandidateConnectionURLs(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServerContext *arg1 = (RobotRaconteur::ServerContext *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServerContext_PrintCandidateConnectionURLs" "', argument " "1"" of type '" "RobotRaconteur::ServerContext *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->PrintCandidateConnectionURLs();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServerContext_LogCandidateConnectionURLs__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServerContext *arg1 = (RobotRaconteur::ServerContext *) 0 ;
  RobotRaconteur::RobotRaconteur_LogLevel arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServerContext_LogCandidateConnectionURLs" "', argument " "1"" of type '" "RobotRaconteur::ServerContext *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ServerContext_LogCandidateConnectionURLs" "', argument " "2"" of type '" "RobotRaconteur::RobotRaconteur_LogLevel""'");
  } 
  arg2 = static_cast< RobotRaconteur::RobotRaconteur_LogLevel >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->LogCandidateConnectionURLs(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServerContext_LogCandidateConnectionURLs__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServerContext *arg1 = (RobotRaconteur::ServerContext *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *smartarg1 = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServerContext_LogCandidateConnectionURLs" "', argument " "1"" of type '" "RobotRaconteur::ServerContext *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->LogCandidateConnectionURLs();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServerContext_LogCandidateConnectionURLs(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "ServerContext_LogCandidateConnectionURLs", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_ServerContext_LogCandidateConnectionURLs__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_ServerContext_LogCandidateConnectionURLs__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'ServerContext_LogCandidateConnectionURLs'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::ServerContext::LogCandidateConnectionURLs(RobotRaconteur::RobotRaconteur_LogLevel)\n"
    "    RobotRaconteur::ServerContext::LogCandidateConnectionURLs()\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_delete_ServerContext(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServerContext *arg1 = (RobotRaconteur::ServerContext *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_ServerContext" "', argument " "1"" of type '" "RobotRaconteur::ServerContext *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerContext > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerContext * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *ServerContext_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_AuthenticatedUser__GetUsername(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AuthenticatedUser *arg1 = (RobotRaconteur::AuthenticatedUser *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::AuthenticatedUser > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::AuthenticatedUser > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__AuthenticatedUser_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AuthenticatedUser__GetUsername" "', argument " "1"" of type '" "RobotRaconteur::AuthenticatedUser *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AuthenticatedUser > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AuthenticatedUser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::AuthenticatedUser * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AuthenticatedUser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::AuthenticatedUser * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetUsername();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_AuthenticatedUser__GetPrivileges(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AuthenticatedUser *arg1 = (RobotRaconteur::AuthenticatedUser *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::AuthenticatedUser > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::AuthenticatedUser > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__AuthenticatedUser_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AuthenticatedUser__GetPrivileges" "', argument " "1"" of type '" "RobotRaconteur::AuthenticatedUser *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AuthenticatedUser > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AuthenticatedUser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::AuthenticatedUser * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AuthenticatedUser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::AuthenticatedUser * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetPrivileges();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< std::string,std::allocator< std::string > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_AuthenticatedUser__GetProperties(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AuthenticatedUser *arg1 = (RobotRaconteur::AuthenticatedUser *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::AuthenticatedUser > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::AuthenticatedUser > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__AuthenticatedUser_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AuthenticatedUser__GetProperties" "', argument " "1"" of type '" "RobotRaconteur::AuthenticatedUser *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AuthenticatedUser > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AuthenticatedUser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::AuthenticatedUser * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AuthenticatedUser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::AuthenticatedUser * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetProperties();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< std::string,std::allocator< std::string > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_AuthenticatedUser__GetLoginTime(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AuthenticatedUser *arg1 = (RobotRaconteur::AuthenticatedUser *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::AuthenticatedUser > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::AuthenticatedUser > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::posix_time::ptime result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__AuthenticatedUser_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AuthenticatedUser__GetLoginTime" "', argument " "1"" of type '" "RobotRaconteur::AuthenticatedUser *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AuthenticatedUser > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AuthenticatedUser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::AuthenticatedUser * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AuthenticatedUser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::AuthenticatedUser * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetLoginTime();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    try {
      resultobj = PyDateTime_FromDateAndTime((int)(&result)->date().year(), (int)(&result)->date().month(), (int)(&result)->date().day(),
        (int)(&result)->time_of_day().hours(), (int)(&result)->time_of_day().minutes(), (int)(&result)->time_of_day().seconds(),
        boost::numeric_cast<int32_t>(((&result)->time_of_day().fractional_seconds() * boost::numeric_cast<int32_t>(pow(10.0,(9-(&result)->time_of_day().num_fractional_digits())))))/1000);
    }
    catch (std::exception&)
    {
      resultobj = SWIG_Py_Void();
    }
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_AuthenticatedUser__GetLastAccessTime(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AuthenticatedUser *arg1 = (RobotRaconteur::AuthenticatedUser *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::AuthenticatedUser > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::AuthenticatedUser > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::posix_time::ptime result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__AuthenticatedUser_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AuthenticatedUser__GetLastAccessTime" "', argument " "1"" of type '" "RobotRaconteur::AuthenticatedUser *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AuthenticatedUser > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AuthenticatedUser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::AuthenticatedUser * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AuthenticatedUser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::AuthenticatedUser * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetLastAccessTime();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    try {
      resultobj = PyDateTime_FromDateAndTime((int)(&result)->date().year(), (int)(&result)->date().month(), (int)(&result)->date().day(),
        (int)(&result)->time_of_day().hours(), (int)(&result)->time_of_day().minutes(), (int)(&result)->time_of_day().seconds(),
        boost::numeric_cast<int32_t>(((&result)->time_of_day().fractional_seconds() * boost::numeric_cast<int32_t>(pow(10.0,(9-(&result)->time_of_day().num_fractional_digits())))))/1000);
    }
    catch (std::exception&)
    {
      resultobj = SWIG_Py_Void();
    }
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_AuthenticatedUser(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AuthenticatedUser *arg1 = (RobotRaconteur::AuthenticatedUser *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::AuthenticatedUser > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::AuthenticatedUser > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__AuthenticatedUser_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_AuthenticatedUser" "', argument " "1"" of type '" "RobotRaconteur::AuthenticatedUser *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AuthenticatedUser > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AuthenticatedUser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::AuthenticatedUser * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::AuthenticatedUser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::AuthenticatedUser * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *AuthenticatedUser_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__AuthenticatedUser_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_ServerEndpoint_GetCurrentEndpoint(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  uint32_t result;
  
  if (!SWIG_Python_UnpackTuple(args, "ServerEndpoint_GetCurrentEndpoint", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (uint32_t)RobotRaconteur_ServerEndpoint_GetCurrentEndpoint();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServerEndpoint_GetCurrentAuthenticatedUser(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::AuthenticatedUser > result;
  
  if (!SWIG_Python_UnpackTuple(args, "ServerEndpoint_GetCurrentAuthenticatedUser", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::ServerEndpoint::GetCurrentAuthenticatedUser();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::AuthenticatedUser > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::AuthenticatedUser >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__AuthenticatedUser_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_ServerEndpoint(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServerEndpoint *arg1 = (RobotRaconteur::ServerEndpoint *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerEndpoint > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServerEndpoint > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerEndpoint_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_ServerEndpoint" "', argument " "1"" of type '" "RobotRaconteur::ServerEndpoint *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerEndpoint > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerEndpoint * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServerEndpoint > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServerEndpoint * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *ServerEndpoint_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerEndpoint_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_WrappedServiceSkelAsyncAdapter_MakeAsync(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkelAsyncAdapter *arg1 = (RobotRaconteur::WrappedServiceSkelAsyncAdapter *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkelAsyncAdapter_MakeAsync" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkelAsyncAdapter *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkelAsyncAdapter * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkelAsyncAdapter * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->MakeAsync();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkelAsyncAdapter_IsAsync(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkelAsyncAdapter *arg1 = (RobotRaconteur::WrappedServiceSkelAsyncAdapter *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkelAsyncAdapter_IsAsync" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkelAsyncAdapter *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkelAsyncAdapter * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkelAsyncAdapter * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->IsAsync();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkelAsyncAdapter_End__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkelAsyncAdapter *arg1 = (RobotRaconteur::WrappedServiceSkelAsyncAdapter *) 0 ;
  RobotRaconteur::HandlerErrorInfo *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkelAsyncAdapter_End" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkelAsyncAdapter *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkelAsyncAdapter * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkelAsyncAdapter * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSkelAsyncAdapter_End" "', argument " "2"" of type '" "RobotRaconteur::HandlerErrorInfo const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSkelAsyncAdapter_End" "', argument " "2"" of type '" "RobotRaconteur::HandlerErrorInfo const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->End((RobotRaconteur::HandlerErrorInfo const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkelAsyncAdapter_End__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkelAsyncAdapter *arg1 = (RobotRaconteur::WrappedServiceSkelAsyncAdapter *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg2 = 0 ;
  RobotRaconteur::HandlerErrorInfo *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp2 ;
  int newmem2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSkelAsyncAdapter_End" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkelAsyncAdapter *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkelAsyncAdapter * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkelAsyncAdapter * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSkelAsyncAdapter_End" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSkelAsyncAdapter_End" "', argument " "3"" of type '" "RobotRaconteur::HandlerErrorInfo const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSkelAsyncAdapter_End" "', argument " "3"" of type '" "RobotRaconteur::HandlerErrorInfo const &""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->End((boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg2,(RobotRaconteur::HandlerErrorInfo const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSkelAsyncAdapter_End(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedServiceSkelAsyncAdapter_End", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo, SWIG_POINTER_NO_NULL | 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_WrappedServiceSkelAsyncAdapter_End__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_WrappedServiceSkelAsyncAdapter_End__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedServiceSkelAsyncAdapter_End'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedServiceSkelAsyncAdapter::End(RobotRaconteur::HandlerErrorInfo const &)\n"
    "    RobotRaconteur::WrappedServiceSkelAsyncAdapter::End(boost::intrusive_ptr< RobotRaconteur::MessageElement > const &,RobotRaconteur::HandlerErrorInfo const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_delete_WrappedServiceSkelAsyncAdapter(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSkelAsyncAdapter *arg1 = (RobotRaconteur::WrappedServiceSkelAsyncAdapter *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedServiceSkelAsyncAdapter" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSkelAsyncAdapter *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkelAsyncAdapter * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSkelAsyncAdapter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSkelAsyncAdapter * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedServiceSkelAsyncAdapter_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_NativeUserAuthenticator__AuthenticateUser(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::UserAuthenticator *arg1 = (RobotRaconteur::UserAuthenticator *) 0 ;
  std::string *arg2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg3 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::UserAuthenticator > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::UserAuthenticator > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 ;
  int res3 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull3 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp3 ;
  int newmem3 = 0 ;
  void *argp4 ;
  int res4 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared4 ;
  PyObject *swig_obj[4] ;
  boost::shared_ptr< RobotRaconteur::AuthenticatedUser > result;
  
  if (!SWIG_Python_UnpackTuple(args, "NativeUserAuthenticator__AuthenticateUser", 4, 4, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UserAuthenticator_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NativeUserAuthenticator__AuthenticateUser" "', argument " "1"" of type '" "RobotRaconteur::UserAuthenticator *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UserAuthenticator > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UserAuthenticator > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UserAuthenticator * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UserAuthenticator > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UserAuthenticator * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "NativeUserAuthenticator__AuthenticateUser" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "NativeUserAuthenticator__AuthenticateUser" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[2] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[2];
  //	temp3 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg3 = &temp3;
  //} else {
  //  arg3 = &tempnull3;
  //}
  
  res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem3);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "NativeUserAuthenticator__AuthenticateUser" "', argument " "3"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp3) {
    temp3 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp3)->get(),true);
    if (newmem3 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp3);
    arg3 = &temp3;
  }
  else {
    arg3 = &tempnull3;
  }
  
  {
    int newmem = 0;
    res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t,  0 , &newmem);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "NativeUserAuthenticator__AuthenticateUser" "', argument " "4"" of type '" "boost::shared_ptr< RobotRaconteur::ServerContext > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp4) tempshared4 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServerContext > * >(argp4);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServerContext > * >(argp4);
      arg4 = &tempshared4;
    } else {
      arg4 = (argp4) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServerContext > * >(argp4) : &tempshared4;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_UserAuthenticator_AuthenticateUser(arg1,(std::string const &)*arg2,(boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg3,(boost::shared_ptr< RobotRaconteur::ServerContext > const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::AuthenticatedUser > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::AuthenticatedUser >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__AuthenticatedUser_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_NativeUserAuthenticator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::UserAuthenticator *arg1 = (RobotRaconteur::UserAuthenticator *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::UserAuthenticator > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::UserAuthenticator > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UserAuthenticator_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_NativeUserAuthenticator" "', argument " "1"" of type '" "RobotRaconteur::UserAuthenticator *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UserAuthenticator > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UserAuthenticator > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UserAuthenticator * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UserAuthenticator > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UserAuthenticator * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *NativeUserAuthenticator_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UserAuthenticator_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_new_ServiceSecurityPolicy(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::UserAuthenticator > *arg1 = 0 ;
  std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > *arg2 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::UserAuthenticator > tempshared1 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  RobotRaconteur::ServiceSecurityPolicy *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_ServiceSecurityPolicy", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UserAuthenticator_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_ServiceSecurityPolicy" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::UserAuthenticator > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::UserAuthenticator > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::UserAuthenticator > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::UserAuthenticator > * >(argp1) : &tempshared1;
    }
  }
  {
    std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > *ptr = (std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_ServiceSecurityPolicy" "', argument " "2"" of type '" "std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_ServiceSecurityPolicy" "', argument " "2"" of type '" "std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::ServiceSecurityPolicy *)new RobotRaconteur::ServiceSecurityPolicy((boost::shared_ptr< RobotRaconteur::UserAuthenticator > const &)*arg1,(std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServiceSecurityPolicy > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ServiceSecurityPolicy >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceSecurityPolicy_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_ServiceSecurityPolicy(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSecurityPolicy *arg1 = (RobotRaconteur::ServiceSecurityPolicy *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceSecurityPolicy > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::ServiceSecurityPolicy > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceSecurityPolicy_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_ServiceSecurityPolicy" "', argument " "1"" of type '" "RobotRaconteur::ServiceSecurityPolicy *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceSecurityPolicy > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceSecurityPolicy > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceSecurityPolicy * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::ServiceSecurityPolicy > * >(argp1);
      arg1 = const_cast< RobotRaconteur::ServiceSecurityPolicy * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *ServiceSecurityPolicy_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceSecurityPolicy_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *ServiceSecurityPolicy_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_new_PasswordFileUserAuthenticator__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  bool arg2 ;
  int res1 = SWIG_OLDOBJ ;
  bool val2 ;
  int ecode2 = 0 ;
  RobotRaconteur::PasswordFileUserAuthenticator *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_PasswordFileUserAuthenticator" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_PasswordFileUserAuthenticator" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "new_PasswordFileUserAuthenticator" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::PasswordFileUserAuthenticator *)new RobotRaconteur::PasswordFileUserAuthenticator((std::string const &)*arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::PasswordFileUserAuthenticator > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::PasswordFileUserAuthenticator >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PasswordFileUserAuthenticator_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_PasswordFileUserAuthenticator__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  RobotRaconteur::PasswordFileUserAuthenticator *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_PasswordFileUserAuthenticator" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_PasswordFileUserAuthenticator" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::PasswordFileUserAuthenticator *)new RobotRaconteur::PasswordFileUserAuthenticator((std::string const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::PasswordFileUserAuthenticator > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::PasswordFileUserAuthenticator >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PasswordFileUserAuthenticator_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_PasswordFileUserAuthenticator(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_PasswordFileUserAuthenticator", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_AsPtr_std_string(argv[0], (std::string**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_PasswordFileUserAuthenticator__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_AsPtr_std_string(argv[0], (std::string**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_bool(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_new_PasswordFileUserAuthenticator__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_PasswordFileUserAuthenticator'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::PasswordFileUserAuthenticator::PasswordFileUserAuthenticator(std::string const &,bool)\n"
    "    RobotRaconteur::PasswordFileUserAuthenticator::PasswordFileUserAuthenticator(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_delete_PasswordFileUserAuthenticator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::PasswordFileUserAuthenticator *arg1 = (RobotRaconteur::PasswordFileUserAuthenticator *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::PasswordFileUserAuthenticator > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::PasswordFileUserAuthenticator > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PasswordFileUserAuthenticator_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_PasswordFileUserAuthenticator" "', argument " "1"" of type '" "RobotRaconteur::PasswordFileUserAuthenticator *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PasswordFileUserAuthenticator > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PasswordFileUserAuthenticator > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PasswordFileUserAuthenticator * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::PasswordFileUserAuthenticator > * >(argp1);
      arg1 = const_cast< RobotRaconteur::PasswordFileUserAuthenticator * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *PasswordFileUserAuthenticator_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__PasswordFileUserAuthenticator_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *PasswordFileUserAuthenticator_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_WrappedUserAuthenticatorDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedUserAuthenticatorDirector *arg1 = (RobotRaconteur::WrappedUserAuthenticatorDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedUserAuthenticatorDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedUserAuthenticatorDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedUserAuthenticatorDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedUserAuthenticatorDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedUserAuthenticatorDirector_AuthenticateUser(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedUserAuthenticatorDirector *arg1 = (RobotRaconteur::WrappedUserAuthenticatorDirector *) 0 ;
  std::string *arg2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > arg3 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 ;
  int res3 = 0 ;
  int newmem3 = 0 ;
  void *argp4 ;
  int res4 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared4 ;
  PyObject *swig_obj[4] ;
  Swig::Director *director = 0;
  bool upcall = false;
  boost::shared_ptr< RobotRaconteur::AuthenticatedUser > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedUserAuthenticatorDirector_AuthenticateUser", 4, 4, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedUserAuthenticatorDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedUserAuthenticatorDirector_AuthenticateUser" "', argument " "1"" of type '" "RobotRaconteur::WrappedUserAuthenticatorDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedUserAuthenticatorDirector * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedUserAuthenticatorDirector_AuthenticateUser" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedUserAuthenticatorDirector_AuthenticateUser" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[2];
  //if (smartarg) {
  //	arg3 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //}
  
  res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem3);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedUserAuthenticatorDirector_AuthenticateUser" "', argument " "3"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement >""'");
  }
  if (argp3) arg3 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp3)->get(),true);
  if (newmem3 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp3);
  
  {
    int newmem = 0;
    res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t,  0 , &newmem);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedUserAuthenticatorDirector_AuthenticateUser" "', argument " "4"" of type '" "boost::shared_ptr< RobotRaconteur::ServerContext > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp4) tempshared4 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServerContext > * >(argp4);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServerContext > * >(argp4);
      arg4 = &tempshared4;
    } else {
      arg4 = (argp4) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServerContext > * >(argp4) : &tempshared4;
    }
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedUserAuthenticatorDirector::AuthenticateUser");
      } else {
        result = (arg1)->AuthenticateUser((std::string const &)*arg2,arg3,(boost::shared_ptr< RobotRaconteur::ServerContext > const &)*arg4);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  {
    boost::shared_ptr<  RobotRaconteur::AuthenticatedUser > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::AuthenticatedUser >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__AuthenticatedUser_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedUserAuthenticatorDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedUserAuthenticatorDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedUserAuthenticatorDirector *)new SwigDirector_WrappedUserAuthenticatorDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedUserAuthenticatorDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedUserAuthenticatorDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedUserAuthenticatorDirector *arg1 = (RobotRaconteur::WrappedUserAuthenticatorDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedUserAuthenticatorDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedUserAuthenticatorDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedUserAuthenticatorDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedUserAuthenticatorDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedUserAuthenticatorDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedUserAuthenticatorDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedUserAuthenticatorDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedUserAuthenticator_SetRRDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedUserAuthenticator *arg1 = (RobotRaconteur::WrappedUserAuthenticator *) 0 ;
  RobotRaconteur::WrappedUserAuthenticatorDirector *arg2 = (RobotRaconteur::WrappedUserAuthenticatorDirector *) 0 ;
  int arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedUserAuthenticator > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedUserAuthenticator > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedUserAuthenticator_SetRRDirector", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedUserAuthenticator_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedUserAuthenticator_SetRRDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedUserAuthenticator *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedUserAuthenticator > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedUserAuthenticator > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedUserAuthenticator * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedUserAuthenticator > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedUserAuthenticator * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__WrappedUserAuthenticatorDirector, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedUserAuthenticator_SetRRDirector" "', argument " "2"" of type '" "RobotRaconteur::WrappedUserAuthenticatorDirector *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedUserAuthenticatorDirector * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedUserAuthenticator_SetRRDirector" "', argument " "3"" of type '" "int""'");
  } 
  arg3 = static_cast< int >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetRRDirector(arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedUserAuthenticator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedUserAuthenticator *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_WrappedUserAuthenticator", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedUserAuthenticator *)new RobotRaconteur::WrappedUserAuthenticator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedUserAuthenticator > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedUserAuthenticator >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedUserAuthenticator_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedUserAuthenticator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedUserAuthenticator *arg1 = (RobotRaconteur::WrappedUserAuthenticator *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedUserAuthenticator > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedUserAuthenticator > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedUserAuthenticator_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedUserAuthenticator" "', argument " "1"" of type '" "RobotRaconteur::WrappedUserAuthenticator *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedUserAuthenticator > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedUserAuthenticator > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedUserAuthenticator * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedUserAuthenticator > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedUserAuthenticator * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedUserAuthenticator_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedUserAuthenticator_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedUserAuthenticator_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_AsyncServiceInfo2VectorReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncServiceInfo2VectorReturnDirector *arg1 = (RobotRaconteur::AsyncServiceInfo2VectorReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncServiceInfo2VectorReturnDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_AsyncServiceInfo2VectorReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncServiceInfo2VectorReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncServiceInfo2VectorReturnDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_AsyncServiceInfo2VectorReturnDirector_handler(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncServiceInfo2VectorReturnDirector *arg1 = (RobotRaconteur::AsyncServiceInfo2VectorReturnDirector *) 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "AsyncServiceInfo2VectorReturnDirector_handler", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncServiceInfo2VectorReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AsyncServiceInfo2VectorReturnDirector_handler" "', argument " "1"" of type '" "RobotRaconteur::AsyncServiceInfo2VectorReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncServiceInfo2VectorReturnDirector * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "AsyncServiceInfo2VectorReturnDirector_handler" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "AsyncServiceInfo2VectorReturnDirector_handler" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > * >(argp2);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::AsyncServiceInfo2VectorReturnDirector::handler");
      } else {
        (arg1)->handler((std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > const &)*arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_AsyncServiceInfo2VectorReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::AsyncServiceInfo2VectorReturnDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::AsyncServiceInfo2VectorReturnDirector *)new SwigDirector_AsyncServiceInfo2VectorReturnDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__AsyncServiceInfo2VectorReturnDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_AsyncServiceInfo2VectorReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncServiceInfo2VectorReturnDirector *arg1 = (RobotRaconteur::AsyncServiceInfo2VectorReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncServiceInfo2VectorReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_AsyncServiceInfo2VectorReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncServiceInfo2VectorReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncServiceInfo2VectorReturnDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *AsyncServiceInfo2VectorReturnDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__AsyncServiceInfo2VectorReturnDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *AsyncServiceInfo2VectorReturnDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_AsyncNodeInfo2VectorReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncNodeInfo2VectorReturnDirector *arg1 = (RobotRaconteur::AsyncNodeInfo2VectorReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncNodeInfo2VectorReturnDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_AsyncNodeInfo2VectorReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncNodeInfo2VectorReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncNodeInfo2VectorReturnDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_AsyncNodeInfo2VectorReturnDirector_handler(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncNodeInfo2VectorReturnDirector *arg1 = (RobotRaconteur::AsyncNodeInfo2VectorReturnDirector *) 0 ;
  std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "AsyncNodeInfo2VectorReturnDirector_handler", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncNodeInfo2VectorReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AsyncNodeInfo2VectorReturnDirector_handler" "', argument " "1"" of type '" "RobotRaconteur::AsyncNodeInfo2VectorReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncNodeInfo2VectorReturnDirector * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "AsyncNodeInfo2VectorReturnDirector_handler" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "AsyncNodeInfo2VectorReturnDirector_handler" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > * >(argp2);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::AsyncNodeInfo2VectorReturnDirector::handler");
      } else {
        (arg1)->handler((std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > const &)*arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_AsyncNodeInfo2VectorReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::AsyncNodeInfo2VectorReturnDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::AsyncNodeInfo2VectorReturnDirector *)new SwigDirector_AsyncNodeInfo2VectorReturnDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__AsyncNodeInfo2VectorReturnDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_AsyncNodeInfo2VectorReturnDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::AsyncNodeInfo2VectorReturnDirector *arg1 = (RobotRaconteur::AsyncNodeInfo2VectorReturnDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__AsyncNodeInfo2VectorReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_AsyncNodeInfo2VectorReturnDirector" "', argument " "1"" of type '" "RobotRaconteur::AsyncNodeInfo2VectorReturnDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::AsyncNodeInfo2VectorReturnDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *AsyncNodeInfo2VectorReturnDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__AsyncNodeInfo2VectorReturnDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *AsyncNodeInfo2VectorReturnDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_ServiceInfo2Wrapped_Name_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceInfo2Wrapped *arg1 = (RobotRaconteur::ServiceInfo2Wrapped *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceInfo2Wrapped_Name_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceInfo2Wrapped_Name_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceInfo2Wrapped *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceInfo2Wrapped * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceInfo2Wrapped_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceInfo2Wrapped_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Name = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceInfo2Wrapped_Name_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceInfo2Wrapped *arg1 = (RobotRaconteur::ServiceInfo2Wrapped *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceInfo2Wrapped_Name_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceInfo2Wrapped *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceInfo2Wrapped * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->Name);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceInfo2Wrapped_RootObjectType_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceInfo2Wrapped *arg1 = (RobotRaconteur::ServiceInfo2Wrapped *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceInfo2Wrapped_RootObjectType_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceInfo2Wrapped_RootObjectType_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceInfo2Wrapped *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceInfo2Wrapped * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceInfo2Wrapped_RootObjectType_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceInfo2Wrapped_RootObjectType_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->RootObjectType = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceInfo2Wrapped_RootObjectType_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceInfo2Wrapped *arg1 = (RobotRaconteur::ServiceInfo2Wrapped *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceInfo2Wrapped_RootObjectType_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceInfo2Wrapped *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceInfo2Wrapped * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->RootObjectType);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceInfo2Wrapped_RootObjectImplements_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceInfo2Wrapped *arg1 = (RobotRaconteur::ServiceInfo2Wrapped *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = (std::vector< std::string,std::allocator< std::string > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceInfo2Wrapped_RootObjectImplements_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceInfo2Wrapped_RootObjectImplements_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceInfo2Wrapped *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceInfo2Wrapped * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceInfo2Wrapped_RootObjectImplements_set" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< std::string,std::allocator< std::string > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->RootObjectImplements = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceInfo2Wrapped_RootObjectImplements_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceInfo2Wrapped *arg1 = (RobotRaconteur::ServiceInfo2Wrapped *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceInfo2Wrapped_RootObjectImplements_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceInfo2Wrapped *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceInfo2Wrapped * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< std::string,std::allocator< std::string > > *)& ((arg1)->RootObjectImplements);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceInfo2Wrapped_ConnectionURL_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceInfo2Wrapped *arg1 = (RobotRaconteur::ServiceInfo2Wrapped *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = (std::vector< std::string,std::allocator< std::string > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceInfo2Wrapped_ConnectionURL_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceInfo2Wrapped_ConnectionURL_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceInfo2Wrapped *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceInfo2Wrapped * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceInfo2Wrapped_ConnectionURL_set" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< std::string,std::allocator< std::string > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ConnectionURL = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceInfo2Wrapped_ConnectionURL_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceInfo2Wrapped *arg1 = (RobotRaconteur::ServiceInfo2Wrapped *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceInfo2Wrapped_ConnectionURL_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceInfo2Wrapped *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceInfo2Wrapped * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< std::string,std::allocator< std::string > > *)& ((arg1)->ConnectionURL);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceInfo2Wrapped_Attributes_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceInfo2Wrapped *arg1 = (RobotRaconteur::ServiceInfo2Wrapped *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceInfo2Wrapped_Attributes_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceInfo2Wrapped_Attributes_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceInfo2Wrapped *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceInfo2Wrapped * >(argp1);
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceInfo2Wrapped_Attributes_set" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Attributes = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceInfo2Wrapped_Attributes_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceInfo2Wrapped *arg1 = (RobotRaconteur::ServiceInfo2Wrapped *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceInfo2Wrapped_Attributes_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceInfo2Wrapped *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceInfo2Wrapped * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::intrusive_ptr< RobotRaconteur::MessageElement > *) & ((arg1)->Attributes);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  
  //if (*result) {
  //  intrusive_ptr_add_ref(result->get());
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result->get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(result)) {
    intrusive_ptr_add_ref(result->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >(result->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceInfo2Wrapped_NodeID_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceInfo2Wrapped *arg1 = (RobotRaconteur::ServiceInfo2Wrapped *) 0 ;
  RobotRaconteur::NodeID *arg2 = (RobotRaconteur::NodeID *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceInfo2Wrapped_NodeID_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceInfo2Wrapped_NodeID_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceInfo2Wrapped *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceInfo2Wrapped * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__NodeID, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceInfo2Wrapped_NodeID_set" "', argument " "2"" of type '" "RobotRaconteur::NodeID *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::NodeID * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->NodeID = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceInfo2Wrapped_NodeID_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceInfo2Wrapped *arg1 = (RobotRaconteur::ServiceInfo2Wrapped *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::NodeID *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceInfo2Wrapped_NodeID_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceInfo2Wrapped *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceInfo2Wrapped * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::NodeID *)& ((arg1)->NodeID);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__NodeID, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceInfo2Wrapped_NodeName_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceInfo2Wrapped *arg1 = (RobotRaconteur::ServiceInfo2Wrapped *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceInfo2Wrapped_NodeName_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceInfo2Wrapped_NodeName_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceInfo2Wrapped *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceInfo2Wrapped * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceInfo2Wrapped_NodeName_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceInfo2Wrapped_NodeName_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->NodeName = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceInfo2Wrapped_NodeName_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceInfo2Wrapped *arg1 = (RobotRaconteur::ServiceInfo2Wrapped *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceInfo2Wrapped_NodeName_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceInfo2Wrapped *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceInfo2Wrapped * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->NodeName);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ServiceInfo2Wrapped(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceInfo2Wrapped *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_ServiceInfo2Wrapped", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::ServiceInfo2Wrapped *)new RobotRaconteur::ServiceInfo2Wrapped();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_ServiceInfo2Wrapped(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceInfo2Wrapped *arg1 = (RobotRaconteur::ServiceInfo2Wrapped *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_ServiceInfo2Wrapped" "', argument " "1"" of type '" "RobotRaconteur::ServiceInfo2Wrapped *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceInfo2Wrapped * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *ServiceInfo2Wrapped_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *ServiceInfo2Wrapped_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedFindServiceByType(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  std::string *arg2 = 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg3 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  PyObject *swig_obj[3] ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedFindServiceByType", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedFindServiceByType" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedFindServiceByType" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedFindServiceByType" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedFindServiceByType" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedFindServiceByType" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::WrappedFindServiceByType((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(std::string const &)*arg2,(std::vector< std::string,std::allocator< std::string > > const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >(static_cast< const std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >& >(result))), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, SWIG_POINTER_OWN |  0 );
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_AsyncWrappedFindServiceByType(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  std::string *arg2 = 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg3 = 0 ;
  int32_t arg4 ;
  RobotRaconteur::AsyncServiceInfo2VectorReturnDirector *arg5 = (RobotRaconteur::AsyncServiceInfo2VectorReturnDirector *) 0 ;
  int32_t arg6 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  int val4 ;
  int ecode4 = 0 ;
  void *argp5 = 0 ;
  int res5 = 0 ;
  int val6 ;
  int ecode6 = 0 ;
  PyObject *swig_obj[6] ;
  
  if (!SWIG_Python_UnpackTuple(args, "AsyncWrappedFindServiceByType", 6, 6, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AsyncWrappedFindServiceByType" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "AsyncWrappedFindServiceByType" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "AsyncWrappedFindServiceByType" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "AsyncWrappedFindServiceByType" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "AsyncWrappedFindServiceByType" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg3 = ptr;
  }
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "AsyncWrappedFindServiceByType" "', argument " "4"" of type '" "int32_t""'");
  } 
  arg4 = static_cast< int32_t >(val4);
  res5 = SWIG_ConvertPtr(swig_obj[4], &argp5,SWIGTYPE_p_RobotRaconteur__AsyncServiceInfo2VectorReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res5)) {
    SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "AsyncWrappedFindServiceByType" "', argument " "5"" of type '" "RobotRaconteur::AsyncServiceInfo2VectorReturnDirector *""'"); 
  }
  arg5 = reinterpret_cast< RobotRaconteur::AsyncServiceInfo2VectorReturnDirector * >(argp5);
  ecode6 = SWIG_AsVal_int(swig_obj[5], &val6);
  if (!SWIG_IsOK(ecode6)) {
    SWIG_exception_fail(SWIG_ArgError(ecode6), "in method '" "AsyncWrappedFindServiceByType" "', argument " "6"" of type '" "int32_t""'");
  } 
  arg6 = static_cast< int32_t >(val6);
  
  try {
    RobotRaconteur::AsyncWrappedFindServiceByType((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(std::string const &)*arg2,(std::vector< std::string,std::allocator< std::string > > const &)*arg3,arg4,arg5,arg6);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_iterator" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped___nonzero__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____nonzero__((std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped___bool__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____bool__((std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped___len__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____len__((std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type arg2 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorserviceinfo2wrapped___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped___getslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorserviceinfo2wrapped___getslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorserviceinfo2wrapped___getslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > *)std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type arg2 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped___setslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorserviceinfo2wrapped___setslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorserviceinfo2wrapped___setslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type arg2 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type arg3 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped___setslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorserviceinfo2wrapped___setslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorserviceinfo2wrapped___setslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type >(val3);
  {
    std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > *ptr = (std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorserviceinfo2wrapped___setslice__" "', argument " "4"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorserviceinfo2wrapped___setslice__" "', argument " "4"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorserviceinfo2wrapped___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vectorserviceinfo2wrapped___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorserviceinfo2wrapped___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorserviceinfo2wrapped___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceInfo2Wrapped >::__setslice__(std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type)\n"
    "    std::vector< RobotRaconteur::ServiceInfo2Wrapped >::__setslice__(std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type,std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type arg2 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorserviceinfo2wrapped___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped___delslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorserviceinfo2wrapped___delslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorserviceinfo2wrapped___delslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped___delitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorserviceinfo2wrapped___delitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped___getitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorserviceinfo2wrapped___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > *)std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorserviceinfo2wrapped___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > *ptr = (std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorserviceinfo2wrapped___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorserviceinfo2wrapped___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____setitem____SWIG_0(arg1,arg2,(std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorserviceinfo2wrapped___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped___delitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorserviceinfo2wrapped___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorserviceinfo2wrapped___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorserviceinfo2wrapped___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorserviceinfo2wrapped___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorserviceinfo2wrapped___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceInfo2Wrapped >::__delitem__(std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type)\n"
    "    std::vector< RobotRaconteur::ServiceInfo2Wrapped >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped___getitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorserviceinfo2wrapped___getitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type *) &std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____getitem____SWIG_1((std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorserviceinfo2wrapped___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorserviceinfo2wrapped___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorserviceinfo2wrapped___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorserviceinfo2wrapped___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceInfo2Wrapped >::__getitem__(PySliceObject *)\n"
    "    std::vector< RobotRaconteur::ServiceInfo2Wrapped >::__getitem__(std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type arg2 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorserviceinfo2wrapped___setitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorserviceinfo2wrapped___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorserviceinfo2wrapped___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type * >(argp3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg____setitem____SWIG_2(arg1,arg2,(RobotRaconteur::ServiceInfo2Wrapped const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorserviceinfo2wrapped___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorserviceinfo2wrapped___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorserviceinfo2wrapped___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorserviceinfo2wrapped___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorserviceinfo2wrapped___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceInfo2Wrapped >::__setitem__(PySliceObject *,std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > const &)\n"
    "    std::vector< RobotRaconteur::ServiceInfo2Wrapped >::__setitem__(PySliceObject *)\n"
    "    std::vector< RobotRaconteur::ServiceInfo2Wrapped >::__setitem__(std::vector< RobotRaconteur::ServiceInfo2Wrapped >::difference_type,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_pop" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type(static_cast< const std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type& >(result))), SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorserviceinfo2wrapped_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_append" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorserviceinfo2wrapped_append" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorserviceinfo2wrapped_append" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg__append(arg1,(RobotRaconteur::ServiceInfo2Wrapped const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorserviceinfo2wrapped__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *)new std::vector< RobotRaconteur::ServiceInfo2Wrapped >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorserviceinfo2wrapped__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > *ptr = (std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vectorserviceinfo2wrapped" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vectorserviceinfo2wrapped" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *)new std::vector< RobotRaconteur::ServiceInfo2Wrapped >((std::vector< RobotRaconteur::ServiceInfo2Wrapped > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_empty" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_size" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorserviceinfo2wrapped_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_swap" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorserviceinfo2wrapped_swap" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorserviceinfo2wrapped_swap" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_begin" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_end" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_rbegin" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServiceInfo2Wrapped >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_rend" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServiceInfo2Wrapped >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_clear" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_get_allocator" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< RobotRaconteur::ServiceInfo2Wrapped >::allocator_type(static_cast< const std::vector< RobotRaconteur::ServiceInfo2Wrapped >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorserviceinfo2wrapped__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorserviceinfo2wrapped" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type""'");
  } 
  arg1 = static_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *)new std::vector< RobotRaconteur::ServiceInfo2Wrapped >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_pop_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_resize" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorserviceinfo2wrapped_resize" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_erase" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorserviceinfo2wrapped_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorserviceinfo2wrapped_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator arg2 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_erase" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorserviceinfo2wrapped_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorserviceinfo2wrapped_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorserviceinfo2wrapped_erase" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorserviceinfo2wrapped_erase" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorserviceinfo2wrapped_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vectorserviceinfo2wrapped_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vectorserviceinfo2wrapped_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorserviceinfo2wrapped_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceInfo2Wrapped >::erase(std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator)\n"
    "    std::vector< RobotRaconteur::ServiceInfo2Wrapped >::erase(std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vectorserviceinfo2wrapped__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type arg1 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorserviceinfo2wrapped" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type""'");
  } 
  arg1 = static_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type >(val1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_vectorserviceinfo2wrapped" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vectorserviceinfo2wrapped" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *)new std::vector< RobotRaconteur::ServiceInfo2Wrapped >(arg1,(std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorserviceinfo2wrapped(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vectorserviceinfo2wrapped", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vectorserviceinfo2wrapped__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vectorserviceinfo2wrapped__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vectorserviceinfo2wrapped__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, SWIG_POINTER_NO_NULL | 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_vectorserviceinfo2wrapped__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vectorserviceinfo2wrapped'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceInfo2Wrapped >::vector()\n"
    "    std::vector< RobotRaconteur::ServiceInfo2Wrapped >::vector(std::vector< RobotRaconteur::ServiceInfo2Wrapped > const &)\n"
    "    std::vector< RobotRaconteur::ServiceInfo2Wrapped >::vector(std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type)\n"
    "    std::vector< RobotRaconteur::ServiceInfo2Wrapped >::vector(std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorserviceinfo2wrapped_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_push_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorserviceinfo2wrapped_push_back" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorserviceinfo2wrapped_push_back" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_front" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type *) &((std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type *) &((std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type arg2 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorserviceinfo2wrapped_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_assign" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorserviceinfo2wrapped_assign" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorserviceinfo2wrapped_assign" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorserviceinfo2wrapped_assign" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type arg2 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_resize" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorserviceinfo2wrapped_resize" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorserviceinfo2wrapped_resize" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorserviceinfo2wrapped_resize" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorserviceinfo2wrapped_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorserviceinfo2wrapped_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorserviceinfo2wrapped_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorserviceinfo2wrapped_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceInfo2Wrapped >::resize(std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type)\n"
    "    std::vector< RobotRaconteur::ServiceInfo2Wrapped >::resize(std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator arg2 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_insert" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorserviceinfo2wrapped_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorserviceinfo2wrapped_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorserviceinfo2wrapped_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorserviceinfo2wrapped_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg__insert__SWIG_0(arg1,arg2,(RobotRaconteur::ServiceInfo2Wrapped const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator arg2 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type arg3 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  void *argp4 = 0 ;
  int res4 = 0 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_insert" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorserviceinfo2wrapped_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorserviceinfo2wrapped_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorserviceinfo2wrapped_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type >(val3);
  res4 = SWIG_ConvertPtr(swig_obj[3], &argp4, SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped,  0  | 0);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorserviceinfo2wrapped_insert" "', argument " "4"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &""'"); 
  }
  if (!argp4) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorserviceinfo2wrapped_insert" "', argument " "4"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &""'"); 
  }
  arg4 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type * >(argp4);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_RobotRaconteur_ServiceInfo2Wrapped_Sg__insert__SWIG_1(arg1,arg2,arg3,(RobotRaconteur::ServiceInfo2Wrapped const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorserviceinfo2wrapped_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator > *>(iter) != 0));
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorserviceinfo2wrapped_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, SWIG_POINTER_NO_NULL | 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorserviceinfo2wrapped_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorserviceinfo2wrapped_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceInfo2Wrapped >::insert(std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &)\n"
    "    std::vector< RobotRaconteur::ServiceInfo2Wrapped >::insert(std::vector< RobotRaconteur::ServiceInfo2Wrapped >::iterator,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type,std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorserviceinfo2wrapped_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_reserve" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorserviceinfo2wrapped_reserve" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorserviceinfo2wrapped_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorserviceinfo2wrapped_capacity" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::ServiceInfo2Wrapped > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vectorserviceinfo2wrapped(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::vector< RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vectorserviceinfo2wrapped" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vectorserviceinfo2wrapped_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vectorserviceinfo2wrapped_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_NodeInfo2_NodeID_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeInfo2 *arg1 = (RobotRaconteur::NodeInfo2 *) 0 ;
  RobotRaconteur::NodeID *arg2 = (RobotRaconteur::NodeID *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "NodeInfo2_NodeID_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeInfo2, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeInfo2_NodeID_set" "', argument " "1"" of type '" "RobotRaconteur::NodeInfo2 *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeInfo2 * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__NodeID, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "NodeInfo2_NodeID_set" "', argument " "2"" of type '" "RobotRaconteur::NodeID *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::NodeID * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->NodeID = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeInfo2_NodeID_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeInfo2 *arg1 = (RobotRaconteur::NodeInfo2 *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::NodeID *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeInfo2, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeInfo2_NodeID_get" "', argument " "1"" of type '" "RobotRaconteur::NodeInfo2 *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeInfo2 * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::NodeID *)& ((arg1)->NodeID);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__NodeID, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeInfo2_NodeName_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeInfo2 *arg1 = (RobotRaconteur::NodeInfo2 *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "NodeInfo2_NodeName_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeInfo2, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeInfo2_NodeName_set" "', argument " "1"" of type '" "RobotRaconteur::NodeInfo2 *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeInfo2 * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "NodeInfo2_NodeName_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "NodeInfo2_NodeName_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->NodeName = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeInfo2_NodeName_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeInfo2 *arg1 = (RobotRaconteur::NodeInfo2 *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeInfo2, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeInfo2_NodeName_get" "', argument " "1"" of type '" "RobotRaconteur::NodeInfo2 *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeInfo2 * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->NodeName);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeInfo2_ConnectionURL_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeInfo2 *arg1 = (RobotRaconteur::NodeInfo2 *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = (std::vector< std::string,std::allocator< std::string > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "NodeInfo2_ConnectionURL_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeInfo2, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeInfo2_ConnectionURL_set" "', argument " "1"" of type '" "RobotRaconteur::NodeInfo2 *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeInfo2 * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "NodeInfo2_ConnectionURL_set" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< std::string,std::allocator< std::string > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ConnectionURL = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeInfo2_ConnectionURL_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeInfo2 *arg1 = (RobotRaconteur::NodeInfo2 *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeInfo2, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeInfo2_ConnectionURL_get" "', argument " "1"" of type '" "RobotRaconteur::NodeInfo2 *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeInfo2 * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< std::string,std::allocator< std::string > > *)& ((arg1)->ConnectionURL);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_NodeInfo2(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeInfo2 *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_NodeInfo2", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::NodeInfo2 *)new RobotRaconteur::NodeInfo2();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__NodeInfo2, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_NodeInfo2(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeInfo2 *arg1 = (RobotRaconteur::NodeInfo2 *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeInfo2, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_NodeInfo2" "', argument " "1"" of type '" "RobotRaconteur::NodeInfo2 *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeInfo2 * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *NodeInfo2_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__NodeInfo2, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *NodeInfo2_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedFindNodeByID(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  RobotRaconteur::NodeID arg2 ;
  std::vector< std::string,std::allocator< std::string > > *arg3 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  void *argp2 ;
  int res2 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  PyObject *swig_obj[3] ;
  std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedFindNodeByID", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedFindNodeByID" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  {
    res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__NodeID,  0  | 0);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedFindNodeByID" "', argument " "2"" of type '" "RobotRaconteur::NodeID""'"); 
    }  
    if (!argp2) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedFindNodeByID" "', argument " "2"" of type '" "RobotRaconteur::NodeID""'");
    } else {
      RobotRaconteur::NodeID * temp = reinterpret_cast< RobotRaconteur::NodeID * >(argp2);
      arg2 = *temp;
      if (SWIG_IsNewObj(res2)) delete temp;
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedFindNodeByID" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedFindNodeByID" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::WrappedFindNodeByID((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,arg2,(std::vector< std::string,std::allocator< std::string > > const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >(static_cast< const std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >& >(result))), SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, SWIG_POINTER_OWN |  0 );
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedFindNodeByName(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  std::string *arg2 = 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg3 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  PyObject *swig_obj[3] ;
  std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedFindNodeByName", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedFindNodeByName" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedFindNodeByName" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedFindNodeByName" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedFindNodeByName" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedFindNodeByName" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::WrappedFindNodeByName((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(std::string const &)*arg2,(std::vector< std::string,std::allocator< std::string > > const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >(static_cast< const std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >& >(result))), SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, SWIG_POINTER_OWN |  0 );
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_AsyncWrappedFindNodeByID(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  RobotRaconteur::NodeID *arg2 = 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg3 = 0 ;
  int32_t arg4 ;
  RobotRaconteur::AsyncNodeInfo2VectorReturnDirector *arg5 = (RobotRaconteur::AsyncNodeInfo2VectorReturnDirector *) 0 ;
  int32_t arg6 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  int val4 ;
  int ecode4 = 0 ;
  void *argp5 = 0 ;
  int res5 = 0 ;
  int val6 ;
  int ecode6 = 0 ;
  PyObject *swig_obj[6] ;
  
  if (!SWIG_Python_UnpackTuple(args, "AsyncWrappedFindNodeByID", 6, 6, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AsyncWrappedFindNodeByID" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__NodeID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "AsyncWrappedFindNodeByID" "', argument " "2"" of type '" "RobotRaconteur::NodeID const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "AsyncWrappedFindNodeByID" "', argument " "2"" of type '" "RobotRaconteur::NodeID const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::NodeID * >(argp2);
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "AsyncWrappedFindNodeByID" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "AsyncWrappedFindNodeByID" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg3 = ptr;
  }
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "AsyncWrappedFindNodeByID" "', argument " "4"" of type '" "int32_t""'");
  } 
  arg4 = static_cast< int32_t >(val4);
  res5 = SWIG_ConvertPtr(swig_obj[4], &argp5,SWIGTYPE_p_RobotRaconteur__AsyncNodeInfo2VectorReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res5)) {
    SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "AsyncWrappedFindNodeByID" "', argument " "5"" of type '" "RobotRaconteur::AsyncNodeInfo2VectorReturnDirector *""'"); 
  }
  arg5 = reinterpret_cast< RobotRaconteur::AsyncNodeInfo2VectorReturnDirector * >(argp5);
  ecode6 = SWIG_AsVal_int(swig_obj[5], &val6);
  if (!SWIG_IsOK(ecode6)) {
    SWIG_exception_fail(SWIG_ArgError(ecode6), "in method '" "AsyncWrappedFindNodeByID" "', argument " "6"" of type '" "int32_t""'");
  } 
  arg6 = static_cast< int32_t >(val6);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur::AsyncWrappedFindNodeByID((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(RobotRaconteur::NodeID const &)*arg2,(std::vector< std::string,std::allocator< std::string > > const &)*arg3,arg4,arg5,arg6);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_AsyncWrappedFindNodeByName(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  std::string *arg2 = 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg3 = 0 ;
  int32_t arg4 ;
  RobotRaconteur::AsyncNodeInfo2VectorReturnDirector *arg5 = (RobotRaconteur::AsyncNodeInfo2VectorReturnDirector *) 0 ;
  int32_t arg6 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  int val4 ;
  int ecode4 = 0 ;
  void *argp5 = 0 ;
  int res5 = 0 ;
  int val6 ;
  int ecode6 = 0 ;
  PyObject *swig_obj[6] ;
  
  if (!SWIG_Python_UnpackTuple(args, "AsyncWrappedFindNodeByName", 6, 6, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AsyncWrappedFindNodeByName" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "AsyncWrappedFindNodeByName" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "AsyncWrappedFindNodeByName" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "AsyncWrappedFindNodeByName" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "AsyncWrappedFindNodeByName" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg3 = ptr;
  }
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "AsyncWrappedFindNodeByName" "', argument " "4"" of type '" "int32_t""'");
  } 
  arg4 = static_cast< int32_t >(val4);
  res5 = SWIG_ConvertPtr(swig_obj[4], &argp5,SWIGTYPE_p_RobotRaconteur__AsyncNodeInfo2VectorReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res5)) {
    SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "AsyncWrappedFindNodeByName" "', argument " "5"" of type '" "RobotRaconteur::AsyncNodeInfo2VectorReturnDirector *""'"); 
  }
  arg5 = reinterpret_cast< RobotRaconteur::AsyncNodeInfo2VectorReturnDirector * >(argp5);
  ecode6 = SWIG_AsVal_int(swig_obj[5], &val6);
  if (!SWIG_IsOK(ecode6)) {
    SWIG_exception_fail(SWIG_ArgError(ecode6), "in method '" "AsyncWrappedFindNodeByName" "', argument " "6"" of type '" "int32_t""'");
  } 
  arg6 = static_cast< int32_t >(val6);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur::AsyncWrappedFindNodeByName((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(std::string const &)*arg2,(std::vector< std::string,std::allocator< std::string > > const &)*arg3,arg4,arg5,arg6);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedUpdateDetectedNodes(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedUpdateDetectedNodes", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedUpdateDetectedNodes" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedUpdateDetectedNodes" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedUpdateDetectedNodes" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur::WrappedUpdateDetectedNodes((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(std::vector< std::string,std::allocator< std::string > > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_AsyncWrappedUpdateDetectedNodes(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  int32_t arg3 ;
  RobotRaconteur::AsyncVoidNoErrReturnDirector *arg4 = (RobotRaconteur::AsyncVoidNoErrReturnDirector *) 0 ;
  int32_t arg5 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  int res2 = SWIG_OLDOBJ ;
  int val3 ;
  int ecode3 = 0 ;
  void *argp4 = 0 ;
  int res4 = 0 ;
  int val5 ;
  int ecode5 = 0 ;
  PyObject *swig_obj[5] ;
  
  if (!SWIG_Python_UnpackTuple(args, "AsyncWrappedUpdateDetectedNodes", 5, 5, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "AsyncWrappedUpdateDetectedNodes" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "AsyncWrappedUpdateDetectedNodes" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "AsyncWrappedUpdateDetectedNodes" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "AsyncWrappedUpdateDetectedNodes" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  res4 = SWIG_ConvertPtr(swig_obj[3], &argp4,SWIGTYPE_p_RobotRaconteur__AsyncVoidNoErrReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "AsyncWrappedUpdateDetectedNodes" "', argument " "4"" of type '" "RobotRaconteur::AsyncVoidNoErrReturnDirector *""'"); 
  }
  arg4 = reinterpret_cast< RobotRaconteur::AsyncVoidNoErrReturnDirector * >(argp4);
  ecode5 = SWIG_AsVal_int(swig_obj[4], &val5);
  if (!SWIG_IsOK(ecode5)) {
    SWIG_exception_fail(SWIG_ArgError(ecode5), "in method '" "AsyncWrappedUpdateDetectedNodes" "', argument " "5"" of type '" "int32_t""'");
  } 
  arg5 = static_cast< int32_t >(val5);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur::AsyncWrappedUpdateDetectedNodes((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(std::vector< std::string,std::allocator< std::string > > const &)*arg2,arg3,arg4,arg5);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedGetDetectedNodes(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedGetDetectedNodes" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::WrappedGetDetectedNodes((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< std::string,std::allocator< std::string > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedGetDetectedNodeCacheInfo(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  RobotRaconteur::NodeID *arg2 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  RobotRaconteur::NodeInfo2 result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedGetDetectedNodeCacheInfo", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedGetDetectedNodeCacheInfo" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__NodeID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedGetDetectedNodeCacheInfo" "', argument " "2"" of type '" "RobotRaconteur::NodeID const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedGetDetectedNodeCacheInfo" "', argument " "2"" of type '" "RobotRaconteur::NodeID const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::NodeID * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::WrappedGetDetectedNodeCacheInfo((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(RobotRaconteur::NodeID const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::NodeInfo2(static_cast< const RobotRaconteur::NodeInfo2& >(result))), SWIGTYPE_p_RobotRaconteur__NodeInfo2, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedTryGetDetectedNodeCacheInfo(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  RobotRaconteur::NodeID *arg2 = 0 ;
  RobotRaconteur::NodeInfo2 *arg3 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  PyObject *swig_obj[3] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedTryGetDetectedNodeCacheInfo", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedTryGetDetectedNodeCacheInfo" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__NodeID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedTryGetDetectedNodeCacheInfo" "', argument " "2"" of type '" "RobotRaconteur::NodeID const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedTryGetDetectedNodeCacheInfo" "', argument " "2"" of type '" "RobotRaconteur::NodeID const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::NodeID * >(argp2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__NodeInfo2,  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedTryGetDetectedNodeCacheInfo" "', argument " "3"" of type '" "RobotRaconteur::NodeInfo2 &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedTryGetDetectedNodeCacheInfo" "', argument " "3"" of type '" "RobotRaconteur::NodeInfo2 &""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::NodeInfo2 * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)RobotRaconteur::WrappedTryGetDetectedNodeCacheInfo((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(RobotRaconteur::NodeID const &)*arg2,*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_iterator" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_RobotRaconteur_NodeInfo2_Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2___nonzero__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____nonzero__((std::vector< RobotRaconteur::NodeInfo2 > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2___bool__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____bool__((std::vector< RobotRaconteur::NodeInfo2 > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::NodeInfo2 >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2___len__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____len__((std::vector< RobotRaconteur::NodeInfo2 > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  std::vector< RobotRaconteur::NodeInfo2 >::difference_type arg2 ;
  std::vector< RobotRaconteur::NodeInfo2 >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectornodeinfo2___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2___getslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectornodeinfo2___getslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::NodeInfo2 >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectornodeinfo2___getslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::NodeInfo2 >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > *)std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  std::vector< RobotRaconteur::NodeInfo2 >::difference_type arg2 ;
  std::vector< RobotRaconteur::NodeInfo2 >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2___setslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectornodeinfo2___setslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::NodeInfo2 >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectornodeinfo2___setslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::NodeInfo2 >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  std::vector< RobotRaconteur::NodeInfo2 >::difference_type arg2 ;
  std::vector< RobotRaconteur::NodeInfo2 >::difference_type arg3 ;
  std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2___setslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectornodeinfo2___setslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::NodeInfo2 >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectornodeinfo2___setslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::NodeInfo2 >::difference_type >(val3);
  {
    std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > *ptr = (std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectornodeinfo2___setslice__" "', argument " "4"" of type '" "std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectornodeinfo2___setslice__" "', argument " "4"" of type '" "std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectornodeinfo2___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vectornodeinfo2___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectornodeinfo2___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectornodeinfo2___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::NodeInfo2 >::__setslice__(std::vector< RobotRaconteur::NodeInfo2 >::difference_type,std::vector< RobotRaconteur::NodeInfo2 >::difference_type)\n"
    "    std::vector< RobotRaconteur::NodeInfo2 >::__setslice__(std::vector< RobotRaconteur::NodeInfo2 >::difference_type,std::vector< RobotRaconteur::NodeInfo2 >::difference_type,std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  std::vector< RobotRaconteur::NodeInfo2 >::difference_type arg2 ;
  std::vector< RobotRaconteur::NodeInfo2 >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectornodeinfo2___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2___delslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectornodeinfo2___delslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::NodeInfo2 >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectornodeinfo2___delslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::NodeInfo2 >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  std::vector< RobotRaconteur::NodeInfo2 >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2___delitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectornodeinfo2___delitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::NodeInfo2 >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2___getitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectornodeinfo2___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > *)std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectornodeinfo2___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > *ptr = (std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectornodeinfo2___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectornodeinfo2___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____setitem____SWIG_0(arg1,arg2,(std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectornodeinfo2___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2___delitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectornodeinfo2___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectornodeinfo2___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectornodeinfo2___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectornodeinfo2___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectornodeinfo2___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::NodeInfo2 >::__delitem__(std::vector< RobotRaconteur::NodeInfo2 >::difference_type)\n"
    "    std::vector< RobotRaconteur::NodeInfo2 >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  std::vector< RobotRaconteur::NodeInfo2 >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< RobotRaconteur::NodeInfo2 >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2___getitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectornodeinfo2___getitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::NodeInfo2 >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::NodeInfo2 >::value_type *) &std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____getitem____SWIG_1((std::vector< RobotRaconteur::NodeInfo2 > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__NodeInfo2, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::NodeInfo2 >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectornodeinfo2___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectornodeinfo2___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectornodeinfo2___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectornodeinfo2___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::NodeInfo2 >::__getitem__(PySliceObject *)\n"
    "    std::vector< RobotRaconteur::NodeInfo2 >::__getitem__(std::vector< RobotRaconteur::NodeInfo2 >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  std::vector< RobotRaconteur::NodeInfo2 >::difference_type arg2 ;
  std::vector< RobotRaconteur::NodeInfo2 >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectornodeinfo2___setitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::NodeInfo2 >::difference_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__NodeInfo2,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectornodeinfo2___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectornodeinfo2___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 >::value_type * >(argp3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_NodeInfo2_Sg____setitem____SWIG_2(arg1,arg2,(RobotRaconteur::NodeInfo2 const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectornodeinfo2___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectornodeinfo2___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectornodeinfo2___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__NodeInfo2, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectornodeinfo2___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectornodeinfo2___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::NodeInfo2 >::__setitem__(PySliceObject *,std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > const &)\n"
    "    std::vector< RobotRaconteur::NodeInfo2 >::__setitem__(PySliceObject *)\n"
    "    std::vector< RobotRaconteur::NodeInfo2 >::__setitem__(std::vector< RobotRaconteur::NodeInfo2 >::difference_type,std::vector< RobotRaconteur::NodeInfo2 >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::NodeInfo2 >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_pop" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = std_vector_Sl_RobotRaconteur_NodeInfo2_Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< RobotRaconteur::NodeInfo2 >::value_type(static_cast< const std::vector< RobotRaconteur::NodeInfo2 >::value_type& >(result))), SWIGTYPE_p_RobotRaconteur__NodeInfo2, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  std::vector< RobotRaconteur::NodeInfo2 >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectornodeinfo2_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_append" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__NodeInfo2,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectornodeinfo2_append" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectornodeinfo2_append" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_RobotRaconteur_NodeInfo2_Sg__append(arg1,(RobotRaconteur::NodeInfo2 const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectornodeinfo2__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::NodeInfo2 > *)new std::vector< RobotRaconteur::NodeInfo2 >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectornodeinfo2__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< RobotRaconteur::NodeInfo2 > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > *ptr = (std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vectornodeinfo2" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vectornodeinfo2" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::NodeInfo2 > *)new std::vector< RobotRaconteur::NodeInfo2 >((std::vector< RobotRaconteur::NodeInfo2 > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_empty" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< RobotRaconteur::NodeInfo2 > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::NodeInfo2 >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_size" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::NodeInfo2 > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  std::vector< RobotRaconteur::NodeInfo2 > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectornodeinfo2_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_swap" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectornodeinfo2_swap" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectornodeinfo2_swap" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::NodeInfo2 >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_begin" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::NodeInfo2 >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::NodeInfo2 >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_end" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::NodeInfo2 >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::NodeInfo2 >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_rbegin" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::NodeInfo2 >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::NodeInfo2 >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_rend" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::NodeInfo2 >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_clear" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< RobotRaconteur::NodeInfo2 > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_get_allocator" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::NodeInfo2 > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< RobotRaconteur::NodeInfo2 >::allocator_type(static_cast< const std::vector< RobotRaconteur::NodeInfo2 >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_RobotRaconteur__NodeInfo2_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectornodeinfo2__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< RobotRaconteur::NodeInfo2 > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectornodeinfo2" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::size_type""'");
  } 
  arg1 = static_cast< std::vector< RobotRaconteur::NodeInfo2 >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::NodeInfo2 > *)new std::vector< RobotRaconteur::NodeInfo2 >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_pop_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  std::vector< RobotRaconteur::NodeInfo2 >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_resize" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectornodeinfo2_resize" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::NodeInfo2 >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  std::vector< RobotRaconteur::NodeInfo2 >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< RobotRaconteur::NodeInfo2 >::iterator result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_erase" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectornodeinfo2_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::NodeInfo2 >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::NodeInfo2 >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectornodeinfo2_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_NodeInfo2_Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::NodeInfo2 >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  std::vector< RobotRaconteur::NodeInfo2 >::iterator arg2 ;
  std::vector< RobotRaconteur::NodeInfo2 >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  std::vector< RobotRaconteur::NodeInfo2 >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_erase" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectornodeinfo2_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::NodeInfo2 >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::NodeInfo2 >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectornodeinfo2_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectornodeinfo2_erase" "', argument " "3"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::NodeInfo2 >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::NodeInfo2 >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectornodeinfo2_erase" "', argument " "3"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_NodeInfo2_Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::NodeInfo2 >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectornodeinfo2_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::NodeInfo2 >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vectornodeinfo2_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::NodeInfo2 >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::NodeInfo2 >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vectornodeinfo2_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectornodeinfo2_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::NodeInfo2 >::erase(std::vector< RobotRaconteur::NodeInfo2 >::iterator)\n"
    "    std::vector< RobotRaconteur::NodeInfo2 >::erase(std::vector< RobotRaconteur::NodeInfo2 >::iterator,std::vector< RobotRaconteur::NodeInfo2 >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vectornodeinfo2__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 >::size_type arg1 ;
  std::vector< RobotRaconteur::NodeInfo2 >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  std::vector< RobotRaconteur::NodeInfo2 > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectornodeinfo2" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::size_type""'");
  } 
  arg1 = static_cast< std::vector< RobotRaconteur::NodeInfo2 >::size_type >(val1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__NodeInfo2,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_vectornodeinfo2" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vectornodeinfo2" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::NodeInfo2 > *)new std::vector< RobotRaconteur::NodeInfo2 >(arg1,(std::vector< RobotRaconteur::NodeInfo2 >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectornodeinfo2(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vectornodeinfo2", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vectornodeinfo2__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vectornodeinfo2__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vectornodeinfo2__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_RobotRaconteur__NodeInfo2, SWIG_POINTER_NO_NULL | 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_vectornodeinfo2__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vectornodeinfo2'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::NodeInfo2 >::vector()\n"
    "    std::vector< RobotRaconteur::NodeInfo2 >::vector(std::vector< RobotRaconteur::NodeInfo2 > const &)\n"
    "    std::vector< RobotRaconteur::NodeInfo2 >::vector(std::vector< RobotRaconteur::NodeInfo2 >::size_type)\n"
    "    std::vector< RobotRaconteur::NodeInfo2 >::vector(std::vector< RobotRaconteur::NodeInfo2 >::size_type,std::vector< RobotRaconteur::NodeInfo2 >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  std::vector< RobotRaconteur::NodeInfo2 >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectornodeinfo2_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_push_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__NodeInfo2,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectornodeinfo2_push_back" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectornodeinfo2_push_back" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< RobotRaconteur::NodeInfo2 >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::NodeInfo2 >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_front" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::NodeInfo2 >::value_type *) &((std::vector< RobotRaconteur::NodeInfo2 > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__NodeInfo2, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::NodeInfo2 >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::NodeInfo2 >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::NodeInfo2 >::value_type *) &((std::vector< RobotRaconteur::NodeInfo2 > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__NodeInfo2, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::NodeInfo2 >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  std::vector< RobotRaconteur::NodeInfo2 >::size_type arg2 ;
  std::vector< RobotRaconteur::NodeInfo2 >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectornodeinfo2_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_assign" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectornodeinfo2_assign" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::NodeInfo2 >::size_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__NodeInfo2,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectornodeinfo2_assign" "', argument " "3"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectornodeinfo2_assign" "', argument " "3"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< RobotRaconteur::NodeInfo2 >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  std::vector< RobotRaconteur::NodeInfo2 >::size_type arg2 ;
  std::vector< RobotRaconteur::NodeInfo2 >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_resize" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectornodeinfo2_resize" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::NodeInfo2 >::size_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__NodeInfo2,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectornodeinfo2_resize" "', argument " "3"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectornodeinfo2_resize" "', argument " "3"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< RobotRaconteur::NodeInfo2 >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectornodeinfo2_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectornodeinfo2_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__NodeInfo2, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectornodeinfo2_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectornodeinfo2_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::NodeInfo2 >::resize(std::vector< RobotRaconteur::NodeInfo2 >::size_type)\n"
    "    std::vector< RobotRaconteur::NodeInfo2 >::resize(std::vector< RobotRaconteur::NodeInfo2 >::size_type,std::vector< RobotRaconteur::NodeInfo2 >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  std::vector< RobotRaconteur::NodeInfo2 >::iterator arg2 ;
  std::vector< RobotRaconteur::NodeInfo2 >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  std::vector< RobotRaconteur::NodeInfo2 >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_insert" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectornodeinfo2_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::NodeInfo2 >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::NodeInfo2 >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectornodeinfo2_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__NodeInfo2,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectornodeinfo2_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectornodeinfo2_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_NodeInfo2_Sg__insert__SWIG_0(arg1,arg2,(RobotRaconteur::NodeInfo2 const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::NodeInfo2 >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  std::vector< RobotRaconteur::NodeInfo2 >::iterator arg2 ;
  std::vector< RobotRaconteur::NodeInfo2 >::size_type arg3 ;
  std::vector< RobotRaconteur::NodeInfo2 >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  void *argp4 = 0 ;
  int res4 = 0 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_insert" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectornodeinfo2_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::NodeInfo2 >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::NodeInfo2 >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectornodeinfo2_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectornodeinfo2_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::size_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::NodeInfo2 >::size_type >(val3);
  res4 = SWIG_ConvertPtr(swig_obj[3], &argp4, SWIGTYPE_p_RobotRaconteur__NodeInfo2,  0  | 0);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectornodeinfo2_insert" "', argument " "4"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::value_type const &""'"); 
  }
  if (!argp4) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectornodeinfo2_insert" "', argument " "4"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::value_type const &""'"); 
  }
  arg4 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 >::value_type * >(argp4);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_RobotRaconteur_NodeInfo2_Sg__insert__SWIG_1(arg1,arg2,arg3,(RobotRaconteur::NodeInfo2 const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectornodeinfo2_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::NodeInfo2 >::iterator > *>(iter) != 0));
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__NodeInfo2, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectornodeinfo2_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::NodeInfo2 >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_RobotRaconteur__NodeInfo2, SWIG_POINTER_NO_NULL | 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectornodeinfo2_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectornodeinfo2_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::NodeInfo2 >::insert(std::vector< RobotRaconteur::NodeInfo2 >::iterator,std::vector< RobotRaconteur::NodeInfo2 >::value_type const &)\n"
    "    std::vector< RobotRaconteur::NodeInfo2 >::insert(std::vector< RobotRaconteur::NodeInfo2 >::iterator,std::vector< RobotRaconteur::NodeInfo2 >::size_type,std::vector< RobotRaconteur::NodeInfo2 >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  std::vector< RobotRaconteur::NodeInfo2 >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectornodeinfo2_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_reserve" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectornodeinfo2_reserve" "', argument " "2"" of type '" "std::vector< RobotRaconteur::NodeInfo2 >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::NodeInfo2 >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectornodeinfo2_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::NodeInfo2 >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectornodeinfo2_capacity" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::NodeInfo2 > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vectornodeinfo2(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::NodeInfo2 > *arg1 = (std::vector< RobotRaconteur::NodeInfo2 > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vectornodeinfo2" "', argument " "1"" of type '" "std::vector< RobotRaconteur::NodeInfo2 > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::NodeInfo2 > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vectornodeinfo2_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vectornodeinfo2_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_new_map_subscriptionserviceinfo2__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::less< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t,  0  | 0);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_map_subscriptionserviceinfo2" "', argument " "1"" of type '" "std::less< RobotRaconteur::ServiceSubscriptionClientID > const &""'"); 
  }
  if (!argp1) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_map_subscriptionserviceinfo2" "', argument " "1"" of type '" "std::less< RobotRaconteur::ServiceSubscriptionClientID > const &""'"); 
  }
  arg1 = reinterpret_cast< std::less< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *)new std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >((std::less< RobotRaconteur::ServiceSubscriptionClientID > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_iterator" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2___nonzero__" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg____nonzero__((std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2___bool__" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg____bool__((std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2___len__" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg____len__((std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2___getitem__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::mapped_type *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionserviceinfo2___getitem__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2___getitem__" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionserviceinfo2___getitem__" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionserviceinfo2___getitem__" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type * >(argp2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::mapped_type *) &std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg____getitem__(arg1,(RobotRaconteur::ServiceSubscriptionClientID const &)*arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2___delitem__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionserviceinfo2___delitem__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2___delitem__" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionserviceinfo2___delitem__" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionserviceinfo2___delitem__" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type * >(argp2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg____delitem__(arg1,(RobotRaconteur::ServiceSubscriptionClientID const &)*arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_has_key(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionserviceinfo2_has_key", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_has_key" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionserviceinfo2_has_key" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionserviceinfo2_has_key" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg__has_key((std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const *)arg1,(RobotRaconteur::ServiceSubscriptionClientID const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_keys(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_keys" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg__keys(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_values(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_values" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg__values(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_items(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_items" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg__items(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2___contains__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionserviceinfo2___contains__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2___contains__" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionserviceinfo2___contains__" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionserviceinfo2___contains__" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg____contains__(arg1,(RobotRaconteur::ServiceSubscriptionClientID const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_key_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_key_iterator" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg__key_iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_value_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_value_iterator" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg__value_iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2___setitem__" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionserviceinfo2___setitem__" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionserviceinfo2___setitem__" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg____setitem____SWIG_0(arg1,(RobotRaconteur::ServiceSubscriptionClientID const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type *arg2 = 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::mapped_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2___setitem__" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionserviceinfo2___setitem__" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionserviceinfo2___setitem__" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type * >(argp2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "map_subscriptionserviceinfo2___setitem__" "', argument " "3"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::mapped_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionserviceinfo2___setitem__" "', argument " "3"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::mapped_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::mapped_type * >(argp3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg____setitem____SWIG_1(arg1,(RobotRaconteur::ServiceSubscriptionClientID const &)*arg2,(RobotRaconteur::ServiceInfo2Wrapped const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "map_subscriptionserviceinfo2___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID, SWIG_POINTER_NO_NULL | 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_map_subscriptionserviceinfo2___setitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID, SWIG_POINTER_NO_NULL | 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_map_subscriptionserviceinfo2___setitem____SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'map_subscriptionserviceinfo2___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::__setitem__(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &)\n"
    "    std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::__setitem__(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &,std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::mapped_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_asdict(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_asdict" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg__asdict(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_map_subscriptionserviceinfo2__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *)new std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_map_subscriptionserviceinfo2__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > > *ptr = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_map_subscriptionserviceinfo2" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_map_subscriptionserviceinfo2" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *)new std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >((std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_map_subscriptionserviceinfo2(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[2] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_map_subscriptionserviceinfo2", 0, 1, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_map_subscriptionserviceinfo2__SWIG_1(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t, SWIG_POINTER_NO_NULL | 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_map_subscriptionserviceinfo2__SWIG_0(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_map_subscriptionserviceinfo2__SWIG_2(self, argc, argv);
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_map_subscriptionserviceinfo2'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::map(std::less< RobotRaconteur::ServiceSubscriptionClientID > const &)\n"
    "    std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::map()\n"
    "    std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::map(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_empty" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_size" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionserviceinfo2_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_swap" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionserviceinfo2_swap" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionserviceinfo2_swap" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_begin" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_end" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_rbegin" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_rend" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_clear" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_get_allocator" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::allocator_type(static_cast< const std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::size_type result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_erase" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionserviceinfo2_erase" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionserviceinfo2_erase" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->erase((std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_count(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::size_type result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionserviceinfo2_count", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_count" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionserviceinfo2_count" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionserviceinfo2_count" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > const *)arg1)->count((std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_erase" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionserviceinfo2_erase" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionserviceinfo2_erase" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg__erase__SWIG_1(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_erase__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator arg2 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_erase" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionserviceinfo2_erase" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionserviceinfo2_erase" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionserviceinfo2_erase" "', argument " "3"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionserviceinfo2_erase" "', argument " "3"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_RobotRaconteur_ServiceInfo2Wrapped_Sg__erase__SWIG_2(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "map_subscriptionserviceinfo2_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID, SWIG_POINTER_NO_NULL | 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_map_subscriptionserviceinfo2_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_map_subscriptionserviceinfo2_erase__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_map_subscriptionserviceinfo2_erase__SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'map_subscriptionserviceinfo2_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::erase(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &)\n"
    "    std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::erase(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator)\n"
    "    std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::erase(std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator,std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_find(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionserviceinfo2_find", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_find" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionserviceinfo2_find" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionserviceinfo2_find" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->find((std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_lower_bound(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionserviceinfo2_lower_bound", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_lower_bound" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionserviceinfo2_lower_bound" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionserviceinfo2_lower_bound" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->lower_bound((std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionserviceinfo2_upper_bound(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionserviceinfo2_upper_bound", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionserviceinfo2_upper_bound" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionserviceinfo2_upper_bound" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionserviceinfo2_upper_bound" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->upper_bound((std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_map_subscriptionserviceinfo2(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_map_subscriptionserviceinfo2" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *map_subscriptionserviceinfo2_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *map_subscriptionserviceinfo2_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_new_map_subscriptionclients__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::less< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t,  0  | 0);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_map_subscriptionclients" "', argument " "1"" of type '" "std::less< RobotRaconteur::ServiceSubscriptionClientID > const &""'"); 
  }
  if (!argp1) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_map_subscriptionclients" "', argument " "1"" of type '" "std::less< RobotRaconteur::ServiceSubscriptionClientID > const &""'"); 
  }
  arg1 = reinterpret_cast< std::less< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *)new std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >((std::less< RobotRaconteur::ServiceSubscriptionClientID > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_iterator" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients___nonzero__" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg____nonzero__((std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients___bool__" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg____bool__((std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients___len__" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg____len__((std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients___getitem__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::mapped_type *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionclients___getitem__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients___getitem__" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionclients___getitem__" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionclients___getitem__" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type * >(argp2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::mapped_type *) &std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg____getitem__(arg1,(RobotRaconteur::ServiceSubscriptionClientID const &)*arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients___delitem__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionclients___delitem__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients___delitem__" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionclients___delitem__" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionclients___delitem__" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type * >(argp2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg____delitem__(arg1,(RobotRaconteur::ServiceSubscriptionClientID const &)*arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_has_key(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionclients_has_key", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_has_key" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionclients_has_key" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionclients_has_key" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg__has_key((std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const *)arg1,(RobotRaconteur::ServiceSubscriptionClientID const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_keys(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_keys" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg__keys(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_values(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_values" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg__values(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_items(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_items" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg__items(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients___contains__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionclients___contains__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients___contains__" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionclients___contains__" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionclients___contains__" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg____contains__(arg1,(RobotRaconteur::ServiceSubscriptionClientID const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_key_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_key_iterator" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg__key_iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_value_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_value_iterator" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg__value_iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients___setitem__" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionclients___setitem__" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionclients___setitem__" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg____setitem____SWIG_0(arg1,(RobotRaconteur::ServiceSubscriptionClientID const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type *arg2 = 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::mapped_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::mapped_type tempshared3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients___setitem__" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionclients___setitem__" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionclients___setitem__" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type * >(argp2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "map_subscriptionclients___setitem__" "', argument " "3"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::mapped_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::mapped_type * >(argp3);
      delete reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::mapped_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::mapped_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg____setitem____SWIG_1(arg1,(RobotRaconteur::ServiceSubscriptionClientID const &)*arg2,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "map_subscriptionclients___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID, SWIG_POINTER_NO_NULL | 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_map_subscriptionclients___setitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID, SWIG_POINTER_NO_NULL | 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_map_subscriptionclients___setitem____SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'map_subscriptionclients___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::__setitem__(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &)\n"
    "    std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::__setitem__(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &,std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::mapped_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_asdict(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_asdict" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg__asdict(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_map_subscriptionclients__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *)new std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_map_subscriptionclients__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > > *ptr = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_map_subscriptionclients" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_map_subscriptionclients" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *)new std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >((std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_map_subscriptionclients(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[2] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_map_subscriptionclients", 0, 1, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_map_subscriptionclients__SWIG_1(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t, SWIG_POINTER_NO_NULL | 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_map_subscriptionclients__SWIG_0(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_map_subscriptionclients__SWIG_2(self, argc, argv);
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_map_subscriptionclients'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::map(std::less< RobotRaconteur::ServiceSubscriptionClientID > const &)\n"
    "    std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::map()\n"
    "    std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::map(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_empty" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_size" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionclients_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_swap" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionclients_swap" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionclients_swap" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_begin" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_end" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_rbegin" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_rend" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_clear" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_get_allocator" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::allocator_type(static_cast< const std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::size_type result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_erase" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionclients_erase" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionclients_erase" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->erase((std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_count(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::size_type result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionclients_count", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_count" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionclients_count" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionclients_count" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > const *)arg1)->count((std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_erase" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionclients_erase" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionclients_erase" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg__erase__SWIG_1(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_erase__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator arg2 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_erase" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionclients_erase" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionclients_erase" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionclients_erase" "', argument " "3"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionclients_erase" "', argument " "3"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_map_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sc_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceStub_Sg__Sg__erase__SWIG_2(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "map_subscriptionclients_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID, SWIG_POINTER_NO_NULL | 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_map_subscriptionclients_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_map_subscriptionclients_erase__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_map_subscriptionclients_erase__SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'map_subscriptionclients_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::erase(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &)\n"
    "    std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::erase(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator)\n"
    "    std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::erase(std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator,std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_find(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionclients_find", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_find" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionclients_find" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionclients_find" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->find((std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_lower_bound(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionclients_lower_bound", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_lower_bound" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionclients_lower_bound" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionclients_lower_bound" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->lower_bound((std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionclients_upper_bound(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionclients_upper_bound", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionclients_upper_bound" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionclients_upper_bound" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionclients_upper_bound" "', argument " "2"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->upper_bound((std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_map_subscriptionclients(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *arg1 = (std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_map_subscriptionclients" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *map_subscriptionclients_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *map_subscriptionclients_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_iterator" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode___nonzero__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____nonzero__((std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode___bool__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____bool__((std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode___len__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____len__((std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_wrappedservicesubscriptionnode___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode___getslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_wrappedservicesubscriptionnode___getslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_wrappedservicesubscriptionnode___getslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode___setslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_wrappedservicesubscriptionnode___setslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_wrappedservicesubscriptionnode___setslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type arg3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode___setslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_wrappedservicesubscriptionnode___setslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_wrappedservicesubscriptionnode___setslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type >(val3);
  {
    std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorptr_wrappedservicesubscriptionnode___setslice__" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_wrappedservicesubscriptionnode___setslice__" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_wrappedservicesubscriptionnode___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vectorptr_wrappedservicesubscriptionnode___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorptr_wrappedservicesubscriptionnode___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_wrappedservicesubscriptionnode___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::__setslice__(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::__setslice__(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_wrappedservicesubscriptionnode___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode___delslice__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_wrappedservicesubscriptionnode___delslice__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_wrappedservicesubscriptionnode___delslice__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode___delitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_wrappedservicesubscriptionnode___delitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode___getitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_wrappedservicesubscriptionnode___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > *)std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_wrappedservicesubscriptionnode___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_wrappedservicesubscriptionnode___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_wrappedservicesubscriptionnode___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____setitem____SWIG_0(arg1,arg2,(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_wrappedservicesubscriptionnode___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode___delitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_wrappedservicesubscriptionnode___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_wrappedservicesubscriptionnode___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_wrappedservicesubscriptionnode___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_wrappedservicesubscriptionnode___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_wrappedservicesubscriptionnode___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::__delitem__(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode___getitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_wrappedservicesubscriptionnode___getitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type *) &std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____getitem____SWIG_1((std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_wrappedservicesubscriptionnode___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_wrappedservicesubscriptionnode___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_wrappedservicesubscriptionnode___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_wrappedservicesubscriptionnode___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::__getitem__(PySliceObject *)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::__getitem__(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type tempshared3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode___setitem__" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_wrappedservicesubscriptionnode___setitem__" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_wrappedservicesubscriptionnode___setitem__" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg____setitem____SWIG_2(arg1,arg2,(boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_wrappedservicesubscriptionnode___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_wrappedservicesubscriptionnode___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_wrappedservicesubscriptionnode___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_wrappedservicesubscriptionnode___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_wrappedservicesubscriptionnode___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::__setitem__(PySliceObject *,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::__setitem__(PySliceObject *)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::__setitem__(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::difference_type,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_pop" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_wrappedservicesubscriptionnode_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_append" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_wrappedservicesubscriptionnode_append" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg__append(arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_wrappedservicesubscriptionnode__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *)new std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_wrappedservicesubscriptionnode__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vectorptr_wrappedservicesubscriptionnode" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vectorptr_wrappedservicesubscriptionnode" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *)new std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >((std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_empty" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_size" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_wrappedservicesubscriptionnode_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_swap" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_wrappedservicesubscriptionnode_swap" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_wrappedservicesubscriptionnode_swap" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_begin" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_end" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_rbegin" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_rend" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_clear" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_get_allocator" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::allocator_type(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_wrappedservicesubscriptionnode__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorptr_wrappedservicesubscriptionnode" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type""'");
  } 
  arg1 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *)new std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_pop_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_resize" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_wrappedservicesubscriptionnode_resize" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_erase" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_wrappedservicesubscriptionnode_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_wrappedservicesubscriptionnode_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_erase" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_wrappedservicesubscriptionnode_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_wrappedservicesubscriptionnode_erase" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_wrappedservicesubscriptionnode_erase" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_wrappedservicesubscriptionnode_erase" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_wrappedservicesubscriptionnode_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vectorptr_wrappedservicesubscriptionnode_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vectorptr_wrappedservicesubscriptionnode_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_wrappedservicesubscriptionnode_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::erase(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::erase(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_wrappedservicesubscriptionnode__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type arg1 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type tempshared2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorptr_wrappedservicesubscriptionnode" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type""'");
  } 
  arg1 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type >(val1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_vectorptr_wrappedservicesubscriptionnode" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *)new std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >(arg1,(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_wrappedservicesubscriptionnode(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vectorptr_wrappedservicesubscriptionnode", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vectorptr_wrappedservicesubscriptionnode__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vectorptr_wrappedservicesubscriptionnode__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vectorptr_wrappedservicesubscriptionnode__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_vectorptr_wrappedservicesubscriptionnode__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vectorptr_wrappedservicesubscriptionnode'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::vector()\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::vector(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_wrappedservicesubscriptionnode_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_push_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_wrappedservicesubscriptionnode_push_back" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp2);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_front" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type *) &((std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_back" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type *) &((std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t, SWIG_POINTER_OWN);
  }
  (void)swig::container_owner<swig::traits<std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type tempshared3 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_wrappedservicesubscriptionnode_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_assign" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_wrappedservicesubscriptionnode_assign" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_wrappedservicesubscriptionnode_assign" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type tempshared3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_resize" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_wrappedservicesubscriptionnode_resize" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type >(val2);
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_wrappedservicesubscriptionnode_resize" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_wrappedservicesubscriptionnode_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_wrappedservicesubscriptionnode_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_wrappedservicesubscriptionnode_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_wrappedservicesubscriptionnode_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::resize(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::resize(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  void *argp3 ;
  int res3 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type tempshared3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_insert" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_wrappedservicesubscriptionnode_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_wrappedservicesubscriptionnode_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator""'");
    }
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_wrappedservicesubscriptionnode_insert" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp3);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg__insert__SWIG_0(arg1,arg2,(boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator arg2 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type arg3 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  void *argp4 ;
  int res4 = 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type tempshared4 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_insert" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_wrappedservicesubscriptionnode_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_wrappedservicesubscriptionnode_insert" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_wrappedservicesubscriptionnode_insert" "', argument " "3"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type""'");
  } 
  arg3 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type >(val3);
  {
    int newmem = 0;
    res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorptr_wrappedservicesubscriptionnode_insert" "', argument " "4"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp4) tempshared4 = *reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp4);
      delete reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp4);
      arg4 = &tempshared4;
    } else {
      arg4 = (argp4) ? reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type * >(argp4) : &tempshared4;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_boost_shared_ptr_Sl_RobotRaconteur_WrappedServiceSubscriptionFilterNode_Sg__Sg__insert__SWIG_1(arg1,arg2,arg3,(boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_wrappedservicesubscriptionnode_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator > *>(iter) != 0));
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_wrappedservicesubscriptionnode_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorptr_wrappedservicesubscriptionnode_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_wrappedservicesubscriptionnode_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::insert(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type const &)\n"
    "    std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::insert(std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::iterator,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type,std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_wrappedservicesubscriptionnode_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_reserve" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_wrappedservicesubscriptionnode_reserve" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type""'");
  } 
  arg2 = static_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_wrappedservicesubscriptionnode_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_wrappedservicesubscriptionnode_capacity" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vectorptr_wrappedservicesubscriptionnode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *arg1 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vectorptr_wrappedservicesubscriptionnode" "', argument " "1"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vectorptr_wrappedservicesubscriptionnode_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vectorptr_wrappedservicesubscriptionnode_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_iterator" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid___nonzero__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____nonzero__((std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid___bool__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____bool__((std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid___len__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____len__((std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type arg2 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_subscriptionclientid___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid___getslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_subscriptionclientid___getslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_subscriptionclientid___getslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > *)std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type arg2 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid___setslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_subscriptionclientid___setslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_subscriptionclientid___setslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type arg2 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type arg3 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid___setslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_subscriptionclientid___setslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_subscriptionclientid___setslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type >(val3);
  {
    std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > *ptr = (std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorptr_subscriptionclientid___setslice__" "', argument " "4"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_subscriptionclientid___setslice__" "', argument " "4"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_subscriptionclientid___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vectorptr_subscriptionclientid___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorptr_subscriptionclientid___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_subscriptionclientid___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionClientID >::__setslice__(std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type)\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionClientID >::__setslice__(std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type,std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type arg2 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_subscriptionclientid___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid___delslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_subscriptionclientid___delslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_subscriptionclientid___delslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid___delitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_subscriptionclientid___delitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid___getitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_subscriptionclientid___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > *)std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_subscriptionclientid___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > *ptr = (std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_subscriptionclientid___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_subscriptionclientid___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____setitem____SWIG_0(arg1,arg2,(std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_subscriptionclientid___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid___delitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_subscriptionclientid___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_subscriptionclientid___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_subscriptionclientid___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_subscriptionclientid___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_subscriptionclientid___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionClientID >::__delitem__(std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type)\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionClientID >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid___getitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_subscriptionclientid___getitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type *) &std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____getitem____SWIG_1((std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_subscriptionclientid___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_subscriptionclientid___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_subscriptionclientid___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_subscriptionclientid___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionClientID >::__getitem__(PySliceObject *)\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionClientID >::__getitem__(std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type arg2 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_subscriptionclientid___setitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_subscriptionclientid___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_subscriptionclientid___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type * >(argp3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg____setitem____SWIG_2(arg1,arg2,(RobotRaconteur::ServiceSubscriptionClientID const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_subscriptionclientid___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vectorptr_subscriptionclientid___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_subscriptionclientid___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_subscriptionclientid___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_subscriptionclientid___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionClientID >::__setitem__(PySliceObject *,std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > const &)\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionClientID >::__setitem__(PySliceObject *)\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionClientID >::__setitem__(std::vector< RobotRaconteur::ServiceSubscriptionClientID >::difference_type,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_pop" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type(static_cast< const std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type& >(result))), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_subscriptionclientid_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_append" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_subscriptionclientid_append" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_subscriptionclientid_append" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg__append(arg1,(RobotRaconteur::ServiceSubscriptionClientID const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_subscriptionclientid__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *)new std::vector< RobotRaconteur::ServiceSubscriptionClientID >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_subscriptionclientid__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > *ptr = (std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vectorptr_subscriptionclientid" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vectorptr_subscriptionclientid" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *)new std::vector< RobotRaconteur::ServiceSubscriptionClientID >((std::vector< RobotRaconteur::ServiceSubscriptionClientID > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_empty" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_size" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_subscriptionclientid_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_swap" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_subscriptionclientid_swap" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_subscriptionclientid_swap" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_begin" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_end" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_rbegin" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServiceSubscriptionClientID >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_rend" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServiceSubscriptionClientID >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_clear" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_get_allocator" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< RobotRaconteur::ServiceSubscriptionClientID >::allocator_type(static_cast< const std::vector< RobotRaconteur::ServiceSubscriptionClientID >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_subscriptionclientid__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorptr_subscriptionclientid" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type""'");
  } 
  arg1 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *)new std::vector< RobotRaconteur::ServiceSubscriptionClientID >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_pop_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_resize" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_subscriptionclientid_resize" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_erase" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_subscriptionclientid_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_subscriptionclientid_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator arg2 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_erase" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_subscriptionclientid_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_subscriptionclientid_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_subscriptionclientid_erase" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_subscriptionclientid_erase" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_subscriptionclientid_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vectorptr_subscriptionclientid_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vectorptr_subscriptionclientid_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_subscriptionclientid_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionClientID >::erase(std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator)\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionClientID >::erase(std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_subscriptionclientid__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type arg1 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vectorptr_subscriptionclientid" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type""'");
  } 
  arg1 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type >(val1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_vectorptr_subscriptionclientid" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vectorptr_subscriptionclientid" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *)new std::vector< RobotRaconteur::ServiceSubscriptionClientID >(arg1,(std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vectorptr_subscriptionclientid(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vectorptr_subscriptionclientid", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vectorptr_subscriptionclientid__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vectorptr_subscriptionclientid__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vectorptr_subscriptionclientid__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID, SWIG_POINTER_NO_NULL | 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_vectorptr_subscriptionclientid__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vectorptr_subscriptionclientid'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionClientID >::vector()\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionClientID >::vector(std::vector< RobotRaconteur::ServiceSubscriptionClientID > const &)\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionClientID >::vector(std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type)\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionClientID >::vector(std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_subscriptionclientid_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_push_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vectorptr_subscriptionclientid_push_back" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_subscriptionclientid_push_back" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_front" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type *) &((std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type *) &((std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type arg2 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_subscriptionclientid_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_assign" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_subscriptionclientid_assign" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_subscriptionclientid_assign" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_subscriptionclientid_assign" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type arg2 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_resize" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_subscriptionclientid_resize" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_subscriptionclientid_resize" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_subscriptionclientid_resize" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_subscriptionclientid_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vectorptr_subscriptionclientid_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_subscriptionclientid_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_subscriptionclientid_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionClientID >::resize(std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type)\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionClientID >::resize(std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator arg2 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_insert" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_subscriptionclientid_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_subscriptionclientid_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vectorptr_subscriptionclientid_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_subscriptionclientid_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg__insert__SWIG_0(arg1,arg2,(RobotRaconteur::ServiceSubscriptionClientID const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator arg2 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type arg3 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  void *argp4 = 0 ;
  int res4 = 0 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_insert" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_subscriptionclientid_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vectorptr_subscriptionclientid_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vectorptr_subscriptionclientid_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type >(val3);
  res4 = SWIG_ConvertPtr(swig_obj[3], &argp4, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vectorptr_subscriptionclientid_insert" "', argument " "4"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &""'"); 
  }
  if (!argp4) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vectorptr_subscriptionclientid_insert" "', argument " "4"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &""'"); 
  }
  arg4 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type * >(argp4);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_RobotRaconteur_ServiceSubscriptionClientID_Sg__insert__SWIG_1(arg1,arg2,arg3,(RobotRaconteur::ServiceSubscriptionClientID const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vectorptr_subscriptionclientid_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator > *>(iter) != 0));
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vectorptr_subscriptionclientid_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID, SWIG_POINTER_NO_NULL | 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vectorptr_subscriptionclientid_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vectorptr_subscriptionclientid_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionClientID >::insert(std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &)\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionClientID >::insert(std::vector< RobotRaconteur::ServiceSubscriptionClientID >::iterator,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type,std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vectorptr_subscriptionclientid_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_reserve" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vectorptr_subscriptionclientid_reserve" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vectorptr_subscriptionclientid_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vectorptr_subscriptionclientid_capacity" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::ServiceSubscriptionClientID > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vectorptr_subscriptionclientid(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionClientID > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vectorptr_subscriptionclientid" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vectorptr_subscriptionclientid_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vectorptr_subscriptionclientid_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_iterator" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___nonzero__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____nonzero__((std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___bool__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____bool__((std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___len__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____len__((std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type arg2 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_wrappedservicesubscriptionmanagerdetails___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___getslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___getslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___getslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > *)std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type arg2 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___setslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___setslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___setslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type arg2 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type arg3 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___setslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___setslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___setslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type >(val3);
  {
    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > *ptr = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___setslice__" "', argument " "4"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_wrappedservicesubscriptionmanagerdetails___setslice__" "', argument " "4"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_wrappedservicesubscriptionmanagerdetails___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vector_wrappedservicesubscriptionmanagerdetails___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vector_wrappedservicesubscriptionmanagerdetails___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_wrappedservicesubscriptionmanagerdetails___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::__setslice__(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type)\n"
    "    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::__setslice__(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type arg2 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_wrappedservicesubscriptionmanagerdetails___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___delslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___delslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___delslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___delitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___delitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___getitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > *)std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > *ptr = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_wrappedservicesubscriptionmanagerdetails___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____setitem____SWIG_0(arg1,arg2,(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___delitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_wrappedservicesubscriptionmanagerdetails___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_wrappedservicesubscriptionmanagerdetails___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_wrappedservicesubscriptionmanagerdetails___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_wrappedservicesubscriptionmanagerdetails___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::__delitem__(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type)\n"
    "    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___getitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___getitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type *) &std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____getitem____SWIG_1((std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_wrappedservicesubscriptionmanagerdetails___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_wrappedservicesubscriptionmanagerdetails___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_wrappedservicesubscriptionmanagerdetails___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_wrappedservicesubscriptionmanagerdetails___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::__getitem__(PySliceObject *)\n"
    "    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::__getitem__(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type arg2 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___setitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_wrappedservicesubscriptionmanagerdetails___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_wrappedservicesubscriptionmanagerdetails___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type * >(argp3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg____setitem____SWIG_2(arg1,arg2,(RobotRaconteur::WrappedServiceSubscriptionManagerDetails const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_wrappedservicesubscriptionmanagerdetails___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_wrappedservicesubscriptionmanagerdetails___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vector_wrappedservicesubscriptionmanagerdetails___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vector_wrappedservicesubscriptionmanagerdetails___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_wrappedservicesubscriptionmanagerdetails___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::__setitem__(PySliceObject *,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > const &)\n"
    "    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::__setitem__(PySliceObject *)\n"
    "    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::__setitem__(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::difference_type,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_pop" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type(static_cast< const std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type& >(result))), SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_wrappedservicesubscriptionmanagerdetails_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_append" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_append" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_wrappedservicesubscriptionmanagerdetails_append" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg__append(arg1,(RobotRaconteur::WrappedServiceSubscriptionManagerDetails const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_wrappedservicesubscriptionmanagerdetails__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *)new std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_wrappedservicesubscriptionmanagerdetails__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > *ptr = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vector_wrappedservicesubscriptionmanagerdetails" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vector_wrappedservicesubscriptionmanagerdetails" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *)new std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >((std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_empty" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_size" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_wrappedservicesubscriptionmanagerdetails_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_swap" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_swap" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_wrappedservicesubscriptionmanagerdetails_swap" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_begin" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_end" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_rbegin" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_rend" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_clear" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_get_allocator" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::allocator_type(static_cast< const std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_wrappedservicesubscriptionmanagerdetails__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vector_wrappedservicesubscriptionmanagerdetails" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type""'");
  } 
  arg1 = static_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *)new std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_pop_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_resize" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_resize" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_erase" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator arg2 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_erase" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_erase" "', argument " "3"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_erase" "', argument " "3"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_wrappedservicesubscriptionmanagerdetails_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vector_wrappedservicesubscriptionmanagerdetails_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vector_wrappedservicesubscriptionmanagerdetails_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_wrappedservicesubscriptionmanagerdetails_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::erase(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator)\n"
    "    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::erase(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vector_wrappedservicesubscriptionmanagerdetails__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type arg1 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vector_wrappedservicesubscriptionmanagerdetails" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type""'");
  } 
  arg1 = static_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type >(val1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_vector_wrappedservicesubscriptionmanagerdetails" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vector_wrappedservicesubscriptionmanagerdetails" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *)new std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >(arg1,(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_wrappedservicesubscriptionmanagerdetails(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vector_wrappedservicesubscriptionmanagerdetails", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vector_wrappedservicesubscriptionmanagerdetails__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vector_wrappedservicesubscriptionmanagerdetails__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vector_wrappedservicesubscriptionmanagerdetails__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, SWIG_POINTER_NO_NULL | 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_vector_wrappedservicesubscriptionmanagerdetails__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vector_wrappedservicesubscriptionmanagerdetails'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::vector()\n"
    "    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::vector(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const &)\n"
    "    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::vector(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type)\n"
    "    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::vector(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_wrappedservicesubscriptionmanagerdetails_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_push_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_push_back" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_wrappedservicesubscriptionmanagerdetails_push_back" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_front" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type *) &((std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type *) &((std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type arg2 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_wrappedservicesubscriptionmanagerdetails_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_assign" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_assign" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_assign" "', argument " "3"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_wrappedservicesubscriptionmanagerdetails_assign" "', argument " "3"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type arg2 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_resize" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_resize" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_resize" "', argument " "3"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_wrappedservicesubscriptionmanagerdetails_resize" "', argument " "3"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_wrappedservicesubscriptionmanagerdetails_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_wrappedservicesubscriptionmanagerdetails_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vector_wrappedservicesubscriptionmanagerdetails_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_wrappedservicesubscriptionmanagerdetails_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::resize(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type)\n"
    "    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::resize(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator arg2 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_insert" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_wrappedservicesubscriptionmanagerdetails_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg__insert__SWIG_0(arg1,arg2,(RobotRaconteur::WrappedServiceSubscriptionManagerDetails const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator arg2 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type arg3 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  void *argp4 = 0 ;
  int res4 = 0 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_insert" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type >(val3);
  res4 = SWIG_ConvertPtr(swig_obj[3], &argp4, SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails,  0  | 0);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_insert" "', argument " "4"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &""'"); 
  }
  if (!argp4) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_wrappedservicesubscriptionmanagerdetails_insert" "', argument " "4"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &""'"); 
  }
  arg4 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type * >(argp4);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_RobotRaconteur_WrappedServiceSubscriptionManagerDetails_Sg__insert__SWIG_1(arg1,arg2,arg3,(RobotRaconteur::WrappedServiceSubscriptionManagerDetails const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_wrappedservicesubscriptionmanagerdetails_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator > *>(iter) != 0));
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vector_wrappedservicesubscriptionmanagerdetails_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, SWIG_POINTER_NO_NULL | 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vector_wrappedservicesubscriptionmanagerdetails_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_wrappedservicesubscriptionmanagerdetails_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::insert(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &)\n"
    "    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::insert(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::iterator,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type,std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_wrappedservicesubscriptionmanagerdetails_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_reserve" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_reserve" "', argument " "2"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_wrappedservicesubscriptionmanagerdetails_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_wrappedservicesubscriptionmanagerdetails_capacity" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vector_wrappedservicesubscriptionmanagerdetails(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *arg1 = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vector_wrappedservicesubscriptionmanagerdetails" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vector_wrappedservicesubscriptionmanagerdetails_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vector_wrappedservicesubscriptionmanagerdetails_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttribute_Name_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttribute *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttribute *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceSubscriptionFilterAttribute_Name_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceSubscriptionFilterAttribute_Name_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttribute *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttribute * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceSubscriptionFilterAttribute_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceSubscriptionFilterAttribute_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Name = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttribute_Name_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttribute *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttribute *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceSubscriptionFilterAttribute_Name_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttribute *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttribute * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->Name);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttribute_Value_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttribute *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttribute *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceSubscriptionFilterAttribute_Value_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceSubscriptionFilterAttribute_Value_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttribute *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttribute * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceSubscriptionFilterAttribute_Value_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceSubscriptionFilterAttribute_Value_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Value = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttribute_Value_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttribute *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttribute *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceSubscriptionFilterAttribute_Value_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttribute *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttribute * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->Value);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttribute_UseRegex_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttribute *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttribute *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceSubscriptionFilterAttribute_UseRegex_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceSubscriptionFilterAttribute_UseRegex_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttribute *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttribute * >(argp1);
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ServiceSubscriptionFilterAttribute_UseRegex_set" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->UseRegex = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttribute_UseRegex_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttribute *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttribute *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceSubscriptionFilterAttribute_UseRegex_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttribute *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttribute * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (bool) ((arg1)->UseRegex);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ServiceSubscriptionFilterAttribute__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  RobotRaconteur::ServiceSubscriptionFilterAttribute *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_ServiceSubscriptionFilterAttribute" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_ServiceSubscriptionFilterAttribute" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::ServiceSubscriptionFilterAttribute *)new RobotRaconteur::ServiceSubscriptionFilterAttribute((std::string const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ServiceSubscriptionFilterAttribute__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  std::string *arg2 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  int res2 = SWIG_OLDOBJ ;
  RobotRaconteur::ServiceSubscriptionFilterAttribute *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_ServiceSubscriptionFilterAttribute" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_ServiceSubscriptionFilterAttribute" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_ServiceSubscriptionFilterAttribute" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_ServiceSubscriptionFilterAttribute" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::ServiceSubscriptionFilterAttribute *)new RobotRaconteur::ServiceSubscriptionFilterAttribute((std::string const &)*arg1,(std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ServiceSubscriptionFilterAttribute(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_ServiceSubscriptionFilterAttribute", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_AsPtr_std_string(argv[0], (std::string**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_ServiceSubscriptionFilterAttribute__SWIG_0(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_AsPtr_std_string(argv[0], (std::string**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_ServiceSubscriptionFilterAttribute__SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_ServiceSubscriptionFilterAttribute'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::ServiceSubscriptionFilterAttribute::ServiceSubscriptionFilterAttribute(std::string const &)\n"
    "    RobotRaconteur::ServiceSubscriptionFilterAttribute::ServiceSubscriptionFilterAttribute(std::string const &,std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttribute_IsMatch__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttribute *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttribute *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  bool result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceSubscriptionFilterAttribute_IsMatch" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttribute const *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttribute * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceSubscriptionFilterAttribute_IsMatch" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceSubscriptionFilterAttribute_IsMatch" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((RobotRaconteur::ServiceSubscriptionFilterAttribute const *)arg1)->IsMatch((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttribute_IsMatch__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttribute *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttribute *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  bool result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceSubscriptionFilterAttribute_IsMatch" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttribute const *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttribute * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceSubscriptionFilterAttribute_IsMatch" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceSubscriptionFilterAttribute_IsMatch" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "ServiceSubscriptionFilterAttribute_IsMatch" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceSubscriptionFilterAttribute_IsMatch" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((RobotRaconteur::ServiceSubscriptionFilterAttribute const *)arg1)->IsMatch((std::string const &)*arg2,(std::string const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttribute_IsMatch__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttribute *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttribute *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  bool result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceSubscriptionFilterAttribute_IsMatch" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttribute const *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttribute * >(argp1);
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceSubscriptionFilterAttribute_IsMatch" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceSubscriptionFilterAttribute_IsMatch" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((RobotRaconteur::ServiceSubscriptionFilterAttribute const *)arg1)->IsMatch((std::vector< std::string,std::allocator< std::string > > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttribute_IsMatch__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttribute *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttribute *) 0 ;
  std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  bool result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceSubscriptionFilterAttribute_IsMatch" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttribute const *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttribute * >(argp1);
  {
    std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > *ptr = (std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceSubscriptionFilterAttribute_IsMatch" "', argument " "2"" of type '" "std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceSubscriptionFilterAttribute_IsMatch" "', argument " "2"" of type '" "std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((RobotRaconteur::ServiceSubscriptionFilterAttribute const *)arg1)->IsMatch((std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttribute_IsMatch(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "ServiceSubscriptionFilterAttribute_IsMatch", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    void *vptr = 0;
    int res = SWIG_ConvertPtr(argv[0], &vptr, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_ServiceSubscriptionFilterAttribute_IsMatch__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    void *vptr = 0;
    int res = SWIG_ConvertPtr(argv[0], &vptr, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_ServiceSubscriptionFilterAttribute_IsMatch__SWIG_2(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    void *vptr = 0;
    int res = SWIG_ConvertPtr(argv[0], &vptr, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_ServiceSubscriptionFilterAttribute_IsMatch__SWIG_3(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    void *vptr = 0;
    int res = SWIG_ConvertPtr(argv[0], &vptr, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_ServiceSubscriptionFilterAttribute_IsMatch__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'ServiceSubscriptionFilterAttribute_IsMatch'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::ServiceSubscriptionFilterAttribute::IsMatch(std::string const &) const\n"
    "    RobotRaconteur::ServiceSubscriptionFilterAttribute::IsMatch(std::string const &,std::string const &) const\n"
    "    RobotRaconteur::ServiceSubscriptionFilterAttribute::IsMatch(std::vector< std::string,std::allocator< std::string > > const &) const\n"
    "    RobotRaconteur::ServiceSubscriptionFilterAttribute::IsMatch(std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > const &) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_delete_ServiceSubscriptionFilterAttribute(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttribute *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttribute *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_ServiceSubscriptionFilterAttribute" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttribute *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttribute * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *ServiceSubscriptionFilterAttribute_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *ServiceSubscriptionFilterAttribute_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_CreateServiceSubscriptionFilterAttributeRegex__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  SwigValueWrapper< RobotRaconteur::ServiceSubscriptionFilterAttribute > result;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "CreateServiceSubscriptionFilterAttributeRegex" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CreateServiceSubscriptionFilterAttributeRegex" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    result = RobotRaconteur::CreateServiceSubscriptionFilterAttributeRegex((std::string const &)*arg1);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::ServiceSubscriptionFilterAttribute(static_cast< const RobotRaconteur::ServiceSubscriptionFilterAttribute& >(result))), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, SWIG_POINTER_OWN |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_CreateServiceSubscriptionFilterAttributeRegex__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  std::string *arg2 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  int res2 = SWIG_OLDOBJ ;
  SwigValueWrapper< RobotRaconteur::ServiceSubscriptionFilterAttribute > result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "CreateServiceSubscriptionFilterAttributeRegex" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CreateServiceSubscriptionFilterAttributeRegex" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "CreateServiceSubscriptionFilterAttributeRegex" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CreateServiceSubscriptionFilterAttributeRegex" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    result = RobotRaconteur::CreateServiceSubscriptionFilterAttributeRegex((std::string const &)*arg1,(std::string const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::ServiceSubscriptionFilterAttribute(static_cast< const RobotRaconteur::ServiceSubscriptionFilterAttribute& >(result))), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, SWIG_POINTER_OWN |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_CreateServiceSubscriptionFilterAttributeRegex(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "CreateServiceSubscriptionFilterAttributeRegex", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_AsPtr_std_string(argv[0], (std::string**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_CreateServiceSubscriptionFilterAttributeRegex__SWIG_0(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_AsPtr_std_string(argv[0], (std::string**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_CreateServiceSubscriptionFilterAttributeRegex__SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'CreateServiceSubscriptionFilterAttributeRegex'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::CreateServiceSubscriptionFilterAttributeRegex(std::string const &)\n"
    "    RobotRaconteur::CreateServiceSubscriptionFilterAttributeRegex(std::string const &,std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_CreateServiceSubscriptionFilterAttributeIdentifier__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  std::string *arg2 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  int res2 = SWIG_OLDOBJ ;
  SwigValueWrapper< RobotRaconteur::ServiceSubscriptionFilterAttribute > result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "CreateServiceSubscriptionFilterAttributeIdentifier" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CreateServiceSubscriptionFilterAttributeIdentifier" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "CreateServiceSubscriptionFilterAttributeIdentifier" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CreateServiceSubscriptionFilterAttributeIdentifier" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    result = RobotRaconteur::CreateServiceSubscriptionFilterAttributeIdentifier((std::string const &)*arg1,(std::string const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::ServiceSubscriptionFilterAttribute(static_cast< const RobotRaconteur::ServiceSubscriptionFilterAttribute& >(result))), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, SWIG_POINTER_OWN |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_CreateServiceSubscriptionFilterAttributeIdentifier__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  SwigValueWrapper< RobotRaconteur::ServiceSubscriptionFilterAttribute > result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "CreateServiceSubscriptionFilterAttributeIdentifier" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CreateServiceSubscriptionFilterAttributeIdentifier" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "CreateServiceSubscriptionFilterAttributeIdentifier" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CreateServiceSubscriptionFilterAttributeIdentifier" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "CreateServiceSubscriptionFilterAttributeIdentifier" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CreateServiceSubscriptionFilterAttributeIdentifier" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    result = RobotRaconteur::CreateServiceSubscriptionFilterAttributeIdentifier((std::string const &)*arg1,(std::string const &)*arg2,(std::string const &)*arg3);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::ServiceSubscriptionFilterAttribute(static_cast< const RobotRaconteur::ServiceSubscriptionFilterAttribute& >(result))), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, SWIG_POINTER_OWN |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_CreateServiceSubscriptionFilterAttributeIdentifier(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "CreateServiceSubscriptionFilterAttributeIdentifier", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_AsPtr_std_string(argv[0], (std::string**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_CreateServiceSubscriptionFilterAttributeIdentifier__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_AsPtr_std_string(argv[0], (std::string**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_CreateServiceSubscriptionFilterAttributeIdentifier__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'CreateServiceSubscriptionFilterAttributeIdentifier'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::CreateServiceSubscriptionFilterAttributeIdentifier(std::string const &,std::string const &)\n"
    "    RobotRaconteur::CreateServiceSubscriptionFilterAttributeIdentifier(std::string const &,std::string const &,std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_CreateServiceSubscriptionFilterAttributeCombinedIdentifier(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< RobotRaconteur::ServiceSubscriptionFilterAttribute > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "CreateServiceSubscriptionFilterAttributeCombinedIdentifier" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CreateServiceSubscriptionFilterAttributeCombinedIdentifier" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    result = RobotRaconteur::CreateServiceSubscriptionFilterAttributeCombinedIdentifier((std::string const &)*arg1);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::ServiceSubscriptionFilterAttribute(static_cast< const RobotRaconteur::ServiceSubscriptionFilterAttribute& >(result))), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, SWIG_POINTER_OWN |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_iterator" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute___nonzero__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____nonzero__((std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute___bool__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____bool__((std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute___len__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____len__((std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute___getslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type arg2 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_subscriptionattribute___getslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute___getslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_subscriptionattribute___getslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_subscriptionattribute___getslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > *)std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____getslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute___setslice____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type arg2 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute___setslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_subscriptionattribute___setslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_subscriptionattribute___setslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____setslice____SWIG_0(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute___setslice____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type arg2 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type arg3 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute___setslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_subscriptionattribute___setslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_subscriptionattribute___setslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type >(val3);
  {
    std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > *ptr = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vector_subscriptionattribute___setslice__" "', argument " "4"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_subscriptionattribute___setslice__" "', argument " "4"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____setslice____SWIG_1(arg1,arg2,arg3,(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > const &)*arg4);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute___setslice__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_subscriptionattribute___setslice__", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_vector_subscriptionattribute___setslice____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        {
          int res = SWIG_AsVal_ptrdiff_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = swig::asptr(argv[3], (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vector_subscriptionattribute___setslice____SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_subscriptionattribute___setslice__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::__setslice__(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type)\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::__setslice__(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute___delslice__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type arg2 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  ptrdiff_t val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_subscriptionattribute___delslice__", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute___delslice__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_subscriptionattribute___delslice__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type >(val2);
  ecode3 = SWIG_AsVal_ptrdiff_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_subscriptionattribute___delslice__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type >(val3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____delslice__(arg1,arg2,arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute___delitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute___delitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_subscriptionattribute___delitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____delitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute___getitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute___getitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_subscriptionattribute___getitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > *)std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____getitem____SWIG_0(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_subscriptionattribute___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  {
    std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > *ptr = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_subscriptionattribute___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_subscriptionattribute___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____setitem____SWIG_0(arg1,arg2,(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_subscriptionattribute___setitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____setitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute___delitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  PySliceObject *arg2 = (PySliceObject *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute___delitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  {
    if (!PySlice_Check(swig_obj[1])) {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_subscriptionattribute___delitem__" "', argument " "2"" of type '" "PySliceObject *""'");
    }
    arg2 = (PySliceObject *) swig_obj[1];
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____delitem____SWIG_1(arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    } catch(std::invalid_argument &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute___delitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_subscriptionattribute___delitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_subscriptionattribute___delitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_subscriptionattribute___delitem____SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_subscriptionattribute___delitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::__delitem__(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type)\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::__delitem__(PySliceObject *)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute___getitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute___getitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_subscriptionattribute___getitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type >(val2);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type *) &std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____getitem____SWIG_1((std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *)arg1,arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute___getitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_subscriptionattribute___getitem__", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_subscriptionattribute___getitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_subscriptionattribute___getitem____SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_subscriptionattribute___getitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::__getitem__(PySliceObject *)\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::__getitem__(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute___setitem____SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type arg2 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  ptrdiff_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute___setitem__" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  ecode2 = SWIG_AsVal_ptrdiff_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_subscriptionattribute___setitem__" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_subscriptionattribute___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_subscriptionattribute___setitem__" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type * >(argp3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____setitem____SWIG_2(arg1,arg2,(RobotRaconteur::ServiceSubscriptionFilterAttribute const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_subscriptionattribute___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        return _wrap_vector_subscriptionattribute___setitem____SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        _v = PySlice_Check(argv[1]);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vector_subscriptionattribute___setitem____SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_ptrdiff_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vector_subscriptionattribute___setitem____SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_subscriptionattribute___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::__setitem__(PySliceObject *,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > const &)\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::__setitem__(PySliceObject *)\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::__setitem__(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::difference_type,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_pop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< RobotRaconteur::ServiceSubscriptionFilterAttribute > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_pop" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__pop(arg1);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type(static_cast< const std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type& >(result))), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_append(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_subscriptionattribute_append", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_append" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vector_subscriptionattribute_append" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_subscriptionattribute_append" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__append(arg1,(RobotRaconteur::ServiceSubscriptionFilterAttribute const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_subscriptionattribute__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *)new std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_subscriptionattribute__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > *ptr = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_vector_subscriptionattribute" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vector_subscriptionattribute" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *)new std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >((std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_empty" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_size" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_subscriptionattribute_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_swap" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vector_subscriptionattribute_swap" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_subscriptionattribute_swap" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_begin" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_end" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_rbegin" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_rend" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_clear" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_get_allocator" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::allocator_type(static_cast< const std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_subscriptionattribute__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type arg1 ;
  size_t val1 ;
  int ecode1 = 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vector_subscriptionattribute" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type""'");
  } 
  arg1 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *)new std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_pop_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_pop_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->pop_back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_resize__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_resize" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_subscriptionattribute_resize" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_erase" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_subscriptionattribute_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_subscriptionattribute_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__erase__SWIG_0(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator arg2 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_erase" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_subscriptionattribute_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_subscriptionattribute_erase" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_subscriptionattribute_erase" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_subscriptionattribute_erase" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__erase__SWIG_1(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_subscriptionattribute_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_vector_subscriptionattribute_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_vector_subscriptionattribute_erase__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_subscriptionattribute_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::erase(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator)\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::erase(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_vector_subscriptionattribute__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type arg1 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type *arg2 = 0 ;
  size_t val1 ;
  int ecode1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_size_t(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_vector_subscriptionattribute" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type""'");
  } 
  arg1 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type >(val1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_vector_subscriptionattribute" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_vector_subscriptionattribute" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *)new std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >(arg1,(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_vector_subscriptionattribute(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_vector_subscriptionattribute", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_vector_subscriptionattribute__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_vector_subscriptionattribute__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_vector_subscriptionattribute__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_size_t(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, SWIG_POINTER_NO_NULL | 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_vector_subscriptionattribute__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_vector_subscriptionattribute'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::vector()\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::vector(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const &)\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::vector(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type)\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::vector(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_push_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_subscriptionattribute_push_back", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_push_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "vector_subscriptionattribute_push_back" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_subscriptionattribute_push_back" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->push_back((std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_front(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_front" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type *) &((std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *)arg1)->front();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_back(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_back" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type *) &((std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *)arg1)->back();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, 0 |  0 );
  (void)swig::container_owner<swig::traits<std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type>::category>::back_reference(resultobj, swig_obj[0]);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_assign(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type arg2 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_subscriptionattribute_assign", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_assign" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_subscriptionattribute_assign" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_subscriptionattribute_assign" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_subscriptionattribute_assign" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->assign(arg2,(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_resize__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type arg2 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_resize" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_subscriptionattribute_resize" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_subscriptionattribute_resize" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_subscriptionattribute_resize" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->resize(arg2,(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_resize(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_subscriptionattribute_resize", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_vector_subscriptionattribute_resize__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vector_subscriptionattribute_resize__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_subscriptionattribute_resize'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::resize(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type)\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::resize(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_insert__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator arg2 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_insert" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_subscriptionattribute_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_subscriptionattribute_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "vector_subscriptionattribute_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_subscriptionattribute_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__insert__SWIG_0(arg1,arg2,(RobotRaconteur::ServiceSubscriptionFilterAttribute const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_insert__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator arg2 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type arg3 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  size_t val3 ;
  int ecode3 = 0 ;
  void *argp4 = 0 ;
  int res4 = 0 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_insert" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_subscriptionattribute_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "vector_subscriptionattribute_insert" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator""'");
    }
  }
  ecode3 = SWIG_AsVal_size_t(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "vector_subscriptionattribute_insert" "', argument " "3"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type""'");
  } 
  arg3 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type >(val3);
  res4 = SWIG_ConvertPtr(swig_obj[3], &argp4, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute,  0  | 0);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "vector_subscriptionattribute_insert" "', argument " "4"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &""'"); 
  }
  if (!argp4) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "vector_subscriptionattribute_insert" "', argument " "4"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &""'"); 
  }
  arg4 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type * >(argp4);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_vector_Sl_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__insert__SWIG_1(arg1,arg2,arg3,(RobotRaconteur::ServiceSubscriptionFilterAttribute const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_insert(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "vector_subscriptionattribute_insert", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *>(iter) != 0));
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_vector_subscriptionattribute_insert__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *>(iter) != 0));
      if (_v) {
        {
          int res = SWIG_AsVal_size_t(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, SWIG_POINTER_NO_NULL | 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_vector_subscriptionattribute_insert__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'vector_subscriptionattribute_insert'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::insert(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &)\n"
    "    std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::insert(std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_reserve(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "vector_subscriptionattribute_reserve", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_reserve" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "vector_subscriptionattribute_reserve" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type""'");
  } 
  arg2 = static_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->reserve(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_vector_subscriptionattribute_capacity(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "vector_subscriptionattribute_capacity" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > const *)arg1)->capacity();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_vector_subscriptionattribute(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_vector_subscriptionattribute" "', argument " "1"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *vector_subscriptionattribute_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *vector_subscriptionattribute_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_new_map_subscriptionattribute__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::less< std::string > *arg1 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_std__lessT_std__string_t,  0  | 0);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_map_subscriptionattribute" "', argument " "1"" of type '" "std::less< std::string > const &""'"); 
  }
  if (!argp1) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_map_subscriptionattribute" "', argument " "1"" of type '" "std::less< std::string > const &""'"); 
  }
  arg1 = reinterpret_cast< std::less< std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *)new std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >((std::less< std::string > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_iterator" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute___nonzero__" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____nonzero__((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute___bool__" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____bool__((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute___len__" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____len__((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute___getitem__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::mapped_type *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionattribute___getitem__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute___getitem__" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattribute___getitem__" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattribute___getitem__" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::mapped_type *) &std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____getitem__(arg1,(std::string const &)*arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, 0 |  0 );
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute___delitem__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionattribute___delitem__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute___delitem__" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattribute___delitem__" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattribute___delitem__" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____delitem__(arg1,(std::string const &)*arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_has_key(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionattribute_has_key", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_has_key" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattribute_has_key" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattribute_has_key" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__has_key((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const *)arg1,(std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_keys(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_keys" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__keys(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_values(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_values" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__values(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_items(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_items" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__items(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute___contains__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionattribute___contains__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute___contains__" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattribute___contains__" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattribute___contains__" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____contains__(arg1,(std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_key_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_key_iterator" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__key_iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_value_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_value_iterator" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__value_iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute___setitem__" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattribute___setitem__" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattribute___setitem__" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____setitem____SWIG_0(arg1,(std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type *arg2 = 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::mapped_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute___setitem__" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattribute___setitem__" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattribute___setitem__" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "map_subscriptionattribute___setitem__" "', argument " "3"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::mapped_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattribute___setitem__" "', argument " "3"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::mapped_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::mapped_type * >(argp3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg____setitem____SWIG_1(arg1,(std::string const &)*arg2,(RobotRaconteur::ServiceSubscriptionFilterAttribute const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "map_subscriptionattribute___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_map_subscriptionattribute___setitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttribute, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_map_subscriptionattribute___setitem____SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'map_subscriptionattribute___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::__setitem__(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &)\n"
    "    std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::__setitem__(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::mapped_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_asdict(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_asdict" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__asdict(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_map_subscriptionattribute__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *)new std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_map_subscriptionattribute__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > > *ptr = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_map_subscriptionattribute" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_map_subscriptionattribute" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *)new std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_map_subscriptionattribute(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[2] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_map_subscriptionattribute", 0, 1, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_map_subscriptionattribute__SWIG_1(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_std__lessT_std__string_t, SWIG_POINTER_NO_NULL | 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_map_subscriptionattribute__SWIG_0(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_map_subscriptionattribute__SWIG_2(self, argc, argv);
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_map_subscriptionattribute'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::map(std::less< std::string > const &)\n"
    "    std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::map()\n"
    "    std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::map(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_empty" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_size" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionattribute_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_swap" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattribute_swap" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattribute_swap" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > &""'"); 
  }
  arg2 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_begin" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_end" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_rbegin" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_rend" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_clear" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_get_allocator" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::allocator_type(static_cast< const std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_erase" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattribute_erase" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattribute_erase" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->erase((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_count(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::size_type result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionattribute_count", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_count" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattribute_count" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattribute_count" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > const *)arg1)->count((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_erase" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionattribute_erase" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionattribute_erase" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__erase__SWIG_1(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_erase__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator arg2 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_erase" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionattribute_erase" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionattribute_erase" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionattribute_erase" "', argument " "3"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionattribute_erase" "', argument " "3"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttribute_Sg__erase__SWIG_2(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "map_subscriptionattribute_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_map_subscriptionattribute_erase__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_map_subscriptionattribute_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_map_subscriptionattribute_erase__SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'map_subscriptionattribute_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::erase(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &)\n"
    "    std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::erase(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator)\n"
    "    std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::erase(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_find(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionattribute_find", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_find" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattribute_find" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattribute_find" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->find((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_lower_bound(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionattribute_lower_bound", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_lower_bound" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattribute_lower_bound" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattribute_lower_bound" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->lower_bound((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattribute_upper_bound(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionattribute_upper_bound", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattribute_upper_bound" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattribute_upper_bound" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattribute_upper_bound" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->upper_bound((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_map_subscriptionattribute(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_map_subscriptionattribute" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *map_subscriptionattribute_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *map_subscriptionattribute_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttributeGroup_Attributes_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > *arg2 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceSubscriptionFilterAttributeGroup_Attributes_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceSubscriptionFilterAttributeGroup_Attributes_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceSubscriptionFilterAttributeGroup_Attributes_set" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Attributes = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttributeGroup_Attributes_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceSubscriptionFilterAttributeGroup_Attributes_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > *)& ((arg1)->Attributes);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttributeGroup_Groups_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *) 0 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > *arg2 = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceSubscriptionFilterAttributeGroup_Groups_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceSubscriptionFilterAttributeGroup_Groups_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceSubscriptionFilterAttributeGroup_Groups_set" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Groups = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttributeGroup_Groups_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceSubscriptionFilterAttributeGroup_Groups_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > *)& ((arg1)->Groups);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttributeGroup_Operation_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *) 0 ;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceSubscriptionFilterAttributeGroup_Operation_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceSubscriptionFilterAttributeGroup_Operation_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup * >(argp1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ServiceSubscriptionFilterAttributeGroup_Operation_set" "', argument " "2"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation""'");
  } 
  arg2 = static_cast< RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Operation = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttributeGroup_Operation_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceSubscriptionFilterAttributeGroup_Operation_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation) ((arg1)->Operation);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttributeGroup_SplitStringAttribute_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceSubscriptionFilterAttributeGroup_SplitStringAttribute_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceSubscriptionFilterAttributeGroup_SplitStringAttribute_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup * >(argp1);
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ServiceSubscriptionFilterAttributeGroup_SplitStringAttribute_set" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->SplitStringAttribute = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttributeGroup_SplitStringAttribute_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceSubscriptionFilterAttributeGroup_SplitStringAttribute_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (bool) ((arg1)->SplitStringAttribute);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttributeGroup_SplitStringDelimiter_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *) 0 ;
  char arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  char val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ServiceSubscriptionFilterAttributeGroup_SplitStringDelimiter_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceSubscriptionFilterAttributeGroup_SplitStringDelimiter_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup * >(argp1);
  ecode2 = SWIG_AsVal_char(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ServiceSubscriptionFilterAttributeGroup_SplitStringDelimiter_set" "', argument " "2"" of type '" "char""'");
  } 
  arg2 = static_cast< char >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->SplitStringDelimiter = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttributeGroup_SplitStringDelimiter_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  char result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceSubscriptionFilterAttributeGroup_SplitStringDelimiter_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (char) ((arg1)->SplitStringDelimiter);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_char(static_cast< char >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ServiceSubscriptionFilterAttributeGroup__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *)new RobotRaconteur::ServiceSubscriptionFilterAttributeGroup();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ServiceSubscriptionFilterAttributeGroup__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation arg1 ;
  int val1 ;
  int ecode1 = 0 ;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_int(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_ServiceSubscriptionFilterAttributeGroup" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation""'");
  } 
  arg1 = static_cast< RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *)new RobotRaconteur::ServiceSubscriptionFilterAttributeGroup(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ServiceSubscriptionFilterAttributeGroup__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation arg1 ;
  std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > arg2 ;
  int val1 ;
  int ecode1 = 0 ;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_int(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_ServiceSubscriptionFilterAttributeGroup" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation""'");
  } 
  arg1 = static_cast< RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation >(val1);
  {
    std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > *ptr = (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > *)0;
    int res = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res) || !ptr) {
      SWIG_exception_fail(SWIG_ArgError((ptr ? res : SWIG_TypeError)), "in method '" "new_ServiceSubscriptionFilterAttributeGroup" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >""'"); 
    }
    arg2 = *ptr;
    if (SWIG_IsNewObj(res)) delete ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *)new RobotRaconteur::ServiceSubscriptionFilterAttributeGroup(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ServiceSubscriptionFilterAttributeGroup__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation arg1 ;
  SwigValueWrapper< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > > arg2 ;
  int val1 ;
  int ecode1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_int(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_ServiceSubscriptionFilterAttributeGroup" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation""'");
  } 
  arg1 = static_cast< RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation >(val1);
  {
    res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t,  0  | 0);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_ServiceSubscriptionFilterAttributeGroup" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > >""'"); 
    }  
    if (!argp2) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_ServiceSubscriptionFilterAttributeGroup" "', argument " "2"" of type '" "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > >""'");
    } else {
      std::vector< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > * temp = reinterpret_cast< std::vector< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > * >(argp2);
      arg2 = *temp;
      if (SWIG_IsNewObj(res2)) delete temp;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *)new RobotRaconteur::ServiceSubscriptionFilterAttributeGroup(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ServiceSubscriptionFilterAttributeGroup(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_ServiceSubscriptionFilterAttributeGroup", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_ServiceSubscriptionFilterAttributeGroup__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_int(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_ServiceSubscriptionFilterAttributeGroup__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_int(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t, SWIG_POINTER_NO_NULL | 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_ServiceSubscriptionFilterAttributeGroup__SWIG_3(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_int(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_ServiceSubscriptionFilterAttributeGroup__SWIG_2(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_ServiceSubscriptionFilterAttributeGroup'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::ServiceSubscriptionFilterAttributeGroup::ServiceSubscriptionFilterAttributeGroup()\n"
    "    RobotRaconteur::ServiceSubscriptionFilterAttributeGroup::ServiceSubscriptionFilterAttributeGroup(RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation)\n"
    "    RobotRaconteur::ServiceSubscriptionFilterAttributeGroup::ServiceSubscriptionFilterAttributeGroup(RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > >)\n"
    "    RobotRaconteur::ServiceSubscriptionFilterAttributeGroup::ServiceSubscriptionFilterAttributeGroup(RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation,std::vector< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > >)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttributeGroup_IsMatch__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  bool result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceSubscriptionFilterAttributeGroup_IsMatch" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttributeGroup const *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceSubscriptionFilterAttributeGroup_IsMatch" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceSubscriptionFilterAttributeGroup_IsMatch" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((RobotRaconteur::ServiceSubscriptionFilterAttributeGroup const *)arg1)->IsMatch((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttributeGroup_IsMatch__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  bool result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceSubscriptionFilterAttributeGroup_IsMatch" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttributeGroup const *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup * >(argp1);
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceSubscriptionFilterAttributeGroup_IsMatch" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceSubscriptionFilterAttributeGroup_IsMatch" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((RobotRaconteur::ServiceSubscriptionFilterAttributeGroup const *)arg1)->IsMatch((std::vector< std::string,std::allocator< std::string > > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttributeGroup_IsMatch__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *) 0 ;
  std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  bool result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ServiceSubscriptionFilterAttributeGroup_IsMatch" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttributeGroup const *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup * >(argp1);
  {
    std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > *ptr = (std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "ServiceSubscriptionFilterAttributeGroup_IsMatch" "', argument " "2"" of type '" "std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ServiceSubscriptionFilterAttributeGroup_IsMatch" "', argument " "2"" of type '" "std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((RobotRaconteur::ServiceSubscriptionFilterAttributeGroup const *)arg1)->IsMatch((std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_ServiceSubscriptionFilterAttributeGroup_IsMatch(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "ServiceSubscriptionFilterAttributeGroup_IsMatch", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    void *vptr = 0;
    int res = SWIG_ConvertPtr(argv[0], &vptr, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_ServiceSubscriptionFilterAttributeGroup_IsMatch__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    void *vptr = 0;
    int res = SWIG_ConvertPtr(argv[0], &vptr, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_ServiceSubscriptionFilterAttributeGroup_IsMatch__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    void *vptr = 0;
    int res = SWIG_ConvertPtr(argv[0], &vptr, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_ServiceSubscriptionFilterAttributeGroup_IsMatch__SWIG_2(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'ServiceSubscriptionFilterAttributeGroup_IsMatch'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::ServiceSubscriptionFilterAttributeGroup::IsMatch(std::string const &) const\n"
    "    RobotRaconteur::ServiceSubscriptionFilterAttributeGroup::IsMatch(std::vector< std::string,std::allocator< std::string > > const &) const\n"
    "    RobotRaconteur::ServiceSubscriptionFilterAttributeGroup::IsMatch(std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > const &) const\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_delete_ServiceSubscriptionFilterAttributeGroup(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *arg1 = (RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_ServiceSubscriptionFilterAttributeGroup" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *ServiceSubscriptionFilterAttributeGroup_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *ServiceSubscriptionFilterAttributeGroup_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_new_map_subscriptionattributegroup__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::less< std::string > *arg1 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_std__lessT_std__string_t,  0  | 0);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_map_subscriptionattributegroup" "', argument " "1"" of type '" "std::less< std::string > const &""'"); 
  }
  if (!argp1) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_map_subscriptionattributegroup" "', argument " "1"" of type '" "std::less< std::string > const &""'"); 
  }
  arg1 = reinterpret_cast< std::less< std::string > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *)new std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >((std::less< std::string > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_iterator" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg__iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup___nonzero__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup___nonzero__" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg____nonzero__((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup___bool__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup___bool__" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg____bool__((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup___len__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup___len__" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg____len__((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const *)arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup___getitem__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::mapped_type *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionattributegroup___getitem__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup___getitem__" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattributegroup___getitem__" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattributegroup___getitem__" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        result = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::mapped_type *) &std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg____getitem__(arg1,(std::string const &)*arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, 0 |  0 );
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup___delitem__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionattributegroup___delitem__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup___delitem__" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattributegroup___delitem__" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattributegroup___delitem__" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg____delitem__(arg1,(std::string const &)*arg2);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_has_key(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionattributegroup_has_key", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_has_key" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattributegroup_has_key" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattributegroup_has_key" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg__has_key((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const *)arg1,(std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_keys(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_keys" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg__keys(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_values(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_values" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg__values(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_items(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_items" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg__items(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup___contains__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionattributegroup___contains__", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup___contains__" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattributegroup___contains__" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattributegroup___contains__" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg____contains__(arg1,(std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_key_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_key_iterator" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg__key_iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_value_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  PyObject **arg2 = (PyObject **) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  swig::SwigPyIterator *result = 0 ;
  
  arg2 = &swig_obj[0];
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_value_iterator" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (swig::SwigPyIterator *)std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg__value_iterator(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_swig__SwigPyIterator, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup___setitem____SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup___setitem__" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattributegroup___setitem__" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattributegroup___setitem__" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg____setitem____SWIG_0(arg1,(std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup___setitem____SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type *arg2 = 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::mapped_type *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup___setitem__" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattributegroup___setitem__" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattributegroup___setitem__" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "map_subscriptionattributegroup___setitem__" "', argument " "3"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::mapped_type const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattributegroup___setitem__" "', argument " "3"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::mapped_type const &""'"); 
  }
  arg3 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::mapped_type * >(argp3);
  
  try {
    try {
      {
        SWIG_PYTHON_THREAD_BEGIN_ALLOW;
        std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg____setitem____SWIG_1(arg1,(std::string const &)*arg2,(RobotRaconteur::ServiceSubscriptionFilterAttributeGroup const &)*arg3);
        SWIG_PYTHON_THREAD_END_ALLOW;
      }
    } catch(std::out_of_range &_e) {
      throw _e;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup___setitem__(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "map_subscriptionattributegroup___setitem__", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_map_subscriptionattributegroup___setitem____SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, SWIG_POINTER_NO_NULL | 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_map_subscriptionattributegroup___setitem____SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'map_subscriptionattributegroup___setitem__'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::__setitem__(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &)\n"
    "    std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::__setitem__(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::mapped_type const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_asdict(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_asdict" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (PyObject *)std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg__asdict(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_map_subscriptionattributegroup__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *)new std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_map_subscriptionattributegroup__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > > *ptr = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_map_subscriptionattributegroup" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_map_subscriptionattributegroup" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *)new std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_map_subscriptionattributegroup(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[2] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_map_subscriptionattributegroup", 0, 1, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_map_subscriptionattributegroup__SWIG_1(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_std__lessT_std__string_t, SWIG_POINTER_NO_NULL | 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_map_subscriptionattributegroup__SWIG_0(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_map_subscriptionattributegroup__SWIG_2(self, argc, argv);
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_map_subscriptionattributegroup'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::map(std::less< std::string > const &)\n"
    "    std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::map()\n"
    "    std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::map(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_empty(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_empty" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const *)arg1)->empty();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_size(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::size_type result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_size" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const *)arg1)->size();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_swap(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionattributegroup_swap", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_swap" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattributegroup_swap" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattributegroup_swap" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > &""'"); 
  }
  arg2 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->swap(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_begin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_begin" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->begin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_end(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_end" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->end();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_rbegin(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_rbegin" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rbegin();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_rend(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::reverse_iterator result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_rend" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->rend();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::reverse_iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_clear(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_clear" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->clear();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_get_allocator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  SwigValueWrapper< std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_get_allocator" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const *)arg1)->get_allocator();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::allocator_type(static_cast< const std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::allocator_type& >(result))), SWIGTYPE_p_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_erase__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::size_type result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_erase" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattributegroup_erase" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattributegroup_erase" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->erase((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_count(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::size_type result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionattributegroup_count", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_count" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattributegroup_count" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattributegroup_count" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = ((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > const *)arg1)->count((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_erase__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_erase" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionattributegroup_erase" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionattributegroup_erase" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg__erase__SWIG_1(arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_erase__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator arg2 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  swig::SwigPyIterator *iter2 = 0 ;
  int res2 ;
  swig::SwigPyIterator *iter3 = 0 ;
  int res3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_erase" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], SWIG_as_voidptrptr(&iter2), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res2) || !iter2) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionattributegroup_erase" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator > *>(iter2);
    if (iter_t) {
      arg2 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionattributegroup_erase" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator""'");
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], SWIG_as_voidptrptr(&iter3), swig::SwigPyIterator::descriptor(), 0);
  if (!SWIG_IsOK(res3) || !iter3) {
    SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionattributegroup_erase" "', argument " "3"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator""'");
  } else {
    swig::SwigPyIterator_T<std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator > *iter_t = dynamic_cast<swig::SwigPyIterator_T<std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator > *>(iter3);
    if (iter_t) {
      arg3 = iter_t->get_current();
    } else {
      SWIG_exception_fail(SWIG_ArgError(SWIG_TypeError), "in method '" "map_subscriptionattributegroup_erase" "', argument " "3"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator""'");
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      std_map_Sl_std_string_Sc_RobotRaconteur_ServiceSubscriptionFilterAttributeGroup_Sg__erase__SWIG_2(arg1,arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_erase(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "map_subscriptionattributegroup_erase", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator > *>(iter) != 0));
      if (_v) {
        return _wrap_map_subscriptionattributegroup_erase__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_map_subscriptionattributegroup_erase__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = swig::asptr(argv[0], (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      swig::SwigPyIterator *iter = 0;
      int res = SWIG_ConvertPtr(argv[1], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
      _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator > *>(iter) != 0));
      if (_v) {
        swig::SwigPyIterator *iter = 0;
        int res = SWIG_ConvertPtr(argv[2], SWIG_as_voidptrptr(&iter), swig::SwigPyIterator::descriptor(), 0);
        _v = (SWIG_IsOK(res) && iter && (dynamic_cast<swig::SwigPyIterator_T<std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator > *>(iter) != 0));
        if (_v) {
          return _wrap_map_subscriptionattributegroup_erase__SWIG_2(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'map_subscriptionattributegroup_erase'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::erase(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &)\n"
    "    std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::erase(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator)\n"
    "    std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::erase(std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator,std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_find(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionattributegroup_find", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_find" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattributegroup_find" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattributegroup_find" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->find((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_lower_bound(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionattributegroup_lower_bound", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_lower_bound" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattributegroup_lower_bound" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattributegroup_lower_bound" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->lower_bound((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_map_subscriptionattributegroup_upper_bound(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator result;
  
  if (!SWIG_Python_UnpackTuple(args, "map_subscriptionattributegroup_upper_bound", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "map_subscriptionattributegroup_upper_bound" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "map_subscriptionattributegroup_upper_bound" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "map_subscriptionattributegroup_upper_bound" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->upper_bound((std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::key_type const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(swig::make_output_iterator(static_cast< const std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::iterator & >(result)),
    swig::SwigPyIterator::descriptor(),SWIG_POINTER_OWN);
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_map_subscriptionattributegroup(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *arg1 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_map_subscriptionattributegroup" "', argument " "1"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *""'"); 
  }
  arg1 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *map_subscriptionattributegroup_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *map_subscriptionattributegroup_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionFilterPredicateDirector_Predicate(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilterPredicateDirector *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilterPredicateDirector *) 0 ;
  RobotRaconteur::ServiceInfo2Wrapped *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionFilterPredicateDirector_Predicate", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionFilterPredicateDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionFilterPredicateDirector_Predicate" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilterPredicateDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionFilterPredicateDirector * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionFilterPredicateDirector_Predicate" "', argument " "2"" of type '" "RobotRaconteur::ServiceInfo2Wrapped const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionFilterPredicateDirector_Predicate" "', argument " "2"" of type '" "RobotRaconteur::ServiceInfo2Wrapped const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::ServiceInfo2Wrapped * >(argp2);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedServiceSubscriptionFilterPredicateDirector::Predicate");
      } else {
        result = (bool)(arg1)->Predicate((RobotRaconteur::ServiceInfo2Wrapped const &)*arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedServiceSubscriptionFilterPredicateDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilterPredicateDirector *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilterPredicateDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionFilterPredicateDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedServiceSubscriptionFilterPredicateDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilterPredicateDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionFilterPredicateDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedServiceSubscriptionFilterPredicateDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedServiceSubscriptionFilterPredicateDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedServiceSubscriptionFilterPredicateDirector *)new SwigDirector_WrappedServiceSubscriptionFilterPredicateDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionFilterPredicateDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedServiceSubscriptionFilterPredicateDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilterPredicateDirector *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilterPredicateDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionFilterPredicateDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedServiceSubscriptionFilterPredicateDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilterPredicateDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionFilterPredicateDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedServiceSubscriptionFilterPredicateDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionFilterPredicateDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedServiceSubscriptionFilterPredicateDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionFilterNode_NodeID_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilterNode *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilterNode *) 0 ;
  ::RobotRaconteur::NodeID *arg2 = (::RobotRaconteur::NodeID *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionFilterNode_NodeID_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionFilterNode_NodeID_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilterNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilterNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilterNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__NodeID, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionFilterNode_NodeID_set" "', argument " "2"" of type '" "::RobotRaconteur::NodeID *""'"); 
  }
  arg2 = reinterpret_cast< ::RobotRaconteur::NodeID * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->NodeID = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionFilterNode_NodeID_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilterNode *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilterNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  ::RobotRaconteur::NodeID *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionFilterNode_NodeID_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilterNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilterNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilterNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (::RobotRaconteur::NodeID *)& ((arg1)->NodeID);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__NodeID, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionFilterNode_NodeName_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilterNode *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilterNode *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionFilterNode_NodeName_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionFilterNode_NodeName_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilterNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilterNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilterNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionFilterNode_NodeName_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionFilterNode_NodeName_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->NodeName = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionFilterNode_NodeName_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilterNode *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilterNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionFilterNode_NodeName_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilterNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilterNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilterNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->NodeName);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionFilterNode_Username_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilterNode *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilterNode *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionFilterNode_Username_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionFilterNode_Username_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilterNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilterNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilterNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionFilterNode_Username_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionFilterNode_Username_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Username = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionFilterNode_Username_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilterNode *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilterNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionFilterNode_Username_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilterNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilterNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilterNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->Username);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionFilterNode_Credentials_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilterNode *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilterNode *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionFilterNode_Credentials_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionFilterNode_Credentials_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilterNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilterNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilterNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionFilterNode_Credentials_set" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Credentials = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionFilterNode_Credentials_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilterNode *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilterNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MessageElementData > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionFilterNode_Credentials_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilterNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilterNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilterNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::intrusive_ptr< RobotRaconteur::MessageElementData > *) & ((arg1)->Credentials);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  
  //if (*result) {
  //  intrusive_ptr_add_ref(result->get());
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >(result->get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElementData >());
  //} else {
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData > **)&resultobj = 0;
  //}
  
  if(*(result)) {
    intrusive_ptr_add_ref(result->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElementData >(result->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElementData >());
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedServiceSubscriptionFilterNode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilterNode *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_WrappedServiceSubscriptionFilterNode", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedServiceSubscriptionFilterNode *)new RobotRaconteur::WrappedServiceSubscriptionFilterNode();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedServiceSubscriptionFilterNode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilterNode *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilterNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedServiceSubscriptionFilterNode" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilterNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilterNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilterNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilterNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedServiceSubscriptionFilterNode_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedServiceSubscriptionFilterNode_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionFilter_Nodes_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilter *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilter *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > *arg2 = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionFilter_Nodes_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionFilter_Nodes_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilter *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionFilter_Nodes_set" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Nodes = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionFilter_Nodes_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilter *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilter *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionFilter_Nodes_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilter *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > *)& ((arg1)->Nodes);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionFilter_ServiceNames_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilter *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilter *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = (std::vector< std::string,std::allocator< std::string > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionFilter_ServiceNames_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionFilter_ServiceNames_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilter *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionFilter_ServiceNames_set" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< std::string,std::allocator< std::string > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ServiceNames = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionFilter_ServiceNames_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilter *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilter *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionFilter_ServiceNames_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilter *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< std::string,std::allocator< std::string > > *)& ((arg1)->ServiceNames);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionFilter_TransportSchemes_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilter *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilter *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = (std::vector< std::string,std::allocator< std::string > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionFilter_TransportSchemes_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionFilter_TransportSchemes_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilter *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionFilter_TransportSchemes_set" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< std::string,std::allocator< std::string > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->TransportSchemes = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionFilter_TransportSchemes_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilter *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilter *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionFilter_TransportSchemes_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilter *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< std::string,std::allocator< std::string > > *)& ((arg1)->TransportSchemes);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionFilter_Attributes_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilter *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilter *) 0 ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > > *arg2 = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionFilter_Attributes_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionFilter_Attributes_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilter *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionFilter_Attributes_set" "', argument " "2"" of type '" "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > > *""'"); 
  }
  arg2 = reinterpret_cast< std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Attributes = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionFilter_Attributes_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilter *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilter *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionFilter_Attributes_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilter *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > > *)& ((arg1)->Attributes);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionFilter_AttributesMatchOperation_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilter *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilter *) 0 ;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionFilter_AttributesMatchOperation_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionFilter_AttributesMatchOperation_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilter *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedServiceSubscriptionFilter_AttributesMatchOperation_set" "', argument " "2"" of type '" "RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation""'");
  } 
  arg2 = static_cast< RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->AttributesMatchOperation = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionFilter_AttributesMatchOperation_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilter *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilter *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionFilter_AttributesMatchOperation_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilter *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation) ((arg1)->AttributesMatchOperation);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionFilter_SetRRPredicateDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilter *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilter *) 0 ;
  RobotRaconteur::WrappedServiceSubscriptionFilterPredicateDirector *arg2 = (RobotRaconteur::WrappedServiceSubscriptionFilterPredicateDirector *) 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionFilter_SetRRPredicateDirector", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionFilter_SetRRPredicateDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilter *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionFilterPredicateDirector, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionFilter_SetRRPredicateDirector" "', argument " "2"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilterPredicateDirector *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionFilterPredicateDirector * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedServiceSubscriptionFilter_SetRRPredicateDirector" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetRRPredicateDirector(arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionFilter_MaxConnections_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilter *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilter *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > *smartarg1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionFilter_MaxConnections_set", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionFilter_MaxConnections_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilter *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedServiceSubscriptionFilter_MaxConnections_set" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->MaxConnections = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionFilter_MaxConnections_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilter *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilter *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionFilter_MaxConnections_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilter *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint32_t) ((arg1)->MaxConnections);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedServiceSubscriptionFilter(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilter *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_WrappedServiceSubscriptionFilter", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedServiceSubscriptionFilter *)new RobotRaconteur::WrappedServiceSubscriptionFilter();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedServiceSubscriptionFilter(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionFilter *arg1 = (RobotRaconteur::WrappedServiceSubscriptionFilter *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedServiceSubscriptionFilter" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionFilter *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionFilter * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedServiceSubscriptionFilter_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedServiceSubscriptionFilter_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionClientID_NodeID_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionClientID *arg1 = (RobotRaconteur::ServiceSubscriptionClientID *) 0 ;
  ::RobotRaconteur::NodeID *arg2 = (::RobotRaconteur::NodeID *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionClientID_NodeID_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionClientID_NodeID_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionClientID *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionClientID * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__NodeID, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionClientID_NodeID_set" "', argument " "2"" of type '" "::RobotRaconteur::NodeID *""'"); 
  }
  arg2 = reinterpret_cast< ::RobotRaconteur::NodeID * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->NodeID = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionClientID_NodeID_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionClientID *arg1 = (RobotRaconteur::ServiceSubscriptionClientID *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  ::RobotRaconteur::NodeID *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionClientID_NodeID_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionClientID *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionClientID * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (::RobotRaconteur::NodeID *)& ((arg1)->NodeID);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__NodeID, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionClientID_ServiceName_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionClientID *arg1 = (RobotRaconteur::ServiceSubscriptionClientID *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionClientID_ServiceName_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionClientID_ServiceName_set" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionClientID *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionClientID * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionClientID_ServiceName_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionClientID_ServiceName_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ServiceName = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionClientID_ServiceName_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionClientID *arg1 = (RobotRaconteur::ServiceSubscriptionClientID *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionClientID_ServiceName_get" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionClientID *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionClientID * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->ServiceName);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionClientID_eq(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionClientID *arg1 = (RobotRaconteur::ServiceSubscriptionClientID *) 0 ;
  RobotRaconteur::ServiceSubscriptionClientID *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionClientID_eq", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionClientID_eq" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionClientID const *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionClientID * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionClientID_eq" "', argument " "2"" of type '" "RobotRaconteur::ServiceSubscriptionClientID const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionClientID_eq" "', argument " "2"" of type '" "RobotRaconteur::ServiceSubscriptionClientID const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionClientID * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((RobotRaconteur::ServiceSubscriptionClientID const *)arg1)->operator ==((RobotRaconteur::ServiceSubscriptionClientID const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  PyErr_Clear();
  Py_INCREF(Py_NotImplemented);
  return Py_NotImplemented;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionClientID_ne(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionClientID *arg1 = (RobotRaconteur::ServiceSubscriptionClientID *) 0 ;
  RobotRaconteur::ServiceSubscriptionClientID *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionClientID_ne", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionClientID_ne" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionClientID const *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionClientID * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionClientID_ne" "', argument " "2"" of type '" "RobotRaconteur::ServiceSubscriptionClientID const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionClientID_ne" "', argument " "2"" of type '" "RobotRaconteur::ServiceSubscriptionClientID const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionClientID * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)((RobotRaconteur::ServiceSubscriptionClientID const *)arg1)->operator !=((RobotRaconteur::ServiceSubscriptionClientID const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  PyErr_Clear();
  Py_INCREF(Py_NotImplemented);
  return Py_NotImplemented;
}


SWIGINTERN PyObject *_wrap_delete_WrappedServiceSubscriptionClientID(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::ServiceSubscriptionClientID *arg1 = (RobotRaconteur::ServiceSubscriptionClientID *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedServiceSubscriptionClientID" "', argument " "1"" of type '" "RobotRaconteur::ServiceSubscriptionClientID *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionClientID * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedServiceSubscriptionClientID_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_WrappedServiceInfo2SubscriptionDirector_ServiceDetected(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceInfo2SubscriptionDirector *arg1 = (RobotRaconteur::WrappedServiceInfo2SubscriptionDirector *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > *arg2 = 0 ;
  RobotRaconteur::ServiceSubscriptionClientID *arg3 = 0 ;
  RobotRaconteur::ServiceInfo2Wrapped *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > tempshared2 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  void *argp4 = 0 ;
  int res4 = 0 ;
  PyObject *swig_obj[4] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceInfo2SubscriptionDirector_ServiceDetected", 4, 4, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceInfo2SubscriptionDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceInfo2SubscriptionDirector_ServiceDetected" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceInfo2SubscriptionDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceInfo2SubscriptionDirector * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceInfo2Subscription_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceInfo2SubscriptionDirector_ServiceDetected" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > * >(argp2) : &tempshared2;
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceInfo2SubscriptionDirector_ServiceDetected" "', argument " "3"" of type '" "RobotRaconteur::ServiceSubscriptionClientID const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceInfo2SubscriptionDirector_ServiceDetected" "', argument " "3"" of type '" "RobotRaconteur::ServiceSubscriptionClientID const &""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionClientID * >(argp3);
  res4 = SWIG_ConvertPtr(swig_obj[3], &argp4, SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped,  0  | 0);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedServiceInfo2SubscriptionDirector_ServiceDetected" "', argument " "4"" of type '" "RobotRaconteur::ServiceInfo2Wrapped const &""'"); 
  }
  if (!argp4) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceInfo2SubscriptionDirector_ServiceDetected" "', argument " "4"" of type '" "RobotRaconteur::ServiceInfo2Wrapped const &""'"); 
  }
  arg4 = reinterpret_cast< RobotRaconteur::ServiceInfo2Wrapped * >(argp4);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedServiceInfo2SubscriptionDirector::ServiceDetected");
      } else {
        (arg1)->ServiceDetected((boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > const &)*arg2,(RobotRaconteur::ServiceSubscriptionClientID const &)*arg3,(RobotRaconteur::ServiceInfo2Wrapped const &)*arg4);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceInfo2SubscriptionDirector_ServiceLost(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceInfo2SubscriptionDirector *arg1 = (RobotRaconteur::WrappedServiceInfo2SubscriptionDirector *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > *arg2 = 0 ;
  RobotRaconteur::ServiceSubscriptionClientID *arg3 = 0 ;
  RobotRaconteur::ServiceInfo2Wrapped *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > tempshared2 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  void *argp4 = 0 ;
  int res4 = 0 ;
  PyObject *swig_obj[4] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceInfo2SubscriptionDirector_ServiceLost", 4, 4, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceInfo2SubscriptionDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceInfo2SubscriptionDirector_ServiceLost" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceInfo2SubscriptionDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceInfo2SubscriptionDirector * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceInfo2Subscription_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceInfo2SubscriptionDirector_ServiceLost" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > * >(argp2) : &tempshared2;
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceInfo2SubscriptionDirector_ServiceLost" "', argument " "3"" of type '" "RobotRaconteur::ServiceSubscriptionClientID const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceInfo2SubscriptionDirector_ServiceLost" "', argument " "3"" of type '" "RobotRaconteur::ServiceSubscriptionClientID const &""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionClientID * >(argp3);
  res4 = SWIG_ConvertPtr(swig_obj[3], &argp4, SWIGTYPE_p_RobotRaconteur__ServiceInfo2Wrapped,  0  | 0);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedServiceInfo2SubscriptionDirector_ServiceLost" "', argument " "4"" of type '" "RobotRaconteur::ServiceInfo2Wrapped const &""'"); 
  }
  if (!argp4) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceInfo2SubscriptionDirector_ServiceLost" "', argument " "4"" of type '" "RobotRaconteur::ServiceInfo2Wrapped const &""'"); 
  }
  arg4 = reinterpret_cast< RobotRaconteur::ServiceInfo2Wrapped * >(argp4);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedServiceInfo2SubscriptionDirector::ServiceLost");
      } else {
        (arg1)->ServiceLost((boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > const &)*arg2,(RobotRaconteur::ServiceSubscriptionClientID const &)*arg3,(RobotRaconteur::ServiceInfo2Wrapped const &)*arg4);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedServiceInfo2SubscriptionDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceInfo2SubscriptionDirector *arg1 = (RobotRaconteur::WrappedServiceInfo2SubscriptionDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceInfo2SubscriptionDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedServiceInfo2SubscriptionDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceInfo2SubscriptionDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceInfo2SubscriptionDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedServiceInfo2SubscriptionDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedServiceInfo2SubscriptionDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedServiceInfo2SubscriptionDirector *)new SwigDirector_WrappedServiceInfo2SubscriptionDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedServiceInfo2SubscriptionDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedServiceInfo2SubscriptionDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceInfo2SubscriptionDirector *arg1 = (RobotRaconteur::WrappedServiceInfo2SubscriptionDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceInfo2SubscriptionDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedServiceInfo2SubscriptionDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceInfo2SubscriptionDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceInfo2SubscriptionDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedServiceInfo2SubscriptionDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedServiceInfo2SubscriptionDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedServiceInfo2SubscriptionDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedServiceInfo2Subscription_GetDetectedServiceInfo2(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceInfo2Subscription *arg1 = (RobotRaconteur::WrappedServiceInfo2Subscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceInfo2Subscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceInfo2Subscription_GetDetectedServiceInfo2" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceInfo2Subscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceInfo2Subscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceInfo2Subscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceInfo2Subscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceInfo2Subscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceInfo2Subscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetDetectedServiceInfo2();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceInfo2Subscription_Close(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceInfo2Subscription *arg1 = (RobotRaconteur::WrappedServiceInfo2Subscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceInfo2Subscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceInfo2Subscription_Close" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceInfo2Subscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceInfo2Subscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceInfo2Subscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceInfo2Subscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceInfo2Subscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceInfo2Subscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Close();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceInfo2Subscription_SetRRDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceInfo2Subscription *arg1 = (RobotRaconteur::WrappedServiceInfo2Subscription *) 0 ;
  RobotRaconteur::WrappedServiceInfo2SubscriptionDirector *arg2 = (RobotRaconteur::WrappedServiceInfo2SubscriptionDirector *) 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceInfo2Subscription_SetRRDirector", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceInfo2Subscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceInfo2Subscription_SetRRDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceInfo2Subscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceInfo2Subscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceInfo2Subscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceInfo2Subscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceInfo2Subscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceInfo2Subscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__WrappedServiceInfo2SubscriptionDirector, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceInfo2Subscription_SetRRDirector" "', argument " "2"" of type '" "RobotRaconteur::WrappedServiceInfo2SubscriptionDirector *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedServiceInfo2SubscriptionDirector * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedServiceInfo2Subscription_SetRRDirector" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetRRDirector(arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceInfo2Subscription_GetNode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceInfo2Subscription *arg1 = (RobotRaconteur::WrappedServiceInfo2Subscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceInfo2Subscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceInfo2Subscription_GetNode" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceInfo2Subscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceInfo2Subscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceInfo2Subscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceInfo2Subscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceInfo2Subscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceInfo2Subscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetNode();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedServiceInfo2Subscription(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceInfo2Subscription *arg1 = (RobotRaconteur::WrappedServiceInfo2Subscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceInfo2Subscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedServiceInfo2Subscription" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceInfo2Subscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceInfo2Subscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceInfo2Subscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceInfo2Subscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceInfo2Subscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceInfo2Subscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedServiceInfo2Subscription_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceInfo2Subscription_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionDirector_ClientConnected(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionDirector *arg1 = (RobotRaconteur::WrappedServiceSubscriptionDirector *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *arg2 = 0 ;
  RobotRaconteur::ServiceSubscriptionClientID *arg3 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared2 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  void *argp4 ;
  int res4 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared4 ;
  PyObject *swig_obj[4] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionDirector_ClientConnected", 4, 4, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionDirector_ClientConnected" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionDirector * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionDirector_ClientConnected" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > * >(argp2) : &tempshared2;
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSubscriptionDirector_ClientConnected" "', argument " "3"" of type '" "RobotRaconteur::ServiceSubscriptionClientID const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionDirector_ClientConnected" "', argument " "3"" of type '" "RobotRaconteur::ServiceSubscriptionClientID const &""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionClientID * >(argp3);
  {
    int newmem = 0;
    res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedServiceSubscriptionDirector_ClientConnected" "', argument " "4"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp4) tempshared4 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp4);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp4);
      arg4 = &tempshared4;
    } else {
      arg4 = (argp4) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp4) : &tempshared4;
    }
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedServiceSubscriptionDirector::ClientConnected");
      } else {
        (arg1)->ClientConnected((boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > const &)*arg2,(RobotRaconteur::ServiceSubscriptionClientID const &)*arg3,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg4);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionDirector_ClientDisconnected(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionDirector *arg1 = (RobotRaconteur::WrappedServiceSubscriptionDirector *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *arg2 = 0 ;
  RobotRaconteur::ServiceSubscriptionClientID *arg3 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared2 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  void *argp4 ;
  int res4 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared4 ;
  PyObject *swig_obj[4] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionDirector_ClientDisconnected", 4, 4, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionDirector_ClientDisconnected" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionDirector * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionDirector_ClientDisconnected" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > * >(argp2) : &tempshared2;
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSubscriptionDirector_ClientDisconnected" "', argument " "3"" of type '" "RobotRaconteur::ServiceSubscriptionClientID const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionDirector_ClientDisconnected" "', argument " "3"" of type '" "RobotRaconteur::ServiceSubscriptionClientID const &""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionClientID * >(argp3);
  {
    int newmem = 0;
    res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedServiceSubscriptionDirector_ClientDisconnected" "', argument " "4"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp4) tempshared4 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp4);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp4);
      arg4 = &tempshared4;
    } else {
      arg4 = (argp4) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp4) : &tempshared4;
    }
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedServiceSubscriptionDirector::ClientDisconnected");
      } else {
        (arg1)->ClientDisconnected((boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > const &)*arg2,(RobotRaconteur::ServiceSubscriptionClientID const &)*arg3,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg4);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionDirector_ClientConnectFailed(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionDirector *arg1 = (RobotRaconteur::WrappedServiceSubscriptionDirector *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *arg2 = 0 ;
  RobotRaconteur::ServiceSubscriptionClientID *arg3 = 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg4 = 0 ;
  RobotRaconteur::HandlerErrorInfo *arg5 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared2 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  int res4 = SWIG_OLDOBJ ;
  void *argp5 = 0 ;
  int res5 = 0 ;
  PyObject *swig_obj[5] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionDirector_ClientConnectFailed", 5, 5, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionDirector_ClientConnectFailed" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionDirector * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionDirector_ClientConnectFailed" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > * >(argp2) : &tempshared2;
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__ServiceSubscriptionClientID,  0  | 0);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSubscriptionDirector_ClientConnectFailed" "', argument " "3"" of type '" "RobotRaconteur::ServiceSubscriptionClientID const &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionDirector_ClientConnectFailed" "', argument " "3"" of type '" "RobotRaconteur::ServiceSubscriptionClientID const &""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::ServiceSubscriptionClientID * >(argp3);
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res4 = swig::asptr(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedServiceSubscriptionDirector_ClientConnectFailed" "', argument " "4"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionDirector_ClientConnectFailed" "', argument " "4"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg4 = ptr;
  }
  res5 = SWIG_ConvertPtr(swig_obj[4], &argp5, SWIGTYPE_p_RobotRaconteur__HandlerErrorInfo,  0 );
  if (!SWIG_IsOK(res5)) {
    SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "WrappedServiceSubscriptionDirector_ClientConnectFailed" "', argument " "5"" of type '" "RobotRaconteur::HandlerErrorInfo &""'"); 
  }
  if (!argp5) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionDirector_ClientConnectFailed" "', argument " "5"" of type '" "RobotRaconteur::HandlerErrorInfo &""'"); 
  }
  arg5 = reinterpret_cast< RobotRaconteur::HandlerErrorInfo * >(argp5);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedServiceSubscriptionDirector::ClientConnectFailed");
      } else {
        (arg1)->ClientConnectFailed((boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > const &)*arg2,(RobotRaconteur::ServiceSubscriptionClientID const &)*arg3,(std::vector< std::string,std::allocator< std::string > > const &)*arg4,*arg5);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedServiceSubscriptionDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionDirector *arg1 = (RobotRaconteur::WrappedServiceSubscriptionDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedServiceSubscriptionDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedServiceSubscriptionDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedServiceSubscriptionDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedServiceSubscriptionDirector *)new SwigDirector_WrappedServiceSubscriptionDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedServiceSubscriptionDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionDirector *arg1 = (RobotRaconteur::WrappedServiceSubscriptionDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedServiceSubscriptionDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedServiceSubscriptionDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedServiceSubscriptionDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_TryDefaultClientRes_res_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes *arg1 = (RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscription_TryDefaultClientRes_res_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscription_TryDefaultClientRes, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_TryDefaultClientRes_res_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes * >(argp1);
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedServiceSubscription_TryDefaultClientRes_res_set" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->res = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_TryDefaultClientRes_res_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes *arg1 = (RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscription_TryDefaultClientRes, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_TryDefaultClientRes_res_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (bool) ((arg1)->res);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_TryDefaultClientRes_client_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes *arg1 = (RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscription_TryDefaultClientRes_client_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscription_TryDefaultClientRes, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_TryDefaultClientRes_client_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscription_TryDefaultClientRes_client_set" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->client = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_TryDefaultClientRes_client_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes *arg1 = (RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscription_TryDefaultClientRes, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_TryDefaultClientRes_client_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *) & ((arg1)->client);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedServiceSubscription_TryDefaultClientRes(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_WrappedServiceSubscription_TryDefaultClientRes", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes *)new RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscription_TryDefaultClientRes, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedServiceSubscription_TryDefaultClientRes(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes *arg1 = (RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscription_TryDefaultClientRes, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedServiceSubscription_TryDefaultClientRes" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedServiceSubscription_TryDefaultClientRes_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscription_TryDefaultClientRes, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedServiceSubscription_TryDefaultClientRes_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_GetConnectedClients(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_GetConnectedClients" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetConnectedClients();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_Close(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_Close" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Close();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_ClaimClient(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscription_ClaimClient", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_ClaimClient" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscription_ClaimClient" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->ClaimClient((boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_ReleaseClient(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscription_ReleaseClient", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_ReleaseClient" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscription_ReleaseClient" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->ReleaseClient((boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_GetConnectRetryDelay(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_GetConnectRetryDelay" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (uint32_t)(arg1)->GetConnectRetryDelay();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_SetConnectRetryDelay(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscription_SetConnectRetryDelay", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_SetConnectRetryDelay" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedServiceSubscription_SetConnectRetryDelay" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetConnectRetryDelay(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_SubscribeWire(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  PyObject *swig_obj[3] ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscription_SubscribeWire", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_SubscribeWire" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscription_SubscribeWire" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_SubscribeWire" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSubscription_SubscribeWire" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_SubscribeWire" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->SubscribeWire((std::string const &)*arg2,(std::string const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_SubscribePipe__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  int32_t arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  int val4 ;
  int ecode4 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > result;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_SubscribePipe" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscription_SubscribePipe" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_SubscribePipe" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSubscription_SubscribePipe" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_SubscribePipe" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedServiceSubscription_SubscribePipe" "', argument " "4"" of type '" "int32_t""'");
  } 
  arg4 = static_cast< int32_t >(val4);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->SubscribePipe((std::string const &)*arg2,(std::string const &)*arg3,arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_SubscribePipe__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_SubscribePipe" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscription_SubscribePipe" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_SubscribePipe" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSubscription_SubscribePipe" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_SubscribePipe" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->SubscribePipe((std::string const &)*arg2,(std::string const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_SubscribePipe(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedServiceSubscription_SubscribePipe", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_WrappedServiceSubscription_SubscribePipe__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          {
            int res = SWIG_AsVal_int(argv[3], NULL);
            _v = SWIG_CheckState(res);
          }
          if (_v) {
            return _wrap_WrappedServiceSubscription_SubscribePipe__SWIG_0(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedServiceSubscription_SubscribePipe'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedServiceSubscription::SubscribePipe(std::string const &,std::string const &,int32_t)\n"
    "    RobotRaconteur::WrappedServiceSubscription::SubscribePipe(std::string const &,std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_GetDefaultClient(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_GetDefaultClient" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetDefaultClient();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_TryGetDefaultClient(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_TryGetDefaultClient" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->TryGetDefaultClient();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes(static_cast< const RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes& >(result))), SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscription_TryDefaultClientRes, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_GetDefaultClientWait__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_GetDefaultClientWait" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedServiceSubscription_GetDefaultClientWait" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetDefaultClientWait(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_GetDefaultClientWait__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > result;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_GetDefaultClientWait" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetDefaultClientWait();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_GetDefaultClientWait(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedServiceSubscription_GetDefaultClientWait", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_WrappedServiceSubscription_GetDefaultClientWait__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_WrappedServiceSubscription_GetDefaultClientWait__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedServiceSubscription_GetDefaultClientWait'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedServiceSubscription::GetDefaultClientWait(int32_t)\n"
    "    RobotRaconteur::WrappedServiceSubscription::GetDefaultClientWait()\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_TryGetDefaultClientWait__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_TryGetDefaultClientWait" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedServiceSubscription_TryGetDefaultClientWait" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->TryGetDefaultClientWait(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes(static_cast< const RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes& >(result))), SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscription_TryDefaultClientRes, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_TryGetDefaultClientWait__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes result;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_TryGetDefaultClientWait" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->TryGetDefaultClientWait();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes(static_cast< const RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes& >(result))), SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscription_TryDefaultClientRes, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_TryGetDefaultClientWait(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedServiceSubscription_TryGetDefaultClientWait", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_WrappedServiceSubscription_TryGetDefaultClientWait__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_WrappedServiceSubscription_TryGetDefaultClientWait__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedServiceSubscription_TryGetDefaultClientWait'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedServiceSubscription::TryGetDefaultClientWait(int32_t)\n"
    "    RobotRaconteur::WrappedServiceSubscription::TryGetDefaultClientWait()\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_AsyncGetDefaultClient(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  int32_t arg2 ;
  RobotRaconteur::AsyncStubReturnDirector *arg3 = (RobotRaconteur::AsyncStubReturnDirector *) 0 ;
  int32_t arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  int val4 ;
  int ecode4 = 0 ;
  PyObject *swig_obj[4] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscription_AsyncGetDefaultClient", 4, 4, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_AsyncGetDefaultClient" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedServiceSubscription_AsyncGetDefaultClient" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__AsyncStubReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSubscription_AsyncGetDefaultClient" "', argument " "3"" of type '" "RobotRaconteur::AsyncStubReturnDirector *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::AsyncStubReturnDirector * >(argp3);
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedServiceSubscription_AsyncGetDefaultClient" "', argument " "4"" of type '" "int32_t""'");
  } 
  arg4 = static_cast< int32_t >(val4);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->AsyncGetDefaultClient(arg2,arg3,arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_SetRRDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  RobotRaconteur::WrappedServiceSubscriptionDirector *arg2 = (RobotRaconteur::WrappedServiceSubscriptionDirector *) 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscription_SetRRDirector", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_SetRRDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionDirector, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscription_SetRRDirector" "', argument " "2"" of type '" "RobotRaconteur::WrappedServiceSubscriptionDirector *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionDirector * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedServiceSubscription_SetRRDirector" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetRRDirector(arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_GetNode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_GetNode" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetNode();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_GetServiceURL(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_GetServiceURL" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetServiceURL();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< std::string,std::allocator< std::string > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_UpdateServiceURL__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  std::string *arg3 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > arg4 ;
  std::string *arg5 = 0 ;
  bool arg6 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  void *argp4 ;
  int res4 = 0 ;
  int newmem4 = 0 ;
  int res5 = SWIG_OLDOBJ ;
  bool val6 ;
  int ecode6 = 0 ;
  
  if ((nobjs < 6) || (nobjs > 6)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[3];
  //if (smartarg) {
  //	arg4 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //}
  
  res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem4);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "4"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData >""'");
  }
  if (argp4) arg4 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4)->get(),true);
  if (newmem4 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4);
  
  {
    std::string *ptr = (std::string *)0;
    res5 = SWIG_AsPtr_std_string(swig_obj[4], &ptr);
    if (!SWIG_IsOK(res5)) {
      SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "5"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "5"" of type '" "std::string const &""'"); 
    }
    arg5 = ptr;
  }
  ecode6 = SWIG_AsVal_bool(swig_obj[5], &val6);
  if (!SWIG_IsOK(ecode6)) {
    SWIG_exception_fail(SWIG_ArgError(ecode6), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "6"" of type '" "bool""'");
  } 
  arg6 = static_cast< bool >(val6);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->UpdateServiceURL((std::vector< std::string,std::allocator< std::string > > const &)*arg2,(std::string const &)*arg3,arg4,(std::string const &)*arg5,arg6);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_UpdateServiceURL__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  std::string *arg3 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > arg4 ;
  std::string *arg5 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  void *argp4 ;
  int res4 = 0 ;
  int newmem4 = 0 ;
  int res5 = SWIG_OLDOBJ ;
  
  if ((nobjs < 5) || (nobjs > 5)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[3];
  //if (smartarg) {
  //	arg4 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //}
  
  res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem4);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "4"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData >""'");
  }
  if (argp4) arg4 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4)->get(),true);
  if (newmem4 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4);
  
  {
    std::string *ptr = (std::string *)0;
    res5 = SWIG_AsPtr_std_string(swig_obj[4], &ptr);
    if (!SWIG_IsOK(res5)) {
      SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "5"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "5"" of type '" "std::string const &""'"); 
    }
    arg5 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->UpdateServiceURL((std::vector< std::string,std::allocator< std::string > > const &)*arg2,(std::string const &)*arg3,arg4,(std::string const &)*arg5);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_UpdateServiceURL__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  std::string *arg3 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  void *argp4 ;
  int res4 = 0 ;
  int newmem4 = 0 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[3];
  //if (smartarg) {
  //	arg4 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //}
  
  res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem4);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "4"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData >""'");
  }
  if (argp4) arg4 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4)->get(),true);
  if (newmem4 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4);
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->UpdateServiceURL((std::vector< std::string,std::allocator< std::string > > const &)*arg2,(std::string const &)*arg3,arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_UpdateServiceURL__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  std::string *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->UpdateServiceURL((std::vector< std::string,std::allocator< std::string > > const &)*arg2,(std::string const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_UpdateServiceURL__SWIG_4(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->UpdateServiceURL((std::vector< std::string,std::allocator< std::string > > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_UpdateServiceURL__SWIG_5(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > arg4 ;
  std::string *arg5 = 0 ;
  bool arg6 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  void *argp4 ;
  int res4 = 0 ;
  int newmem4 = 0 ;
  int res5 = SWIG_OLDOBJ ;
  bool val6 ;
  int ecode6 = 0 ;
  
  if ((nobjs < 6) || (nobjs > 6)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[3];
  //if (smartarg) {
  //	arg4 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //}
  
  res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem4);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "4"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData >""'");
  }
  if (argp4) arg4 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4)->get(),true);
  if (newmem4 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4);
  
  {
    std::string *ptr = (std::string *)0;
    res5 = SWIG_AsPtr_std_string(swig_obj[4], &ptr);
    if (!SWIG_IsOK(res5)) {
      SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "5"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "5"" of type '" "std::string const &""'"); 
    }
    arg5 = ptr;
  }
  ecode6 = SWIG_AsVal_bool(swig_obj[5], &val6);
  if (!SWIG_IsOK(ecode6)) {
    SWIG_exception_fail(SWIG_ArgError(ecode6), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "6"" of type '" "bool""'");
  } 
  arg6 = static_cast< bool >(val6);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->UpdateServiceURL((std::string const &)*arg2,(std::string const &)*arg3,arg4,(std::string const &)*arg5,arg6);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_UpdateServiceURL__SWIG_6(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > arg4 ;
  std::string *arg5 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  void *argp4 ;
  int res4 = 0 ;
  int newmem4 = 0 ;
  int res5 = SWIG_OLDOBJ ;
  
  if ((nobjs < 5) || (nobjs > 5)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[3];
  //if (smartarg) {
  //	arg4 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //}
  
  res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem4);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "4"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData >""'");
  }
  if (argp4) arg4 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4)->get(),true);
  if (newmem4 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4);
  
  {
    std::string *ptr = (std::string *)0;
    res5 = SWIG_AsPtr_std_string(swig_obj[4], &ptr);
    if (!SWIG_IsOK(res5)) {
      SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "5"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "5"" of type '" "std::string const &""'"); 
    }
    arg5 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->UpdateServiceURL((std::string const &)*arg2,(std::string const &)*arg3,arg4,(std::string const &)*arg5);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_UpdateServiceURL__SWIG_7(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  void *argp4 ;
  int res4 = 0 ;
  int newmem4 = 0 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[3];
  //if (smartarg) {
  //	arg4 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //}
  
  res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem4);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "4"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData >""'");
  }
  if (argp4) arg4 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4)->get(),true);
  if (newmem4 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4);
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->UpdateServiceURL((std::string const &)*arg2,(std::string const &)*arg3,arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_UpdateServiceURL__SWIG_8(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->UpdateServiceURL((std::string const &)*arg2,(std::string const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_UpdateServiceURL__SWIG_9(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceURL" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->UpdateServiceURL((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_UpdateServiceURL(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[7] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedServiceSubscription_UpdateServiceURL", 0, 6, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_WrappedServiceSubscription_UpdateServiceURL__SWIG_9(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_WrappedServiceSubscription_UpdateServiceURL__SWIG_4(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_WrappedServiceSubscription_UpdateServiceURL__SWIG_8(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_WrappedServiceSubscription_UpdateServiceURL__SWIG_3(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_WrappedServiceSubscription_UpdateServiceURL__SWIG_7(self, argc, argv);
          }
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_WrappedServiceSubscription_UpdateServiceURL__SWIG_2(self, argc, argv);
          }
        }
      }
    }
  }
  if (argc == 5) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            int res = SWIG_AsPtr_std_string(argv[4], (std::string**)(0));
            _v = SWIG_CheckState(res);
            if (_v) {
              return _wrap_WrappedServiceSubscription_UpdateServiceURL__SWIG_6(self, argc, argv);
            }
          }
        }
      }
    }
  }
  if (argc == 5) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            int res = SWIG_AsPtr_std_string(argv[4], (std::string**)(0));
            _v = SWIG_CheckState(res);
            if (_v) {
              return _wrap_WrappedServiceSubscription_UpdateServiceURL__SWIG_1(self, argc, argv);
            }
          }
        }
      }
    }
  }
  if (argc == 6) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            int res = SWIG_AsPtr_std_string(argv[4], (std::string**)(0));
            _v = SWIG_CheckState(res);
            if (_v) {
              {
                int res = SWIG_AsVal_bool(argv[5], NULL);
                _v = SWIG_CheckState(res);
              }
              if (_v) {
                return _wrap_WrappedServiceSubscription_UpdateServiceURL__SWIG_5(self, argc, argv);
              }
            }
          }
        }
      }
    }
  }
  if (argc == 6) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            int res = SWIG_AsPtr_std_string(argv[4], (std::string**)(0));
            _v = SWIG_CheckState(res);
            if (_v) {
              {
                int res = SWIG_AsVal_bool(argv[5], NULL);
                _v = SWIG_CheckState(res);
              }
              if (_v) {
                return _wrap_WrappedServiceSubscription_UpdateServiceURL__SWIG_0(self, argc, argv);
              }
            }
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedServiceSubscription_UpdateServiceURL'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedServiceSubscription::UpdateServiceURL(std::vector< std::string,std::allocator< std::string > > const &,std::string const &,boost::intrusive_ptr< RobotRaconteur::MessageElementData >,std::string const &,bool)\n"
    "    RobotRaconteur::WrappedServiceSubscription::UpdateServiceURL(std::vector< std::string,std::allocator< std::string > > const &,std::string const &,boost::intrusive_ptr< RobotRaconteur::MessageElementData >,std::string const &)\n"
    "    RobotRaconteur::WrappedServiceSubscription::UpdateServiceURL(std::vector< std::string,std::allocator< std::string > > const &,std::string const &,boost::intrusive_ptr< RobotRaconteur::MessageElementData >)\n"
    "    RobotRaconteur::WrappedServiceSubscription::UpdateServiceURL(std::vector< std::string,std::allocator< std::string > > const &,std::string const &)\n"
    "    RobotRaconteur::WrappedServiceSubscription::UpdateServiceURL(std::vector< std::string,std::allocator< std::string > > const &)\n"
    "    RobotRaconteur::WrappedServiceSubscription::UpdateServiceURL(std::string const &,std::string const &,boost::intrusive_ptr< RobotRaconteur::MessageElementData >,std::string const &,bool)\n"
    "    RobotRaconteur::WrappedServiceSubscription::UpdateServiceURL(std::string const &,std::string const &,boost::intrusive_ptr< RobotRaconteur::MessageElementData >,std::string const &)\n"
    "    RobotRaconteur::WrappedServiceSubscription::UpdateServiceURL(std::string const &,std::string const &,boost::intrusive_ptr< RobotRaconteur::MessageElementData >)\n"
    "    RobotRaconteur::WrappedServiceSubscription::UpdateServiceURL(std::string const &,std::string const &)\n"
    "    RobotRaconteur::WrappedServiceSubscription::UpdateServiceURL(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_UpdateServiceByType__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 ;
  int res3 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > tempshared3 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_UpdateServiceByType" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscription_UpdateServiceByType" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceByType" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSubscription_UpdateServiceByType" "', argument " "3"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp3);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp3) : &tempshared3;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->UpdateServiceByType((std::vector< std::string,std::allocator< std::string > > const &)*arg2,(boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_UpdateServiceByType__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_UpdateServiceByType" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscription_UpdateServiceByType" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_UpdateServiceByType" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->UpdateServiceByType((std::vector< std::string,std::allocator< std::string > > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_UpdateServiceByType(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedServiceSubscription_UpdateServiceByType", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_WrappedServiceSubscription_UpdateServiceByType__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_WrappedServiceSubscription_UpdateServiceByType__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedServiceSubscription_UpdateServiceByType'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedServiceSubscription::UpdateServiceByType(std::vector< std::string,std::allocator< std::string > > const &,boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > const &)\n"
    "    RobotRaconteur::WrappedServiceSubscription::UpdateServiceByType(std::vector< std::string,std::allocator< std::string > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_SubscribeSubObject__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  boost::shared_ptr< RobotRaconteur::WrappedSubObjectSubscription > result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_SubscribeSubObject" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscription_SubscribeSubObject" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_SubscribeSubObject" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedServiceSubscription_SubscribeSubObject" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_SubscribeSubObject" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->SubscribeSubObject((std::string const &)*arg2,(std::string const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_SubscribeSubObject__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  boost::shared_ptr< RobotRaconteur::WrappedSubObjectSubscription > result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscription_SubscribeSubObject" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscription_SubscribeSubObject" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscription_SubscribeSubObject" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->SubscribeSubObject((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscription_SubscribeSubObject(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedServiceSubscription_SubscribeSubObject", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_WrappedServiceSubscription_SubscribeSubObject__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_WrappedServiceSubscription_SubscribeSubObject__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedServiceSubscription_SubscribeSubObject'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedServiceSubscription::SubscribeSubObject(std::string const &,std::string const &)\n"
    "    RobotRaconteur::WrappedServiceSubscription::SubscribeSubObject(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_delete_WrappedServiceSubscription(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscription *arg1 = (RobotRaconteur::WrappedServiceSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedServiceSubscription" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedServiceSubscription_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_WrappedWireSubscriptionDirector_WireValueChanged(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireSubscriptionDirector *arg1 = (RobotRaconteur::WrappedWireSubscriptionDirector *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > *arg2 = 0 ;
  RobotRaconteur::WrappedService_typed_packet *arg3 = 0 ;
  RobotRaconteur::TimeSpec *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > tempshared2 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  void *argp4 = 0 ;
  int res4 = 0 ;
  PyObject *swig_obj[4] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireSubscriptionDirector_WireValueChanged", 4, 4, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireSubscriptionDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireSubscriptionDirector_WireValueChanged" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireSubscriptionDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireSubscriptionDirector * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireSubscriptionDirector_WireValueChanged" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > * >(argp2) : &tempshared2;
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3, SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet,  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedWireSubscriptionDirector_WireValueChanged" "', argument " "3"" of type '" "RobotRaconteur::WrappedService_typed_packet &""'"); 
  }
  if (!argp3) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedWireSubscriptionDirector_WireValueChanged" "', argument " "3"" of type '" "RobotRaconteur::WrappedService_typed_packet &""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::WrappedService_typed_packet * >(argp3);
  res4 = SWIG_ConvertPtr(swig_obj[3], &argp4, SWIGTYPE_p_RobotRaconteur__TimeSpec,  0  | 0);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedWireSubscriptionDirector_WireValueChanged" "', argument " "4"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  if (!argp4) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedWireSubscriptionDirector_WireValueChanged" "', argument " "4"" of type '" "RobotRaconteur::TimeSpec const &""'"); 
  }
  arg4 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp4);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedWireSubscriptionDirector::WireValueChanged");
      } else {
        (arg1)->WireValueChanged((boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > const &)*arg2,*arg3,(RobotRaconteur::TimeSpec const &)*arg4);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedWireSubscriptionDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireSubscriptionDirector *arg1 = (RobotRaconteur::WrappedWireSubscriptionDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireSubscriptionDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedWireSubscriptionDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireSubscriptionDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireSubscriptionDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedWireSubscriptionDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedWireSubscriptionDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedWireSubscriptionDirector *)new SwigDirector_WrappedWireSubscriptionDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedWireSubscriptionDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedWireSubscriptionDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireSubscriptionDirector *arg1 = (RobotRaconteur::WrappedWireSubscriptionDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireSubscriptionDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedWireSubscriptionDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireSubscriptionDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireSubscriptionDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedWireSubscriptionDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedWireSubscriptionDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedWireSubscriptionDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedWireSubscription_GetInValue__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireSubscription *arg1 = (RobotRaconteur::WrappedWireSubscription *) 0 ;
  RobotRaconteur::TimeSpec *arg2 = (RobotRaconteur::TimeSpec *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  RobotRaconteur::WrappedService_typed_packet result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireSubscription_GetInValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__TimeSpec, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireSubscription_GetInValue" "', argument " "2"" of type '" "RobotRaconteur::TimeSpec *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetInValue(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::WrappedService_typed_packet(static_cast< const RobotRaconteur::WrappedService_typed_packet& >(result))), SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireSubscription_GetInValue__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireSubscription *arg1 = (RobotRaconteur::WrappedWireSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > *smartarg1 = 0 ;
  RobotRaconteur::WrappedService_typed_packet result;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireSubscription_GetInValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetInValue();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::WrappedService_typed_packet(static_cast< const RobotRaconteur::WrappedService_typed_packet& >(result))), SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireSubscription_GetInValue(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedWireSubscription_GetInValue", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_WrappedWireSubscription_GetInValue__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      void *vptr = 0;
      int res = SWIG_ConvertPtr(argv[1], &vptr, SWIGTYPE_p_RobotRaconteur__TimeSpec, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_WrappedWireSubscription_GetInValue__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedWireSubscription_GetInValue'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedWireSubscription::GetInValue(RobotRaconteur::TimeSpec *)\n"
    "    RobotRaconteur::WrappedWireSubscription::GetInValue()\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedWireSubscription_TryGetInValue__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireSubscription *arg1 = (RobotRaconteur::WrappedWireSubscription *) 0 ;
  RobotRaconteur::WrappedService_typed_packet *arg2 = 0 ;
  RobotRaconteur::TimeSpec *arg3 = (RobotRaconteur::TimeSpec *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  bool result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireSubscription_TryGetInValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireSubscription_TryGetInValue" "', argument " "2"" of type '" "RobotRaconteur::WrappedService_typed_packet &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedWireSubscription_TryGetInValue" "', argument " "2"" of type '" "RobotRaconteur::WrappedService_typed_packet &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedService_typed_packet * >(argp2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__TimeSpec, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedWireSubscription_TryGetInValue" "', argument " "3"" of type '" "RobotRaconteur::TimeSpec *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::TimeSpec * >(argp3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->TryGetInValue(*arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireSubscription_TryGetInValue__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireSubscription *arg1 = (RobotRaconteur::WrappedWireSubscription *) 0 ;
  RobotRaconteur::WrappedService_typed_packet *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  bool result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireSubscription_TryGetInValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireSubscription_TryGetInValue" "', argument " "2"" of type '" "RobotRaconteur::WrappedService_typed_packet &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedWireSubscription_TryGetInValue" "', argument " "2"" of type '" "RobotRaconteur::WrappedService_typed_packet &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedService_typed_packet * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->TryGetInValue(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireSubscription_TryGetInValue(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedWireSubscription_TryGetInValue", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      void *vptr = 0;
      int res = SWIG_ConvertPtr(argv[1], &vptr, SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet, SWIG_POINTER_NO_NULL);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_WrappedWireSubscription_TryGetInValue__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      void *vptr = 0;
      int res = SWIG_ConvertPtr(argv[1], &vptr, SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet, SWIG_POINTER_NO_NULL);
      _v = SWIG_CheckState(res);
      if (_v) {
        void *vptr = 0;
        int res = SWIG_ConvertPtr(argv[2], &vptr, SWIGTYPE_p_RobotRaconteur__TimeSpec, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_WrappedWireSubscription_TryGetInValue__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedWireSubscription_TryGetInValue'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedWireSubscription::TryGetInValue(RobotRaconteur::WrappedService_typed_packet &,RobotRaconteur::TimeSpec *)\n"
    "    RobotRaconteur::WrappedWireSubscription::TryGetInValue(RobotRaconteur::WrappedService_typed_packet &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedWireSubscription_WaitInValueValid__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireSubscription *arg1 = (RobotRaconteur::WrappedWireSubscription *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  bool result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireSubscription_WaitInValueValid" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedWireSubscription_WaitInValueValid" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->WaitInValueValid(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireSubscription_WaitInValueValid__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireSubscription *arg1 = (RobotRaconteur::WrappedWireSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > *smartarg1 = 0 ;
  bool result;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireSubscription_WaitInValueValid" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->WaitInValueValid();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireSubscription_WaitInValueValid(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedWireSubscription_WaitInValueValid", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_WrappedWireSubscription_WaitInValueValid__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_WrappedWireSubscription_WaitInValueValid__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedWireSubscription_WaitInValueValid'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedWireSubscription::WaitInValueValid(int32_t)\n"
    "    RobotRaconteur::WrappedWireSubscription::WaitInValueValid()\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedWireSubscription_GetIgnoreInValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireSubscription *arg1 = (RobotRaconteur::WrappedWireSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireSubscription_GetIgnoreInValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->GetIgnoreInValue();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireSubscription_SetIgnoreInValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireSubscription *arg1 = (RobotRaconteur::WrappedWireSubscription *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > *smartarg1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireSubscription_SetIgnoreInValue", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireSubscription_SetIgnoreInValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedWireSubscription_SetIgnoreInValue" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetIgnoreInValue(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireSubscription_GetInValueLifespan(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireSubscription *arg1 = (RobotRaconteur::WrappedWireSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireSubscription_GetInValueLifespan" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (int32_t)(arg1)->GetInValueLifespan();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireSubscription_SetInValueLifespan(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireSubscription *arg1 = (RobotRaconteur::WrappedWireSubscription *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireSubscription_SetInValueLifespan", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireSubscription_SetInValueLifespan" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedWireSubscription_SetInValueLifespan" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetInValueLifespan(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireSubscription_GetActiveWireConnectionCount(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireSubscription *arg1 = (RobotRaconteur::WrappedWireSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  size_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireSubscription_GetActiveWireConnectionCount" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetActiveWireConnectionCount();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireSubscription_Close(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireSubscription *arg1 = (RobotRaconteur::WrappedWireSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireSubscription_Close" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Close();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireSubscription_SetRRDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireSubscription *arg1 = (RobotRaconteur::WrappedWireSubscription *) 0 ;
  RobotRaconteur::WrappedWireSubscriptionDirector *arg2 = (RobotRaconteur::WrappedWireSubscriptionDirector *) 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireSubscription_SetRRDirector", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireSubscription_SetRRDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__WrappedWireSubscriptionDirector, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireSubscription_SetRRDirector" "', argument " "2"" of type '" "RobotRaconteur::WrappedWireSubscriptionDirector *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedWireSubscriptionDirector * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedWireSubscription_SetRRDirector" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetRRDirector(arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireSubscription_GetNode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireSubscription *arg1 = (RobotRaconteur::WrappedWireSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireSubscription_GetNode" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetNode();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedWireSubscription(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireSubscription *arg1 = (RobotRaconteur::WrappedWireSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedWireSubscription" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedWireSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedWireSubscription_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_new_WrappedWireSubscription_send_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > tempshared1 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedWireSubscription_send_iterator *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_WrappedWireSubscription_send_iterator" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedWireSubscription_send_iterator *)new RobotRaconteur::WrappedWireSubscription_send_iterator((boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedWireSubscription_send_iterator, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireSubscription_send_iterator_Next(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireSubscription_send_iterator *arg1 = (RobotRaconteur::WrappedWireSubscription_send_iterator *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::WrappedWireConnection > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireSubscription_send_iterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireSubscription_send_iterator_Next" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireSubscription_send_iterator *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireSubscription_send_iterator * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->Next();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedWireConnection > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedWireConnection >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireSubscription_send_iterator_SetOutValue(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireSubscription_send_iterator *arg1 = (RobotRaconteur::WrappedWireSubscription_send_iterator *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedWireSubscription_send_iterator_SetOutValue", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireSubscription_send_iterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireSubscription_send_iterator_SetOutValue" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireSubscription_send_iterator *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireSubscription_send_iterator * >(argp1);
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedWireSubscription_send_iterator_SetOutValue" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetOutValue((boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireSubscription_send_iterator_GetType(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireSubscription_send_iterator *arg1 = (RobotRaconteur::WrappedWireSubscription_send_iterator *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireSubscription_send_iterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireSubscription_send_iterator_GetType" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireSubscription_send_iterator *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireSubscription_send_iterator * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetType();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::TypeDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::TypeDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedWireSubscription_send_iterator_GetStub(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireSubscription_send_iterator *arg1 = (RobotRaconteur::WrappedWireSubscription_send_iterator *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireSubscription_send_iterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedWireSubscription_send_iterator_GetStub" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireSubscription_send_iterator *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireSubscription_send_iterator * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetStub();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedWireSubscription_send_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedWireSubscription_send_iterator *arg1 = (RobotRaconteur::WrappedWireSubscription_send_iterator *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedWireSubscription_send_iterator, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedWireSubscription_send_iterator" "', argument " "1"" of type '" "RobotRaconteur::WrappedWireSubscription_send_iterator *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedWireSubscription_send_iterator * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedWireSubscription_send_iterator_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedWireSubscription_send_iterator, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedWireSubscription_send_iterator_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedPipeSubscriptionDirector_PipePacketReceived(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeSubscriptionDirector *arg1 = (RobotRaconteur::WrappedPipeSubscriptionDirector *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > tempshared2 ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeSubscriptionDirector_PipePacketReceived", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPipeSubscriptionDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeSubscriptionDirector_PipePacketReceived" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeSubscriptionDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPipeSubscriptionDirector * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPipeSubscriptionDirector_PipePacketReceived" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > * >(argp2) : &tempshared2;
    }
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::WrappedPipeSubscriptionDirector::PipePacketReceived");
      } else {
        (arg1)->PipePacketReceived((boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > const &)*arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedPipeSubscriptionDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeSubscriptionDirector *arg1 = (RobotRaconteur::WrappedPipeSubscriptionDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPipeSubscriptionDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedPipeSubscriptionDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeSubscriptionDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPipeSubscriptionDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedPipeSubscriptionDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedPipeSubscriptionDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::WrappedPipeSubscriptionDirector *)new SwigDirector_WrappedPipeSubscriptionDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedPipeSubscriptionDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_WrappedPipeSubscriptionDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeSubscriptionDirector *arg1 = (RobotRaconteur::WrappedPipeSubscriptionDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPipeSubscriptionDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_WrappedPipeSubscriptionDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeSubscriptionDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPipeSubscriptionDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedPipeSubscriptionDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedPipeSubscriptionDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedPipeSubscriptionDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedPipeSubscription_ReceivePacket(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeSubscription *arg1 = (RobotRaconteur::WrappedPipeSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedService_typed_packet result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeSubscription_ReceivePacket" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->ReceivePacket();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::WrappedService_typed_packet(static_cast< const RobotRaconteur::WrappedService_typed_packet& >(result))), SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeSubscription_TryReceivePacket(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeSubscription *arg1 = (RobotRaconteur::WrappedPipeSubscription *) 0 ;
  RobotRaconteur::WrappedService_typed_packet *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeSubscription_TryReceivePacket", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeSubscription_TryReceivePacket" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPipeSubscription_TryReceivePacket" "', argument " "2"" of type '" "RobotRaconteur::WrappedService_typed_packet &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedPipeSubscription_TryReceivePacket" "', argument " "2"" of type '" "RobotRaconteur::WrappedService_typed_packet &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedService_typed_packet * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->TryReceivePacket(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeSubscription_TryReceivePacketWait__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeSubscription *arg1 = (RobotRaconteur::WrappedPipeSubscription *) 0 ;
  RobotRaconteur::WrappedService_typed_packet *arg2 = 0 ;
  int32_t arg3 ;
  bool arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  bool val4 ;
  int ecode4 = 0 ;
  bool result;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeSubscription_TryReceivePacketWait" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPipeSubscription_TryReceivePacketWait" "', argument " "2"" of type '" "RobotRaconteur::WrappedService_typed_packet &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedPipeSubscription_TryReceivePacketWait" "', argument " "2"" of type '" "RobotRaconteur::WrappedService_typed_packet &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedService_typed_packet * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedPipeSubscription_TryReceivePacketWait" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  ecode4 = SWIG_AsVal_bool(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedPipeSubscription_TryReceivePacketWait" "', argument " "4"" of type '" "bool""'");
  } 
  arg4 = static_cast< bool >(val4);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->TryReceivePacketWait(*arg2,arg3,arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeSubscription_TryReceivePacketWait__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeSubscription *arg1 = (RobotRaconteur::WrappedPipeSubscription *) 0 ;
  RobotRaconteur::WrappedService_typed_packet *arg2 = 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  bool result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeSubscription_TryReceivePacketWait" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPipeSubscription_TryReceivePacketWait" "', argument " "2"" of type '" "RobotRaconteur::WrappedService_typed_packet &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedPipeSubscription_TryReceivePacketWait" "', argument " "2"" of type '" "RobotRaconteur::WrappedService_typed_packet &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedService_typed_packet * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedPipeSubscription_TryReceivePacketWait" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->TryReceivePacketWait(*arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeSubscription_TryReceivePacketWait__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeSubscription *arg1 = (RobotRaconteur::WrappedPipeSubscription *) 0 ;
  RobotRaconteur::WrappedService_typed_packet *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  bool result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeSubscription_TryReceivePacketWait" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPipeSubscription_TryReceivePacketWait" "', argument " "2"" of type '" "RobotRaconteur::WrappedService_typed_packet &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedPipeSubscription_TryReceivePacketWait" "', argument " "2"" of type '" "RobotRaconteur::WrappedService_typed_packet &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedService_typed_packet * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->TryReceivePacketWait(*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeSubscription_TryReceivePacketWait(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedPipeSubscription_TryReceivePacketWait", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      void *vptr = 0;
      int res = SWIG_ConvertPtr(argv[1], &vptr, SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet, SWIG_POINTER_NO_NULL);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_WrappedPipeSubscription_TryReceivePacketWait__SWIG_2(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      void *vptr = 0;
      int res = SWIG_ConvertPtr(argv[1], &vptr, SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet, SWIG_POINTER_NO_NULL);
      _v = SWIG_CheckState(res);
      if (_v) {
        {
          int res = SWIG_AsVal_int(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_WrappedPipeSubscription_TryReceivePacketWait__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      void *vptr = 0;
      int res = SWIG_ConvertPtr(argv[1], &vptr, SWIGTYPE_p_RobotRaconteur__WrappedService_typed_packet, SWIG_POINTER_NO_NULL);
      _v = SWIG_CheckState(res);
      if (_v) {
        {
          int res = SWIG_AsVal_int(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          {
            int res = SWIG_AsVal_bool(argv[3], NULL);
            _v = SWIG_CheckState(res);
          }
          if (_v) {
            return _wrap_WrappedPipeSubscription_TryReceivePacketWait__SWIG_0(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedPipeSubscription_TryReceivePacketWait'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedPipeSubscription::TryReceivePacketWait(RobotRaconteur::WrappedService_typed_packet &,int32_t,bool)\n"
    "    RobotRaconteur::WrappedPipeSubscription::TryReceivePacketWait(RobotRaconteur::WrappedService_typed_packet &,int32_t)\n"
    "    RobotRaconteur::WrappedPipeSubscription::TryReceivePacketWait(RobotRaconteur::WrappedService_typed_packet &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedPipeSubscription_Available(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeSubscription *arg1 = (RobotRaconteur::WrappedPipeSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  size_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeSubscription_Available" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->Available();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeSubscription_GetActivePipeEndpointCount(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeSubscription *arg1 = (RobotRaconteur::WrappedPipeSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  size_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeSubscription_GetActivePipeEndpointCount" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetActivePipeEndpointCount();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_size_t(static_cast< size_t >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeSubscription_GetIgnoreReceived(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeSubscription *arg1 = (RobotRaconteur::WrappedPipeSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeSubscription_GetIgnoreReceived" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->GetIgnoreReceived();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeSubscription_SetIgnoreReceived(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeSubscription *arg1 = (RobotRaconteur::WrappedPipeSubscription *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > *smartarg1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeSubscription_SetIgnoreReceived", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeSubscription_SetIgnoreReceived" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedPipeSubscription_SetIgnoreReceived" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetIgnoreReceived(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeSubscription_Close(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeSubscription *arg1 = (RobotRaconteur::WrappedPipeSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeSubscription_Close" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Close();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeSubscription_SetRRDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeSubscription *arg1 = (RobotRaconteur::WrappedPipeSubscription *) 0 ;
  RobotRaconteur::WrappedPipeSubscriptionDirector *arg2 = (RobotRaconteur::WrappedPipeSubscriptionDirector *) 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeSubscription_SetRRDirector", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeSubscription_SetRRDirector" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__WrappedPipeSubscriptionDirector, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPipeSubscription_SetRRDirector" "', argument " "2"" of type '" "RobotRaconteur::WrappedPipeSubscriptionDirector *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedPipeSubscriptionDirector * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedPipeSubscription_SetRRDirector" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetRRDirector(arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeSubscription_GetNode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeSubscription *arg1 = (RobotRaconteur::WrappedPipeSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeSubscription_GetNode" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetNode();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedPipeSubscription(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeSubscription *arg1 = (RobotRaconteur::WrappedPipeSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedPipeSubscription" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedPipeSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedPipeSubscription_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_new_WrappedPipeSubscription_send_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > tempshared1 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedPipeSubscription_send_iterator *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_WrappedPipeSubscription_send_iterator" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedPipeSubscription_send_iterator *)new RobotRaconteur::WrappedPipeSubscription_send_iterator((boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedPipeSubscription_send_iterator, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeSubscription_send_iterator_Next(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeSubscription_send_iterator *arg1 = (RobotRaconteur::WrappedPipeSubscription_send_iterator *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPipeSubscription_send_iterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeSubscription_send_iterator_Next" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeSubscription_send_iterator *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPipeSubscription_send_iterator * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->Next();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedPipeEndpoint >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeSubscription_send_iterator_AsyncSendPacket(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeSubscription_send_iterator *arg1 = (RobotRaconteur::WrappedPipeSubscription_send_iterator *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedPipeSubscription_send_iterator_AsyncSendPacket", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPipeSubscription_send_iterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeSubscription_send_iterator_AsyncSendPacket" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeSubscription_send_iterator *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPipeSubscription_send_iterator * >(argp1);
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedPipeSubscription_send_iterator_AsyncSendPacket" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->AsyncSendPacket((boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeSubscription_send_iterator_GetType(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeSubscription_send_iterator *arg1 = (RobotRaconteur::WrappedPipeSubscription_send_iterator *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPipeSubscription_send_iterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeSubscription_send_iterator_GetType" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeSubscription_send_iterator *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPipeSubscription_send_iterator * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetType();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::TypeDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::TypeDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedPipeSubscription_send_iterator_GetStub(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeSubscription_send_iterator *arg1 = (RobotRaconteur::WrappedPipeSubscription_send_iterator *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPipeSubscription_send_iterator, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedPipeSubscription_send_iterator_GetStub" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeSubscription_send_iterator *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPipeSubscription_send_iterator * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetStub();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedPipeSubscription_send_iterator(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedPipeSubscription_send_iterator *arg1 = (RobotRaconteur::WrappedPipeSubscription_send_iterator *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedPipeSubscription_send_iterator, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedPipeSubscription_send_iterator" "', argument " "1"" of type '" "RobotRaconteur::WrappedPipeSubscription_send_iterator *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedPipeSubscription_send_iterator * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedPipeSubscription_send_iterator_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedPipeSubscription_send_iterator, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedPipeSubscription_send_iterator_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedSubObjectSubscription_GetDefaultClient(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedSubObjectSubscription *arg1 = (RobotRaconteur::WrappedSubObjectSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedSubObjectSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedSubObjectSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedSubObjectSubscription_GetDefaultClient" "', argument " "1"" of type '" "RobotRaconteur::WrappedSubObjectSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedSubObjectSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedSubObjectSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetDefaultClient();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedSubObjectSubscription_TryGetDefaultClient(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedSubObjectSubscription *arg1 = (RobotRaconteur::WrappedSubObjectSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedSubObjectSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedSubObjectSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedSubObjectSubscription_TryGetDefaultClient" "', argument " "1"" of type '" "RobotRaconteur::WrappedSubObjectSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedSubObjectSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedSubObjectSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->TryGetDefaultClient();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes(static_cast< const RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes& >(result))), SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscription_TryDefaultClientRes, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedSubObjectSubscription_GetDefaultClientWait__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedSubObjectSubscription *arg1 = (RobotRaconteur::WrappedSubObjectSubscription *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedSubObjectSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedSubObjectSubscription > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedSubObjectSubscription_GetDefaultClientWait" "', argument " "1"" of type '" "RobotRaconteur::WrappedSubObjectSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedSubObjectSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedSubObjectSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedSubObjectSubscription_GetDefaultClientWait" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetDefaultClientWait(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedSubObjectSubscription_GetDefaultClientWait__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedSubObjectSubscription *arg1 = (RobotRaconteur::WrappedSubObjectSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedSubObjectSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedSubObjectSubscription > *smartarg1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > result;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedSubObjectSubscription_GetDefaultClientWait" "', argument " "1"" of type '" "RobotRaconteur::WrappedSubObjectSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedSubObjectSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedSubObjectSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetDefaultClientWait();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedSubObjectSubscription_GetDefaultClientWait(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedSubObjectSubscription_GetDefaultClientWait", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_WrappedSubObjectSubscription_GetDefaultClientWait__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_WrappedSubObjectSubscription_GetDefaultClientWait__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedSubObjectSubscription_GetDefaultClientWait'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedSubObjectSubscription::GetDefaultClientWait(int32_t)\n"
    "    RobotRaconteur::WrappedSubObjectSubscription::GetDefaultClientWait()\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedSubObjectSubscription_TryGetDefaultClientWait__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedSubObjectSubscription *arg1 = (RobotRaconteur::WrappedSubObjectSubscription *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedSubObjectSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedSubObjectSubscription > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedSubObjectSubscription_TryGetDefaultClientWait" "', argument " "1"" of type '" "RobotRaconteur::WrappedSubObjectSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedSubObjectSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedSubObjectSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedSubObjectSubscription_TryGetDefaultClientWait" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->TryGetDefaultClientWait(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes(static_cast< const RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes& >(result))), SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscription_TryDefaultClientRes, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedSubObjectSubscription_TryGetDefaultClientWait__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedSubObjectSubscription *arg1 = (RobotRaconteur::WrappedSubObjectSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedSubObjectSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedSubObjectSubscription > *smartarg1 = 0 ;
  RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes result;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedSubObjectSubscription_TryGetDefaultClientWait" "', argument " "1"" of type '" "RobotRaconteur::WrappedSubObjectSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedSubObjectSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedSubObjectSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->TryGetDefaultClientWait();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes(static_cast< const RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes& >(result))), SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscription_TryDefaultClientRes, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedSubObjectSubscription_TryGetDefaultClientWait(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedSubObjectSubscription_TryGetDefaultClientWait", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_WrappedSubObjectSubscription_TryGetDefaultClientWait__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_WrappedSubObjectSubscription_TryGetDefaultClientWait__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedSubObjectSubscription_TryGetDefaultClientWait'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedSubObjectSubscription::TryGetDefaultClientWait(int32_t)\n"
    "    RobotRaconteur::WrappedSubObjectSubscription::TryGetDefaultClientWait()\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedSubObjectSubscription_AsyncGetDefaultClient(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedSubObjectSubscription *arg1 = (RobotRaconteur::WrappedSubObjectSubscription *) 0 ;
  int32_t arg2 ;
  RobotRaconteur::AsyncStubReturnDirector *arg3 = (RobotRaconteur::AsyncStubReturnDirector *) 0 ;
  int32_t arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedSubObjectSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedSubObjectSubscription > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  int val4 ;
  int ecode4 = 0 ;
  PyObject *swig_obj[4] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedSubObjectSubscription_AsyncGetDefaultClient", 4, 4, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedSubObjectSubscription_AsyncGetDefaultClient" "', argument " "1"" of type '" "RobotRaconteur::WrappedSubObjectSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedSubObjectSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedSubObjectSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedSubObjectSubscription_AsyncGetDefaultClient" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__AsyncStubReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedSubObjectSubscription_AsyncGetDefaultClient" "', argument " "3"" of type '" "RobotRaconteur::AsyncStubReturnDirector *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::AsyncStubReturnDirector * >(argp3);
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "WrappedSubObjectSubscription_AsyncGetDefaultClient" "', argument " "4"" of type '" "int32_t""'");
  } 
  arg4 = static_cast< int32_t >(val4);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->AsyncGetDefaultClient(arg2,arg3,arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedSubObjectSubscription_Close(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedSubObjectSubscription *arg1 = (RobotRaconteur::WrappedSubObjectSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedSubObjectSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedSubObjectSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedSubObjectSubscription_Close" "', argument " "1"" of type '" "RobotRaconteur::WrappedSubObjectSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedSubObjectSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedSubObjectSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Close();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedSubObjectSubscription_GetNode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedSubObjectSubscription *arg1 = (RobotRaconteur::WrappedSubObjectSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedSubObjectSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedSubObjectSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedSubObjectSubscription_GetNode" "', argument " "1"" of type '" "RobotRaconteur::WrappedSubObjectSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedSubObjectSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedSubObjectSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetNode();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedSubObjectSubscription(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedSubObjectSubscription *arg1 = (RobotRaconteur::WrappedSubObjectSubscription *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedSubObjectSubscription > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedSubObjectSubscription > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedSubObjectSubscription" "', argument " "1"" of type '" "RobotRaconteur::WrappedSubObjectSubscription *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedSubObjectSubscription * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedSubObjectSubscription > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedSubObjectSubscription * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedSubObjectSubscription_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_WrappedServiceInfo2SubscriptionServicesToVector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > > *arg1 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t,  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceInfo2SubscriptionServicesToVector" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > > &""'"); 
  }
  if (!argp1) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceInfo2SubscriptionServicesToVector" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > > &""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > > * >(argp1);
  
  try {
    result = RobotRaconteur::WrappedServiceInfo2SubscriptionServicesToVector(*arg1);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedSubscribeServiceInfo2__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > arg3 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 ;
  int res3 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedSubscribeServiceInfo2" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedSubscribeServiceInfo2" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedSubscribeServiceInfo2" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedSubscribeServiceInfo2" "', argument " "3"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter >""'");
    }
    if (argp3) arg3 = *(reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp3));
    if (newmem & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp3);
  }
  
  try {
    result = RobotRaconteur::WrappedSubscribeServiceInfo2((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(std::vector< std::string,std::allocator< std::string > > const &)*arg2,arg3);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceInfo2Subscription > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceInfo2Subscription >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceInfo2Subscription_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedSubscribeServiceInfo2__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  int res2 = SWIG_OLDOBJ ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedSubscribeServiceInfo2" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedSubscribeServiceInfo2" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedSubscribeServiceInfo2" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    result = RobotRaconteur::WrappedSubscribeServiceInfo2((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(std::vector< std::string,std::allocator< std::string > > const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceInfo2Subscription > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceInfo2Subscription >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceInfo2Subscription_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedSubscribeServiceInfo2(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedSubscribeServiceInfo2", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_WrappedSubscribeServiceInfo2__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_WrappedSubscribeServiceInfo2__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedSubscribeServiceInfo2'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedSubscribeServiceInfo2(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &,std::vector< std::string,std::allocator< std::string > > const &,boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter >)\n"
    "    RobotRaconteur::WrappedSubscribeServiceInfo2(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &,std::vector< std::string,std::allocator< std::string > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionClientsToVector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > > *arg1 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t,  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionClientsToVector" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > > &""'"); 
  }
  if (!argp1) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionClientsToVector" "', argument " "1"" of type '" "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > > &""'"); 
  }
  arg1 = reinterpret_cast< std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > > * >(argp1);
  
  try {
    result = RobotRaconteur::WrappedServiceSubscriptionClientsToVector(*arg1);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedSubscribeServiceByType__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > arg3 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  int res2 = SWIG_OLDOBJ ;
  void *argp3 ;
  int res3 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedSubscribeServiceByType" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedSubscribeServiceByType" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedSubscribeServiceByType" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedSubscribeServiceByType" "', argument " "3"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter >""'");
    }
    if (argp3) arg3 = *(reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp3));
    if (newmem & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp3);
  }
  
  try {
    result = RobotRaconteur::WrappedSubscribeServiceByType((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(std::vector< std::string,std::allocator< std::string > > const &)*arg2,arg3);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedSubscribeServiceByType__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  int res2 = SWIG_OLDOBJ ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedSubscribeServiceByType" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedSubscribeServiceByType" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedSubscribeServiceByType" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    result = RobotRaconteur::WrappedSubscribeServiceByType((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(std::vector< std::string,std::allocator< std::string > > const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedSubscribeServiceByType(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedSubscribeServiceByType", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_WrappedSubscribeServiceByType__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_WrappedSubscribeServiceByType__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedSubscribeServiceByType'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedSubscribeServiceByType(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &,std::vector< std::string,std::allocator< std::string > > const &,boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter >)\n"
    "    RobotRaconteur::WrappedSubscribeServiceByType(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &,std::vector< std::string,std::allocator< std::string > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedSubscribeService__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  std::string *arg3 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > arg4 ;
  std::string *arg5 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  void *argp4 ;
  int res4 = 0 ;
  int newmem4 = 0 ;
  int res5 = SWIG_OLDOBJ ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > result;
  
  if ((nobjs < 5) || (nobjs > 5)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedSubscribeService" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedSubscribeService" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedSubscribeService" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedSubscribeService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedSubscribeService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[3];
  //if (smartarg) {
  //	arg4 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //}
  
  res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem4);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedSubscribeService" "', argument " "4"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData >""'");
  }
  if (argp4) arg4 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4)->get(),true);
  if (newmem4 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4);
  
  {
    std::string *ptr = (std::string *)0;
    res5 = SWIG_AsPtr_std_string(swig_obj[4], &ptr);
    if (!SWIG_IsOK(res5)) {
      SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "WrappedSubscribeService" "', argument " "5"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedSubscribeService" "', argument " "5"" of type '" "std::string const &""'"); 
    }
    arg5 = ptr;
  }
  
  try {
    result = RobotRaconteur::WrappedSubscribeService((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(std::vector< std::string,std::allocator< std::string > > const &)*arg2,(std::string const &)*arg3,arg4,(std::string const &)*arg5);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedSubscribeService__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  std::string *arg3 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > arg4 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  void *argp4 ;
  int res4 = 0 ;
  int newmem4 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > result;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedSubscribeService" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedSubscribeService" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedSubscribeService" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedSubscribeService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedSubscribeService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[3];
  //if (smartarg) {
  //	arg4 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //}
  
  res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem4);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedSubscribeService" "', argument " "4"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData >""'");
  }
  if (argp4) arg4 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4)->get(),true);
  if (newmem4 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4);
  
  
  try {
    result = RobotRaconteur::WrappedSubscribeService((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(std::vector< std::string,std::allocator< std::string > > const &)*arg2,(std::string const &)*arg3,arg4);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedSubscribeService__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  std::string *arg3 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedSubscribeService" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedSubscribeService" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedSubscribeService" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedSubscribeService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedSubscribeService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    result = RobotRaconteur::WrappedSubscribeService((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(std::vector< std::string,std::allocator< std::string > > const &)*arg2,(std::string const &)*arg3);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedSubscribeService__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  int res2 = SWIG_OLDOBJ ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedSubscribeService" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedSubscribeService" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedSubscribeService" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    result = RobotRaconteur::WrappedSubscribeService((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(std::vector< std::string,std::allocator< std::string > > const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedSubscribeService__SWIG_4(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > arg4 ;
  std::string *arg5 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  void *argp4 ;
  int res4 = 0 ;
  int newmem4 = 0 ;
  int res5 = SWIG_OLDOBJ ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > result;
  
  if ((nobjs < 5) || (nobjs > 5)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedSubscribeService" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedSubscribeService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedSubscribeService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedSubscribeService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedSubscribeService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[3];
  //if (smartarg) {
  //	arg4 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //}
  
  res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem4);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedSubscribeService" "', argument " "4"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData >""'");
  }
  if (argp4) arg4 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4)->get(),true);
  if (newmem4 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4);
  
  {
    std::string *ptr = (std::string *)0;
    res5 = SWIG_AsPtr_std_string(swig_obj[4], &ptr);
    if (!SWIG_IsOK(res5)) {
      SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "WrappedSubscribeService" "', argument " "5"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedSubscribeService" "', argument " "5"" of type '" "std::string const &""'"); 
    }
    arg5 = ptr;
  }
  
  try {
    result = RobotRaconteur::WrappedSubscribeService((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(std::string const &)*arg2,(std::string const &)*arg3,arg4,(std::string const &)*arg5);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res5)) delete arg5;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedSubscribeService__SWIG_5(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > arg4 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  void *argp4 ;
  int res4 = 0 ;
  int newmem4 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > result;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedSubscribeService" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedSubscribeService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedSubscribeService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedSubscribeService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedSubscribeService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[3];
  //if (smartarg) {
  //	arg4 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //}
  
  res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem4);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "WrappedSubscribeService" "', argument " "4"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData >""'");
  }
  if (argp4) arg4 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4)->get(),true);
  if (newmem4 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4);
  
  
  try {
    result = RobotRaconteur::WrappedSubscribeService((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(std::string const &)*arg2,(std::string const &)*arg3,arg4);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedSubscribeService__SWIG_6(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedSubscribeService" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedSubscribeService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedSubscribeService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "WrappedSubscribeService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedSubscribeService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    result = RobotRaconteur::WrappedSubscribeService((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(std::string const &)*arg2,(std::string const &)*arg3);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedSubscribeService__SWIG_7(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  std::string *arg2 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  int res2 = SWIG_OLDOBJ ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedSubscribeService" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedSubscribeService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedSubscribeService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    result = RobotRaconteur::WrappedSubscribeService((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(std::string const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedSubscribeService(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[6] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedSubscribeService", 0, 5, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_WrappedSubscribeService__SWIG_7(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_WrappedSubscribeService__SWIG_3(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_WrappedSubscribeService__SWIG_6(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_WrappedSubscribeService__SWIG_2(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_WrappedSubscribeService__SWIG_5(self, argc, argv);
          }
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_WrappedSubscribeService__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  if (argc == 5) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            int res = SWIG_AsPtr_std_string(argv[4], (std::string**)(0));
            _v = SWIG_CheckState(res);
            if (_v) {
              return _wrap_WrappedSubscribeService__SWIG_4(self, argc, argv);
            }
          }
        }
      }
    }
  }
  if (argc == 5) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            int res = SWIG_AsPtr_std_string(argv[4], (std::string**)(0));
            _v = SWIG_CheckState(res);
            if (_v) {
              return _wrap_WrappedSubscribeService__SWIG_0(self, argc, argv);
            }
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedSubscribeService'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedSubscribeService(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &,std::vector< std::string,std::allocator< std::string > > const &,std::string const &,boost::intrusive_ptr< RobotRaconteur::MessageElementData >,std::string const &)\n"
    "    RobotRaconteur::WrappedSubscribeService(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &,std::vector< std::string,std::allocator< std::string > > const &,std::string const &,boost::intrusive_ptr< RobotRaconteur::MessageElementData >)\n"
    "    RobotRaconteur::WrappedSubscribeService(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &,std::vector< std::string,std::allocator< std::string > > const &,std::string const &)\n"
    "    RobotRaconteur::WrappedSubscribeService(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &,std::vector< std::string,std::allocator< std::string > > const &)\n"
    "    RobotRaconteur::WrappedSubscribeService(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &,std::string const &,std::string const &,boost::intrusive_ptr< RobotRaconteur::MessageElementData >,std::string const &)\n"
    "    RobotRaconteur::WrappedSubscribeService(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &,std::string const &,std::string const &,boost::intrusive_ptr< RobotRaconteur::MessageElementData >)\n"
    "    RobotRaconteur::WrappedSubscribeService(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &,std::string const &,std::string const &)\n"
    "    RobotRaconteur::WrappedSubscribeService(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &,std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManagerDetails_Name_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionManagerDetails_Name_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManagerDetails_Name_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManagerDetails *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionManagerDetails * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionManagerDetails_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionManagerDetails_Name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Name = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManagerDetails_Name_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManagerDetails_Name_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManagerDetails *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionManagerDetails * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->Name);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManagerDetails_ConnectionMethod_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *) 0 ;
  RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionManagerDetails_ConnectionMethod_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManagerDetails_ConnectionMethod_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManagerDetails *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionManagerDetails * >(argp1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedServiceSubscriptionManagerDetails_ConnectionMethod_set" "', argument " "2"" of type '" "RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD""'");
  } 
  arg2 = static_cast< RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ConnectionMethod = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManagerDetails_ConnectionMethod_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManagerDetails_ConnectionMethod_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManagerDetails *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionManagerDetails * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD) ((arg1)->ConnectionMethod);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManagerDetails_Urls_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = (std::vector< std::string,std::allocator< std::string > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionManagerDetails_Urls_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManagerDetails_Urls_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManagerDetails *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionManagerDetails * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionManagerDetails_Urls_set" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< std::string,std::allocator< std::string > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Urls = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManagerDetails_Urls_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManagerDetails_Urls_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManagerDetails *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionManagerDetails * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< std::string,std::allocator< std::string > > *)& ((arg1)->Urls);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManagerDetails_UrlUsername_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionManagerDetails_UrlUsername_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManagerDetails_UrlUsername_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManagerDetails *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionManagerDetails * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionManagerDetails_UrlUsername_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionManagerDetails_UrlUsername_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->UrlUsername = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManagerDetails_UrlUsername_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManagerDetails_UrlUsername_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManagerDetails *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionManagerDetails * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->UrlUsername);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManagerDetails_UrlCredentials_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *) 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > tempnull2 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > temp2 ;
  int newmem2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionManagerDetails_UrlCredentials_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManagerDetails_UrlCredentials_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManagerDetails *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionManagerDetails * >(argp1);
  
  // intrusive_ptr by reference
  //if ( swig_obj[1] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[1];
  //	temp2 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //	arg2 = &temp2;
  //} else {
  //  arg2 = &tempnull2;
  //}
  
  res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem2);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionManagerDetails_UrlCredentials_set" "', argument " "2"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &""'");
  }
  if (argp2) {
    temp2 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp2)->get(),true);
    if (newmem2 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp2);
    arg2 = &temp2;
  }
  else {
    arg2 = &tempnull2;
  }
  
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->UrlCredentials = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManagerDetails_UrlCredentials_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::MessageElementData > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManagerDetails_UrlCredentials_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManagerDetails *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionManagerDetails * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::intrusive_ptr< RobotRaconteur::MessageElementData > *) & ((arg1)->UrlCredentials);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  
  //if (*result) {
  //  intrusive_ptr_add_ref(result->get());
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >(result->get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElementData >());
  //} else {
  //  *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData > **)&resultobj = 0;
  //}
  
  if(*(result)) {
    intrusive_ptr_add_ref(result->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElementData >(result->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElementData >());
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManagerDetails_ServiceTypes_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = (std::vector< std::string,std::allocator< std::string > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionManagerDetails_ServiceTypes_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManagerDetails_ServiceTypes_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManagerDetails *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionManagerDetails * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionManagerDetails_ServiceTypes_set" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< std::string,std::allocator< std::string > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ServiceTypes = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManagerDetails_ServiceTypes_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManagerDetails_ServiceTypes_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManagerDetails *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionManagerDetails * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< std::string,std::allocator< std::string > > *)& ((arg1)->ServiceTypes);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManagerDetails_Filter_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionManagerDetails_Filter_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManagerDetails_Filter_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManagerDetails *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionManagerDetails * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionManagerDetails_Filter_set" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp2) : &tempshared2;
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Filter = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManagerDetails_Filter_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManagerDetails_Filter_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManagerDetails *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionManagerDetails * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > *) & ((arg1)->Filter);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionFilter >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManagerDetails_Enabled_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionManagerDetails_Enabled_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManagerDetails_Enabled_set" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManagerDetails *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionManagerDetails * >(argp1);
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedServiceSubscriptionManagerDetails_Enabled_set" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Enabled = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManagerDetails_Enabled_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  bool result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManagerDetails_Enabled_get" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManagerDetails *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionManagerDetails * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (bool) ((arg1)->Enabled);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedServiceSubscriptionManagerDetails__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *)new RobotRaconteur::WrappedServiceSubscriptionManagerDetails();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedServiceSubscriptionManagerDetails__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD arg2 ;
  std::vector< std::string,std::allocator< std::string > > *arg3 = 0 ;
  std::string *arg4 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > *arg5 = 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg6 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > *arg7 = 0 ;
  bool arg8 ;
  int res1 = SWIG_OLDOBJ ;
  int val2 ;
  int ecode2 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  int res4 = SWIG_OLDOBJ ;
  void *argp5 ;
  int res5 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > tempnull5 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > temp5 ;
  int newmem5 = 0 ;
  int res6 = SWIG_OLDOBJ ;
  void *argp7 ;
  int res7 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > tempshared7 ;
  bool val8 ;
  int ecode8 = 0 ;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *result = 0 ;
  
  if ((nobjs < 8) || (nobjs > 8)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "2"" of type '" "RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD""'");
  } 
  arg2 = static_cast< RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD >(val2);
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg3 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res4 = SWIG_AsPtr_std_string(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "4"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "4"" of type '" "std::string const &""'"); 
    }
    arg4 = ptr;
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[4] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[4];
  //	temp5 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //	arg5 = &temp5;
  //} else {
  //  arg5 = &tempnull5;
  //}
  
  res5 = SWIG_ConvertPtrAndOwn(swig_obj[4], &argp5, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem5);
  if (!SWIG_IsOK(res5)) {
    SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "5"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &""'");
  }
  if (argp5) {
    temp5 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp5)->get(),true);
    if (newmem5 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp5);
    arg5 = &temp5;
  }
  else {
    arg5 = &tempnull5;
  }
  
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res6 = swig::asptr(swig_obj[5], &ptr);
    if (!SWIG_IsOK(res6)) {
      SWIG_exception_fail(SWIG_ArgError(res6), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "6"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "6"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg6 = ptr;
  }
  {
    int newmem = 0;
    res7 = SWIG_ConvertPtrAndOwn(swig_obj[6], &argp7, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t,  0 , &newmem);
    if (!SWIG_IsOK(res7)) {
      SWIG_exception_fail(SWIG_ArgError(res7), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "7"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp7) tempshared7 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp7);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp7);
      arg7 = &tempshared7;
    } else {
      arg7 = (argp7) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp7) : &tempshared7;
    }
  }
  ecode8 = SWIG_AsVal_bool(swig_obj[7], &val8);
  if (!SWIG_IsOK(ecode8)) {
    SWIG_exception_fail(SWIG_ArgError(ecode8), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "8"" of type '" "bool""'");
  } 
  arg8 = static_cast< bool >(val8);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *)new RobotRaconteur::WrappedServiceSubscriptionManagerDetails((std::string const &)*arg1,arg2,(std::vector< std::string,std::allocator< std::string > > const &)*arg3,(std::string const &)*arg4,(boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &)*arg5,(std::vector< std::string,std::allocator< std::string > > const &)*arg6,(boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > const &)*arg7,arg8);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  if (SWIG_IsNewObj(res6)) delete arg6;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  if (SWIG_IsNewObj(res6)) delete arg6;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedServiceSubscriptionManagerDetails__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD arg2 ;
  std::vector< std::string,std::allocator< std::string > > *arg3 = 0 ;
  std::string *arg4 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > *arg5 = 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg6 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > *arg7 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  int val2 ;
  int ecode2 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  int res4 = SWIG_OLDOBJ ;
  void *argp5 ;
  int res5 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > tempnull5 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > temp5 ;
  int newmem5 = 0 ;
  int res6 = SWIG_OLDOBJ ;
  void *argp7 ;
  int res7 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > tempshared7 ;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *result = 0 ;
  
  if ((nobjs < 7) || (nobjs > 7)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "2"" of type '" "RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD""'");
  } 
  arg2 = static_cast< RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD >(val2);
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg3 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res4 = SWIG_AsPtr_std_string(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "4"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "4"" of type '" "std::string const &""'"); 
    }
    arg4 = ptr;
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[4] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[4];
  //	temp5 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //	arg5 = &temp5;
  //} else {
  //  arg5 = &tempnull5;
  //}
  
  res5 = SWIG_ConvertPtrAndOwn(swig_obj[4], &argp5, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem5);
  if (!SWIG_IsOK(res5)) {
    SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "5"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &""'");
  }
  if (argp5) {
    temp5 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp5)->get(),true);
    if (newmem5 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp5);
    arg5 = &temp5;
  }
  else {
    arg5 = &tempnull5;
  }
  
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res6 = swig::asptr(swig_obj[5], &ptr);
    if (!SWIG_IsOK(res6)) {
      SWIG_exception_fail(SWIG_ArgError(res6), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "6"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "6"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg6 = ptr;
  }
  {
    int newmem = 0;
    res7 = SWIG_ConvertPtrAndOwn(swig_obj[6], &argp7, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t,  0 , &newmem);
    if (!SWIG_IsOK(res7)) {
      SWIG_exception_fail(SWIG_ArgError(res7), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "7"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp7) tempshared7 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp7);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp7);
      arg7 = &tempshared7;
    } else {
      arg7 = (argp7) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > * >(argp7) : &tempshared7;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *)new RobotRaconteur::WrappedServiceSubscriptionManagerDetails((std::string const &)*arg1,arg2,(std::vector< std::string,std::allocator< std::string > > const &)*arg3,(std::string const &)*arg4,(boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &)*arg5,(std::vector< std::string,std::allocator< std::string > > const &)*arg6,(boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > const &)*arg7);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  if (SWIG_IsNewObj(res6)) delete arg6;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  if (SWIG_IsNewObj(res6)) delete arg6;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedServiceSubscriptionManagerDetails__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD arg2 ;
  std::vector< std::string,std::allocator< std::string > > *arg3 = 0 ;
  std::string *arg4 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > *arg5 = 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg6 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  int val2 ;
  int ecode2 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  int res4 = SWIG_OLDOBJ ;
  void *argp5 ;
  int res5 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > tempnull5 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > temp5 ;
  int newmem5 = 0 ;
  int res6 = SWIG_OLDOBJ ;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *result = 0 ;
  
  if ((nobjs < 6) || (nobjs > 6)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "2"" of type '" "RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD""'");
  } 
  arg2 = static_cast< RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD >(val2);
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg3 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res4 = SWIG_AsPtr_std_string(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "4"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "4"" of type '" "std::string const &""'"); 
    }
    arg4 = ptr;
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[4] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[4];
  //	temp5 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //	arg5 = &temp5;
  //} else {
  //  arg5 = &tempnull5;
  //}
  
  res5 = SWIG_ConvertPtrAndOwn(swig_obj[4], &argp5, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem5);
  if (!SWIG_IsOK(res5)) {
    SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "5"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &""'");
  }
  if (argp5) {
    temp5 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp5)->get(),true);
    if (newmem5 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp5);
    arg5 = &temp5;
  }
  else {
    arg5 = &tempnull5;
  }
  
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res6 = swig::asptr(swig_obj[5], &ptr);
    if (!SWIG_IsOK(res6)) {
      SWIG_exception_fail(SWIG_ArgError(res6), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "6"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "6"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg6 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *)new RobotRaconteur::WrappedServiceSubscriptionManagerDetails((std::string const &)*arg1,arg2,(std::vector< std::string,std::allocator< std::string > > const &)*arg3,(std::string const &)*arg4,(boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &)*arg5,(std::vector< std::string,std::allocator< std::string > > const &)*arg6);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  if (SWIG_IsNewObj(res6)) delete arg6;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  if (SWIG_IsNewObj(res6)) delete arg6;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedServiceSubscriptionManagerDetails__SWIG_4(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD arg2 ;
  std::vector< std::string,std::allocator< std::string > > *arg3 = 0 ;
  std::string *arg4 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > *arg5 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  int val2 ;
  int ecode2 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  int res4 = SWIG_OLDOBJ ;
  void *argp5 ;
  int res5 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > tempnull5 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > temp5 ;
  int newmem5 = 0 ;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *result = 0 ;
  
  if ((nobjs < 5) || (nobjs > 5)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "2"" of type '" "RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD""'");
  } 
  arg2 = static_cast< RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD >(val2);
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg3 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res4 = SWIG_AsPtr_std_string(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "4"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "4"" of type '" "std::string const &""'"); 
    }
    arg4 = ptr;
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[4] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[4];
  //	temp5 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //	arg5 = &temp5;
  //} else {
  //  arg5 = &tempnull5;
  //}
  
  res5 = SWIG_ConvertPtrAndOwn(swig_obj[4], &argp5, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem5);
  if (!SWIG_IsOK(res5)) {
    SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "5"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &""'");
  }
  if (argp5) {
    temp5 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp5)->get(),true);
    if (newmem5 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp5);
    arg5 = &temp5;
  }
  else {
    arg5 = &tempnull5;
  }
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *)new RobotRaconteur::WrappedServiceSubscriptionManagerDetails((std::string const &)*arg1,arg2,(std::vector< std::string,std::allocator< std::string > > const &)*arg3,(std::string const &)*arg4,(boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &)*arg5);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedServiceSubscriptionManagerDetails__SWIG_5(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD arg2 ;
  std::vector< std::string,std::allocator< std::string > > *arg3 = 0 ;
  std::string *arg4 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  int val2 ;
  int ecode2 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  int res4 = SWIG_OLDOBJ ;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *result = 0 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "2"" of type '" "RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD""'");
  } 
  arg2 = static_cast< RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD >(val2);
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg3 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res4 = SWIG_AsPtr_std_string(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "4"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "4"" of type '" "std::string const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *)new RobotRaconteur::WrappedServiceSubscriptionManagerDetails((std::string const &)*arg1,arg2,(std::vector< std::string,std::allocator< std::string > > const &)*arg3,(std::string const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedServiceSubscriptionManagerDetails__SWIG_6(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD arg2 ;
  std::vector< std::string,std::allocator< std::string > > *arg3 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  int val2 ;
  int ecode2 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *result = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "2"" of type '" "RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD""'");
  } 
  arg2 = static_cast< RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD >(val2);
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *)new RobotRaconteur::WrappedServiceSubscriptionManagerDetails((std::string const &)*arg1,arg2,(std::vector< std::string,std::allocator< std::string > > const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedServiceSubscriptionManagerDetails__SWIG_7(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD arg2 ;
  int res1 = SWIG_OLDOBJ ;
  int val2 ;
  int ecode2 = 0 ;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "2"" of type '" "RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD""'");
  } 
  arg2 = static_cast< RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *)new RobotRaconteur::WrappedServiceSubscriptionManagerDetails((std::string const &)*arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedServiceSubscriptionManagerDetails__SWIG_8(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManagerDetails" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *)new RobotRaconteur::WrappedServiceSubscriptionManagerDetails((std::string const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, SWIG_POINTER_NEW |  0 );
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedServiceSubscriptionManagerDetails(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[9] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_WrappedServiceSubscriptionManagerDetails", 0, 8, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_WrappedServiceSubscriptionManagerDetails__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    int res = SWIG_AsPtr_std_string(argv[0], (std::string**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_WrappedServiceSubscriptionManagerDetails__SWIG_8(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_AsPtr_std_string(argv[0], (std::string**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_new_WrappedServiceSubscriptionManagerDetails__SWIG_7(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_AsPtr_std_string(argv[0], (std::string**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< std::string,std::allocator< std::string > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_new_WrappedServiceSubscriptionManagerDetails__SWIG_6(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = SWIG_AsPtr_std_string(argv[0], (std::string**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< std::string,std::allocator< std::string > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_AsPtr_std_string(argv[3], (std::string**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_new_WrappedServiceSubscriptionManagerDetails__SWIG_5(self, argc, argv);
          }
        }
      }
    }
  }
  if (argc == 5) {
    int _v;
    int res = SWIG_AsPtr_std_string(argv[0], (std::string**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< std::string,std::allocator< std::string > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_AsPtr_std_string(argv[3], (std::string**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            int res = SWIG_ConvertPtr(argv[4], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0);
            _v = SWIG_CheckState(res);
            if (_v) {
              return _wrap_new_WrappedServiceSubscriptionManagerDetails__SWIG_4(self, argc, argv);
            }
          }
        }
      }
    }
  }
  if (argc == 6) {
    int _v;
    int res = SWIG_AsPtr_std_string(argv[0], (std::string**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< std::string,std::allocator< std::string > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_AsPtr_std_string(argv[3], (std::string**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            int res = SWIG_ConvertPtr(argv[4], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0);
            _v = SWIG_CheckState(res);
            if (_v) {
              int res = swig::asptr(argv[5], (std::vector< std::string,std::allocator< std::string > >**)(0));
              _v = SWIG_CheckState(res);
              if (_v) {
                return _wrap_new_WrappedServiceSubscriptionManagerDetails__SWIG_3(self, argc, argv);
              }
            }
          }
        }
      }
    }
  }
  if (argc == 7) {
    int _v;
    int res = SWIG_AsPtr_std_string(argv[0], (std::string**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< std::string,std::allocator< std::string > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_AsPtr_std_string(argv[3], (std::string**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            int res = SWIG_ConvertPtr(argv[4], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0);
            _v = SWIG_CheckState(res);
            if (_v) {
              int res = swig::asptr(argv[5], (std::vector< std::string,std::allocator< std::string > >**)(0));
              _v = SWIG_CheckState(res);
              if (_v) {
                int res = SWIG_ConvertPtr(argv[6], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t, 0);
                _v = SWIG_CheckState(res);
                if (_v) {
                  return _wrap_new_WrappedServiceSubscriptionManagerDetails__SWIG_2(self, argc, argv);
                }
              }
            }
          }
        }
      }
    }
  }
  if (argc == 8) {
    int _v;
    int res = SWIG_AsPtr_std_string(argv[0], (std::string**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_int(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        int res = swig::asptr(argv[2], (std::vector< std::string,std::allocator< std::string > >**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_AsPtr_std_string(argv[3], (std::string**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            int res = SWIG_ConvertPtr(argv[4], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0);
            _v = SWIG_CheckState(res);
            if (_v) {
              int res = swig::asptr(argv[5], (std::vector< std::string,std::allocator< std::string > >**)(0));
              _v = SWIG_CheckState(res);
              if (_v) {
                int res = SWIG_ConvertPtr(argv[6], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t, 0);
                _v = SWIG_CheckState(res);
                if (_v) {
                  {
                    int res = SWIG_AsVal_bool(argv[7], NULL);
                    _v = SWIG_CheckState(res);
                  }
                  if (_v) {
                    return _wrap_new_WrappedServiceSubscriptionManagerDetails__SWIG_1(self, argc, argv);
                  }
                }
              }
            }
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_WrappedServiceSubscriptionManagerDetails'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedServiceSubscriptionManagerDetails::WrappedServiceSubscriptionManagerDetails()\n"
    "    RobotRaconteur::WrappedServiceSubscriptionManagerDetails::WrappedServiceSubscriptionManagerDetails(std::string const &,RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD,std::vector< std::string,std::allocator< std::string > > const &,std::string const &,boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &,std::vector< std::string,std::allocator< std::string > > const &,boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > const &,bool)\n"
    "    RobotRaconteur::WrappedServiceSubscriptionManagerDetails::WrappedServiceSubscriptionManagerDetails(std::string const &,RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD,std::vector< std::string,std::allocator< std::string > > const &,std::string const &,boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &,std::vector< std::string,std::allocator< std::string > > const &,boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > const &)\n"
    "    RobotRaconteur::WrappedServiceSubscriptionManagerDetails::WrappedServiceSubscriptionManagerDetails(std::string const &,RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD,std::vector< std::string,std::allocator< std::string > > const &,std::string const &,boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &,std::vector< std::string,std::allocator< std::string > > const &)\n"
    "    RobotRaconteur::WrappedServiceSubscriptionManagerDetails::WrappedServiceSubscriptionManagerDetails(std::string const &,RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD,std::vector< std::string,std::allocator< std::string > > const &,std::string const &,boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &)\n"
    "    RobotRaconteur::WrappedServiceSubscriptionManagerDetails::WrappedServiceSubscriptionManagerDetails(std::string const &,RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD,std::vector< std::string,std::allocator< std::string > > const &,std::string const &)\n"
    "    RobotRaconteur::WrappedServiceSubscriptionManagerDetails::WrappedServiceSubscriptionManagerDetails(std::string const &,RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD,std::vector< std::string,std::allocator< std::string > > const &)\n"
    "    RobotRaconteur::WrappedServiceSubscriptionManagerDetails::WrappedServiceSubscriptionManagerDetails(std::string const &,RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD)\n"
    "    RobotRaconteur::WrappedServiceSubscriptionManagerDetails::WrappedServiceSubscriptionManagerDetails(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_delete_WrappedServiceSubscriptionManagerDetails(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManagerDetails *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedServiceSubscriptionManagerDetails" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManagerDetails *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionManagerDetails * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedServiceSubscriptionManagerDetails_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedServiceSubscriptionManagerDetails_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_new_WrappedServiceSubscriptionManager__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManager *result = 0 ;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedServiceSubscriptionManager *)new RobotRaconteur::WrappedServiceSubscriptionManager();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedServiceSubscriptionManager__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  RobotRaconteur::WrappedServiceSubscriptionManager *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_WrappedServiceSubscriptionManager" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedServiceSubscriptionManager *)new RobotRaconteur::WrappedServiceSubscriptionManager((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedServiceSubscriptionManager__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  RobotRaconteur::WrappedServiceSubscriptionManager *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > *ptr = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_WrappedServiceSubscriptionManager" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManager" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedServiceSubscriptionManager *)new RobotRaconteur::WrappedServiceSubscriptionManager((std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedServiceSubscriptionManager__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > *arg1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg2 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared2 ;
  RobotRaconteur::WrappedServiceSubscriptionManager *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > *ptr = (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_WrappedServiceSubscriptionManager" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedServiceSubscriptionManager" "', argument " "1"" of type '" "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > const &""'"); 
    }
    arg1 = ptr;
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_WrappedServiceSubscriptionManager" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::WrappedServiceSubscriptionManager *)new RobotRaconteur::WrappedServiceSubscriptionManager((std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > const &)*arg1,(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedServiceSubscriptionManager(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_WrappedServiceSubscriptionManager", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_new_WrappedServiceSubscriptionManager__SWIG_0(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_WrappedServiceSubscriptionManager__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 1) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_new_WrappedServiceSubscriptionManager__SWIG_2(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = swig::asptr(argv[0], (std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > >**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_WrappedServiceSubscriptionManager__SWIG_3(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_WrappedServiceSubscriptionManager'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedServiceSubscriptionManager::WrappedServiceSubscriptionManager()\n"
    "    RobotRaconteur::WrappedServiceSubscriptionManager::WrappedServiceSubscriptionManager(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)\n"
    "    RobotRaconteur::WrappedServiceSubscriptionManager::WrappedServiceSubscriptionManager(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > const &)\n"
    "    RobotRaconteur::WrappedServiceSubscriptionManager::WrappedServiceSubscriptionManager(std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > const &,boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManager_AddSubscription(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManager *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManager *) 0 ;
  RobotRaconteur::WrappedServiceSubscriptionManagerDetails *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionManager_AddSubscription", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManager_AddSubscription" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManager *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionManager_AddSubscription" "', argument " "2"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManagerDetails const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionManager_AddSubscription" "', argument " "2"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManagerDetails const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::WrappedServiceSubscriptionManagerDetails * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->AddSubscription((RobotRaconteur::WrappedServiceSubscriptionManagerDetails const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManager_RemoveSubscription__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManager *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManager *) 0 ;
  std::string *arg2 = 0 ;
  bool arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  bool val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManager_RemoveSubscription" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManager *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionManager_RemoveSubscription" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionManager_RemoveSubscription" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  ecode3 = SWIG_AsVal_bool(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedServiceSubscriptionManager_RemoveSubscription" "', argument " "3"" of type '" "bool""'");
  } 
  arg3 = static_cast< bool >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->RemoveSubscription((std::string const &)*arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManager_RemoveSubscription__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManager *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManager *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManager_RemoveSubscription" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManager *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionManager_RemoveSubscription" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionManager_RemoveSubscription" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->RemoveSubscription((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManager_RemoveSubscription(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionManager_RemoveSubscription", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_WrappedServiceSubscriptionManager_RemoveSubscription__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        {
          int res = SWIG_AsVal_bool(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_WrappedServiceSubscriptionManager_RemoveSubscription__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedServiceSubscriptionManager_RemoveSubscription'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedServiceSubscriptionManager::RemoveSubscription(std::string const &,bool)\n"
    "    RobotRaconteur::WrappedServiceSubscriptionManager::RemoveSubscription(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManager_EnableSubscription(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManager *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManager *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionManager_EnableSubscription", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManager_EnableSubscription" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManager *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionManager_EnableSubscription" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionManager_EnableSubscription" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->EnableSubscription((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManager_DisableSubscription__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManager *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManager *) 0 ;
  std::string *arg2 = 0 ;
  bool arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  bool val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManager_DisableSubscription" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManager *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionManager_DisableSubscription" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionManager_DisableSubscription" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  ecode3 = SWIG_AsVal_bool(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedServiceSubscriptionManager_DisableSubscription" "', argument " "3"" of type '" "bool""'");
  } 
  arg3 = static_cast< bool >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->DisableSubscription((std::string const &)*arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManager_DisableSubscription__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManager *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManager *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManager_DisableSubscription" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManager *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionManager_DisableSubscription" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionManager_DisableSubscription" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->DisableSubscription((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManager_DisableSubscription(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionManager_DisableSubscription", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_WrappedServiceSubscriptionManager_DisableSubscription__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        {
          int res = SWIG_AsVal_bool(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_WrappedServiceSubscriptionManager_DisableSubscription__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedServiceSubscriptionManager_DisableSubscription'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedServiceSubscriptionManager::DisableSubscription(std::string const &,bool)\n"
    "    RobotRaconteur::WrappedServiceSubscriptionManager::DisableSubscription(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManager_GetSubscription__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManager *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManager *) 0 ;
  std::string *arg2 = 0 ;
  bool arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  bool val3 ;
  int ecode3 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManager_GetSubscription" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManager *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionManager_GetSubscription" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionManager_GetSubscription" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  ecode3 = SWIG_AsVal_bool(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "WrappedServiceSubscriptionManager_GetSubscription" "', argument " "3"" of type '" "bool""'");
  } 
  arg3 = static_cast< bool >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetSubscription((std::string const &)*arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManager_GetSubscription__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManager *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManager *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManager_GetSubscription" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManager *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionManager_GetSubscription" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionManager_GetSubscription" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetSubscription((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscription >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManager_GetSubscription(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionManager_GetSubscription", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_WrappedServiceSubscriptionManager_GetSubscription__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        {
          int res = SWIG_AsVal_bool(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_WrappedServiceSubscriptionManager_GetSubscription__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedServiceSubscriptionManager_GetSubscription'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedServiceSubscriptionManager::GetSubscription(std::string const &,bool)\n"
    "    RobotRaconteur::WrappedServiceSubscriptionManager::GetSubscription(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManager_IsConnected(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManager *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManager *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionManager_IsConnected", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManager_IsConnected" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManager *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionManager_IsConnected" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionManager_IsConnected" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->IsConnected((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManager_IsEnabled(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManager *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManager *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionManager_IsEnabled", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManager_IsEnabled" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManager *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "WrappedServiceSubscriptionManager_IsEnabled" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "WrappedServiceSubscriptionManager_IsEnabled" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->IsEnabled((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManager_Close__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManager *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManager *) 0 ;
  bool arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > *smartarg1 = 0 ;
  bool val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManager_Close" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManager *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_bool(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "WrappedServiceSubscriptionManager_Close" "', argument " "2"" of type '" "bool""'");
  } 
  arg2 = static_cast< bool >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Close(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManager_Close__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManager *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManager *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > *smartarg1 = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManager_Close" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManager *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Close();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManager_Close(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "WrappedServiceSubscriptionManager_Close", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_WrappedServiceSubscriptionManager_Close__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_bool(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_WrappedServiceSubscriptionManager_Close__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'WrappedServiceSubscriptionManager_Close'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::WrappedServiceSubscriptionManager::Close(bool)\n"
    "    RobotRaconteur::WrappedServiceSubscriptionManager::Close()\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManager_GetSubscriptionNames(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManager *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManager *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManager_GetSubscriptionNames" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManager *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetSubscriptionNames();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< std::string,std::allocator< std::string > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManager_GetSubscriptionDetails(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManager *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManager *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManager_GetSubscriptionDetails" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManager *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetSubscriptionDetails();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedServiceSubscriptionManager_GetNode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManager *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManager *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedServiceSubscriptionManager_GetNode" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManager *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetNode();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedServiceSubscriptionManager(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::WrappedServiceSubscriptionManager *arg1 = (RobotRaconteur::WrappedServiceSubscriptionManager *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedServiceSubscriptionManager" "', argument " "1"" of type '" "RobotRaconteur::WrappedServiceSubscriptionManager *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::WrappedServiceSubscriptionManager > * >(argp1);
      arg1 = const_cast< RobotRaconteur::WrappedServiceSubscriptionManager * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *WrappedServiceSubscriptionManager_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedServiceSubscriptionManager_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_RRLogRecord__GetNode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord__GetNode" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_RRLogRecord_GetNode(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord__SetNode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RRLogRecord__SetNode", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord__SetNode" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RRLogRecord__SetNode" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur_RRLogRecord_SetNode(arg1,(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_Level_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  RobotRaconteur::RobotRaconteur_LogLevel arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RRLogRecord_Level_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_Level_set" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "RRLogRecord_Level_set" "', argument " "2"" of type '" "RobotRaconteur::RobotRaconteur_LogLevel""'");
  } 
  arg2 = static_cast< RobotRaconteur::RobotRaconteur_LogLevel >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Level = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_Level_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::RobotRaconteur_LogLevel result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_Level_get" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::RobotRaconteur_LogLevel) ((arg1)->Level);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_Component_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  RobotRaconteur::RobotRaconteur_LogComponent arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RRLogRecord_Component_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_Component_set" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "RRLogRecord_Component_set" "', argument " "2"" of type '" "RobotRaconteur::RobotRaconteur_LogComponent""'");
  } 
  arg2 = static_cast< RobotRaconteur::RobotRaconteur_LogComponent >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Component = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_Component_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::RobotRaconteur_LogComponent result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_Component_get" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::RobotRaconteur_LogComponent) ((arg1)->Component);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_ComponentName_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RRLogRecord_ComponentName_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_ComponentName_set" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RRLogRecord_ComponentName_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RRLogRecord_ComponentName_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ComponentName = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_ComponentName_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_ComponentName_get" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->ComponentName);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_ComponentObjectID_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RRLogRecord_ComponentObjectID_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_ComponentObjectID_set" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RRLogRecord_ComponentObjectID_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RRLogRecord_ComponentObjectID_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ComponentObjectID = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_ComponentObjectID_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_ComponentObjectID_get" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->ComponentObjectID);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_Endpoint_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  int64_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  long val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RRLogRecord_Endpoint_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_Endpoint_set" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  ecode2 = SWIG_AsVal_long(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "RRLogRecord_Endpoint_set" "', argument " "2"" of type '" "int64_t""'");
  } 
  arg2 = static_cast< int64_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Endpoint = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_Endpoint_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  int64_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_Endpoint_get" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (int64_t) ((arg1)->Endpoint);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_long(static_cast< long >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_ServicePath_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RRLogRecord_ServicePath_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_ServicePath_set" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RRLogRecord_ServicePath_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RRLogRecord_ServicePath_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ServicePath = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_ServicePath_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_ServicePath_get" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->ServicePath);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_Member_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RRLogRecord_Member_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_Member_set" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RRLogRecord_Member_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RRLogRecord_Member_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Member = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_Member_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_Member_get" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->Member);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_Message_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RRLogRecord_Message_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_Message_set" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RRLogRecord_Message_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RRLogRecord_Message_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Message = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_Message_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_Message_get" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->Message);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_Time_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  boost::posix_time::ptime arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RRLogRecord_Time_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_Time_set" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  {
    if (PyDateTime_Check(swig_obj[1])) {
      arg2 = boost::posix_time::ptime(boost::gregorian::date((int)PyDateTime_GET_YEAR(swig_obj[1]),(int)PyDateTime_GET_MONTH(swig_obj[1]),(int)PyDateTime_GET_DAY(swig_obj[1])),
        boost::posix_time::time_duration((int)PyDateTime_DATE_GET_HOUR(swig_obj[1]),(int)PyDateTime_DATE_GET_MINUTE(swig_obj[1]),(int)PyDateTime_DATE_GET_SECOND(swig_obj[1])))
      + boost::posix_time::microseconds((int)PyDateTime_DATE_GET_MICROSECOND(swig_obj[1]));
    }
    else
    {
      PyErr_SetString(PyExc_TypeError, "not a datetime");
      SWIG_fail;
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->Time = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_Time_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::posix_time::ptime result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_Time_get" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result =  ((arg1)->Time);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  {
    try {
      resultobj = PyDateTime_FromDateAndTime((int)(&result)->date().year(), (int)(&result)->date().month(), (int)(&result)->date().day(),
        (int)(&result)->time_of_day().hours(), (int)(&result)->time_of_day().minutes(), (int)(&result)->time_of_day().seconds(),
        boost::numeric_cast<int32_t>(((&result)->time_of_day().fractional_seconds() * boost::numeric_cast<int32_t>(pow(10.0,(9-(&result)->time_of_day().num_fractional_digits())))))/1000);
    }
    catch (std::exception&)
    {
      resultobj = SWIG_Py_Void();
    }
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_SourceFile_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RRLogRecord_SourceFile_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_SourceFile_set" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RRLogRecord_SourceFile_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RRLogRecord_SourceFile_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->SourceFile = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_SourceFile_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_SourceFile_get" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->SourceFile);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_SourceLine_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RRLogRecord_SourceLine_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_SourceLine_set" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "RRLogRecord_SourceLine_set" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->SourceLine = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_SourceLine_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_SourceLine_get" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (uint32_t) ((arg1)->SourceLine);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_ThreadID_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RRLogRecord_ThreadID_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_ThreadID_set" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RRLogRecord_ThreadID_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RRLogRecord_ThreadID_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->ThreadID = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_ThreadID_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_ThreadID_get" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->ThreadID);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_FiberID_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RRLogRecord_FiberID_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_FiberID_set" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RRLogRecord_FiberID_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RRLogRecord_FiberID_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->FiberID = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord_FiberID_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord_FiberID_get" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->FiberID);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRLogRecord___str__(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RRLogRecord___str__" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_RRLogRecord_ToString(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_RRLogRecord(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_RRLogRecord", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::RRLogRecord *)new RobotRaconteur::RRLogRecord();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__RRLogRecord, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_RRLogRecord(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRLogRecord *arg1 = (RobotRaconteur::RRLogRecord *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRLogRecord, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_RRLogRecord" "', argument " "1"" of type '" "RobotRaconteur::RRLogRecord *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *RRLogRecord_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__RRLogRecord, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *RRLogRecord_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_LogRecordHandler(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::LogRecordHandler *arg1 = (RobotRaconteur::LogRecordHandler *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::LogRecordHandler > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::LogRecordHandler > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LogRecordHandler_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_LogRecordHandler" "', argument " "1"" of type '" "RobotRaconteur::LogRecordHandler *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LogRecordHandler > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LogRecordHandler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LogRecordHandler * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::LogRecordHandler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::LogRecordHandler * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *LogRecordHandler_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LogRecordHandler_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_FileLogRecordHandler_OpenFile__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::FileLogRecordHandler *arg1 = (RobotRaconteur::FileLogRecordHandler *) 0 ;
  std::string *arg2 = 0 ;
  bool arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::FileLogRecordHandler > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::FileLogRecordHandler > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  bool val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__FileLogRecordHandler_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "FileLogRecordHandler_OpenFile" "', argument " "1"" of type '" "RobotRaconteur::FileLogRecordHandler *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FileLogRecordHandler > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FileLogRecordHandler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FileLogRecordHandler * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FileLogRecordHandler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FileLogRecordHandler * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "FileLogRecordHandler_OpenFile" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "FileLogRecordHandler_OpenFile" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  ecode3 = SWIG_AsVal_bool(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "FileLogRecordHandler_OpenFile" "', argument " "3"" of type '" "bool""'");
  } 
  arg3 = static_cast< bool >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->OpenFile((std::string const &)*arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_FileLogRecordHandler_OpenFile__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::FileLogRecordHandler *arg1 = (RobotRaconteur::FileLogRecordHandler *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::FileLogRecordHandler > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::FileLogRecordHandler > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__FileLogRecordHandler_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "FileLogRecordHandler_OpenFile" "', argument " "1"" of type '" "RobotRaconteur::FileLogRecordHandler *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FileLogRecordHandler > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FileLogRecordHandler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FileLogRecordHandler * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FileLogRecordHandler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FileLogRecordHandler * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "FileLogRecordHandler_OpenFile" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "FileLogRecordHandler_OpenFile" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->OpenFile((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_FileLogRecordHandler_OpenFile(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "FileLogRecordHandler_OpenFile", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__FileLogRecordHandler_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_FileLogRecordHandler_OpenFile__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__FileLogRecordHandler_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        {
          int res = SWIG_AsVal_bool(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_FileLogRecordHandler_OpenFile__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'FileLogRecordHandler_OpenFile'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::FileLogRecordHandler::OpenFile(std::string const &,bool)\n"
    "    RobotRaconteur::FileLogRecordHandler::OpenFile(std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_FileLogRecordHandler(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::FileLogRecordHandler *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_FileLogRecordHandler", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::FileLogRecordHandler *)new RobotRaconteur::FileLogRecordHandler();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::FileLogRecordHandler > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::FileLogRecordHandler >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__FileLogRecordHandler_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_FileLogRecordHandler(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::FileLogRecordHandler *arg1 = (RobotRaconteur::FileLogRecordHandler *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::FileLogRecordHandler > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::FileLogRecordHandler > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__FileLogRecordHandler_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_FileLogRecordHandler" "', argument " "1"" of type '" "RobotRaconteur::FileLogRecordHandler *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FileLogRecordHandler > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FileLogRecordHandler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FileLogRecordHandler * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::FileLogRecordHandler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::FileLogRecordHandler * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *FileLogRecordHandler_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__FileLogRecordHandler_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *FileLogRecordHandler_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_UserLogRecordHandlerDirector_HandleLogRecord(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::UserLogRecordHandlerDirector *arg1 = (RobotRaconteur::UserLogRecordHandlerDirector *) 0 ;
  RobotRaconteur::RRLogRecord *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "UserLogRecordHandlerDirector_HandleLogRecord", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__UserLogRecordHandlerDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "UserLogRecordHandlerDirector_HandleLogRecord" "', argument " "1"" of type '" "RobotRaconteur::UserLogRecordHandlerDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::UserLogRecordHandlerDirector * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__RRLogRecord,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "UserLogRecordHandlerDirector_HandleLogRecord" "', argument " "2"" of type '" "RobotRaconteur::RRLogRecord const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "UserLogRecordHandlerDirector_HandleLogRecord" "', argument " "2"" of type '" "RobotRaconteur::RRLogRecord const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp2);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        Swig::DirectorPureVirtualException::raise("RobotRaconteur::UserLogRecordHandlerDirector::HandleLogRecord");
      } else {
        (arg1)->HandleLogRecord((RobotRaconteur::RRLogRecord const &)*arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_UserLogRecordHandlerDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::UserLogRecordHandlerDirector *arg1 = (RobotRaconteur::UserLogRecordHandlerDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__UserLogRecordHandlerDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_UserLogRecordHandlerDirector" "', argument " "1"" of type '" "RobotRaconteur::UserLogRecordHandlerDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::UserLogRecordHandlerDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_UserLogRecordHandlerDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::UserLogRecordHandlerDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (RobotRaconteur::UserLogRecordHandlerDirector *)new SwigDirector_UserLogRecordHandlerDirector(arg1); 
      } else {
        SWIG_SetErrorMsg(PyExc_RuntimeError,"accessing abstract class or protected constructor"); 
        SWIG_fail;
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__UserLogRecordHandlerDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_UserLogRecordHandlerDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::UserLogRecordHandlerDirector *arg1 = (RobotRaconteur::UserLogRecordHandlerDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__UserLogRecordHandlerDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_UserLogRecordHandlerDirector" "', argument " "1"" of type '" "RobotRaconteur::UserLogRecordHandlerDirector *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::UserLogRecordHandlerDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *UserLogRecordHandlerDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__UserLogRecordHandlerDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *UserLogRecordHandlerDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_UserLogRecordHandlerBase__SetHandler(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::UserLogRecordHandlerBase *arg1 = (RobotRaconteur::UserLogRecordHandlerBase *) 0 ;
  RobotRaconteur::UserLogRecordHandlerDirector *arg2 = (RobotRaconteur::UserLogRecordHandlerDirector *) 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::UserLogRecordHandlerBase > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::UserLogRecordHandlerBase > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "UserLogRecordHandlerBase__SetHandler", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UserLogRecordHandlerBase_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "UserLogRecordHandlerBase__SetHandler" "', argument " "1"" of type '" "RobotRaconteur::UserLogRecordHandlerBase *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UserLogRecordHandlerBase > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UserLogRecordHandlerBase > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UserLogRecordHandlerBase * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UserLogRecordHandlerBase > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UserLogRecordHandlerBase * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__UserLogRecordHandlerDirector, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "UserLogRecordHandlerBase__SetHandler" "', argument " "2"" of type '" "RobotRaconteur::UserLogRecordHandlerDirector *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::UserLogRecordHandlerDirector * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "UserLogRecordHandlerBase__SetHandler" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetHandler(arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_UserLogRecordHandlerBase(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::UserLogRecordHandlerBase *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_UserLogRecordHandlerBase", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::UserLogRecordHandlerBase *)new RobotRaconteur::UserLogRecordHandlerBase();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::UserLogRecordHandlerBase > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::UserLogRecordHandlerBase >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UserLogRecordHandlerBase_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_UserLogRecordHandlerBase(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::UserLogRecordHandlerBase *arg1 = (RobotRaconteur::UserLogRecordHandlerBase *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::UserLogRecordHandlerBase > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::UserLogRecordHandlerBase > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UserLogRecordHandlerBase_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_UserLogRecordHandlerBase" "', argument " "1"" of type '" "RobotRaconteur::UserLogRecordHandlerBase *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UserLogRecordHandlerBase > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UserLogRecordHandlerBase > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UserLogRecordHandlerBase * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::UserLogRecordHandlerBase > * >(argp1);
      arg1 = const_cast< RobotRaconteur::UserLogRecordHandlerBase * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *UserLogRecordHandlerBase_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__UserLogRecordHandlerBase_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *UserLogRecordHandlerBase_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_NodeDirectories_system_data_dir_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories *arg1 = (RobotRaconteur::NodeDirectories *) 0 ;
  boost::filesystem::path *arg2 = (boost::filesystem::path *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "NodeDirectories_system_data_dir_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeDirectories, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectories_system_data_dir_set" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectories *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectories * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_boost__filesystem__path, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "NodeDirectories_system_data_dir_set" "', argument " "2"" of type '" "boost::filesystem::path *""'"); 
  }
  arg2 = reinterpret_cast< boost::filesystem::path * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->system_data_dir = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectories_system_data_dir_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories *arg1 = (RobotRaconteur::NodeDirectories *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::filesystem::path *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeDirectories, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectories_system_data_dir_get" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectories *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectories * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::filesystem::path *)& ((arg1)->system_data_dir);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_boost__filesystem__path, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectories_system_config_dir_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories *arg1 = (RobotRaconteur::NodeDirectories *) 0 ;
  boost::filesystem::path *arg2 = (boost::filesystem::path *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "NodeDirectories_system_config_dir_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeDirectories, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectories_system_config_dir_set" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectories *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectories * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_boost__filesystem__path, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "NodeDirectories_system_config_dir_set" "', argument " "2"" of type '" "boost::filesystem::path *""'"); 
  }
  arg2 = reinterpret_cast< boost::filesystem::path * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->system_config_dir = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectories_system_config_dir_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories *arg1 = (RobotRaconteur::NodeDirectories *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::filesystem::path *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeDirectories, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectories_system_config_dir_get" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectories *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectories * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::filesystem::path *)& ((arg1)->system_config_dir);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_boost__filesystem__path, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectories_system_state_dir_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories *arg1 = (RobotRaconteur::NodeDirectories *) 0 ;
  boost::filesystem::path *arg2 = (boost::filesystem::path *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "NodeDirectories_system_state_dir_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeDirectories, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectories_system_state_dir_set" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectories *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectories * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_boost__filesystem__path, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "NodeDirectories_system_state_dir_set" "', argument " "2"" of type '" "boost::filesystem::path *""'"); 
  }
  arg2 = reinterpret_cast< boost::filesystem::path * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->system_state_dir = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectories_system_state_dir_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories *arg1 = (RobotRaconteur::NodeDirectories *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::filesystem::path *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeDirectories, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectories_system_state_dir_get" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectories *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectories * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::filesystem::path *)& ((arg1)->system_state_dir);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_boost__filesystem__path, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectories_system_cache_dir_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories *arg1 = (RobotRaconteur::NodeDirectories *) 0 ;
  boost::filesystem::path *arg2 = (boost::filesystem::path *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "NodeDirectories_system_cache_dir_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeDirectories, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectories_system_cache_dir_set" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectories *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectories * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_boost__filesystem__path, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "NodeDirectories_system_cache_dir_set" "', argument " "2"" of type '" "boost::filesystem::path *""'"); 
  }
  arg2 = reinterpret_cast< boost::filesystem::path * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->system_cache_dir = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectories_system_cache_dir_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories *arg1 = (RobotRaconteur::NodeDirectories *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::filesystem::path *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeDirectories, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectories_system_cache_dir_get" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectories *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectories * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::filesystem::path *)& ((arg1)->system_cache_dir);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_boost__filesystem__path, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectories_system_run_dir_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories *arg1 = (RobotRaconteur::NodeDirectories *) 0 ;
  boost::filesystem::path *arg2 = (boost::filesystem::path *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "NodeDirectories_system_run_dir_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeDirectories, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectories_system_run_dir_set" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectories *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectories * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_boost__filesystem__path, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "NodeDirectories_system_run_dir_set" "', argument " "2"" of type '" "boost::filesystem::path *""'"); 
  }
  arg2 = reinterpret_cast< boost::filesystem::path * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->system_run_dir = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectories_system_run_dir_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories *arg1 = (RobotRaconteur::NodeDirectories *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::filesystem::path *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeDirectories, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectories_system_run_dir_get" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectories *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectories * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::filesystem::path *)& ((arg1)->system_run_dir);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_boost__filesystem__path, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectories_user_data_dir_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories *arg1 = (RobotRaconteur::NodeDirectories *) 0 ;
  boost::filesystem::path *arg2 = (boost::filesystem::path *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "NodeDirectories_user_data_dir_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeDirectories, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectories_user_data_dir_set" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectories *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectories * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_boost__filesystem__path, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "NodeDirectories_user_data_dir_set" "', argument " "2"" of type '" "boost::filesystem::path *""'"); 
  }
  arg2 = reinterpret_cast< boost::filesystem::path * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->user_data_dir = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectories_user_data_dir_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories *arg1 = (RobotRaconteur::NodeDirectories *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::filesystem::path *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeDirectories, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectories_user_data_dir_get" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectories *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectories * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::filesystem::path *)& ((arg1)->user_data_dir);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_boost__filesystem__path, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectories_user_config_dir_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories *arg1 = (RobotRaconteur::NodeDirectories *) 0 ;
  boost::filesystem::path *arg2 = (boost::filesystem::path *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "NodeDirectories_user_config_dir_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeDirectories, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectories_user_config_dir_set" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectories *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectories * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_boost__filesystem__path, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "NodeDirectories_user_config_dir_set" "', argument " "2"" of type '" "boost::filesystem::path *""'"); 
  }
  arg2 = reinterpret_cast< boost::filesystem::path * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->user_config_dir = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectories_user_config_dir_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories *arg1 = (RobotRaconteur::NodeDirectories *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::filesystem::path *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeDirectories, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectories_user_config_dir_get" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectories *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectories * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::filesystem::path *)& ((arg1)->user_config_dir);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_boost__filesystem__path, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectories_user_state_dir_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories *arg1 = (RobotRaconteur::NodeDirectories *) 0 ;
  boost::filesystem::path *arg2 = (boost::filesystem::path *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "NodeDirectories_user_state_dir_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeDirectories, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectories_user_state_dir_set" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectories *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectories * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_boost__filesystem__path, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "NodeDirectories_user_state_dir_set" "', argument " "2"" of type '" "boost::filesystem::path *""'"); 
  }
  arg2 = reinterpret_cast< boost::filesystem::path * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->user_state_dir = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectories_user_state_dir_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories *arg1 = (RobotRaconteur::NodeDirectories *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::filesystem::path *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeDirectories, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectories_user_state_dir_get" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectories *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectories * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::filesystem::path *)& ((arg1)->user_state_dir);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_boost__filesystem__path, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectories_user_cache_dir_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories *arg1 = (RobotRaconteur::NodeDirectories *) 0 ;
  boost::filesystem::path *arg2 = (boost::filesystem::path *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "NodeDirectories_user_cache_dir_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeDirectories, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectories_user_cache_dir_set" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectories *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectories * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_boost__filesystem__path, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "NodeDirectories_user_cache_dir_set" "', argument " "2"" of type '" "boost::filesystem::path *""'"); 
  }
  arg2 = reinterpret_cast< boost::filesystem::path * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->user_cache_dir = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectories_user_cache_dir_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories *arg1 = (RobotRaconteur::NodeDirectories *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::filesystem::path *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeDirectories, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectories_user_cache_dir_get" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectories *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectories * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::filesystem::path *)& ((arg1)->user_cache_dir);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_boost__filesystem__path, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectories_user_run_dir_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories *arg1 = (RobotRaconteur::NodeDirectories *) 0 ;
  boost::filesystem::path *arg2 = (boost::filesystem::path *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "NodeDirectories_user_run_dir_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeDirectories, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectories_user_run_dir_set" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectories *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectories * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_boost__filesystem__path, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "NodeDirectories_user_run_dir_set" "', argument " "2"" of type '" "boost::filesystem::path *""'"); 
  }
  arg2 = reinterpret_cast< boost::filesystem::path * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->user_run_dir = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectories_user_run_dir_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories *arg1 = (RobotRaconteur::NodeDirectories *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::filesystem::path *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeDirectories, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectories_user_run_dir_get" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectories *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectories * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::filesystem::path *)& ((arg1)->user_run_dir);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_boost__filesystem__path, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_NodeDirectories(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_NodeDirectories", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::NodeDirectories *)new RobotRaconteur::NodeDirectories();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__NodeDirectories, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_NodeDirectories(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories *arg1 = (RobotRaconteur::NodeDirectories *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeDirectories, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_NodeDirectories" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectories *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectories * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *NodeDirectories_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__NodeDirectories, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *NodeDirectories_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_delete_NodeDirectoriesFD(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectoriesFD *arg1 = (RobotRaconteur::NodeDirectoriesFD *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::NodeDirectoriesFD > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::NodeDirectoriesFD > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__NodeDirectoriesFD_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_NodeDirectoriesFD" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectoriesFD *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::NodeDirectoriesFD > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::NodeDirectoriesFD > * >(argp1);
      arg1 = const_cast< RobotRaconteur::NodeDirectoriesFD * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::NodeDirectoriesFD > * >(argp1);
      arg1 = const_cast< RobotRaconteur::NodeDirectoriesFD * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *NodeDirectoriesFD_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__NodeDirectoriesFD_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *_wrap_NodeDirectoriesUtil_GetDefaultNodeDirectories__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > arg1 ;
  void *argp1 ;
  int res1 = 0 ;
  RobotRaconteur::NodeDirectories result;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectoriesUtil_GetDefaultNodeDirectories" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode >""'");
    }
    if (argp1) arg1 = *(reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1));
    if (newmem & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::NodeDirectoriesUtil::GetDefaultNodeDirectories(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::NodeDirectories(static_cast< const RobotRaconteur::NodeDirectories& >(result))), SWIGTYPE_p_RobotRaconteur__NodeDirectories, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectoriesUtil_GetDefaultNodeDirectories__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **SWIGUNUSEDPARM(swig_obj)) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories result;
  
  if ((nobjs < 0) || (nobjs > 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::NodeDirectoriesUtil::GetDefaultNodeDirectories();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::NodeDirectories(static_cast< const RobotRaconteur::NodeDirectories& >(result))), SWIGTYPE_p_RobotRaconteur__NodeDirectories, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectoriesUtil_GetDefaultNodeDirectories(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[2] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "NodeDirectoriesUtil_GetDefaultNodeDirectories", 0, 1, argv))) SWIG_fail;
  --argc;
  if (argc == 0) {
    return _wrap_NodeDirectoriesUtil_GetDefaultNodeDirectories__SWIG_1(self, argc, argv);
  }
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_NodeDirectoriesUtil_GetDefaultNodeDirectories__SWIG_0(self, argc, argv);
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'NodeDirectoriesUtil_GetDefaultNodeDirectories'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::NodeDirectoriesUtil::GetDefaultNodeDirectories(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode >)\n"
    "    RobotRaconteur::NodeDirectoriesUtil::GetDefaultNodeDirectories()\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_NodeDirectoriesUtil_GetLogonUserName(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::string result;
  
  if (!SWIG_Python_UnpackTuple(args, "NodeDirectoriesUtil_GetLogonUserName", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::NodeDirectoriesUtil::GetLogonUserName();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectoriesUtil_IsLogonUserRoot(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "NodeDirectoriesUtil_IsLogonUserRoot", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)RobotRaconteur::NodeDirectoriesUtil::IsLogonUserRoot();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectoriesUtil_ReadInfoFile(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::filesystem::path *arg1 = 0 ;
  std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "NodeDirectoriesUtil_ReadInfoFile", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_boost__filesystem__path,  0  | 0);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectoriesUtil_ReadInfoFile" "', argument " "1"" of type '" "boost::filesystem::path const &""'"); 
  }
  if (!argp1) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "NodeDirectoriesUtil_ReadInfoFile" "', argument " "1"" of type '" "boost::filesystem::path const &""'"); 
  }
  arg1 = reinterpret_cast< boost::filesystem::path * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t,  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "NodeDirectoriesUtil_ReadInfoFile" "', argument " "2"" of type '" "std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "NodeDirectoriesUtil_ReadInfoFile" "', argument " "2"" of type '" "std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > &""'"); 
  }
  arg2 = reinterpret_cast< std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > * >(argp2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)RobotRaconteur::NodeDirectoriesUtil::ReadInfoFile((boost::filesystem::path const &)*arg1,*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectoriesUtil_CreatePidFile(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::filesystem::path *arg1 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::NodeDirectoriesFD > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_boost__filesystem__path,  0  | 0);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectoriesUtil_CreatePidFile" "', argument " "1"" of type '" "boost::filesystem::path const &""'"); 
  }
  if (!argp1) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "NodeDirectoriesUtil_CreatePidFile" "', argument " "1"" of type '" "boost::filesystem::path const &""'"); 
  }
  arg1 = reinterpret_cast< boost::filesystem::path * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::NodeDirectoriesUtil::CreatePidFile((boost::filesystem::path const &)*arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::NodeDirectoriesFD > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::NodeDirectoriesFD >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__NodeDirectoriesFD_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectoriesUtil_CreateInfoFile(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::filesystem::path *arg1 = 0 ;
  std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::NodeDirectoriesFD > result;
  
  if (!SWIG_Python_UnpackTuple(args, "NodeDirectoriesUtil_CreateInfoFile", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_boost__filesystem__path,  0  | 0);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectoriesUtil_CreateInfoFile" "', argument " "1"" of type '" "boost::filesystem::path const &""'"); 
  }
  if (!argp1) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "NodeDirectoriesUtil_CreateInfoFile" "', argument " "1"" of type '" "boost::filesystem::path const &""'"); 
  }
  arg1 = reinterpret_cast< boost::filesystem::path * >(argp1);
  {
    std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > *ptr = (std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > *)0;
    int res = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res) || !ptr) {
      SWIG_exception_fail(SWIG_ArgError((ptr ? res : SWIG_TypeError)), "in method '" "NodeDirectoriesUtil_CreateInfoFile" "', argument " "2"" of type '" "std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > >""'"); 
    }
    arg2 = *ptr;
    if (SWIG_IsNewObj(res)) delete ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur::NodeDirectoriesUtil::CreateInfoFile((boost::filesystem::path const &)*arg1,arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::NodeDirectoriesFD > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::NodeDirectoriesFD >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__NodeDirectoriesFD_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_NodeDirectoriesUtil_RefreshInfoFile(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::NodeDirectoriesFD > *arg1 = 0 ;
  std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > *arg2 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::NodeDirectoriesFD > tempshared1 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "NodeDirectoriesUtil_RefreshInfoFile", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__NodeDirectoriesFD_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "NodeDirectoriesUtil_RefreshInfoFile" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::NodeDirectoriesFD > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::NodeDirectoriesFD > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::NodeDirectoriesFD > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::NodeDirectoriesFD > * >(argp1) : &tempshared1;
    }
  }
  {
    std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > *ptr = (std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "NodeDirectoriesUtil_RefreshInfoFile" "', argument " "2"" of type '" "std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "NodeDirectoriesUtil_RefreshInfoFile" "', argument " "2"" of type '" "std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur::NodeDirectoriesUtil::RefreshInfoFile((boost::shared_ptr< RobotRaconteur::NodeDirectoriesFD > const &)*arg1,(std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_NodeDirectoriesUtil(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectoriesUtil *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_NodeDirectoriesUtil", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::NodeDirectoriesUtil *)new RobotRaconteur::NodeDirectoriesUtil();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__NodeDirectoriesUtil, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_NodeDirectoriesUtil(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectoriesUtil *arg1 = (RobotRaconteur::NodeDirectoriesUtil *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeDirectoriesUtil, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_NodeDirectoriesUtil" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectoriesUtil *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectoriesUtil * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *NodeDirectoriesUtil_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__NodeDirectoriesUtil, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *NodeDirectoriesUtil_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_GetUuidForNameAndLockResult_uuid_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::GetUuidForNameAndLockResult *arg1 = (RobotRaconteur::GetUuidForNameAndLockResult *) 0 ;
  RobotRaconteur::NodeID *arg2 = (RobotRaconteur::NodeID *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "GetUuidForNameAndLockResult_uuid_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__GetUuidForNameAndLockResult, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "GetUuidForNameAndLockResult_uuid_set" "', argument " "1"" of type '" "RobotRaconteur::GetUuidForNameAndLockResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::GetUuidForNameAndLockResult * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__NodeID, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "GetUuidForNameAndLockResult_uuid_set" "', argument " "2"" of type '" "RobotRaconteur::NodeID *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::NodeID * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->uuid = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_GetUuidForNameAndLockResult_uuid_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::GetUuidForNameAndLockResult *arg1 = (RobotRaconteur::GetUuidForNameAndLockResult *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::NodeID *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__GetUuidForNameAndLockResult, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "GetUuidForNameAndLockResult_uuid_get" "', argument " "1"" of type '" "RobotRaconteur::GetUuidForNameAndLockResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::GetUuidForNameAndLockResult * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (RobotRaconteur::NodeID *)& ((arg1)->uuid);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__NodeID, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_GetUuidForNameAndLockResult_name_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::GetUuidForNameAndLockResult *arg1 = (RobotRaconteur::GetUuidForNameAndLockResult *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "GetUuidForNameAndLockResult_name_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__GetUuidForNameAndLockResult, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "GetUuidForNameAndLockResult_name_set" "', argument " "1"" of type '" "RobotRaconteur::GetUuidForNameAndLockResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::GetUuidForNameAndLockResult * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "GetUuidForNameAndLockResult_name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "GetUuidForNameAndLockResult_name_set" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->name = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_GetUuidForNameAndLockResult_name_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::GetUuidForNameAndLockResult *arg1 = (RobotRaconteur::GetUuidForNameAndLockResult *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__GetUuidForNameAndLockResult, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "GetUuidForNameAndLockResult_name_get" "', argument " "1"" of type '" "RobotRaconteur::GetUuidForNameAndLockResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::GetUuidForNameAndLockResult * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::string *) & ((arg1)->name);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_std_string(static_cast< std::string >(*result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_GetUuidForNameAndLockResult_scope_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::GetUuidForNameAndLockResult *arg1 = (RobotRaconteur::GetUuidForNameAndLockResult *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = (std::vector< std::string,std::allocator< std::string > > *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "GetUuidForNameAndLockResult_scope_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__GetUuidForNameAndLockResult, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "GetUuidForNameAndLockResult_scope_set" "', argument " "1"" of type '" "RobotRaconteur::GetUuidForNameAndLockResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::GetUuidForNameAndLockResult * >(argp1);
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "GetUuidForNameAndLockResult_scope_set" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > *""'"); 
  }
  arg2 = reinterpret_cast< std::vector< std::string,std::allocator< std::string > > * >(argp2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->scope = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_GetUuidForNameAndLockResult_scope_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::GetUuidForNameAndLockResult *arg1 = (RobotRaconteur::GetUuidForNameAndLockResult *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__GetUuidForNameAndLockResult, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "GetUuidForNameAndLockResult_scope_get" "', argument " "1"" of type '" "RobotRaconteur::GetUuidForNameAndLockResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::GetUuidForNameAndLockResult * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (std::vector< std::string,std::allocator< std::string > > *)& ((arg1)->scope);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0 |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_GetUuidForNameAndLockResult_fd_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::GetUuidForNameAndLockResult *arg1 = (RobotRaconteur::GetUuidForNameAndLockResult *) 0 ;
  boost::shared_ptr< RobotRaconteur::NodeDirectoriesFD > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::NodeDirectoriesFD > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "GetUuidForNameAndLockResult_fd_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__GetUuidForNameAndLockResult, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "GetUuidForNameAndLockResult_fd_set" "', argument " "1"" of type '" "RobotRaconteur::GetUuidForNameAndLockResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::GetUuidForNameAndLockResult * >(argp1);
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__NodeDirectoriesFD_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "GetUuidForNameAndLockResult_fd_set" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::NodeDirectoriesFD > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::NodeDirectoriesFD > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::NodeDirectoriesFD > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::NodeDirectoriesFD > * >(argp2) : &tempshared2;
    }
  }
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->fd = *arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_GetUuidForNameAndLockResult_fd_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::GetUuidForNameAndLockResult *arg1 = (RobotRaconteur::GetUuidForNameAndLockResult *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::NodeDirectoriesFD > *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__GetUuidForNameAndLockResult, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "GetUuidForNameAndLockResult_fd_get" "', argument " "1"" of type '" "RobotRaconteur::GetUuidForNameAndLockResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::GetUuidForNameAndLockResult * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (boost::shared_ptr< RobotRaconteur::NodeDirectoriesFD > *) & ((arg1)->fd);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  {
    boost::shared_ptr<  RobotRaconteur::NodeDirectoriesFD > *smartresult = *result ? new boost::shared_ptr<  RobotRaconteur::NodeDirectoriesFD >(*result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__NodeDirectoriesFD_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_GetUuidForNameAndLockResult(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::GetUuidForNameAndLockResult *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_GetUuidForNameAndLockResult", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::GetUuidForNameAndLockResult *)new RobotRaconteur::GetUuidForNameAndLockResult();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__GetUuidForNameAndLockResult, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_GetUuidForNameAndLockResult(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::GetUuidForNameAndLockResult *arg1 = (RobotRaconteur::GetUuidForNameAndLockResult *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__GetUuidForNameAndLockResult, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_GetUuidForNameAndLockResult" "', argument " "1"" of type '" "RobotRaconteur::GetUuidForNameAndLockResult *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::GetUuidForNameAndLockResult * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *GetUuidForNameAndLockResult_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__GetUuidForNameAndLockResult, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *GetUuidForNameAndLockResult_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_GetUuidForNameAndLock(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectories *arg1 = 0 ;
  std::string *arg2 = 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  PyObject *swig_obj[3] ;
  RobotRaconteur::GetUuidForNameAndLockResult result;
  
  if (!SWIG_Python_UnpackTuple(args, "GetUuidForNameAndLock", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1, SWIGTYPE_p_RobotRaconteur__NodeDirectories,  0  | 0);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "GetUuidForNameAndLock" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectories const &""'"); 
  }
  if (!argp1) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "GetUuidForNameAndLock" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectories const &""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectories * >(argp1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "GetUuidForNameAndLock" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "GetUuidForNameAndLock" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res3 = swig::asptr(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "GetUuidForNameAndLock" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "GetUuidForNameAndLock" "', argument " "3"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    result = RobotRaconteur::GetUuidForNameAndLock((RobotRaconteur::NodeDirectories const &)*arg1,(std::string const &)*arg2,(std::vector< std::string,std::allocator< std::string > > const &)*arg3);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::GetUuidForNameAndLockResult(static_cast< const RobotRaconteur::GetUuidForNameAndLockResult& >(result))), SWIGTYPE_p_RobotRaconteur__GetUuidForNameAndLockResult, SWIG_POINTER_OWN |  0 );
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_NodeDirectoriesResourceAlreadyInUse(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectoriesResourceAlreadyInUse *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_NodeDirectoriesResourceAlreadyInUse", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::NodeDirectoriesResourceAlreadyInUse *)new RobotRaconteur::NodeDirectoriesResourceAlreadyInUse();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__NodeDirectoriesResourceAlreadyInUse, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_NodeDirectoriesResourceAlreadyInUse(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::NodeDirectoriesResourceAlreadyInUse *arg1 = (RobotRaconteur::NodeDirectoriesResourceAlreadyInUse *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__NodeDirectoriesResourceAlreadyInUse, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_NodeDirectoriesResourceAlreadyInUse" "', argument " "1"" of type '" "RobotRaconteur::NodeDirectoriesResourceAlreadyInUse *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::NodeDirectoriesResourceAlreadyInUse * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *NodeDirectoriesResourceAlreadyInUse_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__NodeDirectoriesResourceAlreadyInUse, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *NodeDirectoriesResourceAlreadyInUse_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_ClientServiceListenerDirector_objectheapid_set(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  ClientServiceListenerDirector *arg1 = (ClientServiceListenerDirector *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "ClientServiceListenerDirector_objectheapid_set", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_ClientServiceListenerDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ClientServiceListenerDirector_objectheapid_set" "', argument " "1"" of type '" "ClientServiceListenerDirector *""'"); 
  }
  arg1 = reinterpret_cast< ClientServiceListenerDirector * >(argp1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ClientServiceListenerDirector_objectheapid_set" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    if (arg1) (arg1)->objectheapid = arg2;
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ClientServiceListenerDirector_objectheapid_get(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  ClientServiceListenerDirector *arg1 = (ClientServiceListenerDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_ClientServiceListenerDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ClientServiceListenerDirector_objectheapid_get" "', argument " "1"" of type '" "ClientServiceListenerDirector *""'"); 
  }
  arg1 = reinterpret_cast< ClientServiceListenerDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    result = (int32_t) ((arg1)->objectheapid);
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_ClientServiceListenerDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  PyObject *swig_obj[1] ;
  ClientServiceListenerDirector *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    arg1 = swig_obj[0];
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      if ( arg1 != Py_None ) {
        /* subclassed */
        result = (ClientServiceListenerDirector *)new SwigDirector_ClientServiceListenerDirector(arg1); 
      } else {
        result = (ClientServiceListenerDirector *)new ClientServiceListenerDirector(); 
      }
      
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_ClientServiceListenerDirector, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ClientServiceListenerDirector_Callback(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  ClientServiceListenerDirector *arg1 = (ClientServiceListenerDirector *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "ClientServiceListenerDirector_Callback", 2, 2, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_ClientServiceListenerDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ClientServiceListenerDirector_Callback" "', argument " "1"" of type '" "ClientServiceListenerDirector *""'"); 
  }
  arg1 = reinterpret_cast< ClientServiceListenerDirector * >(argp1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ClientServiceListenerDirector_Callback" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        (arg1)->ClientServiceListenerDirector::Callback(arg2);
      } else {
        (arg1)->Callback(arg2);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_ClientServiceListenerDirector_Callback2(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  ClientServiceListenerDirector *arg1 = (ClientServiceListenerDirector *) 0 ;
  int32_t arg2 ;
  std::string *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  PyObject *swig_obj[3] ;
  Swig::Director *director = 0;
  bool upcall = false;
  
  if (!SWIG_Python_UnpackTuple(args, "ClientServiceListenerDirector_Callback2", 3, 3, swig_obj)) SWIG_fail;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_ClientServiceListenerDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "ClientServiceListenerDirector_Callback2" "', argument " "1"" of type '" "ClientServiceListenerDirector *""'"); 
  }
  arg1 = reinterpret_cast< ClientServiceListenerDirector * >(argp1);
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "ClientServiceListenerDirector_Callback2" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "ClientServiceListenerDirector_Callback2" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "ClientServiceListenerDirector_Callback2" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  director = SWIG_DIRECTOR_CAST(arg1);
  upcall = (director && (director->swig_get_self()==swig_obj[0]));
  try {
    try {
      if (upcall) {
        (arg1)->ClientServiceListenerDirector::Callback2(arg2,(std::string const &)*arg3);
      } else {
        (arg1)->Callback2(arg2,(std::string const &)*arg3);
      }
    }
    
    catch (RobotRaconteurException &e) {
      //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      RRExceptionToPythonError(e);
      goto fail;
    }
    catch (std::exception &e) {
      PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
      goto fail;
    }
    /*catch (...)
        {
    		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
    		goto fail;
        }*/
    
    
  } catch (Swig::DirectorException&) {
    SWIG_fail;
  }
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_ClientServiceListenerDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  ClientServiceListenerDirector *arg1 = (ClientServiceListenerDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_ClientServiceListenerDirector, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_ClientServiceListenerDirector" "', argument " "1"" of type '" "ClientServiceListenerDirector *""'"); 
  }
  arg1 = reinterpret_cast< ClientServiceListenerDirector * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_disown_ClientServiceListenerDirector(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  ClientServiceListenerDirector *arg1 = (ClientServiceListenerDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_ClientServiceListenerDirector, 0 |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "disown_ClientServiceListenerDirector" "', argument " "1"" of type '" "ClientServiceListenerDirector *""'"); 
  }
  arg1 = reinterpret_cast< ClientServiceListenerDirector * >(argp1);
  {
    SWIG_PYTHON_THREAD_BEGIN_ALLOW;
    {
      Swig::Director *director = SWIG_DIRECTOR_CAST(arg1);
      if (director) director->swig_disown();
    }
    
    SWIG_PYTHON_THREAD_END_ALLOW;
  }
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *ClientServiceListenerDirector_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_ClientServiceListenerDirector, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *ClientServiceListenerDirector_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_RobotRaconteurNode_RegisterTransport(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::Transport > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::Transport > tempshared2 ;
  PyObject *swig_obj[2] ;
  uint32_t result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode_RegisterTransport", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode_RegisterTransport" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Transport_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode_RegisterTransport" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::Transport > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::Transport > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::Transport > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::Transport > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (uint32_t)(arg1)->RegisterTransport((boost::shared_ptr< RobotRaconteur::Transport > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__GetRequestTimeout(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__GetRequestTimeout" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (uint32_t)(arg1)->GetRequestTimeout();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__SetRequestTimeout(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__SetRequestTimeout", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__SetRequestTimeout" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "RobotRaconteurNode__SetRequestTimeout" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetRequestTimeout(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__GetTransportInactivityTimeout(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__GetTransportInactivityTimeout" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (uint32_t)(arg1)->GetTransportInactivityTimeout();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__SetTransportInactivityTimeout(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__SetTransportInactivityTimeout", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__SetTransportInactivityTimeout" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "RobotRaconteurNode__SetTransportInactivityTimeout" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetTransportInactivityTimeout(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__GetEndpointInactivityTimeout(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__GetEndpointInactivityTimeout" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (uint32_t)(arg1)->GetEndpointInactivityTimeout();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__SetEndpointInactivityTimeout(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__SetEndpointInactivityTimeout", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__SetEndpointInactivityTimeout" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "RobotRaconteurNode__SetEndpointInactivityTimeout" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetEndpointInactivityTimeout(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__GetMemoryMaxTransferSize(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__GetMemoryMaxTransferSize" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (uint32_t)(arg1)->GetMemoryMaxTransferSize();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__SetMemoryMaxTransferSize(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__SetMemoryMaxTransferSize", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__SetMemoryMaxTransferSize" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "RobotRaconteurNode__SetMemoryMaxTransferSize" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetMemoryMaxTransferSize(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__GetNodeDiscoveryMaxCacheCount(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  uint32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__GetNodeDiscoveryMaxCacheCount" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (uint32_t)(arg1)->GetNodeDiscoveryMaxCacheCount();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__SetNodeDiscoveryMaxCacheCount(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__SetNodeDiscoveryMaxCacheCount", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__SetNodeDiscoveryMaxCacheCount" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "RobotRaconteurNode__SetNodeDiscoveryMaxCacheCount" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetNodeDiscoveryMaxCacheCount(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_UnregisterServiceType(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode_UnregisterServiceType", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode_UnregisterServiceType" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode_UnregisterServiceType" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode_UnregisterServiceType" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->UnregisterServiceType((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__ConnectService__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > arg4 ;
  ClientServiceListenerDirector *arg5 = (ClientServiceListenerDirector *) 0 ;
  std::string *arg6 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  void *argp4 ;
  int res4 = 0 ;
  int newmem4 = 0 ;
  void *argp5 = 0 ;
  int res5 = 0 ;
  int res6 = SWIG_OLDOBJ ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > result;
  
  if ((nobjs < 6) || (nobjs > 6)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__ConnectService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__ConnectService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[3];
  //if (smartarg) {
  //	arg4 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //}
  
  res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem4);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "4"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData >""'");
  }
  if (argp4) arg4 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4)->get(),true);
  if (newmem4 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4);
  
  res5 = SWIG_ConvertPtr(swig_obj[4], &argp5,SWIGTYPE_p_ClientServiceListenerDirector, 0 |  0 );
  if (!SWIG_IsOK(res5)) {
    SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "5"" of type '" "ClientServiceListenerDirector *""'"); 
  }
  arg5 = reinterpret_cast< ClientServiceListenerDirector * >(argp5);
  {
    std::string *ptr = (std::string *)0;
    res6 = SWIG_AsPtr_std_string(swig_obj[5], &ptr);
    if (!SWIG_IsOK(res6)) {
      SWIG_exception_fail(SWIG_ArgError(res6), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "6"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__ConnectService" "', argument " "6"" of type '" "std::string const &""'"); 
    }
    arg6 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_RobotRaconteurNode_ConnectService__SWIG_0(arg1,(std::string const &)*arg2,(std::string const &)*arg3,arg4,arg5,(std::string const &)*arg6);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res6)) delete arg6;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res6)) delete arg6;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__ConnectService__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > arg4 ;
  ClientServiceListenerDirector *arg5 = (ClientServiceListenerDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  void *argp4 ;
  int res4 = 0 ;
  int newmem4 = 0 ;
  void *argp5 = 0 ;
  int res5 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > result;
  
  if ((nobjs < 5) || (nobjs > 5)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__ConnectService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__ConnectService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[3];
  //if (smartarg) {
  //	arg4 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //}
  
  res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem4);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "4"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData >""'");
  }
  if (argp4) arg4 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4)->get(),true);
  if (newmem4 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4);
  
  res5 = SWIG_ConvertPtr(swig_obj[4], &argp5,SWIGTYPE_p_ClientServiceListenerDirector, 0 |  0 );
  if (!SWIG_IsOK(res5)) {
    SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "5"" of type '" "ClientServiceListenerDirector *""'"); 
  }
  arg5 = reinterpret_cast< ClientServiceListenerDirector * >(argp5);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_RobotRaconteurNode_ConnectService__SWIG_0(arg1,(std::string const &)*arg2,(std::string const &)*arg3,arg4,arg5);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__ConnectService__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  void *argp4 ;
  int res4 = 0 ;
  int newmem4 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > result;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__ConnectService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__ConnectService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[3];
  //if (smartarg) {
  //	arg4 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //}
  
  res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem4);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "4"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData >""'");
  }
  if (argp4) arg4 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4)->get(),true);
  if (newmem4 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4);
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_RobotRaconteurNode_ConnectService__SWIG_0(arg1,(std::string const &)*arg2,(std::string const &)*arg3,arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__ConnectService__SWIG_3(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__ConnectService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__ConnectService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_RobotRaconteurNode_ConnectService__SWIG_0(arg1,(std::string const &)*arg2,(std::string const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__ConnectService__SWIG_4(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__ConnectService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_RobotRaconteurNode_ConnectService__SWIG_0(arg1,(std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__ConnectService__SWIG_5(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  std::string *arg3 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > arg4 ;
  ClientServiceListenerDirector *arg5 = (ClientServiceListenerDirector *) 0 ;
  std::string *arg6 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  void *argp4 ;
  int res4 = 0 ;
  int newmem4 = 0 ;
  void *argp5 = 0 ;
  int res5 = 0 ;
  int res6 = SWIG_OLDOBJ ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > result;
  
  if ((nobjs < 6) || (nobjs > 6)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__ConnectService" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__ConnectService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[3];
  //if (smartarg) {
  //	arg4 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //}
  
  res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem4);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "4"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData >""'");
  }
  if (argp4) arg4 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4)->get(),true);
  if (newmem4 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4);
  
  res5 = SWIG_ConvertPtr(swig_obj[4], &argp5,SWIGTYPE_p_ClientServiceListenerDirector, 0 |  0 );
  if (!SWIG_IsOK(res5)) {
    SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "5"" of type '" "ClientServiceListenerDirector *""'"); 
  }
  arg5 = reinterpret_cast< ClientServiceListenerDirector * >(argp5);
  {
    std::string *ptr = (std::string *)0;
    res6 = SWIG_AsPtr_std_string(swig_obj[5], &ptr);
    if (!SWIG_IsOK(res6)) {
      SWIG_exception_fail(SWIG_ArgError(res6), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "6"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__ConnectService" "', argument " "6"" of type '" "std::string const &""'"); 
    }
    arg6 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_RobotRaconteurNode_ConnectService__SWIG_5(arg1,(std::vector< std::string,std::allocator< std::string > > const &)*arg2,(std::string const &)*arg3,arg4,arg5,(std::string const &)*arg6);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res6)) delete arg6;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res6)) delete arg6;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__ConnectService__SWIG_6(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  std::string *arg3 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > arg4 ;
  ClientServiceListenerDirector *arg5 = (ClientServiceListenerDirector *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  void *argp4 ;
  int res4 = 0 ;
  int newmem4 = 0 ;
  void *argp5 = 0 ;
  int res5 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > result;
  
  if ((nobjs < 5) || (nobjs > 5)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__ConnectService" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__ConnectService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[3];
  //if (smartarg) {
  //	arg4 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //}
  
  res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem4);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "4"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData >""'");
  }
  if (argp4) arg4 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4)->get(),true);
  if (newmem4 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4);
  
  res5 = SWIG_ConvertPtr(swig_obj[4], &argp5,SWIGTYPE_p_ClientServiceListenerDirector, 0 |  0 );
  if (!SWIG_IsOK(res5)) {
    SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "5"" of type '" "ClientServiceListenerDirector *""'"); 
  }
  arg5 = reinterpret_cast< ClientServiceListenerDirector * >(argp5);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_RobotRaconteurNode_ConnectService__SWIG_5(arg1,(std::vector< std::string,std::allocator< std::string > > const &)*arg2,(std::string const &)*arg3,arg4,arg5);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__ConnectService__SWIG_7(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  std::string *arg3 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  void *argp4 ;
  int res4 = 0 ;
  int newmem4 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > result;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__ConnectService" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__ConnectService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  // intrusive_ptr by value
  //smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[3];
  //if (smartarg) {
  //	arg4 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //}
  
  res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem4);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "4"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData >""'");
  }
  if (argp4) arg4 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4)->get(),true);
  if (newmem4 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4);
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_RobotRaconteurNode_ConnectService__SWIG_5(arg1,(std::vector< std::string,std::allocator< std::string > > const &)*arg2,(std::string const &)*arg3,arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__ConnectService__SWIG_8(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  std::string *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__ConnectService" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__ConnectService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_RobotRaconteurNode_ConnectService__SWIG_5(arg1,(std::vector< std::string,std::allocator< std::string > > const &)*arg2,(std::string const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__ConnectService__SWIG_9(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__ConnectService" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__ConnectService" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_RobotRaconteurNode_ConnectService__SWIG_5(arg1,(std::vector< std::string,std::allocator< std::string > > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::WrappedServiceStub > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::WrappedServiceStub >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__ConnectService(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[7] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__ConnectService", 0, 6, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_RobotRaconteurNode__ConnectService__SWIG_4(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_RobotRaconteurNode__ConnectService__SWIG_9(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_RobotRaconteurNode__ConnectService__SWIG_3(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_RobotRaconteurNode__ConnectService__SWIG_8(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_RobotRaconteurNode__ConnectService__SWIG_2(self, argc, argv);
          }
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_RobotRaconteurNode__ConnectService__SWIG_7(self, argc, argv);
          }
        }
      }
    }
  }
  if (argc == 5) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            void *vptr = 0;
            int res = SWIG_ConvertPtr(argv[4], &vptr, SWIGTYPE_p_ClientServiceListenerDirector, 0);
            _v = SWIG_CheckState(res);
            if (_v) {
              return _wrap_RobotRaconteurNode__ConnectService__SWIG_1(self, argc, argv);
            }
          }
        }
      }
    }
  }
  if (argc == 5) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            void *vptr = 0;
            int res = SWIG_ConvertPtr(argv[4], &vptr, SWIGTYPE_p_ClientServiceListenerDirector, 0);
            _v = SWIG_CheckState(res);
            if (_v) {
              return _wrap_RobotRaconteurNode__ConnectService__SWIG_6(self, argc, argv);
            }
          }
        }
      }
    }
  }
  if (argc == 6) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            void *vptr = 0;
            int res = SWIG_ConvertPtr(argv[4], &vptr, SWIGTYPE_p_ClientServiceListenerDirector, 0);
            _v = SWIG_CheckState(res);
            if (_v) {
              int res = SWIG_AsPtr_std_string(argv[5], (std::string**)(0));
              _v = SWIG_CheckState(res);
              if (_v) {
                return _wrap_RobotRaconteurNode__ConnectService__SWIG_0(self, argc, argv);
              }
            }
          }
        }
      }
    }
  }
  if (argc == 6) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            void *vptr = 0;
            int res = SWIG_ConvertPtr(argv[4], &vptr, SWIGTYPE_p_ClientServiceListenerDirector, 0);
            _v = SWIG_CheckState(res);
            if (_v) {
              int res = SWIG_AsPtr_std_string(argv[5], (std::string**)(0));
              _v = SWIG_CheckState(res);
              if (_v) {
                return _wrap_RobotRaconteurNode__ConnectService__SWIG_5(self, argc, argv);
              }
            }
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'RobotRaconteurNode__ConnectService'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::RobotRaconteurNode::ConnectService(std::string const &,std::string const &,boost::intrusive_ptr< RobotRaconteur::MessageElementData >,ClientServiceListenerDirector *,std::string const &)\n"
    "    RobotRaconteur::RobotRaconteurNode::ConnectService(std::string const &,std::string const &,boost::intrusive_ptr< RobotRaconteur::MessageElementData >,ClientServiceListenerDirector *)\n"
    "    RobotRaconteur::RobotRaconteurNode::ConnectService(std::string const &,std::string const &,boost::intrusive_ptr< RobotRaconteur::MessageElementData >)\n"
    "    RobotRaconteur::RobotRaconteurNode::ConnectService(std::string const &,std::string const &)\n"
    "    RobotRaconteur::RobotRaconteurNode::ConnectService(std::string const &)\n"
    "    RobotRaconteur::RobotRaconteurNode::ConnectService(std::vector< std::string,std::allocator< std::string > > const &,std::string const &,boost::intrusive_ptr< RobotRaconteur::MessageElementData >,ClientServiceListenerDirector *,std::string const &)\n"
    "    RobotRaconteur::RobotRaconteurNode::ConnectService(std::vector< std::string,std::allocator< std::string > > const &,std::string const &,boost::intrusive_ptr< RobotRaconteur::MessageElementData >,ClientServiceListenerDirector *)\n"
    "    RobotRaconteur::RobotRaconteurNode::ConnectService(std::vector< std::string,std::allocator< std::string > > const &,std::string const &,boost::intrusive_ptr< RobotRaconteur::MessageElementData >)\n"
    "    RobotRaconteur::RobotRaconteurNode::ConnectService(std::vector< std::string,std::allocator< std::string > > const &,std::string const &)\n"
    "    RobotRaconteur::RobotRaconteurNode::ConnectService(std::vector< std::string,std::allocator< std::string > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__AsyncConnectService__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > *arg4 = 0 ;
  ClientServiceListenerDirector *arg5 = (ClientServiceListenerDirector *) 0 ;
  std::string *arg6 = 0 ;
  int32_t arg7 ;
  RobotRaconteur::AsyncStubReturnDirector *arg8 = (RobotRaconteur::AsyncStubReturnDirector *) 0 ;
  int32_t arg9 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  void *argp4 ;
  int res4 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > tempnull4 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > temp4 ;
  int newmem4 = 0 ;
  void *argp5 = 0 ;
  int res5 = 0 ;
  int res6 = SWIG_OLDOBJ ;
  int val7 ;
  int ecode7 = 0 ;
  void *argp8 = 0 ;
  int res8 = 0 ;
  int val9 ;
  int ecode9 = 0 ;
  
  if ((nobjs < 9) || (nobjs > 9)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[3] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[3];
  //	temp4 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //	arg4 = &temp4;
  //} else {
  //  arg4 = &tempnull4;
  //}
  
  res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem4);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "4"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &""'");
  }
  if (argp4) {
    temp4 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4)->get(),true);
    if (newmem4 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4);
    arg4 = &temp4;
  }
  else {
    arg4 = &tempnull4;
  }
  
  res5 = SWIG_ConvertPtr(swig_obj[4], &argp5,SWIGTYPE_p_ClientServiceListenerDirector, 0 |  0 );
  if (!SWIG_IsOK(res5)) {
    SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "5"" of type '" "ClientServiceListenerDirector *""'"); 
  }
  arg5 = reinterpret_cast< ClientServiceListenerDirector * >(argp5);
  {
    std::string *ptr = (std::string *)0;
    res6 = SWIG_AsPtr_std_string(swig_obj[5], &ptr);
    if (!SWIG_IsOK(res6)) {
      SWIG_exception_fail(SWIG_ArgError(res6), "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "6"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "6"" of type '" "std::string const &""'"); 
    }
    arg6 = ptr;
  }
  ecode7 = SWIG_AsVal_int(swig_obj[6], &val7);
  if (!SWIG_IsOK(ecode7)) {
    SWIG_exception_fail(SWIG_ArgError(ecode7), "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "7"" of type '" "int32_t""'");
  } 
  arg7 = static_cast< int32_t >(val7);
  res8 = SWIG_ConvertPtr(swig_obj[7], &argp8,SWIGTYPE_p_RobotRaconteur__AsyncStubReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res8)) {
    SWIG_exception_fail(SWIG_ArgError(res8), "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "8"" of type '" "RobotRaconteur::AsyncStubReturnDirector *""'"); 
  }
  arg8 = reinterpret_cast< RobotRaconteur::AsyncStubReturnDirector * >(argp8);
  ecode9 = SWIG_AsVal_int(swig_obj[8], &val9);
  if (!SWIG_IsOK(ecode9)) {
    SWIG_exception_fail(SWIG_ArgError(ecode9), "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "9"" of type '" "int32_t""'");
  } 
  arg9 = static_cast< int32_t >(val9);
  
  try {
    RobotRaconteur_RobotRaconteurNode_AsyncConnectService__SWIG_0(arg1,(std::string const &)*arg2,(std::string const &)*arg3,(boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &)*arg4,arg5,(std::string const &)*arg6,arg7,arg8,arg9);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res6)) delete arg6;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res6)) delete arg6;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__AsyncConnectService__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  std::string *arg3 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > *arg4 = 0 ;
  ClientServiceListenerDirector *arg5 = (ClientServiceListenerDirector *) 0 ;
  std::string *arg6 = 0 ;
  int32_t arg7 ;
  RobotRaconteur::AsyncStubReturnDirector *arg8 = (RobotRaconteur::AsyncStubReturnDirector *) 0 ;
  int32_t arg9 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  void *argp4 ;
  int res4 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > tempnull4 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElementData > temp4 ;
  int newmem4 = 0 ;
  void *argp5 = 0 ;
  int res5 = 0 ;
  int res6 = SWIG_OLDOBJ ;
  int val7 ;
  int ecode7 = 0 ;
  void *argp8 = 0 ;
  int res8 = 0 ;
  int val9 ;
  int ecode9 = 0 ;
  
  if ((nobjs < 9) || (nobjs > 9)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[3] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElementData >**)&swig_obj[3];
  //	temp4 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElementData >(smartarg->get(), true);
  //	arg4 = &temp4;
  //} else {
  //  arg4 = &tempnull4;
  //}
  
  res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,  0 , &newmem4);
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "4"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &""'");
  }
  if (argp4) {
    temp4 = boost::intrusive_ptr<  RobotRaconteur::MessageElementData >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4)->get(),true);
    if (newmem4 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElementData >* >(argp4);
    arg4 = &temp4;
  }
  else {
    arg4 = &tempnull4;
  }
  
  res5 = SWIG_ConvertPtr(swig_obj[4], &argp5,SWIGTYPE_p_ClientServiceListenerDirector, 0 |  0 );
  if (!SWIG_IsOK(res5)) {
    SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "5"" of type '" "ClientServiceListenerDirector *""'"); 
  }
  arg5 = reinterpret_cast< ClientServiceListenerDirector * >(argp5);
  {
    std::string *ptr = (std::string *)0;
    res6 = SWIG_AsPtr_std_string(swig_obj[5], &ptr);
    if (!SWIG_IsOK(res6)) {
      SWIG_exception_fail(SWIG_ArgError(res6), "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "6"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "6"" of type '" "std::string const &""'"); 
    }
    arg6 = ptr;
  }
  ecode7 = SWIG_AsVal_int(swig_obj[6], &val7);
  if (!SWIG_IsOK(ecode7)) {
    SWIG_exception_fail(SWIG_ArgError(ecode7), "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "7"" of type '" "int32_t""'");
  } 
  arg7 = static_cast< int32_t >(val7);
  res8 = SWIG_ConvertPtr(swig_obj[7], &argp8,SWIGTYPE_p_RobotRaconteur__AsyncStubReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res8)) {
    SWIG_exception_fail(SWIG_ArgError(res8), "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "8"" of type '" "RobotRaconteur::AsyncStubReturnDirector *""'"); 
  }
  arg8 = reinterpret_cast< RobotRaconteur::AsyncStubReturnDirector * >(argp8);
  ecode9 = SWIG_AsVal_int(swig_obj[8], &val9);
  if (!SWIG_IsOK(ecode9)) {
    SWIG_exception_fail(SWIG_ArgError(ecode9), "in method '" "RobotRaconteurNode__AsyncConnectService" "', argument " "9"" of type '" "int32_t""'");
  } 
  arg9 = static_cast< int32_t >(val9);
  
  try {
    RobotRaconteur_RobotRaconteurNode_AsyncConnectService__SWIG_1(arg1,(std::vector< std::string,std::allocator< std::string > > const &)*arg2,(std::string const &)*arg3,(boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &)*arg4,arg5,(std::string const &)*arg6,arg7,arg8,arg9);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res6)) delete arg6;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res6)) delete arg6;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__AsyncConnectService(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[10] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__AsyncConnectService", 0, 9, argv))) SWIG_fail;
  --argc;
  if (argc == 9) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            void *vptr = 0;
            int res = SWIG_ConvertPtr(argv[4], &vptr, SWIGTYPE_p_ClientServiceListenerDirector, 0);
            _v = SWIG_CheckState(res);
            if (_v) {
              int res = SWIG_AsPtr_std_string(argv[5], (std::string**)(0));
              _v = SWIG_CheckState(res);
              if (_v) {
                {
                  int res = SWIG_AsVal_int(argv[6], NULL);
                  _v = SWIG_CheckState(res);
                }
                if (_v) {
                  void *vptr = 0;
                  int res = SWIG_ConvertPtr(argv[7], &vptr, SWIGTYPE_p_RobotRaconteur__AsyncStubReturnDirector, 0);
                  _v = SWIG_CheckState(res);
                  if (_v) {
                    {
                      int res = SWIG_AsVal_int(argv[8], NULL);
                      _v = SWIG_CheckState(res);
                    }
                    if (_v) {
                      return _wrap_RobotRaconteurNode__AsyncConnectService__SWIG_0(self, argc, argv);
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  }
  if (argc == 9) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            void *vptr = 0;
            int res = SWIG_ConvertPtr(argv[4], &vptr, SWIGTYPE_p_ClientServiceListenerDirector, 0);
            _v = SWIG_CheckState(res);
            if (_v) {
              int res = SWIG_AsPtr_std_string(argv[5], (std::string**)(0));
              _v = SWIG_CheckState(res);
              if (_v) {
                {
                  int res = SWIG_AsVal_int(argv[6], NULL);
                  _v = SWIG_CheckState(res);
                }
                if (_v) {
                  void *vptr = 0;
                  int res = SWIG_ConvertPtr(argv[7], &vptr, SWIGTYPE_p_RobotRaconteur__AsyncStubReturnDirector, 0);
                  _v = SWIG_CheckState(res);
                  if (_v) {
                    {
                      int res = SWIG_AsVal_int(argv[8], NULL);
                      _v = SWIG_CheckState(res);
                    }
                    if (_v) {
                      return _wrap_RobotRaconteurNode__AsyncConnectService__SWIG_1(self, argc, argv);
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'RobotRaconteurNode__AsyncConnectService'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::RobotRaconteurNode::AsyncConnectService(std::string const &,std::string const &,boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &,ClientServiceListenerDirector *,std::string const &,int32_t,RobotRaconteur::AsyncStubReturnDirector *,int32_t)\n"
    "    RobotRaconteur::RobotRaconteurNode::AsyncConnectService(std::vector< std::string,std::allocator< std::string > > const &,std::string const &,boost::intrusive_ptr< RobotRaconteur::MessageElementData > const &,ClientServiceListenerDirector *,std::string const &,int32_t,RobotRaconteur::AsyncStubReturnDirector *,int32_t)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__DisconnectService(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__DisconnectService", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__DisconnectService" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__DisconnectService" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur_RobotRaconteurNode_DisconnectService(arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__AsyncDisconnectService(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  RobotRaconteur::AsyncVoidNoErrReturnDirector *arg3 = (RobotRaconteur::AsyncVoidNoErrReturnDirector *) 0 ;
  int32_t arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  void *argp3 = 0 ;
  int res3 = 0 ;
  int val4 ;
  int ecode4 = 0 ;
  PyObject *swig_obj[4] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__AsyncDisconnectService", 4, 4, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__AsyncDisconnectService" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__AsyncDisconnectService" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  res3 = SWIG_ConvertPtr(swig_obj[2], &argp3,SWIGTYPE_p_RobotRaconteur__AsyncVoidNoErrReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "RobotRaconteurNode__AsyncDisconnectService" "', argument " "3"" of type '" "RobotRaconteur::AsyncVoidNoErrReturnDirector *""'"); 
  }
  arg3 = reinterpret_cast< RobotRaconteur::AsyncVoidNoErrReturnDirector * >(argp3);
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "RobotRaconteurNode__AsyncDisconnectService" "', argument " "4"" of type '" "int32_t""'");
  } 
  arg4 = static_cast< int32_t >(val4);
  
  try {
    RobotRaconteur_RobotRaconteurNode_AsyncDisconnectService(arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2,arg3,arg4);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_SelectRemoteNodeURL(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::vector< std::string,std::allocator< std::string > > *arg1 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res1 = swig::asptr(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode_SelectRemoteNodeURL" "', argument " "1"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode_SelectRemoteNodeURL" "', argument " "1"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg1 = ptr;
  }
  
  try {
    result = RobotRaconteur::RobotRaconteurNode::SelectRemoteNodeURL((std::vector< std::string,std::allocator< std::string > > const &)*arg1);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__RegisterServiceType__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__RegisterServiceType" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__RegisterServiceType" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__RegisterServiceType" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    RobotRaconteur_RobotRaconteurNode_RegisterServiceType__SWIG_0(arg1,(std::string const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__RegisterServiceTypes__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__RegisterServiceTypes" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__RegisterServiceTypes" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__RegisterServiceTypes" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    RobotRaconteur_RobotRaconteurNode_RegisterServiceTypes__SWIG_0(arg1,(std::vector< std::string,std::allocator< std::string > > const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__RegisterServiceType__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > tempshared2 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__RegisterServiceType" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__RegisterServiceType" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceDefinition > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    RobotRaconteur_RobotRaconteurNode_RegisterServiceType__SWIG_1(arg1,(boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__RegisterServiceType(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__RegisterServiceType", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_RobotRaconteurNode__RegisterServiceType__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_RobotRaconteurNode__RegisterServiceType__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'RobotRaconteurNode__RegisterServiceType'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::RobotRaconteurNode::RegisterServiceType(std::string const &)\n"
    "    RobotRaconteur::RobotRaconteurNode::RegisterServiceType(boost::shared_ptr< RobotRaconteur::ServiceDefinition > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__RegisterServiceTypes__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__RegisterServiceTypes" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > *ptr = (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__RegisterServiceTypes" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__RegisterServiceTypes" "', argument " "2"" of type '" "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    RobotRaconteur_RobotRaconteurNode_RegisterServiceTypes__SWIG_1(arg1,(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__RegisterServiceTypes(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__RegisterServiceTypes", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< std::string,std::allocator< std::string > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_RobotRaconteurNode__RegisterServiceTypes__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = swig::asptr(argv[1], (std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > >**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_RobotRaconteurNode__RegisterServiceTypes__SWIG_1(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'RobotRaconteurNode__RegisterServiceTypes'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::RobotRaconteurNode::RegisterServiceTypes(std::vector< std::string,std::allocator< std::string > > const &)\n"
    "    RobotRaconteur::RobotRaconteurNode::RegisterServiceTypes(std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__GetServiceType(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__GetServiceType", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__GetServiceType" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__GetServiceType" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__GetServiceType" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    result = RobotRaconteur_RobotRaconteurNode_GetServiceType(arg1,(std::string const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServiceDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ServiceDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__GetPulledServiceType(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::RRObject > *arg2 = 0 ;
  std::string *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRObject > tempshared2 ;
  int res3 = SWIG_OLDOBJ ;
  PyObject *swig_obj[3] ;
  boost::shared_ptr< RobotRaconteur::ServiceDefinition > result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__GetPulledServiceType", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__GetPulledServiceType" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRObject_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__GetPulledServiceType" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::RRObject > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp2) : &tempshared2;
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "RobotRaconteurNode__GetPulledServiceType" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__GetPulledServiceType" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    result = RobotRaconteur_RobotRaconteurNode_GetPulledServiceType(arg1,(boost::shared_ptr< RobotRaconteur::RRObject > const &)*arg2,(std::string const &)*arg3);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServiceDefinition > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ServiceDefinition >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_IsServiceTypeRegistered(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode_IsServiceTypeRegistered", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode_IsServiceTypeRegistered" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode_IsServiceTypeRegistered" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode_IsServiceTypeRegistered" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    result = (bool)(arg1)->IsServiceTypeRegistered((std::string const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__GetRegisteredServiceTypes(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::vector< std::string,std::allocator< std::string > > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__GetRegisteredServiceTypes" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->GetRegisteredServiceTypes();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< std::string,std::allocator< std::string > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__GetPulledServiceTypes(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::RRObject > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRObject > tempshared2 ;
  PyObject *swig_obj[2] ;
  std::vector< std::string,std::allocator< std::string > > result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__GetPulledServiceTypes", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__GetPulledServiceTypes" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRObject_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__GetPulledServiceTypes" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::RRObject > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    result = (arg1)->GetPulledServiceTypes((boost::shared_ptr< RobotRaconteur::RRObject > const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = swig::from(static_cast< std::vector< std::string,std::allocator< std::string > > >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__get_s(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__get_s", 0, 0, 0)) SWIG_fail;
  
  try {
    result = RobotRaconteur::RobotRaconteurNode::sp();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__Shutdown(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__Shutdown" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Shutdown();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__RequestObjectLock(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  RobotRaconteur::RobotRaconteurObjectLockFlags arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  std::string result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__RequestObjectLock", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__RequestObjectLock" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__RequestObjectLock" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "RobotRaconteurNode__RequestObjectLock" "', argument " "3"" of type '" "RobotRaconteur::RobotRaconteurObjectLockFlags""'");
  } 
  arg3 = static_cast< RobotRaconteur::RobotRaconteurObjectLockFlags >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_RobotRaconteurNode_RequestObjectLock(arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__ReleaseObjectLock(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  PyObject *swig_obj[2] ;
  std::string result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__ReleaseObjectLock", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__ReleaseObjectLock" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__ReleaseObjectLock" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = RobotRaconteur_RobotRaconteurNode_ReleaseObjectLock(arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__MonitorEnter__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  int val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__MonitorEnter" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__MonitorEnter" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "RobotRaconteurNode__MonitorEnter" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur_RobotRaconteurNode_MonitorEnter__SWIG_0(arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__MonitorEnter__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__MonitorEnter" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__MonitorEnter" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur_RobotRaconteurNode_MonitorEnter__SWIG_0(arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__MonitorEnter(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__MonitorEnter", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_RobotRaconteurNode__MonitorEnter__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        {
          int res = SWIG_AsVal_int(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_RobotRaconteurNode__MonitorEnter__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'RobotRaconteurNode__MonitorEnter'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::RobotRaconteurNode::MonitorEnter(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &,int32_t)\n"
    "    RobotRaconteur::RobotRaconteurNode::MonitorEnter(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__MonitorExit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__MonitorExit", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__MonitorExit" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__MonitorExit" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur_RobotRaconteurNode_MonitorExit(arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__NowUTC(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::posix_time::ptime result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__NowUTC" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->NowUTC();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    try {
      resultobj = PyDateTime_FromDateAndTime((int)(&result)->date().year(), (int)(&result)->date().month(), (int)(&result)->date().day(),
        (int)(&result)->time_of_day().hours(), (int)(&result)->time_of_day().minutes(), (int)(&result)->time_of_day().seconds(),
        boost::numeric_cast<int32_t>(((&result)->time_of_day().fractional_seconds() * boost::numeric_cast<int32_t>(pow(10.0,(9-(&result)->time_of_day().num_fractional_digits())))))/1000);
    }
    catch (std::exception&)
    {
      resultobj = SWIG_Py_Void();
    }
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__NowTimeSpec(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::TimeSpec result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__NowTimeSpec" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->NowTimeSpec();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::TimeSpec(static_cast< const RobotRaconteur::TimeSpec& >(result))), SWIGTYPE_p_RobotRaconteur__TimeSpec, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__NowNodeTime(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::posix_time::ptime result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__NowNodeTime" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->NowNodeTime();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    try {
      resultobj = PyDateTime_FromDateAndTime((int)(&result)->date().year(), (int)(&result)->date().month(), (int)(&result)->date().day(),
        (int)(&result)->time_of_day().hours(), (int)(&result)->time_of_day().minutes(), (int)(&result)->time_of_day().seconds(),
        boost::numeric_cast<int32_t>(((&result)->time_of_day().fractional_seconds() * boost::numeric_cast<int32_t>(pow(10.0,(9-(&result)->time_of_day().num_fractional_digits())))))/1000);
    }
    catch (std::exception&)
    {
      resultobj = SWIG_Py_Void();
    }
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__NodeSyncTimeUTC(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::posix_time::ptime result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__NodeSyncTimeUTC" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->NodeSyncTimeUTC();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    try {
      resultobj = PyDateTime_FromDateAndTime((int)(&result)->date().year(), (int)(&result)->date().month(), (int)(&result)->date().day(),
        (int)(&result)->time_of_day().hours(), (int)(&result)->time_of_day().minutes(), (int)(&result)->time_of_day().seconds(),
        boost::numeric_cast<int32_t>(((&result)->time_of_day().fractional_seconds() * boost::numeric_cast<int32_t>(pow(10.0,(9-(&result)->time_of_day().num_fractional_digits())))))/1000);
    }
    catch (std::exception&)
    {
      resultobj = SWIG_Py_Void();
    }
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__NodeSyncTimeSpec(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::TimeSpec result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__NodeSyncTimeSpec" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->NodeSyncTimeSpec();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::TimeSpec(static_cast< const RobotRaconteur::TimeSpec& >(result))), SWIGTYPE_p_RobotRaconteur__TimeSpec, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__AsyncRequestObjectLock(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  RobotRaconteur::RobotRaconteurObjectLockFlags arg3 ;
  int32_t arg4 ;
  RobotRaconteur::AsyncStringReturnDirector *arg5 = (RobotRaconteur::AsyncStringReturnDirector *) 0 ;
  int32_t arg6 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  int val3 ;
  int ecode3 = 0 ;
  int val4 ;
  int ecode4 = 0 ;
  void *argp5 = 0 ;
  int res5 = 0 ;
  int val6 ;
  int ecode6 = 0 ;
  PyObject *swig_obj[6] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__AsyncRequestObjectLock", 6, 6, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__AsyncRequestObjectLock" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__AsyncRequestObjectLock" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "RobotRaconteurNode__AsyncRequestObjectLock" "', argument " "3"" of type '" "RobotRaconteur::RobotRaconteurObjectLockFlags""'");
  } 
  arg3 = static_cast< RobotRaconteur::RobotRaconteurObjectLockFlags >(val3);
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "RobotRaconteurNode__AsyncRequestObjectLock" "', argument " "4"" of type '" "int32_t""'");
  } 
  arg4 = static_cast< int32_t >(val4);
  res5 = SWIG_ConvertPtr(swig_obj[4], &argp5,SWIGTYPE_p_RobotRaconteur__AsyncStringReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res5)) {
    SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "RobotRaconteurNode__AsyncRequestObjectLock" "', argument " "5"" of type '" "RobotRaconteur::AsyncStringReturnDirector *""'"); 
  }
  arg5 = reinterpret_cast< RobotRaconteur::AsyncStringReturnDirector * >(argp5);
  ecode6 = SWIG_AsVal_int(swig_obj[5], &val6);
  if (!SWIG_IsOK(ecode6)) {
    SWIG_exception_fail(SWIG_ArgError(ecode6), "in method '" "RobotRaconteurNode__AsyncRequestObjectLock" "', argument " "6"" of type '" "int32_t""'");
  } 
  arg6 = static_cast< int32_t >(val6);
  
  try {
    RobotRaconteur_RobotRaconteurNode_AsyncRequestObjectLock(arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2,arg3,arg4,arg5,arg6);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__AsyncReleaseObjectLock(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  int32_t arg3 ;
  RobotRaconteur::AsyncStringReturnDirector *arg4 = (RobotRaconteur::AsyncStringReturnDirector *) 0 ;
  int32_t arg5 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  int val3 ;
  int ecode3 = 0 ;
  void *argp4 = 0 ;
  int res4 = 0 ;
  int val5 ;
  int ecode5 = 0 ;
  PyObject *swig_obj[5] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__AsyncReleaseObjectLock", 5, 5, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__AsyncReleaseObjectLock" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__AsyncReleaseObjectLock" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "RobotRaconteurNode__AsyncReleaseObjectLock" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  res4 = SWIG_ConvertPtr(swig_obj[3], &argp4,SWIGTYPE_p_RobotRaconteur__AsyncStringReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "RobotRaconteurNode__AsyncReleaseObjectLock" "', argument " "4"" of type '" "RobotRaconteur::AsyncStringReturnDirector *""'"); 
  }
  arg4 = reinterpret_cast< RobotRaconteur::AsyncStringReturnDirector * >(argp4);
  ecode5 = SWIG_AsVal_int(swig_obj[4], &val5);
  if (!SWIG_IsOK(ecode5)) {
    SWIG_exception_fail(SWIG_ArgError(ecode5), "in method '" "RobotRaconteurNode__AsyncReleaseObjectLock" "', argument " "5"" of type '" "int32_t""'");
  } 
  arg5 = static_cast< int32_t >(val5);
  
  try {
    RobotRaconteur_RobotRaconteurNode_AsyncReleaseObjectLock(arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2,arg3,arg4,arg5);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__GetServiceAttributes(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__GetServiceAttributes", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__GetServiceAttributes" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__GetServiceAttributes" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    result = RobotRaconteur_RobotRaconteurNode_GetServiceAttributes(arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__GetServiceNodeID(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  PyObject *swig_obj[2] ;
  RobotRaconteur::NodeID result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__GetServiceNodeID", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__GetServiceNodeID" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__GetServiceNodeID" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    result = RobotRaconteur_RobotRaconteurNode_GetServiceNodeID(arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::NodeID(static_cast< const RobotRaconteur::NodeID& >(result))), SWIGTYPE_p_RobotRaconteur__NodeID, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__GetServiceNodeName(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  PyObject *swig_obj[2] ;
  std::string result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__GetServiceNodeName", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__GetServiceNodeName" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__GetServiceNodeName" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    result = RobotRaconteur_RobotRaconteurNode_GetServiceNodeName(arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__GetServiceName(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  PyObject *swig_obj[2] ;
  std::string result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__GetServiceName", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__GetServiceName" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__GetServiceName" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    result = RobotRaconteur_RobotRaconteurNode_GetServiceName(arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__GetObjectServicePath(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  PyObject *swig_obj[2] ;
  std::string result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__GetObjectServicePath", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__GetObjectServicePath" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__GetObjectServicePath" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    result = RobotRaconteur_RobotRaconteurNode_GetObjectServicePath(arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__GetObjectType(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  PyObject *swig_obj[2] ;
  std::string result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__GetObjectType", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__GetObjectType" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__GetObjectType" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    result = RobotRaconteur_RobotRaconteurNode_GetObjectType(arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__RegisterService__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRObject > *arg4 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceSecurityPolicy > arg5 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  void *argp4 ;
  int res4 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRObject > tempshared4 ;
  void *argp5 ;
  int res5 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > result;
  
  if ((nobjs < 5) || (nobjs > 5)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__RegisterService" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__RegisterService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__RegisterService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "RobotRaconteurNode__RegisterService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__RegisterService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  {
    int newmem = 0;
    res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRObject_t,  0 , &newmem);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "RobotRaconteurNode__RegisterService" "', argument " "4"" of type '" "boost::shared_ptr< RobotRaconteur::RRObject > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp4) tempshared4 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp4);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp4);
      arg4 = &tempshared4;
    } else {
      arg4 = (argp4) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp4) : &tempshared4;
    }
  }
  {
    int newmem = 0;
    res5 = SWIG_ConvertPtrAndOwn(swig_obj[4], &argp5, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceSecurityPolicy_t,  0 , &newmem);
    if (!SWIG_IsOK(res5)) {
      SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "RobotRaconteurNode__RegisterService" "', argument " "5"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceSecurityPolicy >""'");
    }
    if (argp5) arg5 = *(reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceSecurityPolicy > * >(argp5));
    if (newmem & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceSecurityPolicy > * >(argp5);
  }
  
  try {
    result = (arg1)->RegisterService((std::string const &)*arg2,(std::string const &)*arg3,(boost::shared_ptr< RobotRaconteur::RRObject > const &)*arg4,arg5);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServerContext > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ServerContext >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__RegisterService__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRObject > *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  void *argp4 ;
  int res4 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRObject > tempshared4 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > result;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__RegisterService" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__RegisterService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__RegisterService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "RobotRaconteurNode__RegisterService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__RegisterService" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  {
    int newmem = 0;
    res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRObject_t,  0 , &newmem);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "RobotRaconteurNode__RegisterService" "', argument " "4"" of type '" "boost::shared_ptr< RobotRaconteur::RRObject > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp4) tempshared4 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp4);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp4);
      arg4 = &tempshared4;
    } else {
      arg4 = (argp4) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp4) : &tempshared4;
    }
  }
  
  try {
    result = (arg1)->RegisterService((std::string const &)*arg2,(std::string const &)*arg3,(boost::shared_ptr< RobotRaconteur::RRObject > const &)*arg4);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::ServerContext > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::ServerContext >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__RegisterService(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[6] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__RegisterService", 0, 5, argv))) SWIG_fail;
  --argc;
  if (argc == 4) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRObject_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_RobotRaconteurNode__RegisterService__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  if (argc == 5) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRObject_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            int res = SWIG_ConvertPtr(argv[4], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceSecurityPolicy_t, 0);
            _v = SWIG_CheckState(res);
            if (_v) {
              return _wrap_RobotRaconteurNode__RegisterService__SWIG_0(self, argc, argv);
            }
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'RobotRaconteurNode__RegisterService'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::RobotRaconteurNode::RegisterService(std::string const &,std::string const &,boost::shared_ptr< RobotRaconteur::RRObject > const &,boost::shared_ptr< RobotRaconteur::ServiceSecurityPolicy >)\n"
    "    RobotRaconteur::RobotRaconteurNode::RegisterService(std::string const &,std::string const &,boost::shared_ptr< RobotRaconteur::RRObject > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_CloseService(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode_CloseService", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode_CloseService" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode_CloseService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode_CloseService" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    (arg1)->CloseService((std::string const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__NodeID(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::NodeID result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__NodeID" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->NodeID();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::NodeID(static_cast< const RobotRaconteur::NodeID& >(result))), SWIGTYPE_p_RobotRaconteur__NodeID, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__SetNodeID(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  RobotRaconteur::NodeID arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__SetNodeID", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__SetNodeID" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__NodeID,  0  | 0);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__SetNodeID" "', argument " "2"" of type '" "RobotRaconteur::NodeID const""'"); 
    }  
    if (!argp2) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__SetNodeID" "', argument " "2"" of type '" "RobotRaconteur::NodeID const""'");
    } else {
      RobotRaconteur::NodeID * temp = reinterpret_cast< RobotRaconteur::NodeID * >(argp2);
      arg2 = *temp;
      if (SWIG_IsNewObj(res2)) delete temp;
    }
  }
  
  try {
    (arg1)->SetNodeID(arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__NodeName(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__NodeName" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->NodeName();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__SetNodeName(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__SetNodeName", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__SetNodeName" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__SetNodeName" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__SetNodeName" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    (arg1)->SetNodeName((std::string const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__GetThreadPoolCount(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  int32_t result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__GetThreadPoolCount" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (int32_t)(arg1)->GetThreadPoolCount();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__SetThreadPoolCount(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  int32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__SetThreadPoolCount", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__SetThreadPoolCount" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "RobotRaconteurNode__SetThreadPoolCount" "', argument " "2"" of type '" "int32_t""'");
  } 
  arg2 = static_cast< int32_t >(val2);
  
  try {
    (arg1)->SetThreadPoolCount(arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__FindObjectType__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::RRObject > *arg2 = 0 ;
  std::string *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRObject > tempshared2 ;
  int res3 = SWIG_OLDOBJ ;
  std::string result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__FindObjectType" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRObject_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__FindObjectType" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::RRObject > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp2) : &tempshared2;
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "RobotRaconteurNode__FindObjectType" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__FindObjectType" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->FindObjectType((boost::shared_ptr< RobotRaconteur::RRObject > const &)*arg2,(std::string const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__FindObjectType__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::RRObject > *arg2 = 0 ;
  std::string *arg3 = 0 ;
  std::string *arg4 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::RRObject > tempshared2 ;
  int res3 = SWIG_OLDOBJ ;
  int res4 = SWIG_OLDOBJ ;
  std::string result;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__FindObjectType" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRObject_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__FindObjectType" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::RRObject > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRObject > * >(argp2) : &tempshared2;
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "RobotRaconteurNode__FindObjectType" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__FindObjectType" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res4 = SWIG_AsPtr_std_string(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "RobotRaconteurNode__FindObjectType" "', argument " "4"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__FindObjectType" "', argument " "4"" of type '" "std::string const &""'"); 
    }
    arg4 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->FindObjectType((boost::shared_ptr< RobotRaconteur::RRObject > const &)*arg2,(std::string const &)*arg3,(std::string const &)*arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__FindObjectType(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__FindObjectType", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRObject_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_RobotRaconteurNode__FindObjectType__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRObject_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_AsPtr_std_string(argv[3], (std::string**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap_RobotRaconteurNode__FindObjectType__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'RobotRaconteurNode__FindObjectType'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::RobotRaconteurNode::FindObjectType(boost::shared_ptr< RobotRaconteur::RRObject > const &,std::string const &)\n"
    "    RobotRaconteur::RobotRaconteurNode::FindObjectType(boost::shared_ptr< RobotRaconteur::RRObject > const &,std::string const &,std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__AsyncFindObjectType__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  std::string *arg3 = 0 ;
  int32_t arg4 ;
  RobotRaconteur::AsyncStringReturnDirector *arg5 = (RobotRaconteur::AsyncStringReturnDirector *) 0 ;
  int32_t arg6 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  int res3 = SWIG_OLDOBJ ;
  int val4 ;
  int ecode4 = 0 ;
  void *argp5 = 0 ;
  int res5 = 0 ;
  int val6 ;
  int ecode6 = 0 ;
  
  if ((nobjs < 6) || (nobjs > 6)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__AsyncFindObjectType" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__AsyncFindObjectType" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "RobotRaconteurNode__AsyncFindObjectType" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__AsyncFindObjectType" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  ecode4 = SWIG_AsVal_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "RobotRaconteurNode__AsyncFindObjectType" "', argument " "4"" of type '" "int32_t""'");
  } 
  arg4 = static_cast< int32_t >(val4);
  res5 = SWIG_ConvertPtr(swig_obj[4], &argp5,SWIGTYPE_p_RobotRaconteur__AsyncStringReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res5)) {
    SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "RobotRaconteurNode__AsyncFindObjectType" "', argument " "5"" of type '" "RobotRaconteur::AsyncStringReturnDirector *""'"); 
  }
  arg5 = reinterpret_cast< RobotRaconteur::AsyncStringReturnDirector * >(argp5);
  ecode6 = SWIG_AsVal_int(swig_obj[5], &val6);
  if (!SWIG_IsOK(ecode6)) {
    SWIG_exception_fail(SWIG_ArgError(ecode6), "in method '" "RobotRaconteurNode__AsyncFindObjectType" "', argument " "6"" of type '" "int32_t""'");
  } 
  arg6 = static_cast< int32_t >(val6);
  
  try {
    RobotRaconteur_RobotRaconteurNode_AsyncFindObjectType__SWIG_0(arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2,(std::string const &)*arg3,arg4,arg5,arg6);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__AsyncFindObjectType__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  std::string *arg3 = 0 ;
  std::string *arg4 = 0 ;
  int32_t arg5 ;
  RobotRaconteur::AsyncStringReturnDirector *arg6 = (RobotRaconteur::AsyncStringReturnDirector *) 0 ;
  int32_t arg7 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  int res3 = SWIG_OLDOBJ ;
  int res4 = SWIG_OLDOBJ ;
  int val5 ;
  int ecode5 = 0 ;
  void *argp6 = 0 ;
  int res6 = 0 ;
  int val7 ;
  int ecode7 = 0 ;
  
  if ((nobjs < 7) || (nobjs > 7)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__AsyncFindObjectType" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__AsyncFindObjectType" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "RobotRaconteurNode__AsyncFindObjectType" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__AsyncFindObjectType" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res4 = SWIG_AsPtr_std_string(swig_obj[3], &ptr);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "RobotRaconteurNode__AsyncFindObjectType" "', argument " "4"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode__AsyncFindObjectType" "', argument " "4"" of type '" "std::string const &""'"); 
    }
    arg4 = ptr;
  }
  ecode5 = SWIG_AsVal_int(swig_obj[4], &val5);
  if (!SWIG_IsOK(ecode5)) {
    SWIG_exception_fail(SWIG_ArgError(ecode5), "in method '" "RobotRaconteurNode__AsyncFindObjectType" "', argument " "5"" of type '" "int32_t""'");
  } 
  arg5 = static_cast< int32_t >(val5);
  res6 = SWIG_ConvertPtr(swig_obj[5], &argp6,SWIGTYPE_p_RobotRaconteur__AsyncStringReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res6)) {
    SWIG_exception_fail(SWIG_ArgError(res6), "in method '" "RobotRaconteurNode__AsyncFindObjectType" "', argument " "6"" of type '" "RobotRaconteur::AsyncStringReturnDirector *""'"); 
  }
  arg6 = reinterpret_cast< RobotRaconteur::AsyncStringReturnDirector * >(argp6);
  ecode7 = SWIG_AsVal_int(swig_obj[6], &val7);
  if (!SWIG_IsOK(ecode7)) {
    SWIG_exception_fail(SWIG_ArgError(ecode7), "in method '" "RobotRaconteurNode__AsyncFindObjectType" "', argument " "7"" of type '" "int32_t""'");
  } 
  arg7 = static_cast< int32_t >(val7);
  
  try {
    RobotRaconteur_RobotRaconteurNode_AsyncFindObjectType__SWIG_1(arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2,(std::string const &)*arg3,(std::string const &)*arg4,arg5,arg6,arg7);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  if (SWIG_IsNewObj(res4)) delete arg4;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__AsyncFindObjectType(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[8] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__AsyncFindObjectType", 0, 7, argv))) SWIG_fail;
  --argc;
  if (argc == 6) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          {
            int res = SWIG_AsVal_int(argv[3], NULL);
            _v = SWIG_CheckState(res);
          }
          if (_v) {
            void *vptr = 0;
            int res = SWIG_ConvertPtr(argv[4], &vptr, SWIGTYPE_p_RobotRaconteur__AsyncStringReturnDirector, 0);
            _v = SWIG_CheckState(res);
            if (_v) {
              {
                int res = SWIG_AsVal_int(argv[5], NULL);
                _v = SWIG_CheckState(res);
              }
              if (_v) {
                return _wrap_RobotRaconteurNode__AsyncFindObjectType__SWIG_0(self, argc, argv);
              }
            }
          }
        }
      }
    }
  }
  if (argc == 7) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_AsPtr_std_string(argv[3], (std::string**)(0));
          _v = SWIG_CheckState(res);
          if (_v) {
            {
              int res = SWIG_AsVal_int(argv[4], NULL);
              _v = SWIG_CheckState(res);
            }
            if (_v) {
              void *vptr = 0;
              int res = SWIG_ConvertPtr(argv[5], &vptr, SWIGTYPE_p_RobotRaconteur__AsyncStringReturnDirector, 0);
              _v = SWIG_CheckState(res);
              if (_v) {
                {
                  int res = SWIG_AsVal_int(argv[6], NULL);
                  _v = SWIG_CheckState(res);
                }
                if (_v) {
                  return _wrap_RobotRaconteurNode__AsyncFindObjectType__SWIG_1(self, argc, argv);
                }
              }
            }
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'RobotRaconteurNode__AsyncFindObjectType'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::RobotRaconteurNode::AsyncFindObjectType(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &,std::string const &,int32_t,RobotRaconteur::AsyncStringReturnDirector *,int32_t)\n"
    "    RobotRaconteur::RobotRaconteurNode::AsyncFindObjectType(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &,std::string const &,std::string const &,int32_t,RobotRaconteur::AsyncStringReturnDirector *,int32_t)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__SetExceptionHandler(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  RobotRaconteur::AsyncVoidReturnDirector *arg2 = (RobotRaconteur::AsyncVoidReturnDirector *) 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__SetExceptionHandler", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__SetExceptionHandler" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__AsyncVoidReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__SetExceptionHandler" "', argument " "2"" of type '" "RobotRaconteur::AsyncVoidReturnDirector *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::AsyncVoidReturnDirector * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "RobotRaconteurNode__SetExceptionHandler" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    RobotRaconteur_RobotRaconteurNode_SetExceptionHandler(arg1,arg2,arg3);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__ClearExceptionHandler(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__ClearExceptionHandler" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    RobotRaconteur_RobotRaconteurNode_ClearExceptionHandler(arg1);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_Sleep(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::posix_time::time_duration *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  boost::posix_time::time_duration tmpDur2 ;
  int64_t pyinputval22 = 0 ;
  double pyinputval2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode_Sleep", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode_Sleep" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  
  
  if (PyInt_Check(swig_obj[1]))
  {
    pyinputval2=boost::lexical_cast<double>(PyInt_AsLong(swig_obj[1]));
  }
  else if (PyLong_Check(swig_obj[1]))
  {
    pyinputval2=boost::lexical_cast<double>(PyLong_AsLong(swig_obj[1]));
  }
  else if (PyFloat_Check(swig_obj[1]))
  {
    pyinputval2=PyFloat_AsDouble(swig_obj[1]);
  }
  else
  {
    PyErr_SetString(PyExc_TypeError,"Input must be an Integer, Long, or Float");
    return NULL;
  }
  
  pyinputval22=boost::lexical_cast<int64_t>(pyinputval2*1000000.0);
  tmpDur2=boost::posix_time::microseconds(pyinputval22);
  arg2=&tmpDur2;
  
  
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Sleep((boost::posix_time::time_duration const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__CreateTimer(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::posix_time::time_duration *arg2 = 0 ;
  bool arg3 ;
  RobotRaconteur::AsyncTimerEventReturnDirector *arg4 = (RobotRaconteur::AsyncTimerEventReturnDirector *) 0 ;
  int32_t arg5 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  boost::posix_time::time_duration tmpDur2 ;
  int64_t pyinputval22 = 0 ;
  double pyinputval2 = 0 ;
  bool val3 ;
  int ecode3 = 0 ;
  void *argp4 = 0 ;
  int res4 = 0 ;
  int val5 ;
  int ecode5 = 0 ;
  PyObject *swig_obj[5] ;
  boost::shared_ptr< RobotRaconteur::Timer > result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__CreateTimer", 5, 5, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__CreateTimer" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  
  
  if (PyInt_Check(swig_obj[1]))
  {
    pyinputval2=boost::lexical_cast<double>(PyInt_AsLong(swig_obj[1]));
  }
  else if (PyLong_Check(swig_obj[1]))
  {
    pyinputval2=boost::lexical_cast<double>(PyLong_AsLong(swig_obj[1]));
  }
  else if (PyFloat_Check(swig_obj[1]))
  {
    pyinputval2=PyFloat_AsDouble(swig_obj[1]);
  }
  else
  {
    PyErr_SetString(PyExc_TypeError,"Input must be an Integer, Long, or Float");
    return NULL;
  }
  
  pyinputval22=boost::lexical_cast<int64_t>(pyinputval2*1000000.0);
  tmpDur2=boost::posix_time::microseconds(pyinputval22);
  arg2=&tmpDur2;
  
  
  ecode3 = SWIG_AsVal_bool(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "RobotRaconteurNode__CreateTimer" "', argument " "3"" of type '" "bool""'");
  } 
  arg3 = static_cast< bool >(val3);
  res4 = SWIG_ConvertPtr(swig_obj[3], &argp4,SWIGTYPE_p_RobotRaconteur__AsyncTimerEventReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res4)) {
    SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "RobotRaconteurNode__CreateTimer" "', argument " "4"" of type '" "RobotRaconteur::AsyncTimerEventReturnDirector *""'"); 
  }
  arg4 = reinterpret_cast< RobotRaconteur::AsyncTimerEventReturnDirector * >(argp4);
  ecode5 = SWIG_AsVal_int(swig_obj[4], &val5);
  if (!SWIG_IsOK(ecode5)) {
    SWIG_exception_fail(SWIG_ArgError(ecode5), "in method '" "RobotRaconteurNode__CreateTimer" "', argument " "5"" of type '" "int32_t""'");
  } 
  arg5 = static_cast< int32_t >(val5);
  
  try {
    result = RobotRaconteur_RobotRaconteurNode_CreateTimer(arg1,(boost::posix_time::time_duration const &)*arg2,arg3,arg4,arg5);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::Timer > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::Timer >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Timer_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_CreateRate(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  double arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  double val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  boost::shared_ptr< RobotRaconteur::Rate > result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode_CreateRate", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode_CreateRate" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_double(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "RobotRaconteurNode_CreateRate" "', argument " "2"" of type '" "double""'");
  } 
  arg2 = static_cast< double >(val2);
  
  try {
    result = (arg1)->CreateRate(arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::Rate > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::Rate >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__Rate_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_CreateAutoResetEvent(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::AutoResetEvent > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode_CreateAutoResetEvent" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->CreateAutoResetEvent();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::AutoResetEvent > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::AutoResetEvent >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__AutoResetEvent_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__PostToThreadPool(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  RobotRaconteur::AsyncVoidNoErrReturnDirector *arg2 = (RobotRaconteur::AsyncVoidNoErrReturnDirector *) 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode__PostToThreadPool", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__PostToThreadPool" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2,SWIGTYPE_p_RobotRaconteur__AsyncVoidNoErrReturnDirector, 0 |  0 );
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode__PostToThreadPool" "', argument " "2"" of type '" "RobotRaconteur::AsyncVoidNoErrReturnDirector *""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::AsyncVoidNoErrReturnDirector * >(argp2);
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "RobotRaconteurNode__PostToThreadPool" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    RobotRaconteur_RobotRaconteurNode_PostToThreadPool(arg1,arg2,arg3);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode__GetRobotRaconteurVersion(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  std::string result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode__GetRobotRaconteurVersion" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->GetRobotRaconteurVersion();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_CompareLogLevel(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  RobotRaconteur::RobotRaconteur_LogLevel arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode_CompareLogLevel", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode_CompareLogLevel" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "RobotRaconteurNode_CompareLogLevel" "', argument " "2"" of type '" "RobotRaconteur::RobotRaconteur_LogLevel""'");
  } 
  arg2 = static_cast< RobotRaconteur::RobotRaconteur_LogLevel >(val2);
  
  try {
    result = (bool)(arg1)->CompareLogLevel(arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_LogMessage(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  RobotRaconteur::RobotRaconteur_LogLevel arg2 ;
  std::string *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  int res3 = SWIG_OLDOBJ ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode_LogMessage", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode_LogMessage" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "RobotRaconteurNode_LogMessage" "', argument " "2"" of type '" "RobotRaconteur::RobotRaconteur_LogLevel""'");
  } 
  arg2 = static_cast< RobotRaconteur::RobotRaconteur_LogLevel >(val2);
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "RobotRaconteurNode_LogMessage" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode_LogMessage" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    (arg1)->LogMessage(arg2,(std::string const &)*arg3);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_LogRecord(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  RobotRaconteur::RRLogRecord *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode_LogRecord", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode_LogRecord" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__RRLogRecord,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode_LogRecord" "', argument " "2"" of type '" "RobotRaconteur::RRLogRecord const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode_LogRecord" "', argument " "2"" of type '" "RobotRaconteur::RRLogRecord const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::RRLogRecord * >(argp2);
  
  try {
    (arg1)->LogRecord((RobotRaconteur::RRLogRecord const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_GetLogLevel(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::RobotRaconteur_LogLevel result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode_GetLogLevel" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (RobotRaconteur::RobotRaconteur_LogLevel)(arg1)->GetLogLevel();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_SetLogLevel(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  RobotRaconteur::RobotRaconteur_LogLevel arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode_SetLogLevel", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode_SetLogLevel" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "RobotRaconteurNode_SetLogLevel" "', argument " "2"" of type '" "RobotRaconteur::RobotRaconteur_LogLevel""'");
  } 
  arg2 = static_cast< RobotRaconteur::RobotRaconteur_LogLevel >(val2);
  
  try {
    (arg1)->SetLogLevel(arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_SetLogLevelFromString(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  RobotRaconteur::RobotRaconteur_LogLevel result;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode_SetLogLevelFromString", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode_SetLogLevelFromString" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode_SetLogLevelFromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode_SetLogLevelFromString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    result = (RobotRaconteur::RobotRaconteur_LogLevel)(arg1)->SetLogLevelFromString((std::string const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_SetLogLevelFromEnvVariable__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  RobotRaconteur::RobotRaconteur_LogLevel result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode_SetLogLevelFromEnvVariable" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode_SetLogLevelFromEnvVariable" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode_SetLogLevelFromEnvVariable" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    result = (RobotRaconteur::RobotRaconteur_LogLevel)(arg1)->SetLogLevelFromEnvVariable((std::string const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_SetLogLevelFromEnvVariable__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  RobotRaconteur::RobotRaconteur_LogLevel result;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode_SetLogLevelFromEnvVariable" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (RobotRaconteur::RobotRaconteur_LogLevel)(arg1)->SetLogLevelFromEnvVariable();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_SetLogLevelFromEnvVariable(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "RobotRaconteurNode_SetLogLevelFromEnvVariable", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_RobotRaconteurNode_SetLogLevelFromEnvVariable__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_RobotRaconteurNode_SetLogLevelFromEnvVariable__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'RobotRaconteurNode_SetLogLevelFromEnvVariable'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::RobotRaconteurNode::SetLogLevelFromEnvVariable(std::string const &)\n"
    "    RobotRaconteur::RobotRaconteurNode::SetLogLevelFromEnvVariable()\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_GetLogRecordHandler(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::LogRecordHandler > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode_GetLogRecordHandler" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->GetLogRecordHandler();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::LogRecordHandler > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::LogRecordHandler >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LogRecordHandler_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_SetLogRecordHandler(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  boost::shared_ptr< RobotRaconteur::LogRecordHandler > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::LogRecordHandler > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode_SetLogRecordHandler", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode_SetLogRecordHandler" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LogRecordHandler_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode_SetLogRecordHandler" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::LogRecordHandler > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::LogRecordHandler > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::LogRecordHandler > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::LogRecordHandler > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    (arg1)->SetLogRecordHandler((boost::shared_ptr< RobotRaconteur::LogRecordHandler > const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_CheckConnection(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode_CheckConnection", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode_CheckConnection" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "RobotRaconteurNode_CheckConnection" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  
  try {
    (arg1)->CheckConnection(arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_GetNodeDirectories(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  RobotRaconteur::NodeDirectories result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode_GetNodeDirectories" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    result = (arg1)->GetNodeDirectories();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj((new RobotRaconteur::NodeDirectories(static_cast< const RobotRaconteur::NodeDirectories& >(result))), SWIGTYPE_p_RobotRaconteur__NodeDirectories, SWIG_POINTER_OWN |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_SetNodeDirectories(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  RobotRaconteur::NodeDirectories *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  void *argp2 = 0 ;
  int res2 = 0 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "RobotRaconteurNode_SetNodeDirectories", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode_SetNodeDirectories" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  res2 = SWIG_ConvertPtr(swig_obj[1], &argp2, SWIGTYPE_p_RobotRaconteur__NodeDirectories,  0  | 0);
  if (!SWIG_IsOK(res2)) {
    SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "RobotRaconteurNode_SetNodeDirectories" "', argument " "2"" of type '" "RobotRaconteur::NodeDirectories const &""'"); 
  }
  if (!argp2) {
    SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "RobotRaconteurNode_SetNodeDirectories" "', argument " "2"" of type '" "RobotRaconteur::NodeDirectories const &""'"); 
  }
  arg2 = reinterpret_cast< RobotRaconteur::NodeDirectories * >(argp2);
  
  try {
    (arg1)->SetNodeDirectories((RobotRaconteur::NodeDirectories const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_Init__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  size_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  size_t val2 ;
  int ecode2 = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode_Init" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_size_t(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "RobotRaconteurNode_Init" "', argument " "2"" of type '" "size_t""'");
  } 
  arg2 = static_cast< size_t >(val2);
  
  try {
    RobotRaconteur_RobotRaconteurNode_Init__SWIG_0(arg1,arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_Init__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "RobotRaconteurNode_Init" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    RobotRaconteur_RobotRaconteurNode_Init__SWIG_0(arg1);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RobotRaconteurNode_Init(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "RobotRaconteurNode_Init", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      return _wrap_RobotRaconteurNode_Init__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      {
        int res = SWIG_AsVal_size_t(argv[1], NULL);
        _v = SWIG_CheckState(res);
      }
      if (_v) {
        return _wrap_RobotRaconteurNode_Init__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'RobotRaconteurNode_Init'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::RobotRaconteurNode::Init(size_t)\n"
    "    RobotRaconteur::RobotRaconteurNode::Init()\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_new_RobotRaconteurNode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_RobotRaconteurNode", 0, 0, 0)) SWIG_fail;
  
  try {
    result = (RobotRaconteur::RobotRaconteurNode *)new RobotRaconteur::RobotRaconteurNode();
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_RobotRaconteurNode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNode *arg1 = (RobotRaconteur::RobotRaconteurNode *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_RobotRaconteurNode" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNode *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNode * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    (void)arg1; delete smartarg1;
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *RobotRaconteurNode_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *RobotRaconteurNode_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap__NewStructure(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg3 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  void *argp3 ;
  int res3 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared3 ;
  PyObject *swig_obj[3] ;
  PyObject *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "_NewStructure", 3, 3, swig_obj)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_NewStructure" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "_NewStructure" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_NewStructure" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "_NewStructure" "', argument " "3"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp3);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp3) : &tempshared3;
    }
  }
  
  try {
    result = (PyObject *)RobotRaconteur::NewStructure((std::string const &)*arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2,(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg3);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap__GetStructureType(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg3 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  void *argp3 ;
  int res3 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared3 ;
  PyObject *swig_obj[3] ;
  PyObject *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "_GetStructureType", 3, 3, swig_obj)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_GetStructureType" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "_GetStructureType" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_GetStructureType" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "_GetStructureType" "', argument " "3"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp3);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp3) : &tempshared3;
    }
  }
  
  try {
    result = (PyObject *)RobotRaconteur::GetStructureType((std::string const &)*arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2,(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg3);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap__GetNumPyDescrForType__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *arg1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg3 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > tempshared1 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  void *argp3 ;
  int res3 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared3 ;
  PyObject *result = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_GetNumPyDescrForType" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > * >(argp1) : &tempshared1;
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_GetNumPyDescrForType" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "_GetNumPyDescrForType" "', argument " "3"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp3);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp3) : &tempshared3;
    }
  }
  
  try {
    result = (PyObject *)RobotRaconteur::GetNumPyDescrForType((boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &)*arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2,(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg3);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap__GetNumPyDescrForType__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  std::string *arg1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg3 = 0 ;
  int res1 = SWIG_OLDOBJ ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  void *argp3 ;
  int res3 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared3 ;
  PyObject *result = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    std::string *ptr = (std::string *)0;
    res1 = SWIG_AsPtr_std_string(swig_obj[0], &ptr);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_GetNumPyDescrForType" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "_GetNumPyDescrForType" "', argument " "1"" of type '" "std::string const &""'"); 
    }
    arg1 = ptr;
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_GetNumPyDescrForType" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "_GetNumPyDescrForType" "', argument " "3"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp3);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp3) : &tempshared3;
    }
  }
  
  try {
    result = (PyObject *)RobotRaconteur::GetNumPyDescrForType((std::string const &)*arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2,(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg3);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  if (SWIG_IsNewObj(res1)) delete arg1;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res1)) delete arg1;
  return NULL;
}


SWIGINTERN PyObject *_wrap__GetNumPyDescrForType__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *arg1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg3 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  void *argp3 ;
  int res3 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared3 ;
  PyObject *result = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_GetNumPyDescrForType" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::TypeDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp1) : &tempshared1;
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_GetNumPyDescrForType" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "_GetNumPyDescrForType" "', argument " "3"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp3);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp3) : &tempshared3;
    }
  }
  
  try {
    result = (PyObject *)RobotRaconteur::GetNumPyDescrForType((boost::shared_ptr< RobotRaconteur::TypeDefinition > const &)*arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2,(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg3);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap__GetNumPyDescrForType(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "_GetNumPyDescrForType", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap__GetNumPyDescrForType__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap__GetNumPyDescrForType__SWIG_2(self, argc, argv);
        }
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_AsPtr_std_string(argv[0], (std::string**)(0));
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap__GetNumPyDescrForType__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function '_GetNumPyDescrForType'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::GetNumPyDescrForType(boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > const &,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &,boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)\n"
    "    RobotRaconteur::GetNumPyDescrForType(std::string const &,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &,boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)\n"
    "    RobotRaconteur::GetNumPyDescrForType(boost::shared_ptr< RobotRaconteur::TypeDefinition > const &,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &,boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap__GetNamedTypeEntryType(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *arg1 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg2 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg3 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared1 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared2 ;
  void *argp3 ;
  int res3 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared3 ;
  PyObject *swig_obj[3] ;
  RobotRaconteur::DataTypes result;
  
  if (!SWIG_Python_UnpackTuple(args, "_GetNamedTypeEntryType", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_GetNamedTypeEntryType" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::TypeDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp1) : &tempshared1;
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_GetNamedTypeEntryType" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp2) : &tempshared2;
    }
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "_GetNamedTypeEntryType" "', argument " "3"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp3);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp3) : &tempshared3;
    }
  }
  
  try {
    result = (RobotRaconteur::DataTypes)RobotRaconteur::GetNamedTypeEntryType((boost::shared_ptr< RobotRaconteur::TypeDefinition > const &)*arg1,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg2,(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg3);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap__PackMessageElement__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *arg2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg3 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg4 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared2 ;
  void *argp3 ;
  int res3 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared3 ;
  void *argp4 ;
  int res4 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared4 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  {
    arg1 = swig_obj[0];
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_PackMessageElement" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::TypeDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2) : &tempshared2;
    }
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "_PackMessageElement" "', argument " "3"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp3);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp3) : &tempshared3;
    }
  }
  {
    int newmem = 0;
    res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "_PackMessageElement" "', argument " "4"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp4) tempshared4 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp4);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp4);
      arg4 = &tempshared4;
    } else {
      arg4 = (argp4) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp4) : &tempshared4;
    }
  }
  
  try {
    result = RobotRaconteur::PackMessageElement(arg1,(boost::shared_ptr< RobotRaconteur::TypeDefinition > const &)*arg2,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg3,(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg4);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap__PackMessageElement__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *arg2 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *arg3 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg4 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared2 ;
  void *argp3 ;
  int res3 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared3 ;
  void *argp4 ;
  int res4 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared4 ;
  boost::shared_ptr< RobotRaconteur::MessageElement > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > result;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  {
    arg1 = swig_obj[0];
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_PackMessageElement" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::TypeDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2) : &tempshared2;
    }
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "_PackMessageElement" "', argument " "3"" of type '" "boost::shared_ptr< RobotRaconteur::ServerContext > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServerContext > * >(argp3);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServerContext > * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServerContext > * >(argp3) : &tempshared3;
    }
  }
  {
    int newmem = 0;
    res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "_PackMessageElement" "', argument " "4"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp4) tempshared4 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp4);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp4);
      arg4 = &tempshared4;
    } else {
      arg4 = (argp4) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp4) : &tempshared4;
    }
  }
  
  try {
    result = RobotRaconteur::PackMessageElement(arg1,(boost::shared_ptr< RobotRaconteur::TypeDefinition > const &)*arg2,(boost::shared_ptr< RobotRaconteur::ServerContext > const &)*arg3,(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg4);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::MessageElement >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::MessageElement >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap__PackMessageElement(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "_PackMessageElement", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 4) {
    int _v;
    _v = (argv[0] != 0);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap__PackMessageElement__SWIG_0(self, argc, argv);
          }
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    _v = (argv[0] != 0);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap__PackMessageElement__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function '_PackMessageElement'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::PackMessageElement(PyObject *,boost::shared_ptr< RobotRaconteur::TypeDefinition > const &,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &,boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)\n"
    "    RobotRaconteur::PackMessageElement(PyObject *,boost::shared_ptr< RobotRaconteur::TypeDefinition > const &,boost::shared_ptr< RobotRaconteur::ServerContext > const &,boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap__UnpackMessageElement__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *arg2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *arg3 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg4 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull1 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp1 ;
  int newmem1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared2 ;
  void *argp3 ;
  int res3 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedServiceStub > tempshared3 ;
  void *argp4 ;
  int res4 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared4 ;
  PyObject *result = 0 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  
  // intrusive_ptr by reference
  //if ( swig_obj[0] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[0];
  //	temp1 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg1 = &temp1;
  //} else {
  //  arg1 = &tempnull1;
  //}
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_UnpackMessageElement" "', argument " "1"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp1) {
    temp1 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp1)->get(),true);
    if (newmem1 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp1);
    arg1 = &temp1;
  }
  else {
    arg1 = &tempnull1;
  }
  
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_UnpackMessageElement" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::TypeDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2) : &tempshared2;
    }
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "_UnpackMessageElement" "', argument " "3"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp3);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedServiceStub > * >(argp3) : &tempshared3;
    }
  }
  {
    int newmem = 0;
    res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "_UnpackMessageElement" "', argument " "4"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp4) tempshared4 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp4);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp4);
      arg4 = &tempshared4;
    } else {
      arg4 = (argp4) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp4) : &tempshared4;
    }
  }
  
  try {
    result = (PyObject *)RobotRaconteur::UnpackMessageElement((boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg1,(boost::shared_ptr< RobotRaconteur::TypeDefinition > const &)*arg2,(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &)*arg3,(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg4);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap__UnpackMessageElement__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > *arg1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *arg2 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *arg3 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg4 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > tempnull1 ;
  boost::intrusive_ptr< RobotRaconteur::MessageElement > temp1 ;
  int newmem1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared2 ;
  void *argp3 ;
  int res3 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared3 ;
  void *argp4 ;
  int res4 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared4 ;
  PyObject *result = 0 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  
  // intrusive_ptr by reference
  //if ( swig_obj[0] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::MessageElement >**)&swig_obj[0];
  //	temp1 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::MessageElement >(smartarg->get(), true);
  //	arg1 = &temp1;
  //} else {
  //  arg1 = &tempnull1;
  //}
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_UnpackMessageElement" "', argument " "1"" of type '" "boost::intrusive_ptr< RobotRaconteur::MessageElement > const &""'");
  }
  if (argp1) {
    temp1 = boost::intrusive_ptr<  RobotRaconteur::MessageElement >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp1)->get(),true);
    if (newmem1 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::MessageElement >* >(argp1);
    arg1 = &temp1;
  }
  else {
    arg1 = &tempnull1;
  }
  
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_UnpackMessageElement" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::TypeDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2) : &tempshared2;
    }
  }
  {
    int newmem = 0;
    res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t,  0 , &newmem);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "_UnpackMessageElement" "', argument " "3"" of type '" "boost::shared_ptr< RobotRaconteur::ServerContext > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp3) tempshared3 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServerContext > * >(argp3);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServerContext > * >(argp3);
      arg3 = &tempshared3;
    } else {
      arg3 = (argp3) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServerContext > * >(argp3) : &tempshared3;
    }
  }
  {
    int newmem = 0;
    res4 = SWIG_ConvertPtrAndOwn(swig_obj[3], &argp4, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "_UnpackMessageElement" "', argument " "4"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp4) tempshared4 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp4);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp4);
      arg4 = &tempshared4;
    } else {
      arg4 = (argp4) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp4) : &tempshared4;
    }
  }
  
  try {
    result = (PyObject *)RobotRaconteur::UnpackMessageElement((boost::intrusive_ptr< RobotRaconteur::MessageElement > const &)*arg1,(boost::shared_ptr< RobotRaconteur::TypeDefinition > const &)*arg2,(boost::shared_ptr< RobotRaconteur::ServerContext > const &)*arg3,(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg4);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap__UnpackMessageElement(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[5] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "_UnpackMessageElement", 0, 4, argv))) SWIG_fail;
  --argc;
  if (argc == 4) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap__UnpackMessageElement__SWIG_0(self, argc, argv);
          }
        }
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_ConvertPtr(argv[2], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0);
        _v = SWIG_CheckState(res);
        if (_v) {
          int res = SWIG_ConvertPtr(argv[3], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
          _v = SWIG_CheckState(res);
          if (_v) {
            return _wrap__UnpackMessageElement__SWIG_1(self, argc, argv);
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function '_UnpackMessageElement'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::UnpackMessageElement(boost::intrusive_ptr< RobotRaconteur::MessageElement > const &,boost::shared_ptr< RobotRaconteur::TypeDefinition > const &,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > const &,boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)\n"
    "    RobotRaconteur::UnpackMessageElement(boost::intrusive_ptr< RobotRaconteur::MessageElement > const &,boost::shared_ptr< RobotRaconteur::TypeDefinition > const &,boost::shared_ptr< RobotRaconteur::ServerContext > const &,boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap__PackToRRArray(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  PyObject *arg1 = (PyObject *) 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *arg2 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::RRBaseArray > *arg3 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared2 ;
  void *argp3 ;
  int res3 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::RRBaseArray > tempnull3 ;
  boost::intrusive_ptr< RobotRaconteur::RRBaseArray > temp3 ;
  int newmem3 = 0 ;
  PyObject *swig_obj[3] ;
  boost::shared_ptr< RobotRaconteur::RRBaseArray > *smartresult = 0 ;
  boost::intrusive_ptr< RobotRaconteur::RRBaseArray > result;
  
  if (!SWIG_Python_UnpackTuple(args, "_PackToRRArray", 3, 3, swig_obj)) SWIG_fail;
  {
    arg1 = swig_obj[0];
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_PackToRRArray" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::TypeDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2) : &tempshared2;
    }
  }
  
  // intrusive_ptr by reference
  //if ( swig_obj[2] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray >**)&swig_obj[2];
  //	temp3 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::RRBaseArray >(smartarg->get(), true);
  //	arg3 = &temp3;
  //} else {
  //  arg3 = &tempnull3;
  //}
  
  res3 = SWIG_ConvertPtrAndOwn(swig_obj[2], &argp3, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t,  0 , &newmem3);
  if (!SWIG_IsOK(res3)) {
    SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "_PackToRRArray" "', argument " "3"" of type '" "boost::intrusive_ptr< RobotRaconteur::RRBaseArray > const &""'");
  }
  if (argp3) {
    temp3 = boost::intrusive_ptr<  RobotRaconteur::RRBaseArray >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRBaseArray >* >(argp3)->get(),true);
    if (newmem3 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRBaseArray >* >(argp3);
    arg3 = &temp3;
  }
  else {
    arg3 = &tempnull3;
  }
  
  
  try {
    result = RobotRaconteur::PackToRRArray(arg1,(boost::shared_ptr< RobotRaconteur::TypeDefinition > const &)*arg2,(boost::intrusive_ptr< RobotRaconteur::RRBaseArray > const &)*arg3);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  
  //if (result) {
  //	intrusive_ptr_add_ref(result.get());
  //	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray > **)&resultobj = new SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray >(result.get(), SWIG_intrusive_deleter<  RobotRaconteur::RRBaseArray >());
  //} else {
  // 	*(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray > **)&resultobj = 0;
  //}
  
  if(*(&result)) {
    intrusive_ptr_add_ref((&result)->get());
    smartresult = new boost::shared_ptr<  RobotRaconteur::RRBaseArray >((&result)->get(),SWIG_intrusive_deleter<  RobotRaconteur::RRBaseArray >());
  }
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t, SWIG_POINTER_OWN);
  
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap__UnpackFromRRArray(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  boost::intrusive_ptr< RobotRaconteur::RRBaseArray > *arg1 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > *arg2 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::intrusive_ptr< RobotRaconteur::RRBaseArray > tempnull1 ;
  boost::intrusive_ptr< RobotRaconteur::RRBaseArray > temp1 ;
  int newmem1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::TypeDefinition > tempshared2 ;
  PyObject *swig_obj[2] ;
  PyObject *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "_UnpackFromRRArray", 2, 2, swig_obj)) SWIG_fail;
  
  // intrusive_ptr by reference
  //if ( swig_obj[0] ) {
  //	smartarg = *(SWIG_SHARED_PTR_QNAMESPACE::shared_ptr<  RobotRaconteur::RRBaseArray >**)&swig_obj[0];
  //	temp1 = SWIG_INTRUSIVE_PTR_QNAMESPACE::intrusive_ptr<  RobotRaconteur::RRBaseArray >(smartarg->get(), true);
  //	arg1 = &temp1;
  //} else {
  //  arg1 = &tempnull1;
  //}
  
  res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t,  0 , &newmem1);
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_UnpackFromRRArray" "', argument " "1"" of type '" "boost::intrusive_ptr< RobotRaconteur::RRBaseArray > const &""'");
  }
  if (argp1) {
    temp1 = boost::intrusive_ptr<  RobotRaconteur::RRBaseArray >(reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRBaseArray >* >(argp1)->get(),true);
    if (newmem1 & SWIG_CAST_NEW_MEMORY) delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RRBaseArray >* >(argp1);
    arg1 = &temp1;
  }
  else {
    arg1 = &tempnull1;
  }
  
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_UnpackFromRRArray" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::TypeDefinition > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::TypeDefinition > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    result = (PyObject *)RobotRaconteur::UnpackFromRRArray((boost::intrusive_ptr< RobotRaconteur::RRBaseArray > const &)*arg1,(boost::shared_ptr< RobotRaconteur::TypeDefinition > const &)*arg2);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap__RRTypeIdToNumPyDataType(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::DataTypes arg1 ;
  int val1 ;
  int ecode1 = 0 ;
  PyObject *swig_obj[1] ;
  PyObject *result = 0 ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  ecode1 = SWIG_AsVal_int(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "_RRTypeIdToNumPyDataType" "', argument " "1"" of type '" "RobotRaconteur::DataTypes""'");
  } 
  arg1 = static_cast< RobotRaconteur::DataTypes >(val1);
  
  try {
    result = (PyObject *)RobotRaconteur::RRTypeIdToNumPyDataType(arg1);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    resultobj = result;
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_CommandLineConfigParser__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  uint32_t arg1 ;
  std::string *arg2 = 0 ;
  unsigned int val1 ;
  int ecode1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  RobotRaconteur::CommandLineConfigParser *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  ecode1 = SWIG_AsVal_unsigned_SS_int(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_CommandLineConfigParser" "', argument " "1"" of type '" "uint32_t""'");
  } 
  arg1 = static_cast< uint32_t >(val1);
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_CommandLineConfigParser" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_CommandLineConfigParser" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::CommandLineConfigParser *)new RobotRaconteur::CommandLineConfigParser(arg1,(std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_CommandLineConfigParser__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  uint32_t arg1 ;
  unsigned int val1 ;
  int ecode1 = 0 ;
  RobotRaconteur::CommandLineConfigParser *result = 0 ;
  
  if ((nobjs < 1) || (nobjs > 1)) SWIG_fail;
  ecode1 = SWIG_AsVal_unsigned_SS_int(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_CommandLineConfigParser" "', argument " "1"" of type '" "uint32_t""'");
  } 
  arg1 = static_cast< uint32_t >(val1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::CommandLineConfigParser *)new RobotRaconteur::CommandLineConfigParser(arg1);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_CommandLineConfigParser(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[3] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_CommandLineConfigParser", 0, 2, argv))) SWIG_fail;
  --argc;
  if (argc == 1) {
    int _v;
    {
      int res = SWIG_AsVal_unsigned_SS_int(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      return _wrap_new_CommandLineConfigParser__SWIG_1(self, argc, argv);
    }
  }
  if (argc == 2) {
    int _v;
    {
      int res = SWIG_AsVal_unsigned_SS_int(argv[0], NULL);
      _v = SWIG_CheckState(res);
    }
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_CommandLineConfigParser__SWIG_0(self, argc, argv);
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_CommandLineConfigParser'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::CommandLineConfigParser::CommandLineConfigParser(uint32_t,std::string const &)\n"
    "    RobotRaconteur::CommandLineConfigParser::CommandLineConfigParser(uint32_t)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_CommandLineConfigParser_SetDefaults(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::CommandLineConfigParser *arg1 = (RobotRaconteur::CommandLineConfigParser *) 0 ;
  std::string *arg2 = 0 ;
  uint16_t arg3 ;
  uint32_t arg4 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  unsigned short val3 ;
  int ecode3 = 0 ;
  unsigned int val4 ;
  int ecode4 = 0 ;
  PyObject *swig_obj[4] ;
  
  if (!SWIG_Python_UnpackTuple(args, "CommandLineConfigParser_SetDefaults", 4, 4, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "CommandLineConfigParser_SetDefaults" "', argument " "1"" of type '" "RobotRaconteur::CommandLineConfigParser *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "CommandLineConfigParser_SetDefaults" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CommandLineConfigParser_SetDefaults" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  ecode3 = SWIG_AsVal_unsigned_SS_short(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "CommandLineConfigParser_SetDefaults" "', argument " "3"" of type '" "uint16_t""'");
  } 
  arg3 = static_cast< uint16_t >(val3);
  ecode4 = SWIG_AsVal_unsigned_SS_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "CommandLineConfigParser_SetDefaults" "', argument " "4"" of type '" "uint32_t""'");
  } 
  arg4 = static_cast< uint32_t >(val4);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetDefaults((std::string const &)*arg2,arg3,arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_CommandLineConfigParser_AddStringOption(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::CommandLineConfigParser *arg1 = (RobotRaconteur::CommandLineConfigParser *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "CommandLineConfigParser_AddStringOption", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "CommandLineConfigParser_AddStringOption" "', argument " "1"" of type '" "RobotRaconteur::CommandLineConfigParser *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "CommandLineConfigParser_AddStringOption" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CommandLineConfigParser_AddStringOption" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "CommandLineConfigParser_AddStringOption" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CommandLineConfigParser_AddStringOption" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->AddStringOption((std::string const &)*arg2,(std::string const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_CommandLineConfigParser_AddBoolOption(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::CommandLineConfigParser *arg1 = (RobotRaconteur::CommandLineConfigParser *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "CommandLineConfigParser_AddBoolOption", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "CommandLineConfigParser_AddBoolOption" "', argument " "1"" of type '" "RobotRaconteur::CommandLineConfigParser *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "CommandLineConfigParser_AddBoolOption" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CommandLineConfigParser_AddBoolOption" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "CommandLineConfigParser_AddBoolOption" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CommandLineConfigParser_AddBoolOption" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->AddBoolOption((std::string const &)*arg2,(std::string const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_CommandLineConfigParser_AddIntOption(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::CommandLineConfigParser *arg1 = (RobotRaconteur::CommandLineConfigParser *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "CommandLineConfigParser_AddIntOption", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "CommandLineConfigParser_AddIntOption" "', argument " "1"" of type '" "RobotRaconteur::CommandLineConfigParser *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "CommandLineConfigParser_AddIntOption" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CommandLineConfigParser_AddIntOption" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "CommandLineConfigParser_AddIntOption" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CommandLineConfigParser_AddIntOption" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->AddIntOption((std::string const &)*arg2,(std::string const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_CommandLineConfigParser_ParseCommandLine(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::CommandLineConfigParser *arg1 = (RobotRaconteur::CommandLineConfigParser *) 0 ;
  std::vector< std::string,std::allocator< std::string > > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "CommandLineConfigParser_ParseCommandLine", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "CommandLineConfigParser_ParseCommandLine" "', argument " "1"" of type '" "RobotRaconteur::CommandLineConfigParser *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res2 = swig::asptr(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "CommandLineConfigParser_ParseCommandLine" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CommandLineConfigParser_ParseCommandLine" "', argument " "2"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->ParseCommandLine((std::vector< std::string,std::allocator< std::string > > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_CommandLineConfigParser_GetOptionOrDefaultAsString__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::CommandLineConfigParser *arg1 = (RobotRaconteur::CommandLineConfigParser *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  std::string result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "CommandLineConfigParser_GetOptionOrDefaultAsString" "', argument " "1"" of type '" "RobotRaconteur::CommandLineConfigParser *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "CommandLineConfigParser_GetOptionOrDefaultAsString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CommandLineConfigParser_GetOptionOrDefaultAsString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetOptionOrDefaultAsString((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_CommandLineConfigParser_GetOptionOrDefaultAsString__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::CommandLineConfigParser *arg1 = (RobotRaconteur::CommandLineConfigParser *) 0 ;
  std::string *arg2 = 0 ;
  std::string *arg3 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int res3 = SWIG_OLDOBJ ;
  std::string result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "CommandLineConfigParser_GetOptionOrDefaultAsString" "', argument " "1"" of type '" "RobotRaconteur::CommandLineConfigParser *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "CommandLineConfigParser_GetOptionOrDefaultAsString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CommandLineConfigParser_GetOptionOrDefaultAsString" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  {
    std::string *ptr = (std::string *)0;
    res3 = SWIG_AsPtr_std_string(swig_obj[2], &ptr);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "CommandLineConfigParser_GetOptionOrDefaultAsString" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CommandLineConfigParser_GetOptionOrDefaultAsString" "', argument " "3"" of type '" "std::string const &""'"); 
    }
    arg3 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetOptionOrDefaultAsString((std::string const &)*arg2,(std::string const &)*arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_std_string(static_cast< std::string >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res3)) delete arg3;
  return NULL;
}


SWIGINTERN PyObject *_wrap_CommandLineConfigParser_GetOptionOrDefaultAsString(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "CommandLineConfigParser_GetOptionOrDefaultAsString", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_CommandLineConfigParser_GetOptionOrDefaultAsString__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        int res = SWIG_AsPtr_std_string(argv[2], (std::string**)(0));
        _v = SWIG_CheckState(res);
        if (_v) {
          return _wrap_CommandLineConfigParser_GetOptionOrDefaultAsString__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'CommandLineConfigParser_GetOptionOrDefaultAsString'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::CommandLineConfigParser::GetOptionOrDefaultAsString(std::string const &)\n"
    "    RobotRaconteur::CommandLineConfigParser::GetOptionOrDefaultAsString(std::string const &,std::string const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_CommandLineConfigParser_GetOptionOrDefaultAsBool__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::CommandLineConfigParser *arg1 = (RobotRaconteur::CommandLineConfigParser *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  bool result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "CommandLineConfigParser_GetOptionOrDefaultAsBool" "', argument " "1"" of type '" "RobotRaconteur::CommandLineConfigParser *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "CommandLineConfigParser_GetOptionOrDefaultAsBool" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CommandLineConfigParser_GetOptionOrDefaultAsBool" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->GetOptionOrDefaultAsBool((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_CommandLineConfigParser_GetOptionOrDefaultAsBool__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::CommandLineConfigParser *arg1 = (RobotRaconteur::CommandLineConfigParser *) 0 ;
  std::string *arg2 = 0 ;
  bool arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  bool val3 ;
  int ecode3 = 0 ;
  bool result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "CommandLineConfigParser_GetOptionOrDefaultAsBool" "', argument " "1"" of type '" "RobotRaconteur::CommandLineConfigParser *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "CommandLineConfigParser_GetOptionOrDefaultAsBool" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CommandLineConfigParser_GetOptionOrDefaultAsBool" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  ecode3 = SWIG_AsVal_bool(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "CommandLineConfigParser_GetOptionOrDefaultAsBool" "', argument " "3"" of type '" "bool""'");
  } 
  arg3 = static_cast< bool >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)(arg1)->GetOptionOrDefaultAsBool((std::string const &)*arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_CommandLineConfigParser_GetOptionOrDefaultAsBool(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "CommandLineConfigParser_GetOptionOrDefaultAsBool", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_CommandLineConfigParser_GetOptionOrDefaultAsBool__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        {
          int res = SWIG_AsVal_bool(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_CommandLineConfigParser_GetOptionOrDefaultAsBool__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'CommandLineConfigParser_GetOptionOrDefaultAsBool'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::CommandLineConfigParser::GetOptionOrDefaultAsBool(std::string const &)\n"
    "    RobotRaconteur::CommandLineConfigParser::GetOptionOrDefaultAsBool(std::string const &,bool)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_CommandLineConfigParser_GetOptionOrDefaultAsInt__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::CommandLineConfigParser *arg1 = (RobotRaconteur::CommandLineConfigParser *) 0 ;
  std::string *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int32_t result;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "CommandLineConfigParser_GetOptionOrDefaultAsInt" "', argument " "1"" of type '" "RobotRaconteur::CommandLineConfigParser *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "CommandLineConfigParser_GetOptionOrDefaultAsInt" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CommandLineConfigParser_GetOptionOrDefaultAsInt" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (int32_t)(arg1)->GetOptionOrDefaultAsInt((std::string const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_CommandLineConfigParser_GetOptionOrDefaultAsInt__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::CommandLineConfigParser *arg1 = (RobotRaconteur::CommandLineConfigParser *) 0 ;
  std::string *arg2 = 0 ;
  int32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > *smartarg1 = 0 ;
  int res2 = SWIG_OLDOBJ ;
  int val3 ;
  int ecode3 = 0 ;
  int32_t result;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "CommandLineConfigParser_GetOptionOrDefaultAsInt" "', argument " "1"" of type '" "RobotRaconteur::CommandLineConfigParser *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "CommandLineConfigParser_GetOptionOrDefaultAsInt" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "CommandLineConfigParser_GetOptionOrDefaultAsInt" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  ecode3 = SWIG_AsVal_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "CommandLineConfigParser_GetOptionOrDefaultAsInt" "', argument " "3"" of type '" "int32_t""'");
  } 
  arg3 = static_cast< int32_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (int32_t)(arg1)->GetOptionOrDefaultAsInt((std::string const &)*arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_int(static_cast< int >(result));
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_CommandLineConfigParser_GetOptionOrDefaultAsInt(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "CommandLineConfigParser_GetOptionOrDefaultAsInt", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_CommandLineConfigParser_GetOptionOrDefaultAsInt__SWIG_0(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        {
          int res = SWIG_AsVal_int(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_CommandLineConfigParser_GetOptionOrDefaultAsInt__SWIG_1(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'CommandLineConfigParser_GetOptionOrDefaultAsInt'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::CommandLineConfigParser::GetOptionOrDefaultAsInt(std::string const &)\n"
    "    RobotRaconteur::CommandLineConfigParser::GetOptionOrDefaultAsInt(std::string const &,int32_t)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_delete_CommandLineConfigParser(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::CommandLineConfigParser *arg1 = (RobotRaconteur::CommandLineConfigParser *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_CommandLineConfigParser" "', argument " "1"" of type '" "RobotRaconteur::CommandLineConfigParser *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > * >(argp1);
      arg1 = const_cast< RobotRaconteur::CommandLineConfigParser * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *CommandLineConfigParser_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *CommandLineConfigParser_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_WrappedRobotRaconteurNodeSetup_GetLocalTransport(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNodeSetup *arg1 = (RobotRaconteur::RobotRaconteurNodeSetup *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNodeSetup > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNodeSetup > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::LocalTransport > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNodeSetup_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedRobotRaconteurNodeSetup_GetLocalTransport" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNodeSetup *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNodeSetup * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNodeSetup * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetLocalTransport();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::LocalTransport > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::LocalTransport >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedRobotRaconteurNodeSetup_GetIntraTransport(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNodeSetup *arg1 = (RobotRaconteur::RobotRaconteurNodeSetup *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNodeSetup > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNodeSetup > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::IntraTransport > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNodeSetup_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedRobotRaconteurNodeSetup_GetIntraTransport" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNodeSetup *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNodeSetup * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNodeSetup * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetIntraTransport();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::IntraTransport > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::IntraTransport >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__IntraTransport_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedRobotRaconteurNodeSetup_GetTcpTransport(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNodeSetup *arg1 = (RobotRaconteur::RobotRaconteurNodeSetup *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNodeSetup > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNodeSetup > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::TcpTransport > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNodeSetup_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedRobotRaconteurNodeSetup_GetTcpTransport" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNodeSetup *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNodeSetup * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNodeSetup * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetTcpTransport();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::TcpTransport > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::TcpTransport >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedRobotRaconteurNodeSetup_GetHardwareTransport(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNodeSetup *arg1 = (RobotRaconteur::RobotRaconteurNodeSetup *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNodeSetup > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNodeSetup > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::HardwareTransport > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNodeSetup_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedRobotRaconteurNodeSetup_GetHardwareTransport" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNodeSetup *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNodeSetup * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNodeSetup * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetHardwareTransport();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::HardwareTransport > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::HardwareTransport >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedRobotRaconteurNodeSetup_GetCommandLineConfig(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNodeSetup *arg1 = (RobotRaconteur::RobotRaconteurNodeSetup *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNodeSetup > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNodeSetup > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > result;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNodeSetup_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedRobotRaconteurNodeSetup_GetCommandLineConfig" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNodeSetup *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNodeSetup * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNodeSetup * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (arg1)->GetCommandLineConfig();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::CommandLineConfigParser >(result) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_WrappedRobotRaconteurNodeSetup_ReleaseNode(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNodeSetup *arg1 = (RobotRaconteur::RobotRaconteurNodeSetup *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNodeSetup > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNodeSetup > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNodeSetup_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "WrappedRobotRaconteurNodeSetup_ReleaseNode" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNodeSetup *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNodeSetup * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNodeSetup * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->ReleaseNode();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_WrappedRobotRaconteurNodeSetup(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RobotRaconteurNodeSetup *arg1 = (RobotRaconteur::RobotRaconteurNodeSetup *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNodeSetup > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNodeSetup > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNodeSetup_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_WrappedRobotRaconteurNodeSetup" "', argument " "1"" of type '" "RobotRaconteur::RobotRaconteurNodeSetup *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNodeSetup * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > * >(argp1);
      arg1 = const_cast< RobotRaconteur::RobotRaconteurNodeSetup * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedRobotRaconteurNodeSetup__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  std::string *arg2 = 0 ;
  uint16_t arg3 ;
  uint32_t arg4 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  int res2 = SWIG_OLDOBJ ;
  unsigned short val3 ;
  int ecode3 = 0 ;
  unsigned int val4 ;
  int ecode4 = 0 ;
  RobotRaconteur::RobotRaconteurNodeSetup *result = 0 ;
  
  if ((nobjs < 4) || (nobjs > 4)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_WrappedRobotRaconteurNodeSetup" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_WrappedRobotRaconteurNodeSetup" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedRobotRaconteurNodeSetup" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  ecode3 = SWIG_AsVal_unsigned_SS_short(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "new_WrappedRobotRaconteurNodeSetup" "', argument " "3"" of type '" "uint16_t""'");
  } 
  arg3 = static_cast< uint16_t >(val3);
  ecode4 = SWIG_AsVal_unsigned_SS_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "new_WrappedRobotRaconteurNodeSetup" "', argument " "4"" of type '" "uint32_t""'");
  } 
  arg4 = static_cast< uint32_t >(val4);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::RobotRaconteurNodeSetup *)new_RobotRaconteur_RobotRaconteurNodeSetup__SWIG_0((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(std::string const &)*arg2,arg3,arg4);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNodeSetup_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedRobotRaconteurNodeSetup__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > *arg2 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > tempshared2 ;
  RobotRaconteur::RobotRaconteurNodeSetup *result = 0 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_WrappedRobotRaconteurNodeSetup" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_WrappedRobotRaconteurNodeSetup" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::RobotRaconteurNodeSetup *)new_RobotRaconteur_RobotRaconteurNodeSetup__SWIG_1((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNodeSetup_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedRobotRaconteurNodeSetup__SWIG_2(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *arg1 = 0 ;
  std::string *arg2 = 0 ;
  uint16_t arg3 ;
  uint32_t arg4 ;
  uint32_t arg5 ;
  std::vector< std::string,std::allocator< std::string > > *arg6 = 0 ;
  void *argp1 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > tempshared1 ;
  int res2 = SWIG_OLDOBJ ;
  unsigned short val3 ;
  int ecode3 = 0 ;
  unsigned int val4 ;
  int ecode4 = 0 ;
  unsigned int val5 ;
  int ecode5 = 0 ;
  int res6 = SWIG_OLDOBJ ;
  RobotRaconteur::RobotRaconteurNodeSetup *result = 0 ;
  
  if ((nobjs < 6) || (nobjs > 6)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "new_WrappedRobotRaconteurNodeSetup" "', argument " "1"" of type '" "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp1) tempshared1 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1);
      arg1 = &tempshared1;
    } else {
      arg1 = (argp1) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > * >(argp1) : &tempshared1;
    }
  }
  {
    std::string *ptr = (std::string *)0;
    res2 = SWIG_AsPtr_std_string(swig_obj[1], &ptr);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "new_WrappedRobotRaconteurNodeSetup" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedRobotRaconteurNodeSetup" "', argument " "2"" of type '" "std::string const &""'"); 
    }
    arg2 = ptr;
  }
  ecode3 = SWIG_AsVal_unsigned_SS_short(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "new_WrappedRobotRaconteurNodeSetup" "', argument " "3"" of type '" "uint16_t""'");
  } 
  arg3 = static_cast< uint16_t >(val3);
  ecode4 = SWIG_AsVal_unsigned_SS_int(swig_obj[3], &val4);
  if (!SWIG_IsOK(ecode4)) {
    SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "new_WrappedRobotRaconteurNodeSetup" "', argument " "4"" of type '" "uint32_t""'");
  } 
  arg4 = static_cast< uint32_t >(val4);
  ecode5 = SWIG_AsVal_unsigned_SS_int(swig_obj[4], &val5);
  if (!SWIG_IsOK(ecode5)) {
    SWIG_exception_fail(SWIG_ArgError(ecode5), "in method '" "new_WrappedRobotRaconteurNodeSetup" "', argument " "5"" of type '" "uint32_t""'");
  } 
  arg5 = static_cast< uint32_t >(val5);
  {
    std::vector< std::string,std::allocator< std::string > > *ptr = (std::vector< std::string,std::allocator< std::string > > *)0;
    res6 = swig::asptr(swig_obj[5], &ptr);
    if (!SWIG_IsOK(res6)) {
      SWIG_exception_fail(SWIG_ArgError(res6), "in method '" "new_WrappedRobotRaconteurNodeSetup" "', argument " "6"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    if (!ptr) {
      SWIG_exception_fail(SWIG_ValueError, "invalid null reference " "in method '" "new_WrappedRobotRaconteurNodeSetup" "', argument " "6"" of type '" "std::vector< std::string,std::allocator< std::string > > const &""'"); 
    }
    arg6 = ptr;
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::RobotRaconteurNodeSetup *)new_RobotRaconteur_RobotRaconteurNodeSetup__SWIG_2((boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &)*arg1,(std::string const &)*arg2,arg3,arg4,arg5,(std::vector< std::string,std::allocator< std::string > > const &)*arg6);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::RobotRaconteurNodeSetup >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNodeSetup_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res6)) delete arg6;
  return resultobj;
fail:
  if (SWIG_IsNewObj(res2)) delete arg2;
  if (SWIG_IsNewObj(res6)) delete arg6;
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_WrappedRobotRaconteurNodeSetup(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[7] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "new_WrappedRobotRaconteurNodeSetup", 0, 6, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_new_WrappedRobotRaconteurNodeSetup__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 4) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        {
          int res = SWIG_AsVal_unsigned_SS_short(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          {
            int res = SWIG_AsVal_unsigned_SS_int(argv[3], NULL);
            _v = SWIG_CheckState(res);
          }
          if (_v) {
            return _wrap_new_WrappedRobotRaconteurNodeSetup__SWIG_0(self, argc, argv);
          }
        }
      }
    }
  }
  if (argc == 6) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_AsPtr_std_string(argv[1], (std::string**)(0));
      _v = SWIG_CheckState(res);
      if (_v) {
        {
          int res = SWIG_AsVal_unsigned_SS_short(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          {
            int res = SWIG_AsVal_unsigned_SS_int(argv[3], NULL);
            _v = SWIG_CheckState(res);
          }
          if (_v) {
            {
              int res = SWIG_AsVal_unsigned_SS_int(argv[4], NULL);
              _v = SWIG_CheckState(res);
            }
            if (_v) {
              int res = swig::asptr(argv[5], (std::vector< std::string,std::allocator< std::string > >**)(0));
              _v = SWIG_CheckState(res);
              if (_v) {
                return _wrap_new_WrappedRobotRaconteurNodeSetup__SWIG_2(self, argc, argv);
              }
            }
          }
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'new_WrappedRobotRaconteurNodeSetup'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::RobotRaconteurNodeSetup::RobotRaconteurNodeSetup(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &,std::string const &,uint16_t,uint32_t)\n"
    "    RobotRaconteur::RobotRaconteurNodeSetup::RobotRaconteurNodeSetup(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &,boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > const &)\n"
    "    RobotRaconteur::RobotRaconteurNodeSetup::RobotRaconteurNodeSetup(boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > const &,std::string const &,uint16_t,uint32_t,uint32_t,std::vector< std::string,std::allocator< std::string > > const &)\n");
  return 0;
}


SWIGINTERN PyObject *WrappedRobotRaconteurNodeSetup_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNodeSetup_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *WrappedRobotRaconteurNodeSetup_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_new_BroadcastDownsampler(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::BroadcastDownsampler *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_BroadcastDownsampler", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::BroadcastDownsampler *)new RobotRaconteur::BroadcastDownsampler();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  {
    boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > *smartresult = result ? new boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler >(result SWIG_NO_NULL_DELETER_SWIG_POINTER_NEW) : 0;
    resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(smartresult), SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__BroadcastDownsampler_t, SWIG_POINTER_NEW | SWIG_POINTER_OWN);
  }
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_BroadcastDownsampler__Init__SWIG_0(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::BroadcastDownsampler *arg1 = (RobotRaconteur::BroadcastDownsampler *) 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *arg2 = 0 ;
  uint32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::BroadcastDownsampler > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::BroadcastDownsampler > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared2 ;
  unsigned int val3 ;
  int ecode3 = 0 ;
  
  if ((nobjs < 3) || (nobjs > 3)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__BroadcastDownsampler_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "BroadcastDownsampler__Init" "', argument " "1"" of type '" "RobotRaconteur::BroadcastDownsampler *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::BroadcastDownsampler * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::BroadcastDownsampler * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "BroadcastDownsampler__Init" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::ServerContext > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServerContext > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServerContext > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServerContext > * >(argp2) : &tempshared2;
    }
  }
  ecode3 = SWIG_AsVal_unsigned_SS_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "BroadcastDownsampler__Init" "', argument " "3"" of type '" "uint32_t""'");
  } 
  arg3 = static_cast< uint32_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Init((boost::shared_ptr< RobotRaconteur::ServerContext > const &)*arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_BroadcastDownsampler__Init__SWIG_1(PyObject *SWIGUNUSEDPARM(self), Py_ssize_t nobjs, PyObject **swig_obj) {
  PyObject *resultobj = 0;
  RobotRaconteur::BroadcastDownsampler *arg1 = (RobotRaconteur::BroadcastDownsampler *) 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::BroadcastDownsampler > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::BroadcastDownsampler > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::ServerContext > tempshared2 ;
  
  if ((nobjs < 2) || (nobjs > 2)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__BroadcastDownsampler_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "BroadcastDownsampler__Init" "', argument " "1"" of type '" "RobotRaconteur::BroadcastDownsampler *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::BroadcastDownsampler * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::BroadcastDownsampler * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "BroadcastDownsampler__Init" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::ServerContext > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServerContext > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServerContext > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::ServerContext > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->Init((boost::shared_ptr< RobotRaconteur::ServerContext > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_BroadcastDownsampler__Init(PyObject *self, PyObject *args) {
  Py_ssize_t argc;
  PyObject *argv[4] = {
    0
  };
  
  if (!(argc = SWIG_Python_UnpackTuple(args, "BroadcastDownsampler__Init", 0, 3, argv))) SWIG_fail;
  --argc;
  if (argc == 2) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__BroadcastDownsampler_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        return _wrap_BroadcastDownsampler__Init__SWIG_1(self, argc, argv);
      }
    }
  }
  if (argc == 3) {
    int _v;
    int res = SWIG_ConvertPtr(argv[0], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__BroadcastDownsampler_t, 0);
    _v = SWIG_CheckState(res);
    if (_v) {
      int res = SWIG_ConvertPtr(argv[1], 0, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0);
      _v = SWIG_CheckState(res);
      if (_v) {
        {
          int res = SWIG_AsVal_unsigned_SS_int(argv[2], NULL);
          _v = SWIG_CheckState(res);
        }
        if (_v) {
          return _wrap_BroadcastDownsampler__Init__SWIG_0(self, argc, argv);
        }
      }
    }
  }
  
fail:
  SWIG_Python_RaiseOrModifyTypeError("Wrong number or type of arguments for overloaded function 'BroadcastDownsampler__Init'.\n"
    "  Possible C/C++ prototypes are:\n"
    "    RobotRaconteur::BroadcastDownsampler::Init(boost::shared_ptr< RobotRaconteur::ServerContext > const &,uint32_t)\n"
    "    RobotRaconteur::BroadcastDownsampler::Init(boost::shared_ptr< RobotRaconteur::ServerContext > const &)\n");
  return 0;
}


SWIGINTERN PyObject *_wrap_BroadcastDownsampler_GetClientDownsample(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::BroadcastDownsampler *arg1 = (RobotRaconteur::BroadcastDownsampler *) 0 ;
  uint32_t arg2 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::BroadcastDownsampler > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::BroadcastDownsampler > *smartarg1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  PyObject *swig_obj[2] ;
  uint32_t result;
  
  if (!SWIG_Python_UnpackTuple(args, "BroadcastDownsampler_GetClientDownsample", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__BroadcastDownsampler_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "BroadcastDownsampler_GetClientDownsample" "', argument " "1"" of type '" "RobotRaconteur::BroadcastDownsampler *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::BroadcastDownsampler * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::BroadcastDownsampler * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "BroadcastDownsampler_GetClientDownsample" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (uint32_t)(arg1)->GetClientDownsample(arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_unsigned_SS_int(static_cast< unsigned int >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_BroadcastDownsampler_SetClientDownsample(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::BroadcastDownsampler *arg1 = (RobotRaconteur::BroadcastDownsampler *) 0 ;
  uint32_t arg2 ;
  uint32_t arg3 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::BroadcastDownsampler > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::BroadcastDownsampler > *smartarg1 = 0 ;
  unsigned int val2 ;
  int ecode2 = 0 ;
  unsigned int val3 ;
  int ecode3 = 0 ;
  PyObject *swig_obj[3] ;
  
  if (!SWIG_Python_UnpackTuple(args, "BroadcastDownsampler_SetClientDownsample", 3, 3, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__BroadcastDownsampler_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "BroadcastDownsampler_SetClientDownsample" "', argument " "1"" of type '" "RobotRaconteur::BroadcastDownsampler *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::BroadcastDownsampler * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::BroadcastDownsampler * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  ecode2 = SWIG_AsVal_unsigned_SS_int(swig_obj[1], &val2);
  if (!SWIG_IsOK(ecode2)) {
    SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "BroadcastDownsampler_SetClientDownsample" "', argument " "2"" of type '" "uint32_t""'");
  } 
  arg2 = static_cast< uint32_t >(val2);
  ecode3 = SWIG_AsVal_unsigned_SS_int(swig_obj[2], &val3);
  if (!SWIG_IsOK(ecode3)) {
    SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "BroadcastDownsampler_SetClientDownsample" "', argument " "3"" of type '" "uint32_t""'");
  } 
  arg3 = static_cast< uint32_t >(val3);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->SetClientDownsample(arg2,arg3);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_BroadcastDownsampler_BeginStep(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::BroadcastDownsampler *arg1 = (RobotRaconteur::BroadcastDownsampler *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::BroadcastDownsampler > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::BroadcastDownsampler > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__BroadcastDownsampler_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "BroadcastDownsampler_BeginStep" "', argument " "1"" of type '" "RobotRaconteur::BroadcastDownsampler *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::BroadcastDownsampler * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::BroadcastDownsampler * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->BeginStep();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_BroadcastDownsampler_EndStep(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::BroadcastDownsampler *arg1 = (RobotRaconteur::BroadcastDownsampler *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::BroadcastDownsampler > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::BroadcastDownsampler > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__BroadcastDownsampler_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "BroadcastDownsampler_EndStep" "', argument " "1"" of type '" "RobotRaconteur::BroadcastDownsampler *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::BroadcastDownsampler * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::BroadcastDownsampler * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->EndStep();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_BroadcastDownsampler__AddPipeBroadcaster(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::BroadcastDownsampler *arg1 = (RobotRaconteur::BroadcastDownsampler *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::BroadcastDownsampler > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::BroadcastDownsampler > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "BroadcastDownsampler__AddPipeBroadcaster", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__BroadcastDownsampler_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "BroadcastDownsampler__AddPipeBroadcaster" "', argument " "1"" of type '" "RobotRaconteur::BroadcastDownsampler *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::BroadcastDownsampler * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::BroadcastDownsampler * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeBroadcaster_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "BroadcastDownsampler__AddPipeBroadcaster" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->AddPipeBroadcaster((boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_BroadcastDownsampler__AddWireBroadcaster(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::BroadcastDownsampler *arg1 = (RobotRaconteur::BroadcastDownsampler *) 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > *arg2 = 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::BroadcastDownsampler > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::BroadcastDownsampler > *smartarg1 = 0 ;
  void *argp2 ;
  int res2 = 0 ;
  boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > tempshared2 ;
  PyObject *swig_obj[2] ;
  
  if (!SWIG_Python_UnpackTuple(args, "BroadcastDownsampler__AddWireBroadcaster", 2, 2, swig_obj)) SWIG_fail;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__BroadcastDownsampler_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "BroadcastDownsampler__AddWireBroadcaster" "', argument " "1"" of type '" "RobotRaconteur::BroadcastDownsampler *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::BroadcastDownsampler * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::BroadcastDownsampler * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  {
    int newmem = 0;
    res2 = SWIG_ConvertPtrAndOwn(swig_obj[1], &argp2, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__WrappedWireBroadcaster_t,  0 , &newmem);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "BroadcastDownsampler__AddWireBroadcaster" "', argument " "2"" of type '" "boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > const &""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      if (argp2) tempshared2 = *reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > * >(argp2);
      delete reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > * >(argp2);
      arg2 = &tempshared2;
    } else {
      arg2 = (argp2) ? reinterpret_cast< boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > * >(argp2) : &tempshared2;
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (arg1)->AddWireBroadcaster((boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > const &)*arg2);
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_BroadcastDownsampler(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::BroadcastDownsampler *arg1 = (RobotRaconteur::BroadcastDownsampler *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  boost::shared_ptr< RobotRaconteur::BroadcastDownsampler > tempshared1 ;
  boost::shared_ptr< RobotRaconteur::BroadcastDownsampler > *smartarg1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  {
    int newmem = 0;
    res1 = SWIG_ConvertPtrAndOwn(swig_obj[0], &argp1, SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__BroadcastDownsampler_t, 0 |  0 , &newmem);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_BroadcastDownsampler" "', argument " "1"" of type '" "RobotRaconteur::BroadcastDownsampler *""'");
    }
    if (newmem & SWIG_CAST_NEW_MEMORY) {
      tempshared1 = *reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      delete reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::BroadcastDownsampler * >(tempshared1.get());
    } else {
      smartarg1 = reinterpret_cast< boost::shared_ptr<  RobotRaconteur::BroadcastDownsampler > * >(argp1);
      arg1 = const_cast< RobotRaconteur::BroadcastDownsampler * >((smartarg1 ? smartarg1->get() : 0));
    }
  }
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      (void)arg1; delete smartarg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *BroadcastDownsampler_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_boost__shared_ptrT_RobotRaconteur__BroadcastDownsampler_t, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *BroadcastDownsampler_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN PyObject *_wrap_RRNativeDirectorSupport_Start(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  
  if (!SWIG_Python_UnpackTuple(args, "RRNativeDirectorSupport_Start", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur::RRNativeDirectorSupport::Start();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRNativeDirectorSupport_Stop(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  
  if (!SWIG_Python_UnpackTuple(args, "RRNativeDirectorSupport_Stop", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      RobotRaconteur::RRNativeDirectorSupport::Stop();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_RRNativeDirectorSupport_IsRunning(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  bool result;
  
  if (!SWIG_Python_UnpackTuple(args, "RRNativeDirectorSupport_IsRunning", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (bool)RobotRaconteur::RRNativeDirectorSupport::IsRunning();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_From_bool(static_cast< bool >(result));
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_new_RRNativeDirectorSupport(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRNativeDirectorSupport *result = 0 ;
  
  if (!SWIG_Python_UnpackTuple(args, "new_RRNativeDirectorSupport", 0, 0, 0)) SWIG_fail;
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      result = (RobotRaconteur::RRNativeDirectorSupport *)new RobotRaconteur::RRNativeDirectorSupport();
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_RobotRaconteur__RRNativeDirectorSupport, SWIG_POINTER_NEW |  0 );
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *_wrap_delete_RRNativeDirectorSupport(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  RobotRaconteur::RRNativeDirectorSupport *arg1 = (RobotRaconteur::RRNativeDirectorSupport *) 0 ;
  void *argp1 = 0 ;
  int res1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  res1 = SWIG_ConvertPtr(swig_obj[0], &argp1,SWIGTYPE_p_RobotRaconteur__RRNativeDirectorSupport, SWIG_POINTER_DISOWN |  0 );
  if (!SWIG_IsOK(res1)) {
    SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_RRNativeDirectorSupport" "', argument " "1"" of type '" "RobotRaconteur::RRNativeDirectorSupport *""'"); 
  }
  arg1 = reinterpret_cast< RobotRaconteur::RRNativeDirectorSupport * >(argp1);
  
  try {
    {
      SWIG_PYTHON_THREAD_BEGIN_ALLOW;
      delete arg1;
      SWIG_PYTHON_THREAD_END_ALLOW;
    }
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


SWIGINTERN PyObject *RRNativeDirectorSupport_swigregister(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *obj;
  if (!SWIG_Python_UnpackTuple(args, "swigregister", 1, 1, &obj)) return NULL;
  SWIG_TypeNewClientData(SWIGTYPE_p_RobotRaconteur__RRNativeDirectorSupport, SWIG_NewClientData(obj));
  return SWIG_Py_Void();
}

SWIGINTERN PyObject *RRNativeDirectorSupport_swiginit(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  return SWIG_Python_InitShadowInstance(args);
}

SWIGINTERN int Swig_var_PythonTracebackPrintExc_set(PyObject *_val) {
  {
    bool val;
    int res = SWIG_AsVal_bool(_val, &val);
    if (!SWIG_IsOK(res)) {
      SWIG_exception_fail(SWIG_ArgError(res), "in variable '""RobotRaconteur::PythonTracebackPrintExc""' of type '""bool""'");
    }
    RobotRaconteur::PythonTracebackPrintExc = static_cast< bool >(val);
  }
  return 0;
fail:
  return 1;
}


SWIGINTERN PyObject *Swig_var_PythonTracebackPrintExc_get(void) {
  PyObject *pyobj = 0;
  
  pyobj = SWIG_From_bool(static_cast< bool >(RobotRaconteur::PythonTracebackPrintExc));
  return pyobj;
}


SWIGINTERN PyObject *_wrap_SetPythonTracebackPrintExc(PyObject *SWIGUNUSEDPARM(self), PyObject *args) {
  PyObject *resultobj = 0;
  bool arg1 ;
  bool val1 ;
  int ecode1 = 0 ;
  PyObject *swig_obj[1] ;
  
  if (!args) SWIG_fail;
  swig_obj[0] = args;
  ecode1 = SWIG_AsVal_bool(swig_obj[0], &val1);
  if (!SWIG_IsOK(ecode1)) {
    SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "SetPythonTracebackPrintExc" "', argument " "1"" of type '" "bool""'");
  } 
  arg1 = static_cast< bool >(val1);
  
  try {
    RobotRaconteur::SetPythonTracebackPrintExc(arg1);
  }
  
  catch (RobotRaconteurException &e) {
    //PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    RRExceptionToPythonError(e);
    goto fail;
  }
  catch (std::exception &e) {
    PyErr_SetString(PyExc_Exception, const_cast<char*>(e.what()));
    goto fail;
  }
  /*catch (...)
      {
  		PyErr_SetString(PyExc_Exception,"Unknown Robot Raconteur error");
  		goto fail;
      }*/
  
  
  resultobj = SWIG_Py_Void();
  return resultobj;
fail:
  return NULL;
}


static PyMethodDef SwigMethods[] = {
	 { "SWIG_PyInstanceMethod_New", SWIG_PyInstanceMethod_New, METH_O, NULL},
	 { "delete_SwigPyIterator", _wrap_delete_SwigPyIterator, METH_O, NULL},
	 { "SwigPyIterator_value", _wrap_SwigPyIterator_value, METH_O, NULL},
	 { "SwigPyIterator_incr", _wrap_SwigPyIterator_incr, METH_VARARGS, NULL},
	 { "SwigPyIterator_decr", _wrap_SwigPyIterator_decr, METH_VARARGS, NULL},
	 { "SwigPyIterator_distance", _wrap_SwigPyIterator_distance, METH_VARARGS, NULL},
	 { "SwigPyIterator_equal", _wrap_SwigPyIterator_equal, METH_VARARGS, NULL},
	 { "SwigPyIterator_copy", _wrap_SwigPyIterator_copy, METH_O, NULL},
	 { "SwigPyIterator_next", _wrap_SwigPyIterator_next, METH_O, NULL},
	 { "SwigPyIterator___next__", _wrap_SwigPyIterator___next__, METH_O, NULL},
	 { "SwigPyIterator_previous", _wrap_SwigPyIterator_previous, METH_O, NULL},
	 { "SwigPyIterator_advance", _wrap_SwigPyIterator_advance, METH_VARARGS, NULL},
	 { "SwigPyIterator___eq__", _wrap_SwigPyIterator___eq__, METH_VARARGS, NULL},
	 { "SwigPyIterator___ne__", _wrap_SwigPyIterator___ne__, METH_VARARGS, NULL},
	 { "SwigPyIterator___iadd__", _wrap_SwigPyIterator___iadd__, METH_VARARGS, NULL},
	 { "SwigPyIterator___isub__", _wrap_SwigPyIterator___isub__, METH_VARARGS, NULL},
	 { "SwigPyIterator___add__", _wrap_SwigPyIterator___add__, METH_VARARGS, NULL},
	 { "SwigPyIterator___sub__", _wrap_SwigPyIterator___sub__, METH_VARARGS, NULL},
	 { "SwigPyIterator_swigregister", SwigPyIterator_swigregister, METH_O, NULL},
	 { "new_FilesystemPath", _wrap_new_FilesystemPath, METH_O, NULL},
	 { "FilesystemPath_string", _wrap_FilesystemPath_string, METH_O, NULL},
	 { "delete_FilesystemPath", _wrap_delete_FilesystemPath, METH_O, NULL},
	 { "FilesystemPath_swigregister", FilesystemPath_swigregister, METH_O, NULL},
	 { "FilesystemPath_swiginit", FilesystemPath_swiginit, METH_VARARGS, NULL},
	 { "vectorstring_iterator", _wrap_vectorstring_iterator, METH_O, NULL},
	 { "vectorstring___nonzero__", _wrap_vectorstring___nonzero__, METH_O, NULL},
	 { "vectorstring___bool__", _wrap_vectorstring___bool__, METH_O, NULL},
	 { "vectorstring___len__", _wrap_vectorstring___len__, METH_O, NULL},
	 { "vectorstring___getslice__", _wrap_vectorstring___getslice__, METH_VARARGS, NULL},
	 { "vectorstring___setslice__", _wrap_vectorstring___setslice__, METH_VARARGS, NULL},
	 { "vectorstring___delslice__", _wrap_vectorstring___delslice__, METH_VARARGS, NULL},
	 { "vectorstring___delitem__", _wrap_vectorstring___delitem__, METH_VARARGS, NULL},
	 { "vectorstring___getitem__", _wrap_vectorstring___getitem__, METH_VARARGS, NULL},
	 { "vectorstring___setitem__", _wrap_vectorstring___setitem__, METH_VARARGS, NULL},
	 { "vectorstring_pop", _wrap_vectorstring_pop, METH_O, NULL},
	 { "vectorstring_append", _wrap_vectorstring_append, METH_VARARGS, NULL},
	 { "vectorstring_empty", _wrap_vectorstring_empty, METH_O, NULL},
	 { "vectorstring_size", _wrap_vectorstring_size, METH_O, NULL},
	 { "vectorstring_swap", _wrap_vectorstring_swap, METH_VARARGS, NULL},
	 { "vectorstring_begin", _wrap_vectorstring_begin, METH_O, NULL},
	 { "vectorstring_end", _wrap_vectorstring_end, METH_O, NULL},
	 { "vectorstring_rbegin", _wrap_vectorstring_rbegin, METH_O, NULL},
	 { "vectorstring_rend", _wrap_vectorstring_rend, METH_O, NULL},
	 { "vectorstring_clear", _wrap_vectorstring_clear, METH_O, NULL},
	 { "vectorstring_get_allocator", _wrap_vectorstring_get_allocator, METH_O, NULL},
	 { "vectorstring_pop_back", _wrap_vectorstring_pop_back, METH_O, NULL},
	 { "vectorstring_erase", _wrap_vectorstring_erase, METH_VARARGS, NULL},
	 { "new_vectorstring", _wrap_new_vectorstring, METH_VARARGS, NULL},
	 { "vectorstring_push_back", _wrap_vectorstring_push_back, METH_VARARGS, NULL},
	 { "vectorstring_front", _wrap_vectorstring_front, METH_O, NULL},
	 { "vectorstring_back", _wrap_vectorstring_back, METH_O, NULL},
	 { "vectorstring_assign", _wrap_vectorstring_assign, METH_VARARGS, NULL},
	 { "vectorstring_resize", _wrap_vectorstring_resize, METH_VARARGS, NULL},
	 { "vectorstring_insert", _wrap_vectorstring_insert, METH_VARARGS, NULL},
	 { "vectorstring_reserve", _wrap_vectorstring_reserve, METH_VARARGS, NULL},
	 { "vectorstring_capacity", _wrap_vectorstring_capacity, METH_O, NULL},
	 { "delete_vectorstring", _wrap_delete_vectorstring, METH_O, NULL},
	 { "vectorstring_swigregister", vectorstring_swigregister, METH_O, NULL},
	 { "vectorstring_swiginit", vectorstring_swiginit, METH_VARARGS, NULL},
	 { "vectorint32_iterator", _wrap_vectorint32_iterator, METH_O, NULL},
	 { "vectorint32___nonzero__", _wrap_vectorint32___nonzero__, METH_O, NULL},
	 { "vectorint32___bool__", _wrap_vectorint32___bool__, METH_O, NULL},
	 { "vectorint32___len__", _wrap_vectorint32___len__, METH_O, NULL},
	 { "vectorint32___getslice__", _wrap_vectorint32___getslice__, METH_VARARGS, NULL},
	 { "vectorint32___setslice__", _wrap_vectorint32___setslice__, METH_VARARGS, NULL},
	 { "vectorint32___delslice__", _wrap_vectorint32___delslice__, METH_VARARGS, NULL},
	 { "vectorint32___delitem__", _wrap_vectorint32___delitem__, METH_VARARGS, NULL},
	 { "vectorint32___getitem__", _wrap_vectorint32___getitem__, METH_VARARGS, NULL},
	 { "vectorint32___setitem__", _wrap_vectorint32___setitem__, METH_VARARGS, NULL},
	 { "vectorint32_pop", _wrap_vectorint32_pop, METH_O, NULL},
	 { "vectorint32_append", _wrap_vectorint32_append, METH_VARARGS, NULL},
	 { "vectorint32_empty", _wrap_vectorint32_empty, METH_O, NULL},
	 { "vectorint32_size", _wrap_vectorint32_size, METH_O, NULL},
	 { "vectorint32_swap", _wrap_vectorint32_swap, METH_VARARGS, NULL},
	 { "vectorint32_begin", _wrap_vectorint32_begin, METH_O, NULL},
	 { "vectorint32_end", _wrap_vectorint32_end, METH_O, NULL},
	 { "vectorint32_rbegin", _wrap_vectorint32_rbegin, METH_O, NULL},
	 { "vectorint32_rend", _wrap_vectorint32_rend, METH_O, NULL},
	 { "vectorint32_clear", _wrap_vectorint32_clear, METH_O, NULL},
	 { "vectorint32_get_allocator", _wrap_vectorint32_get_allocator, METH_O, NULL},
	 { "vectorint32_pop_back", _wrap_vectorint32_pop_back, METH_O, NULL},
	 { "vectorint32_erase", _wrap_vectorint32_erase, METH_VARARGS, NULL},
	 { "new_vectorint32", _wrap_new_vectorint32, METH_VARARGS, NULL},
	 { "vectorint32_push_back", _wrap_vectorint32_push_back, METH_VARARGS, NULL},
	 { "vectorint32_front", _wrap_vectorint32_front, METH_O, NULL},
	 { "vectorint32_back", _wrap_vectorint32_back, METH_O, NULL},
	 { "vectorint32_assign", _wrap_vectorint32_assign, METH_VARARGS, NULL},
	 { "vectorint32_resize", _wrap_vectorint32_resize, METH_VARARGS, NULL},
	 { "vectorint32_insert", _wrap_vectorint32_insert, METH_VARARGS, NULL},
	 { "vectorint32_reserve", _wrap_vectorint32_reserve, METH_VARARGS, NULL},
	 { "vectorint32_capacity", _wrap_vectorint32_capacity, METH_O, NULL},
	 { "delete_vectorint32", _wrap_delete_vectorint32, METH_O, NULL},
	 { "vectorint32_swigregister", vectorint32_swigregister, METH_O, NULL},
	 { "vectorint32_swiginit", vectorint32_swiginit, METH_VARARGS, NULL},
	 { "vectoruint32_iterator", _wrap_vectoruint32_iterator, METH_O, NULL},
	 { "vectoruint32___nonzero__", _wrap_vectoruint32___nonzero__, METH_O, NULL},
	 { "vectoruint32___bool__", _wrap_vectoruint32___bool__, METH_O, NULL},
	 { "vectoruint32___len__", _wrap_vectoruint32___len__, METH_O, NULL},
	 { "vectoruint32___getslice__", _wrap_vectoruint32___getslice__, METH_VARARGS, NULL},
	 { "vectoruint32___setslice__", _wrap_vectoruint32___setslice__, METH_VARARGS, NULL},
	 { "vectoruint32___delslice__", _wrap_vectoruint32___delslice__, METH_VARARGS, NULL},
	 { "vectoruint32___delitem__", _wrap_vectoruint32___delitem__, METH_VARARGS, NULL},
	 { "vectoruint32___getitem__", _wrap_vectoruint32___getitem__, METH_VARARGS, NULL},
	 { "vectoruint32___setitem__", _wrap_vectoruint32___setitem__, METH_VARARGS, NULL},
	 { "vectoruint32_pop", _wrap_vectoruint32_pop, METH_O, NULL},
	 { "vectoruint32_append", _wrap_vectoruint32_append, METH_VARARGS, NULL},
	 { "vectoruint32_empty", _wrap_vectoruint32_empty, METH_O, NULL},
	 { "vectoruint32_size", _wrap_vectoruint32_size, METH_O, NULL},
	 { "vectoruint32_swap", _wrap_vectoruint32_swap, METH_VARARGS, NULL},
	 { "vectoruint32_begin", _wrap_vectoruint32_begin, METH_O, NULL},
	 { "vectoruint32_end", _wrap_vectoruint32_end, METH_O, NULL},
	 { "vectoruint32_rbegin", _wrap_vectoruint32_rbegin, METH_O, NULL},
	 { "vectoruint32_rend", _wrap_vectoruint32_rend, METH_O, NULL},
	 { "vectoruint32_clear", _wrap_vectoruint32_clear, METH_O, NULL},
	 { "vectoruint32_get_allocator", _wrap_vectoruint32_get_allocator, METH_O, NULL},
	 { "vectoruint32_pop_back", _wrap_vectoruint32_pop_back, METH_O, NULL},
	 { "vectoruint32_erase", _wrap_vectoruint32_erase, METH_VARARGS, NULL},
	 { "new_vectoruint32", _wrap_new_vectoruint32, METH_VARARGS, NULL},
	 { "vectoruint32_push_back", _wrap_vectoruint32_push_back, METH_VARARGS, NULL},
	 { "vectoruint32_front", _wrap_vectoruint32_front, METH_O, NULL},
	 { "vectoruint32_back", _wrap_vectoruint32_back, METH_O, NULL},
	 { "vectoruint32_assign", _wrap_vectoruint32_assign, METH_VARARGS, NULL},
	 { "vectoruint32_resize", _wrap_vectoruint32_resize, METH_VARARGS, NULL},
	 { "vectoruint32_insert", _wrap_vectoruint32_insert, METH_VARARGS, NULL},
	 { "vectoruint32_reserve", _wrap_vectoruint32_reserve, METH_VARARGS, NULL},
	 { "vectoruint32_capacity", _wrap_vectoruint32_capacity, METH_O, NULL},
	 { "delete_vectoruint32", _wrap_delete_vectoruint32, METH_O, NULL},
	 { "vectoruint32_swigregister", vectoruint32_swigregister, METH_O, NULL},
	 { "vectoruint32_swiginit", vectoruint32_swiginit, METH_VARARGS, NULL},
	 { "vector_int8_t_iterator", _wrap_vector_int8_t_iterator, METH_O, NULL},
	 { "vector_int8_t___nonzero__", _wrap_vector_int8_t___nonzero__, METH_O, NULL},
	 { "vector_int8_t___bool__", _wrap_vector_int8_t___bool__, METH_O, NULL},
	 { "vector_int8_t___len__", _wrap_vector_int8_t___len__, METH_O, NULL},
	 { "vector_int8_t___getslice__", _wrap_vector_int8_t___getslice__, METH_VARARGS, NULL},
	 { "vector_int8_t___setslice__", _wrap_vector_int8_t___setslice__, METH_VARARGS, NULL},
	 { "vector_int8_t___delslice__", _wrap_vector_int8_t___delslice__, METH_VARARGS, NULL},
	 { "vector_int8_t___delitem__", _wrap_vector_int8_t___delitem__, METH_VARARGS, NULL},
	 { "vector_int8_t___getitem__", _wrap_vector_int8_t___getitem__, METH_VARARGS, NULL},
	 { "vector_int8_t___setitem__", _wrap_vector_int8_t___setitem__, METH_VARARGS, NULL},
	 { "vector_int8_t_pop", _wrap_vector_int8_t_pop, METH_O, NULL},
	 { "vector_int8_t_append", _wrap_vector_int8_t_append, METH_VARARGS, NULL},
	 { "vector_int8_t_empty", _wrap_vector_int8_t_empty, METH_O, NULL},
	 { "vector_int8_t_size", _wrap_vector_int8_t_size, METH_O, NULL},
	 { "vector_int8_t_swap", _wrap_vector_int8_t_swap, METH_VARARGS, NULL},
	 { "vector_int8_t_begin", _wrap_vector_int8_t_begin, METH_O, NULL},
	 { "vector_int8_t_end", _wrap_vector_int8_t_end, METH_O, NULL},
	 { "vector_int8_t_rbegin", _wrap_vector_int8_t_rbegin, METH_O, NULL},
	 { "vector_int8_t_rend", _wrap_vector_int8_t_rend, METH_O, NULL},
	 { "vector_int8_t_clear", _wrap_vector_int8_t_clear, METH_O, NULL},
	 { "vector_int8_t_get_allocator", _wrap_vector_int8_t_get_allocator, METH_O, NULL},
	 { "vector_int8_t_pop_back", _wrap_vector_int8_t_pop_back, METH_O, NULL},
	 { "vector_int8_t_erase", _wrap_vector_int8_t_erase, METH_VARARGS, NULL},
	 { "new_vector_int8_t", _wrap_new_vector_int8_t, METH_VARARGS, NULL},
	 { "vector_int8_t_push_back", _wrap_vector_int8_t_push_back, METH_VARARGS, NULL},
	 { "vector_int8_t_front", _wrap_vector_int8_t_front, METH_O, NULL},
	 { "vector_int8_t_back", _wrap_vector_int8_t_back, METH_O, NULL},
	 { "vector_int8_t_assign", _wrap_vector_int8_t_assign, METH_VARARGS, NULL},
	 { "vector_int8_t_resize", _wrap_vector_int8_t_resize, METH_VARARGS, NULL},
	 { "vector_int8_t_insert", _wrap_vector_int8_t_insert, METH_VARARGS, NULL},
	 { "vector_int8_t_reserve", _wrap_vector_int8_t_reserve, METH_VARARGS, NULL},
	 { "vector_int8_t_capacity", _wrap_vector_int8_t_capacity, METH_O, NULL},
	 { "delete_vector_int8_t", _wrap_delete_vector_int8_t, METH_O, NULL},
	 { "vector_int8_t_swigregister", vector_int8_t_swigregister, METH_O, NULL},
	 { "vector_int8_t_swiginit", vector_int8_t_swiginit, METH_VARARGS, NULL},
	 { "vector_uint8_t_iterator", _wrap_vector_uint8_t_iterator, METH_O, NULL},
	 { "vector_uint8_t___nonzero__", _wrap_vector_uint8_t___nonzero__, METH_O, NULL},
	 { "vector_uint8_t___bool__", _wrap_vector_uint8_t___bool__, METH_O, NULL},
	 { "vector_uint8_t___len__", _wrap_vector_uint8_t___len__, METH_O, NULL},
	 { "vector_uint8_t___getslice__", _wrap_vector_uint8_t___getslice__, METH_VARARGS, NULL},
	 { "vector_uint8_t___setslice__", _wrap_vector_uint8_t___setslice__, METH_VARARGS, NULL},
	 { "vector_uint8_t___delslice__", _wrap_vector_uint8_t___delslice__, METH_VARARGS, NULL},
	 { "vector_uint8_t___delitem__", _wrap_vector_uint8_t___delitem__, METH_VARARGS, NULL},
	 { "vector_uint8_t___getitem__", _wrap_vector_uint8_t___getitem__, METH_VARARGS, NULL},
	 { "vector_uint8_t___setitem__", _wrap_vector_uint8_t___setitem__, METH_VARARGS, NULL},
	 { "vector_uint8_t_pop", _wrap_vector_uint8_t_pop, METH_O, NULL},
	 { "vector_uint8_t_append", _wrap_vector_uint8_t_append, METH_VARARGS, NULL},
	 { "vector_uint8_t_empty", _wrap_vector_uint8_t_empty, METH_O, NULL},
	 { "vector_uint8_t_size", _wrap_vector_uint8_t_size, METH_O, NULL},
	 { "vector_uint8_t_swap", _wrap_vector_uint8_t_swap, METH_VARARGS, NULL},
	 { "vector_uint8_t_begin", _wrap_vector_uint8_t_begin, METH_O, NULL},
	 { "vector_uint8_t_end", _wrap_vector_uint8_t_end, METH_O, NULL},
	 { "vector_uint8_t_rbegin", _wrap_vector_uint8_t_rbegin, METH_O, NULL},
	 { "vector_uint8_t_rend", _wrap_vector_uint8_t_rend, METH_O, NULL},
	 { "vector_uint8_t_clear", _wrap_vector_uint8_t_clear, METH_O, NULL},
	 { "vector_uint8_t_get_allocator", _wrap_vector_uint8_t_get_allocator, METH_O, NULL},
	 { "vector_uint8_t_pop_back", _wrap_vector_uint8_t_pop_back, METH_O, NULL},
	 { "vector_uint8_t_erase", _wrap_vector_uint8_t_erase, METH_VARARGS, NULL},
	 { "new_vector_uint8_t", _wrap_new_vector_uint8_t, METH_VARARGS, NULL},
	 { "vector_uint8_t_push_back", _wrap_vector_uint8_t_push_back, METH_VARARGS, NULL},
	 { "vector_uint8_t_front", _wrap_vector_uint8_t_front, METH_O, NULL},
	 { "vector_uint8_t_back", _wrap_vector_uint8_t_back, METH_O, NULL},
	 { "vector_uint8_t_assign", _wrap_vector_uint8_t_assign, METH_VARARGS, NULL},
	 { "vector_uint8_t_resize", _wrap_vector_uint8_t_resize, METH_VARARGS, NULL},
	 { "vector_uint8_t_insert", _wrap_vector_uint8_t_insert, METH_VARARGS, NULL},
	 { "vector_uint8_t_reserve", _wrap_vector_uint8_t_reserve, METH_VARARGS, NULL},
	 { "vector_uint8_t_capacity", _wrap_vector_uint8_t_capacity, METH_O, NULL},
	 { "delete_vector_uint8_t", _wrap_delete_vector_uint8_t, METH_O, NULL},
	 { "vector_uint8_t_swigregister", vector_uint8_t_swigregister, METH_O, NULL},
	 { "vector_uint8_t_swiginit", vector_uint8_t_swiginit, METH_VARARGS, NULL},
	 { "vector_uint64_t_iterator", _wrap_vector_uint64_t_iterator, METH_O, NULL},
	 { "vector_uint64_t___nonzero__", _wrap_vector_uint64_t___nonzero__, METH_O, NULL},
	 { "vector_uint64_t___bool__", _wrap_vector_uint64_t___bool__, METH_O, NULL},
	 { "vector_uint64_t___len__", _wrap_vector_uint64_t___len__, METH_O, NULL},
	 { "vector_uint64_t___getslice__", _wrap_vector_uint64_t___getslice__, METH_VARARGS, NULL},
	 { "vector_uint64_t___setslice__", _wrap_vector_uint64_t___setslice__, METH_VARARGS, NULL},
	 { "vector_uint64_t___delslice__", _wrap_vector_uint64_t___delslice__, METH_VARARGS, NULL},
	 { "vector_uint64_t___delitem__", _wrap_vector_uint64_t___delitem__, METH_VARARGS, NULL},
	 { "vector_uint64_t___getitem__", _wrap_vector_uint64_t___getitem__, METH_VARARGS, NULL},
	 { "vector_uint64_t___setitem__", _wrap_vector_uint64_t___setitem__, METH_VARARGS, NULL},
	 { "vector_uint64_t_pop", _wrap_vector_uint64_t_pop, METH_O, NULL},
	 { "vector_uint64_t_append", _wrap_vector_uint64_t_append, METH_VARARGS, NULL},
	 { "vector_uint64_t_empty", _wrap_vector_uint64_t_empty, METH_O, NULL},
	 { "vector_uint64_t_size", _wrap_vector_uint64_t_size, METH_O, NULL},
	 { "vector_uint64_t_swap", _wrap_vector_uint64_t_swap, METH_VARARGS, NULL},
	 { "vector_uint64_t_begin", _wrap_vector_uint64_t_begin, METH_O, NULL},
	 { "vector_uint64_t_end", _wrap_vector_uint64_t_end, METH_O, NULL},
	 { "vector_uint64_t_rbegin", _wrap_vector_uint64_t_rbegin, METH_O, NULL},
	 { "vector_uint64_t_rend", _wrap_vector_uint64_t_rend, METH_O, NULL},
	 { "vector_uint64_t_clear", _wrap_vector_uint64_t_clear, METH_O, NULL},
	 { "vector_uint64_t_get_allocator", _wrap_vector_uint64_t_get_allocator, METH_O, NULL},
	 { "vector_uint64_t_pop_back", _wrap_vector_uint64_t_pop_back, METH_O, NULL},
	 { "vector_uint64_t_erase", _wrap_vector_uint64_t_erase, METH_VARARGS, NULL},
	 { "new_vector_uint64_t", _wrap_new_vector_uint64_t, METH_VARARGS, NULL},
	 { "vector_uint64_t_push_back", _wrap_vector_uint64_t_push_back, METH_VARARGS, NULL},
	 { "vector_uint64_t_front", _wrap_vector_uint64_t_front, METH_O, NULL},
	 { "vector_uint64_t_back", _wrap_vector_uint64_t_back, METH_O, NULL},
	 { "vector_uint64_t_assign", _wrap_vector_uint64_t_assign, METH_VARARGS, NULL},
	 { "vector_uint64_t_resize", _wrap_vector_uint64_t_resize, METH_VARARGS, NULL},
	 { "vector_uint64_t_insert", _wrap_vector_uint64_t_insert, METH_VARARGS, NULL},
	 { "vector_uint64_t_reserve", _wrap_vector_uint64_t_reserve, METH_VARARGS, NULL},
	 { "vector_uint64_t_capacity", _wrap_vector_uint64_t_capacity, METH_O, NULL},
	 { "delete_vector_uint64_t", _wrap_delete_vector_uint64_t, METH_O, NULL},
	 { "vector_uint64_t_swigregister", vector_uint64_t_swigregister, METH_O, NULL},
	 { "vector_uint64_t_swiginit", vector_uint64_t_swiginit, METH_VARARGS, NULL},
	 { "map_strstr_iterator", _wrap_map_strstr_iterator, METH_O, NULL},
	 { "map_strstr___nonzero__", _wrap_map_strstr___nonzero__, METH_O, NULL},
	 { "map_strstr___bool__", _wrap_map_strstr___bool__, METH_O, NULL},
	 { "map_strstr___len__", _wrap_map_strstr___len__, METH_O, NULL},
	 { "map_strstr___getitem__", _wrap_map_strstr___getitem__, METH_VARARGS, NULL},
	 { "map_strstr___delitem__", _wrap_map_strstr___delitem__, METH_VARARGS, NULL},
	 { "map_strstr_has_key", _wrap_map_strstr_has_key, METH_VARARGS, NULL},
	 { "map_strstr_keys", _wrap_map_strstr_keys, METH_O, NULL},
	 { "map_strstr_values", _wrap_map_strstr_values, METH_O, NULL},
	 { "map_strstr_items", _wrap_map_strstr_items, METH_O, NULL},
	 { "map_strstr___contains__", _wrap_map_strstr___contains__, METH_VARARGS, NULL},
	 { "map_strstr_key_iterator", _wrap_map_strstr_key_iterator, METH_O, NULL},
	 { "map_strstr_value_iterator", _wrap_map_strstr_value_iterator, METH_O, NULL},
	 { "map_strstr___setitem__", _wrap_map_strstr___setitem__, METH_VARARGS, NULL},
	 { "map_strstr_asdict", _wrap_map_strstr_asdict, METH_O, NULL},
	 { "new_map_strstr", _wrap_new_map_strstr, METH_VARARGS, NULL},
	 { "map_strstr_empty", _wrap_map_strstr_empty, METH_O, NULL},
	 { "map_strstr_size", _wrap_map_strstr_size, METH_O, NULL},
	 { "map_strstr_swap", _wrap_map_strstr_swap, METH_VARARGS, NULL},
	 { "map_strstr_begin", _wrap_map_strstr_begin, METH_O, NULL},
	 { "map_strstr_end", _wrap_map_strstr_end, METH_O, NULL},
	 { "map_strstr_rbegin", _wrap_map_strstr_rbegin, METH_O, NULL},
	 { "map_strstr_rend", _wrap_map_strstr_rend, METH_O, NULL},
	 { "map_strstr_clear", _wrap_map_strstr_clear, METH_O, NULL},
	 { "map_strstr_get_allocator", _wrap_map_strstr_get_allocator, METH_O, NULL},
	 { "map_strstr_count", _wrap_map_strstr_count, METH_VARARGS, NULL},
	 { "map_strstr_erase", _wrap_map_strstr_erase, METH_VARARGS, NULL},
	 { "map_strstr_find", _wrap_map_strstr_find, METH_VARARGS, NULL},
	 { "map_strstr_lower_bound", _wrap_map_strstr_lower_bound, METH_VARARGS, NULL},
	 { "map_strstr_upper_bound", _wrap_map_strstr_upper_bound, METH_VARARGS, NULL},
	 { "delete_map_strstr", _wrap_delete_map_strstr, METH_O, NULL},
	 { "map_strstr_swigregister", map_strstr_swigregister, METH_O, NULL},
	 { "map_strstr_swiginit", map_strstr_swiginit, METH_VARARGS, NULL},
	 { "GetRRDataTypeString", _wrap_GetRRDataTypeString, METH_O, NULL},
	 { "IsTypeRRArray", _wrap_IsTypeRRArray, METH_O, NULL},
	 { "IsTypeNumeric", _wrap_IsTypeNumeric, METH_O, NULL},
	 { "delete_RRObject", _wrap_delete_RRObject, METH_O, NULL},
	 { "RRObject_swigregister", RRObject_swigregister, METH_O, NULL},
	 { "delete_RRValue", _wrap_delete_RRValue, METH_O, NULL},
	 { "RRValue_swigregister", RRValue_swigregister, METH_O, NULL},
	 { "MessageElementData_GetTypeString", _wrap_MessageElementData_GetTypeString, METH_O, NULL},
	 { "MessageElementData_GetTypeID", _wrap_MessageElementData_GetTypeID, METH_O, NULL},
	 { "delete_MessageElementData", _wrap_delete_MessageElementData, METH_O, NULL},
	 { "MessageElementData_swigregister", MessageElementData_swigregister, METH_O, NULL},
	 { "RRBaseArray_GetTypeString", _wrap_RRBaseArray_GetTypeString, METH_O, NULL},
	 { "RRBaseArray_size", _wrap_RRBaseArray_size, METH_O, NULL},
	 { "RRBaseArray_void_ptr", _wrap_RRBaseArray_void_ptr, METH_O, NULL},
	 { "RRBaseArray_ElementSize", _wrap_RRBaseArray_ElementSize, METH_O, NULL},
	 { "RRBaseArray_GetTypeID", _wrap_RRBaseArray_GetTypeID, METH_O, NULL},
	 { "delete_RRBaseArray", _wrap_delete_RRBaseArray, METH_O, NULL},
	 { "RRBaseArray_swigregister", RRBaseArray_swigregister, METH_O, NULL},
	 { "AllocateRRArrayByType", _wrap_AllocateRRArrayByType, METH_VARARGS, NULL},
	 { "RRMultiDimArrayUntyped_Dims_set", _wrap_RRMultiDimArrayUntyped_Dims_set, METH_VARARGS, NULL},
	 { "RRMultiDimArrayUntyped_Dims_get", _wrap_RRMultiDimArrayUntyped_Dims_get, METH_O, NULL},
	 { "RRMultiDimArrayUntyped_Array_set", _wrap_RRMultiDimArrayUntyped_Array_set, METH_VARARGS, NULL},
	 { "RRMultiDimArrayUntyped_Array_get", _wrap_RRMultiDimArrayUntyped_Array_get, METH_O, NULL},
	 { "new_RRMultiDimArrayUntyped", _wrap_new_RRMultiDimArrayUntyped, METH_NOARGS, NULL},
	 { "delete_RRMultiDimArrayUntyped", _wrap_delete_RRMultiDimArrayUntyped, METH_O, NULL},
	 { "RRMultiDimArrayUntyped_swigregister", RRMultiDimArrayUntyped_swigregister, METH_O, NULL},
	 { "RRMultiDimArrayUntyped_swiginit", RRMultiDimArrayUntyped_swiginit, METH_VARARGS, NULL},
	 { "new_ServicePathSegment", _wrap_new_ServicePathSegment, METH_VARARGS, "\n"
		"\n"
		"Construct a new ServicePathSegment with a name\n"
		"\n"
		":param name: The name of the segment\n"
		":type name: str\n"
		"\n"
		""},
	 { "ServicePathSegment_name_set", _wrap_ServicePathSegment_name_set, METH_VARARGS, "The name of the segment"},
	 { "ServicePathSegment_name_get", _wrap_ServicePathSegment_name_get, METH_O, "The name of the segment"},
	 { "ServicePathSegment_index_set", _wrap_ServicePathSegment_index_set, METH_VARARGS, "The index of the segment"},
	 { "ServicePathSegment_index_get", _wrap_ServicePathSegment_index_get, METH_O, "The index of the segment"},
	 { "delete_ServicePathSegment", _wrap_delete_ServicePathSegment, METH_O, NULL},
	 { "ServicePathSegment_swigregister", ServicePathSegment_swigregister, METH_O, NULL},
	 { "ServicePathSegment_swiginit", ServicePathSegment_swiginit, METH_VARARGS, NULL},
	 { "ServicePathSegments_iterator", _wrap_ServicePathSegments_iterator, METH_O, NULL},
	 { "ServicePathSegments___nonzero__", _wrap_ServicePathSegments___nonzero__, METH_O, NULL},
	 { "ServicePathSegments___bool__", _wrap_ServicePathSegments___bool__, METH_O, NULL},
	 { "ServicePathSegments___len__", _wrap_ServicePathSegments___len__, METH_O, NULL},
	 { "ServicePathSegments___getslice__", _wrap_ServicePathSegments___getslice__, METH_VARARGS, NULL},
	 { "ServicePathSegments___setslice__", _wrap_ServicePathSegments___setslice__, METH_VARARGS, NULL},
	 { "ServicePathSegments___delslice__", _wrap_ServicePathSegments___delslice__, METH_VARARGS, NULL},
	 { "ServicePathSegments___delitem__", _wrap_ServicePathSegments___delitem__, METH_VARARGS, NULL},
	 { "ServicePathSegments___getitem__", _wrap_ServicePathSegments___getitem__, METH_VARARGS, NULL},
	 { "ServicePathSegments___setitem__", _wrap_ServicePathSegments___setitem__, METH_VARARGS, NULL},
	 { "ServicePathSegments_pop", _wrap_ServicePathSegments_pop, METH_O, NULL},
	 { "ServicePathSegments_append", _wrap_ServicePathSegments_append, METH_VARARGS, NULL},
	 { "ServicePathSegments_empty", _wrap_ServicePathSegments_empty, METH_O, NULL},
	 { "ServicePathSegments_size", _wrap_ServicePathSegments_size, METH_O, NULL},
	 { "ServicePathSegments_swap", _wrap_ServicePathSegments_swap, METH_VARARGS, NULL},
	 { "ServicePathSegments_begin", _wrap_ServicePathSegments_begin, METH_O, NULL},
	 { "ServicePathSegments_end", _wrap_ServicePathSegments_end, METH_O, NULL},
	 { "ServicePathSegments_rbegin", _wrap_ServicePathSegments_rbegin, METH_O, NULL},
	 { "ServicePathSegments_rend", _wrap_ServicePathSegments_rend, METH_O, NULL},
	 { "ServicePathSegments_clear", _wrap_ServicePathSegments_clear, METH_O, NULL},
	 { "ServicePathSegments_get_allocator", _wrap_ServicePathSegments_get_allocator, METH_O, NULL},
	 { "ServicePathSegments_pop_back", _wrap_ServicePathSegments_pop_back, METH_O, NULL},
	 { "ServicePathSegments_erase", _wrap_ServicePathSegments_erase, METH_VARARGS, NULL},
	 { "new_ServicePathSegments", _wrap_new_ServicePathSegments, METH_VARARGS, NULL},
	 { "ServicePathSegments_push_back", _wrap_ServicePathSegments_push_back, METH_VARARGS, NULL},
	 { "ServicePathSegments_front", _wrap_ServicePathSegments_front, METH_O, NULL},
	 { "ServicePathSegments_back", _wrap_ServicePathSegments_back, METH_O, NULL},
	 { "ServicePathSegments_assign", _wrap_ServicePathSegments_assign, METH_VARARGS, NULL},
	 { "ServicePathSegments_resize", _wrap_ServicePathSegments_resize, METH_VARARGS, NULL},
	 { "ServicePathSegments_insert", _wrap_ServicePathSegments_insert, METH_VARARGS, NULL},
	 { "ServicePathSegments_reserve", _wrap_ServicePathSegments_reserve, METH_VARARGS, NULL},
	 { "ServicePathSegments_capacity", _wrap_ServicePathSegments_capacity, METH_O, NULL},
	 { "delete_ServicePathSegments", _wrap_delete_ServicePathSegments, METH_O, NULL},
	 { "ServicePathSegments_swigregister", ServicePathSegments_swigregister, METH_O, NULL},
	 { "ServicePathSegments_swiginit", ServicePathSegments_swiginit, METH_VARARGS, NULL},
	 { "EncodeServicePathIndex", _wrap_EncodeServicePathIndex, METH_O, "\n"
		"\n"
		"Encode a service path index for use in a Robot Raconteur service path\n"
		"\n"
		":param index: The index to encode\n"
		":type index: str\n"
		":return: The encoded index\n"
		":rtype: str\n"
		"\n"
		""},
	 { "DecodeServicePathIndex", _wrap_DecodeServicePathIndex, METH_O, "\n"
		"\n"
		"Decode a service path index from a Robot Raconteur service path\n"
		"\n"
		":param index: The index to decode\n"
		":type index: str\n"
		":return: The decoded index\n"
		":rtype: str\n"
		"\n"
		""},
	 { "ParseServicePath", _wrap_ParseServicePath, METH_O, "\n"
		"\n"
		"Split a service path into segments\n"
		"\n"
		":param path: The path to split\n"
		":type path: str\n"
		":return: The segments of the path\n"
		":rtype: List[RobotRaconteur.ServicePathSegment]\n"
		"\n"
		""},
	 { "BuildServicePath", _wrap_BuildServicePath, METH_O, "\n"
		"\n"
		"Join service path segments into a path\n"
		"\n"
		":param segments: The segments to join\n"
		":type segments: List[RobotRaconteur.ServicePathSegment]\n"
		":return: The joined path\n"
		":rtype: str\n"
		"\n"
		""},
	 { "IsStringName", _wrap_IsStringName, METH_O, NULL},
	 { "IsStringScopedName", _wrap_IsStringScopedName, METH_O, NULL},
	 { "IsStringUUID", _wrap_IsStringUUID, METH_O, NULL},
	 { "IsStringIdentifier", _wrap_IsStringIdentifier, METH_O, NULL},
	 { "MultiDimArray_CalculateCopyIndicesIter_Next", _wrap_MultiDimArray_CalculateCopyIndicesIter_Next, METH_O, NULL},
	 { "delete_MultiDimArray_CalculateCopyIndicesIter", _wrap_delete_MultiDimArray_CalculateCopyIndicesIter, METH_O, NULL},
	 { "MultiDimArray_CalculateCopyIndicesIter_swigregister", MultiDimArray_CalculateCopyIndicesIter_swigregister, METH_O, NULL},
	 { "MultiDimArray_CalculateCopyIndicesBeginIter", _wrap_MultiDimArray_CalculateCopyIndicesBeginIter, METH_VARARGS, NULL},
	 { "NodeID_ToString", _wrap_NodeID_ToString, METH_VARARGS, NULL},
	 { "NodeID_NewUniqueID", _wrap_NodeID_NewUniqueID, METH_NOARGS, "\n"
		"\n"
		"Generate a new random NodeID UUID\n"
		"\n"
		"Returned UUID is statistically guaranteed to be unique\n"
		"\n"
		":return: NodeID The newly generated UUID\n"
		":rtype: RobotRaconteur.NodeID\n"
		"\n"
		""},
	 { "NodeID___eq__", _wrap_NodeID___eq__, METH_VARARGS, NULL},
	 { "NodeID___ne__", _wrap_NodeID___ne__, METH_VARARGS, NULL},
	 { "NodeID___lt__", _wrap_NodeID___lt__, METH_VARARGS, NULL},
	 { "NodeID_IsAnyNode", _wrap_NodeID_IsAnyNode, METH_O, "\n"
		"\n"
		"Is the NodeID UUID all zeros\n"
		"\n"
		"The all zero UUID respresents \"any\" node, or an unset NodeID\n"
		"\n"
		":return: True The NodeID UUID is all zeros, representing any node, False The NodeID UUID is not all zeros\n"
		":rtype: bool\n"
		"\n"
		""},
	 { "NodeID_GetAny", _wrap_NodeID_GetAny, METH_NOARGS, "\n"
		"\n"
		"Get the \"any\" NodeId\n"
		"\n"
		":return: The \"any\" NodeID\n"
		":rtype: RobotRaconteur.NodeID\n"
		"\n"
		""},
	 { "new_NodeID", _wrap_new_NodeID, METH_VARARGS, NULL},
	 { "NodeID_ToByteArray", _wrap_NodeID_ToByteArray, METH_O, "\n"
		"\n"
		"Convert the NodeID UUID to bytes\n"
		"\n"
		":return: The UUID as bytes\n"
		":rtype: bytearray\n"
		"\n"
		""},
	 { "NodeID_FromByteArray", _wrap_NodeID_FromByteArray, METH_O, "\n"
		"\n"
		"Returns a new NodeID from bytes\n"
		"\n"
		":param bytes: The bytes\n"
		":type bytes: bytearray\n"
		":return: The new NodeID instance with the specified UUID\n"
		":rtype: RobotRaconteur.NodeID\n"
		"\n"
		""},
	 { "delete_NodeID", _wrap_delete_NodeID, METH_O, NULL},
	 { "NodeID_swigregister", NodeID_swigregister, METH_O, NULL},
	 { "NodeID_swiginit", NodeID_swiginit, METH_VARARGS, NULL},
	 { "vectorptr_messageentry_iterator", _wrap_vectorptr_messageentry_iterator, METH_O, NULL},
	 { "vectorptr_messageentry___nonzero__", _wrap_vectorptr_messageentry___nonzero__, METH_O, NULL},
	 { "vectorptr_messageentry___bool__", _wrap_vectorptr_messageentry___bool__, METH_O, NULL},
	 { "vectorptr_messageentry___len__", _wrap_vectorptr_messageentry___len__, METH_O, NULL},
	 { "vectorptr_messageentry___getslice__", _wrap_vectorptr_messageentry___getslice__, METH_VARARGS, NULL},
	 { "vectorptr_messageentry___setslice__", _wrap_vectorptr_messageentry___setslice__, METH_VARARGS, NULL},
	 { "vectorptr_messageentry___delslice__", _wrap_vectorptr_messageentry___delslice__, METH_VARARGS, NULL},
	 { "vectorptr_messageentry___delitem__", _wrap_vectorptr_messageentry___delitem__, METH_VARARGS, NULL},
	 { "vectorptr_messageentry___getitem__", _wrap_vectorptr_messageentry___getitem__, METH_VARARGS, NULL},
	 { "vectorptr_messageentry___setitem__", _wrap_vectorptr_messageentry___setitem__, METH_VARARGS, NULL},
	 { "vectorptr_messageentry_pop", _wrap_vectorptr_messageentry_pop, METH_O, NULL},
	 { "vectorptr_messageentry_append", _wrap_vectorptr_messageentry_append, METH_VARARGS, NULL},
	 { "vectorptr_messageentry_empty", _wrap_vectorptr_messageentry_empty, METH_O, NULL},
	 { "vectorptr_messageentry_size", _wrap_vectorptr_messageentry_size, METH_O, NULL},
	 { "vectorptr_messageentry_swap", _wrap_vectorptr_messageentry_swap, METH_VARARGS, NULL},
	 { "vectorptr_messageentry_begin", _wrap_vectorptr_messageentry_begin, METH_O, NULL},
	 { "vectorptr_messageentry_end", _wrap_vectorptr_messageentry_end, METH_O, NULL},
	 { "vectorptr_messageentry_rbegin", _wrap_vectorptr_messageentry_rbegin, METH_O, NULL},
	 { "vectorptr_messageentry_rend", _wrap_vectorptr_messageentry_rend, METH_O, NULL},
	 { "vectorptr_messageentry_clear", _wrap_vectorptr_messageentry_clear, METH_O, NULL},
	 { "vectorptr_messageentry_get_allocator", _wrap_vectorptr_messageentry_get_allocator, METH_O, NULL},
	 { "vectorptr_messageentry_pop_back", _wrap_vectorptr_messageentry_pop_back, METH_O, NULL},
	 { "vectorptr_messageentry_erase", _wrap_vectorptr_messageentry_erase, METH_VARARGS, NULL},
	 { "new_vectorptr_messageentry", _wrap_new_vectorptr_messageentry, METH_VARARGS, NULL},
	 { "vectorptr_messageentry_push_back", _wrap_vectorptr_messageentry_push_back, METH_VARARGS, NULL},
	 { "vectorptr_messageentry_front", _wrap_vectorptr_messageentry_front, METH_O, NULL},
	 { "vectorptr_messageentry_back", _wrap_vectorptr_messageentry_back, METH_O, NULL},
	 { "vectorptr_messageentry_assign", _wrap_vectorptr_messageentry_assign, METH_VARARGS, NULL},
	 { "vectorptr_messageentry_resize", _wrap_vectorptr_messageentry_resize, METH_VARARGS, NULL},
	 { "vectorptr_messageentry_insert", _wrap_vectorptr_messageentry_insert, METH_VARARGS, NULL},
	 { "vectorptr_messageentry_reserve", _wrap_vectorptr_messageentry_reserve, METH_VARARGS, NULL},
	 { "vectorptr_messageentry_capacity", _wrap_vectorptr_messageentry_capacity, METH_O, NULL},
	 { "delete_vectorptr_messageentry", _wrap_delete_vectorptr_messageentry, METH_O, NULL},
	 { "vectorptr_messageentry_swigregister", vectorptr_messageentry_swigregister, METH_O, NULL},
	 { "vectorptr_messageentry_swiginit", vectorptr_messageentry_swiginit, METH_VARARGS, NULL},
	 { "vectorptr_messageelement_iterator", _wrap_vectorptr_messageelement_iterator, METH_O, NULL},
	 { "vectorptr_messageelement___nonzero__", _wrap_vectorptr_messageelement___nonzero__, METH_O, NULL},
	 { "vectorptr_messageelement___bool__", _wrap_vectorptr_messageelement___bool__, METH_O, NULL},
	 { "vectorptr_messageelement___len__", _wrap_vectorptr_messageelement___len__, METH_O, NULL},
	 { "vectorptr_messageelement___getslice__", _wrap_vectorptr_messageelement___getslice__, METH_VARARGS, NULL},
	 { "vectorptr_messageelement___setslice__", _wrap_vectorptr_messageelement___setslice__, METH_VARARGS, NULL},
	 { "vectorptr_messageelement___delslice__", _wrap_vectorptr_messageelement___delslice__, METH_VARARGS, NULL},
	 { "vectorptr_messageelement___delitem__", _wrap_vectorptr_messageelement___delitem__, METH_VARARGS, NULL},
	 { "vectorptr_messageelement___getitem__", _wrap_vectorptr_messageelement___getitem__, METH_VARARGS, NULL},
	 { "vectorptr_messageelement___setitem__", _wrap_vectorptr_messageelement___setitem__, METH_VARARGS, NULL},
	 { "vectorptr_messageelement_pop", _wrap_vectorptr_messageelement_pop, METH_O, NULL},
	 { "vectorptr_messageelement_append", _wrap_vectorptr_messageelement_append, METH_VARARGS, NULL},
	 { "vectorptr_messageelement_empty", _wrap_vectorptr_messageelement_empty, METH_O, NULL},
	 { "vectorptr_messageelement_size", _wrap_vectorptr_messageelement_size, METH_O, NULL},
	 { "vectorptr_messageelement_swap", _wrap_vectorptr_messageelement_swap, METH_VARARGS, NULL},
	 { "vectorptr_messageelement_begin", _wrap_vectorptr_messageelement_begin, METH_O, NULL},
	 { "vectorptr_messageelement_end", _wrap_vectorptr_messageelement_end, METH_O, NULL},
	 { "vectorptr_messageelement_rbegin", _wrap_vectorptr_messageelement_rbegin, METH_O, NULL},
	 { "vectorptr_messageelement_rend", _wrap_vectorptr_messageelement_rend, METH_O, NULL},
	 { "vectorptr_messageelement_clear", _wrap_vectorptr_messageelement_clear, METH_O, NULL},
	 { "vectorptr_messageelement_get_allocator", _wrap_vectorptr_messageelement_get_allocator, METH_O, NULL},
	 { "vectorptr_messageelement_pop_back", _wrap_vectorptr_messageelement_pop_back, METH_O, NULL},
	 { "vectorptr_messageelement_erase", _wrap_vectorptr_messageelement_erase, METH_VARARGS, NULL},
	 { "new_vectorptr_messageelement", _wrap_new_vectorptr_messageelement, METH_VARARGS, NULL},
	 { "vectorptr_messageelement_push_back", _wrap_vectorptr_messageelement_push_back, METH_VARARGS, NULL},
	 { "vectorptr_messageelement_front", _wrap_vectorptr_messageelement_front, METH_O, NULL},
	 { "vectorptr_messageelement_back", _wrap_vectorptr_messageelement_back, METH_O, NULL},
	 { "vectorptr_messageelement_assign", _wrap_vectorptr_messageelement_assign, METH_VARARGS, NULL},
	 { "vectorptr_messageelement_resize", _wrap_vectorptr_messageelement_resize, METH_VARARGS, NULL},
	 { "vectorptr_messageelement_insert", _wrap_vectorptr_messageelement_insert, METH_VARARGS, NULL},
	 { "vectorptr_messageelement_reserve", _wrap_vectorptr_messageelement_reserve, METH_VARARGS, NULL},
	 { "vectorptr_messageelement_capacity", _wrap_vectorptr_messageelement_capacity, METH_O, NULL},
	 { "delete_vectorptr_messageelement", _wrap_delete_vectorptr_messageelement, METH_O, NULL},
	 { "vectorptr_messageelement_swigregister", vectorptr_messageelement_swigregister, METH_O, NULL},
	 { "vectorptr_messageelement_swiginit", vectorptr_messageelement_swiginit, METH_VARARGS, NULL},
	 { "Message_header_set", _wrap_Message_header_set, METH_VARARGS, NULL},
	 { "Message_header_get", _wrap_Message_header_get, METH_O, NULL},
	 { "Message_entries_set", _wrap_Message_entries_set, METH_VARARGS, NULL},
	 { "Message_entries_get", _wrap_Message_entries_get, METH_O, NULL},
	 { "new_Message", _wrap_new_Message, METH_NOARGS, NULL},
	 { "Message_ComputeSize", _wrap_Message_ComputeSize, METH_O, NULL},
	 { "Message_FindEntry", _wrap_Message_FindEntry, METH_VARARGS, NULL},
	 { "Message_AddEntry", _wrap_Message_AddEntry, METH_VARARGS, NULL},
	 { "delete_Message", _wrap_delete_Message, METH_O, NULL},
	 { "Message_swigregister", Message_swigregister, METH_O, NULL},
	 { "Message_swiginit", Message_swiginit, METH_VARARGS, NULL},
	 { "MessageHeader_HeaderSize_set", _wrap_MessageHeader_HeaderSize_set, METH_VARARGS, NULL},
	 { "MessageHeader_HeaderSize_get", _wrap_MessageHeader_HeaderSize_get, METH_O, NULL},
	 { "MessageHeader_MessageFlags_set", _wrap_MessageHeader_MessageFlags_set, METH_VARARGS, NULL},
	 { "MessageHeader_MessageFlags_get", _wrap_MessageHeader_MessageFlags_get, METH_O, NULL},
	 { "MessageHeader_SenderEndpoint_set", _wrap_MessageHeader_SenderEndpoint_set, METH_VARARGS, NULL},
	 { "MessageHeader_SenderEndpoint_get", _wrap_MessageHeader_SenderEndpoint_get, METH_O, NULL},
	 { "MessageHeader_ReceiverEndpoint_set", _wrap_MessageHeader_ReceiverEndpoint_set, METH_VARARGS, NULL},
	 { "MessageHeader_ReceiverEndpoint_get", _wrap_MessageHeader_ReceiverEndpoint_get, METH_O, NULL},
	 { "MessageHeader_SenderNodeName_set", _wrap_MessageHeader_SenderNodeName_set, METH_VARARGS, NULL},
	 { "MessageHeader_SenderNodeName_get", _wrap_MessageHeader_SenderNodeName_get, METH_O, NULL},
	 { "MessageHeader_ReceiverNodeName_set", _wrap_MessageHeader_ReceiverNodeName_set, METH_VARARGS, NULL},
	 { "MessageHeader_ReceiverNodeName_get", _wrap_MessageHeader_ReceiverNodeName_get, METH_O, NULL},
	 { "MessageHeader_SenderNodeID_set", _wrap_MessageHeader_SenderNodeID_set, METH_VARARGS, NULL},
	 { "MessageHeader_SenderNodeID_get", _wrap_MessageHeader_SenderNodeID_get, METH_O, NULL},
	 { "MessageHeader_ReceiverNodeID_set", _wrap_MessageHeader_ReceiverNodeID_set, METH_VARARGS, NULL},
	 { "MessageHeader_ReceiverNodeID_get", _wrap_MessageHeader_ReceiverNodeID_get, METH_O, NULL},
	 { "MessageHeader_MetaData_set", _wrap_MessageHeader_MetaData_set, METH_VARARGS, NULL},
	 { "MessageHeader_MetaData_get", _wrap_MessageHeader_MetaData_get, METH_O, NULL},
	 { "MessageHeader_EntryCount_set", _wrap_MessageHeader_EntryCount_set, METH_VARARGS, NULL},
	 { "MessageHeader_EntryCount_get", _wrap_MessageHeader_EntryCount_get, METH_O, NULL},
	 { "MessageHeader_MessageID_set", _wrap_MessageHeader_MessageID_set, METH_VARARGS, NULL},
	 { "MessageHeader_MessageID_get", _wrap_MessageHeader_MessageID_get, METH_O, NULL},
	 { "MessageHeader_MessageResID_set", _wrap_MessageHeader_MessageResID_set, METH_VARARGS, NULL},
	 { "MessageHeader_MessageResID_get", _wrap_MessageHeader_MessageResID_get, METH_O, NULL},
	 { "MessageHeader_MessageSize_set", _wrap_MessageHeader_MessageSize_set, METH_VARARGS, NULL},
	 { "MessageHeader_MessageSize_get", _wrap_MessageHeader_MessageSize_get, METH_O, NULL},
	 { "MessageHeader_Priority_set", _wrap_MessageHeader_Priority_set, METH_VARARGS, NULL},
	 { "MessageHeader_Priority_get", _wrap_MessageHeader_Priority_get, METH_O, NULL},
	 { "MessageHeader_Extended_set", _wrap_MessageHeader_Extended_set, METH_VARARGS, NULL},
	 { "MessageHeader_Extended_get", _wrap_MessageHeader_Extended_get, METH_O, NULL},
	 { "MessageHeader_ComputeSize", _wrap_MessageHeader_ComputeSize, METH_O, NULL},
	 { "MessageHeader_UpdateHeader", _wrap_MessageHeader_UpdateHeader, METH_VARARGS, NULL},
	 { "new_MessageHeader", _wrap_new_MessageHeader, METH_NOARGS, NULL},
	 { "delete_MessageHeader", _wrap_delete_MessageHeader, METH_O, NULL},
	 { "MessageHeader_swigregister", MessageHeader_swigregister, METH_O, NULL},
	 { "MessageHeader_swiginit", MessageHeader_swiginit, METH_VARARGS, NULL},
	 { "MessageEntry_EntrySize_set", _wrap_MessageEntry_EntrySize_set, METH_VARARGS, NULL},
	 { "MessageEntry_EntrySize_get", _wrap_MessageEntry_EntrySize_get, METH_O, NULL},
	 { "MessageEntry_EntryFlags_set", _wrap_MessageEntry_EntryFlags_set, METH_VARARGS, NULL},
	 { "MessageEntry_EntryFlags_get", _wrap_MessageEntry_EntryFlags_get, METH_O, NULL},
	 { "MessageEntry_EntryType_set", _wrap_MessageEntry_EntryType_set, METH_VARARGS, NULL},
	 { "MessageEntry_EntryType_get", _wrap_MessageEntry_EntryType_get, METH_O, NULL},
	 { "MessageEntry_ServicePath_set", _wrap_MessageEntry_ServicePath_set, METH_VARARGS, NULL},
	 { "MessageEntry_ServicePath_get", _wrap_MessageEntry_ServicePath_get, METH_O, NULL},
	 { "MessageEntry_MemberName_set", _wrap_MessageEntry_MemberName_set, METH_VARARGS, NULL},
	 { "MessageEntry_MemberName_get", _wrap_MessageEntry_MemberName_get, METH_O, NULL},
	 { "MessageEntry_MemberNameCode_set", _wrap_MessageEntry_MemberNameCode_set, METH_VARARGS, NULL},
	 { "MessageEntry_MemberNameCode_get", _wrap_MessageEntry_MemberNameCode_get, METH_O, NULL},
	 { "MessageEntry_RequestID_set", _wrap_MessageEntry_RequestID_set, METH_VARARGS, NULL},
	 { "MessageEntry_RequestID_get", _wrap_MessageEntry_RequestID_get, METH_O, NULL},
	 { "MessageEntry_Error_set", _wrap_MessageEntry_Error_set, METH_VARARGS, NULL},
	 { "MessageEntry_Error_get", _wrap_MessageEntry_Error_get, METH_O, NULL},
	 { "MessageEntry_MetaData_set", _wrap_MessageEntry_MetaData_set, METH_VARARGS, NULL},
	 { "MessageEntry_MetaData_get", _wrap_MessageEntry_MetaData_get, METH_O, NULL},
	 { "MessageEntry_Extended_set", _wrap_MessageEntry_Extended_set, METH_VARARGS, NULL},
	 { "MessageEntry_Extended_get", _wrap_MessageEntry_Extended_get, METH_O, NULL},
	 { "MessageEntry_elements_set", _wrap_MessageEntry_elements_set, METH_VARARGS, NULL},
	 { "MessageEntry_elements_get", _wrap_MessageEntry_elements_get, METH_O, NULL},
	 { "new_MessageEntry", _wrap_new_MessageEntry, METH_VARARGS, NULL},
	 { "MessageEntry_ComputeSize", _wrap_MessageEntry_ComputeSize, METH_O, NULL},
	 { "MessageEntry_FindElement", _wrap_MessageEntry_FindElement, METH_VARARGS, NULL},
	 { "MessageEntry_AddElement", _wrap_MessageEntry_AddElement, METH_VARARGS, NULL},
	 { "delete_MessageEntry", _wrap_delete_MessageEntry, METH_O, NULL},
	 { "MessageEntry_swigregister", MessageEntry_swigregister, METH_O, NULL},
	 { "MessageEntry_swiginit", MessageEntry_swiginit, METH_VARARGS, NULL},
	 { "MessageElement_ElementSize_set", _wrap_MessageElement_ElementSize_set, METH_VARARGS, NULL},
	 { "MessageElement_ElementSize_get", _wrap_MessageElement_ElementSize_get, METH_O, NULL},
	 { "MessageElement_ElementFlags_set", _wrap_MessageElement_ElementFlags_set, METH_VARARGS, NULL},
	 { "MessageElement_ElementFlags_get", _wrap_MessageElement_ElementFlags_get, METH_O, NULL},
	 { "MessageElement_ElementName_set", _wrap_MessageElement_ElementName_set, METH_VARARGS, NULL},
	 { "MessageElement_ElementName_get", _wrap_MessageElement_ElementName_get, METH_O, NULL},
	 { "MessageElement_ElementNameCode_set", _wrap_MessageElement_ElementNameCode_set, METH_VARARGS, NULL},
	 { "MessageElement_ElementNameCode_get", _wrap_MessageElement_ElementNameCode_get, METH_O, NULL},
	 { "MessageElement_ElementNumber_set", _wrap_MessageElement_ElementNumber_set, METH_VARARGS, NULL},
	 { "MessageElement_ElementNumber_get", _wrap_MessageElement_ElementNumber_get, METH_O, NULL},
	 { "MessageElement_ElementType_set", _wrap_MessageElement_ElementType_set, METH_VARARGS, NULL},
	 { "MessageElement_ElementType_get", _wrap_MessageElement_ElementType_get, METH_O, NULL},
	 { "MessageElement_ElementTypeName_set", _wrap_MessageElement_ElementTypeName_set, METH_VARARGS, NULL},
	 { "MessageElement_ElementTypeName_get", _wrap_MessageElement_ElementTypeName_get, METH_O, NULL},
	 { "MessageElement_ElementTypeNameCode_set", _wrap_MessageElement_ElementTypeNameCode_set, METH_VARARGS, NULL},
	 { "MessageElement_ElementTypeNameCode_get", _wrap_MessageElement_ElementTypeNameCode_get, METH_O, NULL},
	 { "MessageElement_MetaData_set", _wrap_MessageElement_MetaData_set, METH_VARARGS, NULL},
	 { "MessageElement_MetaData_get", _wrap_MessageElement_MetaData_get, METH_O, NULL},
	 { "MessageElement_Extended_set", _wrap_MessageElement_Extended_set, METH_VARARGS, NULL},
	 { "MessageElement_Extended_get", _wrap_MessageElement_Extended_get, METH_O, NULL},
	 { "MessageElement_DataCount_set", _wrap_MessageElement_DataCount_set, METH_VARARGS, NULL},
	 { "MessageElement_DataCount_get", _wrap_MessageElement_DataCount_get, METH_O, NULL},
	 { "new_MessageElement", _wrap_new_MessageElement, METH_NOARGS, NULL},
	 { "MessageElement__GetData", _wrap_MessageElement__GetData, METH_O, NULL},
	 { "MessageElement__SetData", _wrap_MessageElement__SetData, METH_VARARGS, NULL},
	 { "MessageElement_ComputeSize", _wrap_MessageElement_ComputeSize, METH_O, NULL},
	 { "MessageElement_UpdateData", _wrap_MessageElement_UpdateData, METH_O, NULL},
	 { "MessageElement_FindElement", _wrap_MessageElement_FindElement, METH_VARARGS, NULL},
	 { "MessageElement_ContainsElement", _wrap_MessageElement_ContainsElement, METH_VARARGS, NULL},
	 { "delete_MessageElement", _wrap_delete_MessageElement, METH_O, NULL},
	 { "MessageElement_swigregister", MessageElement_swigregister, METH_O, NULL},
	 { "MessageElement_swiginit", MessageElement_swiginit, METH_VARARGS, NULL},
	 { "MessageElementNestedElementList_Type_set", _wrap_MessageElementNestedElementList_Type_set, METH_VARARGS, NULL},
	 { "MessageElementNestedElementList_Type_get", _wrap_MessageElementNestedElementList_Type_get, METH_O, NULL},
	 { "MessageElementNestedElementList_TypeName_set", _wrap_MessageElementNestedElementList_TypeName_set, METH_VARARGS, NULL},
	 { "MessageElementNestedElementList_TypeName_get", _wrap_MessageElementNestedElementList_TypeName_get, METH_O, NULL},
	 { "MessageElementNestedElementList_Elements_set", _wrap_MessageElementNestedElementList_Elements_set, METH_VARARGS, NULL},
	 { "MessageElementNestedElementList_Elements_get", _wrap_MessageElementNestedElementList_Elements_get, METH_O, NULL},
	 { "new_MessageElementNestedElementList", _wrap_new_MessageElementNestedElementList, METH_VARARGS, NULL},
	 { "MessageElementNestedElementList_GetTypeString", _wrap_MessageElementNestedElementList_GetTypeString, METH_O, NULL},
	 { "MessageElementNestedElementList_GetTypeID", _wrap_MessageElementNestedElementList_GetTypeID, METH_O, NULL},
	 { "delete_MessageElementNestedElementList", _wrap_delete_MessageElementNestedElementList, METH_O, NULL},
	 { "MessageElementNestedElementList_swigregister", MessageElementNestedElementList_swigregister, METH_O, NULL},
	 { "MessageElementNestedElementList_swiginit", MessageElementNestedElementList_swiginit, METH_VARARGS, NULL},
	 { "MessageElementDataUtil_ToRRBaseArray", _wrap_MessageElementDataUtil_ToRRBaseArray, METH_O, NULL},
	 { "MessageElementDataUtil_ToMessageElementNestedElementList", _wrap_MessageElementDataUtil_ToMessageElementNestedElementList, METH_O, NULL},
	 { "new_MessageElementDataUtil", _wrap_new_MessageElementDataUtil, METH_NOARGS, NULL},
	 { "delete_MessageElementDataUtil", _wrap_delete_MessageElementDataUtil, METH_O, NULL},
	 { "MessageElementDataUtil_swigregister", MessageElementDataUtil_swigregister, METH_O, NULL},
	 { "MessageElementDataUtil_swiginit", MessageElementDataUtil_swiginit, METH_VARARGS, NULL},
	 { "MessageFromBytes", _wrap_MessageFromBytes, METH_O, NULL},
	 { "MessageLengthFromBytes", _wrap_MessageLengthFromBytes, METH_O, NULL},
	 { "MessageToBytes", _wrap_MessageToBytes, METH_VARARGS, NULL},
	 { "MessageElementFromBytes", _wrap_MessageElementFromBytes, METH_O, NULL},
	 { "MessageElementLengthFromBytes", _wrap_MessageElementLengthFromBytes, METH_O, NULL},
	 { "MessageElementToBytes", _wrap_MessageElementToBytes, METH_VARARGS, NULL},
	 { "HandlerErrorInfo_error_code_set", _wrap_HandlerErrorInfo_error_code_set, METH_VARARGS, NULL},
	 { "HandlerErrorInfo_error_code_get", _wrap_HandlerErrorInfo_error_code_get, METH_O, NULL},
	 { "HandlerErrorInfo_errorname_set", _wrap_HandlerErrorInfo_errorname_set, METH_VARARGS, NULL},
	 { "HandlerErrorInfo_errorname_get", _wrap_HandlerErrorInfo_errorname_get, METH_O, NULL},
	 { "HandlerErrorInfo_errormessage_set", _wrap_HandlerErrorInfo_errormessage_set, METH_VARARGS, NULL},
	 { "HandlerErrorInfo_errormessage_get", _wrap_HandlerErrorInfo_errormessage_get, METH_O, NULL},
	 { "HandlerErrorInfo_errorsubname_set", _wrap_HandlerErrorInfo_errorsubname_set, METH_VARARGS, NULL},
	 { "HandlerErrorInfo_errorsubname_get", _wrap_HandlerErrorInfo_errorsubname_get, METH_O, NULL},
	 { "HandlerErrorInfo_param__set", _wrap_HandlerErrorInfo_param__set, METH_VARARGS, NULL},
	 { "HandlerErrorInfo_param__get", _wrap_HandlerErrorInfo_param__get, METH_O, NULL},
	 { "new_HandlerErrorInfo", _wrap_new_HandlerErrorInfo, METH_NOARGS, NULL},
	 { "delete_HandlerErrorInfo", _wrap_delete_HandlerErrorInfo, METH_O, NULL},
	 { "HandlerErrorInfo_swigregister", HandlerErrorInfo_swigregister, METH_O, NULL},
	 { "HandlerErrorInfo_swiginit", HandlerErrorInfo_swiginit, METH_VARARGS, NULL},
	 { "delete_AsyncRequestDirector", _wrap_delete_AsyncRequestDirector, METH_O, NULL},
	 { "AsyncRequestDirector_handler", _wrap_AsyncRequestDirector_handler, METH_VARARGS, NULL},
	 { "new_AsyncRequestDirector", _wrap_new_AsyncRequestDirector, METH_O, NULL},
	 { "disown_AsyncRequestDirector", _wrap_disown_AsyncRequestDirector, METH_O, NULL},
	 { "AsyncRequestDirector_swigregister", AsyncRequestDirector_swigregister, METH_O, NULL},
	 { "AsyncRequestDirector_swiginit", AsyncRequestDirector_swiginit, METH_VARARGS, NULL},
	 { "delete_AsyncVoidReturnDirector", _wrap_delete_AsyncVoidReturnDirector, METH_O, NULL},
	 { "AsyncVoidReturnDirector_handler", _wrap_AsyncVoidReturnDirector_handler, METH_VARARGS, NULL},
	 { "new_AsyncVoidReturnDirector", _wrap_new_AsyncVoidReturnDirector, METH_O, NULL},
	 { "disown_AsyncVoidReturnDirector", _wrap_disown_AsyncVoidReturnDirector, METH_O, NULL},
	 { "AsyncVoidReturnDirector_swigregister", AsyncVoidReturnDirector_swigregister, METH_O, NULL},
	 { "AsyncVoidReturnDirector_swiginit", AsyncVoidReturnDirector_swiginit, METH_VARARGS, NULL},
	 { "delete_AsyncVoidNoErrReturnDirector", _wrap_delete_AsyncVoidNoErrReturnDirector, METH_O, NULL},
	 { "AsyncVoidNoErrReturnDirector_handler", _wrap_AsyncVoidNoErrReturnDirector_handler, METH_O, NULL},
	 { "new_AsyncVoidNoErrReturnDirector", _wrap_new_AsyncVoidNoErrReturnDirector, METH_O, NULL},
	 { "disown_AsyncVoidNoErrReturnDirector", _wrap_disown_AsyncVoidNoErrReturnDirector, METH_O, NULL},
	 { "AsyncVoidNoErrReturnDirector_swigregister", AsyncVoidNoErrReturnDirector_swigregister, METH_O, NULL},
	 { "AsyncVoidNoErrReturnDirector_swiginit", AsyncVoidNoErrReturnDirector_swiginit, METH_VARARGS, NULL},
	 { "delete_AsyncStringReturnDirector", _wrap_delete_AsyncStringReturnDirector, METH_O, NULL},
	 { "AsyncStringReturnDirector_handler", _wrap_AsyncStringReturnDirector_handler, METH_VARARGS, NULL},
	 { "new_AsyncStringReturnDirector", _wrap_new_AsyncStringReturnDirector, METH_O, NULL},
	 { "disown_AsyncStringReturnDirector", _wrap_disown_AsyncStringReturnDirector, METH_O, NULL},
	 { "AsyncStringReturnDirector_swigregister", AsyncStringReturnDirector_swigregister, METH_O, NULL},
	 { "AsyncStringReturnDirector_swiginit", AsyncStringReturnDirector_swiginit, METH_VARARGS, NULL},
	 { "delete_AsyncUInt32ReturnDirector", _wrap_delete_AsyncUInt32ReturnDirector, METH_O, NULL},
	 { "AsyncUInt32ReturnDirector_handler", _wrap_AsyncUInt32ReturnDirector_handler, METH_VARARGS, NULL},
	 { "new_AsyncUInt32ReturnDirector", _wrap_new_AsyncUInt32ReturnDirector, METH_O, NULL},
	 { "disown_AsyncUInt32ReturnDirector", _wrap_disown_AsyncUInt32ReturnDirector, METH_O, NULL},
	 { "AsyncUInt32ReturnDirector_swigregister", AsyncUInt32ReturnDirector_swigregister, METH_O, NULL},
	 { "AsyncUInt32ReturnDirector_swiginit", AsyncUInt32ReturnDirector_swiginit, METH_VARARGS, NULL},
	 { "vectorptr_typedefinition_iterator", _wrap_vectorptr_typedefinition_iterator, METH_O, NULL},
	 { "vectorptr_typedefinition___nonzero__", _wrap_vectorptr_typedefinition___nonzero__, METH_O, NULL},
	 { "vectorptr_typedefinition___bool__", _wrap_vectorptr_typedefinition___bool__, METH_O, NULL},
	 { "vectorptr_typedefinition___len__", _wrap_vectorptr_typedefinition___len__, METH_O, NULL},
	 { "vectorptr_typedefinition___getslice__", _wrap_vectorptr_typedefinition___getslice__, METH_VARARGS, NULL},
	 { "vectorptr_typedefinition___setslice__", _wrap_vectorptr_typedefinition___setslice__, METH_VARARGS, NULL},
	 { "vectorptr_typedefinition___delslice__", _wrap_vectorptr_typedefinition___delslice__, METH_VARARGS, NULL},
	 { "vectorptr_typedefinition___delitem__", _wrap_vectorptr_typedefinition___delitem__, METH_VARARGS, NULL},
	 { "vectorptr_typedefinition___getitem__", _wrap_vectorptr_typedefinition___getitem__, METH_VARARGS, NULL},
	 { "vectorptr_typedefinition___setitem__", _wrap_vectorptr_typedefinition___setitem__, METH_VARARGS, NULL},
	 { "vectorptr_typedefinition_pop", _wrap_vectorptr_typedefinition_pop, METH_O, NULL},
	 { "vectorptr_typedefinition_append", _wrap_vectorptr_typedefinition_append, METH_VARARGS, NULL},
	 { "vectorptr_typedefinition_empty", _wrap_vectorptr_typedefinition_empty, METH_O, NULL},
	 { "vectorptr_typedefinition_size", _wrap_vectorptr_typedefinition_size, METH_O, NULL},
	 { "vectorptr_typedefinition_swap", _wrap_vectorptr_typedefinition_swap, METH_VARARGS, NULL},
	 { "vectorptr_typedefinition_begin", _wrap_vectorptr_typedefinition_begin, METH_O, NULL},
	 { "vectorptr_typedefinition_end", _wrap_vectorptr_typedefinition_end, METH_O, NULL},
	 { "vectorptr_typedefinition_rbegin", _wrap_vectorptr_typedefinition_rbegin, METH_O, NULL},
	 { "vectorptr_typedefinition_rend", _wrap_vectorptr_typedefinition_rend, METH_O, NULL},
	 { "vectorptr_typedefinition_clear", _wrap_vectorptr_typedefinition_clear, METH_O, NULL},
	 { "vectorptr_typedefinition_get_allocator", _wrap_vectorptr_typedefinition_get_allocator, METH_O, NULL},
	 { "vectorptr_typedefinition_pop_back", _wrap_vectorptr_typedefinition_pop_back, METH_O, NULL},
	 { "vectorptr_typedefinition_erase", _wrap_vectorptr_typedefinition_erase, METH_VARARGS, NULL},
	 { "new_vectorptr_typedefinition", _wrap_new_vectorptr_typedefinition, METH_VARARGS, NULL},
	 { "vectorptr_typedefinition_push_back", _wrap_vectorptr_typedefinition_push_back, METH_VARARGS, NULL},
	 { "vectorptr_typedefinition_front", _wrap_vectorptr_typedefinition_front, METH_O, NULL},
	 { "vectorptr_typedefinition_back", _wrap_vectorptr_typedefinition_back, METH_O, NULL},
	 { "vectorptr_typedefinition_assign", _wrap_vectorptr_typedefinition_assign, METH_VARARGS, NULL},
	 { "vectorptr_typedefinition_resize", _wrap_vectorptr_typedefinition_resize, METH_VARARGS, NULL},
	 { "vectorptr_typedefinition_insert", _wrap_vectorptr_typedefinition_insert, METH_VARARGS, NULL},
	 { "vectorptr_typedefinition_reserve", _wrap_vectorptr_typedefinition_reserve, METH_VARARGS, NULL},
	 { "vectorptr_typedefinition_capacity", _wrap_vectorptr_typedefinition_capacity, METH_O, NULL},
	 { "delete_vectorptr_typedefinition", _wrap_delete_vectorptr_typedefinition, METH_O, NULL},
	 { "vectorptr_typedefinition_swigregister", vectorptr_typedefinition_swigregister, METH_O, NULL},
	 { "vectorptr_typedefinition_swiginit", vectorptr_typedefinition_swiginit, METH_VARARGS, NULL},
	 { "vectorptr_memberdefinition_iterator", _wrap_vectorptr_memberdefinition_iterator, METH_O, NULL},
	 { "vectorptr_memberdefinition___nonzero__", _wrap_vectorptr_memberdefinition___nonzero__, METH_O, NULL},
	 { "vectorptr_memberdefinition___bool__", _wrap_vectorptr_memberdefinition___bool__, METH_O, NULL},
	 { "vectorptr_memberdefinition___len__", _wrap_vectorptr_memberdefinition___len__, METH_O, NULL},
	 { "vectorptr_memberdefinition___getslice__", _wrap_vectorptr_memberdefinition___getslice__, METH_VARARGS, NULL},
	 { "vectorptr_memberdefinition___setslice__", _wrap_vectorptr_memberdefinition___setslice__, METH_VARARGS, NULL},
	 { "vectorptr_memberdefinition___delslice__", _wrap_vectorptr_memberdefinition___delslice__, METH_VARARGS, NULL},
	 { "vectorptr_memberdefinition___delitem__", _wrap_vectorptr_memberdefinition___delitem__, METH_VARARGS, NULL},
	 { "vectorptr_memberdefinition___getitem__", _wrap_vectorptr_memberdefinition___getitem__, METH_VARARGS, NULL},
	 { "vectorptr_memberdefinition___setitem__", _wrap_vectorptr_memberdefinition___setitem__, METH_VARARGS, NULL},
	 { "vectorptr_memberdefinition_pop", _wrap_vectorptr_memberdefinition_pop, METH_O, NULL},
	 { "vectorptr_memberdefinition_append", _wrap_vectorptr_memberdefinition_append, METH_VARARGS, NULL},
	 { "vectorptr_memberdefinition_empty", _wrap_vectorptr_memberdefinition_empty, METH_O, NULL},
	 { "vectorptr_memberdefinition_size", _wrap_vectorptr_memberdefinition_size, METH_O, NULL},
	 { "vectorptr_memberdefinition_swap", _wrap_vectorptr_memberdefinition_swap, METH_VARARGS, NULL},
	 { "vectorptr_memberdefinition_begin", _wrap_vectorptr_memberdefinition_begin, METH_O, NULL},
	 { "vectorptr_memberdefinition_end", _wrap_vectorptr_memberdefinition_end, METH_O, NULL},
	 { "vectorptr_memberdefinition_rbegin", _wrap_vectorptr_memberdefinition_rbegin, METH_O, NULL},
	 { "vectorptr_memberdefinition_rend", _wrap_vectorptr_memberdefinition_rend, METH_O, NULL},
	 { "vectorptr_memberdefinition_clear", _wrap_vectorptr_memberdefinition_clear, METH_O, NULL},
	 { "vectorptr_memberdefinition_get_allocator", _wrap_vectorptr_memberdefinition_get_allocator, METH_O, NULL},
	 { "vectorptr_memberdefinition_pop_back", _wrap_vectorptr_memberdefinition_pop_back, METH_O, NULL},
	 { "vectorptr_memberdefinition_erase", _wrap_vectorptr_memberdefinition_erase, METH_VARARGS, NULL},
	 { "new_vectorptr_memberdefinition", _wrap_new_vectorptr_memberdefinition, METH_VARARGS, NULL},
	 { "vectorptr_memberdefinition_push_back", _wrap_vectorptr_memberdefinition_push_back, METH_VARARGS, NULL},
	 { "vectorptr_memberdefinition_front", _wrap_vectorptr_memberdefinition_front, METH_O, NULL},
	 { "vectorptr_memberdefinition_back", _wrap_vectorptr_memberdefinition_back, METH_O, NULL},
	 { "vectorptr_memberdefinition_assign", _wrap_vectorptr_memberdefinition_assign, METH_VARARGS, NULL},
	 { "vectorptr_memberdefinition_resize", _wrap_vectorptr_memberdefinition_resize, METH_VARARGS, NULL},
	 { "vectorptr_memberdefinition_insert", _wrap_vectorptr_memberdefinition_insert, METH_VARARGS, NULL},
	 { "vectorptr_memberdefinition_reserve", _wrap_vectorptr_memberdefinition_reserve, METH_VARARGS, NULL},
	 { "vectorptr_memberdefinition_capacity", _wrap_vectorptr_memberdefinition_capacity, METH_O, NULL},
	 { "delete_vectorptr_memberdefinition", _wrap_delete_vectorptr_memberdefinition, METH_O, NULL},
	 { "vectorptr_memberdefinition_swigregister", vectorptr_memberdefinition_swigregister, METH_O, NULL},
	 { "vectorptr_memberdefinition_swiginit", vectorptr_memberdefinition_swiginit, METH_VARARGS, NULL},
	 { "vectorptr_serviceentrydefinition_iterator", _wrap_vectorptr_serviceentrydefinition_iterator, METH_O, NULL},
	 { "vectorptr_serviceentrydefinition___nonzero__", _wrap_vectorptr_serviceentrydefinition___nonzero__, METH_O, NULL},
	 { "vectorptr_serviceentrydefinition___bool__", _wrap_vectorptr_serviceentrydefinition___bool__, METH_O, NULL},
	 { "vectorptr_serviceentrydefinition___len__", _wrap_vectorptr_serviceentrydefinition___len__, METH_O, NULL},
	 { "vectorptr_serviceentrydefinition___getslice__", _wrap_vectorptr_serviceentrydefinition___getslice__, METH_VARARGS, NULL},
	 { "vectorptr_serviceentrydefinition___setslice__", _wrap_vectorptr_serviceentrydefinition___setslice__, METH_VARARGS, NULL},
	 { "vectorptr_serviceentrydefinition___delslice__", _wrap_vectorptr_serviceentrydefinition___delslice__, METH_VARARGS, NULL},
	 { "vectorptr_serviceentrydefinition___delitem__", _wrap_vectorptr_serviceentrydefinition___delitem__, METH_VARARGS, NULL},
	 { "vectorptr_serviceentrydefinition___getitem__", _wrap_vectorptr_serviceentrydefinition___getitem__, METH_VARARGS, NULL},
	 { "vectorptr_serviceentrydefinition___setitem__", _wrap_vectorptr_serviceentrydefinition___setitem__, METH_VARARGS, NULL},
	 { "vectorptr_serviceentrydefinition_pop", _wrap_vectorptr_serviceentrydefinition_pop, METH_O, NULL},
	 { "vectorptr_serviceentrydefinition_append", _wrap_vectorptr_serviceentrydefinition_append, METH_VARARGS, NULL},
	 { "vectorptr_serviceentrydefinition_empty", _wrap_vectorptr_serviceentrydefinition_empty, METH_O, NULL},
	 { "vectorptr_serviceentrydefinition_size", _wrap_vectorptr_serviceentrydefinition_size, METH_O, NULL},
	 { "vectorptr_serviceentrydefinition_swap", _wrap_vectorptr_serviceentrydefinition_swap, METH_VARARGS, NULL},
	 { "vectorptr_serviceentrydefinition_begin", _wrap_vectorptr_serviceentrydefinition_begin, METH_O, NULL},
	 { "vectorptr_serviceentrydefinition_end", _wrap_vectorptr_serviceentrydefinition_end, METH_O, NULL},
	 { "vectorptr_serviceentrydefinition_rbegin", _wrap_vectorptr_serviceentrydefinition_rbegin, METH_O, NULL},
	 { "vectorptr_serviceentrydefinition_rend", _wrap_vectorptr_serviceentrydefinition_rend, METH_O, NULL},
	 { "vectorptr_serviceentrydefinition_clear", _wrap_vectorptr_serviceentrydefinition_clear, METH_O, NULL},
	 { "vectorptr_serviceentrydefinition_get_allocator", _wrap_vectorptr_serviceentrydefinition_get_allocator, METH_O, NULL},
	 { "vectorptr_serviceentrydefinition_pop_back", _wrap_vectorptr_serviceentrydefinition_pop_back, METH_O, NULL},
	 { "vectorptr_serviceentrydefinition_erase", _wrap_vectorptr_serviceentrydefinition_erase, METH_VARARGS, NULL},
	 { "new_vectorptr_serviceentrydefinition", _wrap_new_vectorptr_serviceentrydefinition, METH_VARARGS, NULL},
	 { "vectorptr_serviceentrydefinition_push_back", _wrap_vectorptr_serviceentrydefinition_push_back, METH_VARARGS, NULL},
	 { "vectorptr_serviceentrydefinition_front", _wrap_vectorptr_serviceentrydefinition_front, METH_O, NULL},
	 { "vectorptr_serviceentrydefinition_back", _wrap_vectorptr_serviceentrydefinition_back, METH_O, NULL},
	 { "vectorptr_serviceentrydefinition_assign", _wrap_vectorptr_serviceentrydefinition_assign, METH_VARARGS, NULL},
	 { "vectorptr_serviceentrydefinition_resize", _wrap_vectorptr_serviceentrydefinition_resize, METH_VARARGS, NULL},
	 { "vectorptr_serviceentrydefinition_insert", _wrap_vectorptr_serviceentrydefinition_insert, METH_VARARGS, NULL},
	 { "vectorptr_serviceentrydefinition_reserve", _wrap_vectorptr_serviceentrydefinition_reserve, METH_VARARGS, NULL},
	 { "vectorptr_serviceentrydefinition_capacity", _wrap_vectorptr_serviceentrydefinition_capacity, METH_O, NULL},
	 { "delete_vectorptr_serviceentrydefinition", _wrap_delete_vectorptr_serviceentrydefinition, METH_O, NULL},
	 { "vectorptr_serviceentrydefinition_swigregister", vectorptr_serviceentrydefinition_swigregister, METH_O, NULL},
	 { "vectorptr_serviceentrydefinition_swiginit", vectorptr_serviceentrydefinition_swiginit, METH_VARARGS, NULL},
	 { "vectorptr_constantdefinition_iterator", _wrap_vectorptr_constantdefinition_iterator, METH_O, NULL},
	 { "vectorptr_constantdefinition___nonzero__", _wrap_vectorptr_constantdefinition___nonzero__, METH_O, NULL},
	 { "vectorptr_constantdefinition___bool__", _wrap_vectorptr_constantdefinition___bool__, METH_O, NULL},
	 { "vectorptr_constantdefinition___len__", _wrap_vectorptr_constantdefinition___len__, METH_O, NULL},
	 { "vectorptr_constantdefinition___getslice__", _wrap_vectorptr_constantdefinition___getslice__, METH_VARARGS, NULL},
	 { "vectorptr_constantdefinition___setslice__", _wrap_vectorptr_constantdefinition___setslice__, METH_VARARGS, NULL},
	 { "vectorptr_constantdefinition___delslice__", _wrap_vectorptr_constantdefinition___delslice__, METH_VARARGS, NULL},
	 { "vectorptr_constantdefinition___delitem__", _wrap_vectorptr_constantdefinition___delitem__, METH_VARARGS, NULL},
	 { "vectorptr_constantdefinition___getitem__", _wrap_vectorptr_constantdefinition___getitem__, METH_VARARGS, NULL},
	 { "vectorptr_constantdefinition___setitem__", _wrap_vectorptr_constantdefinition___setitem__, METH_VARARGS, NULL},
	 { "vectorptr_constantdefinition_pop", _wrap_vectorptr_constantdefinition_pop, METH_O, NULL},
	 { "vectorptr_constantdefinition_append", _wrap_vectorptr_constantdefinition_append, METH_VARARGS, NULL},
	 { "vectorptr_constantdefinition_empty", _wrap_vectorptr_constantdefinition_empty, METH_O, NULL},
	 { "vectorptr_constantdefinition_size", _wrap_vectorptr_constantdefinition_size, METH_O, NULL},
	 { "vectorptr_constantdefinition_swap", _wrap_vectorptr_constantdefinition_swap, METH_VARARGS, NULL},
	 { "vectorptr_constantdefinition_begin", _wrap_vectorptr_constantdefinition_begin, METH_O, NULL},
	 { "vectorptr_constantdefinition_end", _wrap_vectorptr_constantdefinition_end, METH_O, NULL},
	 { "vectorptr_constantdefinition_rbegin", _wrap_vectorptr_constantdefinition_rbegin, METH_O, NULL},
	 { "vectorptr_constantdefinition_rend", _wrap_vectorptr_constantdefinition_rend, METH_O, NULL},
	 { "vectorptr_constantdefinition_clear", _wrap_vectorptr_constantdefinition_clear, METH_O, NULL},
	 { "vectorptr_constantdefinition_get_allocator", _wrap_vectorptr_constantdefinition_get_allocator, METH_O, NULL},
	 { "vectorptr_constantdefinition_pop_back", _wrap_vectorptr_constantdefinition_pop_back, METH_O, NULL},
	 { "vectorptr_constantdefinition_erase", _wrap_vectorptr_constantdefinition_erase, METH_VARARGS, NULL},
	 { "new_vectorptr_constantdefinition", _wrap_new_vectorptr_constantdefinition, METH_VARARGS, NULL},
	 { "vectorptr_constantdefinition_push_back", _wrap_vectorptr_constantdefinition_push_back, METH_VARARGS, NULL},
	 { "vectorptr_constantdefinition_front", _wrap_vectorptr_constantdefinition_front, METH_O, NULL},
	 { "vectorptr_constantdefinition_back", _wrap_vectorptr_constantdefinition_back, METH_O, NULL},
	 { "vectorptr_constantdefinition_assign", _wrap_vectorptr_constantdefinition_assign, METH_VARARGS, NULL},
	 { "vectorptr_constantdefinition_resize", _wrap_vectorptr_constantdefinition_resize, METH_VARARGS, NULL},
	 { "vectorptr_constantdefinition_insert", _wrap_vectorptr_constantdefinition_insert, METH_VARARGS, NULL},
	 { "vectorptr_constantdefinition_reserve", _wrap_vectorptr_constantdefinition_reserve, METH_VARARGS, NULL},
	 { "vectorptr_constantdefinition_capacity", _wrap_vectorptr_constantdefinition_capacity, METH_O, NULL},
	 { "delete_vectorptr_constantdefinition", _wrap_delete_vectorptr_constantdefinition, METH_O, NULL},
	 { "vectorptr_constantdefinition_swigregister", vectorptr_constantdefinition_swigregister, METH_O, NULL},
	 { "vectorptr_constantdefinition_swiginit", vectorptr_constantdefinition_swiginit, METH_VARARGS, NULL},
	 { "vectorptr_enumdefinition_iterator", _wrap_vectorptr_enumdefinition_iterator, METH_O, NULL},
	 { "vectorptr_enumdefinition___nonzero__", _wrap_vectorptr_enumdefinition___nonzero__, METH_O, NULL},
	 { "vectorptr_enumdefinition___bool__", _wrap_vectorptr_enumdefinition___bool__, METH_O, NULL},
	 { "vectorptr_enumdefinition___len__", _wrap_vectorptr_enumdefinition___len__, METH_O, NULL},
	 { "vectorptr_enumdefinition___getslice__", _wrap_vectorptr_enumdefinition___getslice__, METH_VARARGS, NULL},
	 { "vectorptr_enumdefinition___setslice__", _wrap_vectorptr_enumdefinition___setslice__, METH_VARARGS, NULL},
	 { "vectorptr_enumdefinition___delslice__", _wrap_vectorptr_enumdefinition___delslice__, METH_VARARGS, NULL},
	 { "vectorptr_enumdefinition___delitem__", _wrap_vectorptr_enumdefinition___delitem__, METH_VARARGS, NULL},
	 { "vectorptr_enumdefinition___getitem__", _wrap_vectorptr_enumdefinition___getitem__, METH_VARARGS, NULL},
	 { "vectorptr_enumdefinition___setitem__", _wrap_vectorptr_enumdefinition___setitem__, METH_VARARGS, NULL},
	 { "vectorptr_enumdefinition_pop", _wrap_vectorptr_enumdefinition_pop, METH_O, NULL},
	 { "vectorptr_enumdefinition_append", _wrap_vectorptr_enumdefinition_append, METH_VARARGS, NULL},
	 { "vectorptr_enumdefinition_empty", _wrap_vectorptr_enumdefinition_empty, METH_O, NULL},
	 { "vectorptr_enumdefinition_size", _wrap_vectorptr_enumdefinition_size, METH_O, NULL},
	 { "vectorptr_enumdefinition_swap", _wrap_vectorptr_enumdefinition_swap, METH_VARARGS, NULL},
	 { "vectorptr_enumdefinition_begin", _wrap_vectorptr_enumdefinition_begin, METH_O, NULL},
	 { "vectorptr_enumdefinition_end", _wrap_vectorptr_enumdefinition_end, METH_O, NULL},
	 { "vectorptr_enumdefinition_rbegin", _wrap_vectorptr_enumdefinition_rbegin, METH_O, NULL},
	 { "vectorptr_enumdefinition_rend", _wrap_vectorptr_enumdefinition_rend, METH_O, NULL},
	 { "vectorptr_enumdefinition_clear", _wrap_vectorptr_enumdefinition_clear, METH_O, NULL},
	 { "vectorptr_enumdefinition_get_allocator", _wrap_vectorptr_enumdefinition_get_allocator, METH_O, NULL},
	 { "vectorptr_enumdefinition_pop_back", _wrap_vectorptr_enumdefinition_pop_back, METH_O, NULL},
	 { "vectorptr_enumdefinition_erase", _wrap_vectorptr_enumdefinition_erase, METH_VARARGS, NULL},
	 { "new_vectorptr_enumdefinition", _wrap_new_vectorptr_enumdefinition, METH_VARARGS, NULL},
	 { "vectorptr_enumdefinition_push_back", _wrap_vectorptr_enumdefinition_push_back, METH_VARARGS, NULL},
	 { "vectorptr_enumdefinition_front", _wrap_vectorptr_enumdefinition_front, METH_O, NULL},
	 { "vectorptr_enumdefinition_back", _wrap_vectorptr_enumdefinition_back, METH_O, NULL},
	 { "vectorptr_enumdefinition_assign", _wrap_vectorptr_enumdefinition_assign, METH_VARARGS, NULL},
	 { "vectorptr_enumdefinition_resize", _wrap_vectorptr_enumdefinition_resize, METH_VARARGS, NULL},
	 { "vectorptr_enumdefinition_insert", _wrap_vectorptr_enumdefinition_insert, METH_VARARGS, NULL},
	 { "vectorptr_enumdefinition_reserve", _wrap_vectorptr_enumdefinition_reserve, METH_VARARGS, NULL},
	 { "vectorptr_enumdefinition_capacity", _wrap_vectorptr_enumdefinition_capacity, METH_O, NULL},
	 { "delete_vectorptr_enumdefinition", _wrap_delete_vectorptr_enumdefinition, METH_O, NULL},
	 { "vectorptr_enumdefinition_swigregister", vectorptr_enumdefinition_swigregister, METH_O, NULL},
	 { "vectorptr_enumdefinition_swiginit", vectorptr_enumdefinition_swiginit, METH_VARARGS, NULL},
	 { "vector_constantdefinition_structfield_iterator", _wrap_vector_constantdefinition_structfield_iterator, METH_O, NULL},
	 { "vector_constantdefinition_structfield___nonzero__", _wrap_vector_constantdefinition_structfield___nonzero__, METH_O, NULL},
	 { "vector_constantdefinition_structfield___bool__", _wrap_vector_constantdefinition_structfield___bool__, METH_O, NULL},
	 { "vector_constantdefinition_structfield___len__", _wrap_vector_constantdefinition_structfield___len__, METH_O, NULL},
	 { "vector_constantdefinition_structfield___getslice__", _wrap_vector_constantdefinition_structfield___getslice__, METH_VARARGS, NULL},
	 { "vector_constantdefinition_structfield___setslice__", _wrap_vector_constantdefinition_structfield___setslice__, METH_VARARGS, NULL},
	 { "vector_constantdefinition_structfield___delslice__", _wrap_vector_constantdefinition_structfield___delslice__, METH_VARARGS, NULL},
	 { "vector_constantdefinition_structfield___delitem__", _wrap_vector_constantdefinition_structfield___delitem__, METH_VARARGS, NULL},
	 { "vector_constantdefinition_structfield___getitem__", _wrap_vector_constantdefinition_structfield___getitem__, METH_VARARGS, NULL},
	 { "vector_constantdefinition_structfield___setitem__", _wrap_vector_constantdefinition_structfield___setitem__, METH_VARARGS, NULL},
	 { "vector_constantdefinition_structfield_pop", _wrap_vector_constantdefinition_structfield_pop, METH_O, NULL},
	 { "vector_constantdefinition_structfield_append", _wrap_vector_constantdefinition_structfield_append, METH_VARARGS, NULL},
	 { "vector_constantdefinition_structfield_empty", _wrap_vector_constantdefinition_structfield_empty, METH_O, NULL},
	 { "vector_constantdefinition_structfield_size", _wrap_vector_constantdefinition_structfield_size, METH_O, NULL},
	 { "vector_constantdefinition_structfield_swap", _wrap_vector_constantdefinition_structfield_swap, METH_VARARGS, NULL},
	 { "vector_constantdefinition_structfield_begin", _wrap_vector_constantdefinition_structfield_begin, METH_O, NULL},
	 { "vector_constantdefinition_structfield_end", _wrap_vector_constantdefinition_structfield_end, METH_O, NULL},
	 { "vector_constantdefinition_structfield_rbegin", _wrap_vector_constantdefinition_structfield_rbegin, METH_O, NULL},
	 { "vector_constantdefinition_structfield_rend", _wrap_vector_constantdefinition_structfield_rend, METH_O, NULL},
	 { "vector_constantdefinition_structfield_clear", _wrap_vector_constantdefinition_structfield_clear, METH_O, NULL},
	 { "vector_constantdefinition_structfield_get_allocator", _wrap_vector_constantdefinition_structfield_get_allocator, METH_O, NULL},
	 { "vector_constantdefinition_structfield_pop_back", _wrap_vector_constantdefinition_structfield_pop_back, METH_O, NULL},
	 { "vector_constantdefinition_structfield_erase", _wrap_vector_constantdefinition_structfield_erase, METH_VARARGS, NULL},
	 { "new_vector_constantdefinition_structfield", _wrap_new_vector_constantdefinition_structfield, METH_VARARGS, NULL},
	 { "vector_constantdefinition_structfield_push_back", _wrap_vector_constantdefinition_structfield_push_back, METH_VARARGS, NULL},
	 { "vector_constantdefinition_structfield_front", _wrap_vector_constantdefinition_structfield_front, METH_O, NULL},
	 { "vector_constantdefinition_structfield_back", _wrap_vector_constantdefinition_structfield_back, METH_O, NULL},
	 { "vector_constantdefinition_structfield_assign", _wrap_vector_constantdefinition_structfield_assign, METH_VARARGS, NULL},
	 { "vector_constantdefinition_structfield_resize", _wrap_vector_constantdefinition_structfield_resize, METH_VARARGS, NULL},
	 { "vector_constantdefinition_structfield_insert", _wrap_vector_constantdefinition_structfield_insert, METH_VARARGS, NULL},
	 { "vector_constantdefinition_structfield_reserve", _wrap_vector_constantdefinition_structfield_reserve, METH_VARARGS, NULL},
	 { "vector_constantdefinition_structfield_capacity", _wrap_vector_constantdefinition_structfield_capacity, METH_O, NULL},
	 { "delete_vector_constantdefinition_structfield", _wrap_delete_vector_constantdefinition_structfield, METH_O, NULL},
	 { "vector_constantdefinition_structfield_swigregister", vector_constantdefinition_structfield_swigregister, METH_O, NULL},
	 { "vector_constantdefinition_structfield_swiginit", vector_constantdefinition_structfield_swiginit, METH_VARARGS, NULL},
	 { "vector_enumdefinitionvalues_iterator", _wrap_vector_enumdefinitionvalues_iterator, METH_O, NULL},
	 { "vector_enumdefinitionvalues___nonzero__", _wrap_vector_enumdefinitionvalues___nonzero__, METH_O, NULL},
	 { "vector_enumdefinitionvalues___bool__", _wrap_vector_enumdefinitionvalues___bool__, METH_O, NULL},
	 { "vector_enumdefinitionvalues___len__", _wrap_vector_enumdefinitionvalues___len__, METH_O, NULL},
	 { "vector_enumdefinitionvalues___getslice__", _wrap_vector_enumdefinitionvalues___getslice__, METH_VARARGS, NULL},
	 { "vector_enumdefinitionvalues___setslice__", _wrap_vector_enumdefinitionvalues___setslice__, METH_VARARGS, NULL},
	 { "vector_enumdefinitionvalues___delslice__", _wrap_vector_enumdefinitionvalues___delslice__, METH_VARARGS, NULL},
	 { "vector_enumdefinitionvalues___delitem__", _wrap_vector_enumdefinitionvalues___delitem__, METH_VARARGS, NULL},
	 { "vector_enumdefinitionvalues___getitem__", _wrap_vector_enumdefinitionvalues___getitem__, METH_VARARGS, NULL},
	 { "vector_enumdefinitionvalues___setitem__", _wrap_vector_enumdefinitionvalues___setitem__, METH_VARARGS, NULL},
	 { "vector_enumdefinitionvalues_pop", _wrap_vector_enumdefinitionvalues_pop, METH_O, NULL},
	 { "vector_enumdefinitionvalues_append", _wrap_vector_enumdefinitionvalues_append, METH_VARARGS, NULL},
	 { "vector_enumdefinitionvalues_empty", _wrap_vector_enumdefinitionvalues_empty, METH_O, NULL},
	 { "vector_enumdefinitionvalues_size", _wrap_vector_enumdefinitionvalues_size, METH_O, NULL},
	 { "vector_enumdefinitionvalues_swap", _wrap_vector_enumdefinitionvalues_swap, METH_VARARGS, NULL},
	 { "vector_enumdefinitionvalues_begin", _wrap_vector_enumdefinitionvalues_begin, METH_O, NULL},
	 { "vector_enumdefinitionvalues_end", _wrap_vector_enumdefinitionvalues_end, METH_O, NULL},
	 { "vector_enumdefinitionvalues_rbegin", _wrap_vector_enumdefinitionvalues_rbegin, METH_O, NULL},
	 { "vector_enumdefinitionvalues_rend", _wrap_vector_enumdefinitionvalues_rend, METH_O, NULL},
	 { "vector_enumdefinitionvalues_clear", _wrap_vector_enumdefinitionvalues_clear, METH_O, NULL},
	 { "vector_enumdefinitionvalues_get_allocator", _wrap_vector_enumdefinitionvalues_get_allocator, METH_O, NULL},
	 { "vector_enumdefinitionvalues_pop_back", _wrap_vector_enumdefinitionvalues_pop_back, METH_O, NULL},
	 { "vector_enumdefinitionvalues_erase", _wrap_vector_enumdefinitionvalues_erase, METH_VARARGS, NULL},
	 { "new_vector_enumdefinitionvalues", _wrap_new_vector_enumdefinitionvalues, METH_VARARGS, NULL},
	 { "vector_enumdefinitionvalues_push_back", _wrap_vector_enumdefinitionvalues_push_back, METH_VARARGS, NULL},
	 { "vector_enumdefinitionvalues_front", _wrap_vector_enumdefinitionvalues_front, METH_O, NULL},
	 { "vector_enumdefinitionvalues_back", _wrap_vector_enumdefinitionvalues_back, METH_O, NULL},
	 { "vector_enumdefinitionvalues_assign", _wrap_vector_enumdefinitionvalues_assign, METH_VARARGS, NULL},
	 { "vector_enumdefinitionvalues_resize", _wrap_vector_enumdefinitionvalues_resize, METH_VARARGS, NULL},
	 { "vector_enumdefinitionvalues_insert", _wrap_vector_enumdefinitionvalues_insert, METH_VARARGS, NULL},
	 { "vector_enumdefinitionvalues_reserve", _wrap_vector_enumdefinitionvalues_reserve, METH_VARARGS, NULL},
	 { "vector_enumdefinitionvalues_capacity", _wrap_vector_enumdefinitionvalues_capacity, METH_O, NULL},
	 { "delete_vector_enumdefinitionvalues", _wrap_delete_vector_enumdefinitionvalues, METH_O, NULL},
	 { "vector_enumdefinitionvalues_swigregister", vector_enumdefinitionvalues_swigregister, METH_O, NULL},
	 { "vector_enumdefinitionvalues_swiginit", vector_enumdefinitionvalues_swiginit, METH_VARARGS, NULL},
	 { "vector_usingdefinition_iterator", _wrap_vector_usingdefinition_iterator, METH_O, NULL},
	 { "vector_usingdefinition___nonzero__", _wrap_vector_usingdefinition___nonzero__, METH_O, NULL},
	 { "vector_usingdefinition___bool__", _wrap_vector_usingdefinition___bool__, METH_O, NULL},
	 { "vector_usingdefinition___len__", _wrap_vector_usingdefinition___len__, METH_O, NULL},
	 { "vector_usingdefinition___getslice__", _wrap_vector_usingdefinition___getslice__, METH_VARARGS, NULL},
	 { "vector_usingdefinition___setslice__", _wrap_vector_usingdefinition___setslice__, METH_VARARGS, NULL},
	 { "vector_usingdefinition___delslice__", _wrap_vector_usingdefinition___delslice__, METH_VARARGS, NULL},
	 { "vector_usingdefinition___delitem__", _wrap_vector_usingdefinition___delitem__, METH_VARARGS, NULL},
	 { "vector_usingdefinition___getitem__", _wrap_vector_usingdefinition___getitem__, METH_VARARGS, NULL},
	 { "vector_usingdefinition___setitem__", _wrap_vector_usingdefinition___setitem__, METH_VARARGS, NULL},
	 { "vector_usingdefinition_pop", _wrap_vector_usingdefinition_pop, METH_O, NULL},
	 { "vector_usingdefinition_append", _wrap_vector_usingdefinition_append, METH_VARARGS, NULL},
	 { "vector_usingdefinition_empty", _wrap_vector_usingdefinition_empty, METH_O, NULL},
	 { "vector_usingdefinition_size", _wrap_vector_usingdefinition_size, METH_O, NULL},
	 { "vector_usingdefinition_swap", _wrap_vector_usingdefinition_swap, METH_VARARGS, NULL},
	 { "vector_usingdefinition_begin", _wrap_vector_usingdefinition_begin, METH_O, NULL},
	 { "vector_usingdefinition_end", _wrap_vector_usingdefinition_end, METH_O, NULL},
	 { "vector_usingdefinition_rbegin", _wrap_vector_usingdefinition_rbegin, METH_O, NULL},
	 { "vector_usingdefinition_rend", _wrap_vector_usingdefinition_rend, METH_O, NULL},
	 { "vector_usingdefinition_clear", _wrap_vector_usingdefinition_clear, METH_O, NULL},
	 { "vector_usingdefinition_get_allocator", _wrap_vector_usingdefinition_get_allocator, METH_O, NULL},
	 { "vector_usingdefinition_pop_back", _wrap_vector_usingdefinition_pop_back, METH_O, NULL},
	 { "vector_usingdefinition_erase", _wrap_vector_usingdefinition_erase, METH_VARARGS, NULL},
	 { "new_vector_usingdefinition", _wrap_new_vector_usingdefinition, METH_VARARGS, NULL},
	 { "vector_usingdefinition_push_back", _wrap_vector_usingdefinition_push_back, METH_VARARGS, NULL},
	 { "vector_usingdefinition_front", _wrap_vector_usingdefinition_front, METH_O, NULL},
	 { "vector_usingdefinition_back", _wrap_vector_usingdefinition_back, METH_O, NULL},
	 { "vector_usingdefinition_assign", _wrap_vector_usingdefinition_assign, METH_VARARGS, NULL},
	 { "vector_usingdefinition_resize", _wrap_vector_usingdefinition_resize, METH_VARARGS, NULL},
	 { "vector_usingdefinition_insert", _wrap_vector_usingdefinition_insert, METH_VARARGS, NULL},
	 { "vector_usingdefinition_reserve", _wrap_vector_usingdefinition_reserve, METH_VARARGS, NULL},
	 { "vector_usingdefinition_capacity", _wrap_vector_usingdefinition_capacity, METH_O, NULL},
	 { "delete_vector_usingdefinition", _wrap_delete_vector_usingdefinition, METH_O, NULL},
	 { "vector_usingdefinition_swigregister", vector_usingdefinition_swigregister, METH_O, NULL},
	 { "vector_usingdefinition_swiginit", vector_usingdefinition_swiginit, METH_VARARGS, NULL},
	 { "vectorptr_exceptiondefinition_iterator", _wrap_vectorptr_exceptiondefinition_iterator, METH_O, NULL},
	 { "vectorptr_exceptiondefinition___nonzero__", _wrap_vectorptr_exceptiondefinition___nonzero__, METH_O, NULL},
	 { "vectorptr_exceptiondefinition___bool__", _wrap_vectorptr_exceptiondefinition___bool__, METH_O, NULL},
	 { "vectorptr_exceptiondefinition___len__", _wrap_vectorptr_exceptiondefinition___len__, METH_O, NULL},
	 { "vectorptr_exceptiondefinition___getslice__", _wrap_vectorptr_exceptiondefinition___getslice__, METH_VARARGS, NULL},
	 { "vectorptr_exceptiondefinition___setslice__", _wrap_vectorptr_exceptiondefinition___setslice__, METH_VARARGS, NULL},
	 { "vectorptr_exceptiondefinition___delslice__", _wrap_vectorptr_exceptiondefinition___delslice__, METH_VARARGS, NULL},
	 { "vectorptr_exceptiondefinition___delitem__", _wrap_vectorptr_exceptiondefinition___delitem__, METH_VARARGS, NULL},
	 { "vectorptr_exceptiondefinition___getitem__", _wrap_vectorptr_exceptiondefinition___getitem__, METH_VARARGS, NULL},
	 { "vectorptr_exceptiondefinition___setitem__", _wrap_vectorptr_exceptiondefinition___setitem__, METH_VARARGS, NULL},
	 { "vectorptr_exceptiondefinition_pop", _wrap_vectorptr_exceptiondefinition_pop, METH_O, NULL},
	 { "vectorptr_exceptiondefinition_append", _wrap_vectorptr_exceptiondefinition_append, METH_VARARGS, NULL},
	 { "vectorptr_exceptiondefinition_empty", _wrap_vectorptr_exceptiondefinition_empty, METH_O, NULL},
	 { "vectorptr_exceptiondefinition_size", _wrap_vectorptr_exceptiondefinition_size, METH_O, NULL},
	 { "vectorptr_exceptiondefinition_swap", _wrap_vectorptr_exceptiondefinition_swap, METH_VARARGS, NULL},
	 { "vectorptr_exceptiondefinition_begin", _wrap_vectorptr_exceptiondefinition_begin, METH_O, NULL},
	 { "vectorptr_exceptiondefinition_end", _wrap_vectorptr_exceptiondefinition_end, METH_O, NULL},
	 { "vectorptr_exceptiondefinition_rbegin", _wrap_vectorptr_exceptiondefinition_rbegin, METH_O, NULL},
	 { "vectorptr_exceptiondefinition_rend", _wrap_vectorptr_exceptiondefinition_rend, METH_O, NULL},
	 { "vectorptr_exceptiondefinition_clear", _wrap_vectorptr_exceptiondefinition_clear, METH_O, NULL},
	 { "vectorptr_exceptiondefinition_get_allocator", _wrap_vectorptr_exceptiondefinition_get_allocator, METH_O, NULL},
	 { "vectorptr_exceptiondefinition_pop_back", _wrap_vectorptr_exceptiondefinition_pop_back, METH_O, NULL},
	 { "vectorptr_exceptiondefinition_erase", _wrap_vectorptr_exceptiondefinition_erase, METH_VARARGS, NULL},
	 { "new_vectorptr_exceptiondefinition", _wrap_new_vectorptr_exceptiondefinition, METH_VARARGS, NULL},
	 { "vectorptr_exceptiondefinition_push_back", _wrap_vectorptr_exceptiondefinition_push_back, METH_VARARGS, NULL},
	 { "vectorptr_exceptiondefinition_front", _wrap_vectorptr_exceptiondefinition_front, METH_O, NULL},
	 { "vectorptr_exceptiondefinition_back", _wrap_vectorptr_exceptiondefinition_back, METH_O, NULL},
	 { "vectorptr_exceptiondefinition_assign", _wrap_vectorptr_exceptiondefinition_assign, METH_VARARGS, NULL},
	 { "vectorptr_exceptiondefinition_resize", _wrap_vectorptr_exceptiondefinition_resize, METH_VARARGS, NULL},
	 { "vectorptr_exceptiondefinition_insert", _wrap_vectorptr_exceptiondefinition_insert, METH_VARARGS, NULL},
	 { "vectorptr_exceptiondefinition_reserve", _wrap_vectorptr_exceptiondefinition_reserve, METH_VARARGS, NULL},
	 { "vectorptr_exceptiondefinition_capacity", _wrap_vectorptr_exceptiondefinition_capacity, METH_O, NULL},
	 { "delete_vectorptr_exceptiondefinition", _wrap_delete_vectorptr_exceptiondefinition, METH_O, NULL},
	 { "vectorptr_exceptiondefinition_swigregister", vectorptr_exceptiondefinition_swigregister, METH_O, NULL},
	 { "vectorptr_exceptiondefinition_swiginit", vectorptr_exceptiondefinition_swiginit, METH_VARARGS, NULL},
	 { "vectorptr_servicedefinition_iterator", _wrap_vectorptr_servicedefinition_iterator, METH_O, NULL},
	 { "vectorptr_servicedefinition___nonzero__", _wrap_vectorptr_servicedefinition___nonzero__, METH_O, NULL},
	 { "vectorptr_servicedefinition___bool__", _wrap_vectorptr_servicedefinition___bool__, METH_O, NULL},
	 { "vectorptr_servicedefinition___len__", _wrap_vectorptr_servicedefinition___len__, METH_O, NULL},
	 { "vectorptr_servicedefinition___getslice__", _wrap_vectorptr_servicedefinition___getslice__, METH_VARARGS, NULL},
	 { "vectorptr_servicedefinition___setslice__", _wrap_vectorptr_servicedefinition___setslice__, METH_VARARGS, NULL},
	 { "vectorptr_servicedefinition___delslice__", _wrap_vectorptr_servicedefinition___delslice__, METH_VARARGS, NULL},
	 { "vectorptr_servicedefinition___delitem__", _wrap_vectorptr_servicedefinition___delitem__, METH_VARARGS, NULL},
	 { "vectorptr_servicedefinition___getitem__", _wrap_vectorptr_servicedefinition___getitem__, METH_VARARGS, NULL},
	 { "vectorptr_servicedefinition___setitem__", _wrap_vectorptr_servicedefinition___setitem__, METH_VARARGS, NULL},
	 { "vectorptr_servicedefinition_pop", _wrap_vectorptr_servicedefinition_pop, METH_O, NULL},
	 { "vectorptr_servicedefinition_append", _wrap_vectorptr_servicedefinition_append, METH_VARARGS, NULL},
	 { "vectorptr_servicedefinition_empty", _wrap_vectorptr_servicedefinition_empty, METH_O, NULL},
	 { "vectorptr_servicedefinition_size", _wrap_vectorptr_servicedefinition_size, METH_O, NULL},
	 { "vectorptr_servicedefinition_swap", _wrap_vectorptr_servicedefinition_swap, METH_VARARGS, NULL},
	 { "vectorptr_servicedefinition_begin", _wrap_vectorptr_servicedefinition_begin, METH_O, NULL},
	 { "vectorptr_servicedefinition_end", _wrap_vectorptr_servicedefinition_end, METH_O, NULL},
	 { "vectorptr_servicedefinition_rbegin", _wrap_vectorptr_servicedefinition_rbegin, METH_O, NULL},
	 { "vectorptr_servicedefinition_rend", _wrap_vectorptr_servicedefinition_rend, METH_O, NULL},
	 { "vectorptr_servicedefinition_clear", _wrap_vectorptr_servicedefinition_clear, METH_O, NULL},
	 { "vectorptr_servicedefinition_get_allocator", _wrap_vectorptr_servicedefinition_get_allocator, METH_O, NULL},
	 { "vectorptr_servicedefinition_pop_back", _wrap_vectorptr_servicedefinition_pop_back, METH_O, NULL},
	 { "vectorptr_servicedefinition_erase", _wrap_vectorptr_servicedefinition_erase, METH_VARARGS, NULL},
	 { "new_vectorptr_servicedefinition", _wrap_new_vectorptr_servicedefinition, METH_VARARGS, NULL},
	 { "vectorptr_servicedefinition_push_back", _wrap_vectorptr_servicedefinition_push_back, METH_VARARGS, NULL},
	 { "vectorptr_servicedefinition_front", _wrap_vectorptr_servicedefinition_front, METH_O, NULL},
	 { "vectorptr_servicedefinition_back", _wrap_vectorptr_servicedefinition_back, METH_O, NULL},
	 { "vectorptr_servicedefinition_assign", _wrap_vectorptr_servicedefinition_assign, METH_VARARGS, NULL},
	 { "vectorptr_servicedefinition_resize", _wrap_vectorptr_servicedefinition_resize, METH_VARARGS, NULL},
	 { "vectorptr_servicedefinition_insert", _wrap_vectorptr_servicedefinition_insert, METH_VARARGS, NULL},
	 { "vectorptr_servicedefinition_reserve", _wrap_vectorptr_servicedefinition_reserve, METH_VARARGS, NULL},
	 { "vectorptr_servicedefinition_capacity", _wrap_vectorptr_servicedefinition_capacity, METH_O, NULL},
	 { "delete_vectorptr_servicedefinition", _wrap_delete_vectorptr_servicedefinition, METH_O, NULL},
	 { "vectorptr_servicedefinition_swigregister", vectorptr_servicedefinition_swigregister, METH_O, NULL},
	 { "vectorptr_servicedefinition_swiginit", vectorptr_servicedefinition_swiginit, METH_VARARGS, NULL},
	 { "ServiceDefinitionParseInfo_ServiceName_set", _wrap_ServiceDefinitionParseInfo_ServiceName_set, METH_VARARGS, NULL},
	 { "ServiceDefinitionParseInfo_ServiceName_get", _wrap_ServiceDefinitionParseInfo_ServiceName_get, METH_O, NULL},
	 { "ServiceDefinitionParseInfo_RobDefFilePath_set", _wrap_ServiceDefinitionParseInfo_RobDefFilePath_set, METH_VARARGS, NULL},
	 { "ServiceDefinitionParseInfo_RobDefFilePath_get", _wrap_ServiceDefinitionParseInfo_RobDefFilePath_get, METH_O, NULL},
	 { "ServiceDefinitionParseInfo_Line_set", _wrap_ServiceDefinitionParseInfo_Line_set, METH_VARARGS, NULL},
	 { "ServiceDefinitionParseInfo_Line_get", _wrap_ServiceDefinitionParseInfo_Line_get, METH_O, NULL},
	 { "ServiceDefinitionParseInfo_LineNumber_set", _wrap_ServiceDefinitionParseInfo_LineNumber_set, METH_VARARGS, NULL},
	 { "ServiceDefinitionParseInfo_LineNumber_get", _wrap_ServiceDefinitionParseInfo_LineNumber_get, METH_O, NULL},
	 { "new_ServiceDefinitionParseInfo", _wrap_new_ServiceDefinitionParseInfo, METH_NOARGS, NULL},
	 { "ServiceDefinitionParseInfo_Reset", _wrap_ServiceDefinitionParseInfo_Reset, METH_O, NULL},
	 { "delete_ServiceDefinitionParseInfo", _wrap_delete_ServiceDefinitionParseInfo, METH_O, NULL},
	 { "ServiceDefinitionParseInfo_swigregister", ServiceDefinitionParseInfo_swigregister, METH_O, NULL},
	 { "ServiceDefinitionParseInfo_swiginit", ServiceDefinitionParseInfo_swiginit, METH_VARARGS, NULL},
	 { "new_RobotRaconteurVersion", _wrap_new_RobotRaconteurVersion, METH_VARARGS, NULL},
	 { "RobotRaconteurVersion_ToString", _wrap_RobotRaconteurVersion_ToString, METH_O, NULL},
	 { "RobotRaconteurVersion_FromString", _wrap_RobotRaconteurVersion_FromString, METH_VARARGS, NULL},
	 { "RobotRaconteurVersion___nonzero__", _wrap_RobotRaconteurVersion___nonzero__, METH_O, NULL},
	 { "RobotRaconteurVersion_major_set", _wrap_RobotRaconteurVersion_major_set, METH_VARARGS, NULL},
	 { "RobotRaconteurVersion_major_get", _wrap_RobotRaconteurVersion_major_get, METH_O, NULL},
	 { "RobotRaconteurVersion_minor_set", _wrap_RobotRaconteurVersion_minor_set, METH_VARARGS, NULL},
	 { "RobotRaconteurVersion_minor_get", _wrap_RobotRaconteurVersion_minor_get, METH_O, NULL},
	 { "RobotRaconteurVersion_patch_set", _wrap_RobotRaconteurVersion_patch_set, METH_VARARGS, NULL},
	 { "RobotRaconteurVersion_patch_get", _wrap_RobotRaconteurVersion_patch_get, METH_O, NULL},
	 { "RobotRaconteurVersion_tweak_set", _wrap_RobotRaconteurVersion_tweak_set, METH_VARARGS, NULL},
	 { "RobotRaconteurVersion_tweak_get", _wrap_RobotRaconteurVersion_tweak_get, METH_O, NULL},
	 { "RobotRaconteurVersion_ParseInfo_set", _wrap_RobotRaconteurVersion_ParseInfo_set, METH_VARARGS, NULL},
	 { "RobotRaconteurVersion_ParseInfo_get", _wrap_RobotRaconteurVersion_ParseInfo_get, METH_O, NULL},
	 { "delete_RobotRaconteurVersion", _wrap_delete_RobotRaconteurVersion, METH_O, NULL},
	 { "RobotRaconteurVersion_swigregister", RobotRaconteurVersion_swigregister, METH_O, NULL},
	 { "RobotRaconteurVersion_swiginit", RobotRaconteurVersion_swiginit, METH_VARARGS, NULL},
	 { "RobotRaconteurVersion_eq", _wrap_RobotRaconteurVersion_eq, METH_VARARGS, NULL},
	 { "RobotRaconteurVersion_ne", _wrap_RobotRaconteurVersion_ne, METH_VARARGS, NULL},
	 { "RobotRaconteurVersion_gt", _wrap_RobotRaconteurVersion_gt, METH_VARARGS, NULL},
	 { "RobotRaconteurVersion_ge", _wrap_RobotRaconteurVersion_ge, METH_VARARGS, NULL},
	 { "RobotRaconteurVersion_lt", _wrap_RobotRaconteurVersion_lt, METH_VARARGS, NULL},
	 { "RobotRaconteurVersion_le", _wrap_RobotRaconteurVersion_le, METH_VARARGS, NULL},
	 { "ServiceDefinition_Name_set", _wrap_ServiceDefinition_Name_set, METH_VARARGS, NULL},
	 { "ServiceDefinition_Name_get", _wrap_ServiceDefinition_Name_get, METH_O, NULL},
	 { "ServiceDefinition_NamedArrays_set", _wrap_ServiceDefinition_NamedArrays_set, METH_VARARGS, NULL},
	 { "ServiceDefinition_NamedArrays_get", _wrap_ServiceDefinition_NamedArrays_get, METH_O, NULL},
	 { "ServiceDefinition_Pods_set", _wrap_ServiceDefinition_Pods_set, METH_VARARGS, NULL},
	 { "ServiceDefinition_Pods_get", _wrap_ServiceDefinition_Pods_get, METH_O, NULL},
	 { "ServiceDefinition_Structures_set", _wrap_ServiceDefinition_Structures_set, METH_VARARGS, NULL},
	 { "ServiceDefinition_Structures_get", _wrap_ServiceDefinition_Structures_get, METH_O, NULL},
	 { "ServiceDefinition_Objects_set", _wrap_ServiceDefinition_Objects_set, METH_VARARGS, NULL},
	 { "ServiceDefinition_Objects_get", _wrap_ServiceDefinition_Objects_get, METH_O, NULL},
	 { "ServiceDefinition_Options_set", _wrap_ServiceDefinition_Options_set, METH_VARARGS, NULL},
	 { "ServiceDefinition_Options_get", _wrap_ServiceDefinition_Options_get, METH_O, NULL},
	 { "ServiceDefinition_Imports_set", _wrap_ServiceDefinition_Imports_set, METH_VARARGS, NULL},
	 { "ServiceDefinition_Imports_get", _wrap_ServiceDefinition_Imports_get, METH_O, NULL},
	 { "ServiceDefinition_Exceptions_set", _wrap_ServiceDefinition_Exceptions_set, METH_VARARGS, NULL},
	 { "ServiceDefinition_Exceptions_get", _wrap_ServiceDefinition_Exceptions_get, METH_O, NULL},
	 { "ServiceDefinition_Using_set", _wrap_ServiceDefinition_Using_set, METH_VARARGS, NULL},
	 { "ServiceDefinition_Using_get", _wrap_ServiceDefinition_Using_get, METH_O, NULL},
	 { "ServiceDefinition_Constants_set", _wrap_ServiceDefinition_Constants_set, METH_VARARGS, NULL},
	 { "ServiceDefinition_Constants_get", _wrap_ServiceDefinition_Constants_get, METH_O, NULL},
	 { "ServiceDefinition_Enums_set", _wrap_ServiceDefinition_Enums_set, METH_VARARGS, NULL},
	 { "ServiceDefinition_Enums_get", _wrap_ServiceDefinition_Enums_get, METH_O, NULL},
	 { "ServiceDefinition_StdVer_set", _wrap_ServiceDefinition_StdVer_set, METH_VARARGS, NULL},
	 { "ServiceDefinition_StdVer_get", _wrap_ServiceDefinition_StdVer_get, METH_O, NULL},
	 { "ServiceDefinition_ParseInfo_set", _wrap_ServiceDefinition_ParseInfo_set, METH_VARARGS, NULL},
	 { "ServiceDefinition_ParseInfo_get", _wrap_ServiceDefinition_ParseInfo_get, METH_O, NULL},
	 { "ServiceDefinition_DocString_set", _wrap_ServiceDefinition_DocString_set, METH_VARARGS, NULL},
	 { "ServiceDefinition_DocString_get", _wrap_ServiceDefinition_DocString_get, METH_O, NULL},
	 { "ServiceDefinition_ToString", _wrap_ServiceDefinition_ToString, METH_O, NULL},
	 { "ServiceDefinition_FromString", _wrap_ServiceDefinition_FromString, METH_VARARGS, NULL},
	 { "new_ServiceDefinition", _wrap_new_ServiceDefinition, METH_NOARGS, NULL},
	 { "ServiceDefinition_Reset", _wrap_ServiceDefinition_Reset, METH_O, NULL},
	 { "delete_ServiceDefinition", _wrap_delete_ServiceDefinition, METH_O, NULL},
	 { "ServiceDefinition_swigregister", ServiceDefinition_swigregister, METH_O, NULL},
	 { "ServiceDefinition_swiginit", ServiceDefinition_swiginit, METH_VARARGS, NULL},
	 { "ServiceEntryDefinition_Name_set", _wrap_ServiceEntryDefinition_Name_set, METH_VARARGS, NULL},
	 { "ServiceEntryDefinition_Name_get", _wrap_ServiceEntryDefinition_Name_get, METH_O, NULL},
	 { "ServiceEntryDefinition_Members_set", _wrap_ServiceEntryDefinition_Members_set, METH_VARARGS, NULL},
	 { "ServiceEntryDefinition_Members_get", _wrap_ServiceEntryDefinition_Members_get, METH_O, NULL},
	 { "ServiceEntryDefinition_EntryType_set", _wrap_ServiceEntryDefinition_EntryType_set, METH_VARARGS, NULL},
	 { "ServiceEntryDefinition_EntryType_get", _wrap_ServiceEntryDefinition_EntryType_get, METH_O, NULL},
	 { "ServiceEntryDefinition_Implements_set", _wrap_ServiceEntryDefinition_Implements_set, METH_VARARGS, NULL},
	 { "ServiceEntryDefinition_Implements_get", _wrap_ServiceEntryDefinition_Implements_get, METH_O, NULL},
	 { "ServiceEntryDefinition_Options_set", _wrap_ServiceEntryDefinition_Options_set, METH_VARARGS, NULL},
	 { "ServiceEntryDefinition_Options_get", _wrap_ServiceEntryDefinition_Options_get, METH_O, NULL},
	 { "ServiceEntryDefinition_Constants_set", _wrap_ServiceEntryDefinition_Constants_set, METH_VARARGS, NULL},
	 { "ServiceEntryDefinition_Constants_get", _wrap_ServiceEntryDefinition_Constants_get, METH_O, NULL},
	 { "ServiceEntryDefinition_ParseInfo_set", _wrap_ServiceEntryDefinition_ParseInfo_set, METH_VARARGS, NULL},
	 { "ServiceEntryDefinition_ParseInfo_get", _wrap_ServiceEntryDefinition_ParseInfo_get, METH_O, NULL},
	 { "ServiceEntryDefinition_DocString_set", _wrap_ServiceEntryDefinition_DocString_set, METH_VARARGS, NULL},
	 { "ServiceEntryDefinition_DocString_get", _wrap_ServiceEntryDefinition_DocString_get, METH_O, NULL},
	 { "new_ServiceEntryDefinition", _wrap_new_ServiceEntryDefinition, METH_O, NULL},
	 { "ServiceEntryDefinition_ToString", _wrap_ServiceEntryDefinition_ToString, METH_O, NULL},
	 { "ServiceEntryDefinition_FromString", _wrap_ServiceEntryDefinition_FromString, METH_VARARGS, NULL},
	 { "ServiceEntryDefinition_GetServiceDefinition", _wrap_ServiceEntryDefinition_GetServiceDefinition, METH_O, NULL},
	 { "ServiceEntryDefinition_SetServiceDefinition", _wrap_ServiceEntryDefinition_SetServiceDefinition, METH_VARARGS, NULL},
	 { "ServiceEntryDefinition_Reset", _wrap_ServiceEntryDefinition_Reset, METH_O, NULL},
	 { "delete_ServiceEntryDefinition", _wrap_delete_ServiceEntryDefinition, METH_O, NULL},
	 { "ServiceEntryDefinition_swigregister", ServiceEntryDefinition_swigregister, METH_O, NULL},
	 { "ServiceEntryDefinition_swiginit", ServiceEntryDefinition_swiginit, METH_VARARGS, NULL},
	 { "MemberDefinition_Name_set", _wrap_MemberDefinition_Name_set, METH_VARARGS, NULL},
	 { "MemberDefinition_Name_get", _wrap_MemberDefinition_Name_get, METH_O, NULL},
	 { "MemberDefinition_DocString_set", _wrap_MemberDefinition_DocString_set, METH_VARARGS, NULL},
	 { "MemberDefinition_DocString_get", _wrap_MemberDefinition_DocString_get, METH_O, NULL},
	 { "new_MemberDefinition", _wrap_new_MemberDefinition, METH_O, NULL},
	 { "MemberDefinition_NoLock", _wrap_MemberDefinition_NoLock, METH_O, NULL},
	 { "MemberDefinition_ToString", _wrap_MemberDefinition_ToString, METH_O, NULL},
	 { "MemberDefinition_GetServiceEntry", _wrap_MemberDefinition_GetServiceEntry, METH_O, NULL},
	 { "MemberDefinition_SetServiceEntry", _wrap_MemberDefinition_SetServiceEntry, METH_VARARGS, NULL},
	 { "MemberDefinition_Modifiers_set", _wrap_MemberDefinition_Modifiers_set, METH_VARARGS, NULL},
	 { "MemberDefinition_Modifiers_get", _wrap_MemberDefinition_Modifiers_get, METH_O, NULL},
	 { "MemberDefinition_ParseInfo_set", _wrap_MemberDefinition_ParseInfo_set, METH_VARARGS, NULL},
	 { "MemberDefinition_ParseInfo_get", _wrap_MemberDefinition_ParseInfo_get, METH_O, NULL},
	 { "MemberDefinition_Reset", _wrap_MemberDefinition_Reset, METH_O, NULL},
	 { "delete_MemberDefinition", _wrap_delete_MemberDefinition, METH_O, NULL},
	 { "MemberDefinition_swigregister", MemberDefinition_swigregister, METH_O, NULL},
	 { "MemberDefinition_swiginit", MemberDefinition_swiginit, METH_VARARGS, NULL},
	 { "PropertyDefinition_Type_set", _wrap_PropertyDefinition_Type_set, METH_VARARGS, NULL},
	 { "PropertyDefinition_Type_get", _wrap_PropertyDefinition_Type_get, METH_O, NULL},
	 { "new_PropertyDefinition", _wrap_new_PropertyDefinition, METH_O, NULL},
	 { "PropertyDefinition_ToString", _wrap_PropertyDefinition_ToString, METH_VARARGS, NULL},
	 { "PropertyDefinition_FromString", _wrap_PropertyDefinition_FromString, METH_VARARGS, NULL},
	 { "delete_PropertyDefinition", _wrap_delete_PropertyDefinition, METH_O, NULL},
	 { "PropertyDefinition_swigregister", PropertyDefinition_swigregister, METH_O, NULL},
	 { "PropertyDefinition_swiginit", PropertyDefinition_swiginit, METH_VARARGS, NULL},
	 { "FunctionDefinition_ReturnType_set", _wrap_FunctionDefinition_ReturnType_set, METH_VARARGS, NULL},
	 { "FunctionDefinition_ReturnType_get", _wrap_FunctionDefinition_ReturnType_get, METH_O, NULL},
	 { "FunctionDefinition_Parameters_set", _wrap_FunctionDefinition_Parameters_set, METH_VARARGS, NULL},
	 { "FunctionDefinition_Parameters_get", _wrap_FunctionDefinition_Parameters_get, METH_O, NULL},
	 { "new_FunctionDefinition", _wrap_new_FunctionDefinition, METH_O, NULL},
	 { "FunctionDefinition_ToString", _wrap_FunctionDefinition_ToString, METH_O, NULL},
	 { "FunctionDefinition_FromString", _wrap_FunctionDefinition_FromString, METH_VARARGS, NULL},
	 { "FunctionDefinition_IsGenerator", _wrap_FunctionDefinition_IsGenerator, METH_O, NULL},
	 { "delete_FunctionDefinition", _wrap_delete_FunctionDefinition, METH_O, NULL},
	 { "FunctionDefinition_swigregister", FunctionDefinition_swigregister, METH_O, NULL},
	 { "FunctionDefinition_swiginit", FunctionDefinition_swiginit, METH_VARARGS, NULL},
	 { "EventDefinition_Parameters_set", _wrap_EventDefinition_Parameters_set, METH_VARARGS, NULL},
	 { "EventDefinition_Parameters_get", _wrap_EventDefinition_Parameters_get, METH_O, NULL},
	 { "new_EventDefinition", _wrap_new_EventDefinition, METH_O, NULL},
	 { "EventDefinition_ToString", _wrap_EventDefinition_ToString, METH_O, NULL},
	 { "EventDefinition_FromString", _wrap_EventDefinition_FromString, METH_VARARGS, NULL},
	 { "delete_EventDefinition", _wrap_delete_EventDefinition, METH_O, NULL},
	 { "EventDefinition_swigregister", EventDefinition_swigregister, METH_O, NULL},
	 { "EventDefinition_swiginit", EventDefinition_swiginit, METH_VARARGS, NULL},
	 { "ObjRefDefinition_ObjectType_set", _wrap_ObjRefDefinition_ObjectType_set, METH_VARARGS, NULL},
	 { "ObjRefDefinition_ObjectType_get", _wrap_ObjRefDefinition_ObjectType_get, METH_O, NULL},
	 { "ObjRefDefinition_ArrayType_set", _wrap_ObjRefDefinition_ArrayType_set, METH_VARARGS, NULL},
	 { "ObjRefDefinition_ArrayType_get", _wrap_ObjRefDefinition_ArrayType_get, METH_O, NULL},
	 { "ObjRefDefinition_ContainerType_set", _wrap_ObjRefDefinition_ContainerType_set, METH_VARARGS, NULL},
	 { "ObjRefDefinition_ContainerType_get", _wrap_ObjRefDefinition_ContainerType_get, METH_O, NULL},
	 { "new_ObjRefDefinition", _wrap_new_ObjRefDefinition, METH_O, NULL},
	 { "ObjRefDefinition_ToString", _wrap_ObjRefDefinition_ToString, METH_O, NULL},
	 { "ObjRefDefinition_FromString", _wrap_ObjRefDefinition_FromString, METH_VARARGS, NULL},
	 { "delete_ObjRefDefinition", _wrap_delete_ObjRefDefinition, METH_O, NULL},
	 { "ObjRefDefinition_swigregister", ObjRefDefinition_swigregister, METH_O, NULL},
	 { "ObjRefDefinition_swiginit", ObjRefDefinition_swiginit, METH_VARARGS, NULL},
	 { "PipeDefinition_Type_set", _wrap_PipeDefinition_Type_set, METH_VARARGS, NULL},
	 { "PipeDefinition_Type_get", _wrap_PipeDefinition_Type_get, METH_O, NULL},
	 { "new_PipeDefinition", _wrap_new_PipeDefinition, METH_O, NULL},
	 { "PipeDefinition_ToString", _wrap_PipeDefinition_ToString, METH_O, NULL},
	 { "PipeDefinition_FromString", _wrap_PipeDefinition_FromString, METH_VARARGS, NULL},
	 { "delete_PipeDefinition", _wrap_delete_PipeDefinition, METH_O, NULL},
	 { "PipeDefinition_swigregister", PipeDefinition_swigregister, METH_O, NULL},
	 { "PipeDefinition_swiginit", PipeDefinition_swiginit, METH_VARARGS, NULL},
	 { "CallbackDefinition_ReturnType_set", _wrap_CallbackDefinition_ReturnType_set, METH_VARARGS, NULL},
	 { "CallbackDefinition_ReturnType_get", _wrap_CallbackDefinition_ReturnType_get, METH_O, NULL},
	 { "CallbackDefinition_Parameters_set", _wrap_CallbackDefinition_Parameters_set, METH_VARARGS, NULL},
	 { "CallbackDefinition_Parameters_get", _wrap_CallbackDefinition_Parameters_get, METH_O, NULL},
	 { "new_CallbackDefinition", _wrap_new_CallbackDefinition, METH_O, NULL},
	 { "CallbackDefinition_ToString", _wrap_CallbackDefinition_ToString, METH_O, NULL},
	 { "CallbackDefinition_FromString", _wrap_CallbackDefinition_FromString, METH_VARARGS, NULL},
	 { "delete_CallbackDefinition", _wrap_delete_CallbackDefinition, METH_O, NULL},
	 { "CallbackDefinition_swigregister", CallbackDefinition_swigregister, METH_O, NULL},
	 { "CallbackDefinition_swiginit", CallbackDefinition_swiginit, METH_VARARGS, NULL},
	 { "WireDefinition_Type_set", _wrap_WireDefinition_Type_set, METH_VARARGS, NULL},
	 { "WireDefinition_Type_get", _wrap_WireDefinition_Type_get, METH_O, NULL},
	 { "new_WireDefinition", _wrap_new_WireDefinition, METH_O, NULL},
	 { "WireDefinition_ToString", _wrap_WireDefinition_ToString, METH_O, NULL},
	 { "WireDefinition_FromString", _wrap_WireDefinition_FromString, METH_VARARGS, NULL},
	 { "delete_WireDefinition", _wrap_delete_WireDefinition, METH_O, NULL},
	 { "WireDefinition_swigregister", WireDefinition_swigregister, METH_O, NULL},
	 { "WireDefinition_swiginit", WireDefinition_swiginit, METH_VARARGS, NULL},
	 { "MemoryDefinition_Type_set", _wrap_MemoryDefinition_Type_set, METH_VARARGS, NULL},
	 { "MemoryDefinition_Type_get", _wrap_MemoryDefinition_Type_get, METH_O, NULL},
	 { "new_MemoryDefinition", _wrap_new_MemoryDefinition, METH_O, NULL},
	 { "MemoryDefinition_ToString", _wrap_MemoryDefinition_ToString, METH_O, NULL},
	 { "MemoryDefinition_FromString", _wrap_MemoryDefinition_FromString, METH_VARARGS, NULL},
	 { "delete_MemoryDefinition", _wrap_delete_MemoryDefinition, METH_O, NULL},
	 { "MemoryDefinition_swigregister", MemoryDefinition_swigregister, METH_O, NULL},
	 { "MemoryDefinition_swiginit", MemoryDefinition_swiginit, METH_VARARGS, NULL},
	 { "TypeDefinition_Name_set", _wrap_TypeDefinition_Name_set, METH_VARARGS, NULL},
	 { "TypeDefinition_Name_get", _wrap_TypeDefinition_Name_get, METH_O, NULL},
	 { "TypeDefinition_Type_set", _wrap_TypeDefinition_Type_set, METH_VARARGS, NULL},
	 { "TypeDefinition_Type_get", _wrap_TypeDefinition_Type_get, METH_O, NULL},
	 { "TypeDefinition_TypeString_set", _wrap_TypeDefinition_TypeString_set, METH_VARARGS, NULL},
	 { "TypeDefinition_TypeString_get", _wrap_TypeDefinition_TypeString_get, METH_O, NULL},
	 { "TypeDefinition_ArrayType_set", _wrap_TypeDefinition_ArrayType_set, METH_VARARGS, NULL},
	 { "TypeDefinition_ArrayType_get", _wrap_TypeDefinition_ArrayType_get, METH_O, NULL},
	 { "TypeDefinition_ArrayVarLength_set", _wrap_TypeDefinition_ArrayVarLength_set, METH_VARARGS, NULL},
	 { "TypeDefinition_ArrayVarLength_get", _wrap_TypeDefinition_ArrayVarLength_get, METH_O, NULL},
	 { "TypeDefinition_ArrayLength_set", _wrap_TypeDefinition_ArrayLength_set, METH_VARARGS, NULL},
	 { "TypeDefinition_ArrayLength_get", _wrap_TypeDefinition_ArrayLength_get, METH_O, NULL},
	 { "TypeDefinition_ContainerType_set", _wrap_TypeDefinition_ContainerType_set, METH_VARARGS, NULL},
	 { "TypeDefinition_ContainerType_get", _wrap_TypeDefinition_ContainerType_get, METH_O, NULL},
	 { "TypeDefinition_ToString", _wrap_TypeDefinition_ToString, METH_O, NULL},
	 { "TypeDefinition_FromString", _wrap_TypeDefinition_FromString, METH_VARARGS, NULL},
	 { "TypeDefinition_DataTypeFromString", _wrap_TypeDefinition_DataTypeFromString, METH_O, NULL},
	 { "TypeDefinition_StringFromDataType", _wrap_TypeDefinition_StringFromDataType, METH_O, NULL},
	 { "TypeDefinition__GetMember", _wrap_TypeDefinition__GetMember, METH_O, NULL},
	 { "TypeDefinition__SetMember", _wrap_TypeDefinition__SetMember, METH_VARARGS, NULL},
	 { "TypeDefinition_CopyTo", _wrap_TypeDefinition_CopyTo, METH_VARARGS, NULL},
	 { "TypeDefinition_Clone", _wrap_TypeDefinition_Clone, METH_O, NULL},
	 { "TypeDefinition_Rename", _wrap_TypeDefinition_Rename, METH_VARARGS, NULL},
	 { "TypeDefinition_RemoveContainers", _wrap_TypeDefinition_RemoveContainers, METH_O, NULL},
	 { "TypeDefinition_RemoveArray", _wrap_TypeDefinition_RemoveArray, METH_O, NULL},
	 { "TypeDefinition_QualifyTypeStringWithUsing", _wrap_TypeDefinition_QualifyTypeStringWithUsing, METH_O, NULL},
	 { "TypeDefinition_UnqualifyTypeStringWithUsing", _wrap_TypeDefinition_UnqualifyTypeStringWithUsing, METH_O, NULL},
	 { "TypeDefinition_Reset", _wrap_TypeDefinition_Reset, METH_O, NULL},
	 { "new_TypeDefinition", _wrap_new_TypeDefinition, METH_VARARGS, NULL},
	 { "delete_TypeDefinition", _wrap_delete_TypeDefinition, METH_O, NULL},
	 { "TypeDefinition_swigregister", TypeDefinition_swigregister, METH_O, NULL},
	 { "TypeDefinition_swiginit", TypeDefinition_swiginit, METH_VARARGS, NULL},
	 { "delete_UsingDefinition", _wrap_delete_UsingDefinition, METH_O, NULL},
	 { "UsingDefinition_QualifiedName_set", _wrap_UsingDefinition_QualifiedName_set, METH_VARARGS, NULL},
	 { "UsingDefinition_QualifiedName_get", _wrap_UsingDefinition_QualifiedName_get, METH_O, NULL},
	 { "UsingDefinition_UnqualifiedName_set", _wrap_UsingDefinition_UnqualifiedName_set, METH_VARARGS, NULL},
	 { "UsingDefinition_UnqualifiedName_get", _wrap_UsingDefinition_UnqualifiedName_get, METH_O, NULL},
	 { "UsingDefinition_ParseInfo_set", _wrap_UsingDefinition_ParseInfo_set, METH_VARARGS, NULL},
	 { "UsingDefinition_ParseInfo_get", _wrap_UsingDefinition_ParseInfo_get, METH_O, NULL},
	 { "UsingDefinition__GetService", _wrap_UsingDefinition__GetService, METH_O, NULL},
	 { "UsingDefinition__SetService", _wrap_UsingDefinition__SetService, METH_VARARGS, NULL},
	 { "new_UsingDefinition", _wrap_new_UsingDefinition, METH_O, NULL},
	 { "UsingDefinition_ToString", _wrap_UsingDefinition_ToString, METH_O, NULL},
	 { "UsingDefinition_FromString", _wrap_UsingDefinition_FromString, METH_VARARGS, NULL},
	 { "UsingDefinition_swigregister", UsingDefinition_swigregister, METH_O, NULL},
	 { "UsingDefinition_swiginit", UsingDefinition_swiginit, METH_VARARGS, NULL},
	 { "ConstantDefinition_StructField_Name_set", _wrap_ConstantDefinition_StructField_Name_set, METH_VARARGS, NULL},
	 { "ConstantDefinition_StructField_Name_get", _wrap_ConstantDefinition_StructField_Name_get, METH_O, NULL},
	 { "ConstantDefinition_StructField_ConstantRefName_set", _wrap_ConstantDefinition_StructField_ConstantRefName_set, METH_VARARGS, NULL},
	 { "ConstantDefinition_StructField_ConstantRefName_get", _wrap_ConstantDefinition_StructField_ConstantRefName_get, METH_O, NULL},
	 { "new_ConstantDefinition_StructField", _wrap_new_ConstantDefinition_StructField, METH_NOARGS, NULL},
	 { "delete_ConstantDefinition_StructField", _wrap_delete_ConstantDefinition_StructField, METH_O, NULL},
	 { "ConstantDefinition_StructField_swigregister", ConstantDefinition_StructField_swigregister, METH_O, NULL},
	 { "ConstantDefinition_StructField_swiginit", ConstantDefinition_StructField_swiginit, METH_VARARGS, NULL},
	 { "delete_ConstantDefinition", _wrap_delete_ConstantDefinition, METH_O, NULL},
	 { "ConstantDefinition_Name_set", _wrap_ConstantDefinition_Name_set, METH_VARARGS, NULL},
	 { "ConstantDefinition_Name_get", _wrap_ConstantDefinition_Name_get, METH_O, NULL},
	 { "ConstantDefinition_DocString_set", _wrap_ConstantDefinition_DocString_set, METH_VARARGS, NULL},
	 { "ConstantDefinition_DocString_get", _wrap_ConstantDefinition_DocString_get, METH_O, NULL},
	 { "ConstantDefinition_Type_set", _wrap_ConstantDefinition_Type_set, METH_VARARGS, NULL},
	 { "ConstantDefinition_Type_get", _wrap_ConstantDefinition_Type_get, METH_O, NULL},
	 { "ConstantDefinition_Value_set", _wrap_ConstantDefinition_Value_set, METH_VARARGS, NULL},
	 { "ConstantDefinition_Value_get", _wrap_ConstantDefinition_Value_get, METH_O, NULL},
	 { "ConstantDefinition_ParseInfo_set", _wrap_ConstantDefinition_ParseInfo_set, METH_VARARGS, NULL},
	 { "ConstantDefinition_ParseInfo_get", _wrap_ConstantDefinition_ParseInfo_get, METH_O, NULL},
	 { "ConstantDefinition__GetService", _wrap_ConstantDefinition__GetService, METH_O, NULL},
	 { "ConstantDefinition__SetService", _wrap_ConstantDefinition__SetService, METH_VARARGS, NULL},
	 { "ConstantDefinition__GetServiceEntry", _wrap_ConstantDefinition__GetServiceEntry, METH_O, NULL},
	 { "ConstantDefinition__SetServiceEntry", _wrap_ConstantDefinition__SetServiceEntry, METH_VARARGS, NULL},
	 { "new_ConstantDefinition", _wrap_new_ConstantDefinition, METH_VARARGS, NULL},
	 { "ConstantDefinition_ToString", _wrap_ConstantDefinition_ToString, METH_O, NULL},
	 { "ConstantDefinition_FromString", _wrap_ConstantDefinition_FromString, METH_VARARGS, NULL},
	 { "ConstantDefinition_Reset", _wrap_ConstantDefinition_Reset, METH_O, NULL},
	 { "ConstantDefinition_VerifyValue", _wrap_ConstantDefinition_VerifyValue, METH_O, NULL},
	 { "ConstantDefinition_ValueToString", _wrap_ConstantDefinition_ValueToString, METH_O, NULL},
	 { "ConstantDefinition_ValueToStructFields", _wrap_ConstantDefinition_ValueToStructFields, METH_O, NULL},
	 { "ConstantDefinition_UnescapeString", _wrap_ConstantDefinition_UnescapeString, METH_O, NULL},
	 { "ConstantDefinition_EscapeString", _wrap_ConstantDefinition_EscapeString, METH_O, NULL},
	 { "ConstantDefinition_swigregister", ConstantDefinition_swigregister, METH_O, NULL},
	 { "ConstantDefinition_swiginit", ConstantDefinition_swiginit, METH_VARARGS, NULL},
	 { "delete_EnumDefinition", _wrap_delete_EnumDefinition, METH_O, NULL},
	 { "EnumDefinition_Name_set", _wrap_EnumDefinition_Name_set, METH_VARARGS, NULL},
	 { "EnumDefinition_Name_get", _wrap_EnumDefinition_Name_get, METH_O, NULL},
	 { "EnumDefinition_DocString_set", _wrap_EnumDefinition_DocString_set, METH_VARARGS, NULL},
	 { "EnumDefinition_DocString_get", _wrap_EnumDefinition_DocString_get, METH_O, NULL},
	 { "EnumDefinition_Values_set", _wrap_EnumDefinition_Values_set, METH_VARARGS, NULL},
	 { "EnumDefinition_Values_get", _wrap_EnumDefinition_Values_get, METH_O, NULL},
	 { "EnumDefinition_ParseInfo_set", _wrap_EnumDefinition_ParseInfo_set, METH_VARARGS, NULL},
	 { "EnumDefinition_ParseInfo_get", _wrap_EnumDefinition_ParseInfo_get, METH_O, NULL},
	 { "EnumDefinition__GetService", _wrap_EnumDefinition__GetService, METH_O, NULL},
	 { "EnumDefinition__SetService", _wrap_EnumDefinition__SetService, METH_VARARGS, NULL},
	 { "new_EnumDefinition", _wrap_new_EnumDefinition, METH_O, NULL},
	 { "EnumDefinition_ToString", _wrap_EnumDefinition_ToString, METH_O, NULL},
	 { "EnumDefinition_FromString", _wrap_EnumDefinition_FromString, METH_VARARGS, NULL},
	 { "EnumDefinition_VerifyValues", _wrap_EnumDefinition_VerifyValues, METH_O, NULL},
	 { "EnumDefinition_Reset", _wrap_EnumDefinition_Reset, METH_O, NULL},
	 { "EnumDefinition_swigregister", EnumDefinition_swigregister, METH_O, NULL},
	 { "EnumDefinition_swiginit", EnumDefinition_swiginit, METH_VARARGS, NULL},
	 { "new_EnumDefinitionValue", _wrap_new_EnumDefinitionValue, METH_NOARGS, NULL},
	 { "EnumDefinitionValue_Name_set", _wrap_EnumDefinitionValue_Name_set, METH_VARARGS, NULL},
	 { "EnumDefinitionValue_Name_get", _wrap_EnumDefinitionValue_Name_get, METH_O, NULL},
	 { "EnumDefinitionValue_DocString_set", _wrap_EnumDefinitionValue_DocString_set, METH_VARARGS, NULL},
	 { "EnumDefinitionValue_DocString_get", _wrap_EnumDefinitionValue_DocString_get, METH_O, NULL},
	 { "EnumDefinitionValue_Value_set", _wrap_EnumDefinitionValue_Value_set, METH_VARARGS, NULL},
	 { "EnumDefinitionValue_Value_get", _wrap_EnumDefinitionValue_Value_get, METH_O, NULL},
	 { "EnumDefinitionValue_ImplicitValue_set", _wrap_EnumDefinitionValue_ImplicitValue_set, METH_VARARGS, NULL},
	 { "EnumDefinitionValue_ImplicitValue_get", _wrap_EnumDefinitionValue_ImplicitValue_get, METH_O, NULL},
	 { "EnumDefinitionValue_HexValue_set", _wrap_EnumDefinitionValue_HexValue_set, METH_VARARGS, NULL},
	 { "EnumDefinitionValue_HexValue_get", _wrap_EnumDefinitionValue_HexValue_get, METH_O, NULL},
	 { "delete_EnumDefinitionValue", _wrap_delete_EnumDefinitionValue, METH_O, NULL},
	 { "EnumDefinitionValue_swigregister", EnumDefinitionValue_swigregister, METH_O, NULL},
	 { "EnumDefinitionValue_swiginit", EnumDefinitionValue_swiginit, METH_VARARGS, NULL},
	 { "delete_ExceptionDefinition", _wrap_delete_ExceptionDefinition, METH_O, NULL},
	 { "ExceptionDefinition_Name_set", _wrap_ExceptionDefinition_Name_set, METH_VARARGS, NULL},
	 { "ExceptionDefinition_Name_get", _wrap_ExceptionDefinition_Name_get, METH_O, NULL},
	 { "ExceptionDefinition_DocString_set", _wrap_ExceptionDefinition_DocString_set, METH_VARARGS, NULL},
	 { "ExceptionDefinition_DocString_get", _wrap_ExceptionDefinition_DocString_get, METH_O, NULL},
	 { "ExceptionDefinition_ParseInfo_set", _wrap_ExceptionDefinition_ParseInfo_set, METH_VARARGS, NULL},
	 { "ExceptionDefinition_ParseInfo_get", _wrap_ExceptionDefinition_ParseInfo_get, METH_O, NULL},
	 { "ExceptionDefinition__GetService", _wrap_ExceptionDefinition__GetService, METH_O, NULL},
	 { "ExceptionDefinition__SetService", _wrap_ExceptionDefinition__SetService, METH_VARARGS, NULL},
	 { "new_ExceptionDefinition", _wrap_new_ExceptionDefinition, METH_O, NULL},
	 { "ExceptionDefinition_ToString", _wrap_ExceptionDefinition_ToString, METH_O, NULL},
	 { "ExceptionDefinition_FromString", _wrap_ExceptionDefinition_FromString, METH_VARARGS, NULL},
	 { "ExceptionDefinition_Reset", _wrap_ExceptionDefinition_Reset, METH_O, NULL},
	 { "ExceptionDefinition_swigregister", ExceptionDefinition_swigregister, METH_O, NULL},
	 { "ExceptionDefinition_swiginit", ExceptionDefinition_swiginit, METH_VARARGS, NULL},
	 { "MemberDefinitionUtil_ToProperty", _wrap_MemberDefinitionUtil_ToProperty, METH_O, NULL},
	 { "MemberDefinitionUtil_ToFunction", _wrap_MemberDefinitionUtil_ToFunction, METH_O, NULL},
	 { "MemberDefinitionUtil_ToObjRef", _wrap_MemberDefinitionUtil_ToObjRef, METH_O, NULL},
	 { "MemberDefinitionUtil_ToEvent", _wrap_MemberDefinitionUtil_ToEvent, METH_O, NULL},
	 { "MemberDefinitionUtil_ToPipe", _wrap_MemberDefinitionUtil_ToPipe, METH_O, NULL},
	 { "MemberDefinitionUtil_ToCallback", _wrap_MemberDefinitionUtil_ToCallback, METH_O, NULL},
	 { "MemberDefinitionUtil_ToWire", _wrap_MemberDefinitionUtil_ToWire, METH_O, NULL},
	 { "MemberDefinitionUtil_ToMemory", _wrap_MemberDefinitionUtil_ToMemory, METH_O, NULL},
	 { "new_MemberDefinitionUtil", _wrap_new_MemberDefinitionUtil, METH_NOARGS, NULL},
	 { "delete_MemberDefinitionUtil", _wrap_delete_MemberDefinitionUtil, METH_O, NULL},
	 { "MemberDefinitionUtil_swigregister", MemberDefinitionUtil_swigregister, METH_O, NULL},
	 { "MemberDefinitionUtil_swiginit", MemberDefinitionUtil_swiginit, METH_VARARGS, NULL},
	 { "Transport_GetUrlSchemeString", _wrap_Transport_GetUrlSchemeString, METH_O, NULL},
	 { "Transport_GetCurrentTransportConnectionURL", _wrap_Transport_GetCurrentTransportConnectionURL, METH_NOARGS, NULL},
	 { "Transport_GetServerListenUrls", _wrap_Transport_GetServerListenUrls, METH_O, NULL},
	 { "delete_Transport", _wrap_delete_Transport, METH_O, NULL},
	 { "Transport_swigregister", Transport_swigregister, METH_O, NULL},
	 { "new_TcpTransport", _wrap_new_TcpTransport, METH_VARARGS, NULL},
	 { "TcpTransport__GetDefaultReceiveTimeout", _wrap_TcpTransport__GetDefaultReceiveTimeout, METH_O, NULL},
	 { "TcpTransport__SetDefaultReceiveTimeout", _wrap_TcpTransport__SetDefaultReceiveTimeout, METH_VARARGS, NULL},
	 { "TcpTransport__GetDefaultConnectTimeout", _wrap_TcpTransport__GetDefaultConnectTimeout, METH_O, NULL},
	 { "TcpTransport__SetDefaultConnectTimeout", _wrap_TcpTransport__SetDefaultConnectTimeout, METH_VARARGS, NULL},
	 { "TcpTransport__GetDefaultHeartbeatPeriod", _wrap_TcpTransport__GetDefaultHeartbeatPeriod, METH_O, NULL},
	 { "TcpTransport__SetDefaultHeartbeatPeriod", _wrap_TcpTransport__SetDefaultHeartbeatPeriod, METH_VARARGS, NULL},
	 { "TcpTransport_GetUrlSchemeString", _wrap_TcpTransport_GetUrlSchemeString, METH_O, NULL},
	 { "TcpTransport_GetServerListenUrls", _wrap_TcpTransport_GetServerListenUrls, METH_O, NULL},
	 { "TcpTransport_GetListenPort", _wrap_TcpTransport_GetListenPort, METH_O, NULL},
	 { "TcpTransport_StartServer", _wrap_TcpTransport_StartServer, METH_VARARGS, NULL},
	 { "TcpTransport_Close", _wrap_TcpTransport_Close, METH_O, "\n"
		"\n"
		"Close the transport. Done automatically by node shutdown.\n"
		"\n"
		""},
	 { "TcpTransport_EnableNodeDiscoveryListening", _wrap_TcpTransport_EnableNodeDiscoveryListening, METH_VARARGS, "\n"
		"\n"
		"Enable node discovery listening\n"
		"\n"
		"By default enables listining on IPv6 link-local scope\n"
		"\n"
		"See IPNodeDiscoveryFlags constants\n"
		"\n"
		":param flags: The flags specifying the scope\n"
		":type flags: int\n"
		"\n"
		""},
	 { "TcpTransport_DisableNodeDiscoveryListening", _wrap_TcpTransport_DisableNodeDiscoveryListening, METH_O, "\n"
		"\n"
		"Disable node discovery listening\n"
		"\n"
		""},
	 { "TcpTransport_EnableNodeAnnounce", _wrap_TcpTransport_EnableNodeAnnounce, METH_VARARGS, "\n"
		"\n"
		"Enable node discovery announce\n"
		"\n"
		"By default enables announce on IPv6 link-local scope\n"
		"\n"
		"See IPNodeDiscoveryFlags constants\n"
		"\n"
		":param flags: The flags specifying the scope\n"
		":type flags: int\n"
		"\n"
		""},
	 { "TcpTransport_DisableNodeAnnounce", _wrap_TcpTransport_DisableNodeAnnounce, METH_O, "\n"
		"\n"
		"Disable node discovery announce\n"
		"\n"
		""},
	 { "TcpTransport__GetNodeAnnouncePeriod", _wrap_TcpTransport__GetNodeAnnouncePeriod, METH_O, NULL},
	 { "TcpTransport__SetNodeAnnouncePeriod", _wrap_TcpTransport__SetNodeAnnouncePeriod, METH_VARARGS, NULL},
	 { "TcpTransport__GetMaxMessageSize", _wrap_TcpTransport__GetMaxMessageSize, METH_O, NULL},
	 { "TcpTransport__SetMaxMessageSize", _wrap_TcpTransport__SetMaxMessageSize, METH_VARARGS, NULL},
	 { "TcpTransport__GetMaxConnectionCount", _wrap_TcpTransport__GetMaxConnectionCount, METH_O, NULL},
	 { "TcpTransport__SetMaxConnectionCount", _wrap_TcpTransport__SetMaxConnectionCount, METH_VARARGS, NULL},
	 { "TcpTransport__GetRequireTls", _wrap_TcpTransport__GetRequireTls, METH_O, NULL},
	 { "TcpTransport__SetRequireTls", _wrap_TcpTransport__SetRequireTls, METH_VARARGS, NULL},
	 { "TcpTransport_LoadTlsNodeCertificate", _wrap_TcpTransport_LoadTlsNodeCertificate, METH_O, NULL},
	 { "TcpTransport__IsTlsNodeCertificateLoaded", _wrap_TcpTransport__IsTlsNodeCertificateLoaded, METH_O, NULL},
	 { "TcpTransport__IsTransportConnectionSecure", _wrap_TcpTransport__IsTransportConnectionSecure, METH_VARARGS, NULL},
	 { "TcpTransport__IsSecurePeerIdentityVerified", _wrap_TcpTransport__IsSecurePeerIdentityVerified, METH_VARARGS, NULL},
	 { "TcpTransport__GetSecurePeerIdentity", _wrap_TcpTransport__GetSecurePeerIdentity, METH_VARARGS, NULL},
	 { "TcpTransport_StartServerUsingPortSharer", _wrap_TcpTransport_StartServerUsingPortSharer, METH_O, NULL},
	 { "TcpTransport__IsPortSharerRunning", _wrap_TcpTransport__IsPortSharerRunning, METH_O, NULL},
	 { "TcpTransport__GetAcceptWebSockets", _wrap_TcpTransport__GetAcceptWebSockets, METH_O, NULL},
	 { "TcpTransport__SetAcceptWebSockets", _wrap_TcpTransport__SetAcceptWebSockets, METH_VARARGS, NULL},
	 { "TcpTransport_GetWebSocketAllowedOrigins", _wrap_TcpTransport_GetWebSocketAllowedOrigins, METH_O, NULL},
	 { "TcpTransport_AddWebSocketAllowedOrigin", _wrap_TcpTransport_AddWebSocketAllowedOrigin, METH_VARARGS, NULL},
	 { "TcpTransport_RemoveWebSocketAllowedOrigin", _wrap_TcpTransport_RemoveWebSocketAllowedOrigin, METH_VARARGS, NULL},
	 { "TcpTransport__GetDisableMessage4", _wrap_TcpTransport__GetDisableMessage4, METH_O, NULL},
	 { "TcpTransport__SetDisableMessage4", _wrap_TcpTransport__SetDisableMessage4, METH_VARARGS, NULL},
	 { "TcpTransport__GetDisableStringTable", _wrap_TcpTransport__GetDisableStringTable, METH_O, NULL},
	 { "TcpTransport__SetDisableStringTable", _wrap_TcpTransport__SetDisableStringTable, METH_VARARGS, NULL},
	 { "TcpTransport__GetDisableAsyncMessageIO", _wrap_TcpTransport__GetDisableAsyncMessageIO, METH_O, NULL},
	 { "TcpTransport__SetDisableAsyncMessageIO", _wrap_TcpTransport__SetDisableAsyncMessageIO, METH_VARARGS, NULL},
	 { "TcpTransport_GetLocalAdapterIPAddresses", _wrap_TcpTransport_GetLocalAdapterIPAddresses, METH_NOARGS, NULL},
	 { "delete_TcpTransport", _wrap_delete_TcpTransport, METH_O, NULL},
	 { "TcpTransport_swigregister", TcpTransport_swigregister, METH_O, NULL},
	 { "TcpTransport_swiginit", TcpTransport_swiginit, METH_VARARGS, NULL},
	 { "new_LocalTransport", _wrap_new_LocalTransport, METH_VARARGS, NULL},
	 { "LocalTransport_IsLocalTransportSupported", _wrap_LocalTransport_IsLocalTransportSupported, METH_NOARGS, "\n"
		"\n"
		"Check if the LocalTransport is supported on the current\n"
		"operating system.\n"
		"\n"
		"Windows versions before Windows 10 1803 do not support the LocalTransport\n"
		"due to lack of UNIX sockets. All other transports will continue to operate\n"
		"normally.\n"
		"\n"
		"All versions of Linux and Mac OSX support the LocalTransport\n"
		"\n"
		"LocalTransport on Android and iOS is not officially supported\n"
		"\n"
		":return: True LocalTransport is supported, otherwise False\n"
		":rtype: bool\n"
		"\n"
		""},
	 { "LocalTransport_GetUrlSchemeString", _wrap_LocalTransport_GetUrlSchemeString, METH_O, NULL},
	 { "LocalTransport_GetServerListenUrls", _wrap_LocalTransport_GetServerListenUrls, METH_O, NULL},
	 { "LocalTransport_StartClientAsNodeName", _wrap_LocalTransport_StartClientAsNodeName, METH_VARARGS, "\n"
		"\n"
		"Initialize the LocalTransport by assigning a NodeID based on NodeName\n"
		"\n"
		"Assigns the specified name to be the NodeName of the node, and manages\n"
		"a corresponding NodeID. See LocalTransport for more information.\n"
		"\n"
		"Throws NodeNameAlreadyInUse if another node is using name\n"
		"\n"
		":param name: The node name\n"
		":type name: str\n"
		"\n"
		""},
	 { "LocalTransport_StartServerAsNodeName", _wrap_LocalTransport_StartServerAsNodeName, METH_VARARGS, NULL},
	 { "LocalTransport_StartServerAsNodeID", _wrap_LocalTransport_StartServerAsNodeID, METH_VARARGS, NULL},
	 { "LocalTransport_Close", _wrap_LocalTransport_Close, METH_O, "\n"
		"\n"
		"Close the transport. Done automatically by node shutdown.\n"
		"\n"
		""},
	 { "LocalTransport__GetMaxMessageSize", _wrap_LocalTransport__GetMaxMessageSize, METH_O, NULL},
	 { "LocalTransport__SetMaxMessageSize", _wrap_LocalTransport__SetMaxMessageSize, METH_VARARGS, NULL},
	 { "LocalTransport__GetDisableMessage4", _wrap_LocalTransport__GetDisableMessage4, METH_O, NULL},
	 { "LocalTransport__SetDisableMessage4", _wrap_LocalTransport__SetDisableMessage4, METH_VARARGS, NULL},
	 { "LocalTransport__GetDisableStringTable", _wrap_LocalTransport__GetDisableStringTable, METH_O, NULL},
	 { "LocalTransport__SetDisableStringTable", _wrap_LocalTransport__SetDisableStringTable, METH_VARARGS, NULL},
	 { "LocalTransport__GetDisableAsyncMessageIO", _wrap_LocalTransport__GetDisableAsyncMessageIO, METH_O, NULL},
	 { "LocalTransport__SetDisableAsyncMessageIO", _wrap_LocalTransport__SetDisableAsyncMessageIO, METH_VARARGS, NULL},
	 { "LocalTransport_EnableNodeDiscoveryListening", _wrap_LocalTransport_EnableNodeDiscoveryListening, METH_O, NULL},
	 { "LocalTransport_DisableNodeDiscoveryListening", _wrap_LocalTransport_DisableNodeDiscoveryListening, METH_O, NULL},
	 { "delete_LocalTransport", _wrap_delete_LocalTransport, METH_O, NULL},
	 { "LocalTransport_swigregister", LocalTransport_swigregister, METH_O, NULL},
	 { "LocalTransport_swiginit", LocalTransport_swiginit, METH_VARARGS, NULL},
	 { "new_HardwareTransport", _wrap_new_HardwareTransport, METH_VARARGS, NULL},
	 { "HardwareTransport_GetUrlSchemeString", _wrap_HardwareTransport_GetUrlSchemeString, METH_O, NULL},
	 { "HardwareTransport_GetServerListenUrls", _wrap_HardwareTransport_GetServerListenUrls, METH_O, NULL},
	 { "HardwareTransport_Close", _wrap_HardwareTransport_Close, METH_O, "\n"
		"\n"
		"Close the transport. Done automatically by node shutdown.\n"
		"\n"
		""},
	 { "HardwareTransport__GetMaxMessageSize", _wrap_HardwareTransport__GetMaxMessageSize, METH_O, NULL},
	 { "HardwareTransport__SetMaxMessageSize", _wrap_HardwareTransport__SetMaxMessageSize, METH_VARARGS, NULL},
	 { "HardwareTransport__GetDisableMessage4", _wrap_HardwareTransport__GetDisableMessage4, METH_O, NULL},
	 { "HardwareTransport__SetDisableMessage4", _wrap_HardwareTransport__SetDisableMessage4, METH_VARARGS, NULL},
	 { "HardwareTransport__GetDisableStringTable", _wrap_HardwareTransport__GetDisableStringTable, METH_O, NULL},
	 { "HardwareTransport__SetDisableStringTable", _wrap_HardwareTransport__SetDisableStringTable, METH_VARARGS, NULL},
	 { "HardwareTransport__GetDisableAsyncMessageIO", _wrap_HardwareTransport__GetDisableAsyncMessageIO, METH_O, NULL},
	 { "HardwareTransport__SetDisableAsyncMessageIO", _wrap_HardwareTransport__SetDisableAsyncMessageIO, METH_VARARGS, NULL},
	 { "delete_HardwareTransport", _wrap_delete_HardwareTransport, METH_O, NULL},
	 { "HardwareTransport_swigregister", HardwareTransport_swigregister, METH_O, NULL},
	 { "HardwareTransport_swiginit", HardwareTransport_swiginit, METH_VARARGS, NULL},
	 { "new_IntraTransport", _wrap_new_IntraTransport, METH_VARARGS, NULL},
	 { "IntraTransport_GetUrlSchemeString", _wrap_IntraTransport_GetUrlSchemeString, METH_O, NULL},
	 { "IntraTransport_GetServerListenUrls", _wrap_IntraTransport_GetServerListenUrls, METH_O, NULL},
	 { "IntraTransport_Close", _wrap_IntraTransport_Close, METH_O, "\n"
		"\n"
		"Close the transport. Done automatically by node shutdown.\n"
		"\n"
		""},
	 { "IntraTransport_StartClient", _wrap_IntraTransport_StartClient, METH_O, "Start the transport as a client"},
	 { "IntraTransport_StartServer", _wrap_IntraTransport_StartServer, METH_O, "Start the server to listen for incoming client connections"},
	 { "delete_IntraTransport", _wrap_delete_IntraTransport, METH_O, NULL},
	 { "IntraTransport_swigregister", IntraTransport_swigregister, METH_O, NULL},
	 { "IntraTransport_swiginit", IntraTransport_swiginit, METH_VARARGS, NULL},
	 { "TimerEvent_stopped_get", _wrap_TimerEvent_stopped_get, METH_O, "\n"
		"\n"
		"True if timer has been stopped\n"
		"\n"
		":rtype: bool\n"
		"\n"
		""},
	 { "TimerEvent_last_expected_get", _wrap_TimerEvent_last_expected_get, METH_O, "\n"
		"\n"
		"The last expected callback invocation time\n"
		"\n"
		":rtype: datetime.datetime\n"
		"\n"
		""},
	 { "TimerEvent_last_real_get", _wrap_TimerEvent_last_real_get, METH_O, "\n"
		"\n"
		"The real last callback invocation time\n"
		"\n"
		":rtype: datetime.datetime\n"
		"\n"
		""},
	 { "TimerEvent_current_expected_get", _wrap_TimerEvent_current_expected_get, METH_O, "\n"
		"\n"
		"The current expected invocation time\n"
		"\n"
		":rtype: datetime.datetime\n"
		"\n"
		""},
	 { "TimerEvent_current_real_get", _wrap_TimerEvent_current_real_get, METH_O, "\n"
		"\n"
		"The current invocation time\n"
		"\n"
		":rtype: datetime.datetime\n"
		"\n"
		""},
	 { "new_TimerEvent", _wrap_new_TimerEvent, METH_NOARGS, NULL},
	 { "delete_TimerEvent", _wrap_delete_TimerEvent, METH_O, NULL},
	 { "TimerEvent_swigregister", TimerEvent_swigregister, METH_O, NULL},
	 { "TimerEvent_swiginit", TimerEvent_swiginit, METH_VARARGS, NULL},
	 { "delete_AsyncTimerEventReturnDirector", _wrap_delete_AsyncTimerEventReturnDirector, METH_O, NULL},
	 { "AsyncTimerEventReturnDirector_handler", _wrap_AsyncTimerEventReturnDirector_handler, METH_VARARGS, NULL},
	 { "new_AsyncTimerEventReturnDirector", _wrap_new_AsyncTimerEventReturnDirector, METH_O, NULL},
	 { "disown_AsyncTimerEventReturnDirector", _wrap_disown_AsyncTimerEventReturnDirector, METH_O, NULL},
	 { "AsyncTimerEventReturnDirector_swigregister", AsyncTimerEventReturnDirector_swigregister, METH_O, NULL},
	 { "AsyncTimerEventReturnDirector_swiginit", AsyncTimerEventReturnDirector_swiginit, METH_VARARGS, NULL},
	 { "Timer_Start", _wrap_Timer_Start, METH_O, "\n"
		"\n"
		"Start the timer\n"
		"\n"
		"Must be called after RobotRaconteurNode.CreateTimer()\n"
		"\n"
		""},
	 { "Timer_Stop", _wrap_Timer_Stop, METH_O, "\n"
		"\n"
		"Stop the timer\n"
		"\n"
		""},
	 { "Timer_TryStop", _wrap_Timer_TryStop, METH_O, "\n"
		"\n"
		"Stop the timer without throwing an exception if the timer is not running\n"
		"\n"
		""},
	 { "Timer_GetPeriod", _wrap_Timer_GetPeriod, METH_O, "\n"
		"\n"
		"Get the period of the timer in seconds\n"
		"\n"
		":rtype: float\n"
		"\n"
		""},
	 { "Timer_SetPeriod", _wrap_Timer_SetPeriod, METH_VARARGS, "\n"
		"\n"
		"Set the period of the timer in seconds\n"
		"\n"
		":param period: Period in seconds\n"
		":type period: float\n"
		"\n"
		""},
	 { "Timer_IsRunning", _wrap_Timer_IsRunning, METH_O, "\n"
		"\n"
		"Get if the timer is running\n"
		"\n"
		":rtype: bool\n"
		"\n"
		""},
	 { "delete_Timer", _wrap_delete_Timer, METH_O, NULL},
	 { "Timer_swigregister", Timer_swigregister, METH_O, NULL},
	 { "WallTimer_Start", _wrap_WallTimer_Start, METH_O, "\n"
		"\n"
		"Start the timer\n"
		"\n"
		"Must be called after RobotRaconteurNode.CreateTimer()\n"
		"\n"
		""},
	 { "WallTimer_Stop", _wrap_WallTimer_Stop, METH_O, "\n"
		"\n"
		"Stop the timer\n"
		"\n"
		""},
	 { "WallTimer_TryStop", _wrap_WallTimer_TryStop, METH_O, "\n"
		"\n"
		"Stop the timer without throwing an exception if the timer is not running\n"
		"\n"
		""},
	 { "WallTimer_GetPeriod", _wrap_WallTimer_GetPeriod, METH_O, "\n"
		"\n"
		"Get the period of the timer in seconds\n"
		"\n"
		":rtype: float\n"
		"\n"
		""},
	 { "WallTimer_SetPeriod", _wrap_WallTimer_SetPeriod, METH_VARARGS, "\n"
		"\n"
		"Set the period of the timer in seconds\n"
		"\n"
		":param period: Period in seconds\n"
		":type period: float\n"
		"\n"
		""},
	 { "WallTimer_IsRunning", _wrap_WallTimer_IsRunning, METH_O, "\n"
		"\n"
		"Get if the timer is running\n"
		"\n"
		":rtype: bool\n"
		"\n"
		""},
	 { "delete_WallTimer", _wrap_delete_WallTimer, METH_O, NULL},
	 { "new_WallTimer", _wrap_new_WallTimer, METH_VARARGS, NULL},
	 { "WallTimer_swigregister", WallTimer_swigregister, METH_O, NULL},
	 { "WallTimer_swiginit", WallTimer_swiginit, METH_VARARGS, NULL},
	 { "Rate_Sleep", _wrap_Rate_Sleep, METH_O, "\n"
		"\n"
		"Sleep the calling thread until the current loop period expires\n"
		"\n"
		""},
	 { "delete_Rate", _wrap_delete_Rate, METH_O, NULL},
	 { "Rate_swigregister", Rate_swigregister, METH_O, NULL},
	 { "new_WallRate", _wrap_new_WallRate, METH_VARARGS, NULL},
	 { "WallRate_Sleep", _wrap_WallRate_Sleep, METH_O, "\n"
		"\n"
		"Sleep the calling thread until the current loop period expires\n"
		"\n"
		""},
	 { "delete_WallRate", _wrap_delete_WallRate, METH_O, NULL},
	 { "WallRate_swigregister", WallRate_swigregister, METH_O, NULL},
	 { "WallRate_swiginit", WallRate_swiginit, METH_VARARGS, NULL},
	 { "new_AutoResetEvent", _wrap_new_AutoResetEvent, METH_NOARGS, NULL},
	 { "delete_AutoResetEvent", _wrap_delete_AutoResetEvent, METH_O, NULL},
	 { "AutoResetEvent_Set", _wrap_AutoResetEvent_Set, METH_O, "\n"
		"\n"
		"Set the event, releasing waiting threads\n"
		"\n"
		""},
	 { "AutoResetEvent_Reset", _wrap_AutoResetEvent_Reset, METH_O, "\n"
		"\n"
		"Reset the event\n"
		"\n"
		""},
	 { "AutoResetEvent_WaitOne", _wrap_AutoResetEvent_WaitOne, METH_VARARGS, NULL},
	 { "AutoResetEvent_swigregister", AutoResetEvent_swigregister, METH_O, NULL},
	 { "AutoResetEvent_swiginit", AutoResetEvent_swiginit, METH_VARARGS, NULL},
	 { "TimeSpec_seconds_set", _wrap_TimeSpec_seconds_set, METH_VARARGS, "\n"
		"\n"
		"Seconds since epoch\n"
		"\n"
		""},
	 { "TimeSpec_seconds_get", _wrap_TimeSpec_seconds_get, METH_O, "\n"
		"\n"
		"Seconds since epoch\n"
		"\n"
		""},
	 { "TimeSpec_nanoseconds_set", _wrap_TimeSpec_nanoseconds_set, METH_VARARGS, "\n"
		"\n"
		"Nanoseconds from epoch. Normalized to be between 0 and 1e9-1\n"
		"\n"
		""},
	 { "TimeSpec_nanoseconds_get", _wrap_TimeSpec_nanoseconds_get, METH_O, "\n"
		"\n"
		"Nanoseconds from epoch. Normalized to be between 0 and 1e9-1\n"
		"\n"
		""},
	 { "new_TimeSpec", _wrap_new_TimeSpec, METH_VARARGS, NULL},
	 { "TimeSpec___eq__", _wrap_TimeSpec___eq__, METH_VARARGS, "equality comparison"},
	 { "TimeSpec___ne__", _wrap_TimeSpec___ne__, METH_VARARGS, "inequality comparison"},
	 { "TimeSpec___sub__", _wrap_TimeSpec___sub__, METH_VARARGS, "subtraction operator"},
	 { "TimeSpec___add__", _wrap_TimeSpec___add__, METH_VARARGS, "addition operator"},
	 { "TimeSpec___gt__", _wrap_TimeSpec___gt__, METH_VARARGS, "greater-than comparison"},
	 { "TimeSpec___ge__", _wrap_TimeSpec___ge__, METH_VARARGS, "greater-than-or-equal comparison"},
	 { "TimeSpec___lt__", _wrap_TimeSpec___lt__, METH_VARARGS, "less-then comparison"},
	 { "TimeSpec___le__", _wrap_TimeSpec___le__, METH_VARARGS, "less-than-or-equal comparison"},
	 { "TimeSpec_cleanup_nanosecs", _wrap_TimeSpec_cleanup_nanosecs, METH_O, "normalize nanoseconds to be within 0 and 1e9-1"},
	 { "delete_TimeSpec", _wrap_delete_TimeSpec, METH_O, NULL},
	 { "TimeSpec_swigregister", TimeSpec_swigregister, METH_O, NULL},
	 { "TimeSpec_swiginit", TimeSpec_swiginit, METH_VARARGS, NULL},
	 { "WrappedService_typed_packet_packet_set", _wrap_WrappedService_typed_packet_packet_set, METH_VARARGS, NULL},
	 { "WrappedService_typed_packet_packet_get", _wrap_WrappedService_typed_packet_packet_get, METH_O, NULL},
	 { "WrappedService_typed_packet_type_set", _wrap_WrappedService_typed_packet_type_set, METH_VARARGS, NULL},
	 { "WrappedService_typed_packet_type_get", _wrap_WrappedService_typed_packet_type_get, METH_O, NULL},
	 { "WrappedService_typed_packet_stub_set", _wrap_WrappedService_typed_packet_stub_set, METH_VARARGS, NULL},
	 { "WrappedService_typed_packet_stub_get", _wrap_WrappedService_typed_packet_stub_get, METH_O, NULL},
	 { "WrappedService_typed_packet_client_set", _wrap_WrappedService_typed_packet_client_set, METH_VARARGS, NULL},
	 { "WrappedService_typed_packet_client_get", _wrap_WrappedService_typed_packet_client_get, METH_O, NULL},
	 { "new_WrappedService_typed_packet", _wrap_new_WrappedService_typed_packet, METH_NOARGS, NULL},
	 { "delete_WrappedService_typed_packet", _wrap_delete_WrappedService_typed_packet, METH_O, NULL},
	 { "WrappedService_typed_packet_swigregister", WrappedService_typed_packet_swigregister, METH_O, NULL},
	 { "WrappedService_typed_packet_swiginit", WrappedService_typed_packet_swiginit, METH_VARARGS, NULL},
	 { "delete_WrappedPipeEndpointDirector", _wrap_delete_WrappedPipeEndpointDirector, METH_O, NULL},
	 { "WrappedPipeEndpointDirector_PipeEndpointClosedCallback", _wrap_WrappedPipeEndpointDirector_PipeEndpointClosedCallback, METH_O, NULL},
	 { "WrappedPipeEndpointDirector_PacketReceivedEvent", _wrap_WrappedPipeEndpointDirector_PacketReceivedEvent, METH_O, NULL},
	 { "WrappedPipeEndpointDirector_PacketAckReceivedEvent", _wrap_WrappedPipeEndpointDirector_PacketAckReceivedEvent, METH_VARARGS, NULL},
	 { "new_WrappedPipeEndpointDirector", _wrap_new_WrappedPipeEndpointDirector, METH_O, NULL},
	 { "disown_WrappedPipeEndpointDirector", _wrap_disown_WrappedPipeEndpointDirector, METH_O, NULL},
	 { "WrappedPipeEndpointDirector_swigregister", WrappedPipeEndpointDirector_swigregister, METH_O, NULL},
	 { "WrappedPipeEndpointDirector_swiginit", WrappedPipeEndpointDirector_swiginit, METH_VARARGS, NULL},
	 { "delete_AsyncPipeEndpointReturnDirector", _wrap_delete_AsyncPipeEndpointReturnDirector, METH_O, NULL},
	 { "AsyncPipeEndpointReturnDirector_handler", _wrap_AsyncPipeEndpointReturnDirector_handler, METH_VARARGS, NULL},
	 { "new_AsyncPipeEndpointReturnDirector", _wrap_new_AsyncPipeEndpointReturnDirector, METH_O, NULL},
	 { "disown_AsyncPipeEndpointReturnDirector", _wrap_disown_AsyncPipeEndpointReturnDirector, METH_O, NULL},
	 { "AsyncPipeEndpointReturnDirector_swigregister", AsyncPipeEndpointReturnDirector_swigregister, METH_O, NULL},
	 { "AsyncPipeEndpointReturnDirector_swiginit", AsyncPipeEndpointReturnDirector_swiginit, METH_VARARGS, NULL},
	 { "WrappedTryReceivePacketWaitResult_res_set", _wrap_WrappedTryReceivePacketWaitResult_res_set, METH_VARARGS, NULL},
	 { "WrappedTryReceivePacketWaitResult_res_get", _wrap_WrappedTryReceivePacketWaitResult_res_get, METH_O, NULL},
	 { "WrappedTryReceivePacketWaitResult_packet_set", _wrap_WrappedTryReceivePacketWaitResult_packet_set, METH_VARARGS, NULL},
	 { "WrappedTryReceivePacketWaitResult_packet_get", _wrap_WrappedTryReceivePacketWaitResult_packet_get, METH_O, NULL},
	 { "new_WrappedTryReceivePacketWaitResult", _wrap_new_WrappedTryReceivePacketWaitResult, METH_NOARGS, NULL},
	 { "delete_WrappedTryReceivePacketWaitResult", _wrap_delete_WrappedTryReceivePacketWaitResult, METH_O, NULL},
	 { "WrappedTryReceivePacketWaitResult_swigregister", WrappedTryReceivePacketWaitResult_swigregister, METH_O, NULL},
	 { "WrappedTryReceivePacketWaitResult_swiginit", WrappedTryReceivePacketWaitResult_swiginit, METH_VARARGS, NULL},
	 { "WrappedPipeEndpoint_SendPacket", _wrap_WrappedPipeEndpoint_SendPacket, METH_VARARGS, NULL},
	 { "WrappedPipeEndpoint_ReceivePacket", _wrap_WrappedPipeEndpoint_ReceivePacket, METH_O, NULL},
	 { "WrappedPipeEndpoint_PeekNextPacket", _wrap_WrappedPipeEndpoint_PeekNextPacket, METH_O, NULL},
	 { "WrappedPipeEndpoint_ReceivePacketWait", _wrap_WrappedPipeEndpoint_ReceivePacketWait, METH_VARARGS, NULL},
	 { "WrappedPipeEndpoint_PeekNextPacketWait", _wrap_WrappedPipeEndpoint_PeekNextPacketWait, METH_VARARGS, NULL},
	 { "WrappedPipeEndpoint_TryReceivePacketWait", _wrap_WrappedPipeEndpoint_TryReceivePacketWait, METH_VARARGS, NULL},
	 { "WrappedPipeEndpoint_Close", _wrap_WrappedPipeEndpoint_Close, METH_O, NULL},
	 { "WrappedPipeEndpoint_GetIndex", _wrap_WrappedPipeEndpoint_GetIndex, METH_O, NULL},
	 { "WrappedPipeEndpoint_GetEndpoint", _wrap_WrappedPipeEndpoint_GetEndpoint, METH_O, NULL},
	 { "WrappedPipeEndpoint_Available", _wrap_WrappedPipeEndpoint_Available, METH_O, NULL},
	 { "WrappedPipeEndpoint_GetRequestPacketAck", _wrap_WrappedPipeEndpoint_GetRequestPacketAck, METH_O, NULL},
	 { "WrappedPipeEndpoint_SetRequestPacketAck", _wrap_WrappedPipeEndpoint_SetRequestPacketAck, METH_VARARGS, NULL},
	 { "WrappedPipeEndpoint_Type_set", _wrap_WrappedPipeEndpoint_Type_set, METH_VARARGS, NULL},
	 { "WrappedPipeEndpoint_Type_get", _wrap_WrappedPipeEndpoint_Type_get, METH_O, NULL},
	 { "WrappedPipeEndpoint_SetRRDirector", _wrap_WrappedPipeEndpoint_SetRRDirector, METH_VARARGS, NULL},
	 { "WrappedPipeEndpoint_IsUnreliable", _wrap_WrappedPipeEndpoint_IsUnreliable, METH_O, NULL},
	 { "WrappedPipeEndpoint_Direction", _wrap_WrappedPipeEndpoint_Direction, METH_O, NULL},
	 { "WrappedPipeEndpoint_AsyncSendPacket", _wrap_WrappedPipeEndpoint_AsyncSendPacket, METH_VARARGS, NULL},
	 { "WrappedPipeEndpoint_AsyncClose", _wrap_WrappedPipeEndpoint_AsyncClose, METH_VARARGS, NULL},
	 { "WrappedPipeEndpoint_GetNode", _wrap_WrappedPipeEndpoint_GetNode, METH_O, NULL},
	 { "WrappedPipeEndpoint_GetIgnoreReceived", _wrap_WrappedPipeEndpoint_GetIgnoreReceived, METH_O, NULL},
	 { "WrappedPipeEndpoint_SetIgnoreReceived", _wrap_WrappedPipeEndpoint_SetIgnoreReceived, METH_VARARGS, NULL},
	 { "delete_WrappedPipeEndpoint", _wrap_delete_WrappedPipeEndpoint, METH_O, NULL},
	 { "WrappedPipeEndpoint_swigregister", WrappedPipeEndpoint_swigregister, METH_O, NULL},
	 { "WrappedPipeClient_Connect", _wrap_WrappedPipeClient_Connect, METH_VARARGS, NULL},
	 { "WrappedPipeClient_AsyncConnect", _wrap_WrappedPipeClient_AsyncConnect, METH_VARARGS, NULL},
	 { "WrappedPipeClient_GetMemberName", _wrap_WrappedPipeClient_GetMemberName, METH_O, NULL},
	 { "WrappedPipeClient_Type_set", _wrap_WrappedPipeClient_Type_set, METH_VARARGS, NULL},
	 { "WrappedPipeClient_Type_get", _wrap_WrappedPipeClient_Type_get, METH_O, NULL},
	 { "WrappedPipeClient_GetNode", _wrap_WrappedPipeClient_GetNode, METH_O, NULL},
	 { "WrappedPipeClient_Direction", _wrap_WrappedPipeClient_Direction, METH_O, NULL},
	 { "delete_WrappedPipeClient", _wrap_delete_WrappedPipeClient, METH_O, NULL},
	 { "WrappedPipeClient_swigregister", WrappedPipeClient_swigregister, METH_O, NULL},
	 { "delete_WrappedPipeServerConnectDirector", _wrap_delete_WrappedPipeServerConnectDirector, METH_O, NULL},
	 { "WrappedPipeServerConnectDirector_PipeConnectCallback", _wrap_WrappedPipeServerConnectDirector_PipeConnectCallback, METH_VARARGS, NULL},
	 { "new_WrappedPipeServerConnectDirector", _wrap_new_WrappedPipeServerConnectDirector, METH_O, NULL},
	 { "disown_WrappedPipeServerConnectDirector", _wrap_disown_WrappedPipeServerConnectDirector, METH_O, NULL},
	 { "WrappedPipeServerConnectDirector_swigregister", WrappedPipeServerConnectDirector_swigregister, METH_O, NULL},
	 { "WrappedPipeServerConnectDirector_swiginit", WrappedPipeServerConnectDirector_swiginit, METH_VARARGS, NULL},
	 { "WrappedPipeServer_GetMemberName", _wrap_WrappedPipeServer_GetMemberName, METH_O, NULL},
	 { "WrappedPipeServer_Type_set", _wrap_WrappedPipeServer_Type_set, METH_VARARGS, NULL},
	 { "WrappedPipeServer_Type_get", _wrap_WrappedPipeServer_Type_get, METH_O, NULL},
	 { "WrappedPipeServer_SetWrappedPipeConnectCallback", _wrap_WrappedPipeServer_SetWrappedPipeConnectCallback, METH_VARARGS, NULL},
	 { "WrappedPipeServer_GetNode", _wrap_WrappedPipeServer_GetNode, METH_O, NULL},
	 { "WrappedPipeServer_Direction", _wrap_WrappedPipeServer_Direction, METH_O, NULL},
	 { "delete_WrappedPipeServer", _wrap_delete_WrappedPipeServer, METH_O, NULL},
	 { "WrappedPipeServer_swigregister", WrappedPipeServer_swigregister, METH_O, NULL},
	 { "WrappedPipeBroadcasterPredicateDirector_Predicate", _wrap_WrappedPipeBroadcasterPredicateDirector_Predicate, METH_VARARGS, NULL},
	 { "delete_WrappedPipeBroadcasterPredicateDirector", _wrap_delete_WrappedPipeBroadcasterPredicateDirector, METH_O, NULL},
	 { "new_WrappedPipeBroadcasterPredicateDirector", _wrap_new_WrappedPipeBroadcasterPredicateDirector, METH_O, NULL},
	 { "disown_WrappedPipeBroadcasterPredicateDirector", _wrap_disown_WrappedPipeBroadcasterPredicateDirector, METH_O, NULL},
	 { "WrappedPipeBroadcasterPredicateDirector_swigregister", WrappedPipeBroadcasterPredicateDirector_swigregister, METH_O, NULL},
	 { "WrappedPipeBroadcasterPredicateDirector_swiginit", WrappedPipeBroadcasterPredicateDirector_swiginit, METH_VARARGS, NULL},
	 { "WrappedPipeBroadcaster_Init", _wrap_WrappedPipeBroadcaster_Init, METH_VARARGS, NULL},
	 { "WrappedPipeBroadcaster_SendPacket", _wrap_WrappedPipeBroadcaster_SendPacket, METH_VARARGS, NULL},
	 { "WrappedPipeBroadcaster_AsyncSendPacket", _wrap_WrappedPipeBroadcaster_AsyncSendPacket, METH_VARARGS, NULL},
	 { "WrappedPipeBroadcaster_GetActivePipeEndpointCount", _wrap_WrappedPipeBroadcaster_GetActivePipeEndpointCount, METH_O, NULL},
	 { "WrappedPipeBroadcaster_SetPredicateDirector", _wrap_WrappedPipeBroadcaster_SetPredicateDirector, METH_VARARGS, NULL},
	 { "WrappedPipeBroadcaster_GetMaxBacklog", _wrap_WrappedPipeBroadcaster_GetMaxBacklog, METH_O, NULL},
	 { "WrappedPipeBroadcaster_SetMaxBacklog", _wrap_WrappedPipeBroadcaster_SetMaxBacklog, METH_VARARGS, NULL},
	 { "new_WrappedPipeBroadcaster", _wrap_new_WrappedPipeBroadcaster, METH_NOARGS, NULL},
	 { "delete_WrappedPipeBroadcaster", _wrap_delete_WrappedPipeBroadcaster, METH_O, NULL},
	 { "WrappedPipeBroadcaster_swigregister", WrappedPipeBroadcaster_swigregister, METH_O, NULL},
	 { "WrappedPipeBroadcaster_swiginit", WrappedPipeBroadcaster_swiginit, METH_VARARGS, NULL},
	 { "delete_WrappedWireConnectionDirector", _wrap_delete_WrappedWireConnectionDirector, METH_O, NULL},
	 { "WrappedWireConnectionDirector_WireValueChanged", _wrap_WrappedWireConnectionDirector_WireValueChanged, METH_VARARGS, NULL},
	 { "WrappedWireConnectionDirector_WireConnectionClosedCallback", _wrap_WrappedWireConnectionDirector_WireConnectionClosedCallback, METH_O, NULL},
	 { "new_WrappedWireConnectionDirector", _wrap_new_WrappedWireConnectionDirector, METH_O, NULL},
	 { "disown_WrappedWireConnectionDirector", _wrap_disown_WrappedWireConnectionDirector, METH_O, NULL},
	 { "WrappedWireConnectionDirector_swigregister", WrappedWireConnectionDirector_swigregister, METH_O, NULL},
	 { "WrappedWireConnectionDirector_swiginit", WrappedWireConnectionDirector_swiginit, METH_VARARGS, NULL},
	 { "delete_AsyncWireConnectionReturnDirector", _wrap_delete_AsyncWireConnectionReturnDirector, METH_O, NULL},
	 { "AsyncWireConnectionReturnDirector_handler", _wrap_AsyncWireConnectionReturnDirector_handler, METH_VARARGS, NULL},
	 { "new_AsyncWireConnectionReturnDirector", _wrap_new_AsyncWireConnectionReturnDirector, METH_O, NULL},
	 { "disown_AsyncWireConnectionReturnDirector", _wrap_disown_AsyncWireConnectionReturnDirector, METH_O, NULL},
	 { "AsyncWireConnectionReturnDirector_swigregister", AsyncWireConnectionReturnDirector_swigregister, METH_O, NULL},
	 { "AsyncWireConnectionReturnDirector_swiginit", AsyncWireConnectionReturnDirector_swiginit, METH_VARARGS, NULL},
	 { "delete_AsyncWirePeekReturnDirector", _wrap_delete_AsyncWirePeekReturnDirector, METH_O, NULL},
	 { "AsyncWirePeekReturnDirector_handler", _wrap_AsyncWirePeekReturnDirector_handler, METH_VARARGS, NULL},
	 { "new_AsyncWirePeekReturnDirector", _wrap_new_AsyncWirePeekReturnDirector, METH_O, NULL},
	 { "disown_AsyncWirePeekReturnDirector", _wrap_disown_AsyncWirePeekReturnDirector, METH_O, NULL},
	 { "AsyncWirePeekReturnDirector_swigregister", AsyncWirePeekReturnDirector_swigregister, METH_O, NULL},
	 { "AsyncWirePeekReturnDirector_swiginit", AsyncWirePeekReturnDirector_swiginit, METH_VARARGS, NULL},
	 { "TryGetValueResult_res_set", _wrap_TryGetValueResult_res_set, METH_VARARGS, NULL},
	 { "TryGetValueResult_res_get", _wrap_TryGetValueResult_res_get, METH_O, NULL},
	 { "TryGetValueResult_value_set", _wrap_TryGetValueResult_value_set, METH_VARARGS, NULL},
	 { "TryGetValueResult_value_get", _wrap_TryGetValueResult_value_get, METH_O, NULL},
	 { "TryGetValueResult_ts_set", _wrap_TryGetValueResult_ts_set, METH_VARARGS, NULL},
	 { "TryGetValueResult_ts_get", _wrap_TryGetValueResult_ts_get, METH_O, NULL},
	 { "new_TryGetValueResult", _wrap_new_TryGetValueResult, METH_NOARGS, NULL},
	 { "delete_TryGetValueResult", _wrap_delete_TryGetValueResult, METH_O, NULL},
	 { "TryGetValueResult_swigregister", TryGetValueResult_swigregister, METH_O, NULL},
	 { "TryGetValueResult_swiginit", TryGetValueResult_swiginit, METH_VARARGS, NULL},
	 { "WrappedWireConnection_GetInValue", _wrap_WrappedWireConnection_GetInValue, METH_O, NULL},
	 { "WrappedWireConnection_GetOutValue", _wrap_WrappedWireConnection_GetOutValue, METH_O, NULL},
	 { "WrappedWireConnection_SetOutValue", _wrap_WrappedWireConnection_SetOutValue, METH_VARARGS, NULL},
	 { "WrappedWireConnection_SetRRDirector", _wrap_WrappedWireConnection_SetRRDirector, METH_VARARGS, NULL},
	 { "WrappedWireConnection_Type_set", _wrap_WrappedWireConnection_Type_set, METH_VARARGS, NULL},
	 { "WrappedWireConnection_Type_get", _wrap_WrappedWireConnection_Type_get, METH_O, NULL},
	 { "WrappedWireConnection_Close", _wrap_WrappedWireConnection_Close, METH_O, NULL},
	 { "WrappedWireConnection_GetEndpoint", _wrap_WrappedWireConnection_GetEndpoint, METH_O, NULL},
	 { "WrappedWireConnection_GetLastValueReceivedTime", _wrap_WrappedWireConnection_GetLastValueReceivedTime, METH_O, NULL},
	 { "WrappedWireConnection_GetLastValueSentTime", _wrap_WrappedWireConnection_GetLastValueSentTime, METH_O, NULL},
	 { "WrappedWireConnection_GetInValueValid", _wrap_WrappedWireConnection_GetInValueValid, METH_O, NULL},
	 { "WrappedWireConnection_GetOutValueValid", _wrap_WrappedWireConnection_GetOutValueValid, METH_O, NULL},
	 { "WrappedWireConnection_TryGetInValue", _wrap_WrappedWireConnection_TryGetInValue, METH_O, NULL},
	 { "WrappedWireConnection_TryGetOutValue", _wrap_WrappedWireConnection_TryGetOutValue, METH_O, NULL},
	 { "WrappedWireConnection_AsyncClose", _wrap_WrappedWireConnection_AsyncClose, METH_VARARGS, NULL},
	 { "WrappedWireConnection_GetNode", _wrap_WrappedWireConnection_GetNode, METH_O, NULL},
	 { "WrappedWireConnection_GetIgnoreInValue", _wrap_WrappedWireConnection_GetIgnoreInValue, METH_O, NULL},
	 { "WrappedWireConnection_SetIgnoreInValue", _wrap_WrappedWireConnection_SetIgnoreInValue, METH_VARARGS, NULL},
	 { "WrappedWireConnection_GetInValueLifespan", _wrap_WrappedWireConnection_GetInValueLifespan, METH_O, NULL},
	 { "WrappedWireConnection_SetInValueLifespan", _wrap_WrappedWireConnection_SetInValueLifespan, METH_VARARGS, NULL},
	 { "WrappedWireConnection_GetOutValueLifespan", _wrap_WrappedWireConnection_GetOutValueLifespan, METH_O, NULL},
	 { "WrappedWireConnection_SetOutValueLifespan", _wrap_WrappedWireConnection_SetOutValueLifespan, METH_VARARGS, NULL},
	 { "WrappedWireConnection_WaitInValueValid", _wrap_WrappedWireConnection_WaitInValueValid, METH_VARARGS, NULL},
	 { "WrappedWireConnection_WaitOutValueValid", _wrap_WrappedWireConnection_WaitOutValueValid, METH_VARARGS, NULL},
	 { "WrappedWireConnection_Direction", _wrap_WrappedWireConnection_Direction, METH_O, NULL},
	 { "delete_WrappedWireConnection", _wrap_delete_WrappedWireConnection, METH_O, NULL},
	 { "WrappedWireConnection_swigregister", WrappedWireConnection_swigregister, METH_O, NULL},
	 { "WrappedWireClient_Connect", _wrap_WrappedWireClient_Connect, METH_O, NULL},
	 { "WrappedWireClient_AsyncConnect", _wrap_WrappedWireClient_AsyncConnect, METH_VARARGS, NULL},
	 { "WrappedWireClient_GetMemberName", _wrap_WrappedWireClient_GetMemberName, METH_O, NULL},
	 { "WrappedWireClient_PeekInValue", _wrap_WrappedWireClient_PeekInValue, METH_VARARGS, NULL},
	 { "WrappedWireClient_PeekOutValue", _wrap_WrappedWireClient_PeekOutValue, METH_VARARGS, NULL},
	 { "WrappedWireClient_PokeOutValue", _wrap_WrappedWireClient_PokeOutValue, METH_VARARGS, NULL},
	 { "WrappedWireClient_AsyncPeekInValue", _wrap_WrappedWireClient_AsyncPeekInValue, METH_VARARGS, NULL},
	 { "WrappedWireClient_AsyncPeekOutValue", _wrap_WrappedWireClient_AsyncPeekOutValue, METH_VARARGS, NULL},
	 { "WrappedWireClient_AsyncPokeOutValue", _wrap_WrappedWireClient_AsyncPokeOutValue, METH_VARARGS, NULL},
	 { "WrappedWireClient_Type_set", _wrap_WrappedWireClient_Type_set, METH_VARARGS, NULL},
	 { "WrappedWireClient_Type_get", _wrap_WrappedWireClient_Type_get, METH_O, NULL},
	 { "WrappedWireClient_GetNode", _wrap_WrappedWireClient_GetNode, METH_O, NULL},
	 { "WrappedWireClient_Direction", _wrap_WrappedWireClient_Direction, METH_O, NULL},
	 { "delete_WrappedWireClient", _wrap_delete_WrappedWireClient, METH_O, NULL},
	 { "WrappedWireClient_swigregister", WrappedWireClient_swigregister, METH_O, NULL},
	 { "delete_WrappedWireServerConnectDirector", _wrap_delete_WrappedWireServerConnectDirector, METH_O, NULL},
	 { "WrappedWireServerConnectDirector_WireConnectCallback", _wrap_WrappedWireServerConnectDirector_WireConnectCallback, METH_VARARGS, NULL},
	 { "new_WrappedWireServerConnectDirector", _wrap_new_WrappedWireServerConnectDirector, METH_O, NULL},
	 { "disown_WrappedWireServerConnectDirector", _wrap_disown_WrappedWireServerConnectDirector, METH_O, NULL},
	 { "WrappedWireServerConnectDirector_swigregister", WrappedWireServerConnectDirector_swigregister, METH_O, NULL},
	 { "WrappedWireServerConnectDirector_swiginit", WrappedWireServerConnectDirector_swiginit, METH_VARARGS, NULL},
	 { "delete_WrappedWireServerPeekValueDirector", _wrap_delete_WrappedWireServerPeekValueDirector, METH_O, NULL},
	 { "WrappedWireServerPeekValueDirector_PeekValue", _wrap_WrappedWireServerPeekValueDirector_PeekValue, METH_VARARGS, NULL},
	 { "new_WrappedWireServerPeekValueDirector", _wrap_new_WrappedWireServerPeekValueDirector, METH_O, NULL},
	 { "disown_WrappedWireServerPeekValueDirector", _wrap_disown_WrappedWireServerPeekValueDirector, METH_O, NULL},
	 { "WrappedWireServerPeekValueDirector_swigregister", WrappedWireServerPeekValueDirector_swigregister, METH_O, NULL},
	 { "WrappedWireServerPeekValueDirector_swiginit", WrappedWireServerPeekValueDirector_swiginit, METH_VARARGS, NULL},
	 { "delete_WrappedWireServerPokeValueDirector", _wrap_delete_WrappedWireServerPokeValueDirector, METH_O, NULL},
	 { "WrappedWireServerPokeValueDirector_PokeValue", _wrap_WrappedWireServerPokeValueDirector_PokeValue, METH_VARARGS, NULL},
	 { "new_WrappedWireServerPokeValueDirector", _wrap_new_WrappedWireServerPokeValueDirector, METH_O, NULL},
	 { "disown_WrappedWireServerPokeValueDirector", _wrap_disown_WrappedWireServerPokeValueDirector, METH_O, NULL},
	 { "WrappedWireServerPokeValueDirector_swigregister", WrappedWireServerPokeValueDirector_swigregister, METH_O, NULL},
	 { "WrappedWireServerPokeValueDirector_swiginit", WrappedWireServerPokeValueDirector_swiginit, METH_VARARGS, NULL},
	 { "WrappedWireServer_GetMemberName", _wrap_WrappedWireServer_GetMemberName, METH_O, NULL},
	 { "WrappedWireServer_Type_set", _wrap_WrappedWireServer_Type_set, METH_VARARGS, NULL},
	 { "WrappedWireServer_Type_get", _wrap_WrappedWireServer_Type_get, METH_O, NULL},
	 { "WrappedWireServer_SetWrappedWireConnectCallback", _wrap_WrappedWireServer_SetWrappedWireConnectCallback, METH_VARARGS, NULL},
	 { "WrappedWireServer_GetNode", _wrap_WrappedWireServer_GetNode, METH_O, NULL},
	 { "WrappedWireServer_SetPeekInValueCallback", _wrap_WrappedWireServer_SetPeekInValueCallback, METH_VARARGS, NULL},
	 { "WrappedWireServer_SetPeekOutValueCallback", _wrap_WrappedWireServer_SetPeekOutValueCallback, METH_VARARGS, NULL},
	 { "WrappedWireServer_SetPokeOutValueCallback", _wrap_WrappedWireServer_SetPokeOutValueCallback, METH_VARARGS, NULL},
	 { "WrappedWireServer_Direction", _wrap_WrappedWireServer_Direction, METH_O, NULL},
	 { "delete_WrappedWireServer", _wrap_delete_WrappedWireServer, METH_O, NULL},
	 { "WrappedWireServer_swigregister", WrappedWireServer_swigregister, METH_O, NULL},
	 { "WrappedWireBroadcasterPredicateDirector_Predicate", _wrap_WrappedWireBroadcasterPredicateDirector_Predicate, METH_VARARGS, NULL},
	 { "delete_WrappedWireBroadcasterPredicateDirector", _wrap_delete_WrappedWireBroadcasterPredicateDirector, METH_O, NULL},
	 { "new_WrappedWireBroadcasterPredicateDirector", _wrap_new_WrappedWireBroadcasterPredicateDirector, METH_O, NULL},
	 { "disown_WrappedWireBroadcasterPredicateDirector", _wrap_disown_WrappedWireBroadcasterPredicateDirector, METH_O, NULL},
	 { "WrappedWireBroadcasterPredicateDirector_swigregister", WrappedWireBroadcasterPredicateDirector_swigregister, METH_O, NULL},
	 { "WrappedWireBroadcasterPredicateDirector_swiginit", WrappedWireBroadcasterPredicateDirector_swiginit, METH_VARARGS, NULL},
	 { "WrappedWireBroadcaster_Init", _wrap_WrappedWireBroadcaster_Init, METH_VARARGS, NULL},
	 { "WrappedWireBroadcaster_SetOutValue", _wrap_WrappedWireBroadcaster_SetOutValue, METH_VARARGS, NULL},
	 { "WrappedWireBroadcaster_GetActiveWireConnectionCount", _wrap_WrappedWireBroadcaster_GetActiveWireConnectionCount, METH_O, NULL},
	 { "WrappedWireBroadcaster_SetPredicateDirector", _wrap_WrappedWireBroadcaster_SetPredicateDirector, METH_VARARGS, NULL},
	 { "WrappedWireBroadcaster_GetOutValueLifespan", _wrap_WrappedWireBroadcaster_GetOutValueLifespan, METH_O, NULL},
	 { "WrappedWireBroadcaster_SetOutValueLifespan", _wrap_WrappedWireBroadcaster_SetOutValueLifespan, METH_VARARGS, NULL},
	 { "WrappedWireBroadcaster_SetPeekInValueCallback", _wrap_WrappedWireBroadcaster_SetPeekInValueCallback, METH_VARARGS, NULL},
	 { "new_WrappedWireBroadcaster", _wrap_new_WrappedWireBroadcaster, METH_NOARGS, NULL},
	 { "delete_WrappedWireBroadcaster", _wrap_delete_WrappedWireBroadcaster, METH_O, NULL},
	 { "WrappedWireBroadcaster_swigregister", WrappedWireBroadcaster_swigregister, METH_O, NULL},
	 { "WrappedWireBroadcaster_swiginit", WrappedWireBroadcaster_swiginit, METH_VARARGS, NULL},
	 { "WrappedWireUnicastReceiver_Init", _wrap_WrappedWireUnicastReceiver_Init, METH_VARARGS, NULL},
	 { "WrappedWireUnicastReceiver_GetInValue", _wrap_WrappedWireUnicastReceiver_GetInValue, METH_VARARGS, NULL},
	 { "WrappedWireUnicastReceiver_AddInValueChangedListener", _wrap_WrappedWireUnicastReceiver_AddInValueChangedListener, METH_VARARGS, NULL},
	 { "WrappedWireUnicastReceiver_GetInValueLifespan", _wrap_WrappedWireUnicastReceiver_GetInValueLifespan, METH_O, NULL},
	 { "WrappedWireUnicastReceiver_SetInValueLifespan", _wrap_WrappedWireUnicastReceiver_SetInValueLifespan, METH_VARARGS, NULL},
	 { "WrappedWireUnicastReceiver_TryGetInValue", _wrap_WrappedWireUnicastReceiver_TryGetInValue, METH_VARARGS, NULL},
	 { "new_WrappedWireUnicastReceiver", _wrap_new_WrappedWireUnicastReceiver, METH_NOARGS, NULL},
	 { "delete_WrappedWireUnicastReceiver", _wrap_delete_WrappedWireUnicastReceiver, METH_O, NULL},
	 { "WrappedWireUnicastReceiver_swigregister", WrappedWireUnicastReceiver_swigregister, METH_O, NULL},
	 { "WrappedWireUnicastReceiver_swiginit", WrappedWireUnicastReceiver_swiginit, METH_VARARGS, NULL},
	 { "ArrayMemoryBase_Length", _wrap_ArrayMemoryBase_Length, METH_O, NULL},
	 { "ArrayMemoryBase_ElementTypeID", _wrap_ArrayMemoryBase_ElementTypeID, METH_O, NULL},
	 { "delete_ArrayMemoryBase", _wrap_delete_ArrayMemoryBase, METH_O, NULL},
	 { "ArrayMemoryBase_swigregister", ArrayMemoryBase_swigregister, METH_O, NULL},
	 { "MultiDimArrayMemoryBase_Dimensions", _wrap_MultiDimArrayMemoryBase_Dimensions, METH_O, NULL},
	 { "MultiDimArrayMemoryBase_DimCount", _wrap_MultiDimArrayMemoryBase_DimCount, METH_O, NULL},
	 { "MultiDimArrayMemoryBase_ElementTypeID", _wrap_MultiDimArrayMemoryBase_ElementTypeID, METH_O, NULL},
	 { "delete_MultiDimArrayMemoryBase", _wrap_delete_MultiDimArrayMemoryBase, METH_O, NULL},
	 { "MultiDimArrayMemoryBase_swigregister", MultiDimArrayMemoryBase_swigregister, METH_O, NULL},
	 { "WrappedArrayMemoryClientUtil_Read", _wrap_WrappedArrayMemoryClientUtil_Read, METH_VARARGS, NULL},
	 { "WrappedArrayMemoryClientUtil_Write", _wrap_WrappedArrayMemoryClientUtil_Write, METH_VARARGS, NULL},
	 { "WrappedArrayMemoryClientUtil_Direction", _wrap_WrappedArrayMemoryClientUtil_Direction, METH_O, NULL},
	 { "new_WrappedArrayMemoryClientUtil", _wrap_new_WrappedArrayMemoryClientUtil, METH_NOARGS, NULL},
	 { "delete_WrappedArrayMemoryClientUtil", _wrap_delete_WrappedArrayMemoryClientUtil, METH_O, NULL},
	 { "WrappedArrayMemoryClientUtil_swigregister", WrappedArrayMemoryClientUtil_swigregister, METH_O, NULL},
	 { "WrappedArrayMemoryClientUtil_swiginit", WrappedArrayMemoryClientUtil_swiginit, METH_VARARGS, NULL},
	 { "WrappedMultiDimArrayMemoryClientUtil_Read", _wrap_WrappedMultiDimArrayMemoryClientUtil_Read, METH_VARARGS, NULL},
	 { "WrappedMultiDimArrayMemoryClientUtil_Write", _wrap_WrappedMultiDimArrayMemoryClientUtil_Write, METH_VARARGS, NULL},
	 { "WrappedMultiDimArrayMemoryClientUtil_Direction", _wrap_WrappedMultiDimArrayMemoryClientUtil_Direction, METH_O, NULL},
	 { "new_WrappedMultiDimArrayMemoryClientUtil", _wrap_new_WrappedMultiDimArrayMemoryClientUtil, METH_NOARGS, NULL},
	 { "delete_WrappedMultiDimArrayMemoryClientUtil", _wrap_delete_WrappedMultiDimArrayMemoryClientUtil, METH_O, NULL},
	 { "WrappedMultiDimArrayMemoryClientUtil_swigregister", WrappedMultiDimArrayMemoryClientUtil_swigregister, METH_O, NULL},
	 { "WrappedMultiDimArrayMemoryClientUtil_swiginit", WrappedMultiDimArrayMemoryClientUtil_swiginit, METH_VARARGS, NULL},
	 { "WrappedPodArrayMemoryClientBuffer_UnpackReadResult", _wrap_WrappedPodArrayMemoryClientBuffer_UnpackReadResult, METH_VARARGS, NULL},
	 { "WrappedPodArrayMemoryClientBuffer_PackWriteRequest", _wrap_WrappedPodArrayMemoryClientBuffer_PackWriteRequest, METH_VARARGS, NULL},
	 { "WrappedPodArrayMemoryClientBuffer_GetBufferLength", _wrap_WrappedPodArrayMemoryClientBuffer_GetBufferLength, METH_O, NULL},
	 { "delete_WrappedPodArrayMemoryClientBuffer", _wrap_delete_WrappedPodArrayMemoryClientBuffer, METH_O, NULL},
	 { "new_WrappedPodArrayMemoryClientBuffer", _wrap_new_WrappedPodArrayMemoryClientBuffer, METH_O, NULL},
	 { "disown_WrappedPodArrayMemoryClientBuffer", _wrap_disown_WrappedPodArrayMemoryClientBuffer, METH_O, NULL},
	 { "WrappedPodArrayMemoryClientBuffer_swigregister", WrappedPodArrayMemoryClientBuffer_swigregister, METH_O, NULL},
	 { "WrappedPodArrayMemoryClientBuffer_swiginit", WrappedPodArrayMemoryClientBuffer_swiginit, METH_VARARGS, NULL},
	 { "WrappedPodArrayMemoryClient_Length", _wrap_WrappedPodArrayMemoryClient_Length, METH_O, NULL},
	 { "WrappedPodArrayMemoryClient_Direction", _wrap_WrappedPodArrayMemoryClient_Direction, METH_O, NULL},
	 { "WrappedPodArrayMemoryClient_Read", _wrap_WrappedPodArrayMemoryClient_Read, METH_VARARGS, NULL},
	 { "WrappedPodArrayMemoryClient_Write", _wrap_WrappedPodArrayMemoryClient_Write, METH_VARARGS, NULL},
	 { "delete_WrappedPodArrayMemoryClient", _wrap_delete_WrappedPodArrayMemoryClient, METH_O, NULL},
	 { "WrappedPodArrayMemoryClient_swigregister", WrappedPodArrayMemoryClient_swigregister, METH_O, NULL},
	 { "WrappedPodMultiDimArrayMemoryClientBuffer_UnpackReadResult", _wrap_WrappedPodMultiDimArrayMemoryClientBuffer_UnpackReadResult, METH_VARARGS, NULL},
	 { "WrappedPodMultiDimArrayMemoryClientBuffer_PackWriteRequest", _wrap_WrappedPodMultiDimArrayMemoryClientBuffer_PackWriteRequest, METH_VARARGS, NULL},
	 { "delete_WrappedPodMultiDimArrayMemoryClientBuffer", _wrap_delete_WrappedPodMultiDimArrayMemoryClientBuffer, METH_O, NULL},
	 { "new_WrappedPodMultiDimArrayMemoryClientBuffer", _wrap_new_WrappedPodMultiDimArrayMemoryClientBuffer, METH_O, NULL},
	 { "disown_WrappedPodMultiDimArrayMemoryClientBuffer", _wrap_disown_WrappedPodMultiDimArrayMemoryClientBuffer, METH_O, NULL},
	 { "WrappedPodMultiDimArrayMemoryClientBuffer_swigregister", WrappedPodMultiDimArrayMemoryClientBuffer_swigregister, METH_O, NULL},
	 { "WrappedPodMultiDimArrayMemoryClientBuffer_swiginit", WrappedPodMultiDimArrayMemoryClientBuffer_swiginit, METH_VARARGS, NULL},
	 { "WrappedPodMultiDimArrayMemoryClient_Dimensions", _wrap_WrappedPodMultiDimArrayMemoryClient_Dimensions, METH_O, NULL},
	 { "WrappedPodMultiDimArrayMemoryClient_DimCount", _wrap_WrappedPodMultiDimArrayMemoryClient_DimCount, METH_O, NULL},
	 { "WrappedPodMultiDimArrayMemoryClient_Direction", _wrap_WrappedPodMultiDimArrayMemoryClient_Direction, METH_O, NULL},
	 { "WrappedPodMultiDimArrayMemoryClient_Read", _wrap_WrappedPodMultiDimArrayMemoryClient_Read, METH_VARARGS, NULL},
	 { "WrappedPodMultiDimArrayMemoryClient_Write", _wrap_WrappedPodMultiDimArrayMemoryClient_Write, METH_VARARGS, NULL},
	 { "delete_WrappedPodMultiDimArrayMemoryClient", _wrap_delete_WrappedPodMultiDimArrayMemoryClient, METH_O, NULL},
	 { "WrappedPodMultiDimArrayMemoryClient_swigregister", WrappedPodMultiDimArrayMemoryClient_swigregister, METH_O, NULL},
	 { "delete_WrappedArrayMemoryDirector", _wrap_delete_WrappedArrayMemoryDirector, METH_O, NULL},
	 { "WrappedArrayMemoryDirector_Length", _wrap_WrappedArrayMemoryDirector_Length, METH_O, NULL},
	 { "WrappedArrayMemoryDirector_Read", _wrap_WrappedArrayMemoryDirector_Read, METH_VARARGS, NULL},
	 { "WrappedArrayMemoryDirector_Write", _wrap_WrappedArrayMemoryDirector_Write, METH_VARARGS, NULL},
	 { "WrappedArrayMemoryDirector_objectheapid_set", _wrap_WrappedArrayMemoryDirector_objectheapid_set, METH_VARARGS, NULL},
	 { "WrappedArrayMemoryDirector_objectheapid_get", _wrap_WrappedArrayMemoryDirector_objectheapid_get, METH_O, NULL},
	 { "new_WrappedArrayMemoryDirector", _wrap_new_WrappedArrayMemoryDirector, METH_O, NULL},
	 { "disown_WrappedArrayMemoryDirector", _wrap_disown_WrappedArrayMemoryDirector, METH_O, NULL},
	 { "WrappedArrayMemoryDirector_swigregister", WrappedArrayMemoryDirector_swigregister, METH_O, NULL},
	 { "WrappedArrayMemoryDirector_swiginit", WrappedArrayMemoryDirector_swiginit, METH_VARARGS, NULL},
	 { "WrappedMultiDimArrayMemoryParams_memorypos_set", _wrap_WrappedMultiDimArrayMemoryParams_memorypos_set, METH_VARARGS, NULL},
	 { "WrappedMultiDimArrayMemoryParams_memorypos_get", _wrap_WrappedMultiDimArrayMemoryParams_memorypos_get, METH_O, NULL},
	 { "WrappedMultiDimArrayMemoryParams_buffer_set", _wrap_WrappedMultiDimArrayMemoryParams_buffer_set, METH_VARARGS, NULL},
	 { "WrappedMultiDimArrayMemoryParams_buffer_get", _wrap_WrappedMultiDimArrayMemoryParams_buffer_get, METH_O, NULL},
	 { "WrappedMultiDimArrayMemoryParams_bufferpos_set", _wrap_WrappedMultiDimArrayMemoryParams_bufferpos_set, METH_VARARGS, NULL},
	 { "WrappedMultiDimArrayMemoryParams_bufferpos_get", _wrap_WrappedMultiDimArrayMemoryParams_bufferpos_get, METH_O, NULL},
	 { "WrappedMultiDimArrayMemoryParams_count_set", _wrap_WrappedMultiDimArrayMemoryParams_count_set, METH_VARARGS, NULL},
	 { "WrappedMultiDimArrayMemoryParams_count_get", _wrap_WrappedMultiDimArrayMemoryParams_count_get, METH_O, NULL},
	 { "new_WrappedMultiDimArrayMemoryParams", _wrap_new_WrappedMultiDimArrayMemoryParams, METH_NOARGS, NULL},
	 { "delete_WrappedMultiDimArrayMemoryParams", _wrap_delete_WrappedMultiDimArrayMemoryParams, METH_O, NULL},
	 { "WrappedMultiDimArrayMemoryParams_swigregister", WrappedMultiDimArrayMemoryParams_swigregister, METH_O, NULL},
	 { "WrappedMultiDimArrayMemoryParams_swiginit", WrappedMultiDimArrayMemoryParams_swiginit, METH_VARARGS, NULL},
	 { "delete_WrappedMultiDimArrayMemoryDirector", _wrap_delete_WrappedMultiDimArrayMemoryDirector, METH_O, NULL},
	 { "WrappedMultiDimArrayMemoryDirector_Dimensions", _wrap_WrappedMultiDimArrayMemoryDirector_Dimensions, METH_O, NULL},
	 { "WrappedMultiDimArrayMemoryDirector_DimCount", _wrap_WrappedMultiDimArrayMemoryDirector_DimCount, METH_O, NULL},
	 { "WrappedMultiDimArrayMemoryDirector_Read", _wrap_WrappedMultiDimArrayMemoryDirector_Read, METH_VARARGS, NULL},
	 { "WrappedMultiDimArrayMemoryDirector_Write", _wrap_WrappedMultiDimArrayMemoryDirector_Write, METH_VARARGS, NULL},
	 { "WrappedMultiDimArrayMemoryDirector_objectheapid_set", _wrap_WrappedMultiDimArrayMemoryDirector_objectheapid_set, METH_VARARGS, NULL},
	 { "WrappedMultiDimArrayMemoryDirector_objectheapid_get", _wrap_WrappedMultiDimArrayMemoryDirector_objectheapid_get, METH_O, NULL},
	 { "new_WrappedMultiDimArrayMemoryDirector", _wrap_new_WrappedMultiDimArrayMemoryDirector, METH_O, NULL},
	 { "disown_WrappedMultiDimArrayMemoryDirector", _wrap_disown_WrappedMultiDimArrayMemoryDirector, METH_O, NULL},
	 { "WrappedMultiDimArrayMemoryDirector_swigregister", WrappedMultiDimArrayMemoryDirector_swigregister, METH_O, NULL},
	 { "WrappedMultiDimArrayMemoryDirector_swiginit", WrappedMultiDimArrayMemoryDirector_swiginit, METH_VARARGS, NULL},
	 { "new_WrappedPodArrayMemoryDirector", _wrap_new_WrappedPodArrayMemoryDirector, METH_O, NULL},
	 { "delete_WrappedPodArrayMemoryDirector", _wrap_delete_WrappedPodArrayMemoryDirector, METH_O, NULL},
	 { "WrappedPodArrayMemoryDirector_Length", _wrap_WrappedPodArrayMemoryDirector_Length, METH_O, NULL},
	 { "WrappedPodArrayMemoryDirector_Read", _wrap_WrappedPodArrayMemoryDirector_Read, METH_VARARGS, NULL},
	 { "WrappedPodArrayMemoryDirector_Write", _wrap_WrappedPodArrayMemoryDirector_Write, METH_VARARGS, NULL},
	 { "WrappedPodArrayMemoryDirector_objectheapid_set", _wrap_WrappedPodArrayMemoryDirector_objectheapid_set, METH_VARARGS, NULL},
	 { "WrappedPodArrayMemoryDirector_objectheapid_get", _wrap_WrappedPodArrayMemoryDirector_objectheapid_get, METH_O, NULL},
	 { "disown_WrappedPodArrayMemoryDirector", _wrap_disown_WrappedPodArrayMemoryDirector, METH_O, NULL},
	 { "WrappedPodArrayMemoryDirector_swigregister", WrappedPodArrayMemoryDirector_swigregister, METH_O, NULL},
	 { "WrappedPodArrayMemoryDirector_swiginit", WrappedPodArrayMemoryDirector_swiginit, METH_VARARGS, NULL},
	 { "delete_WrappedPodMultiDimArrayMemoryDirector", _wrap_delete_WrappedPodMultiDimArrayMemoryDirector, METH_O, NULL},
	 { "WrappedPodMultiDimArrayMemoryDirector_Dimensions", _wrap_WrappedPodMultiDimArrayMemoryDirector_Dimensions, METH_O, NULL},
	 { "WrappedPodMultiDimArrayMemoryDirector_DimCount", _wrap_WrappedPodMultiDimArrayMemoryDirector_DimCount, METH_O, NULL},
	 { "WrappedPodMultiDimArrayMemoryDirector_Read", _wrap_WrappedPodMultiDimArrayMemoryDirector_Read, METH_VARARGS, NULL},
	 { "WrappedPodMultiDimArrayMemoryDirector_Write", _wrap_WrappedPodMultiDimArrayMemoryDirector_Write, METH_VARARGS, NULL},
	 { "WrappedPodMultiDimArrayMemoryDirector_objectheapid_set", _wrap_WrappedPodMultiDimArrayMemoryDirector_objectheapid_set, METH_VARARGS, NULL},
	 { "WrappedPodMultiDimArrayMemoryDirector_objectheapid_get", _wrap_WrappedPodMultiDimArrayMemoryDirector_objectheapid_get, METH_O, NULL},
	 { "new_WrappedPodMultiDimArrayMemoryDirector", _wrap_new_WrappedPodMultiDimArrayMemoryDirector, METH_O, NULL},
	 { "disown_WrappedPodMultiDimArrayMemoryDirector", _wrap_disown_WrappedPodMultiDimArrayMemoryDirector, METH_O, NULL},
	 { "WrappedPodMultiDimArrayMemoryDirector_swigregister", WrappedPodMultiDimArrayMemoryDirector_swigregister, METH_O, NULL},
	 { "WrappedPodMultiDimArrayMemoryDirector_swiginit", WrappedPodMultiDimArrayMemoryDirector_swiginit, METH_VARARGS, NULL},
	 { "WrappedNamedArrayMemoryClientBuffer_UnpackReadResult", _wrap_WrappedNamedArrayMemoryClientBuffer_UnpackReadResult, METH_VARARGS, NULL},
	 { "WrappedNamedArrayMemoryClientBuffer_PackWriteRequest", _wrap_WrappedNamedArrayMemoryClientBuffer_PackWriteRequest, METH_VARARGS, NULL},
	 { "WrappedNamedArrayMemoryClientBuffer_GetBufferLength", _wrap_WrappedNamedArrayMemoryClientBuffer_GetBufferLength, METH_O, NULL},
	 { "delete_WrappedNamedArrayMemoryClientBuffer", _wrap_delete_WrappedNamedArrayMemoryClientBuffer, METH_O, NULL},
	 { "new_WrappedNamedArrayMemoryClientBuffer", _wrap_new_WrappedNamedArrayMemoryClientBuffer, METH_O, NULL},
	 { "disown_WrappedNamedArrayMemoryClientBuffer", _wrap_disown_WrappedNamedArrayMemoryClientBuffer, METH_O, NULL},
	 { "WrappedNamedArrayMemoryClientBuffer_swigregister", WrappedNamedArrayMemoryClientBuffer_swigregister, METH_O, NULL},
	 { "WrappedNamedArrayMemoryClientBuffer_swiginit", WrappedNamedArrayMemoryClientBuffer_swiginit, METH_VARARGS, NULL},
	 { "WrappedNamedArrayMemoryClient_Length", _wrap_WrappedNamedArrayMemoryClient_Length, METH_O, NULL},
	 { "WrappedNamedArrayMemoryClient_Direction", _wrap_WrappedNamedArrayMemoryClient_Direction, METH_O, NULL},
	 { "WrappedNamedArrayMemoryClient_Read", _wrap_WrappedNamedArrayMemoryClient_Read, METH_VARARGS, NULL},
	 { "WrappedNamedArrayMemoryClient_Write", _wrap_WrappedNamedArrayMemoryClient_Write, METH_VARARGS, NULL},
	 { "delete_WrappedNamedArrayMemoryClient", _wrap_delete_WrappedNamedArrayMemoryClient, METH_O, NULL},
	 { "WrappedNamedArrayMemoryClient_swigregister", WrappedNamedArrayMemoryClient_swigregister, METH_O, NULL},
	 { "WrappedNamedMultiDimArrayMemoryClientBuffer_UnpackReadResult", _wrap_WrappedNamedMultiDimArrayMemoryClientBuffer_UnpackReadResult, METH_VARARGS, NULL},
	 { "WrappedNamedMultiDimArrayMemoryClientBuffer_PackWriteRequest", _wrap_WrappedNamedMultiDimArrayMemoryClientBuffer_PackWriteRequest, METH_VARARGS, NULL},
	 { "delete_WrappedNamedMultiDimArrayMemoryClientBuffer", _wrap_delete_WrappedNamedMultiDimArrayMemoryClientBuffer, METH_O, NULL},
	 { "new_WrappedNamedMultiDimArrayMemoryClientBuffer", _wrap_new_WrappedNamedMultiDimArrayMemoryClientBuffer, METH_O, NULL},
	 { "disown_WrappedNamedMultiDimArrayMemoryClientBuffer", _wrap_disown_WrappedNamedMultiDimArrayMemoryClientBuffer, METH_O, NULL},
	 { "WrappedNamedMultiDimArrayMemoryClientBuffer_swigregister", WrappedNamedMultiDimArrayMemoryClientBuffer_swigregister, METH_O, NULL},
	 { "WrappedNamedMultiDimArrayMemoryClientBuffer_swiginit", WrappedNamedMultiDimArrayMemoryClientBuffer_swiginit, METH_VARARGS, NULL},
	 { "WrappedNamedMultiDimArrayMemoryClient_Dimensions", _wrap_WrappedNamedMultiDimArrayMemoryClient_Dimensions, METH_O, NULL},
	 { "WrappedNamedMultiDimArrayMemoryClient_DimCount", _wrap_WrappedNamedMultiDimArrayMemoryClient_DimCount, METH_O, NULL},
	 { "WrappedNamedMultiDimArrayMemoryClient_Direction", _wrap_WrappedNamedMultiDimArrayMemoryClient_Direction, METH_O, NULL},
	 { "WrappedNamedMultiDimArrayMemoryClient_Read", _wrap_WrappedNamedMultiDimArrayMemoryClient_Read, METH_VARARGS, NULL},
	 { "WrappedNamedMultiDimArrayMemoryClient_Write", _wrap_WrappedNamedMultiDimArrayMemoryClient_Write, METH_VARARGS, NULL},
	 { "delete_WrappedNamedMultiDimArrayMemoryClient", _wrap_delete_WrappedNamedMultiDimArrayMemoryClient, METH_O, NULL},
	 { "WrappedNamedMultiDimArrayMemoryClient_swigregister", WrappedNamedMultiDimArrayMemoryClient_swigregister, METH_O, NULL},
	 { "new_WrappedNamedArrayMemoryDirector", _wrap_new_WrappedNamedArrayMemoryDirector, METH_O, NULL},
	 { "delete_WrappedNamedArrayMemoryDirector", _wrap_delete_WrappedNamedArrayMemoryDirector, METH_O, NULL},
	 { "WrappedNamedArrayMemoryDirector_Length", _wrap_WrappedNamedArrayMemoryDirector_Length, METH_O, NULL},
	 { "WrappedNamedArrayMemoryDirector_Read", _wrap_WrappedNamedArrayMemoryDirector_Read, METH_VARARGS, NULL},
	 { "WrappedNamedArrayMemoryDirector_Write", _wrap_WrappedNamedArrayMemoryDirector_Write, METH_VARARGS, NULL},
	 { "WrappedNamedArrayMemoryDirector_objectheapid_set", _wrap_WrappedNamedArrayMemoryDirector_objectheapid_set, METH_VARARGS, NULL},
	 { "WrappedNamedArrayMemoryDirector_objectheapid_get", _wrap_WrappedNamedArrayMemoryDirector_objectheapid_get, METH_O, NULL},
	 { "disown_WrappedNamedArrayMemoryDirector", _wrap_disown_WrappedNamedArrayMemoryDirector, METH_O, NULL},
	 { "WrappedNamedArrayMemoryDirector_swigregister", WrappedNamedArrayMemoryDirector_swigregister, METH_O, NULL},
	 { "WrappedNamedArrayMemoryDirector_swiginit", WrappedNamedArrayMemoryDirector_swiginit, METH_VARARGS, NULL},
	 { "delete_WrappedNamedMultiDimArrayMemoryDirector", _wrap_delete_WrappedNamedMultiDimArrayMemoryDirector, METH_O, NULL},
	 { "WrappedNamedMultiDimArrayMemoryDirector_Dimensions", _wrap_WrappedNamedMultiDimArrayMemoryDirector_Dimensions, METH_O, NULL},
	 { "WrappedNamedMultiDimArrayMemoryDirector_DimCount", _wrap_WrappedNamedMultiDimArrayMemoryDirector_DimCount, METH_O, NULL},
	 { "WrappedNamedMultiDimArrayMemoryDirector_Read", _wrap_WrappedNamedMultiDimArrayMemoryDirector_Read, METH_VARARGS, NULL},
	 { "WrappedNamedMultiDimArrayMemoryDirector_Write", _wrap_WrappedNamedMultiDimArrayMemoryDirector_Write, METH_VARARGS, NULL},
	 { "WrappedNamedMultiDimArrayMemoryDirector_objectheapid_set", _wrap_WrappedNamedMultiDimArrayMemoryDirector_objectheapid_set, METH_VARARGS, NULL},
	 { "WrappedNamedMultiDimArrayMemoryDirector_objectheapid_get", _wrap_WrappedNamedMultiDimArrayMemoryDirector_objectheapid_get, METH_O, NULL},
	 { "new_WrappedNamedMultiDimArrayMemoryDirector", _wrap_new_WrappedNamedMultiDimArrayMemoryDirector, METH_O, NULL},
	 { "disown_WrappedNamedMultiDimArrayMemoryDirector", _wrap_disown_WrappedNamedMultiDimArrayMemoryDirector, METH_O, NULL},
	 { "WrappedNamedMultiDimArrayMemoryDirector_swigregister", WrappedNamedMultiDimArrayMemoryDirector_swigregister, METH_O, NULL},
	 { "WrappedNamedMultiDimArrayMemoryDirector_swiginit", WrappedNamedMultiDimArrayMemoryDirector_swiginit, METH_VARARGS, NULL},
	 { "WrappedGeneratorClient_TryGetNextResult_value_set", _wrap_WrappedGeneratorClient_TryGetNextResult_value_set, METH_VARARGS, NULL},
	 { "WrappedGeneratorClient_TryGetNextResult_value_get", _wrap_WrappedGeneratorClient_TryGetNextResult_value_get, METH_O, NULL},
	 { "WrappedGeneratorClient_TryGetNextResult_res_set", _wrap_WrappedGeneratorClient_TryGetNextResult_res_set, METH_VARARGS, NULL},
	 { "WrappedGeneratorClient_TryGetNextResult_res_get", _wrap_WrappedGeneratorClient_TryGetNextResult_res_get, METH_O, NULL},
	 { "delete_WrappedGeneratorClient_TryGetNextResult", _wrap_delete_WrappedGeneratorClient_TryGetNextResult, METH_O, NULL},
	 { "WrappedGeneratorClient_TryGetNextResult_swigregister", WrappedGeneratorClient_TryGetNextResult_swigregister, METH_O, NULL},
	 { "WrappedGeneratorClient_Next", _wrap_WrappedGeneratorClient_Next, METH_VARARGS, NULL},
	 { "WrappedGeneratorClient_TryNext", _wrap_WrappedGeneratorClient_TryNext, METH_VARARGS, NULL},
	 { "WrappedGeneratorClient_AsyncNext", _wrap_WrappedGeneratorClient_AsyncNext, METH_VARARGS, NULL},
	 { "WrappedGeneratorClient_Abort", _wrap_WrappedGeneratorClient_Abort, METH_O, NULL},
	 { "WrappedGeneratorClient_AsyncAbort", _wrap_WrappedGeneratorClient_AsyncAbort, METH_VARARGS, NULL},
	 { "WrappedGeneratorClient_Close", _wrap_WrappedGeneratorClient_Close, METH_O, NULL},
	 { "WrappedGeneratorClient_AsyncClose", _wrap_WrappedGeneratorClient_AsyncClose, METH_VARARGS, NULL},
	 { "WrappedGeneratorClient_NextAll", _wrap_WrappedGeneratorClient_NextAll, METH_O, NULL},
	 { "delete_WrappedGeneratorClient", _wrap_delete_WrappedGeneratorClient, METH_O, NULL},
	 { "WrappedGeneratorClient_swigregister", WrappedGeneratorClient_swigregister, METH_O, NULL},
	 { "delete_AsyncGeneratorClientReturnDirector", _wrap_delete_AsyncGeneratorClientReturnDirector, METH_O, NULL},
	 { "AsyncGeneratorClientReturnDirector_handler", _wrap_AsyncGeneratorClientReturnDirector_handler, METH_VARARGS, NULL},
	 { "new_AsyncGeneratorClientReturnDirector", _wrap_new_AsyncGeneratorClientReturnDirector, METH_O, NULL},
	 { "disown_AsyncGeneratorClientReturnDirector", _wrap_disown_AsyncGeneratorClientReturnDirector, METH_O, NULL},
	 { "AsyncGeneratorClientReturnDirector_swigregister", AsyncGeneratorClientReturnDirector_swigregister, METH_O, NULL},
	 { "AsyncGeneratorClientReturnDirector_swiginit", AsyncGeneratorClientReturnDirector_swiginit, METH_VARARGS, NULL},
	 { "new_WrappedGeneratorServerDirector", _wrap_new_WrappedGeneratorServerDirector, METH_O, NULL},
	 { "delete_WrappedGeneratorServerDirector", _wrap_delete_WrappedGeneratorServerDirector, METH_O, NULL},
	 { "WrappedGeneratorServerDirector_Next", _wrap_WrappedGeneratorServerDirector_Next, METH_VARARGS, NULL},
	 { "WrappedGeneratorServerDirector_Abort", _wrap_WrappedGeneratorServerDirector_Abort, METH_VARARGS, NULL},
	 { "WrappedGeneratorServerDirector_Close", _wrap_WrappedGeneratorServerDirector_Close, METH_VARARGS, NULL},
	 { "WrappedGeneratorServerDirector_objectheapid_set", _wrap_WrappedGeneratorServerDirector_objectheapid_set, METH_VARARGS, NULL},
	 { "WrappedGeneratorServerDirector_objectheapid_get", _wrap_WrappedGeneratorServerDirector_objectheapid_get, METH_O, NULL},
	 { "disown_WrappedGeneratorServerDirector", _wrap_disown_WrappedGeneratorServerDirector, METH_O, NULL},
	 { "WrappedGeneratorServerDirector_swigregister", WrappedGeneratorServerDirector_swigregister, METH_O, NULL},
	 { "WrappedGeneratorServerDirector_swiginit", WrappedGeneratorServerDirector_swiginit, METH_VARARGS, NULL},
	 { "delete_WrappedServiceStubDirector", _wrap_delete_WrappedServiceStubDirector, METH_O, NULL},
	 { "WrappedServiceStubDirector_DispatchEvent", _wrap_WrappedServiceStubDirector_DispatchEvent, METH_VARARGS, NULL},
	 { "WrappedServiceStubDirector_CallbackCall", _wrap_WrappedServiceStubDirector_CallbackCall, METH_VARARGS, NULL},
	 { "new_WrappedServiceStubDirector", _wrap_new_WrappedServiceStubDirector, METH_O, NULL},
	 { "disown_WrappedServiceStubDirector", _wrap_disown_WrappedServiceStubDirector, METH_O, NULL},
	 { "WrappedServiceStubDirector_swigregister", WrappedServiceStubDirector_swigregister, METH_O, NULL},
	 { "WrappedServiceStubDirector_swiginit", WrappedServiceStubDirector_swiginit, METH_VARARGS, NULL},
	 { "delete_AsyncStubReturnDirector", _wrap_delete_AsyncStubReturnDirector, METH_O, NULL},
	 { "AsyncStubReturnDirector_handler", _wrap_AsyncStubReturnDirector_handler, METH_VARARGS, NULL},
	 { "new_AsyncStubReturnDirector", _wrap_new_AsyncStubReturnDirector, METH_O, NULL},
	 { "disown_AsyncStubReturnDirector", _wrap_disown_AsyncStubReturnDirector, METH_O, NULL},
	 { "AsyncStubReturnDirector_swigregister", AsyncStubReturnDirector_swigregister, METH_O, NULL},
	 { "AsyncStubReturnDirector_swiginit", AsyncStubReturnDirector_swiginit, METH_VARARGS, NULL},
	 { "WrappedDirectorLock_valid", _wrap_WrappedDirectorLock_valid, METH_O, NULL},
	 { "delete_WrappedDirectorLock", _wrap_delete_WrappedDirectorLock, METH_O, NULL},
	 { "WrappedDirectorLock_swigregister", WrappedDirectorLock_swigregister, METH_O, NULL},
	 { "WrappedServiceStub_PropertyGet", _wrap_WrappedServiceStub_PropertyGet, METH_VARARGS, NULL},
	 { "WrappedServiceStub_PropertySet", _wrap_WrappedServiceStub_PropertySet, METH_VARARGS, NULL},
	 { "WrappedServiceStub_FunctionCall", _wrap_WrappedServiceStub_FunctionCall, METH_VARARGS, NULL},
	 { "WrappedServiceStub_GeneratorFunctionCall", _wrap_WrappedServiceStub_GeneratorFunctionCall, METH_VARARGS, NULL},
	 { "WrappedServiceStub_async_PropertyGet", _wrap_WrappedServiceStub_async_PropertyGet, METH_VARARGS, NULL},
	 { "WrappedServiceStub_async_PropertySet", _wrap_WrappedServiceStub_async_PropertySet, METH_VARARGS, NULL},
	 { "WrappedServiceStub_async_FunctionCall", _wrap_WrappedServiceStub_async_FunctionCall, METH_VARARGS, NULL},
	 { "WrappedServiceStub_async_GeneratorFunctionCall", _wrap_WrappedServiceStub_async_GeneratorFunctionCall, METH_VARARGS, NULL},
	 { "WrappedServiceStub_async_FindObjRef", _wrap_WrappedServiceStub_async_FindObjRef, METH_VARARGS, NULL},
	 { "WrappedServiceStub_async_FindObjRefTyped", _wrap_WrappedServiceStub_async_FindObjRefTyped, METH_VARARGS, NULL},
	 { "WrappedServiceStub_GetPipe", _wrap_WrappedServiceStub_GetPipe, METH_VARARGS, NULL},
	 { "WrappedServiceStub_GetWire", _wrap_WrappedServiceStub_GetWire, METH_VARARGS, NULL},
	 { "WrappedServiceStub_GetObjectHeapID", _wrap_WrappedServiceStub_GetObjectHeapID, METH_O, NULL},
	 { "WrappedServiceStub__FindObjRef", _wrap_WrappedServiceStub__FindObjRef, METH_VARARGS, NULL},
	 { "WrappedServiceStub__FindObjRefTyped", _wrap_WrappedServiceStub__FindObjRefTyped, METH_VARARGS, NULL},
	 { "WrappedServiceStub_GetArrayMemory", _wrap_WrappedServiceStub_GetArrayMemory, METH_VARARGS, NULL},
	 { "WrappedServiceStub_GetMultiDimArrayMemory", _wrap_WrappedServiceStub_GetMultiDimArrayMemory, METH_VARARGS, NULL},
	 { "WrappedServiceStub_GetPodArrayMemory", _wrap_WrappedServiceStub_GetPodArrayMemory, METH_VARARGS, NULL},
	 { "WrappedServiceStub_GetPodMultiDimArrayMemory", _wrap_WrappedServiceStub_GetPodMultiDimArrayMemory, METH_VARARGS, NULL},
	 { "WrappedServiceStub_GetNamedArrayMemory", _wrap_WrappedServiceStub_GetNamedArrayMemory, METH_VARARGS, NULL},
	 { "WrappedServiceStub_GetNamedMultiDimArrayMemory", _wrap_WrappedServiceStub_GetNamedMultiDimArrayMemory, METH_VARARGS, NULL},
	 { "WrappedServiceStub_RRClose", _wrap_WrappedServiceStub_RRClose, METH_O, NULL},
	 { "WrappedServiceStub_RR_objecttype_set", _wrap_WrappedServiceStub_RR_objecttype_set, METH_VARARGS, NULL},
	 { "WrappedServiceStub_RR_objecttype_get", _wrap_WrappedServiceStub_RR_objecttype_get, METH_O, NULL},
	 { "WrappedServiceStub_SetRRDirector", _wrap_WrappedServiceStub_SetRRDirector, METH_VARARGS, NULL},
	 { "WrappedServiceStub_GetRRDirectorLock", _wrap_WrappedServiceStub_GetRRDirectorLock, METH_O, NULL},
	 { "WrappedServiceStub_RRGetNode", _wrap_WrappedServiceStub_RRGetNode, METH_O, NULL},
	 { "WrappedServiceStub_GetPyStub", _wrap_WrappedServiceStub_GetPyStub, METH_O, NULL},
	 { "WrappedServiceStub_SetPyStub", _wrap_WrappedServiceStub_SetPyStub, METH_VARARGS, NULL},
	 { "delete_WrappedServiceStub", _wrap_delete_WrappedServiceStub, METH_O, NULL},
	 { "WrappedServiceStub_swigregister", WrappedServiceStub_swigregister, METH_O, NULL},
	 { "delete_WrappedServiceSkelDirector", _wrap_delete_WrappedServiceSkelDirector, METH_O, NULL},
	 { "WrappedServiceSkelDirector_Init", _wrap_WrappedServiceSkelDirector_Init, METH_VARARGS, NULL},
	 { "WrappedServiceSkelDirector__CallGetProperty", _wrap_WrappedServiceSkelDirector__CallGetProperty, METH_VARARGS, NULL},
	 { "WrappedServiceSkelDirector__CallSetProperty", _wrap_WrappedServiceSkelDirector__CallSetProperty, METH_VARARGS, NULL},
	 { "WrappedServiceSkelDirector__CallFunction", _wrap_WrappedServiceSkelDirector__CallFunction, METH_VARARGS, NULL},
	 { "WrappedServiceSkelDirector__GetSubObj", _wrap_WrappedServiceSkelDirector__GetSubObj, METH_VARARGS, NULL},
	 { "WrappedServiceSkelDirector__GetArrayMemory", _wrap_WrappedServiceSkelDirector__GetArrayMemory, METH_VARARGS, NULL},
	 { "WrappedServiceSkelDirector__GetMultiDimArrayMemory", _wrap_WrappedServiceSkelDirector__GetMultiDimArrayMemory, METH_VARARGS, NULL},
	 { "WrappedServiceSkelDirector__GetPodArrayMemory", _wrap_WrappedServiceSkelDirector__GetPodArrayMemory, METH_VARARGS, NULL},
	 { "WrappedServiceSkelDirector__GetPodMultiDimArrayMemory", _wrap_WrappedServiceSkelDirector__GetPodMultiDimArrayMemory, METH_VARARGS, NULL},
	 { "WrappedServiceSkelDirector__GetNamedArrayMemory", _wrap_WrappedServiceSkelDirector__GetNamedArrayMemory, METH_VARARGS, NULL},
	 { "WrappedServiceSkelDirector__GetNamedMultiDimArrayMemory", _wrap_WrappedServiceSkelDirector__GetNamedMultiDimArrayMemory, METH_VARARGS, NULL},
	 { "WrappedServiceSkelDirector_MonitorEnter", _wrap_WrappedServiceSkelDirector_MonitorEnter, METH_VARARGS, NULL},
	 { "WrappedServiceSkelDirector_MonitorExit", _wrap_WrappedServiceSkelDirector_MonitorExit, METH_O, NULL},
	 { "WrappedServiceSkelDirector_ReleaseCastObject", _wrap_WrappedServiceSkelDirector_ReleaseCastObject, METH_O, NULL},
	 { "new_WrappedServiceSkelDirector", _wrap_new_WrappedServiceSkelDirector, METH_O, NULL},
	 { "disown_WrappedServiceSkelDirector", _wrap_disown_WrappedServiceSkelDirector, METH_O, NULL},
	 { "WrappedServiceSkelDirector_swigregister", WrappedServiceSkelDirector_swigregister, METH_O, NULL},
	 { "WrappedServiceSkelDirector_swiginit", WrappedServiceSkelDirector_swiginit, METH_VARARGS, NULL},
	 { "new_WrappedRRObject", _wrap_new_WrappedRRObject, METH_VARARGS, NULL},
	 { "WrappedRRObject_Type_set", _wrap_WrappedRRObject_Type_set, METH_VARARGS, NULL},
	 { "WrappedRRObject_Type_get", _wrap_WrappedRRObject_Type_get, METH_O, NULL},
	 { "delete_WrappedRRObject", _wrap_delete_WrappedRRObject, METH_O, NULL},
	 { "WrappedRRObject_swigregister", WrappedRRObject_swigregister, METH_O, NULL},
	 { "WrappedRRObject_swiginit", WrappedRRObject_swiginit, METH_VARARGS, NULL},
	 { "WrappedServiceSkel_Type_set", _wrap_WrappedServiceSkel_Type_set, METH_VARARGS, NULL},
	 { "WrappedServiceSkel_Type_get", _wrap_WrappedServiceSkel_Type_get, METH_O, NULL},
	 { "WrappedServiceSkel_castobj_set", _wrap_WrappedServiceSkel_castobj_set, METH_VARARGS, NULL},
	 { "WrappedServiceSkel_castobj_get", _wrap_WrappedServiceSkel_castobj_get, METH_O, NULL},
	 { "WrappedServiceSkel_GetPipe", _wrap_WrappedServiceSkel_GetPipe, METH_VARARGS, NULL},
	 { "WrappedServiceSkel_GetWire", _wrap_WrappedServiceSkel_GetWire, METH_VARARGS, NULL},
	 { "WrappedServiceSkel_WrappedDispatchEvent", _wrap_WrappedServiceSkel_WrappedDispatchEvent, METH_VARARGS, NULL},
	 { "WrappedServiceSkel_WrappedCallbackCall", _wrap_WrappedServiceSkel_WrappedCallbackCall, METH_VARARGS, NULL},
	 { "WrappedServiceSkel_RRGetNode", _wrap_WrappedServiceSkel_RRGetNode, METH_O, NULL},
	 { "WrappedServiceSkel_RegisterGeneratorServer", _wrap_WrappedServiceSkel_RegisterGeneratorServer, METH_VARARGS, NULL},
	 { "WrappedServiceSkel_GetServicePath", _wrap_WrappedServiceSkel_GetServicePath, METH_O, NULL},
	 { "WrappedServiceSkel_GetContext", _wrap_WrappedServiceSkel_GetContext, METH_O, NULL},
	 { "new_WrappedServiceSkel", _wrap_new_WrappedServiceSkel, METH_NOARGS, NULL},
	 { "delete_WrappedServiceSkel", _wrap_delete_WrappedServiceSkel, METH_O, NULL},
	 { "WrappedServiceSkel_swigregister", WrappedServiceSkel_swigregister, METH_O, NULL},
	 { "WrappedServiceSkel_swiginit", WrappedServiceSkel_swiginit, METH_VARARGS, NULL},
	 { "ServerServiceListenerDirector_objectheapid_set", _wrap_ServerServiceListenerDirector_objectheapid_set, METH_VARARGS, NULL},
	 { "ServerServiceListenerDirector_objectheapid_get", _wrap_ServerServiceListenerDirector_objectheapid_get, METH_O, NULL},
	 { "new_ServerServiceListenerDirector", _wrap_new_ServerServiceListenerDirector, METH_O, NULL},
	 { "ServerServiceListenerDirector_Callback", _wrap_ServerServiceListenerDirector_Callback, METH_VARARGS, NULL},
	 { "delete_ServerServiceListenerDirector", _wrap_delete_ServerServiceListenerDirector, METH_O, NULL},
	 { "disown_ServerServiceListenerDirector", _wrap_disown_ServerServiceListenerDirector, METH_O, NULL},
	 { "ServerServiceListenerDirector_swigregister", ServerServiceListenerDirector_swigregister, METH_O, NULL},
	 { "ServerServiceListenerDirector_swiginit", ServerServiceListenerDirector_swiginit, METH_VARARGS, NULL},
	 { "ServerContext_GetCurrentServicePath", _wrap_ServerContext_GetCurrentServicePath, METH_NOARGS, "\n"
		"\n"
		"Get the current object service path\n"
		"\n"
		"Returns the service path of the current object during a request or\n"
		"packet event.\n"
		"This is a thread-specific value and only\n"
		"valid during the initial request or packet event invocation.\n"
		"\n"
		":return: The current object service path\n"
		":rtype: str\n"
		"\n"
		""},
	 { "ServerContext__ReleaseServicePath", _wrap_ServerContext__ReleaseServicePath, METH_VARARGS, NULL},
	 { "ServerContext_GetCurrentServerContext", _wrap_ServerContext_GetCurrentServerContext, METH_NOARGS, "\n"
		"\n"
		"Get the current ServerContext\n"
		"\n"
		"Returns the current server context during a request or packet event.\n"
		"This is a thread-specific value and only\n"
		"valid during the initial request or packet event invocation.\n"
		"\n"
		":return: The current server context\n"
		":rtype: RobotRaconteur.ServerContext\n"
		"\n"
		""},
	 { "ServerContext_KickUser", _wrap_ServerContext_KickUser, METH_VARARGS, "\n"
		"\n"
		"Kicks a user with the specified username\n"
		"\n"
		"User must be authenticated.\n"
		"\n"
		":param username: The username to kick\n"
		":type username: str\n"
		"\n"
		""},
	 { "ServerContext__AddServerServiceListener", _wrap_ServerContext__AddServerServiceListener, METH_VARARGS, NULL},
	 { "ServerContext__SetServiceAttributes", _wrap_ServerContext__SetServiceAttributes, METH_VARARGS, NULL},
	 { "ServerContext_RequestObjectLock", _wrap_ServerContext_RequestObjectLock, METH_VARARGS, "\n"
		"\n"
		"Request an object lock on servicepath for user username\n"
		"\n"
		"This function handles incoming client requests, but may also be used\n"
		"by the service directly\n"
		"\n"
		":param servicepath: The service path to lock\n"
		":type servicepath: str\n"
		":param username: The name of the user owning the lock\n"
		":type username: str\n"
		"\n"
		""},
	 { "ServerContext_RequestClientObjectLock", _wrap_ServerContext_RequestClientObjectLock, METH_VARARGS, "\n"
		"\n"
		"Request a client lock on servicepath for a specific client connection\n"
		"\n"
		"This function handles incoming client requests, but may also be used\n"
		"by the service directly. Client object locks lock for a specific client connection,\n"
		"while client locks lock for a specific user. The specific client connection is\n"
		"specified using endpoint.\n"
		"\n"
		":param servicepath: The service path to lock\n"
		":type servicepath: str\n"
		":param username: The name of the user owning the lock\n"
		":type username: str\n"
		":param endpoint: The client endpoint ID of the client owning the lock\n"
		":type int:\n"
		"\n"
		""},
	 { "ServerContext_ReleaseObjectLock", _wrap_ServerContext_ReleaseObjectLock, METH_VARARGS, "\n"
		"\n"
		"Release a client lock on servicepath\n"
		"\n"
		"This function handles incoming client requests, but may also be used\n"
		"by the service directly. Client locks can be released by the user that\n"
		"created them if override_ is false, or by any user if override_ is true.\n"
		"\n"
		"The override_ parameter is set to true for client requests if the client has\n"
		"the \"objectlockoverride\" permission.\n"
		"\n"
		":param servicepath: The service path to release lock\n"
		":type str:\n"
		":param username: The username requesting the lock release\n"
		":type username:\n"
		":param override_: If False, only the creating username can release the lock. If True,\n"
		" any username can release the lock\n"
		":type override_: bool\n"
		"\n"
		""},
	 { "ServerContext_GetObjectLockUsername", _wrap_ServerContext_GetObjectLockUsername, METH_VARARGS, "\n"
		"\n"
		"Get the name of the user that has locked the specified service path\n"
		"\n"
		":param servicepath: The service path\n"
		":type servicepath: str\n"
		":return: The user owning the lock, or empty servicepath is not locked\n"
		":rtype: str\n"
		"\n"
		""},
	 { "ServerContext_GetNode", _wrap_ServerContext_GetNode, METH_O, "\n"
		"\n"
		"Get the Node\n"
		"\n"
		":rtype: RobotRaconteur.RobotRaconteurNode\n"
		"\n"
		""},
	 { "ServerContext_GetExtraImports", _wrap_ServerContext_GetExtraImports, METH_O, "\n"
		"\n"
		"Get the current list of extra service definition imports\n"
		"\n"
		":return: The list of extra imports\n"
		":rtype: List[str]\n"
		"\n"
		""},
	 { "ServerContext_AddExtraImport", _wrap_ServerContext_AddExtraImport, METH_VARARGS, "\n"
		"\n"
		"Add an extra service definition import\n"
		"\n"
		"Clients using dynamic typing will not automatically pull service definitions unless\n"
		"imported by the root object or an objref. If new \"struct\", \"po\", or \"namedarray\" types\n"
		"are introduced in a new service definition type without a corresponding object, an error will\n"
		"occur. Use AddExtraImport() to add the name of the new service definition to add it to the\n"
		"list of service definitions the client will pull.\n"
		"\n"
		"Service definition must have been registered using RobotRaconteurNode::RegisterServiceType()\n"
		"\n"
		":param import_: The name of the service definition\n"
		":type import_: str\n"
		"\n"
		""},
	 { "ServerContext_RemoveExtraImport", _wrap_ServerContext_RemoveExtraImport, METH_VARARGS, "\n"
		"\n"
		"Removes an extra import service definition registered with AddExtraImport()\n"
		"\n"
		"See AddExtraImport()\n"
		"\n"
		":param import_: The name of the service definition\n"
		":type import_: str\n"
		":return: True The service definition was removed, False The service definition was not found in the extra imports vector\n"
		":rtype: bool\n"
		"\n"
		""},
	 { "ServerContext_GetCandidateConnectionURLs", _wrap_ServerContext_GetCandidateConnectionURLs, METH_O, "\n"
		"\n"
		"Get the candidate connection URLs for this service. The candidate connection URLs are\n"
		"the URLs that can be used to connect to the service. The correct URL to use depends on the\n"
		"transport being used to connect to the service, and the network configuration of the client and service.\n"
		"\n"
		":return: The candidate connection URLs\n"
		":rtype: List[str]\n"
		"\n"
		""},
	 { "ServerContext_PrintCandidateConnectionURLs", _wrap_ServerContext_PrintCandidateConnectionURLs, METH_O, "\n"
		"\n"
		"Print the candidate connection URLs for this service. See GetCandidateConnectionURLs() for more information.\n"
		"\n"
		"\n"
		""},
	 { "ServerContext_LogCandidateConnectionURLs", _wrap_ServerContext_LogCandidateConnectionURLs, METH_VARARGS, "\n"
		"\n"
		"Log the candidate connection URLs for this service. See GetCandidateConnectionURLs() for more information.\n"
		"\n"
		":param level: The log level to use. Defaults to RobotRaconteur.LogLevel_Info\n"
		":type level: RobotRaconteur.LogLevel\n"
		"\n"
		""},
	 { "delete_ServerContext", _wrap_delete_ServerContext, METH_O, NULL},
	 { "ServerContext_swigregister", ServerContext_swigregister, METH_O, NULL},
	 { "AuthenticatedUser__GetUsername", _wrap_AuthenticatedUser__GetUsername, METH_O, NULL},
	 { "AuthenticatedUser__GetPrivileges", _wrap_AuthenticatedUser__GetPrivileges, METH_O, NULL},
	 { "AuthenticatedUser__GetProperties", _wrap_AuthenticatedUser__GetProperties, METH_O, NULL},
	 { "AuthenticatedUser__GetLoginTime", _wrap_AuthenticatedUser__GetLoginTime, METH_O, NULL},
	 { "AuthenticatedUser__GetLastAccessTime", _wrap_AuthenticatedUser__GetLastAccessTime, METH_O, NULL},
	 { "delete_AuthenticatedUser", _wrap_delete_AuthenticatedUser, METH_O, NULL},
	 { "AuthenticatedUser_swigregister", AuthenticatedUser_swigregister, METH_O, NULL},
	 { "ServerEndpoint_GetCurrentEndpoint", _wrap_ServerEndpoint_GetCurrentEndpoint, METH_NOARGS, "\n"
		"\n"
		"Returns the current server endpoint\n"
		"\n"
		"Returns the current server endpoint during a request or packet event.\n"
		"This is a thread-specific value and only valid during the initial\n"
		"request or packet event invocation.\n"
		"\n"
		"Throws InvalidOperationException if not during a request or packet event\n"
		"\n"
		":return: The current server endpoint id\n"
		":rtype: int\n"
		"\n"
		""},
	 { "ServerEndpoint_GetCurrentAuthenticatedUser", _wrap_ServerEndpoint_GetCurrentAuthenticatedUser, METH_NOARGS, "\n"
		"\n"
		"Returns the current authenticated user\n"
		"\n"
		"Users that have been authenticated have a corresponding\n"
		"AuthenticatedUser object associated with the ServerEndpoint.\n"
		"GetCurrentAuthenticatedUser() returns the AuthenticatedUser\n"
		"associated with the current ServerEndpoint during a request\n"
		"or packet event. This is a thread-specific value and only valid during\n"
		"the initial request or packet event invocation.\n"
		"\n"
		"Throws PermissionDeniedException or AuthenticationException\n"
		"if there is no AuthenticatedUser set in the current thread.\n"
		"\n"
		":return: The AuthenticatedUser\n"
		":rtype: RobotRaconteur.AuthenticatedUser\n"
		"\n"
		""},
	 { "delete_ServerEndpoint", _wrap_delete_ServerEndpoint, METH_O, NULL},
	 { "ServerEndpoint_swigregister", ServerEndpoint_swigregister, METH_O, NULL},
	 { "WrappedServiceSkelAsyncAdapter_MakeAsync", _wrap_WrappedServiceSkelAsyncAdapter_MakeAsync, METH_O, NULL},
	 { "WrappedServiceSkelAsyncAdapter_IsAsync", _wrap_WrappedServiceSkelAsyncAdapter_IsAsync, METH_O, NULL},
	 { "WrappedServiceSkelAsyncAdapter_End", _wrap_WrappedServiceSkelAsyncAdapter_End, METH_VARARGS, NULL},
	 { "delete_WrappedServiceSkelAsyncAdapter", _wrap_delete_WrappedServiceSkelAsyncAdapter, METH_O, NULL},
	 { "WrappedServiceSkelAsyncAdapter_swigregister", WrappedServiceSkelAsyncAdapter_swigregister, METH_O, NULL},
	 { "NativeUserAuthenticator__AuthenticateUser", _wrap_NativeUserAuthenticator__AuthenticateUser, METH_VARARGS, NULL},
	 { "delete_NativeUserAuthenticator", _wrap_delete_NativeUserAuthenticator, METH_O, NULL},
	 { "NativeUserAuthenticator_swigregister", NativeUserAuthenticator_swigregister, METH_O, NULL},
	 { "new_ServiceSecurityPolicy", _wrap_new_ServiceSecurityPolicy, METH_VARARGS, NULL},
	 { "delete_ServiceSecurityPolicy", _wrap_delete_ServiceSecurityPolicy, METH_O, NULL},
	 { "ServiceSecurityPolicy_swigregister", ServiceSecurityPolicy_swigregister, METH_O, NULL},
	 { "ServiceSecurityPolicy_swiginit", ServiceSecurityPolicy_swiginit, METH_VARARGS, NULL},
	 { "new_PasswordFileUserAuthenticator", _wrap_new_PasswordFileUserAuthenticator, METH_VARARGS, NULL},
	 { "delete_PasswordFileUserAuthenticator", _wrap_delete_PasswordFileUserAuthenticator, METH_O, NULL},
	 { "PasswordFileUserAuthenticator_swigregister", PasswordFileUserAuthenticator_swigregister, METH_O, NULL},
	 { "PasswordFileUserAuthenticator_swiginit", PasswordFileUserAuthenticator_swiginit, METH_VARARGS, NULL},
	 { "delete_WrappedUserAuthenticatorDirector", _wrap_delete_WrappedUserAuthenticatorDirector, METH_O, NULL},
	 { "WrappedUserAuthenticatorDirector_AuthenticateUser", _wrap_WrappedUserAuthenticatorDirector_AuthenticateUser, METH_VARARGS, NULL},
	 { "new_WrappedUserAuthenticatorDirector", _wrap_new_WrappedUserAuthenticatorDirector, METH_O, NULL},
	 { "disown_WrappedUserAuthenticatorDirector", _wrap_disown_WrappedUserAuthenticatorDirector, METH_O, NULL},
	 { "WrappedUserAuthenticatorDirector_swigregister", WrappedUserAuthenticatorDirector_swigregister, METH_O, NULL},
	 { "WrappedUserAuthenticatorDirector_swiginit", WrappedUserAuthenticatorDirector_swiginit, METH_VARARGS, NULL},
	 { "WrappedUserAuthenticator_SetRRDirector", _wrap_WrappedUserAuthenticator_SetRRDirector, METH_VARARGS, NULL},
	 { "new_WrappedUserAuthenticator", _wrap_new_WrappedUserAuthenticator, METH_NOARGS, NULL},
	 { "delete_WrappedUserAuthenticator", _wrap_delete_WrappedUserAuthenticator, METH_O, NULL},
	 { "WrappedUserAuthenticator_swigregister", WrappedUserAuthenticator_swigregister, METH_O, NULL},
	 { "WrappedUserAuthenticator_swiginit", WrappedUserAuthenticator_swiginit, METH_VARARGS, NULL},
	 { "delete_AsyncServiceInfo2VectorReturnDirector", _wrap_delete_AsyncServiceInfo2VectorReturnDirector, METH_O, NULL},
	 { "AsyncServiceInfo2VectorReturnDirector_handler", _wrap_AsyncServiceInfo2VectorReturnDirector_handler, METH_VARARGS, NULL},
	 { "new_AsyncServiceInfo2VectorReturnDirector", _wrap_new_AsyncServiceInfo2VectorReturnDirector, METH_O, NULL},
	 { "disown_AsyncServiceInfo2VectorReturnDirector", _wrap_disown_AsyncServiceInfo2VectorReturnDirector, METH_O, NULL},
	 { "AsyncServiceInfo2VectorReturnDirector_swigregister", AsyncServiceInfo2VectorReturnDirector_swigregister, METH_O, NULL},
	 { "AsyncServiceInfo2VectorReturnDirector_swiginit", AsyncServiceInfo2VectorReturnDirector_swiginit, METH_VARARGS, NULL},
	 { "delete_AsyncNodeInfo2VectorReturnDirector", _wrap_delete_AsyncNodeInfo2VectorReturnDirector, METH_O, NULL},
	 { "AsyncNodeInfo2VectorReturnDirector_handler", _wrap_AsyncNodeInfo2VectorReturnDirector_handler, METH_VARARGS, NULL},
	 { "new_AsyncNodeInfo2VectorReturnDirector", _wrap_new_AsyncNodeInfo2VectorReturnDirector, METH_O, NULL},
	 { "disown_AsyncNodeInfo2VectorReturnDirector", _wrap_disown_AsyncNodeInfo2VectorReturnDirector, METH_O, NULL},
	 { "AsyncNodeInfo2VectorReturnDirector_swigregister", AsyncNodeInfo2VectorReturnDirector_swigregister, METH_O, NULL},
	 { "AsyncNodeInfo2VectorReturnDirector_swiginit", AsyncNodeInfo2VectorReturnDirector_swiginit, METH_VARARGS, NULL},
	 { "ServiceInfo2Wrapped_Name_set", _wrap_ServiceInfo2Wrapped_Name_set, METH_VARARGS, NULL},
	 { "ServiceInfo2Wrapped_Name_get", _wrap_ServiceInfo2Wrapped_Name_get, METH_O, NULL},
	 { "ServiceInfo2Wrapped_RootObjectType_set", _wrap_ServiceInfo2Wrapped_RootObjectType_set, METH_VARARGS, NULL},
	 { "ServiceInfo2Wrapped_RootObjectType_get", _wrap_ServiceInfo2Wrapped_RootObjectType_get, METH_O, NULL},
	 { "ServiceInfo2Wrapped_RootObjectImplements_set", _wrap_ServiceInfo2Wrapped_RootObjectImplements_set, METH_VARARGS, NULL},
	 { "ServiceInfo2Wrapped_RootObjectImplements_get", _wrap_ServiceInfo2Wrapped_RootObjectImplements_get, METH_O, NULL},
	 { "ServiceInfo2Wrapped_ConnectionURL_set", _wrap_ServiceInfo2Wrapped_ConnectionURL_set, METH_VARARGS, NULL},
	 { "ServiceInfo2Wrapped_ConnectionURL_get", _wrap_ServiceInfo2Wrapped_ConnectionURL_get, METH_O, NULL},
	 { "ServiceInfo2Wrapped_Attributes_set", _wrap_ServiceInfo2Wrapped_Attributes_set, METH_VARARGS, NULL},
	 { "ServiceInfo2Wrapped_Attributes_get", _wrap_ServiceInfo2Wrapped_Attributes_get, METH_O, NULL},
	 { "ServiceInfo2Wrapped_NodeID_set", _wrap_ServiceInfo2Wrapped_NodeID_set, METH_VARARGS, NULL},
	 { "ServiceInfo2Wrapped_NodeID_get", _wrap_ServiceInfo2Wrapped_NodeID_get, METH_O, NULL},
	 { "ServiceInfo2Wrapped_NodeName_set", _wrap_ServiceInfo2Wrapped_NodeName_set, METH_VARARGS, NULL},
	 { "ServiceInfo2Wrapped_NodeName_get", _wrap_ServiceInfo2Wrapped_NodeName_get, METH_O, NULL},
	 { "new_ServiceInfo2Wrapped", _wrap_new_ServiceInfo2Wrapped, METH_NOARGS, NULL},
	 { "delete_ServiceInfo2Wrapped", _wrap_delete_ServiceInfo2Wrapped, METH_O, NULL},
	 { "ServiceInfo2Wrapped_swigregister", ServiceInfo2Wrapped_swigregister, METH_O, NULL},
	 { "ServiceInfo2Wrapped_swiginit", ServiceInfo2Wrapped_swiginit, METH_VARARGS, NULL},
	 { "WrappedFindServiceByType", _wrap_WrappedFindServiceByType, METH_VARARGS, NULL},
	 { "AsyncWrappedFindServiceByType", _wrap_AsyncWrappedFindServiceByType, METH_VARARGS, NULL},
	 { "vectorserviceinfo2wrapped_iterator", _wrap_vectorserviceinfo2wrapped_iterator, METH_O, NULL},
	 { "vectorserviceinfo2wrapped___nonzero__", _wrap_vectorserviceinfo2wrapped___nonzero__, METH_O, NULL},
	 { "vectorserviceinfo2wrapped___bool__", _wrap_vectorserviceinfo2wrapped___bool__, METH_O, NULL},
	 { "vectorserviceinfo2wrapped___len__", _wrap_vectorserviceinfo2wrapped___len__, METH_O, NULL},
	 { "vectorserviceinfo2wrapped___getslice__", _wrap_vectorserviceinfo2wrapped___getslice__, METH_VARARGS, NULL},
	 { "vectorserviceinfo2wrapped___setslice__", _wrap_vectorserviceinfo2wrapped___setslice__, METH_VARARGS, NULL},
	 { "vectorserviceinfo2wrapped___delslice__", _wrap_vectorserviceinfo2wrapped___delslice__, METH_VARARGS, NULL},
	 { "vectorserviceinfo2wrapped___delitem__", _wrap_vectorserviceinfo2wrapped___delitem__, METH_VARARGS, NULL},
	 { "vectorserviceinfo2wrapped___getitem__", _wrap_vectorserviceinfo2wrapped___getitem__, METH_VARARGS, NULL},
	 { "vectorserviceinfo2wrapped___setitem__", _wrap_vectorserviceinfo2wrapped___setitem__, METH_VARARGS, NULL},
	 { "vectorserviceinfo2wrapped_pop", _wrap_vectorserviceinfo2wrapped_pop, METH_O, NULL},
	 { "vectorserviceinfo2wrapped_append", _wrap_vectorserviceinfo2wrapped_append, METH_VARARGS, NULL},
	 { "vectorserviceinfo2wrapped_empty", _wrap_vectorserviceinfo2wrapped_empty, METH_O, NULL},
	 { "vectorserviceinfo2wrapped_size", _wrap_vectorserviceinfo2wrapped_size, METH_O, NULL},
	 { "vectorserviceinfo2wrapped_swap", _wrap_vectorserviceinfo2wrapped_swap, METH_VARARGS, NULL},
	 { "vectorserviceinfo2wrapped_begin", _wrap_vectorserviceinfo2wrapped_begin, METH_O, NULL},
	 { "vectorserviceinfo2wrapped_end", _wrap_vectorserviceinfo2wrapped_end, METH_O, NULL},
	 { "vectorserviceinfo2wrapped_rbegin", _wrap_vectorserviceinfo2wrapped_rbegin, METH_O, NULL},
	 { "vectorserviceinfo2wrapped_rend", _wrap_vectorserviceinfo2wrapped_rend, METH_O, NULL},
	 { "vectorserviceinfo2wrapped_clear", _wrap_vectorserviceinfo2wrapped_clear, METH_O, NULL},
	 { "vectorserviceinfo2wrapped_get_allocator", _wrap_vectorserviceinfo2wrapped_get_allocator, METH_O, NULL},
	 { "vectorserviceinfo2wrapped_pop_back", _wrap_vectorserviceinfo2wrapped_pop_back, METH_O, NULL},
	 { "vectorserviceinfo2wrapped_erase", _wrap_vectorserviceinfo2wrapped_erase, METH_VARARGS, NULL},
	 { "new_vectorserviceinfo2wrapped", _wrap_new_vectorserviceinfo2wrapped, METH_VARARGS, NULL},
	 { "vectorserviceinfo2wrapped_push_back", _wrap_vectorserviceinfo2wrapped_push_back, METH_VARARGS, NULL},
	 { "vectorserviceinfo2wrapped_front", _wrap_vectorserviceinfo2wrapped_front, METH_O, NULL},
	 { "vectorserviceinfo2wrapped_back", _wrap_vectorserviceinfo2wrapped_back, METH_O, NULL},
	 { "vectorserviceinfo2wrapped_assign", _wrap_vectorserviceinfo2wrapped_assign, METH_VARARGS, NULL},
	 { "vectorserviceinfo2wrapped_resize", _wrap_vectorserviceinfo2wrapped_resize, METH_VARARGS, NULL},
	 { "vectorserviceinfo2wrapped_insert", _wrap_vectorserviceinfo2wrapped_insert, METH_VARARGS, NULL},
	 { "vectorserviceinfo2wrapped_reserve", _wrap_vectorserviceinfo2wrapped_reserve, METH_VARARGS, NULL},
	 { "vectorserviceinfo2wrapped_capacity", _wrap_vectorserviceinfo2wrapped_capacity, METH_O, NULL},
	 { "delete_vectorserviceinfo2wrapped", _wrap_delete_vectorserviceinfo2wrapped, METH_O, NULL},
	 { "vectorserviceinfo2wrapped_swigregister", vectorserviceinfo2wrapped_swigregister, METH_O, NULL},
	 { "vectorserviceinfo2wrapped_swiginit", vectorserviceinfo2wrapped_swiginit, METH_VARARGS, NULL},
	 { "NodeInfo2_NodeID_set", _wrap_NodeInfo2_NodeID_set, METH_VARARGS, NULL},
	 { "NodeInfo2_NodeID_get", _wrap_NodeInfo2_NodeID_get, METH_O, NULL},
	 { "NodeInfo2_NodeName_set", _wrap_NodeInfo2_NodeName_set, METH_VARARGS, NULL},
	 { "NodeInfo2_NodeName_get", _wrap_NodeInfo2_NodeName_get, METH_O, NULL},
	 { "NodeInfo2_ConnectionURL_set", _wrap_NodeInfo2_ConnectionURL_set, METH_VARARGS, NULL},
	 { "NodeInfo2_ConnectionURL_get", _wrap_NodeInfo2_ConnectionURL_get, METH_O, NULL},
	 { "new_NodeInfo2", _wrap_new_NodeInfo2, METH_NOARGS, NULL},
	 { "delete_NodeInfo2", _wrap_delete_NodeInfo2, METH_O, NULL},
	 { "NodeInfo2_swigregister", NodeInfo2_swigregister, METH_O, NULL},
	 { "NodeInfo2_swiginit", NodeInfo2_swiginit, METH_VARARGS, NULL},
	 { "WrappedFindNodeByID", _wrap_WrappedFindNodeByID, METH_VARARGS, NULL},
	 { "WrappedFindNodeByName", _wrap_WrappedFindNodeByName, METH_VARARGS, NULL},
	 { "AsyncWrappedFindNodeByID", _wrap_AsyncWrappedFindNodeByID, METH_VARARGS, NULL},
	 { "AsyncWrappedFindNodeByName", _wrap_AsyncWrappedFindNodeByName, METH_VARARGS, NULL},
	 { "WrappedUpdateDetectedNodes", _wrap_WrappedUpdateDetectedNodes, METH_VARARGS, NULL},
	 { "AsyncWrappedUpdateDetectedNodes", _wrap_AsyncWrappedUpdateDetectedNodes, METH_VARARGS, NULL},
	 { "WrappedGetDetectedNodes", _wrap_WrappedGetDetectedNodes, METH_O, NULL},
	 { "WrappedGetDetectedNodeCacheInfo", _wrap_WrappedGetDetectedNodeCacheInfo, METH_VARARGS, NULL},
	 { "WrappedTryGetDetectedNodeCacheInfo", _wrap_WrappedTryGetDetectedNodeCacheInfo, METH_VARARGS, NULL},
	 { "vectornodeinfo2_iterator", _wrap_vectornodeinfo2_iterator, METH_O, NULL},
	 { "vectornodeinfo2___nonzero__", _wrap_vectornodeinfo2___nonzero__, METH_O, NULL},
	 { "vectornodeinfo2___bool__", _wrap_vectornodeinfo2___bool__, METH_O, NULL},
	 { "vectornodeinfo2___len__", _wrap_vectornodeinfo2___len__, METH_O, NULL},
	 { "vectornodeinfo2___getslice__", _wrap_vectornodeinfo2___getslice__, METH_VARARGS, NULL},
	 { "vectornodeinfo2___setslice__", _wrap_vectornodeinfo2___setslice__, METH_VARARGS, NULL},
	 { "vectornodeinfo2___delslice__", _wrap_vectornodeinfo2___delslice__, METH_VARARGS, NULL},
	 { "vectornodeinfo2___delitem__", _wrap_vectornodeinfo2___delitem__, METH_VARARGS, NULL},
	 { "vectornodeinfo2___getitem__", _wrap_vectornodeinfo2___getitem__, METH_VARARGS, NULL},
	 { "vectornodeinfo2___setitem__", _wrap_vectornodeinfo2___setitem__, METH_VARARGS, NULL},
	 { "vectornodeinfo2_pop", _wrap_vectornodeinfo2_pop, METH_O, NULL},
	 { "vectornodeinfo2_append", _wrap_vectornodeinfo2_append, METH_VARARGS, NULL},
	 { "vectornodeinfo2_empty", _wrap_vectornodeinfo2_empty, METH_O, NULL},
	 { "vectornodeinfo2_size", _wrap_vectornodeinfo2_size, METH_O, NULL},
	 { "vectornodeinfo2_swap", _wrap_vectornodeinfo2_swap, METH_VARARGS, NULL},
	 { "vectornodeinfo2_begin", _wrap_vectornodeinfo2_begin, METH_O, NULL},
	 { "vectornodeinfo2_end", _wrap_vectornodeinfo2_end, METH_O, NULL},
	 { "vectornodeinfo2_rbegin", _wrap_vectornodeinfo2_rbegin, METH_O, NULL},
	 { "vectornodeinfo2_rend", _wrap_vectornodeinfo2_rend, METH_O, NULL},
	 { "vectornodeinfo2_clear", _wrap_vectornodeinfo2_clear, METH_O, NULL},
	 { "vectornodeinfo2_get_allocator", _wrap_vectornodeinfo2_get_allocator, METH_O, NULL},
	 { "vectornodeinfo2_pop_back", _wrap_vectornodeinfo2_pop_back, METH_O, NULL},
	 { "vectornodeinfo2_erase", _wrap_vectornodeinfo2_erase, METH_VARARGS, NULL},
	 { "new_vectornodeinfo2", _wrap_new_vectornodeinfo2, METH_VARARGS, NULL},
	 { "vectornodeinfo2_push_back", _wrap_vectornodeinfo2_push_back, METH_VARARGS, NULL},
	 { "vectornodeinfo2_front", _wrap_vectornodeinfo2_front, METH_O, NULL},
	 { "vectornodeinfo2_back", _wrap_vectornodeinfo2_back, METH_O, NULL},
	 { "vectornodeinfo2_assign", _wrap_vectornodeinfo2_assign, METH_VARARGS, NULL},
	 { "vectornodeinfo2_resize", _wrap_vectornodeinfo2_resize, METH_VARARGS, NULL},
	 { "vectornodeinfo2_insert", _wrap_vectornodeinfo2_insert, METH_VARARGS, NULL},
	 { "vectornodeinfo2_reserve", _wrap_vectornodeinfo2_reserve, METH_VARARGS, NULL},
	 { "vectornodeinfo2_capacity", _wrap_vectornodeinfo2_capacity, METH_O, NULL},
	 { "delete_vectornodeinfo2", _wrap_delete_vectornodeinfo2, METH_O, NULL},
	 { "vectornodeinfo2_swigregister", vectornodeinfo2_swigregister, METH_O, NULL},
	 { "vectornodeinfo2_swiginit", vectornodeinfo2_swiginit, METH_VARARGS, NULL},
	 { "map_subscriptionserviceinfo2_iterator", _wrap_map_subscriptionserviceinfo2_iterator, METH_O, NULL},
	 { "map_subscriptionserviceinfo2___nonzero__", _wrap_map_subscriptionserviceinfo2___nonzero__, METH_O, NULL},
	 { "map_subscriptionserviceinfo2___bool__", _wrap_map_subscriptionserviceinfo2___bool__, METH_O, NULL},
	 { "map_subscriptionserviceinfo2___len__", _wrap_map_subscriptionserviceinfo2___len__, METH_O, NULL},
	 { "map_subscriptionserviceinfo2___getitem__", _wrap_map_subscriptionserviceinfo2___getitem__, METH_VARARGS, NULL},
	 { "map_subscriptionserviceinfo2___delitem__", _wrap_map_subscriptionserviceinfo2___delitem__, METH_VARARGS, NULL},
	 { "map_subscriptionserviceinfo2_has_key", _wrap_map_subscriptionserviceinfo2_has_key, METH_VARARGS, NULL},
	 { "map_subscriptionserviceinfo2_keys", _wrap_map_subscriptionserviceinfo2_keys, METH_O, NULL},
	 { "map_subscriptionserviceinfo2_values", _wrap_map_subscriptionserviceinfo2_values, METH_O, NULL},
	 { "map_subscriptionserviceinfo2_items", _wrap_map_subscriptionserviceinfo2_items, METH_O, NULL},
	 { "map_subscriptionserviceinfo2___contains__", _wrap_map_subscriptionserviceinfo2___contains__, METH_VARARGS, NULL},
	 { "map_subscriptionserviceinfo2_key_iterator", _wrap_map_subscriptionserviceinfo2_key_iterator, METH_O, NULL},
	 { "map_subscriptionserviceinfo2_value_iterator", _wrap_map_subscriptionserviceinfo2_value_iterator, METH_O, NULL},
	 { "map_subscriptionserviceinfo2___setitem__", _wrap_map_subscriptionserviceinfo2___setitem__, METH_VARARGS, NULL},
	 { "map_subscriptionserviceinfo2_asdict", _wrap_map_subscriptionserviceinfo2_asdict, METH_O, NULL},
	 { "new_map_subscriptionserviceinfo2", _wrap_new_map_subscriptionserviceinfo2, METH_VARARGS, NULL},
	 { "map_subscriptionserviceinfo2_empty", _wrap_map_subscriptionserviceinfo2_empty, METH_O, NULL},
	 { "map_subscriptionserviceinfo2_size", _wrap_map_subscriptionserviceinfo2_size, METH_O, NULL},
	 { "map_subscriptionserviceinfo2_swap", _wrap_map_subscriptionserviceinfo2_swap, METH_VARARGS, NULL},
	 { "map_subscriptionserviceinfo2_begin", _wrap_map_subscriptionserviceinfo2_begin, METH_O, NULL},
	 { "map_subscriptionserviceinfo2_end", _wrap_map_subscriptionserviceinfo2_end, METH_O, NULL},
	 { "map_subscriptionserviceinfo2_rbegin", _wrap_map_subscriptionserviceinfo2_rbegin, METH_O, NULL},
	 { "map_subscriptionserviceinfo2_rend", _wrap_map_subscriptionserviceinfo2_rend, METH_O, NULL},
	 { "map_subscriptionserviceinfo2_clear", _wrap_map_subscriptionserviceinfo2_clear, METH_O, NULL},
	 { "map_subscriptionserviceinfo2_get_allocator", _wrap_map_subscriptionserviceinfo2_get_allocator, METH_O, NULL},
	 { "map_subscriptionserviceinfo2_count", _wrap_map_subscriptionserviceinfo2_count, METH_VARARGS, NULL},
	 { "map_subscriptionserviceinfo2_erase", _wrap_map_subscriptionserviceinfo2_erase, METH_VARARGS, NULL},
	 { "map_subscriptionserviceinfo2_find", _wrap_map_subscriptionserviceinfo2_find, METH_VARARGS, NULL},
	 { "map_subscriptionserviceinfo2_lower_bound", _wrap_map_subscriptionserviceinfo2_lower_bound, METH_VARARGS, NULL},
	 { "map_subscriptionserviceinfo2_upper_bound", _wrap_map_subscriptionserviceinfo2_upper_bound, METH_VARARGS, NULL},
	 { "delete_map_subscriptionserviceinfo2", _wrap_delete_map_subscriptionserviceinfo2, METH_O, NULL},
	 { "map_subscriptionserviceinfo2_swigregister", map_subscriptionserviceinfo2_swigregister, METH_O, NULL},
	 { "map_subscriptionserviceinfo2_swiginit", map_subscriptionserviceinfo2_swiginit, METH_VARARGS, NULL},
	 { "map_subscriptionclients_iterator", _wrap_map_subscriptionclients_iterator, METH_O, NULL},
	 { "map_subscriptionclients___nonzero__", _wrap_map_subscriptionclients___nonzero__, METH_O, NULL},
	 { "map_subscriptionclients___bool__", _wrap_map_subscriptionclients___bool__, METH_O, NULL},
	 { "map_subscriptionclients___len__", _wrap_map_subscriptionclients___len__, METH_O, NULL},
	 { "map_subscriptionclients___getitem__", _wrap_map_subscriptionclients___getitem__, METH_VARARGS, NULL},
	 { "map_subscriptionclients___delitem__", _wrap_map_subscriptionclients___delitem__, METH_VARARGS, NULL},
	 { "map_subscriptionclients_has_key", _wrap_map_subscriptionclients_has_key, METH_VARARGS, NULL},
	 { "map_subscriptionclients_keys", _wrap_map_subscriptionclients_keys, METH_O, NULL},
	 { "map_subscriptionclients_values", _wrap_map_subscriptionclients_values, METH_O, NULL},
	 { "map_subscriptionclients_items", _wrap_map_subscriptionclients_items, METH_O, NULL},
	 { "map_subscriptionclients___contains__", _wrap_map_subscriptionclients___contains__, METH_VARARGS, NULL},
	 { "map_subscriptionclients_key_iterator", _wrap_map_subscriptionclients_key_iterator, METH_O, NULL},
	 { "map_subscriptionclients_value_iterator", _wrap_map_subscriptionclients_value_iterator, METH_O, NULL},
	 { "map_subscriptionclients___setitem__", _wrap_map_subscriptionclients___setitem__, METH_VARARGS, NULL},
	 { "map_subscriptionclients_asdict", _wrap_map_subscriptionclients_asdict, METH_O, NULL},
	 { "new_map_subscriptionclients", _wrap_new_map_subscriptionclients, METH_VARARGS, NULL},
	 { "map_subscriptionclients_empty", _wrap_map_subscriptionclients_empty, METH_O, NULL},
	 { "map_subscriptionclients_size", _wrap_map_subscriptionclients_size, METH_O, NULL},
	 { "map_subscriptionclients_swap", _wrap_map_subscriptionclients_swap, METH_VARARGS, NULL},
	 { "map_subscriptionclients_begin", _wrap_map_subscriptionclients_begin, METH_O, NULL},
	 { "map_subscriptionclients_end", _wrap_map_subscriptionclients_end, METH_O, NULL},
	 { "map_subscriptionclients_rbegin", _wrap_map_subscriptionclients_rbegin, METH_O, NULL},
	 { "map_subscriptionclients_rend", _wrap_map_subscriptionclients_rend, METH_O, NULL},
	 { "map_subscriptionclients_clear", _wrap_map_subscriptionclients_clear, METH_O, NULL},
	 { "map_subscriptionclients_get_allocator", _wrap_map_subscriptionclients_get_allocator, METH_O, NULL},
	 { "map_subscriptionclients_count", _wrap_map_subscriptionclients_count, METH_VARARGS, NULL},
	 { "map_subscriptionclients_erase", _wrap_map_subscriptionclients_erase, METH_VARARGS, NULL},
	 { "map_subscriptionclients_find", _wrap_map_subscriptionclients_find, METH_VARARGS, NULL},
	 { "map_subscriptionclients_lower_bound", _wrap_map_subscriptionclients_lower_bound, METH_VARARGS, NULL},
	 { "map_subscriptionclients_upper_bound", _wrap_map_subscriptionclients_upper_bound, METH_VARARGS, NULL},
	 { "delete_map_subscriptionclients", _wrap_delete_map_subscriptionclients, METH_O, NULL},
	 { "map_subscriptionclients_swigregister", map_subscriptionclients_swigregister, METH_O, NULL},
	 { "map_subscriptionclients_swiginit", map_subscriptionclients_swiginit, METH_VARARGS, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_iterator", _wrap_vectorptr_wrappedservicesubscriptionnode_iterator, METH_O, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode___nonzero__", _wrap_vectorptr_wrappedservicesubscriptionnode___nonzero__, METH_O, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode___bool__", _wrap_vectorptr_wrappedservicesubscriptionnode___bool__, METH_O, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode___len__", _wrap_vectorptr_wrappedservicesubscriptionnode___len__, METH_O, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode___getslice__", _wrap_vectorptr_wrappedservicesubscriptionnode___getslice__, METH_VARARGS, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode___setslice__", _wrap_vectorptr_wrappedservicesubscriptionnode___setslice__, METH_VARARGS, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode___delslice__", _wrap_vectorptr_wrappedservicesubscriptionnode___delslice__, METH_VARARGS, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode___delitem__", _wrap_vectorptr_wrappedservicesubscriptionnode___delitem__, METH_VARARGS, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode___getitem__", _wrap_vectorptr_wrappedservicesubscriptionnode___getitem__, METH_VARARGS, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode___setitem__", _wrap_vectorptr_wrappedservicesubscriptionnode___setitem__, METH_VARARGS, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_pop", _wrap_vectorptr_wrappedservicesubscriptionnode_pop, METH_O, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_append", _wrap_vectorptr_wrappedservicesubscriptionnode_append, METH_VARARGS, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_empty", _wrap_vectorptr_wrappedservicesubscriptionnode_empty, METH_O, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_size", _wrap_vectorptr_wrappedservicesubscriptionnode_size, METH_O, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_swap", _wrap_vectorptr_wrappedservicesubscriptionnode_swap, METH_VARARGS, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_begin", _wrap_vectorptr_wrappedservicesubscriptionnode_begin, METH_O, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_end", _wrap_vectorptr_wrappedservicesubscriptionnode_end, METH_O, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_rbegin", _wrap_vectorptr_wrappedservicesubscriptionnode_rbegin, METH_O, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_rend", _wrap_vectorptr_wrappedservicesubscriptionnode_rend, METH_O, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_clear", _wrap_vectorptr_wrappedservicesubscriptionnode_clear, METH_O, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_get_allocator", _wrap_vectorptr_wrappedservicesubscriptionnode_get_allocator, METH_O, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_pop_back", _wrap_vectorptr_wrappedservicesubscriptionnode_pop_back, METH_O, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_erase", _wrap_vectorptr_wrappedservicesubscriptionnode_erase, METH_VARARGS, NULL},
	 { "new_vectorptr_wrappedservicesubscriptionnode", _wrap_new_vectorptr_wrappedservicesubscriptionnode, METH_VARARGS, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_push_back", _wrap_vectorptr_wrappedservicesubscriptionnode_push_back, METH_VARARGS, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_front", _wrap_vectorptr_wrappedservicesubscriptionnode_front, METH_O, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_back", _wrap_vectorptr_wrappedservicesubscriptionnode_back, METH_O, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_assign", _wrap_vectorptr_wrappedservicesubscriptionnode_assign, METH_VARARGS, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_resize", _wrap_vectorptr_wrappedservicesubscriptionnode_resize, METH_VARARGS, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_insert", _wrap_vectorptr_wrappedservicesubscriptionnode_insert, METH_VARARGS, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_reserve", _wrap_vectorptr_wrappedservicesubscriptionnode_reserve, METH_VARARGS, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_capacity", _wrap_vectorptr_wrappedservicesubscriptionnode_capacity, METH_O, NULL},
	 { "delete_vectorptr_wrappedservicesubscriptionnode", _wrap_delete_vectorptr_wrappedservicesubscriptionnode, METH_O, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_swigregister", vectorptr_wrappedservicesubscriptionnode_swigregister, METH_O, NULL},
	 { "vectorptr_wrappedservicesubscriptionnode_swiginit", vectorptr_wrappedservicesubscriptionnode_swiginit, METH_VARARGS, NULL},
	 { "vectorptr_subscriptionclientid_iterator", _wrap_vectorptr_subscriptionclientid_iterator, METH_O, NULL},
	 { "vectorptr_subscriptionclientid___nonzero__", _wrap_vectorptr_subscriptionclientid___nonzero__, METH_O, NULL},
	 { "vectorptr_subscriptionclientid___bool__", _wrap_vectorptr_subscriptionclientid___bool__, METH_O, NULL},
	 { "vectorptr_subscriptionclientid___len__", _wrap_vectorptr_subscriptionclientid___len__, METH_O, NULL},
	 { "vectorptr_subscriptionclientid___getslice__", _wrap_vectorptr_subscriptionclientid___getslice__, METH_VARARGS, NULL},
	 { "vectorptr_subscriptionclientid___setslice__", _wrap_vectorptr_subscriptionclientid___setslice__, METH_VARARGS, NULL},
	 { "vectorptr_subscriptionclientid___delslice__", _wrap_vectorptr_subscriptionclientid___delslice__, METH_VARARGS, NULL},
	 { "vectorptr_subscriptionclientid___delitem__", _wrap_vectorptr_subscriptionclientid___delitem__, METH_VARARGS, NULL},
	 { "vectorptr_subscriptionclientid___getitem__", _wrap_vectorptr_subscriptionclientid___getitem__, METH_VARARGS, NULL},
	 { "vectorptr_subscriptionclientid___setitem__", _wrap_vectorptr_subscriptionclientid___setitem__, METH_VARARGS, NULL},
	 { "vectorptr_subscriptionclientid_pop", _wrap_vectorptr_subscriptionclientid_pop, METH_O, NULL},
	 { "vectorptr_subscriptionclientid_append", _wrap_vectorptr_subscriptionclientid_append, METH_VARARGS, NULL},
	 { "vectorptr_subscriptionclientid_empty", _wrap_vectorptr_subscriptionclientid_empty, METH_O, NULL},
	 { "vectorptr_subscriptionclientid_size", _wrap_vectorptr_subscriptionclientid_size, METH_O, NULL},
	 { "vectorptr_subscriptionclientid_swap", _wrap_vectorptr_subscriptionclientid_swap, METH_VARARGS, NULL},
	 { "vectorptr_subscriptionclientid_begin", _wrap_vectorptr_subscriptionclientid_begin, METH_O, NULL},
	 { "vectorptr_subscriptionclientid_end", _wrap_vectorptr_subscriptionclientid_end, METH_O, NULL},
	 { "vectorptr_subscriptionclientid_rbegin", _wrap_vectorptr_subscriptionclientid_rbegin, METH_O, NULL},
	 { "vectorptr_subscriptionclientid_rend", _wrap_vectorptr_subscriptionclientid_rend, METH_O, NULL},
	 { "vectorptr_subscriptionclientid_clear", _wrap_vectorptr_subscriptionclientid_clear, METH_O, NULL},
	 { "vectorptr_subscriptionclientid_get_allocator", _wrap_vectorptr_subscriptionclientid_get_allocator, METH_O, NULL},
	 { "vectorptr_subscriptionclientid_pop_back", _wrap_vectorptr_subscriptionclientid_pop_back, METH_O, NULL},
	 { "vectorptr_subscriptionclientid_erase", _wrap_vectorptr_subscriptionclientid_erase, METH_VARARGS, NULL},
	 { "new_vectorptr_subscriptionclientid", _wrap_new_vectorptr_subscriptionclientid, METH_VARARGS, NULL},
	 { "vectorptr_subscriptionclientid_push_back", _wrap_vectorptr_subscriptionclientid_push_back, METH_VARARGS, NULL},
	 { "vectorptr_subscriptionclientid_front", _wrap_vectorptr_subscriptionclientid_front, METH_O, NULL},
	 { "vectorptr_subscriptionclientid_back", _wrap_vectorptr_subscriptionclientid_back, METH_O, NULL},
	 { "vectorptr_subscriptionclientid_assign", _wrap_vectorptr_subscriptionclientid_assign, METH_VARARGS, NULL},
	 { "vectorptr_subscriptionclientid_resize", _wrap_vectorptr_subscriptionclientid_resize, METH_VARARGS, NULL},
	 { "vectorptr_subscriptionclientid_insert", _wrap_vectorptr_subscriptionclientid_insert, METH_VARARGS, NULL},
	 { "vectorptr_subscriptionclientid_reserve", _wrap_vectorptr_subscriptionclientid_reserve, METH_VARARGS, NULL},
	 { "vectorptr_subscriptionclientid_capacity", _wrap_vectorptr_subscriptionclientid_capacity, METH_O, NULL},
	 { "delete_vectorptr_subscriptionclientid", _wrap_delete_vectorptr_subscriptionclientid, METH_O, NULL},
	 { "vectorptr_subscriptionclientid_swigregister", vectorptr_subscriptionclientid_swigregister, METH_O, NULL},
	 { "vectorptr_subscriptionclientid_swiginit", vectorptr_subscriptionclientid_swiginit, METH_VARARGS, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_iterator", _wrap_vector_wrappedservicesubscriptionmanagerdetails_iterator, METH_O, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails___nonzero__", _wrap_vector_wrappedservicesubscriptionmanagerdetails___nonzero__, METH_O, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails___bool__", _wrap_vector_wrappedservicesubscriptionmanagerdetails___bool__, METH_O, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails___len__", _wrap_vector_wrappedservicesubscriptionmanagerdetails___len__, METH_O, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails___getslice__", _wrap_vector_wrappedservicesubscriptionmanagerdetails___getslice__, METH_VARARGS, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails___setslice__", _wrap_vector_wrappedservicesubscriptionmanagerdetails___setslice__, METH_VARARGS, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails___delslice__", _wrap_vector_wrappedservicesubscriptionmanagerdetails___delslice__, METH_VARARGS, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails___delitem__", _wrap_vector_wrappedservicesubscriptionmanagerdetails___delitem__, METH_VARARGS, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails___getitem__", _wrap_vector_wrappedservicesubscriptionmanagerdetails___getitem__, METH_VARARGS, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails___setitem__", _wrap_vector_wrappedservicesubscriptionmanagerdetails___setitem__, METH_VARARGS, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_pop", _wrap_vector_wrappedservicesubscriptionmanagerdetails_pop, METH_O, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_append", _wrap_vector_wrappedservicesubscriptionmanagerdetails_append, METH_VARARGS, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_empty", _wrap_vector_wrappedservicesubscriptionmanagerdetails_empty, METH_O, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_size", _wrap_vector_wrappedservicesubscriptionmanagerdetails_size, METH_O, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_swap", _wrap_vector_wrappedservicesubscriptionmanagerdetails_swap, METH_VARARGS, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_begin", _wrap_vector_wrappedservicesubscriptionmanagerdetails_begin, METH_O, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_end", _wrap_vector_wrappedservicesubscriptionmanagerdetails_end, METH_O, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_rbegin", _wrap_vector_wrappedservicesubscriptionmanagerdetails_rbegin, METH_O, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_rend", _wrap_vector_wrappedservicesubscriptionmanagerdetails_rend, METH_O, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_clear", _wrap_vector_wrappedservicesubscriptionmanagerdetails_clear, METH_O, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_get_allocator", _wrap_vector_wrappedservicesubscriptionmanagerdetails_get_allocator, METH_O, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_pop_back", _wrap_vector_wrappedservicesubscriptionmanagerdetails_pop_back, METH_O, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_erase", _wrap_vector_wrappedservicesubscriptionmanagerdetails_erase, METH_VARARGS, NULL},
	 { "new_vector_wrappedservicesubscriptionmanagerdetails", _wrap_new_vector_wrappedservicesubscriptionmanagerdetails, METH_VARARGS, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_push_back", _wrap_vector_wrappedservicesubscriptionmanagerdetails_push_back, METH_VARARGS, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_front", _wrap_vector_wrappedservicesubscriptionmanagerdetails_front, METH_O, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_back", _wrap_vector_wrappedservicesubscriptionmanagerdetails_back, METH_O, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_assign", _wrap_vector_wrappedservicesubscriptionmanagerdetails_assign, METH_VARARGS, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_resize", _wrap_vector_wrappedservicesubscriptionmanagerdetails_resize, METH_VARARGS, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_insert", _wrap_vector_wrappedservicesubscriptionmanagerdetails_insert, METH_VARARGS, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_reserve", _wrap_vector_wrappedservicesubscriptionmanagerdetails_reserve, METH_VARARGS, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_capacity", _wrap_vector_wrappedservicesubscriptionmanagerdetails_capacity, METH_O, NULL},
	 { "delete_vector_wrappedservicesubscriptionmanagerdetails", _wrap_delete_vector_wrappedservicesubscriptionmanagerdetails, METH_O, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_swigregister", vector_wrappedservicesubscriptionmanagerdetails_swigregister, METH_O, NULL},
	 { "vector_wrappedservicesubscriptionmanagerdetails_swiginit", vector_wrappedservicesubscriptionmanagerdetails_swiginit, METH_VARARGS, NULL},
	 { "ServiceSubscriptionFilterAttribute_Name_set", _wrap_ServiceSubscriptionFilterAttribute_Name_set, METH_VARARGS, "(str) The attribute name. Empty for no name"},
	 { "ServiceSubscriptionFilterAttribute_Name_get", _wrap_ServiceSubscriptionFilterAttribute_Name_get, METH_O, "(str) The attribute name. Empty for no name"},
	 { "ServiceSubscriptionFilterAttribute_Value_set", _wrap_ServiceSubscriptionFilterAttribute_Value_set, METH_VARARGS, "(str) The string value for the attribute"},
	 { "ServiceSubscriptionFilterAttribute_Value_get", _wrap_ServiceSubscriptionFilterAttribute_Value_get, METH_O, "(str) The string value for the attribute"},
	 { "ServiceSubscriptionFilterAttribute_UseRegex_set", _wrap_ServiceSubscriptionFilterAttribute_UseRegex_set, METH_VARARGS, NULL},
	 { "ServiceSubscriptionFilterAttribute_UseRegex_get", _wrap_ServiceSubscriptionFilterAttribute_UseRegex_get, METH_O, NULL},
	 { "new_ServiceSubscriptionFilterAttribute", _wrap_new_ServiceSubscriptionFilterAttribute, METH_VARARGS, NULL},
	 { "ServiceSubscriptionFilterAttribute_IsMatch", _wrap_ServiceSubscriptionFilterAttribute_IsMatch, METH_VARARGS, NULL},
	 { "delete_ServiceSubscriptionFilterAttribute", _wrap_delete_ServiceSubscriptionFilterAttribute, METH_O, NULL},
	 { "ServiceSubscriptionFilterAttribute_swigregister", ServiceSubscriptionFilterAttribute_swigregister, METH_O, NULL},
	 { "ServiceSubscriptionFilterAttribute_swiginit", ServiceSubscriptionFilterAttribute_swiginit, METH_VARARGS, NULL},
	 { "CreateServiceSubscriptionFilterAttributeRegex", _wrap_CreateServiceSubscriptionFilterAttributeRegex, METH_VARARGS, "\n"
		"\n"
		"\n"
		"Create a ServiceSubscriptionFilterAttribute from a regex string\n"
		"\n"
		":param regex_value: The regex string\n"
		":type regex_value: str\n"
		":return: The ServiceSubscriptionFilterAttribute\n"
		":rtype: RobotRaconteur.ServiceSubscriptionFilterAttribute\n"
		"\n"
		""},
	 { "CreateServiceSubscriptionFilterAttributeIdentifier", _wrap_CreateServiceSubscriptionFilterAttributeIdentifier, METH_VARARGS, NULL},
	 { "CreateServiceSubscriptionFilterAttributeCombinedIdentifier", _wrap_CreateServiceSubscriptionFilterAttributeCombinedIdentifier, METH_O, NULL},
	 { "vector_subscriptionattribute_iterator", _wrap_vector_subscriptionattribute_iterator, METH_O, NULL},
	 { "vector_subscriptionattribute___nonzero__", _wrap_vector_subscriptionattribute___nonzero__, METH_O, NULL},
	 { "vector_subscriptionattribute___bool__", _wrap_vector_subscriptionattribute___bool__, METH_O, NULL},
	 { "vector_subscriptionattribute___len__", _wrap_vector_subscriptionattribute___len__, METH_O, NULL},
	 { "vector_subscriptionattribute___getslice__", _wrap_vector_subscriptionattribute___getslice__, METH_VARARGS, NULL},
	 { "vector_subscriptionattribute___setslice__", _wrap_vector_subscriptionattribute___setslice__, METH_VARARGS, NULL},
	 { "vector_subscriptionattribute___delslice__", _wrap_vector_subscriptionattribute___delslice__, METH_VARARGS, NULL},
	 { "vector_subscriptionattribute___delitem__", _wrap_vector_subscriptionattribute___delitem__, METH_VARARGS, NULL},
	 { "vector_subscriptionattribute___getitem__", _wrap_vector_subscriptionattribute___getitem__, METH_VARARGS, NULL},
	 { "vector_subscriptionattribute___setitem__", _wrap_vector_subscriptionattribute___setitem__, METH_VARARGS, NULL},
	 { "vector_subscriptionattribute_pop", _wrap_vector_subscriptionattribute_pop, METH_O, NULL},
	 { "vector_subscriptionattribute_append", _wrap_vector_subscriptionattribute_append, METH_VARARGS, NULL},
	 { "vector_subscriptionattribute_empty", _wrap_vector_subscriptionattribute_empty, METH_O, NULL},
	 { "vector_subscriptionattribute_size", _wrap_vector_subscriptionattribute_size, METH_O, NULL},
	 { "vector_subscriptionattribute_swap", _wrap_vector_subscriptionattribute_swap, METH_VARARGS, NULL},
	 { "vector_subscriptionattribute_begin", _wrap_vector_subscriptionattribute_begin, METH_O, NULL},
	 { "vector_subscriptionattribute_end", _wrap_vector_subscriptionattribute_end, METH_O, NULL},
	 { "vector_subscriptionattribute_rbegin", _wrap_vector_subscriptionattribute_rbegin, METH_O, NULL},
	 { "vector_subscriptionattribute_rend", _wrap_vector_subscriptionattribute_rend, METH_O, NULL},
	 { "vector_subscriptionattribute_clear", _wrap_vector_subscriptionattribute_clear, METH_O, NULL},
	 { "vector_subscriptionattribute_get_allocator", _wrap_vector_subscriptionattribute_get_allocator, METH_O, NULL},
	 { "vector_subscriptionattribute_pop_back", _wrap_vector_subscriptionattribute_pop_back, METH_O, NULL},
	 { "vector_subscriptionattribute_erase", _wrap_vector_subscriptionattribute_erase, METH_VARARGS, NULL},
	 { "new_vector_subscriptionattribute", _wrap_new_vector_subscriptionattribute, METH_VARARGS, NULL},
	 { "vector_subscriptionattribute_push_back", _wrap_vector_subscriptionattribute_push_back, METH_VARARGS, NULL},
	 { "vector_subscriptionattribute_front", _wrap_vector_subscriptionattribute_front, METH_O, NULL},
	 { "vector_subscriptionattribute_back", _wrap_vector_subscriptionattribute_back, METH_O, NULL},
	 { "vector_subscriptionattribute_assign", _wrap_vector_subscriptionattribute_assign, METH_VARARGS, NULL},
	 { "vector_subscriptionattribute_resize", _wrap_vector_subscriptionattribute_resize, METH_VARARGS, NULL},
	 { "vector_subscriptionattribute_insert", _wrap_vector_subscriptionattribute_insert, METH_VARARGS, NULL},
	 { "vector_subscriptionattribute_reserve", _wrap_vector_subscriptionattribute_reserve, METH_VARARGS, NULL},
	 { "vector_subscriptionattribute_capacity", _wrap_vector_subscriptionattribute_capacity, METH_O, NULL},
	 { "delete_vector_subscriptionattribute", _wrap_delete_vector_subscriptionattribute, METH_O, NULL},
	 { "vector_subscriptionattribute_swigregister", vector_subscriptionattribute_swigregister, METH_O, NULL},
	 { "vector_subscriptionattribute_swiginit", vector_subscriptionattribute_swiginit, METH_VARARGS, NULL},
	 { "map_subscriptionattribute_iterator", _wrap_map_subscriptionattribute_iterator, METH_O, NULL},
	 { "map_subscriptionattribute___nonzero__", _wrap_map_subscriptionattribute___nonzero__, METH_O, NULL},
	 { "map_subscriptionattribute___bool__", _wrap_map_subscriptionattribute___bool__, METH_O, NULL},
	 { "map_subscriptionattribute___len__", _wrap_map_subscriptionattribute___len__, METH_O, NULL},
	 { "map_subscriptionattribute___getitem__", _wrap_map_subscriptionattribute___getitem__, METH_VARARGS, NULL},
	 { "map_subscriptionattribute___delitem__", _wrap_map_subscriptionattribute___delitem__, METH_VARARGS, NULL},
	 { "map_subscriptionattribute_has_key", _wrap_map_subscriptionattribute_has_key, METH_VARARGS, NULL},
	 { "map_subscriptionattribute_keys", _wrap_map_subscriptionattribute_keys, METH_O, NULL},
	 { "map_subscriptionattribute_values", _wrap_map_subscriptionattribute_values, METH_O, NULL},
	 { "map_subscriptionattribute_items", _wrap_map_subscriptionattribute_items, METH_O, NULL},
	 { "map_subscriptionattribute___contains__", _wrap_map_subscriptionattribute___contains__, METH_VARARGS, NULL},
	 { "map_subscriptionattribute_key_iterator", _wrap_map_subscriptionattribute_key_iterator, METH_O, NULL},
	 { "map_subscriptionattribute_value_iterator", _wrap_map_subscriptionattribute_value_iterator, METH_O, NULL},
	 { "map_subscriptionattribute___setitem__", _wrap_map_subscriptionattribute___setitem__, METH_VARARGS, NULL},
	 { "map_subscriptionattribute_asdict", _wrap_map_subscriptionattribute_asdict, METH_O, NULL},
	 { "new_map_subscriptionattribute", _wrap_new_map_subscriptionattribute, METH_VARARGS, NULL},
	 { "map_subscriptionattribute_empty", _wrap_map_subscriptionattribute_empty, METH_O, NULL},
	 { "map_subscriptionattribute_size", _wrap_map_subscriptionattribute_size, METH_O, NULL},
	 { "map_subscriptionattribute_swap", _wrap_map_subscriptionattribute_swap, METH_VARARGS, NULL},
	 { "map_subscriptionattribute_begin", _wrap_map_subscriptionattribute_begin, METH_O, NULL},
	 { "map_subscriptionattribute_end", _wrap_map_subscriptionattribute_end, METH_O, NULL},
	 { "map_subscriptionattribute_rbegin", _wrap_map_subscriptionattribute_rbegin, METH_O, NULL},
	 { "map_subscriptionattribute_rend", _wrap_map_subscriptionattribute_rend, METH_O, NULL},
	 { "map_subscriptionattribute_clear", _wrap_map_subscriptionattribute_clear, METH_O, NULL},
	 { "map_subscriptionattribute_get_allocator", _wrap_map_subscriptionattribute_get_allocator, METH_O, NULL},
	 { "map_subscriptionattribute_count", _wrap_map_subscriptionattribute_count, METH_VARARGS, NULL},
	 { "map_subscriptionattribute_erase", _wrap_map_subscriptionattribute_erase, METH_VARARGS, NULL},
	 { "map_subscriptionattribute_find", _wrap_map_subscriptionattribute_find, METH_VARARGS, NULL},
	 { "map_subscriptionattribute_lower_bound", _wrap_map_subscriptionattribute_lower_bound, METH_VARARGS, NULL},
	 { "map_subscriptionattribute_upper_bound", _wrap_map_subscriptionattribute_upper_bound, METH_VARARGS, NULL},
	 { "delete_map_subscriptionattribute", _wrap_delete_map_subscriptionattribute, METH_O, NULL},
	 { "map_subscriptionattribute_swigregister", map_subscriptionattribute_swigregister, METH_O, NULL},
	 { "map_subscriptionattribute_swiginit", map_subscriptionattribute_swiginit, METH_VARARGS, NULL},
	 { "ServiceSubscriptionFilterAttributeGroup_Attributes_set", _wrap_ServiceSubscriptionFilterAttributeGroup_Attributes_set, METH_VARARGS, "(list) The list of attributes in the group"},
	 { "ServiceSubscriptionFilterAttributeGroup_Attributes_get", _wrap_ServiceSubscriptionFilterAttributeGroup_Attributes_get, METH_O, "(list) The list of attributes in the group"},
	 { "ServiceSubscriptionFilterAttributeGroup_Groups_set", _wrap_ServiceSubscriptionFilterAttributeGroup_Groups_set, METH_VARARGS, "(list) The nested groups in the group"},
	 { "ServiceSubscriptionFilterAttributeGroup_Groups_get", _wrap_ServiceSubscriptionFilterAttributeGroup_Groups_get, METH_O, "(list) The nested groups in the group"},
	 { "ServiceSubscriptionFilterAttributeGroup_Operation_set", _wrap_ServiceSubscriptionFilterAttributeGroup_Operation_set, METH_VARARGS, "(RobotRaconteur.ServiceSubscriptionFilterAttributeGroupOperation) The operation to use to combine the attributes"},
	 { "ServiceSubscriptionFilterAttributeGroup_Operation_get", _wrap_ServiceSubscriptionFilterAttributeGroup_Operation_get, METH_O, "(RobotRaconteur.ServiceSubscriptionFilterAttributeGroupOperation) The operation to use to combine the attributes"},
	 { "ServiceSubscriptionFilterAttributeGroup_SplitStringAttribute_set", _wrap_ServiceSubscriptionFilterAttributeGroup_SplitStringAttribute_set, METH_VARARGS, "(bool) True if string attributes will be split into a list with delimiter (default ',')"},
	 { "ServiceSubscriptionFilterAttributeGroup_SplitStringAttribute_get", _wrap_ServiceSubscriptionFilterAttributeGroup_SplitStringAttribute_get, METH_O, "(bool) True if string attributes will be split into a list with delimiter (default ',')"},
	 { "ServiceSubscriptionFilterAttributeGroup_SplitStringDelimiter_set", _wrap_ServiceSubscriptionFilterAttributeGroup_SplitStringDelimiter_set, METH_VARARGS, "(str) The delimiter to use for splitting string attributes (default ',')"},
	 { "ServiceSubscriptionFilterAttributeGroup_SplitStringDelimiter_get", _wrap_ServiceSubscriptionFilterAttributeGroup_SplitStringDelimiter_get, METH_O, "(str) The delimiter to use for splitting string attributes (default ',')"},
	 { "new_ServiceSubscriptionFilterAttributeGroup", _wrap_new_ServiceSubscriptionFilterAttributeGroup, METH_VARARGS, NULL},
	 { "ServiceSubscriptionFilterAttributeGroup_IsMatch", _wrap_ServiceSubscriptionFilterAttributeGroup_IsMatch, METH_VARARGS, NULL},
	 { "delete_ServiceSubscriptionFilterAttributeGroup", _wrap_delete_ServiceSubscriptionFilterAttributeGroup, METH_O, NULL},
	 { "ServiceSubscriptionFilterAttributeGroup_swigregister", ServiceSubscriptionFilterAttributeGroup_swigregister, METH_O, NULL},
	 { "ServiceSubscriptionFilterAttributeGroup_swiginit", ServiceSubscriptionFilterAttributeGroup_swiginit, METH_VARARGS, NULL},
	 { "map_subscriptionattributegroup_iterator", _wrap_map_subscriptionattributegroup_iterator, METH_O, NULL},
	 { "map_subscriptionattributegroup___nonzero__", _wrap_map_subscriptionattributegroup___nonzero__, METH_O, NULL},
	 { "map_subscriptionattributegroup___bool__", _wrap_map_subscriptionattributegroup___bool__, METH_O, NULL},
	 { "map_subscriptionattributegroup___len__", _wrap_map_subscriptionattributegroup___len__, METH_O, NULL},
	 { "map_subscriptionattributegroup___getitem__", _wrap_map_subscriptionattributegroup___getitem__, METH_VARARGS, NULL},
	 { "map_subscriptionattributegroup___delitem__", _wrap_map_subscriptionattributegroup___delitem__, METH_VARARGS, NULL},
	 { "map_subscriptionattributegroup_has_key", _wrap_map_subscriptionattributegroup_has_key, METH_VARARGS, NULL},
	 { "map_subscriptionattributegroup_keys", _wrap_map_subscriptionattributegroup_keys, METH_O, NULL},
	 { "map_subscriptionattributegroup_values", _wrap_map_subscriptionattributegroup_values, METH_O, NULL},
	 { "map_subscriptionattributegroup_items", _wrap_map_subscriptionattributegroup_items, METH_O, NULL},
	 { "map_subscriptionattributegroup___contains__", _wrap_map_subscriptionattributegroup___contains__, METH_VARARGS, NULL},
	 { "map_subscriptionattributegroup_key_iterator", _wrap_map_subscriptionattributegroup_key_iterator, METH_O, NULL},
	 { "map_subscriptionattributegroup_value_iterator", _wrap_map_subscriptionattributegroup_value_iterator, METH_O, NULL},
	 { "map_subscriptionattributegroup___setitem__", _wrap_map_subscriptionattributegroup___setitem__, METH_VARARGS, NULL},
	 { "map_subscriptionattributegroup_asdict", _wrap_map_subscriptionattributegroup_asdict, METH_O, NULL},
	 { "new_map_subscriptionattributegroup", _wrap_new_map_subscriptionattributegroup, METH_VARARGS, NULL},
	 { "map_subscriptionattributegroup_empty", _wrap_map_subscriptionattributegroup_empty, METH_O, NULL},
	 { "map_subscriptionattributegroup_size", _wrap_map_subscriptionattributegroup_size, METH_O, NULL},
	 { "map_subscriptionattributegroup_swap", _wrap_map_subscriptionattributegroup_swap, METH_VARARGS, NULL},
	 { "map_subscriptionattributegroup_begin", _wrap_map_subscriptionattributegroup_begin, METH_O, NULL},
	 { "map_subscriptionattributegroup_end", _wrap_map_subscriptionattributegroup_end, METH_O, NULL},
	 { "map_subscriptionattributegroup_rbegin", _wrap_map_subscriptionattributegroup_rbegin, METH_O, NULL},
	 { "map_subscriptionattributegroup_rend", _wrap_map_subscriptionattributegroup_rend, METH_O, NULL},
	 { "map_subscriptionattributegroup_clear", _wrap_map_subscriptionattributegroup_clear, METH_O, NULL},
	 { "map_subscriptionattributegroup_get_allocator", _wrap_map_subscriptionattributegroup_get_allocator, METH_O, NULL},
	 { "map_subscriptionattributegroup_count", _wrap_map_subscriptionattributegroup_count, METH_VARARGS, NULL},
	 { "map_subscriptionattributegroup_erase", _wrap_map_subscriptionattributegroup_erase, METH_VARARGS, NULL},
	 { "map_subscriptionattributegroup_find", _wrap_map_subscriptionattributegroup_find, METH_VARARGS, NULL},
	 { "map_subscriptionattributegroup_lower_bound", _wrap_map_subscriptionattributegroup_lower_bound, METH_VARARGS, NULL},
	 { "map_subscriptionattributegroup_upper_bound", _wrap_map_subscriptionattributegroup_upper_bound, METH_VARARGS, NULL},
	 { "delete_map_subscriptionattributegroup", _wrap_delete_map_subscriptionattributegroup, METH_O, NULL},
	 { "map_subscriptionattributegroup_swigregister", map_subscriptionattributegroup_swigregister, METH_O, NULL},
	 { "map_subscriptionattributegroup_swiginit", map_subscriptionattributegroup_swiginit, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionFilterPredicateDirector_Predicate", _wrap_WrappedServiceSubscriptionFilterPredicateDirector_Predicate, METH_VARARGS, NULL},
	 { "delete_WrappedServiceSubscriptionFilterPredicateDirector", _wrap_delete_WrappedServiceSubscriptionFilterPredicateDirector, METH_O, NULL},
	 { "new_WrappedServiceSubscriptionFilterPredicateDirector", _wrap_new_WrappedServiceSubscriptionFilterPredicateDirector, METH_O, NULL},
	 { "disown_WrappedServiceSubscriptionFilterPredicateDirector", _wrap_disown_WrappedServiceSubscriptionFilterPredicateDirector, METH_O, NULL},
	 { "WrappedServiceSubscriptionFilterPredicateDirector_swigregister", WrappedServiceSubscriptionFilterPredicateDirector_swigregister, METH_O, NULL},
	 { "WrappedServiceSubscriptionFilterPredicateDirector_swiginit", WrappedServiceSubscriptionFilterPredicateDirector_swiginit, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionFilterNode_NodeID_set", _wrap_WrappedServiceSubscriptionFilterNode_NodeID_set, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionFilterNode_NodeID_get", _wrap_WrappedServiceSubscriptionFilterNode_NodeID_get, METH_O, NULL},
	 { "WrappedServiceSubscriptionFilterNode_NodeName_set", _wrap_WrappedServiceSubscriptionFilterNode_NodeName_set, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionFilterNode_NodeName_get", _wrap_WrappedServiceSubscriptionFilterNode_NodeName_get, METH_O, NULL},
	 { "WrappedServiceSubscriptionFilterNode_Username_set", _wrap_WrappedServiceSubscriptionFilterNode_Username_set, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionFilterNode_Username_get", _wrap_WrappedServiceSubscriptionFilterNode_Username_get, METH_O, NULL},
	 { "WrappedServiceSubscriptionFilterNode_Credentials_set", _wrap_WrappedServiceSubscriptionFilterNode_Credentials_set, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionFilterNode_Credentials_get", _wrap_WrappedServiceSubscriptionFilterNode_Credentials_get, METH_O, NULL},
	 { "new_WrappedServiceSubscriptionFilterNode", _wrap_new_WrappedServiceSubscriptionFilterNode, METH_NOARGS, NULL},
	 { "delete_WrappedServiceSubscriptionFilterNode", _wrap_delete_WrappedServiceSubscriptionFilterNode, METH_O, NULL},
	 { "WrappedServiceSubscriptionFilterNode_swigregister", WrappedServiceSubscriptionFilterNode_swigregister, METH_O, NULL},
	 { "WrappedServiceSubscriptionFilterNode_swiginit", WrappedServiceSubscriptionFilterNode_swiginit, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionFilter_Nodes_set", _wrap_WrappedServiceSubscriptionFilter_Nodes_set, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionFilter_Nodes_get", _wrap_WrappedServiceSubscriptionFilter_Nodes_get, METH_O, NULL},
	 { "WrappedServiceSubscriptionFilter_ServiceNames_set", _wrap_WrappedServiceSubscriptionFilter_ServiceNames_set, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionFilter_ServiceNames_get", _wrap_WrappedServiceSubscriptionFilter_ServiceNames_get, METH_O, NULL},
	 { "WrappedServiceSubscriptionFilter_TransportSchemes_set", _wrap_WrappedServiceSubscriptionFilter_TransportSchemes_set, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionFilter_TransportSchemes_get", _wrap_WrappedServiceSubscriptionFilter_TransportSchemes_get, METH_O, NULL},
	 { "WrappedServiceSubscriptionFilter_Attributes_set", _wrap_WrappedServiceSubscriptionFilter_Attributes_set, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionFilter_Attributes_get", _wrap_WrappedServiceSubscriptionFilter_Attributes_get, METH_O, NULL},
	 { "WrappedServiceSubscriptionFilter_AttributesMatchOperation_set", _wrap_WrappedServiceSubscriptionFilter_AttributesMatchOperation_set, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionFilter_AttributesMatchOperation_get", _wrap_WrappedServiceSubscriptionFilter_AttributesMatchOperation_get, METH_O, NULL},
	 { "WrappedServiceSubscriptionFilter_SetRRPredicateDirector", _wrap_WrappedServiceSubscriptionFilter_SetRRPredicateDirector, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionFilter_MaxConnections_set", _wrap_WrappedServiceSubscriptionFilter_MaxConnections_set, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionFilter_MaxConnections_get", _wrap_WrappedServiceSubscriptionFilter_MaxConnections_get, METH_O, NULL},
	 { "new_WrappedServiceSubscriptionFilter", _wrap_new_WrappedServiceSubscriptionFilter, METH_NOARGS, NULL},
	 { "delete_WrappedServiceSubscriptionFilter", _wrap_delete_WrappedServiceSubscriptionFilter, METH_O, NULL},
	 { "WrappedServiceSubscriptionFilter_swigregister", WrappedServiceSubscriptionFilter_swigregister, METH_O, NULL},
	 { "WrappedServiceSubscriptionFilter_swiginit", WrappedServiceSubscriptionFilter_swiginit, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionClientID_NodeID_set", _wrap_WrappedServiceSubscriptionClientID_NodeID_set, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionClientID_NodeID_get", _wrap_WrappedServiceSubscriptionClientID_NodeID_get, METH_O, NULL},
	 { "WrappedServiceSubscriptionClientID_ServiceName_set", _wrap_WrappedServiceSubscriptionClientID_ServiceName_set, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionClientID_ServiceName_get", _wrap_WrappedServiceSubscriptionClientID_ServiceName_get, METH_O, NULL},
	 { "WrappedServiceSubscriptionClientID_eq", _wrap_WrappedServiceSubscriptionClientID_eq, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionClientID_ne", _wrap_WrappedServiceSubscriptionClientID_ne, METH_VARARGS, NULL},
	 { "delete_WrappedServiceSubscriptionClientID", _wrap_delete_WrappedServiceSubscriptionClientID, METH_O, NULL},
	 { "WrappedServiceSubscriptionClientID_swigregister", WrappedServiceSubscriptionClientID_swigregister, METH_O, NULL},
	 { "WrappedServiceInfo2SubscriptionDirector_ServiceDetected", _wrap_WrappedServiceInfo2SubscriptionDirector_ServiceDetected, METH_VARARGS, NULL},
	 { "WrappedServiceInfo2SubscriptionDirector_ServiceLost", _wrap_WrappedServiceInfo2SubscriptionDirector_ServiceLost, METH_VARARGS, NULL},
	 { "delete_WrappedServiceInfo2SubscriptionDirector", _wrap_delete_WrappedServiceInfo2SubscriptionDirector, METH_O, NULL},
	 { "new_WrappedServiceInfo2SubscriptionDirector", _wrap_new_WrappedServiceInfo2SubscriptionDirector, METH_O, NULL},
	 { "disown_WrappedServiceInfo2SubscriptionDirector", _wrap_disown_WrappedServiceInfo2SubscriptionDirector, METH_O, NULL},
	 { "WrappedServiceInfo2SubscriptionDirector_swigregister", WrappedServiceInfo2SubscriptionDirector_swigregister, METH_O, NULL},
	 { "WrappedServiceInfo2SubscriptionDirector_swiginit", WrappedServiceInfo2SubscriptionDirector_swiginit, METH_VARARGS, NULL},
	 { "WrappedServiceInfo2Subscription_GetDetectedServiceInfo2", _wrap_WrappedServiceInfo2Subscription_GetDetectedServiceInfo2, METH_O, NULL},
	 { "WrappedServiceInfo2Subscription_Close", _wrap_WrappedServiceInfo2Subscription_Close, METH_O, NULL},
	 { "WrappedServiceInfo2Subscription_SetRRDirector", _wrap_WrappedServiceInfo2Subscription_SetRRDirector, METH_VARARGS, NULL},
	 { "WrappedServiceInfo2Subscription_GetNode", _wrap_WrappedServiceInfo2Subscription_GetNode, METH_O, NULL},
	 { "delete_WrappedServiceInfo2Subscription", _wrap_delete_WrappedServiceInfo2Subscription, METH_O, NULL},
	 { "WrappedServiceInfo2Subscription_swigregister", WrappedServiceInfo2Subscription_swigregister, METH_O, NULL},
	 { "WrappedServiceSubscriptionDirector_ClientConnected", _wrap_WrappedServiceSubscriptionDirector_ClientConnected, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionDirector_ClientDisconnected", _wrap_WrappedServiceSubscriptionDirector_ClientDisconnected, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionDirector_ClientConnectFailed", _wrap_WrappedServiceSubscriptionDirector_ClientConnectFailed, METH_VARARGS, NULL},
	 { "delete_WrappedServiceSubscriptionDirector", _wrap_delete_WrappedServiceSubscriptionDirector, METH_O, NULL},
	 { "new_WrappedServiceSubscriptionDirector", _wrap_new_WrappedServiceSubscriptionDirector, METH_O, NULL},
	 { "disown_WrappedServiceSubscriptionDirector", _wrap_disown_WrappedServiceSubscriptionDirector, METH_O, NULL},
	 { "WrappedServiceSubscriptionDirector_swigregister", WrappedServiceSubscriptionDirector_swigregister, METH_O, NULL},
	 { "WrappedServiceSubscriptionDirector_swiginit", WrappedServiceSubscriptionDirector_swiginit, METH_VARARGS, NULL},
	 { "WrappedServiceSubscription_TryDefaultClientRes_res_set", _wrap_WrappedServiceSubscription_TryDefaultClientRes_res_set, METH_VARARGS, NULL},
	 { "WrappedServiceSubscription_TryDefaultClientRes_res_get", _wrap_WrappedServiceSubscription_TryDefaultClientRes_res_get, METH_O, NULL},
	 { "WrappedServiceSubscription_TryDefaultClientRes_client_set", _wrap_WrappedServiceSubscription_TryDefaultClientRes_client_set, METH_VARARGS, NULL},
	 { "WrappedServiceSubscription_TryDefaultClientRes_client_get", _wrap_WrappedServiceSubscription_TryDefaultClientRes_client_get, METH_O, NULL},
	 { "new_WrappedServiceSubscription_TryDefaultClientRes", _wrap_new_WrappedServiceSubscription_TryDefaultClientRes, METH_NOARGS, NULL},
	 { "delete_WrappedServiceSubscription_TryDefaultClientRes", _wrap_delete_WrappedServiceSubscription_TryDefaultClientRes, METH_O, NULL},
	 { "WrappedServiceSubscription_TryDefaultClientRes_swigregister", WrappedServiceSubscription_TryDefaultClientRes_swigregister, METH_O, NULL},
	 { "WrappedServiceSubscription_TryDefaultClientRes_swiginit", WrappedServiceSubscription_TryDefaultClientRes_swiginit, METH_VARARGS, NULL},
	 { "WrappedServiceSubscription_GetConnectedClients", _wrap_WrappedServiceSubscription_GetConnectedClients, METH_O, NULL},
	 { "WrappedServiceSubscription_Close", _wrap_WrappedServiceSubscription_Close, METH_O, NULL},
	 { "WrappedServiceSubscription_ClaimClient", _wrap_WrappedServiceSubscription_ClaimClient, METH_VARARGS, NULL},
	 { "WrappedServiceSubscription_ReleaseClient", _wrap_WrappedServiceSubscription_ReleaseClient, METH_VARARGS, NULL},
	 { "WrappedServiceSubscription_GetConnectRetryDelay", _wrap_WrappedServiceSubscription_GetConnectRetryDelay, METH_O, NULL},
	 { "WrappedServiceSubscription_SetConnectRetryDelay", _wrap_WrappedServiceSubscription_SetConnectRetryDelay, METH_VARARGS, NULL},
	 { "WrappedServiceSubscription_SubscribeWire", _wrap_WrappedServiceSubscription_SubscribeWire, METH_VARARGS, NULL},
	 { "WrappedServiceSubscription_SubscribePipe", _wrap_WrappedServiceSubscription_SubscribePipe, METH_VARARGS, NULL},
	 { "WrappedServiceSubscription_GetDefaultClient", _wrap_WrappedServiceSubscription_GetDefaultClient, METH_O, NULL},
	 { "WrappedServiceSubscription_TryGetDefaultClient", _wrap_WrappedServiceSubscription_TryGetDefaultClient, METH_O, NULL},
	 { "WrappedServiceSubscription_GetDefaultClientWait", _wrap_WrappedServiceSubscription_GetDefaultClientWait, METH_VARARGS, NULL},
	 { "WrappedServiceSubscription_TryGetDefaultClientWait", _wrap_WrappedServiceSubscription_TryGetDefaultClientWait, METH_VARARGS, NULL},
	 { "WrappedServiceSubscription_AsyncGetDefaultClient", _wrap_WrappedServiceSubscription_AsyncGetDefaultClient, METH_VARARGS, NULL},
	 { "WrappedServiceSubscription_SetRRDirector", _wrap_WrappedServiceSubscription_SetRRDirector, METH_VARARGS, NULL},
	 { "WrappedServiceSubscription_GetNode", _wrap_WrappedServiceSubscription_GetNode, METH_O, NULL},
	 { "WrappedServiceSubscription_GetServiceURL", _wrap_WrappedServiceSubscription_GetServiceURL, METH_O, NULL},
	 { "WrappedServiceSubscription_UpdateServiceURL", _wrap_WrappedServiceSubscription_UpdateServiceURL, METH_VARARGS, NULL},
	 { "WrappedServiceSubscription_UpdateServiceByType", _wrap_WrappedServiceSubscription_UpdateServiceByType, METH_VARARGS, NULL},
	 { "WrappedServiceSubscription_SubscribeSubObject", _wrap_WrappedServiceSubscription_SubscribeSubObject, METH_VARARGS, NULL},
	 { "delete_WrappedServiceSubscription", _wrap_delete_WrappedServiceSubscription, METH_O, NULL},
	 { "WrappedServiceSubscription_swigregister", WrappedServiceSubscription_swigregister, METH_O, NULL},
	 { "WrappedWireSubscriptionDirector_WireValueChanged", _wrap_WrappedWireSubscriptionDirector_WireValueChanged, METH_VARARGS, NULL},
	 { "delete_WrappedWireSubscriptionDirector", _wrap_delete_WrappedWireSubscriptionDirector, METH_O, NULL},
	 { "new_WrappedWireSubscriptionDirector", _wrap_new_WrappedWireSubscriptionDirector, METH_O, NULL},
	 { "disown_WrappedWireSubscriptionDirector", _wrap_disown_WrappedWireSubscriptionDirector, METH_O, NULL},
	 { "WrappedWireSubscriptionDirector_swigregister", WrappedWireSubscriptionDirector_swigregister, METH_O, NULL},
	 { "WrappedWireSubscriptionDirector_swiginit", WrappedWireSubscriptionDirector_swiginit, METH_VARARGS, NULL},
	 { "WrappedWireSubscription_GetInValue", _wrap_WrappedWireSubscription_GetInValue, METH_VARARGS, NULL},
	 { "WrappedWireSubscription_TryGetInValue", _wrap_WrappedWireSubscription_TryGetInValue, METH_VARARGS, NULL},
	 { "WrappedWireSubscription_WaitInValueValid", _wrap_WrappedWireSubscription_WaitInValueValid, METH_VARARGS, NULL},
	 { "WrappedWireSubscription_GetIgnoreInValue", _wrap_WrappedWireSubscription_GetIgnoreInValue, METH_O, NULL},
	 { "WrappedWireSubscription_SetIgnoreInValue", _wrap_WrappedWireSubscription_SetIgnoreInValue, METH_VARARGS, NULL},
	 { "WrappedWireSubscription_GetInValueLifespan", _wrap_WrappedWireSubscription_GetInValueLifespan, METH_O, NULL},
	 { "WrappedWireSubscription_SetInValueLifespan", _wrap_WrappedWireSubscription_SetInValueLifespan, METH_VARARGS, NULL},
	 { "WrappedWireSubscription_GetActiveWireConnectionCount", _wrap_WrappedWireSubscription_GetActiveWireConnectionCount, METH_O, NULL},
	 { "WrappedWireSubscription_Close", _wrap_WrappedWireSubscription_Close, METH_O, NULL},
	 { "WrappedWireSubscription_SetRRDirector", _wrap_WrappedWireSubscription_SetRRDirector, METH_VARARGS, NULL},
	 { "WrappedWireSubscription_GetNode", _wrap_WrappedWireSubscription_GetNode, METH_O, NULL},
	 { "delete_WrappedWireSubscription", _wrap_delete_WrappedWireSubscription, METH_O, NULL},
	 { "WrappedWireSubscription_swigregister", WrappedWireSubscription_swigregister, METH_O, NULL},
	 { "new_WrappedWireSubscription_send_iterator", _wrap_new_WrappedWireSubscription_send_iterator, METH_O, NULL},
	 { "WrappedWireSubscription_send_iterator_Next", _wrap_WrappedWireSubscription_send_iterator_Next, METH_O, NULL},
	 { "WrappedWireSubscription_send_iterator_SetOutValue", _wrap_WrappedWireSubscription_send_iterator_SetOutValue, METH_VARARGS, NULL},
	 { "WrappedWireSubscription_send_iterator_GetType", _wrap_WrappedWireSubscription_send_iterator_GetType, METH_O, NULL},
	 { "WrappedWireSubscription_send_iterator_GetStub", _wrap_WrappedWireSubscription_send_iterator_GetStub, METH_O, NULL},
	 { "delete_WrappedWireSubscription_send_iterator", _wrap_delete_WrappedWireSubscription_send_iterator, METH_O, NULL},
	 { "WrappedWireSubscription_send_iterator_swigregister", WrappedWireSubscription_send_iterator_swigregister, METH_O, NULL},
	 { "WrappedWireSubscription_send_iterator_swiginit", WrappedWireSubscription_send_iterator_swiginit, METH_VARARGS, NULL},
	 { "WrappedPipeSubscriptionDirector_PipePacketReceived", _wrap_WrappedPipeSubscriptionDirector_PipePacketReceived, METH_VARARGS, NULL},
	 { "delete_WrappedPipeSubscriptionDirector", _wrap_delete_WrappedPipeSubscriptionDirector, METH_O, NULL},
	 { "new_WrappedPipeSubscriptionDirector", _wrap_new_WrappedPipeSubscriptionDirector, METH_O, NULL},
	 { "disown_WrappedPipeSubscriptionDirector", _wrap_disown_WrappedPipeSubscriptionDirector, METH_O, NULL},
	 { "WrappedPipeSubscriptionDirector_swigregister", WrappedPipeSubscriptionDirector_swigregister, METH_O, NULL},
	 { "WrappedPipeSubscriptionDirector_swiginit", WrappedPipeSubscriptionDirector_swiginit, METH_VARARGS, NULL},
	 { "WrappedPipeSubscription_ReceivePacket", _wrap_WrappedPipeSubscription_ReceivePacket, METH_O, NULL},
	 { "WrappedPipeSubscription_TryReceivePacket", _wrap_WrappedPipeSubscription_TryReceivePacket, METH_VARARGS, NULL},
	 { "WrappedPipeSubscription_TryReceivePacketWait", _wrap_WrappedPipeSubscription_TryReceivePacketWait, METH_VARARGS, NULL},
	 { "WrappedPipeSubscription_Available", _wrap_WrappedPipeSubscription_Available, METH_O, NULL},
	 { "WrappedPipeSubscription_GetActivePipeEndpointCount", _wrap_WrappedPipeSubscription_GetActivePipeEndpointCount, METH_O, NULL},
	 { "WrappedPipeSubscription_GetIgnoreReceived", _wrap_WrappedPipeSubscription_GetIgnoreReceived, METH_O, NULL},
	 { "WrappedPipeSubscription_SetIgnoreReceived", _wrap_WrappedPipeSubscription_SetIgnoreReceived, METH_VARARGS, NULL},
	 { "WrappedPipeSubscription_Close", _wrap_WrappedPipeSubscription_Close, METH_O, NULL},
	 { "WrappedPipeSubscription_SetRRDirector", _wrap_WrappedPipeSubscription_SetRRDirector, METH_VARARGS, NULL},
	 { "WrappedPipeSubscription_GetNode", _wrap_WrappedPipeSubscription_GetNode, METH_O, NULL},
	 { "delete_WrappedPipeSubscription", _wrap_delete_WrappedPipeSubscription, METH_O, NULL},
	 { "WrappedPipeSubscription_swigregister", WrappedPipeSubscription_swigregister, METH_O, NULL},
	 { "new_WrappedPipeSubscription_send_iterator", _wrap_new_WrappedPipeSubscription_send_iterator, METH_O, NULL},
	 { "WrappedPipeSubscription_send_iterator_Next", _wrap_WrappedPipeSubscription_send_iterator_Next, METH_O, NULL},
	 { "WrappedPipeSubscription_send_iterator_AsyncSendPacket", _wrap_WrappedPipeSubscription_send_iterator_AsyncSendPacket, METH_VARARGS, NULL},
	 { "WrappedPipeSubscription_send_iterator_GetType", _wrap_WrappedPipeSubscription_send_iterator_GetType, METH_O, NULL},
	 { "WrappedPipeSubscription_send_iterator_GetStub", _wrap_WrappedPipeSubscription_send_iterator_GetStub, METH_O, NULL},
	 { "delete_WrappedPipeSubscription_send_iterator", _wrap_delete_WrappedPipeSubscription_send_iterator, METH_O, NULL},
	 { "WrappedPipeSubscription_send_iterator_swigregister", WrappedPipeSubscription_send_iterator_swigregister, METH_O, NULL},
	 { "WrappedPipeSubscription_send_iterator_swiginit", WrappedPipeSubscription_send_iterator_swiginit, METH_VARARGS, NULL},
	 { "WrappedSubObjectSubscription_GetDefaultClient", _wrap_WrappedSubObjectSubscription_GetDefaultClient, METH_O, NULL},
	 { "WrappedSubObjectSubscription_TryGetDefaultClient", _wrap_WrappedSubObjectSubscription_TryGetDefaultClient, METH_O, NULL},
	 { "WrappedSubObjectSubscription_GetDefaultClientWait", _wrap_WrappedSubObjectSubscription_GetDefaultClientWait, METH_VARARGS, NULL},
	 { "WrappedSubObjectSubscription_TryGetDefaultClientWait", _wrap_WrappedSubObjectSubscription_TryGetDefaultClientWait, METH_VARARGS, NULL},
	 { "WrappedSubObjectSubscription_AsyncGetDefaultClient", _wrap_WrappedSubObjectSubscription_AsyncGetDefaultClient, METH_VARARGS, NULL},
	 { "WrappedSubObjectSubscription_Close", _wrap_WrappedSubObjectSubscription_Close, METH_O, NULL},
	 { "WrappedSubObjectSubscription_GetNode", _wrap_WrappedSubObjectSubscription_GetNode, METH_O, NULL},
	 { "delete_WrappedSubObjectSubscription", _wrap_delete_WrappedSubObjectSubscription, METH_O, NULL},
	 { "WrappedSubObjectSubscription_swigregister", WrappedSubObjectSubscription_swigregister, METH_O, NULL},
	 { "WrappedServiceInfo2SubscriptionServicesToVector", _wrap_WrappedServiceInfo2SubscriptionServicesToVector, METH_O, NULL},
	 { "WrappedSubscribeServiceInfo2", _wrap_WrappedSubscribeServiceInfo2, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionClientsToVector", _wrap_WrappedServiceSubscriptionClientsToVector, METH_O, NULL},
	 { "WrappedSubscribeServiceByType", _wrap_WrappedSubscribeServiceByType, METH_VARARGS, NULL},
	 { "WrappedSubscribeService", _wrap_WrappedSubscribeService, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionManagerDetails_Name_set", _wrap_WrappedServiceSubscriptionManagerDetails_Name_set, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionManagerDetails_Name_get", _wrap_WrappedServiceSubscriptionManagerDetails_Name_get, METH_O, NULL},
	 { "WrappedServiceSubscriptionManagerDetails_ConnectionMethod_set", _wrap_WrappedServiceSubscriptionManagerDetails_ConnectionMethod_set, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionManagerDetails_ConnectionMethod_get", _wrap_WrappedServiceSubscriptionManagerDetails_ConnectionMethod_get, METH_O, NULL},
	 { "WrappedServiceSubscriptionManagerDetails_Urls_set", _wrap_WrappedServiceSubscriptionManagerDetails_Urls_set, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionManagerDetails_Urls_get", _wrap_WrappedServiceSubscriptionManagerDetails_Urls_get, METH_O, NULL},
	 { "WrappedServiceSubscriptionManagerDetails_UrlUsername_set", _wrap_WrappedServiceSubscriptionManagerDetails_UrlUsername_set, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionManagerDetails_UrlUsername_get", _wrap_WrappedServiceSubscriptionManagerDetails_UrlUsername_get, METH_O, NULL},
	 { "WrappedServiceSubscriptionManagerDetails_UrlCredentials_set", _wrap_WrappedServiceSubscriptionManagerDetails_UrlCredentials_set, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionManagerDetails_UrlCredentials_get", _wrap_WrappedServiceSubscriptionManagerDetails_UrlCredentials_get, METH_O, NULL},
	 { "WrappedServiceSubscriptionManagerDetails_ServiceTypes_set", _wrap_WrappedServiceSubscriptionManagerDetails_ServiceTypes_set, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionManagerDetails_ServiceTypes_get", _wrap_WrappedServiceSubscriptionManagerDetails_ServiceTypes_get, METH_O, NULL},
	 { "WrappedServiceSubscriptionManagerDetails_Filter_set", _wrap_WrappedServiceSubscriptionManagerDetails_Filter_set, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionManagerDetails_Filter_get", _wrap_WrappedServiceSubscriptionManagerDetails_Filter_get, METH_O, NULL},
	 { "WrappedServiceSubscriptionManagerDetails_Enabled_set", _wrap_WrappedServiceSubscriptionManagerDetails_Enabled_set, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionManagerDetails_Enabled_get", _wrap_WrappedServiceSubscriptionManagerDetails_Enabled_get, METH_O, NULL},
	 { "new_WrappedServiceSubscriptionManagerDetails", _wrap_new_WrappedServiceSubscriptionManagerDetails, METH_VARARGS, NULL},
	 { "delete_WrappedServiceSubscriptionManagerDetails", _wrap_delete_WrappedServiceSubscriptionManagerDetails, METH_O, NULL},
	 { "WrappedServiceSubscriptionManagerDetails_swigregister", WrappedServiceSubscriptionManagerDetails_swigregister, METH_O, NULL},
	 { "WrappedServiceSubscriptionManagerDetails_swiginit", WrappedServiceSubscriptionManagerDetails_swiginit, METH_VARARGS, NULL},
	 { "new_WrappedServiceSubscriptionManager", _wrap_new_WrappedServiceSubscriptionManager, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionManager_AddSubscription", _wrap_WrappedServiceSubscriptionManager_AddSubscription, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionManager_RemoveSubscription", _wrap_WrappedServiceSubscriptionManager_RemoveSubscription, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionManager_EnableSubscription", _wrap_WrappedServiceSubscriptionManager_EnableSubscription, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionManager_DisableSubscription", _wrap_WrappedServiceSubscriptionManager_DisableSubscription, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionManager_GetSubscription", _wrap_WrappedServiceSubscriptionManager_GetSubscription, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionManager_IsConnected", _wrap_WrappedServiceSubscriptionManager_IsConnected, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionManager_IsEnabled", _wrap_WrappedServiceSubscriptionManager_IsEnabled, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionManager_Close", _wrap_WrappedServiceSubscriptionManager_Close, METH_VARARGS, NULL},
	 { "WrappedServiceSubscriptionManager_GetSubscriptionNames", _wrap_WrappedServiceSubscriptionManager_GetSubscriptionNames, METH_O, NULL},
	 { "WrappedServiceSubscriptionManager_GetSubscriptionDetails", _wrap_WrappedServiceSubscriptionManager_GetSubscriptionDetails, METH_O, NULL},
	 { "WrappedServiceSubscriptionManager_GetNode", _wrap_WrappedServiceSubscriptionManager_GetNode, METH_O, NULL},
	 { "delete_WrappedServiceSubscriptionManager", _wrap_delete_WrappedServiceSubscriptionManager, METH_O, NULL},
	 { "WrappedServiceSubscriptionManager_swigregister", WrappedServiceSubscriptionManager_swigregister, METH_O, NULL},
	 { "WrappedServiceSubscriptionManager_swiginit", WrappedServiceSubscriptionManager_swiginit, METH_VARARGS, NULL},
	 { "RRLogRecord__GetNode", _wrap_RRLogRecord__GetNode, METH_O, NULL},
	 { "RRLogRecord__SetNode", _wrap_RRLogRecord__SetNode, METH_VARARGS, NULL},
	 { "RRLogRecord_Level_set", _wrap_RRLogRecord_Level_set, METH_VARARGS, "(int) The log level "},
	 { "RRLogRecord_Level_get", _wrap_RRLogRecord_Level_get, METH_O, "(int) The log level "},
	 { "RRLogRecord_Component_set", _wrap_RRLogRecord_Component_set, METH_VARARGS, "(int) The source component"},
	 { "RRLogRecord_Component_get", _wrap_RRLogRecord_Component_get, METH_O, "(int) The source component"},
	 { "RRLogRecord_ComponentName_set", _wrap_RRLogRecord_ComponentName_set, METH_VARARGS, "(str) The source component name"},
	 { "RRLogRecord_ComponentName_get", _wrap_RRLogRecord_ComponentName_get, METH_O, "(str) The source component name"},
	 { "RRLogRecord_ComponentObjectID_set", _wrap_RRLogRecord_ComponentObjectID_set, METH_VARARGS, "(str) The source component object ID"},
	 { "RRLogRecord_ComponentObjectID_get", _wrap_RRLogRecord_ComponentObjectID_get, METH_O, "(str) The source component object ID"},
	 { "RRLogRecord_Endpoint_set", _wrap_RRLogRecord_Endpoint_set, METH_VARARGS, "(int) The source endpoint"},
	 { "RRLogRecord_Endpoint_get", _wrap_RRLogRecord_Endpoint_get, METH_O, "(int) The source endpoint"},
	 { "RRLogRecord_ServicePath_set", _wrap_RRLogRecord_ServicePath_set, METH_VARARGS, "(str) The service path of the source object"},
	 { "RRLogRecord_ServicePath_get", _wrap_RRLogRecord_ServicePath_get, METH_O, "(str) The service path of the source object"},
	 { "RRLogRecord_Member_set", _wrap_RRLogRecord_Member_set, METH_VARARGS, "(str) The source member"},
	 { "RRLogRecord_Member_get", _wrap_RRLogRecord_Member_get, METH_O, "(str) The source member"},
	 { "RRLogRecord_Message_set", _wrap_RRLogRecord_Message_set, METH_VARARGS, "(str) Human readable log message"},
	 { "RRLogRecord_Message_get", _wrap_RRLogRecord_Message_get, METH_O, "(str) Human readable log message"},
	 { "RRLogRecord_Time_set", _wrap_RRLogRecord_Time_set, METH_VARARGS, "(datetime.datetime) Time of logging event"},
	 { "RRLogRecord_Time_get", _wrap_RRLogRecord_Time_get, METH_O, "(datetime.datetime) Time of logging event"},
	 { "RRLogRecord_SourceFile_set", _wrap_RRLogRecord_SourceFile_set, METH_VARARGS, "(str) The sourcecode filename"},
	 { "RRLogRecord_SourceFile_get", _wrap_RRLogRecord_SourceFile_get, METH_O, "(str) The sourcecode filename"},
	 { "RRLogRecord_SourceLine_set", _wrap_RRLogRecord_SourceLine_set, METH_VARARGS, "(int) The line within the sourcecode file"},
	 { "RRLogRecord_SourceLine_get", _wrap_RRLogRecord_SourceLine_get, METH_O, "(int) The line within the sourcecode file"},
	 { "RRLogRecord_ThreadID_set", _wrap_RRLogRecord_ThreadID_set, METH_VARARGS, "(str) The source thread"},
	 { "RRLogRecord_ThreadID_get", _wrap_RRLogRecord_ThreadID_get, METH_O, "(str) The source thread"},
	 { "RRLogRecord_FiberID_set", _wrap_RRLogRecord_FiberID_set, METH_VARARGS, "(str) The source coroutine fiber"},
	 { "RRLogRecord_FiberID_get", _wrap_RRLogRecord_FiberID_get, METH_O, "(str) The source coroutine fiber"},
	 { "RRLogRecord___str__", _wrap_RRLogRecord___str__, METH_O, NULL},
	 { "new_RRLogRecord", _wrap_new_RRLogRecord, METH_NOARGS, NULL},
	 { "delete_RRLogRecord", _wrap_delete_RRLogRecord, METH_O, NULL},
	 { "RRLogRecord_swigregister", RRLogRecord_swigregister, METH_O, NULL},
	 { "RRLogRecord_swiginit", RRLogRecord_swiginit, METH_VARARGS, NULL},
	 { "delete_LogRecordHandler", _wrap_delete_LogRecordHandler, METH_O, NULL},
	 { "LogRecordHandler_swigregister", LogRecordHandler_swigregister, METH_O, NULL},
	 { "FileLogRecordHandler_OpenFile", _wrap_FileLogRecordHandler_OpenFile, METH_VARARGS, "\n"
		"\n"
		"Open a file to store log records\n"
		"\n"
		":param filename: The filename\n"
		":type filename: str\n"
		":param append: If True, log messages are appended. If False, the file is truncated when opened\n"
		":type append: bool\n"
		"\n"
		""},
	 { "new_FileLogRecordHandler", _wrap_new_FileLogRecordHandler, METH_NOARGS, NULL},
	 { "delete_FileLogRecordHandler", _wrap_delete_FileLogRecordHandler, METH_O, NULL},
	 { "FileLogRecordHandler_swigregister", FileLogRecordHandler_swigregister, METH_O, NULL},
	 { "FileLogRecordHandler_swiginit", FileLogRecordHandler_swiginit, METH_VARARGS, NULL},
	 { "UserLogRecordHandlerDirector_HandleLogRecord", _wrap_UserLogRecordHandlerDirector_HandleLogRecord, METH_VARARGS, NULL},
	 { "delete_UserLogRecordHandlerDirector", _wrap_delete_UserLogRecordHandlerDirector, METH_O, NULL},
	 { "new_UserLogRecordHandlerDirector", _wrap_new_UserLogRecordHandlerDirector, METH_O, NULL},
	 { "disown_UserLogRecordHandlerDirector", _wrap_disown_UserLogRecordHandlerDirector, METH_O, NULL},
	 { "UserLogRecordHandlerDirector_swigregister", UserLogRecordHandlerDirector_swigregister, METH_O, NULL},
	 { "UserLogRecordHandlerDirector_swiginit", UserLogRecordHandlerDirector_swiginit, METH_VARARGS, NULL},
	 { "UserLogRecordHandlerBase__SetHandler", _wrap_UserLogRecordHandlerBase__SetHandler, METH_VARARGS, NULL},
	 { "new_UserLogRecordHandlerBase", _wrap_new_UserLogRecordHandlerBase, METH_NOARGS, NULL},
	 { "delete_UserLogRecordHandlerBase", _wrap_delete_UserLogRecordHandlerBase, METH_O, NULL},
	 { "UserLogRecordHandlerBase_swigregister", UserLogRecordHandlerBase_swigregister, METH_O, NULL},
	 { "UserLogRecordHandlerBase_swiginit", UserLogRecordHandlerBase_swiginit, METH_VARARGS, NULL},
	 { "NodeDirectories_system_data_dir_set", _wrap_NodeDirectories_system_data_dir_set, METH_VARARGS, NULL},
	 { "NodeDirectories_system_data_dir_get", _wrap_NodeDirectories_system_data_dir_get, METH_O, NULL},
	 { "NodeDirectories_system_config_dir_set", _wrap_NodeDirectories_system_config_dir_set, METH_VARARGS, NULL},
	 { "NodeDirectories_system_config_dir_get", _wrap_NodeDirectories_system_config_dir_get, METH_O, NULL},
	 { "NodeDirectories_system_state_dir_set", _wrap_NodeDirectories_system_state_dir_set, METH_VARARGS, NULL},
	 { "NodeDirectories_system_state_dir_get", _wrap_NodeDirectories_system_state_dir_get, METH_O, NULL},
	 { "NodeDirectories_system_cache_dir_set", _wrap_NodeDirectories_system_cache_dir_set, METH_VARARGS, NULL},
	 { "NodeDirectories_system_cache_dir_get", _wrap_NodeDirectories_system_cache_dir_get, METH_O, NULL},
	 { "NodeDirectories_system_run_dir_set", _wrap_NodeDirectories_system_run_dir_set, METH_VARARGS, NULL},
	 { "NodeDirectories_system_run_dir_get", _wrap_NodeDirectories_system_run_dir_get, METH_O, NULL},
	 { "NodeDirectories_user_data_dir_set", _wrap_NodeDirectories_user_data_dir_set, METH_VARARGS, NULL},
	 { "NodeDirectories_user_data_dir_get", _wrap_NodeDirectories_user_data_dir_get, METH_O, NULL},
	 { "NodeDirectories_user_config_dir_set", _wrap_NodeDirectories_user_config_dir_set, METH_VARARGS, NULL},
	 { "NodeDirectories_user_config_dir_get", _wrap_NodeDirectories_user_config_dir_get, METH_O, NULL},
	 { "NodeDirectories_user_state_dir_set", _wrap_NodeDirectories_user_state_dir_set, METH_VARARGS, NULL},
	 { "NodeDirectories_user_state_dir_get", _wrap_NodeDirectories_user_state_dir_get, METH_O, NULL},
	 { "NodeDirectories_user_cache_dir_set", _wrap_NodeDirectories_user_cache_dir_set, METH_VARARGS, NULL},
	 { "NodeDirectories_user_cache_dir_get", _wrap_NodeDirectories_user_cache_dir_get, METH_O, NULL},
	 { "NodeDirectories_user_run_dir_set", _wrap_NodeDirectories_user_run_dir_set, METH_VARARGS, NULL},
	 { "NodeDirectories_user_run_dir_get", _wrap_NodeDirectories_user_run_dir_get, METH_O, NULL},
	 { "new_NodeDirectories", _wrap_new_NodeDirectories, METH_NOARGS, NULL},
	 { "delete_NodeDirectories", _wrap_delete_NodeDirectories, METH_O, NULL},
	 { "NodeDirectories_swigregister", NodeDirectories_swigregister, METH_O, NULL},
	 { "NodeDirectories_swiginit", NodeDirectories_swiginit, METH_VARARGS, NULL},
	 { "delete_NodeDirectoriesFD", _wrap_delete_NodeDirectoriesFD, METH_O, NULL},
	 { "NodeDirectoriesFD_swigregister", NodeDirectoriesFD_swigregister, METH_O, NULL},
	 { "NodeDirectoriesUtil_GetDefaultNodeDirectories", _wrap_NodeDirectoriesUtil_GetDefaultNodeDirectories, METH_VARARGS, NULL},
	 { "NodeDirectoriesUtil_GetLogonUserName", _wrap_NodeDirectoriesUtil_GetLogonUserName, METH_NOARGS, NULL},
	 { "NodeDirectoriesUtil_IsLogonUserRoot", _wrap_NodeDirectoriesUtil_IsLogonUserRoot, METH_NOARGS, NULL},
	 { "NodeDirectoriesUtil_ReadInfoFile", _wrap_NodeDirectoriesUtil_ReadInfoFile, METH_VARARGS, NULL},
	 { "NodeDirectoriesUtil_CreatePidFile", _wrap_NodeDirectoriesUtil_CreatePidFile, METH_O, NULL},
	 { "NodeDirectoriesUtil_CreateInfoFile", _wrap_NodeDirectoriesUtil_CreateInfoFile, METH_VARARGS, NULL},
	 { "NodeDirectoriesUtil_RefreshInfoFile", _wrap_NodeDirectoriesUtil_RefreshInfoFile, METH_VARARGS, NULL},
	 { "new_NodeDirectoriesUtil", _wrap_new_NodeDirectoriesUtil, METH_NOARGS, NULL},
	 { "delete_NodeDirectoriesUtil", _wrap_delete_NodeDirectoriesUtil, METH_O, NULL},
	 { "NodeDirectoriesUtil_swigregister", NodeDirectoriesUtil_swigregister, METH_O, NULL},
	 { "NodeDirectoriesUtil_swiginit", NodeDirectoriesUtil_swiginit, METH_VARARGS, NULL},
	 { "GetUuidForNameAndLockResult_uuid_set", _wrap_GetUuidForNameAndLockResult_uuid_set, METH_VARARGS, NULL},
	 { "GetUuidForNameAndLockResult_uuid_get", _wrap_GetUuidForNameAndLockResult_uuid_get, METH_O, NULL},
	 { "GetUuidForNameAndLockResult_name_set", _wrap_GetUuidForNameAndLockResult_name_set, METH_VARARGS, NULL},
	 { "GetUuidForNameAndLockResult_name_get", _wrap_GetUuidForNameAndLockResult_name_get, METH_O, NULL},
	 { "GetUuidForNameAndLockResult_scope_set", _wrap_GetUuidForNameAndLockResult_scope_set, METH_VARARGS, NULL},
	 { "GetUuidForNameAndLockResult_scope_get", _wrap_GetUuidForNameAndLockResult_scope_get, METH_O, NULL},
	 { "GetUuidForNameAndLockResult_fd_set", _wrap_GetUuidForNameAndLockResult_fd_set, METH_VARARGS, NULL},
	 { "GetUuidForNameAndLockResult_fd_get", _wrap_GetUuidForNameAndLockResult_fd_get, METH_O, NULL},
	 { "new_GetUuidForNameAndLockResult", _wrap_new_GetUuidForNameAndLockResult, METH_NOARGS, NULL},
	 { "delete_GetUuidForNameAndLockResult", _wrap_delete_GetUuidForNameAndLockResult, METH_O, NULL},
	 { "GetUuidForNameAndLockResult_swigregister", GetUuidForNameAndLockResult_swigregister, METH_O, NULL},
	 { "GetUuidForNameAndLockResult_swiginit", GetUuidForNameAndLockResult_swiginit, METH_VARARGS, NULL},
	 { "GetUuidForNameAndLock", _wrap_GetUuidForNameAndLock, METH_VARARGS, NULL},
	 { "new_NodeDirectoriesResourceAlreadyInUse", _wrap_new_NodeDirectoriesResourceAlreadyInUse, METH_NOARGS, NULL},
	 { "delete_NodeDirectoriesResourceAlreadyInUse", _wrap_delete_NodeDirectoriesResourceAlreadyInUse, METH_O, NULL},
	 { "NodeDirectoriesResourceAlreadyInUse_swigregister", NodeDirectoriesResourceAlreadyInUse_swigregister, METH_O, NULL},
	 { "NodeDirectoriesResourceAlreadyInUse_swiginit", NodeDirectoriesResourceAlreadyInUse_swiginit, METH_VARARGS, NULL},
	 { "ClientServiceListenerDirector_objectheapid_set", _wrap_ClientServiceListenerDirector_objectheapid_set, METH_VARARGS, NULL},
	 { "ClientServiceListenerDirector_objectheapid_get", _wrap_ClientServiceListenerDirector_objectheapid_get, METH_O, NULL},
	 { "new_ClientServiceListenerDirector", _wrap_new_ClientServiceListenerDirector, METH_O, NULL},
	 { "ClientServiceListenerDirector_Callback", _wrap_ClientServiceListenerDirector_Callback, METH_VARARGS, NULL},
	 { "ClientServiceListenerDirector_Callback2", _wrap_ClientServiceListenerDirector_Callback2, METH_VARARGS, NULL},
	 { "delete_ClientServiceListenerDirector", _wrap_delete_ClientServiceListenerDirector, METH_O, NULL},
	 { "disown_ClientServiceListenerDirector", _wrap_disown_ClientServiceListenerDirector, METH_O, NULL},
	 { "ClientServiceListenerDirector_swigregister", ClientServiceListenerDirector_swigregister, METH_O, NULL},
	 { "ClientServiceListenerDirector_swiginit", ClientServiceListenerDirector_swiginit, METH_VARARGS, NULL},
	 { "RobotRaconteurNode_RegisterTransport", _wrap_RobotRaconteurNode_RegisterTransport, METH_VARARGS, "\n"
		"\n"
		"\n"
		"Register a transport for use by the node\n"
		"\n"
		":param transport: The transport to register\n"
		":type transport: RobotRaconteur.Transport\n"
		":return: The transport internal id\n"
		":rtype: int\n"
		"\n"
		"\n"
		""},
	 { "RobotRaconteurNode__GetRequestTimeout", _wrap_RobotRaconteurNode__GetRequestTimeout, METH_O, NULL},
	 { "RobotRaconteurNode__SetRequestTimeout", _wrap_RobotRaconteurNode__SetRequestTimeout, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__GetTransportInactivityTimeout", _wrap_RobotRaconteurNode__GetTransportInactivityTimeout, METH_O, NULL},
	 { "RobotRaconteurNode__SetTransportInactivityTimeout", _wrap_RobotRaconteurNode__SetTransportInactivityTimeout, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__GetEndpointInactivityTimeout", _wrap_RobotRaconteurNode__GetEndpointInactivityTimeout, METH_O, NULL},
	 { "RobotRaconteurNode__SetEndpointInactivityTimeout", _wrap_RobotRaconteurNode__SetEndpointInactivityTimeout, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__GetMemoryMaxTransferSize", _wrap_RobotRaconteurNode__GetMemoryMaxTransferSize, METH_O, NULL},
	 { "RobotRaconteurNode__SetMemoryMaxTransferSize", _wrap_RobotRaconteurNode__SetMemoryMaxTransferSize, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__GetNodeDiscoveryMaxCacheCount", _wrap_RobotRaconteurNode__GetNodeDiscoveryMaxCacheCount, METH_O, NULL},
	 { "RobotRaconteurNode__SetNodeDiscoveryMaxCacheCount", _wrap_RobotRaconteurNode__SetNodeDiscoveryMaxCacheCount, METH_VARARGS, NULL},
	 { "RobotRaconteurNode_UnregisterServiceType", _wrap_RobotRaconteurNode_UnregisterServiceType, METH_VARARGS, "\n"
		"\n"
		"\n"
		"Unregister a previously registered service type\n"
		"\n"
		"This function is not recommended as the results can be unpredictable\n"
		"\n"
		":param type: The service type to unregister\n"
		":type type: str\n"
		"\n"
		""},
	 { "RobotRaconteurNode__ConnectService", _wrap_RobotRaconteurNode__ConnectService, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__AsyncConnectService", _wrap_RobotRaconteurNode__AsyncConnectService, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__DisconnectService", _wrap_RobotRaconteurNode__DisconnectService, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__AsyncDisconnectService", _wrap_RobotRaconteurNode__AsyncDisconnectService, METH_VARARGS, NULL},
	 { "RobotRaconteurNode_SelectRemoteNodeURL", _wrap_RobotRaconteurNode_SelectRemoteNodeURL, METH_O, "\n"
		"\n"
		"\n"
		"Select the \"best\" URL from a std::vector of candidates\n"
		"\n"
		"Service discovery will often return a list of candidate URLs to\n"
		"use to connect to a node. This function uses heuristics to select\n"
		"the \"best\" URL to use. The selection criteria ranks URLs in roughly\n"
		"the following order, lower number being better:\n"
		"\n"
		"1. \"rr+intra\" for IntraTransport\n"
		"2. \"rr+local\" for LocalTransport\n"
		"3. \"rr+pci\" or \"rr+usb\" for HardwareTransport\n"
		"4. \"rrs+tcp://127.0.0.1\" for secure TcpTransport loopback\n"
		"5. \"rrs+tcp://[::1]\" for secure TcpTransport IPv6 loopback\n"
		"6. \"rrs+tcp://localhost\" for secure TcpTransport loopback\n"
		"7. \"rrs+tcp://[fe80\" for secure TcpTransport link-local IPv6\n"
		"8. \"rrs+tcp://\" for any secure TcpTransport\n"
		"9. \"rr+tcp://127.0.0.1\" for TcpTransport loopback\n"
		"10. \"rr+tcp://[::1]\" for TcpTransport IPv6 loopback\n"
		"11. \"rr+tcp://localhost\" for TcpTransport loopback\n"
		"12. \"rr+tcp://[fe80\" for TcpTransport link-local IPv6\n"
		"13. \"rr+tcp://\" for any TcpTransport\n"
		"\n"
		":param urls: The candidate URLs\n"
		":type urls: List[str]\n"
		":return: The \"best\" URL out of the candidates\n"
		":rtype: str\n"
		"\n"
		""},
	 { "RobotRaconteurNode__RegisterServiceType", _wrap_RobotRaconteurNode__RegisterServiceType, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__RegisterServiceTypes", _wrap_RobotRaconteurNode__RegisterServiceTypes, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__GetServiceType", _wrap_RobotRaconteurNode__GetServiceType, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__GetPulledServiceType", _wrap_RobotRaconteurNode__GetPulledServiceType, METH_VARARGS, NULL},
	 { "RobotRaconteurNode_IsServiceTypeRegistered", _wrap_RobotRaconteurNode_IsServiceTypeRegistered, METH_VARARGS, "\n"
		"\n"
		"\n"
		"Test if a service type has been registered\n"
		"\n"
		":param type: The name of the service type to check\n"
		":type type: str\n"
		":rtype: bool\n"
		"\n"
		""},
	 { "RobotRaconteurNode__GetRegisteredServiceTypes", _wrap_RobotRaconteurNode__GetRegisteredServiceTypes, METH_O, NULL},
	 { "RobotRaconteurNode__GetPulledServiceTypes", _wrap_RobotRaconteurNode__GetPulledServiceTypes, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__get_s", _wrap_RobotRaconteurNode__get_s, METH_NOARGS, NULL},
	 { "RobotRaconteurNode__Shutdown", _wrap_RobotRaconteurNode__Shutdown, METH_O, NULL},
	 { "RobotRaconteurNode__RequestObjectLock", _wrap_RobotRaconteurNode__RequestObjectLock, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__ReleaseObjectLock", _wrap_RobotRaconteurNode__ReleaseObjectLock, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__MonitorEnter", _wrap_RobotRaconteurNode__MonitorEnter, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__MonitorExit", _wrap_RobotRaconteurNode__MonitorExit, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__NowUTC", _wrap_RobotRaconteurNode__NowUTC, METH_O, NULL},
	 { "RobotRaconteurNode__NowTimeSpec", _wrap_RobotRaconteurNode__NowTimeSpec, METH_O, NULL},
	 { "RobotRaconteurNode__NowNodeTime", _wrap_RobotRaconteurNode__NowNodeTime, METH_O, NULL},
	 { "RobotRaconteurNode__NodeSyncTimeUTC", _wrap_RobotRaconteurNode__NodeSyncTimeUTC, METH_O, NULL},
	 { "RobotRaconteurNode__NodeSyncTimeSpec", _wrap_RobotRaconteurNode__NodeSyncTimeSpec, METH_O, NULL},
	 { "RobotRaconteurNode__AsyncRequestObjectLock", _wrap_RobotRaconteurNode__AsyncRequestObjectLock, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__AsyncReleaseObjectLock", _wrap_RobotRaconteurNode__AsyncReleaseObjectLock, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__GetServiceAttributes", _wrap_RobotRaconteurNode__GetServiceAttributes, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__GetServiceNodeID", _wrap_RobotRaconteurNode__GetServiceNodeID, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__GetServiceNodeName", _wrap_RobotRaconteurNode__GetServiceNodeName, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__GetServiceName", _wrap_RobotRaconteurNode__GetServiceName, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__GetObjectServicePath", _wrap_RobotRaconteurNode__GetObjectServicePath, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__GetObjectType", _wrap_RobotRaconteurNode__GetObjectType, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__RegisterService", _wrap_RobotRaconteurNode__RegisterService, METH_VARARGS, NULL},
	 { "RobotRaconteurNode_CloseService", _wrap_RobotRaconteurNode_CloseService, METH_VARARGS, "\n"
		"\n"
		"\n"
		"Closes a previously registered service\n"
		"\n"
		"Services are automatically closed by Shutdown, so this function\n"
		"is rarely used.\n"
		"\n"
		":param sname: The name of the service to close\n"
		":type sname: str\n"
		"\n"
		""},
	 { "RobotRaconteurNode__NodeID", _wrap_RobotRaconteurNode__NodeID, METH_O, NULL},
	 { "RobotRaconteurNode__SetNodeID", _wrap_RobotRaconteurNode__SetNodeID, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__NodeName", _wrap_RobotRaconteurNode__NodeName, METH_O, NULL},
	 { "RobotRaconteurNode__SetNodeName", _wrap_RobotRaconteurNode__SetNodeName, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__GetThreadPoolCount", _wrap_RobotRaconteurNode__GetThreadPoolCount, METH_O, NULL},
	 { "RobotRaconteurNode__SetThreadPoolCount", _wrap_RobotRaconteurNode__SetThreadPoolCount, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__FindObjectType", _wrap_RobotRaconteurNode__FindObjectType, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__AsyncFindObjectType", _wrap_RobotRaconteurNode__AsyncFindObjectType, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__SetExceptionHandler", _wrap_RobotRaconteurNode__SetExceptionHandler, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__ClearExceptionHandler", _wrap_RobotRaconteurNode__ClearExceptionHandler, METH_O, NULL},
	 { "RobotRaconteurNode_Sleep", _wrap_RobotRaconteurNode_Sleep, METH_VARARGS, "\n"
		"\n"
		"\n"
		"Sleeps for a specified duration\n"
		"\n"
		"Normally will sleep based on the system clock, but in certain\n"
		"circumstances will use simulation time\n"
		"\n"
		":param duration: Duration to sleep in seconds\n"
		":type duration: float\n"
		"\n"
		""},
	 { "RobotRaconteurNode__CreateTimer", _wrap_RobotRaconteurNode__CreateTimer, METH_VARARGS, NULL},
	 { "RobotRaconteurNode_CreateRate", _wrap_RobotRaconteurNode_CreateRate, METH_VARARGS, "\n"
		"\n"
		"\n"
		"Create a Rate object\n"
		"\n"
		"Rate is used to stabilize periodic loops to a specified frequency\n"
		"\n"
		"This function will normally return a WallRate instance\n"
		"\n"
		":param frequency: Frequency of loop in Hz\n"
		":type frequency: float\n"
		":return: The new Rate object\n"
		":rtype: RobotRaconteur.Rate\n"
		"\n"
		""},
	 { "RobotRaconteurNode_CreateAutoResetEvent", _wrap_RobotRaconteurNode_CreateAutoResetEvent, METH_O, "\n"
		"\n"
		"Create an AutoResetEvent object\n"
		"\n"
		"Normally the AutoResetEvent will use the system clock for timeouts,\n"
		"but in certain circumstances will use simulation time\n"
		"\n"
		":return: The new AutoResetEvent object\n"
		":rtype: RobotRaconteur.AutoResetEvent\n"
		"\n"
		""},
	 { "RobotRaconteurNode__PostToThreadPool", _wrap_RobotRaconteurNode__PostToThreadPool, METH_VARARGS, NULL},
	 { "RobotRaconteurNode__GetRobotRaconteurVersion", _wrap_RobotRaconteurNode__GetRobotRaconteurVersion, METH_O, NULL},
	 { "RobotRaconteurNode_CompareLogLevel", _wrap_RobotRaconteurNode_CompareLogLevel, METH_VARARGS, "\n"
		"\n"
		"\n"
		"Test if the specified log level would be accepted\n"
		"\n"
		":param log_level: Log level to test\n"
		":type log_level: int\n"
		":return: True if the log would be accepted, False if it would be ignored\n"
		":rtype: bool\n"
		"\n"
		""},
	 { "RobotRaconteurNode_LogMessage", _wrap_RobotRaconteurNode_LogMessage, METH_VARARGS, "\n"
		"\n"
		"\n"
		"Log a simple message using the current node\n"
		"\n"
		"The record will be sent to the configured log handler,\n"
		"or sent to std::cerr if none is configured\n"
		"\n"
		"If the level of the message is below the current log level\n"
		"for the node, the record will be ignored\n"
		"\n"
		"\n"
		""},
	 { "RobotRaconteurNode_LogRecord", _wrap_RobotRaconteurNode_LogRecord, METH_VARARGS, "\n"
		"\n"
		"\n"
		"Log a record to the node.\n"
		"\n"
		"The record will be sent to the configured log handler,\n"
		"or sent to std::cerr if none is configured\n"
		"\n"
		"If the level of the message is below the current log level\n"
		"for the node, it will be ignored\n"
		"\n"
		":param record: The record to log\n"
		":type record: RRLogRecord\n"
		"\n"
		""},
	 { "RobotRaconteurNode_GetLogLevel", _wrap_RobotRaconteurNode_GetLogLevel, METH_O, "\n"
		"\n"
		"\n"
		"Get the current log level for the node\n"
		"\n"
		"Default level is \"info\"\n"
		"\n"
		":return: The current log level\n"
		":rtype: int\n"
		"\n"
		""},
	 { "RobotRaconteurNode_SetLogLevel", _wrap_RobotRaconteurNode_SetLogLevel, METH_VARARGS, "\n"
		"\n"
		"\n"
		"Set the log level for the node\n"
		"\n"
		"Set RobotRaconteur.RobotRaconteur_LogLevel_Disable to disable logging\n"
		"\n"
		":param level: The desired log level\n"
		":type level: int\n"
		"\n"
		""},
	 { "RobotRaconteurNode_SetLogLevelFromString", _wrap_RobotRaconteurNode_SetLogLevelFromString, METH_VARARGS, "\n"
		"\n"
		"\n"
		"Set the log level for the node from a string\n"
		"\n"
		"Must be one of the following values: DISABLE, FATAL, ERROR, WARNING, INFO, DEBUG, TRACE\n"
		"\n"
		"Defaults to WARNING\n"
		"\n"
		":param level: The desired log level\n"
		":type level: str\n"
		":return: The log level\n"
		":rtype: int\n"
		"\n"
		""},
	 { "RobotRaconteurNode_SetLogLevelFromEnvVariable", _wrap_RobotRaconteurNode_SetLogLevelFromEnvVariable, METH_VARARGS, "\n"
		"\n"
		"\n"
		"Set the log level for the node from specified environmental variable\n"
		"\n"
		"Retrieves the specified environmental variable and sets the log level based\n"
		"on one of the following values: DISABLE, FATAL, ERROR, WARNING, INFO, DEBUG, TRACE\n"
		"\n"
		"If an invalid value or the variable does not exist, the log level is left unchanged.\n"
		"\n"
		":param env_variable_name: The environmental variable to use. Defaults to\n"
		" ``ROBOTRACONTEUR_LOG_LEVEL``\n"
		":type env_variable_name: str\n"
		":return: The log level\n"
		":rtype: int\n"
		"\n"
		""},
	 { "RobotRaconteurNode_GetLogRecordHandler", _wrap_RobotRaconteurNode_GetLogRecordHandler, METH_O, "\n"
		"\n"
		"\n"
		"Get the currently configured log record handler\n"
		"\n"
		"If None, records are sent to ``stdout``\n"
		"\n"
		":return: The log record handler\n"
		":rtype: LogRecordHandler\n"
		"\n"
		""},
	 { "RobotRaconteurNode_SetLogRecordHandler", _wrap_RobotRaconteurNode_SetLogRecordHandler, METH_VARARGS, "\n"
		"\n"
		"\n"
		"Set the handler for log records\n"
		"\n"
		"If handler is NULL, records are sent to std::cerr\n"
		"\n"
		":param handler: The log record handler function\n"
		":type handler: RobotRaconteur.LogRecordHandler\n"
		"\n"
		""},
	 { "RobotRaconteurNode_CheckConnection", _wrap_RobotRaconteurNode_CheckConnection, METH_VARARGS, "\n"
		"\n"
		"\n"
		"Check that the TransportConnection associated with an endpoint\n"
		"is connected\n"
		"\n"
		"Raises `RobotRaconteur.ConnectionException` or other exception if connection is invalid.\n"
		"\n"
		":param endpoint: The LocalEndpoint identifier to check\n"
		":type endpoint: int\n"
		"\n"
		""},
	 { "RobotRaconteurNode_GetNodeDirectories", _wrap_RobotRaconteurNode_GetNodeDirectories, METH_O, NULL},
	 { "RobotRaconteurNode_SetNodeDirectories", _wrap_RobotRaconteurNode_SetNodeDirectories, METH_VARARGS, NULL},
	 { "RobotRaconteurNode_Init", _wrap_RobotRaconteurNode_Init, METH_VARARGS, "\n"
		"Initialize the node. Called automatically for ``RRN``\n"
		"\n"
		"This function must be called to initialize background tasks before\n"
		"using the node. It is called automatically by the\n"
		"singleton accessors, so the user only needs to call this function\n"
		"when not using the singleton. If a custom thread pool is being\n"
		"used, the thread pool factory must be specified before\n"
		"calling init.\n"
		"\n"
		":param thread_count: The initial number of threads in the thread pool (default 20)\n"
		":type thread_count: int\n"
		"\n"
		""},
	 { "new_RobotRaconteurNode", _wrap_new_RobotRaconteurNode, METH_NOARGS, NULL},
	 { "delete_RobotRaconteurNode", _wrap_delete_RobotRaconteurNode, METH_O, NULL},
	 { "RobotRaconteurNode_swigregister", RobotRaconteurNode_swigregister, METH_O, NULL},
	 { "RobotRaconteurNode_swiginit", RobotRaconteurNode_swiginit, METH_VARARGS, NULL},
	 { "_NewStructure", _wrap__NewStructure, METH_VARARGS, NULL},
	 { "_GetStructureType", _wrap__GetStructureType, METH_VARARGS, NULL},
	 { "_GetNumPyDescrForType", _wrap__GetNumPyDescrForType, METH_VARARGS, NULL},
	 { "_GetNamedTypeEntryType", _wrap__GetNamedTypeEntryType, METH_VARARGS, NULL},
	 { "_PackMessageElement", _wrap__PackMessageElement, METH_VARARGS, NULL},
	 { "_UnpackMessageElement", _wrap__UnpackMessageElement, METH_VARARGS, NULL},
	 { "_PackToRRArray", _wrap__PackToRRArray, METH_VARARGS, NULL},
	 { "_UnpackFromRRArray", _wrap__UnpackFromRRArray, METH_VARARGS, NULL},
	 { "_RRTypeIdToNumPyDataType", _wrap__RRTypeIdToNumPyDataType, METH_O, NULL},
	 { "new_CommandLineConfigParser", _wrap_new_CommandLineConfigParser, METH_VARARGS, NULL},
	 { "CommandLineConfigParser_SetDefaults", _wrap_CommandLineConfigParser_SetDefaults, METH_VARARGS, "\n"
		"\n"
		"Set the default NodeName, TCP port, and flags\n"
		"\n"
		"The command line options will be allowed to override the options\n"
		"specified in allowed_overrides passed to CommandLineConfigParser().\n"
		"\n"
		":param node_name: The default NodeName\n"
		":type node_name str\n"
		":param tcp_port: The default TCP port\n"
		":type tcp_port: int\n"
		":param default_flags: The default flags\n"
		":type default_flags: int\n"
		"\n"
		""},
	 { "CommandLineConfigParser_AddStringOption", _wrap_CommandLineConfigParser_AddStringOption, METH_VARARGS, "\n"
		"\n"
		"Add a new string option\n"
		"\n"
		":param name: The name of the option\n"
		":type name: str\n"
		":param descr: Description of the option\n"
		":type descr: str\n"
		"\n"
		""},
	 { "CommandLineConfigParser_AddBoolOption", _wrap_CommandLineConfigParser_AddBoolOption, METH_VARARGS, "\n"
		"\n"
		"Add a new bool option\n"
		"\n"
		":param name: The name of the option\n"
		":type name: str\n"
		":param descr: Description of the option\n"
		":type descr: str\n"
		"\n"
		""},
	 { "CommandLineConfigParser_AddIntOption", _wrap_CommandLineConfigParser_AddIntOption, METH_VARARGS, "\n"
		"\n"
		"Add a new int32_t option\n"
		"\n"
		":param name: The name of the option\n"
		":type name: str\n"
		":param descr: Description of the option\n"
		":type descr: str\n"
		"\n"
		""},
	 { "CommandLineConfigParser_ParseCommandLine", _wrap_CommandLineConfigParser_ParseCommandLine, METH_VARARGS, "\n"
		"\n"
		"Parse a specified string vector containing the options\n"
		"\n"
		"Results are stored in the instance\n"
		"\n"
		":param args: The options as a string list\n"
		":type args: List[str]\n"
		"\n"
		""},
	 { "CommandLineConfigParser_GetOptionOrDefaultAsString", _wrap_CommandLineConfigParser_GetOptionOrDefaultAsString, METH_VARARGS, "\n"
		"\n"
		"Get the option value as a string\n"
		"\n"
		"Returns empty string if option not specified on command line\n"
		"\n"
		":param option: The name of the option\n"
		":type option: str\n"
		":return: The option value, or an empty string\n"
		":rtype: str\n"
		"\n"
		""},
	 { "CommandLineConfigParser_GetOptionOrDefaultAsBool", _wrap_CommandLineConfigParser_GetOptionOrDefaultAsBool, METH_VARARGS, "\n"
		"\n"
		"Get the option value as a bool\n"
		"\n"
		"Returns false if option not specified on command line\n"
		"\n"
		":param option: The name of the option\n"
		":type option: str\n"
		":return: The option value, or False\n"
		":rtype: bool\n"
		"\n"
		""},
	 { "CommandLineConfigParser_GetOptionOrDefaultAsInt", _wrap_CommandLineConfigParser_GetOptionOrDefaultAsInt, METH_VARARGS, "\n"
		"\n"
		"Get the option value as an int\n"
		"\n"
		"Returns -1 if option not specified on command line\n"
		"\n"
		":param option: The name of the option\n"
		":type option: str\n"
		":return: The option value, or -1\n"
		":rtype: int\n"
		"\n"
		""},
	 { "delete_CommandLineConfigParser", _wrap_delete_CommandLineConfigParser, METH_O, NULL},
	 { "CommandLineConfigParser_swigregister", CommandLineConfigParser_swigregister, METH_O, NULL},
	 { "CommandLineConfigParser_swiginit", CommandLineConfigParser_swiginit, METH_VARARGS, NULL},
	 { "WrappedRobotRaconteurNodeSetup_GetLocalTransport", _wrap_WrappedRobotRaconteurNodeSetup_GetLocalTransport, METH_O, NULL},
	 { "WrappedRobotRaconteurNodeSetup_GetIntraTransport", _wrap_WrappedRobotRaconteurNodeSetup_GetIntraTransport, METH_O, NULL},
	 { "WrappedRobotRaconteurNodeSetup_GetTcpTransport", _wrap_WrappedRobotRaconteurNodeSetup_GetTcpTransport, METH_O, NULL},
	 { "WrappedRobotRaconteurNodeSetup_GetHardwareTransport", _wrap_WrappedRobotRaconteurNodeSetup_GetHardwareTransport, METH_O, NULL},
	 { "WrappedRobotRaconteurNodeSetup_GetCommandLineConfig", _wrap_WrappedRobotRaconteurNodeSetup_GetCommandLineConfig, METH_O, NULL},
	 { "WrappedRobotRaconteurNodeSetup_ReleaseNode", _wrap_WrappedRobotRaconteurNodeSetup_ReleaseNode, METH_O, NULL},
	 { "delete_WrappedRobotRaconteurNodeSetup", _wrap_delete_WrappedRobotRaconteurNodeSetup, METH_O, NULL},
	 { "new_WrappedRobotRaconteurNodeSetup", _wrap_new_WrappedRobotRaconteurNodeSetup, METH_VARARGS, NULL},
	 { "WrappedRobotRaconteurNodeSetup_swigregister", WrappedRobotRaconteurNodeSetup_swigregister, METH_O, NULL},
	 { "WrappedRobotRaconteurNodeSetup_swiginit", WrappedRobotRaconteurNodeSetup_swiginit, METH_VARARGS, NULL},
	 { "new_BroadcastDownsampler", _wrap_new_BroadcastDownsampler, METH_NOARGS, NULL},
	 { "BroadcastDownsampler__Init", _wrap_BroadcastDownsampler__Init, METH_VARARGS, NULL},
	 { "BroadcastDownsampler_GetClientDownsample", _wrap_BroadcastDownsampler_GetClientDownsample, METH_VARARGS, "\n"
		"\n"
		"Get the downsample for the specified client\n"
		"\n"
		":param ep: The endpoint ID of the client\n"
		":type ep: int\n"
		":return: The downsample\n"
		":rtype: int\n"
		"\n"
		""},
	 { "BroadcastDownsampler_SetClientDownsample", _wrap_BroadcastDownsampler_SetClientDownsample, METH_VARARGS, "\n"
		"\n"
		"Set the downsample for the specified client\n"
		"\n"
		":param ep: The endpoint ID of the client\n"
		":type ep: int\n"
		":param downsample: The desired downsample\n"
		":type downsample: int\n"
		"\n"
		""},
	 { "BroadcastDownsampler_BeginStep", _wrap_BroadcastDownsampler_BeginStep, METH_O, "\n"
		"\n"
		"Begin the update loop step\n"
		"\n"
		"Use BroadcastDownsamplerStep for automatic stepping\n"
		"\n"
		""},
	 { "BroadcastDownsampler_EndStep", _wrap_BroadcastDownsampler_EndStep, METH_O, "\n"
		"\n"
		"End the update loop step\n"
		"\n"
		"Use BroadcastDownsamplerStep for automatic stepping\n"
		"\n"
		""},
	 { "BroadcastDownsampler__AddPipeBroadcaster", _wrap_BroadcastDownsampler__AddPipeBroadcaster, METH_VARARGS, NULL},
	 { "BroadcastDownsampler__AddWireBroadcaster", _wrap_BroadcastDownsampler__AddWireBroadcaster, METH_VARARGS, NULL},
	 { "delete_BroadcastDownsampler", _wrap_delete_BroadcastDownsampler, METH_O, NULL},
	 { "BroadcastDownsampler_swigregister", BroadcastDownsampler_swigregister, METH_O, NULL},
	 { "BroadcastDownsampler_swiginit", BroadcastDownsampler_swiginit, METH_VARARGS, NULL},
	 { "RRNativeDirectorSupport_Start", _wrap_RRNativeDirectorSupport_Start, METH_NOARGS, NULL},
	 { "RRNativeDirectorSupport_Stop", _wrap_RRNativeDirectorSupport_Stop, METH_NOARGS, NULL},
	 { "RRNativeDirectorSupport_IsRunning", _wrap_RRNativeDirectorSupport_IsRunning, METH_NOARGS, NULL},
	 { "new_RRNativeDirectorSupport", _wrap_new_RRNativeDirectorSupport, METH_NOARGS, NULL},
	 { "delete_RRNativeDirectorSupport", _wrap_delete_RRNativeDirectorSupport, METH_O, NULL},
	 { "RRNativeDirectorSupport_swigregister", RRNativeDirectorSupport_swigregister, METH_O, NULL},
	 { "RRNativeDirectorSupport_swiginit", RRNativeDirectorSupport_swiginit, METH_VARARGS, NULL},
	 { "SetPythonTracebackPrintExc", _wrap_SetPythonTracebackPrintExc, METH_O, NULL},
	 { NULL, NULL, 0, NULL }
};

static PyMethodDef SwigMethods_proxydocs[] = {
	 { NULL, NULL, 0, NULL }
};


/* -------- TYPE CONVERSION AND EQUIVALENCE RULES (BEGIN) -------- */

static void *_p_RobotRaconteur__TcpTransportTo_p_RobotRaconteur__Transport(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::Transport *)  ((RobotRaconteur::TcpTransport *) x));
}
static void *_p_RobotRaconteur__LocalTransportTo_p_RobotRaconteur__Transport(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::Transport *)  ((RobotRaconteur::LocalTransport *) x));
}
static void *_p_RobotRaconteur__HardwareTransportTo_p_RobotRaconteur__Transport(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::Transport *)  ((RobotRaconteur::HardwareTransport *) x));
}
static void *_p_RobotRaconteur__IntraTransportTo_p_RobotRaconteur__Transport(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::Transport *)  ((RobotRaconteur::IntraTransport *) x));
}
static void *_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_tTo_p_boost__shared_ptrT_RobotRaconteur__RRObject_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::RRObject >(*(boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *)x);
}
static void *_p_boost__shared_ptrT_RobotRaconteur__WrappedRRObject_tTo_p_boost__shared_ptrT_RobotRaconteur__RRObject_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::RRObject >(*(boost::shared_ptr< RobotRaconteur::WrappedRRObject > *)x);
}
static void *_p_RobotRaconteur__PropertyDefinitionTo_p_RobotRaconteur__MemberDefinition(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::MemberDefinition *)  ((RobotRaconteur::PropertyDefinition *) x));
}
static void *_p_RobotRaconteur__FunctionDefinitionTo_p_RobotRaconteur__MemberDefinition(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::MemberDefinition *)  ((RobotRaconteur::FunctionDefinition *) x));
}
static void *_p_RobotRaconteur__EventDefinitionTo_p_RobotRaconteur__MemberDefinition(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::MemberDefinition *)  ((RobotRaconteur::EventDefinition *) x));
}
static void *_p_RobotRaconteur__ObjRefDefinitionTo_p_RobotRaconteur__MemberDefinition(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::MemberDefinition *)  ((RobotRaconteur::ObjRefDefinition *) x));
}
static void *_p_RobotRaconteur__PipeDefinitionTo_p_RobotRaconteur__MemberDefinition(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::MemberDefinition *)  ((RobotRaconteur::PipeDefinition *) x));
}
static void *_p_RobotRaconteur__CallbackDefinitionTo_p_RobotRaconteur__MemberDefinition(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::MemberDefinition *)  ((RobotRaconteur::CallbackDefinition *) x));
}
static void *_p_RobotRaconteur__WireDefinitionTo_p_RobotRaconteur__MemberDefinition(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::MemberDefinition *)  ((RobotRaconteur::WireDefinition *) x));
}
static void *_p_RobotRaconteur__MemoryDefinitionTo_p_RobotRaconteur__MemberDefinition(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::MemberDefinition *)  ((RobotRaconteur::MemoryDefinition *) x));
}
static void *_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_tTo_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::MessageElementData >(*(boost::shared_ptr< RobotRaconteur::RRBaseArray > *)x);
}
static void *_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_tTo_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::MessageElementData >(*(boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *)x);
}
static void *_p_boost__shared_ptrT_RobotRaconteur__WallTimer_tTo_p_boost__shared_ptrT_RobotRaconteur__Timer_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::Timer >(*(boost::shared_ptr< RobotRaconteur::WallTimer > *)x);
}
static void *_p_boost__shared_ptrT_RobotRaconteur__PipeDefinition_tTo_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::MemberDefinition >(*(boost::shared_ptr< RobotRaconteur::PipeDefinition > *)x);
}
static void *_p_boost__shared_ptrT_RobotRaconteur__WireDefinition_tTo_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::MemberDefinition >(*(boost::shared_ptr< RobotRaconteur::WireDefinition > *)x);
}
static void *_p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_tTo_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::MemberDefinition >(*(boost::shared_ptr< RobotRaconteur::ObjRefDefinition > *)x);
}
static void *_p_boost__shared_ptrT_RobotRaconteur__MemoryDefinition_tTo_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::MemberDefinition >(*(boost::shared_ptr< RobotRaconteur::MemoryDefinition > *)x);
}
static void *_p_boost__shared_ptrT_RobotRaconteur__EventDefinition_tTo_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::MemberDefinition >(*(boost::shared_ptr< RobotRaconteur::EventDefinition > *)x);
}
static void *_p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_tTo_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::MemberDefinition >(*(boost::shared_ptr< RobotRaconteur::PropertyDefinition > *)x);
}
static void *_p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_tTo_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::MemberDefinition >(*(boost::shared_ptr< RobotRaconteur::FunctionDefinition > *)x);
}
static void *_p_boost__shared_ptrT_RobotRaconteur__CallbackDefinition_tTo_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::MemberDefinition >(*(boost::shared_ptr< RobotRaconteur::CallbackDefinition > *)x);
}
static void *_p_RobotRaconteur__WrappedServiceStubTo_p_RobotRaconteur__RRObject(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::RRObject *)  ((RobotRaconteur::WrappedServiceStub *) x));
}
static void *_p_RobotRaconteur__WrappedRRObjectTo_p_RobotRaconteur__RRObject(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::RRObject *)  ((RobotRaconteur::WrappedRRObject *) x));
}
static void *_p_RobotRaconteur__UserLogRecordHandlerBaseTo_p_RobotRaconteur__LogRecordHandler(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::LogRecordHandler *)  ((RobotRaconteur::UserLogRecordHandlerBase *) x));
}
static void *_p_boost__shared_ptrT_RobotRaconteur__WallRate_tTo_p_boost__shared_ptrT_RobotRaconteur__Rate_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::Rate >(*(boost::shared_ptr< RobotRaconteur::WallRate > *)x);
}
static void *_p_RobotRaconteur__PasswordFileUserAuthenticatorTo_p_RobotRaconteur__UserAuthenticator(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::UserAuthenticator *)  ((RobotRaconteur::PasswordFileUserAuthenticator *) x));
}
static void *_p_RobotRaconteur__WrappedUserAuthenticatorTo_p_RobotRaconteur__UserAuthenticator(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::UserAuthenticator *)  ((RobotRaconteur::WrappedUserAuthenticator *) x));
}
static void *_p_RobotRaconteur__WallRateTo_p_RobotRaconteur__Rate(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::Rate *)  ((RobotRaconteur::WallRate *) x));
}
static void *_p_RobotRaconteur__MessageElementTo_p_RobotRaconteur__RRValue(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::RRValue *)  ((RobotRaconteur::MessageElement *) x));
}
static void *_p_RobotRaconteur__RRBaseArrayTo_p_RobotRaconteur__RRValue(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::RRValue *) (RobotRaconteur::MessageElementData *) ((RobotRaconteur::RRBaseArray *) x));
}
static void *_p_RobotRaconteur__MessageEntryTo_p_RobotRaconteur__RRValue(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::RRValue *)  ((RobotRaconteur::MessageEntry *) x));
}
static void *_p_RobotRaconteur__MessageElementNestedElementListTo_p_RobotRaconteur__RRValue(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::RRValue *) (RobotRaconteur::MessageElementData *) ((RobotRaconteur::MessageElementNestedElementList *) x));
}
static void *_p_RobotRaconteur__MessageTo_p_RobotRaconteur__RRValue(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::RRValue *)  ((RobotRaconteur::Message *) x));
}
static void *_p_RobotRaconteur__MessageHeaderTo_p_RobotRaconteur__RRValue(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::RRValue *)  ((RobotRaconteur::MessageHeader *) x));
}
static void *_p_RobotRaconteur__MessageElementDataTo_p_RobotRaconteur__RRValue(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::RRValue *)  ((RobotRaconteur::MessageElementData *) x));
}
static void *_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_tTo_p_boost__shared_ptrT_RobotRaconteur__Transport_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::Transport >(*(boost::shared_ptr< RobotRaconteur::LocalTransport > *)x);
}
static void *_p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_tTo_p_boost__shared_ptrT_RobotRaconteur__Transport_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::Transport >(*(boost::shared_ptr< RobotRaconteur::HardwareTransport > *)x);
}
static void *_p_boost__shared_ptrT_RobotRaconteur__IntraTransport_tTo_p_boost__shared_ptrT_RobotRaconteur__Transport_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::Transport >(*(boost::shared_ptr< RobotRaconteur::IntraTransport > *)x);
}
static void *_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_tTo_p_boost__shared_ptrT_RobotRaconteur__Transport_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::Transport >(*(boost::shared_ptr< RobotRaconteur::TcpTransport > *)x);
}
static void *_p_RobotRaconteur__WallTimerTo_p_RobotRaconteur__Timer(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::Timer *)  ((RobotRaconteur::WallTimer *) x));
}
static void *_p_boost__shared_ptrT_RobotRaconteur__UserLogRecordHandlerBase_tTo_p_boost__shared_ptrT_RobotRaconteur__LogRecordHandler_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::LogRecordHandler >(*(boost::shared_ptr< RobotRaconteur::UserLogRecordHandlerBase > *)x);
}
static void *_p_boost__shared_ptrT_RobotRaconteur__PasswordFileUserAuthenticator_tTo_p_boost__shared_ptrT_RobotRaconteur__UserAuthenticator_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::UserAuthenticator >(*(boost::shared_ptr< RobotRaconteur::PasswordFileUserAuthenticator > *)x);
}
static void *_p_boost__shared_ptrT_RobotRaconteur__WrappedUserAuthenticator_tTo_p_boost__shared_ptrT_RobotRaconteur__UserAuthenticator_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::UserAuthenticator >(*(boost::shared_ptr< RobotRaconteur::WrappedUserAuthenticator > *)x);
}
static void *_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_tTo_p_boost__shared_ptrT_RobotRaconteur__RRValue_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::RRValue >(*(boost::shared_ptr< RobotRaconteur::RRBaseArray > *)x);
}
static void *_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_tTo_p_boost__shared_ptrT_RobotRaconteur__RRValue_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::RRValue >(*(boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *)x);
}
static void *_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_tTo_p_boost__shared_ptrT_RobotRaconteur__RRValue_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::RRValue >(*(boost::shared_ptr< RobotRaconteur::MessageEntry > *)x);
}
static void *_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_tTo_p_boost__shared_ptrT_RobotRaconteur__RRValue_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::RRValue >(*(boost::shared_ptr< RobotRaconteur::MessageElementData > *)x);
}
static void *_p_boost__shared_ptrT_RobotRaconteur__Message_tTo_p_boost__shared_ptrT_RobotRaconteur__RRValue_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::RRValue >(*(boost::shared_ptr< RobotRaconteur::Message > *)x);
}
static void *_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_tTo_p_boost__shared_ptrT_RobotRaconteur__RRValue_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::RRValue >(*(boost::shared_ptr< RobotRaconteur::MessageHeader > *)x);
}
static void *_p_boost__shared_ptrT_RobotRaconteur__MessageElement_tTo_p_boost__shared_ptrT_RobotRaconteur__RRValue_t(void *x, int *newmemory) {
    *newmemory = SWIG_CAST_NEW_MEMORY;
    return (void *) new boost::shared_ptr< RobotRaconteur::RRValue >(*(boost::shared_ptr< RobotRaconteur::MessageElement > *)x);
}
static void *_p_RobotRaconteur__NodeDirectoriesResourceAlreadyInUseTo_p_std__runtime_error(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((std::runtime_error *)  ((RobotRaconteur::NodeDirectoriesResourceAlreadyInUse *) x));
}
static void *_p_RobotRaconteur__RRBaseArrayTo_p_RobotRaconteur__MessageElementData(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::MessageElementData *)  ((RobotRaconteur::RRBaseArray *) x));
}
static void *_p_RobotRaconteur__MessageElementNestedElementListTo_p_RobotRaconteur__MessageElementData(void *x, int *SWIGUNUSEDPARM(newmemory)) {
    return (void *)((RobotRaconteur::MessageElementData *)  ((RobotRaconteur::MessageElementNestedElementList *) x));
}
static swig_type_info _swigt__p_ClientServiceListenerDirector = {"_p_ClientServiceListenerDirector", "ClientServiceListenerDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_MemberDefinitionUtil = {"_p_MemberDefinitionUtil", "MemberDefinitionUtil *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_MessageElementDataUtil = {"_p_MessageElementDataUtil", "MessageElementDataUtil *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__ArrayMemoryBase = {"_p_RobotRaconteur__ArrayMemoryBase", "RobotRaconteur::ArrayMemoryBase *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__AsyncGeneratorClientReturnDirector = {"_p_RobotRaconteur__AsyncGeneratorClientReturnDirector", "RobotRaconteur::AsyncGeneratorClientReturnDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__AsyncNodeInfo2VectorReturnDirector = {"_p_RobotRaconteur__AsyncNodeInfo2VectorReturnDirector", "RobotRaconteur::AsyncNodeInfo2VectorReturnDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__AsyncPipeEndpointReturnDirector = {"_p_RobotRaconteur__AsyncPipeEndpointReturnDirector", "RobotRaconteur::AsyncPipeEndpointReturnDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__AsyncRequestDirector = {"_p_RobotRaconteur__AsyncRequestDirector", "RobotRaconteur::AsyncRequestDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__AsyncServiceInfo2VectorReturnDirector = {"_p_RobotRaconteur__AsyncServiceInfo2VectorReturnDirector", "RobotRaconteur::AsyncServiceInfo2VectorReturnDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__AsyncStringReturnDirector = {"_p_RobotRaconteur__AsyncStringReturnDirector", "RobotRaconteur::AsyncStringReturnDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__AsyncStubReturnDirector = {"_p_RobotRaconteur__AsyncStubReturnDirector", "RobotRaconteur::AsyncStubReturnDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__AsyncTimerEventReturnDirector = {"_p_RobotRaconteur__AsyncTimerEventReturnDirector", "RobotRaconteur::AsyncTimerEventReturnDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__AsyncUInt32ReturnDirector = {"_p_RobotRaconteur__AsyncUInt32ReturnDirector", "RobotRaconteur::AsyncUInt32ReturnDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__AsyncVoidNoErrReturnDirector = {"_p_RobotRaconteur__AsyncVoidNoErrReturnDirector", "RobotRaconteur::AsyncVoidNoErrReturnDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__AsyncVoidReturnDirector = {"_p_RobotRaconteur__AsyncVoidReturnDirector", "RobotRaconteur::AsyncVoidReturnDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__AsyncWireConnectionReturnDirector = {"_p_RobotRaconteur__AsyncWireConnectionReturnDirector", "RobotRaconteur::AsyncWireConnectionReturnDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__AsyncWirePeekReturnDirector = {"_p_RobotRaconteur__AsyncWirePeekReturnDirector", "RobotRaconteur::AsyncWirePeekReturnDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__AuthenticatedUser = {"_p_RobotRaconteur__AuthenticatedUser", "RobotRaconteur::AuthenticatedUser *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__AutoResetEvent = {"_p_RobotRaconteur__AutoResetEvent", "RobotRaconteur::AutoResetEvent *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__BroadcastDownsampler = {"_p_RobotRaconteur__BroadcastDownsampler", "RobotRaconteur::BroadcastDownsampler *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__CallbackDefinition = {"_p_RobotRaconteur__CallbackDefinition", "RobotRaconteur::CallbackDefinition *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__CommandLineConfigParser = {"_p_RobotRaconteur__CommandLineConfigParser", "RobotRaconteur::CommandLineConfigParser *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__ConstantDefinition = {"_p_RobotRaconteur__ConstantDefinition", "RobotRaconteur::ConstantDefinition *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__ConstantDefinition_StructField = {"_p_RobotRaconteur__ConstantDefinition_StructField", "std::vector< RobotRaconteur::ConstantDefinition_StructField >::value_type *|RobotRaconteur::ConstantDefinition_StructField *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__EnumDefinition = {"_p_RobotRaconteur__EnumDefinition", "RobotRaconteur::EnumDefinition *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__EnumDefinitionValue = {"_p_RobotRaconteur__EnumDefinitionValue", "RobotRaconteur::EnumDefinitionValue *|std::vector< RobotRaconteur::EnumDefinitionValue >::value_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__EventDefinition = {"_p_RobotRaconteur__EventDefinition", "RobotRaconteur::EventDefinition *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__ExceptionDefinition = {"_p_RobotRaconteur__ExceptionDefinition", "RobotRaconteur::ExceptionDefinition *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__FileLogRecordHandler = {"_p_RobotRaconteur__FileLogRecordHandler", "RobotRaconteur::FileLogRecordHandler *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__FunctionDefinition = {"_p_RobotRaconteur__FunctionDefinition", "RobotRaconteur::FunctionDefinition *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__GetUuidForNameAndLockResult = {"_p_RobotRaconteur__GetUuidForNameAndLockResult", "RobotRaconteur::GetUuidForNameAndLockResult *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__HandlerErrorInfo = {"_p_RobotRaconteur__HandlerErrorInfo", "RobotRaconteur::HandlerErrorInfo *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__HardwareTransport = {"_p_RobotRaconteur__HardwareTransport", "RobotRaconteur::HardwareTransport *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__IntraTransport = {"_p_RobotRaconteur__IntraTransport", "RobotRaconteur::IntraTransport *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__LocalTransport = {"_p_RobotRaconteur__LocalTransport", "RobotRaconteur::LocalTransport *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__LogRecordHandler = {"_p_RobotRaconteur__LogRecordHandler", "RobotRaconteur::LogRecordHandler *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__MemberDefinition = {"_p_RobotRaconteur__MemberDefinition", "RobotRaconteur::MemberDefinition *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__MemoryDefinition = {"_p_RobotRaconteur__MemoryDefinition", "RobotRaconteur::MemoryDefinition *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__Message = {"_p_RobotRaconteur__Message", "RobotRaconteur::Message *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__MessageElement = {"_p_RobotRaconteur__MessageElement", "RobotRaconteur::MessageElement *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__MessageElementData = {"_p_RobotRaconteur__MessageElementData", "RobotRaconteur::MessageElementData *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__MessageElementNestedElementList = {"_p_RobotRaconteur__MessageElementNestedElementList", "RobotRaconteur::MessageElementNestedElementList *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__MessageEntry = {"_p_RobotRaconteur__MessageEntry", "RobotRaconteur::MessageEntry *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__MessageHeader = {"_p_RobotRaconteur__MessageHeader", "RobotRaconteur::MessageHeader *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__MultiDimArrayMemoryBase = {"_p_RobotRaconteur__MultiDimArrayMemoryBase", "RobotRaconteur::MultiDimArrayMemoryBase *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__NodeDirectories = {"_p_RobotRaconteur__NodeDirectories", "RobotRaconteur::NodeDirectories *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__NodeDirectoriesFD = {"_p_RobotRaconteur__NodeDirectoriesFD", "RobotRaconteur::NodeDirectoriesFD *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__NodeDirectoriesResourceAlreadyInUse = {"_p_RobotRaconteur__NodeDirectoriesResourceAlreadyInUse", "RobotRaconteur::NodeDirectoriesResourceAlreadyInUse *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__NodeDirectoriesUtil = {"_p_RobotRaconteur__NodeDirectoriesUtil", "RobotRaconteur::NodeDirectoriesUtil *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__NodeID = {"_p_RobotRaconteur__NodeID", "RobotRaconteur::NodeID *|::RobotRaconteur::NodeID *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__NodeInfo2 = {"_p_RobotRaconteur__NodeInfo2", "std::vector< RobotRaconteur::NodeInfo2 >::value_type *|RobotRaconteur::NodeInfo2 *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__ObjRefDefinition = {"_p_RobotRaconteur__ObjRefDefinition", "RobotRaconteur::ObjRefDefinition *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__PasswordFileUserAuthenticator = {"_p_RobotRaconteur__PasswordFileUserAuthenticator", "RobotRaconteur::PasswordFileUserAuthenticator *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__PipeDefinition = {"_p_RobotRaconteur__PipeDefinition", "RobotRaconteur::PipeDefinition *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__PropertyDefinition = {"_p_RobotRaconteur__PropertyDefinition", "RobotRaconteur::PropertyDefinition *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__RRBaseArray = {"_p_RobotRaconteur__RRBaseArray", "RobotRaconteur::RRBaseArray *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__RRLogRecord = {"_p_RobotRaconteur__RRLogRecord", "RobotRaconteur::RRLogRecord *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__RRMultiDimArrayUntyped = {"_p_RobotRaconteur__RRMultiDimArrayUntyped", "RobotRaconteur::RRMultiDimArrayUntyped *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__RRNativeDirectorSupport = {"_p_RobotRaconteur__RRNativeDirectorSupport", "RobotRaconteur::RRNativeDirectorSupport *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__RRObject = {"_p_RobotRaconteur__RRObject", "RobotRaconteur::RRObject *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__RRValue = {"_p_RobotRaconteur__RRValue", "RobotRaconteur::RRValue *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__Rate = {"_p_RobotRaconteur__Rate", "RobotRaconteur::Rate *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__RobotRaconteurNode = {"_p_RobotRaconteur__RobotRaconteurNode", "RobotRaconteur::RobotRaconteurNode *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__RobotRaconteurNodeSetup = {"_p_RobotRaconteur__RobotRaconteurNodeSetup", "RobotRaconteur::RobotRaconteurNodeSetup *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__RobotRaconteurVersion = {"_p_RobotRaconteur__RobotRaconteurVersion", "RobotRaconteur::RobotRaconteurVersion *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__ServerContext = {"_p_RobotRaconteur__ServerContext", "RobotRaconteur::ServerContext *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__ServerEndpoint = {"_p_RobotRaconteur__ServerEndpoint", "RobotRaconteur::ServerEndpoint *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__ServiceDefinition = {"_p_RobotRaconteur__ServiceDefinition", "RobotRaconteur::ServiceDefinition *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__ServiceDefinitionParseInfo = {"_p_RobotRaconteur__ServiceDefinitionParseInfo", "RobotRaconteur::ServiceDefinitionParseInfo *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__ServiceEntryDefinition = {"_p_RobotRaconteur__ServiceEntryDefinition", "RobotRaconteur::ServiceEntryDefinition *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__ServiceInfo2Wrapped = {"_p_RobotRaconteur__ServiceInfo2Wrapped", "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::mapped_type *|std::vector< RobotRaconteur::ServiceInfo2Wrapped >::value_type *|RobotRaconteur::ServiceInfo2Wrapped *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__ServicePathSegment = {"_p_RobotRaconteur__ServicePathSegment", "RobotRaconteur::ServicePathSegment *|std::vector< RobotRaconteur::ServicePathSegment >::value_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__ServiceSecurityPolicy = {"_p_RobotRaconteur__ServiceSecurityPolicy", "RobotRaconteur::ServiceSecurityPolicy *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__ServiceSubscriptionClientID = {"_p_RobotRaconteur__ServiceSubscriptionClientID", "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::key_type *|std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::key_type *|RobotRaconteur::ServiceSubscriptionClientID *|std::vector< RobotRaconteur::ServiceSubscriptionClientID >::value_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__ServiceSubscriptionFilterAttribute = {"_p_RobotRaconteur__ServiceSubscriptionFilterAttribute", "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::value_type *|RobotRaconteur::ServiceSubscriptionFilterAttribute *|std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::mapped_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup = {"_p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup", "RobotRaconteur::ServiceSubscriptionFilterAttributeGroup *|std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::mapped_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__TcpTransport = {"_p_RobotRaconteur__TcpTransport", "RobotRaconteur::TcpTransport *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__TimeSpec = {"_p_RobotRaconteur__TimeSpec", "RobotRaconteur::TimeSpec *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__Timer = {"_p_RobotRaconteur__Timer", "RobotRaconteur::Timer *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__TimerEvent = {"_p_RobotRaconteur__TimerEvent", "RobotRaconteur::TimerEvent *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__Transport = {"_p_RobotRaconteur__Transport", "RobotRaconteur::Transport *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__TryGetValueResult = {"_p_RobotRaconteur__TryGetValueResult", "RobotRaconteur::TryGetValueResult *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__TypeDefinition = {"_p_RobotRaconteur__TypeDefinition", "RobotRaconteur::TypeDefinition *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__UserAuthenticator = {"_p_RobotRaconteur__UserAuthenticator", "RobotRaconteur::UserAuthenticator *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__UserLogRecordHandlerBase = {"_p_RobotRaconteur__UserLogRecordHandlerBase", "RobotRaconteur::UserLogRecordHandlerBase *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__UserLogRecordHandlerDirector = {"_p_RobotRaconteur__UserLogRecordHandlerDirector", "RobotRaconteur::UserLogRecordHandlerDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__UsingDefinition = {"_p_RobotRaconteur__UsingDefinition", "RobotRaconteur::UsingDefinition *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WallRate = {"_p_RobotRaconteur__WallRate", "RobotRaconteur::WallRate *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WallTimer = {"_p_RobotRaconteur__WallTimer", "RobotRaconteur::WallTimer *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WireDefinition = {"_p_RobotRaconteur__WireDefinition", "RobotRaconteur::WireDefinition *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedArrayMemoryClientUtil = {"_p_RobotRaconteur__WrappedArrayMemoryClientUtil", "RobotRaconteur::WrappedArrayMemoryClientUtil *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedArrayMemoryDirector = {"_p_RobotRaconteur__WrappedArrayMemoryDirector", "RobotRaconteur::WrappedArrayMemoryDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedDirectorLock = {"_p_RobotRaconteur__WrappedDirectorLock", "RobotRaconteur::WrappedDirectorLock *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedGeneratorClient = {"_p_RobotRaconteur__WrappedGeneratorClient", "RobotRaconteur::WrappedGeneratorClient *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedGeneratorClient_TryGetNextResult = {"_p_RobotRaconteur__WrappedGeneratorClient_TryGetNextResult", "RobotRaconteur::WrappedGeneratorClient_TryGetNextResult *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedGeneratorServerDirector = {"_p_RobotRaconteur__WrappedGeneratorServerDirector", "RobotRaconteur::WrappedGeneratorServerDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedMultiDimArrayMemoryClientUtil = {"_p_RobotRaconteur__WrappedMultiDimArrayMemoryClientUtil", "RobotRaconteur::WrappedMultiDimArrayMemoryClientUtil *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedMultiDimArrayMemoryDirector = {"_p_RobotRaconteur__WrappedMultiDimArrayMemoryDirector", "RobotRaconteur::WrappedMultiDimArrayMemoryDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedMultiDimArrayMemoryParams = {"_p_RobotRaconteur__WrappedMultiDimArrayMemoryParams", "RobotRaconteur::WrappedMultiDimArrayMemoryParams *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedNamedArrayMemoryClient = {"_p_RobotRaconteur__WrappedNamedArrayMemoryClient", "RobotRaconteur::WrappedNamedArrayMemoryClient *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedNamedArrayMemoryClientBuffer = {"_p_RobotRaconteur__WrappedNamedArrayMemoryClientBuffer", "RobotRaconteur::WrappedNamedArrayMemoryClientBuffer *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedNamedArrayMemoryDirector = {"_p_RobotRaconteur__WrappedNamedArrayMemoryDirector", "RobotRaconteur::WrappedNamedArrayMemoryDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClient = {"_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClient", "RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClientBuffer = {"_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClientBuffer", "RobotRaconteur::WrappedNamedMultiDimArrayMemoryClientBuffer *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryDirector = {"_p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryDirector", "RobotRaconteur::WrappedNamedMultiDimArrayMemoryDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedPipeBroadcaster = {"_p_RobotRaconteur__WrappedPipeBroadcaster", "RobotRaconteur::WrappedPipeBroadcaster *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedPipeBroadcasterPredicateDirector = {"_p_RobotRaconteur__WrappedPipeBroadcasterPredicateDirector", "RobotRaconteur::WrappedPipeBroadcasterPredicateDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedPipeClient = {"_p_RobotRaconteur__WrappedPipeClient", "RobotRaconteur::WrappedPipeClient *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedPipeEndpoint = {"_p_RobotRaconteur__WrappedPipeEndpoint", "RobotRaconteur::WrappedPipeEndpoint *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedPipeEndpointDirector = {"_p_RobotRaconteur__WrappedPipeEndpointDirector", "RobotRaconteur::WrappedPipeEndpointDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedPipeServer = {"_p_RobotRaconteur__WrappedPipeServer", "RobotRaconteur::WrappedPipeServer *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedPipeServerConnectDirector = {"_p_RobotRaconteur__WrappedPipeServerConnectDirector", "RobotRaconteur::WrappedPipeServerConnectDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedPipeSubscription = {"_p_RobotRaconteur__WrappedPipeSubscription", "RobotRaconteur::WrappedPipeSubscription *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedPipeSubscriptionDirector = {"_p_RobotRaconteur__WrappedPipeSubscriptionDirector", "RobotRaconteur::WrappedPipeSubscriptionDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedPipeSubscription_send_iterator = {"_p_RobotRaconteur__WrappedPipeSubscription_send_iterator", "RobotRaconteur::WrappedPipeSubscription_send_iterator *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedPodArrayMemoryClient = {"_p_RobotRaconteur__WrappedPodArrayMemoryClient", "RobotRaconteur::WrappedPodArrayMemoryClient *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedPodArrayMemoryClientBuffer = {"_p_RobotRaconteur__WrappedPodArrayMemoryClientBuffer", "RobotRaconteur::WrappedPodArrayMemoryClientBuffer *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedPodArrayMemoryDirector = {"_p_RobotRaconteur__WrappedPodArrayMemoryDirector", "RobotRaconteur::WrappedPodArrayMemoryDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedPodMultiDimArrayMemoryClient = {"_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryClient", "RobotRaconteur::WrappedPodMultiDimArrayMemoryClient *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedPodMultiDimArrayMemoryClientBuffer = {"_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryClientBuffer", "RobotRaconteur::WrappedPodMultiDimArrayMemoryClientBuffer *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedPodMultiDimArrayMemoryDirector = {"_p_RobotRaconteur__WrappedPodMultiDimArrayMemoryDirector", "RobotRaconteur::WrappedPodMultiDimArrayMemoryDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedRRObject = {"_p_RobotRaconteur__WrappedRRObject", "RobotRaconteur::WrappedRRObject *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedServiceInfo2Subscription = {"_p_RobotRaconteur__WrappedServiceInfo2Subscription", "RobotRaconteur::WrappedServiceInfo2Subscription *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedServiceInfo2SubscriptionDirector = {"_p_RobotRaconteur__WrappedServiceInfo2SubscriptionDirector", "RobotRaconteur::WrappedServiceInfo2SubscriptionDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedServiceSkel = {"_p_RobotRaconteur__WrappedServiceSkel", "RobotRaconteur::WrappedServiceSkel *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedServiceSkelAsyncAdapter = {"_p_RobotRaconteur__WrappedServiceSkelAsyncAdapter", "RobotRaconteur::WrappedServiceSkelAsyncAdapter *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedServiceSkelDirector = {"_p_RobotRaconteur__WrappedServiceSkelDirector", "RobotRaconteur::WrappedServiceSkelDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedServiceStub = {"_p_RobotRaconteur__WrappedServiceStub", "RobotRaconteur::WrappedServiceStub *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedServiceStubDirector = {"_p_RobotRaconteur__WrappedServiceStubDirector", "RobotRaconteur::WrappedServiceStubDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedServiceSubscription = {"_p_RobotRaconteur__WrappedServiceSubscription", "RobotRaconteur::WrappedServiceSubscription *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedServiceSubscriptionDirector = {"_p_RobotRaconteur__WrappedServiceSubscriptionDirector", "RobotRaconteur::WrappedServiceSubscriptionDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedServiceSubscriptionFilter = {"_p_RobotRaconteur__WrappedServiceSubscriptionFilter", "RobotRaconteur::WrappedServiceSubscriptionFilter *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedServiceSubscriptionFilterNode = {"_p_RobotRaconteur__WrappedServiceSubscriptionFilterNode", "RobotRaconteur::WrappedServiceSubscriptionFilterNode *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedServiceSubscriptionFilterPredicateDirector = {"_p_RobotRaconteur__WrappedServiceSubscriptionFilterPredicateDirector", "RobotRaconteur::WrappedServiceSubscriptionFilterPredicateDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedServiceSubscriptionManager = {"_p_RobotRaconteur__WrappedServiceSubscriptionManager", "RobotRaconteur::WrappedServiceSubscriptionManager *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails = {"_p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails", "RobotRaconteur::WrappedServiceSubscriptionManagerDetails *|std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::value_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedServiceSubscription_TryDefaultClientRes = {"_p_RobotRaconteur__WrappedServiceSubscription_TryDefaultClientRes", "RobotRaconteur::WrappedServiceSubscription_TryDefaultClientRes *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedService_typed_packet = {"_p_RobotRaconteur__WrappedService_typed_packet", "RobotRaconteur::WrappedService_typed_packet *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedSubObjectSubscription = {"_p_RobotRaconteur__WrappedSubObjectSubscription", "RobotRaconteur::WrappedSubObjectSubscription *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedTryReceivePacketWaitResult = {"_p_RobotRaconteur__WrappedTryReceivePacketWaitResult", "RobotRaconteur::WrappedTryReceivePacketWaitResult *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedUserAuthenticator = {"_p_RobotRaconteur__WrappedUserAuthenticator", "RobotRaconteur::WrappedUserAuthenticator *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedUserAuthenticatorDirector = {"_p_RobotRaconteur__WrappedUserAuthenticatorDirector", "RobotRaconteur::WrappedUserAuthenticatorDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedWireBroadcaster = {"_p_RobotRaconteur__WrappedWireBroadcaster", "RobotRaconteur::WrappedWireBroadcaster *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedWireBroadcasterPredicateDirector = {"_p_RobotRaconteur__WrappedWireBroadcasterPredicateDirector", "RobotRaconteur::WrappedWireBroadcasterPredicateDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedWireClient = {"_p_RobotRaconteur__WrappedWireClient", "RobotRaconteur::WrappedWireClient *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedWireConnection = {"_p_RobotRaconteur__WrappedWireConnection", "RobotRaconteur::WrappedWireConnection *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedWireConnectionDirector = {"_p_RobotRaconteur__WrappedWireConnectionDirector", "RobotRaconteur::WrappedWireConnectionDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedWireServer = {"_p_RobotRaconteur__WrappedWireServer", "RobotRaconteur::WrappedWireServer *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedWireServerConnectDirector = {"_p_RobotRaconteur__WrappedWireServerConnectDirector", "RobotRaconteur::WrappedWireServerConnectDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedWireServerPeekValueDirector = {"_p_RobotRaconteur__WrappedWireServerPeekValueDirector", "RobotRaconteur::WrappedWireServerPeekValueDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedWireServerPokeValueDirector = {"_p_RobotRaconteur__WrappedWireServerPokeValueDirector", "RobotRaconteur::WrappedWireServerPokeValueDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedWireSubscription = {"_p_RobotRaconteur__WrappedWireSubscription", "RobotRaconteur::WrappedWireSubscription *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedWireSubscriptionDirector = {"_p_RobotRaconteur__WrappedWireSubscriptionDirector", "RobotRaconteur::WrappedWireSubscriptionDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedWireSubscription_send_iterator = {"_p_RobotRaconteur__WrappedWireSubscription_send_iterator", "RobotRaconteur::WrappedWireSubscription_send_iterator *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__WrappedWireUnicastReceiver = {"_p_RobotRaconteur__WrappedWireUnicastReceiver", "RobotRaconteur::WrappedWireUnicastReceiver *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_RobotRaconteur__detail__MultiDimArray_CalculateCopyIndicesIter = {"_p_RobotRaconteur__detail__MultiDimArray_CalculateCopyIndicesIter", "RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_ServerServiceListenerDirector = {"_p_ServerServiceListenerDirector", "ServerServiceListenerDirector *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_allocator_type = {"_p_allocator_type", "allocator_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__arrayT_unsigned_char_16_t = {"_p_boost__arrayT_unsigned_char_16_t", "boost::array< unsigned char,16 > *|boost::array< uint8_t,16 > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__filesystem__path = {"_p_boost__filesystem__path", "boost::filesystem::path *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__optionalT_std__string_t = {"_p_boost__optionalT_std__string_t", "boost::optional< std::string > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__posix_time__ptime = {"_p_boost__posix_time__ptime", "boost::posix_time::ptime *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__posix_time__time_duration = {"_p_boost__posix_time__time_duration", "boost::posix_time::time_duration *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__ArrayMemoryBase_t = {"_p_boost__shared_ptrT_RobotRaconteur__ArrayMemoryBase_t", "boost::shared_ptr< RobotRaconteur::ArrayMemoryBase > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__AuthenticatedUser_t = {"_p_boost__shared_ptrT_RobotRaconteur__AuthenticatedUser_t", "boost::shared_ptr< RobotRaconteur::AuthenticatedUser > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__AutoResetEvent_t = {"_p_boost__shared_ptrT_RobotRaconteur__AutoResetEvent_t", "boost::shared_ptr< RobotRaconteur::AutoResetEvent > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__BroadcastDownsampler_t = {"_p_boost__shared_ptrT_RobotRaconteur__BroadcastDownsampler_t", "boost::shared_ptr< RobotRaconteur::BroadcastDownsampler > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__CallbackDefinition_t = {"_p_boost__shared_ptrT_RobotRaconteur__CallbackDefinition_t", "boost::shared_ptr< RobotRaconteur::CallbackDefinition > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t = {"_p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t", "boost::shared_ptr< RobotRaconteur::CommandLineConfigParser > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t = {"_p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t", "boost::shared_ptr< RobotRaconteur::ConstantDefinition > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t = {"_p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t", "boost::shared_ptr< RobotRaconteur::EnumDefinition > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__EventDefinition_t = {"_p_boost__shared_ptrT_RobotRaconteur__EventDefinition_t", "boost::shared_ptr< RobotRaconteur::EventDefinition > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t = {"_p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t", "boost::shared_ptr< RobotRaconteur::ExceptionDefinition > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__FileLogRecordHandler_t = {"_p_boost__shared_ptrT_RobotRaconteur__FileLogRecordHandler_t", "boost::shared_ptr< RobotRaconteur::FileLogRecordHandler > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_t = {"_p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_t", "boost::shared_ptr< RobotRaconteur::FunctionDefinition > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t = {"_p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t", "boost::shared_ptr< RobotRaconteur::HardwareTransport > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__IntraTransport_t = {"_p_boost__shared_ptrT_RobotRaconteur__IntraTransport_t", "boost::shared_ptr< RobotRaconteur::IntraTransport > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t = {"_p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t", "boost::shared_ptr< RobotRaconteur::LocalTransport > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__LogRecordHandler_t = {"_p_boost__shared_ptrT_RobotRaconteur__LogRecordHandler_t", "boost::shared_ptr< RobotRaconteur::LogRecordHandler > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t = {"_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t", "boost::shared_ptr< RobotRaconteur::MemberDefinition > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__MemoryDefinition_t = {"_p_boost__shared_ptrT_RobotRaconteur__MemoryDefinition_t", "boost::shared_ptr< RobotRaconteur::MemoryDefinition > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t = {"_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t", "boost::shared_ptr< RobotRaconteur::MessageElementData > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t = {"_p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t", "boost::shared_ptr< RobotRaconteur::MessageElementNestedElementList > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__MessageElement_t = {"_p_boost__shared_ptrT_RobotRaconteur__MessageElement_t", "boost::shared_ptr< RobotRaconteur::MessageElement > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t = {"_p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t", "boost::shared_ptr< RobotRaconteur::MessageEntry > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t = {"_p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t", "boost::shared_ptr< RobotRaconteur::MessageHeader > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__Message_t = {"_p_boost__shared_ptrT_RobotRaconteur__Message_t", "boost::shared_ptr< RobotRaconteur::Message > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__MultiDimArrayMemoryBase_t = {"_p_boost__shared_ptrT_RobotRaconteur__MultiDimArrayMemoryBase_t", "boost::shared_ptr< RobotRaconteur::MultiDimArrayMemoryBase > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__NodeDirectoriesFD_t = {"_p_boost__shared_ptrT_RobotRaconteur__NodeDirectoriesFD_t", "boost::shared_ptr< RobotRaconteur::NodeDirectoriesFD > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_t = {"_p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_t", "boost::shared_ptr< RobotRaconteur::ObjRefDefinition > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__PasswordFileUserAuthenticator_t = {"_p_boost__shared_ptrT_RobotRaconteur__PasswordFileUserAuthenticator_t", "boost::shared_ptr< RobotRaconteur::PasswordFileUserAuthenticator > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__PipeDefinition_t = {"_p_boost__shared_ptrT_RobotRaconteur__PipeDefinition_t", "boost::shared_ptr< RobotRaconteur::PipeDefinition > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_t = {"_p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_t", "boost::shared_ptr< RobotRaconteur::PropertyDefinition > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t = {"_p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t", "boost::shared_ptr< RobotRaconteur::RRBaseArray > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__RRMultiDimArrayUntyped_t = {"_p_boost__shared_ptrT_RobotRaconteur__RRMultiDimArrayUntyped_t", "boost::shared_ptr< RobotRaconteur::RRMultiDimArrayUntyped > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__RRObject_t = {"_p_boost__shared_ptrT_RobotRaconteur__RRObject_t", "boost::shared_ptr< RobotRaconteur::RRObject > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__RRValue_t = {"_p_boost__shared_ptrT_RobotRaconteur__RRValue_t", "boost::shared_ptr< RobotRaconteur::RRValue > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__Rate_t = {"_p_boost__shared_ptrT_RobotRaconteur__Rate_t", "boost::shared_ptr< RobotRaconteur::Rate > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNodeSetup_t = {"_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNodeSetup_t", "boost::shared_ptr< RobotRaconteur::RobotRaconteurNodeSetup > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t = {"_p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t", "boost::shared_ptr< RobotRaconteur::RobotRaconteurNode > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__ServerContext_t = {"_p_boost__shared_ptrT_RobotRaconteur__ServerContext_t", "boost::shared_ptr< RobotRaconteur::ServerContext > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__ServerEndpoint_t = {"_p_boost__shared_ptrT_RobotRaconteur__ServerEndpoint_t", "boost::shared_ptr< RobotRaconteur::ServerEndpoint > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t = {"_p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t", "boost::shared_ptr< RobotRaconteur::ServiceDefinition > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t = {"_p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t", "boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__ServiceSecurityPolicy_t = {"_p_boost__shared_ptrT_RobotRaconteur__ServiceSecurityPolicy_t", "boost::shared_ptr< RobotRaconteur::ServiceSecurityPolicy > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t = {"_p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t", "boost::shared_ptr< RobotRaconteur::TcpTransport > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__Timer_t = {"_p_boost__shared_ptrT_RobotRaconteur__Timer_t", "boost::shared_ptr< RobotRaconteur::Timer > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__Transport_t = {"_p_boost__shared_ptrT_RobotRaconteur__Transport_t", "boost::shared_ptr< RobotRaconteur::Transport > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t = {"_p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t", "boost::shared_ptr< RobotRaconteur::TypeDefinition > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__UserAuthenticator_t = {"_p_boost__shared_ptrT_RobotRaconteur__UserAuthenticator_t", "boost::shared_ptr< RobotRaconteur::UserAuthenticator > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__UserLogRecordHandlerBase_t = {"_p_boost__shared_ptrT_RobotRaconteur__UserLogRecordHandlerBase_t", "boost::shared_ptr< RobotRaconteur::UserLogRecordHandlerBase > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t = {"_p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t", "boost::shared_ptr< RobotRaconteur::UsingDefinition > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WallRate_t = {"_p_boost__shared_ptrT_RobotRaconteur__WallRate_t", "boost::shared_ptr< RobotRaconteur::WallRate > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WallTimer_t = {"_p_boost__shared_ptrT_RobotRaconteur__WallTimer_t", "boost::shared_ptr< RobotRaconteur::WallTimer > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WireDefinition_t = {"_p_boost__shared_ptrT_RobotRaconteur__WireDefinition_t", "boost::shared_ptr< RobotRaconteur::WireDefinition > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedDirectorLock_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedDirectorLock_t", "boost::shared_ptr< RobotRaconteur::WrappedDirectorLock > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedGeneratorClient_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedGeneratorClient_t", "boost::shared_ptr< RobotRaconteur::WrappedGeneratorClient > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedNamedArrayMemoryClient_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedNamedArrayMemoryClient_t", "boost::shared_ptr< RobotRaconteur::WrappedNamedArrayMemoryClient > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClient_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClient_t", "boost::shared_ptr< RobotRaconteur::WrappedNamedMultiDimArrayMemoryClient > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeBroadcaster_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeBroadcaster_t", "boost::shared_ptr< RobotRaconteur::WrappedPipeBroadcaster > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeClient_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeClient_t", "boost::shared_ptr< RobotRaconteur::WrappedPipeClient > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t", "boost::shared_ptr< RobotRaconteur::WrappedPipeEndpoint > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeServer_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeServer_t", "boost::shared_ptr< RobotRaconteur::WrappedPipeServer > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t", "boost::shared_ptr< RobotRaconteur::WrappedPipeSubscription > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedPodArrayMemoryClient_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedPodArrayMemoryClient_t", "boost::shared_ptr< RobotRaconteur::WrappedPodArrayMemoryClient > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedPodMultiDimArrayMemoryClient_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedPodMultiDimArrayMemoryClient_t", "boost::shared_ptr< RobotRaconteur::WrappedPodMultiDimArrayMemoryClient > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedRRObject_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedRRObject_t", "boost::shared_ptr< RobotRaconteur::WrappedRRObject > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceInfo2Subscription_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceInfo2Subscription_t", "boost::shared_ptr< RobotRaconteur::WrappedServiceInfo2Subscription > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t", "boost::shared_ptr< RobotRaconteur::WrappedServiceSkelAsyncAdapter > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t", "boost::shared_ptr< RobotRaconteur::WrappedServiceSkel > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t", "boost::shared_ptr< RobotRaconteur::WrappedServiceStub > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t", "boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t", "boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilter > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t", "boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionManager > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t", "boost::shared_ptr< RobotRaconteur::WrappedServiceSubscription > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t", "boost::shared_ptr< RobotRaconteur::WrappedSubObjectSubscription > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedUserAuthenticator_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedUserAuthenticator_t", "boost::shared_ptr< RobotRaconteur::WrappedUserAuthenticator > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedWireBroadcaster_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedWireBroadcaster_t", "boost::shared_ptr< RobotRaconteur::WrappedWireBroadcaster > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedWireClient_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedWireClient_t", "boost::shared_ptr< RobotRaconteur::WrappedWireClient > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t", "boost::shared_ptr< RobotRaconteur::WrappedWireConnection > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedWireServer_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedWireServer_t", "boost::shared_ptr< RobotRaconteur::WrappedWireServer > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t", "boost::shared_ptr< RobotRaconteur::WrappedWireSubscription > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedWireUnicastReceiver_t = {"_p_boost__shared_ptrT_RobotRaconteur__WrappedWireUnicastReceiver_t", "boost::shared_ptr< RobotRaconteur::WrappedWireUnicastReceiver > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__shared_ptrT_RobotRaconteur__detail__MultiDimArray_CalculateCopyIndicesIter_t = {"_p_boost__shared_ptrT_RobotRaconteur__detail__MultiDimArray_CalculateCopyIndicesIter_t", "boost::shared_ptr< RobotRaconteur::detail::MultiDimArray_CalculateCopyIndicesIter > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_boost__string_ref = {"_p_boost__string_ref", "boost::string_ref *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_char = {"_p_char", "char *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_difference_type = {"_p_difference_type", "difference_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_int = {"_p_int", "int *|int_least32_t *|int32_t *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_key_type = {"_p_key_type", "key_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_long = {"_p_long", "intptr_t *|int_least64_t *|int_fast32_t *|int_fast64_t *|int64_t *|long *|int_fast16_t *|intmax_t *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_mapped_type = {"_p_mapped_type", "mapped_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p_PyObject = {"_p_p_PyObject", "PyObject **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_short = {"_p_short", "short *|int_least16_t *|int16_t *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_signed_char = {"_p_signed_char", "signed char *|int_least8_t *|int_fast8_t *|int8_t *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_size_type = {"_p_size_type", "size_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t = {"_p_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t", "std::vector< RobotRaconteur::ConstantDefinition_StructField >::allocator_type *|std::allocator< RobotRaconteur::ConstantDefinition_StructField > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t = {"_p_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t", "std::allocator< RobotRaconteur::EnumDefinitionValue > *|std::vector< RobotRaconteur::EnumDefinitionValue >::allocator_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_RobotRaconteur__NodeInfo2_t = {"_p_std__allocatorT_RobotRaconteur__NodeInfo2_t", "std::vector< RobotRaconteur::NodeInfo2 >::allocator_type *|std::allocator< RobotRaconteur::NodeInfo2 > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t = {"_p_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t", "std::allocator< RobotRaconteur::ServiceInfo2Wrapped > *|std::vector< RobotRaconteur::ServiceInfo2Wrapped >::allocator_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_RobotRaconteur__ServicePathSegment_t = {"_p_std__allocatorT_RobotRaconteur__ServicePathSegment_t", "std::allocator< RobotRaconteur::ServicePathSegment > *|std::vector< RobotRaconteur::ServicePathSegment >::allocator_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t = {"_p_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t", "std::allocator< RobotRaconteur::ServiceSubscriptionClientID > *|std::vector< RobotRaconteur::ServiceSubscriptionClientID >::allocator_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t = {"_p_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t", "std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > *|std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute >::allocator_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t = {"_p_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t", "std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *|std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails >::allocator_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t = {"_p_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t", "std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *|std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > >::allocator_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t = {"_p_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t", "std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *|std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > >::allocator_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t = {"_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t", "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > >::allocator_type *|std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t = {"_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t", "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > >::allocator_type *|std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t = {"_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t", "std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *|std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > >::allocator_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t = {"_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t", "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > >::allocator_type *|std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t = {"_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t", "std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *|std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > >::allocator_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t = {"_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t", "std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *|std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > >::allocator_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t = {"_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t", "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > >::allocator_type *|std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t = {"_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t", "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > >::allocator_type *|std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t = {"_p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t", "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > >::allocator_type *|std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_int_t = {"_p_std__allocatorT_int_t", "std::vector< int >::allocator_type *|std::allocator< int > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_signed_char_t = {"_p_std__allocatorT_signed_char_t", "std::vector< signed char >::allocator_type *|std::allocator< signed char > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t = {"_p_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t", "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped >::allocator_type *|std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t = {"_p_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t", "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > >::allocator_type *|std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t = {"_p_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t", "std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > *|std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup >::allocator_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t = {"_p_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t", "std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > *|std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute >::allocator_type *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_std__pairT_std__string_const_std__string_t_t = {"_p_std__allocatorT_std__pairT_std__string_const_std__string_t_t", "std::map< std::string,std::string >::allocator_type *|std::allocator< std::pair< std::string const,std::string > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_std__string_t = {"_p_std__allocatorT_std__string_t", "std::vector< std::string >::allocator_type *|std::allocator< std::string > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_unsigned_char_t = {"_p_std__allocatorT_unsigned_char_t", "std::vector< unsigned char >::allocator_type *|std::allocator< unsigned char > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_unsigned_int_t = {"_p_std__allocatorT_unsigned_int_t", "std::vector< unsigned int >::allocator_type *|std::allocator< unsigned int > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__allocatorT_unsigned_long_t = {"_p_std__allocatorT_unsigned_long_t", "std::vector< unsigned long >::allocator_type *|std::allocator< unsigned long > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__invalid_argument = {"_p_std__invalid_argument", "std::invalid_argument *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t = {"_p_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t", "std::less< RobotRaconteur::ServiceSubscriptionClientID > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__lessT_std__string_t = {"_p_std__lessT_std__string_t", "std::less< std::string > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t = {"_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t", "std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped > *|std::map< RobotRaconteur::ServiceSubscriptionClientID,RobotRaconteur::ServiceInfo2Wrapped,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,RobotRaconteur::ServiceInfo2Wrapped > > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t = {"_p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t", "std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > *|std::map< RobotRaconteur::ServiceSubscriptionClientID,boost::shared_ptr< RobotRaconteur::WrappedServiceStub >,std::less< RobotRaconteur::ServiceSubscriptionClientID >,std::allocator< std::pair< RobotRaconteur::ServiceSubscriptionClientID const,boost::shared_ptr< RobotRaconteur::WrappedServiceStub > > > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t = {"_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t", "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > *|std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t = {"_p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t", "std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute > *|std::map< std::string,RobotRaconteur::ServiceSubscriptionFilterAttribute,std::less< std::string >,std::allocator< std::pair< std::string const,RobotRaconteur::ServiceSubscriptionFilterAttribute > > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t = {"_p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t", "std::map< std::string,std::string > *|std::map< std::string,std::string,std::less< std::string >,std::allocator< std::pair< std::string const,std::string > > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__runtime_error = {"_p_std__runtime_error", "std::runtime_error *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t = {"_p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t", "std::vector< RobotRaconteur::ConstantDefinition_StructField > *|std::vector< RobotRaconteur::ConstantDefinition_StructField,std::allocator< RobotRaconteur::ConstantDefinition_StructField > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t = {"_p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t", "std::vector< RobotRaconteur::EnumDefinitionValue,std::allocator< RobotRaconteur::EnumDefinitionValue > > *|std::vector< RobotRaconteur::EnumDefinitionValue > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t = {"_p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t", "std::vector< RobotRaconteur::NodeInfo2 > *|std::vector< RobotRaconteur::NodeInfo2,std::allocator< RobotRaconteur::NodeInfo2 > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t = {"_p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t", "std::vector< RobotRaconteur::ServiceInfo2Wrapped,std::allocator< RobotRaconteur::ServiceInfo2Wrapped > > *|std::vector< RobotRaconteur::ServiceInfo2Wrapped > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t = {"_p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t", "std::vector< RobotRaconteur::ServicePathSegment > *|std::vector< RobotRaconteur::ServicePathSegment,std::allocator< RobotRaconteur::ServicePathSegment > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t = {"_p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t", "std::vector< RobotRaconteur::ServiceSubscriptionClientID > *|std::vector< RobotRaconteur::ServiceSubscriptionClientID,std::allocator< RobotRaconteur::ServiceSubscriptionClientID > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t = {"_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t", "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttributeGroup > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t = {"_p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t", "std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute > *|std::vector< RobotRaconteur::ServiceSubscriptionFilterAttribute,std::allocator< RobotRaconteur::ServiceSubscriptionFilterAttribute > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t = {"_p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t", "std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > *|std::vector< RobotRaconteur::WrappedServiceSubscriptionManagerDetails,std::allocator< RobotRaconteur::WrappedServiceSubscriptionManagerDetails > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t = {"_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t", "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement > > *|std::vector< boost::intrusive_ptr< RobotRaconteur::MessageElement >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageElement > > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t = {"_p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t", "std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > *|std::vector< boost::intrusive_ptr< RobotRaconteur::MessageEntry >,std::allocator< boost::intrusive_ptr< RobotRaconteur::MessageEntry > > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t = {"_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t", "std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > *|std::vector< boost::shared_ptr< RobotRaconteur::ConstantDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ConstantDefinition > > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t = {"_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t", "std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition > > *|std::vector< boost::shared_ptr< RobotRaconteur::EnumDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::EnumDefinition > > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t = {"_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t", "std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > *|std::vector< boost::shared_ptr< RobotRaconteur::ExceptionDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ExceptionDefinition > > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t = {"_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t", "std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition > > *|std::vector< boost::shared_ptr< RobotRaconteur::MemberDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::MemberDefinition > > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t = {"_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t", "std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > *|std::vector< boost::shared_ptr< RobotRaconteur::ServiceDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceDefinition > > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t = {"_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t", "std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > *|std::vector< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::ServiceEntryDefinition > > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t = {"_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t", "std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition > > *|std::vector< boost::shared_ptr< RobotRaconteur::TypeDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::TypeDefinition > > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t = {"_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t", "std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition > > *|std::vector< boost::shared_ptr< RobotRaconteur::UsingDefinition >,std::allocator< boost::shared_ptr< RobotRaconteur::UsingDefinition > > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t = {"_p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t", "std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > *|std::vector< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode >,std::allocator< boost::shared_ptr< RobotRaconteur::WrappedServiceSubscriptionFilterNode > > > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_int_std__allocatorT_int_t_t = {"_p_std__vectorT_int_std__allocatorT_int_t_t", "std::vector< int,std::allocator< int > > *|std::vector< int32_t,std::allocator< int32_t > > *|std::vector< int32_t > *|std::vector< int > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t = {"_p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t", "std::vector< signed char > *|std::vector< signed char,std::allocator< signed char > > *|std::vector< int8_t > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_std__string_std__allocatorT_std__string_t_t = {"_p_std__vectorT_std__string_std__allocatorT_std__string_t_t", "std::vector< std::string,std::allocator< std::string > > *|std::vector< std::string > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t = {"_p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t", "std::vector< uint8_t,std::allocator< uint8_t > > *|std::vector< unsigned char > *|std::vector< unsigned char,std::allocator< unsigned char > > *|std::vector< uint8_t > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t = {"_p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t", "std::vector< unsigned int,std::allocator< unsigned int > > *|std::vector< uint32_t > *|std::vector< unsigned int > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t = {"_p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t", "std::vector< unsigned long > *|std::vector< unsigned long,std::allocator< unsigned long > > *|std::vector< uint64_t,std::allocator< uint64_t > > *|std::vector< uint64_t > *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_swig__SwigPyIterator = {"_p_swig__SwigPyIterator", "swig::SwigPyIterator *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_unsigned_char = {"_p_unsigned_char", "unsigned char *|uint_least8_t *|uint_fast8_t *|uint8_t *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_unsigned_int = {"_p_unsigned_int", "uint_least32_t *|uint32_t *|unsigned int *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_unsigned_long = {"_p_unsigned_long", "uintptr_t *|uint_least64_t *|uint_fast32_t *|uint_fast64_t *|uint64_t *|unsigned long *|uint_fast16_t *|uintmax_t *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_unsigned_short = {"_p_unsigned_short", "unsigned short *|uint_least16_t *|uint16_t *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_value_type = {"_p_value_type", "value_type *", 0, 0, (void*)0, 0};

static swig_type_info *swig_type_initial[] = {
  &_swigt__p_ClientServiceListenerDirector,
  &_swigt__p_MemberDefinitionUtil,
  &_swigt__p_MessageElementDataUtil,
  &_swigt__p_RobotRaconteur__ArrayMemoryBase,
  &_swigt__p_RobotRaconteur__AsyncGeneratorClientReturnDirector,
  &_swigt__p_RobotRaconteur__AsyncNodeInfo2VectorReturnDirector,
  &_swigt__p_RobotRaconteur__AsyncPipeEndpointReturnDirector,
  &_swigt__p_RobotRaconteur__AsyncRequestDirector,
  &_swigt__p_RobotRaconteur__AsyncServiceInfo2VectorReturnDirector,
  &_swigt__p_RobotRaconteur__AsyncStringReturnDirector,
  &_swigt__p_RobotRaconteur__AsyncStubReturnDirector,
  &_swigt__p_RobotRaconteur__AsyncTimerEventReturnDirector,
  &_swigt__p_RobotRaconteur__AsyncUInt32ReturnDirector,
  &_swigt__p_RobotRaconteur__AsyncVoidNoErrReturnDirector,
  &_swigt__p_RobotRaconteur__AsyncVoidReturnDirector,
  &_swigt__p_RobotRaconteur__AsyncWireConnectionReturnDirector,
  &_swigt__p_RobotRaconteur__AsyncWirePeekReturnDirector,
  &_swigt__p_RobotRaconteur__AuthenticatedUser,
  &_swigt__p_RobotRaconteur__AutoResetEvent,
  &_swigt__p_RobotRaconteur__BroadcastDownsampler,
  &_swigt__p_RobotRaconteur__CallbackDefinition,
  &_swigt__p_RobotRaconteur__CommandLineConfigParser,
  &_swigt__p_RobotRaconteur__ConstantDefinition,
  &_swigt__p_RobotRaconteur__ConstantDefinition_StructField,
  &_swigt__p_RobotRaconteur__EnumDefinition,
  &_swigt__p_RobotRaconteur__EnumDefinitionValue,
  &_swigt__p_RobotRaconteur__EventDefinition,
  &_swigt__p_RobotRaconteur__ExceptionDefinition,
  &_swigt__p_RobotRaconteur__FileLogRecordHandler,
  &_swigt__p_RobotRaconteur__FunctionDefinition,
  &_swigt__p_RobotRaconteur__GetUuidForNameAndLockResult,
  &_swigt__p_RobotRaconteur__HandlerErrorInfo,
  &_swigt__p_RobotRaconteur__HardwareTransport,
  &_swigt__p_RobotRaconteur__IntraTransport,
  &_swigt__p_RobotRaconteur__LocalTransport,
  &_swigt__p_RobotRaconteur__LogRecordHandler,
  &_swigt__p_RobotRaconteur__MemberDefinition,
  &_swigt__p_RobotRaconteur__MemoryDefinition,
  &_swigt__p_RobotRaconteur__Message,
  &_swigt__p_RobotRaconteur__MessageElement,
  &_swigt__p_RobotRaconteur__MessageElementData,
  &_swigt__p_RobotRaconteur__MessageElementNestedElementList,
  &_swigt__p_RobotRaconteur__MessageEntry,
  &_swigt__p_RobotRaconteur__MessageHeader,
  &_swigt__p_RobotRaconteur__MultiDimArrayMemoryBase,
  &_swigt__p_RobotRaconteur__NodeDirectories,
  &_swigt__p_RobotRaconteur__NodeDirectoriesFD,
  &_swigt__p_RobotRaconteur__NodeDirectoriesResourceAlreadyInUse,
  &_swigt__p_RobotRaconteur__NodeDirectoriesUtil,
  &_swigt__p_RobotRaconteur__NodeID,
  &_swigt__p_RobotRaconteur__NodeInfo2,
  &_swigt__p_RobotRaconteur__ObjRefDefinition,
  &_swigt__p_RobotRaconteur__PasswordFileUserAuthenticator,
  &_swigt__p_RobotRaconteur__PipeDefinition,
  &_swigt__p_RobotRaconteur__PropertyDefinition,
  &_swigt__p_RobotRaconteur__RRBaseArray,
  &_swigt__p_RobotRaconteur__RRLogRecord,
  &_swigt__p_RobotRaconteur__RRMultiDimArrayUntyped,
  &_swigt__p_RobotRaconteur__RRNativeDirectorSupport,
  &_swigt__p_RobotRaconteur__RRObject,
  &_swigt__p_RobotRaconteur__RRValue,
  &_swigt__p_RobotRaconteur__Rate,
  &_swigt__p_RobotRaconteur__RobotRaconteurNode,
  &_swigt__p_RobotRaconteur__RobotRaconteurNodeSetup,
  &_swigt__p_RobotRaconteur__RobotRaconteurVersion,
  &_swigt__p_RobotRaconteur__ServerContext,
  &_swigt__p_RobotRaconteur__ServerEndpoint,
  &_swigt__p_RobotRaconteur__ServiceDefinition,
  &_swigt__p_RobotRaconteur__ServiceDefinitionParseInfo,
  &_swigt__p_RobotRaconteur__ServiceEntryDefinition,
  &_swigt__p_RobotRaconteur__ServiceInfo2Wrapped,
  &_swigt__p_RobotRaconteur__ServicePathSegment,
  &_swigt__p_RobotRaconteur__ServiceSecurityPolicy,
  &_swigt__p_RobotRaconteur__ServiceSubscriptionClientID,
  &_swigt__p_RobotRaconteur__ServiceSubscriptionFilterAttribute,
  &_swigt__p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup,
  &_swigt__p_RobotRaconteur__TcpTransport,
  &_swigt__p_RobotRaconteur__TimeSpec,
  &_swigt__p_RobotRaconteur__Timer,
  &_swigt__p_RobotRaconteur__TimerEvent,
  &_swigt__p_RobotRaconteur__Transport,
  &_swigt__p_RobotRaconteur__TryGetValueResult,
  &_swigt__p_RobotRaconteur__TypeDefinition,
  &_swigt__p_RobotRaconteur__UserAuthenticator,
  &_swigt__p_RobotRaconteur__UserLogRecordHandlerBase,
  &_swigt__p_RobotRaconteur__UserLogRecordHandlerDirector,
  &_swigt__p_RobotRaconteur__UsingDefinition,
  &_swigt__p_RobotRaconteur__WallRate,
  &_swigt__p_RobotRaconteur__WallTimer,
  &_swigt__p_RobotRaconteur__WireDefinition,
  &_swigt__p_RobotRaconteur__WrappedArrayMemoryClientUtil,
  &_swigt__p_RobotRaconteur__WrappedArrayMemoryDirector,
  &_swigt__p_RobotRaconteur__WrappedDirectorLock,
  &_swigt__p_RobotRaconteur__WrappedGeneratorClient,
  &_swigt__p_RobotRaconteur__WrappedGeneratorClient_TryGetNextResult,
  &_swigt__p_RobotRaconteur__WrappedGeneratorServerDirector,
  &_swigt__p_RobotRaconteur__WrappedMultiDimArrayMemoryClientUtil,
  &_swigt__p_RobotRaconteur__WrappedMultiDimArrayMemoryDirector,
  &_swigt__p_RobotRaconteur__WrappedMultiDimArrayMemoryParams,
  &_swigt__p_RobotRaconteur__WrappedNamedArrayMemoryClient,
  &_swigt__p_RobotRaconteur__WrappedNamedArrayMemoryClientBuffer,
  &_swigt__p_RobotRaconteur__WrappedNamedArrayMemoryDirector,
  &_swigt__p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClient,
  &_swigt__p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClientBuffer,
  &_swigt__p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryDirector,
  &_swigt__p_RobotRaconteur__WrappedPipeBroadcaster,
  &_swigt__p_RobotRaconteur__WrappedPipeBroadcasterPredicateDirector,
  &_swigt__p_RobotRaconteur__WrappedPipeClient,
  &_swigt__p_RobotRaconteur__WrappedPipeEndpoint,
  &_swigt__p_RobotRaconteur__WrappedPipeEndpointDirector,
  &_swigt__p_RobotRaconteur__WrappedPipeServer,
  &_swigt__p_RobotRaconteur__WrappedPipeServerConnectDirector,
  &_swigt__p_RobotRaconteur__WrappedPipeSubscription,
  &_swigt__p_RobotRaconteur__WrappedPipeSubscriptionDirector,
  &_swigt__p_RobotRaconteur__WrappedPipeSubscription_send_iterator,
  &_swigt__p_RobotRaconteur__WrappedPodArrayMemoryClient,
  &_swigt__p_RobotRaconteur__WrappedPodArrayMemoryClientBuffer,
  &_swigt__p_RobotRaconteur__WrappedPodArrayMemoryDirector,
  &_swigt__p_RobotRaconteur__WrappedPodMultiDimArrayMemoryClient,
  &_swigt__p_RobotRaconteur__WrappedPodMultiDimArrayMemoryClientBuffer,
  &_swigt__p_RobotRaconteur__WrappedPodMultiDimArrayMemoryDirector,
  &_swigt__p_RobotRaconteur__WrappedRRObject,
  &_swigt__p_RobotRaconteur__WrappedServiceInfo2Subscription,
  &_swigt__p_RobotRaconteur__WrappedServiceInfo2SubscriptionDirector,
  &_swigt__p_RobotRaconteur__WrappedServiceSkel,
  &_swigt__p_RobotRaconteur__WrappedServiceSkelAsyncAdapter,
  &_swigt__p_RobotRaconteur__WrappedServiceSkelDirector,
  &_swigt__p_RobotRaconteur__WrappedServiceStub,
  &_swigt__p_RobotRaconteur__WrappedServiceStubDirector,
  &_swigt__p_RobotRaconteur__WrappedServiceSubscription,
  &_swigt__p_RobotRaconteur__WrappedServiceSubscriptionDirector,
  &_swigt__p_RobotRaconteur__WrappedServiceSubscriptionFilter,
  &_swigt__p_RobotRaconteur__WrappedServiceSubscriptionFilterNode,
  &_swigt__p_RobotRaconteur__WrappedServiceSubscriptionFilterPredicateDirector,
  &_swigt__p_RobotRaconteur__WrappedServiceSubscriptionManager,
  &_swigt__p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails,
  &_swigt__p_RobotRaconteur__WrappedServiceSubscription_TryDefaultClientRes,
  &_swigt__p_RobotRaconteur__WrappedService_typed_packet,
  &_swigt__p_RobotRaconteur__WrappedSubObjectSubscription,
  &_swigt__p_RobotRaconteur__WrappedTryReceivePacketWaitResult,
  &_swigt__p_RobotRaconteur__WrappedUserAuthenticator,
  &_swigt__p_RobotRaconteur__WrappedUserAuthenticatorDirector,
  &_swigt__p_RobotRaconteur__WrappedWireBroadcaster,
  &_swigt__p_RobotRaconteur__WrappedWireBroadcasterPredicateDirector,
  &_swigt__p_RobotRaconteur__WrappedWireClient,
  &_swigt__p_RobotRaconteur__WrappedWireConnection,
  &_swigt__p_RobotRaconteur__WrappedWireConnectionDirector,
  &_swigt__p_RobotRaconteur__WrappedWireServer,
  &_swigt__p_RobotRaconteur__WrappedWireServerConnectDirector,
  &_swigt__p_RobotRaconteur__WrappedWireServerPeekValueDirector,
  &_swigt__p_RobotRaconteur__WrappedWireServerPokeValueDirector,
  &_swigt__p_RobotRaconteur__WrappedWireSubscription,
  &_swigt__p_RobotRaconteur__WrappedWireSubscriptionDirector,
  &_swigt__p_RobotRaconteur__WrappedWireSubscription_send_iterator,
  &_swigt__p_RobotRaconteur__WrappedWireUnicastReceiver,
  &_swigt__p_RobotRaconteur__detail__MultiDimArray_CalculateCopyIndicesIter,
  &_swigt__p_ServerServiceListenerDirector,
  &_swigt__p_allocator_type,
  &_swigt__p_boost__arrayT_unsigned_char_16_t,
  &_swigt__p_boost__filesystem__path,
  &_swigt__p_boost__optionalT_std__string_t,
  &_swigt__p_boost__posix_time__ptime,
  &_swigt__p_boost__posix_time__time_duration,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__ArrayMemoryBase_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__AuthenticatedUser_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__AutoResetEvent_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__BroadcastDownsampler_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__CallbackDefinition_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__EventDefinition_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__FileLogRecordHandler_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__IntraTransport_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__LogRecordHandler_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__MemoryDefinition_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__Message_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__MultiDimArrayMemoryBase_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__NodeDirectoriesFD_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__PasswordFileUserAuthenticator_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__PipeDefinition_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__RRMultiDimArrayUntyped_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__RRObject_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__RRValue_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__Rate_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNodeSetup_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__ServerContext_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__ServerEndpoint_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__ServiceSecurityPolicy_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__Timer_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__Transport_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__UserAuthenticator_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__UserLogRecordHandlerBase_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WallRate_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WallTimer_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WireDefinition_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedDirectorLock_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedGeneratorClient_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedNamedArrayMemoryClient_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClient_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeBroadcaster_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeClient_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeServer_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedPodArrayMemoryClient_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedPodMultiDimArrayMemoryClient_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedRRObject_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceInfo2Subscription_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedUserAuthenticator_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedWireBroadcaster_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedWireClient_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedWireServer_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedWireUnicastReceiver_t,
  &_swigt__p_boost__shared_ptrT_RobotRaconteur__detail__MultiDimArray_CalculateCopyIndicesIter_t,
  &_swigt__p_boost__string_ref,
  &_swigt__p_char,
  &_swigt__p_difference_type,
  &_swigt__p_int,
  &_swigt__p_key_type,
  &_swigt__p_long,
  &_swigt__p_mapped_type,
  &_swigt__p_p_PyObject,
  &_swigt__p_short,
  &_swigt__p_signed_char,
  &_swigt__p_size_type,
  &_swigt__p_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t,
  &_swigt__p_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t,
  &_swigt__p_std__allocatorT_RobotRaconteur__NodeInfo2_t,
  &_swigt__p_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t,
  &_swigt__p_std__allocatorT_RobotRaconteur__ServicePathSegment_t,
  &_swigt__p_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t,
  &_swigt__p_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t,
  &_swigt__p_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t,
  &_swigt__p_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t,
  &_swigt__p_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t,
  &_swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t,
  &_swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t,
  &_swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t,
  &_swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t,
  &_swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t,
  &_swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t,
  &_swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t,
  &_swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t,
  &_swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t,
  &_swigt__p_std__allocatorT_int_t,
  &_swigt__p_std__allocatorT_signed_char_t,
  &_swigt__p_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t,
  &_swigt__p_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t,
  &_swigt__p_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t,
  &_swigt__p_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t,
  &_swigt__p_std__allocatorT_std__pairT_std__string_const_std__string_t_t,
  &_swigt__p_std__allocatorT_std__string_t,
  &_swigt__p_std__allocatorT_unsigned_char_t,
  &_swigt__p_std__allocatorT_unsigned_int_t,
  &_swigt__p_std__allocatorT_unsigned_long_t,
  &_swigt__p_std__invalid_argument,
  &_swigt__p_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t,
  &_swigt__p_std__lessT_std__string_t,
  &_swigt__p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t,
  &_swigt__p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t,
  &_swigt__p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t,
  &_swigt__p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t,
  &_swigt__p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t,
  &_swigt__p_std__runtime_error,
  &_swigt__p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t,
  &_swigt__p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t,
  &_swigt__p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t,
  &_swigt__p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t,
  &_swigt__p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t,
  &_swigt__p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t,
  &_swigt__p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t,
  &_swigt__p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t,
  &_swigt__p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t,
  &_swigt__p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t,
  &_swigt__p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t,
  &_swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t,
  &_swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t,
  &_swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t,
  &_swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t,
  &_swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t,
  &_swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t,
  &_swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t,
  &_swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t,
  &_swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t,
  &_swigt__p_std__vectorT_int_std__allocatorT_int_t_t,
  &_swigt__p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t,
  &_swigt__p_std__vectorT_std__string_std__allocatorT_std__string_t_t,
  &_swigt__p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t,
  &_swigt__p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t,
  &_swigt__p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t,
  &_swigt__p_swig__SwigPyIterator,
  &_swigt__p_unsigned_char,
  &_swigt__p_unsigned_int,
  &_swigt__p_unsigned_long,
  &_swigt__p_unsigned_short,
  &_swigt__p_value_type,
};

static swig_cast_info _swigc__p_ClientServiceListenerDirector[] = {  {&_swigt__p_ClientServiceListenerDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_MemberDefinitionUtil[] = {  {&_swigt__p_MemberDefinitionUtil, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_MessageElementDataUtil[] = {  {&_swigt__p_MessageElementDataUtil, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__ArrayMemoryBase[] = {  {&_swigt__p_RobotRaconteur__ArrayMemoryBase, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__AsyncGeneratorClientReturnDirector[] = {  {&_swigt__p_RobotRaconteur__AsyncGeneratorClientReturnDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__AsyncNodeInfo2VectorReturnDirector[] = {  {&_swigt__p_RobotRaconteur__AsyncNodeInfo2VectorReturnDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__AsyncPipeEndpointReturnDirector[] = {  {&_swigt__p_RobotRaconteur__AsyncPipeEndpointReturnDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__AsyncRequestDirector[] = {  {&_swigt__p_RobotRaconteur__AsyncRequestDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__AsyncServiceInfo2VectorReturnDirector[] = {  {&_swigt__p_RobotRaconteur__AsyncServiceInfo2VectorReturnDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__AsyncStringReturnDirector[] = {  {&_swigt__p_RobotRaconteur__AsyncStringReturnDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__AsyncStubReturnDirector[] = {  {&_swigt__p_RobotRaconteur__AsyncStubReturnDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__AsyncTimerEventReturnDirector[] = {  {&_swigt__p_RobotRaconteur__AsyncTimerEventReturnDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__AsyncUInt32ReturnDirector[] = {  {&_swigt__p_RobotRaconteur__AsyncUInt32ReturnDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__AsyncVoidNoErrReturnDirector[] = {  {&_swigt__p_RobotRaconteur__AsyncVoidNoErrReturnDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__AsyncVoidReturnDirector[] = {  {&_swigt__p_RobotRaconteur__AsyncVoidReturnDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__AsyncWireConnectionReturnDirector[] = {  {&_swigt__p_RobotRaconteur__AsyncWireConnectionReturnDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__AsyncWirePeekReturnDirector[] = {  {&_swigt__p_RobotRaconteur__AsyncWirePeekReturnDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__AuthenticatedUser[] = {  {&_swigt__p_RobotRaconteur__AuthenticatedUser, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__AutoResetEvent[] = {  {&_swigt__p_RobotRaconteur__AutoResetEvent, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__BroadcastDownsampler[] = {  {&_swigt__p_RobotRaconteur__BroadcastDownsampler, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__CallbackDefinition[] = {  {&_swigt__p_RobotRaconteur__CallbackDefinition, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__CommandLineConfigParser[] = {  {&_swigt__p_RobotRaconteur__CommandLineConfigParser, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__ConstantDefinition[] = {  {&_swigt__p_RobotRaconteur__ConstantDefinition, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__ConstantDefinition_StructField[] = {  {&_swigt__p_RobotRaconteur__ConstantDefinition_StructField, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__EnumDefinition[] = {  {&_swigt__p_RobotRaconteur__EnumDefinition, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__EnumDefinitionValue[] = {  {&_swigt__p_RobotRaconteur__EnumDefinitionValue, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__EventDefinition[] = {  {&_swigt__p_RobotRaconteur__EventDefinition, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__ExceptionDefinition[] = {  {&_swigt__p_RobotRaconteur__ExceptionDefinition, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__FileLogRecordHandler[] = {  {&_swigt__p_RobotRaconteur__FileLogRecordHandler, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__FunctionDefinition[] = {  {&_swigt__p_RobotRaconteur__FunctionDefinition, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__GetUuidForNameAndLockResult[] = {  {&_swigt__p_RobotRaconteur__GetUuidForNameAndLockResult, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__HandlerErrorInfo[] = {  {&_swigt__p_RobotRaconteur__HandlerErrorInfo, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__HardwareTransport[] = {  {&_swigt__p_RobotRaconteur__HardwareTransport, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__IntraTransport[] = {  {&_swigt__p_RobotRaconteur__IntraTransport, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__LocalTransport[] = {  {&_swigt__p_RobotRaconteur__LocalTransport, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__LogRecordHandler[] = {  {&_swigt__p_RobotRaconteur__UserLogRecordHandlerBase, _p_RobotRaconteur__UserLogRecordHandlerBaseTo_p_RobotRaconteur__LogRecordHandler, 0, 0},  {&_swigt__p_RobotRaconteur__LogRecordHandler, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__MemberDefinition[] = {  {&_swigt__p_RobotRaconteur__MemberDefinition, 0, 0, 0},  {&_swigt__p_RobotRaconteur__PropertyDefinition, _p_RobotRaconteur__PropertyDefinitionTo_p_RobotRaconteur__MemberDefinition, 0, 0},  {&_swigt__p_RobotRaconteur__FunctionDefinition, _p_RobotRaconteur__FunctionDefinitionTo_p_RobotRaconteur__MemberDefinition, 0, 0},  {&_swigt__p_RobotRaconteur__EventDefinition, _p_RobotRaconteur__EventDefinitionTo_p_RobotRaconteur__MemberDefinition, 0, 0},  {&_swigt__p_RobotRaconteur__ObjRefDefinition, _p_RobotRaconteur__ObjRefDefinitionTo_p_RobotRaconteur__MemberDefinition, 0, 0},  {&_swigt__p_RobotRaconteur__PipeDefinition, _p_RobotRaconteur__PipeDefinitionTo_p_RobotRaconteur__MemberDefinition, 0, 0},  {&_swigt__p_RobotRaconteur__CallbackDefinition, _p_RobotRaconteur__CallbackDefinitionTo_p_RobotRaconteur__MemberDefinition, 0, 0},  {&_swigt__p_RobotRaconteur__WireDefinition, _p_RobotRaconteur__WireDefinitionTo_p_RobotRaconteur__MemberDefinition, 0, 0},  {&_swigt__p_RobotRaconteur__MemoryDefinition, _p_RobotRaconteur__MemoryDefinitionTo_p_RobotRaconteur__MemberDefinition, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__MemoryDefinition[] = {  {&_swigt__p_RobotRaconteur__MemoryDefinition, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__Message[] = {  {&_swigt__p_RobotRaconteur__Message, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__MessageElement[] = {  {&_swigt__p_RobotRaconteur__MessageElement, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__MessageElementData[] = {  {&_swigt__p_RobotRaconteur__RRBaseArray, _p_RobotRaconteur__RRBaseArrayTo_p_RobotRaconteur__MessageElementData, 0, 0},  {&_swigt__p_RobotRaconteur__MessageElementNestedElementList, _p_RobotRaconteur__MessageElementNestedElementListTo_p_RobotRaconteur__MessageElementData, 0, 0},  {&_swigt__p_RobotRaconteur__MessageElementData, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__MessageElementNestedElementList[] = {  {&_swigt__p_RobotRaconteur__MessageElementNestedElementList, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__MessageEntry[] = {  {&_swigt__p_RobotRaconteur__MessageEntry, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__MessageHeader[] = {  {&_swigt__p_RobotRaconteur__MessageHeader, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__MultiDimArrayMemoryBase[] = {  {&_swigt__p_RobotRaconteur__MultiDimArrayMemoryBase, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__NodeDirectories[] = {  {&_swigt__p_RobotRaconteur__NodeDirectories, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__NodeDirectoriesFD[] = {  {&_swigt__p_RobotRaconteur__NodeDirectoriesFD, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__NodeDirectoriesResourceAlreadyInUse[] = {  {&_swigt__p_RobotRaconteur__NodeDirectoriesResourceAlreadyInUse, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__NodeDirectoriesUtil[] = {  {&_swigt__p_RobotRaconteur__NodeDirectoriesUtil, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__NodeID[] = {  {&_swigt__p_RobotRaconteur__NodeID, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__NodeInfo2[] = {  {&_swigt__p_RobotRaconteur__NodeInfo2, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__ObjRefDefinition[] = {  {&_swigt__p_RobotRaconteur__ObjRefDefinition, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__PasswordFileUserAuthenticator[] = {  {&_swigt__p_RobotRaconteur__PasswordFileUserAuthenticator, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__PipeDefinition[] = {  {&_swigt__p_RobotRaconteur__PipeDefinition, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__PropertyDefinition[] = {  {&_swigt__p_RobotRaconteur__PropertyDefinition, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__RRBaseArray[] = {  {&_swigt__p_RobotRaconteur__RRBaseArray, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__RRLogRecord[] = {  {&_swigt__p_RobotRaconteur__RRLogRecord, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__RRMultiDimArrayUntyped[] = {  {&_swigt__p_RobotRaconteur__RRMultiDimArrayUntyped, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__RRNativeDirectorSupport[] = {  {&_swigt__p_RobotRaconteur__RRNativeDirectorSupport, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__RRObject[] = {  {&_swigt__p_RobotRaconteur__RRObject, 0, 0, 0},  {&_swigt__p_RobotRaconteur__WrappedServiceStub, _p_RobotRaconteur__WrappedServiceStubTo_p_RobotRaconteur__RRObject, 0, 0},  {&_swigt__p_RobotRaconteur__WrappedRRObject, _p_RobotRaconteur__WrappedRRObjectTo_p_RobotRaconteur__RRObject, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__RRValue[] = {  {&_swigt__p_RobotRaconteur__MessageElement, _p_RobotRaconteur__MessageElementTo_p_RobotRaconteur__RRValue, 0, 0},  {&_swigt__p_RobotRaconteur__RRBaseArray, _p_RobotRaconteur__RRBaseArrayTo_p_RobotRaconteur__RRValue, 0, 0},  {&_swigt__p_RobotRaconteur__MessageEntry, _p_RobotRaconteur__MessageEntryTo_p_RobotRaconteur__RRValue, 0, 0},  {&_swigt__p_RobotRaconteur__RRValue, 0, 0, 0},  {&_swigt__p_RobotRaconteur__MessageElementNestedElementList, _p_RobotRaconteur__MessageElementNestedElementListTo_p_RobotRaconteur__RRValue, 0, 0},  {&_swigt__p_RobotRaconteur__Message, _p_RobotRaconteur__MessageTo_p_RobotRaconteur__RRValue, 0, 0},  {&_swigt__p_RobotRaconteur__MessageHeader, _p_RobotRaconteur__MessageHeaderTo_p_RobotRaconteur__RRValue, 0, 0},  {&_swigt__p_RobotRaconteur__MessageElementData, _p_RobotRaconteur__MessageElementDataTo_p_RobotRaconteur__RRValue, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__Rate[] = {  {&_swigt__p_RobotRaconteur__Rate, 0, 0, 0},  {&_swigt__p_RobotRaconteur__WallRate, _p_RobotRaconteur__WallRateTo_p_RobotRaconteur__Rate, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__RobotRaconteurNode[] = {  {&_swigt__p_RobotRaconteur__RobotRaconteurNode, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__RobotRaconteurNodeSetup[] = {  {&_swigt__p_RobotRaconteur__RobotRaconteurNodeSetup, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__RobotRaconteurVersion[] = {  {&_swigt__p_RobotRaconteur__RobotRaconteurVersion, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__ServerContext[] = {  {&_swigt__p_RobotRaconteur__ServerContext, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__ServerEndpoint[] = {  {&_swigt__p_RobotRaconteur__ServerEndpoint, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__ServiceDefinition[] = {  {&_swigt__p_RobotRaconteur__ServiceDefinition, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__ServiceDefinitionParseInfo[] = {  {&_swigt__p_RobotRaconteur__ServiceDefinitionParseInfo, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__ServiceEntryDefinition[] = {  {&_swigt__p_RobotRaconteur__ServiceEntryDefinition, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__ServiceInfo2Wrapped[] = {  {&_swigt__p_RobotRaconteur__ServiceInfo2Wrapped, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__ServicePathSegment[] = {  {&_swigt__p_RobotRaconteur__ServicePathSegment, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__ServiceSecurityPolicy[] = {  {&_swigt__p_RobotRaconteur__ServiceSecurityPolicy, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__ServiceSubscriptionClientID[] = {  {&_swigt__p_RobotRaconteur__ServiceSubscriptionClientID, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__ServiceSubscriptionFilterAttribute[] = {  {&_swigt__p_RobotRaconteur__ServiceSubscriptionFilterAttribute, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup[] = {  {&_swigt__p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__TcpTransport[] = {  {&_swigt__p_RobotRaconteur__TcpTransport, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__TimeSpec[] = {  {&_swigt__p_RobotRaconteur__TimeSpec, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__Timer[] = {  {&_swigt__p_RobotRaconteur__WallTimer, _p_RobotRaconteur__WallTimerTo_p_RobotRaconteur__Timer, 0, 0},  {&_swigt__p_RobotRaconteur__Timer, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__TimerEvent[] = {  {&_swigt__p_RobotRaconteur__TimerEvent, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__Transport[] = {  {&_swigt__p_RobotRaconteur__Transport, 0, 0, 0},  {&_swigt__p_RobotRaconteur__TcpTransport, _p_RobotRaconteur__TcpTransportTo_p_RobotRaconteur__Transport, 0, 0},  {&_swigt__p_RobotRaconteur__LocalTransport, _p_RobotRaconteur__LocalTransportTo_p_RobotRaconteur__Transport, 0, 0},  {&_swigt__p_RobotRaconteur__HardwareTransport, _p_RobotRaconteur__HardwareTransportTo_p_RobotRaconteur__Transport, 0, 0},  {&_swigt__p_RobotRaconteur__IntraTransport, _p_RobotRaconteur__IntraTransportTo_p_RobotRaconteur__Transport, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__TryGetValueResult[] = {  {&_swigt__p_RobotRaconteur__TryGetValueResult, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__TypeDefinition[] = {  {&_swigt__p_RobotRaconteur__TypeDefinition, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__UserAuthenticator[] = {  {&_swigt__p_RobotRaconteur__UserAuthenticator, 0, 0, 0},  {&_swigt__p_RobotRaconteur__PasswordFileUserAuthenticator, _p_RobotRaconteur__PasswordFileUserAuthenticatorTo_p_RobotRaconteur__UserAuthenticator, 0, 0},  {&_swigt__p_RobotRaconteur__WrappedUserAuthenticator, _p_RobotRaconteur__WrappedUserAuthenticatorTo_p_RobotRaconteur__UserAuthenticator, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__UserLogRecordHandlerBase[] = {  {&_swigt__p_RobotRaconteur__UserLogRecordHandlerBase, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__UserLogRecordHandlerDirector[] = {  {&_swigt__p_RobotRaconteur__UserLogRecordHandlerDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__UsingDefinition[] = {  {&_swigt__p_RobotRaconteur__UsingDefinition, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WallRate[] = {  {&_swigt__p_RobotRaconteur__WallRate, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WallTimer[] = {  {&_swigt__p_RobotRaconteur__WallTimer, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WireDefinition[] = {  {&_swigt__p_RobotRaconteur__WireDefinition, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedArrayMemoryClientUtil[] = {  {&_swigt__p_RobotRaconteur__WrappedArrayMemoryClientUtil, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedArrayMemoryDirector[] = {  {&_swigt__p_RobotRaconteur__WrappedArrayMemoryDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedDirectorLock[] = {  {&_swigt__p_RobotRaconteur__WrappedDirectorLock, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedGeneratorClient[] = {  {&_swigt__p_RobotRaconteur__WrappedGeneratorClient, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedGeneratorClient_TryGetNextResult[] = {  {&_swigt__p_RobotRaconteur__WrappedGeneratorClient_TryGetNextResult, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedGeneratorServerDirector[] = {  {&_swigt__p_RobotRaconteur__WrappedGeneratorServerDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedMultiDimArrayMemoryClientUtil[] = {  {&_swigt__p_RobotRaconteur__WrappedMultiDimArrayMemoryClientUtil, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedMultiDimArrayMemoryDirector[] = {  {&_swigt__p_RobotRaconteur__WrappedMultiDimArrayMemoryDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedMultiDimArrayMemoryParams[] = {  {&_swigt__p_RobotRaconteur__WrappedMultiDimArrayMemoryParams, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedNamedArrayMemoryClient[] = {  {&_swigt__p_RobotRaconteur__WrappedNamedArrayMemoryClient, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedNamedArrayMemoryClientBuffer[] = {  {&_swigt__p_RobotRaconteur__WrappedNamedArrayMemoryClientBuffer, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedNamedArrayMemoryDirector[] = {  {&_swigt__p_RobotRaconteur__WrappedNamedArrayMemoryDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClient[] = {  {&_swigt__p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClient, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClientBuffer[] = {  {&_swigt__p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClientBuffer, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryDirector[] = {  {&_swigt__p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedPipeBroadcaster[] = {  {&_swigt__p_RobotRaconteur__WrappedPipeBroadcaster, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedPipeBroadcasterPredicateDirector[] = {  {&_swigt__p_RobotRaconteur__WrappedPipeBroadcasterPredicateDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedPipeClient[] = {  {&_swigt__p_RobotRaconteur__WrappedPipeClient, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedPipeEndpoint[] = {  {&_swigt__p_RobotRaconteur__WrappedPipeEndpoint, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedPipeEndpointDirector[] = {  {&_swigt__p_RobotRaconteur__WrappedPipeEndpointDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedPipeServer[] = {  {&_swigt__p_RobotRaconteur__WrappedPipeServer, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedPipeServerConnectDirector[] = {  {&_swigt__p_RobotRaconteur__WrappedPipeServerConnectDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedPipeSubscription[] = {  {&_swigt__p_RobotRaconteur__WrappedPipeSubscription, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedPipeSubscriptionDirector[] = {  {&_swigt__p_RobotRaconteur__WrappedPipeSubscriptionDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedPipeSubscription_send_iterator[] = {  {&_swigt__p_RobotRaconteur__WrappedPipeSubscription_send_iterator, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedPodArrayMemoryClient[] = {  {&_swigt__p_RobotRaconteur__WrappedPodArrayMemoryClient, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedPodArrayMemoryClientBuffer[] = {  {&_swigt__p_RobotRaconteur__WrappedPodArrayMemoryClientBuffer, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedPodArrayMemoryDirector[] = {  {&_swigt__p_RobotRaconteur__WrappedPodArrayMemoryDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedPodMultiDimArrayMemoryClient[] = {  {&_swigt__p_RobotRaconteur__WrappedPodMultiDimArrayMemoryClient, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedPodMultiDimArrayMemoryClientBuffer[] = {  {&_swigt__p_RobotRaconteur__WrappedPodMultiDimArrayMemoryClientBuffer, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedPodMultiDimArrayMemoryDirector[] = {  {&_swigt__p_RobotRaconteur__WrappedPodMultiDimArrayMemoryDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedRRObject[] = {  {&_swigt__p_RobotRaconteur__WrappedRRObject, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedServiceInfo2Subscription[] = {  {&_swigt__p_RobotRaconteur__WrappedServiceInfo2Subscription, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedServiceInfo2SubscriptionDirector[] = {  {&_swigt__p_RobotRaconteur__WrappedServiceInfo2SubscriptionDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedServiceSkel[] = {  {&_swigt__p_RobotRaconteur__WrappedServiceSkel, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedServiceSkelAsyncAdapter[] = {  {&_swigt__p_RobotRaconteur__WrappedServiceSkelAsyncAdapter, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedServiceSkelDirector[] = {  {&_swigt__p_RobotRaconteur__WrappedServiceSkelDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedServiceStub[] = {  {&_swigt__p_RobotRaconteur__WrappedServiceStub, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedServiceStubDirector[] = {  {&_swigt__p_RobotRaconteur__WrappedServiceStubDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedServiceSubscription[] = {  {&_swigt__p_RobotRaconteur__WrappedServiceSubscription, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedServiceSubscriptionDirector[] = {  {&_swigt__p_RobotRaconteur__WrappedServiceSubscriptionDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedServiceSubscriptionFilter[] = {  {&_swigt__p_RobotRaconteur__WrappedServiceSubscriptionFilter, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedServiceSubscriptionFilterNode[] = {  {&_swigt__p_RobotRaconteur__WrappedServiceSubscriptionFilterNode, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedServiceSubscriptionFilterPredicateDirector[] = {  {&_swigt__p_RobotRaconteur__WrappedServiceSubscriptionFilterPredicateDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedServiceSubscriptionManager[] = {  {&_swigt__p_RobotRaconteur__WrappedServiceSubscriptionManager, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails[] = {  {&_swigt__p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedServiceSubscription_TryDefaultClientRes[] = {  {&_swigt__p_RobotRaconteur__WrappedServiceSubscription_TryDefaultClientRes, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedService_typed_packet[] = {  {&_swigt__p_RobotRaconteur__WrappedService_typed_packet, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedSubObjectSubscription[] = {  {&_swigt__p_RobotRaconteur__WrappedSubObjectSubscription, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedTryReceivePacketWaitResult[] = {  {&_swigt__p_RobotRaconteur__WrappedTryReceivePacketWaitResult, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedUserAuthenticator[] = {  {&_swigt__p_RobotRaconteur__WrappedUserAuthenticator, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedUserAuthenticatorDirector[] = {  {&_swigt__p_RobotRaconteur__WrappedUserAuthenticatorDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedWireBroadcaster[] = {  {&_swigt__p_RobotRaconteur__WrappedWireBroadcaster, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedWireBroadcasterPredicateDirector[] = {  {&_swigt__p_RobotRaconteur__WrappedWireBroadcasterPredicateDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedWireClient[] = {  {&_swigt__p_RobotRaconteur__WrappedWireClient, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedWireConnection[] = {  {&_swigt__p_RobotRaconteur__WrappedWireConnection, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedWireConnectionDirector[] = {  {&_swigt__p_RobotRaconteur__WrappedWireConnectionDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedWireServer[] = {  {&_swigt__p_RobotRaconteur__WrappedWireServer, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedWireServerConnectDirector[] = {  {&_swigt__p_RobotRaconteur__WrappedWireServerConnectDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedWireServerPeekValueDirector[] = {  {&_swigt__p_RobotRaconteur__WrappedWireServerPeekValueDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedWireServerPokeValueDirector[] = {  {&_swigt__p_RobotRaconteur__WrappedWireServerPokeValueDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedWireSubscription[] = {  {&_swigt__p_RobotRaconteur__WrappedWireSubscription, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedWireSubscriptionDirector[] = {  {&_swigt__p_RobotRaconteur__WrappedWireSubscriptionDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedWireSubscription_send_iterator[] = {  {&_swigt__p_RobotRaconteur__WrappedWireSubscription_send_iterator, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__WrappedWireUnicastReceiver[] = {  {&_swigt__p_RobotRaconteur__WrappedWireUnicastReceiver, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RobotRaconteur__detail__MultiDimArray_CalculateCopyIndicesIter[] = {  {&_swigt__p_RobotRaconteur__detail__MultiDimArray_CalculateCopyIndicesIter, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_ServerServiceListenerDirector[] = {  {&_swigt__p_ServerServiceListenerDirector, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_allocator_type[] = {  {&_swigt__p_allocator_type, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__arrayT_unsigned_char_16_t[] = {  {&_swigt__p_boost__arrayT_unsigned_char_16_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__filesystem__path[] = {  {&_swigt__p_boost__filesystem__path, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__optionalT_std__string_t[] = {  {&_swigt__p_boost__optionalT_std__string_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__posix_time__ptime[] = {  {&_swigt__p_boost__posix_time__ptime, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__posix_time__time_duration[] = {  {&_swigt__p_boost__posix_time__time_duration, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__ArrayMemoryBase_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__ArrayMemoryBase_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__AuthenticatedUser_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__AuthenticatedUser_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__AutoResetEvent_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__AutoResetEvent_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__BroadcastDownsampler_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__BroadcastDownsampler_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__CallbackDefinition_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__CallbackDefinition_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__EventDefinition_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__EventDefinition_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__FileLogRecordHandler_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__FileLogRecordHandler_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__IntraTransport_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__IntraTransport_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__LogRecordHandler_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__UserLogRecordHandlerBase_t, _p_boost__shared_ptrT_RobotRaconteur__UserLogRecordHandlerBase_tTo_p_boost__shared_ptrT_RobotRaconteur__LogRecordHandler_t, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__LogRecordHandler_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_t, _p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_tTo_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_t, _p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_tTo_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__MemoryDefinition_t, _p_boost__shared_ptrT_RobotRaconteur__MemoryDefinition_tTo_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__PipeDefinition_t, _p_boost__shared_ptrT_RobotRaconteur__PipeDefinition_tTo_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WireDefinition_t, _p_boost__shared_ptrT_RobotRaconteur__WireDefinition_tTo_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__EventDefinition_t, _p_boost__shared_ptrT_RobotRaconteur__EventDefinition_tTo_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_t, _p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_tTo_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__CallbackDefinition_t, _p_boost__shared_ptrT_RobotRaconteur__CallbackDefinition_tTo_p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__MemoryDefinition_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__MemoryDefinition_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t, _p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_tTo_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, _p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_tTo_p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__MessageElement_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__Message_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__Message_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__MultiDimArrayMemoryBase_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__MultiDimArrayMemoryBase_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__NodeDirectoriesFD_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__NodeDirectoriesFD_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__PasswordFileUserAuthenticator_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__PasswordFileUserAuthenticator_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__PipeDefinition_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__PipeDefinition_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__RRMultiDimArrayUntyped_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__RRMultiDimArrayUntyped_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__RRObject_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, _p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_tTo_p_boost__shared_ptrT_RobotRaconteur__RRObject_t, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedRRObject_t, _p_boost__shared_ptrT_RobotRaconteur__WrappedRRObject_tTo_p_boost__shared_ptrT_RobotRaconteur__RRObject_t, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__RRObject_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__RRValue_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t, _p_boost__shared_ptrT_RobotRaconteur__MessageEntry_tTo_p_boost__shared_ptrT_RobotRaconteur__RRValue_t, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t, _p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_tTo_p_boost__shared_ptrT_RobotRaconteur__RRValue_t, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__Message_t, _p_boost__shared_ptrT_RobotRaconteur__Message_tTo_p_boost__shared_ptrT_RobotRaconteur__RRValue_t, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t, _p_boost__shared_ptrT_RobotRaconteur__MessageHeader_tTo_p_boost__shared_ptrT_RobotRaconteur__RRValue_t, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__RRValue_t, 0, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t, _p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_tTo_p_boost__shared_ptrT_RobotRaconteur__RRValue_t, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t, _p_boost__shared_ptrT_RobotRaconteur__MessageElementData_tTo_p_boost__shared_ptrT_RobotRaconteur__RRValue_t, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__MessageElement_t, _p_boost__shared_ptrT_RobotRaconteur__MessageElement_tTo_p_boost__shared_ptrT_RobotRaconteur__RRValue_t, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__Rate_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__Rate_t, 0, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WallRate_t, _p_boost__shared_ptrT_RobotRaconteur__WallRate_tTo_p_boost__shared_ptrT_RobotRaconteur__Rate_t, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNodeSetup_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNodeSetup_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__ServerContext_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__ServerContext_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__ServerEndpoint_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__ServerEndpoint_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__ServiceSecurityPolicy_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__ServiceSecurityPolicy_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__Timer_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__Timer_t, 0, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WallTimer_t, _p_boost__shared_ptrT_RobotRaconteur__WallTimer_tTo_p_boost__shared_ptrT_RobotRaconteur__Timer_t, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__Transport_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t, _p_boost__shared_ptrT_RobotRaconteur__LocalTransport_tTo_p_boost__shared_ptrT_RobotRaconteur__Transport_t, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__IntraTransport_t, _p_boost__shared_ptrT_RobotRaconteur__IntraTransport_tTo_p_boost__shared_ptrT_RobotRaconteur__Transport_t, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t, _p_boost__shared_ptrT_RobotRaconteur__TcpTransport_tTo_p_boost__shared_ptrT_RobotRaconteur__Transport_t, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__Transport_t, 0, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t, _p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_tTo_p_boost__shared_ptrT_RobotRaconteur__Transport_t, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__UserAuthenticator_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__UserAuthenticator_t, 0, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__PasswordFileUserAuthenticator_t, _p_boost__shared_ptrT_RobotRaconteur__PasswordFileUserAuthenticator_tTo_p_boost__shared_ptrT_RobotRaconteur__UserAuthenticator_t, 0, 0},  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedUserAuthenticator_t, _p_boost__shared_ptrT_RobotRaconteur__WrappedUserAuthenticator_tTo_p_boost__shared_ptrT_RobotRaconteur__UserAuthenticator_t, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__UserLogRecordHandlerBase_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__UserLogRecordHandlerBase_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WallRate_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WallRate_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WallTimer_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WallTimer_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WireDefinition_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WireDefinition_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedDirectorLock_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedDirectorLock_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedGeneratorClient_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedGeneratorClient_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedNamedArrayMemoryClient_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedNamedArrayMemoryClient_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClient_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClient_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeBroadcaster_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeBroadcaster_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeClient_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeClient_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeServer_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeServer_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedPodArrayMemoryClient_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedPodArrayMemoryClient_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedPodMultiDimArrayMemoryClient_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedPodMultiDimArrayMemoryClient_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedRRObject_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedRRObject_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceInfo2Subscription_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceInfo2Subscription_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedUserAuthenticator_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedUserAuthenticator_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedWireBroadcaster_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedWireBroadcaster_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedWireClient_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedWireClient_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedWireServer_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedWireServer_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedWireUnicastReceiver_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__WrappedWireUnicastReceiver_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__shared_ptrT_RobotRaconteur__detail__MultiDimArray_CalculateCopyIndicesIter_t[] = {  {&_swigt__p_boost__shared_ptrT_RobotRaconteur__detail__MultiDimArray_CalculateCopyIndicesIter_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_boost__string_ref[] = {  {&_swigt__p_boost__string_ref, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_char[] = {  {&_swigt__p_char, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_difference_type[] = {  {&_swigt__p_difference_type, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_int[] = {  {&_swigt__p_int, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_key_type[] = {  {&_swigt__p_key_type, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_long[] = {  {&_swigt__p_long, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_mapped_type[] = {  {&_swigt__p_mapped_type, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p_PyObject[] = {  {&_swigt__p_p_PyObject, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_short[] = {  {&_swigt__p_short, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_signed_char[] = {  {&_swigt__p_signed_char, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_size_type[] = {  {&_swigt__p_size_type, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t[] = {  {&_swigt__p_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t[] = {  {&_swigt__p_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_RobotRaconteur__NodeInfo2_t[] = {  {&_swigt__p_std__allocatorT_RobotRaconteur__NodeInfo2_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t[] = {  {&_swigt__p_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_RobotRaconteur__ServicePathSegment_t[] = {  {&_swigt__p_std__allocatorT_RobotRaconteur__ServicePathSegment_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t[] = {  {&_swigt__p_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t[] = {  {&_swigt__p_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t[] = {  {&_swigt__p_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t[] = {  {&_swigt__p_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t[] = {  {&_swigt__p_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t[] = {  {&_swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t[] = {  {&_swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t[] = {  {&_swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t[] = {  {&_swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t[] = {  {&_swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t[] = {  {&_swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t[] = {  {&_swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t[] = {  {&_swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t[] = {  {&_swigt__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_int_t[] = {  {&_swigt__p_std__allocatorT_int_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_signed_char_t[] = {  {&_swigt__p_std__allocatorT_signed_char_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t[] = {  {&_swigt__p_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t[] = {  {&_swigt__p_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t[] = {  {&_swigt__p_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t[] = {  {&_swigt__p_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_std__pairT_std__string_const_std__string_t_t[] = {  {&_swigt__p_std__allocatorT_std__pairT_std__string_const_std__string_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_std__string_t[] = {  {&_swigt__p_std__allocatorT_std__string_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_unsigned_char_t[] = {  {&_swigt__p_std__allocatorT_unsigned_char_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_unsigned_int_t[] = {  {&_swigt__p_std__allocatorT_unsigned_int_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__allocatorT_unsigned_long_t[] = {  {&_swigt__p_std__allocatorT_unsigned_long_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__invalid_argument[] = {  {&_swigt__p_std__invalid_argument, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t[] = {  {&_swigt__p_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__lessT_std__string_t[] = {  {&_swigt__p_std__lessT_std__string_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t[] = {  {&_swigt__p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t[] = {  {&_swigt__p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t[] = {  {&_swigt__p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t[] = {  {&_swigt__p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t[] = {  {&_swigt__p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__runtime_error[] = {  {&_swigt__p_std__runtime_error, 0, 0, 0},  {&_swigt__p_RobotRaconteur__NodeDirectoriesResourceAlreadyInUse, _p_RobotRaconteur__NodeDirectoriesResourceAlreadyInUseTo_p_std__runtime_error, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t[] = {  {&_swigt__p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t[] = {  {&_swigt__p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t[] = {  {&_swigt__p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t[] = {  {&_swigt__p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t[] = {  {&_swigt__p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t[] = {  {&_swigt__p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t[] = {  {&_swigt__p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t[] = {  {&_swigt__p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t[] = {  {&_swigt__p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t[] = {  {&_swigt__p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t[] = {  {&_swigt__p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t[] = {  {&_swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t[] = {  {&_swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t[] = {  {&_swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t[] = {  {&_swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t[] = {  {&_swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t[] = {  {&_swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t[] = {  {&_swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t[] = {  {&_swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t[] = {  {&_swigt__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_int_std__allocatorT_int_t_t[] = {  {&_swigt__p_std__vectorT_int_std__allocatorT_int_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t[] = {  {&_swigt__p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_std__string_std__allocatorT_std__string_t_t[] = {  {&_swigt__p_std__vectorT_std__string_std__allocatorT_std__string_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t[] = {  {&_swigt__p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t[] = {  {&_swigt__p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t[] = {  {&_swigt__p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_swig__SwigPyIterator[] = {  {&_swigt__p_swig__SwigPyIterator, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_unsigned_char[] = {  {&_swigt__p_unsigned_char, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_unsigned_int[] = {  {&_swigt__p_unsigned_int, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_unsigned_long[] = {  {&_swigt__p_unsigned_long, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_unsigned_short[] = {  {&_swigt__p_unsigned_short, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_value_type[] = {  {&_swigt__p_value_type, 0, 0, 0},{0, 0, 0, 0}};

static swig_cast_info *swig_cast_initial[] = {
  _swigc__p_ClientServiceListenerDirector,
  _swigc__p_MemberDefinitionUtil,
  _swigc__p_MessageElementDataUtil,
  _swigc__p_RobotRaconteur__ArrayMemoryBase,
  _swigc__p_RobotRaconteur__AsyncGeneratorClientReturnDirector,
  _swigc__p_RobotRaconteur__AsyncNodeInfo2VectorReturnDirector,
  _swigc__p_RobotRaconteur__AsyncPipeEndpointReturnDirector,
  _swigc__p_RobotRaconteur__AsyncRequestDirector,
  _swigc__p_RobotRaconteur__AsyncServiceInfo2VectorReturnDirector,
  _swigc__p_RobotRaconteur__AsyncStringReturnDirector,
  _swigc__p_RobotRaconteur__AsyncStubReturnDirector,
  _swigc__p_RobotRaconteur__AsyncTimerEventReturnDirector,
  _swigc__p_RobotRaconteur__AsyncUInt32ReturnDirector,
  _swigc__p_RobotRaconteur__AsyncVoidNoErrReturnDirector,
  _swigc__p_RobotRaconteur__AsyncVoidReturnDirector,
  _swigc__p_RobotRaconteur__AsyncWireConnectionReturnDirector,
  _swigc__p_RobotRaconteur__AsyncWirePeekReturnDirector,
  _swigc__p_RobotRaconteur__AuthenticatedUser,
  _swigc__p_RobotRaconteur__AutoResetEvent,
  _swigc__p_RobotRaconteur__BroadcastDownsampler,
  _swigc__p_RobotRaconteur__CallbackDefinition,
  _swigc__p_RobotRaconteur__CommandLineConfigParser,
  _swigc__p_RobotRaconteur__ConstantDefinition,
  _swigc__p_RobotRaconteur__ConstantDefinition_StructField,
  _swigc__p_RobotRaconteur__EnumDefinition,
  _swigc__p_RobotRaconteur__EnumDefinitionValue,
  _swigc__p_RobotRaconteur__EventDefinition,
  _swigc__p_RobotRaconteur__ExceptionDefinition,
  _swigc__p_RobotRaconteur__FileLogRecordHandler,
  _swigc__p_RobotRaconteur__FunctionDefinition,
  _swigc__p_RobotRaconteur__GetUuidForNameAndLockResult,
  _swigc__p_RobotRaconteur__HandlerErrorInfo,
  _swigc__p_RobotRaconteur__HardwareTransport,
  _swigc__p_RobotRaconteur__IntraTransport,
  _swigc__p_RobotRaconteur__LocalTransport,
  _swigc__p_RobotRaconteur__LogRecordHandler,
  _swigc__p_RobotRaconteur__MemberDefinition,
  _swigc__p_RobotRaconteur__MemoryDefinition,
  _swigc__p_RobotRaconteur__Message,
  _swigc__p_RobotRaconteur__MessageElement,
  _swigc__p_RobotRaconteur__MessageElementData,
  _swigc__p_RobotRaconteur__MessageElementNestedElementList,
  _swigc__p_RobotRaconteur__MessageEntry,
  _swigc__p_RobotRaconteur__MessageHeader,
  _swigc__p_RobotRaconteur__MultiDimArrayMemoryBase,
  _swigc__p_RobotRaconteur__NodeDirectories,
  _swigc__p_RobotRaconteur__NodeDirectoriesFD,
  _swigc__p_RobotRaconteur__NodeDirectoriesResourceAlreadyInUse,
  _swigc__p_RobotRaconteur__NodeDirectoriesUtil,
  _swigc__p_RobotRaconteur__NodeID,
  _swigc__p_RobotRaconteur__NodeInfo2,
  _swigc__p_RobotRaconteur__ObjRefDefinition,
  _swigc__p_RobotRaconteur__PasswordFileUserAuthenticator,
  _swigc__p_RobotRaconteur__PipeDefinition,
  _swigc__p_RobotRaconteur__PropertyDefinition,
  _swigc__p_RobotRaconteur__RRBaseArray,
  _swigc__p_RobotRaconteur__RRLogRecord,
  _swigc__p_RobotRaconteur__RRMultiDimArrayUntyped,
  _swigc__p_RobotRaconteur__RRNativeDirectorSupport,
  _swigc__p_RobotRaconteur__RRObject,
  _swigc__p_RobotRaconteur__RRValue,
  _swigc__p_RobotRaconteur__Rate,
  _swigc__p_RobotRaconteur__RobotRaconteurNode,
  _swigc__p_RobotRaconteur__RobotRaconteurNodeSetup,
  _swigc__p_RobotRaconteur__RobotRaconteurVersion,
  _swigc__p_RobotRaconteur__ServerContext,
  _swigc__p_RobotRaconteur__ServerEndpoint,
  _swigc__p_RobotRaconteur__ServiceDefinition,
  _swigc__p_RobotRaconteur__ServiceDefinitionParseInfo,
  _swigc__p_RobotRaconteur__ServiceEntryDefinition,
  _swigc__p_RobotRaconteur__ServiceInfo2Wrapped,
  _swigc__p_RobotRaconteur__ServicePathSegment,
  _swigc__p_RobotRaconteur__ServiceSecurityPolicy,
  _swigc__p_RobotRaconteur__ServiceSubscriptionClientID,
  _swigc__p_RobotRaconteur__ServiceSubscriptionFilterAttribute,
  _swigc__p_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup,
  _swigc__p_RobotRaconteur__TcpTransport,
  _swigc__p_RobotRaconteur__TimeSpec,
  _swigc__p_RobotRaconteur__Timer,
  _swigc__p_RobotRaconteur__TimerEvent,
  _swigc__p_RobotRaconteur__Transport,
  _swigc__p_RobotRaconteur__TryGetValueResult,
  _swigc__p_RobotRaconteur__TypeDefinition,
  _swigc__p_RobotRaconteur__UserAuthenticator,
  _swigc__p_RobotRaconteur__UserLogRecordHandlerBase,
  _swigc__p_RobotRaconteur__UserLogRecordHandlerDirector,
  _swigc__p_RobotRaconteur__UsingDefinition,
  _swigc__p_RobotRaconteur__WallRate,
  _swigc__p_RobotRaconteur__WallTimer,
  _swigc__p_RobotRaconteur__WireDefinition,
  _swigc__p_RobotRaconteur__WrappedArrayMemoryClientUtil,
  _swigc__p_RobotRaconteur__WrappedArrayMemoryDirector,
  _swigc__p_RobotRaconteur__WrappedDirectorLock,
  _swigc__p_RobotRaconteur__WrappedGeneratorClient,
  _swigc__p_RobotRaconteur__WrappedGeneratorClient_TryGetNextResult,
  _swigc__p_RobotRaconteur__WrappedGeneratorServerDirector,
  _swigc__p_RobotRaconteur__WrappedMultiDimArrayMemoryClientUtil,
  _swigc__p_RobotRaconteur__WrappedMultiDimArrayMemoryDirector,
  _swigc__p_RobotRaconteur__WrappedMultiDimArrayMemoryParams,
  _swigc__p_RobotRaconteur__WrappedNamedArrayMemoryClient,
  _swigc__p_RobotRaconteur__WrappedNamedArrayMemoryClientBuffer,
  _swigc__p_RobotRaconteur__WrappedNamedArrayMemoryDirector,
  _swigc__p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClient,
  _swigc__p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClientBuffer,
  _swigc__p_RobotRaconteur__WrappedNamedMultiDimArrayMemoryDirector,
  _swigc__p_RobotRaconteur__WrappedPipeBroadcaster,
  _swigc__p_RobotRaconteur__WrappedPipeBroadcasterPredicateDirector,
  _swigc__p_RobotRaconteur__WrappedPipeClient,
  _swigc__p_RobotRaconteur__WrappedPipeEndpoint,
  _swigc__p_RobotRaconteur__WrappedPipeEndpointDirector,
  _swigc__p_RobotRaconteur__WrappedPipeServer,
  _swigc__p_RobotRaconteur__WrappedPipeServerConnectDirector,
  _swigc__p_RobotRaconteur__WrappedPipeSubscription,
  _swigc__p_RobotRaconteur__WrappedPipeSubscriptionDirector,
  _swigc__p_RobotRaconteur__WrappedPipeSubscription_send_iterator,
  _swigc__p_RobotRaconteur__WrappedPodArrayMemoryClient,
  _swigc__p_RobotRaconteur__WrappedPodArrayMemoryClientBuffer,
  _swigc__p_RobotRaconteur__WrappedPodArrayMemoryDirector,
  _swigc__p_RobotRaconteur__WrappedPodMultiDimArrayMemoryClient,
  _swigc__p_RobotRaconteur__WrappedPodMultiDimArrayMemoryClientBuffer,
  _swigc__p_RobotRaconteur__WrappedPodMultiDimArrayMemoryDirector,
  _swigc__p_RobotRaconteur__WrappedRRObject,
  _swigc__p_RobotRaconteur__WrappedServiceInfo2Subscription,
  _swigc__p_RobotRaconteur__WrappedServiceInfo2SubscriptionDirector,
  _swigc__p_RobotRaconteur__WrappedServiceSkel,
  _swigc__p_RobotRaconteur__WrappedServiceSkelAsyncAdapter,
  _swigc__p_RobotRaconteur__WrappedServiceSkelDirector,
  _swigc__p_RobotRaconteur__WrappedServiceStub,
  _swigc__p_RobotRaconteur__WrappedServiceStubDirector,
  _swigc__p_RobotRaconteur__WrappedServiceSubscription,
  _swigc__p_RobotRaconteur__WrappedServiceSubscriptionDirector,
  _swigc__p_RobotRaconteur__WrappedServiceSubscriptionFilter,
  _swigc__p_RobotRaconteur__WrappedServiceSubscriptionFilterNode,
  _swigc__p_RobotRaconteur__WrappedServiceSubscriptionFilterPredicateDirector,
  _swigc__p_RobotRaconteur__WrappedServiceSubscriptionManager,
  _swigc__p_RobotRaconteur__WrappedServiceSubscriptionManagerDetails,
  _swigc__p_RobotRaconteur__WrappedServiceSubscription_TryDefaultClientRes,
  _swigc__p_RobotRaconteur__WrappedService_typed_packet,
  _swigc__p_RobotRaconteur__WrappedSubObjectSubscription,
  _swigc__p_RobotRaconteur__WrappedTryReceivePacketWaitResult,
  _swigc__p_RobotRaconteur__WrappedUserAuthenticator,
  _swigc__p_RobotRaconteur__WrappedUserAuthenticatorDirector,
  _swigc__p_RobotRaconteur__WrappedWireBroadcaster,
  _swigc__p_RobotRaconteur__WrappedWireBroadcasterPredicateDirector,
  _swigc__p_RobotRaconteur__WrappedWireClient,
  _swigc__p_RobotRaconteur__WrappedWireConnection,
  _swigc__p_RobotRaconteur__WrappedWireConnectionDirector,
  _swigc__p_RobotRaconteur__WrappedWireServer,
  _swigc__p_RobotRaconteur__WrappedWireServerConnectDirector,
  _swigc__p_RobotRaconteur__WrappedWireServerPeekValueDirector,
  _swigc__p_RobotRaconteur__WrappedWireServerPokeValueDirector,
  _swigc__p_RobotRaconteur__WrappedWireSubscription,
  _swigc__p_RobotRaconteur__WrappedWireSubscriptionDirector,
  _swigc__p_RobotRaconteur__WrappedWireSubscription_send_iterator,
  _swigc__p_RobotRaconteur__WrappedWireUnicastReceiver,
  _swigc__p_RobotRaconteur__detail__MultiDimArray_CalculateCopyIndicesIter,
  _swigc__p_ServerServiceListenerDirector,
  _swigc__p_allocator_type,
  _swigc__p_boost__arrayT_unsigned_char_16_t,
  _swigc__p_boost__filesystem__path,
  _swigc__p_boost__optionalT_std__string_t,
  _swigc__p_boost__posix_time__ptime,
  _swigc__p_boost__posix_time__time_duration,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__ArrayMemoryBase_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__AuthenticatedUser_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__AutoResetEvent_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__BroadcastDownsampler_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__CallbackDefinition_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__CommandLineConfigParser_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__EventDefinition_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__FileLogRecordHandler_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__FunctionDefinition_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__HardwareTransport_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__IntraTransport_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__LocalTransport_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__LogRecordHandler_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__MemoryDefinition_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__MessageElementData_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__MessageElementNestedElementList_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__MessageElement_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__MessageEntry_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__MessageHeader_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__Message_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__MultiDimArrayMemoryBase_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__NodeDirectoriesFD_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__ObjRefDefinition_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__PasswordFileUserAuthenticator_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__PipeDefinition_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__PropertyDefinition_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__RRBaseArray_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__RRMultiDimArrayUntyped_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__RRObject_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__RRValue_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__Rate_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNodeSetup_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__RobotRaconteurNode_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__ServerContext_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__ServerEndpoint_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__ServiceSecurityPolicy_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__TcpTransport_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__Timer_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__Transport_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__UserAuthenticator_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__UserLogRecordHandlerBase_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WallRate_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WallTimer_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WireDefinition_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedDirectorLock_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedGeneratorClient_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedNamedArrayMemoryClient_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedNamedMultiDimArrayMemoryClient_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeBroadcaster_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeClient_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeEndpoint_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeServer_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedPipeSubscription_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedPodArrayMemoryClient_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedPodMultiDimArrayMemoryClient_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedRRObject_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceInfo2Subscription_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkelAsyncAdapter_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSkel_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilter_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionManager_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscription_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedSubObjectSubscription_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedUserAuthenticator_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedWireBroadcaster_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedWireClient_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedWireConnection_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedWireServer_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedWireSubscription_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__WrappedWireUnicastReceiver_t,
  _swigc__p_boost__shared_ptrT_RobotRaconteur__detail__MultiDimArray_CalculateCopyIndicesIter_t,
  _swigc__p_boost__string_ref,
  _swigc__p_char,
  _swigc__p_difference_type,
  _swigc__p_int,
  _swigc__p_key_type,
  _swigc__p_long,
  _swigc__p_mapped_type,
  _swigc__p_p_PyObject,
  _swigc__p_short,
  _swigc__p_signed_char,
  _swigc__p_size_type,
  _swigc__p_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t,
  _swigc__p_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t,
  _swigc__p_std__allocatorT_RobotRaconteur__NodeInfo2_t,
  _swigc__p_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t,
  _swigc__p_std__allocatorT_RobotRaconteur__ServicePathSegment_t,
  _swigc__p_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t,
  _swigc__p_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t,
  _swigc__p_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t,
  _swigc__p_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t,
  _swigc__p_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t,
  _swigc__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t,
  _swigc__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t,
  _swigc__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t,
  _swigc__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t,
  _swigc__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t,
  _swigc__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t,
  _swigc__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t,
  _swigc__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t,
  _swigc__p_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t,
  _swigc__p_std__allocatorT_int_t,
  _swigc__p_std__allocatorT_signed_char_t,
  _swigc__p_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t,
  _swigc__p_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t,
  _swigc__p_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t,
  _swigc__p_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t,
  _swigc__p_std__allocatorT_std__pairT_std__string_const_std__string_t_t,
  _swigc__p_std__allocatorT_std__string_t,
  _swigc__p_std__allocatorT_unsigned_char_t,
  _swigc__p_std__allocatorT_unsigned_int_t,
  _swigc__p_std__allocatorT_unsigned_long_t,
  _swigc__p_std__invalid_argument,
  _swigc__p_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t,
  _swigc__p_std__lessT_std__string_t,
  _swigc__p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_RobotRaconteur__ServiceInfo2Wrapped_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_RobotRaconteur__ServiceInfo2Wrapped_t_t_t,
  _swigc__p_std__mapT_RobotRaconteur__ServiceSubscriptionClientID_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_std__lessT_RobotRaconteur__ServiceSubscriptionClientID_t_std__allocatorT_std__pairT_RobotRaconteur__ServiceSubscriptionClientID_const_boost__shared_ptrT_RobotRaconteur__WrappedServiceStub_t_t_t_t,
  _swigc__p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t_t,
  _swigc__p_std__mapT_std__string_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t_t,
  _swigc__p_std__mapT_std__string_std__string_std__lessT_std__string_t_std__allocatorT_std__pairT_std__string_const_std__string_t_t_t,
  _swigc__p_std__runtime_error,
  _swigc__p_std__vectorT_RobotRaconteur__ConstantDefinition_StructField_std__allocatorT_RobotRaconteur__ConstantDefinition_StructField_t_t,
  _swigc__p_std__vectorT_RobotRaconteur__EnumDefinitionValue_std__allocatorT_RobotRaconteur__EnumDefinitionValue_t_t,
  _swigc__p_std__vectorT_RobotRaconteur__NodeInfo2_std__allocatorT_RobotRaconteur__NodeInfo2_t_t,
  _swigc__p_std__vectorT_RobotRaconteur__ServiceInfo2Wrapped_std__allocatorT_RobotRaconteur__ServiceInfo2Wrapped_t_t,
  _swigc__p_std__vectorT_RobotRaconteur__ServicePathSegment_std__allocatorT_RobotRaconteur__ServicePathSegment_t_t,
  _swigc__p_std__vectorT_RobotRaconteur__ServiceSubscriptionClientID_std__allocatorT_RobotRaconteur__ServiceSubscriptionClientID_t_t,
  _swigc__p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttributeGroup_t_t,
  _swigc__p_std__vectorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_std__allocatorT_RobotRaconteur__ServiceSubscriptionFilterAttribute_t_t,
  _swigc__p_std__vectorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_std__allocatorT_RobotRaconteur__WrappedServiceSubscriptionManagerDetails_t_t,
  _swigc__p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageElement_t_t_t,
  _swigc__p_std__vectorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_std__allocatorT_boost__intrusive_ptrT_RobotRaconteur__MessageEntry_t_t_t,
  _swigc__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ConstantDefinition_t_t_t,
  _swigc__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__EnumDefinition_t_t_t,
  _swigc__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ExceptionDefinition_t_t_t,
  _swigc__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__MemberDefinition_t_t_t,
  _swigc__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceDefinition_t_t_t,
  _swigc__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__ServiceEntryDefinition_t_t_t,
  _swigc__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__TypeDefinition_t_t_t,
  _swigc__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__UsingDefinition_t_t_t,
  _swigc__p_std__vectorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_std__allocatorT_boost__shared_ptrT_RobotRaconteur__WrappedServiceSubscriptionFilterNode_t_t_t,
  _swigc__p_std__vectorT_int_std__allocatorT_int_t_t,
  _swigc__p_std__vectorT_signed_char_std__allocatorT_signed_char_t_t,
  _swigc__p_std__vectorT_std__string_std__allocatorT_std__string_t_t,
  _swigc__p_std__vectorT_unsigned_char_std__allocatorT_unsigned_char_t_t,
  _swigc__p_std__vectorT_unsigned_int_std__allocatorT_unsigned_int_t_t,
  _swigc__p_std__vectorT_unsigned_long_std__allocatorT_unsigned_long_t_t,
  _swigc__p_swig__SwigPyIterator,
  _swigc__p_unsigned_char,
  _swigc__p_unsigned_int,
  _swigc__p_unsigned_long,
  _swigc__p_unsigned_short,
  _swigc__p_value_type,
};


/* -------- TYPE CONVERSION AND EQUIVALENCE RULES (END) -------- */

static swig_const_info swig_const_table[] = {
{0, 0, 0, 0.0, 0, 0}};

#ifdef __cplusplus
}
#endif
/* -----------------------------------------------------------------------------
 * Type initialization:
 * This problem is tough by the requirement that no dynamic
 * memory is used. Also, since swig_type_info structures store pointers to
 * swig_cast_info structures and swig_cast_info structures store pointers back
 * to swig_type_info structures, we need some lookup code at initialization.
 * The idea is that swig generates all the structures that are needed.
 * The runtime then collects these partially filled structures.
 * The SWIG_InitializeModule function takes these initial arrays out of
 * swig_module, and does all the lookup, filling in the swig_module.types
 * array with the correct data and linking the correct swig_cast_info
 * structures together.
 *
 * The generated swig_type_info structures are assigned statically to an initial
 * array. We just loop through that array, and handle each type individually.
 * First we lookup if this type has been already loaded, and if so, use the
 * loaded structure instead of the generated one. Then we have to fill in the
 * cast linked list. The cast data is initially stored in something like a
 * two-dimensional array. Each row corresponds to a type (there are the same
 * number of rows as there are in the swig_type_initial array). Each entry in
 * a column is one of the swig_cast_info structures for that type.
 * The cast_initial array is actually an array of arrays, because each row has
 * a variable number of columns. So to actually build the cast linked list,
 * we find the array of casts associated with the type, and loop through it
 * adding the casts to the list. The one last trick we need to do is making
 * sure the type pointer in the swig_cast_info struct is correct.
 *
 * First off, we lookup the cast->type name to see if it is already loaded.
 * There are three cases to handle:
 *  1) If the cast->type has already been loaded AND the type we are adding
 *     casting info to has not been loaded (it is in this module), THEN we
 *     replace the cast->type pointer with the type pointer that has already
 *     been loaded.
 *  2) If BOTH types (the one we are adding casting info to, and the
 *     cast->type) are loaded, THEN the cast info has already been loaded by
 *     the previous module so we just ignore it.
 *  3) Finally, if cast->type has not already been loaded, then we add that
 *     swig_cast_info to the linked list (because the cast->type) pointer will
 *     be correct.
 * ----------------------------------------------------------------------------- */

#ifdef __cplusplus
extern "C" {
#if 0
} /* c-mode */
#endif
#endif

#if 0
#define SWIGRUNTIME_DEBUG
#endif


SWIGRUNTIME void
SWIG_InitializeModule(void *clientdata) {
  size_t i;
  swig_module_info *module_head, *iter;
  int init;
  
  /* check to see if the circular list has been setup, if not, set it up */
  if (swig_module.next==0) {
    /* Initialize the swig_module */
    swig_module.type_initial = swig_type_initial;
    swig_module.cast_initial = swig_cast_initial;
    swig_module.next = &swig_module;
    init = 1;
  } else {
    init = 0;
  }
  
  /* Try and load any already created modules */
  module_head = SWIG_GetModule(clientdata);
  if (!module_head) {
    /* This is the first module loaded for this interpreter */
    /* so set the swig module into the interpreter */
    SWIG_SetModule(clientdata, &swig_module);
  } else {
    /* the interpreter has loaded a SWIG module, but has it loaded this one? */
    iter=module_head;
    do {
      if (iter==&swig_module) {
        /* Our module is already in the list, so there's nothing more to do. */
        return;
      }
      iter=iter->next;
    } while (iter!= module_head);
    
    /* otherwise we must add our module into the list */
    swig_module.next = module_head->next;
    module_head->next = &swig_module;
  }
  
  /* When multiple interpreters are used, a module could have already been initialized in
       a different interpreter, but not yet have a pointer in this interpreter.
       In this case, we do not want to continue adding types... everything should be
       set up already */
  if (init == 0) return;
  
  /* Now work on filling in swig_module.types */
#ifdef SWIGRUNTIME_DEBUG
  printf("SWIG_InitializeModule: size %lu\n", (unsigned long)swig_module.size);
#endif
  for (i = 0; i < swig_module.size; ++i) {
    swig_type_info *type = 0;
    swig_type_info *ret;
    swig_cast_info *cast;
    
#ifdef SWIGRUNTIME_DEBUG
    printf("SWIG_InitializeModule: type %lu %s\n", (unsigned long)i, swig_module.type_initial[i]->name);
#endif
    
    /* if there is another module already loaded */
    if (swig_module.next != &swig_module) {
      type = SWIG_MangledTypeQueryModule(swig_module.next, &swig_module, swig_module.type_initial[i]->name);
    }
    if (type) {
      /* Overwrite clientdata field */
#ifdef SWIGRUNTIME_DEBUG
      printf("SWIG_InitializeModule: found type %s\n", type->name);
#endif
      if (swig_module.type_initial[i]->clientdata) {
        type->clientdata = swig_module.type_initial[i]->clientdata;
#ifdef SWIGRUNTIME_DEBUG
        printf("SWIG_InitializeModule: found and overwrite type %s \n", type->name);
#endif
      }
    } else {
      type = swig_module.type_initial[i];
    }
    
    /* Insert casting types */
    cast = swig_module.cast_initial[i];
    while (cast->type) {
      /* Don't need to add information already in the list */
      ret = 0;
#ifdef SWIGRUNTIME_DEBUG
      printf("SWIG_InitializeModule: look cast %s\n", cast->type->name);
#endif
      if (swig_module.next != &swig_module) {
        ret = SWIG_MangledTypeQueryModule(swig_module.next, &swig_module, cast->type->name);
#ifdef SWIGRUNTIME_DEBUG
        if (ret) printf("SWIG_InitializeModule: found cast %s\n", ret->name);
#endif
      }
      if (ret) {
        if (type == swig_module.type_initial[i]) {
#ifdef SWIGRUNTIME_DEBUG
          printf("SWIG_InitializeModule: skip old type %s\n", ret->name);
#endif
          cast->type = ret;
          ret = 0;
        } else {
          /* Check for casting already in the list */
          swig_cast_info *ocast = SWIG_TypeCheck(ret->name, type);
#ifdef SWIGRUNTIME_DEBUG
          if (ocast) printf("SWIG_InitializeModule: skip old cast %s\n", ret->name);
#endif
          if (!ocast) ret = 0;
        }
      }
      
      if (!ret) {
#ifdef SWIGRUNTIME_DEBUG
        printf("SWIG_InitializeModule: adding cast %s\n", cast->type->name);
#endif
        if (type->cast) {
          type->cast->prev = cast;
          cast->next = type->cast;
        }
        type->cast = cast;
      }
      cast++;
    }
    /* Set entry in modules->types array equal to the type */
    swig_module.types[i] = type;
  }
  swig_module.types[i] = 0;
  
#ifdef SWIGRUNTIME_DEBUG
  printf("**** SWIG_InitializeModule: Cast List ******\n");
  for (i = 0; i < swig_module.size; ++i) {
    int j = 0;
    swig_cast_info *cast = swig_module.cast_initial[i];
    printf("SWIG_InitializeModule: type %lu %s\n", (unsigned long)i, swig_module.type_initial[i]->name);
    while (cast->type) {
      printf("SWIG_InitializeModule: cast type %s\n", cast->type->name);
      cast++;
      ++j;
    }
    printf("---- Total casts: %d\n",j);
  }
  printf("**** SWIG_InitializeModule: Cast List ******\n");
#endif
}

/* This function will propagate the clientdata field of type to
* any new swig_type_info structures that have been added into the list
* of equivalent types.  It is like calling
* SWIG_TypeClientData(type, clientdata) a second time.
*/
SWIGRUNTIME void
SWIG_PropagateClientData(void) {
  size_t i;
  swig_cast_info *equiv;
  static int init_run = 0;
  
  if (init_run) return;
  init_run = 1;
  
  for (i = 0; i < swig_module.size; i++) {
    if (swig_module.types[i]->clientdata) {
      equiv = swig_module.types[i]->cast;
      while (equiv) {
        if (!equiv->converter) {
          if (equiv->type && !equiv->type->clientdata)
          SWIG_TypeClientData(equiv->type, swig_module.types[i]->clientdata);
        }
        equiv = equiv->next;
      }
    }
  }
}

#ifdef __cplusplus
#if 0
{
  /* c-mode */
#endif
}
#endif



#ifdef __cplusplus
extern "C" {
#endif
  
  /* Python-specific SWIG API */
#define SWIG_newvarlink()                             SWIG_Python_newvarlink()
#define SWIG_addvarlink(p, name, get_attr, set_attr)  SWIG_Python_addvarlink(p, name, get_attr, set_attr)
#define SWIG_InstallConstants(d, constants)           SWIG_Python_InstallConstants(d, constants)
  
  /* -----------------------------------------------------------------------------
   * global variable support code.
   * ----------------------------------------------------------------------------- */
  
  typedef struct swig_globalvar {
    char       *name;                  /* Name of global variable */
    PyObject *(*get_attr)(void);       /* Return the current value */
    int       (*set_attr)(PyObject *); /* Set the value */
    struct swig_globalvar *next;
  } swig_globalvar;
  
  typedef struct swig_varlinkobject {
    PyObject_HEAD
    swig_globalvar *vars;
  } swig_varlinkobject;
  
  SWIGINTERN PyObject *
  swig_varlink_repr(swig_varlinkobject *SWIGUNUSEDPARM(v)) {
#if PY_VERSION_HEX >= 0x03000000
    return PyUnicode_InternFromString("<Swig global variables>");
#else
    return PyString_FromString("<Swig global variables>");
#endif
  }
  
  SWIGINTERN PyObject *
  swig_varlink_str(swig_varlinkobject *v) {
#if PY_VERSION_HEX >= 0x03000000
    PyObject *str = PyUnicode_InternFromString("(");
    PyObject *tail;
    PyObject *joined;
    swig_globalvar *var;
    for (var = v->vars; var; var=var->next) {
      tail = PyUnicode_FromString(var->name);
      joined = PyUnicode_Concat(str, tail);
      Py_DecRef(str);
      Py_DecRef(tail);
      str = joined;
      if (var->next) {
        tail = PyUnicode_InternFromString(", ");
        joined = PyUnicode_Concat(str, tail);
        Py_DecRef(str);
        Py_DecRef(tail);
        str = joined;
      }
    }
    tail = PyUnicode_InternFromString(")");
    joined = PyUnicode_Concat(str, tail);
    Py_DecRef(str);
    Py_DecRef(tail);
    str = joined;
#else
    PyObject *str = PyString_FromString("(");
    swig_globalvar *var;
    for (var = v->vars; var; var=var->next) {
      PyString_ConcatAndDel(&str,PyString_FromString(var->name));
      if (var->next) PyString_ConcatAndDel(&str,PyString_FromString(", "));
    }
    PyString_ConcatAndDel(&str,PyString_FromString(")"));
#endif
    return str;
  }
  
  SWIGINTERN void
  swig_varlink_dealloc(swig_varlinkobject *v) {
    swig_globalvar *var = v->vars;
    while (var) {
      swig_globalvar *n = var->next;
      free(var->name);
      free(var);
      var = n;
    }
  }
  
  SWIGINTERN PyObject *
  swig_varlink_getattr(swig_varlinkobject *v, char *n) {
    PyObject *res = NULL;
    swig_globalvar *var = v->vars;
    while (var) {
      if (strcmp(var->name,n) == 0) {
        res = (*var->get_attr)();
        break;
      }
      var = var->next;
    }
    if (res == NULL && !PyErr_Occurred()) {
      PyErr_Format(PyExc_AttributeError, "Unknown C global variable '%s'", n);
    }
    return res;
  }
  
  SWIGINTERN int
  swig_varlink_setattr(swig_varlinkobject *v, char *n, PyObject *p) {
    int res = 1;
    swig_globalvar *var = v->vars;
    while (var) {
      if (strcmp(var->name,n) == 0) {
        res = (*var->set_attr)(p);
        break;
      }
      var = var->next;
    }
    if (res == 1 && !PyErr_Occurred()) {
      PyErr_Format(PyExc_AttributeError, "Unknown C global variable '%s'", n);
    }
    return res;
  }
  
  SWIGINTERN PyTypeObject*
  swig_varlink_type(void) {
    static char varlink__doc__[] = "Swig var link object";
    static PyTypeObject varlink_type;
    static int type_init = 0;
    if (!type_init) {
      const PyTypeObject tmp = {
#if PY_VERSION_HEX >= 0x03000000
        PyVarObject_HEAD_INIT(NULL, 0)
#else
        PyObject_HEAD_INIT(NULL)
        0,                                  /* ob_size */
#endif
        "swigvarlink",                      /* tp_name */
        sizeof(swig_varlinkobject),         /* tp_basicsize */
        0,                                  /* tp_itemsize */
        (destructor) swig_varlink_dealloc,  /* tp_dealloc */
        0,                                  /* tp_print */
        (getattrfunc) swig_varlink_getattr, /* tp_getattr */
        (setattrfunc) swig_varlink_setattr, /* tp_setattr */
        0,                                  /* tp_compare */
        (reprfunc) swig_varlink_repr,       /* tp_repr */
        0,                                  /* tp_as_number */
        0,                                  /* tp_as_sequence */
        0,                                  /* tp_as_mapping */
        0,                                  /* tp_hash */
        0,                                  /* tp_call */
        (reprfunc) swig_varlink_str,        /* tp_str */
        0,                                  /* tp_getattro */
        0,                                  /* tp_setattro */
        0,                                  /* tp_as_buffer */
        0,                                  /* tp_flags */
        varlink__doc__,                     /* tp_doc */
        0,                                  /* tp_traverse */
        0,                                  /* tp_clear */
        0,                                  /* tp_richcompare */
        0,                                  /* tp_weaklistoffset */
        0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, /* tp_iter -> tp_weaklist */
        0,                                  /* tp_del */
        0,                                  /* tp_version_tag */
#if PY_VERSION_HEX >= 0x03040000
        0,                                  /* tp_finalize */
#endif
#if PY_VERSION_HEX >= 0x03080000
        0,                                  /* tp_vectorcall */
#endif
#if (PY_VERSION_HEX >= 0x03080000) && (PY_VERSION_HEX < 0x03090000)
        0,                                  /* tp_print */
#endif
#ifdef COUNT_ALLOCS
        0,                                  /* tp_allocs */
        0,                                  /* tp_frees */
        0,                                  /* tp_maxalloc */
        0,                                  /* tp_prev */
        0                                   /* tp_next */
#endif
      };
      varlink_type = tmp;
      type_init = 1;
      if (PyType_Ready(&varlink_type) < 0)
      return NULL;
    }
    return &varlink_type;
  }
  
  /* Create a variable linking object for use later */
  SWIGINTERN PyObject *
  SWIG_Python_newvarlink(void) {
    swig_varlinkobject *result = PyObject_NEW(swig_varlinkobject, swig_varlink_type());
    if (result) {
      result->vars = 0;
    }
    return ((PyObject*) result);
  }
  
  SWIGINTERN void 
  SWIG_Python_addvarlink(PyObject *p, const char *name, PyObject *(*get_attr)(void), int (*set_attr)(PyObject *p)) {
    swig_varlinkobject *v = (swig_varlinkobject *) p;
    swig_globalvar *gv = (swig_globalvar *) malloc(sizeof(swig_globalvar));
    if (gv) {
      size_t size = strlen(name)+1;
      gv->name = (char *)malloc(size);
      if (gv->name) {
        memcpy(gv->name, name, size);
        gv->get_attr = get_attr;
        gv->set_attr = set_attr;
        gv->next = v->vars;
      }
    }
    v->vars = gv;
  }
  
  SWIGINTERN PyObject *
  SWIG_globals(void) {
    static PyObject *globals = 0;
    if (!globals) {
      globals = SWIG_newvarlink();
    }
    return globals;
  }
  
  /* -----------------------------------------------------------------------------
   * constants/methods manipulation
   * ----------------------------------------------------------------------------- */
  
  /* Install Constants */
  SWIGINTERN void
  SWIG_Python_InstallConstants(PyObject *d, swig_const_info constants[]) {
    PyObject *obj = 0;
    size_t i;
    for (i = 0; constants[i].type; ++i) {
      switch(constants[i].type) {
      case SWIG_PY_POINTER:
        obj = SWIG_InternalNewPointerObj(constants[i].pvalue, *(constants[i]).ptype,0);
        break;
      case SWIG_PY_BINARY:
        obj = SWIG_NewPackedObj(constants[i].pvalue, constants[i].lvalue, *(constants[i].ptype));
        break;
      default:
        obj = 0;
        break;
      }
      if (obj) {
        PyDict_SetItemString(d, constants[i].name, obj);
        Py_DECREF(obj);
      }
    }
  }
  
  /* -----------------------------------------------------------------------------*/
  /* Fix SwigMethods to carry the callback ptrs when needed */
  /* -----------------------------------------------------------------------------*/
  
  SWIGINTERN void
  SWIG_Python_FixMethods(PyMethodDef *methods,
    swig_const_info *const_table,
    swig_type_info **types,
    swig_type_info **types_initial) {
    size_t i;
    for (i = 0; methods[i].ml_name; ++i) {
      const char *c = methods[i].ml_doc;
      if (!c) continue;
      c = strstr(c, "swig_ptr: ");
      if (c) {
        int j;
        swig_const_info *ci = 0;
        const char *name = c + 10;
        for (j = 0; const_table[j].type; ++j) {
          if (strncmp(const_table[j].name, name, 
              strlen(const_table[j].name)) == 0) {
            ci = &(const_table[j]);
            break;
          }
        }
        if (ci) {
          void *ptr = (ci->type == SWIG_PY_POINTER) ? ci->pvalue : 0;
          if (ptr) {
            size_t shift = (ci->ptype) - types;
            swig_type_info *ty = types_initial[shift];
            size_t ldoc = (c - methods[i].ml_doc);
            size_t lptr = strlen(ty->name)+2*sizeof(void*)+2;
            char *ndoc = (char*)malloc(ldoc + lptr + 10);
            if (ndoc) {
              char *buff = ndoc;
              memcpy(buff, methods[i].ml_doc, ldoc);
              buff += ldoc;
              memcpy(buff, "swig_ptr: ", 10);
              buff += 10;
              SWIG_PackVoidPtr(buff, ptr, ty->name, lptr);
              methods[i].ml_doc = ndoc;
            }
          }
        }
      }
    }
  } 
  
  /* -----------------------------------------------------------------------------
   * Method creation and docstring support functions
   * ----------------------------------------------------------------------------- */
  
  /* -----------------------------------------------------------------------------
   * Function to find the method definition with the correct docstring for the
   * proxy module as opposed to the low-level API
   * ----------------------------------------------------------------------------- */
  
  SWIGINTERN PyMethodDef *SWIG_PythonGetProxyDoc(const char *name) {
    /* Find the function in the modified method table */
    size_t offset = 0;
    int found = 0;
    while (SwigMethods_proxydocs[offset].ml_meth != NULL) {
      if (strcmp(SwigMethods_proxydocs[offset].ml_name, name) == 0) {
        found = 1;
        break;
      }
      offset++;
    }
    /* Use the copy with the modified docstring if available */
    return found ? &SwigMethods_proxydocs[offset] : NULL;
  }
  
  /* -----------------------------------------------------------------------------
   * Wrapper of PyInstanceMethod_New() used in Python 3
   * It is exported to the generated module, used for -fastproxy
   * ----------------------------------------------------------------------------- */
  
  SWIGINTERN PyObject *SWIG_PyInstanceMethod_New(PyObject *SWIGUNUSEDPARM(self), PyObject *func) {
    if (PyCFunction_Check(func)) {
      PyCFunctionObject *funcobj = (PyCFunctionObject *)func;
      PyMethodDef *ml = SWIG_PythonGetProxyDoc(funcobj->m_ml->ml_name);
      if (ml)
      func = PyCFunction_NewEx(ml, funcobj->m_self, funcobj->m_module);
    }
#if PY_VERSION_HEX >= 0x03000000
    return PyInstanceMethod_New(func);
#else
    return PyMethod_New(func, NULL, NULL);
#endif
  }
  
  /* -----------------------------------------------------------------------------
   * Wrapper of PyStaticMethod_New()
   * It is exported to the generated module, used for -fastproxy
   * ----------------------------------------------------------------------------- */
  
  SWIGINTERN PyObject *SWIG_PyStaticMethod_New(PyObject *SWIGUNUSEDPARM(self), PyObject *func) {
    if (PyCFunction_Check(func)) {
      PyCFunctionObject *funcobj = (PyCFunctionObject *)func;
      PyMethodDef *ml = SWIG_PythonGetProxyDoc(funcobj->m_ml->ml_name);
      if (ml)
      func = PyCFunction_NewEx(ml, funcobj->m_self, funcobj->m_module);
    }
    return PyStaticMethod_New(func);
  }
  
#ifdef __cplusplus
}
#endif

/* -----------------------------------------------------------------------------*
 *  Partial Init method
 * -----------------------------------------------------------------------------*/

#ifdef __cplusplus
extern "C"
#endif

SWIGEXPORT 
#if PY_VERSION_HEX >= 0x03000000
PyObject*
#else
void
#endif
SWIG_init(void) {
  PyObject *m, *d, *md, *globals;
  
#if PY_VERSION_HEX >= 0x03000000
  static struct PyModuleDef SWIG_module = {
    PyModuleDef_HEAD_INIT,
    SWIG_name,
    NULL,
    -1,
    SwigMethods,
    NULL,
    NULL,
    NULL,
    NULL
  };
#endif
  
#if defined(SWIGPYTHON_BUILTIN)
  static SwigPyClientData SwigPyObject_clientdata = {
    0, 0, 0, 0, 0, 0, 0
  };
  static PyGetSetDef this_getset_def = {
    (char *)"this", &SwigPyBuiltin_ThisClosure, NULL, NULL, NULL
  };
  static SwigPyGetSet thisown_getset_closure = {
    SwigPyObject_own,
    SwigPyObject_own
  };
  static PyGetSetDef thisown_getset_def = {
    (char *)"thisown", SwigPyBuiltin_GetterClosure, SwigPyBuiltin_SetterClosure, NULL, &thisown_getset_closure
  };
  PyTypeObject *builtin_pytype;
  int builtin_base_count;
  swig_type_info *builtin_basetype;
  PyObject *tuple;
  PyGetSetDescrObject *static_getset;
  PyTypeObject *metatype;
  PyTypeObject *swigpyobject;
  SwigPyClientData *cd;
  PyObject *public_interface, *public_symbol;
  PyObject *this_descr;
  PyObject *thisown_descr;
  PyObject *self = 0;
  int i;
  
  (void)builtin_pytype;
  (void)builtin_base_count;
  (void)builtin_basetype;
  (void)tuple;
  (void)static_getset;
  (void)self;
  
  /* Metaclass is used to implement static member variables */
  metatype = SwigPyObjectType();
  assert(metatype);
#endif
  
  (void)globals;
  
  /* Create singletons now to avoid potential deadlocks with multi-threaded usage after module initialization */
  SWIG_This();
  SWIG_Python_TypeCache();
  SwigPyPacked_type();
#ifndef SWIGPYTHON_BUILTIN
  SwigPyObject_type();
#endif
  
  /* Fix SwigMethods to carry the callback ptrs when needed */
  SWIG_Python_FixMethods(SwigMethods, swig_const_table, swig_types, swig_type_initial);
  
#if PY_VERSION_HEX >= 0x03000000
  m = PyModule_Create(&SWIG_module);
#else
  m = Py_InitModule(SWIG_name, SwigMethods);
#endif
  
  md = d = PyModule_GetDict(m);
  (void)md;
  
  SWIG_InitializeModule(0);
  
#ifdef SWIGPYTHON_BUILTIN
  swigpyobject = SwigPyObject_TypeOnce();
  
  SwigPyObject_stype = SWIG_MangledTypeQuery("_p_SwigPyObject");
  assert(SwigPyObject_stype);
  cd = (SwigPyClientData*) SwigPyObject_stype->clientdata;
  if (!cd) {
    SwigPyObject_stype->clientdata = &SwigPyObject_clientdata;
    SwigPyObject_clientdata.pytype = swigpyobject;
  } else if (swigpyobject->tp_basicsize != cd->pytype->tp_basicsize) {
    PyErr_SetString(PyExc_RuntimeError, "Import error: attempted to load two incompatible swig-generated modules.");
# if PY_VERSION_HEX >= 0x03000000
    return NULL;
# else
    return;
# endif
  }
  
  /* All objects have a 'this' attribute */
  this_descr = PyDescr_NewGetSet(SwigPyObject_type(), &this_getset_def);
  (void)this_descr;
  
  /* All objects have a 'thisown' attribute */
  thisown_descr = PyDescr_NewGetSet(SwigPyObject_type(), &thisown_getset_def);
  (void)thisown_descr;
  
  public_interface = PyList_New(0);
  public_symbol = 0;
  (void)public_symbol;
  
  PyDict_SetItemString(md, "__all__", public_interface);
  Py_DECREF(public_interface);
  for (i = 0; SwigMethods[i].ml_name != NULL; ++i)
  SwigPyBuiltin_AddPublicSymbol(public_interface, SwigMethods[i].ml_name);
  for (i = 0; swig_const_table[i].name != 0; ++i)
  SwigPyBuiltin_AddPublicSymbol(public_interface, swig_const_table[i].name);
#endif
  
  SWIG_InstallConstants(d,swig_const_table);
  
  SWIG_Python_SetConstant(d, "SHARED_PTR_DISOWN",SWIG_From_int(static_cast< int >(0)));
  
  
  PyDateTime_IMPORT;
  
  RobotRaconteur::PythonTypeSupport_Init();
  
  RobotRaconteur::RobotRaconteurNode::s()->SetDynamicServiceFactory(RR_MAKE_SHARED<RobotRaconteur::WrappedDynamicServiceFactory>());
  RobotRaconteur::RobotRaconteurNode::s()->SetThreadPoolFactory(RR_MAKE_SHARED<RobotRaconteur::PythonThreadPoolFactory>());
  //RobotRaconteur::RobotRaconteurNode::s()->UnregisterServiceType("RobotRaconteurServiceIndex");
  //RobotRaconteur::RobotRaconteurNode::s()->RegisterServiceType(RR_MAKE_SHARED<RobotRaconteur::WrappedServiceFactory>(RR_MAKE_SHARED<RobotRaconteurServiceIndex::RobotRaconteurServiceIndexFactory>()->DefString()));
  
  SWIG_Python_SetConstant(d, "DataTypes_void_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_void_t)));
  SWIG_Python_SetConstant(d, "DataTypes_double_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_double_t)));
  SWIG_Python_SetConstant(d, "DataTypes_single_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_single_t)));
  SWIG_Python_SetConstant(d, "DataTypes_int8_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_int8_t)));
  SWIG_Python_SetConstant(d, "DataTypes_uint8_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_uint8_t)));
  SWIG_Python_SetConstant(d, "DataTypes_int16_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_int16_t)));
  SWIG_Python_SetConstant(d, "DataTypes_uint16_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_uint16_t)));
  SWIG_Python_SetConstant(d, "DataTypes_int32_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_int32_t)));
  SWIG_Python_SetConstant(d, "DataTypes_uint32_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_uint32_t)));
  SWIG_Python_SetConstant(d, "DataTypes_int64_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_int64_t)));
  SWIG_Python_SetConstant(d, "DataTypes_uint64_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_uint64_t)));
  SWIG_Python_SetConstant(d, "DataTypes_string_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_string_t)));
  SWIG_Python_SetConstant(d, "DataTypes_cdouble_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_cdouble_t)));
  SWIG_Python_SetConstant(d, "DataTypes_csingle_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_csingle_t)));
  SWIG_Python_SetConstant(d, "DataTypes_bool_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_bool_t)));
  SWIG_Python_SetConstant(d, "DataTypes_structure_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_structure_t)));
  SWIG_Python_SetConstant(d, "DataTypes_vector_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_vector_t)));
  SWIG_Python_SetConstant(d, "DataTypes_dictionary_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_dictionary_t)));
  SWIG_Python_SetConstant(d, "DataTypes_object_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_object_t)));
  SWIG_Python_SetConstant(d, "DataTypes_varvalue_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_varvalue_t)));
  SWIG_Python_SetConstant(d, "DataTypes_varobject_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_varobject_t)));
  SWIG_Python_SetConstant(d, "DataTypes_list_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_list_t)));
  SWIG_Python_SetConstant(d, "DataTypes_pod_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_pod_t)));
  SWIG_Python_SetConstant(d, "DataTypes_pod_array_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_pod_array_t)));
  SWIG_Python_SetConstant(d, "DataTypes_pod_multidimarray_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_pod_multidimarray_t)));
  SWIG_Python_SetConstant(d, "DataTypes_enum_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_enum_t)));
  SWIG_Python_SetConstant(d, "DataTypes_namedtype_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_namedtype_t)));
  SWIG_Python_SetConstant(d, "DataTypes_namedarray_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_namedarray_t)));
  SWIG_Python_SetConstant(d, "DataTypes_namedarray_array_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_namedarray_array_t)));
  SWIG_Python_SetConstant(d, "DataTypes_namedarray_multidimarray_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_namedarray_multidimarray_t)));
  SWIG_Python_SetConstant(d, "DataTypes_multidimarray_t",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_multidimarray_t)));
  SWIG_Python_SetConstant(d, "DataTypes_ArrayTypes_none",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_ArrayTypes_none)));
  SWIG_Python_SetConstant(d, "DataTypes_ArrayTypes_array",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_ArrayTypes_array)));
  SWIG_Python_SetConstant(d, "DataTypes_ArrayTypes_multidimarray",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_ArrayTypes_multidimarray)));
  SWIG_Python_SetConstant(d, "DataTypes_ContainerTypes_none",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_ContainerTypes_none)));
  SWIG_Python_SetConstant(d, "DataTypes_ContainerTypes_list",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_ContainerTypes_list)));
  SWIG_Python_SetConstant(d, "DataTypes_ContainerTypes_map_int32",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_ContainerTypes_map_int32)));
  SWIG_Python_SetConstant(d, "DataTypes_ContainerTypes_map_string",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_ContainerTypes_map_string)));
  SWIG_Python_SetConstant(d, "DataTypes_ContainerTypes_generator",SWIG_From_int(static_cast< int >(RobotRaconteur::DataTypes_ContainerTypes_generator)));
  globals = SWIG_globals();
  if (!globals) {
    PyErr_SetString(PyExc_TypeError, "Failure to create SWIG globals.");
#if PY_VERSION_HEX >= 0x03000000
    return NULL;
#else
    return;
#endif
  }
  PyDict_SetItemString(md, "cvar", globals);
  Py_DECREF(globals);
  SWIG_addvarlink(globals, "MessageFlags_ROUTING_INFO", Swig_var_MessageFlags_ROUTING_INFO_get, Swig_var_MessageFlags_ROUTING_INFO_set);
  SWIG_addvarlink(globals, "MessageFlags_ENDPOINT_INFO", Swig_var_MessageFlags_ENDPOINT_INFO_get, Swig_var_MessageFlags_ENDPOINT_INFO_set);
  SWIG_addvarlink(globals, "MessageFlags_PRIORITY", Swig_var_MessageFlags_PRIORITY_get, Swig_var_MessageFlags_PRIORITY_set);
  SWIG_addvarlink(globals, "MessageFlags_UNRELIABLE", Swig_var_MessageFlags_UNRELIABLE_get, Swig_var_MessageFlags_UNRELIABLE_set);
  SWIG_addvarlink(globals, "MessageFlags_META_INFO", Swig_var_MessageFlags_META_INFO_get, Swig_var_MessageFlags_META_INFO_set);
  SWIG_addvarlink(globals, "MessageFlags_STRING_TABLE", Swig_var_MessageFlags_STRING_TABLE_get, Swig_var_MessageFlags_STRING_TABLE_set);
  SWIG_addvarlink(globals, "MessageFlags_MULTIPLE_ENTRIES", Swig_var_MessageFlags_MULTIPLE_ENTRIES_get, Swig_var_MessageFlags_MULTIPLE_ENTRIES_set);
  SWIG_addvarlink(globals, "MessageFlags_EXTENDED", Swig_var_MessageFlags_EXTENDED_get, Swig_var_MessageFlags_EXTENDED_set);
  SWIG_addvarlink(globals, "MessageFlags_Version2Compat", Swig_var_MessageFlags_Version2Compat_get, Swig_var_MessageFlags_Version2Compat_set);
  SWIG_addvarlink(globals, "MessageEntryFlags_SERVICE_PATH_STR", Swig_var_MessageEntryFlags_SERVICE_PATH_STR_get, Swig_var_MessageEntryFlags_SERVICE_PATH_STR_set);
  SWIG_addvarlink(globals, "MessageEntryFlags_SERVICE_PATH_CODE", Swig_var_MessageEntryFlags_SERVICE_PATH_CODE_get, Swig_var_MessageEntryFlags_SERVICE_PATH_CODE_set);
  SWIG_addvarlink(globals, "MessageEntryFlags_MEMBER_NAME_STR", Swig_var_MessageEntryFlags_MEMBER_NAME_STR_get, Swig_var_MessageEntryFlags_MEMBER_NAME_STR_set);
  SWIG_addvarlink(globals, "MessageEntryFlags_MEMBER_NAME_CODE", Swig_var_MessageEntryFlags_MEMBER_NAME_CODE_get, Swig_var_MessageEntryFlags_MEMBER_NAME_CODE_set);
  SWIG_addvarlink(globals, "MessageEntryFlags_REQUEST_ID", Swig_var_MessageEntryFlags_REQUEST_ID_get, Swig_var_MessageEntryFlags_REQUEST_ID_set);
  SWIG_addvarlink(globals, "MessageEntryFlags_ERROR", Swig_var_MessageEntryFlags_ERROR_get, Swig_var_MessageEntryFlags_ERROR_set);
  SWIG_addvarlink(globals, "MessageEntryFlags_META_INFO", Swig_var_MessageEntryFlags_META_INFO_get, Swig_var_MessageEntryFlags_META_INFO_set);
  SWIG_addvarlink(globals, "MessageEntryFlags_EXTENDED", Swig_var_MessageEntryFlags_EXTENDED_get, Swig_var_MessageEntryFlags_EXTENDED_set);
  SWIG_addvarlink(globals, "MessageEntryFlags_Version2Compat", Swig_var_MessageEntryFlags_Version2Compat_get, Swig_var_MessageEntryFlags_Version2Compat_set);
  SWIG_addvarlink(globals, "MessageElementFlags_ELEMENT_NAME_STR", Swig_var_MessageElementFlags_ELEMENT_NAME_STR_get, Swig_var_MessageElementFlags_ELEMENT_NAME_STR_set);
  SWIG_addvarlink(globals, "MessageElementFlags_ELEMENT_NAME_CODE", Swig_var_MessageElementFlags_ELEMENT_NAME_CODE_get, Swig_var_MessageElementFlags_ELEMENT_NAME_CODE_set);
  SWIG_addvarlink(globals, "MessageElementFlags_ELEMENT_NUMBER", Swig_var_MessageElementFlags_ELEMENT_NUMBER_get, Swig_var_MessageElementFlags_ELEMENT_NUMBER_set);
  SWIG_addvarlink(globals, "MessageElementFlags_ELEMENT_TYPE_NAME_STR", Swig_var_MessageElementFlags_ELEMENT_TYPE_NAME_STR_get, Swig_var_MessageElementFlags_ELEMENT_TYPE_NAME_STR_set);
  SWIG_addvarlink(globals, "MessageElementFlags_ELEMENT_TYPE_NAME_CODE", Swig_var_MessageElementFlags_ELEMENT_TYPE_NAME_CODE_get, Swig_var_MessageElementFlags_ELEMENT_TYPE_NAME_CODE_set);
  SWIG_addvarlink(globals, "MessageElementFlags_META_INFO", Swig_var_MessageElementFlags_META_INFO_get, Swig_var_MessageElementFlags_META_INFO_set);
  SWIG_addvarlink(globals, "MessageElementFlags_EXTENDED", Swig_var_MessageElementFlags_EXTENDED_get, Swig_var_MessageElementFlags_EXTENDED_set);
  SWIG_addvarlink(globals, "MessageElementFlags_Version2Compat", Swig_var_MessageElementFlags_Version2Compat_get, Swig_var_MessageElementFlags_Version2Compat_set);
  SWIG_Python_SetConstant(d, "MessageEntryType_Null",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_Null)));
  SWIG_Python_SetConstant(d, "MessageEntryType_StreamOp",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_StreamOp)));
  SWIG_Python_SetConstant(d, "MessageEntryType_StreamOpRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_StreamOpRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_StreamCheckCapability",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_StreamCheckCapability)));
  SWIG_Python_SetConstant(d, "MessageEntryType_StreamCheckCapabilityRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_StreamCheckCapabilityRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_GetServiceDesc",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_GetServiceDesc)));
  SWIG_Python_SetConstant(d, "MessageEntryType_GetServiceDescRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_GetServiceDescRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_ObjectTypeName",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_ObjectTypeName)));
  SWIG_Python_SetConstant(d, "MessageEntryType_ObjectTypeNameRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_ObjectTypeNameRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_ServiceClosed",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_ServiceClosed)));
  SWIG_Python_SetConstant(d, "MessageEntryType_ServiceClosedRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_ServiceClosedRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_ConnectClient",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_ConnectClient)));
  SWIG_Python_SetConstant(d, "MessageEntryType_ConnectClientRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_ConnectClientRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_DisconnectClient",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_DisconnectClient)));
  SWIG_Python_SetConstant(d, "MessageEntryType_DisconnectClientRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_DisconnectClientRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_ConnectionTest",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_ConnectionTest)));
  SWIG_Python_SetConstant(d, "MessageEntryType_ConnectionTestRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_ConnectionTestRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_GetNodeInfo",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_GetNodeInfo)));
  SWIG_Python_SetConstant(d, "MessageEntryType_GetNodeInfoRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_GetNodeInfoRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_ReconnectClient",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_ReconnectClient)));
  SWIG_Python_SetConstant(d, "MessageEntryType_ReconnectClientRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_ReconnectClientRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_NodeCheckCapability",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_NodeCheckCapability)));
  SWIG_Python_SetConstant(d, "MessageEntryType_NodeCheckCapabilityRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_NodeCheckCapabilityRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_GetServiceAttributes",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_GetServiceAttributes)));
  SWIG_Python_SetConstant(d, "MessageEntryType_GetServiceAttributesRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_GetServiceAttributesRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_ConnectClientCombined",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_ConnectClientCombined)));
  SWIG_Python_SetConstant(d, "MessageEntryType_ConnectClientCombinedRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_ConnectClientCombinedRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_EndpointCheckCapability",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_EndpointCheckCapability)));
  SWIG_Python_SetConstant(d, "MessageEntryType_EndpointCheckCapabilityRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_EndpointCheckCapabilityRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_ServiceCheckCapabilityReq",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_ServiceCheckCapabilityReq)));
  SWIG_Python_SetConstant(d, "MessageEntryType_ServiceCheckCapabilityRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_ServiceCheckCapabilityRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_ClientKeepAliveReq",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_ClientKeepAliveReq)));
  SWIG_Python_SetConstant(d, "MessageEntryType_ClientKeepAliveRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_ClientKeepAliveRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_ClientSessionOpReq",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_ClientSessionOpReq)));
  SWIG_Python_SetConstant(d, "MessageEntryType_ClientSessionOpRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_ClientSessionOpRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_ServicePathReleasedReq",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_ServicePathReleasedReq)));
  SWIG_Python_SetConstant(d, "MessageEntryType_ServicePathReleasedRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_ServicePathReleasedRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_PropertyGetReq",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_PropertyGetReq)));
  SWIG_Python_SetConstant(d, "MessageEntryType_PropertyGetRes",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_PropertyGetRes)));
  SWIG_Python_SetConstant(d, "MessageEntryType_PropertySetReq",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_PropertySetReq)));
  SWIG_Python_SetConstant(d, "MessageEntryType_PropertySetRes",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_PropertySetRes)));
  SWIG_Python_SetConstant(d, "MessageEntryType_FunctionCallReq",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_FunctionCallReq)));
  SWIG_Python_SetConstant(d, "MessageEntryType_FunctionCallRes",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_FunctionCallRes)));
  SWIG_Python_SetConstant(d, "MessageEntryType_GeneratorNextReq",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_GeneratorNextReq)));
  SWIG_Python_SetConstant(d, "MessageEntryType_GeneratorNextRes",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_GeneratorNextRes)));
  SWIG_Python_SetConstant(d, "MessageEntryType_EventReq",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_EventReq)));
  SWIG_Python_SetConstant(d, "MessageEntryType_EventRes",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_EventRes)));
  SWIG_Python_SetConstant(d, "MessageEntryType_PipePacket",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_PipePacket)));
  SWIG_Python_SetConstant(d, "MessageEntryType_PipePacketRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_PipePacketRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_PipeConnectReq",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_PipeConnectReq)));
  SWIG_Python_SetConstant(d, "MessageEntryType_PipeConnectRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_PipeConnectRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_PipeDisconnectReq",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_PipeDisconnectReq)));
  SWIG_Python_SetConstant(d, "MessageEntryType_PipeDisconnectRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_PipeDisconnectRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_PipeClosed",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_PipeClosed)));
  SWIG_Python_SetConstant(d, "MessageEntryType_PipeClosedRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_PipeClosedRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_CallbackCallReq",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_CallbackCallReq)));
  SWIG_Python_SetConstant(d, "MessageEntryType_CallbackCallRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_CallbackCallRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_WirePacket",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_WirePacket)));
  SWIG_Python_SetConstant(d, "MessageEntryType_WirePacketRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_WirePacketRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_WireConnectReq",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_WireConnectReq)));
  SWIG_Python_SetConstant(d, "MessageEntryType_WireConnectRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_WireConnectRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_WireDisconnectReq",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_WireDisconnectReq)));
  SWIG_Python_SetConstant(d, "MessageEntryType_WireDisconnectRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_WireDisconnectRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_WireClosed",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_WireClosed)));
  SWIG_Python_SetConstant(d, "MessageEntryType_WireClosedRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_WireClosedRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_MemoryRead",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_MemoryRead)));
  SWIG_Python_SetConstant(d, "MessageEntryType_MemoryReadRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_MemoryReadRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_MemoryWrite",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_MemoryWrite)));
  SWIG_Python_SetConstant(d, "MessageEntryType_MemoryWriteRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_MemoryWriteRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_MemoryGetParam",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_MemoryGetParam)));
  SWIG_Python_SetConstant(d, "MessageEntryType_MemoryGetParamRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_MemoryGetParamRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_WirePeekInValueReq",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_WirePeekInValueReq)));
  SWIG_Python_SetConstant(d, "MessageEntryType_WirePeekInValueRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_WirePeekInValueRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_WirePeekOutValueReq",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_WirePeekOutValueReq)));
  SWIG_Python_SetConstant(d, "MessageEntryType_WirePeekOutValueRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_WirePeekOutValueRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_WirePokeOutValueReq",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_WirePokeOutValueReq)));
  SWIG_Python_SetConstant(d, "MessageEntryType_WirePokeOutValueRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_WirePokeOutValueRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_WireTransportOpReq",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_WireTransportOpReq)));
  SWIG_Python_SetConstant(d, "MessageEntryType_WireTransportOpRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_WireTransportOpRet)));
  SWIG_Python_SetConstant(d, "MessageEntryType_WireTransportEvent",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_WireTransportEvent)));
  SWIG_Python_SetConstant(d, "MessageEntryType_WireTransportEventRet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageEntryType_WireTransportEventRet)));
  SWIG_Python_SetConstant(d, "MessageErrorType_None",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_None)));
  SWIG_Python_SetConstant(d, "MessageErrorType_ConnectionError",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_ConnectionError)));
  SWIG_Python_SetConstant(d, "MessageErrorType_ProtocolError",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_ProtocolError)));
  SWIG_Python_SetConstant(d, "MessageErrorType_ServiceNotFound",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_ServiceNotFound)));
  SWIG_Python_SetConstant(d, "MessageErrorType_ObjectNotFound",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_ObjectNotFound)));
  SWIG_Python_SetConstant(d, "MessageErrorType_InvalidEndpoint",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_InvalidEndpoint)));
  SWIG_Python_SetConstant(d, "MessageErrorType_EndpointCommunicationFatalError",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_EndpointCommunicationFatalError)));
  SWIG_Python_SetConstant(d, "MessageErrorType_NodeNotFound",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_NodeNotFound)));
  SWIG_Python_SetConstant(d, "MessageErrorType_ServiceError",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_ServiceError)));
  SWIG_Python_SetConstant(d, "MessageErrorType_MemberNotFound",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_MemberNotFound)));
  SWIG_Python_SetConstant(d, "MessageErrorType_MemberFormatMismatch",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_MemberFormatMismatch)));
  SWIG_Python_SetConstant(d, "MessageErrorType_DataTypeMismatch",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_DataTypeMismatch)));
  SWIG_Python_SetConstant(d, "MessageErrorType_DataTypeError",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_DataTypeError)));
  SWIG_Python_SetConstant(d, "MessageErrorType_DataSerializationError",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_DataSerializationError)));
  SWIG_Python_SetConstant(d, "MessageErrorType_MessageEntryNotFound",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_MessageEntryNotFound)));
  SWIG_Python_SetConstant(d, "MessageErrorType_MessageElementNotFound",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_MessageElementNotFound)));
  SWIG_Python_SetConstant(d, "MessageErrorType_UnknownError",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_UnknownError)));
  SWIG_Python_SetConstant(d, "MessageErrorType_InvalidOperation",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_InvalidOperation)));
  SWIG_Python_SetConstant(d, "MessageErrorType_InvalidArgument",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_InvalidArgument)));
  SWIG_Python_SetConstant(d, "MessageErrorType_OperationFailed",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_OperationFailed)));
  SWIG_Python_SetConstant(d, "MessageErrorType_NullValue",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_NullValue)));
  SWIG_Python_SetConstant(d, "MessageErrorType_InternalError",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_InternalError)));
  SWIG_Python_SetConstant(d, "MessageErrorType_SystemResourcePermissionDenied",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_SystemResourcePermissionDenied)));
  SWIG_Python_SetConstant(d, "MessageErrorType_OutOfSystemResource",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_OutOfSystemResource)));
  SWIG_Python_SetConstant(d, "MessageErrorType_SystemResourceError",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_SystemResourceError)));
  SWIG_Python_SetConstant(d, "MessageErrorType_ResourceNotFound",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_ResourceNotFound)));
  SWIG_Python_SetConstant(d, "MessageErrorType_IOError",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_IOError)));
  SWIG_Python_SetConstant(d, "MessageErrorType_BufferLimitViolation",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_BufferLimitViolation)));
  SWIG_Python_SetConstant(d, "MessageErrorType_ServiceDefinitionError",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_ServiceDefinitionError)));
  SWIG_Python_SetConstant(d, "MessageErrorType_OutOfRange",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_OutOfRange)));
  SWIG_Python_SetConstant(d, "MessageErrorType_KeyNotFound",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_KeyNotFound)));
  SWIG_Python_SetConstant(d, "MessageErrorType_InvalidConfiguration",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_InvalidConfiguration)));
  SWIG_Python_SetConstant(d, "MessageErrorType_InvalidState",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_InvalidState)));
  SWIG_Python_SetConstant(d, "MessageErrorType_RemoteError",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_RemoteError)));
  SWIG_Python_SetConstant(d, "MessageErrorType_RequestTimeout",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_RequestTimeout)));
  SWIG_Python_SetConstant(d, "MessageErrorType_ReadOnlyMember",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_ReadOnlyMember)));
  SWIG_Python_SetConstant(d, "MessageErrorType_WriteOnlyMember",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_WriteOnlyMember)));
  SWIG_Python_SetConstant(d, "MessageErrorType_NotImplementedError",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_NotImplementedError)));
  SWIG_Python_SetConstant(d, "MessageErrorType_MemberBusy",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_MemberBusy)));
  SWIG_Python_SetConstant(d, "MessageErrorType_ValueNotSet",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_ValueNotSet)));
  SWIG_Python_SetConstant(d, "MessageErrorType_AbortOperation",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_AbortOperation)));
  SWIG_Python_SetConstant(d, "MessageErrorType_OperationAborted",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_OperationAborted)));
  SWIG_Python_SetConstant(d, "MessageErrorType_StopIteration",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_StopIteration)));
  SWIG_Python_SetConstant(d, "MessageErrorType_OperationTimeout",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_OperationTimeout)));
  SWIG_Python_SetConstant(d, "MessageErrorType_OperationCancelled",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_OperationCancelled)));
  SWIG_Python_SetConstant(d, "MessageErrorType_AuthenticationError",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_AuthenticationError)));
  SWIG_Python_SetConstant(d, "MessageErrorType_ObjectLockedError",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_ObjectLockedError)));
  SWIG_Python_SetConstant(d, "MessageErrorType_PermissionDenied",SWIG_From_int(static_cast< int >(RobotRaconteur::MessageErrorType_PermissionDenied)));
  SWIG_Python_SetConstant(d, "ClientServiceListenerEventType_ClientClosed",SWIG_From_int(static_cast< int >(RobotRaconteur::ClientServiceListenerEventType_ClientClosed)));
  SWIG_Python_SetConstant(d, "ClientServiceListenerEventType_ClientConnectionTimeout",SWIG_From_int(static_cast< int >(RobotRaconteur::ClientServiceListenerEventType_ClientConnectionTimeout)));
  SWIG_Python_SetConstant(d, "ClientServiceListenerEventType_TransportConnectionConnected",SWIG_From_int(static_cast< int >(RobotRaconteur::ClientServiceListenerEventType_TransportConnectionConnected)));
  SWIG_Python_SetConstant(d, "ClientServiceListenerEventType_TransportConnectionClosed",SWIG_From_int(static_cast< int >(RobotRaconteur::ClientServiceListenerEventType_TransportConnectionClosed)));
  SWIG_Python_SetConstant(d, "ClientServiceListenerEventType_ServicePathReleased",SWIG_From_int(static_cast< int >(RobotRaconteur::ClientServiceListenerEventType_ServicePathReleased)));
  SWIG_Python_SetConstant(d, "ServerServiceListenerEventType_ServiceClosed",SWIG_From_int(static_cast< int >(RobotRaconteur::ServerServiceListenerEventType_ServiceClosed)));
  SWIG_Python_SetConstant(d, "ServerServiceListenerEventType_ClientConnected",SWIG_From_int(static_cast< int >(RobotRaconteur::ServerServiceListenerEventType_ClientConnected)));
  SWIG_Python_SetConstant(d, "ServerServiceListenerEventType_ClientDisconnected",SWIG_From_int(static_cast< int >(RobotRaconteur::ServerServiceListenerEventType_ClientDisconnected)));
  SWIG_Python_SetConstant(d, "MemberDefinition_Direction_both",SWIG_From_int(static_cast< int >(RobotRaconteur::MemberDefinition_Direction_both)));
  SWIG_Python_SetConstant(d, "MemberDefinition_Direction_readonly",SWIG_From_int(static_cast< int >(RobotRaconteur::MemberDefinition_Direction_readonly)));
  SWIG_Python_SetConstant(d, "MemberDefinition_Direction_writeonly",SWIG_From_int(static_cast< int >(RobotRaconteur::MemberDefinition_Direction_writeonly)));
  SWIG_Python_SetConstant(d, "MemberDefinition_NoLock_none",SWIG_From_int(static_cast< int >(RobotRaconteur::MemberDefinition_NoLock_none)));
  SWIG_Python_SetConstant(d, "MemberDefinition_NoLock_all",SWIG_From_int(static_cast< int >(RobotRaconteur::MemberDefinition_NoLock_all)));
  SWIG_Python_SetConstant(d, "MemberDefinition_NoLock_read",SWIG_From_int(static_cast< int >(RobotRaconteur::MemberDefinition_NoLock_read)));
  SWIG_Python_SetConstant(d, "RR_TIMEOUT_INFINITE",SWIG_From_int(static_cast< int >((-1))));
  SWIG_Python_SetConstant(d, "RR_VALUE_LIFESPAN_INFINITE",SWIG_From_int(static_cast< int >((-1))));
  SWIG_addvarlink(globals, "TranspartCapabilityCode_PAGE_MASK", Swig_var_TranspartCapabilityCode_PAGE_MASK_get, Swig_var_TranspartCapabilityCode_PAGE_MASK_set);
  SWIG_addvarlink(globals, "TransportCapabilityCode_MESSAGE2_BASIC_PAGE", Swig_var_TransportCapabilityCode_MESSAGE2_BASIC_PAGE_get, Swig_var_TransportCapabilityCode_MESSAGE2_BASIC_PAGE_set);
  SWIG_addvarlink(globals, "TransportCapabilityCode_MESSAGE2_BASIC_ENABLE", Swig_var_TransportCapabilityCode_MESSAGE2_BASIC_ENABLE_get, Swig_var_TransportCapabilityCode_MESSAGE2_BASIC_ENABLE_set);
  SWIG_addvarlink(globals, "TransportCapabilityCode_MESSAGE2_BASIC_CONNECTCOMBINED", Swig_var_TransportCapabilityCode_MESSAGE2_BASIC_CONNECTCOMBINED_get, Swig_var_TransportCapabilityCode_MESSAGE2_BASIC_CONNECTCOMBINED_set);
  SWIG_addvarlink(globals, "TransportCapabilityCode_MESSAGE4_BASIC_PAGE", Swig_var_TransportCapabilityCode_MESSAGE4_BASIC_PAGE_get, Swig_var_TransportCapabilityCode_MESSAGE4_BASIC_PAGE_set);
  SWIG_addvarlink(globals, "TransportCapabilityCode_MESSAGE4_BASIC_ENABLE", Swig_var_TransportCapabilityCode_MESSAGE4_BASIC_ENABLE_get, Swig_var_TransportCapabilityCode_MESSAGE4_BASIC_ENABLE_set);
  SWIG_addvarlink(globals, "TransportCapabilityCode_MESSAGE4_BASIC_CONNECTCOMBINED", Swig_var_TransportCapabilityCode_MESSAGE4_BASIC_CONNECTCOMBINED_get, Swig_var_TransportCapabilityCode_MESSAGE4_BASIC_CONNECTCOMBINED_set);
  SWIG_addvarlink(globals, "TransportCapabilityCode_MESSAGE4_STRINGTABLE_PAGE", Swig_var_TransportCapabilityCode_MESSAGE4_STRINGTABLE_PAGE_get, Swig_var_TransportCapabilityCode_MESSAGE4_STRINGTABLE_PAGE_set);
  SWIG_addvarlink(globals, "TransportCapabilityCode_MESSAGE4_STRINGTABLE_ENABLE", Swig_var_TransportCapabilityCode_MESSAGE4_STRINGTABLE_ENABLE_get, Swig_var_TransportCapabilityCode_MESSAGE4_STRINGTABLE_ENABLE_set);
  SWIG_addvarlink(globals, "TransportCapabilityCode_MESSAGE4_STRINGTABLE_MESSAGE_LOCAL", Swig_var_TransportCapabilityCode_MESSAGE4_STRINGTABLE_MESSAGE_LOCAL_get, Swig_var_TransportCapabilityCode_MESSAGE4_STRINGTABLE_MESSAGE_LOCAL_set);
  SWIG_addvarlink(globals, "TransportCapabilityCode_MESSAGE4_STRINGTABLE_STANDARD_TABLE", Swig_var_TransportCapabilityCode_MESSAGE4_STRINGTABLE_STANDARD_TABLE_get, Swig_var_TransportCapabilityCode_MESSAGE4_STRINGTABLE_STANDARD_TABLE_set);
  SWIG_Python_SetConstant(d, "LogLevel_Trace",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogLevel_Trace)));
  SWIG_Python_SetConstant(d, "LogLevel_Debug",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogLevel_Debug)));
  SWIG_Python_SetConstant(d, "LogLevel_Info",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogLevel_Info)));
  SWIG_Python_SetConstant(d, "LogLevel_Warning",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogLevel_Warning)));
  SWIG_Python_SetConstant(d, "LogLevel_Error",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogLevel_Error)));
  SWIG_Python_SetConstant(d, "LogLevel_Fatal",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogLevel_Fatal)));
  SWIG_Python_SetConstant(d, "LogLevel_Disable",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogLevel_Disable)));
  SWIG_Python_SetConstant(d, "LogComponent_Default",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogComponent_Default)));
  SWIG_Python_SetConstant(d, "LogComponent_Node",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogComponent_Node)));
  SWIG_Python_SetConstant(d, "LogComponent_Transport",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogComponent_Transport)));
  SWIG_Python_SetConstant(d, "LogComponent_Message",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogComponent_Message)));
  SWIG_Python_SetConstant(d, "LogComponent_Client",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogComponent_Client)));
  SWIG_Python_SetConstant(d, "LogComponent_Service",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogComponent_Service)));
  SWIG_Python_SetConstant(d, "LogComponent_Member",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogComponent_Member)));
  SWIG_Python_SetConstant(d, "LogComponent_Pack",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogComponent_Pack)));
  SWIG_Python_SetConstant(d, "LogComponent_Unpack",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogComponent_Unpack)));
  SWIG_Python_SetConstant(d, "LogComponent_ServiceDefinition",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogComponent_ServiceDefinition)));
  SWIG_Python_SetConstant(d, "LogComponent_Discovery",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogComponent_Discovery)));
  SWIG_Python_SetConstant(d, "LogComponent_Subscription",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogComponent_Subscription)));
  SWIG_Python_SetConstant(d, "LogComponent_NodeSetup",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogComponent_NodeSetup)));
  SWIG_Python_SetConstant(d, "LogComponent_Utility",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogComponent_Utility)));
  SWIG_Python_SetConstant(d, "LogComponent_RobDefLib",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogComponent_RobDefLib)));
  SWIG_Python_SetConstant(d, "LogComponent_User",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogComponent_User)));
  SWIG_Python_SetConstant(d, "LogComponent_UserClient",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogComponent_UserClient)));
  SWIG_Python_SetConstant(d, "LogComponent_UserService",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogComponent_UserService)));
  SWIG_Python_SetConstant(d, "LogComponent_ThirdParty",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteur_LogComponent_ThirdParty)));
  
  // thread safe initialization
  swig::container_owner_attribute();
  
  SWIG_Python_SetConstant(d, "IPNodeDiscoveryFlags_NODE_LOCAL",SWIG_From_int(static_cast< int >(RobotRaconteur::IPNodeDiscoveryFlags_NODE_LOCAL)));
  SWIG_Python_SetConstant(d, "IPNodeDiscoveryFlags_LINK_LOCAL",SWIG_From_int(static_cast< int >(RobotRaconteur::IPNodeDiscoveryFlags_LINK_LOCAL)));
  SWIG_Python_SetConstant(d, "IPNodeDiscoveryFlags_SITE_LOCAL",SWIG_From_int(static_cast< int >(RobotRaconteur::IPNodeDiscoveryFlags_SITE_LOCAL)));
  SWIG_Python_SetConstant(d, "IPNodeDiscoveryFlags_IPV4_BROADCAST",SWIG_From_int(static_cast< int >(RobotRaconteur::IPNodeDiscoveryFlags_IPV4_BROADCAST)));
  SWIG_Python_SetConstant(d, "ServiceSubscriptionFilterAttributeGroupOperation_OR",SWIG_From_int(static_cast< int >(RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation_OR)));
  SWIG_Python_SetConstant(d, "ServiceSubscriptionFilterAttributeGroupOperation_AND",SWIG_From_int(static_cast< int >(RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation_AND)));
  SWIG_Python_SetConstant(d, "ServiceSubscriptionFilterAttributeGroupOperation_NOR",SWIG_From_int(static_cast< int >(RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation_NOR)));
  SWIG_Python_SetConstant(d, "ServiceSubscriptionFilterAttributeGroupOperation_NAND",SWIG_From_int(static_cast< int >(RobotRaconteur::ServiceSubscriptionFilterAttributeGroupOperation_NAND)));
  SWIG_Python_SetConstant(d, "ServiceSubscriptionManager_CONNECTION_METHOD_DEFAULT",SWIG_From_int(static_cast< int >(RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD_DEFAULT)));
  SWIG_Python_SetConstant(d, "ServiceSubscriptionManager_CONNECTION_METHOD_URL",SWIG_From_int(static_cast< int >(RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD_URL)));
  SWIG_Python_SetConstant(d, "ServiceSubscriptionManager_CONNECTION_METHOD_TYPE",SWIG_From_int(static_cast< int >(RobotRaconteur::ServiceSubscriptionManager_CONNECTION_METHOD_TYPE)));
  SWIG_Python_SetConstant(d, "RobotRaconteurObjectLockFlags_USER_LOCK",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurObjectLockFlags_USER_LOCK)));
  SWIG_Python_SetConstant(d, "RobotRaconteurObjectLockFlags_CLIENT_LOCK",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurObjectLockFlags_CLIENT_LOCK)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_NONE",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_NONE)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_ENABLE_NODE_DISCOVERY_LISTENING",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_ENABLE_NODE_DISCOVERY_LISTENING)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_ENABLE_NODE_ANNOUNCE",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_ENABLE_NODE_ANNOUNCE)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_ENABLE_LOCAL_TRANSPORT",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_ENABLE_LOCAL_TRANSPORT)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_ENABLE_TCP_TRANSPORT",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_ENABLE_TCP_TRANSPORT)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_ENABLE_HARDWARE_TRANSPORT",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_ENABLE_HARDWARE_TRANSPORT)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_LOCAL_TRANSPORT_START_SERVER",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_LOCAL_TRANSPORT_START_SERVER)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_LOCAL_TRANSPORT_START_CLIENT",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_LOCAL_TRANSPORT_START_CLIENT)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_TCP_TRANSPORT_START_SERVER",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_TCP_TRANSPORT_START_SERVER)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_TCP_TRANSPORT_START_SERVER_PORT_SHARER",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_TCP_TRANSPORT_START_SERVER_PORT_SHARER)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_DISABLE_MESSAGE4",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_DISABLE_MESSAGE4)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_DISABLE_STRINGTABLE",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_DISABLE_STRINGTABLE)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_DISABLE_TIMEOUTS",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_DISABLE_TIMEOUTS)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_LOAD_TLS_CERT",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_LOAD_TLS_CERT)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_REQUIRE_TLS",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_REQUIRE_TLS)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_LOCAL_TRANSPORT_SERVER_PUBLIC",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_LOCAL_TRANSPORT_SERVER_PUBLIC)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_TCP_TRANSPORT_LISTEN_LOCALHOST",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_TCP_TRANSPORT_LISTEN_LOCALHOST)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_NODENAME_OVERRIDE",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_NODENAME_OVERRIDE)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_NODEID_OVERRIDE",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_NODEID_OVERRIDE)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_TCP_PORT_OVERRIDE",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_TCP_PORT_OVERRIDE)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_TCP_WEBSOCKET_ORIGIN_OVERRIDE",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_TCP_WEBSOCKET_ORIGIN_OVERRIDE)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_ENABLE_INTRA_TRANSPORT",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_ENABLE_INTRA_TRANSPORT)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_INTRA_TRANSPORT_START_SERVER",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_INTRA_TRANSPORT_START_SERVER)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_TCP_TRANSPORT_IPV4_DISCOVERY",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_TCP_TRANSPORT_IPV4_DISCOVERY)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_TCP_TRANSPORT_IPV6_DISCOVERY",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_TCP_TRANSPORT_IPV6_DISCOVERY)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_LOCAL_TAP_ENABLE",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_LOCAL_TAP_ENABLE)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_LOCAL_TAP_NAME",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_LOCAL_TAP_NAME)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_JUMBO_MESSAGE",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_JUMBO_MESSAGE)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_ENABLE_ALL_TRANSPORTS",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_ENABLE_ALL_TRANSPORTS)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_CLIENT_DEFAULT",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_CLIENT_DEFAULT)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_CLIENT_DEFAULT_ALLOWED_OVERRIDE",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_CLIENT_DEFAULT_ALLOWED_OVERRIDE)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_SERVER_DEFAULT",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_SERVER_DEFAULT)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_SERVER_DEFAULT_ALLOWED_OVERRIDE",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_SERVER_DEFAULT_ALLOWED_OVERRIDE)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_SECURE_SERVER_DEFAULT",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_SECURE_SERVER_DEFAULT)));
  SWIG_Python_SetConstant(d, "RobotRaconteurNodeSetupFlags_SECURE_SERVER_DEFAULT_ALLOWED_OVERRIDE",SWIG_From_int(static_cast< int >(RobotRaconteur::RobotRaconteurNodeSetupFlags_SECURE_SERVER_DEFAULT_ALLOWED_OVERRIDE)));
  SWIG_addvarlink(globals, "PythonTracebackPrintExc", Swig_var_PythonTracebackPrintExc_get, Swig_var_PythonTracebackPrintExc_set);
  
  RobotRaconteur::InitPythonTracebackPrintExc();
  
  
  /* Initialize threading */
  SWIG_PYTHON_INITIALIZE_THREADS;
#if PY_VERSION_HEX >= 0x03000000
  return m;
#else
  return;
#endif
}

